package network

import (
	"strings"
)

var manuf = map[string]string {
    "741AE09": "Private",
    "C85CE27": "SYNERGY SYSTEMS AND SOLUTIONS",
    "C85CE2A": "San Telequip (P) Ltd.,",
    "B84C874": "Blum Novotest GmbH",
    "B84C879": "Airgain Inc.",
    "D0D94F7": "Mitsubishi Electric US, Inc.",
    "C022F12": "TSURUGA Electric Corporation",
    "C022F15": "Canon Electronic Business Machines (H.K.) Co., Ltd.",
    "C022F17": "Shenzhen Chengfenghao Electronics Co.;LTD.",
    "48DA353": "Xiamen Magnetic North Co., Ltd",
    "48DA35C": "Guangzhou Xinhong Communication Technology Co.,Ltd",
    "C022F19": "MAHINDR & MAHINDRA",
    "C022F1B": "Lafayette AB",
    "5491AFC": "DanuTech Europe Kft",
    "D015BBD": "Bluewaves Mobility Innovation Inc",
    "48DA351": "Think Engineering",
    "5491AF3": "IronLink",
    "88A6EF6": "ShenZhen KZIot Technology LLC.",
    "88A6EFD": "Hash Mining s.r.o.",
    "38B8EBB": "PEZY Computing K.K.",
    "88A6EF8": "TechPLEX Inc.",
    "50482C9": "Soter Technologies",
    "50482CD": "KIDO SPORTS CO., LTD.",
    "68DA73A": "Shenzhen Haiyingzhilian Industrial Co., Ltd.",
    "68DA739": "Nadex Machinery(Shanghai) Co.,Ltd",
    "58C41EC": "PQTEL Network Technology Co. , Ltd.",
    "58C41ED": "Munich Electrification GmbH",
    "C86BBC6": "Drowsy Digital Inc",
    "BC3198E": "RADAR",
    "BC31989": "Baisstar (Shenzhen) Intelligence Co., Ltd.",
    "BC31985": "Hunan Gukam Railway Equipment Co.,Ltd",
    "BC3198B": "Suzhou Anchi Control system.,Co.Ltd",
    "C86BBC9": "Osee Technology LTD.",
    "BC34000": "Redvision CCTV",
    "B0FF721": "Guangzhou Senguang Communication Technology Co., Ltd",
    "1C21D18": "Cleaveland/Price, Inc.",
    "B0FF725": "Shenzhen Ruilian Electronic Technology Co.,Ltd",
    "B0FF724": "Jiangxi Xingchi Electronic Technology Co.,Ltd.",
    "90F4213": "Sinpeng(Guangzhou)Technology Co.,Ltd",
    "FCCD2F5": "QCTEK CO.,LTD.",
    "B44D43E": "GearUP Portal Pte. Ltd.",
    "6C2ADF3": "Johnson Controls IR, Sabroe Controls",
    "6C2ADF6": "ITI Limited",
    "7006922": "Scud (Fujian) Electronics Co.,Ltd",
    "700692B": "SWIT Electronics Co.,Ltd",
    "7006926": "Hangzhou Clounix Technology Limited",
    "700692E": "Ganghsan Guanglian",
    "7006929": "Shenzhen Lingwei Technology Co., Ltd",
    "7006924": "Fusiostor Technologies Private Limited",
    "6C2ADFA": "JBF",
    "E03C1C5": "Semic Inc.",
    "54083BD": "BHS Corrugated Maschinen- und Anlagenbau GmbH",
    "54083BC": "FairPhone B.V.",
    "F40E11F": "Private",
    "885D90F": "Private",
    "54083BE": "Sinclair Technologies",
    "54083BB": "Korea Bus Broadcasting",
    "A44F29F": "Private",
    "F80278F": "Private",
    "B01F81F": "Private",
    "B437D1F": "Private",
    "E81863F": "Private",
    "B0C5CAF": "Private",
    "3C39E7F": "Private",
    "E03C1CD": "Sprintshield d.o.o.",
    "E03C1CA": "MELAG Medizintechnik GmbH & Co. KG",
    "D016F0C": "Peralex Electronics (Pty) Ltd",
    "D016F00": "Shenzhen Lesingle Technology CO., LTD.",
    "D09395B": "Invendis Technologies India Pvt Ltd",
    "E03C1C8": "Jiangsu Riying Electronics Co.,Ltd.",
    "D093956": "Annapurna labs",
    "5C6AECB": "Shenzhen Anked vision Electronics Co.Ltd",
    "5C6AEC2": "Shenzhen Mingyue Technology lnnovation Co.,Ltd",
    "5C6AEC1": "Shanghai Smilembb Technology Co.,LTD",
    "7C45F9D": "Mobilaris Industrial Solutions",
    "7C45F99": "MIJ CO LTD",
    "5C6AECD": "DarkVision Technologies Inc.",
    "C0EAC36": "Worldpass industrial Company Limited ",
    "C0EAC31": "Dongguan Wecxw CO.,Ltd.",
    "C0EAC33": "Hangzhou Qixun Technology Co., Ltd",
    "8C5DB26": "SmartMore Corporation Limited",
    "C0EAC3B": "SeongHo Information and Communication Corp.",
    "8C5DB28": "Guangzhou Phimax Electronic Technology Co.,Ltd",
    "8C5DB29": "ISSENDORFF KG",
    "8C5DB2D": "Guandong Yuhang Automation Technology Co.,Ltd",
    "8C5DB21": "DAYOUPLUS",
    "705A6F2": "Tyromotion GmbH",
    "705A6F0": "Thyracont Vacuum Instruments GmbH",
    "C4A559E": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "705A6F8": "LUAN Industry and Commerce Co., Ltd",
    "705A6FD": "PICadvanced SA",
    "705A6F4": "Vaiotik Co., Ltd",
    "C4A559B": "SMH Technologies SRL",
    "84B3869": "Weiss Robotics GmbH & Co. KG",
    "C4A5593": "X-speed lnformation Technology Co.,Ltd",
    "E0382D2": "Xi'an Xiangxun Technology Co., Ltd.",
    "E0382D8": "Shenzhen iTest Technology Co.,Ltd",
    "0CCC473": "Shimane Masuda Electronics CO.,LTD.",
    "E0382D1": "Annapurna labs",
    "84B3864": "COBHAM",
    "0CCC479": "OptConnect",
    "C083597": "Fuzhou Fdlinker Technology Co.,LTD",
    "0CCC478": "NINGBO QIXIANG INFORMATION TECHNOLOGY CO., LTD",
    "F0221DE": "oToBrite Electronics, Inc.",
    "F0221DD": "Schleissheimer Soft- und Hardwareentwicklung GmbH",
    "F0221DC": "Estone Technology LTD",
    "E0382D4": "Qingdao Unovo Technologies Co., Ltd",
    "E0382D5": "Weishi Intelligent Information Technology (Guangzhou) Co., LTD",
    "F0221D0": "THANHBINH COMPANY - E111 FACTORY",
    "D46137C": "MUSASHI ENGINEERING,INC.",
    "4C93A65": "Fastenal IP Company",
    "4C74A7D": "ddcpersia",
    "6CDFFB3": "Beijing Ainemo Co Ltd",
    "D096865": "Annapurna labs",
    "D09686B": "Changsha keruijie lnformation Technology Co.,Ltd",
    "D096868": "Energiekonzepte Deutschland GmbH",
    "C49894A": "Neron Informatics Pvt Ltd",
    "C48372B": "care.ai",
    "C483722": "AI-RIDER CORPORATION",
    "C48372D": "Annapurna labs",
    "C483728": "ACCELECOM LTD.",
    "D420007": "Annapurna labs",
    "5847CA0": "LITUM BILGI TEKNOLOJILERI SAN. VE TIC. A.S.",
    "D420001": "Zelus(HuangZhou) Technology Ltd.",
    "D420000": "Wattsense",
    "C4A10E0": "HYOSUNG Heavy Industries Corporation",
    "5847CA8": "Birger Engineering, Inc.",
    "5847CA4": "Future Tech Development FZC LLC",
    "883CC5E": "myUpTech AB",
    "883CC5D": "Lenard Enterprises Inc",
    "18C3F4B": "VECTOR TECHNOLOGIES, LLC",
    "883CC59": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "883CC5C": "HDL da Amaz\u00f4nia Industria Eletr\u00f4nica Ltda",
    "2C691D4": "SPEEDTECH CORP.",
    "2C691D8": "IBM",
    "303D510": "Fink Telecom Services GmbH",
    "303D516": "Amber-Link Network Technology Co.,Ltd.",
    "18C3F49": "Ningbo Yuda Communication Technology Co.,Ltd",
    "303D51B": "Labman Automation",
    "18C3F4C": "HANGZHOU ZHONGKEJIGUANG TECHNOLOGY CO., LTD",
    "FC61795": "Qisda Corporation",
    "FC6179C": "Shenzhen Xmitech Electronic Co.,Ltd",
    "303D511": "SHENZHEN WLINK TECHNOLOGY CO., LTD.",
    "0C7FED0": "Guangdong Tianshu New Energy Technology Co., Ltd",
    "0C7FEDA": "Annapurna labs",
    "8C51095": "Heliox Automotive B.V.",
    "8C51091": "Amzetta Technologies, LLC",
    "7050E7E": "KFBIO (KONFOONG BIOINFORMATION TECH CO.,LTD)",
    "7050E78": "Shenzhen Dangs Science and Technology CO.,Ltd.",
    "8002F4B": "Baicells Technologies Co., Ltd",
    "8002F40": "BK Networks Co,. Ltd.",
    "C4A10E8": "Ayla Networks (Shenzhen) Co., Ltd.",
    "C4A10E3": "Consolinno Energy GmbH",
    "C4A10ED": "Connectlab SRL",
    "C4A10E1": "BARTEC PIXAVI AS",
    "9C431EE": "SHURE INCORPORATED",
    "1C5974E": "Globe Tracker ApS",
    "18A59CB": "CAL-COMP INDUSTRIA E COMERCIO DE ELETRONICOS E INFORMATICA LTDA",
    "381F261": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "18A59C5": "Thermia AB",
    "381F269": "SMS Evoko Group AB",
    "18A59CC": "BlueEyes Technology",
    "3043D73": "Luxshare Electronic Technology (Kunshan) LTD",
    "3043D77": "DIGICITI Technology Co.,Ltd",
    "381F268": "Avon Protection",
    "3043D72": "Apollo Infoways Private Limited",
    "6C93081": "WATERFORD CONSULTANTS LLC",
    "0C86299": "HONGKONG SAINT TECH INDUSTRIAL LIMITED",
    "6C93080": "Braums",
    "6C93085": "Shenzhen C & D Electronics Co., Ltd.",
    "0C8629B": "Akribis Systems",
    "50FF991": "COYOTE SYSTEM",
    "0C8629D": "BEIJING BEIBIANZHIDA TECHNOLOGY CO.,LTD",
    "0C86295": "Shenzhen protostellar technology Co., Ltd",
    "988FE03": "Empowerment Technologies Inc.",
    "988FE0B": "Dongguan Synst Electronics Co., LTD.",
    "988FE02": "vhf elektronik GmbH",
    "988FE0C": "Shenzhen Micro&Nano Perception Computing Technology Co.,Ltd",
    "0826AE3": "Shenzhen Hai yingZhilian Industrial Co., Ltd.",
    "DC36434": "Fresenius Medical Care R&D (Shanghai) Co.,Ltd.",
    "0826AE2": "ZaiNar",
    "18D793A": "zhejiang Anhong technology co.,ltd",
    "18D793D": "Kraken Technologies Ltd",
    "18D7939": "Clarity Medical Pvt Ltd",
    "04EEE8C": "daishin",
    "04EEE8D": "SHENZHEN TOPWELL TECHNOLOGY CO., LTD.",
    "50A030E": "HANKOOK CTEC CO,. LTD.",
    "18D7931": "Annapurna labs",
    "94C9B71": "C-Mer Rainsoptics Limited",
    "50A030B": "SHANGHAI ZXELINK Co.,Ltd",
    "50A030D": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "94C9B72": "Annapurna labs",
    "50A0305": "Jiangsu Jinshi Legend Technology Co.,Ltd",
    "1845B32": "Haier cloud Health Technology (Qingdao) Co., Ltd",
    "F4A454A": "Annapurna labs",
    "1845B38": "ShenZhen Topband Co.,Ltd",
    "F4A4545": "Denshijiki Industry Co.,Ltd",
    "F4A4544": "Earshots",
    "1845B30": "leetop tech co.,ltd",
    "1845B31": "Pfannenberg GmbH",
    "F4700C2": "Beijing ASU Tech Co., Ltd.",
    "F4700C9": "Annapurna labs",
    "F4700C6": "Jinan USR IOT Technology Limited",
    "F4700CD": "Freeus LLC",
    "7872648": "Gsou Technology(Shenzhen)Co.,Ltd",
    "7872649": "SHENZHEN FANGZHICHENG TECHNOLOGY CO., LTD.",
    "787264D": "QT systems ab",
    "9880BB4": "Shenzhen Ginto E-commerce CO.,LTD",
    "1CAE3EA": "Beijing SuperCloud Technology Co., Ltd.",
    "986EE84": "Fujitsu component limited",
    "986EE8C": "Sercomm Corporation.",
    "1CAE3ED": "FORME",
    "1CAE3E6": "Annapurna labs",
    "7813057": "E-Stone Electronics Co., Ltd",
    "7C83345": "PRO BRAND TECHNOLOGY (TW)",
    "38A8CDE": "OUTFORM",
    "785EE86": "Guangdong COROS Sports Technology Co., Ltd",
    "6433B52": "Adesso, Inc",
    "2836137": "shenzhen technology limited",
    "6433B5C": "Geksacon",
    "44A92C6": "Ningbo joyson new energy automotive technology Co.,Ltd",
    "44A92CB": "Amethystum Storage Technology Co., Ltd",
    "44A92C8": "RT-Systemtechnik GmbH",
    "5848492": "X-speed lnformation Technology Co.,Ltd",
    "44A92C2": "Anhui Zhongxin Electronic Technology Co., Ltd.",
    "A85B36C": "ATER Technologies Co Ltd",
    "F02A2B3": "Frigotel SRL",
    "A85B365": "JUGANU LTD",
    "44A92C0": "ZHEJIANG HISING TECHNOLOGY CO.,LTD",
    "F02A2BC": "Comexio GmbH",
    "A85B362": "Loomanet Inc.",
    "E878294": "Annapurna labs",
    "E878297": "FAIOT Co., LTD",
    "781305C": "Brigates Microelectronics Co., Ltd.",
    "781305D": "Shanghai Siminics Optoelectronic Technology Co., Ltd",
    "7813052": "LEAFF ENGINEERING SRL",
    "7813053": "microtec Sicherheitstechnik GmbH",
    "A85B363": "Shenzhen Dandelion Intelligent Cloud Technology Development Co., LTD",
    "FCD2B6D": "Bee Smart(Changzhou) Information Technology Co., Ltd",
    "986D359": "Advanced Diagnostics LTD",
    "1874E22": "Shenzhen WITSTECH Co.,Ltd.",
    "1874E26": "Beijing Jrunion Technology Co., Ltd.",
    "1874E29": "SHENZHEN AORO COMMUNICATION EQUIPMENT CO., LTD",
    "1874E2B": "Shenzhen Jooan Technology  Co., Ltd",
    "E878298": "JVISMall CO.,LTD",
    "1874E25": "HANGZHOU ZHOUJU ELECTRONIC TECHNOLOGICAL CO.,LTD ",
    "C0FBF96": "IVT corporation",
    "C0FBF92": "Dongguan Chuan OptoElectronics Limited",
    "C8F5D64": "EVOTOR LLC",
    "C8F5D61": "Valeo Interior Controls (Shenzhen) Co.,Ltd",
    "C8F5D68": "Yarward Electronics  Co., Ltd.",
    "88C9B37": "Robert Bosch JuP1",
    "C8F5D62": "Qbic Technology Co., Ltd",
    "C0FBF98": "Dongmengling",
    "88C9B3B": "Shenzhen MMUI Co.,Ltd",
    "1CA0EF1": "Wisnu and Supak Co.,Ltd.",
    "1CA0EFD": "Shenzhen Liandian Communication Technology Co.LTD",
    "0C5CB5B": "ADI Global Distribution",
    "245DFC4": "Suzhou Jiangzhi electronic technology co., Ltd",
    "0C5CB55": "The Raymond Corporation",
    "6015920": "S Labs sp. z o.o.",
    "6015924": "Zaptec",
    "601592B": "Annapurna labs",
    "E86CC78": "Lighthouse EIP",
    "E86CC72": "Xirgo Technologies LLC",
    "E86CC74": "Koal Software Co., Ltd",
    "245DFC8": "Cosmicnode",
    "E86CC7C": "Limited Liability Company M.S.Korp",
    "0411197": "Herrick Tech Labs",
    "9827824": "Dspread Technology (Beijing) Inc.",
    "9827827": "KORTEK CORPORATION",
    "446FD80": "Sichuan subao network technology ltd.co.",
    "78D4F17": "Famar Fueguina S.A.",
    "78D4F14": "BYD Auto lndustry Co.,Ltd",
    "446FD85": "ZHEJIANG SHIP ELECTRONICS & TECHNOLOGY CO., LTD.",
    "446FD8C": "Changzhou Haitu Electronic Technology Co.,Ltd",
    "78D4F15": "Huaqin Telecom Technology Co.,Ltd.",
    "78D4F19": "shanghai baudcom communication device co.,ltd",
    "885FE81": "Apoidea Technology Co., Ltd.",
    "A0024A6": "Xiaojie Technology (Shenzhen) Co., Ltd",
    "A453EE9": "Dongguan HuaFuu industrial co., LTD",
    "A453EE2": "Ubisafe Smart Devices",
    "A453EE1": "Stellamore",
    "8CAE49E": "Shenzhen C & D Electronics Co., Ltd.",
    "8CAE496": "Chengdu BillDTE Technology Co., Ltd",
    "8CAE49A": "Gigawave",
    "8C476E9": "Xertified AB",
    "8C476E0": "Chipsafer Pte. Ltd.",
    "8CAE498": "LLC Taipit - Measuring Equipment",
    "8CAE497": "Precitec Optronik GmbH",
    "8CAE49C": "Parametric GmbH",
    "34049EC": "ClearCaptions LLC",
    "8C476E2": "HuiZhou MIKI Communication Equipment Co.,LTD",
    "8CAE499": "TTR Corporation",
    "6879122": "CNDI CO.,LTD",
    "687912B": "Swisscom Broadcast Ltd",
    "687912A": "Wingtech Mobile Communications Co., Ltd.",
    "6879125": "Copper Labs, Inc.",
    "8411C20": "Kazdream Technologies LLP",
    "DC4A9ED": "HAPPIEST BABY INC.",
    "DC4A9E6": "TATTILE SRL",
    "8411C22": "Futurecom Systems Group",
    "8411C24": "LLC STC MZTA",
    "5895D86": "Norgren Manufacturing Co., Ltd.",
    "5895D89": "Loftie",
    "5895D8A": "Peak Communications Limited",
    "DC4A9E7": "Astrogate Inc.",
    "5895D8C": "LOCTEK ERGONOMIC TECHNOLOGY CORP.",
    "5895D80": "Shenzhen DOOGEE Hengtong Technology CO.,LTD",
    "DC4A9E8": "Methodex Systems Pvt. Ltd.",
    "CC4F5CC": "Beijing Cotytech Technology Co.,LTD.",
    "CC4F5C7": "Smiths US Innovation LLC",
    "CC4F5C1": "lesswire GmbH",
    "CC4F5C6": "Watertech S.p.A.",
    "CC4F5CB": "Ontex BV",
    "FCCD2F0": "Ningbo Bull Digital Technology Co., LTD",
    "CC4F5C4": "Spark Biomedical",
    "18FDCB3": "Staclar, Inc.",
    "18FDCBC": "Ark Vision Systems GmbH & Co. KG",
    "18FDCBE": "KWANG YANG MOTOR CO.,LTD",
    "98FC84C": "Shenzhen Incar Technology Co., Ltd.",
    "98FC84D": "Jazwares LLC",
    "18FDCB6": "SKA Organisation",
    "98FC840": "Leia, Inc",
    "98FC843": "ZeXin (Shanghai) Information Technologies Co.,Ltd",
    "98FC844": "Sferrum GmbH",
    "C0619AD": "Uhnder",
    "28B77C8": "Shenzhen PUAS Industrial Co.,LTD",
    "C0619A5": "Nanjing Balance Network Technology Co., Ltd",
    "28B77C1": "SolarEdge Technologies",
    "C0619A7": "MAD PIECE LLC.",
    "4C93A62": "Diehl Controls Nanjing Co., Ltd.",
    "F469D52": "Pulsar Engineering srl",
    "F469D5C": "Huaqin Telecom Technology Co.,Ltd.",
    "4C93A63": "Commsignia, Ltd.",
    "F469D57": "Rosco, Inc",
    "4C93A64": "4TheWall - 4D Sistem A.S",
    "F469D5A": "ShenZhenShi EVADA technology Co.,Ltd",
    "F469D50": "Mossman Limited",
    "5C857E6": "ProdataKey",
    "5C857E4": "Shenzhen IP3 Century Intelligent Technology CO.,Ltd",
    "5C857E5": "Shanghai Yanhe automation technology co.,LTD",
    "D01411B": "CYLTek Limited",
    "D014114": "powerall",
    "3049502": "Sercomm Corporation.",
    "3049501": "ATLI WORLD LIMITED",
    "304950C": "Anacove LLC",
    "D014119": "Airthings",
    "304950A": "Ledworks SRL",
    "D01411A": "ABB EVI SPA",
    "D014115": "Superlead",
    "F0D7AF3": "720?bei jing?Health iTech Co.,Ltd",
    "F0D7AFB": "EVCO SPA",
    "DCE5338": "JB-Lighting Lichtanlagen GmbH",
    "F0D7AF7": "Rievtech Electronic Co.,Ltd ",
    "F0D7AFC": "Shenzhen Virtual Clusters Information Technology Co.,Ltd.",
    "7069794": "SelectTech GeoSpatial, LLC",
    "706979A": "Foxconn Brasil Industria e Comercio Ltda",
    "CCC2618": "RoomMate AS",
    "706979C": "Rivian Automotive LLC",
    "94FBA74": "UOI TECHNOLOGY CORPORATION",
    "38F7CD3": "VANGUARD",
    "38F7CD2": "RIPower Co.,Ltd",
    "38F7CD6": "Fast Cotton(Beijing) Limited",
    "38F7CDB": "Fibergate Inc.",
    "94FBA70": "Reichert Inc.",
    "E8B4707": "Tibit Communications",
    "D425CC4": "Barobo, Inc.",
    "14AE852": "Qingdao iTechene Technologies Co., Ltd.",
    "F041C84": "Candelic Limited",
    "94FBA79": "Shanghai Hyco Genyong Technology Co., Ltd.",
    "E8B4704": "YAWATA ELECTRIC INDUSTRIAL CO.,LTD.",
    "C09BF46": "LTD Delovoy Office",
    "F490CBB": "A-dec Inc.",
    "F490CB7": "TEQ SA",
    "9405BBE": "BAE Systems",
    "9405BB2": "Dongguan CXWE Technology Co.,Ltd.",
    "9405BB0": "Qingdao Maotran Electronics co., ltd",
    "9405BB9": "Zimmer GmbH",
    "F490CBC": "Cheetah Medical",
    "9405BB7": "closip Inc.",
    "9405BB8": "iungo",
    "6462660": "MiiVii Dynamics Technology CO.,LTD",
    "94CC04A": "hyBee Inc.",
    "94CC04D": "Hanzhuo Information Technology(Shanghai) Ltd.",
    "94CC046": "Sam Nazarko Trading Ltd",
    "94CC042": "Nanjing Yacer Communication Technology Co. Ltd.",
    "646266E": "Shenzhen Jie Shi Lian Industrial Co., LTD",
    "6462668": "Leontech Limited",
    "6462661": "Annapurna labs",
    "6462665": "B\u00fchler AG",
    "14AE851": "Henfred Technology Co., Ltd.",
    "14AE85A": "MTA Systems",
    "B0B353E": "Nanjing Yining Intelligent Technology Co., Ltd.",
    "B0B353D": "IPvideo Corporation",
    "B0B3530": "Blake UK",
    "B0B353B": "Zoox",
    "B0B353C": "Beijing Geekplus Technology Co.,Ltd.",
    "3CFAD3B": " Corelink Technology Co.,Ltd",
    "B0B3537": "WUUK LABS CORP.",
    "3CFAD37": "LIPS Corporation",
    "3CFAD39": "Shenzhen Vplus Communication Intelligent Co., Ltd.",
    "3CFAD31": "Annapurna labs",
    "90E2FC0": "PEA (DBT-Technology)",
    "50DE199": "AEG Identifikationssysteme GmbH",
    "402C761": "Shanghai Dahua Scale Factory",
    "50DE19A": "Tannak International AB",
    "402C764": "Beijing Smarot Technology Co., Ltd.",
    "50DE193": "TRAXENS",
    "50DE191": "Clear Flow by Antiference",
    "50DE197": "Tianjin Natianal Health Technology Co.,Ltd",
    "50DE192": "SPII SPA",
    "200A0DE": "HANGZHOU DANGBEI NETWORK TECH.Co.,Ltd",
    "200A0DD": "Bently & EL  Co. Ltd.",
    "402C763": "EmbeddedArt AB",
    "200A0D6": "Austin Hughes Electronics Ltd.",
    "200A0DB": "Amazon Technologies Inc.",
    "200A0D3": "Clearly IP Inc",
    "A0224EE": "Hunan Youmei Science&Technology Development Co.,Ltd.",
    "A0224E6": "MESIT asd, s.r.o.",
    "0069672": "Ningbo Shen Link Communication Technology Co.,Ltd",
    "006967A": "Zhejiang Holip Electronic Technology Co.,Ltd",
    "0069671": "miliwave",
    "C4954DA": "KAT Mekatronik Urunleri AS",
    "C4954D4": "GL Solutions Inc.",
    "0069676": "Comcast-SRL",
    "A0224EC": "Standartoptic, Limited Liability Company",
    "A83FA18": "Neos Ventures Limited",
    "006967D": "aversix",
    "C4954D1": "Teletronik AG",
    "C4954D6": "AKKA Germany GmbH",
    "C4954D8": "Xinjiang Golden Calf Energy IOT Technology Co., Ltd ",
    "80E4DA0": "Wheatstone Corporation",
    "C4954D0": "BA International Electronics Co. Ltd.",
    "C4954D7": "LLC \"TechnoEnergo\"",
    "6431391": "Livongo Health",
    "506255E": "SHINSOFT CO., LTD.",
    "4403772": "Exsom Computers LLC",
    "4403776": "SHEN ZHEN HUAWANG TECHNOLOGY CO; LTD",
    "4403771": "Atari, Inc.",
    "440377E": "Bolin Technology Co., Ltd",
    "4403774": "Lenovo Image(Tianjin) Technology Ltd.",
    "506255D": "COTT Electronics",
    "04D16E5": "Dspread Technology (Beijing) Inc.",
    "04D16EE": "Evolute Systems Private Limited",
    "54A4933": "I-MOON TECHNOLOGY CO., LIMITED",
    "54A4930": "Intelligent Surveillance Corp",
    "401175E": "NIBBLE",
    "4011751": "Fujian Kuke3D Technology Co.,LTD",
    "4011755": "MIRC ELECTRONICS LTD",
    "4011754": "Table Trac Inc",
    "10DCB67": "Moya Commumication Technology (Shenzhen) Co.,Ltd.",
    "10DCB6A": "Pickering Interfaces Ltd",
    "4011750": "Lexi Devices, Inc.",
    "4C4BF9A": "Electrolux Professional AB",
    "208593B": "IOG Products LLC",
    "2415105": "GANZHOU DEHUIDA TECHNOLOGY CO., LTD",
    "2415102": "Nile Global Inc",
    "241510D": "Helen of Troy",
    "9806375": "GS GLOBAL SECURITY INC",
    "9806370": "Zoleo Inc.",
    "9806371": "E. P. Schlumberger",
    "208593D": "Shanghai Kenmyond Industrial Network Equipment Co.,Ltd",
    "208593A": "H3 Industries, Inc.",
    "241510B": "Teknic, Inc.",
    "44D5F2A": "SYS TEC electronic GmbH",
    "44D5F2D": "Shenzhen Nation RFID Technology Co.,Ltd.",
    "B4A2EBC": "Shanghai Shenou Communication Equipment Co., Ltd.",
    "D05F64E": "Montblanc-Simplo GmbH",
    "D05F648": "TytoCare LTD.",
    "44D5F26": "Beam Communications Pty Ltd",
    "D05F646": "Cyrus Technology GmbH",
    "FCA47A7": "Innovative Advantage",
    "2C16BDB": "LINGDONG TECHNOLOGY (BEIJING) CO. LTD",
    "B4A2EB9": "CURRENT WAYS, INC.",
    "B4A2EB5": "Annapurna labs",
    "B4A2EBA": "Hengkang\uff08Hangzhou\uff09Co.,Ltd",
    "2C16BDC": "Beijing CHJ Automotive Co., Ltd.",
    "FCA47AA": " Shenzhen Elebao Technology Co., Ltd",
    "FCA47AC": "Shenzhen ALFEYE Technology CO.,Ltd",
    "B4A2EBB": "Quantitec GmbH",
    "8C593C4": "Guralp Systems Limited",
    "D0C8575": "Beijing Inspiry Technology Co., Ltd. ",
    "D0C857E": "E-T-A Elektrotechnische Apparate GmbH",
    "8C593C9": "GENIS",
    "8C593C2": "Beida Jade Bird Universal Fire Alarm Device CO.,LTD.",
    "8C593C7": "OBO Pro.2 Inc.",
    "BC97401": "comtac AG",
    "BC97403": "Precision Galaxy Pvt. Ltd",
    "6095CE0": "Siema Applications",
    "6095CE8": "Trophy SAS",
    "6095CE6": "Xiamen Sigmastar Technology Ltd.",
    "6095CEB": "Beijing Sinomedisite Bio-tech Co.,Ltd",
    "BC97407": "Airfi Oy AB",
    "BC97405": "Shanghai Laisi Information Technology Co.,Ltd",
    "6095CEE": "VNS Inc.",
    "848BCD5": "exodraft a/s",
    "1C82591": "3xLOGIC Inc.",
    "848BCD6": "TWTG R&D B.V. ",
    "1C8259B": "KeyWest Networks, Inc",
    "848BCD2": "CCX Technologies Inc.",
    "B0FD0B8": "eSenseLab Ltd.",
    "B0FD0BA": "TEMCO JAPAN CO., LTD.",
    "B0FD0B3": "DMAC Security LLC",
    "C82C2BD": "UBITRON Co.,LTD",
    "C82C2B1": "Galgus",
    "C82C2BB": "Kunshan SVL Electric  Co.,Ltd",
    "C82C2B6": "Grav I.T.",
    "C82C2B7": "Merpa Bilgi Islem Ltd.Sti",
    "C863149": "Maxcom S.A.",
    "E41E0A8": "SAGE Glass",
    "C863148": "Thinci, Inc.",
    "C863141": "Autonics Co., Ltd.",
    "FCD2B63": "Coet Costruzioni Elettrotecniche",
    "745BC5D": "CELYSS SAS",
    "745BC5C": "ComNot",
    "A83FA12": "MEDCAPTAIN MEDICAL TECHNOLOGY CO., LTD.",
    "745BC5A": "Fournie Grospaud Energie SASU",
    "E44CC72": "Doowon Electronics & Telecom Co.,Ltd",
    "4CBC98A": "Shenzhen Cogitation Technology Co.,Ltd.",
    "E05A9F6": "Fibrain",
    "E44CC7C": "EPS Bio",
    "4CBC98E": "Wonder Workshop",
    "4CBC981": "JSC NIC",
    "E05A9F9": "Gemalto \"Document Readers\"",
    "D8860B5": "CAMTRACE",
    "38B19EB": "System Q Ltd",
    "D8860B7": "Gr\u00fcnbeck Wasseraufbereitung GmbH",
    "D8860B0": "Inspur Group Co., Ltd.",
    "CCD39D5": "SHENZHEN ROYOLE TECHNOLOGIES CO., LTD.",
    "CCD39DD": "Ethernity Networks",
    "9C69B45": "Elesta GmbH",
    "9C69B4A": "BEIJING PICOHOOD TECHNOLOGY CO.,LTD",
    "9C69B4B": "Toughdog Security Systems",
    "6CDFFB1": "Chongqing Baoli Yota Technologies Limited",
    "6CDFFB7": "Hashtrend AG",
    "4C917A2": "Chongqing Unisinsight Technology Co.,Ltd.",
    "4C917AD": "Shenzhen bankledger Technology Co, Ltd",
    "6CDFFBE": "Beijing Fimi Technology Co., Ltd.",
    "4C917A0": "Shenzhen Dangs Science & Technology CO.,LTD",
    "7CBC845": "Nanning auto digital technology co.,LTD",
    "98F9C75": "Tonycore Technology Co.,Ltd.",
    "98F9C7D": "hangzhou soar security technologies limited liability company",
    "2C4835E": "IROOTECH TECHNOLOGY CO.,LTD",
    "98F9C76": "GoodBox",
    "0CFE5DE": "NEWGREEN TECH CO., LTD.",
    "98F9C79": "Koala Technology CO., LTD.",
    "6C5C3D8": "GUANGZHOU GUANGRI ELEVATOR INDUSTRY CO.,LTD",
    "0CFE5DC": "Bepal Technology Co.,Ltd.",
    "0CFE5D0": "Chengdu Ledong Information & Technology Co., Ltd. ",
    "6C5C3D0": "ShenZhen Hugsun Technology Co.,Ltd.",
    "6C5C3D5": "Unitel Engineering",
    "A83FA16": "BEGLEC",
    "1CFD088": "ShenZhen DeLippo Technology Co., LTD",
    "1CFD089": "Cobham Slip Rings",
    "3C6A2CC": "Eltov System",
    "3C6A2CA": "Metro",
    "A83FA11": "GTDevice LLC",
    "3C6A2C8": "TP Radio",
    "A4ED437": "Wuxi Junction Infomation Technology Incorporated Company",
    "3C6A2C4": "XI'AN YEP TELECOM TECHNOLOGY CO.,LTD",
    "300A60A": "Ampetronic Ltd",
    "3C6A2CD": "Xiamen Smarttek CO., Ltd.",
    "300A60B": "Giax GmbH",
    "300A60C": "Thermo Process Instruments, LP",
    "3C6A2C1": "Olibra LLC",
    "300A609": "WINTEK System Co., Ltd",
    "A4ED43D": "Brand New Brand Nordic AB",
    "A4ED438": "Linseis Messgeraete GmbH",
    "A028330": "GERSYS GmbH",
    "A028336": "Xiamen Caimore Communication Technology Co.,Ltd.",
    "8489EC4": "Vayyar Imaging Ltd.",
    "0055DAE": "Victorsure Limited",
    "8489EC1": "Research Electronics International, LLC.",
    "3009F93": "OOO \"Microlink-Svyaz\"",
    "A02833D": "Audix",
    "A028332": "Shanghai Nohmi Secom Fire Protection  Equipment Co.,Ltd.",
    "8489ECE": "Shenzhen Intellifusion Technologies Co., Ltd.",
    "8489EC6": "POCT biotechnology",
    "8489EC8": "Arts Digital Technology (HK) Ltd.",
    "3009F9D": "Technology for Humankind",
    "9CF6DDB": "Guangzhou LANGO Electronics Technology Co., Ltd.",
    "C08359A": "SHANGHAI CHARMHOPE INFORMATION TECHNOLOGY CO.,LTD.",
    "9CF6DD1": "Ithor IT Co.,Ltd.",
    "9CF6DD9": "CAMA\uff08Luoyang\uff09Electronics Co.\uff0cLtd",
    "C083598": "ista International GmbH",
    "04C3E6A": "Sealed Unit Parts Co., Inc.",
    "C083593": "PCH Engineering A/S",
    "C08359B": "Suzhou Siheng Science and Technology Ltd.",
    "3C427EC": "Privacy Labs",
    "3C427E3": "Shenzhen VETAS Communication Technology Co , Ltd.",
    "B44BD6B": "DongYoung media",
    "B44BD66": "Perspicace Intellegince Technology",
    "B44BD63": "Huizhou Sunoda Technology Co. Ltd",
    "B44BD64": "Shenzhen Hi-Net Technology Co., Ltd.",
    "B44BD67": "Taizhou convergence Information technology Co.,LTD",
    "B44BD60": "G4S Monitoring Technologies Ltd",
    "3C427E2": "Starloop Tech Co., Ltd.",
    "3C427E5": "Geoplan Korea",
    "D47C44E": "SHENZHEN ANYSEC TECHNOLOGY CO. LTD",
    "A019B2B": "HangZhou iMagic Technology Co., Ltd ",
    "A019B2D": "RYD Electronic Technology Co.,Ltd.",
    "A019B27": "ARIMA Communications Corp.",
    "1CA0D3E": "Exicom Tele-Systems Ltd.",
    "D47C44C": "STRIVE ORTHOPEDICS INC",
    "D47C44B": "OPTiM Corporation",
    "A019B26": "GfG mbH",
    "D47C447": "Pongee Industries Co., Ltd.",
    "2C4835D": "Phasor Solutions Ltd",
    "2C48357": "FAST",
    "0C73EB4": "U-PASS.CO.,LTD",
    "3C24F07": "Swissdotnet SA",
    "480BB29": "Microprogram Information Co., Ltd",
    "480BB24": "Hangzhou Freely Communication Co., Ltd.",
    "480BB2C": "SHENZHEN TOPWELL TECHNOLOGY CO..LTD",
    "480BB28": "BravoCom\uff08xiamen\uff09TechCo.Ltd",
    "885FE8B": "Shenzhen ORVIBO Technology Co., Ltd",
    "885FE88": "Changsha Xiangji-Haidun Technology Co., Ltd",
    "301F9AB": "Smart Component Technologies LTD",
    "301F9A2": "CHISON Medical Technologies Co., Ltd.",
    "F041C8A": "Telstra",
    "F041C81": "DongGuan Siyoto Electronics Co., Ltd       ",
    "F041C8D": "ATN Media Group FZ LLC",
    "88A9A70": "Shenzhenshi kechuangzhixian technology Co.LTD",
    "88A9A75": "Volterman Inc.",
    "88A9A7D": "AVLINK INDUSTRIAL CO., LTD",
    "A4DA223": "DURATECH Enterprise,LLC",
    "A4DA22A": "Grundig",
    "A4DA224": "LORIOT AG",
    "A4DA22D": "Shen Zhen City YaKun Electronics Co., Ltd",
    "A4DA22B": "Klashwerks Inc.",
    "A4DA227": "Hydro Electronic Devices, Inc.",
    "A4DA225": "Original Products Pvt. Ltd.",
    "DCE533B": "Tintel Hongkong Co.Ltd",
    "A4DA221": "T2T System",
    "DCE5332": "Remko GmbH & Co. KG",
    "DCE5331": "Ambi Labs Limited",
    "C4FFBC5": "comtime GmbH",
    "C4FFBC1": "VISATECH C0., LTD.",
    "C4FFBC9": "GSM Innovations Pty Ltd",
    "1C21D1E": "p2-plus inc.",
    "34298FC": "Albert Handtmann Maschinenfabrik GmbH&Co.KG",
    "9C431ED": "HK ELEPHONE Communication Tech Co.,Limited",
    "282C025": "LLC \"MICROTEH\"",
    "282C020": "SAKATA DENKI Co., Ltd.",
    "282C02A": "Tokin Limited",
    "9C431E0": "Antailiye Technology Co.,Ltd",
    "282C02B": "ThirdReality, Inc",
    "F8B5684": "Combiwins Technology Co.,Limited",
    "F8B5685": "etectRx",
    "F8B5680": "LifePrint Products, Inc.",
    "F8B5683": "Dongwoo Engineering Co.,Ltd",
    "3873EAE": "Shenzhen Jixian Technology Co., Ltd.",
    "4048FD9": "Plus One Global Ltd.",
    "3873EAD": "Annapurna labs",
    "3873EA7": "PingGPS Inc",
    "3873EAA": "SHENZHEN CSE TECHNOLOGY CO., LTD",
    "3873EA3": "Proch plastic Co., Ltd.",
    "3873EAC": "LG Electronics",
    "EC9F0D4": "WisIOE",
    "8C147D9": "Anyware Solutions ApS",
    "34D0B80": "Captec Ltd",
    "34D0B8A": "Meatest sro",
    "AC1DDFB": "FINEpowerX INC",
    "AC1DDFD": "Elekon AG",
    "AC1DDF1": "HellaStorm, Inc.",
    "741AE0C": "bistos.co.ltd",
    "741AE00": "Huano International Technology Limited",
    "741AE04": "Revl Inc.",
    "AC1DDFA": "WESCO INTEGRATED SUPPLY",
    "741AE02": "NURA HOLDINGS PTY LTD",
    "CC22371": "Terma Sp. z o.o.",
    "741AE05": "FUJIAN TAILI COMMUNICATION TECHNOLOGY CO.,LTD",
    "2C279E6": "Rutledge Omni Services Pte Ltd",
    "2C279E0": "Changzhou WEBO Weighing Device & System CO.,LTD",
    "2C279E2": "Kunyi electronic technology (Shanghai) Co., Ltd.",
    "2C279EB": "Forties Inc.",
    "2C279ED": "Jiangsu JianHu Science & Technology Co., Ltd.",
    "904E911": "Apollo Video Technology",
    "189BA5C": "Christ Electronic System GmbH",
    "E818632": "AVCON Information Technology Co.,Ltd",
    "34298F4": "ISRA Vision AG",
    "34298F9": "Wiesheu GmbH",
    "34298F5": "Highlite International B.V.",
    "34008A7": "uberGARD Pte. Ltd.",
    "34008A8": "Shenzhen Andakai Technologies Co., Ltd.",
    "34008AA": "Hibertek International Limited",
    "28F537A": "Honeywell Safety Products USA, Inc",
    "7CBACC6": "Fossil Power Systems Inc",
    "7CBACC5": "Fortem Technologies, Inc.",
    "7CBACC9": "Yongguan Electronic Technology (D.G)LTD",
    "78D800E": "CL International ",
    "7CBACCA": "Annapurna labs",
    "78D800D": "Korea Micro Wireless Co.,Ltd.",
    "7CBACC3": "Izkare",
    "F88A3C7": "Josh.ai",
    "4C65A81": "Beijing Bluehalo Internet Inc.",
    "4C65A80": "WELT Corporation",
    "4C65A8C": "Fuse",
    "4C65A89": "SHENZHEN LISAIER TRONICS CO.,LTD",
    "4C65A87": "Wuhan MoreQuick Network Technology Co., Ltd.",
    "A0C5F2A": "Serious Integrated, Inc.",
    "A0C5F2B": "Oray.com co., LTD.",
    "8C147D5": "Unwired Networks",
    "F023B9E": "Domotz Ltd",
    "F023B9C": "Shenzhen Lachesis Mhealth Co., Ltd.",
    "8C147D6": "Shenzhen Meidou Technology Co, Ltd.",
    "04714BA": "Observables, Inc.",
    "60D7E38": "HindlePower, Inc",
    "60D7E3B": "Nextivity",
    "60D7E3C": "Zhejiang Send Intelligent Technology,Ltd",
    "04714B1": "uAvionix Corporation",
    "04714B4": "Apparatebau Gauting GmbH",
    "60D7E3E": "HuBDIC CO.,LTD",
    "04714B8": "Energport Inc",
    "60D7E34": "Hemisphere GNSS",
    "60D7E3D": "Quantronix, Inc.",
    "98AAFC4": "RPE \"RADICO\"",
    "98AAFC3": "Nexus Electrical(Jiaxing) Limited",
    "98AAFC8": "Beijing Tiandi-Marco Electro-Hydraulic Control System Company Ltd.",
    "08ED029": "Savox Communications ",
    "08ED02B": "Szok Energy and Communication Co., Ltd.",
    "08ED02C": "Guard RFID Solutions",
    "144FD76": "i-SENS, Inc.",
    "98AAFC6": "Mekotronics Co., Ltd",
    "98AAFCD": "MCS Micronic Computer Systeme GmbH",
    "144FD7B": "Arkus-ST Ltd",
    "A411639": "accesso Technology Group",
    "1CA0D39": "Cirque Audio Technology Co., Ltd",
    "1CA0D3B": "Guang Dong He Zheng Network Technology Co.,Ltd",
    "40F3857": "PALAZZETTI LELIO SPA",
    "8CC8F46": "SHENZHEN D-light Technolgy Limited",
    "8CC8F4A": "Trilux Group Management GmbH",
    "8CC8F41": "Lanhomex Technology(Shen Zhen)Co.,Ltd. ",
    "40F3854": "Embedded IQ",
    "40F3853": "IntelliDesign Pty Ltd",
    "8CC8F43": "TOHO DENKI IND.CO.,LTD",
    "8CC8F40": "Guardtec,Inc",
    "50A4D00": "TRAXENS",
    "50A4D08": "XinLian'AnBao\uff08Beijing\uff09Technology Co.\uff0cLTD.",
    "8CC8F49": "Swift Navigation Inc",
    "50A4D07": "Shanghai Pujiang Smart Card Systems Co., Ltd.",
    "50A4D05": "TREXOM S.r.l.",
    "50A4D09": "OEM PRODUCTION INC.",
    "40ED98B": "Siebert Industrieelektronik GmbH",
    "40ED987": "Vaisala Oyj",
    "34049E8": "Eclipse Information Technologies",
    "34049EB": "Eginity, Inc.",
    "34049ED": "uikismart",
    "40ED980": "Tsinghua Tongfang Co., LTD",
    "A4580FD": "EYE IO, LLC",
    "A4580F6": "Astro, Inc",
    "A4580FA": "GUANGZHOU OPTICAL BRIDGE COMMUNICATION EQUIPMENT CO.,LTD.",
    "40ED986": "Shanghai Broadwan Communications Co.,Ltd",
    "40ED985": "Cape",
    "500B918": "Panasonic Enterprise Solutions Company",
    "244E7BE": "WithWin Technology ShenZhen CO.,LTD",
    "244E7B5": "Jiangsu Xuanbo Electronic Technologies Co.,Ltd",
    "244E7B2": "RCC TIME CO .,LIMITED",
    "244E7B4": "Leshi Internet Information & Technology (Beijing) Corp.",
    "500B917": "Shenzhen Xinfa Electronic Co.,ltd",
    "7CCBE28": "Polarteknik Oy",
    "7CCBE2C": "mirakonta s.l.",
    "1CC0E10": "Shenzhen Highsharp Electronics Ltd.",
    "1CC0E18": "LX Corporation Pty Ltd",
    "4865EE4": "Mission Microwave Technologies, Inc",
    "4CE173E": "Plus One Japan Limited",
    "4865EE7": "Venture Research Inc.",
    "383A216": "Shenzhen Smart-core Technology co., Ltd.",
    "0CEFAF0": "Kenmore",
    "383A212": "Shenzhen HS Fiber Communication Equipment CO., LTD",
    "4CE1733": "outpaceIO",
    "AC64DD4": "8Cups",
    "AC64DD1": "JSC InfoTeCS",
    "AC64DDB": "Groupe Citypassenger Inc",
    "4CE173C": "REMONDE NETWORK ",
    "F81D784": "Digital Imaging Technology",
    "F81D783": "SHANGHAI SUN TELECOMMUNICATION CO., LTD.",
    "F81D78D": "Tofino",
    "70F8E7C": "Fixstars Corporation",
    "383A21D": "Colooc AB",
    "F81D78A": "AVPro Global Holdings LLC",
    "383A214": "Dongguan Innovation Technology Co Ltd",
    "70F8E72": "VOXX International",
    "D096864": "Houston Radar LLC",
    "84E0F42": "Hangzhou Uni-Ubi Co.,Ltd.",
    "84E0F4E": "Scale-Tec Ltd.",
    "58E876D": "Xiamen Cacamle Technology Co.,Ltd.",
    "C0D3919": "xxter bv",
    "C0D3917": "ALNETz Co.,LTD",
    "58E8765": "Broad Air Technology Co., LTD.",
    "58E876B": "Annapurna labs",
    "58E8763": "McWong International Inc",
    "F0ACD71": "Intenta GmbH",
    "F0ACD72": "QUANTUM POWER SYSTEMS",
    "F0ACD73": "Med-Pat/Inn-Phone",
    "F0ACD70": " Guilin glsun Science and Tech Co.,LTD",
    "283638E": "SCA Hygiene Products AB",
    "283638B": "ShangHai Canall Information Technology Co.,Ltd",
    "2836388": "Havells India Limited",
    "2836387": "Innovative Technology Ltd",
    "8C192D1": "Shenzhen Huanuo Internet Technology Co.,Ltd",
    "8C192DA": "TeleAlarm SA",
    "D0D94F8": "Apption Labs Limited",
    "D0D94F5": "Optigo Networks",
    "D0D94F3": "Beijing Yiwangxuntong Technology",
    "D0D94F4": "peiker CEE",
    "8C192D2": "DataRemote Inc.",
    "D0D94F0": "Perfant Technology Co., Ltd",
    "6891D0B": "Altis Technology ",
    "C47C8D7": "Awiselink Co., Ltd.",
    "C47C8DC": "INOTEC Sicherheitstechnik GmbH",
    "C47C8D5": "PASCAL Co., Ltd.",
    "C47C8D6": "HHCC Plant Technology Co.,Ltd.",
    "CCD31EB": "Elk Products",
    "CCD31EA": "Haishu Technology LIMITED",
    "C47C8D9": "Airbus DS - SLC",
    "6891D0A": "WiseCube",
    "6891D07": "Omniimpex GmbH",
    "E0B6F54": "Agora",
    "E0B6F5A": "Folksam AB",
    "E0B6F5D": "ITEL MOBILE LIMITED",
    "50FF99C": "Goetting KG",
    "50FF99D": "Shenzhen Haipengxin Electronic Co., Ltd.",
    "50FF99B": "Sichuan Dowlab Electronics Technology Co. Ltd",
    "5CF2863": "beijing your wonderful control system technology co.,ltd",
    "5CF2865": "EUROIMMUN Medizinische Labordiagnostika AG",
    "5CF2866": "VPInstruments",
    "5CF2862": "Shanghai Notion Information Technology CO.,LTD.",
    "5CF286C": "Sunpet Industries Limited",
    "5CF2868": "SHENZHEN HIVT TECHNOLOGY CO.,LTD",
    "5CF286D": "BrightSky, LLC",
    "986D351": "Shenzhen cositea electronics technology co.,LTD",
    "986D358": "Beijing 3CAVI Tech Co.,Ltd",
    "38FDFEE": "iSmart electronic technology co.,LTD",
    "7C477CC": "Annapurna labs",
    "7C477C3": "EyeLock LLC",
    "38FDFE7": "Rademacher Geraete-Elektronik GmbH",
    "38FDFE2": "Smart Solution Technology, Inc",
    "78CA830": "DAINCUBE",
    "78CA831": "Excelocity Inc.",
    "78CA839": "Louroe Electronics",
    "38B8EB5": "Dojo-Labs Ltd",
    "1C88792": "Airsmart System Co.,Ltd",
    "1C88796": "Eolos IT Corp",
    "1C8879C": "Accriva",
    "1C87761": "EBS Sp. z o.o.",
    "1C8779D": "Shenzhen Innovaconn Systems Co.,Ltd",
    "1C8779C": "AllThingsTalk",
    "1C87796": "Shenzhen Shouxin Tongda Technology Co.,Ltd",
    "1C87795": "BEIDIAN  GROUP",
    "8439BED": "Shenzhen Lidaxun Digital Technology Co.,Ltd",
    "8439BEC": "EDC Electronic Design Chemnitz GmbH",
    "8439BEA": "Emotiq s.r.l.",
    "40A36B9": "PH Technical Labs",
    "800A802": "Sumitomo Wiring Systems, Ltd.",
    "800A804": "LLVISION TECHNOLOGY CO.,LTD",
    "800A803": "Beijing VControl Technology Co., Ltd.",
    "0055DAB": "Interaxon Inc",
    "0055DAD": "Arrow Electronics,Inc.",
    "A03E6B2": "Videx Electronics S.p.A.",
    "A03E6B8": "718th  Research  Institute  of  CSIC",
    "A03E6B9": "Incogniteam Ltd.",
    "0055DA2": "Beijing Connected Information Technology Co.,Ltd.",
    "CC1BE09": "MobiStor Technology Inc.",
    "CC1BE08": "MDT technologies GmbH",
    "A03E6B5": "Friday Lab, UAB",
    "CC1BE0D": "NEWSTAR (HK) ELECTRONIC DEVELOPMENT LIMITED",
    "CC1BE03": "Shenzhen Vanstor Technology Co.,Ltd",
    "CC1BE06": "IC RealTech",
    "A03E6BC": "Qunar.com",
    "C88ED16": "Shenyang Machine Tool(Group) Research & Design Institute Co., Ltd, Shanghai Branch",
    "C88ED1A": "AP Sensing GmbH",
    "DC44278": "Wharton Electronics Ltd",
    "DC44273": "General Microsystems Sdn Bhd",
    "DC44270": "Suritel",
    "C88ED12": "ROTRONIC AG",
    "1C21D19": "Dynojet Research",
    "1C21D15": "B-Scada Inc.",
    "B0C5CA8": "Astyx GmbH",
    "B0C5CA1": "IVK-SAYANY",
    "B0C5CA5": "SYSTOVI",
    "78C2C00": "Shenzhen ELI Technology co.,ltd",
    "78C2C0B": "Wan Chao An (Beijing) Technology Co., Ltd.",
    "885D907": "Schmidt & Co.,(H.K.)Ltd.",
    "885D904": "Wuhan Strong Electronics Co., Ltd",
    "885D90C": "iRoom GmbH",
    "885D90B": "Premier Merchandises Limited",
    "B437D11": "Alturna Networks",
    "B437D12": "Fibersystem AB",
    "B437D13": "DIMTON\u00a0CO.,LTD.",
    "74F8DB5": "Provision-ISR",
    "64FB815": "Kay Schulze & Karsten Pohle GbR",
    "807B859": "SMART ELECTRONICS NZ LIMITED",
    "549A111": "SpearX Inc.",
    "807B85C": "SCALA Digital Technology(Ningbo) CO, LTD",
    "807B85B": "Oliotalo Oy",
    "64FB814": "Pricer AB",
    "64FB81A": "Bronkhorst High-Tech BV",
    "64FB818": "NPG Technology S.A.",
    "64FB813": "MOBILUS Inc.",
    "1CCAE31": "PGA ELECTRONIC",
    "80E4DA9": "Elcus",
    "2CD141E": "CITA SMART SOLUTIONS LTD",
    "80E4DA7": "Shortcut Labs",
    "2CD141A": "Fiberroad Technology Co., Ltd.",
    "2CD1415": "ZENIC INC.",
    "1CCAE30": "Private",
    "9802D8E": "Private",
    "2C6A6F1": "ELKO EP, s.r.o.",
    "A0BB3E5": "ManTech International Corporation",
    "90C6820": "Shenzhen Lencotion Technology Co.,Ltd",
    "A0BB3EA": "Filo SRL",
    "2C6A6F9": "Logic IO Aps",
    "2C6A6F7": "SM DSP CO.,LTD.",
    "9802D84": "Zedi, Inc.",
    "9802D88": "Simplo Technology Co.,LTD",
    "F802784": "CLARUS Korea Co., Ltd",
    "28FD80C": "Airbus Defence and Space Oy",
    "28FD802": "Zhixiang Technology Co., Ltd.",
    "0CEFAFC": "GainStrong Industry Co.,Ltd",
    "A0BB3E1": "IVision Electronics Co.,Ltd",
    "A0BB3E3": "WiteRiver Technology LLC",
    "2C265F2": "Jiangsu JARI Technology Group Co., LTD",
    "F80278B": "Rosemount Analytical",
    "2C265FA": "Polara Engineering",
    "A44F29A": "HTD",
    "A44F29B": "GUANGDONG REAL-DESIGN INTELLIGENT TECHNOLOGY CO.,LTD",
    "A44F290": "Dermalog Identification Systems GmbH",
    "0CEFAFA": "chengdu joyotime Technology Co., Ltd.",
    "3C39E7C": "VANSTONE ELECTRONIC (BEIJING)CO,. LTD.",
    "3C39E78": "Martem AS",
    "0CEFAF1": "Goerlitz AG",
    "1007235": "BEIJING SOOALL INFORMATION TECHNOLOGY CO.,LTD",
    "E81863B": "Protek Electronics Group Co.,LTD",
    "D022124": "Viatron GmbH",
    "D022120": "Spirit IT B.V.",
    "3C39E72": "HomeWizard B.V.",
    "3C39E75": "Attrackting AG",
    "74E14A9": "Kanto Aircraft Instrument Co., Ltd.",
    "E4956ED": "Shanghai Tieda Telecommunications Equipment Co.,LTD.",
    "E4956E8": "PT.MLWTelecom",
    "B8D812A": "Kiwigrid GmbH",
    "B8D8124": "V5 Technology Corporation",
    "74E14A7": "APM Technologies (DongGuan) Ltd",
    "BC66411": "Global China Technology Limited",
    "58FCDB9": "Hi-Target Surveying Instrument Co., Ltd.",
    "B01F817": "Aether Services, Inc.",
    "BC6641D": "UtilLighting Co.,Ltd.",
    "B01F81C": "Access Device Integrated Communications Corp.",
    "58FCDB4": "Inforce Computing Inc.",
    "58FCDB0": "Spang Power Electronics",
    "E4956E0": "SMC Networks, Inc",
    "BC66416": "Intuitive Surgical, Inc",
    "141FBAE": "POS Systema LLC",
    "F40E111": "BEIJING DONGJIN AERO-TECH CO., LTD",
    "141FBA7": "Wisnetworks Technologies Co., Ltd.",
    "F40E11A": "Kodpro Ltd.",
    "141FBA4": "BYZERO",
    "7C70BC4": "K-Vision Technology (Shanghai), Ltd",
    "7C70BC0": "Shanghai magcomm communication technology co ltd",
    "BC34003": "Altronix Corporation",
    "7C70BC7": "Nomad Digital Ltd.",
    "BC34007": "Q-PRODUCTS a. s.",
    "A43BFA6": "Recognition Systems LLC",
    "A43BFA7": "Deatronic srl",
    "D076504": "Private",
    "7419F89": "Princip a.s.",
    "7419F8A": "Tanjarine",
    "7419F88": "Quest Payment Systems",
    "D076508": "Accumulate AB",
    "7419F85": "Starcor Beijing Co.,Limited",
    "7419F82": "Symtop Instrument Co.",
    "C85CE2D": "UNILUMIN GROUP CO., LTD.",
    "4CEA416": "Gopod Group Limited",
    "E86CC70": "Trapeze Switzerland GmbH",
    "5CF8384": "Shanghai Zenchant Electornics Co.,LTD",
    "5CF8388": "Stonex srl",
    "4CEA415": "ZICHAN J TECHNOLOGY CO.,LTD",
    "5CF838A": "Semsotec GmbH",
    "5CF8381": "Bergische Ingenieure GmbH",
    "5CF8382": "The idiot company",
    "5CF838D": "Watts A\\S",
    "5CF8387": "T-CHIP INTELLIGENT TECHNOLOGY CO.,LTD.",
    "F41A793": "SHENZHEN XIANXIN ELECTRONICS CO., LTD.",
    "F41A797": "Avedis Zildjian Company",
    "5CF838E": "Guangzhou Duge Technology Co.,LTD",
    "5CF838B": "trilogik GmbH",
    "2CC44FA": "Shenzhen OneThing Technologies Co.,Ltd.",
    "1006485": "Annapurna labs",
    "2CC44F8": "Wuxi Yikesen Intelligent Technology Co., Ltd",
    "1006487": "Wilson Electronics",
    "100648E": "Kloud-12 LLC",
    "008DF43": "Algodue Elettronica Srl",
    "2CC44F3": "somon",
    "EC9A0C5": "SmartRay GmbH",
    "EC9A0C4": "Hangzhou Saicom Communication Technology Co., LTD",
    "EC9A0C3": "Protect Animals with Satellites LLC",
    "EC9A0C6": "Seek Thermal",
    "008DF4E": "Relay, Inc.",
    "EC9A0C0": "SHENZHEN HEXINDA SUPPLY CHAIN MANAGEMENT CO.LTD",
    "008DF41": "Beijing Infinitesensing Technology Co.,Ltd",
    "008DF4C": "Creative Security Technology Inc.",
    "C898DBA": "CHAMPIN",
    "EC9A0CA": "Fancom",
    "C898DB6": "Quantum Co., Ltd.",
    "78392D7": "Shenzhen C & D Electronics Co., Ltd.",
    "78392D8": "Dreamtek",
    "78392D2": "Chengdu Shiketong Technology Co.,Ltd",
    "D016F0E": "BBPOS Limited",
    "CCD31E6": "BBPOS Limited",
    "50482C2": "BBPOS Limited",
    "78392DB": "MedRx, Inc",
    "78392D5": "IO Master Technology",
    "8C55708": "Nayax LTD",
    "28F8C6B": "3PI Tech Solutions",
    "28F8C62": "Intermatic AG",
    "E8FF1E7": "NRT Technology Taiwan Co., Ltd.",
    "28F8C64": "Beijing HengzhengTC Sci-Tech Co., Ltd",
    "28F8C6A": "Videndum Media Solutions Spa",
    "38B8EBE": "WT-Consulting SAS",
    "E8FF1E4": "HMN Technologies Co., Ltd",
    "E8FF1ED": "Shenzhen AZW Technology Co., Ltd.",
    "28F8C65": "Sarco Equipments PVT. LTD.",
    "E8FF1E8": "Annapurna labs",
    "8CA6825": "Qstar Technology Co,Ltd",
    "E8FF1E0": "Private",
    "E8FF1EE": "Shenzhen kstar Science & Technology Co., Ltd",
    "E8FF1EA": "Geotab Inc.",
    "8CA682A": "Schok LLC",
    "8CA6828": "Schok LLC",
    "8CA682E": "Texys International",
    "D0A0110": "SIONYX, LLC",
    "78E9965": "Chuangming Futre Technology Co., Ltd.",
    "D0A0118": "Annapurna labs",
    "D0A0119": "Vastai Technologies(Shanghai)Inc",
    "78E9961": "COGITO TECH COMPANY LIMITED",
    "78E996D": "Lorch Schweisstechnik GmbH",
    "34C8D67": "ZXCLAA Technology(Suzhou) Co.,Ltd",
    "34C8D61": "Shenzhen Xmitech Electronic Co.,Ltd",
    "34C8D62": "Guangzhou Linkpi Technology Co. Ltd",
    "34C8D64": "Chengdu Decentest technology Co., Ltd.",
    "34C8D6C": "Huizhou KDT Intelligent Display Technology Co. Ltd",
    "34C8D68": "Illumina",
    "34C8D6E": "Shenzhen C & D Electronics Co., Ltd.",
    "04C3E6E": "Great Talent Technology Limited",
    "9015640": "Fengzhushou Co., Ltd.",
    "9015642": "PIN GENIE, INC.DBA LOCKLY",
    "9015643": "Clinton Electronics Corporation",
    "9015645": "Relectrify Pty Ltd",
    "0863323": "Umano Medical Inc.",
    "0863322": "innovative specialized security solutions",
    "086332B": "ShenZhen YuanXiang Digital Technology Co., Ltd",
    "0863327": "SOFTWARE-AUTOMATION-CONTROL JOINT STOCK COMPANY (CADPRO., JSC)",
    "48E6637": "EARWEISS TECHNOLOGY (SHENZHEN) CO,. LTD",
    "48E6639": "AceZone Aps",
    "48E6638": "Shenzhen Peicheng Technology Co., LTD. ",
    "C4CC377": "Vicinity Technologies Limited",
    "C4CC37C": "CHAINTECH Technology Corp.",
    "C4CC375": "Changzhou runningtech industrial technology Co., Ltd.",
    "C4CC37A": "INPAQ Technology Co., Ltd",
    "C4CC376": "Safety System Product GmbH & Co. KG",
    "60A434D": "Kaynes technology India Ltd",
    "60A4346": "Lechpol Electronics Leszek Sp.k.",
    "60A4340": "UNIQON",
    "60A4347": "Drov Technologies",
    "60A434E": "KNVISION",
    "C8FFBFA": "Maestro Food Co.",
    "C8FFBF9": "Shandong Wanshuo Optoelectronic Equipment Co.,Ltd",
    "C8FFBFC": "Density.IO",
    "C8FFBFE": "HT ITALIA SRL",
    "1063A3B": "Shen zhen shi shang mei dian zi shang wu you xian gong si",
    "1063A38": "Shenzhen C & D Electronics Co., Ltd.",
    "1063A36": "Morgan Schaffer",
    "1063A3D": "Triton Sensors",
    "5C87D87": "Tradewinds Networks Incorporated",
    "5C87D83": "Shenzhen Beiens Import and Export Co.,Ltd",
    "5C87D8B": "Credo Diagnostics Biomedical Pte. Ltd. Taiwan branch(SINGAPORE)",
    "5C5A4CB": "ESME SOLUTIONS",
    "5C5A4C8": "Spot AI, Inc.",
    "5C5A4C7": "Ace Computers",
    "F87A391": "Hangzhou Jiemu Electronic Technology Co.,Ltd",
    "F87A39D": "Xiamen Tonmind Technology Co.,Ltd",
    "EC5BCDD": "ASHIDA Electronics Pvt. Ltd",
    "EC5BCD9": "C&D Technologies",
    "EC5BCD0": "DONG GUAN YUNG FU ELECTRONICS LTD.",
    "EC5BCD1": "Ingersoll Rand",
    "3446630": "China Drive Electric Co.,Ltd\uff08Zhe Jiang\uff09",
    "3446634": "Luminys Systems Corporation",
    "08DA33C": "GITHON TECHNOLOGY CO., LTD.",
    "08DA333": "AI Storm",
    "48E6C6C": "Odin Solutions, S.L. - B73845893",
    "48E6C6B": "Q-PAC",
    "48E6C67": "Macriot",
    "F420552": "Elehear inc.",
    "F42055C": "Shenzhen Guangjian Technology Co.,Ltd",
    "F420553": "Annapurna labs",
    "F42055D": "Wuxi Sunning Smart Devices Co.,Ltd",
    "F420550": "AsicFlag",
    "2486252": "Emerson Automation FCP Kft.",
    "F42055B": "Synaps Technology S.r.l.",
    "248625A": "TAS INDIA PVT LTD",
    "2486254": "Codepoint Technologies, Inc.",
    "2486255": "METRO ELECTRONICS",
    "AC86D1D": "Krixe Pte. Ltd.",
    "485E0E3": "Shenzhen Anycon Electronics Technology Co.,Ltd",
    "485E0E9": "Cnergee Technologies Private Limited",
    "C4FF840": "ZHUHAI NINESTAR INFORMATION TECHNOLOGY CO., LTD.",
    "C4FF84B": "Elaraby Company For Engineering Industries",
    "ECA7B11": "AKOM Technologies Private Limited",
    "ECA7B1B": "Shenzhen Adreamer Elite Co.,Ltd",
    "ECA7B18": "Shenzhen zhst video technology co.,ltd",
    "B84C873": "Shenzhen Link-all Technology Co., Ltd",
    "B84C87C": "SOND",
    "B84C87B": "Beijing Yunji Technology Co., Ltd.",
    "B84C875": "Psync Labs, Inc.",
    "C022F10": "RCT Power GmbH",
    "5491AF6": "Star Systems International Limited",
    "5491AF7": "Hong Telecom Equipment Service Limited",
    "5491AF8": "Hunan Quanying Electronics Co. , Ltd.",
    "5491AF1": "4MITech",
    "5491AFB": "Hyperconn Pte. ltd",
    "50482CA": "JUNG HO",
    "D015BBE": "PHYTEC EMBEDDED PVT LTD",
    "D015BBB": "EdgeDX",
    "88A6EF3": "Enlaps",
    "88A6EF5": "Labpano Technology (Changzhou) Co., Ltd.",
    "88A6EF2": "METRO ELECTRONICS",
    "D015BB5": "Listen Technologies",
    "50482CE": "Harbin Nosean Tese And Control Technology Co.,Ltd",
    "6431394": "Active Brains",
    "D015BB8": "ALEKTO-SYSTEMS LTD",
    "50482C1": "Annapurna labs",
    "50482C7": "Oliver IQ, Inc.",
    "68DA73E": "Synamedia",
    "68DA734": "Agramkow A/S",
    "68DA736": "Global Networks ZEN-EI Co., Ltd",
    "68DA731": "DTEN Inc.",
    "58C41EB": "Pulse Structural Monitoring Ltd",
    "58C41E3": "Lemco IKE",
    "58C41E7": "HwaCom Systems Inc.",
    "C86BBCB": "HAI ROBOTICS Co.,Ltd.",
    "BC3198D": "Shanghai Sigen New Energy Technology Co., Ltd",
    "58C41E9": "ShenZhen Heng Yue Industry Co.,Ltd",
    "C86BBC0": "SafelyYou",
    "B0FF723": "Hammerhead Navigation Inc.",
    "B0FF726": "Tachyon Energy",
    "90F421D": "Taichitel Technology Shanghai Co.,Ltd.",
    "90F4217": "Senstar Corporation",
    "90F4210": "Gemstone Lights",
    "90F4215": "DESKO GmbH",
    "B0FF728": "ERA RF Technologies",
    "90F421A": "Proqualit Telecom LTDA",
    "B0FF72B": "Launch Tech Co., Ltd.",
    "B44D43D": "Shenzhen CreatLentem Technology Co.,Ltd",
    "B44D434": "ALL.SPACE Networks Ltd",
    "B44D430": "Mihoyo",
    "6C2ADF1": "Xi'an Xindian Equipment Engineering Center Co., Ltd",
    "B44D436": "Spot AI, Inc.",
    "2C279E3": "Private",
    "700692A": "Munters",
    "6C2ADF5": "Xinjiang Ying Sheng Information Technology Co., Ltd.",
    "700692D": "Skyware Protech Limited",
    "54083B2": "NAVITUS LT",
    "2415108": "Medicomp, Inc",
    "2CD141F": "Private",
    "54083B9": "Unicompute Technology Co.,Ltd.",
    "E4956EF": "Private",
    "7419F8F": "Private",
    "E03C1C1": "Shikino High-Tech Co., Ltd.",
    "D093951": "Hefei Siqiang Electronic Technology Co.,Ltd",
    "D09395E": "Shenzhen Hotack Technology Co.,Ltd",
    "D093954": "DAESUNG CELTIC ENERSYS",
    "D016F07": "Hydac Electronic",
    "D016F0A": "OPTITERA GLOBAL NETWORKS PRIVATE LIMITED",
    "D016F09": "Crystal Alarm AB",
    "549A11F": "Private",
    "0055DAF": "Private",
    "D093950": "Zhejiang Ruiyi lntelligent Technology Co. Ltd",
    "5C6AEC4": "GeneTouch Corp.",
    "5C6AEC3": "Shanghai Yunsilicon Technology Co., Ltd.",
    "5C6AEC5": "Exaterra Ltd.",
    "7C45F92": "Dongguan Boyye Industrial Co., Ltd",
    "C0EAC30": "Anhui Shengjiaruiduo Electronic Technology Co., Ltd.",
    "C0EAC35": "Techem Energy Services GmbH",
    "7C45F98": "Feller AG",
    "C0EAC3D": "Kontron Asia Technology Inc.",
    "A0224E1": "PoE Texas",
    "7C45F97": "Georg Fischer Piping Systems Ltd.",
    "705A6F6": "Earfun Technology (HK) Limited",
    "8C5DB22": "F+ Networks",
    "C4A5591": "Motive Technologies, Inc.",
    "C4A5592": "SHENZHEN ORFA TECH CO., LTD",
    "84B3860": "Nan Jing WZX Technology Limited",
    "84B3867": "FOTILE GROUP NINGBO FOTILE KITCHENWARE Co.,Ltd",
    "84B386C": "Palomar Products Inc",
    "84B386B": "Sineng electric CO., Ltd",
    "F0221D3": "ShenZhen Shizao Electronic Technology",
    "E0382DE": "Anysafe",
    "E0382DB": "SERCOMM PHILIPPINES INC",
    "E0382D7": "Famar Fueguina S.A.",
    "F0221D4": "Synergies Intelligent Systems Inc.",
    "D461370": "Wistron Corporation",
    "D461375": "Estelle AB",
    "4C74A78": "Annapurna labs",
    "F0221D6": "Vcognition Technologies Inc.",
    "D096860": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "C498943": "BTL Industries JSC",
    "C498949": "Shenzhen  Hexin Automation Technology Co.,Ltd.",
    "C498948": "Pliem (Shanghai) Intelligent Technology Co., Ltd",
    "C49894E": "Hans Sasserath GmbH & Co. KG",
    "C49894C": "Zhejiang Rexense loT Technology Co., Ltd",
    "C498942": "Metasphere Ltd",
    "D09686A": "Hero Health Inc.",
    "D420008": "Dalian Baishengyuan Technology Co.,Ltd",
    "C483723": "NextSilicon",
    "D42000B": "Shenzhen Volt IoT technology co.,ltd.",
    "C48372E": "Suzhou LZY technology Co.,Ltd",
    "5847CA1": "Hexagon Metrology Services Ltd.",
    "883CC56": "mfJebsen Electronics Ltd.",
    "E86CC77": "Huaqin Technology Co.,Ltd.",
    "883CC51": "Hanwei Electronics Group Corporation",
    "883CC58": "Wuhan Guangying Intelligence Technology Co., Ltd",
    "18C3F46": "VeriFone Systems (China), Inc.",
    "18C3F42": "Changsha Kiloview Electronics Co., Ltd.",
    "2C691DA": "Panasonic Appliances Marketing Asia Pacific",
    "2C691D7": "Shenzhen Gigalight Technology Co., Ltd",
    "303D51C": "TalkGo, Inc.",
    "2C691D1": "KATEK SE",
    "FC61797": "Kvaliteta Systems and Solutions Private Limited ",
    "0C7FED9": "Shenzhen ORVIBO Technology Co., Ltd.",
    "8C51090": "TianJin JointOptic Technology Co., LTD.",
    "8C51094": "Shenzhen WOWOTO Technology Co., Ltd.",
    "0C7FED6": "Netweb Technologies India Pvt Ltd",
    "0C7FEDD": "ALT Co., Ltd.",
    "8C5109A": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "8C5109B": "Beijing Superhexa Century Technology Co., Ltd.",
    "8002F43": "Shenzhen Suanzi Technology Co., Ltd",
    "7050E74": "Quantumdoor Technologies, Inc.",
    "7050E7C": "shenzhen newbridge communication equipment CO.,LTD",
    "6C15246": "Kunshan Abram Software Technology Co.,Ltd.",
    "6C15244": "Magicyo Technology CO., LTD.",
    "6C15249": "D-HOME SMAART",
    "1C59743": "Jiangsu Welm Technology Co.,Ltd",
    "1C59741": "Logical Infrastructure PTY LTD",
    "1C59747": "Lynxi Technologies Co.,Ltd.",
    "1C5974D": "Shenzhen Geshem Technology Co Ltd",
    "1C5974C": "King-On Technology Ltd.",
    "1C59745": "Shenzhen Shi Fang Communication Technology Co., Ltd",
    "1C59748": "Topway Global Technology Limited",
    "6C15243": "Forcite Helmet Systems Pty Ltd",
    "381F26C": "Jade Bird Fire Co., Ltd.",
    "381F265": "Zhejiang Huazhou Intelligent Equipment Co,. Ltd",
    "18A59CE": "BMC Messsysteme GmbH",
    "18A59C9": "estun automation co.,ltd",
    "18A59C0": "Omwave",
    "18A59C6": "INTEGRAL PLUS",
    "18A59CD": "Annapurna labs",
    "381F262": "Synamedia",
    "3043D7D": "Annapurna labs",
    "6C9308E": "ANDDORO LLC",
    "3043D70": "SYMES SA",
    "3043D75": "Shenzhen Mees Hi-Tech Co., Ltd",
    "6C9308A": "Shenzhen TOPWAY Technology Co.,LTD",
    "6C93089": "Shenzhen DOOGEE Hengtong Technology CO., LTD",
    "0C86298": "MyGregor Ltd",
    "6C9308D": "Annapurna labs",
    "988FE0D": "Shenzhen Vitalitim Technology Co., Ltd",
    "1054D29": "Bamboo Dynamics Corporation., Ltd.",
    "1054D24": "Raylogic Control Systems Private Limited",
    "0C86290": "Shanghai Prophet Electronic Technology Co.,Ltd",
    "988FE01": "GUANGZHOU HERYSTORM TECHNOLOGY CO.,LTD.",
    "0C86291": "Beijing Qinmu Data Technology Co., Ltd.",
    "0C86296": "C&A Marketing, INC.",
    "1054D27": "SHENZHEN CARSAFE TECHNOLOGY DEVELOPMENT CO.,LTD",
    "1054D25": "Sybersense",
    "0826AE9": "Annapurna labs",
    "0826AE8": "ShineTech Electronics Co., Ltd",
    "0826AE5": "SHENNAN CIRCUITS CO.,LTD",
    "0826AEB": "F-Plus Mobile LLC",
    "0826AE1": "Beijing Silion Technology Corp.,Ltd.",
    "18D7930": "Shenzhen JieXingTong Technology Co.,LTD",
    "18D7933": "Verification & Validation Technology Co.,Ltd",
    "DC36436": "Shenzhen smart-core technology co.,ltd.",
    "DC36432": "Wuhan Linptech Co. ,Ltd.",
    "DC3643E": "Beijing L&S Lancom Platform Tech. Co., Ltd.",
    "DC36437": "UKG",
    "18D7938": "Torsa Global",
    "04EEE8E": "Best Integration Technology Co., Ltd.",
    "04EEE80": "Zoomlion Huanuo(Beijing)Technology Co.,Ltd",
    "04EEE89": "Privacy Hero",
    "04EEE8B": "Hunan Yaguan Communication Technology Co.,Ltd",
    "50A0303": "RealWear (Shanghai) Intelligent Technology Co. Ltd",
    "04EEE86": "NALSSEN INC.",
    "50A0308": "GE Medical System China Co. Ltd.",
    "50A0300": "Gopod Group Limited",
    "50A030C": "GUANGZHOU UNIPOWER COMPUTER CO.,LTD",
    "94C9B74": "Zhejiang Hengjie Communication Technology Co,. Ltd.",
    "1845B33": "Ancsonic (chongqing) Electronic Science& Technology Co.,Ltd",
    "08F80D6": "SEDA CHEMICAL PRODUCTS CO., LTD.",
    "08F80D9": "Benelink Technology Inc.",
    "1845B39": "Teko Telecom Srl",
    "F4A454D": "SAEL SRL",
    "F4A4541": "PT Telkom Indonesia",
    "1845B3B": "Hangzhou CCRFID Microelectronic Co., Ltd.",
    "F4A454B": "Graco Inc.",
    "1845B3D": "Taicang T&W Electronics",
    "F4A454E": "Care Bloom, LLC",
    "F4700CA": "Jinan Huake Electrical Device Co., Ltd.",
    "74F8DB3": "ATX",
    "F4700C8": "Shenzhen Focuscom Communication Technology Co.,Ltd.",
    "F4700CC": "G.S.D GROUP INC.",
    "F4700C3": "Union Source Technology(HK)LTD",
    "7872642": "BZBGEAR",
    "7872643": "Guangdong Hongqin Telecom Technology Co.,Ltd.",
    "9880BB7": "Guangdong-Hong Kong-Macao Greater Bay Area Research Innovation Institute for Nanotechnology ",
    "9880BB9": "Shenzhen Hebang Electronic Co., Ltd",
    "7872640": "SmartMore Co.,ltd.",
    "9880BBB": "Hilo",
    "1CAE3E3": "HagerEnergy GmbH",
    "1CAE3EC": "QuEST Rail LLC",
    "986EE88": "Sisgeo Srl",
    "7C8334E": "Balter GmbH",
    "7C83344": "Fusus",
    "7C83346": "Wojinxin Beijing Technology Co., LTD",
    "38A8CD2": "Beijing Porient Technology Co., Ltd",
    "38A8CDB": "Qingdao Hisense Hitachi Air-conditioning Systems Co.,Ltd.",
    "7C83342": "PEMtronics LLC",
    "38A8CDC": "Beijing Aumiwalker technology CO.,LTD",
    "38A8CD8": "Max Way Electronics Co., Ltd.",
    "38A8CD6": "cal4care Pte Ltd",
    "38A8CD7": "Shenzhen C & D Electronics Co., Ltd.",
    "38A8CD0": "ACiiST Smart Networks Ltd.",
    "7C83341": "Linear Logic LLC",
    "785EE89": "TOPDON TECHNOLOGY Co.,Ltd.",
    "2836139": "Qingdao Airpoint Electronics Co.,Ltd.",
    "2836134": "Elytone Electronic Co., Ltd.",
    "785EE81": "RIKEN KEIKI NARA MFG. Co., Ltd.",
    "5848497": "Shandong Aotai Electric Co., LTD.",
    "6433B50": "Duomondi International Development Co., Ltd.",
    "6433B5E": "University of Texas at Austin",
    "5848496": "Shenzhen hongqifu Technology Co., Ltd",
    "584849B": "Daatrics LTD",
    "F02A2B7": "Protronix s.r.o.",
    "F02A2B1": "Tobi Tribe Inc.",
    "F02A2B9": "ZiGong Pengcheng Technology Co.,Ltd",
    "A85B36A": "TAIDEN INDUSTRIAL CO.,LTD",
    "A85B364": "Luoxian (Guandong) Technology Co., Ltd",
    "A85B369": "Avista Edge",
    "1874E28": "Kano Computing Limited",
    "D09FD9C": "Fujian Newland Auto-ID Tech. Co,.Ltd.",
    "D09FD94": "Poten (Shanghai) Technology Co.,Ltd.",
    "D09FD9D": "Shenzhen eloT Technology Co.,Ltd",
    "C0FBF9E": "Navitas Digital Safety Ltd",
    "D09FD98": "Queclink Wireless Solutions Co., Ltd.",
    "D09FD97": "Raymax Technology Ltd.",
    "C0FBF9A": "Tiandi(Changzhou) Automation Co., Ltd.",
    "D09FD90": "Lemei Intelligent IOT (Shenzhen) Co., Ltd",
    "88C9B32": "shenzhen franklin ESS technology CO.,Ltd",
    "C8F5D67": "Oscars Pro",
    "C8F5D65": "Pinmicro K K",
    "2CD141D": "Square Inc.",
    "88C9B31": "Cervoz Technology Co; Ltd.",
    "1CA0EF0": "Tangshan Liulin Automation Equipment Co., Ltd.",
    "1CA0EFC": "LLC \"Gagar.In\"",
    "1CA0EF8": "Zillnk",
    "20CE2A1": "Shanghai Digicube Info&Tech Co.,Ltd.",
    "20CE2A8": "Intelligraphics",
    "1CA0EF4": "Leviathan Solutions Ltd.",
    "0C5CB5C": "Hunan Newman Car NetworKing Technology Co.,Ltd",
    "0C5CB56": "S2C limited",
    "245DFC9": "TORGOVYY DOM  TEHNOLOGIY LLC",
    "245DFC0": "CompanyDeep",
    "245DFC5": "ContactProximity Inc",
    "F023B9D": "Shenyang Ali Technology Company Limited",
    "245DFC6": "Guangzhou Lango Electronics Technology Co.,Ltd.",
    "0411192": "Alethea Communications Technologies Pvt. Ltd.",
    "38B8EB7": "Sirin Mobile Technologies",
    "E86CC7A": "CoxSpace",
    "0411191": "Acentury",
    "446FD83": "Shenzhen Mestechs Technology CO., LTD",
    "446FD8A": "ZHEJIANG HIKAILINK TECHNOLOGY Co., Ltd",
    "9827825": "Guangzhou Wuzhou Technology Co, Ltd.",
    "982782A": "Nanjing BianYu Future Home Technology Co.Ltd",
    "9827829": "Wuxi GuoYiHaiJu Technology Co.,Ltd.",
    "9827828": "CATS Power design",
    "78D4F18": "Xiamen Cheerzing IOT Technology Co.,Ltd.",
    "A0024A9": "Kontakt Micro-Location Sp z o.o.",
    "A0024AB": "Xi'an Yingsheng Electric Technology Co.,Ltd.",
    "A0024A2": "Danriver Technologies Corp.",
    "3C39E71": "BEWATEC Kommunikationstechnik GmbH",
    "A0024A4": "Argos Solutions AS",
    "78D4F1D": "Quidel Corporation",
    "8CAE491": "H3 Platform",
    "A453EE0": "MAHLE ELECTRONICS, SLU",
    "A453EED": "SSK CORPORATION",
    "687912E": "Ametek Solidstate Controls",
    "34049E9": "Church & Dwight Co., Inc.",
    "8C476EA": "AU Optronics Corporation",
    "8C476E8": "IntelliVIX Co. Ltd.",
    "F0D7AFD": "Dongguan Gedi Electrons Techeology Co.,LTD",
    "6879127": "Babbit and Friends, SIA",
    "8411C28": "Leybold GmbH",
    "DC4A9E3": "LEACH INTERNATIONAL EUROPE",
    "DC4A9EA": " LongSung Technology (Shanghai) Co.,Ltd.   ",
    "DC4A9E5": "Nuove Tecnologie srl",
    "DC4A9E9": "AiSight GmbH",
    "5895D82": "Sercomm Corporation.",
    "FCCD2F9": "Aroma Retail",
    "FCCD2F2": "Loupedeck Oy",
    "58208A1": "BEIJING SENFETECH CORPORATION LTD.",
    "58208AC": "Jiangsu Zhonganzhixin Communication Technology Co.",
    "58208A6": "Shangyin Intelligence Technology Shandong Co.,Ltd",
    "58208A0": "Annapurna labs",
    "CC4F5C5": "Kymati GmbH",
    "58208AE": "UPM Technology, Inc",
    "58208A4": "TRING",
    "58208A9": "Suzhou Ruilisi Technology Ltd.",
    "98FC842": "Juketek Co., Ltd.",
    "18FDCBA": "Sercomm Corporation.",
    "18FDCB8": "CISTECH Solutions",
    "98FC84A": "Shield Inc.",
    "18FDCB1": "SOTHIS CIC TEC (Shanghai) Co., Ltd",
    "C0619AA": "Gronn Kontakt AS",
    "C0619A8": "Nanjing SinoVatio Technology Co., Ltd",
    "C0619A9": "Wingtech Mobile Communications  Co.,Ltd.",
    "28B77C7": "Convertertec Deutschland GmbH",
    "4C93A68": "Sercomm Corporation.",
    "4C93A60": "Vestaboard, Inc.",
    "4C93A6E": "CELLTRON",
    "4C93A67": "5Voxel Co., Ltd.",
    "28B77C4": "Annapurna labs",
    "4C93A66": "Shandong Senter Electronic Co., Ltd",
    "28B77C2": "Zhuhai RongBang Electronic Technology Co., Ltd.",
    "F469D59": "Terminus (Shanghai) Technology Co.,Ltd.",
    "5C857E2": "mobilogix HongKong",
    "5C857EC": "Annapurna labs",
    "3049506": "Curb, Inc.",
    "3049509": "Shanghai gatang technology CO.,LTD",
    "F0D7AFE": "Wren Associates, LTD",
    "F0D7AF1": "Beijing Serviatech lnformation Tech Co.,Ltd",
    "F0D7AF0": "ID Tech Japan Co.,Ltd.",
    "304950B": "HANGZHOU EV-TECH CO.,LTD",
    "CCC2617": "Ability Enterprise Co., Ltd",
    "CCC2614": "EDAG Engineering GmbH",
    "F041C8C": " Shanghai Think-Force Electronic Technology Co. Ltd",
    "7069793": "Hebei Baina Xinda Technology Co., Ltd.",
    "7069792": "Graphcore Ltd",
    "38F7CD1": "NZIA Connect Inc",
    "94FBA7C": "Solaborate Inc.",
    "94FBA76": "Sercomm Corporation.",
    "94FBA7B": "Shenzhen Golden Star Technology Ltd",
    "94FBA77": "Anvil Systems Group, Inc.",
    "38F7CDA": "Distech Controls",
    "E8B470C": "Anduril Industries",
    "C09BF4E": "Continental Automotive Component Malaysia Sdn.Bhd.",
    "E8B4703": "Webfleet Solutions B.V.",
    "E8B470D": "Medica Corporation",
    "94FBA75": "CAVITY EYE",
    "94FBA73": "GUANG DONG TAKSTAR ELECTRONIC CO.,LTD.",
    "C09BF49": "Alcatraz AI Inc.",
    "C09BF40": "Annapurna labs",
    "C09BF47": "Big Dutchman International GmbH",
    "F490CBE": "RSAE Labs Inc",
    "9405BB6": "ZIGPOS GmbH",
    "9405BBA": "SolarEdge Technologies",
    "F490CB6": "Airbeam Wireless Technologies Inc.",
    "9405BB5": "Chengdu Zhongheng Network Co.,Ltd.",
    "94CC04C": "Shanxi Baixin Information Technology Co., Ltd.",
    "94CC043": "Shenzhen Link technology Co.,Ltd",
    "94CC045": "SHENZHEN SANRAY TECHNOLOGY CO.,LTD",
    "94CC041": "GOCOAX, INC",
    "14AE859": "Veo Technologies",
    "14AE857": "SHENZHEN HONOR ELECTRONIC CO.,LTD",
    "90E2FCB": "Shenzhen Dingsheng Intelligent Technology Co., Ltd",
    "3CFAD3C": "Shenzhen zhong ju  Fiber optical Co.Ltd",
    "90E2FC9": "Huddly AS",
    "3CFAD32": "Naruida Technology Ltd.",
    "50DE19C": "Shenzhen Vipstech Co., Ltd",
    "402C762": "Annapurna labs",
    "50DE190": "Telic AG",
    "200A0D7": "Tecnint HTE SRL",
    "402C76A": "NowTechnologies Zrt",
    "402C765": "Baumer Bourdon-Haenni",
    "200A0DA": "IRSAP",
    "A0224EA": "IST ElektronikgesmbH",
    "006967E": "Tianjin Lianwu Technology Co., Ltd. ",
    "0069679": "Hangzhou Wise IOT Technology Co.,Ltd",
    "0069678": "Ambient-System sp. z o.o.",
    "006967B": "Datapan d.o.o.",
    "C4954D3": "Sercomm Corporation.",
    "C4954DC": "SolidGear Corporation",
    "6431396": "Hunan Voc Acoustics Technology Co., Ltd.",
    "6431393": "KOANGYOW INTEGRATION MACHINE CO., LTD.",
    "643139C": "SHEN ZHEN FUCHANG TECHNOLOGY Co.,Ltd.",
    "6431399": "Honeywell Analytics Ltd",
    "506255C": "AED Distribution",
    "5062555": "Suzhou Ruixinjie Information Technology Co.,Ltd",
    "5062553": "Hypertech Advance Co., LTD",
    "5062557": "AVTECH Software, Inc.",
    "440377B": "Hangzhou Asia Infrastructure Tech. Co., Ltd.",
    "04D16EC": "PacPort Corporation",
    "244E7BD": "Church & Dwight Co., Inc.",
    "04D16E1": "Launch Tech Co., Ltd.",
    "04D16E7": "Envision Energy",
    "04D16E4": "ShenZhen Huafu Information technology Co.?Ltd",
    "04D16E8": "CHENGDU INTERLINK SCIENCE AND TECHNOLOGY CO.,LTD",
    "4403777": "Stara S/A Ind\u00fastria de Implementos Agr\u00edcolas",
    "4011758": "Beijing Gemotech Intelligent Technology Co., Ltd.",
    "4011759": "ADH Guardian USA",
    "4C4BF94": "Shenzhen dingsheng technology co., LTD",
    "4C4BF9C": "Connected IO",
    "4C4BF98": "Zivid AS",
    "8439BE2": "Cheng Du virtual world Technology Limited.",
    "4C4BF91": "Jiangsu acrel Co., Ltd.",
    "4C4BF97": "GLONEXS",
    "208593E": "Dynaudio",
    "2085930": "Hemina Spa",
    "208593C": "Regloplas AG",
    "4011752": "Kanda Kogyo",
    "4C4BF9E": "Beijing AutoAi Technology co. LTD",
    "2085934": "Kloudspot Inc",
    "2085931": "Networking Services Corp",
    "4C4BF95": "Remedee Labs",
    "241510C": "Shenzhen Xtooltech Co., Ltd",
    "241510E": "Satellite Link Technology CO.,LTD",
    "9806377": "SAMWONTECH",
    "980637C": "HwaCom Systems Inc.",
    "2415101": "SMaBiT GmbH",
    "241510A": "Unitronux(Shenzhen) Intelligence Technology Co.,Ltd",
    "980637D": "VR Technology(Shenzhen) Limited",
    "9806379": "NAB co,.LTD",
    "2415100": "Safetrust Inc",
    "9806373": "Hangzhou Sanxin Network Technology Co.,Ltd",
    "980637B": "Petersime",
    "D05F649": "Shanghai Luying International Trade Co.,Ltd",
    "D05F645": "Atoll Solutions Private Limited",
    "D05F642": "SHANGHAI ZHONGMI COMMUNICATION TECHNOLOGY CO.,LTD",
    "D05F643": "HUAQIN TELECOM HONG KONG LTD",
    "44D5F28": "CETC Avionics.L td",
    "44D5F23": "VURO LLC",
    "D05F64B": "North American Blue Tiger Company, LLC",
    "44D5F21": "SIMPLERED TECHNOLOGY LTD.",
    "44D5F24": "APPOTRONICS CO., LTD",
    "44D5F20": "TIBA Research & Development (1986) LTD",
    "FCA47A0": "Broadcom Inc.",
    "FCA47AB": "Shenzhen Nokelock Technology Co, Ltd.",
    "2C16BD4": "Sunit Oy",
    "FCA47A6": "Token",
    "8C593CE": "Shenzhen Tian-Power Technology Co.,Ltd.",
    "B4A2EB7": "Kona I",
    "8C593CB": "Scharfe-Sicht GmbH",
    "8C593CA": "ecom instruments GmbH",
    "B4A2EB0": "QKM Technology(Dongguan)Co.,Ltd",
    "D0C8573": "Mobicon",
    "D0C857D": "IFLYTEK CO.,LTD.",
    "D0C8577": "Eco Mobile ",
    "6095CEC": "Synamedia",
    "6095CE4": "Untangle, Inc.",
    "1C82596": "CGI IT UK LIMITED",
    "1C8259A": "ESTec Corporation",
    "848BCD0": "SouXin Corporate",
    "C82C2B8": "Verifone Systems (China),lnc.",
    "B0FD0BE": "Shenzhen FEIBIT Electronic Technology Co.,LTD",
    "C82C2B2": "Repp Health",
    "C82C2BC": "Smart Wires Inc",
    "B0FD0B1": "IDspire Corporation Ltd.",
    "C82C2BE": "Fr\u00e4nkische Rohrwerke Gebr. Kirchner GmbH & Co. KG",
    "C86314E": "Taylor Dynamometer",
    "C86314C": "Freeus LLC",
    "C863140": "Western Reserve Controls, Inc.",
    "E41E0AA": "FireAngel Safety Technology Ltd",
    "C863145": "Meyer Electronics Limited",
    "E41E0A3": "Avast Software s.r.o.",
    "FCD2B6C": "Silicon (Shenzhen) Electronic Technology Co.,Ltd.",
    "FCD2B6A": "NREAL TECHNOLOGY LIMITED",
    "34E1D14": "ASA Innovation & Technology Ltd.",
    "34E1D16": "Ningbo Hua Gao Mdt Info Tech Ltd",
    "34E1D13": "Rinco Ultrasonics AG",
    "FCD2B6E": "Univer S.p.A.",
    "34E1D17": "Genius Pros",
    "34E1D19": "Biamp",
    "745BC59": "Haikou Frun Flash&Mcu Microcontrol Technology Development Co.,Ltd",
    "745BC5B": "Smartiply Inc.",
    "745BC53": "OXON AG",
    "E44CC77": "Channel Enterprises (HK) Ltd.",
    "4CBC98B": "Dongguan SmartAction Technology Co.,Ltd",
    "E44CC76": "HANGZHOU OLE-SYSTEMS CO., LTD",
    "4CBC985": "Gronic Systems GmbH",
    "4CBC982": "Quake Global Inc",
    "4CBC983": "Machine Max",
    "E44CC75": "CE LABS, LLC",
    "38B19E6": "Thrust Networks",
    "E05A9FB": "Shenzhen Rongan Networks Technology Co.,Ltd",
    "E05A9F3": "Link of Things Co., Ltd.",
    "D8860BA": "GLO Science",
    "D8860B9": "DIGITAL CONCEPTS",
    "E05A9F4": "Hale Products",
    "D8860B6": "SCANMATIK",
    "D8860B4": "Teplovodokhran Ltd.",
    "38B19E3": "AVO DEVELOPMENT LTD",
    "38B19ED": "Dallas Delta Corporation",
    "8439BE0": "HINO ENGINEERING, INC",
    "CCD39D7": "Glenair",
    "D425CCE": "Coperion",
    "D425CCA": "E-MetroTel",
    "D425CC1": "Eware Information Technology com.,Ltd",
    "CCD39D3": "MagTarget LLC",
    "D425CC0": "NORDI TELEKOMMUNIKATSIOONI O\u00dc",
    "D425CC3": "EISST Ltd",
    "9C69B48": "Skydock do Brasil Ltda",
    "9C69B47": "PCI Limited",
    "9C69B49": "Teptron AB",
    "4C917A9": "Hangzhou Hangtu Technology Co.,Ltd.",
    "4C917A4": "LumiGrow Inc.",
    "4C917A8": "Camsat Przemys\u0142aw Gralak",
    "7CBC847": "Xuji Changnan Communication Equipment Co., Ltd.",
    "6CDFFB6": "AAON",
    "6CDFFBB": "CELL System Co.,Ltd.",
    "6CDFFB9": "YongTechs Electric Co. Ltd",
    "6CDFFB8": "Hardmeier",
    "7CBC84A": "OPNT BV",
    "7CBC846": "Soci\u00e9t\u00e9 de Transport de Montr\u00e9al",
    "7CBC841": "Xiamen Mage Information Technology Co.,Ltd.",
    "98F9C77": "ARIMA Communications Corp.",
    "0CFE5D3": "Beijing WayClouds Technology Co., Ltd.",
    "98F9C7A": "MSB Elektronik und Ger\u00e4tebau GmbH",
    "6C5C3DB": "Reconova Technologies",
    "6C5C3DD": "Syowatsusinkougyo Co.,Ltd.",
    "6C5C3DA": "krtkl inc.",
    "6C5C3DC": "choyang powertech",
    "1CFD086": "A&B Technology",
    "6C5C3D7": "SOUNDKING ELECTRONICS&SOUND CO., LTD.",
    "A83FA1B": "Exel s.r.l. unipersonale",
    "A83FA13": "Guangzhou Tupu Internet Technology Co., Ltd.",
    "A83FA1D": "Shenzhen BIO I/E Co.,Ltd",
    "3C6A2C5": "Qingdao iGuan Technology Co., Ltd.",
    "3C6A2C2": "Bosch Automotive Products (Suzhou) Co., Ltd.",
    "3C6A2CB": "Phytium Technology Co., Ltd.",
    "A83FA14": "Zhejiang Wellsun Intelligent Technology Co.,Ltd.",
    "1CFD085": "Beijing Hengxin Rainbow Information Technology Co.,Ltd",
    "A4ED43E": "TOEC TECHNOLOGY CO.\uff0cLTD.",
    "A4ED433": "Dongguan Mingji Electronics technology Group Co., Ltd.",
    "A4ED43C": "leakSMART",
    "300A607": "Newtons4th Ltd",
    "A4ED431": "INGELABS S.L.",
    "A02833C": "Kalray S.A.",
    "8489EC7": "BYDA Co. Ltd., ",
    "3009F94": "Punkt Tronics AG",
    "8489ECC": "SHINKAWA LTD.",
    "C083596": "Beijing Cloud Fly Technology Development Co.Ltd",
    "9CF6DD7": "KXT Technology Co., Ltd.",
    "9CF6DD4": "Capital Engineering & Research Incorporation Ltd.",
    "C08359D": "Gardner Denver Thomas GmbH",
    "9CF6DDA": "AVI Pty Ltd",
    "04C3E6D": "Amiosec Ltd",
    "04C3E64": "Innovusion Inc.",
    "04C3E68": "SLOC GmbH",
    "3C427EA": "snap40 Ltd",
    "D47C448": "Beijing Maystar Information Technology Co., Ltd.",
    "B44BD61": "SHENZHEN TITA INTERACTIVE TECHNOLOGY CO.,LTD",
    "D47C445": "LS Communication Co.,Ltd.",
    "3C427ED": "ROBOX SMART MOTION (WUHU) CO.,LTD",
    "A019B28": "MIS Industrie Systeme GmbH & Co. KG",
    "A019B22": "Beijing Deephi Intelligent Technology Co., Ltd",
    "D47C446": "ASDA ICT Co., Ltd.",
    "A019B2E": "Ahgora Sistemas SA",
    "2C48355": "Scout Security, Inc.",
    "2C48351": "Advanced Electronics Company Ltd",
    "2C48353": "Newtrax Technologies Inc",
    "8C1CDA5": "Septentrio NV",
    "8C1CDAD": "Riegl Laser Measurement Systems GmbH",
    "8C1CDA7": "K Technology Corporation",
    "3C24F08": "Sivat Technology Co.,Ltd.",
    "3C24F09": "Siemens AG - Siemens Deutschland Mobility",
    "8C1CDA0": "CEOS Pty Ltd",
    "0C73EB3": "Tiinlab Acoustic Technology (Shenzhen) Co., Ltd.",
    "0C73EBD": "D-Link \uff08Shanghai\uff09Limited Corp.",
    "0C73EBB": "Synaccess Networks",
    "0C73EB2": "Deltapath, Inc.",
    "0C73EBA": "Dana",
    "0C73EB7": "Dinkle Enterprise Co., Ltd.",
    "0C73EBE": "Taiwan Pulse Motion Co., Ltd.",
    "4865EE8": "SmartDisplayer Technology Co., Ltd.",
    "3C24F06": "Inter Action Corporation",
    "3C24F0B": "COMATIS",
    "480BB26": "Annapurna labs",
    "480BB2B": "Popit Oy",
    "885FE89": "Sowee",
    "885FE87": "Red Technologies, LLC.",
    "301F9A3": "MICOMSOFT CO.,LTD.",
    "301F9AA": "HUNAN CHANGSHA HENGJIAN TECHNOLDGY DEVELPMENT CO.,LTD.",
    "885FE83": "Sonnet Labs Inc.",
    "40A36B4": "SKS-Kinkel Elektronik GmbH",
    "885FE85": "Hauch & Bach ApS",
    "F041C8E": "Shenzhen Umind Technology Co., Ltd.",
    "F041C82": "Shenzhen Medica Technology Development Co., Ltd.",
    "88A9A76": "Sieper L\u00fcdenscheid GmbH & Co. KG",
    "A4DA228": "SolidPro Technology Corporation",
    "DCE5337": "SAN Engineering",
    "DCE5333": "ShenZhen C&D Electronics CO.Ltd.",
    "9C431E1": "Symfun Telecom Ltd",
    "C4FFBCB": "KAGA ELECTRONICS CO.,LTD.",
    "9C431E4": "Wireless Environment, LLC",
    "9C431E7": "Optris GmbH",
    "282C022": "Shenzhen emb-star technology co. LTD",
    "4048FD6": "Swarco Technology ApS",
    "F8B568B": "Whizpace Pte. Ltd.",
    "F8B5688": "Maven Wireless AB",
    "4048FDC": "Ecotap B.V.",
    "4048FD2": "MITHRAS Technology Co., LTD",
    "3873EA4": "Light Blue Optics Ltd.",
    "3873EA5": "ISTCONTROL",
    "EC9F0DB": "CRRC QINGDAO SIFANG ROLLING STOCK RESEARCH INSTITUTE CO.,LTD",
    "EC9F0DA": "flexlog GmbH",
    "34D0B89": "Skytech Creations Limited",
    "34D0B8C": "Glory Mark Electronic Ltd. Taiwan Branch (B.V.I.)",
    "34D0B8D": "NTX Embedded",
    "34D0B84": "EQPlay Intelligent Technology(Kunshan) Co,Ltd.",
    "AC1DDF8": "Sichuan Odot Automation System Co.,Ltd.",
    "AC1DDF9": "Solare Datensysteme GmbH",
    "AC1DDF2": "ConectaIP Tecnologia S.L.",
    "741AE0E": "ITS Partner (O.B.S) S.L.",
    "741AE08": "Broadcast Wireless Systems Ltd",
    "741AE07": "B\u00c4R Bahnsicherung AG",
    "CC2237E": "MANUFACTURAS Y TRANSFORMADOS AB, S.L.",
    "CC2237B": "Tolomatic, Inc.",
    "CC22372": "Apeiron Data Systems",
    "2C279EE": "Amaryllo International Inc.",
    "CC2237D": "SHENZHEN HOOENERGY TECHNOLOGY CO.,LTD",
    "04714B2": "Shenzhen WayOS Technology Crop., Ltd.",
    "189BA58": "Shenzhen Tong Tai Yi information Technology Co.,Ltd",
    "8439BE3": "ShenZhen Fudeyu Technology co.,Ltd",
    "904E91D": "SKODA ELECTRIC a.s.",
    "2C279E9": "octoScope, Inc.",
    "2C279E4": "Shijiazhuang King Transportation Equipment Co.,Ltd",
    "904E914": "Wrtnode technology Inc.",
    "2C279E8": "Institut Dr. Foerster GmbH & Co. KG",
    "34298F3": "Beijing Vorx Telecommunications Co., Ltd.",
    "34298F1": "Chengdu Meross Technology Co., Ltd.",
    "34008AE": "SHENZHEN WXL ELECTRONICS CO., LTD.",
    "34008AB": "Project Engineering srl",
    "34298F6": "Bellman & Symfon",
    "34298FB": "Schnick-Schnack-Systems GmbH",
    "34298FA": "Virtual Trunk Pte Ltd",
    "34298F8": "Nanjing Sandemarine Electric Co.,Ltd",
    "189BA53": "PHINETWORKS",
    "189BA59": "APANA Inc.",
    "34008A0": "Angee Technologies Ltd.",
    "7CBACCC": "Flying Loft Inc.",
    "78D800C": "Shenzhen Chenzhuo Technology Co., Ltd.",
    "78D8008": "Salunda Ltd",
    "78D8001": "Shenzhen Envicool Information Technology Co., Ltd",
    "78D8003": "Shenzhen Scodeno Technology Co,. Ltd.",
    "7CBACCE": "ALPHA TECHNOLOGIES, LLC",
    "4C65A8E": "High Infinity Germany",
    "4C65A86": "Nuviz Oy",
    "F88A3C0": "ART SPA",
    "F88A3CB": "FARA AS",
    "40A36B8": "SFT Co., Ltd.",
    "F88A3CE": "Avateq Corp.",
    "A0C5F2D": "UnaliWear, Inc.",
    "A0C5F29": "Impulse Networks Pte Ltd",
    "A0C5F28": "CoolR Group Inc",
    "4C65A88": "Instant Byte, S.L.",
    "4C65A82": "Orica Europe Pty Ltd & Co KG",
    "A0C5F2C": "Glooko inc",
    "F023B96": "Xiamen Jinhaode Electronic Co.,Ltd",
    "F023B90": "Aquametro AG",
    "F023B98": "G3 TECHNOLOGIES< INC",
    "04714BE": "Gimso Mobile Ltd",
    "8C147D4": "Nanjing bilian information Technology Co.,Ltd.",
    "8C147DA": "Bluemega Document & Print Services",
    "50FF994": "IPC Global",
    "F023B94": "EZVIS LIMITED",
    "60D7E30": "Avalun",
    "60D7E3A": "Wilderness Labs Inc.",
    "60D7E35": "Revol Technologies inc",
    "60D7E33": "SKS Automaatio oy",
    "98AAFCE": "Comarch S.A. ",
    "98AAFCC": "dots Inc.",
    "98AAFC9": "BEAM Authentic",
    "08ED021": "Imperx, Inc",
    "08ED020": "D2SLink Systems",
    "144FD70": "Annapurna labs",
    "144FD78": "NPort Networks Inc.,",
    "144FD7C": "D&S Cable Industries (HK) Limited",
    "144FD72": "FedEx Services OTI",
    "F802787": "BETTINI SRL",
    "144FD71": "Zehnder Group AG",
    "A411638": "Dspread Technology (Beijing) Inc.",
    "A411633": "Pax",
    "1CA0D35": "Dynamic Connect (Suzhou) Hi-Tech Electronic Co.,Ltd.",
    "1CA0D36": "Intertecno SRL \"NISUTA\"",
    "40F385E": "BBB Inc.",
    "1CA0D34": "NPO TELECOM JSC",
    "1CA0D33": "SAVELEC",
    "40F3855": "KATO ENGINEERING INC.",
    "8CC8F44": "ITECH Electronic Co.,ltd.",
    "34049EE": "ND SatCom GmbH",
    "34049E5": "Seeiner Technology Co.,LTD",
    "34049E3": "Nanjing Mythware Information Technology Co., Ltd.",
    "A4580FE": "Finetree Communications Inc",
    "7CCBE2B": "Easy Broadband Technology Co., Ltd.",
    "7CCBE27": "Hangzhou Kaicom Communication Co.,Ltd",
    "7CCBE23": "Astrum Technologies CC",
    "500B91D": "Shenzhen Lucky Sonics Co .,Ltd",
    "A4580F5": "CoAsia Microelectronics Corp.",
    "500B91E": "Shenzhen zhong ju  Fiber optical Co.Ltd",
    "A4580F4": "Shenzhen City billion Leiden science and Technology Co., Ltd.",
    "500B91C": "Diamond Traffic Products, Inc",
    "1CC0E14": "Videri Inc.",
    "7CCBE20": "Heyuan Yongyida Technology Holdings Co.,Ltd.",
    "244E7BA": "Shenzhen AWT science &  technology limited",
    "4865EE1": "Gopod Group Limited",
    "4865EEB": "EnBW Energie Baden-W\u00fcrttemberg AG",
    "244E7B0": "Tekelek Europe Ltd",
    "4865EED": "Winn Technology Co.,Ltd",
    "244E7B9": "UniMAT Automation Technology Co., Ltd.",
    "7CCBE21": "CeoTronics AG",
    "4CE1734": "Huizhou Dehong Technology Co., Ltd.",
    "1CC0E13": "HANGZHOU SOFTEL OPTIC CO., LTD",
    "4CE1736": "DAIKOKU DENKI CO.,LTD.",
    "4CE1738": "Nanjing Tongke Technology Development Co., LTD",
    "1CC0E1A": "SECHERON SA",
    "4865EE3": "Data Technology Inc.",
    "4CE173B": "Shanghai Ehong Technology Co.,Ltd",
    "4CE1735": "NewVastek",
    "383A217": "Chengdu Krosslan Technology Inc.",
    "383A219": "Skylark Wireless LLC",
    "383A21E": "SDNware technology co.,LTD",
    "AC64DDE": "DIGIBIRD TECHNOLOGY CO., LTD.",
    "AC64DD3": "infypower Co., Ltd",
    "383A21B": "Pactron",
    "70F8E76": "Flexim Security Oy",
    "F81D781": "ADTECHNO Inc.",
    "F81D78C": "SHENZHUOYUE TECHNOLOGY.,LTD",
    "70F8E71": "System Level Solutions (India) Pvt.",
    "70F8E79": "Kontech Electronics Co., Ltd",
    "70F8E70": "SHENZHEN Xin JiuNing Electronics Co Ltd",
    "70F8E78": "Eclipse Security",
    "2C48350": "Progress Rail Services, Inspection and Information Systems",
    "4048FD7": "Cloud4Wi",
    "84E0F46": "Liaoning IK'SONYA Science and Technology Co., Ltd.",
    "84E0F41": "MedicusTek Inc.",
    "84E0F4A": "iSolution Technologies Co.,Ltd.",
    "84E0F47": "Dantherm",
    "84E0F48": "RAY Co.,LTD",
    "C0D391D": "REGULUS CO.,LTD.",
    "C0D3918": "XENA SECURITY LIMITED",
    "C0D3912": "Hofon Automation Co.,Ltd",
    "986D354": "blossom communications corp.",
    "C0D3913": "IXON B.V.",
    "58E8768": "Chengdu Vision-Zenith Technology Co.,Ltd",
    "58E876C": "KUSTOM SIGNALS INC",
    "58E876A": "SHENZHEN DIGISSIN TECHNOLOGY",
    "F0ACD74": "Sercomm Corporation.",
    "F0ACD77": "Hanju Network Technologies Co.",
    "F0ACD78": "Telefonix Incorporated",
    "F0ACD76": "Suzhou Pairlink Network Technology",
    "F0ACD75": "PAVO TASARIM URETIM TICARET A.S.",
    "2836380": "Knowles Electronics LLC",
    "283638D": "APPEAK Technology System Co.Ltd.",
    "2836384": "Dspread Technology (Beijing) Inc.",
    "283638A": "Bluekey Pty Ltd",
    "8C192D4": "Charmlink Tech(HK) Co.,Limited ",
    "8C192D7": "SRETT",
    "8C192D8": "Shenzhen Cylan Technology Co.,Ltd",
    "8C192DB": "Abside Networks, Inc.",
    "8C192D9": "ViaWear, Inc.",
    "8C192DC": "You Zhengcheng co.,ltd",
    "CCD31E9": "Siemens AG, MO MLT BG",
    "D0D94FD": "DUKSANMECASYS CO., LTD.",
    "C47C8DB": "GC AUTOMATION CO,LTD",
    "6891D0E": "Outstanding Technology Co., Ltd.",
    "C47C8DA": "Silvus technologies inc",
    "C47C8D4": "ROBOSTAR",
    "C47C8D8": "GETEMED Medizin- und Informationstechnik AG",
    "C47C8D3": "Watec Co., Ltd.",
    "6891D05": "NIPK Electron Co.",
    "E0B6F5C": "funktel GmbH",
    "6891D01": "Multi Alarm Zrt.",
    "6891D09": "QUANTEX",
    "E0B6F56": "POMCube Inc. ",
    "E0B6F58": "Yuneec International\uff08China\uff09Co.\uff0cLtd",
    "50FF992": " SHENZHEN KINGVT ELECTRONICS CO.,LTD",
    "E0B6F53": "Huizhou GISUN Industrial CO. LTD",
    "986D356": "Vitronic Dr.-Ing. Stein Bildverarbeitungssysteme GmbH",
    "986D35D": "Praesideo B.V.",
    "50FF997": "Honeywell International",
    "986D357": "Zhejiang Hanshow Technology Co., Ltd.",
    "7C477C6": "Zerosystem LTD.Co",
    "7C477C5": "Midwest Microwave Solutions",
    "5CF286E": "Daisen Electronic Industrial Co., Ltd.",
    "7C477C0": "BungBungame Inc",
    "7C477C1": "Photosynth Inc.",
    "7C477C8": "Shenzhen Eunicum Electric Co.,Ltd.",
    "5CF286A": "Unfors Raysafe AB",
    "38B8EB8": "CeeNex Inc",
    "38FDFE4": "New Telecom Solutions LLC",
    "38FDFE5": "CaptiveAire Systems Inc.",
    "78CA837": "Beijing CarePulse Electronic Technology ",
    "78CA83C": "Elanview Technology Co.,Ltd",
    "78CA838": "IHM",
    "78CA83B": "Zhejiang Science Electronic Tech Co., Ltd",
    "1C87745": "Xiaoxinge (Tangshan) Electronic Technology Co., Ltd.",
    "1C8774C": "New Nordic Engineering",
    "1C88798": "Toshiba Toko Meter Systems Co., LTD.",
    "1C87744": "Weber Marking Systems GmbH",
    "1C8774D": "CLABER SPA",
    "1C87799": "Istria soluciones de criptografia, S. A.",
    "1C87747": "Ing Buero Ziegler",
    "8439BE8": "Diamond Products LLC",
    "40A36BB": "Amobile Intelligent Corp.",
    "1C87791": "A-GEAR COMPANY LIMITED ",
    "1C8779A": "Hangzhou Xiaowen Intelligent Technology Co., Ltd.",
    "1C87798": "ZHEJIANG ITENAL TECHNOLOGY CO.,LTD",
    "8439BEB": "Shenzhen Horn Audio Co.,Ltd.",
    "40A36B7": "Pella Corporation",
    "40A36B0": "Fin Robotics Inc",
    "40A36B3": "Omnitracs, LLC",
    "800A806": "Beijing Gooagoo Technical Service Co.,Ltd.",
    "800A805": "Shenzhen Zidoo Technology Co., Ltd.",
    "A03E6BE": "Nanjing zhanyi software technology co., LTD",
    "0055DAC": "Donguan WideLink Communication Technology Co.,Ltd.",
    "0055DA7": "LUCISTECHNOLOGIES\uff08SHANGHAI\uff09CO.,LTD",
    "0055DA3": "Novexx Solutions GmbH",
    "CC1BE05": "Earphone Connection, Ubc.",
    "DC4427B": "Nautilus Infotech CO., Ltd.",
    "DC4427C": "Pyrexx Technologies GmbH",
    "1C21D11": "Ognios GmbH",
    "1C21D1C": "Private",
    "C88ED11": "German Pipe GmbH",
    "A03E6B1": "Business Support Consultant Co.,Ltd",
    "DC4427A": "Shanghai Huahong Integrated Circuit Co.,Ltd",
    "DC44274": "Nex Technologies PTY LTD",
    "B0C5CA9": "D&T Inc.",
    "78C2C0A": "Ombitron, Inc.",
    "78C2C08": "Beijing Coilabs technology co.,ltd",
    "78C2C0D": "KORF Inc.",
    "78C2C07": "Guangzhou Hongcai Stage Equipment co.,ltd",
    "B437D16": "Yireh Auto Tech Co.,Ltd.",
    "B437D10": "Lezyne INC USA",
    "74F8DB7": "Wuhan Tianyu Information Industry Co., Ltd.",
    "74F8DB4": "WiFi Hotspots, SL",
    "74F8DB2": "Shenzhen Ruishi Information Technology Co.,Ltd.",
    "B437D17": "GE Power Management",
    "549A114": "eTauro LLC",
    "549A113": "Royal Boon Edam International BV",
    "549A110": "Shenzhen Excera Technology Co.,Ltd.",
    "64FB81E": "ChengDu KeChuang LongXin Sci-tech Co.,Ltd",
    "64FB819": "hiQview Corporation",
    "549A117": "Niveo International BV",
    "807B85A": "Interplan Co., Ltd.",
    "64FB81D": "Dongyang unitech.co.ltd",
    "64FB81C": "Bridgeport Instruments, LLC",
    "549A11E": "Beijing HTSmartech Co.,Ltd",
    "807B856": "Quickte Technology Co.,Ltd",
    "80E4DA4": "Beijing Yuantel Technolgy Co.,Ltd-Shenzhen Branch",
    "80E4DA3": "Beijing Gaokezhongtian Technology Co Ltd",
    "64FB812": "Seven Solutions S.L",
    "1CCAE3D": "eSight Corporation",
    "80E4DA8": "Krizer international Co,. Ltd.",
    "80E4DAB": "Nanjing LILO Technology Co. Ltd.",
    "80E4DAC": "EVER Sp. z o.o.",
    "90C6828": "Teletek Electronics",
    "90C6826": "Nanjing Jiexi Technologies Co., Ltd.",
    "2CD1417": "XiaMen 35.com Technology Co,.Ltd.",
    "90C6827": "Cinet Inc",
    "90C6823": "Innovative Electronic Technology",
    "2C6A6F0": "Shanghai Shuncom Electronic Technology Co.,Ltd",
    "2CD1416": "Bowei Technology Company Limited",
    "90C682E": "Shanghai HuRong Communication Technology Development Co., Ltd.",
    "2C6A6FA": "Wellntel, Inc.",
    "A0BB3E8": "AutarcTech GmbH",
    "A0BB3E9": "Sandal Plc",
    "28FD804": "Digital Signal Corp",
    "2C265FE": "Hysentel Technology Co., Ltd",
    "A0BB3E6": "Xiamen Kehua Hengsheng Co.,Ltd",
    "2C265F5": "Motec GmbH",
    "F802789": "Beijing Redcdn Technology, Co., Ltd",
    "0CEFAF4": "Sentry360",
    "F802782": "Innodisk",
    "3C39E74": "University of British Columbia",
    "1007230": "RippleTek Tech Ltd",
    "A44F293": "Comsel System Ltd",
    "E818631": "clabsys",
    "B8D812D": "Lam Research",
    "B8D812C": "Yuwei Info&Tech Development Co.,Ltd",
    "B8D8121": "VOTEM",
    "E818637": "Siliconcube",
    "D022128": "Shenzhen SIC Technology. Co., Ltd.",
    "D022126": "URANO INDUSTRIA DE BALANCAS E EQUIPAMENTOS LTDA",
    "B8D8126": "Vonger Electronic Technology Co.,Ltd.",
    "E818639": "BSM Wireless Inc.",
    "D022122": "RHENAC Systems GmbH",
    "D022121": "AIM",
    "E4956E4": "Guang Lian Zhi Tong Technology Limited",
    "E4956E1": "Tband srl",
    "BC6641A": "EBlink",
    "BC66418": "Shenzhen Yaguang communication CO.,LTD",
    "E4956E7": "NationalchipKorea",
    "E4956E3": "Shanghai DGE Co., Ltd",
    "74E14A3": "emz-Hanauer GmbH & Co. KGaA",
    "74E14A4": "open joint stock company \"YUG-SISTEMA plus\"",
    "E4956EE": "Tacom Projetos Bilhetagem Inteligente ltda",
    "BC66412": "Process-Electronic Sp. z o.o.",
    "58FCDBE": "Applied Device Technologies",
    "B01F81B": "Rademacher Geraete-Elektronik GmbH",
    "B01F814": "SHENZHEN YIFANG DIGITAL TECHNOLOGY CO.,LTD.",
    "58FCDB2": "Beseye Cloud Security Co. Ltd.",
    "58FCDBD": "XIAMEN LEELEN TECHNOLOGY CO.,LTD",
    "F40E119": "Sterna Security",
    "F40E112": "Axel srl",
    "141FBAB": "Newings Communication CO., LTD.",
    "7C70BC3": "FLEXIM GmbH",
    "F40E110": "realphone technology co.,ltd",
    "7C70BCB": "Tohan\u3000Engineering\u3000Corporation",
    "141FBA3": "Private",
    "D07650D": "tecnotron elekronik gmbh",
    "BC34006": "Cameron",
    "A43BFA9": "SHEN ZHEN PASUN TECH CO.LTD.",
    "A43BFA3": "Circus World Displays Ltd",
    "A43BFA1": "Beijing Uniwill Science and Technology Co,Ltd",
    "A43BFA0": "Chengdu Territory Technology Co.,Ltd",
    "BC3400A": "AURALIC LIMITED",
    "D076509": "Greenwave Scientific",
    "7419F87": "Heptagon Systems PTY. LTD.",
    "7419F8B": "IDEXX Laboratories, Inc",
    "D076500": "CentrAlert, Inc.",
    "7419F8C": "Bach Icon ApS",
    "C85CE25": "Cranns Limited",
    "C85CE23": "ECOCHIP Communication Technology(shenzhen)Co.Ltd.",
    "C85CE21": "Annapurna labs",
    "4CEA41C": "hogotech",
    "1CCAE37": "Bird Home Automation GmbH",
    "4CEA41A": "Vortex Infotech Private Limited",
    "4CEA414": "Eltroplan Engineering GmbH",
    "4CEA41E": "Aztech Group DOOEL",
    "F41A798": "Annapurna labs",
    "F41A79D": "Directed Electronics OE Pty Ltd",
    "F41A792": "HCTEK Co.,Ltd.",
    "5CF8385": "DesignA Electronics Limited",
    "48DA359": "Flextronics International Kft",
    "100648A": "Dreame Innovation Technology(Suzhou) Co.,Ltd.",
    "2CC44FB": "CAMS New Energy Technology Co., Ltd.",
    "1006484": "Beijing Cheering Networks Technology Co., Ltd.",
    "008DF4B": "Guangzhou Legendview Electronic Technology Co.,Ltd",
    "008DF46": "Annapurna labs",
    "008DF42": "Symetrics Industries d.b.a. Extant Aerospace",
    "008DF45": "Schneider Electric",
    "EC9A0C2": "Shenzhen Yitoa Digital Technology Co., Ltd.",
    "EC9A0C1": "Shenzhen Naxiang Technology Co., Ltd",
    "008DF4A": "Adel System srl",
    "C898DBB": "Shenzhen Hooolink Technology Co., LTD",
    "C898DB4": "Shenzhen IBD Intelligence Technology Co,.Ltd.",
    "78392D0": "Neuron GmbH",
    "C898DB7": "J&R Technology Limited",
    "C898DBD": "Quilt Systems, Inc",
    "78392DC": "Jiangsu Yibang New Energy Technology Co., LTD",
    "8C55709": "Neptronic Ltd",
    "8C55707": "Eideal Company Limited",
    "8C5570A": "Fortune Brands Innovations, Inc.",
    "8C55703": "ScandiNova Systems",
    "8C5570E": "Johnson Health Tech. (Shanghai) Co.,Ltd.",
    "8C55701": "LLC Katusha Print",
    "28F8C67": "ASES GROUP, s.r.o.",
    "28F8C60": "2iC-Care Ltd",
    "28F8C6E": "COMEM SpA",
    "28F8C68": "Shenzhen Xifanlina Industrial Co.,Ltd",
    "28F8C69": "secuever",
    "E8FF1E2": "Minami Medical Technology (Guangdong) Co.,Ltd",
    "E8FF1EC": "Fracarro Radioindustrie Srl",
    "E8FF1E3": "LLP \"NIC \"FORS\"",
    "8CA682D": "EFit partners",
    "8CA6822": "Lion Energy",
    "D0A011D": "REMOWIRELESS COMMUNICATION INTERNATIONAL CO.,LIMITED",
    "D0A011B": "TYRI Sweden AB",
    "D0A0112": "Shanghai Hongqia Industry and Trade Co., LTD",
    "78E996C": "GARANTIR TECHNOLOGIES PRIVATE LIMITED",
    "78E9969": "ATM SOLUTIONS",
    "78E9963": "ACL Co.,Ltd.",
    "78E9966": "Kilews",
    "78E9962": "STIEBEL ELTRON GMBH & CO. KG",
    "34C8D6B": "Yuanzhou Intelligent (Shenzhen) Co., Ltd.",
    "80A5797": "Siemens Energy Global GmbH & Co. KG",
    "80A5791": "Zhe Jiang EV-Tech Co.,Ltd",
    "08F80DD": "Zhe Jiang EV-Tech Co.,Ltd",
    "80A5798": "DI3 INFOTECH LLP",
    "80A5799": "Yovil Ltd.",
    "9015646": "final Inc.",
    "901564B": "Shanghai AMP&MOONS'Automation Co.,Ltd.",
    "9015644": "Kontakt Micro-Location Sp z o.o.",
    "901564E": "Guiyag Electrlcal Control Equipment Co.,Ltd",
    "6879129": "LEAPS s.r.o.",
    "0863324": "Beijing KELIBANG Information technology Co.,LTD",
    "48E6630": "Huaqian Beijing Technology Co., Ltd",
    "687912C": "Globus Infocom Limited",
    "48E663A": "Shenzhen Jointelli Technologies Co.,Ltd ",
    "C4CC374": "Pineberry Pi ",
    "C4CC372": "CIX TECHNOLOGY INC.",
    "60A434A": "Scancom",
    "C8FFBF5": "Chongqing Zhizhu Huaxin Technology Co.,Ltd",
    "C8FFBF8": "Shenzhen Fengrunda Technology Co.,Ltd",
    "8CA6827": "Anhui seeker electronic technology Co.,LTD",
    "C8FFBF3": "TECTOY S.A",
    "C8FFBF1": "robert juliat",
    "1063A3C": "FLAT-PRO LLC",
    "5C87D80": "Nanjing Shufan Information Technology Co., Ltd.",
    "1063A34": "GANTECH E TECHNOLOGIES PRIVATE LIMITED",
    "1063A33": "Nextvision Stabilized Systems LTD",
    "1063A37": "NRS Co., Ltd.",
    "5C87D88": "fmad engineering",
    "5C5A4C2": "ITS Partner (O.B.S) S.L.",
    "F87A398": "Beijing Zhongyuan Yishang Technology Co.,LTD",
    "EC5BCD3": "Hefei BOE Vision-electronic Technology Co.,Ltd.",
    "F87A39E": "Cognosos, Inc.",
    "EC5BCD7": "Annapurna labs",
    "344663A": "Shenzhen Shenhong Communication Technology Co., Ltd",
    "EC5BCDE": "Autel Robotics USA LLC",
    "EC5BCD8": "Doosan Bobcat North America",
    "3446637": "SHENZHEN HTFUTURE CO., LTD",
    "3446632": "Amcrest Technologies",
    "08DA335": "Shengqing Acoustics LLC",
    "08DA334": "Cleverfox Equipments Private Limited",
    "08DA331": "TransTera Technology (HK) Limited",
    "48E6C61": "DEICO MUH. A.S.",
    "48E6C63": "Varjo Technologies Oy",
    "48E6C60": "Shenzhen Oumeihua Technology Co.,Ltd",
    "48E6C69": "Takiguchi Corporation",
    "48E6C65": "Annapurna labs",
    "48E6C66": "Moff Inc.",
    "F42055A": "Shenzhen C & D Electronics Co., Ltd.",
    "2486250": "Shanghai Xizhi Technology Co., Ltd.",
    "248625C": "L-LIGHT Co., Ltd.",
    "F420556": "Shenzhen GuangXu Electronic Technology Co.,Ltd.",
    "2486257": "Tianjin Optical Electrical Juneng Communication Co.,Ltd.",
    "2486259": "Vtron Pty Ltd",
    "AC86D12": "Hamsa India",
    "AC86D11": "Beijing Fantasy Mofang Technology Co.,Ltd",
    "AC86D16": "Adveco Technology Co., Ltd",
    "485E0ED": "Huaqin Technology Co.,Ltd",
    "485E0E2": "Shenzhen Shouchuang Micro Technology Co., Ltd",
    "485E0E6": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "ECA7B1C": "Hiromokuten Co.,Ltd.",
    "C4FF841": "Turing Machines Inc.",
    "ECA7B1D": "USU TELECOM INDIA PRIVATE LIMITED",
    "B84C87A": "Altronix , Corp",
    "8CC8F47": "TableSafe",
    "58E8760": "Zhuhai Raysharp Technology Co.,Ltd",
    "B84C872": "Shenzhen Huixiangfeng Electronic Technology Co., Ltd.",
    "B84C878": "Fujian Morefun Electronic Technology Co., Ltd.",
    "C022F1E": "Masimo Corporation",
    "48DA35B": "Vivid-Hosting, LLC.",
    "C022F18": "UtopiaTech Private Limited",
    "5491AF5": "Shenzhen IDSTE Information Technology Co., LTD",
    "5491AFD": "Ningbo Joynext Technology Corporation",
    "48DA356": "Shenzhen Sipeed Technology Co., Ltd",
    "5491AF0": "Opal-RT Technologies Inc.",
    "5491AF9": "Asiga Pty Ltd",
    "88A6EF9": "Kii Audio GmbH",
    "88A6EF4": "PT communication Systems Pvt LTD",
    "88A6EF1": "Shenzhen YAKO Automation Technology Co.,Ltd.",
    "D015BB6": "ShenZhen Zhongke GuanJie Data Technology Co.,Ltd.",
    "D015BB1": "Jiangsu Eastone Technology Co.,Ltd",
    "50482C8": "Dongguan Amdolla Electric & Light Material Manufacture Co., Ltd",
    "50482CB": "SL Process",
    "68DA730": "Annapurna labs",
    "68DA737": "Haven Lighting",
    "68DA735": "Shenzhen Xin hang xian Electronics Co., LTD",
    "58C41E2": "Truesense Srl",
    "58C41E8": "Xiaomi EV Technology Co., Ltd.",
    "C86BBCC": "ZEUS",
    "C86BBCE": "Waterkotte GmbH",
    "C86BBC7": "Shenzhen smart-core technology co.,ltd.",
    "C86BBC4": "Liuzhou Zuo You Trade Co., Ltd.",
    "C86BBC3": "Antevia Networks",
    "B0FF722": "MVT Elektrik Sanayi ve Ticaret Limited Sirketi",
    "BC31987": "Zhejiang Delixi Electric Appliance Co., Ltd",
    "BC31982": "JSC Megapolis-telecom region",
    "BC31980": "THINKCAR TECH CO.,LTD.",
    "90F421B": "Jiangsu MSInfo Technology Co.,Ltd.",
    "8C147D1": "Private",
    "90F4216": "Wuxi Sunning Smart Devices Co.,Ltd",
    "B0FF72A": "Simple Things",
    "C08359C": "Private",
    "B0FF720": "ShenYang LeShun Technology Co.,Ltd",
    "6C2ADF8": "Beijing Yisheng Chuanqi Technology Co., Ltd.",
    "B44D439": "AD HOC DEVELOPMENTS S.L",
    "6C2ADFE": "WeatherFlow-Tempest, Inc",
    "C0FBF91": "LIXIL Corporation",
    "B44D433": "ETSME Technologies C0., Ltd.",
    "B44D43A": "UAV Navigation",
    "6C2ADF0": "Ademco Inc. dba ADI Global Distribution",
    "B44D432": "RG SOLUTIONS LTD",
    "B44D431": "iLine Microsystems S.L. (B20956751)",
    "B44D438": "ShenZhen Launch-Wonder Technology co., LTD",
    "6C2ADFB": "MOBA Mobile Automation AG",
    "6C2ADF4": "Zhejiang Eternal Automation Sci-Tec Co., Ltd",
    "7006927": "DCNET SOLUTIONS INDIA PVT LTD",
    "54083B4": "Toray Medical Company Limited",
    "CC1BE0F": "Private",
    "7006921": "Beijing Fortech Microsystems., Co., Ltd.",
    "54083B5": "shenzhen HAIOT technology co.,ltd",
    "54083B3": "Dhyan Networks and Technologies, Inc",
    "DC4427F": "Private",
    "9802D8F": "Private",
    "74E14AF": "Private",
    "1CCAE3F": "Private",
    "C88ED1F": "Private",
    "D02212F": "Private",
    "E03C1C3": "Dewetron GmbH",
    "E03C1C6": "GhinF Digital information technology (hangzhou) Co., Ltd",
    "E03C1CC": "Meferi Technologies Co.,Ltd.",
    "E03C1C4": "Earable Inc.",
    "E03C1C0": "Scangrip",
    "E03C1C7": "Tap Home, s.r.o.",
    "E03C1CE": "Annapurna labs",
    "D093957": "iSolution Technologies Co.,Ltd.",
    "D093955": "FungHwa i-Link Technology CO., LTD",
    "D016F05": "wuxi high information Security Technolog",
    "D016F08": "Shenzhen DOOGEE Hengtong Technology CO.,LTD",
    "D093959": "NINGBO SUNNY OPOTECH CO.,LTD",
    "0CEFAFF": "Private",
    "1C21D1F": "Private",
    "E03C1C2": "Hoplite Industries, Inc.",
    "D016F03": "Sofinet LLC",
    "D09395A": "Automatic Devices",
    "D09395D": "T-COM LLC",
    "D09395C": "BRICK4U GmbH",
    "5C6AECC": "Suzhou Huaqi Intelligent Technology Co., Ltd.",
    "5C6AEC8": "Optiver Services B.V.",
    "7C45F91": "Hunan Shengyun Photoelectric Technology Co., LTD",
    "88C9B3A": "WEG AUTOMATION EUROPE S.R.L.",
    "7C45F9E": "Scania CV AB",
    "C0EAC38": "CDSTech",
    "C0EAC37": "Annapurna labs",
    "705A6FA": "Annapurna labs",
    "8C5DB25": "Unite Audio",
    "8C5DB20": "NPP NTT LLC",
    "705A6FC": "CoolR Group Inc",
    "705A6FB": "Callidus trading, spol. s r.o.",
    "705A6F1": "BMR s.r.o.",
    "94C9B79": "Titanium union(shenzhen)technology co.,ltd",
    "705A6F3": "Wavelab Telecom Equipment (GZ) Ltd.",
    "C4A559D": "MINOLTA SECURITY",
    "C4A5597": "Aviron Interactive Inc.",
    "C4A5598": "METICS",
    "0CCC471": "General Industrial Controls Pvt Ltd",
    "84B3866": "ALPHA Corporation",
    "84B3862": "Annapurna labs",
    "84B386D": "Dongguan Amsamotion Automation Technology Co., Ltd",
    "E0382DD": "KEPLER COMMUNICATIONS INC.",
    "0CCC470": "Shenzhen Jooan Technology  Co., Ltd",
    "0CCC474": "Qingdao Geesatcom Technology Co., Ltd",
    "0CCC477": "Cyrus Audio LTD",
    "E0382DA": "4D Photonics GmbH",
    "E0382D9": "Velvac Incorporated",
    "F0221DB": "LK Systems AB",
    "D46137A": "Shenzhen Xunjie International Trade Co., LTD",
    "F0221D1": "Dr. Eberl MBE Komponenten GmbH",
    "D461378": "Beijing Digital China Yunke Technology Limited",
    "D46137B": "KunPeng Instrument (Dalian)Co.,Ltd.",
    "D46137D": "IPTECHVIEW",
    "4C74A79": "Suzhou XiongLi Technology Inc.",
    "4C74A70": "Shenzhen Timekettle Technologies Co.,Ltd",
    "4C74A74": "Wuxi Micro Innovation Integrated Circuit Design Co., Ltd",
    "4C74A77": "COREIP TECHNOLOGY PRIVATE LIMITED",
    "4C74A7A": "RAONARK",
    "D4BABA3": "Shenzhen Pu Ying Innovation Technology Corporation Limited",
    "D4BABA8": "Chengdu Ba SAN SI YI Information Technology Co., LTD",
    "D4BABA1": "Annapurna labs",
    "D4BABA9": "Shenzhen Chuangyou Acoustic Technology Co., Ltd.",
    "D4BABAD": "AADONA Communication Pvt Ltd",
    "D4BABA5": "ReeR SpA",
    "D4BABAC": "Rusatom Automated Control Systems, Joint-Stock Company",
    "D096863": "EPHI B.V.",
    "D096866": "Shenzhen Ntmer Technology Co., Ltd.",
    "C498941": "SEAVIEW TELECOM",
    "986EE8B": "Private",
    "C483727": "clk2.inc",
    "C48372C": "Acenew technology(shenzhen) limited company",
    "D42000C": "Gentec Systems  Co.",
    "C483726": "Netplus Co., Ltd.",
    "D42000E": "RPUSI Communication Technology  Co.,Ltd.",
    "D42000D": "ZUUM",
    "D420004": "EVOC VIN Technology Co.,Ltd",
    "D42000A": "BirdDog Australia",
    "5847CAA": "Powder Watts, LLC",
    "5847CAB": "Suzhou Laisai Intelligence Technology Co.,Ltd",
    "883CC5A": "Corigine,Inc.",
    "883CC54": "Swabian Instruments GmbH",
    "5847CAC": "SMS ELECTRIC CO., LTD ZHENGZHOU",
    "18C3F4A": "Shenzhen Yunlianxin Technology Co., Ltd.",
    "18C3F44": "Annapurna labs",
    "883CC5B": "Shenzhen shijia chuangxin Technology Co., Ltd",
    "18C3F47": "Shenzhen Yecon-Tech Co.,Ltd.",
    "18C3F4D": "Shenzhen C & D Electronics Co., Ltd.",
    "2C691DE": "Chengdu Qianhong Communication Co., Ltd.",
    "2C691D6": "Carnegie Robotics",
    "2C691DC": "Aparian, Inc.",
    "2C691D9": "SHENZHEN EX-LINK TECHNOLOGY CO.,LTD",
    "2C691D3": "Sunsa, Inc",
    "300A603": "Intergard do Brasil Ind e Com de Eletr e Mec Ltda",
    "10DCB60": "Apex Supply Chain Technologies",
    "303D514": "Dspread Technology (Beijing) Inc.",
    "303D513": "S & A Systems",
    "303D51E": "Percent.com",
    "FC61792": "Shenzhen Shenshui Electronic Commerce Co.,Ltd",
    "FC61790": "Zhuhai Anjubao Electronics Technology Co., Ltd.",
    "FC61798": "Annapurna labs",
    "FC61799": "MACH SYSTEMS s.r.o.",
    "303D515": "Media Hub Digital Smart Home Pty Ltd.",
    "303D51A": "TeraNXT Global India Pvt Ltd.",
    "0C7FED3": "Soft dB",
    "0C7FED8": "U-tec Group Inc.",
    "8002F46": "Mech-Mind Robotics Technologies Ltd.",
    "8C51097": "ENPLUG Co., Ltd.",
    "0C7FED5": "ShenZhen TianGang Micro Technology CO.LTD",
    "8C51099": "Frontmatec",
    "7050E71": "Annapurna labs",
    "8002F49": "XUNDI(XIAMEN) ELECTRONIC TECHNOLOGY CO.,LTD.",
    "8002F45": "Sichuan Fanyi Technology Co. Ltd.",
    "8002F4D": "Jiangsu Vedkang Medicl Sclence and Technology Co.,Ltd",
    "8002F4A": "PassiveLogic",
    "8002F44": "Infors AG",
    "C4A10E7": "Guangzhou South Satellite Navigation Instrument Co., Ltd.",
    "C4A10EB": "Clinton Electronics Corporation",
    "C4A10E2": "Wistron InfoComn (Kunshan) Co., Ltd.",
    "6C1524E": "AEC s.r.l.",
    "7050E7B": "Beijing Shannoncyber Technology Co.,Ltd",
    "7050E70": "Shenzhen C & D Electronics Co., Ltd.",
    "C4A10E4": "Harbour Cross Technology Ltd",
    "1C59740": "Shenzhen Hanshine Technology Co.Ltd.",
    "18A59C3": "Beijing QS Medical Technology Co., Ltd.",
    "6C15241": "Telsonic AG",
    "6C15247": "Motium Pty Ltd",
    "6C1524D": "SYMLINK CORPORATION",
    "18A59C7": "ePower Network Solution Co., Ltd.",
    "381F263": "Bosch Automotive Electronics India Pvt. Ltd.",
    "1C59742": "Chongqing Taishan Cable Co., Ltd",
    "18A59CA": "Erba Lachema s.r.o.",
    "3043D7E": "Guangdong Hongqin Telecom  Technology Co. Ltd.",
    "3043D71": "Shenzhen juduoping Technology Co.,Ltd",
    "3043D76": "Sprocomm Technologies Co., Ltd.Guangming Branch",
    "6C9308C": "Shenzhen haichangxing Technology Co., Ltd.",
    "0C8629E": "FX TECHNOLOGY LIMITED",
    "6C93083": "LightnTec GmbH",
    "1054D2E": "COSMO AIOT TECHNOLOGY CO LTD",
    "1054D23": "Little Array Technology (Shenzhen) Co., Ltd.",
    "1054D2C": "LUXSHARE-ICT Co., Ltd.",
    "1054D20": "GIPS Technology Co., Ltd.",
    "1054D2A": "Embion B.V.",
    "0C8629A": "Nipron Co.,Ltd",
    "0C8629C": "SHENZHEN YINGMU TECHNOLOGY.,LTD",
    "988FE0A": "Pavana Technologies JSC.",
    "988FE04": "Schmid AG, energy solutions",
    "988FE08": "Changzhou Perceptime Technology Co.,Ltd.",
    "988FE06": "Huaqin Technology Co.,Ltd.",
    "DC3643D": "Hangzhou Huanyu Vision Technology Co., Ltd",
    "18D793E": "Teegarden Applied Science Inc",
    "0826AEC": "Brannstrom Sweden AB",
    "DC36431": "Dongguan Pengchen Earth Instrument CO. LT",
    "DC36433": "WIS Networks",
    "DC36430": "Meier Tobler AG",
    "0826AE6": "Newcapec co.,Ltd",
    "0826AE4": "BANGJOO Co., Ltd.",
    "DC3643B": "nami.ai",
    "04EEE83": "Fluid Management Technology",
    "04EEE85": "RealWear",
    "04EEE82": "Hengke Technology Industry Co., Ltd.",
    "04EEE84": "Shenzhen Daotong Technology Co.,Ltd",
    "18D793B": "EcoG",
    "50A0302": "Annapurna labs",
    "28FD80A": "Apollo Digital (Taiwan) Ltd.",
    "94C9B75": "Beijing Anyunshiji Technology Co., Ltd.",
    "94C9B7A": "ShenZhen Beide Technology Co.,LTD",
    "94C9B70": "Fairy Devices Inc.",
    "94C9B7E": "shenzhen UDD Technologies,co.,Ltd",
    "50A0309": "DPA Microphones A/S",
    "50A0306": "Abacus Research AG",
    "50A0304": "Alert Innovation",
    "94C9B7B": "3D Biomedicine Science & Technology Co., Limited",
    "08F80DC": "ZMBIZI APP LLC",
    "08F80D3": "Annapurna labs",
    "1845B3C": "Bdf Digital",
    "08F80D4": "FG-Lab Inc.",
    "F4A454C": "Integrated Dynamics Engineering GmbH",
    "F4A4549": "Lonton infomation tech Ltd., Co",
    "1845B36": "Harmonic Technology Limited",
    "1845B3A": "Guangzhou Aoshi Internet Information & Technology Co.,Ltd.",
    "1845B34": "Mission Secure Inc",
    "F4700C5": "Shenzhen Lidaxun Digital Technology Co., LTD",
    "F4A4547": "Advanced Mechanical Technology, Inc. d/b/a AMTI",
    "9880BB8": "Jyh Eng Technology Co., Ltd",
    "9880BB2": "Shanghai ECone Technology Co.,Ltd.",
    "7872646": "Shenzhen C-DIGI Technology Co.,Ltd.",
    "9880BB5": "Melexis Technologies NV",
    "1CAE3E1": "IPROAD,Inc",
    "1CAE3EE": "Broachlink Technology Co.,Limited",
    "1CAE3E2": "LINKWISE TECHNOLOGIES CO., LIMITED",
    "1CAE3EB": "Beijing Boyan-rd Technology Development CO.,LTD",
    "1CAE3E8": "JingQi(tianjin) technology Co., Ltd",
    "1CAE3E5": "Netvio Ltd",
    "9880BBA": "Guangzhou Shortcut Technology Co.,Ltd.",
    "986EE82": "Ugreen Group Limited",
    "986EE81": "Shanghai Pixsur Smart Technology Co.,Ltd",
    "38A8CD3": "Dongguan Fyrnetics Co., Ltd",
    "7C83340": "Thermalimage",
    "38A8CD4": "WHITEvoid GmbH",
    "785EE8B": "Lantern Engineering (Pty) Ltd",
    "785EE87": "MT B?LG? TEKNOLOJ?LER? VE DI? T?C. A.?.",
    "785EE85": "INFOMOBILITY S.R.L.",
    "785EE80": "Youtransactor",
    "785EE8A": "Yake (Tianjin) Technology Co.,Ltd.",
    "6433B5A": "Hometek Eletronics Co., Ltd",
    "584849E": "Avadesign Technology Co. Ltd.",
    "2836131": "Hi-p (Suzhou) Electronics Co,Ltd",
    "6433B51": "Huaqin Telecom Technology Co.,Ltd.",
    "6433B53": "Wingtech Mobile Communications Co.,Ltd",
    "6433B5D": "IIYAMA CORPORATION",
    "2836133": "Linear Computing Inc.",
    "5848499": "Shenzhen Tongye Technology Co.,Ltd",
    "44A92CE": "Annapurna labs",
    "5848493": "Viper Design LLC",
    "5848495": "Hubei Shudi Communication Technology Co., Ltd",
    "5848490": "Beijing Zhongyuanyishang Technology Co Ltd",
    "5848491": "SKAARHOJ ApS",
    "44A92CA": "Digiport OU",
    "44A92CC": "Cubitech",
    "44A92C5": "Shenzhen Lianfaxun Electronic Technology Co.,Ltd",
    "F02A2B2": "Shanghai Armour Technology Co., Ltd.",
    "F02A2BB": "EL.MO. spa",
    "F02A2B4": "Onclave Networks",
    "F02A2BA": "Navigil Ltd",
    "A85B36B": "\"Lampyris Plant\" LLC",
    "F02A2BD": "Definitely Win Corp.,Ltd.",
    "F02A2B0": "Merlin Security Inc.",
    "F02A2B5": "Agile Sports Technologies, dba Hudl",
    "A85B368": "ShangHai SnowLake Technology Co.,LTD.",
    "7813051": "Global Media Streaming LLC",
    "781305A": "Leonardo SpA - Montevarchi ",
    "7813050": "InnoSenT",
    "7813054": "Jiangxi Winsky Intelligence Technology Co., Ltd",
    "7813055": "ATS-CONVERS,LLC",
    "E878290": "Tanz Security Technology Ltd.",
    "E878296": "AXING AG",
    "1874E2E": "G&O Audio Co.,LTD",
    "E878293": "Electronic Controlled Systems, Inc.",
    "D09FD95": "Carbon Mobile GmbH",
    "D09FD99": "ENTTEC Pty Ltd.",
    "D09FD91": "elecgator bvba",
    "D09FD9E": "Minibems Ltd",
    "C0FBF95": "HAGUENET",
    "C0FBF9B": "SHENZHEN COMIX HST CLOUD COMPUTING CO., LTD.",
    "C0FBF93": "SHENZHEN HEQIANG ELECTRONICS LIMITED",
    "C8F5D66": "Jabil",
    "C8F5D6D": "Volansys technologies pvt ltd",
    "88C9B30": "ADOPT NETTECH PVT LTD",
    "20CE2A2": "Jabil",
    "20CE2AC": "Ariston Thermo s.p.a.",
    "20CE2A7": "Beijing Huadianzhongxin Tech.Co.,Ltd",
    "0C5CB53": "iH&S Technology Limited",
    "0C5CB59": "Colordeve International",
    "0C5CB57": "Energybox Limited",
    "0C5CB58": "Shenzhen C & D Electronics Co., Ltd.",
    "1CA0EF6": "HANJEN.CHIN  CO., LTD.",
    "0C5CB54": "Annapurna labs",
    "601592E": "Annapurna labs",
    "6015929": "JIANGSU SUNFY TECHNOLOGIES HOLDING CO.,LTD.",
    "601592C": "PSS Co., Ltd",
    "6015921": "RTDS Technologies Inc.",
    "6015923": "OSI TECHNOLOGY CO.,LTD.",
    "DCE533A": "Amazinglayer Network Co., Ltd.",
    "245DFC7": "LTY LLC",
    "E86CC73": "Shenzhen Yibaifen Industrial Co.,Ltd.",
    "E86CC7D": "z-max mediasolution",
    "041119A": "CyOne Security AG",
    "245DFC2": "Blue Iris Labs",
    "245DFC1": "ARTICONA - Bechtle Logistik & Service GmbH",
    "E86CC7E": "Annapurna labs",
    "0411193": "SUZHOU RIBAO TECHNOLOGY CO.,LTD.",
    "0411199": "AC Power Distribution / ACT Entmt.",
    "041119D": "Nuance Hearing Ltd.",
    "9827826": "WESTERN SECURITY SOLUTIONS",
    "982782D": "Thorlabs GmbH",
    "9827823": "Danfoss Power Solutions",
    "9827820": "SHENZHEN HEROFUN BIO-TECH CO., LTD",
    "982782B": "RayTron, INC.",
    "446FD86": "Anhui GuDao Tech",
    "446FD89": "Annapurna labs",
    "78D4F1E": "Blue Sparq, Inc.",
    "78D4F1A": "BONENG TRANSMISSION(SUZHOU)CO.,LTD",
    "78D4F12": "Lyngsoe Systems",
    "446FD8D": "SCAIME",
    "A0024A0": "Zhejiang Hechuan Technology Co.,Ltd",
    "A0024A3": "SomaDetect Inc",
    "A0024AD": "bitbee Inc",
    "A453EEE": "MEGACOUNT",
    "A453EE7": "Beijing Lanke Science and Technology Co.,LTd.",
    "A453EE8": "T-Touching Co., Ltd.",
    "8C476E6": "Oxford Nanopore Technologies Ltd.",
    "3009F91": "Shenzhen Sunvell Electronics Co., Ltd.",
    "8C476ED": "innolectric AG",
    "8CAE49D": "Larch Networks",
    "8411C26": "KESSEL AG",
    "6879121": "Annapurna labs",
    "6879124": "McDonald's Corporation",
    "8411C2A": "igus GmbH",
    "687912D": "Neurolab",
    "DC4A9EE": "SES-imagotag Deutschland GmbH",
    "DC4A9EC": "HEFEI DATANG STORAGE TECHNOLOGY CO.,LTD",
    "DC4A9E1": "Advanced Electronics Ltd",
    "FCCD2F7": "Suzhou lehui display co.,ltd",
    "DC4A9E4": "ADIAL",
    "5895D8E": "Gmv sistemas SAU",
    "5895D84": "Unity Surveillance, Inc.",
    "5895D88": "Shenzhen C & D Electronics Co., Ltd.",
    "FCCD2FE": "Eltek brojila d.o.o.",
    "FCCD2F8": "Asesorias y Servicios Innovaxxion SPA",
    "5895D83": "Tonnet Telecommunication International Co., Ltd.",
    "FCCD2FB": "HEAD-DIRECT (KUNSHAN) Co. Ltd",
    "58208A7": "pureLiFi Ltd",
    "FCCD2F6": "Annapurna labs",
    "CC4F5C8": "Feelmore Labs",
    "58208A2": "MARS DIGI TECH CO .,LTD",
    "58208A3": "Aggregate Co.,Ltd.",
    "18FDCBD": "StreamLocator",
    "18FDCB4": "Gosuncn Technology Group Co.,LTD.",
    "D01411E": "Tecnosoft srl",
    "C0619AB": "Victron Energy B.V.",
    "98FC84E": "Dongguan Kingtron Electronics Tech Co., Ltd",
    "98FC849": "Fath Mechatronics",
    "C86314A": "Optictimes Co.,Ltd",
    "C0619A6": "IPG Automotive GmbH",
    "C0619A2": "Grup Arge Enerji ve Kontrol Sistemleri",
    "98FC847": "Broadtech Technologies Co., Ltd.",
    "4C93A6B": "Felten Electronics",
    "28B77CB": "Vehant Technologies Pvt Ltd.",
    "28B77C3": "Beijing Kitten&Puppy Technology Co.,Ltd.",
    "28B77CA": "Simaudio Ltd",
    "28B77C0": "SHENZHEN EVIEW GPS TECHNOLOGY",
    "4C93A69": "Advantics",
    "C0619A4": "Stello",
    "28B77C5": "GROTHE GmbH",
    "F469D5E": "ORtek Technology, Inc.",
    "F469D56": "TianJin KCHT Information Technology Co., Ltd.",
    "F469D55": "Hefei STAROT Technology Co.,Ltd",
    "F469D5B": "Konntek Inc",
    "5C857E8": "BeiJing Xinsheng Technology Co.,Ltd",
    "5C857EB": "HHCC Plant Technology Co., Ltd.",
    "D014113": "iLOQ Oy",
    "5C857E0": "28 Gorilla",
    "D014117": "Realwave Inc.",
    "3049508": "SHENZHEN LDROBOT CO., LTD.",
    "D014118": "Video Security, Inc.",
    "304950D": "Merlyn Mind, Inc.",
    "F0D7AFA": "MSTAR TECHNOLOGIES,INC",
    "F0D7AF2": "Blacknight Internet Solutions Limited",
    "CCC2613": "NETRADYNE, INC.",
    "F0D7AF5": "Dongguan Huili electroacoustic Industrial Co.,ltd",
    "F0D7AF9": "New IT Project LLC",
    "CCC261B": "Winterthur Gas & Diesel Ltd.",
    "CCC2611": "NWL Inc.",
    "CCC2616": "Guardiar USA",
    "CCC261C": "Nortek Security & Control",
    "38F7CDE": "APT MOBILE SATCOM LIMITED",
    "7069791": "Linksys Telecom Shenzhen CO., LTD",
    "706979D": "FREUND ELEKTRONIKA D.O.O., IP-INTEGRA TECHNOLOGIES",
    "C09BF4B": "NUCTECH COMPANY LIMITED",
    "C09BF43": "Osprey Video, Inc",
    "E8B4702": "internet domain name system beijing engineering research center ltd",
    "E8B470A": "plc2 Design GmbH",
    "E8B4709": "Miltek Industries Pte Ltd",
    "C09BF45": "Infiot Inc.",
    "E8B4700": "DongGuan Ramaxel Memory Technology",
    "E8B4705": "Alperia Fiber srl ",
    "E8B4708": "DEHN SE + Co KG",
    "F041C85": "XI'AN MEI SHANG MEI WIRELESS TECHNOLOGY.Co., Ltd.",
    "E8B4706": "Elcoma",
    "F490CB9": "Fractyl Labs",
    "C09BF44": "JSC NPK ATRONIK",
    "C09BF4C": "Pinpark Inc.",
    "F490CB4": "OmniNet",
    "A4DA229": "Malldon Technology Limited",
    "A019B29": "Lon Microsystems Inc.",
    "F490CB0": "Epitel, Inc.",
    "9405BBC": "LAO INDUSTRIA LTDA",
    "F490CB5": "Avilution",
    "9405BBB": "AUSTAR HEARING SCIENCE AND TECHNILIGY(XIAMEN)CO.,LTD",
    "6462667": "Shenzhen C & D Electronics Co., Ltd.",
    "6462664": "Redstone Systems, Inc.",
    "6462669": "Chunghwa System Integration Co., Ltd.",
    "646266D": "Kobol Innovations Pte. Ltd.",
    "6462662": "Protectli",
    "14AE858": "Trimble LEM",
    "14AE850": "Kayamatics Limited",
    "646266B": "Signal Hound",
    "B0B3532": "Rizhao SUNWAM International Co., Ltd.",
    "3CFAD33": "Harman Connected Services, Inc.",
    "3CFAD3A": "UltiMachine",
    "B0B353A": "Ledger",
    "402C76D": "Guangzhou Qi'an Technology Co., Ltd.",
    "402C76B": "Beijing Kuaiyu Electronic Co., Ltd.",
    "50DE194": "Langogo Technology Co., Ltd.",
    "3CFAD34": "GRG Banking Technology Co.,Ltd",
    "200A0D5": "Shenzhen Zhangyue Technology Co.,Ltd ",
    "A0224E4": "TMGcore, Inc.",
    "A0224E8": "EISST  International Ltd",
    "200A0DC": "sehwa",
    "200A0D1": "Wideband Systems, Inc.",
    "0069677": "PANGAEA SOLUTION INC",
    "006967C": "Desird Design R&D",
    "C4954DE": "Canare Electric Co., Ltd.",
    "A0224E0": "Kyung In Electronics",
    "C4954DB": "Multicom, Inc",
    "0069674": "Command Alkon, Inc",
    "643139A": "Product Development Associates, Inc.",
    "5062551": "Hagiwara Solutions Co., Ltd ",
    "506255A": "CCTV Manufacturer",
    "6431392": "Smartplus Inc.",
    "6431395": "Shenzhen He&e Technology Co.,Ltd.",
    "C4954DD": "Newland Era Edu Hi-Tech(BeiJing)Co.,Ltd",
    "5062558": "Roda industrial development Co.,Ltd.",
    "6431398": "Shenzhen Huanyin Electronics Ltd.",
    "4403775": "Norden Communication UK Ltd.",
    "F8B568E": "ZAO &quot;RADIUS Avtomatika&quot;",
    "4403773": "Annapurna labs",
    "4403770": "Musashi Seimitsu Industry Co.,Ltd",
    "440377A": "symplr",
    "54A493B": "Advice",
    "54A4937": "RED Hydrogen LLC",
    "54A4939": "Do Easy International Limited",
    "54A493A": "Wonders Technology Co., Ltd.",
    "04D16E9": "FUZHOU ZHUOYI ELECTRONIC CO.,LTD",
    "04D16E2": "s.d.i. s.p.a.",
    "401175A": "BWT Tianjin Ltd.",
    "2085936": "Eilersen Electric A/S",
    "4C4BF9D": "Shenzhen Haichuan Intelligent Information Technology Co., Ltd.",
    "2085933": "UNILUMIN GROUP CO.,LTD",
    "4C4BF9B": "Stored Energy Systems",
    "9806378": "Shenzhen Y&D Electronics Information Co., Ltd",
    "2CD1412": "IntelliLUM",
    "2415103": "Kaiyun",
    "2415106": "SHANDONG KEHUI POWER AUTOMATION CO. LTD.",
    "9806376": "BOEING SSG",
    "D05F64D": "Shenzhen Canzone Technology Co.,Ltd.",
    "FCA47A8": "KARRY COMMUNICATION LIMITED",
    "2C16BD1": "Curtiss-Wright Drive Technology",
    "2C16BD2": "AIMCO",
    "2C16BD7": "SCT OPTRONICS CO., LTD",
    "2C16BD0": "Beijing Jishi Huitong Technology Co., Ltd.",
    "2C16BD3": "Saft AB",
    "2C16BD9": "Shanghai Walktech Information Technology Co.,Ltd.",
    "B4A2EBD": "SALZBRENNER media GmbH",
    "B4A2EB8": "SHENZHEN ZHUIFENGMA TECHNOLOGY CO., LTD",
    "FCA47AD": "SHENZHEN KUKU TECHNOLOGY CO.,LTD",
    "FCA47A9": "Oberix Group Pty Ltd",
    "D0C8578": "Nanjing Magewell Electronics Co.,Ltd",
    "D0C857C": "Dante Security Inc.",
    "D0C8570": "YUAN High-Tech Development Co., Ltd.",
    "8C593C1": "Future Robot Technology Co., Limited",
    "8C593C5": "Spectranetix",
    "8C593C8": "Nanonord A/S",
    "6095CE2": "Q-SENTECH Co.,Ltd.",
    "BC9740E": "B4ComTechnologies LLC",
    "BC97406": "Shenzhen Colorwin Optical Technology Co.,Ltd",
    "BC97404": "Wind Mobility Technology (Beijing) Co., Ltd",
    "BC9740A": "Amap Information Technology Co., Ltd",
    "BC9740C": "LISTEC GmbH",
    "1C8259E": "Microtronics Engineering GmbH",
    "BC97409": "Direct Communication Solutions",
    "848BCD4": "Logic Supply",
    "848BCDC": "WORMIT",
    "B0FD0B7": "Everynet Oy",
    "B0FD0B5": "Taian Yuqi Communication Technology Co., Ltd",
    "B0FD0B9": "Eagle Acoustics Manufacturing, LLC ",
    "1C82592": "Diatrend Corporation",
    "1C82598": "SHENZHEN AOA TECHNOLOGY CO.,LTD",
    "1C82599": "Shanghai Xiaoyan Technology Co., Ltd.",
    "C82C2B4": "iWave Systems Tech Pvt Ltd",
    "E41E0A7": "Tritium Pty Ltd",
    "E41E0A0": "Zavod \u2116 423",
    "B0FD0B2": "Vista Manufacturing",
    "C82C2B5": "DALCO AG",
    "C82C2B3": "RF Engineering and Energy Resource",
    "34E1D1C": "CREW by True Rowing, Inc.",
    "CCD39DE": "Shanghai tongli information technology co. LTD",
    "34E1D11": "SAMA NextGen PTE Limited",
    "E41E0A5": "Aeroel srl",
    "FCD2B69": "Winglet Systems Inc.",
    "34E1D1D": "HI-TECH.ORG",
    "FCD2B62": "Soma GmbH",
    "E44CC74": "Beijing Zhongchuangwei Nanjing Quantum Communication Technology Co., Ltd.",
    "E44CC70": "Alert Alarm AB",
    "D47C44D": "Huaqin Telecom Technology Co.,Ltd.",
    "4CBC988": "Shenzhen Shanling Digital Technology Development Co.,Ltd.",
    "E44CC73": "JSC \"Svyaz Inginiring M\"",
    "E05A9F7": "OMB Guitars LLC",
    "E05A9F1": "AITEC SYSTEM CO., LTD.",
    "4CBC984": "Nemon Co., Ltd.",
    "E05A9FE": "ShenZhen Arts Changhua Intelligent Technology Co., Ltd",
    "D8860B2": "Get SAT",
    "38B19E4": "Basalte BVBA",
    "C4FFBCE": "viRaTec GmbH",
    "CCD39D2": "Continental Control Systems",
    "CCD39DC": "Hangzhou Scooper Technology Co.,Ltd.",
    "CCD39D0": "INX CO.,LTD.",
    "9C69B42": "MOZI (Shenzhen) Artificial Intelligence Technology Co., Ltd. ",
    "D425CCD": "Combined Energy Technologies Pty Ltd",
    "D425CC6": "Nanjing LES Information Technology Co., Ltd",
    "D425CCB": "Veea",
    "9C69B44": "Globalcom Engineering SPA",
    "9C69B43": "Appareo Systems, LLC",
    "6CDFFBA": "Guilin Zhishen Information TechonlogyCO.,Ltd",
    "6CDFFB5": "Greenbird Vertriebs GmbH",
    "4C917A1": "Inster Tecnolog\u00eda y Comunicaciones SAU",
    "6CDFFBC": "Toucan Systems Ltd",
    "98F9C74": "Promess GmbH",
    "98F9C71": "HighSecLabs",
    "98F9C78": "Renalsense",
    "6C5C3DE": "Clinton Electronics Corporation",
    "0CFE5DA": "Fujian Jieyu Computer Technology Co., Ltd.",
    "0CFE5D2": "Dspread International Co.,Limited",
    "98F9C70": "SHENZHEN HUNTKEY ELECTRIC CO., LTD.",
    "0CFE5D8": "CTK Contact Electronics co., Ltd.",
    "A4580FC": "Homebeaver",
    "1CFD087": "sunweit industrial limited",
    "1CFD080": "InSeat Solutions, LLC",
    "1CFD08B": "guangzhou huiqun intelligent technology co. LTD",
    "1CFD084": "SABIK Offshore GmbH",
    "1CFD082": "HiHi Ltd",
    "1CFD083": "Umeox Innovations Co.,Ltd",
    "6C5C3D6": "Hangzhou Netease Yanxuan Trading Co.,Ltd",
    "6C5C3D1": "Shenzhen Justek Technology Co., Ltd",
    "3C6A2CE": "Beijing Donghua Hongtai Polytron Technologies Inc",
    "3C6A2C7": "Homegear GmbH",
    "300A60D": "Sixth Energy Technologies Private Limited",
    "3C6A2C0": "Rio Lago Technologies  LLC",
    "300A60E": "Imageo s.r.o.",
    "3C6A2C6": "La Barri\u00e8re Automatique",
    "300A605": "A9",
    "300A600": "KAZUtechnica Co.,Ltd.",
    "A4ED439": "Heyuan intelligence technology CO.,Ltd",
    "A028335": "JGR Optics Inc",
    "A028334": "Firm INFORMTEST Ltd.",
    "A028339": "IMESHX CORPORATION LIMITED",
    "A028337": "Kryptus Information Security S/A",
    "3009F95": "VELSITEC-CLIBASE",
    "9CF6DD8": "Savari Inc",
    "3009F98": "essence security",
    "3009F97": "Maytronics Ltd.",
    "3009F9A": "Shenzhen Tencent Computer System Co., Ltd.",
    "3009F96": "Beijing Mydreamplus Information Technology Co., Ltd.",
    "8489ECA": "Newell Brands",
    "8489ECB": "EPSa Elektronik & Pr\u00e4zisionsbau Saalfeld GmbH",
    "9CF6DD3": "RYEEX Technology Co.,Ltd.",
    "9CF6DDC": "Lighting New Energy Technology Co., Ltd.",
    "C083592": "Huaxin SM Optics Co. LTD.",
    "C083599": "Shenzhen Pay Device Technology Co., Ltd.",
    "04C3E6B": "Flintec UK Ltd.",
    "C083595": "Viper Design, LLC",
    "3C427E0": "Grandway Technology (Shenzhen) Limited",
    "B44BD6E": "CHUNGHSIN INTERNATIONAL ELECTRONICS CO.,LTD.",
    "3C427E4": "Teknoware Oy",
    "04C3E63": "Extech Electronics Co., LTD.",
    "04C3E65": "Invasys",
    "04C3E69": "Ekin Teknoloji San ve Tic A.S.",
    "D47C44A": "Tendzone International Pte Ltd",
    "D47C441": "Innoviz Technologies LTD",
    "D47C440": "Exafore Oy",
    "D47C443": "OMRON SENTECH CO., LTD.",
    "B44BD68": "Arnouse Digital Devices Corp",
    "2C48352": "Rheonik Messtechnik GmbH",
    "2C48359": "SureFlap Ltd",
    "CCD31EE": "ShenZhenBoryNet Co.,LTD.",
    "A019B2A": "Adomi",
    "A019B24": "Osatec",
    "2C4835C": "Santec Corporation",
    "2C48356": "Exertus Oy",
    "8C1CDAE": "Electronic Controlled Systems, Inc.",
    "3C24F04": "Inter-Coastal Electronics",
    "3C24F03": "Wisycom",
    "3C24F00": "SHENZHEN PINSIDA TECHNOLOGY CO.,LTD.",
    "8C1CDA2": "GEOMC",
    "8C1CDA6": "LocoLabs LLC",
    "8C1CDAC": "Alcidae Inc",
    "3C24F02": "Laipac Technology Inc.",
    "0C73EB9": "Beijing L&S Lancom Platform Tech. Co., Ltd. ",
    "0C73EB1": "EVERSEC TECHNOLOGY CORPORATION",
    "0C73EB8": "Beijing Miiiw Technology Co., Ltd",
    "0C73EBC": "Shenzhen Samchung Video Technology Co., Ltd.",
    "480BB2A": "XIAMEN RONGTA TECHNOLOGY CO.,LTD.",
    "480BB27": "Beijing Dragon Resources Limited.",
    "480BB22": "Thales CETCA Avionics CO., Ltd",
    "885FE8C": "Inor Process AB",
    "885FE84": "Beijing laiwei Technology  Co.,Ltd",
    "885FE8D": "zhejiang yuanwang communication technolgy co.,ltd",
    "301F9AC": "Origami Group Limited",
    "B8D8128": "Visual Productions BV",
    "301F9AE": "Shenzhen Fengliyuan Energy Conservating Technology Co. Ltd",
    "301F9A7": "Triax A/S",
    "F041C88": "POSTIUM KOREA CO., LTD.",
    "301F9A4": "NCM Supplies, Inc.",
    "88A9A7B": "TWK-ELEKTRONIK",
    "88A9A7C": "AndroVideo Inc.",
    "F041C80": "LINPA ACOUSTIC TECHNOLOGY CO.,LTD ",
    "88A9A79": "FlashForge Corporation",
    "88A9A73": "Mikroelektronika",
    "88A9A72": "Honeywell spol. s.r.o. HTS CZ o.z. ",
    "DCE5336": "WECAN Solution Inc.",
    "A4DA226": "AURANEXT",
    "C4FFBC6": "Shenzhen C & D Electronics Co., Ltd.",
    "C4FFBC8": "ShenZhen ZYT Technology co., Ltd",
    "C4FFBCD": "Beijing  KDF information technology co. LTD.",
    "C4FFBCA": "Advanced Navigation",
    "9C431E8": "Wunda Group plc",
    "9C431E9": "\"CONTINENT\" Co. Ltd",
    "9C431E6": "R-S-I Elektrotechnik GmbH  CO KG",
    "C4FFBC2": "Mobiletron Electronics Co., Ltd",
    "C4FFBCC": "KyongBo Electric Co., Ltd.",
    "282C026": "Lookman Electroplast Industries Ltd",
    "282C023": "Dexin Digital Technology Corp. Ltd.",
    "9C431E3": "Advanced Logic Technology (ALT) sa",
    "9C431E5": "ProMOS Technologies Inc.",
    "282C027": "Telecom and Microelectonic Industries",
    "282C029": "Systec Intelligent Building Technology (Tianjin) Co.,Ltd.",
    "F8B568A": "SinePulse GmbH",
    "F8B568C": "3SI Security Systems, Inc",
    "F8B5682": "Shenzhen New-Bund Technology Co., Ltd.",
    "F8B5689": "Beijing Wanji Techonology Co., Ltd.",
    "F8B5687": "CloudMinds (Shenzhen) Holdings Co., Ltd",
    "4048FD8": "Dorel Juvenile",
    "4048FD5": "The 52nd Research Institute of China Electronic Technology Group Corporation",
    "4048FD0": "BEIJING C&W ELECTRONICS(GROUP)CO.,LTD",
    "4048FDD": "NOX Systems AG",
    "3873EA9": "Lightform, Inc.",
    "3873EA6": "Live Sentinel",
    "EC9F0D1": "Simula Technology Inc.",
    "EC9F0D8": "Zhejiang HEJU Communication Technology Co., Ltd",
    "EC9F0D6": "Shenzhen Compare Electronics Co., Ltd",
    "34D0B8B": "OROSOUND SAS",
    "AC1DDF4": "Motec Pty Ltd",
    "AC1DDF7": "Green IT Korea Co., Ltd.",
    "AC1DDF5": "Shenzhen Ouzheng Electronic Tech Co,.Ltd",
    "CC22378": "Safilo S.p.A.",
    "CC2237A": "shenzhen zonglian network technology limited",
    "741AE01": "Socionext Inc.",
    "2C279E1": "Electronique Bluewave Inc.",
    "B8D8120": "Glamo Inc.",
    "CC22379": "E Ink Corp",
    "2C279E7": "FOCAL-JMLab",
    "2C279EA": "Exegy Inc",
    "904E91C": "Showtacle s.r.o.",
    "189BA5E": "Taiwan Name Plate Co.,LTD",
    "189BA55": "Starfire Industries LLC",
    "189BA5D": "legendsky tech ",
    "189BA5A": "SHENZHEN FIONEXX TECHNOLOGIES LTD.",
    "189BA57": "Beijing Xinertel Technology Co., Ltd.",
    "189BA50": "Dectris Ltd.",
    "904E91B": "Shanghai JaWay Information Technology Co., Ltd.",
    "CC1BE00": "Microtech System,Inc  ",
    "34298FD": "Keystone Electronic Solutions",
    "28F537E": "Performance Motion Devices",
    "28F5379": "Herbert Waldmann GmbH & Co. KG",
    "28F5376": "MyOmega Systems GmbH",
    "78D8009": "SightLine Applications",
    "78D800B": "Maddalena S.p.A.",
    "78D8005": "Bj\u00f6rkviks Consulting AB",
    "7CBACCD": "SIGMA-ELEKTRO GmbH",
    "F88A3C8": "Cadmus Electronic Co.,Ltd.",
    "F88A3C1": "Carefree of Colorado",
    "7CBACCB": "Briowireless Inc.",
    "7CBACC7": "Virgin Orbit",
    "F88A3C5": "KOKKIA INC",
    "F88A3C6": "Beijing Zhong Chuang Communication Technology Ltd.",
    "F88A3C4": "GO-LINK TECHNOLOGY CO., LTD.",
    "F88A3CA": "Protos GmbH",
    "8C147DE": "Electrical & Automation Larsen & Toubro Limited",
    "8C147DB": "Bausch Datacom NV/SA",
    "4C65A84": "Plus One Japan Limited",
    "4C65A85": "TEL-Electronics Ltd",
    "A0C5F24": "AiCare Corp.",
    "F023B91": "Ubiant",
    "F023B9B": "Q Core Medical Ltd",
    "04714BD": "Shenzhen BoClouds Technology Co.,Ltd.",
    "F023B9A": "Annapurna labs",
    "F023B93": "BSP RUS Ltd.",
    "8C147DD": "Shenzhen  Lanxus  technology Co. Ltd.",
    "40ED98A": "Integrated Design Ltd",
    "8C147D0": "Nio",
    "60D7E36": "Ameli s.r.l.",
    "04714B5": "Bureau Electronique Appliquee",
    "04714B3": "Griesser Electronic AG",
    "08ED02D": "Origami Energy Ltd",
    "08ED02E": "Telstra Corporation Limited",
    "98AAFC1": "SURTEC",
    "144FD7D": "Shanghai B&A Technology Co., Ltd",
    "144FD79": "Emerson Network Power (India) Pvt. Ltd.",
    "A411637": "SHENZHEN YIWANJIA INFORMATION TECHNOLOGY CO.,LTD",
    "98AAFC0": "Dalian Eastern Display Co., Ltd.",
    "144FD7E": "Edan Instruments, Inc.",
    "144FD75": "FLS FINLAND OY",
    "40F3850": "SubPac",
    "A411635": "Carbon, Inc.",
    "A41163D": "SHENZHEN ZHISHI TECHNOLOGY CO., LTD.",
    "1CA0D30": "OOO Tekhnotronika",
    "40A36B6": "Securiton AG",
    "A41163B": "Moog Music Inc.",
    "40F385A": "Creanord",
    "40F385D": "Digital Bros S.p.A.",
    "40F3852": "Beijing Zongheng Electro-Mechanical Technology Development Co.",
    "8CC8F4E": "Evaporcool Solutions",
    "8CC8F48": "Strongbyte Solutions Limited",
    "8CC8F42": "Dark Horse Connect LLC",
    "8CC8F4C": "Shenzhen KSTAR Science and Technology Co., Ltd",
    "50A4D0E": "Sagetech Corporation",
    "50A4D0D": "Axel Technology",
    "50A4D0A": "Changsha SinoCare, Inc",
    "50A4D04": "Raven Industries Inc.",
    "40ED982": "A-IOX INC.",
    "40ED981": "GuangZhou FiiO Electronics Technology Co.,Ltd",
    "40ED98C": "BloomSky,Inc.",
    "34049E0": "GoChip Inc.",
    "34049E6": "Life Interface Co., Ltd.",
    "A4580F1": "Stone Lock Global, Inc.",
    "A4580F8": "AIR LIQUIDE MEDICAL SYSTEMS",
    "500B910": "Igor, Inc.",
    "7CCBE2E": "Aplex Technology Inc.",
    "A4580F9": "Ksenia Security srl",
    "500B912": "Annapurna labs",
    "500B914": "Sinope technologies Inc",
    "7CCBE22": "1000eyes GmbH",
    "7CCBE26": "SY Electronics Limited",
    "7CCBE29": "Hangzhou Haohaokaiche Technology Co.,Ltd.",
    "244E7B7": "Nanjing Wanlida Technology Co., Ltd.",
    "7CCBE25": "DTECH Labs, Inc.",
    "4CE1739": "Shenzhen Evolution Dynamics Co., Ltd.",
    "1CC0E1B": "Exigent Sensors",
    "4865EE6": "shenzhen sunflower technologies CO., LIMITED",
    "1CC0E19": "Ospicon Company Limited",
    "1CC0E1D": "NewLand (NZ) Communication Tech Limited",
    "4865EE0": "DefPower Ltd",
    "4CE1731": "Nexoforge Inc.",
    "AC64DD8": "PFDC ELANCYL",
    "AC64DD2": "Shenzhen PuHua Technology Co., Ltd",
    "AC64DDA": "Bluewave Global Manufacturing Limited",
    "AC64DD7": "Wittmann Kunststoffger\u00e4te GmbH",
    "383A215": "OOO NPP Uraltechnologiya",
    "383A210": "R3C Information(Shenzhen) Co.\uff0cLtd.",
    "383A211": "HOBART GmbH",
    "F81D78E": "GUANGDONG ENOK COMMUNICATION CO., LTD.",
    "F81D787": "WUHAN GUIDE INFRARED CO.,LTD",
    "F81D78B": "SigmaConnectivityAB",
    "F0ACD7E": "Fiziico Co., Ltd.",
    "C0D3911": "B9Creations",
    "986D35C": "my-PV GmbH",
    "F0ACD7B": "Zhejiang Makepower Electronics,Inc.",
    "B0C5CA0": "EM-Tech",
    "2836386": "Georg Neumann GmbH",
    "2836385": "CHARGELIB",
    "8C192D6": "smartHome Partner GmbH",
    "1C87760": "Dspread Technology (Beijing) Inc.",
    "8C192DD": "Pyras Technology Inc.",
    "D0D94FA": "Shenzhen FDC Electuonic Co.,Ltd.",
    "8C192D0": "Noritsu Precision Co., Ltd.",
    "78CA83A": "Eksagate Elektronik M\u00fchendislik ve Bilgisayar San. Tic. A.\u015e.",
    "D0D94FC": "ARROWAVE TECHNOLOGIES LIMITED",
    "CCD31E4": "PJG Systementwicklung GmbH",
    "CCD31EC": "NantEnergy",
    "E0B6F5E": "Advatek Lighting Pty Ltd",
    "2C265FC": "AATON DIGITAL",
    "E0B6F55": "Shenzhen Civicom Technology Co.,Limited",
    "E0B6F59": "Motiveprime Consumer Electronics Pvt Ltd",
    "E0B6F52": " Shanghai- British Information Technology Co., Ltd",
    "50FF993": "Yongjing Shanghai Electronic Science and Technology ",
    "50FF995": "Garrison Technology",
    "986D355": "PDAHL ",
    "7C477CE": "I-Convergence.com",
    "7C477C4": "RLC Electronics Systems",
    "7C477C9": "DaLian Cheering Tech Co.,Ltd",
    "38B8EBC": "Ajax Systems Inc",
    "5CF2867": "Access IS",
    "38FDFE8": "Indra Navia AS",
    "38FDFE9": "OOO Group of Industrial Technologies",
    "38FDFEB": "Swedish Adrenaline AB",
    "38B8EB9": "NHS Sistemas de Energia",
    "38FDFE0": "Edge I&D Co., Ltd.",
    "38FDFEA": "Management Service Corporation",
    "38B8EB0": "Bumjin C&L Co., Ltd.",
    "38B8EBA": "SECAD SA",
    "38B8EB4": "UMLOGICS",
    "78CA834": "Pinhole (Beijing) Technology Co., Ltd.",
    "78CA833": "Neofon GmbH",
    "78CA832": "APC",
    "78CA836": "Nomiku",
    "1C8879E": "Orion Labs inc",
    "1C8879D": "Beijing Raycores Technology Co.,Ltd",
    "1C87748": "Surtec Industries, Inc",
    "1C88794": "Ultraflux",
    "1C8774A": "Nebbiolo Technologies",
    "1C87749": "Wide World Trade HK ltd.",
    "1C8776D": "Qivivo",
    "1C87741": "SIGFOX",
    "1C87743": "Silora R&D",
    "1C87769": "Tokyo Drawing Ltd.",
    "1C87768": "Guangzhou Video-Star Electronics Co.,Ltd.",
    "1C87763": "Unjo AB",
    "1C87767": "Corporate Systems Engineering ",
    "1C87766": "philandro Software GmbH",
    "1C87762": "Ibeo Automotive Systems GmbH",
    "40A36BD": "FAOD Co.,Ltd.",
    "8439BE9": "Guangdong SunMeng Information Technology Co. Ltd.",
    "8439BE6": "Shenzhen IP3 Century Intelligent Technology Co., Ltd",
    "1C87793": "Visual Land Inc.",
    "1C87790": "Wurm GmbH & Co. KG Elektronische Systeme",
    "70886BB": "Beijing Strongleader Science & Technology Co., Ltd.",
    "70886BC": "MAX4G, Inc.",
    "70886B0": "Veracity UK Ltd",
    "40A36B2": "TOPROOTTechnology Corp. Ltd.",
    "70886B4": "HORI CO., LTD.",
    "70886BA": "RHXTune Technology Co.,Ltd",
    "800A801": "Dongguan I-Chime electrinics Co.,Ltd",
    "CC1BE02": "i-Trinetech Co.,Ltd.",
    "CC1BE0B": "ART&CORE Inc",
    "CC1BE07": "Sichuan Dianjia network technology Co.Ltd.",
    "0055DA0": "Shinko Technos co.,ltd.",
    "1C21D13": "Microview Science and Technology Co.,Ltd",
    "1C21D14": "Scientific-Production Enterprise Dynamics",
    "C88ED1E": "Aventics GmbH",
    "A03E6B6": "Wuhan Rui Ying Tong Network Technology Co., Ltd(China)",
    "A03E6B3": "iLoda Solutions Limited",
    "C88ED10": "AISWORLD PRIVATE LIMITED",
    "A03E6B7": "SinoGrid Software Systems Inc.",
    "78C2C0E": "Huwomobility",
    "B0C5CAC": "XMetrics",
    "B0C5CA3": "abode systems, inc.",
    "B0C5CAE": "Audio Elektronik \u0130thalat \u0130hracat San ve Tic A.\u015e.",
    "B0C5CAD": "Private",
    "DC44279": "Neusoft Corporation",
    "DC44275": "Century Audio, Inc.",
    "78C2C09": "SES",
    "885D90D": "Hexaglobe",
    "74F8DBD": "Simon Electric (China) Co.,ltd",
    "B437D1B": "NSI Co., Ltd.",
    "74F8DB8": "Songam Syscom Co. LTD.",
    "74F8DB6": "Shenzhen Melon Electronics Co.,Ltd",
    "78C2C01": "XRONOS-INC",
    "B437D1E": "Union Tecnologica Noxium S.L.",
    "B437D1D": "ZXY Sport Tracking",
    "885D903": "CPAC Systems",
    "885D905": "Shenzhen JingHanDa Electronics Co.Ltd",
    "807B85E": "Mersen",
    "807B85D": "Kaynes Technology India Pvt Ltd",
    "64FB816": "XIMO Communication Technology Co., Ltd",
    "549A11C": "Xi'an Hua Fan Technology Co.,Ltd.",
    "885D900": "FOSHAN HUAGUO OPTICAL CO.,LTD",
    "549A116": "Orient Direct, Inc.",
    "80E4DAD": "Dalian Roiland Technology Co.,Ltd",
    "80E4DA5": "CAVALRY STORAGE INC",
    "80E4DA6": "BroadMedia Co., Ltd.",
    "64FB810": "SHANGHAI   SIMCOM   LIMITED",
    "1CCAE38": "OxySec S.r.l.",
    "1CCAE39": "SHIN-YOSHA CORPORATION",
    "1CCAE33": "Shenzhen Smart Device Technology Co.,LTD",
    "1CCAE3E": "Dabi Atlante S/A Industrias Medico Odontol\u00f3gicas",
    "1CCAE34": "Sunray Medical Apparatus Co.,Ltd.",
    "2CD1411": "Ezee Systems Limited",
    "2C6A6F8": "Milbank Manufacturing Co.",
    "90C682B": "Lachmann & Rink GmbH",
    "90C682C": "Li Seng Technology Ltd.",
    "90C6825": "S.A.E.T. S.R.L.",
    "2C6A6FE": "EATON FHF Funke + Huster Fernsig GmbH",
    "2C6A6F5": "SHEN ZHEN SIS SCIENCE & TECHNOLOGY LTD.",
    "2C6A6F2": "NanChang LangJie Technology Co.,Ltd",
    "2CD141B": "Resus Industries",
    "9802D8C": "AGV spa",
    "9802D82": "United Power Research Technology Corp.",
    "A0BB3ED": "Shenzhen Talent Technology company limited",
    "A0BB3EC": "Ewig Industries Macao Commercial Offshore Ltd",
    "A0BB3E2": "DirectOut GmbH",
    "28FD806": "Vigil Monitoring",
    "28FD805": "Xiaocong Network Limited",
    "28FD80B": "Poket Hardware GmbH",
    "28FD809": "JINLITONG INTERNATIONAL CO.,LTD",
    "28FD808": "Jasco Products Company",
    "2C265FB": "Rexgen Inc.",
    "2C265FD": "E Core Corporation",
    "2C265F9": "Br\u00fcel & Kjaer Vibro GmbH",
    "9802D85": "EBI  Ltd.",
    "0CEFAFE": "Infinisource Inc.",
    "0CEFAFD": "CJSC \u00abSvyaz Engineering\u00bb",
    "F802786": "Witium Co., Ltd",
    "0CEFAF7": "Syntrans AB",
    "F802788": "EMBUX Technology Co., Ltd.",
    "100723A": "TESSERA TECHNOLOGY INC.",
    "3C39E70": "Hannstar Display Corp",
    "A44F29C": "Shenzhen Huadoo Bright Group Limitied",
    "1007239": "Wireless input technology Inc.",
    "3C39E7E": "MARPOSS SPA",
    "A44F297": "Protean Payment",
    "A44F298": "Innovations in Optics, Inc.",
    "1007234": "Audio Engineering Ltd.",
    "1007233": "Tongfang computer co.Ltd.",
    "3C39E76": "RO.VE.R. Laboratories S.p.A",
    "A44F299": "Certi Networks Sdn Bhd",
    "E81863C": "Shenzhen Hipad Telecommunication Technology Co.,Ltd",
    "E81863D": "DIGITAL DYNAMICS, INC.",
    "D02212D": "SHENZHEN ZHONGXI SECURITY CO.,LTD",
    "B8D8127": "Neuropace Inc.",
    "E818635": "WETEK ELECTRONICS LIMITED",
    "74E14AE": "Diamond Kinetics",
    "74E14AC": "Wuhan Shenghong Laser Projection Technology Co.,LTD",
    "E4956E6": "SHENZHEN JOYETECH ELECTRONICS CO., LTD.",
    "E4956E2": "Shanghai Hoping Technology Co., Ltd.",
    "BC66419": "Shenzhen General Measure Technology Co., Ltd",
    "BC66415": "Scientific Games",
    "74E14A2": "KLIMAT SOLEC Sp. z o.o.",
    "74E14A0": "Altenburger Electronic GmbH",
    "74E14A1": "Cerevo Inc.",
    "74E14A6": "Emerging Technology (Holdings) Ltd.",
    "58FCDB7": "Prometheus Security Group Global, Inc.",
    "58FCDB8": "Shanghai Qianjin Electronic Equipment Co. Ltd",
    "58FCDB5": "Shenzhen Siecom Communication Technology Development Co.,Ltd.",
    "B01F818": "Technion Oy",
    "58FCDBC": "Excenon Mobile Technology Co., Ltd.",
    "B01F81E": "Advanced & Wise Technology Corp.",
    "B01F811": "Uvax Concepts",
    "B01F810": "Dalian GigaTec Electronics Co.,Ltd",
    "58FCDB1": "Certis Technology International",
    "58FCDB3": "Custom Biogenic Systems",
    "B01F816": "COMOTA Co., Ltd.",
    "BC66413": "Solectria Renewables, LLC",
    "BC66410": "InSync Technology Ltd",
    "141FBA2": "Deutsche Energieversorgung GmbH",
    "7C70BCE": "HOPERUN MMAX DIGITAL PTE. LTD.",
    "7C70BCC": "Lukup Media",
    "7C70BC9": "dogtra",
    "7C70BC8": "Mennekes Elektrotechnik GmbH & Co. KG",
    "7C70BC5": "Canary Connect, Inc.",
    "F40E11D": "DXG Technology Corp.",
    "F40E114": "Dayang Technology Development Inc.",
    "141FBA9": "Black Moth Technologies",
    "A43BFAD": "JSC \u201cComponent-ASU\u201d",
    "A43BFA4": "Maxon Australia",
    "D07650E": "Revox Inc.",
    "A43BFA8": "Alpwise",
    "7C70BC2": "Digital Lumens",
    "BC34004": "Dexcel Design Pvt Ltd",
    "7419F8D": "Ansjer Electronics Co., Ltd.",
    "D076506": "Picobrew LLC",
    "D076505": "Annapurna Labs",
    "C85CE29": "Quthc Limited ",
    "C85CE24": "Jector Digital Corporation",
    "C85CE26": "brinfotec",
    "C85CE2E": "Wonder Education Tech Limited",
    "4CEA417": "Atos spa",
    "4CEA411": "HawkEye Technology Co.,Ltd",
    "4CEA419": "Annapurna labs",
    "5CF8386": "Kunshan Baifeng Intelligent Technology Co.,Ltd",
    "F41A794": "Guangdong Sygole Intelligent Technology Co.,Ltd",
    "5CF838C": "Sichuan Zhongguang Lightning Protection Technologies Co., Ltd.",
    "90A9F79": "Zekler Safety AB",
    "90A9F78": "BAODING FORLINX EMBEDDEDTECHNOLOGY CO., LTD",
    "90A9F73": "Suzhou Etag-Technology Corporation",
    "90A9F75": "Shenzhen DOOGEE Hengtong Technology CO.,LTD",
    "90A9F74": "LAB-EL ELEKTRONIKA LABORATORYJNA",
    "100648B": "Shenzhen smart-core technology co.,ltd.",
    "1006486": "Hong Kong BOZZ Co., Limited.",
    "90A9F7E": "The Engineerix Group",
    "2CC44F7": "NSK Dental Italy",
    "1006482": "Dynics",
    "2CC44F2": "Falcon V Systems S. A.",
    "2CC44F4": "Beijing Siling Robot Technology Co.,Ltd",
    "008DF49": "Relay, Inc.",
    "008DF40": "Sensata Technologies",
    "EC9A0CD": "ViGEM GmbH",
    "C898DB8": "Voleatech GmbH",
    "C898DB2": "freecle Inc.",
    "C898DB9": "MINDTEC",
    "C898DB5": "Private",
    "78392D6": "Annapurna labs",
    "C4A5594": "National Company of Telecommunication and Information Security",
    "78392D4": "Annapurna labs",
    "78392DA": "Edgenectar Inc.",
    "8C55706": "Joule Group Limites",
    "8C55700": "AST International GmbH",
    "8C55705": "Private",
    "8C55702": "Microvision Inc",
    "28F8C6C": "Annapurna labs",
    "E8FF1EB": "Melisono AB",
    "C08359E": "Trystar, LLC",
    "8CA6821": "FightCamp",
    "D0A011C": "TASKA Prosthetics",
    "D0A0111": "Shanghai Railway Communication Co. Ltd.",
    "D0A0117": "Shenzhen Liandian Communication Technology Co.LTD",
    "78E9967": " Beisit Electric Tech(Hangzhou)Co.,Ltd.",
    "78E9960": "SHENZHEN EEGUARD TECHNOLOGY CO.,LIMITED",
    "D0A0113": "V-Count Teknoloji A.\u015e.",
    "34C8D65": "PRIZMA GROUP DISTRIBUTION & CONSULTING D.O.O - RUBISEC",
    "40F3858": "Great Talent Technology Limited",
    "904E913": "Great Talent Technology Limited",
    "901564C": "LINAK A/S",
    "9015649": "Annapurna labs",
    "44A92C3": "Luxonis Holding Corporation",
    "086332C": "Swiftronix AB",
    "086332E": "C-VUE (SHANGHAI) AUDIO TECHNOLOGY CO.,LTD",
    "48E6634": "Smile Security and Survillence Private Limited",
    "0863321": "Shanghai eCloud Technologies Co.,ltd",
    "48E6636": "Shenzhen Huabao New Energy Co.,Ltd",
    "48E6631": "Nanning Nislight Communication Technology Co.,Ltd.,",
    "C4CC370": "KAIS Co.,Ltd.",
    "0863320": "Eaton Corporation",
    "60A4348": "TIME ENGINEERING CO., LTD.",
    "60A4342": "Hangzhou Zhongxinhui lntelligent Technology Co.,Ltd.",
    "60A4344": "Human-life Information Platforms Institute",
    "60A4341": "EEG Enterprises Inc",
    "60A4349": "Shenzhen HantangFengyun Technology Co.,Ltd",
    "C8FFBF2": "Cognizant Mobility GmbH",
    "60A434B": "Bweetech Electronics Technology (Shanghai) Co.,Ltd",
    "1063A35": "Lianxin (Dalian) Technology Co.,Ltd",
    "C8FFBF6": "Accuphy Technologies Beijing Ltd",
    "5C87D8A": "Piscis Networks Private Limited",
    "1063A32": "Sichuan Puhui Zhida Communication Equipment Co. Ltd.",
    "1063A30": "Changsha Sunvote Limited",
    "5C87D81": "COMET SYSTEM, s.r.o.",
    "5C5A4C5": "Annapurna labs",
    "5C5A4C3": "Ferroamp AB (publ)",
    "F87A39C": "Flextronics International Kft",
    "F87A394": "Zsystem technology co.,",
    "F87A39A": "FELTRIN INDUSTRIA E COMERCIO",
    "F87A39B": "FuyanshengElectronicFujian Co.ltd",
    "EC5BCD5": "Shenzhen Qunfang Technology Co., LTD.",
    "EC5BCD6": "Jiangsu Fushi Electronic Technology Co., Ltd",
    "344663C": "mirle automation corporation",
    "EC5BCDB": "StepOver GmbH",
    "08DA336": "Videoline Surveillance Services Pvt. Ltd. ",
    "EC5BCD4": "Green Solutions (Chengdu) Co., Ltd",
    "344663D": "HANGZHOU TASHI INTERNET OF THINGS TECHNOLOGY CO., LTD",
    "3446635": "Wuhan IDXLINK Technology Co., Ltd",
    "08DA339": "Lens Technology (Xiangtan) Co.,Ltd",
    "08DA338": "AMPACS Corporation",
    "08DA33E": "Telo Communication\uff08Shenzhen\uff09Co.,Ltd",
    "48E6C64": "Shenzhen ZK Technology CO.,LTD.",
    "4C917A7": "MECCALTE SPA",
    "B44BD69": "Private",
    "2486253": "ViewSec Co., Ltd.",
    "AC86D10": "Advanced Rugged System Co., LTD",
    "485E0EB": "Vital Oricraft Flows Technology Co., Ltd.",
    "485E0E0": "Shenzhen C & D Electronics Co., Ltd.",
    "AC86D14": "RayThink Technology Co.,Ltd",
    "485E0E4": "MHE Electronics",
    "C4FF849": "Atlas Copco",
    "485E0EC": "Transom Post OpCo LLC dba Bose Professional",
    "C4FF845": "Eastern Acoustic Works",
    "C4FF84C": "KSB SE & Co. KGaA",
    "C4FF848": "IBM",
    "C4FF842": "ShenZhen BoQiao Technologies CO.,LTD.",
    "ECA7B10": "Nanjing Xinyun Technology Co., Ltd",
    "ECA7B14": "Sichuan Ruiting Zhihui Technology Co., Ltd",
    "C4FF843": "Cloudhop Inc",
    "ECA7B17": "Shenzhen Lanroot Technology Co., Ltd",
    "ECA7B1A": "Annapurna labs",
    "D014111": "P.B. Elettronica srl",
    "B84C876": "HORIBA Precision Instruments (Beijing) Co.,Ltd",
    "C022F13": "Spectra Technologies India Private Limited",
    "B84C870": "Annapurna labs",
    "48DA354": "Sphere Com Services Pvt Ltd",
    "48DA355": "Beijing keshengte communication equipment co., ltd",
    "48DA35E": "Neps Technologies Private Limited",
    "C022F16": "Pony.AI, INC.",
    "FCCD2FA": "Scopus International Pvt. Ltd.",
    "48DA358": "Shenzhen Qianhong Technology Co.,Ltd.",
    "5491AFE": "Jiangxi Anbaichuan Electric Co.(ABC),Ltd",
    "48DA35A": "Auto Meter Products Inc.",
    "48DA350": "RBS LLC",
    "48DA35D": "NACON LIMITED (HK) LTD",
    "88A6EFB": "Beijing ThinRedline Technology Co.,Ltd.",
    "88A6EFC": "Shenzhen C & D Electronics Co., Ltd.",
    "D015BB2": "Beijing Guangshu Zhiying Technology Development Co., Ltd.",
    "D015BB7": "New Tech IoT",
    "D015BBA": "HONG KONG COHONEST TECHNOLOGY LIMITED",
    "88A6EFA": "Draper, Inc.",
    "D015BBC": "Shenzhen Waystar Communication Technology Co. Ltd.",
    "50482C5": "Bluefin International Inc",
    "50482C0": "Landatel Comunicaciones SL",
    "58C41EA": "GeBE Elektronik und Feinwerktechnik GmbH",
    "58C41E6": "NetChain Co.,Ltd.",
    "C86BBC5": "Shenzhen Hebang Electronic Co., Ltd",
    "D09FD93": "GS Yuasa International Ltd.",
    "C86BBC8": "Sinsegye Beijing Technology Co., Ltd",
    "8C476E7": "Syng, Inc.",
    "58C41E4": "BEIJING FIBRLINK COMMUNICATIONS CO.,LTD.",
    "BC31984": "Chongqing e-skybest ELECT CO.,LIMITED",
    "BC31986": "ntc mekhanotronnika",
    "C86BBC1": "WeLink Solutions, Inc.",
    "BC31988": "Temposonics,LLC",
    "B0FF72D": "TBB Renewable (Xiamen) Co Ltd",
    "B0FF729": "JIUYEE?shenzhen) Medical Technology Co.,Ltd",
    "B0FF727": "BL Innovare",
    "90F421E": "Velan Studios Inc.",
    "B0FF72E": "HANNING & KAHL GmbH & Co. KG",
    "B44D435": "halstrup-walcher GmbH",
    "B44D437": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "90F421C": "ACOBA",
    "90F4214": "Sansap Technology Pvt. Ltd.",
    "B44D43C": "SHENZHEN KOSKY TECHNOLOGY CO.,LTD.",
    "B44D43B": "Paulmann Licht GmbH",
    "6C2ADF9": "Simpleway Europe a.s.",
    "6C2ADFC": "VNETS INFORMATION TECHNOLOGY LTD.",
    "6C2ADFD": "Sichuan Huidian Qiming Intelligent Technology Co.,Ltd",
    "1CA0D37": "U-TX Technologies Ltd",
    "54083B7": "ASCS Sp. z o.o.",
    "7006928": "JMA Wireless",
    "58FCDBF": "Private",
    "28FD80F": "Private",
    "141FBAF": "Private",
    "700692C": "ScoreBird, LLC",
    "D07650F": "Private",
    "54083B1": "Annapurna labs",
    "54083BA": "Silex Ipari Automatiz\u00e1l\u00e1si Zrt.",
    "BC3400F": "Private",
    "78C2C0F": "Private",
    "100723F": "Private",
    "807B85F": "Private",
    "A03E6BF": "Private",
    "64FB81F": "Private",
    "54083B6": "Vector Atomic",
    "A43BFAF": "Private",
    "7C70BCF": "Private",
    "E03C1CB": "Hangzhou Uni-Ubi Co.,Ltd.",
    "D016F02": "RYSE Inc.",
    "D016F0D": "Top Guard Technologies",
    "74F8DBF": "Private",
    "BC6641F": "Private",
    "D016F06": "Tornado Modular Systems",
    "D093952": "AT&T",
    "5C6AECE": "Saab Seaeye Ltd",
    "5C6AEC9": "Shanghai Alway Information Technology Co., Ltd",
    "5C6AEC0": "Acuity Brands Lighting",
    "7C45F96": "HANK ELECTRONICS CO., LTD",
    "7C45F9C": "Xemex NV",
    "7C45F95": "Interactive Technologies, Inc.",
    "C0EAC3A": "VOLT EQUIPAMENTOS ELETRONICOS LTDA",
    "5C6AEC7": "Nippon Pulse Motor Co., Ltd.",
    "7C45F94": "SPECS Surface Nano Analysis GmbH",
    "C0EAC39": "OLEDCOMM",
    "8C5DB2C": "HEXIN Technologies Co., Ltd.",
    "18D793C": "Private",
    "705A6FE": "Hall Technologies",
    "8C5DB2E": "Surbhi Satcom Pvt Ltd",
    "8C5DB2A": "Beijing Scistor Technologies Co., Ltd",
    "8C5DB2B": "NADDOD",
    "C4A559A": "Hebei Far-East Communication System Engineerning Co.,Ltd.",
    "94C9B78": "OSOM Products Inc",
    "705A6F9": "Annapurna labs",
    "84B3865": "Fusus",
    "0CCC47E": "Foxconn Brasil Industria e Comercio Ltda",
    "C4A5595": "Moultrie Mobile",
    "C4A5596": "Annapurna labs",
    "84B3861": "Sichuan Huakun Zhenyu Intelligent Technology Co., Ltd",
    "84B3863": "Phonesuite",
    "E0382D3": "Annapurna labs",
    "E0382D0": "Beijing Cgprintech Technology Co.,Ltd",
    "E0382D6": "iTracxing",
    "E0382DC": "SiLAND Chengdu Technology Co., Ltd",
    "F0221D8": "Shenzhen Glazero Technology Co., Ltd.",
    "F0221D5": "Shenzhen SuyuVisonTechnology Co.,Ltd",
    "F0221D9": "Shanghai Gfanxvision Intelligent Technology Co.Ltd",
    "F0221D7": "Bulat Co., Limited",
    "D46137E": "UAB Brolis sensor technology",
    "D461371": "Shenzhen smart-core technology co.,ltd.",
    "D461372": "Robert Bosch Elektronikai Kft.",
    "D461373": "APPOTRONICS CO., LTD",
    "D461376": "Securus CCTV India",
    "4C74A73": "GoCodeIT Inc",
    "C498940": "Shenzhen Megmeet Drive Technology Co.,Ltd.",
    "C498944": "Alpine Electronics Marketing, Inc.",
    "D096869": "Camfil",
    "D096861": "PROVCOM LTD",
    "C49894B": "Shanghai YVR Technology Co., Ltd.",
    "C498946": "Aetina Corporation ",
    "D4BABA4": "Beijing Yuanxin Junsheng Technology Co.,ltd",
    "D420002": "Shenzhen AI Develop & Manufacture Co.,LTD.",
    "C483725": "Wuhan Da Ta Technologies Co.,Ltd.",
    "C483729": "Biwave Technologies, Inc.",
    "5847CA2": "ONAWHIM (OAW) INC.",
    "5847CA5": "Huizhou Jiemeisi Technology Co., Ltd",
    "5847CA7": "Shenzhen Meigao Electronic Equipment Co.,Ltd",
    "5847CAD": "PRACTEK Technology Co., Ltd.",
    "18C3F4E": "SHENZHEN MEGMEET ELECTRICAL CO., LTD",
    "18C3F40": "Scati Labs, S.A.",
    "883CC57": "KMtronic ltd",
    "883CC50": "Chengdu Data Sky Technology Co., Ltd.",
    "303D517": "Destiny Automate Limited",
    "2C691DD": "Ascentac Inc.",
    "2C691DB": "Shenzhen Daren HI-Tech Electronics Co., Ltd.",
    "2C691D0": "Hunan Xiangjiang Kunpeng Information Technology Co., Ltd.",
    "2C691D2": "Abode Systems Inc",
    "303D519": "Annapurna labs",
    "F490CBA": "Fend Incorporated",
    "FC61796": "Hangzhou LiDe Communication Co.,Ltd",
    "FC61794": "CHOEUNENG",
    "FC61793": "EchoStar Mobile",
    "0C7FEDE": "environmental systems corporation",
    "303D512": "Harman Connected Services Corporation India Pvt. Ltd.",
    "FC6179E": "ACCO Brands USA LLC",
    "8C51092": "PROCET Technology Co., Ltd(HK)",
    "0C7FED4": "Purple Mountain ,Inc",
    "0C7FED2": "Tango Networks Inc",
    "8C5109C": "SpotterRF LLC",
    "8002F4E": "Alfred Systems Inc",
    "7050E76": "Nippon Pulse America, Inc.",
    "8002F4C": "Wuhan Glory Road Intelligent Technology Co., Ltd.",
    "8002F41": "Sichuan lookout environment protection technology co.,Ltd",
    "C4A10EC": "Focus-on",
    "C4A10E5": "O-NET Industrial Technologies (Shenzhen) Limited",
    "7050E7D": "Eta Compute Inc.",
    "6C1524B": "Annapurna labs",
    "1C59746": "Square Inc.",
    "6C15245": "Shenzhen Electron Technology Co., LTD.",
    "6C15248": "ShenZhen Chainway Information Technology Co., Ltd.",
    "6C15240": "DEFA AS",
    "6C15242": "Linkplay",
    "6C1524C": "CORAL-TAIYI",
    "1C5974B": "Beijing Flintec Electronic Technology Co.,Ltd.",
    "1C59744": "Syntax technology(tianjin)Co.,LTD",
    "18A59C4": "IT-1",
    "18A59C8": "Residence Control Ltd",
    "1C59749": "Shanghai Laisi Information Technology Co.,Ltd",
    "18A59C2": "Actiontec Electronics Inc.",
    "0826AEA": "Flextronics International Kft",
    "381F26D": "HWACHANG CORPORATION ",
    "381F264": "Airmaster A/S",
    "3043D7C": "Xiaoniu network technology (Shanghai) Co., Ltd.",
    "6C9308B": "Shenzhen EZpro Sound & Light Technology Co., Ltd.",
    "0C86297": "HagerEnergy GmbH",
    "6C93084": "Estelar s.r.o",
    "1054D21": "Jiangxi Ofilm&Jvneng IoT Tech Co., Ltd.",
    "0C86292": "BADA SYSTEM co., Ltd",
    "1054D2B": "Shenzhen Dinstech Technology Co.,Ltd.",
    "1054D2D": "Sun wealth technology corporation limited",
    "1054D22": "ComNav Technology Ltd.",
    "988FE00": "Valinso B.V.",
    "988FE09": "Nawon Machinery",
    "988FE05": "KuaiZhu SmartTechnology?Suzhou?CO.,Ltd",
    "0826AE7": "EVTECH SOLUTIONS LTD. DBA 3D-P",
    "DC36439": "Hefei EA Excelsior Information Security Co., Ltd.",
    "0826AE0": "Wuhan Tianyu Information Industry Co., Ltd.",
    "18D7934": "Remote Engineer B.V.",
    "18D7936": "Autel lntelligent Technology Corp.,Ltd",
    "04EEE87": "Shenzhen C & D Electronics Co., Ltd.",
    "DC36435": "Hangzhou Chingan Tech Co., Ltd.",
    "50A0307": "Shenzhen Hewang Electric Co.,Ltd",
    "50A030A": "Missing-Link Oy",
    "50A0301": "XEPIC Corporation Limited",
    "04EEE81": "Shanghai ZLAN Information Technology Co.,Ltd",
    "08F80D0": "Huizhou changfei Optoelectruonics Technology Co.,Ltd",
    "08F80D7": "HANGZHOU YILI Communication Equipment Ltd",
    "08F80D5": "Zhejiang Luci Technology Co., Ltd",
    "94C9B76": "Realtimes Beijing Technology Co., Ltd.",
    "94C9B7D": "Dspread Technology (Beijing) Inc.",
    "94C9B73": "Sitronics JSC",
    "08F80DA": "MICKEY INDUSTRY,LTD.",
    "C8F5D69": "Shanghai Mo xiang Network Technology CO.,ltd",
    "08F80D2": "Shanghai Mininglamp AI Group Co.,Ltd",
    "08F80DB": "Vont Innovations",
    "94C9B7C": "Jinjin Technology (Shenzhen) Co., Ltd",
    "94C9B77": "MAMMOTHTEK CLOUD(DONG GUAN)TECHNOLOGY CO., LTD",
    "1845B3E": "Sleep Number",
    "1845B35": "ELPITECH LLC",
    "F4A4548": "Shenzhen Cudy Technology Co., Ltd.",
    "F4700C1": "Shenzhen Excelland Technology Co., Ltd.",
    "F4700C7": "Changde xsound lnnovation technologies co;ltd.",
    "787264E": "Heltec Automation",
    "7872645": "CALTTA TECHNOLOGIES CO.,LTD.",
    "7872641": "Zhengzhou Reform Intelligent Device Co., Ltd",
    "9880BBE": "D.Med Technical Products GmbH",
    "9880BB1": "GreatWall Information Co.,Ltd",
    "9880BB6": "Neusoft Reach Automotive Technology (Shenyang) Co.,Ltd",
    "9880BB0": "RYEEX Technology Co.,Ltd.",
    "787264B": "digades GmbH",
    "787264C": "Comcast-SRL",
    "1CAE3E7": "NextDrive Co.",
    "1CAE3E0": "DAO QIN TECHNOLOGY CO.LTD.",
    "1CAE3E4": "P.H.U. Metering Anna Moder",
    "986EE83": "ReeR SpA",
    "986EE8E": "First Design System Inc.",
    "986EE80": "Sbarco Technology CO., Ltd.",
    "986EE89": "Span.IO, Inc.",
    "986EE8A": "Logos Payment Solutions A/S",
    "986EE86": "Blair Companies",
    "50DE19E": "DTEN Inc.",
    "38A8CD9": "PT Supertone",
    "38A8CDA": "NIC Technologii",
    "38A8CD5": "Revo Infratech USA Ltd",
    "7C8334A": "ENGINETECH (TIANJIN) COMPUTER CO.,LTD.",
    "7C8334C": "Hunan Datang Xianyi Technology Co.,Ltd",
    "7C83348": "Silicon Xpandas Electronics Co., Ltd.",
    "7C8334B": "Shenzhen AZW Technology Co., Ltd.",
    "785EE8E": "Suzhou Tianping Advanced Digital Technologies Co.Ltd",
    "785EE82": "Vega-Absolute",
    "785EE88": "Jiangxi guoxuan radio and television technology Co.,Ltd",
    "785EE83": "Incontrol LLC",
    "2836132": "Shenzhen HQVT TECHNOLOGY Co.,LTD",
    "2836136": "ESI Ventures, LLC",
    "283613D": "AVYCON",
    "6433B55": "Revo Smart Technologies co.,limited",
    "2836130": "Shandong SIASUN Industrial Software Research Institute Co., Ltd",
    "584849C": "Haag-Streit AG",
    "44A92CD": "NPP KOMETEH JSC",
    "44A92C9": "China Electronics Corporation Greatwall Shengfeifan information system Co.,ltd. Hu'nan computer R.&D. Center",
    "5848494": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "584849D": "Telegaertner Elektronik GmbH",
    "5848498": "STACKFORCE GmbH",
    "A85B36E": "ORBITVU Sp. z o. o.",
    "F02A2B6": "Shenzhen ORVIBO Technology Co., Ltd.",
    "A85B360": "Bluesoo Tech (HongKong) Co.,Limited",
    "A85B367": "Louis Vuitton Malletier",
    "E87829E": "Solos Technology Limited",
    "3049505": "IK Elektronik GmbH",
    "E878291": "Shenzhen Jointelli Technologies Co.,Ltd ",
    "E87829A": "METZ CONNECT GmbH",
    "7813056": "CRRC Nangjing Puzhen Haitai Brake Equipment Co., LTD ",
    "7813058": "Shenzhen AV-Display Co.,Ltd",
    "781305B": "Bithouse Oy",
    "1874E2D": "Samriddi Automations Pvt. Ltd.",
    "1874E2A": "Linux Automation GmbH",
    "E87829D": "Bernd Walter Computer Technology",
    "D09FD9A": "Eurolan Ltd",
    "E05A9F8": "Fujian Newland Auto-ID Tech. Co,.Ltd.",
    "D09FD96": "Elevoc Technology Co., Ltd.",
    "C0FBF9C": "SHENZHEN ELSKY TECHNOLOGY CO., LTD",
    "1874E2C": "NextGen RF Design, Inc.",
    "1874E21": "Sartorius Lab Instruments GmbH & Co. KG",
    "C8F5D6C": "Eltako GmbH",
    "C0FBF9D": "Dropbeats Technology Co., Ltd.",
    "88C9B3D": "Origins Technology Limited",
    "88C9B3E": "Sercomm Corporation.",
    "88C9B38": "Divelbiss Corporation",
    "C8F5D6B": "United Barcode Systems",
    "88C9B35": "Brabender Technologie GmbH & Co, KG",
    "20CE2AE": "Funkwerk Systems GmbH",
    "88C9B34": "Hasbro Inc",
    "20CE2AD": "LAUDA DR R WOBSER GMBH & CO KG",
    "20CE2A5": "Zaber Technologies Inc.",
    "20CE2AB": "Swarovski Optik KG",
    "20CE2A9": "Rugged Monitoring",
    "20CE2AA": "MeshPlusPlus, Inc.",
    "1CA0EF3": "Sequent AG",
    "6015925": "Comfit HealthCare Devices Limited",
    "0C5CB51": "avxav Electronic Trading LLC",
    "1CA0EF7": "tec5AG",
    "0C5CB50": "Yamasei",
    "0C5CB52": "HongKong Blossom Limited",
    "0C5CB5E": "Munters Europe AB",
    "601592D": "REMOWIRELESS COMMUNICATION INTERNATIONAL CO.,LIMITED",
    "601592A": "insensiv GmbH",
    "6015922": "EDA Technology Co.,LTD",
    "245DFCE": "Dodge",
    "E86CC79": "Hangzhou Lanxum Security Technology Co., Ltd",
    "E86CC76": "KLAB",
    "0411198": "Shenzhen YIZHENG Technology Co.,Ltd",
    "245DFCC": "Senix Corporation",
    "245DFCD": "Hunan Honestone lntelligence Technology Co.,Ltd",
    "041119E": "JULIDA LIMITED",
    "0411190": "FORT Robotics Inc.",
    "E86CC75": "Shenzhen Rongda Computer Co.,Ltd",
    "041119B": "Hubei Baobao Intelligent Technology Co.,LTD",
    "446FD81": "Shenzhen Furuilian Electronic Co.,Ltd.",
    "446FD84": "lb Lautsprecher gmbH",
    "446FD8B": "Beijing gpthink technology co.,LTD.",
    "78D4F13": "Ekoenergetyka - Polska S.A.",
    "78D4F16": "Guangzhou Kingray information technology Co.,Ltd.",
    "446FD88": "Global Telecom Engineering, Inc",
    "A0024AE": "IoTecha Corp",
    "78D4F1C": "TNB",
    "78D4F10": "Burisch Elektronik Bauteile GmbH",
    "A453EEA": "shanggong technology Ltd",
    "A453EE5": "Foshan Yisihang Electrical Technology Co., Ltd.",
    "A453EE3": "Larva.io O\u00dc",
    "885FE8E": "Unicom Global, Inc.",
    "8CAE495": "Gati Information Technolog(Kunshan)Co.,Ltd.",
    "0055DAA": "Speechlab",
    "A453EE4": "Williamson Corporation",
    "A453EEC": "SOS LAB Co., Ltd.",
    "8CAE493": "BERTIN TECHNOLOGIES",
    "8CAE492": "SEVERIN Elektroger\u00e4te GmbH",
    "8411C21": "Beijing Dayu Technology Co., Ltd.",
    "8411C29": "C TECH BILISIM TEKNOLOJILERI SAN. VE TIC. A.S.",
    "8411C23": "\t FUJIFILM Healthcare Corporation",
    "DC4A9E0": "Dongguan Huili electroacoustic Industrial Co.,ltd",
    "8411C2B": "Guangdong Creator&Flyaudio Electronic Technology Co.,LTD",
    "8411C2E": "Dangerous Music Group, LLC",
    "5895D8B": "SuZhou Ruishengwei Intelligent Technology Co.,Ltd",
    "5895D87": "Epiphan Systems Inc",
    "2085939": "Mastodon Design",
    "5895D8D": "Alunos AG",
    "CC4F5C0": "Chengdu Ren Heng Mei Guang Technology Co.,Ltd.",
    "FCCD2FC": "Spedos ADS a.s.",
    "58208AB": "Infodev Electronic Designers Intl.",
    "58208AD": "SAMBO HITECH",
    "58208AA": "Conductix-Wampfler",
    "58208A5": "JIA HUANG JHAN YE CO.,LTD",
    "CC4F5C2": "MatchX GmbH",
    "98FC84B": "chiconypower",
    "98FC848": "Guangdong DE at science and technology co., LTD",
    "C863142": "Tymphany Acoustic Technology (Huizhou) Co.,  Ltd.",
    "18FDCB2": "Cabtronix AG",
    "18FDCB5": "Accel Robotics",
    "28B77C9": "Anser Coding Inc.",
    "28B77CE": "Ray Pte Ltd",
    "4C93A6C": "Wuhan Maiwe communication Co.,Ltd",
    "4C93A6D": "Cantronic Systems (Canada) Inc",
    "4C93A61": "Atrie Technology Fzc",
    "F469D53": "ITS Co., Ltd. ",
    "5C857E1": "Sichuan C.H Control Technology Co., Ltd.",
    "5C857EA": "Zhejiang Jetron Ark Digital Technology Co., Ltd",
    "5C857E7": "Beijing HZFD Technology Co., Ltd",
    "F469D58": "WiFi Nation Ltd",
    "F469D51": "Junchuang (Xiamen) Automation Technology Co.,Ltd",
    "5C857E3": "Cable Matters Inc.",
    "5C857ED": "Nautech Electronics Ltd",
    "3049504": "ADVANCED MICROWAVE ENGINEERING SRL",
    "D014110": "EkkoSense Ltd",
    "D014112": "Evoco Labs CO., LTD",
    "D01411C": "Shen Zhen HaiHe Hi-Tech Co., Ltd",
    "3049507": "Shenzhen iTG robot Co.,Ltd.",
    "E41E0A1": "Connected Cars A/S",
    "8489ECD": "Price Industries Limited",
    "CCC261E": "Toong In Electronic Corp.",
    "CCC2612": "Tecnoideal Srl",
    "7069798": "An Phat Information Technology Co., Ltd",
    "706979B": "Liquid Instruments Pty Ltd",
    "7069795": "Ibyte",
    "CCC261A": "Shenzhen Uyesee Technology Co.,Ltd",
    "706979E": "BAS-IP LP",
    "7069790": "Full Solution Telecom",
    "38F7CD7": "ARUNAS PTY LTD",
    "38F7CDD": "Macherey-Nagel GmbH & Co. KG",
    "94FBA7E": "Skyring Smart Technologies(Shenzhen) Co., Ltd.",
    "94FBA7A": "ELKRON ",
    "94FBA72": "Beijing Leja Tech co., Ltd.",
    "38F7CD0": "Polska Fabryka Wodomierzy i Ciep?omierzy FILA ",
    "94FBA71": "Inaxsys Security Systems inc.",
    "F490CB3": "Ricker Lyman Robotic",
    "F490CBD": "Simavita (Aust) Pty Ltd",
    "F490CB8": "Beijing Penslink Co., Ltd.",
    "C09BF4A": "Inveo",
    "C09BF4D": "The Professional Monitor Company Ltd",
    "C09BF41": "Connected Space Management",
    "94CC04E": "SynchronicIT BV",
    "94CC04B": "Shandong free optical technology co., ltd.",
    "94CC049": "ENTEC Electric & Electronic Co., LTD.",
    "6462663": "FaceHeart Inc.",
    "646266A": "Sensoro Co., Ltd.",
    "6462666": "Pass & Seymour, Inc d/b/a Legrand",
    "90E2FCC": "Stanley Security",
    "90E2FCD": "Beijing Lanxum Computer Technology CO.,LTD.",
    "14AE85D": "iSolution Technologies Co.,Ltd.",
    "90E2FC2": "ShenZhen Temwey Innovation Technology Co.,Ltd.",
    "90E2FC8": "bitsensing Inc.",
    "90E2FC4": "Dongguan Kangyong electronics technology Co. Ltd",
    "3CFAD38": "Energous Corporation",
    "3CFAD35": "Gulf Security Technology Co., Ltd",
    "3CFAD3D": "AMobile Solutions (Xiamen) CO. , LTD.",
    "B0B3535": "Zenlayer",
    "B0B3539": "HANMECIPS CO.",
    "3CFAD30": "Home Control AS",
    "402C768": "Suteng Innovation Technology Co., Ltd.",
    "402C767": "Zhejiang Guoli Security Technology Co., Ltd.",
    "402C760": "Lista AG",
    "50DE19D": "Penny & Giles Aerospace Ltd",
    "50DE195": "Bliq B.V.",
    "50DE19B": "BRAINWARE TERAHERTA INFORMATION TECHNOLOGY CO.,LTD.",
    "402C766": "Guangzhou LANGO Electronics Technology Co., Ltd.",
    "200A0D2": "Netinovo Technologies(Shenzhen) Ltd",
    "402C769": "Annapurna labs",
    "5062550": "Ufanet SC",
    "3C39E7B": "chipsguide technology Co.,LTD",
    "200A0D8": "bcheck NV",
    "A0224E7": "Applied Information, Inc.",
    "A0224E3": "ProPhotonix",
    "C4954D9": "Shenzhen Xtooltech Co., Ltd",
    "6431397": "Dongguan Huili electroacoustic Industrial Co.,ltd",
    "5062559": "Southern Ground Audio LLC",
    "C4954D2": "Shen\u00a0Zhen\u00a0Euse\u00a0Technology\u00a0Co.,Ltd",
    "440377D": "OMNISENSE SYSTEMS PRIVATE LIMITED TAIWAN BRANCH",
    "10DCB62": "CAL-COMP INDUSTRIA E COMERCIO DE ELETRONICOS E INFORMATICA LTDA",
    "10DCB66": "Prolan Zrt.",
    "54A4934": "Shenzhen C & D Electronics Co., Ltd.",
    "54A4931": "ShenZhen Smart&Aspiration Co.,LTD",
    "54A493D": "ASSEM TECHNOLOGY CO.,LTD.",
    "10DCB6E": "Shenzhen Sunwoda intelligent hardware Co.,Ltd",
    "10DCB68": "Sanofi (Beijing) Pharmaceutical Co., Ltd.",
    "54A4938": "Chengdu EVECCA Technology Co.,Ltd.",
    "04D16ED": "Elotec Fischer Elektronik GmbH",
    "54A493E": "Nederman Holding AB",
    "54A4935": "AUSOUNDS INTELLIGENCE, LLC",
    "401175B": "Chongqing IQIYI Intelligence Technology Co., Ltd.",
    "401175D": "NanJing HuaStart Network Technology Co.,Ltd.",
    "401175C": "disguise Technologies Limited",
    "4011753": "Beijing Hexinruitong Electric Power Technology Co., Ltd.",
    "4011757": "Guangzhou RALID Information System Co.Ltd",
    "4C4BF92": "Shenzhen HommPro Technology Co.,Ltd",
    "10DCB64": "Annapurna labs",
    "2415109": "Topgolf Sweden AB",
    "2415107": "SuZhou A-rack Information Technology Co.,Ltd",
    "4C4BF93": "Power Active Co., Ltd",
    "4C4BF90": "Multitek Elektronik Sanayi ve Ticaret A.S.",
    "4C4BF96": "Shandong Linkotech Electronic Co., Ltd.",
    "2085932": "Mid Continent Controls, Inc.",
    "34049E1": "Connected IO",
    "2085938": "AASSET SECURITY",
    "2085935": "Wave-In Communication",
    "44D5F22": "Shenzhen Hebang Electronic Co., Ltd",
    "9806374": "Chengdu Shuwei Communication Technology Co.,Ltd",
    "44D5F2B": "Valeo Interior Controls (Shenzhen) Co.,Ltd",
    "44D5F25": "tiga.eleven GmbH",
    "980637A": "Angora Networks",
    "D05F64A": "PartnerNET LTD",
    "FCA47AE": "Hefei Feier Smart Science&Technology Co. Ltd",
    "FCA47A4": "HOOC AG",
    "D05F64C": "Nanjing Huamai Technology Co.,Ltd",
    "D05F644": "wallbe GmbH",
    "D05F640": "Decathlon SA",
    "2C16BD6": "CLOUDWALK TECHNOLOGY CO.,LTD",
    "B4A2EB1": " DCI International, LLC.",
    "B4A2EB6": "ShenZhen Lark Acoustics Co., Ltd. ",
    "2C16BDD": "Hangzhou Yanzhi Technology Co.,Ltd.",
    "B4A2EBE": "Dongguan Finslink Communication Technology Co.,Ltd.",
    "B4A2EB3": "Canaan Creative Co.,Ltd.",
    "2C16BD5": "Beijing Zhijian Link Technology Co., Ltd.",
    "FCA47A2": "Ant Financial\uff08Hang Zhou\uff09Network Technology Co.,Ltd.",
    "2C16BDE": "Molex Incorporated ",
    "FCA47A5": "Syfer",
    "300A601": "Beijing Ruiteng Zhongtian TECH Ltd.,Co",
    "D0C8574": "Imin Technology Pte Ltd",
    "745BC51": "Beijing Inspiry Technology Co., Ltd. ",
    "D0C857A": "shenzhen cnsun",
    "8C593CC": "Dantherm Cooling Inc.",
    "D0C8579": "Shenzhen xiaosha  Intelligence  Technology Co. Ltd",
    "D0C8576": "Innovative Industrial(HK)Co., Limited",
    "D0C8572": "FORGAMERS INC.",
    "1C8259C": "Evondos Oy",
    "1C82595": "Fagus-GreCon Greten GmbH & Co. KG",
    "1C8259D": "Applied Concepts, Inc.",
    "BC9740D": "Rollock Oy",
    "BC97402": "Lattec I/S",
    "BC97400": "Alpha ESS Co., Ltd.",
    "6095CEA": "(UN)MANNED",
    "6095CE9": "Jlztlink Industry(ShenZhen)Co.,Ltd.",
    "6095CE3": "Robot S.A.",
    "6095CE7": "Cadmo Soluciones SAC",
    "6095CED": "GovComm",
    "6095CE1": "Ponoor Experiments Inc.",
    "1C82590": "Shandong Luneng Intelligence Technology CO., Ltd",
    "1C82597": "Jump Trading",
    "1C82594": "winsun AG",
    "F81D788": "TELEOFIS",
    "848BCDB": "CHONGQING HUAYI KANGDAO TECHNOLOGY CO.,LTD.",
    "848BCD9": "NORALSY",
    "B0FD0B6": "DNESO TEN Ltd.",
    "1C82593": "C&A Marketing, INC.",
    "848BCD1": "Shenzhen\u00a0LTIME\u00a0In-Vehicle\u00a0Entertainment\u00a0System\u00a0Company\u00a0Limited",
    "B0FD0BB": "MartinLogan, Ltd.",
    "E41E0A2": "IDvaco Private Limited",
    "B0FD0BC": "Haltian Products Oy",
    "E41E0AB": "Safety Vision, LLC",
    "E41E0A6": "SFC Energy AG",
    "E41E0A4": "XPR Group",
    "E41E0A9": "B METERS S.R.L.",
    "C86314B": "Shenzhen Lihewei Electronics Co.,Ltd.Hunan Branch",
    "34E1D1A": "OrCam Technologies",
    "34E1D18": "Hubitat Inc.",
    "34E1D1B": "APG Cash Drawer, LLC",
    "FCD2B66": "Cirque Audio Technology Co.,Ltd",
    "FCD2B61": "LINK (FAR-EAST) CORPORATION",
    "FCD2B64": "SHEN ZHEN XIN HAO YUAN PRECISION TECHNOLOGY CO.,L TD",
    "FCD2B67": "Teamly Digital",
    "745BC52": "SIGLENT TECHNOLOGIES CO., LTD.",
    "745BC5E": "Qingdao Wintec System Co., Ltd",
    "E44CC79": "Ottomate International Pvt. Ltd.",
    "E44CC71": "ACS-Solutions GmbH",
    "E44CC7D": "Telo Systems Limitd",
    "745BC56": "Yekani Manufacturing PTY Ltd",
    "745BC57": "SHENZHEN ATX TECHNOLOGY CO.,LTD ",
    "E44CC7B": "SmallHD",
    "745BC55": "SpringCard",
    "745BC50": "IRS Systementwicklung GmbH",
    "D8860BE": "Shenzhen Yidong Technology Co.,Ltd",
    "D8860B8": "VRINDA NANO TECHNOLOGIES PVT LTD",
    "E05A9FC": "ShenZhen Mornsun Smartlinker Limited Co., LTD",
    "4CBC980": "Charge-Amps AB",
    "4CBC98D": "Elink Technology (Shenzhen) Co., Limited",
    "38B19E5": "Star Electronics GmbH & CoKG",
    "38B19E2": "HDANYWHERE",
    "38B19EA": "Aeroespacial Guosheng Technology Co., Ltd",
    "38B19EE": "ShenZhen ShuaiXian Electronic Equipment Co.Ltd",
    "D8860BC": " YUSAN INDUSTRIES LIMITED \u2002",
    "D8860BB": "Library Ideas",
    "D8860BD": "ComNav Technology Ltd.",
    "D8860B3": "Auvidea GmbH",
    "28F5378": "1MORE",
    "38B19E0": "Triple Jump Medical",
    "38B19EC": "Gesellschaft industrieller Technologien ",
    "CCD39D8": "Obelisk Inc.",
    "38B19E8": "BoCo Inc.",
    "CCD39D9": "Bejing Nexsec Inc.",
    "CCD39D6": "Krontech",
    "CCD39DB": "Q-Branch Labs, Inc.",
    "CCD39D4": "Shenzhen Chenggu Technology Co., Ltd",
    "9C69B40": "Suzhou Fitcan Technology Co.,LTD",
    "9C69B4C": "Guangdong Hanwei intergration Co.,Ltd",
    "D425CC9": "TAKUMI JAPAN LTD",
    "9C69B4E": "NINGBO SHEN LINK COMMUNICATION TECHNOLOGY CO., LTD",
    "9C69B46": "Shenzhen jiahua zhongli technology co.LTD",
    "4C917A6": "Openeye",
    "4C917A3": "Smart Access",
    "6CDFFBD": "Nanjing Buruike Electronics Technology Co., Ltd.",
    "7CBC84E": "Beijing Topnew Group Co., Ltd",
    "7CBC848": "Shenzhen Kuang-chi  Space Technology Co., Ltd.",
    "6CDFFB4": "Lineable Inc",
    "4C917AC": "Alibaba (Beijing)\u2002Software Service Inc.",
    "4C917A5": "mtekvision",
    "7CBC84D": "VANTAGE INTEGRATED SECURITY SOLUTIONS PVT LTD",
    "98F9C7E": "NC-LINK Technology Co., Ltd.",
    "7CBC843": "Shanghai Yitu Technology Co. Ltd",
    "0CFE5D6": "Antailiye Technology Co.,Ltd",
    "0CFE5D7": "Vermes Microdispensing GmbH",
    "0CFE5D4": "Yantai Dongfang Wisdom Electic Co.,Ltd.",
    "0CFE5D5": "SELECTRIC Nachrichten-Systeme GmbH",
    "98F9C72": "Pozyx NV",
    "6C5C3D9": "IskraUralTEL",
    "A83FA1C": "Laonz Co.,Ltd",
    "A83FA1E": "Guangzhou Navigateworx Technologies Co., Limited",
    "A83FA1A": "Shanghai East China Computer Co., Ltd",
    "A83FA17": "Plejd AB",
    "6C5C3D4": "HTI Co., LTD.",
    "3C6A2C3": "figur8, Inc.",
    "300A602": "Advanced Electronic Designs, Inc.",
    "300A606": "Realtime biometrics India pvt ltd",
    "A4ED436": "Shanghai  Facom  Electronics Technology  Co, ltd.",
    "A028333": "SHANGHAI XUNTAI INFORMATION TECHNOLOGY CO.,LTD.",
    "A028331": "Ordercube GmbH",
    "A02833A": "Medical Evolution Kft",
    "A4ED43A": "Guangzhou Maxfaith Communication Technology Co.,LTD.",
    "A4ED434": "NETAS TELEKOMUNIKASYON A.S.",
    "A4ED435": "Beijing ICPC CO.,Ltd.",
    "8489EC2": "thousand star tech LTD.",
    "A028338": "HZHY TECHNOLOGY",
    "9CF6DDE": "Shanxi ZhuoZhi fei High Electronic Technology Co. Ltd.",
    "B8D8122": "IPM Sales and service Co.,Ltd.",
    "9CF6DD2": "Beijing Sifang Automation Co., Ltd.",
    "9CF6DD5": "b8ta Inc.",
    "3009F9E": "ZhongLi HengFeng (Shenzhen) Technology co.,Ltd.",
    "3009F9B": "Sichuan Nebula Networks Co.,LTD.",
    "9CF6DD0": "Annapurna labs",
    "C083590": "CHONGQING JIUYU SMART TECHNOLOGY CO.LTD.",
    "04C3E66": "Shenzhen Shuotian Information Technology Co., LTD",
    "04C3E62": "SiS Technology",
    "04C3E67": "Advanced Digital Technologies, s.r.o.",
    "04C3E61": " Guangdong New Pulse Electric Co., Ltd.",
    "3C427EE": "Xiaoniu network technology (Shanghai) Co., Ltd.",
    "B44BD6C": "Impakt S.A.",
    "B44BD6A": " Shenzhen Huabai Intelligent Technology Co., Ltd.",
    "3C427E7": "GJS Co., Ltd.",
    "3C427E9": "TAITEX CORPORATION",
    "3C427E1": "Dongguan Taide Industrial Co.,Ltd.",
    "D47C442": "YunDing Network Technology (Beijing) Co., Ltd",
    "A019B23": "Power Diagnostic Service Co., LTD.",
    "CCD31ED": "CUJO LLC",
    "D47C449": "Suzhou Wan Dian Zhang Network Technology Co., Ltd",
    "74F8DBA": "Ballard Technology, Inc,",
    "A019B2C": "LDA Technologies",
    "A019B20": "Vast Production Services",
    "2C48358": "DPS Electronics",
    "2C4835B": "Shanghai Visteon Automotive Electronics System CO. Ltd.",
    "3C24F05": "CASKY eTech Co., Ltd.",
    "8C1CDAA": "China Potevio Co., Ltd",
    "8C1CDA4": "Anntec \uff08Beijing\uff09 Technology Co.,Ltd.",
    "3C24F0D": "Travis Holding B.V.",
    "3C24F01": "Abrites Ltd.",
    "480BB25": "Solaredge LTD.",
    "480BB23": "shanghai Rinlink  Intelligent Technology Co., Ltd.",
    "0C73EB6": "Green Fox Electro AS",
    "301F9A8": " FINE TRIUMPH TECHNOLOGY CORP.,LTD.",
    "301F9A5": "Beijing Surestar Technology Co. Ltd,",
    "301F9A1": "Dewesoft d.o.o.",
    "301F9A0": "ILSAN ELECTRONICS",
    "88A9A74": "Thomas & Darden, Inc",
    "F041C83": "SHENZHEN WISEWING INTERNET TECHNOLOGY CO.,LTD",
    "88A9A7A": "Zhejiang Haoteng Electronic Technology Co.,Ltd.",
    "F041C8B": "Powervault Ltd",
    "A4DA22C": "EHO.LINK",
    "DCE533D": "Suzhou ATES electronic technology co.LTD",
    "DCE533C": "BRCK",
    "DCE5334": "shenzhen bangying electronics co,.ltd",
    "DCE533E": "Giant Power Technology Biomedical Corporation",
    "DCE5339": "Tiertime Corporation",
    "DCE5330": "FLYHT Aerospace",
    "9C431EB": "JNL Technologies Inc",
    "C4FFBC4": "iMageTech CO.,LTD.",
    "9C431EC": "SuZhou Jinruiyang Information Technology CO.,LTD",
    "C4FFBC3": "SHENZHEN KALIF ELECTRONICS CO.,LTD",
    "282C024": "EFENTO T P SZYD\u0141OWSKI K ZAR\u0118BA SP\u00d3\u0141KA JAWNA",
    "F8B5681": "PT. Eyro Digital Teknologi",
    "282C02C": "Epoch International Enterprises, Inc.",
    "4048FD3": "RL Controls LLC.",
    "4048FD4": "Dynamic Engineering",
    "4048FDB": "Magenta Labs, Inc.",
    "3873EA8": "Rock Electronic Co., Ltd.",
    "4048FD1": "Fast Programming",
    "4048FDE": "SMART SENSOR DEVICES AB",
    "3873EA2": "Eyesight(Shanghai)Communication Technology Co.,Ltd.",
    "3873EA0": "L-3 Communications Mobile-Vision, Inc.",
    "EC9F0D0": "Hesai Photonics Technology Co., Ltd",
    "EC9F0D7": "Bei jing Lian Shan times Techonology Co.Ltd",
    "EC9F0D5": "Paw-Taw-John Services, Inc.",
    "EC9F0DC": "Sarcos Corp",
    "EC9F0D9": "FCI",
    "34D0B83": "Tascent, Inc.",
    "34D0B88": "Vtrek Group International Ltd.",
    "34D0B82": "Blustream Pty Ltd",
    "34D0B85": "eesy-innovation GmbH",
    "AC1DDF3": "CRDE",
    "549A115": "Elotech Industrieelektronik GmbH",
    "741AE0A": "SAIERCOM CORPORATION",
    "741AE06": "Blocks Wearables Inc.",
    "741AE0D": "Voltaware Services Limited",
    "CC2237C": "Hebei ZHSF Technology Co.,Ltd.",
    "CC22373": "XConnect Professional Services",
    "904E916": "Nuwa Robotics (HK) Limited Taiwan Branch",
    "904E910": "Spirtech",
    "904E919": "CUTTER Systems spol. s r.o.",
    "904E915": "mcf88 SRL",
    "904E917": "IBM",
    "904E912": "North Pole Engineering, Inc.",
    "50FF99E": "Informa LLC",
    "2C279E5": "AudioNord Distribution A/S",
    "CC22374": "SHANGHAI CARGOA M.&E.EQUIPMENT CO.LTD",
    "34298F2": "Shenzhen Advance River System Technology Co., Ltd",
    "189BA51": "ChengDu Vantron Technology, Ltd.",
    "34008AD": "ChengDu HuiZhong Cloud Information Technology Co., Ltd.",
    "28F5377": "Shenzhen Modern Cowboy Technology Co.,Ltd.     ",
    "28F5373": "PRIMETECH ENGINEERING CORP.",
    "34008A9": "Keruyun Technoligies(Beijing) Corporation Limited",
    "34008A2": "RPE \"Monitor\"",
    "34008AC": "Shenzhen Eternal Idea Tech Co.,Ltd",
    "34008A4": "Fotonic i Norden AB",
    "28F537B": "LogiM GmbH Software und Entwicklung",
    "28F5375": "Atomrock LLC",
    "28F5371": "Umojo",
    "78D800A": "Insignal Co., Ltd.",
    "78D8004": "CS Instruments GmbH",
    "28F537D": "Skyrockettoys LLC",
    "28F5370": "Valeo Siemens eAutomotive Norway",
    "7CBACC1": "Changsha SUNYE Electric Co., Ltd.",
    "7CBACC4": "Sun Asia Trade Co.",
    "4C65A8A": "Suzhou Embedded Electronic Technology Co., Ltd.",
    "F88A3C3": "Shenzhen Shengyuan Tech Ltd.",
    "F88A3C2": "KLATU Networks Inc",
    "4C65A83": "Roost",
    "A0C5F2E": "Synapsys Solutions Ltd.",
    "A0C5F21": "KNS Group LLC (YADRO Company)",
    "A0C5F22": "Speedgoat GmbH",
    "A0C5F27": "Viettronimex JSC",
    "F023B95": "Audeara Pty. Ltd.",
    "04714B0": "Neurio Technology Inc.",
    "F023B99": "Emu Technology",
    "04714BB": "DIGIBEST TECHNOLOGY CO., LTD.",
    "04714B9": "Lighthouse AI, Inc",
    "04714BC": "KittyHawk Corporation",
    "1CC0E17": "SHENZHEN KINSTONE D&T DEVELOP CO.,LTD",
    "60D7E31": "Elap s.r.l.",
    "08ED027": "Eleven Engineering Incorporated",
    "08ED022": "TES Touch Embedded Solutions Inc.",
    "98AAFC2": "Shenzhen UniStrong Science & Technology Co., Ltd",
    "98AAFCB": "Resonant Systems Inc.",
    "98AAFCA": "SENKO Co.,Ltd.",
    "08ED028": "HANTAS CO., LTD.",
    "144FD74": "Red Technology Limited",
    "144FD73": "Qingdao Wodatong Electronics Co., Ltd.",
    "144FD7A": "Unirobot Corporation",
    "A411636": "Beijing XiaoRui Technology Co., Ltd",
    "1CA0D32": "NovTech, Inc.",
    "40F3851": "Johnson Matthey",
    "40F385C": "Clixxo Broadband Private Limited",
    "8CC8F4B": "PTYPE Co., LTD.",
    "8CC8F4D": "Beijing Xinxunxintong Eletronics Co.,Ltd",
    "40F385B": "URMET Home & Building Solutions Pty Ltd",
    "8CC8F45": "Beijing KXWELL Technology CO., LTD ",
    "50A4D03": "Guangzhou Hysoon Electronic Co., Ltd.",
    "50A4D0C": "Beijing YangLian Networks Technology co., LTD",
    "A4580F7": "Changsha Tai Hui Network Technology Co.,Ltd",
    "34049E4": "Harbin Yantuo Science and Technology Development Co., Ltd",
    "40ED989": "TeraTron GmbH",
    "40ED98E": "BORDA TECHNOLOGY",
    "40ED98D": "Hangzhou GANX Technology Co.,Ltd.",
    "A4580FB": "ABB AB PGHV",
    "7CCBE2D": "optilink networks pvt ltd",
    "A4580F3": "Engineered SA",
    "500B913": "EWIN TECHNOLOGY LIMITED",
    "500B916": "Security Alarms & Co. S.A.",
    "244E7B3": "Shenzhen Ruixunyun Technology Co.,Ltd.",
    "4865EEE": "CNU",
    "4865EE5": "Swistec Systems AG",
    "244E7BB": "Mighty Audio, Inc.",
    "7CCBE24": "Ningbo bird sales co.,LTD",
    "100723C": "Shenzhen Xinfa Electronic Co.,ltd",
    "244E7B1": "sonoscape",
    "244E7BC": "CHUNGHSIN TECHNOLOGY GROUP CO.,LTD",
    "1CC0E16": "Monument Labs, Inc.",
    "4CE173D": "KTC(K-TEL)",
    "1CC0E12": "Abbott Medical Optics Inc.",
    "4CE1730": " Beijing Sutongwang E-Business Co., Ltd",
    "383A21C": "Mission Embedded GmbH",
    "AC64DDC": "Beijing Hamigua Technology Co., Ltd.",
    "383A213": "Shanghai Greatwall Safety System Co.,Ltd",
    "F81D785": "DACONS",
    "F81D782": "Xperio Labs Limited",
    "70F8E7A": "TiVACI CORPORATION PTE LTD",
    "70F8E73": "Dr. Simon Consulting GmbH",
    "F81D789": "Ophrys Syst\u00e8mes",
    "C483724": "Transact Technologies Inc",
    "AC64DDD": "HMicro Inc",
    "3009F9C": "Honeywell",
    "84E0F4C": "AIMTRON CORPORATION",
    "84E0F4B": "Orchard Electronics Co., Ltd.",
    "84E0F43": "ASL Intercom B.V.",
    "84E0F40": "ShenZhen Panrich Technology Limited",
    "C0D391E": "SAMSARA NETWORKS INC",
    "C0D391A": "Alpha Audiotronics, Inc.",
    "C0D391C": "Zhinengguo technology company limited",
    "C0D3915": "WiTagg, Inc",
    "58E8769": "TEM Mobile Limited",
    "58E8764": "PROBIT SRL",
    "F0ACD7A": "Groupeer Technologies",
    "F0ACD7D": "Smart Power Technology Co., Ltd.",
    "5CF2860": "Hangzhou Signwei Electronics Technology Co., Ltd",
    "2836381": "Panasonic System Solutions Europe",
    "2836389": "Shenzhen  Zhi Hua  Creative Technology  Co., Ltd.",
    "D0D94F2": "Teco Image Systems Co., Ltd.",
    "D0D94F6": "Hyundai Autohow",
    "D0D94F9": "Hangzhou xiaoben technology co.,Ltd",
    "8C192D5": "ELCO(TIANJIN)ELECTRONICS CO.,LTD.",
    "CCD31E3": "KEN A/S",
    "CCD31E2": "Neptune Systems",
    "D0D94FB": "MAX Smart Home, LLC",
    "D0D94F1": "mycable GmbH",
    "CCD31E0": "SAMIM Co",
    "CCD31E1": "Rondo Burgdorf AG",
    "6891D06": "femrice",
    "C47C8D0": "ATI",
    "C47C8DE": "Labor Strauss Sicherungsanlagenbau GmbH",
    "C47C8D1": "LYNX INNOVATION LITIMED",
    "6891D00": "Central Railway Manufacturing",
    "6891D02": "Shenzhen NeaTech Intelligence Technology Co., Ltd.",
    "E0B6F57": "Shenzhen Xrinda Technology Ltd",
    "50FF999": "Sea Eagle Optoelectronic Information Technology(Tianjin)co,Ltd",
    "E0B6F50": "BeSTAR Corporation",
    "50FF990": "Simicon",
    "50FF996": "LEGEND WINNER LIMITED",
    "986D350": "Shenzhen MALATA Mobile Communication Co.,LTD",
    "986D353": "DH Mechatronic AG",
    "7C477CA": "Dspread Technology (Beijing) Inc.",
    "7C477C7": "BlueSmart Technology Corporation",
    "7C477CD": "Speedifi Inc",
    "5CF2861": "iSon Tech",
    "5CF2864": "CHIPSEN Co.,Ltd.",
    "38FDFE6": "Inspero Inc",
    "38FDFED": "FUBA Automotive Electronics GmbH",
    "38B8EB1": "1.A Connect GmbH",
    "38B8EB3": "Aina Wireless Inc",
    "78CA83E": "Konecranes",
    "78CA835": "Huatune Technology (Shanghai) Co., Ltd.",
    "1C88797": "Sensys Networks, Inc.",
    "1C8774B": "HABEY USA Inc.",
    "1C87746": "Schawbel Technologies LLC",
    "1C88795": "SHENZHENFREELINK ELECTRONIC CO.,LTD",
    "1C88793": "Shenzhen Xiaoxi Technology Co., Ltd.",
    "1C8776C": "Strone Technology",
    "1C8776A": "Jiangsu ETERN COMMUNICATION Co.,ltd",
    "1C87765": "Zhuhai MYZR Technology Co.,Ltd",
    "8439BE4": "Shenzhen Ramos Digital Technology Co,.Ltd.",
    "1C87794": "Novetta",
    "40A36B5": "National Research Council of Canada",
    "8439BE5": "Neat S.r.l.",
    "1C88799": "Xingtera China Ltd",
    "70886B5": "Chengdu Ophylink Communication Technology Ltd.",
    "70886B2": "CVnet",
    "40A36BA": "Embrionix Design Inc.",
    "CC1BE0C": "Guangzhou Southelectric Power Science Technology Development Co.,Ltd.",
    "0055DA1": "KoolPOS Inc.",
    "C88ED1B": "Advanced Micro Controls Inc.",
    "C88ED18": "Electronic Controls Design, Inc.",
    "C88ED19": "Focalcrest, Ltd.",
    "A03E6BA": "Shenzhen Neostra Technology Co.Ltd",
    "A03E6BD": "Jining SmartCity Infotech Co.Ltd.",
    "0055DA5": "Nanoleaf",
    "0055DA4": "Datapath Limited",
    "0055DA8": "BroadSoft, Inc.",
    "CC1BE04": "Laserworld (Switzerland) AG",
    "DC44272": "Skywave Technology Co,.Ltd.",
    "DC44276": "EK-TEAM Elektronik- u. Kunststoff-Technik GmbH",
    "DC44277": "EcoGuard AB",
    "B0C5CA6": "SunTech Medical, Inc.",
    "B0C5CA4": "shanghai University Ding-Tech software Corp.,ltd",
    "B437D1C": "NANJING PUTIAN TELECOMMUNICATIONS TECHNOLOGY CO.,LTD.",
    "B437D15": "Stratom, Inc.",
    "B437D14": "KOMSIS ELEKTRONIK SISTEMLERI SAN. TIC. LTD.STI",
    "B437D19": "Nanjing yuekong Intelligent Technology",
    "B437D18": "eInfochips Limited",
    "78C2C03": "Ningbo Sanxing Electric Co., Ltd.",
    "78C2C02": "RONIX incorporated",
    "74F8DBC": "TBM CO., LTD.",
    "74F8DBE": "Bernard Krone Holding GmbH & Co. KG",
    "74F8DBB": "Capwave Technologies Inc",
    "78C2C0C": "Shanghai Hanyi Technologies Co,.Ltd.",
    "74F8DB9": "Avantree Corporation",
    "549A11B": "Elite Silicon Technology, Inc.",
    "549A118": "Tite, Inc.",
    "549A112": "Torrap Design Limited",
    "885D908": "Creative Sensor Inc.",
    "549A119": "Alfen BV",
    "885D90E": "Unitac Technology Limited",
    "885D90A": "Shenzhen Speedrun Technologies Co.,Ltd.",
    "807B854": "Quantel USA, Inc.",
    "74F8DB0": "Enercon Technologies",
    "885D902": "DAIDONG Industrial System Co., Ltd.",
    "807B855": "EFCO",
    "807B853": "Zhuhai TOP Intelligence Electric Co., Ltd.",
    "807B851": "Hangzhou Synway Information Engineering Co., Ltd",
    "1CCAE3A": "SIREA",
    "1CCAE35": "TengFeng",
    "64FB817": "Securosys SA",
    "80E4DA1": "Guangzhou Pinzhong Electronic Technology CO., LTD",
    "80E4DA2": "Thurlby Thandar Instruments LTD",
    "2CD141C": "PIN SHANG LED Co., LTD.",
    "2CD1414": "Shanghai RW ELE&TEC CO.,LTD",
    "90C682D": "PowerShield Limited",
    "2C6A6FD": "Holjeron",
    "2C6A6FC": "Sensity Systems",
    "2C6A6F4": "TINYCO",
    "2C6A6F6": "Beep, Inc.",
    "2C6A6F3": "Cloudproject Generation Srl",
    "90C6824": "Neone, Inc.",
    "9802D89": "Navroom Beijing, China",
    "9802D8A": "HySecurity",
    "9802D87": "Ormazabal Protection&Automation",
    "28FD800": "Millcode",
    "A0BB3EB": "Beijing Techshino Technology Co., Ltd.",
    "9802D80": "Stoerk-Tronic, Stoerk GmbH & Co.KG",
    "9802D8D": "Promicon Elektronik GmbH + Co.KG",
    "9802D8B": "HANSHIN MEDICAL CO., LTD.",
    "A0BB3E0": "Link Labs",
    "28FD807": "University of York",
    "28FD801": "Galileo, Inc.",
    "28FD80D": "Grandway Technology (Shenzhen) Limited",
    "A0BB3E7": "SIMTEC Elektronik GmbH",
    "2C265F8": "Itus Networks, LLC",
    "2C265F7": "Coremate Technical Co., Ltd",
    "2C265F4": "GTA Electronics Co., Ltd.",
    "2C265F1": "Griessbach",
    "F80278D": "Dueton Systems s.r.o.",
    "F80278A": "Luxul Technology Inc",
    "0CEFAF9": "Rotel",
    "F802781": "Reason Tecnologia SA",
    "A44F294": "DGC Access AB",
    "A44F296": "Selektro Power Inc",
    "A44F295": "Shanghai KuanYu Industrial Network Equipment Co.,Ltd",
    "A44F292": "LUCEOR",
    "A44F291": "Olssen B.V.",
    "3C39E73": "ELSA Japan Inc.",
    "0CEFAF6": "Firmware Design AS",
    "0CEFAF3": "Engineering Center ENERGOSERVICE",
    "E818633": "DongGuan Pengxun Electronics Technology Co., Ltd.",
    "E818634": "Guangzhou Tianyi Electronics Co., Ltd",
    "B8D812E": "ZheJiang FangTai Electirc Co., Ltd",
    "E818630": "DigiMagus Technology (Shenzhen) Co., Ltd",
    "1007231": "Beijing Assem Technology Co., ltd",
    "D02212A": "GNS-GmbH",
    "E81863A": "JDM Mobile Internet Solution(Shanghai) Co., Ltd.",
    "D02212C": "Xperio Labs Ltd.",
    "1007236": "ESTONE TECHNOLOGY INC",
    "E818636": "ARTECH SOLUTION CO.,LTD",
    "B8D8125": "XIAMEN XINDECO LTD.",
    "B8D8123": "iModesty Technology Corp.",
    "74E14A8": "aritec gmbh",
    "74E14AA": "AStar Design Service Technologies Co., Ltd.",
    "B8D8129": "Entotem LTD",
    "E4956E9": "eZeLink LLC",
    "BC6641E": "Lucent Trans Electronics Co., Ltd",
    "BC6641C": "Shenzhen Crave Communication Co.,ltd",
    "B01F813": "Sound United",
    "58FCDBB": "SWARCO TRAFFIC SYSTEMS GMBH",
    "BC6641B": "Sidus Novum Sp. z o. o.",
    "BC66417": "VSN Mobil",
    "B01F81D": "TAIWAN Anjie Electronics Co.,Ltd.",
    "58FCDBA": "Xmodus Systems GmbH",
    "F40E11C": "NIHON MEGA LOGIC CO.,LTD.",
    "F40E115": "E-SONG",
    "F40E113": "Shenzhen headsun technology",
    "141FBAC": "Swiss Electronic (Shenzhen) Co., Ltd",
    "B01F815": "SHENZHEN GRID TECHNOLOGY CO.,LTD",
    "B01F812": "Private",
    "F40E117": "Shenzhen Grandsun Electronic Co.,Ltd.",
    "141FBA1": "GloQuad",
    "F40E11E": "Elektronika Naglic d.o.o.",
    "141FBA8": "Shenzhen CATIC Information Technology Industry Co.,Ltd",
    "141FBAA": "Winsonic Electronics Co., Ltd.",
    "BC3400C": "Parlay Labs dba Highfive",
    "A43BFAA": "Plus One Japan Ltd.",
    "A43BFAC": "SHANGHAI XIETONG TECHNOLOGY INC.",
    "A43BFAB": "ALSTOM Strongwish (Shenzhen) Co., Ltd",
    "A43BFAE": "The Magstim Company Ltd.",
    "BC34001": "IPLINK Technology Corp",
    "7C70BCA": "Motec GmbH",
    "7C70BC1": "XD-GE Automation CO.,LTD",
    "A43BFA2": "Powell Industries",
    "D076501": "DAIKEN AUTOMACAO LTDA",
    "D07650B": "PelKorea",
    "D07650A": "InventDesign",
    "7419F84": "Cloudvue Technologies Corporation",
    "D076507": "ENCORED Technologies, Inc.",
    "7419F83": "Essential Trading Systems Corp",
    "7419F81": "Trend-tech Technology Co., Limited",
    "C85CE28": "LYNX Technik AG",
    "C85CE22": "SamabaNova Systems",
    "C85CE2C": "Shanghai Gaviota Intelligent Technology Co.,Ltd.",
    "4CEA413": "YENSHOW TECHNOLOGY CO.,LTD",
    "4CEA410": "Airflying",
    "4C74A7C": "N3com",
    "4CEA41D": "Jiangsu TSD Electronics Technology Co., Ltd",
    "F41A791": "Jide Car Rastreamento e Monitoramento LTDA",
    "5CF8383": "Shenzhen Tuozhu Technology Co., Ltd.",
    "F41A799": "TERZ Industrial Electronics GmbH",
    "90A9F72": "Suzhou Lingchen Acquisition Computer",
    "F41A79E": "Shenzhen Yizhao Innovation Technology Co., Ltd.",
    "90A9F7C": "SkyLine Limited Technologies Co., Ltd",
    "90A9F7D": "Shanghai Jiehezhi Technology Co., Ltd.",
    "90A9F71": "Shenzhen Chainway Information Technology Co., Ltd",
    "90A9F70": "Versta",
    "2CC44F1": "Joyoful",
    "2CC44F0": "Shenzhen Syeconmax Technology Co. Ltd",
    "2CC44FD": "Shanghai Fanlian Yunduan Electronic Technology Co.,Ltd",
    "1006481": "KYTRONICS",
    "100648C": "Dongguan Hongyexiang Industrial Co.,Ltd",
    "2CC44F9": "Kaspersky Lab Middle-East FZ-LLC",
    "1006488": "Zhejiang Chenghao Technology Co. , Ltd.",
    "2CC44FC": "Beijing Xiaoqu Zhipin Technology Co., Ltd",
    "EC9A0CC": "Fujian Zhongliang Zhihui Technology Co., Ltd",
    "EC9A0C8": "NXC Systems LLC",
    "A85B366": "Versuni",
    "EC9A0CB": "ReVibe Energy AB",
    "008DF47": "HIMSA",
    "008DF4D": "ID TECH SOLUTIONS PVT LTD",
    "EC9A0C7": "Jiangsu Maodu Yunke Medical Technology Co., Ltd",
    "008DF48": "ERA TOYS LIMITED",
    "008DF44": "Energy Team S.p.A.",
    "78392D3": "Zeta Alarms Limited",
    "8C5570B": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "8C55704": "Next Vision Tech(Ningbo)Co.,LTD",
    "28F8C6D": "ShenZhen Goodtimes Technoogy CO.,LTD",
    "28F8C63": "PANASONIC AUTOMOTIVE SYSTEM MALAYSIA",
    "28F8C61": "Annapurna labs",
    "28F8C66": "Shenzhen Hongdian technologies corporation.",
    "E8FF1E9": "Shenzhen Justinfo Technology Co.,Ltd",
    "E8FF1E5": "CARVE VIET NAM TECHNOLOGY COMPANY LIMITED",
    "8CA6823": "Royal Way General Trading LLC",
    "8CA682B": "POCKETALK CORP.",
    "8CA682C": "Netskope",
    "D0A011A": "Wuhan Tabebuia Technology Co.Ltd",
    "D0A0116": "OCEM Energy Technology",
    "D0A0115": "Shenzhen DOOGEE Hengtong Technology CO.,LTD",
    "D0A011E": "Annapurna labs",
    "78E9964": "Shenzhen Jin Hao Mi Technology Co., LTD",
    "78E996A": "Shenzhen Farben lnformation Technology CO.,Ltd.",
    "78E996B": "CelAudio (Beijing) Technology Inc",
    "34C8D6A": "Shenzhen Jooan Technology  Co., Ltd",
    "80A579A": "ViewSonic Corp",
    "80A5796": "Xiamen Pinnacle Electrical Co., Ltd",
    "80A579B": "Ground Control Technologies UK Ltd.",
    "80A5790": "Benano Inc.",
    "80A5793": "Tool-Temp AG",
    "80A5792": "Jiangsu wonder-working electric co., LTD",
    "901564A": "Bunka Shutter Co., Ltd.",
    "0863326": "OVT India pvt Ltd ",
    "0863329": "TZMedical Inc.",
    "48E663B": "Nakamura-Tome Precision  Industry Co.,Ltd.",
    "0863325": "A Paul Software Systems Pvt. Ltd.",
    "48E663E": "Matribox Intelligent Technology co.Ltd.",
    "48E663C": "BEIJING CUNYIN CHENGQI TECHNOLOGY CO., LTD.",
    "48E6635": "Hoypower Energy Co.,Ltd",
    "C4CC371": "Taiwan Inpro International Co.Ltd",
    "C4CC37B": "DRS Naval Power Systems, Inc.",
    "C4CC37E": "Joby Aviation, Inc.",
    "C4CC373": "Qingdao Goertek Intelligent Sensor Co.,Ltd",
    "C8FFBF0": "Shenzhen HC Electronic Technology Co.,LTD",
    "60A4345": "Hangzhou Lanly Technology Co., Ltd.",
    "C8FFBF4": "Beijing Jingyibeifang Instrument Co.,Ltd.",
    "C8FFBFB": "PubliBike SA",
    "C8FFBFD": "ALDES DomNexX",
    "1063A39": "Nexite",
    "3049503": "Morgan Schaffer",
    "5C87D84": "TELFI TECHNOLOGIES PRIVATE LIMITED",
    "1063A31": "Jacobs Technology, Inc.",
    "1063A3A": "ITAB Shop products",
    "5C87D86": "Shanghai Jianyi Technology Co., Ltd",
    "5C87D8D": "Zilia Technologies",
    "5C87D85": "Hangzhou Advanced Intelligent Manufacturing Systems Co.,Ltd.",
    "5C87D89": "Annapurna labs",
    "5C87D82": "Freeus LLC",
    "5C5A4C1": "Orchid Products Limited",
    "5C5A4CA": "Chengdu Skysoft Info&Tech Co.,Ltd.",
    "5C5A4CE": "AI-RIDER CORPORATION",
    "F87A396": "Annapurna labs",
    "F87A390": "HANGZHOU YONGXIE TECHNOLOGY CO., LTD",
    "F87A392": "Total-one TECHNOLOGY CO., LTD.",
    "F87A399": "Shenzhenshilemaikejiyouxiangongsi",
    "F87A397": "Shenzhen Electron Technology Co., LTD.",
    "3446631": "Chain Reaction Ltd",
    "3446633": "LA Clippers",
    "3446636": "China Motor Corporation",
    "08DA33B": "Shenzhen Fanxiang lnformation Technology Co.,Ltd",
    "08DA33D": "Shanghai ReeLink Global Communication Company LTD",
    "344663E": "Shenzhen ELECQ Technology Co.,Ltd",
    "08DA33A": "JAKA Robotics Co., Ltd.",
    "08DA330": "Smart and connective",
    "48E6C6D": "Digital Matter Pty Ltd",
    "48E6C6A": "AL HAMI INFORMATION TECHNOLOGY - L.L.C",
    "48E6C62": "Satlab",
    "F420559": "Lens Technology (Xiangtan) Co.,Ltd",
    "F420555": "Proqualit Telecom LTDA",
    "248625B": "Wynd Labs",
    "2486251": "Shenzhen LianwangRuijie Communication Technology Co, Ltd.",
    "48E6C68": "SANWA ELECTRONIC INSTRUMENT CO.,LTD",
    "248625D": "Chengdu HOLDTECS Co.,Ltd",
    "AC86D15": "Hangzhou Lingban Technology Co., Ltd",
    "AC86D13": "Inovaxe Corporation",
    "485E0E7": "Shanghai B&A automation.net",
    "AC86D18": "PK Sound",
    "485E0E1": "Dongguan YEHUO Technology Co.,LTD",
    "485E0E8": "ITFC",
    "485E0EE": "ELOOM SYSTEM",
    "C4FF847": "NEKMA M.Mesek P.Mesek B.Mesek",
    "C4FF846": "Lens Technology (Xiangtan) Co.,Ltd",
    "C4FF844": "MALHOTRA ELECTRONICS PRIVATE LIMITED",
    "ECA7B16": "Digital Audio Labs",
    "B84C87E": "Private",
    "C85CE20": "Fela Management AG ",
    "301F9A9": "Deep Sentinel ",
    "B84C871": "em-trak",
    "B84C877": "Beijing Jiyuan Automation Technology CO.,LTD",
    "B84C87D": "DFUN (ZHUHAI) CO,. LTD.",
    "C022F14": "Andritz AB",
    "C022F11": "COMMUP WUHAN NETWORK TECHNOLOGY CO.,LTD",
    "C022F1A": "Bosch Automotive Products (Suzhou) Co., Ltd.",
    "C022F1C": "Accelsius LLC",
    "C022F1D": "Envent Engineering",
    "50482C6": "WIKA Mobile Control GmbH & Co.KG",
    "5491AF2": "SHENZHEN SANMI INTELLIGENT CO.,LTD",
    "5491AF4": "DDPAI Technology Co.,Ltd",
    "48DA352": "Annapurna labs",
    "48DA357": "ERAESEEDS Co.,Ltd.",
    "5491AFA": "Zhuhai SHIXI Technology Co.,Ltd",
    "88A6EFE": "IONA Tech",
    "88A6EF7": "TRUWIN",
    "88A6EF0": "Energet LLC",
    "D015BB0": "FORTUNE MARKETING PRIVATE LIMITED",
    "B4A2EB2": "ONX Inc.",
    "D015BB4": "Esders GmbH",
    "D015BB3": "TePS'EG",
    "D015BB9": "Stellar Blu Solutions",
    "50482CC": "Telecam Technology Co.,Ltd",
    "C8F5D63": "BBPOS Limited",
    "68DA73B": "Gamber-Johnson LLC",
    "68DA73C": "Delta Surge Inc.",
    "68DA738": "STEL FIBER ELECTRONICS INDIA PRIVATE LIMITED",
    "68DA732": "SHENZHEN ALLDOCUBE SCIENCE AND TECHNOLOGY CO., LTD.",
    "50482C3": "Immunity Networks and Technologies Pvt Ltd",
    "58C41E1": "JLZTLink Industry ?Shen Zhen?Co., Ltd.",
    "58C41EE": "Beijing Qiangyun Innovation Technology Co.,Ltd",
    "68DA733": "Softronics Ltd",
    "68DA73D": "Sichuan GFS Information Technology Co.Ltd",
    "C86BBC2": "Vipaks + Ltd",
    "C86BBCD": "SCANTECH(HANGZHOU)CO.,LTD",
    "BC31983": "Shenzhen Qichang Intelligent Technology Co., Ltd.",
    "58C41E0": "Guangzhou TeleStar Communication Consulting Service Co., Ltd",
    "58C41E5": "Zhejiang Cainiao Supply Chain Management Co.,Ltd",
    "C86BBCA": "Alpha Bridge Technologies Private Limited",
    "BC3198A": "FUJITSU COMPONENT LIMIED",
    "B0FF72C": "Hopewheel info.Co.,Ltd.",
    "BC31981": "swiss-sonic Ultraschall AG",
    "90F4218": "Mi-Jack Products",
    "BC3198C": "Innoflight, Inc.",
    "90F4212": "Catvision Ltd.",
    "90F4219": "Twunicom Life Tech. Co., Ltd.",
    "90F4211": "BeEnergy SG GmbH",
    "80E4DAF": "Private",
    "6C2ADF2": "DAIKO ELECTRIC CO.,LTD",
    "FC61791": "Signalinks Communication Technology Co., Ltd",
    "6C2ADF7": "RootV",
    "7006925": "CANAAN CREATIVE CO.,LTD.",
    "7006920": "Techology, LLC",
    "7006923": "BOSSCCTV CO., LTD",
    "800A80F": "Private",
    "54083B0": "Shenzhen Liandian Communication Technology Co.LTD",
    "B8D812F": "Private",
    "54083B8": "Update Systems Inc.",
    "2C6A6FF": "Private",
    "D016F04": "BEIJING XIAOYUAN WENHUA CULTURE COMMUNICATION CO., LTD.",
    "D016F01": "QBIC COMMUNICATIONS DMCC",
    "6015927": "Unipi Technology s.r.o.",
    "D016F0B": "worldcns inc.",
    "E03C1C9": "Ocamar Technologies (Shanghai) Co.,Ltd.",
    "2C265FF": "Private",
    "90C682F": "Private",
    "A0BB3EF": "Private",
    "D093958": "Annapurna labs",
    "D093953": "Nesecure Telecom Pvt Ltd",
    "5C6AEC6": "FEMTOCELL",
    "7C45F93": "Hangzhou LUXAR Technologies Co., Ltd",
    "7C45F9B": "IngDan China-chip Electronic Technology(Wuxi) Co.,Ltd.",
    "C0EAC3E": "Beijing Zhongyuanyishang Technology Co Ltd",
    "C0EAC34": "Tokoz a.s.",
    "7C45F90": "SENSeOR",
    "5C6AECA": "Shenzhen Olax Technology CO.,Ltd",
    "7C45F9A": "qiio AG",
    "8C5DB27": "Cleartex s.r.o.",
    "8C5DB24": "CoreTigo",
    "C0EAC3C": "Trumeter",
    "C0EAC32": "NEXSEC Incorporated",
    "705A6F5": "Acer Connect",
    "8C5DB23": "Yuzhou Zhongnan lnformation Technology Co.,Ltd",
    "705A6F7": "WiBASE Industrial Solutions Inc.",
    "C4A5590": "Archermind Japan Co.,Ltd.",
    "84B386E": "NINGBO XINSUAN TECHNOLOGY CO.,LTD",
    "84B3868": "NetworX",
    "C4A559C": "ALTAM SYSTEMS SL",
    "C4A5599": "Shenzhen Meishifu Technology Co.,Ltd.",
    "0CCC47D": "GODOX Photo Equipment Co., Ltd.",
    "0CCC476": "Annapurna labs",
    "0CCC472": "Sun Yan International Trading Ltd.",
    "0CCC47A": "Rich Source Precision IND., Co., LTD.",
    "0CCC475": "DMECOM TELECOM CO.,LTD.",
    "84B386A": "Velocio Networks, Inc.",
    "0CCC47B": "Spot AI, Inc.",
    "0CCC47C": "KUMI ELECTRONIC COMPONENTS",
    "D461374": "Beijing TAIXINYUN Technology Co.,Ltd",
    "F0221D2": "Chonel Industry?shanghai?Co., Ltd.",
    "D461377": "Beijing Shudun Information Technology Co., Ltd",
    "A0024A5": "Dongguan Amsamotion Automation Technology Co., Ltd",
    "F0221DA": "Hangzhou Gold Electronic Equipment Co., Ltd",
    "4C74A75": "AGILITY ROBOTICS, INC.",
    "4C74A7E": "KYOCERA CORPORATION",
    "4C74A7B": "traplinked Gmbh",
    "4C74A71": "Shenzhen Hollyland Technology Co.,Ltd",
    "4C74A76": "ABB LV Installation Materials Co., Ltd. Beijing",
    "4C74A72": "Cyanview",
    "D09686C": "ECS s.r.l.",
    "D096862": "TMLake Technology Ltd.,",
    "D096867": "Private",
    "C49894D": "Jiangsu AIDriving Co.,Ltd.",
    "C498947": "Shenzhen C & D Electronics Co., Ltd.",
    "C498945": "shenzhen lanodo technology Co., Ltd",
    "D09686E": "withnetworks",
    "D09686D": " CertusNet Information Technology Co.,LTD",
    "C483721": "Shenzhen King Will Technology Co., LTD",
    "D4BABA6": "Shenzhen Yueer Innovation Technology Co., Ltd",
    "D4BABAB": "Qingdao Vzense Technology Co., Ltd.",
    "D4BABAA": "Actiontec Electronics Inc.",
    "D4BABA7": "Shenzhen Double Better Technology Co., Ltd",
    "D4BABA0": "SHENZHEN ACTION TECHNOLOGIES CO., LTD.",
    "D4BABA2": "GuangZhou Ostec Electronic Technology Co.,Limited",
    "C483720": "Compumedics Germany GmbH",
    "D4BABAE": "Camozzi Automation SpA",
    "C48372A": "VIE STYLE,INC,",
    "D420006": "HiAR Information Technology Co.,Ltd",
    "D420005": "Monolith Electric?Changzhou?Co.,Ltd.",
    "5847CAE": "AZURE SUMMIT TECHNOLOGY",
    "D420009": "WEATHEX CO., LTD.",
    "5847CA6": "Shenzhen C & D Electronics Co., Ltd.",
    "5847CA9": "Kingnuo Intelligent Technology (Jiaxing) Co., Ltd.",
    "E87829B": "Ampner Ltd",
    "D420003": "Arbe Robotics Ltd.",
    "5847CA3": "Fujian Helios Technologies Co., Ltd.",
    "883CC53": "shenzhen Feng Jing Sheng Electronics Technology Co.,Ltd",
    "883CC55": "Shanghai Ucan Automation Equipment Co., Ltd.",
    "1874E24": "Aggressive Electronics Manufacturing Services Pvt Limited",
    "883CC52": "NETGEN HITECH SOLUTIONS LLP",
    "18C3F48": "Shenzhen Liandian Communication Technology Co.LTD",
    "18C3F41": "Enertex Bayern GmbH",
    "18C3F43": "General Test Systems",
    "10DCB61": "Hitachi Energy Switzerland Ltd",
    "18C3F45": "Synaptics, Inc",
    "2C691D5": "LG Electronics Inc.",
    "FC6179B": "Fulian Precision Electronics(Tianjin) Co., Ltd",
    "303D518": "The Heil Co dba AWTI 3rd Eye Cam",
    "303D51D": "XOR UK Corporation Limited",
    "C0D391B": " Celliber Technologies Pvt Limited",
    "FC6179A": "Shenzhen Dptek Technology Co., Ltd.",
    "FC6179D": "Int'Act Pty Ltd",
    "0C7FED7": "Grandway Technology (Shenzhen) Limited",
    "0C7FED1": "Toast, Inc.",
    "0C7FEDC": "Shenzhen MoreSense Technology Co., Ltd. ",
    "8C5109E": "IROOTELLUCKY Corp.",
    "0C7FEDB": "TelX Systems",
    "8002F47": "Lazer Safe Pty Ltd",
    "8002F42": "Beijing Cybercore",
    "8C51096": "Avxav Electronic Trading LLC",
    "8002F48": "Annapurna labs",
    "8C51093": "SHENZHEN LDROBOT CO., LTD.",
    "8C51098": "nerospec",
    "8C5109D": "Surpedia Technologies Co., Ltd.",
    "7050E75": "Wall Box Chargers, S.L.",
    "7050E7A": "Guangzhou Tianhe High Tech Industrial Development Zone Zhongsheng Electrical Limited Company",
    "7050E79": "Elastics.cloud",
    "7050E72": "Electronic's Time SRL",
    "7050E73": "Skychers Creations ShenZhen Limited",
    "C4A10EE": "Alio, Inc",
    "C4A10E9": "XI'AN YEP TELECOM TECHNOLOGY CO.,LTD",
    "244E7B6": "Owasys Advanced Wireless Devices",
    "7050E77": "Yoctopuce",
    "C4A10E6": "Hainan World Electronic Science and Techology Co.,Ltd",
    "6C1524A": "STERIS",
    "C4A10EA": "Jiangsu Perceive World Technology Co.,Ltd.",
    "18A59C1": "Cuman",
    "3043D74": "FIBERME COMMUNICATIONS LLC",
    "381F26E": "Annapurna labs",
    "381F267": "RCE systems s.r.o.",
    "381F26B": "Deutronic Elektronik GmbH",
    "381F26A": "Sercomm Corporation.",
    "381F260": "JAESUNG INFORMATION & COMMUNICATION CO.LTD",
    "381F266": "NOITAC sp. z o.o. sp.k.",
    "6C93082": "ZHEJIANG XIAN DA Environmental Technology Co., Ltd",
    "3043D78": "Kesu (Shanghai) Electronic Technology Co., Ltd",
    "3043D7A": "Bodhi",
    "3043D79": "PK Solutions LLC",
    "3043D7B": "Motec GmbH",
    "6C93086": "Uconfree technology(shenzhen)limited",
    "6C93087": "Liberty AV Solutions",
    "6C93088": "Hangzhou Risco System Co.,Ltd",
    "0C86294": "Ag Express Electronics",
    "0C86293": "Annapurna labs",
    "1054D26": "Lanao Communication Technology Limited",
    "1054D28": "Annapurna labs",
    "988FE0E": "CEL Terminus (Shanghai) Information Technologies Co.,Ltd.",
    "988FE07": "China Huaxin Post and Telecom Technologies Co., Ltd.",
    "0826AED": "Veth Propulsion bv",
    "0826AEE": "Mass Electronics Pty Ltd",
    "DC3643C": "Orlaco Products B.V.",
    "18D7937": "JFA Electronics Industry and Commerce EIRELI",
    "DC3643A": "KUANTECH (CAMBODIA) CORPORATION LIMITED",
    "DC36438": "OAK INFORMATION SYSTEM CO.",
    "18D7932": "Hydrotechnik GmbH",
    "18D7935": "DongGuan Orient Electronics & Metal Co.,Ltd",
    "04EEE88": "MPEON Co.,Ltd",
    "04EEE8A": "Shenzhen JoiningFree Technology Co.,Ltd",
    "08F80D1": "Shenzhen DophiGo IoT Technology Co.,Ltd",
    "08F80DE": "Suzhou Sidi Information Technology Co., Ltd.",
    "1845B37": "Shenzhen Incar Technology Co., Ltd.",
    "08F80D8": "OpenYard LLC",
    "F4700CE": "Shenzhen WeProTalk Technology Co., Ltd.",
    "F4700CB": "Shanghai Risingpo Electronics CO.,LTD",
    "F4700C4": "Shenzhen Anycon Electronics Technology Co.,Ltd",
    "F4A4542": "TRI WORKS",
    "F4A4543": "Chongqing Hengxun Liansheng Industrial Co.,Ltd",
    "F4A4546": "Introl Design",
    "F4A4540": "NKT Photonics A/S",
    "7872647": "Conjing Networks Inc.",
    "787264A": "Typhoon HIL, Inc.",
    "7872644": "Asustor Inc.",
    "F4700C0": "HYUNSUNG CONVERGENCE ",
    "9880BBC": "Shenzhen Xin Kingbrand Enterprises Co., Ltd",
    "9880BBD": "Wyebot, Inc.",
    "9880BB3": "Annapurna labs",
    "986EE87": "Centro de Pesquisas Av Wernher Von Braun",
    "A0BB3E4": "COMSYS Communications Systems Service GmbH",
    "986EE85": "SUZHOU AUDITORYWORKS CO., LTD.",
    "986EE8D": "Changzhou Jiahao Radio&TV device CO.,LTD",
    "7C83349": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "7C83347": "ChengDU Yi Gong Intelligence Technology Co., Ltd.",
    "7C8334D": "MSV elektronika s.r.o.",
    "1CAE3E9": "China Convert Technology Co., Ltd.",
    "785EE84": "beijing Areospace Hongda optoelectronics technology co.,ltd",
    "785EE8C": "FINETOOLING TECHNOLOGY(HONG KONG)CO.,LIMITED",
    "38A8CD1": "Fujica System Co., ltd",
    "7C83343": "Beijing Changkun Technology Co., Ltd.",
    "38A8CDD": "Annapurna labs",
    "785EE8D": "Tachyon Networks",
    "283613E": "EGMedical, s.r.o.",
    "2836135": "Turing Video",
    "2836138": "Fuzhou Lesi Intelligent Technology Co., Ltd",
    "283613B": "Qorvo, Inc.",
    "283613C": "midBit Technologies, LLC",
    "6433B54": "Eagle Eye Networks, Inc",
    "283613A": "MAKEEN Energy",
    "6433B56": "MICROIT SRL",
    "6433B59": "Annapurna labs",
    "6433B58": "LACO Technologies",
    "6433B5B": "electroCore Inc.",
    "6433B57": "ABB Electrification Smart Power (ELSP)",
    "584849A": "Waoo",
    "44A92C7": "Efficient Building Automation Corp.",
    "44A92C1": "uimcom",
    "44A92C4": "NetX Networks a.s.",
    "F02A2BE": "Shenzhen CUCO Technology Co., Ltd",
    "F02A2B8": "Tenways Engineering Service Ltd",
    "A85B361": "PARMA LLC",
    "781305E": "Dongguan zhenxing electronic technology co.,limited",
    "A85B36D": "Adam Hall GmbH",
    "7813059": "Shenzhen C & D Electronics Co., Ltd.",
    "E878295": "SHEN ZHEN SKYSI WISDOM TECHNOLOGY CO.,LTD.",
    "E878299": "Ryu Tech. LTD",
    "E878292": "Galcon",
    "1874E20": "Ensor AG",
    "E87829C": "FairPhone B.V.",
    "1874E23": "CT Company",
    "D09FD9B": "Cablewireless Laboratory Co., Ltd",
    "1874E27": "Sansec Technology Co.,Ltd",
    "D09FD92": "Westar Display Technologies",
    "C0FBF94": "Minato Advanced Technologies inc",
    "C0FBF97": " LongSung Technology (Shanghai) Co.,Ltd.   ",
    "C8F5D6E": "HEITEC AG",
    "C8F5D6A": "HENAN FOXSTAR DIGITAL DISPLAY Co.,Ltd.",
    "88C9B33": "Fortive Setra-ICG(Tianjin)Co.,Ltd",
    "C0FBF99": "zxsolution",
    "C0FBF90": "Xerox Corporation",
    "C8F5D60": "MEIRYO TECHNICA CORPORATION",
    "88C9B36": "Hugo Techno",
    "20CE2A3": "Cuculus GmbH",
    "20CE2A0": "Annapurna labs",
    "78C2C06": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "88C9B3C": "Shenzhen Viewsmart Technology Co.,Ltd",
    "88C9B39": "Richbeam (Beijing) Technology Co., Ltd.",
    "20CE2A4": "Annapurna labs",
    "20CE2A6": "Radarxense BV",
    "1CA0EF5": "Nanjing Bilin Intelligent Identification Technology Co.,Ltd",
    "1CA0EF2": "Schneider-Electric(China)Co.Ltd,Shenzhen Branch",
    "1CA0EFA": "Henrich Electronics Corporation",
    "1CA0EF9": "Atlas Aerospace",
    "1CA0EFB": " BMK professional electronics GmbH",
    "1CA0EFE": "RDA Microelectronics Technologies (Shanghai) Co. , Ltd",
    "0C5CB5A": "Zhengzhou coal machinery hydraulic electric control Co.,Ltd",
    "6015928": "Yangzhou Wanfang Electronic Technology,CO .,Ltd.",
    "A453EE6": "Shenzhen Xunqi Interconnet Technology Co., Ltd",
    "6015926": "BEIJING KUANGSHI TECHNOLOGY CO., LTD",
    "0C5CB5D": "BSU Inc",
    "245DFCA": "Tata Sky Limited",
    "245DFC3": "Shenzhen Hailuck Electronic Technology CO.,LTD",
    "245DFCB": "ONLY",
    "0411195": "CEITA COMMUNICATION TECHNOLOGY CO.,LTD",
    "982782E": "SureFlap Ltd",
    "982782C": "KRISTECH Krzysztof Kajstura",
    "E86CC7B": "MORNSUN Guangzhou Science & Technology Co., Ltd.",
    "E86CC71": "ASSA ABLOY(GuangZhou) Smart Technology Co., Ltd",
    "0411194": "Bolicom Innovation Technology (BeiJing) Co.,LTD.",
    "041119C": "Haerbin Donglin Technology Co., Ltd.",
    "0411196": "ZPD technology Co., Ltd",
    "9827822": "Anhui Shengren Electronic Technology Co., Ltd",
    "446FD8E": "CTE",
    "9827821": "INFODAS GmbH",
    "446FD82": "BAYKON End\u00fcstriyel Kontrol Sistemleri San. ve Tic. A.\u015e.",
    "78D4F1B": "Jiangsu byzoro intelligent technology Co.,Ltd",
    "446FD87": "ITC",
    "A0024AA": "Guangdong Jinpeng Technology Co. LTD",
    "78D4F11": "Silla Industries",
    "A0024AC": "Encroute AB",
    "A0024A8": "Beijing Lyratone Technology Co., Ltd",
    "A453EEB": "Viper Design, LLC",
    "8CAE490": "Ouman Oy",
    "A0024A7": "ENNEBI ELETTRONICA SRL",
    "8CAE49B": "Suzhou Guowang Electronics Technology Co., Ltd.",
    "8C476EB": "Faravid Communication&Data Analysis",
    "8C476E1": "TelWare Corporation",
    "8C476E4": "Shenzhen Juding Electronics Co., Ltd.",
    "8C476EE": "Annapurna labs",
    "8C476EC": "Edge Networks Inc",
    "8C476E3": "Shanghai Satellite Communication Technology Co.,Ltd",
    "8C476E5": "Square Inc.",
    "8CAE494": "Jiangsu Sixingda Information Technology Co., Ltd.",
    "6879120": "PCTEL, Inc.",
    "6879126": "APPOTRONICS CO., LTD",
    "A03E6B0": "s&t embedded GmbH",
    "8411C2C": "Provision-ISR",
    "8411C2D": "Goldmund Switzerland",
    "8411C27": "Ei3 Corporation",
    "8411C25": "AIBIoT GmbH",
    "6879128": "ShangHai Aigentoo Information Technology Co., Ltd",
    "6879123": "Stephan Electronics SARL",
    "5895D85": "elgris UG",
    "DC4A9E2": "Annapurna labs",
    "DC4A9EB": "Maxvision Technology Corp.",
    "FCCD2F4": "Genitek Engineering sprl",
    "CC4F5CE": "Beijing Techao Weijia Technology Limited",
    "FCCD2F1": "Siren Care(Shanghai) information and technology company",
    "FCCD2FD": "Shenzhen Smartbyte Technology Co., Ltd.",
    "FCCD2F3": "Xmitech Technology Co., Limited",
    "5895D81": "shenzhen UDD Technologies,co.,Ltd",
    "CC4F5C3": "Shanghai Zenchant Electornics Co.,LTD",
    "CC4F5C9": "Dtrovision",
    "CC4F5CA": "AZ-TECHNOLOGY SDN BHD",
    "CC4F5CD": "Beijing Neutron Technology CO.,LTD.",
    "98FC846": "ZERONE CO., LTD.",
    "18FDCB7": "ENERGIE IP",
    "18FDCB0": "Shenzhen Rui jiali Electronic Technology Co. Ltd.",
    "58208A8": "SAMIL CTS Co., Ltd.",
    "18FDCBB": "TRANSLITE GLOBAL LLC",
    "98FC845": "Zymbit",
    "18FDCB9": "CreyNox GmbH",
    "C0619AE": "Zhejiang Haikang Science And Technology Co.,Ltd",
    "C0619A0": "Paragon Robotics LLC",
    "C0619A3": "LYAND ACOUSTIC TECHNOLOGY CO.,LTD.",
    "C0619A1": "KidKraft",
    "C0619AC": "JAM-Labs Corp",
    "98FC841": "go-e GmbH",
    "4C93A6A": "Hanwang Technology Co.,Ltd",
    "28B77C6": "Shanghai Taiji Software Co.,Limited",
    "28B77CD": "Enedo Finland Oy",
    "28B77CC": "AnyLink LLC",
    "5C857EE": "Guoyi Liangzi (Hefei) Technology Co., Ltd(CIQTEK)",
    "F469D54": "Stype CS d.o.o.",
    "F469D5D": "Nantong ZYDZ Electronic.,Co.Ltd",
    "5C857E9": "Express LUCK Industrial Ltd.",
    "D014116": "Ahnnet",
    "1C88791": "ANDRA Sp. z o. o.",
    "E4956EC": "Shenzhen Arronna Telecom Co.,Ltd",
    "D01411D": "Guangdong Shiqi Manufacture Co., Ltd.",
    "304950E": "IoTmaxx GmbH",
    "3049500": "Guangzhou Lian-med Technology Co.,Ltd.",
    "F0D7AF8": "SHEN ZHEN MICHIP TECHNOLOGIES CO.,LTD.",
    "94FBA7D": "Rosenberger Technologies Co.,Ltd.",
    "3CFAD36": "Nox Medical",
    "CCC2610": "Ebiologic Technology Co., Ltd.",
    "F0D7AF4": "ADAM Audio GmbH",
    "CCC261D": "Dspread Technology (Beijing) Inc.",
    "CCC2615": "Viper Design, LLC",
    "CCC2619": "BYTERG LLC",
    "480BB2E": "Beijing MFOX technology Co., Ltd.",
    "F0D7AF6": "Anord Mardix (USA) Inc.",
    "38F7CDC": "Shenzhen MADIGI Electronic Technology Co., Ltd",
    "38F7CD8": "BlastWave Inc.",
    "38F7CD4": "NORDI TELEKOMMUNIKATSIOONI O\u00dc",
    "38F7CD9": "RFbeam Microwave GmbH",
    "7069797": "Intelitech SIA",
    "7069799": "Faurecia Clarion Electronics (Dongguan) Co., Ltd",
    "38F7CD5": "Shanghai qinzhuo Electronic Co., Ltd.",
    "7069796": "Beijing Security Union Information Technology Co.,Ltd",
    "E8B470E": "UNICACCES GROUPE",
    "E8B470B": "Digifocus Technology Inc.",
    "94FBA78": "Silver-I Co.,LTD.",
    "E8B4701": "Autocom Diagnostic Partner AB",
    "F490CB2": "ICE Gateway GmbH",
    "C09BF42": "Hitachi High-Tech Materials Corporation",
    "848BCD3": "Annapurna labs",
    "C09BF48": "SHENZHEN WINS ELECTRONIC TECHNOLOGY CO., LTD",
    "F490CB1": "DELEM BV",
    "9405BBD": "Sunthink S&T Development Co.,Ltd",
    "9405BB1": "Dongguan Kingtron Electronics Tech Co., Ltd",
    "9405BB3": "Neutrik AG",
    "DC44271": "Tesla,Inc.",
    "94CC048": "CircuitWerkes, Inc.",
    "94CC044": "ProConnections, Inc.",
    "9405BB4": "Shenzhen Baolijie Technology Co., Ltd.",
    "94CC040": "Hangzhou Yongkong Technology Co., Ltd.",
    "646266C": "Jiangsu Aisida Electronic Co.,Ltd",
    "94CC047": "Gowing Business And Contracting Wenzhou Co., LTD",
    "14AE85C": "IO Industries Inc.",
    "14AE855": "AZ-TECHNOLOGY SDN BHD",
    "14AE854": "CENTERVUE SPA",
    "14AE85E": "Sercomm Corporation.",
    "14AE85B": "NTC SOFT",
    "14AE853": "IFLYTEK CO.,LTD.",
    "14AE856": "TMG TE GmbH",
    "90E2FCA": "Power Engineering & Manufacturing, Inc.",
    "90E2FC6": "Sindoh Techno Co., Ltd.",
    "90E2FC3": "Shenzhen Hisource Technology Development CO.,Ltd.",
    "90E2FC5": "TOTALONE TECHNOLOGY CO., LTD.",
    "90E2FC1": "Yite technology",
    "B0B3533": "AD HOC DEVELOPMENTS S.L",
    "B0B3538": "VOXISCOM",
    "B0B3534": "Innotas Elektronik GmbH",
    "B0B3531": "Sprocomm Technologies CO.,LTD.",
    "90E2FCE": "DevCom spol. s r.o.",
    "90E2FC7": "Fair Winds Digital srl",
    "B0B3536": "Hangzhou Hikrobot Technology Co., Ltd.",
    "3CFAD3E": "Mirico",
    "402C76E": "LS Energy Solutions",
    "402C76C": "gridX GmbH",
    "50DE196": "OCEANCCTV LTD",
    "50DE198": "IVATIV, INC",
    "200A0D4": "Virtium",
    "A0224EB": "All Inspire Health Inc.",
    "200A0D9": "Welzek (Beijing) Technologies  Co, Ltd",
    "200A0D0": "halstrup-walcher GmbH",
    "0069675": "Shenzhen Xiao Bi En Culture Education Technology Co.,Ltd.",
    "0069670": "Annapurna labs",
    "0069673": "Suzhou Radiant Lighting Technology Co.,Ltd",
    "A0224E5": "Zhuhai Cheer Technology Co., LTD.",
    "A0224ED": "Digifocus Technology Inc.",
    "8C147DC": "Reynaers Aluminium",
    "A0224E2": "Closed Joint-Stock Company \"NORSI-TRANS\"",
    "A0224E9": "Delta Tau Data Systems, Inc.",
    "643139D": "ZHEJIANG MOORGEN INTELLIGENT TECHNOLOGY CO.,LTD",
    "6431390": "SHENZHEN EMEET INTELLIGENT TECHNOLOGY CO., LTD.",
    "5062552": "ShenZhen ChuangMo Electronics Technology Co., Ltd",
    "643139E": "ATG UV Technology",
    "C4954D5": "Marble Automation",
    "643139B": "Alphago GmbH",
    "5062554": "XSLAB Inc.",
    "5062556": "Shenzhen Sinway South Technology Co., Ltd",
    "4403778": "Gemmy Electronics (Shenzhen) Co, Ltd",
    "4403779": "SHENZHEN UT-KING TECHNOLOGY CO.,LTD",
    "506255B": "CHENGDU COVE TECHNOLOGY CO.,LTD",
    "440377C": "BIG Climatic Manufacture, Co. LTD, Zhongshan Branch",
    "04D16EA": "Metra Electronics",
    "04D16E6": "ETL Elektrotechnik Lauter GmbH",
    "04D16E3": " Beijing Huaxia Qixin Technology Co., Ltd.",
    "54A493C": "BJ COTYTECH TECHNOLOGY CO.,LTD",
    "54A4932": "genua GmbH",
    "04D16E0": "INTRIPLE, a.s.",
    "04D16EB": "National Radio & Telecommunication Corporation - NRTC",
    "10DCB6D": "LeoLabs",
    "54A4936": "Hannto Technology Co., Ltd",
    "10DCB69": "Fuzhou Rockchip Electronics Co.,Ltd",
    "10DCB63": "HANACNS ",
    "10DCB65": "Milesight Taiwan",
    "10DCB6B": "Eyeball Fintech Company",
    "4011756": "ShenZhen LanShuo Communication Equipment CO.,LTD.",
    "2085937": "Great Lite International",
    "4C4BF99": "Tecnoplus Srl",
    "44D5F29": "Auctus Technologies Co.,Ltd.",
    "44D5F27": "Shenzhen Qiutian Technology Co.,Ltd",
    "44D5F2E": "Joint-Stock Company Research and Development Center \"ELVEES\"",
    "9806372": "Summa nv",
    "980637E": "Shanghai Jinnian information technology Co. Ltd",
    "2415104": "Annapurna labs",
    "44D5F2C": "neocontrol solu\u00e7\u00f5es em automa\u00e7\u00e3o",
    "D05F641": "Hangzhou ToupTek Photonics Co., Ltd.",
    "D05F647": "Beijing Core Shield Group Co., Ltd.",
    "2C16BD8": "Shenzhen elink smart Co., ltd",
    "2C16BDA": "Shenzhen Haiying Wire Tech Co., Ltd.",
    "FCA47A1": "Shenzhen VMAX New Energy Co., Ltd.",
    "FCA47A3": "Cliptech Industria e Comercio Ltda",
    "B4A2EB4": "Softel SA de CV",
    "D0C857B": "CHUNGHSIN INTERNATIONAL ELECTRONICS CO.,LTD.",
    "8C593C6": "Qbic Technology Co., Ltd",
    "8C593CD": "IDRO-ELETTRICA S.P.A.",
    "D0C8571": "DALI A/S",
    "8C593C3": "Chongqing beimoting technology co.ltd",
    "8C593C0": "Fujian Chaozhi Group Co., Ltd.",
    "BC97408": "Gaodi Rus",
    "6095CE5": "AdvanWISE Corporation",
    "BC9740B": "ForoTel",
    "848BCDD": "ENGISAT LDA",
    "848BCD7": "Smart\u00a0Code (Shenzhen) Technology Co.,Ltd",
    "848BCD8": "Dunst tronic GmbH",
    "848BCDE": "Emotiv Inc",
    "848BCDA": "Sphera Telecom",
    "B0FD0B4": "Fasii Information Technology (Shanghai) Ltd.",
    "B0FD0B0": "TAE HYUNG Industrial Electronics Co., Ltd.",
    "C82C2B0": "Fungible, Inc.",
    "E41E0AE": "Shanghai LeXiang Technology Co., Ltd",
    "E41E0AD": "ROMO Wind A/S",
    "E41E0AC": "TELETASK BELGIUM",
    "C82C2BA": "Shiftall Inc.",
    "C82C2B9": "BIOT Sp. z o.o.",
    "C863144": "Shenzhen Zero Zero Infinity Technology Co.\uff0cLtd.",
    "C86314D": "Telematix AG",
    "34E1D1E": "Annapurna labs",
    "34E1D10": "Tianjin Sublue Ocean Science & Technology Co., Ltd",
    "C863146": "GRINBI PARTNERS",
    "C863143": "TrackMan",
    "34E1D12": "Teton Camera LLC",
    "34E1D15": "Doki Technologies Limited",
    "C863147": "Shenzhen Wesion Technology Co., Ltd",
    "FCD2B68": "Oviss Labs Inc.",
    "FCD2B65": "Grandway Technology (Shenzhen) Limited",
    "FCD2B60": "CG POWER AND INDUSTRIAL SOLUTIONS LTD",
    "FCD2B6B": "T CHIP DIGITAL TECHNOLOGY CO.LTD",
    "745BC54": "uGrid Network Inc.",
    "E44CC7E": "FLK information security technology Co,. Ltd",
    "E44CC78": "IAG GROUP LTD",
    "745BC58": "EDOMO Systems GmbH",
    "E44CC7A": "Muzik Inc",
    "4CBC98C": "Heliotis AG",
    "4CBC989": "Airtex Manufacturing Partnership",
    "4CBC987": "Voegtlin Instruments GmbH",
    "4CBC986": "Humanplus Intelligent Robotics Technology Co.,Ltd.",
    "E05A9F0": "Annapurna labs",
    "E05A9FD": "Mountz, Inc.",
    "E05A9F5": "TRYEN",
    "E05A9F2": "Chengdu Song Yuan Electronic Technology Co.,Ltd",
    "E05A9FA": "Contemporary Amperex Technology Co., Limited",
    "4C65A8B": "ZMIN Technologies",
    "38B19E1": "Freedompro Srl",
    "283638C": "Swisson AG",
    "D8860B1": "Krspace",
    "38B19E9": "Doepke Schaltger\u00e4te GmbH",
    "D425CC8": "DOLBY LABORATORIES, INC.",
    "D425CC7": "BlueCats US, LLC",
    "D425CC2": "MusicLens Inc.",
    "D425CC5": "bvk technology",
    "D425CCC": "POSNET Polska S.A.",
    "CCD39D1": "Evoko Unlimited AB",
    "CCD39DA": "Lubelskie Fabryki Wag FAWAG S.A.",
    "38B19E7": "Beijing Memblaze Technology Co Ltd",
    "9C69B41": "EA Technology Ltd",
    "4C917AA": "Erlab DFS SAS",
    "4C917AB": "AvertX",
    "4C917AE": "Annapurna labs",
    "9C69B4D": "\"Intellect module\" LLC",
    "6891D04": "G-TECH Instruments Inc.",
    "6CDFFB0": "Shenzhen HDCVT Technology",
    "6CDFFB2": "Sercomm Corporation.",
    "98F9C73": "Beijing Horizon Information Technology Co., Ltd",
    "9C431E2": "HAESUNG DS",
    "7CBC84B": "Guangzhou Puppyrobot Technology Co.Ltd Beijing Branch",
    "7CBC840": "AG Neovo",
    "7CBC849": "HITIQ LIMITED",
    "7CBC84C": "Tibit Communications",
    "7CBC842": "3S Technology Co., Ltd.",
    "98F9C7C": "ShenZhen Chuangwei Electronic Appliance Co.,Ltd",
    "7CBC844": "CONTINENTAL",
    "98F9C7B": "HIROIA Communications Pte. Ltd. Taiwan Branch",
    "0CFE5D1": "Fender Musical Instrument",
    "0CFE5DB": "YINUO-LINK LIMITED",
    "0CFE5DD": "Maksat Technologies P Ltd",
    "6C5C3D2": "Vertiv Industrial Systems",
    "0CFE5D9": "Celerway Communication AS",
    "1CFD08E": "MESHBOX FOUNDATION PTE. LTD.",
    "1CFD08D": "Tianjin Keyvia Electric Co.,Ltd",
    "A83FA15": "Sercomm Corporation.",
    "A83FA10": "Imecon Engineering SrL",
    "1CFD08A": "Banmak Technogies Co.,Ltd",
    "1CFD081": "Shenzhen SEWO Technology Co.,Ltd.",
    "6C5C3D3": "KWONG MING ELECTRICAL MANUFACTORY LIMITED",
    "3C6A2C9": "WICKS Co., Ltd.",
    "A83FA19": "Shenzhen ITLONG Intelligent Technology Co.,Ltd",
    "1CFD08C": "Shanghai YottaTech Co Ltd (\u4e0a\u6d77\u5c27\u5b83\u79d1\u6280\u6709\u9650\u516c\u53f8\uff09",
    "300A604": "AVIC JONHON OPTRONIC TECHNOLOGY CO., LTD.",
    "300A608": "Bronkhorst High-Tech BV",
    "A02833E": "Precision Planting, LLC.",
    "A4ED43B": "Paragon Business Solutions Ltd.",
    "A4ED432": "Shanghai Mission Information Technologies (Group) Co.,Ltd",
    "A4ED430": "Sweam AB",
    "8489EC3": "Aerionics Inc.",
    "8489EC0": "SmartGiant Technology",
    "8489EC5": "Zephyr Engineering, Inc.",
    "8489EC9": "Shenzhen Xtooltech Co., Ltd",
    "A02833B": "FlexLink AB",
    "6891D03": "Ambitio LLC",
    "D47C444": "Sammi Onformation Systems",
    "9CF6DD6": "Shenzhen Xtooltech Co., Ltd",
    "3009F92": "Beijing Netswift Technology Co.,Ltd.",
    "3009F99": "Bonraybio",
    "3009F90": "Hurray Cloud Technology Co., Ltd.",
    "78C2C04": "Ory Laboratory Co., Ltd.",
    "C083594": "ANTS",
    "9CF6DDD": "Foshan Synwit Technology Co.,Ltd.",
    "04C3E60": "DREAMKAS LLC",
    "04C3E6C": "SHANTOU YINGSHENG IMPORT & EXPORT TRADING CO.,LTD.",
    "C083591": "Gemvax Technology ,. Co.Ltd",
    "3C427E6": "Edit Srl",
    "3C427E8": "UBTECH ROBOTICS CORP",
    "A0C5F20": "Quantlab Financial, LLC",
    "B44BD6D": "ELLETA SOLUTIONS LTD",
    "B44BD65": "ShenZhen Comstar Technology Company",
    "B44BD62": "Shenzhen Cudy Technology Co., Ltd.",
    "3C427EB": "Compal Electronics INC.",
    "A019B25": "SZBROAD   TECHNOLOGY (HK) CO.,LTMITED",
    "A019B21": "El Sewedy Electrometer Egypt S.A.E.",
    "2C48354": "GEARTECH LTD",
    "2C4835A": "Collatz+Trojan GmbH",
    "8C1CDA9": "Raychem RPG PVT. LTD.",
    "8C1CDA8": "ATOL LLC",
    "8C1CDA1": "GESAS GmbH",
    "3C24F0E": "GETMOBIT LLC",
    "3C24F0C": "Authentico Technologies",
    "8C1CDA3": "Structura Technology & Innovation",
    "8C1CDAB": "T+A elektroakustik GmbH & Co.KG",
    "480BB2D": "M2Lab Ltd.",
    "480BB20": "Ridango AS",
    "480BB21": "BAJA ELECTRONICS TECHNOLOGY LIMITED",
    "3C24F0A": "Shenzhen Bestway Technology Co., Ltd",
    "0C73EB5": "Husty M.Styczen J.Hupert Sp.J.",
    "0C73EB0": "Gemini Data Loggers (UK) Limited",
    "885FE86": "Shenzhen Xin Kingbrand Enterprises Co.,Ltd     ",
    "885FE8A": "Lisle Design Ltd",
    "301F9A6": "YiSheng technology  co.,LTD",
    "2CD1419": "Beijing Hexing Chuangxiang Technology Co., Ltd.",
    "301F9AD": "OLIMEX Ltd",
    "885FE80": "Jungheinrich Norderstedt AG & Co. KG",
    "885FE82": "Opto Engineering",
    "F041C89": "Shenzhen  Nufilo Electronic Technology Co., Ltd.",
    "88A9A7E": "Impact Distribution",
    "88A9A78": "psb intralogistics GmbH",
    "88A9A71": "Solaredge LTD.",
    "F041C86": "AED Engineering GmbH",
    "F041C87": "Nanchang BlackShark Co.,Ltd.",
    "88A9A77": "kimura giken corporation",
    "A4DA220": "General Electric Company",
    "DCE5335": "Controls Inc",
    "A4DA222": "Wyze Labs Inc",
    "A4DA22E": "Quuppa Oy",
    "9C431EA": "ST Access Control System Corp.",
    "282C021": "Astronics AES",
    "282C02E": "Capintec, Inc.",
    "282C02D": "SHENZHEN DOMENOR TECHNOLOGY LLC",
    "189BA54": "Innominds Software Inc",
    "4048FDA": "Shenzhen Yifang Digital Technology Co., LTD. ",
    "F8B5686": "Package Guard, Inc",
    "F8B568D": "Solarius",
    "3873EA1": "KingWay Information Co.,Ltd.",
    "3873EAB": "Shanghai ZoomSmart Technology Co., Ltd.",
    "EC9F0DE": "MAX Technologies",
    "AC1DDFE": "Duravit AG",
    "AC1DDFC": "Beijing Chunhong Technology Co., Ltd.",
    "EC9F0D3": "Waverly Labs Inc.",
    "34D0B86": "NumberFour AG",
    "34D0B81": "Shenzhen Bao Lai Wei Intelligent Technology Co., L",
    "EC9F0DD": "SKS Control Oy",
    "34D0B8E": "Kongqiguanjia (Beijing)Technology co.\uff0cltd",
    "34D0B87": "Shenzhen Rikomagic Tech Corp.,Ltd",
    "EC9F0D2": "DRB Systems",
    "AC1DDF0": "PiOctave Solutions Pvt Ltd",
    "AC1DDF6": "Shenzheng SenseTime Technology Co. Ltd",
    "741AE03": "Philips Personal Health Solutions",
    "CC22375": "Beijing Safesoft Greatmaker Co.,ltd",
    "CC22376": "Siemens AG Austria",
    "CC22377": "Shanghai Doit IOT Technology Co.,Ltd.",
    "741AE0B": "SHEN ZHEN YINGJIACHUANG ELECTRONICS TECHNOLOGY CO.,LTD.",
    "CC22370": "MEDCOM sp. z o.o.",
    "904E91E": "Shenzhen Cloudynamo Internet Technologies Co.,LTD.",
    "904E918": "CommandScape, Inc.",
    "904E91A": "Kaertech Limited",
    "2C279EC": "WAYCOM Technology Co.,Ltd",
    "189BA5B": "Eutron SPA",
    "34008A5": "Federal Aviation Administration ",
    "34298F7": "Dongguan Kingtron Electronics Tech Co., Ltd",
    "34298F0": "BlackEdge Capital",
    "189BA56": "Mantra Softech India Pvt Ltd",
    "189BA52": "Airprotec",
    "34298FE": "ARC Technology Co., Ltd",
    "34008A3": "Globex 99 LTD",
    "0CEFAFB": "Hubei Century Network Technology Co., Ltd",
    "34008A1": "ZQAM Communications",
    "28F5372": "Unicair Communication Tec Co., Ltd.",
    "78D8000": "Kverneland Group Mechatronics",
    "78D8006": "Alango Technologies Ltd",
    "28F5374": "Phyn LLC",
    "78D8007": "NimbeLink Corp",
    "28F537C": "Matricx Singapore Pte Ltd",
    "78D8002": "Shanghai Espacetime Technology Co.,Ltd.",
    "7CBACC8": "Collinear Networks Inc.",
    "F88A3C9": "withus",
    "7CBACC0": "TGT Limited",
    "7CBACC2": "Maco Lighting Pty. Ltd.",
    "A0C5F23": "Shenzhen Feima Robotics Technology Co.,Ltd",
    "4C65A8D": "Qingping Technology (Beijing) Co., Ltd.",
    "F88A3CC": "EXCETOP TECHNOLOGY (BEIJING) CO., LTD.",
    "F88A3CD": "THK Co.,LTD.",
    "8C147D3": "Remotec Technology Limited",
    "F023B92": "Raysgem Electronics and Technology Co.Ltd",
    "A0C5F25": "Spacepath Communications Ltd",
    "8C147D8": "V2 S.p.A.",
    "A0C5F26": "ShenZhen JuWangShi Tech",
    "8C147D7": "UrbanHello",
    "8C147D2": "Agilent S.p.A",
    "F023B97": "Transcend Building Automation control network corporation",
    "04714B7": "Omylis Pte Ltd",
    "04714B6": "Armstrong Fluid Technology",
    "08ED024": "Fio Corporation",
    "60D7E39": " LongSung Technology (Shanghai) Co.,Ltd.   ",
    "60D7E37": "Phase One A/S",
    "08ED02A": "Victiana SRL",
    "60D7E32": "Novo innovations Ltd",
    "08ED025": "Vigitron Inc.",
    "08ED026": "SANGO ELECTRONICS CO",
    "08ED023": "Jiangsu Logread Network Technology Co., LTD.",
    "98AAFC5": "SPM Instrument AB",
    "98AAFC7": "Shenzhen Hubsan Technology Co.\uff0cLTD.",
    "A411632": "Allgo Tech. (Beijing) Co.,Ltd",
    "A41163C": "Viloc",
    "1CC0E1C": "Nitto Seiko",
    "A411630": "Adetel Equipment",
    "A41163A": "ISE GmbH",
    "A411631": "INTER CONTROL Hermann K\u00f6hler Elektrik GmbH & Co.KG",
    "A41163E": "tinylogics",
    "144FD77": "Shenzhen V-Streaming Technology Co., Ltd.",
    "1CA0D3C": "LYT inc.",
    "1CA0D38": "Desarrollos y Soluciones Guinea I+D S.L.",
    "1CA0D3A": "DSM Messtechnik GmbH",
    "1CA0D3D": "ERATO (HK) Corporation Limited",
    "58E8766": "DivioTec Inc.",
    "A411634": "AlterG, Inc.",
    "1CA0D31": "Jabil circuit italia srl",
    "40F3856": "Lennox International Incorporated",
    "40F3859": "Fast Precision Technologies Co. Ltd.",
    "38FDFEC": "New Garden Co., Ltd.",
    "50A4D0B": "ZHENG DIAN ELECTRONICS LIMITED",
    "50A4D01": "Beijing ANTVR Technology Co., LTD",
    "50A4D02": "Seneco A/S",
    "807B857": "Chendu Ningshui Technology Co.,Ltd",
    "50A4D06": "PointGrab",
    "34049E7": "Pebble Technology",
    "34049E2": "EFD Induction",
    "34049EA": "i3 International Inc.",
    "A4580F0": "INNOPRO",
    "A4580F2": "BLOKS. GmbH",
    "40ED984": "Kendrion Kuhnke Automation GmbH",
    "40ED988": "GUANGZHOU AURIC INTELLIGENT TECHNOLOGY CO.,LTD.",
    "40ED983": "Knox Company",
    "500B919": "Machfu, Inc.",
    "500B915": "jiangsu zhongling high-tech CO.,LTD.",
    "500B91A": "New Audio LLC",
    "7CCBE2A": "Shanghai Institute of Applied Physics, Chinese Academy of Sciences",
    "500B911": "SPD Development Company Ltd",
    "500B91B": "thumbzup UK Limited",
    "4865EE9": "VideoStitch, Inc",
    "4865EE2": "CaptionCall",
    "4865EEA": "Shenzhen Inpor cloud Computing Co., Ltd.",
    "4865EEC": "DNV GL",
    "244E7B8": "Cyber1st",
    "AC64DD6": "Kpnetworks Ltd.",
    "AC64DD0": "Jia-Teng",
    "AC64DD9": "Micro Connect Pty Ltd",
    "AC64DD5": "SHANGHAI ZTE TECHNOLOGIES CO.,LTD",
    "1CC0E1E": "Yun Yang Fire Safety Equipment Co.,Ltd.",
    "1CC0E15": "Kids Wireless Inc",
    "4CE173A": "jvi",
    "4CE1732": "Lenovo Data Center Group",
    "4CE1737": "Ers\u00fales Limited",
    "1CC0E11": "Hangzhou Kaierda Electric Welding Machine Co.,Ltd",
    "383A218": "Alicat Scientific",
    "F81D780": "Dongguan Shun Hing Plastics Limited",
    "383A21A": "Foresight Sports",
    "70F8E75": "Beijing Eehuu Technology Co.,Ltd.",
    "F81D786": "Zengge Co., Limited",
    "70F8E7E": "CUAV",
    "70F8E7B": "Photonfocus AG",
    "C4FFBC7": "Critical Link",
    "84E0F45": "Hangzhou Nationalchip Science & Technology Co.,Ltd.",
    "70F8E77": "NST Technology Limited Co.,Ltd.",
    "70F8E74": "CLIP Inc.",
    "84E0F49": "SHENZHEN HCN.ELECTRONICS CO.,LTD.",
    "84E0F44": "PetroInTrade",
    "70F8E7D": "System-on-Chip engineering",
    "84E0F4D": "Logos01 Srl",
    "C0D3914": "Vernier Software & Technology",
    "C0D3916": "Ernitec",
    "5CF2869": "Shenzhen VST Automotive Electronics Co., LTD",
    "C0D3910": "Fuzhou Jinshi Technology Co.,Ltd.",
    "58E8761": "Beijing Perabytes IS Technology Co., Ltd",
    "58E876E": " Baoruh Electronic Co., Ltd.",
    "58E8762": "Coala Life AB",
    "58E8767": "Chronos Technology Ltd.",
    "F0ACD7C": "Simprints Technology Ltd",
    "B0C5CA7": "SHENZHEN KTC TECHNOLOGY GROUP ",
    "F0ACD79": "U3storage Technologies Co., Ltd",
    "2836382": "SHENZHEN GOSPELL SMARTHOME ELECTRONIC CO., LTD.",
    "78CA83D": "Hubei Boyuan Zhijia Network Media Co. Ltd.",
    "2836383": "Sabinetek",
    "8C192DE": "Elcon AB",
    "D0D94FE": "APPOTRONICS CO., LTD",
    "8C192D3": "Greenfield Technology",
    "CCD31E5": "NTmore.Co.,Ltd",
    "CCD31E7": "Shenzhen Decnta Technology Co.,LTD.",
    "6891D0C": "Spraying Systems Co.",
    "6891D0D": "Fuzhou x-speed information technology Co.,Ltd.",
    "CCD31E8": "inoage GmbH",
    "C47C8D2": "Star2Star Communications, LLC",
    "C47C8DD": "Anhui GuangXing Linked-Video Communication Technology Co, Ltd.",
    "6891D08": "solvimus GmbH",
    "E0B6F51": "START TODAY CO.,LTD.",
    "50FF998": "Dolphin Concepts Limited",
    "50FF99A": "metraTec GmbH",
    "E0B6F5B": "Moog Crossbow",
    "986D352": "SHENZHEN FISE TECHNOLOGY HOLDING CO.,LTD.",
    "986D35A": "iWave Japan, Inc.",
    "1007232": "Diginet Control Systems Pty Ltd",
    "7C477CB": "Hangzhou Yiyitaidi Information Technology Co., Ltd.",
    "986D35B": "INTECH",
    "986D35E": "BAYCOM OPTO-ELECTRONICS TECHNOLGY CO., LTD.",
    "38FDFE1": "WAYTONE (BEIIJNG) COMMUNICATIONS CO.,LTD",
    "7C477C2": "POWERLAND LIMITED",
    "5CF286B": "Itron UK Limited",
    "38B8EB2": "barox Kommunikation GmbH",
    "38B8EB6": "MATRIXSTREAM TECHNOLOGIES, INC.",
    "38B8EBD": "Yellowbrick Data, Inc.",
    "1C8879B": "gekartel AG",
    "1C88790": "Newps co.,ltd",
    "1C8774E": "Quest Integrity",
    "1C8879A": "ITW-FEG",
    "1C8779E": "ASSYSTEM France",
    "1C8776E": "Artis GmbH",
    "1C87740": "Philips Personal Health Solutions",
    "1C8776B": "Hekatron Vertriebs GmbH",
    "1C87742": "Nichigaku",
    "1C87764": "RDP.RU",
    "1C87792": "SMARTMOVT TECHNOLOGY Co.\uff0c LTD",
    "1C87797": "TASC Systems Inc.",
    "1C8779B": "Beijing Geedeen Technology Co., Ltd",
    "8439BE1": "Guangzhou Heygears Technology Ltd",
    "40A36BC": "Onion Corporation",
    "70886B6": "Church & Dwight Co., Inc.",
    "70886B1": "Bitfinder Inc",
    "70886B8": "Cable Matters Inc.",
    "70886B9": "Shenzhen Coolhear Information Technology Co., Ltd.",
    "40A36B1": "TW-TeamWare",
    "CC1BE0E": "Cassia Networks",
    "800A800": "Golana Technology (Shenzhen) Co., Ltd.",
    "CC1BE0A": "Matter Labs Pty Ltd",
    "0055DA6": "OOO \"DEKATRON\"",
    "0055DA9": "Quantum Communication Technology Co., Ltd.,Anhui",
    "CC1BE01": "Beijing Daotongtianxia Co.Ltd.",
    "A03E6B4": "Shenzhen Nufilo Inc.",
    "A03E6BB": "KoCoS Messtechnik AG",
    "C88ED1D": "PHOENIX ENGINEERING CORP.",
    "C88ED1C": "Shanghai Bwave Technology Co.,Ltd",
    "1C21D1D": "Liscotech System Co., Ltd.",
    "1C21D1A": "LG CNS",
    "1C21D1B": "Global Design Solutions Ltd",
    "1C21D16": "Wuhan TieChi Detection Technology Co., Ltd.",
    "1C21D10": "Toyo System CO.,LTD.",
    "1C21D12": "Varaani Works Oy",
    "DC4427D": "Rohde&Schwarz Topex SA",
    "DC4427E": "VerifEye Technologies",
    "1C21D17": "Soundtrack Your Brand Sweden AB",
    "C88ED13": "Linx Technologies",
    "C88ED14": "Comlab AG",
    "C88ED17": "Ube, Inc. (dba Plum)",
    "C88ED15": "Fibergate.Inc",
    "B0C5CAA": "TEM Mobile Limited",
    "B0C5CAB": "RISECOMM  (HK) TECHNOLOGY CO. LIMITED",
    "B0C5CA2": "LOWOTEC GmbH",
    "78C2C05": "ShenZhen TuLing Robot CO.,LTD",
    "B437D1A": "Axiomatic Technologies Corporation",
    "549A11D": "Hangzhou duotin Technology Co., Ltd.",
    "74F8DB1": "GHL Advanced Technology GmbH & Co. KG",
    "885D909": "Gigatech R&D Corp.",
    "885D901": "ShenZhen Yuyangsheng technology company LTD",
    "885D906": "Hi-Profile Achievement (M) Sdn Bhd",
    "807B852": "Phoenix Co.,Ltd.",
    "64FB81B": "Sichuan Haige Actec Communication Technology Co.,Ltd.",
    "64FB811": "Narrative AB",
    "807B850": "Shiroshita Industrial Co., Ltd.",
    "807B858": "IDair, LLC",
    "549A11A": "VendNovation LLC",
    "80E4DAA": "Neutronics",
    "1CCAE32": "Insigma Inc",
    "80E4DAE": "Akenori PTE LTD",
    "2CD1418": "Minno LLC",
    "2CD1413": "AOptix Technologies, Inc",
    "1CCAE3B": "Dream Visions Co., LTD",
    "1CCAE3C": "Gahdeung Elecom",
    "1CCAE36": "TOKAI RIKA CO., LTD.",
    "9802D83": "Grammer EiA Electronics nv",
    "A0BB3EE": "Messtechnik Sachs GmbH",
    "9802D81": "SHENZHEN ATEKO PHOTOELECTRICITY CO LTD",
    "90C6822": "ekey biometric systems gmbh",
    "90C6821": "Shenzhen Photon Broadband Technology CO., LTD",
    "90C6829": "ACT",
    "2CD1410": "iCIRROUND Inc",
    "90C682A": "Beijing Acorn Networks Corporation",
    "9802D86": "Fritz Kuebler GmbH",
    "2C6A6FB": "Schneider Electric Korea",
    "2C265F6": "Appostar Technology Co. Ltd",
    "2C265F3": "shenzhen Clever Electronic Co., Ltd.",
    "2C265F0": "XIAMEN VORLINK IOT TECHNOLOGY CO.,LTD.",
    "F80278E": "Lit Technologies",
    "28FD80E": "T-Radio AS",
    "28FD803": "NUUO, Inc.",
    "0CEFAF5": "PREMIUM SA",
    "0CEFAF8": "BSX Athletics",
    "0CEFAF2": "LUMEL S.A.",
    "A44F29E": "Neotech Systems Pvt. Ltd.",
    "A44F29D": "HALLIBURTON",
    "F802785": "Electric Objects",
    "F802783": "3Shape Holding A/S",
    "F80278C": "Technology Research, LLC",
    "F802780": "Digatron Power Electronics GmbH",
    "1007238": "Ion Professional Solutions",
    "100723E": "First Chair Acoustics Co., Ltd.",
    "100723B": "Fujian Quanzhou Dong Ang Electronics Co., Ltd.",
    "1007237": "nanoTech Co., Ltd.",
    "D022129": "UAB &quot;SALDA&quot;",
    "3C39E7A": "iiM AG",
    "3C39E79": "Zone Controls AB",
    "3C39E77": "Sensor to Image GmbH",
    "D02212B": "Schleifenbauer Holding BV",
    "D022127": "Cliptech Industria e Comercio Ltda",
    "D02212E": "u::Lux GmbH",
    "74E14AD": "Knog Pty Ltd",
    "74E14AB": "Loctek Visual Technology Corp.",
    "D022125": "Shanghai Routech Co., Ltd",
    "B8D812B": "Docobo Limited",
    "E81863E": "Acopian Technical Company",
    "BC66414": "ARGUS-SPECTRUM",
    "E4956EB": "iConservo Inc",
    "E4956EA": "Red Point Positioning, Corp.",
    "74E14A5": "UTU Oy",
    "F40E116": "Alpha Design Technologies Pvt Ltd",
    "B01F81A": "Steffens Systems GmbH",
    "B01F819": "CIDE Interactive",
    "58FCDB6": "Timex Group USA Inc",
    "F40E11B": "BRADAR INDUSTRIA SA",
    "F40E118": "Zeepro Inc.",
    "BC3400D": "Hangzhou Linker Digital Technology Co., Ltd",
    "BC3400E": "LLD Technology Ltd.",
    "BC3400B": "FARO TECHNOLOGIES, INC.",
    "BC34008": "MATICA TECHNOLOGIES AG",
    "7C70BC6": "Bidgely",
    "141FBA5": "Inttelix Brasil Tecnologia e Sistemas Ltda",
    "7C70BCD": "mk-messtechnik GmbH",
    "BC34009": "Shenzhen PHilorise Technical Limited",
    "BC34005": "NDSL, Inc.",
    "141FBAD": "AJIS(DALIAN)co.,LTD",
    "141FBA0": "Shenzhen Mining Technology Co.,Ltd.",
    "D076502": "Happo Solutions Oy",
    "BC34002": "LifeSmart",
    "D07650C": "Electro-Motive Diesel",
    "A43BFA5": "BOI Solutions",
    "D076503": "TAPKO Technologies GmbH",
    "7419F8E": "Volacomm Co., Ltd",
    "7419F80": "Marmitek",
    "7419F86": "Baudisch Electronic GmbH",
    "38FDFE3": "Siemens AG, DI PA AE",
    "C85CE2B": "AloT Tech",
    "C4FFBC0": "Danego BV",
    "4CEA41B": "Hangzhou Hortwork Technology Co.,Ltd.",
    "4CEA418": "WUXI LATCOS TECHNOLOGY AUTOMATION Co.Ltd.",
    "4CEA412": "SHENZHEN ATC Technology Co., Ltd",
    "5CF8380": "Hunan Guoke supercomputer Technologu Co.,LTD",
    "5CF8389": "Benison Tech",
    "F41A79B": "Shenzhen High Speed Technology Co., Ltd",
    "F41A79C": "TA Technology (Shanghai) Co., Ltd",
    "F41A79A": "BIRTECH TECHNOLOGY",
    "F41A796": "eddylab GmbH",
    "F41A795": "Meta-Bounds Inc.",
    "1C5974A": "Viridi Parente, Inc.",
    "E4956E5": "Bucher Automation Budapest",
    "90A9F76": "Shenzhen Eevin Technology Co.,Ltd",
    "F41A790": "MetCom Solutions GmbH",
    "1006483": "illuminous LLC",
    "1006480": "FLEXTRONICS TECHNOLOGIES (INDIA) PVT LTD",
    "90A9F7B": "Kranze Technology Solutions, Inc.",
    "90A9F77": "GUANGDONG KERUIDE ELECTRICAL TECHNOLOGY CO., LTD.",
    "90A9F7A": "Chi Geng Technology Co., Ltd",
    "1006489": "Annapurna labs",
    "100648D": "Microvast Energy",
    "B0FD0BD": "Habana Labs LTD.",
    "2CC44FE": "Elcos srl",
    "2CC44F5": "MOHAN ELECTRONICS AND SYSTEMS (OPTIVISION)",
    "2CC44F6": "HM Corporation Ltd.",
    "EC9A0C9": "TIANJIN MEITENG TECHNOLOGY CO.,LTD",
    "EC9A0CE": "Risuntek Inc",
    "C898DB1": "Block, Inc.",
    "C898DBE": "qihangzhitong",
    "C898DBC": "Smartrend Manufacturing Group",
    "C898DB0": "Unicore Communications Inc.",
    "C898DB3": "Shenzhen Kedakeda Technology Co., Ltd.",
    "78392D9": "AVATR Co., LTD.",
    "78392DD": "Avantree Corporation",
    "10DCB6C": "BBPOS Limited",
    "78392D1": "LivEye GmbH",
    "78392DE": "Planeta Inform\u00e1tica Ltda",
    "D461379": "Mavenir Systems, Inc.",
    "8C5570D": "EPSa Elektronik & Pr\u00e4zisionsbau Saalfeld GmbH",
    "A0024A1": "Videndum Media Solutions Spa",
    "8C5570C": "Antronix Inc.,",
    "34008A6": "Sithon Technologies",
    "50482C4": "HY-LINE Technology GmbH",
    "E8FF1E6": "Partizan Security s.r.o.",
    "E8FF1E1": "hard & softWERK GmbH",
    "8CA6824": "China Information Technology Designing&Consulting Institute Co., Ltd.",
    "8CA6829": "Barkodes Bilgisayar Sistemleri Bilgi Iletisim ve Y",
    "8CA6820": "AI SECURITY Co.Ltd.",
    "8CA6826": "ShangHai Huijue Network Communication Equipment CO., Ltd.",
    "D0A0114": "SUMICO",
    "282C028": "Shenzhen Neoway Technology Co.,Ltd.",
    "78E9968": "Shenzhen Zhiting Technology Co.,Ltd",
    "34C8D69": "Laxton Group Limited",
    "78E996E": "Bita-International Co.,Ltd",
    "34C8D66": "Signalwing Corporation",
    "34C8D63": "kratos network pte.ltd.",
    "34C8D6D": "eight",
    "34C8D60": "Shenzhen Zhangyue Technology Co., Ltd",
    "80A5795": "Potron Technology Co.,Ltd.,",
    "80A579C": "BluArmor",
    "80A5794": "Hardened Networks",
    "80A579D": "Guangdong Province Ivsuan technology co., ltd",
    "80A579E": "Unplugged Technologies Ltd.",
    "9015647": "Hangzhou System Technology Co.,Ltd",
    "9015648": "Heliogen",
    "901564D": "Jiangsu Kangjie Data Co., Ltd.",
    "086332A": "Dynacom Communication",
    "0863328": "in.hub GmbH",
    "086332D": "akYtec GmbH",
    "48E6632": "Beijing Jingdong Qianshi Technology Co.,Ltd.",
    "48E6633": "MAKERFABS",
    "C4CC37D": "Netavo Global Data Services Ltd",
    "48E663D": "Neureality ltd",
    "C4CC379": "Skychers Creations ShenZhen Limited",
    "C4CC378": "SmartLicht systems co., ltd",
    "60A4343": "Shenzhen lncar Technology Co.,Ltd",
    "60A434C": "Annapurna labs",
    "C8FFBF7": "Indra Renewable Technologies",
    "5C5A4C9": "Linktech Systerm Technology Co.,Ltd",
    "5C87D8E": "Beijing Townsky Technology Co.,Ltd",
    "5C5A4C4": "YIHUA COMMUNICATIONS(HUIZHOU)CO.,LTD",
    "5C87D8C": "DRAGONGLASS TECHNOLOGY(SHENZHEN)CO.,LTD.",
    "1063A3E": "Annapurna labs",
    "5C5A4C0": "Jinchuan Group Co.,Ltd",
    "5C5A4CD": "Aeva, Inc.",
    "5C5A4CC": "tarm AG",
    "5C5A4C6": "Shenzhen Sunsoont Technology Co.,Ltd",
    "F87A393": "Overview Limited",
    "EC5BCDA": "CareSix Inc.",
    "EC5BCDC": "Quicklert Inc",
    "F87A395": "tatwah SA",
    "EC5BCD2": "Sfera Labs S.r.l.",
    "3446639": "Shenzhen C & D Electronics Co., Ltd.",
    "344663B": "Grohe AG",
    "3446638": "Bluesoo Tech (HongKong) Co.,Limited",
    "08DA337": "Origalys ElectroChem SAS",
    "08DA332": "Shenzhen Zhenghao lnnovation Techenology Co.,LTD",
    "F420558": "Huzhou Luxshare Precision Industry Co.LTD",
    "48E6C6E": "BEUPSYS",
    "F420557": "Beyond Laser Systems LLC",
    "F42055E": "Shenzhen Keyking Technology Limited",
    "F420551": "Sichuan Gengyuan Technology Co.,Ltd",
    "2486258": "Ningbo Sigmatek Automation Co., Ltd.",
    "F420554": "Pulsotronic-Anlagentechnik GmbH",
    "2486256": "ADTEK",
    "248625E": "Hangzhou UPAI Technology Co., Ltd",
    "AC86D1B": "shenzhen ceita communications technology co.,ltd",
    "AC86D1A": "EMIT GmbH",
    "AC86D17": "Quantum-Systems GmbH",
    "AC86D1E": "Retina Development B.V.",
    "AC86D1C": "MOTUSTECHS(Wuhan)Co.,Ltd.",
    "AC86D19": "Annapurna labs",
    "485E0EA": "AIGNEP SPA",
    "485E0E5": "GlobalXtreme",
    "C4FF84A": "Openeye",
    "C4FF84E": "Elbit system EW and SIGINT Elisra ltd",
    "C4FF84D": "AOSAISHI (HONG KONG) CO.,LIMITED",
    "141FBA6": "Hitachi Rail RCS France SAS",
    "ECA7B19": "Print International Limited",
    "286FB9": "Nokia Shanghai Bell Co., Ltd.",
    "08EA44": "Extreme Networks Headquarters",
    "F4EAB5": "Extreme Networks Headquarters",
    "B87CF2": "Extreme Networks Headquarters",
    "E0A129": "Extreme Networks Headquarters",
    "A8C647": "Extreme Networks Headquarters",
    "A473AB": "Extreme Networks Headquarters",
    "0C9B78": "Extreme Networks Headquarters",
    "A4C7F6": "Extreme Networks Headquarters",
    "B42D56": "Extreme Networks Headquarters",
    "887E25": "Extreme Networks Headquarters",
    "001977": "Extreme Networks Headquarters",
    "38E2CA": "Katun Corporation",
    "B05B99": "Sagemcom Broadband SAS",
    "E80AB9": "Cisco Systems, Inc",
    "78465F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "9075DE": "Zebra Technologies Inc.",
    "80BA16": "Micas Networks Inc.",
    "10E992": "INGRAM MICRO SERVICES",
    "78F276": "Cyklop Fastjet Technologies (Shanghai) Inc.",
    "20F83B": "Nabu Casa, Inc.",
    "F01B24": "zte corporation",
    "B84C87": "IEEE Registration Authority",
    "E4F14C": "Private",
    "10061C": "Espressif Inc.",
    "E00630": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04D168": "Sunplus Technology Co., Ltd.",
    "644C69": "CONPROVE",
    "DCD160": "Tianjin Changdatong Technology Co.,LTD",
    "644ED7": "HP Inc.",
    "9CE330": "Cisco Meraki",
    "04A959": "New H3C Technologies Co., Ltd",
    "0CB983": "Honor Device Co., Ltd.",
    "2CB301": "Honor Device Co., Ltd.",
    "000E72": "Arca Technologies S.r.l.",
    "E0C2B7": "Masimo Corporation",
    "C88BE8": "Masimo Corporation",
    "E4C767": "Intel Corporate",
    "902D77": "Edgecore Americas Networking Corporation",
    "6CFBED": "GN Audio A/S",
    "E4F27C": "Juniper Networks",
    "DC7CF7": "China Mobile Group Device Co.,Ltd.",
    "906C4B": "Advance Security Electronics",
    "D48AFC": "Espressif Inc.",
    "E465B8": "Espressif Inc.",
    "E001C7": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "B4DF91": "Cisco Meraki",
    "98EE8C": "zte corporation",
    "FC500C": "Sitehop Ltd",
    "50E478": "Sichuan AI-Link Technology Co., Ltd.",
    "90C710": "zte corporation",
    "48D475": "Lampuga GmbH",
    "601AC7": "Nintendo Co.,Ltd",
    "48DA35": "IEEE Registration Authority",
    "A881FE": "Luxul Tech Co., Ltd",
    "649A08": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "50F222": "EM Microelectronic",
    "307A57": "Accuenergy (CANADA) Inc",
    "CCEB5E": "Xiaomi Communications Co Ltd",
    "60C78D": "Juniper Networks",
    "DC5193": "zte corporation",
    "7478A6": "Fortinet, Inc.",
    "001982": "SmarDTV Corporation",
    "50FE0C": "AzureWave Technology Inc.",
    "BCFD0C": "Shenzhen Phaten Tech. LTD",
    "1865C7": "Dongguan YIMO Technology Co.LTD",
    "0C01C8": "DENSO Co.,Ltd",
    "F42E48": "zte corporation",
    "ACD75B": "Sagemcom Broadband SAS",
    "B4C2E0": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "040986": "Arcadyan Corporation",
    "481BA4": "Cisco Systems, Inc",
    "F0EE7A": "Apple, Inc.",
    "203AEB": "zte corporation",
    "5876B3": "Ubee Interactive Co., Limited",
    "E87072": "Hangzhou BroadLink Technology Co.,Ltd",
    "D8DAF1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54443B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C7075": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC03FA": "Vantiva USA LLC",
    "48F7C0": "Vantiva USA LLC",
    "480033": "Vantiva USA LLC",
    "14987D": "Vantiva USA LLC",
    "58238C": "Vantiva USA LLC",
    "80B234": "Vantiva USA LLC",
    "8417EF": "Vantiva USA LLC",
    "A456CC": "Vantiva USA LLC",
    "70037E": "Vantiva USA LLC",
    "441C12": "Vantiva USA LLC",
    "5C7695": "Vantiva USA LLC",
    "AC4CA5": "Vantiva USA LLC",
    "00CB7A": "Vantiva USA LLC",
    "F8D2AC": "Vantiva USA LLC",
    "CCF3C8": "Vantiva USA LLC",
    "40F3B0": "Texas Instruments",
    "149CEF": "Texas Instruments",
    "80C41B": "Texas Instruments",
    "3468B5": "Texas Instruments",
    "58AD12": "Apple, Inc.",
    "60FDA6": "Apple, Inc.",
    "80A997": "Apple, Inc.",
    "348C5E": "Apple, Inc.",
    "D0B2C4": "Vantiva USA LLC",
    "80C6AB": "Vantiva USA LLC",
    "383FB3": "Vantiva USA LLC",
    "641236": "Vantiva USA LLC",
    "0C75D2": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "D87766": "Nurivoice Co., Ltd",
    "B8A75E": "Wuxi Xinjie Electric Co.,Ltd",
    "8C53E6": "Wistron Neweb Corporation",
    "345E08": "Roku, Inc",
    "708185": "New H3C Technologies Co., Ltd",
    "50D33B": "cloudnineinfo",
    "84C8A0": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "C0E911": "RealNetworks",
    "8C3B4A": "Universal Global Scientific Industrial Co., Ltd.",
    "A88B28": "SHENZHEN DIYANG SMART TECHNOLOGY CO.,LTD.",
    "08F01E": "eero inc.",
    "18A5FF": "Arcadyan Corporation",
    "8C1AF3": "Shenzhen Gooxi Information Security CO.,Ltd.",
    "F4B6C6": "Indra Heera Technology LLP",
    "6C03B5": "Cisco Systems, Inc",
    "58C41E": "IEEE Registration Authority",
    "A89162": "Sophos Ltd",
    "709883": "SHENZHEN KAYAN ELECTRONICS., LTD.",
    "F43A7B": "zte corporation",
    "689E29": "zte corporation",
    "C87023": "Altice Labs S.A.",
    "E44097": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "DCB4CA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "8CD54A": "TAIYO YUDEN CO.,LTD",
    "A002A5": "Intel Corporate",
    "ACC048": "OnePlus Technology (Shenzhen) Co., Ltd",
    "408432": "Microchip Technology Inc.",
    "88B5FF": "Shenzhen iComm Semiconductor CO.,LTD",
    "44D5C1": "EM Microelectronic",
    "943BB1": "Kaon Group Co., Ltd.",
    "44F034": "Kaon Group Co., Ltd.",
    "888F10": "Shenzhen Max Infinite Technology Co.,Ltd.",
    "C4799F": "Haiguang Smart Device Co.,Ltd.",
    "842859": "Amazon Technologies Inc.",
    "5016F4": "Motorola Mobility LLC, a Lenovo Company",
    "3C08CD": "Juniper Networks",
    "68505D": "Halo Technologies",
    "7C6A8A": "SINOBONDER Technology Co., Ltd.",
    "90ECE3": "Nokia",
    "887B2C": "zte corporation",
    "60DBEF": "Unify Software and Solutions GmbH & Co. KG",
    "0016F6": "Nevion",
    "40B7FC": "Phyplus Microelectronics Limited",
    "682C4F": "leerang corporation",
    "68DDB7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "14D864": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B851A9": "Nokia",
    "D077CE": "Edgecore Networks Corporation",
    "ECAFF9": "Hailo Technologies Ltd.",
    "FC59C0": "Arista Networks",
    "641B2F": "Samsung Electronics Co.,Ltd",
    "9C73B1": "Samsung Electronics Co.,Ltd",
    "388A06": "Samsung Electronics Co.,Ltd",
    "B0D888": "Panasonic Automotive Systems Co.,Ltd",
    "28EC22": "eero inc.",
    "40D4F6": "Honor Device Co., Ltd.",
    "7C3180": "SMK corporation",
    "905607": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "483133": "Robert Bosch Elektronika Kft.",
    "B8AB61": "Cisco Meraki",
    "40ED00": "TP-Link Corporation Limited",
    "782DAD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D06158": "HUAWEI TECHNOLOGIES CO.,LTD",
    "244BF1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60152B": "Palo Alto Networks",
    "44D77E": "Robert Bosch GmbH",
    "908855": "Cisco Systems, Inc",
    "687161": "Cisco Systems, Inc",
    "883F37": "UHTEK CO., LTD.",
    "0080B8": "DMG MORI Digital Co., LTD",
    "2873F6": "Amazon Technologies Inc.",
    "C4EBFF": "zte corporation",
    "4893DC": "UNIWAY INFOCOM PVT LTD",
    "5CE688": "VECOS Europe B.V.",
    "5C2886": "Inventec(Chongqing) Corporation",
    "D4BAFA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "4CEC0F": "Cisco Systems, Inc",
    "843E1D": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "E0CB1D": "Amazon Technologies Inc.",
    "FC84A7": "Murata Manufacturing Co., Ltd.",
    "3CCB4D": "Avikus Co., Ltd",
    "601B52": "Vodafone Italia S.p.A.",
    "24A42C": "NETIO products a.s.",
    "702F86": "Marquardt GmbH",
    "C08F20": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "08FF24": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "90B67A": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "249AC8": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "A8881F": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "005079": "Private",
    "28C01B": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "E028B1": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "303180": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "201582": "Apple, Inc.",
    "40921A": "Apple, Inc.",
    "10E2C9": "Apple, Inc.",
    "0436B8": "I&C Technology",
    "D0431E": "Dell Inc.",
    "F0A0B1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "404F42": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3086F1": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00C04F": "Dell Inc.",
    "00B0D0": "Dell Inc.",
    "0019B9": "Dell Inc.",
    "001AA0": "Dell Inc.",
    "002564": "Dell Inc.",
    "A4BADB": "Dell Inc.",
    "38A851": "Quickset Defense Technologies, LLC",
    "00CB00": "Private",
    "1100AA": "Private",
    "9C93E4": "Private",
    "782BCB": "Dell Inc.",
    "14FEB5": "Dell Inc.",
    "180373": "Dell Inc.",
    "74867A": "Dell Inc.",
    "204747": "Dell Inc.",
    "000BDB": "Dell Inc.",
    "00123F": "Dell Inc.",
    "A41F72": "Dell Inc.",
    "001C23": "Dell Inc.",
    "847BEB": "Dell Inc.",
    "989096": "Dell Inc.",
    "801844": "Dell Inc.",
    "9840BB": "Dell Inc.",
    "D481D7": "Dell Inc.",
    "F80DA9": "Zyxel Communications Corporation",
    "54BF64": "Dell Inc.",
    "CCC5E5": "Dell Inc.",
    "4CD98F": "Dell Inc.",
    "DCF401": "Dell Inc.",
    "6C2B59": "Dell Inc.",
    "C8F750": "Dell Inc.",
    "98E743": "Dell Inc.",
    "185A58": "Dell Inc.",
    "D08E79": "Dell Inc.",
    "B44506": "Dell Inc.",
    "485A0D": "Juniper Networks",
    "E0D848": "Dell Inc.",
    "04BF1B": "Dell Inc.",
    "581031": "Hon Hai Precision IND.CO.,LTD",
    "AC919B": "Wistron Neweb Corporation",
    "E03C1C": "IEEE Registration Authority",
    "000993": "Visteon Corporation",
    "EC41CA": "Shenzhen TecAnswer Technology co.,ltd",
    "148477": "New H3C Technologies Co., Ltd",
    "14962D": "New H3C Technologies Co., Ltd",
    "C4A64E": "Quectel Wireless Solutions Co.,Ltd.",
    "7066B9": "Huawei Device Co., Ltd.",
    "C4A1AE": "Huawei Device Co., Ltd.",
    "486F33": "KYUNGWOO.SYSTEM, INC.",
    "442063": "Continental Automotive Technologies GmbH",
    "086195": "Rockwell Automation",
    "24BBC9": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "0000BC": "Rockwell Automation",
    "FCD749": "Amazon Technologies Inc.",
    "A87116": "Earda Technologies co Ltd",
    "E8EBDD": "Guangzhou Qingying Acoustics Technology Co., Ltd",
    "548C81": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "244845": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "0050CA": "DZS Inc.",
    "74EE8D": "Apollo Intelligent Connectivity (Beijing) Technology Co., Ltd.",
    "D858C6": "Katch Asset Tracking Pty Limited ",
    "2CC6A0": "Lumacron Technology Ltd.",
    "ECC89C": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "8CE748": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "2428FD": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "ACB92F": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "D4E853": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "240F9B": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "C06DED": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "2432AE": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "E0BAAD": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "E0CA3C": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "DC07F8": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "64DB8B": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "94E1AC": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "5803FB": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "4447CC": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "98DF82": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "C056E3": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "BCAD28": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "88B6BD": "Flaircomm Microelectronics, Inc.",
    "2C9682": "MitraStar Technology Corp.",
    "BC49B2": "SHENZHEN ALONG COMMUNICATION TECH CO., LTD",
    "9497AE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "0CBD75": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "AC9073": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000351": "Diebold Nixdorf",
    "F09FC2": "Ubiquiti Inc",
    "802AA8": "Ubiquiti Inc",
    "FC1D3A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4BEFB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58F8D7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "788A20": "Ubiquiti Inc",
    "7483C2": "Ubiquiti Inc",
    "E063DA": "Ubiquiti Inc",
    "245A4C": "Ubiquiti Inc",
    "602232": "Ubiquiti Inc",
    "E43883": "Ubiquiti Inc",
    "788CB5": "TP-Link Corporation Limited",
    "84B59C": "Juniper Networks",
    "5C4527": "Juniper Networks",
    "EC3EF7": "Juniper Networks",
    "002159": "Juniper Networks",
    "00239C": "Juniper Networks",
    "50C58D": "Juniper Networks",
    "28C0DA": "Juniper Networks",
    "288A1C": "Juniper Networks",
    "B8EA98": "Xiaomi Communications Co Ltd",
    "10C4CA": "HUMAX Co., Ltd.",
    "AC800A": "Sony Corporation",
    "40A677": "Juniper Networks",
    "D818D3": "Juniper Networks",
    "F04B3A": "Juniper Networks",
    "C042D0": "Juniper Networks",
    "001BC0": "Juniper Networks",
    "44ECCE": "Juniper Networks",
    "CCE194": "Juniper Networks",
    "E45D37": "Juniper Networks",
    "94F7AD": "Juniper Networks",
    "784F9B": "Juniper Networks",
    "88D98F": "Juniper Networks",
    "78507C": "Juniper Networks",
    "F07CC7": "Juniper Networks",
    "000585": "Juniper Networks",
    "889009": "Juniper Networks",
    "00CC34": "Juniper Networks",
    "E030F9": "Juniper Networks",
    "204E71": "Juniper Networks",
    "D404FF": "Juniper Networks",
    "84C1C1": "Juniper Networks",
    "4C734F": "Juniper Networks",
    "D45A3F": "Juniper Networks",
    "04698F": "Juniper Networks",
    "58C935": "Chiun Mai Communication System, Inc",
    "7CE2CA": "Juniper Networks",
    "A4A528": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "E878EE": "New H3C Technologies Co., Ltd",
    "ECA7AD": "Barrot Technology Co.,Ltd.",
    "00302B": "Inalp Solutions AG",
    "F0748D": "Ruijie Networks Co.,LTD",
    "48814E": "E&M SOLUTION CO,.Ltd",
    "7C4D8F": "HP Inc.",
    "9CCBF7": "CLOUD STAR TECHNOLOGY CO., LTD.",
    "74D558": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F46412": "Sony Interactive Entertainment Inc.",
    "3CF9F0": "zte corporation",
    "6877DA": "zte corporation",
    "8858BE": "kuosheng.com",
    "44DBD2": "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD.",
    "E0DCA0": "Siemens Industrial Automation Products Ltd., Chengdu",
    "1C3B62": "HMD Global Oy",
    "5C64F1": "Cisco Systems, Inc",
    "9CDBAF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C2E57": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8D775": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64EC65": "vivo Mobile Communication Co., Ltd.",
    "F42462": "Selcom Electronics (Shanghai) Co., Ltd",
    "F4A17F": "Marquardt Electronics Technology (Shanghai) Co.Ltd",
    "48BCE1": "Samsung Electronics Co.,Ltd",
    "B838EF": "ADVA Optical Networking Ltd.",
    "084218": "Asyril SA",
    "D85B22": "Shenzhen Hohunet Technology Co., Ltd",
    "D0D003": "Samsung Electronics Co.,Ltd",
    "5C76D5": "Nokia",
    "8C7A00": "Nokia",
    "A4FC14": "Apple, Inc.",
    "A81AF1": "Apple, Inc.",
    "705846": "Trig Avionics Limited",
    "0891A3": "Amazon Technologies Inc.",
    "0CD923": "GOCLOUD Networks(GAOKE Networks)",
    "84398F": "Fortinet, Inc.",
    "B4EDD5": "Quectel Wireless Solutions Co.,Ltd.",
    "2CD1C6": "Murata Manufacturing Co., Ltd.",
    "B85CEE": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "CC08FA": "Apple, Inc.",
    "909B6F": "Apple, Inc.",
    "7473B4": "Apple, Inc.",
    "F83C80": "MITSUMI ELECTRIC CO.,LTD.",
    "D83ADD": "Raspberry Pi Trading Ltd",
    "70F8AE": "Microsoft Corporation",
    "000F2A": "Cableware Electronics",
    "BC64D9": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "301F48": "zte corporation",
    "7CC74A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "38FC34": "Huawei Device Co., Ltd.",
    "3CF692": "Huawei Device Co., Ltd.",
    "940EE7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8B271": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6405E4": "ALPSALPINE CO,.LTD",
    "DCAC6F": "Everytale Inc",
    "F41532": "PETAiO (NanJing), Inc.",
    "2CAB33": "Texas Instruments",
    "28B5E8": "Texas Instruments",
    "08C3B3": "TCL King Electrical Appliances(Huizhou)Co.,Ltd",
    "204569": "ITEL MOBILE LIMITED",
    "D46352": "Vutility Inc.",
    "D821DA": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "08F1B3": "Cisco Meraki",
    "BCE8FA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "505A65": "AzureWave Technology Inc.",
    "C4A559": "IEEE Registration Authority",
    "6C0C9A": "Amazon Technologies Inc.",
    "CC00F1": "Sagemcom Broadband SAS",
    "308216": "Apple, Inc.",
    "8CD0B2": "Beijing Xiaomi Mobile Software Co., Ltd",
    "C4A052": "Motorola Mobility LLC, a Lenovo Company",
    "84900A": "Arcadyan Corporation",
    "787689": "eero inc.",
    "8076C2": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "F04A3D": "Bosch Thermotechnik GmbH",
    "941FA2": "Wuhan YuXin Semiconductor Co., Ltd.",
    "B49DFD": "Shenzhen SDMC Technology CO.,Ltd.",
    "F41A9C": "Xiaomi Communications Co Ltd",
    "001AEB": "Allied Telesis K.K.",
    "DC0682": "Accessia Technology Ltd.",
    "44D267": "Snorble",
    "B0BC7A": "Harman/Becker Automotive Systems GmbH",
    "487706": "NXP Semiconductor (Tianjin) LTD.",
    "0CCC47": "IEEE Registration Authority",
    "5C3E06": "Cisco Systems, Inc",
    "C828E5": "Cisco Systems, Inc",
    "B0FC88": "Sagemcom Broadband SAS",
    "34873D": "Quectel Wireless Solutions Co.,Ltd.",
    "C44137": "Quectel Wireless Solutions Co.,Ltd.",
    "6CD199": "vivo Mobile Communication Co., Ltd.",
    "D040BE": "NPO RPS LLC",
    "10D680": "Tendyron Corporation",
    "784946": "Cambridge Mobile Telematics, Inc.",
    "003E73": "Mist Systems, Inc.",
    "0CFC18": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F44DAD": "Cable Matters Inc.",
    "80AB4D": "Nokia Solutions and Networks GmbH & Co. KG",
    "4CBA7D": "Gemtek Technology Co., Ltd.",
    "500238": "Nokia Shanghai Bell Co., Ltd.",
    "ACAD4B": "zte corporation",
    "A8169D": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "F0221D": "IEEE Registration Authority",
    "9C5440": "ChengDu TD Tech",
    "5026EF": "Murata Manufacturing Co., Ltd.",
    "444988": "Intel Corporate",
    "FC9189": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "1C1A1B": "Shanghai Sunmi Technology Co.,Ltd.",
    "8038D4": "Fibercentury Network Technology Co.,Ltd.",
    "7C296F": "Apple, Inc.",
    "40EDCF": "Apple, Inc.",
    "74057C": "Qorvo International Pte. Ltd.",
    "8C986B": "Apple, Inc.",
    "1C8682": "Apple, Inc.",
    "8054E3": "Apple, Inc.",
    "102E00": "Intel Corporate",
    "203A43": "Intel Corporate",
    "3C3B99": "ITEL MOBILE LIMITED",
    "FC3357": "KAGA FEI Co., Ltd.",
    "9CDE4D": "ML vision Co.,LTD",
    "F8F0C5": "Suzhou Kuhan Information Technologies Co.,Ltd.",
    "F8DE73": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC1A02": "HUAWEI TECHNOLOGIES CO.,LTD",
    "902778": "Open Infrastructure",
    "68418F": "Telechips, Inc.",
    "BC9EBB": "Nintendo Co.,Ltd",
    "D009C8": "Cisco Systems, Inc",
    "D03957": "Liteon Technology Corporation",
    "48E729": "Espressif Inc.",
    "E8BAE2": "Xplora Technologies AS",
    "5CA06C": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "A83B76": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "40E11C": "shenzhen Cloud High Communication Technology Co.,Ltd",
    "44643C": "Cisco Systems, Inc",
    "24161B": "Cisco Systems, Inc",
    "A8400B": "Visteon Corporation",
    "20318D": "Giax GmbH",
    "3CB07E": "Arounds Intelligent Equipment Co., Ltd.",
    "C04E8A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "08569B": "WiZ",
    "081AFD": "Huawei Device Co., Ltd.",
    "C0BFAC": "Huawei Device Co., Ltd.",
    "44272E": "Huawei Device Co., Ltd.",
    "106650": "Robert Bosch JuP1",
    "A80C03": "Florawise",
    "90DAF9": "Siemens Rail Automation SAU",
    "F8009D": "INTRACOM DEFENSE S.A.",
    "201642": "Microsoft Corporation",
    "8C946A": "New H3C Technologies Co., Ltd",
    "FC1165": "Cambium Networks Limited",
    "CC8DB5": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "ACEE64": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "0012F3": "u-blox AG",
    "547068": "VTech Communications Limited",
    "C84052": "PAX Computer Technology(Shenzhen) Ltd.",
    "68EE4B": "Sharetronic Data Technology Co.,Ltd",
    "48C35A": "LENOVO(BEIJING)CO., LTD.",
    "C461C7": "Microsoft Corporation",
    "ACCB36": "Fiberhome Telecommunication Technologies Co.,LTD",
    "60706C": "Google, Inc.",
    "C82ADD": "Google, Inc.",
    "240935": "Samsung Electronics Co.,Ltd",
    "B4DB91": "CELESTICA INC.",
    "C48372": "IEEE Registration Authority",
    "C0A5E8": "Intel Corporate",
    "906584": "Intel Corporate",
    "28C5D2": "Intel Corporate",
    "E48C73": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "2CB6C8": "Raisecom Technology CO., LTD",
    "D834D1": "Shenzhen Orange Digital Technology Co.,Ltd",
    "842289": "Samsung Electronics Co.,Ltd",
    "84AAA4": "SONoC Corp.",
    "883C93": "Alcatel-Lucent Enterprise",
    "FCDF00": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "E8BFDB": "Inodesign Group",
    "287E80": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "0C9F71": "Dolphin Electronics (DongGuan) Co., Ltd.",
    "1C8B76": "Calix Inc.",
    "745889": "Multilaser Industrial S.A.",
    "E8FF98": "Huawei Device Co., Ltd.",
    "241551": "Huawei Device Co., Ltd.",
    "58957E": "Huawei Device Co., Ltd.",
    "2C9452": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6001B1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8144D": "Apple, Inc.",
    "EC28D3": "Apple, Inc.",
    "5CBBEE": "zte corporation",
    "086518": "Apple, Inc.",
    "2C57CE": "Apple, Inc.",
    "80398C": "Samsung Electronics Co.,Ltd",
    "980D6F": "Samsung Electronics Co.,Ltd",
    "1C90FF": "Tuya Smart Inc.",
    "10823D": "Ruijie Networks Co.,LTD",
    "982044": "New H3C Technologies Co., Ltd",
    "D8A0E8": "zte corporation",
    "80563C": "ZF",
    "A044F3": "RafaelMicro",
    "F42B7D": "Chipsguide technology CO.,LTD.",
    "8C5219": "SHARP Corporation",
    "2C9E00": "Sony Interactive Entertainment Inc.",
    "DC6AE7": "Xiaomi Communications Co Ltd",
    "7CA449": "Xiaomi Communications Co Ltd",
    "D0066A": "Cornelis Networks, Inc.",
    "883CC5": "IEEE Registration Authority",
    "C404D8": "Aviva Links Inc.",
    "AC965B": "Lucid Motors",
    "089115": "Amazon Technologies Inc.",
    "74D423": "Amazon Technologies Inc.",
    "E04735": "Ericsson AB",
    "909877": "Vestel Elektronik San ve Tic. A.S.",
    "ECA138": "Amazon Technologies Inc.",
    "E0EF02": "Chengdu Quanjing Intelligent Technology Co.,Ltd",
    "4CA3A7": "TECNO MOBILE LIMITED",
    "B067B5": "Apple, Inc.",
    "5C5284": "Apple, Inc.",
    "C0956D": "Apple, Inc.",
    "3C39C8": "Apple, Inc.",
    "A8ABB5": "Apple, Inc.",
    "5864C4": "Apple, Inc.",
    "40A53B": "Nokia",
    "74803F": "Renesas Electronics (Penang) Sdn. Bhd.",
    "900A62": "Inventus Power Eletronica do Brasil LTDA",
    "505FB5": "ASKEY COMPUTER CORP",
    "88DE7C": "ASKEY COMPUTER CORP",
    "045747": "GoPro",
    "5C60BA": "HP Inc.",
    "BCF88B": "zte corporation",
    "68539D": "EM Microelectronic",
    "0443FD": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "A4897E": "Guangzhou Yuhong Technology Co.,Ltd.",
    "BCD767": "BAE Systems Apllied Intelligence",
    "EC1D9E": "Quectel Wireless Solutions Co.,Ltd.",
    "081A1E": "Shenzhen iComm Semiconductor CO.,LTD",
    "80646F": "Espressif Inc.",
    "3CA7AE": "zte corporation",
    "AC3184": "Huawei Device Co., Ltd.",
    "503F50": "Huawei Device Co., Ltd.",
    "2C691D": "IEEE Registration Authority",
    "CC29BD": "zte corporation",
    "385CFB": "Silicon Laboratories",
    "C43D1A": "Intel Corporate",
    "04E8B9": "Intel Corporate",
    "E02E0B": "Intel Corporate",
    "E8CC8C": "Chengdu Jiarui Hualian Communication Technology Co",
    "E46564": "SHENZHEN KTC TECHNOLOGY CO.,LTD",
    "3CE064": "Texas Instruments",
    "E0928F": "Texas Instruments",
    "CC037B": "Texas Instruments",
    "581CF8": "Intel Corporate",
    "AC198E": "Intel Corporate",
    "C85EA9": "Intel Corporate",
    "C8BF4C": "Beijing Xiaomi Mobile Software Co., Ltd",
    "7CDE78": "New H3C Technologies Co., Ltd",
    "B8FBAF": "Xiamen IPRT Technology CO.,LTD",
    "348518": "Espressif Inc.",
    "E051D8": "China Dragon Technology Limited",
    "549A11": "IEEE Registration Authority",
    "ECE6A2": "Fiberhome Telecommunication Technologies Co.,LTD",
    "AC84C6": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "1C0ED3": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "380A4F": "PRACHI ENTERPRISES",
    "A0CDF3": "Murata Manufacturing Co., Ltd.",
    "AC5AF0": "LG Electronics",
    "ECA62F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0CBEF1": "Huawei Device Co., Ltd.",
    "AC936A": "Huawei Device Co., Ltd.",
    "38A44B": "Huawei Device Co., Ltd.",
    "801970": "Samsung Electronics Co.,Ltd",
    "5C53C3": "Ubee Interactive Co., Limited",
    "B48A0A": "Espressif Inc.",
    "C83A1B": "Toshiba TEC Corporation Inc",
    "E0F728": "Amazon Technologies Inc.",
    "242934": "Google, Inc.",
    "3822F4": "Huawei Device Co., Ltd.",
    "8002F4": "IEEE Registration Authority",
    "149BF3": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "10071D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C8C13C": "RuggedTek Hangzhou Co., Ltd",
    "B850D8": "Beijing Xiaomi Mobile Software Co., Ltd",
    "F4B3B1": "Silicon Laboratories",
    "C09F51": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "10B232": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "8470D7": "eero inc.",
    "581DD8": "Sagemcom Broadband SAS",
    "30D587": "Samsung Electronics Co.,Ltd",
    "14F592": "Shenzhen SDG DONZHI Technology Co., Ltd",
    "4C09FA": "FRONTIER SMART TECHNOLOGIES LTD",
    "242CFE": "Zhejiang Tmall Technology Co., Ltd.",
    "A042D1": "Huawei Device Co., Ltd.",
    "58879F": "Huawei Device Co., Ltd.",
    "9C1FCA": "Hangzhou AlmightyDigit Technology Co., Ltd",
    "2CDC78": "Descartes Systems (USA) LLC",
    "68A7B4": "Honor Device Co., Ltd.",
    "803C20": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4DD58": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8D87E": "Amazon Technologies Inc.",
    "78034F": "Nokia",
    "9C57BC": "eero inc.",
    "0015A6": "Digital Electronics Products Ltd.",
    "A8A237": "Arcadyan Corporation",
    "F8AB82": "Xiaomi Communications Co Ltd",
    "EC30B3": "Xiaomi Communications Co Ltd",
    "08E63B": "zte corporation",
    "88C174": "zte corporation",
    "C89E61": "Lyngsoe Systems LTd",
    "902CFB": "CanTops Co,.Ltd.",
    "1CAF4A": "Samsung Electronics Co.,Ltd",
    "C8120B": "Samsung Electronics Co.,Ltd",
    "6C1524": "IEEE Registration Authority",
    "C08D51": "Amazon Technologies Inc.",
    "44B4B2": "Amazon Technologies Inc.",
    "ACCCFC": "Amazon Technologies Inc.",
    "E8DC6C": "Cisco Systems, Inc",
    "B03F64": "Apple, Inc.",
    "2C8217": "Apple, Inc.",
    "142D4D": "Apple, Inc.",
    "EC42CC": "Apple, Inc.",
    "B8211C": "Apple, Inc.",
    "28BE43": "vivo Mobile Communication Co., Ltd.",
    "2CFC8B": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "002604": "WorldCast Systems",
    "F8AD24": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "A8C98A": "New H3C Technologies Co., Ltd",
    "E0276C": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "DC8E95": "Silicon Laboratories",
    "90935A": "ARRIS Group, Inc.",
    "AC8FA9": "Nokia Solutions and Networks GmbH & Co. KG",
    "000AD0": "Niigata Develoment Center,  F.I.T. Co., Ltd.",
    "7CEF40": "Nextorage Corporation",
    "C0E01C": "IoT Security Group, SL",
    "AC51AB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48CDD3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC6618": "Adtran Inc",
    "44291E": "AltoBeam (China) Inc.",
    "24EBED": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2874F5": "Nokia Solutions and Networks GmbH & Co. KG",
    "A04466": "Intellics",
    "446D7F": "Amazon Technologies Inc.",
    "2406F2": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "6818D9": "Hill AFB - CAPRE Group",
    "1C61B4": "TP-Link Corporation Limited",
    "9CA2F4": "TP-Link Corporation Limited",
    "18BC57": "ADVA Optical Networking Ltd.",
    "D8E2DF": "Microsoft Corporation",
    "388F30": "Samsung Electronics Co.,Ltd",
    "6C0F61": "Hypervolt Ltd",
    "C0C170": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "504289": "zte corporation",
    "84C692": "Texas Instruments",
    "6CB2FD": "Texas Instruments",
    "302BDC": "Top-Unum Electronics Co., LTD",
    "90F7B2": "New H3C Technologies Co., Ltd",
    "1C47F6": "Zhidao Network Technology(Shenzhen) Co.,Ltd",
    "000CD6": "PARTNER TECH",
    "6C9308": "IEEE Registration Authority",
    "E8EBD3": "Mellanox Technologies, Inc.",
    "E881AB": "Beijing Sankuai Online Technology Co.,Ltd",
    "D4BD4F": "Ruckus Wireless",
    "5CC9C0": "Renesas Electronics (Penang) Sdn. Bhd.",
    "8C1553": "Beijing Memblaze Technology Co Ltd",
    "6CB158": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B0A4F0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04E31A": "Sagemcom Broadband SAS",
    "7C6A60": "China Mobile Group Device Co.,Ltd.",
    "B4695F": "TCT mobile ltd",
    "24753A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C85895": "Motorola Mobility LLC, a Lenovo Company",
    "6C999D": "Amazon Technologies Inc.",
    "C0060C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C3341": "China Mobile IOT Company Limited",
    "AC567B": "Sunnovo International Limited",
    "34D737": "IBG Industriebeteiligungsgesellschaft mbH &b Co. KG",
    "A8537D": "Mist Systems, Inc.",
    "303422": "eero inc.",
    "E4DADF": "Taicang T&W Electronics",
    "7C35F8": "Zhejiang Tmall Technology Co., Ltd.",
    "B05C16": "Fiberhome Telecommunication Technologies Co.,LTD",
    "08E021": "Honor Device Co., Ltd.",
    "9CC12D": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "7426FF": "zte corporation",
    "C42728": "zte corporation",
    "8CC7C3": "NETLINK ICT ",
    "E8D322": "Cisco Systems, Inc",
    "A0B4BF": "InfiNet LLC",
    "7CC95E": "Dongguan Liesheng Electronic Co., Ltd.",
    "D0EDFF": "ZF CVCS",
    "988FE0": "IEEE Registration Authority",
    "F8A91F": "ZVISION Technologies Co., Ltd",
    "C07982": "TCL King Electrical Appliances(Huizhou)Co.,Ltd",
    "080076": "PC LAN TECHNOLOGIES",
    "7CCCFC": "Quectel Wireless Solutions Co.,Ltd.",
    "DC8084": "Apple, Inc.",
    "4CBAD7": "LG Innotek",
    "34B883": "Cisco Systems, Inc",
    "80657C": "Apple, Inc.",
    "2CFDB4": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "38C804": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "E007C2": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "18D793": "IEEE Registration Authority",
    "102C8D": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "D8B673": "Texas Instruments",
    "B4F267": "Compal Broadband Networks, Inc.",
    "B0DD74": "Heimgard Technologies AS",
    "C0FBC1": "ITEL MOBILE LIMITED",
    "9850A3": "SIGNALTEK JSC",
    "B48C9D": "AzureWave Technology Inc.",
    "50B3B4": "Shenzhen Furuilian Electronic Co.,Ltd.",
    "94E686": "Espressif Inc.",
    "F828C9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC1193": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC3643": "IEEE Registration Authority",
    "50874D": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A41752": "Hifocus Electronics India Private Limited",
    "00620B": "Broadcom Limited",
    "40FE95": "New H3C Technologies Co., Ltd",
    "2CDD5F": "Shenzhen iComm Semiconductor CO.,LTD",
    "A05394": "Shenzhen zediel co., Ltd.",
    "04EEE8": "IEEE Registration Authority",
    "9CB8B4": "AMPAK Technology,Inc.",
    "84B4DB": "Silicon Laboratories",
    "1C70C9": "Jiangsu Aisida Electronic Co., Ltd",
    "3CF7D1": "OMRON Corporation",
    "DCCCE6": "Samsung Electronics Co.,Ltd",
    "F065AE": "Samsung Electronics Co.,Ltd",
    "5089D1": "Huawei Device Co., Ltd.",
    "BCD074": "Apple, Inc.",
    "0C1EF7": "Omni-ID",
    "E85177": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "388A21": "UAB \"Teltonika Telematics\"",
    "A45FB9": "DreamBig Semiconductor, Inc.",
    "345D9E": "Sagemcom Broadband SAS",
    "000F32": "Lootom Telcovideo Network (Wuxi) Co Ltd",
    "18C23C": "Lumi United Technology Co., Ltd",
    "0475F9": "Taicang T&W Electronics",
    "E06D18": "PIONEERCORPORATION",
    "8CF8C5": "Intel Corporate",
    "A05950": "Intel Corporate",
    "D4354A": "ALAXALA Networks Corporation",
    "40E99B": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "90FFD6": "Honor Device Co., Ltd.",
    "CCA3BD": "ITEL MOBILE LIMITED",
    "BCAF87": "smartAC.com, Inc.",
    "C02B31": "Phytium Technology Co.,Ltd.",
    "58CF79": "Espressif Inc.",
    "786299": "BITSTREAM sp. z o.o.",
    "CC5B31": "Nintendo Co.,Ltd",
    "C0280B": "Honor Device Co., Ltd.",
    "9CEA97": "Honor Device Co., Ltd.",
    "C89BAD": "Honor Device Co., Ltd.",
    "B4D286": "Telechips, Inc.",
    "08F80D": "IEEE Registration Authority",
    "4CC449": "Icotera A/S",
    "BC455B": "Samsung Electronics Co.,Ltd",
    "D42C46": "BUFFALO.INC",
    "1091A8": "Espressif Inc.",
    "78047A": "Edge Networks LLC",
    "B03CDC": "Intel Corporate",
    "208097": "Shenzhen OXO Technology limited",
    "7C1689": "Sagemcom Broadband SAS",
    "5C83CD": "New platforms",
    "10E8A7": "Wistron Neweb Corporation",
    "8415D3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D49400": "HUAWEI TECHNOLOGIES CO.,LTD",
    "384B24": "SIEMENS AG",
    "F0AE66": "Cosonic Intelligent Technologies Co., Ltd.",
    "3CA916": "Huawei Device Co., Ltd.",
    "D46C6D": "ARRIS Group, Inc.",
    "A0E7AE": "ARRIS Group, Inc.",
    "806A00": "Cisco Systems, Inc",
    "A06C65": "Texas Instruments",
    "448502": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "F4A454": "IEEE Registration Authority",
    "58FCC6": "TOZO INC",
    "C41C07": "Samsung Electronics Co.,Ltd",
    "4011C3": "Samsung Electronics Co.,Ltd",
    "C47D9F": "Samsung Electronics Co.,Ltd",
    "A4EF15": "AltoBeam (China) Inc.",
    "1C4586": "Nintendo Co.,Ltd",
    "185B00": "Nokia",
    "A861DF": "China Mobile Group Device Co.,Ltd.",
    "305A99": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "604DE1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "704E6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "603D29": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10A51D": "Intel Corporate",
    "589B4A": "DWnet Technologies(Suzhou) Corporation",
    "507C6F": "Intel Corporate",
    "98DD60": "Apple, Inc.",
    "C04442": "Apple, Inc.",
    "D468AA": "Apple, Inc.",
    "F8C3CC": "Apple, Inc.",
    "54C480": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D866EE": "BOXIN COMMUNICATION CO.,LTD.",
    "F4573E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "70C6DD": "New H3C Technologies Co., Ltd",
    "3C9EC7": "SKY UK LIMITED",
    "088E90": "Intel Corporate",
    "282A87": "ITEL MOBILE LIMITED",
    "A8B57C": "Roku, Inc",
    "2426BA": "Shenzhen Toptel Technology Co., Ltd.",
    "F06C73": "Nokia",
    "9C756E": "Ajax Systems DMCC",
    "E8F9D4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0C787": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C4F9B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "482FD7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C214A": "Intel Corporate",
    "508492": "Intel Corporate",
    "9880BB": "IEEE Registration Authority",
    "B812DA": " LVSWITCHES INC.",
    "A0D7F3": "Samsung Electronics Co.,Ltd",
    "7890A2": "zte corporation",
    "ACBCD9": "Cisco Systems, Inc",
    "D8B053": "Xiaomi Communications Co Ltd",
    "6CF784": "Xiaomi Communications Co Ltd",
    "546CEB": "Intel Corporate",
    "009337": "Intel Corporate",
    "58CE2A": "Intel Corporate",
    "042B58": "Shenzhen Hanzsung Technology Co.,Ltd",
    "B43D08": "GX International BV",
    "7089F5": "Dongguan Lingjie IOT Co., LTD",
    "9431CB": "vivo Mobile Communication Co., Ltd.",
    "B8208E": "Panasonic Connect Co., Ltd.",
    "847B57": "Intel Corporate",
    "4006D5": "Cisco Systems, Inc",
    "C4CA2B": "Arista Networks",
    "607DDD": "Shenzhen Shichuangyi Electronics Co.,Ltd",
    "AC5E14": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20DF73": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48128F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64CBE9": "LG Innotek",
    "24D81E": "MirWifi,Joint-Stock Company",
    "90380C": "Espressif Inc.",
    "BC2228": "D-Link International",
    "40CA63": "Seongji Industry Company",
    "40B0A1": "VALCOM CO.,LTD.",
    "103C59": "zte corporation",
    "D48A3B": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "6CDDEF": "EPCOMM Inc.",
    "10B7A8": "CableFree Networks Limited",
    "089BF1": "eero inc.",
    "78444A": "Shenzhen Aiwinn information Technology Co., Ltd.",
    "D0F121": "Xi'an LINKSCI Technology Co., Ltd",
    "9C5636": "Huawei Device Co., Ltd.",
    "947F1D": "Shenzhen Fastrain Technology Co., Ltd.",
    "D0F520": "KYOCERA Corporation ",
    "70B64F": "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.",
    "B89470": "Calix Inc.",
    "E4293D": "Shenzhen Sy-Fiber Optical Communication Technology.Co.,Ltd",
    "70B9BB": "Shenzhen Hankvision Technology CO.,LTD",
    "B4205B": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A4C69A": "Samsung Electronics Co.,Ltd",
    "8CB87E": "Intel Corporate",
    "701AB8": "Intel Corporate",
    "5CDF89": "Ruckus Wireless",
    "B88C29": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "5444A3": "Samsung Electronics Co.,Ltd",
    "D03F27": "Wyze Labs Inc",
    "50297B": "China Mobile Group Device Co.,Ltd.",
    "3003C8": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "089E84": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1082D7": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "785EE8": "IEEE Registration Authority",
    "9C40CD": "Synclayer Inc.",
    "D8BE1F": "Apple, Inc.",
    "98502E": "Apple, Inc.",
    "580AD4": "Apple, Inc.",
    "A477F3": "Apple, Inc.",
    "CCBCE3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "088EDC": "Apple, Inc.",
    "A84A28": "Apple, Inc.",
    "BC6AD1": "Xiaomi Communications Co Ltd",
    "0845D1": "Cisco Systems, Inc",
    "18E215": "Nokia",
    "EC1C5D": "Siemens AG",
    "8427B6": "China Mobile IOT Company Limited",
    "807EB4": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "2CB8ED": "SonicWall",
    "500A52": "Huiwan Technologies Co. Ltd",
    "54A9D4": "Minibar Systems",
    "000062": "BULL HN INFORMATION SYSTEMS",
    "C8C9A3": "Espressif Inc.",
    "E43BC9": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "E89526": "Luxshare Precision Industry CO., LTD.",
    "104121": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "D49390": "CLEVO CO.",
    "BC062D": "Wacom Co.,Ltd.",
    "3C93F4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6433B5": "IEEE Registration Authority",
    "504B9E": "Huawei Device Co., Ltd.",
    "047AAE": "Huawei Device Co., Ltd.",
    "409CA6": "Curvalux",
    "8C4B14": "Espressif Inc.",
    "D88083": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "FC4265": "Zhejiang Tmall Technology Co., Ltd.",
    "187758": "Audoo Limited (UK)",
    "44D454": "Sagemcom Broadband SAS",
    "6887C6": "Cisco Systems, Inc",
    "80248F": "Cisco Systems, Inc",
    "4C50F1": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "B4E454": "Amazon Technologies Inc.",
    "0C43F9": "Amazon Technologies Inc.",
    "D494FB": "Continental Automotive Systems Inc.",
    "44A92C": "IEEE Registration Authority",
    "84AC16": "Apple, Inc.",
    "2CBC87": "Apple, Inc.",
    "5078B0": "Huawei Device Co., Ltd.",
    "E4072B": "Huawei Device Co., Ltd.",
    "2446E4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50E7A0": "Renesas Electronics (Penang) Sdn. Bhd.",
    "90E868": "AzureWave Technology Inc.",
    "68F0D0": "SkyBell Technologies Inc.",
    "FCA89B": "Texas Instruments",
    "98F07B": "Texas Instruments",
    "CC896C": "GN Hearing A/S",
    "A4FF95": "Nokia",
    "D47350": "DBG Commnunications Technology  Co., Ltd.",
    "70DA17": "Austrian Audio GmbH",
    "3861A5": "Grabango Co",
    "78D3ED": "NORMA",
    "8C1ED9": "Beijing Unigroup Tsingteng Microsystem Co., LTD.",
    "34B472": "Espressif Inc.",
    "F80C58": "Taicang T&W Electronics",
    "50411C": "AMPAK Technology,Inc.",
    "F42679": "Intel Corporate",
    "B03795": "LG Electronics",
    "3C7AF0": "ITEL MOBILE LIMITED",
    "48F3F3": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "081C6E": "Xiaomi Communications Co Ltd",
    "508140": "HP Inc.",
    "FC58DF": "Interphone Service",
    "981082": "Nsolution Co., Ltd.",
    "38A659": "Sagemcom Broadband SAS",
    "781305": "IEEE Registration Authority",
    "34243E": "zte corporation",
    "98ED7E": "eero inc.",
    "34FE9E": "Fujitsu Limited",
    "38453B": "Ruckus Wireless",
    "6450D6": "Liquidtool Systems",
    "F44637": "Intel Corporate",
    "0C718C": "TCT mobile ltd",
    "24085D": "Continental Aftermarket & Services GmbH",
    "A848FA": "Espressif Inc.",
    "505D7A": "zte corporation",
    "C41234": "Apple, Inc.",
    "3CA6F6": "Apple, Inc.",
    "681BEF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4DC5F": "Cofractal, Inc.",
    "4CAB4F": "Apple, Inc.",
    "9C583C": "Apple, Inc.",
    "000901": "Shenzhen Shixuntong Information & Technoligy Co",
    "F463E7": "Nanjing Maxon O.E. Tech. Co., LTD",
    "88AEDD": "EliteGroup Computer Systems Co., LTD",
    "7806C9": "Huawei Device Co., Ltd.",
    "E8A6CA": "Huawei Device Co., Ltd.",
    "CCFA66": "Huawei Device Co., Ltd.",
    "68966A": "OHSUNG",
    "A0E70B": "Intel Corporate",
    "04EEEE": "Laplace System Co., Ltd.",
    "1C3CD4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4E451": "HUAWEI TECHNOLOGIES CO.,LTD",
    "482218": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "F88EA1": "Edgecore Networks Corporation",
    "2C4881": "vivo Mobile Communication Co., Ltd.",
    "E40CFD": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "58D697": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "30A176": "Fiberhome Telecommunication Technologies Co.,LTD",
    "48E7DA": "AzureWave Technology Inc.",
    "047153": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "40C48C": "N-iTUS CO.,LTD.",
    "5437BB": "Taicang T&W Electronics",
    "60F8F2": "Synaptec",
    "AC74B1": "Intel Corporate",
    "F46077": "Texas Instruments",
    "C03C04": "Sagemcom Broadband SAS",
    "A4D73C": "Seiko Epson Corporation",
    "000808": "PPT Vision, Inc.",
    "6C108B": "WeLink Communications",
    "4C7167": "PoLabs d.o.o.",
    "7C8530": "Nokia",
    "24A799": "Huawei Device Co., Ltd.",
    "7C3E74": "Huawei Device Co., Ltd.",
    "148919": "2bps",
    "58FD5D": "Hangzhou Xinyun technology Co., Ltd.",
    "185207": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "E0C63C": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "F8BAE6": "Nokia",
    "BCECA0": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "A45590": "Xiaomi Communications Co Ltd",
    "08010F": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "C0CC42": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "806559": "EM Microelectronic",
    "D047C1": "Elma Electronic AG",
    "308E7A": "Shenzhen iComm Semiconductor CO.,LTD",
    "B8A377": "Cisco Systems, Inc",
    "E44E2D": "Cisco Systems, Inc",
    "984265": "Sagemcom Broadband SAS",
    "DC215C": "Intel Corporate",
    "C42360": "Intel Corporate",
    "78D9E9": "MOMENTUM IOT",
    "103D1C": "Intel Corporate",
    "3887D5": "Intel Corporate",
    "1CD107": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "9C1C37": "AltoBeam (China) Inc.",
    "2C0823": "Sercomm France Sarl",
    "A8934A": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "C87B23": "Bose Corporation",
    "00047D": "Motorola Solutions Inc.",
    "8C19B5": "Arcadyan Corporation",
    "34AB95": "Espressif Inc.",
    "C491CF": "Luxul ",
    "D89AC1": "Nokia",
    "F0B11D": "Nokia",
    "7CD9F4": "UAB \"Teltonika Telematics\"",
    "A4D795": "Wingtech Mobile Communications Co.,Ltd",
    "84AB26": "Tiinlab Corporation",
    "58356B": "TECNO MOBILE LIMITED",
    "F89753": "Huawei Device Co., Ltd.",
    "5894AE": "Huawei Device Co., Ltd.",
    "B03ACE": "Huawei Device Co., Ltd.",
    "F84CDA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BCFF4D": "Espressif Inc.",
    "A4056E": "Tiinlab Corporation",
    "FC4EA4": "Apple, Inc.",
    "F4BEEC": "Apple, Inc.",
    "54E61B": "Apple, Inc.",
    "CC68B6": "TP-Link Corporation Limited",
    "14DD9C": "vivo Mobile Communication Co., Ltd.",
    "64644A": "Beijing Xiaomi Mobile Software Co., Ltd",
    "E89E0C": "MAX8USA DISTRIBUTORS INC.",
    "64808B": "VG Controls, Inc.",
    "FC13F0": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "20896F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F4FBB8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A070B7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78B554": "Huawei Device Co., Ltd.",
    "689E6A": "Huawei Device Co., Ltd.",
    "282B96": "Huawei Device Co., Ltd.",
    "A439B6": "SHENZHEN PEIZHE MICROELECTRONICS CO .LTD",
    "0004AD": "Malibu Networks",
    "C45BBE": "Espressif Inc.",
    "002618": "ASUSTek COMPUTER INC.",
    "CC3331": "Texas Instruments",
    "FC9C98": "Arlo Technology",
    "1C6EE6": "NHNETWORKS",
    "08F606": "zte corporation",
    "B814DB": "OHSUNG",
    "1CD1E0": "Cisco Systems, Inc",
    "E8FD35": "Huawei Device Co., Ltd.",
    "ECC5D2": "Huawei Device Co., Ltd.",
    "4044FD": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "646EE0": "Intel Corporate",
    "0456E5": "Intel Corporate",
    "884604": "Xiaomi Communications Co Ltd",
    "841EA3": "Sagemcom Broadband SAS",
    "8C2A8E": "DongGuan Ramaxel Memory Technology",
    "80071B": "VSOLUTION TELECOMMUNICATION TECHNOLOGY CO.,LTD.",
    "24456B": "Huawei Device Co., Ltd.",
    "483871": "Huawei Device Co., Ltd.",
    "44BDDE": "BHTC GmbH",
    "B4608C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F40223": "PAX Computer Technology(Shenzhen) Ltd.",
    "6479F0": "Intel Corporate",
    "081086": "NEC Platforms, Ltd.",
    "78653B": "Shaoxing Ourten Electronics Co., Ltd.",
    "E0E656": "Nethesis  srl",
    "4CF202": "Xiaomi Communications Co Ltd",
    "A877E5": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "10A4DA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC584A": "xiamenshi c-chip technology co., ltd",
    "84D608": "Wingtech Mobile Communications Co., Ltd.",
    "346893": "Tecnovideo Srl",
    "9023B4": "New H3C Technologies Co., Ltd",
    "882A5E": "New H3C Technologies Co., Ltd",
    "246968": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "FCA9DC": "Renesas Electronics (Penang) Sdn. Bhd.",
    "3C7AAA": "China Dragon Technology Limited",
    "84FD27": "Silicon Laboratories",
    "28D3EA": "Huawei Device Co., Ltd.",
    "A8F266": "Huawei Device Co., Ltd.",
    "CC9C3E": "Cisco Meraki",
    "84267A": "GUANGDONG TAIDE ZHILIAN TECHNOLOGY CO.,LTD",
    "6C4760": "Sunitec Enterprise Co.,Ltd",
    "183219": "EM Microelectronic",
    "0020C1": "SAXA, Inc.",
    "4829E4": "AO ",
    "3C9BC6": "Huawei Device Co., Ltd.",
    "0036BE": "Northwest Towers",
    "A45E5A": "ACTIVIO Inc.",
    "7C5079": "Intel Corporate",
    "8038FB": "Intel Corporate",
    "501FC6": "Apple, Inc.",
    "CC69FA": "Apple, Inc.",
    "10CEE9": "Apple, Inc.",
    "5C56A4": "Wanan Hongsheng Electronic Co.Ltd",
    "642656": "Shenzhen Fanweitai Technology Service Co.,Ltd",
    "AC8247": "Intel Corporate",
    "F0258E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9C746F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC74C4": "Maytronics Ltd.",
    "540910": "Apple, Inc.",
    "9CFC28": "Apple, Inc.",
    "B485E1": "Apple, Inc.",
    "0C19F8": "Apple, Inc.",
    "08B4B1": "Google, Inc.",
    "382028": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E47727": "HUAWEI TECHNOLOGIES CO.,LTD",
    "105107": "Intel Corporate",
    "749AC0": "Cachengo, Inc.",
    "34587C": "MIRAE INFORMATION TECHNOLOGY CO., LTD.",
    "A03B01": "Kyung In Electronics",
    "145E69": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C4E287": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54F607": "Huawei Device Co., Ltd.",
    "A0A3F0": "D-Link International",
    "E8A0CD": "Nintendo Co.,Ltd",
    "9C823F": "Huawei Device Co., Ltd.",
    "E06C4E": "Shenzhen TINNO Mobile Technology Corp.",
    "50558D": "China Mobile IOT Company Limited",
    "64E003": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "34F716": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "846569": "New H3C Technologies Co., Ltd",
    "281B04": "Zalliant LLC",
    "7C5259": "Sichuan Jiuzhou Electronic Technology Co., Ltd.",
    "F4B301": "Intel Corporate",
    "E8D2FF": "Sagemcom Broadband SAS",
    "0C96CD": "MERCURY CORPORATION",
    "18CE94": "Samsung Electronics Co.,Ltd",
    "783716": "Samsung Electronics Co.,Ltd",
    "5C6F69": "Broadcom Limited",
    "D012CB": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "78CF2F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A83B5C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "045FB9": "Cisco Systems, Inc",
    "102FA3": "Shenzhen Uvision-tech Technology Co.Ltd",
    "04495D": "Huawei Device Co., Ltd.",
    "F0FEE7": "Huawei Device Co., Ltd.",
    "E8F408": "Intel Corporate",
    "58B0FE": "Team EPS GmbH",
    "90C792": "ARRIS Group, Inc.",
    "DCCD74": "Japan E.M.Solutions Co., Ltd.",
    "748B29": "Micobiomed",
    "484C29": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4D438": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54211D": "Huawei Device Co., Ltd.",
    "143FA6": "Sony Home Entertainment&Sound Products Inc",
    "44D453": "Sagemcom Broadband SAS",
    "DCA120": "Nokia",
    "50523B": "Nokia",
    "2406AA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "8CAE49": "IEEE Registration Authority",
    "20FF36": "IFLYTEK CO.,LTD.",
    "24649F": "Huawei Device Co., Ltd.",
    "0C1773": "Huawei Device Co., Ltd.",
    "E81E92": "Huawei Device Co., Ltd.",
    "888E68": "Huawei Device Co., Ltd.",
    "807484": "ALL Winner (Hong Kong) Limited",
    "88238C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A8CC6F": "HMD Global Oy",
    "A4CCB9": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "6CB881": "zte corporation",
    "989AB9": "zte corporation",
    "008A55": "Huawei Device Co., Ltd.",
    "64A28A": "Huawei Device Co., Ltd.",
    "AC471B": "Huawei Device Co., Ltd.",
    "003192": "TP-Link Corporation Limited",
    "0865F0": "JM Zengge Co., Ltd",
    "14AB02": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B04530": "SKY UK LIMITED",
    "6CA0B4": "SKY UK LIMITED",
    "EC0DE4": "Amazon Technologies Inc.",
    "B456E3": "Apple, Inc.",
    "843095": "Hon Hai Precision IND.CO.,LTD",
    "3C6105": "Espressif Inc.",
    "4C20B8": "Apple, Inc.",
    "1488E6": "Apple, Inc.",
    "7895EB": "ITEL MOBILE LIMITED",
    "B0BBE5": "Sagemcom Broadband SAS",
    "1C9F4E": "COOSEA GROUP (HK) COMPANY LIMITED",
    "A468BC": "Oakley Inc.",
    "8CFDDE": "Sagemcom Broadband SAS",
    "48D890": "FN-LINK TECHNOLOGY LIMITED",
    "B0C53C": "Cisco Systems, Inc",
    "ECCE13": "Cisco Systems, Inc",
    "C418E9": "Samsung Electronics Co.,Ltd",
    "1CE57F": "Samsung Electronics Co.,Ltd",
    "085531": "Routerboard.com",
    "BC5BD5": "ARRIS Group, Inc.",
    "241145": "Xiaomi Communications Co Ltd",
    "00042B": "IT Access Co., Ltd.",
    "D44F67": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8B95A": "LG Innotek",
    "781F11": "RAB Lighting",
    "107100": "Huawei Device Co., Ltd.",
    "607EA4": "Shanghai Imilab Technology Co.Ltd",
    "B4FF98": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0838E6": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "E8C2DD": "Infinix mobility limited",
    "C41C9C": "JiQiDao",
    "98F181": "New H3C Technologies Co., Ltd",
    "8C55BB": "Songwoo Information & Technology Co., Ltd",
    "7C8FDE": "DWnet Technologies(Suzhou) Corporation",
    "90808F": "Huawei Device Co., Ltd.",
    "184516": "Texas Instruments",
    "D02EAB": "Texas Instruments",
    "847127": "Silicon Laboratories",
    "F0F7E7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "849DC2": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "40A9CF": "Amazon Technologies Inc.",
    "5895D8": "IEEE Registration Authority",
    "F80DAC": "HP Inc.",
    "8850F6": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "0405DD": "Shenzhen Cultraview Digital Technology Co., Ltd",
    "3897A4": "ELECOM CO.,LTD.",
    "E433AE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "387A3C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8CCE4E": "Espressif Inc.",
    "40B5C1": "Cisco Systems, Inc",
    "E44791": "Iris ID Systems, Inc.",
    "F013C1": "Hannto Technology Co., Ltd",
    "8454DF": "Huawei Device Co., Ltd.",
    "E884A5": "Intel Corporate",
    "00C035": "QUINTAR COMPANY",
    "1C501E": "Sunplus Technology Co., Ltd.",
    "FC66CF": "Apple, Inc.",
    "AC1D06": "Apple, Inc.",
    "44A8FC": "Apple, Inc.",
    "F81093": "Apple, Inc.",
    "DCE994": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "409505": "ACOINFO TECHNOLOGY CO.,LTD",
    "5CD89E": "Huawei Device Co., Ltd.",
    "B82D28": "AMPAK Technology,Inc.",
    "A4134E": "Luxul ",
    "B85F98": "Amazon Technologies Inc.",
    "D8F883": "Intel Corporate",
    "B436D1": "Renesas Electronics (Penang) Sdn. Bhd.",
    "101965": "New H3C Technologies Co., Ltd",
    "94FF61": "China Mobile Group Device Co.,Ltd.",
    "DC774C": "Cisco Systems, Inc",
    "204441": "Remote Solution",
    "FC4482": "Intel Corporate",
    "A085FC": "Microsoft Corporation",
    "14A9D0": "F5 Networks, Inc.",
    "000A49": "F5 Networks, Inc.",
    "0094A1": "F5 Networks, Inc.",
    "80F1F1": "Tech4home, Lda",
    "006151": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC76C5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C0272": "Silicon Laboratories",
    "000430": "Netgem",
    "0446CF": "Beijing Venustech Cybervision Co.,Ltd.",
    "B88035": "Shenzhen Qihu Intelligent Technology Company Limited",
    "9012A1": "We Corporation Inc.",
    "64B623": "Schrack Seconet Care Communication GmbH",
    "000EFF": "Megasolution,Inc.",
    "702C09": "Nintendo Co.,Ltd",
    "E4D373": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C0BC9A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E81B69": "Sercomm Corporation.",
    "6852D6": "UGame Technology Co.,Ltd",
    "FC1999": "Xiaomi Communications Co Ltd",
    "CC874A": "Nokia",
    "A45129": "XAG",
    "B4C26A": "Garmin International",
    "78F8B8": "Rako Controls Ltd",
    "6CE5F7": "New H3C Technologies Co., Ltd",
    "B8DD71": "zte corporation",
    "F4E578": "LLC Proizvodstvennaya Kompania \"TransService\"",
    "0854BB": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "3C9C0F": "Intel Corporate",
    "1C5D80": "Mitubishi Hitachi Power Systems Industries Co., Ltd.",
    "3CB15B": "Avaya Inc",
    "5865E6": "infomark",
    "08FBEA": "AMPAK Technology,Inc.",
    "60EB5A": "Asterfusion Data Technologies Co.,Ltd",
    "5C443E": "Skullcandy",
    "F88200": "CaptionCall",
    "0050F1": "Maxlinear, Inc",
    "801605": "Vodafone Italia S.p.A.",
    "A802DB": "zte corporation",
    "74F7F6": "Shanghai Sunmi Technology Co.,Ltd.",
    "241AE6": "Huawei Device Co., Ltd.",
    "C45444": "Quanta Computer Inc.",
    "001B24": "Quanta Computer Inc.",
    "00C09F": "Quanta Computer Inc.",
    "C80AA9": "Quanta Computer Inc.",
    "60EB69": "Quanta Computer Inc.",
    "B42330": "Itron Inc",
    "000895": "DIRC Technologie GmbH & Co.KG",
    "00269E": "Quanta Computer Inc.",
    "A022DE": "vivo Mobile Communication Co., Ltd.",
    "FC73FB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60B76E": "Google, Inc.",
    "00927D": "Ficosa Internationa(Taicang) C0.,Ltd.",
    "441622": "Microsoft Corporation",
    "044562": "ANDRA Sp. z o. o.",
    "94AEF0": "Cisco Systems, Inc",
    "DCD9AE": "Nokia Shanghai Bell Co., Ltd.",
    "C4366C": "LG Innotek",
    "1C08C1": "LG Innotek",
    "BCA993": "Cambium Networks Limited",
    "A497B1": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "44E6B0": "China Mobile IOT Company Limited",
    "747A90": "Murata Manufacturing Co., Ltd.",
    "58B623": "Beijing Xiaomi Mobile Software Co., Ltd",
    "14007D": "zte corporation",
    "24E9CA": "Huawei Device Co., Ltd.",
    "D440D0": "OCOSMOS Co., LTD",
    "98B8BC": "Samsung Electronics Co.,Ltd",
    "703AA6": "New H3C Technologies Co., Ltd",
    "8CCEFD": "Shenzhen zhouhai technology co.,LTD",
    "50FB19": "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.",
    "9408C7": "Huawei Device Co., Ltd.",
    "C8CA63": "Huawei Device Co., Ltd.",
    "9437F7": "Huawei Device Co., Ltd.",
    "184E16": "Samsung Electronics Co.,Ltd",
    "C03D03": "Samsung Electronics Co.,Ltd",
    "10746F": "MOTOROLA SOLUTIONS MALAYSIA SDN. BHD.",
    "D0C637": "Intel Corporate",
    "D49234": "NEC Corporation",
    "386893": "Intel Corporate",
    "903FEA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20AB48": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CCD73C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94E70B": "Intel Corporate",
    "046C59": "Intel Corporate",
    "D4ABCD": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "A0CAA5": "INTELLIGENCE TECHNOLOGY OF CEC CO., LTD",
    "E8854B": "Apple, Inc.",
    "E00EE4": "DWnet Technologies(Suzhou) Corporation",
    "58FDB1": "LG Electronics",
    "34EAE7": "Shanghai High-Flying Electronics  Technology Co., Ltd",
    "CC9ECA": "HMD Global Oy",
    "28EC95": "Apple, Inc.",
    "E02B96": "Apple, Inc.",
    "184593": "Taicang T&W Electronics",
    "F86C03": "Shenzhen Teleone Technology Co., Ltd",
    "105DDC": "Huawei Device Co., Ltd.",
    "DC7385": "Huawei Device Co., Ltd.",
    "5455D5": "Huawei Device Co., Ltd.",
    "F041C6": "Heat Tech Company, Ltd.",
    "C49886": "Qorvo International Pte. Ltd.",
    "E43A65": "MofiNetwork Inc",
    "54219D": "Samsung Electronics Co.,Ltd",
    "3C306F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "80E1BF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "482CD0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D8714D": "Texas Instruments",
    "0CEC80": "Texas Instruments",
    "70039F": "Espressif Inc.",
    "709F2D": "zte corporation",
    "5C0FFB": "Amino Communications Ltd",
    "E82A44": "Liteon Technology Corporation",
    "48E1E9": "Chengdu Meross Technology Co., Ltd.",
    "08B055": "ASKEY COMPUTER CORP",
    "A0DE0F": "Huawei Device Co., Ltd.",
    "F487C5": "Huawei Device Co., Ltd.",
    "882B94": "MADOKA SYSTEM Co.,Ltd.",
    "7CEF61": "STR Elektronik Josef Schlechtinger GmbH",
    "400589": "T-Mobile, USA",
    "64A965": "Linkflow Co., Ltd.",
    "3CF652": "zte corporation",
    "A80577": "Netlist, Inc.",
    "404028": "ZIV",
    "7458F3": "Amazon Technologies Inc.",
    "08AA55": "Motorola Mobility LLC, a Lenovo Company",
    "001354": "Zcomax Technologies, Inc.",
    "C09BF4": "IEEE Registration Authority",
    "30B237": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "68E209": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4308B": "Xiaomi Communications Co Ltd",
    "DC6B12": "worldcns inc.",
    "AC4A56": "Cisco Systems, Inc",
    "B0B5C3": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "282373": "Digita",
    "D0ABD5": "Intel Corporate",
    "D4C1C8": "zte corporation",
    "88D274": "zte corporation",
    "BC5A56": "Cisco Systems, Inc",
    "704A0E": "AMPAK Technology,Inc.",
    "BC0F9A": "D-Link International",
    "4CCE2D": "Danlaw Inc",
    "0C35FE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8C83DF": "Nokia",
    "30B9B0": "Intracom Asia Co., Ltd",
    "D4DACD": "SKY UK LIMITED",
    "F4B78D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A416E7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B40931": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94E7EA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94E4BA": "Huawei Device Co., Ltd.",
    "347146": "Huawei Device Co., Ltd.",
    "2CC546": "Huawei Device Co., Ltd.",
    "0C839A": "Huawei Device Co., Ltd.",
    "E0E0FC": "Huawei Device Co., Ltd.",
    "AC4B1E": "Integri-Sys.Com LLC",
    "6869CA": "Hitachi, Ltd.",
    "B0E4D5": "Google, Inc.",
    "88C397": "Beijing Xiaomi Mobile Software Co., Ltd",
    "F0F6C1": "Sonos, Inc.",
    "E0D4E8": "Intel Corporate",
    "305075": "GN Audio A/S",
    "90EC77": "silicom",
    "B440A4": "Apple, Inc.",
    "48B8A3": "Apple, Inc.",
    "F4DBE3": "Apple, Inc.",
    "BC428C": "ALPSALPINE CO,.LTD",
    "04BDBF": "Samsung Electronics Co.,Ltd",
    "BC7ABF": "Samsung Electronics Co.,Ltd",
    "60684E": "Samsung Electronics Co.,Ltd",
    "8020FD": "Samsung Electronics Co.,Ltd",
    "B4CE40": "Samsung Electronics Co.,Ltd",
    "D45EEC": "Beijing Xiaomi Electronics Co., Ltd.",
    "4C4576": "China Mobile(Hangzhou) Information Technology Co.,Ltd.",
    "74C929": "Zhejiang Dahua Technology Co., Ltd.",
    "94CC04": "IEEE Registration Authority",
    "9CC9EB": "NETGEAR",
    "5CB29E": "ASCO Power Technologies",
    "347839": "zte corporation",
    "D84DB9": "Wu Qi Technologies,Inc.",
    "A04F85": "LG Electronics (Mobile Communications)",
    "2C1A01": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24169D": "Cisco Systems, Inc",
    "F419E2": "Volterra",
    "64F2FB": "Hangzhou Ezviz Software Co.,Ltd.",
    "D807B6": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "646E97": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "6C0D34": "Nokia",
    "00233D": "Laird Technologies",
    "6C1632": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30809B": "New H3C Technologies Co., Ltd",
    "7422BB": "Huawei Device Co., Ltd.",
    "4C6371": "Xiaomi Communications Co Ltd",
    "5CBAEF": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "000ADB": "Trilliant",
    "30AB6A": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "8C6078": "Swissbit AG",
    "F80DF0": "zte corporation",
    "9CE91C": "zte corporation",
    "749BE8": "Hitron Technologies. Inc",
    "00DD25": "Shenzhen hechengdong Technology Co., Ltd",
    "0445A1": "NIRIT- Xinwei  Telecom Technology Co., Ltd.",
    "8020E1": "BVBA DPTechnics",
    "8C97EA": "FREEBOX SAS",
    "FC8E6E": "StreamCCTV, LLC",
    "788B2A": "Zhen Shi Information Technology (Shanghai) Co., Ltd.",
    "AC64CF": "FN-LINK TECHNOLOGY LIMITED",
    "3CFAD3": "IEEE Registration Authority",
    "5C17CF": "OnePlus Technology (Shenzhen) Co., Ltd",
    "102959": "Apple, Inc.",
    "F06728": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "E02AE6": "Fiberhome Telecommunication Technologies Co.,LTD",
    "14F6D8": "Intel Corporate",
    "3CDA6D": "Tiandy Technologies CO.,LTD",
    "A4FA76": "New H3C Technologies Co., Ltd",
    "9CE176": "Cisco Systems, Inc",
    "B41A1D": "Samsung Electronics Co.,Ltd",
    "E47684": "Apple, Inc.",
    "F05CD5": "Apple, Inc.",
    "70EA5A": "Apple, Inc.",
    "4CE176": "Cisco Systems, Inc",
    "1434F6": "LV SOLUTION SDN. BHD.",
    "18AFA1": "Shenzhen Yifang Network Technology Co., Ltd.",
    "54CE69": "Hikari Trading Co.,Ltd.",
    "6CAEF6": "eero inc.",
    "C014B8": "Nokia",
    "D028BA": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "A428B7": "Yangtze Memory Technologies Co., Ltd.",
    "9492D2": "KCF Technologies, Inc.",
    "E4A8DF": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "702F35": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C53C3": "Beijing Xiaomi Mobile Software Co., Ltd",
    "D83BBF": "Intel Corporate",
    "000DBB": "Nippon Dentsu Co.,Ltd.",
    "0CEE99": "Amazon Technologies Inc.",
    "FCF5C4": "Espressif Inc.",
    "E826B6": "Companies House to GlucoRx Technologies Ltd.",
    "00763D": "Veea",
    "48B02D": "NVIDIA Corporation",
    "588E81": "Silicon Laboratories",
    "902B34": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "94DE80": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "74D435": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "408D5C": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "6CC63B": "Taicang T&W Electronics",
    "3093BC": "Sagemcom Broadband SAS",
    "F4FEFB": "Samsung Electronics Co.,Ltd",
    "3C410E": "Cisco Systems, Inc",
    "207454": "vivo Mobile Communication Co., Ltd.",
    "B8C9B5": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "984914": "Wistron Neweb Corporation",
    "F86FDE": "Shenzhen Goodix Technology Co.,Ltd.",
    "0045E2": "CyberTAN Technology Inc.",
    "043F72": "Mellanox Technologies, Inc.",
    "1C4D66": "Amazon Technologies Inc.",
    "646624": "Sagemcom Broadband SAS",
    "B0F530": "Hitron Technologies. Inc",
    "001FF6": "PS Audio International",
    "006967": "IEEE Registration Authority",
    "001EB8": "Aloys, Inc",
    "0CE4A0": "Huawei Device Co., Ltd.",
    "BC1AE4": "Huawei Device Co., Ltd.",
    "740AE1": "Huawei Device Co., Ltd.",
    "B4A898": "Huawei Device Co., Ltd.",
    "502CC6": "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI",
    "0027E3": "Cisco Systems, Inc",
    "089C86": "Nokia Shanghai Bell Co., Ltd.",
    "F05136": "TCT mobile ltd",
    "105932": "Roku, Inc",
    "7817BE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "90AFD1": "netKTI Co., Ltd",
    "E04007": "Huawei Device Co., Ltd.",
    "90FD73": "zte corporation",
    "88ACC0": "Zyxel Communications Corporation",
    "1C1E38": "PCCW Global, Inc.",
    "90812A": "Apple, Inc.",
    "B87BC5": "Apple, Inc.",
    "4070F5": "Apple, Inc.",
    "B035B5": "Apple, Inc.",
    "800C67": "Apple, Inc.",
    "F072EA": "Google, Inc.",
    "EC9C32": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "4CADA8": "PANOPTICS CORP.",
    "981BB5": "ASSA ABLOY Korea Co., Ltd iRevo",
    "D42DC5": "i-PRO Co., Ltd.",
    "B887C6": "Prudential Technology co.,LTD",
    "FC1CA1": "Nokia",
    "6CD2BA": "zte corporation",
    "1C1ADF": "Microsoft Corporation",
    "D4F547": "Google, Inc.",
    "34CB1A": "Procter & Gamble Company",
    "F0B107": "Ericsson AB",
    "E8D03C": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "70CE8C": "Samsung Electronics Co.,Ltd",
    "783A6C": "TECNO MOBILE LIMITED",
    "C4E0DE": "Zhengzhou XindaJiean Information Technology Co.,Ltd.",
    "901A4F": "EM Microelectronic",
    "0004C9": "Micro Electron Co., Ltd.",
    "C43A35": "FN-LINK TECHNOLOGY LIMITED",
    "04D16E": "IEEE Registration Authority",
    "D88ADC": "Huawei Device Co., Ltd.",
    "CC6A10": "The Chamberlain Group, Inc",
    "C84F0E": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "10E953": "Huawei Device Co., Ltd.",
    "303ABA": "Guangzhou BaoLun Electronics Co., Ltd",
    "7C48B2": "Vida Resources Lte Ltd",
    "040E3C": "HP Inc.",
    "6C1C71": "Zhejiang Dahua Technology Co., Ltd.",
    "0C2FB0": "Samsung Electronics Co.,Ltd",
    "F03F95": "HUAWEI TECHNOLOGIES CO.,LTD",
    "185644": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9C69D1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78C881": "Sony Interactive Entertainment Inc.",
    "7048F7": "Nintendo Co.,Ltd",
    "18E1CA": "wanze",
    "CCEF03": "Hunan Keyshare Communication Technology Co., Ltd.",
    "ECBEDD": "Sagemcom Broadband SAS",
    "88571D": "Seongji Industry Company",
    "B04FC3": "Shenzhen NVC Cloud Technology Co., Ltd.",
    "309176": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "B40216": "Cisco Systems, Inc",
    "54A493": "IEEE Registration Authority",
    "8CC84B": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "749EA5": "OHSUNG",
    "D44F68": "Eidetic Communications Inc",
    "340F66": "Web Sensing LLC",
    "002272": "American Micro-Fuel Device Corp.",
    "00D0EF": "IGT",
    "D89790": "Commonwealth Scientific and Industrial Research Organisation",
    "405582": "Nokia",
    "A4E31B": "Nokia",
    "B8A58D": "Axe Group Holdings Limited",
    "50CEE3": "Gigafirm.co.LTD",
    "F497C2": "Nebulon Inc",
    "30FBB8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4BD9E": "Cisco Systems, Inc",
    "5885E9": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "A44519": "Xiaomi Communications Co Ltd",
    "8446FE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D82918": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C1D96": "Intel Corporate",
    "BC2392": "BYD Precision Manufacture Company Ltd.",
    "94E6F7": "Intel Corporate",
    "086083": "zte corporation",
    "E01954": "zte corporation",
    "7CF31B": "LG Electronics (Mobile Communications)",
    "10327E": "Huawei Device Co., Ltd.",
    "C419D1": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "64B21D": "Chengdu Phycom Tech Co., Ltd.",
    "C42996": "Signify B.V.",
    "B065F1": "WIO Manufacturing HK Limited",
    "901234": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "542A1B": "Sonos, Inc.",
    "38C4E8": "NSS Sp. z o.o.",
    "34DD7E": "Umeox Innovations Co.,Ltd",
    "CCCD64": "SM-Electronic GmbH",
    "24DFA7": "Hangzhou BroadLink Technology Co.,Ltd",
    "5C925E": "Zioncom Electronics (Shenzhen) Ltd.",
    "084FA9": "Cisco Systems, Inc",
    "084FF9": "Cisco Systems, Inc",
    "980637": "IEEE Registration Authority",
    "8CB84A": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "98E8FA": "Nintendo Co.,Ltd",
    "F8084F": "Sagemcom Broadband SAS",
    "1802AE": "vivo Mobile Communication Co., Ltd.",
    "0C20D3": "vivo Mobile Communication Co., Ltd.",
    "68DBF5": "Amazon Technologies Inc.",
    "2446C8": "Motorola Mobility LLC, a Lenovo Company",
    "5098B8": "New H3C Technologies Co., Ltd",
    "44D791": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B84DEE": "Hisense broadband multimedia technology Co.,Ltd",
    "D46BA6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC0577": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98BA39": "Doro AB",
    "E4CC9D": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "A8D0E3": "Systech Electronics Ltd",
    "308BB2": "Cisco Systems, Inc",
    "A89352": "SHANGHAI ZHONGMI COMMUNICATION TECHNOLOGY CO.,LTD",
    "E0EB62": "Shanghai Hulu Devices Co., Ltd",
    "6CE8C6": "Earda Technologies co Ltd",
    "1C4176": "China Mobile Group Device Co.,Ltd.",
    "48216C": "China Mobile IOT Company Limited",
    "8CBE24": "Tashang Semiconductor(Shanghai) Co., Ltd.",
    "94DC4E": "AEV, spol. s r. o.",
    "54DED0": "Sevio Srl",
    "08B3AF": "vivo Mobile Communication Co., Ltd.",
    "30862D": "Arista Network, Inc.",
    "E86F38": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "08688D": "New H3C Technologies Co., Ltd",
    "401920": "Movon Corporation",
    "608B0E": "Apple, Inc.",
    "1442FC": "Texas Instruments",
    "AC5D5C": "FN-LINK TECHNOLOGY LIMITED",
    "A4AE11": "Hon Hai Precision Industry Co., Ltd.",
    "2C1E4F": "Chengdu Qianli Network Technology Co., Ltd.",
    "009052": "SELCOM ELETTRONICA S.R.L.",
    "1871D5": "Hazens Automotive Electronics(SZ)Co.,Ltd.",
    "ACB1EE": "SHENZHEN FENDA TECHNOLOGY CO., LTD",
    "F8ADCB": "HMD Global Oy",
    "D03745": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "603A7C": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "000178": "MARGI Systems, Inc.",
    "001A83": "Pegasus Technologies Inc.",
    "50E085": "Intel Corporate",
    "6C5E3B": "Cisco Systems, Inc",
    "58C876": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "D462EA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54BAD6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0CB771": "ARRIS Group, Inc.",
    "24166D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "940B19": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70C7F2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DCA632": "Raspberry Pi Trading Ltd",
    "88F56E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C8C2FA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88B362": "Nokia Shanghai Bell Co., Ltd.",
    "0847D0": "Nokia Shanghai Bell Co., Ltd.",
    "CC9093": "Hansong Tehnologies",
    "BC97E1": "Broadcom Limited",
    "28D1B7": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "3894ED": "NETGEAR",
    "700433": "California Things Inc.",
    "CC64A6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "148430": "MITAC COMPUTING TECHNOLOGY CORPORATION",
    "B8A44F": "Axis Communications AB",
    "0024EB": "ClearPath Networks, Inc.",
    "D81399": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "54E019": "Ring LLC",
    "F0A968": "Antailiye Technology Co.,Ltd",
    "50AF4D": "zte corporation",
    "C8EAF8": "zte corporation",
    "848BCD": "IEEE Registration Authority",
    "08EDED": "Zhejiang Dahua Technology Co., Ltd.",
    "383B26": "Jiangsu Qinheng Co., Ltd.",
    "30317D": "Hosiden Corporation",
    "5CFAFB": "Acubit",
    "9C7BEF": "Hewlett Packard",
    "88299C": "Samsung Electronics Co.,Ltd",
    "7C8956": "Samsung Electronics Co.,Ltd",
    "742EDB": "Perinet GmbH",
    "201742": "LG Electronics",
    "CC8826": "LG Innotek",
    "EC5B73": "Advanced & Wise Technology Corp.",
    "D09C7A": "Xiaomi Communications Co Ltd",
    "1C697A": "EliteGroup Computer Systems Co., LTD",
    "4C1744": "Amazon Technologies Inc.",
    "B03055": "China Mobile IOT Company Limited",
    "14C03E": "ARRIS Group, Inc.",
    "C089AB": "ARRIS Group, Inc.",
    "D44DA4": "Murata Manufacturing Co., Ltd.",
    "DC7196": "Intel Corporate",
    "6882F2": "grandcentrix GmbH",
    "905C34": "Sirius Electronic Systems Srl",
    "D46A35": "Cisco Systems, Inc",
    "F8E5CF": "CGI IT UK LIMITED",
    "C82C2B": "IEEE Registration Authority",
    "8020DA": "Sagemcom Broadband SAS",
    "D420B0": "Mist Systems, Inc.",
    "0022AF": "Safety Vision, LLC",
    "A091A2": "OnePlus Electronics (Shenzhen) Co., Ltd.",
    "0080B5": "UNITED NETWORKS INC.",
    "00A0B0": "I-O DATA DEVICE,INC.",
    "9C497F": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "C4E39F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "2479F3": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "80A235": "Edgecore Networks Corporation",
    "402343": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "0021B7": "LEXMARK INTERNATIONAL, INC.",
    "C8C64A": "Flextronics Tech.(Ind) Pvt Ltd",
    "30EA26": "Sycada BV",
    "18F18E": "ChipER Technology co. ltd",
    "F89A78": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88F872": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC5623": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5486BC": "Cisco Systems, Inc",
    "B808CF": "Intel Corporate",
    "68847E": "FUJITSU LIMITED",
    "003085": "Cisco Systems, Inc",
    "6092F5": "ARRIS Group, Inc.",
    "605F8D": "eero inc.",
    "C4B36A": "Cisco Systems, Inc",
    "70F754": "AMPAK Technology,Inc.",
    "6C8BD3": "Cisco Systems, Inc",
    "68974B": "Shenzhen Costar Electronics Co. Ltd.",
    "34E1D1": "IEEE Registration Authority",
    "14B457": "Silicon Laboratories",
    "DC962C": "NST Audio Ltd",
    "50EC50": "Beijing Xiaomi Mobile Software Co., Ltd",
    "6063F9": "Ciholas, Inc.",
    "AC8FF8": "Nokia",
    "6003A6": "Inteno Broadband Technology AB",
    "44B295": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "9424E1": "Alcatel-Lucent Enterprise",
    "108286": "Luxshare Precision Industry Co.,Ltd",
    "6061DF": "Z-meta Research LLC",
    "7057BF": "New H3C Technologies Co., Ltd",
    "089798": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "18022D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D8BC59": "Shenzhen DAPU Microelectronics Co., Ltd",
    "8C79F5": "Samsung Electronics Co.,Ltd",
    "246F28": "Espressif Inc.",
    "00122A": "VTech Telecommunications Ltd.",
    "000422": "Studio Technologies, Inc",
    "80DA13": "eero inc.",
    "48F8DB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C08ACD": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "D81EDD": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "D43FCB": "ARRIS Group, Inc.",
    "B0518E": "Holl technology CO.Ltd.",
    "681729": "Intel Corporate",
    "2852E0": "Layon international Electronic & Telecom Co.,Ltd",
    "7C6166": "Amazon Technologies Inc.",
    "989BCB": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "ACF6F7": "LG Electronics (Mobile Communications)",
    "58CB52": "Google, Inc.",
    "F8CA59": "NetComm Wireless",
    "88B291": "Apple, Inc.",
    "C42AD0": "Apple, Inc.",
    "CCD281": "Apple, Inc.",
    "F84D33": "Fiberhome Telecommunication Technologies Co.,LTD",
    "107717": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "A86D5F": "Raisecom Technology CO., LTD",
    "48E6C0": "SIMCom Wireless Solutions Co.,Ltd.",
    "383C9C": "Fujian Newland Payment Technology Co.,Ltd.",
    "C02E25": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "200DB0": "Shenzhen Four Seas Global Link Network Technology Co., Ltd.",
    "100C6B": "NETGEAR",
    "58ECED": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "C4F0EC": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D88DC8": "Atil Technology Co., LTD",
    "E80FC8": "Universal Electronics, Inc.",
    "C8AACC": "Private",
    "002167": "HWA JIN T&I Corp.",
    "04EE03": "Texas Instruments",
    "4C2498": "Texas Instruments",
    "7CD95C": "Google, Inc.",
    "1C2E1B": "Suzhou Tremenet Communication Technology Co., Ltd.",
    "907A58": "Zegna-Daidong Limited",
    "E009BF": "SHENZHEN\u2002TONG BO WEI\u2002TECHNOLOGY Co.,LTD",
    "B00073": "Wistron Neweb Corporation",
    "001BF7": "Lund IP Products AB",
    "846991": "Nokia",
    "00131E": "peiker acustic GmbH",
    "783607": "Cermate Technologies Inc.",
    "1C24EB": "Burlywood",
    "001013": "Kontron America, Inc.",
    "A8E2C1": "Texas Instruments",
    "909A77": "Texas Instruments",
    "00EEAB": "Cisco Systems, Inc",
    "54A703": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "58C6F0": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "84A06E": "Sagemcom Broadband SAS",
    "884A18": "Opulinks",
    "A43EA0": "iComm HK LIMITED",
    "64C2DE": "LG Electronics (Mobile Communications)",
    "8C444F": "HUMAX Co., Ltd.",
    "006762": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2CC407": "machineQ",
    "0007CB": "FREEBOX SAS",
    "149FB6": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "00115A": "Ivoclar Vivadent AG",
    "40DF02": "LINE BIZ Plus",
    "DC31D1": "vivo Mobile Communication Co., Ltd.",
    "D43B04": "Intel Corporate",
    "B4ED19": "Pie Digital, Inc.",
    "84139F": "zte corporation",
    "2C2BF9": "LG Innotek",
    "900218": "SKY UK LIMITED",
    "144E2A": "Ciena Corporation",
    "7495EC": "ALPSALPINE CO,.LTD",
    "AC5AEE": "China Mobile Group Device Co.,Ltd.",
    "F051EA": "Fitbit, Inc.",
    "5033F0": "YICHEN (SHENZHEN) TECHNOLOGY CO.LTD",
    "4CE19E": "TECNO MOBILE LIMITED",
    "441AFA": "New H3C Technologies Co., Ltd",
    "04072E": "VTech Electronics Ltd.",
    "FC2BB2": "Actiontec Electronics, Inc",
    "E09F2A": "Iton Technology Corp. ",
    "70BC10": "Microsoft Corporation",
    "9C69B4": "IEEE Registration Authority",
    "0006F7": "ALPSALPINE CO,.LTD",
    "000704": "ALPSALPINE CO,.LTD",
    "0006F5": "ALPSALPINE CO,.LTD",
    "34C731": "ALPSALPINE CO,.LTD",
    "64D4BD": "ALPSALPINE CO,.LTD",
    "0498F3": "ALPSALPINE CO,.LTD",
    "500084": "Siemens Canada",
    "44B433": "tide.co.,ltd",
    "00214F": "ALPSALPINE CO,.LTD",
    "247D4D": "Texas Instruments",
    "DC21B9": "Sentec Co.Ltd",
    "6CDFFB": "IEEE Registration Authority",
    "203233": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "E498BB": "Phyplus Microelectronics Limited",
    "705E55": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "B0D568": "Shenzhen Cultraview Digital Technology Co., Ltd",
    "F00EBF": "ZettaHash Inc.",
    "04EA56": "Intel Corporate",
    "60A11E": "Wuhan Maxsine Electric Co.,Ltd.",
    "C45BF7": "ants",
    "8CDF9D": "NEC Corporation",
    "703509": "Cisco Systems, Inc",
    "5C415A": "Amazon.com, LLC",
    "A8B456": "Cisco Systems, Inc",
    "A49426": "Elgama-Elektronika Ltd.",
    "2CA9F0": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "549B72": "Ericsson AB",
    "9454DF": "YST CORP.",
    "780ED1": "TRUMPF Werkzeugmaschinen GmbH+Co.KG",
    "F82F08": "Molex CMS",
    "A047D7": "Best IT World (India) Pvt Ltd",
    "6899CD": "Cisco Systems, Inc",
    "1040F3": "Apple, Inc.",
    "4428A3": "Jiangsu fulian  Communication Technology Co., Ltd.",
    "10C595": "Lenovo",
    "6829DC": "Ficosa Electronics S.L.U.",
    "F80DF1": "Sontex SA",
    "7CBC84": "IEEE Registration Authority",
    "44E66E": "Apple, Inc.",
    "C0E862": "Apple, Inc.",
    "F40616": "Apple, Inc.",
    "586B14": "Apple, Inc.",
    "BCB863": "Apple, Inc.",
    "0CFE5D": "IEEE Registration Authority",
    "3C8D20": "Google, Inc.",
    "601D91": "Motorola Mobility LLC, a Lenovo Company",
    "D4C94B": "Motorola Mobility LLC, a Lenovo Company",
    "AC1585": "silergy corp",
    "AC5093": "Magna Electronics Europe GmbH & Co. OHG",
    "9CC8FC": "ARRIS Group, Inc.",
    "70BBE9": "Xiaomi Communications Co Ltd",
    "00D02D": "Resideo",
    "2C1CF6": "Alien Green LLC",
    "B07E11": "Texas Instruments",
    "08351B": "Shenzhen Jialihua Electronic Technology Co., Ltd",
    "806940": "LEXAR CO.,LIMITED",
    "64F81C": "Huawei Technologies Co., Ltd.",
    "10C753": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "F4951B": "Hefei Radio Communication Technology Co., Ltd ",
    "6C3845": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00B1E3": "Cisco Systems, Inc",
    "A41194": "Lenovo",
    "2C6104": "SHENZHEN FENGLIAN TECHNOLOGY CO., LTD.",
    "BC9325": "Ningbo Joyson Preh Car Connect Co.,Ltd.  ",
    "0C1C19": "LONGCONN ELECTRONICS(SHENZHEN) CO.,LTD",
    "E013B5": "vivo Mobile Communication Co., Ltd.",
    "E0795E": "Wuxi Xiaohu Technology Co.,Ltd.",
    "8C85E6": "Cleondris GmbH",
    "90E710": "New H3C Technologies Co., Ltd",
    "302952": "Hillstone Networks Inc",
    "049226": "ASUSTek COMPUTER INC.",
    "E8ADA6": "Sagemcom Broadband SAS",
    "D0B60A": "Xingluo Technology Company  Limited",
    "A4FC77": "Mega Well Limited",
    "50A0A4": "Nokia",
    "1098C3": "Murata Manufacturing Co., Ltd.",
    "84326F": "GUANGZHOU AVA ELECTRONICS TECHNOLOGY CO.,LTD ",
    "C89C13": "Inspiremobile",
    "E4B2FB": "Apple, Inc.",
    "2CCA0C": "WITHUS PLANET",
    "440049": "Amazon Technologies Inc.",
    "1838AE": "CONSPIN SOLUTION",
    "04CF8C": "XIAOMI Electronics,CO.,LTD",
    "0C7512": "Shenzhen Kunlun TongTai Technology  Co.,Ltd.",
    "0C4101": "Ruichi Auto Technology (Guangzhou) Co., Ltd.",
    "00B771": "Cisco Systems, Inc",
    "807D14": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20283E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F83880": "Apple, Inc.",
    "4C569D": "Apple, Inc.",
    "38539C": "Apple, Inc.",
    "402619": "Apple, Inc.",
    "6CE85C": "Apple, Inc.",
    "B0AE25": "Varikorea",
    "049162": "Microchip Technology Inc.",
    "983B8F": "Intel Corporate",
    "54278D": "NXP (China) Management Ltd.",
    "B0BE76": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "D003DF": "Samsung Electronics Co.,Ltd",
    "6CC374": "Texas Instruments",
    "684749": "Texas Instruments",
    "F8D9B8": "Open Mesh, Inc.",
    "7C696B": "Atmosic Technologies",
    "5CD20B": "Yytek Co., Ltd.",
    "4C1265": "ARRIS Group, Inc.",
    "00500C": "e-Tek Labs, Inc.",
    "485F99": "Cloud Network Technology (Samoa) Limited",
    "88F7BF": "vivo Mobile Communication Co., Ltd.",
    "D87D7F": "Sagemcom Broadband SAS",
    "ECC40D": "Nintendo Co.,Ltd",
    "50579C": "Seiko Epson Corporation",
    "8834FE": "Bosch Automotive Products (Suzhou) Co. Ltd",
    "2C79D7": "Sagemcom Broadband SAS",
    "00B4F5": "DongGuan Siyoto Electronics Co., Ltd       ",
    "4C1B86": "Arcadyan Corporation",
    "58B568": "SECURITAS DIRECT ESPA\u00d1A, SAU",
    "C423A2": "PT. Emsonic Indonesia",
    "B4CB57": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "FC039F": "Samsung Electronics Co.,Ltd",
    "00073A": "INVENTEL",
    "00266C": "INVENTEC CORPORATION",
    "008CFA": "INVENTEC CORPORATION",
    "846A66": "Sumitomo Kizai  Co.,Ltd.",
    "5CFB7C": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "10A24E": "GOLD3LINK ELECTRONICS CO., LTD",
    "88108F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4631F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A49B4F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00051A": "3COM EUROPE LTD",
    "08004E": "3COM EUROPE LTD",
    "00301E": "3COM EUROPE LTD",
    "005004": "3COM",
    "000103": "3COM",
    "02C08C": "3COM",
    "18D717": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "80B624": "IVS",
    "DCF505": "AzureWave Technology Inc.",
    "CCF0FD": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "8489EC": "IEEE Registration Authority",
    "1C1BB5": "Intel Corporate",
    "0057C1": "LG Electronics (Mobile Communications)",
    "7C240C": "Telechips, Inc.",
    "00203D": "Honeywell Environmental & Combustion Controls",
    "004084": "Honeywell",
    "D4741B": "Beijing HuaDa ZhiBao Electronic System Co.,Ltd.",
    "A0950C": "China Mobile IOT Company Limited",
    "2C15E1": "Phicomm (Shanghai) Co., Ltd.",
    "A4D990": "Samsung Electronics Co.,Ltd",
    "DCF719": "Cisco Systems, Inc",
    "30D16B": "Liteon Technology Corporation",
    "006087": "KANSAI ELECTRIC CO., LTD.",
    "98AE71": "VVDN Technologies Pvt Ltd",
    "AC6E1A": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "30D32D": "devolo AG",
    "0080EB": "COMPCONTROL B.V.",
    "0002EB": "Pico Communications",
    "342EB6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC9232": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC8B28": "Intel Corporate",
    "B869F4": "Routerboard.com",
    "000FB0": "Compal Electronics INC.",
    "00023F": "Compal Electronics INC.",
    "001555": "DFM GmbH",
    "BCF2AF": "devolo AG",
    "E0AF4F": "Deutsche Telekom AG",
    "68A8E1": "Wacom Co.,Ltd.",
    "283A4D": "Cloud Network Technology (Samoa) Limited",
    "B87C6F": "NXP (China) Management Ltd.",
    "305DA6": "ADVALY SYSTEM Inc.",
    "BC30D9": "Arcadyan Corporation",
    "0479B7": "Texas Instruments",
    "C0D0FF": "China Mobile IOT Company Limited",
    "88DF9E": "New H3C Technologies Co., Ltd",
    "2C7CE4": "Wuhan Tianyu Information Industry Co., Ltd.",
    "24D76B": "Syntronic AB",
    "C4FEE2": "AMICCOM Electronics Corporation",
    "144802": "THE YEOLRIM Co.,Ltd.",
    "FC4AE9": "Castlenet Technology Inc.",
    "40313C": "XIAOMI Electronics,CO.,LTD",
    "FC4596": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "A0E534": "Stratec Biomedical AG",
    "1C7508": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "001B38": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "00235A": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "444B5D": "GE Healthcare",
    "8CA048": "Beijing NeTopChip Technology Co.,LTD",
    "24D3F2": "zte corporation",
    "D469A5": "Miura Systems Ltd.",
    "E01283": " Shenzhen Fanzhuo Communication Technology Co., Lt",
    "E8C57A": "Ufispace Co., LTD.",
    "A0CF5B": "Cisco Systems, Inc",
    "002421": "MICRO-STAR INT'L CO., LTD.",
    "0060D1": "CASCADE COMMUNICATIONS",
    "0C8C24": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "805E4F": "FN-LINK TECHNOLOGY LIMITED",
    "8C6D77": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C8126": "ARCOM",
    "D47C44": "IEEE Registration Authority",
    "302432": "Intel Corporate",
    "24F57E": "HWH CO., LTD.",
    "84C9C6": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "780CF0": "Cisco Systems, Inc",
    "7054B4": "Vestel Elektronik San ve Tic. A.S.",
    "20A60C": "Xiaomi Communications Co Ltd",
    "505967": "Intent Solutions Inc",
    "000680": "Card Access, Inc.",
    "88B66B": "easynetworks",
    "488AD2": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "DCE838": "CK Telecom (Shenzhen) Limited",
    "A8D498": "Avira Operations GmbH & Co. KG",
    "60D02C": "Ruckus Wireless",
    "D058FC": "SKY UK LIMITED",
    "3856B5": "Peerbridge Health Inc",
    "14579F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B44326": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C576C": "Samsung Electronics Co.,Ltd",
    "841766": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "2C4D79": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "04AB18": "ELECOM CO.,LTD.",
    "D0C5D8": "LATECOERE",
    "000C42": "Routerboard.com",
    "0026BD": "JTEC Card &amp; Communication Co., Ltd",
    "7C96D2": "Fihonest communication co.,Ltd",
    "78D294": "NETGEAR",
    "709FA9": "TECNO MOBILE LIMITED",
    "0C01DB": "Infinix mobility limited",
    "08C5E1": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "1866C7": "Shenzhen Libre Technology Co., Ltd",
    "5CB3F6": "Human, Incorporated",
    "2C4835": "IEEE Registration Authority",
    "482AE3": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "B0EB57": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1869DA": "China Mobile Group Device Co.,Ltd.",
    "FCA6CD": "Fiberhome Telecommunication Technologies Co.,LTD",
    "44C874": "China Mobile Group Device Co.,Ltd.",
    "74C14F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A85B6C": "Robert Bosch Gmbh, CM-CI2",
    "C8B1EE": "Qorvo",
    "00CBB4": "SHENZHEN ATEKO PHOTOELECTRICITY CO.,LTD",
    "4CC00A": "vivo Mobile Communication Co., Ltd.",
    "9CE82B": "vivo Mobile Communication Co., Ltd.",
    "7079B3": "Cisco Systems, Inc",
    "149B2F": "JiangSu ZhongXie Intelligent Technology co., LTD",
    "48DD9D": "ITEL MOBILE LIMITED",
    "A075EA": "BoxLock, Inc.",
    "F04CD5": "Maxlinear, Inc",
    "00FCBA": "Cisco Systems, Inc",
    "F85C4D": "Nokia",
    "2C584F": "ARRIS Group, Inc.",
    "3835FB": "Sagemcom Broadband SAS",
    "0001AE": "Trex Enterprises",
    "00E009": "Stratus Technologies",
    "E4EA83": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "74EC42": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D4FC13": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A0B045": "Halong Mining",
    "ECAF97": "GIT",
    "E0BAB4": "Arrcus, Inc",
    "807D3A": "Espressif Inc.",
    "9C7F57": "UNIC Memory Technology Co Ltd",
    "B4E01D": "CONCEPTION ELECTRONIQUE",
    "1C0042": "NARI Technology Co., Ltd.",
    "68CAE4": "Cisco Systems, Inc",
    "781D4A": "zte corporation",
    "001027": "L-3 COMMUNICATIONS EAST",
    "589B0B": "Shineway Technologies, Inc.",
    "D8160A": "Nippon Electro-Sensory Devices",
    "10C07C": "Blu-ray Disc Association",
    "E4B021": "Samsung Electronics Co.,Ltd",
    "4434A7": "ARRIS Group, Inc.",
    "3CE1A1": "Universal Global Scientific Industrial Co., Ltd.",
    "F898EF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58F987": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8F5AC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58BAD4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC2643": "Elprotronic Inc.",
    "04E229": "Qingdao Haier Technology Co.,Ltd",
    "701D08": "99IOT Shenzhen co.,ltd",
    "7412BB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00BE3B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CA177": "HUAWEI TECHNOLOGIES CO.,LTD",
    "242E02": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78B6EC": "Scuf Gaming International LLC",
    "8035C1": "Xiaomi Communications Co Ltd",
    "9CE895": "New H3C Technologies Co., Ltd",
    "00583F": "PC Aquarius",
    "903D68": "G-Printec, Inc.",
    "1094BB": "Apple, Inc.",
    "08DFCB": "Systrome Networks",
    "A4933F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "34E894": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "28AC9E": "Cisco Systems, Inc",
    "F86FC1": "Apple, Inc.",
    "28FF3C": "Apple, Inc.",
    "F099B6": "Apple, Inc.",
    "88E9FE": "Apple, Inc.",
    "38892C": "Apple, Inc.",
    "348B75": "LAVA INTERNATIONAL(H.K) LIMITED",
    "749EAF": "Apple, Inc.",
    "94BF2D": "Apple, Inc.",
    "2047DA": "Xiaomi Communications Co Ltd",
    "58B3FC": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "2429FE": "KYOCERA Corporation ",
    "7C49EB": "XIAOMI Electronics,CO.,LTD",
    "C43306": "China Mobile Group Device Co.,Ltd.",
    "984562": "Shanghai Baud Data Communication Co.,Ltd.",
    "68D482": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "10E7C6": "Hewlett Packard",
    "202D23": "Collinear Networks Inc.",
    "90834B": "BEIJING YUNYI TIMES TECHNOLOGY CO,.LTD",
    "18502A": "SOARNEX",
    "A433D7": "MitraStar Technology Corp.",
    "A8367A": "frogblue TECHNOLOGY GmbH",
    "B0ACD2": "zte corporation",
    "200F70": "FOXTECH",
    "007147": "Amazon Technologies Inc.",
    "00BE75": "Cisco Systems, Inc",
    "6CE4DA": "NEC Platforms, Ltd.",
    "E4C483": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "001FBA": "Boyoung Tech",
    "DCA333": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "1831BF": "ASUSTek COMPUTER INC.",
    "C048E6": "Samsung Electronics Co.,Ltd",
    "882E5A": "storONE",
    "788C54": "Ping Communication",
    "F00FEC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC075F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8AF67": "Hewlett Packard",
    "C098DA": "China Mobile IOT Company Limited",
    "C8FAE1": "ARQ Digital LLC",
    "10A4B9": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "80CE62": "Hewlett Packard",
    "801F12": "Microchip Technology Inc.",
    "506CBE": "InnosiliconTechnology Ltd",
    "247E12": "Cisco Systems, Inc",
    "501CB0": "Cisco Systems, Inc",
    "04C241": "Nokia",
    "78257A": "LEO Innovation Lab",
    "8CF710": "AMPAK Technology, Inc.",
    "307BAC": "New H3C Technologies Co., Ltd",
    "3C479B": "Theissen Training Systems, Inc.",
    "606BFF": "Nintendo Co.,Ltd",
    "785DC8": "LG Electronics",
    "0C5415": "Intel Corporate",
    "3C0461": "ARRIS Group, Inc.",
    "E8C1B8": " Nanjing Bangzhong Electronic Commerce Limited",
    "D8D775": "Sagemcom Broadband SAS",
    "B4A8B9": "Cisco Systems, Inc",
    "50DCE7": "Amazon Technologies Inc.",
    "649829": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "081DC4": "Thermo Fisher Scientific Messtechnik GmbH",
    "785364": "SHIFT GmbH",
    "883D24": "Google, Inc.",
    "04AC44": "Holtek Semiconductor Inc.",
    "F4DCA5": "DAWON DNS",
    "D460E3": "Sercomm Corporation.",
    "40CBC0": "Apple, Inc.",
    "E8330D": "Xaptec GmbH",
    "38E60A": "Xiaomi Communications Co Ltd",
    "08AED6": "Samsung Electronics Co.,Ltd",
    "CC4D38": "Carnegie Technologies",
    "005D73": "Cisco Systems, Inc",
    "606D3C": "Luxshare Precision Industry Company Limited",
    "002790": "Cisco Systems, Inc",
    "5C5F67": "Intel Corporate",
    "803A59": "AT&T",
    "588D64": "Xi'an Clevbee Technology Co.,Ltd",
    "34BA38": "PAL MOHAN ELECTRONICS PVT LTD",
    "9829A6": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "DCBFE9": "Motorola Mobility LLC, a Lenovo Company",
    "CC6EA4": "Samsung Electronics Co.,Ltd",
    "A816D0": "Samsung Electronics Co.,Ltd",
    "A46CF1": "Samsung Electronics Co.,Ltd",
    "C4618B": "Apple, Inc.",
    "08E689": "Apple, Inc.",
    "DC56E7": "Apple, Inc.",
    "4064A4": "THE FURUKAWA ELECTRIC CO., LTD",
    "D45DDF": "PEGATRON CORPORATION",
    "F065C2": "Yanfeng Visteon Electronics Technology (Shanghai) Co.,Ltd.",
    "70B7E2": "Jiangsu Miter Technology Co.,Ltd.",
    "A09D86": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "00C0FF": "Seagate Cloud Systems Inc",
    "88D171": "BEGHELLI S.P.A",
    "A825EB": "Cambridge Industries(Group) Co.,Ltd.",
    "34E380": "Genexis B.V.",
    "5C5819": "Jingsheng Technology Co., Ltd.",
    "B8CA04": "Holtek Semiconductor Inc.",
    "C4C563": "TECNO MOBILE LIMITED",
    "F8B568": "IEEE Registration Authority",
    "2C6B7D": "Texas Instruments",
    "6CB2AE": "Cisco Systems, Inc",
    "80B708": "Blue Danube Systems, Inc",
    "B0982B": "Sagemcom Broadband SAS",
    "34FA9F": "Ruckus Wireless",
    "A09DC1": "China Dragon Technology Limited",
    "2C4205": "Lytx",
    "6C7660": "KYOCERA CORPORATION ",
    "002102": "UpdateLogic Inc.",
    "F41E5E": "RtBrick Inc.",
    "505800": "WyTec International, Inc.",
    "0CEAC9": "ARRIS Group, Inc.",
    "C8D12A": "Comtrend Corporation",
    "BCF292": "PLANTRONICS, INC.",
    "0450DA": "Qiku Internet Network Scientific (Shenzhen) Co., Ltd",
    "E820E2": "HUMAX Co., Ltd.",
    "1890D8": "Sagemcom Broadband SAS",
    "88835D": "FN-LINK TECHNOLOGY LIMITED",
    "F81D0F": "Hitron Technologies. Inc",
    "0005A7": "HYPERCHIP Inc.",
    "0026A8": "DAEHAP HYPER-TECH",
    "2031EB": "HDSN",
    "F8C96C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "844823": "WOXTER TECHNOLOGY Co. Ltd",
    "08BC20": "Hangzhou Royal Cloud Technology Co., Ltd",
    "942A3F": "Diversey Inc",
    "785C28": "Prime Motion Inc.",
    "EC9F0D": "IEEE Registration Authority",
    "E078A3": "Shanghai Winner Information Technology Co.,Inc",
    "10A4BE": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "947BBE": "Ubicquia LLC",
    "ECC06A": "PowerChord Group Limited",
    "944996": "WiSilica Inc",
    "68A682": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "0CB2B7": "Texas Instruments",
    "587A62": "Texas Instruments",
    "547A52": "CTE International srl",
    "3408BC": "Apple, Inc.",
    "1C36BB": "Apple, Inc.",
    "3C2EFF": "Apple, Inc.",
    "8C5BF0": "ARRIS Group, Inc.",
    "088466": "Novartis Pharma AG",
    "309FFB": "Ardomus Networks Corporation",
    "F06E0B": "Microsoft Corporation",
    "346FED": "Enovation Controls",
    "24F677": "Apple, Inc.",
    "B0CA68": "Apple, Inc.",
    "C83C85": "Apple, Inc.",
    "5433CB": "Apple, Inc.",
    "00E025": "dit Co., Ltd.",
    "001530": "Dell EMC",
    "78870D": "Unifiedgateways India Private Limited",
    "A88200": "Hisense Electric Co.,Ltd",
    "3820A8": "ColorTokens, Inc.",
    "705896": "InShow Technology",
    "00B69F": "Latch",
    "9CE063": "Samsung Electronics Co.,Ltd",
    "D03169": "Samsung Electronics Co.,Ltd",
    "842C80": "Sichuan Changhong Electric Ltd.",
    "3CC079": "Shenzhen One-Nine Intelligent Electronic Science and Technology Co., Ltd",
    "000589": "National Datacomputer",
    "3CA616": "vivo Mobile Communication Co., Ltd.",
    "98C5DB": "Ericsson AB",
    "BC0543": "AVM GmbH",
    "149F3C": "Samsung Electronics Co.,Ltd",
    "FCEEE6": "FORMIKE ELECTRONIC CO., LTD",
    "84E327": "TAILYN TECHNOLOGIES INC",
    "0021B8": "Inphi Corporation",
    "0C9160": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "D8ED1C": "Magna Technology SL",
    "D83134": "Roku, Inc",
    "A0648F": "ASKEY COMPUTER CORP",
    "C850E9": "Raisecom Technology CO., LTD",
    "F46E24": "NEC Personal Computers, Ltd.",
    "888279": "Shenzhen RB-LINK Intelligent Technology Co.Ltd",
    "10F163": "TNK CO.,LTD",
    "88DA1A": "Redpine Signals, Inc.",
    "98EF9B": "OHSUNG",
    "14CF8D": "OHSUNG",
    "104400": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0E17E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4A7C5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0FE61": "Vivint Wireless Inc. ",
    "5C2BF5": "Vivint Wireless Inc. ",
    "2C279E": "IEEE Registration Authority",
    "B430C0": "York Instruments Ltd",
    "E81DA8": "Ruckus Wireless",
    "5C8D2D": "Shanghai Wellpay Information Technology Co., Ltd",
    "68C63A": "Espressif Inc.",
    "408BF6": "Shenzhen TCL New Technology Co., Ltd",
    "00006B": "Silicon Graphics",
    "74373B": "UNINET Co.,Ltd.",
    "7C6456": "Samsung Electronics Co.,Ltd",
    "90FD9F": "Silicon Laboratories",
    "F03D03": "TECNO MOBILE LIMITED",
    "DCF090": "Nubia Technology Co.,Ltd.",
    "CC5A53": "Cisco Systems, Inc",
    "00C0EE": "KYOCERA Display Corporation",
    "245FDF": "KYOCERA CORPORATION ",
    "646E69": "Liteon Technology Corporation",
    "706BB9": "Cisco Systems, Inc",
    "006088": "Analog Devices, Inc.",
    "084ACF": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "9C63ED": "zte corporation",
    "74F661": "Schneider Electric Fire & Security Oy",
    "8C5F48": "Continental Intelligent Transportation Systems LLC",
    "947EB9": "National Narrowband Network Communications Pty Ltd",
    "D4389C": "Sony Corporation",
    "001BD3": "Panasonic Corporation AVC Networks Company",
    "CC7EE7": "Panasonic Corporation AVC Networks Company",
    "20C6EB": "Panasonic Corporation AVC Networks Company",
    "64B5C6": "Nintendo Co.,Ltd",
    "001987": "Panasonic Mobile Communications Co.,Ltd.",
    "BCC342": "Panasonic Communications Co., Ltd.",
    "2CB21A": "Phicomm (Shanghai) Co., Ltd.",
    "00289F": "Semptian Co., Ltd.",
    "AC1F74": "Apple, Inc.",
    "A85C2C": "Apple, Inc.",
    "00DB70": "Apple, Inc.",
    "B8634D": "Apple, Inc.",
    "A4E975": "Apple, Inc.",
    "3035AD": "Apple, Inc.",
    "844167": "Apple, Inc.",
    "9800C6": "Apple, Inc.",
    "FCE557": "Nokia Corporation",
    "48C58D": "Lear Corporation GmbH",
    "24C42F": "Philips Lifeline",
    "E470B8": "Intel Corporate",
    "9C305B": "Hon Hai Precision Ind. Co.,Ltd.",
    "D4D2E5": "BKAV Corporation",
    "0050B5": "FICHET SECURITE ELECTRONIQUE",
    "001439": "Blonder Tongue Laboratories, Inc",
    "00869C": "Palo Alto Networks",
    "104E89": "Garmin International",
    "D8C497": "Quanta Computer Inc.",
    "C84029": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F86EEE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "84802D": "Cisco Systems, Inc",
    "00188D": "Nokia Danmark A/S",
    "0015AB": "PRO CO SOUND INC",
    "5876C5": "DIGI I'S LTD",
    "94D9B3": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "00A0AC": "GILAT SATELLITE NETWORKS, LTD.",
    "741C27": "ITEL MOBILE LIMITED",
    "2830AC": "Frontiir Co. Ltd.",
    "78D800": "IEEE Registration Authority",
    "0835B2": "CoreEdge Networks Co., Ltd",
    "4C49E3": "Xiaomi Communications Co Ltd",
    "002609": "Phyllis Co., Ltd.",
    "28F537": "IEEE Registration Authority",
    "7802B1": "Cisco Systems, Inc",
    "F83441": "Intel Corporate",
    "44EA4B": "Actlas Inc.",
    "C4CB6B": "Airista Flow, Inc.",
    "54666C": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "A89675": "Motorola Mobility LLC, a Lenovo Company",
    "B4C170": "Yi chip Microelectronics (Hangzhou) Co., Ltd",
    "28D436": "Jiangsu dewosi electric co., LTD",
    "D4B27A": "ARRIS Group, Inc.",
    "389AF6": "Samsung Electronics Co.,Ltd",
    "E0AA96": "Samsung Electronics Co.,Ltd",
    "507705": "Samsung Electronics Co.,Ltd",
    "245880": "VIZEO",
    "188090": "Cisco Systems, Inc",
    "B40F3B": "Tenda Technology Co.,Ltd.Dongguan branch",
    "A8B2DA": "FUJITSU LIMITED",
    "786256": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B05508": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70DF2F": "Cisco Systems, Inc",
    "3894E0": "Syrotech Networks. Ltd.",
    "B875C0": "PayPal, Inc.",
    "001C71": "Emergent Electronics",
    "B090D4": "Shenzhen Hoin Internet Technology Co., Ltd",
    "8C9F3B": "Qingdao Hisense Communications Co.,Ltd.",
    "001F41": "Ruckus Wireless",
    "842096": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "C4108A": "Ruckus Wireless",
    "F0B052": "Ruckus Wireless",
    "AC6706": "Ruckus Wireless",
    "044FAA": "Ruckus Wireless",
    "589396": "Ruckus Wireless",
    "0014B8": "Hill-Rom",
    "ACED5C": "Intel Corporate",
    "001A93": "ERCO Leuchten GmbH",
    "94F665": "Ruckus Wireless",
    "10CDB6": "Essential Products, Inc.",
    "9061AE": "Intel Corporate",
    "50184C": "Platina Systems Inc.",
    "00EC0A": "Xiaomi Communications Co Ltd",
    "C81FEA": "Avaya Inc",
    "34F64B": "Intel Corporate",
    "A4F3E7": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "E43A6E": "Shenzhen Zeroone Technology CO.,LTD",
    "6CF9D2": "CHENGDU POVODO ELECTRONIC TECHNOLOGY CO., LTD",
    "F46BEF": "Sagemcom Broadband SAS",
    "08306B": "Palo Alto Networks",
    "60271C": "VIDEOR E. Hartig GmbH",
    "0018AE": "TVT CO.,LTD",
    "8891DD": "Racktivity",
    "F4B7B3": "vivo Mobile Communication Co., Ltd.",
    "30B62D": " Mojo Networks, Inc.",
    "6C60EB": "ZHI YUAN ELECTRONICS CO., LIMITED",
    "74DADA": "D-Link International",
    "D8F1F0": "Pepxim International Limited",
    "9097F3": "Samsung Electronics Co.,Ltd",
    "7C1C68": "Samsung Electronics Co.,Ltd",
    "C087EB": "Samsung Electronics Co.,Ltd",
    "C8D7B0": "Samsung Electronics Co.,Ltd",
    "FCA667": "Amazon Technologies Inc.",
    "447BBB": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "1C4593": "Texas Instruments",
    "90EC50": "C.O.B.O. SPA",
    "2C41A1": "Bose Corporation",
    "4C38D8": "ARRIS Group, Inc.",
    "9C7BD2": "NEOLAB Convergence",
    "D0F88C": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "2CB115": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "DCC8F5": "Shanghai UMEinfo CO.,LTD.",
    "88D7F6": "ASUSTek COMPUTER INC.",
    "10D07A": "AMPAK Technology, Inc.",
    "C4ABB2": "vivo Mobile Communication Co., Ltd.",
    "44B412": "SIUS AG",
    "0CB912": "JM-DATA GmbH",
    "3CA308": "Texas Instruments",
    "00E18C": "Intel Corporate",
    "881544": "Cisco Meraki",
    "002449": "Shen Zhen Lite Star Electronics Technology Co., Ltd",
    "04714B": "IEEE Registration Authority",
    "847933": "profichip GmbH",
    "6045CB": "ASUSTek COMPUTER INC.",
    "F43E61": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "B4417A": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "D4503F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "388C50": "LG Electronics",
    "DC0856": "Alcatel-Lucent Enterprise",
    "E442A6": "Intel Corporate",
    "3C678C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28A6DB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14A0F8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98AAFC": "IEEE Registration Authority",
    "1CDA27": "vivo Mobile Communication Co., Ltd.",
    "90F305": "HUMAX Co., Ltd.",
    "4C8120": "Taicang T&W Electronics",
    "00118B": "Alcatel-Lucent Enterprise",
    "00E0B1": "Alcatel-Lucent Enterprise",
    "00E0DA": "Alcatel-Lucent Enterprise",
    "4095BD": "NTmore.Co.,Ltd",
    "E8C1D7": "Philips",
    "C8F86D": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "F8BE0D": "A2UICT Co.,Ltd.",
    "00143F": "Hotway Technology Corporation",
    "F8FF0B": "Electronic Technology Inc.",
    "144FD7": "IEEE Registration Authority",
    "B85510": "Zioncom Electronics (Shenzhen) Ltd.",
    "70F087": "Apple, Inc.",
    "00D318": "SPG Controls",
    "D055B2": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "7C6BF7": "NTI co., ltd.",
    "245BA7": "Apple, Inc.",
    "6091F3": "vivo Mobile Communication Co., Ltd.",
    "28395E": "Samsung Electronics Co.,Ltd",
    "38295A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "88E628": "Shenzhen Kezhonglong Optoelectronic Technology Co.,Ltd",
    "000C46": "Allied Telesyn Inc.",
    "001F72": "QingDao Hiphone Technology Co,.Ltd",
    "002365": "Insta Elektro GmbH",
    "40FA7F": "Preh Car Connect GmbH",
    "CC82EB": "KYOCERA CORPORATION ",
    "D4CF37": "Symbolic IO",
    "409F38": "AzureWave Technology Inc.",
    "049573": "zte corporation",
    "F0D7AA": "Motorola Mobility LLC, a Lenovo Company",
    "3096FB": "Samsung Electronics Co.,Ltd",
    "4827EA": "Samsung Electronics Co.,Ltd",
    "000631": "Calix Inc.",
    "BC2F3D": "vivo Mobile Communication Co., Ltd.",
    "7C787E": "Samsung Electronics Co.,Ltd",
    "D47DFC": "TECNO MOBILE LIMITED",
    "B816DB": "CHANT SINCERE CO.,LTD",
    "E05163": "Arcadyan Corporation",
    "54E3F6": "Alcatel-Lucent",
    "40B034": "Hewlett Packard",
    "3034D2": "Availink, Inc.",
    "504061": "Nokia",
    "00108E": "HUGH SYMONS CONCEPT Technologies Ltd.",
    "641A22": "Heliospectra AB",
    "8CF5A3": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "F42B48": "Ubiqam",
    "2C7E81": "ARRIS Group, Inc.",
    "D47AE2": "Samsung Electronics Co.,Ltd",
    "3CBD3E": "Beijing Xiaomi Electronics Co., Ltd.",
    "40B4CD": "Amazon Technologies Inc.",
    "407D0F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68CC6E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50F14A": "Texas Instruments",
    "04DEF2": "Shenzhen ECOM Technology Co. Ltd",
    "A084CB": "SonicSensory,Inc.",
    "78C1A7": "zte corporation",
    "4C7872": "Cav. Uff. Giacomo Cimberio S.p.A. ",
    "D8E0E1": "Samsung Electronics Co.,Ltd",
    "00A0C8": "Adtran Inc",
    "D8C771": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4B169": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "E44790": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "8CC8F4": "IEEE Registration Authority",
    "F483E1": "Shanghai Clouder Semiconductor Co.,Ltd",
    "083E5D": "Sagemcom Broadband SAS",
    "540384": "Hongkong Nano IC Technologies Co., Ltd",
    "6854FD": "Amazon Technologies Inc.",
    "70918F": "Weber-Stephen Products LLC",
    "0003BC": "COT GmbH",
    "7C1015": "Brilliant Home Technology, Inc.",
    "CCB8A8": "AMPAK Technology, Inc.",
    "34049E": "IEEE Registration Authority",
    "94FB29": "Zebra Technologies Inc.",
    "74614B": "Chongqing Huijiatong Information Technology Co., Ltd.",
    "98D293": "Google, Inc.",
    "6C19C0": "Apple, Inc.",
    "E02A82": "Universal Global Scientific Industrial Co., Ltd.",
    "B0F963": "Hangzhou H3C Technologies Co., Limited",
    "D490E0": "Topcon Electronics GmbH & Co. KG",
    "E02202": "ARRIS Group, Inc.",
    "D825B0": "Rockeetech Systems Co.,Ltd.",
    "F49651": "NAKAYO Inc",
    "446246": "Comat AG",
    "38454C": "Light Labs, Inc.",
    "B0702D": "Apple, Inc.",
    "00204F": "DEUTSCHE AEROSPACE AG",
    "8CE117": "zte corporation",
    "688AF0": "zte corporation",
    "C0210D": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "4CE2F1": "Udino srl",
    "00DBDF": "Intel Corporate",
    "A84041": "Dragino Technology Co., Limited",
    "1077B0": "Fiberhome Telecommunication Technologies Co.,LTD",
    "80D4A5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04B0E7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "446A2E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A06FAA": "LG Innotek",
    "94A04E": "Bostex Technology Co., LTD",
    "FC10C6": "Taicang T&W Electronics",
    "F01DBC": "Microsoft Corporation",
    "B0EE7B": "Roku, Inc",
    "AC587B": "JCT Healthcare",
    "B47447": "CoreOS",
    "0026AB": "Seiko Epson Corporation",
    "1062EB": "D-Link International",
    "000894": "InnoVISION Multimedia Ltd.",
    "504B5B": "CONTROLtronic GmbH",
    "C8662C": "Beijing Haitai Fangyuan High Technology Co,.Ltd.",
    "8096CA": "Hon Hai Precision Ind. Co.,Ltd.",
    "186571": "Top Victory Electronics (Taiwan) Co., Ltd.",
    "2C6FC9": "Hon Hai Precision Ind. Co.,Ltd.",
    "000B4F": "Verifone",
    "60C798": "Verifone",
    "D058A8": "zte corporation",
    "D071C4": "zte corporation",
    "B4B384": "ShenZhen Figigantic Electronic Co.,Ltd",
    "F83F51": "Samsung Electronics Co.,Ltd",
    "50D753": "CONELCOM GmbH",
    "34D270": "Amazon Technologies Inc.",
    "0CC47A": "Super Micro Computer, Inc.",
    "A0CC2B": "Murata Manufacturing Co., Ltd.",
    "00C003": "GLOBALNET COMMUNICATIONS",
    "00234A": "Private",
    "7828CA": "Sonos, Inc.",
    "6CA7FA": "YOUNGBO ENGINEERING INC.",
    "8C7EB3": "Lytro, Inc.",
    "E0D9E3": "Eltex Enterprise Ltd.",
    "805EC0": "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD.",
    "0023D2": "Inhand Electronics, Inc.",
    "00271C": "MERCURY CORPORATION",
    "1C40E8": "SHENZHEN PROGRESS&WIN TECHNOLOGY CO.,LTD",
    "2C402B": "Smart iBlue Technology Limited",
    "5C6B4F": "Hello Inc.",
    "404E36": "HTC Corporation",
    "50795B": "Interexport Telecomunicaciones S.A.",
    "0016D9": "NINGBO BIRD CO.,LTD.",
    "5001D9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C9924": "ARRIS Group, Inc.",
    "A8E705": "Fiberhome Telecommunication Technologies Co.,LTD",
    "60A4D0": "Samsung Electronics Co.,Ltd",
    "886B0F": "Bluegiga Technologies OY",
    "AC84C9": "Sagemcom Broadband SAS",
    "9C62AB": "Sumavision Technologies Co.,Ltd",
    "188B15": "ShenZhen ZhongRuiJing Technology co.,LTD",
    "4CE173": "IEEE Registration Authority",
    "0CD86C": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "DC0B34": "LG Electronics (Mobile Communications)",
    "049790": "Lartech telecom LLC",
    "28EED3": "Shenzhen Super D Technology Co., Ltd",
    "24C44A": "zte corporation",
    "98541B": "Intel Corporate",
    "007B18": "SENTRY Co., LTD.",
    "144D67": "Zioncom Electronics (Shenzhen) Ltd.",
    "0060D6": "NovAtel Inc.",
    "008701": "Samsung Electronics Co.,Ltd",
    "5C9960": "Samsung Electronics Co.,Ltd",
    "245CBF": "NCSE",
    "B0958E": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C025E9": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "2C3361": "Apple, Inc.",
    "C8F946": "LOCOSYS Technology Inc.",
    "00137E": "CorEdge Networks, Inc.",
    "D814D6": "SURE SYSTEM Co Ltd",
    "6CEFC6": "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD.",
    "101DC0": "Samsung Electronics Co.,Ltd",
    "487B6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "883FD3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CA933": "Luma Home",
    "C8028F": "Nova Electronics (Shanghai) Co., Ltd.",
    "000B14": "ViewSonic Corporation",
    "A46011": "Verifone",
    "240D65": "Shenzhen Vsun Communication Technology Co., Ltd.",
    "78471D": "Samsung Electronics Co.,Ltd",
    "A07591": "Samsung Electronics Co.,Ltd",
    "0CDFA4": "Samsung Electronics Co.,Ltd",
    "68EBAE": "Samsung Electronics Co.,Ltd",
    "444E1A": "Samsung Electronics Co.,Ltd",
    "002341": "Vanderbilt International (SWE) AB ",
    "407C7D": "Nokia",
    "24590B": "White Sky Inc. Limited",
    "143365": "TEM Mobile Limited",
    "6CDEA9": "Cisco Meraki",
    "A8469D": "Cisco Meraki",
    "D8EC5E": "Belkin International Inc.",
    "E89F80": "Belkin International Inc.",
    "0050CB": "Bucher Automation AG",
    "B072BF": "Murata Manufacturing Co., Ltd.",
    "701DC4": "NorthStar Battery Company, LLC",
    "58EF68": "Belkin International Inc.",
    "CC03D9": "Cisco Meraki",
    "0C8DDB": "Cisco Meraki",
    "5C8613": "Beijing Zhoenet Technology Co., Ltd",
    "CC7314": "HONG KONG WHEATEK TECHNOLOGY LIMITED",
    "B8EE65": "Liteon Technology Corporation",
    "985BB0": "KMDATA INC.",
    "0011FF": "Digitro Tecnologia Ltda",
    "001B94": "T.E.M.A. S.p.A.",
    "BC8556": "Hon Hai Precision Ind. Co.,Ltd.",
    "342387": "Hon Hai Precision Ind. Co.,Ltd.",
    "64DAA0": "Robert Bosch Smart Home GmbH",
    "14B837": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "E47CF9": "Samsung Electronics Co.,Ltd",
    "4844F7": "Samsung Electronics Co.,Ltd",
    "001377": "Samsung Electronics Co.,Ltd",
    "002454": "Samsung Electronics Co.,Ltd",
    "E81132": "Samsung Electronics Co.,Ltd",
    "50B7C3": "Samsung Electronics Co.,Ltd",
    "1C5A3E": "Samsung Electronics Co.,Ltd",
    "A02195": "Samsung Electronics Co.,Ltd",
    "B07870": "Wi-NEXT, Inc.",
    "5CA39D": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "90187C": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "50CCF8": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "002637": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "002119": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "F4D9FB": "Samsung Electronics Co.,Ltd",
    "3C6200": "Samsung Electronics Co.,Ltd",
    "9439E5": "Hon Hai Precision Ind. Co.,Ltd.",
    "642737": "Hon Hai Precision Ind. Co.,Ltd.",
    "A41731": "Hon Hai Precision Ind. Co.,Ltd.",
    "E006E6": "Hon Hai Precision Ind. Co.,Ltd.",
    "F0F002": "Hon Hai Precision Ind. Co.,Ltd.",
    "C0CB38": "Hon Hai Precision Ind. Co.,Ltd.",
    "F07BCB": "Hon Hai Precision Ind. Co.,Ltd.",
    "C417FE": "Hon Hai Precision Ind. Co.,Ltd.",
    "00223B": "Communication Networks, LLC",
    "C0F8DA": "Hon Hai Precision Ind. Co.,Ltd.",
    "0017D5": "Samsung Electronics Co.,Ltd",
    "001E7D": "Samsung Electronics Co.,Ltd",
    "001DF6": "Samsung Electronics Co.,Ltd",
    "001D25": "Samsung Electronics Co.,Ltd",
    "E4C1F1": "SHENZHEN SPOTMAU INFORMATION TECHNOLIGY CO., Ltd  ",
    "240AC4": "Espressif Inc.",
    "F06BCA": "Samsung Electronics Co.,Ltd",
    "AC3613": "Samsung Electronics Co.,Ltd",
    "BC8CCD": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "D022BE": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "EC9BF3": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "343111": "Samsung Electronics Co.,Ltd",
    "08FD0E": "Samsung Electronics Co.,Ltd",
    "041BBA": "Samsung Electronics Co.,Ltd",
    "889B39": "Samsung Electronics Co.,Ltd",
    "E432CB": "Samsung Electronics Co.,Ltd",
    "10D542": "Samsung Electronics Co.,Ltd",
    "A0821F": "Samsung Electronics Co.,Ltd",
    "F409D8": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "18D276": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C06599": "Samsung Electronics Co.,Ltd",
    "BC79AD": "Samsung Electronics Co.,Ltd",
    "4C3C16": "Samsung Electronics Co.,Ltd",
    "0073E0": "Samsung Electronics Co.,Ltd",
    "F0E77E": "Samsung Electronics Co.,Ltd",
    "F008F1": "Samsung Electronics Co.,Ltd",
    "58C38B": "Samsung Electronics Co.,Ltd",
    "002611": "Licera AB",
    "005218": "Wuxi Keboda Electron Co.Ltd",
    "00E3B2": "Samsung Electronics Co.,Ltd",
    "301966": "Samsung Electronics Co.,Ltd",
    "94350A": "Samsung Electronics Co.,Ltd",
    "F0038C": "AzureWave Technology Inc.",
    "A81B6A": "Texas Instruments",
    "985DAD": "Texas Instruments",
    "D43639": "Texas Instruments",
    "BC282C": "e-Smart Systems Pvt. Ltd",
    "A40DBC": "Xiamen Intretech Inc.",
    "B0F893": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "001E81": "CNB Technology Inc.",
    "641269": "ARRIS Group, Inc.",
    "0002C9": "Mellanox Technologies, Inc.",
    "7CA97D": "Objenious",
    "A8A648": "Qingdao Hisense Communications Co.,Ltd.",
    "84EF18": "Intel Corporate",
    "C411E0": "Bull Group Co., Ltd",
    "343DC4": "BUFFALO.INC",
    "005094": "ARRIS Group, Inc.",
    "E0B7B1": "ARRIS Group, Inc.",
    "D82522": "ARRIS Group, Inc.",
    "28C87A": "ARRIS Group, Inc.",
    "7C0623": "Ultra Electronics Sonar System Division",
    "00126C": "Visonic Technologies 1993 Ltd.",
    "0080C7": "XIRCOM",
    "000138": "XAVi Technologies Corp.",
    "00166D": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "3C9157": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "0000D8": "Novell, Inc.",
    "14825B": "Hefei Radio Communication Technology Co., Ltd ",
    "AC6175": "HUAWEI TECHNOLOGIES CO.,LTD",
    "244427": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48FD8E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28AC67": "Mach Power, Rappresentanze Internazionali s.r.l.",
    "080051": "ExperData",
    "001F46": "Nortel Networks",
    "003093": "Sonnet Technologies, Inc",
    "00034B": "Nortel Networks",
    "000E40": "Nortel Networks",
    "001C9C": "Nortel Networks",
    "001B25": "Nortel Networks",
    "0019E1": "Nortel Networks",
    "28BE03": "TCT mobile ltd",
    "903AE6": "PARROT SA",
    "A098ED": "Shandong Intelligent Optical Communication Development Co., Ltd.",
    "000EF4": "Kasda Networks Inc",
    "00167A": "Skyworth Overseas Development Ltd.",
    "002561": "ProCurve Networking by HP",
    "008058": "PRINTER SYSTEMS CORP.",
    "00157D": "POSDATA",
    "4849C7": "Samsung Electronics Co.,Ltd",
    "10E878": "Nokia",
    "FCB0C4": "Shanghai DareGlobal Technologies Co.,Ltd",
    "A89DD2": "Shanghai DareGlobal Technologies Co.,Ltd",
    "00E00F": "Shanghai Baud Data Communication Co.,Ltd.",
    "00140D": "Nortel Networks",
    "849866": "Samsung Electronics Co.,Ltd",
    "1880F5": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "001D42": "Nortel Networks",
    "A42940": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "E4A387": "Control Solutions LLC",
    "10F96F": "LG Electronics (Mobile Communications)",
    "C4438F": "LG Electronics (Mobile Communications)",
    "A09169": "LG Electronics (Mobile Communications)",
    "286C07": "XIAOMI Electronics,CO.,LTD",
    "002280": "A2B Electronics AB",
    "404AD4": "Widex A/S",
    "9893CC": "LG ELECTRONICS INC",
    "3CCD93": "LG ELECTRONICS INC",
    "2021A5": "LG Electronics (Mobile Communications)",
    "6CD68A": "LG Electronics (Mobile Communications)",
    "002483": "LG Electronics (Mobile Communications)",
    "001FE3": "LG Electronics (Mobile Communications)",
    "2C54CF": "LG Electronics (Mobile Communications)",
    "485929": "LG Electronics (Mobile Communications)",
    "58A2B5": "LG Electronics (Mobile Communications)",
    "000F62": "Alcatel Bell Space N.V.",
    "F485C6": "FDT Technologies",
    "0019AB": "Raycom CO ., LTD",
    "4C334E": "HIGHTECH",
    "CC79CF": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "001925": "Intelicis Corporation",
    "9476B7": "Samsung Electronics Co.,Ltd",
    "B8D812": "IEEE Registration Authority",
    "1C21D1": "IEEE Registration Authority",
    "7C70BC": "IEEE Registration Authority",
    "E81863": "IEEE Registration Authority",
    "2CD141": "IEEE Registration Authority",
    "3C39E7": "IEEE Registration Authority",
    "BC6641": "IEEE Registration Authority",
    "80E4DA": "IEEE Registration Authority",
    "885D90": "IEEE Registration Authority",
    "C88ED1": "IEEE Registration Authority",
    "B01F81": "IEEE Registration Authority",
    "88124E": "Qualcomm Inc.",
    "001B32": "QLogic Corporation",
    "0017CA": "Qisda Corporation",
    "F0421C": "Intel Corporate",
    "001CD8": "BlueAnt Wireless",
    "283638": "IEEE Registration Authority",
    "C8755B": "Quantify Technology Pty. Ltd.",
    "4C6641": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "A854B2": "Wistron Neweb Corporation",
    "E09579": "ORTHOsoft inc, d/b/a Zimmer CAS",
    "A0ADA1": "JMR Electronics, Inc",
    "E0ACF1": "Cisco Systems, Inc",
    "00015B": "ITALTEL S.p.A/RF-UP-I",
    "BCC00F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "9CA5C0": "vivo Mobile Communication Co., Ltd.",
    "3044A1": "Shanghai Nanchao Information Technology",
    "00A0A8": "RENEX CORPORATION",
    "70F395": "Universal Global Scientific Industrial Co., Ltd.",
    "90A4DE": "Wistron Neweb Corporation",
    "70E284": "Wistron Infocomm (Zhongshan) Corporation",
    "0015B7": "Toshiba",
    "E89D87": "Toshiba",
    "001428": "Vocollect Inc",
    "006B9E": "Vizio, Inc",
    "5CA86A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00C0AB": "Telco Systems, Inc. ",
    "001D7E": "Cisco-Linksys, LLC",
    "E4FB8F": "MOBIWIRE MOBILES (NINGBO) CO.,LTD",
    "3876CA": "Shenzhen Smart Intelligent Technology Co.Ltd",
    "F40A4A": "INDUSNET Communication Technology Co.,LTD",
    "0023F8": "Zyxel Communications Corporation",
    "0019CB": "Zyxel Communications Corporation",
    "10BD55": "Q-Lab Corporation",
    "B40418": "Smartchip Integrated Inc.",
    "90CF7D": "Qingdao Hisense Communications Co.,Ltd.",
    "C449BB": "MITSUMI ELECTRIC CO.,LTD.",
    "FC2D5E": "zte corporation",
    "2C094D": "Raptor Engineering, LLC",
    "AC3743": "HTC Corporation",
    "F85A00": "Sanford LP",
    "B8BBAF": "Samsung Electronics Co.,Ltd",
    "60C5AD": "Samsung Electronics Co.,Ltd",
    "442C05": "AMPAK Technology, Inc.",
    "6038E0": "Belkin International Inc.",
    "8C897A": "AUGTEK",
    "002550": "Riverbed Technology, Inc.",
    "FC55DC": "Baltic Latvian Universal Electronics LLC",
    "042758": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9CE374": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CD2E9": "YOKOTE SEIKO CO., LTD.",
    "50AB3E": "Qibixx AG",
    "24F094": "Apple, Inc.",
    "086D41": "Apple, Inc.",
    "ECADB8": "Apple, Inc.",
    "9801A7": "Apple, Inc.",
    "002552": "VXi Corporation",
    "F4911E": "ZHUHAI EWPE INFORMATION TECHNOLOGY INC",
    "7C738B": "Cocoon Alarm Ltd",
    "B4D5BD": "Intel Corporate",
    "98AA3C": "Will i-tech Co., Ltd.",
    "DC3752": "GE",
    "6879ED": "SHARP Corporation",
    "608D17": "Sentrus Government Systems Division, Inc",
    "002382": "Lih Rong electronic Enterprise Co., Ltd.",
    "F845AD": "Konka Group Co., Ltd.",
    "000FE2": "Hangzhou H3C Technologies Co., Limited",
    "6CE983": "Gastron Co., LTD.",
    "28E31F": "Xiaomi Communications Co Ltd",
    "DCD916": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00022E": "TEAC Corp. R& D",
    "0060B0": "Hewlett Packard",
    "F80F84": "Natural Security SAS",
    "44A42D": "TCT mobile ltd",
    "80F62E": "Hangzhou H3C Technologies Co., Limited",
    "70F96D": "Hangzhou H3C Technologies Co., Limited",
    "083FBC": "zte corporation",
    "00C0F0": "Kingston Technology Company, Inc.",
    "0018D7": "JAVAD GNSS, Inc.",
    "001F09": "Jastec",
    "AC620D": "Jabil Circuit(Wuxi) Co.,Ltd",
    "1C7370": "Neotech",
    "30E37A": "Intel Corporate",
    "BC6A44": "Commend International GmbH",
    "F0EE58": "PACE Telematics GmbH",
    "4CA003": "VITEC",
    "08000D": "International Computers, Ltd",
    "B8AEED": "Elitegroup Computer Systems Co.,Ltd.",
    "0000C9": "Emulex Corporation",
    "0040AA": "Valmet Automation",
    "7050AF": "SKY UK LIMITED",
    "F4EF9E": "SGSG SCIENCE & TECHNOLOGY CO. LTD",
    "1C740D": "Zyxel Communications Corporation",
    "603ECA": "Cambridge Medical Robotics Ltd",
    "9C741A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4A8B6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "244C07": "HUAWEI TECHNOLOGIES CO.,LTD",
    "746FF7": "Wistron Neweb Corporation",
    "000DB0": "Olym-tech Co.,Ltd.",
    "30F6B9": "Ecocentric Energy",
    "D0B0CD": "Moen",
    "00020E": "ECI Telecom Ltd.",
    "200A5E": "Xiangshan Giant Eagle Technology Developing Co., Ltd.",
    "001F1F": "Edimax Technology Co. Ltd.",
    "B0E2E5": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00C164": "Cisco Systems, Inc",
    "CCB11A": "Samsung Electronics Co.,Ltd",
    "703C03": "RadiAnt Co.,Ltd",
    "40476A": "Astro Gaming",
    "001FA7": "Sony Interactive Entertainment Inc.",
    "9046A2": "Tedipay UK Ltd",
    "6479A7": "Phison Electronics Corp.   ",
    "1C5F2B": "D-Link International",
    "CCD3E2": "Jiangsu Yinhe  Electronics Co.,Ltd.",
    "E4FAED": "Samsung Electronics Co.,Ltd",
    "288335": "Samsung Electronics Co.,Ltd",
    "DCCF96": "Samsung Electronics Co.,Ltd",
    "AC44F2": "YAMAHA CORPORATION",
    "1CEA1B": "Nokia",
    "D4612E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54511B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68536C": "SPnS Co.,Ltd",
    "DC2DCB": "Beijing Unis HengYue Technology Co., Ltd.",
    "2C9662": "Invenit BV",
    "84683E": "Intel Corporate",
    "743E2B": "Ruckus Wireless",
    "E0C767": "Apple, Inc.",
    "80ED2C": "Apple, Inc.",
    "CCB3AB": "shenzhen Biocare Bio-Medical Equipment Co.,Ltd.",
    "E4B318": "Intel Corporate",
    "E0CDFD": "Beijing E3Control Technology Co, LTD",
    "F03404": "TCT mobile ltd",
    "80D160": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "047E4A": "moobox CO., Ltd.",
    "0080E5": "NetApp",
    "9C5C8E": "ASUSTek COMPUTER INC.",
    "30785C": "Partow Tamas Novin (Parman)",
    "60ACC8": "KunTeng Inc.",
    "C88722": "Lumenpulse",
    "70661B": "Sonova AG",
    "B07FB9": "NETGEAR",
    "3C591E": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "002682": "Gemtek Technology Co., Ltd.",
    "0009E1": "Gemtek Technology Co., Ltd.",
    "14C126": "Nokia Corporation",
    "600194": "Espressif Inc.",
    "D8F710": "Libre Wireless Technologies Inc.",
    "84BA3B": "CANON INC.",
    "A87E33": "Nokia Danmark A/S",
    "002403": "Nokia Danmark A/S",
    "002404": "Nokia Danmark A/S",
    "8CA2FD": "Starry, Inc.",
    "001B33": "Nokia Danmark A/S",
    "00194F": "Nokia Danmark A/S",
    "00BD3A": "Nokia Corporation",
    "80501B": "Nokia Corporation",
    "A04E04": "Nokia Corporation",
    "AC61EA": "Apple, Inc.",
    "38B54D": "Apple, Inc.",
    "1C5CF2": "Apple, Inc.",
    "001262": "Nokia Danmark A/S",
    "0014A7": "Nokia Danmark A/S",
    "0015A0": "Nokia Danmark A/S",
    "0016BC": "Nokia Danmark A/S",
    "00174B": "Nokia Danmark A/S",
    "0019B7": "Nokia Danmark A/S",
    "0017B0": "Nokia Danmark A/S",
    "0015DE": "Nokia Danmark A/S",
    "0002EE": "Nokia Danmark A/S",
    "002669": "Nokia Danmark A/S",
    "002109": "Nokia Danmark A/S",
    "002108": "Nokia Danmark A/S",
    "A4C7DE": "Cambridge Industries(Group) Co.,Ltd.",
    "0019C7": "Cambridge Industries(Group) Co.,Ltd.",
    "70D931": "Cambridge Industries(Group) Co.,Ltd.",
    "C43655": "Shenzhen Fenglian Technology Co., Ltd.",
    "E0B9E5": "Technicolor Delivery Technologies Belgium NV",
    "0270B3": "DATA RECALL LTD.",
    "000136": "CyberTAN Technology Inc.",
    "0030DA": "Comtrend Corporation",
    "64680C": "Comtrend Corporation",
    "3872C0": "Comtrend Corporation",
    "A80600": "Samsung Electronics Co.,Ltd",
    "0060BB": "Cabletron Systems, Inc.",
    "D8B8F6": "Nantworks",
    "008077": "Brother industries, LTD.",
    "C4576E": "Samsung Electronics Co.,Ltd",
    "508569": "Samsung Electronics Co.,Ltd",
    "029D8E": "CARDIAC RECORDERS, INC.",
    "00402A": "Canoga Perkins Corporation",
    "A0B4A5": "Samsung Electronics Co.,Ltd",
    "E4F8EF": "Samsung Electronics Co.,Ltd",
    "24F5AA": "Samsung Electronics Co.,Ltd",
    "988389": "Samsung Electronics Co.,Ltd",
    "84A466": "Samsung Electronics Co.,Ltd",
    "F05A09": "Samsung Electronics Co.,Ltd",
    "503275": "Samsung Electronics Co.,Ltd",
    "08FC88": "Samsung Electronics Co.,Ltd",
    "F8D0BD": "Samsung Electronics Co.,Ltd",
    "78595E": "Samsung Electronics Co.,Ltd",
    "0C1420": "Samsung Electronics Co.,Ltd",
    "94B10A": "Samsung Electronics Co.,Ltd",
    "3CBBFD": "Samsung Electronics Co.,Ltd",
    "A48431": "Samsung Electronics Co.,Ltd",
    "140C76": "FREEBOX SAS",
    "D04D2C": "Roku, Inc.",
    "B0A737": "Roku, Inc.",
    "001BE9": "Broadcom",
    "DC64B8": "Shenzhen JingHanDa Electronics Co.Ltd",
    "44EE02": "MTI Ltd.",
    "5856E8": "ARRIS Group, Inc.",
    "F80BBE": "ARRIS Group, Inc.",
    "001333": "BaudTec Corporation",
    "58671A": "Barnes&Noble",
    "002675": "Aztech Electronics Pte Ltd",
    "0024FE": "AVM GmbH",
    "C02506": "AVM GmbH",
    "405D82": "NETGEAR",
    "DCEF09": "NETGEAR",
    "DC446D": "Allwinner Technology Co., Ltd",
    "745AAA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04FE8D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8D3F7": "Arcadyan Technology Corporation",
    "000D92": "ARIMA Communications Corp.",
    "002163": "ASKEY COMPUTER CORP",
    "DC4517": "ARRIS Group, Inc.",
    "C8AA21": "ARRIS Group, Inc.",
    "F87B7A": "ARRIS Group, Inc.",
    "4C60DE": "NETGEAR",
    "C43DC7": "NETGEAR",
    "489D24": "BlackBerry RTS",
    "08BD43": "NETGEAR",
    "0015CE": "ARRIS Group, Inc.",
    "0015A2": "ARRIS Group, Inc.",
    "0015A3": "ARRIS Group, Inc.",
    "0015A4": "ARRIS Group, Inc.",
    "0000CA": "ARRIS Group, Inc.",
    "001FE4": "Sony Corporation",
    "002345": "Sony Corporation",
    "6C0E0D": "Sony Corporation",
    "6C23B9": "Sony Corporation",
    "707E43": "ARRIS Group, Inc.",
    "00152F": "ARRIS Group, Inc.",
    "0017EE": "ARRIS Group, Inc.",
    "00111A": "ARRIS Group, Inc.",
    "000F9F": "ARRIS Group, Inc.",
    "002642": "ARRIS Group, Inc.",
    "0024A1": "ARRIS Group, Inc.",
    "002210": "ARRIS Group, Inc.",
    "0022B4": "ARRIS Group, Inc.",
    "002395": "ARRIS Group, Inc.",
    "001A80": "Sony Corporation",
    "0012EE": "Sony Corporation",
    "001620": "Sony Corporation",
    "001963": "Sony Corporation",
    "0004BD": "ARRIS Group, Inc.",
    "00E06F": "ARRIS Group, Inc.",
    "8096B1": "ARRIS Group, Inc.",
    "0023AF": "ARRIS Group, Inc.",
    "001FC4": "ARRIS Group, Inc.",
    "001CFB": "ARRIS Group, Inc.",
    "00149A": "ARRIS Group, Inc.",
    "0014E8": "ARRIS Group, Inc.",
    "0019C0": "ARRIS Group, Inc.",
    "3017C8": "Sony Corporation",
    "205476": "Sony Corporation",
    "703C39": "SEAWING Kft",
    "9097D5": "Espressif Inc.",
    "ACD074": "Espressif Inc.",
    "38E3C5": "Taicang T&W Electronics",
    "8841FC": "AirTies Wireless Networks",
    "98743D": "Shenzhen Jun Kai Hengye Technology Co. Ltd",
    "A0F459": "FN-LINK TECHNOLOGY LIMITED",
    "586356": "FN-LINK TECHNOLOGY LIMITED",
    "0025D3": "AzureWave Technology Inc.",
    "A04FD4": "ADB Broadband Italia",
    "8CB864": "AcSiP Technology Corp.",
    "5CE2F4": "AcSiP Technology Corp.",
    "B8616F": "Accton Technology Corp",
    "0012CF": "Accton Technology Corp",
    "0030F1": "Accton Technology Corp",
    "705A0F": "Hewlett Packard",
    "4495FA": "Qingdao Santong Digital Technology Co.Ltd",
    "74C246": "Amazon Technologies Inc.",
    "000FA3": "Alpha Networks Inc.",
    "001D6A": "Alpha Networks Inc.",
    "0000F4": "Allied Telesis, Inc.",
    "001577": "Allied Telesis, Inc.",
    "1C4BD6": "AzureWave Technology Inc.",
    "08A95A": "AzureWave Technology Inc.",
    "94DBC9": "AzureWave Technology Inc.",
    "240A64": "AzureWave Technology Inc.",
    "40E230": "AzureWave Technology Inc.",
    "80D21D": "AzureWave Technology Inc.",
    "709E29": "Sony Interactive Entertainment Inc.",
    "A4DB30": "Liteon Technology Corporation",
    "40F02F": "Liteon Technology Corporation",
    "0030D3": "Agilent Technologies, Inc.",
    "00A02F": "ADB Broadband Italia",
    "D4D184": "ADB Broadband Italia",
    "D00ED9": "Taicang T&W Electronics",
    "541473": " Wingtech Group (HongKong\uff09Limited",
    "0CCC26": "Airenetworks",
    "001F3B": "Intel Corporate",
    "00215D": "Intel Corporate",
    "00216A": "Intel Corporate",
    "001676": "Intel Corporate",
    "0016EA": "Intel Corporate",
    "90E2BA": "Intel Corporate",
    "0026C7": "Intel Corporate",
    "0026C6": "Intel Corporate",
    "88532E": "Intel Corporate",
    "E09467": "Intel Corporate",
    "08D40C": "Intel Corporate",
    "6C8814": "Intel Corporate",
    "7CCCB8": "Intel Corporate",
    "F40669": "Intel Corporate",
    "001DE1": "Intel Corporate",
    "E09D31": "Intel Corporate",
    "8086F2": "Intel Corporate",
    "303A64": "Intel Corporate",
    "ACFDCE": "Intel Corporate",
    "74C99A": "Ericsson AB",
    "5CC213": "Fr. Sauter AG",
    "28101B": "MagnaCom",
    "001495": "2Wire Inc",
    "C46699": "vivo Mobile Communication Co., Ltd.",
    "C8F230": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "8C0EE3": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "982CBE": "2Wire Inc",
    "640F28": "2Wire Inc",
    "884AEA": "Texas Instruments",
    "001B77": "Intel Corporate",
    "001CC0": "Intel Corporate",
    "104A7D": "Intel Corporate",
    "001556": "Sagemcom Broadband SAS",
    "C0D044": "Sagemcom Broadband SAS",
    "A01B29": "Sagemcom Broadband SAS",
    "74E14A": "IEEE Registration Authority",
    "0CEFAF": "IEEE Registration Authority",
    "F80278": "IEEE Registration Authority",
    "A0BB3E": "IEEE Registration Authority",
    "B07994": "Motorola Mobility LLC, a Lenovo Company",
    "A470D6": "Motorola Mobility LLC, a Lenovo Company",
    "C8CD72": "Sagemcom Broadband SAS",
    "0022A4": "2Wire Inc",
    "B499BA": "Hewlett Packard",
    "0C47C9": "Amazon Technologies Inc.",
    "0050BA": "D-Link Corporation",
    "00179A": "D-Link Corporation",
    "001CF0": "D-Link Corporation",
    "001E58": "D-Link Corporation",
    "0022B0": "D-Link Corporation",
    "002401": "D-Link Corporation",
    "1CAFF7": "D-Link International",
    "14D64D": "D-Link International",
    "9094E4": "D-Link International",
    "0030C5": "CADENCE DESIGN SYSTEMS, INC.",
    "348AAE": "Sagemcom Broadband SAS",
    "F82C18": "2Wire Inc",
    "18017D": "Harbin Arteor technology co., LTD",
    "944452": "Belkin International Inc.",
    "0014BF": "Cisco-Linksys, LLC",
    "BC0DA5": "Texas Instruments",
    "CC8CE3": "Texas Instruments",
    "E0D7BA": "Texas Instruments",
    "7C03D8": "Sagemcom Broadband SAS",
    "C0AC54": "Sagemcom Broadband SAS",
    "2C3996": "Sagemcom Broadband SAS",
    "F08261": "Sagemcom Broadband SAS",
    "D08CB5": "Texas Instruments",
    "00182F": "Texas Instruments",
    "0017EA": "Texas Instruments",
    "0021BA": "Texas Instruments",
    "1CE2CC": "Texas Instruments",
    "985945": "Texas Instruments",
    "D494A1": "Texas Instruments",
    "B0B448": "Texas Instruments",
    "287CDB": "Hefei  Toycloud Technology Co.,ltd",
    "CCB255": "D-Link International",
    "28107B": "D-Link International",
    "FC7516": "D-Link International",
    "84C9B2": "D-Link International",
    "C8D3A3": "D-Link International",
    "3CBB73": "Shenzhen Xinguodu Technology Co., Ltd.",
    "047863": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "806AB0": "Shenzhen TINNO Mobile Technology Corp.",
    "48AD08": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C5AB4": "Google, Inc.",
    "F4F5E8": "Google, Inc.",
    "94EB2C": "Google, Inc.",
    "4CFB45": "HUAWEI TECHNOLOGIES CO.,LTD",
    "009ACD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70B3D5": "IEEE Registration Authority",
    "28ED6A": "Apple, Inc.",
    "409F87": "Jide Technology (Hong Kong) Limited",
    "0CF9C0": "SKY UK LIMITED",
    "4CFF12": "Fuze Entertainment Co., ltd",
    "0CC731": "Currant, Inc.",
    "AC9A22": "NXP Semiconductors",
    "8461A0": "ARRIS Group, Inc.",
    "5C8FE0": "ARRIS Group, Inc.",
    "BCCAB5": "ARRIS Group, Inc.",
    "D039B3": "ARRIS Group, Inc.",
    "000FCC": "ARRIS Group, Inc.",
    "38F23E": "Microsoft Mobile Oy",
    "ACB313": "ARRIS Group, Inc.",
    "0CF893": "ARRIS Group, Inc.",
    "3CDFA9": "ARRIS Group, Inc.",
    "0015D1": "ARRIS Group, Inc.",
    "001DD0": "ARRIS Group, Inc.",
    "001DD3": "ARRIS Group, Inc.",
    "000740": "BUFFALO.INC",
    "0024A5": "BUFFALO.INC",
    "CCE1D5": "BUFFALO.INC",
    "E4F89C": "Intel Corporate",
    "2C4138": "Hewlett Packard",
    "441EA1": "Hewlett Packard",
    "78E7D1": "Hewlett Packard",
    "001CC4": "Hewlett Packard",
    "001E0B": "Hewlett Packard",
    "6CA100": "Intel Corporate",
    "A402B9": "Intel Corporate",
    "DC5360": "Intel Corporate",
    "000CF1": "Intel Corporation",
    "0012F0": "Intel Corporate",
    "5CB901": "Hewlett Packard",
    "DC4A3E": "Hewlett Packard",
    "9CB654": "Hewlett Packard",
    "6C3BE5": "Hewlett Packard",
    "A0B3CC": "Hewlett Packard",
    "ECB1D7": "Hewlett Packard",
    "784859": "Hewlett Packard",
    "3464A9": "Hewlett Packard",
    "3863BB": "Hewlett Packard",
    "002264": "Hewlett Packard",
    "0025B3": "Hewlett Packard",
    "643150": "Hewlett Packard",
    "58DC6D": "Exceptional Innovation, Inc.",
    "902155": "HTC Corporation",
    "D8B377": "HTC Corporation",
    "B0F1A3": "Fengfan (BeiJing) Technology Co., Ltd. ",
    "7CB15D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88CF98": "HUAWEI TECHNOLOGIES CO.,LTD",
    "140467": "SNK Technologies Co.,Ltd.",
    "8030DC": "Texas Instruments",
    "B05ADA": "Hewlett Packard",
    "001083": "Hewlett Packard",
    "A4D18C": "Apple, Inc.",
    "241EEB": "Apple, Inc.",
    "CC25EF": "Apple, Inc.",
    "942CB3": "HUMAX Co., Ltd.",
    "002719": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "40169F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F4EC38": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A4D578": "Texas Instruments",
    "246081": "razberi technologies",
    "00265E": "Hon Hai Precision Ind. Co.,Ltd.",
    "00242C": "Hon Hai Precision Ind. Co.,Ltd.",
    "808917": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A09347": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "E8BBA8": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "B0AA36": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "784B87": "Murata Manufacturing Co., Ltd.",
    "E4CE02": "WyreStorm Technologies Ltd",
    "9CD24B": "zte corporation",
    "C87B5B": "zte corporation",
    "0019C6": "zte corporation",
    "001DD9": "Hon Hai Precision Ind. Co.,Ltd.",
    "00197D": "Hon Hai Precision Ind. Co.,Ltd.",
    "90F652": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "14CF92": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "20DCE6": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "14CC20": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C03E0F": "SKY UK LIMITED",
    "7C4CA5": "SKY UK LIMITED",
    "CC4E24": "Brocade Communications Systems LLC",
    "00E052": "Brocade Communications Systems LLC",
    "0014A4": "Hon Hai Precision Ind. Co.,Ltd.",
    "78DD08": "Hon Hai Precision Ind. Co.,Ltd.",
    "9CD21E": "Hon Hai Precision Ind. Co.,Ltd.",
    "F80D43": "Hon Hai Precision Ind. Co.,Ltd.",
    "00010F": "Brocade Communications Systems LLC",
    "080088": "Brocade Communications Systems LLC",
    "84742A": "zte corporation",
    "38F889": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D07AB5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "904E2B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2008ED": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40F308": "Murata Manufacturing Co., Ltd.",
    "0007D8": "Hitron Technologies. Inc",
    "B43052": "HUAWEI TECHNOLOGIES CO.,LTD",
    "80D09B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C8E5C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "002568": "HUAWEI TECHNOLOGIES CO.,LTD",
    "781DBA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00259E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3475C7": "Avaya Inc",
    "B0ADAA": "Avaya Inc",
    "FC48EF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "707BE8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C1FCC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4B110": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E468A3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8853D4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04C06F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "202BC1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54A51B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "006B8E": "Shanghai Feixun Communication Co.,Ltd.",
    "D46AA8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8BC1B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "582AF7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0034FE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C85195": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40CBA8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D46E5C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C8BEF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B4475E": "Avaya Inc",
    "BCADAB": "Avaya Inc",
    "FCA841": "Avaya Inc",
    "24D921": "Avaya Inc",
    "848371": "Avaya Inc",
    "001B4F": "Avaya Inc",
    "3400A3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00E08F": "Cisco Systems, Inc",
    "001868": "Cisco SPVTG",
    "887556": "Cisco Systems, Inc",
    "FC9947": "Cisco Systems, Inc",
    "6C2056": "Cisco Systems, Inc",
    "ACF2C5": "Cisco Systems, Inc",
    "0003DD": "Comark Interactive Solutions",
    "005053": "Cisco Systems, Inc",
    "005050": "Cisco Systems, Inc",
    "00906D": "Cisco Systems, Inc",
    "0090AB": "Cisco Systems, Inc",
    "005054": "Cisco Systems, Inc",
    "00500B": "Cisco Systems, Inc",
    "2401C7": "Cisco Systems, Inc",
    "6886A7": "Cisco Systems, Inc",
    "08CC68": "Cisco Systems, Inc",
    "247189": "Texas Instruments",
    "987BF3": "Texas Instruments",
    "A0F6FD": "Texas Instruments",
    "D0B5C2": "Texas Instruments",
    "78A504": "Texas Instruments",
    "6CECEB": "Texas Instruments",
    "5006AB": "Cisco Systems, Inc",
    "00E0A3": "Cisco Systems, Inc",
    "00902B": "Cisco Systems, Inc",
    "3C0E23": "Cisco Systems, Inc",
    "00E018": "ASUSTek COMPUTER INC.",
    "000C6E": "ASUSTek COMPUTER INC.",
    "000EA6": "ASUSTek COMPUTER INC.",
    "001D60": "ASUSTek COMPUTER INC.",
    "0015F2": "ASUSTek COMPUTER INC.",
    "90E6BA": "ASUSTek COMPUTER INC.",
    "0C2724": "Cisco Systems, Inc",
    "6C416A": "Cisco Systems, Inc",
    "ECE1A9": "Cisco Systems, Inc",
    "1CE85D": "Cisco Systems, Inc",
    "A89D21": "Cisco Systems, Inc",
    "689CE2": "Cisco Systems, Inc",
    "C067AF": "Cisco Systems, Inc",
    "C0255C": "Cisco Systems, Inc",
    "F44E05": "Cisco Systems, Inc",
    "881DFC": "Cisco Systems, Inc",
    "A0F849": "Cisco Systems, Inc",
    "6CFA89": "Cisco Systems, Inc",
    "F46D04": "ASUSTek COMPUTER INC.",
    "745E1C": "PIONEER CORPORATION",
    "001011": "Cisco Systems, Inc",
    "00000C": "Cisco Systems, Inc",
    "046273": "Cisco Systems, Inc",
    "28CFE9": "Apple, Inc.",
    "002608": "Apple, Inc.",
    "0026B0": "Apple, Inc.",
    "0026BB": "Apple, Inc.",
    "D49A20": "Apple, Inc.",
    "F81EDF": "Apple, Inc.",
    "D8B190": "Cisco Systems, Inc",
    "80E86F": "Cisco Systems, Inc",
    "AC7E8A": "Cisco Systems, Inc",
    "185933": "Cisco SPVTG",
    "445829": "Cisco SPVTG",
    "602AD0": "Cisco SPVTG",
    "001451": "Apple, Inc.",
    "001E52": "Apple, Inc.",
    "0021E9": "Apple, Inc.",
    "CC08E0": "Apple, Inc.",
    "F0B479": "Apple, Inc.",
    "00A040": "Apple, Inc.",
    "003065": "Apple, Inc.",
    "0023BE": "Cisco SPVTG",
    "3CD0F8": "Apple, Inc.",
    "680927": "Apple, Inc.",
    "6CC26B": "Apple, Inc.",
    "44D884": "Apple, Inc.",
    "64200C": "Apple, Inc.",
    "40B395": "Apple, Inc.",
    "44FB42": "Apple, Inc.",
    "E88D28": "Apple, Inc.",
    "949426": "Apple, Inc.",
    "207D74": "Apple, Inc.",
    "F4F15A": "Apple, Inc.",
    "C86F1D": "Apple, Inc.",
    "B065BD": "Apple, Inc.",
    "8C2DAA": "Apple, Inc.",
    "848506": "Apple, Inc.",
    "98FE94": "Apple, Inc.",
    "D8004D": "Apple, Inc.",
    "1093E9": "Apple, Inc.",
    "442A60": "Apple, Inc.",
    "C82A14": "Apple, Inc.",
    "3C0754": "Apple, Inc.",
    "C8334B": "Apple, Inc.",
    "64E682": "Apple, Inc.",
    "9C207B": "Apple, Inc.",
    "A4B197": "Apple, Inc.",
    "A4D1D2": "Apple, Inc.",
    "28CFDA": "Apple, Inc.",
    "542696": "Apple, Inc.",
    "64A3CB": "Apple, Inc.",
    "30F7C5": "Apple, Inc.",
    "3090AB": "Apple, Inc.",
    "045453": "Apple, Inc.",
    "F4F951": "Apple, Inc.",
    "C06394": "Apple, Inc.",
    "18AF8F": "Apple, Inc.",
    "C8B5B7": "Apple, Inc.",
    "90B21F": "Apple, Inc.",
    "B8E856": "Apple, Inc.",
    "2078F0": "Apple, Inc.",
    "E0ACCB": "Apple, Inc.",
    "A0999B": "Apple, Inc.",
    "24240E": "Apple, Inc.",
    "903C92": "Apple, Inc.",
    "AC87A3": "Apple, Inc.",
    "D8BB2C": "Apple, Inc.",
    "D04F7E": "Apple, Inc.",
    "9CF387": "Apple, Inc.",
    "B418D1": "Apple, Inc.",
    "F0DBF8": "Apple, Inc.",
    "48746E": "Apple, Inc.",
    "34363B": "Apple, Inc.",
    "D0A637": "Apple, Inc.",
    "789F70": "Apple, Inc.",
    "A85B78": "Apple, Inc.",
    "FCE998": "Apple, Inc.",
    "0CBC9F": "Apple, Inc.",
    "D89695": "Apple, Inc.",
    "1499E2": "Apple, Inc.",
    "A88E24": "Apple, Inc.",
    "68AE20": "Apple, Inc.",
    "54AE27": "Apple, Inc.",
    "C8F650": "Apple, Inc.",
    "50D59C": "Thai Habel Industrial Co., Ltd.",
    "6C4A39": "BITA",
    "847D50": "Holley Metering Limited",
    "D81D72": "Apple, Inc.",
    "341298": "Apple, Inc.",
    "70E72C": "Apple, Inc.",
    "70ECE4": "Apple, Inc.",
    "E034E4": "Feit Electric Company, Inc.",
    "34C9F0": "LM Technologies Ltd",
    "681401": "Hon Hai Precision Ind. Co.,Ltd.",
    "50A9DE": "Smartcom - Bulgaria AD",
    "3052CB": "Liteon Technology Corporation",
    "049645": "WUXI SKY CHIP INTERCONNECTION TECHNOLOGY CO.,LTD.",
    "1CCDE5": "Shanghai Wind Technologies Co.,Ltd",
    "AC1FD7": "Real Vision Technology Co.,Ltd.",
    "88C242": "Poynt Co.",
    "DC82F6": "iPort",
    "68A828": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC20E8": "Apple, Inc.",
    "6C9354": "Yaojin Technology (Shenzhen) Co., LTD.",
    "DCDC07": "TRP Systems BV",
    "3891D5": "Hangzhou H3C Technologies Co., Limited",
    "BC9C31": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70480F": "Apple, Inc.",
    "3CB72B": "PLUMgrid Inc",
    "382187": "Midea Group Co., Ltd.",
    "78D6B2": "Toshiba",
    "8CE2DA": "Circle Media Inc",
    "489A42": "Technomate Ltd",
    "20D160": "Private",
    "E8DED6": "Intrising Networks, Inc.",
    "381C23": "Hilan Technology CO.,LTD",
    "D88B4C": "KingTing Tech.",
    "48BF74": "Baicells Technologies Co.,LTD",
    "384C90": "ARRIS Group, Inc.",
    "F0B0E7": "Apple, Inc.",
    "649A12": "P2 Mobile Technologies Limited",
    "E4C2D1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0469F8": "Apple, Inc.",
    "AC8995": "AzureWave Technology Inc.",
    "3029BE": "Shanghai MRDcom Co.,Ltd",
    "48B620": "ROLI Ltd.",
    "20C3A4": "RetailNext",
    "D4F4BE": "Palo Alto Networks",
    "D888CE": "RF Technology Pty Ltd",
    "845B12": "HUAWEI TECHNOLOGIES CO.,LTD",
    "205531": "Samsung Electronics Co.,Ltd",
    "307CB2": "ANOV FRANCE",
    "847973": "Shanghai Baud Data Communication Co.,Ltd.",
    "E8B4C8": "Samsung Electronics Co.,Ltd",
    "D087E2": "Samsung Electronics Co.,Ltd",
    "F05B7B": "Samsung Electronics Co.,Ltd",
    "B047BF": "Samsung Electronics Co.,Ltd",
    "7C0BC6": "Samsung Electronics Co.,Ltd",
    "906FA9": "NANJING PUTIAN TELECOMMUNICATIONS TECHNOLOGY CO.,LTD.",
    "60FD56": "WOORISYSTEMS CO., Ltd",
    "7CFE90": "Mellanox Technologies, Inc.",
    "0C756C": "Anaren Microwave, Inc.",
    "10868C": "ARRIS Group, Inc.",
    "44FDA3": "Everysight LTD.",
    "D47BB0": "ASKEY COMPUTER CORP",
    "F0224E": "Esan electronic co.",
    "C8F9C8": "NewSharp Technology(SuZhou)Co,Ltd",
    "A8A795": "Hon Hai Precision Ind. Co.,Ltd.",
    "9870E8": "INNATECH SDN BHD",
    "44656A": "Mega Video Electronic(HK) Industry Co., Ltd",
    "C412F5": "D-Link International",
    "68EDA4": "Shenzhen Seavo Technology Co.,Ltd",
    "2CC5D3": "Ruckus Wireless",
    "80C5E6": "Microsoft Corporation",
    "10DF8B": "Shenzhen CareDear Communication Technology Co.,Ltd",
    "606DC7": "Hon Hai Precision Ind. Co.,Ltd.",
    "CC5FBF": "Topwise 3G Communication Co., Ltd.",
    "E02CB2": "Lenovo Mobile Communication (Wuhan) Company Limited",
    "78EB39": "Instituto Nacional de Tecnolog\u00eda Industrial",
    "5440AD": "Samsung Electronics Co.,Ltd",
    "FCE1FB": "Array Networks",
    "E89120": "Motorola Mobility LLC, a Lenovo Company",
    "D4F9A1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "145A83": "Logi-D inc",
    "D0C0BF": "Actions Microelectronics Co., Ltd",
    "FC8F90": "Samsung Electronics Co.,Ltd",
    "54CD10": "Panasonic Mobile Communications Co.,Ltd.",
    "A424DD": "Cambrionix Ltd",
    "804E81": "Samsung Electronics Co.,Ltd",
    "48066A": "Tempered Networks, Inc.",
    "74042B": "Lenovo Mobile Communication (Wuhan) Company Limited",
    "D05C7A": "Sartura d.o.o.",
    "64167F": "Polycom",
    "C09A71": "XIAMEN MEITU MOBILE TECHNOLOGY CO.LTD",
    "340B40": "MIOS ELETTRONICA SRL",
    "144146": "Honeywell (China) Co., LTD",
    "BC5C4C": "ELECOM CO.,LTD.",
    "887033": "Hangzhou Silan Microelectronic Inc",
    "DC56E6": "Shenzhen Bococom Technology Co.,LTD",
    "900A39": "Wiio, Inc.",
    "BCF811": "Xiamen DNAKE Technology Co.,Ltd",
    "E861BE": "Melec Inc.",
    "5870C6": "Shanghai Xiaoyi Technology Co., Ltd.",
    "402814": "RFI Engineering",
    "F87AEF": "Rosonix Technology, Inc.",
    "2028BC": "Visionscape Co,. Ltd.",
    "40A5EF": "Shenzhen Four Seas Global Link Network Technology Co., Ltd.",
    "B856BD": "ITT LLC",
    "C4924C": "KEISOKUKI CENTER CO.,LTD.",
    "749CE3": "KodaCloud Canada, Inc",
    "CC9635": "LVS Co.,Ltd.",
    "241C04": "SHENZHEN JEHE TECHNOLOGY DEVELOPMENT CO., LTD.",
    "4480EB": "Motorola Mobility LLC, a Lenovo Company",
    "50502A": "Egardia",
    "94F19E": "HUIZHOU MAORONG INTELLIGENT TECHNOLOGY CO.,LTD",
    "D45556": "Fiber Mountain Inc.",
    "DC60A1": "Teledyne DALSA Professional Imaging",
    "4CA515": "Baikal Electronics JSC",
    "7C3CB6": "Shenzhen Homecare Technology Co.,Ltd.",
    "1C7D22": "FUJIFILM Business Innovation Corp.",
    "749637": "Todaair Electronic Co., Ltd",
    "88E603": "Avotek corporation",
    "84C3E8": "Vaillant GmbH",
    "2CAD13": "SHENZHEN ZHILU TECHNOLOGY CO.,LTD",
    "C48E8F": "Hon Hai Precision Ind. Co.,Ltd.",
    "54369B": "1Verge Internet Technology (Beijing) Co., Ltd.",
    "4062B6": "Tele system communication",
    "60128B": "CANON INC.",
    "78ACBF": "Igneous Systems",
    "40D28A": "Nintendo Co., Ltd.",
    "9CD35B": "Samsung Electronics Co.,Ltd",
    "A89FBA": "Samsung Electronics Co.,Ltd",
    "B88EC6": "Stateless Networks",
    "BCD165": "Cisco SPVTG",
    "28E476": "Pi-Coral",
    "74A34A": "ZIMI CORPORATION",
    "20C06D": "SHENZHEN SPACETEK TECHNOLOGY CO.,LTD",
    "08A5C8": "Sunnovo International Limited",
    "D05BA8": "zte corporation",
    "A86405": "nimbus 9, Inc",
    "E4BAD9": "360 Fly Inc.",
    "10C67E": "SHENZHEN JUCHIN TECHNOLOGY CO., LTD",
    "6828F6": "Vubiq Networks, Inc.",
    "10A659": "Mobile Create Co.,Ltd.",
    "58856E": "QSC AG",
    "4886E8": "Microsoft Corporation",
    "A0A3E2": "Actiontec Electronics, Inc",
    "74547D": "Cisco SPVTG",
    "8C9109": "Toyoshima Electric Technoeogy(Suzhou) Co.,Ltd.",
    "54098D": "deister electronic GmbH",
    "50F43C": "Leeo Inc",
    "E887A3": "Loxley Public Company Limited",
    "D8CB8A": "Micro-Star INTL CO., LTD.",
    "F88479": "Yaojin Technology(Shenzhen)Co.,Ltd",
    "2C2997": "Microsoft Corporation",
    "00E6E8": "Netzin Technology Corporation,.Ltd.",
    "70FC8C": "OneAccess SA",
    "2C3796": "CYBO CO.,LTD.",
    "587FB7": "SONAR INDUSTRIAL CO., LTD.",
    "EC59E7": "Microsoft Corporation",
    "C035C5": "Prosoft Systems LTD",
    "908C09": "Total Phase",
    "DC2F03": "Step forward Group Co., Ltd.",
    "380E7B": "V.P.S. Thai Co., Ltd",
    "08EB29": "Jiangsu Huitong Group Co.,Ltd.",
    "409B0D": "Shenzhen Yourf Kwan Industrial Co., Ltd",
    "4C2C83": "Zhejiang KaNong Network Technology Co.,Ltd.",
    "D46132": "Pro Concept Manufacturer Co.,Ltd.",
    "54A050": "ASUSTek COMPUTER INC.",
    "849681": "Cathay Communication Co.,Ltd",
    "2CA30E": "POWER DRAGON DEVELOPMENT LIMITED",
    "8486F3": "Greenvity Communications",
    "6050C1": "Kinetek Sports",
    "2C1A31": "Electronics Company Limited",
    "2012D5": "Scientech Materials Corporation",
    "8C5D60": "UCI Corporation Co.,Ltd.",
    "6099D1": "Vuzix / Lenovo",
    "B04515": "mira fitness,LLC.",
    "BC6B4D": "Nokia",
    "C8D019": "Shanghai Tigercel Communication Technology Co.,Ltd",
    "902181": "Shanghai Huaqin Telecom Technology Co.,Ltd",
    "58108C": "Intelbras",
    "4C7403": "BQ",
    "1C5216": "DONGGUAN HELE ELECTRONICS CO., LTD",
    "14488B": "Shenzhen Doov Technology Co.,Ltd",
    "80F8EB": "RayTight",
    "D437D7": "zte corporation",
    "2C010B": "NASCENT Technology, LLC - RemKon",
    "784561": "CyberTAN Technology Inc.",
    "94CE31": "CTS Limited",
    "6C6EFE": "Core Logic Inc.",
    "3808FD": "Silca Spa",
    "20ED74": "Ability enterprise co.,Ltd.",
    "6C0273": "Shenzhen Jin Yun Video Equipment Co., Ltd.",
    "A056B2": "Harman/Becker Automotive Systems GmbH",
    "44666E": "IP-LINE",
    "5CB6CC": "NovaComm Technologies Inc.",
    "5C1515": "ADVAN",
    "244F1D": "iRule LLC",
    "94AEE3": "Belden Hirschmann Industries (Suzhou) Ltd.",
    "04DEDB": "Rockport Networks Inc",
    "1C4840": "IMS Messsysteme GmbH",
    "54DF00": "Ulterius Technologies, LLC",
    "34C5D0": "Hagleitner Hygiene International GmbH",
    "10C37B": "ASUSTek COMPUTER INC.",
    "B40B44": "Smartisan Technology Co., Ltd.",
    "90B686": "Murata Manufacturing Co., Ltd.",
    "ECD9D1": "Shenzhen TG-NET Botone Technology Co.,Ltd.",
    "2829CC": "Corsa Technology Incorporated",
    "3431C4": "AVM GmbH",
    "184A6F": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "E85D6B": "Luminate Wireless",
    "50FEF2": "Sify Technologies Ltd",
    "705B2E": "M2Communication Inc.",
    "B40AC6": "DEXON Systems Ltd.",
    "D059E4": "Samsung Electronics Co.,Ltd",
    "14A364": "Samsung Electronics Co.,Ltd",
    "481A84": "Pointer Telocation Ltd",
    "DC663A": "Apacer Technology Inc.",
    "08DF1F": "Bose Corporation",
    "4826E8": "Tek-Air Systems, Inc.",
    "581F67": "Open-m technology limited",
    "A012DB": "TABUCHI ELECTRIC CO.,LTD",
    "488244": "Life Fitness / Div. of Brunswick",
    "C83168": "eZEX corporation",
    "D05AF1": "Shenzhen Pulier Tech CO.,Ltd",
    "40B3CD": "Chiyoda Electronics Co.,Ltd.",
    "30918F": "Technicolor Delivery Technologies Belgium NV",
    "5CF4AB": "Zyxel Communications Corporation",
    "08B2A3": "Cynny Italia S.r.L.",
    "FCE186": "A3M Co., LTD",
    "54EF92": "Shenzhen Elink Technology Co., LTD",
    "0C2026": "noax Technologies AG",
    "202564": "PEGATRON CORPORATION",
    "C8D590": "FLIGHT DATA SYSTEMS",
    "A0DA92": "Nanjing Glarun Atten Technology Co. Ltd.",
    "54C80F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "E4D332": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "70305D": "Ubiquoss Inc",
    "D4CFF9": "Shenzhen SEI Robotics Co.,Ltd",
    "24336C": "Private",
    "ECE512": "tado GmbH",
    "7CFF62": "Huizhou Super Electron Technology Co.,Ltd.",
    "283B96": "Cool Control LTD",
    "648D9E": "IVT Electronic Co.,Ltd",
    "949F3F": "Optek Digital Technology company limited",
    "78FEE2": "Shanghai Diveo Technology Co., Ltd",
    "CC95D7": "Vizio, Inc",
    "FC09F6": "GUANGDONG TONZE ELECTRIC CO.,LTD",
    "BCF61C": "Geomodeling Wuxi Technology Co. Ltd.",
    "086DF2": "Shenzhen MIMOWAVE Technology Co.,Ltd",
    "687848": "WESTUNITIS CO., LTD.",
    "48D0CF": "Universal Electronics, Inc.",
    "DCC793": "Nokia Corporation",
    "444891": "HDMI Licensing, LLC",
    "FC923B": "Nokia Corporation",
    "E03F49": "ASUSTek COMPUTER INC.",
    "28656B": "Keystone Microtech Corporation",
    "CC9F35": "Transbit Sp. z o.o.",
    "F03FF8": "R L Drake",
    "5C254C": "Avire Global Pte Ltd",
    "D42F23": "Akenori PTE Ltd",
    "98C0EB": "Global Regency Ltd",
    "D09C30": "Foster Electric Company, Limited",
    "ECF72B": "HD DIGITAL TECH CO., LTD.",
    "4CF45B": "Blue Clover Devices",
    "B06971": "DEI Sales, Inc.",
    "C09D26": "Topicon HK Lmd.",
    "442938": "NietZsche enterprise Co.Ltd.",
    "9C3EAA": "EnvyLogic Co.,Ltd.",
    "F8F005": "Newport Media Inc.",
    "1CFCBB": "Realfiction ApS",
    "B0C554": "D-Link International",
    "C4E984": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "48EE86": "UTStarcom (China) Co.,Ltd",
    "888914": "All Components Incorporated",
    "F0321A": "Mita-Teknik A/S",
    "24050F": "MTN Electronic Co. Ltd",
    "AC2DA3": "TXTR GmbH",
    "14C089": "DUNE HD LTD",
    "F4B6E5": "TerraSem Co.,Ltd",
    "A881F1": "BMEYE B.V.",
    "D069D0": "Verto Medical Solutions, LLC",
    "889CA6": "BTB Korea INC",
    "90837A": "General Electric Water & Process Technologies",
    "A43A69": "Vers Inc",
    "B0D7C5": "Logipix Ltd",
    "9C44A6": "SwiftTest, Inc.",
    "FC07A0": "LRE Medical GmbH",
    "1CEEE8": "Ilshin Elecom",
    "FC1349": "Global Apps Corp.",
    "848433": "Paradox Engineering SA",
    "44C306": "SIFROM Inc.",
    "602103": "I4VINE, INC",
    "60FFDD": "C.E. ELECTRONICS, INC",
    "88A73C": "Ragentek Technology Group",
    "88D962": "Canopus Systems US LLC",
    "9031CD": "Onyx Healthcare Inc.",
    "E48184": "Nokia",
    "E056F4": "AxesNetwork Solutions inc.",
    "84569C": "Coho Data, Inc.,",
    "78AE0C": "Far South Networks",
    "84A783": "Alcatel Lucent",
    "80BAE6": "Neets",
    "F8A2B4": "RHEWA-WAAGENFABRIK August Freudewald GmbH &amp;Co. KG",
    "B024F3": "Progeny Systems",
    "0C54A5": "PEGATRON CORPORATION",
    "8C4DB9": "Unmonday Ltd",
    "4CD9C4": "Magneti Marelli Automotive Electronics (Guangzhou) Co. Ltd",
    "2C5A05": "Nokia Corporation",
    "D481CA": "iDevices, LLC",
    "D858D7": "CZ.NIC, z.s.p.o.",
    "7C6AB3": "IBC TECHNOLOGIES INC.",
    "181BEB": "Actiontec Electronics, Inc",
    "78CA5E": "ELNO",
    "B4827B": "AKG Acoustics GmbH",
    "DC5E36": "Paterson Technology",
    "50E0C7": "TurControlSystme AG",
    "DCAD9E": "GreenPriz",
    "B8DF6B": "SpotCam Co., Ltd.",
    "9C8888": "Simac Techniek NV",
    "041A04": "WaveIP",
    "18C8E7": "Shenzhen Hualistone Technology Co.,Ltd",
    "A03B1B": "Inspire Tech",
    "7CCD3C": "Guangzhou Juzing Technology Co., Ltd",
    "448A5B": "Micro-Star INT'L CO., LTD.",
    "0CCB8D": "ASCO Numatics GmbH",
    "6C4B7F": "Vossloh-Schwabe Deutschland GmbH",
    "688AB5": "EDP Servicos",
    "2493CA": "Voxtronic Austria",
    "10B26B": "base Co.,Ltd.",
    "4CF02E": "Vifa Denmark A/S",
    "DCCEBC": "Shenzhen JSR Technology Co.,Ltd.",
    "68764F": "Sony Corporation",
    "38DBBB": "Sunbow Telecom Co., Ltd.",
    "9843DA": "INTERTECH",
    "08CA45": "Toyou Feiji Electronics Co., Ltd.",
    "3CCA87": "Iders Incorporated",
    "908C44": "H.K ZONGMU TECHNOLOGY CO., LTD.",
    "34A5E1": "Sensorist ApS",
    "6C8366": "Nanjing SAC Power Grid Automation Co., Ltd.",
    "78D99F": "NuCom HK Ltd.",
    "142BD2": "Armtel Ltd.",
    "9CA10A": "SCLE SFE",
    "88789C": "Game Technologies SA",
    "8079AE": "ShanDong Tecsunrise  Co.,Ltd",
    "7CBD06": "AE REFUsol",
    "94BA56": "Shenzhen Coship Electronics Co., Ltd.",
    "740EDB": "Optowiz Co., Ltd",
    "A4895B": "ARK INFOSOLUTIONS PVT LTD",
    "D09D0A": "LINKCOM",
    "EC219F": "VidaBox LLC",
    "A8CCC5": "Saab AB (publ)",
    "988E4A": "NOXUS(BEIJING) TECHNOLOGY CO.,LTD",
    "50B888": "wi2be Tecnologia S/A",
    "54BEF7": "PEGATRON CORPORATION",
    "907990": "Benchmark Electronics Romania SRL",
    "EC2AF0": "Ypsomed AG",
    "F854AF": "ECI Telecom Ltd.",
    "F0F5AE": "Adaptrum Inc.",
    "F83D4E": "Softlink Automation System Co., Ltd",
    "7CE56B": "ESEN Optoelectronics Technology Co.,Ltd.",
    "D44C9C": "Shenzhen YOOBAO Technology Co.Ltd",
    "20CEC4": "Peraso Technologies",
    "CC4703": "Intercon Systems Co., Ltd.",
    "DC1792": "Captivate Network",
    "442AFF": "E3 Technology, Inc.",
    "7C8306": "Glen Dimplex Nordic as",
    "140D4F": "Flextronics International",
    "FC3FAB": "Henan Lanxin Technology Co., Ltd",
    "58468F": "Koncar Electronics and Informatics",
    "985D46": "PeopleNet Communication",
    "F89FB8": "YAZAKI Energy System Corporation",
    "540536": "Vivago Oy",
    "6C90B1": "SanLogic Inc",
    "E0FAEC": "Platan sp. z o.o. sp. k.",
    "446755": "Orbit Irrigation",
    "5422F8": "zte corporation",
    "486E73": "Pica8, Inc.",
    "CC7B35": "zte corporation",
    "04D437": "ZNV",
    "CCF407": "EUKREA ELECTROMATIQUE SARL",
    "088E4F": "SF Software Solutions",
    "ACCA8E": "ODA Technologies",
    "6405BE": "NEW LIGHT LED",
    "E8BB3D": "Sino Prime-Tech Limited",
    "28A241": "exlar corp",
    "30786B": "TIANJIN Golden Pentagon Electronics Co., Ltd.",
    "20DF3F": "Nanjing SAC Power Grid Automation Co., Ltd.",
    "4CD637": "Qsono Electronics Co., Ltd",
    "9436E0": "Sichuan Bihong Broadcast &amp; Television New Technologies Co.,Ltd",
    "34A68C": "Shine Profit Development Limited",
    "949FB4": "ChengDu JiaFaAnTai Technology Co.,Ltd",
    "509871": "Inventum Technologies Private Limited",
    "048C03": "ThinPAD Technology (Shenzhen)CO.,LTD",
    "88462A": "Telechips Inc.",
    "C80258": "ITW GSE ApS",
    "F42012": "Cuciniale GmbH",
    "98B039": "Nokia",
    "B830A8": "Road-Track Telematics Development",
    "406826": "Thales UK Limited",
    "F82BC8": "Jiangsu Switter Co., Ltd",
    "60C397": "2Wire Inc",
    "FC6018": "Zhejiang Kangtai Electric Co., Ltd.",
    "0CF019": "Malgn Technology Co., Ltd.",
    "28285D": "Zyxel Communications Corporation",
    "646EEA": "Iskratel d.o.o.",
    "D0737F": "Mini-Circuits",
    "58E02C": "Micro Technic A/S",
    "78B3CE": "Elo touch solutions",
    "F8516D": "Denwa Technology Corp.",
    "444A65": "Silverflare Ltd.",
    "744BE9": "EXPLORER HYPERTECH CO.,LTD",
    "E8CE06": "SkyHawke Technologies, LLC.",
    "C8F386": "Shenzhen Xiaoniao Technology Co.,Ltd",
    "2C72C3": "Soundmatters",
    "2C69BA": "RF Controls, LLC",
    "D4BF7F": "UPVEL",
    "C8DDC9": "Lenovo Mobile Communication Technology Ltd.",
    "6C8686": "Technonia",
    "D4AC4E": "BODi rS, LLC",
    "A4FCCE": "Security Expert Ltd.",
    "E07F88": "EVIDENCE Network SIA",
    "1C7CC7": "Coriant GmbH",
    "341B22": "Grandbeing Technology Co., Ltd",
    "842F75": "Innokas Group",
    "CC3C3F": "SA.S.S. Datentechnik AG",
    "341A4C": "SHENZHEN WEIBU ELECTRONICS CO.,LTD.",
    "88142B": "Protonic Holland",
    "70C6AC": "Bosch Automotive Aftermarket",
    "0488E2": "Beats Electronics LLC",
    "40560C": "In Home Displays Ltd",
    "A0861D": "Chengdu Fuhuaxin Technology co.,Ltd",
    "508D6F": "CHAHOO Limited",
    "308999": "Guangdong East Power Co.,",
    "A47ACF": "VIBICOM COMMUNICATIONS INC.",
    "BC261D": "HONG KONG TECON TECHNOLOGY",
    "204C6D": "Hugo Brennenstuhl Gmbh & Co. KG.",
    "9C9726": "Technicolor Delivery Technologies Belgium NV",
    "7C0507": "PEGATRON CORPORATION",
    "880905": "MTMCommunications",
    "F4B381": "WindowMaster A/S",
    "74F102": "Beijing HCHCOM Technology Co., Ltd",
    "F0F260": "Mobitec AB",
    "282CB2": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "64E599": "EFM Networks",
    "40C4D6": "ChongQing Camyu Technology Development Co.,Ltd.",
    "A8294C": "Precision Optical Transceivers, Inc.",
    "30D46A": "Autosales Incorporated",
    "E8DE27": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C89346": "MXCHIP Company Limited",
    "94ACCA": "trivum technologies GmbH",
    "908260": "IEEE 1904.1 Working Group",
    "D4EE07": "HIWIFI Co., Ltd.",
    "FCAD0F": "QTS NETWORKS",
    "984C04": "Zhangzhou Keneng Electrical Equipment Co Ltd",
    "ACDBDA": "Shenzhen Geniatech Inc, Ltd",
    "A4E991": "SISTEMAS AUDIOVISUALES ITELSIS S.L.",
    "3C86A8": "Sangshin elecom .co,, LTD",
    "84F493": "OMS spol. s.r.o.",
    "BCD177": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "7CD762": "Freestyle Technology Pty Ltd",
    "901D27": "zte corporation",
    "F05DC8": "Duracell Powermat",
    "CC5D57": "Information  System Research Institute,Inc.",
    "081DFB": "Shanghai Mexon Communication Technology Co.,Ltd",
    "C44EAC": "Shenzhen Shiningworth Technology Co., Ltd.",
    "044CEF": "Fujian Sanao Technology Co.,Ltd",
    "4C804F": "Armstrong Monitoring Corp",
    "64C667": "Barnes&Noble",
    "103DEA": "HFC Technology (Beijing) Ltd. Co.",
    "D42751": "Infopia Co., Ltd",
    "68A40E": "BSH Hausger\u00e4te GmbH",
    "F4C6D7": "blackned GmbH",
    "4CCA53": "Skyera, Inc.",
    "90FF79": "Metro Ethernet Forum",
    "D4CA6E": "u-blox AG",
    "B01408": "LIGHTSPEED INTERNATIONAL CO.",
    "9C3178": "Foshan Huadian Intelligent Communications Teachnologies Co.,Ltd",
    "983F9F": "China SSJ (Suzhou) Network Technology Inc.",
    "B838CA": "Kyokko Tsushin System CO.,LTD",
    "B0C95B": "Beijing Symtech CO.,LTD",
    "38B5BD": "E.G.O. Elektro-Ger",
    "5C43D2": "HAZEMEYER",
    "D809C3": "Cercacor Labs",
    "2CE871": "Alert Metalguard ApS",
    "F87B62": "FASTWEL INTERNATIONAL CO., LTD. Taiwan Branch",
    "40270B": "Mobileeco Co., Ltd",
    "E4EEFD": "MR&D Manufacturing",
    "105CBF": "DuroByte Inc",
    "20918A": "PROFALUX",
    "A80180": "IMAGO Technologies GmbH",
    "0C5521": "Axiros GmbH",
    "7898FD": "Q9 Networks Inc.",
    "D063B4": "SolidRun Ltd.",
    "2C441B": "Spectrum Medical Limited",
    "E4E409": "LEIFHEIT AG",
    "68B8D9": "Act KDE, Inc.",
    "90CC24": "Synaptics, Inc",
    "C88A83": "Dongguan HuaHong Electronics Co.,Ltd",
    "8CC5E1": "ShenZhen Konka Telecommunication Technology Co.,Ltd",
    "64A341": "Wonderlan (Beijing) Technology Co., Ltd.",
    "74FE48": "ADVANTECH CO., LTD.",
    "80B95C": "ELFTECH Co., Ltd.",
    "A01917": "Bertel S.p.a.",
    "24F2DD": "Radiant Zemax LLC",
    "A46E79": "DFT System Co.Ltd",
    "B877C3": "METER Group",
    "004D32": "Andon Health Co.,Ltd.",
    "FCA9B0": "MIARTECH (SHANGHAI),INC.",
    "9C8D1A": "INTEG process group inc",
    "480362": "DESAY ELECTRONICS(HUIZHOU)CO.,LTD",
    "18673F": "Hanover Displays Limited",
    "7C0A50": "J-MEX Inc.",
    "40F2E9": "IBM",
    "744D79": "Arrive Systems Inc.",
    "5011EB": "SilverNet Ltd",
    "54DF63": "Intrakey technologies GmbH",
    "D493A0": "Fidelix Oy",
    "08EBED": "World Elite Technology Co.,LTD",
    "EC4993": "Qihan Technology Co., Ltd ",
    "B0ACFA": "FUJITSU LIMITED",
    "8CE081": "zte corporation",
    "3C83B5": "Advance Vision Electronics Co. Ltd.",
    "28A192": "GERP Solution",
    "9C541C": "Shenzhen My-power Technology Co.,Ltd",
    "8C3330": "EmFirst Co., Ltd.",
    "087BAA": "SVYAZKOMPLEKTSERVICE, LLC",
    "C44567": "SAMBON PRECISON and ELECTRONICS",
    "E45614": "Suttle Apparatus",
    "A44E2D": "Adaptive Wireless Solutions, LLC",
    "0CCDFB": "EDIC Systems Inc.",
    "9C0473": "Tecmobile (International) Ltd. ",
    "B01203": "Dynamics Hong Kong Limited",
    "9886B1": "Flyaudio corporation (China)",
    "7093F8": "Space Monkey, Inc.",
    "28B3AB": "Genmark Automation",
    "C4E7BE": "SCSpro Co.,Ltd",
    "58874C": "LITE-ON CLEAN ENERGY TECHNOLOGY CORP.",
    "2891D0": "Stage Tec Entwicklungsgesellschaft f\u00fcr professionelle Audiotechnik mbH",
    "C0BD42": "ZPA Smart Energy a.s.",
    "2074CF": "Shenzhen Voxtech Co.,Ltd",
    "FC52CE": "Control iD",
    "5C4A26": "Enguity Technology Corp",
    "D8D27C": "JEMA ENERGY, SA",
    "E0F5CA": "CHENG UEI PRECISION INDUSTRY CO.,LTD.",
    "A8EF26": "Tritonwave",
    "60F2EF": "VisionVera International Co., Ltd.",
    "C03F2A": "Biscotti, Inc.",
    "381C4A": "SIMCom Wireless Solutions Co.,Ltd.",
    "48F8B3": "Cisco-Linksys, LLC",
    "B4DFFA": "Litemax Electronics Inc.",
    "681CA2": "Rosewill Inc.",
    "604616": "XIAMEN VANN INTELLIGENT CO., LTD",
    "749975": "IBM Corporation",
    "D8C691": "Hichan Technology Corp.",
    "E43FA2": "Wuxi DSP Technologies Inc.",
    "F4B72A": "TIME INTERCONNECT LTD",
    "DC9FA4": "Nokia Corporation",
    "44C39B": "OOO RUBEZH NPO",
    "6C40C6": "Nimbus Data, Inc.",
    "1048B1": "Beijing Duokan Technology Limited",
    "106FEF": "Ad-Sol Nissin Corp",
    "2C625A": "Finest Security Systems Co., Ltd",
    "20DC93": "Cheetah Hi-Tech, Inc.",
    "4423AA": "Farmage Co., Ltd.",
    "7CFE28": "Salutron Inc.",
    "E8102E": "Really Simple Software, Inc",
    "0C565C": "HyBroad Vision (Hong Kong) Technology Co Ltd",
    "8C6AE4": "Viogem Limited",
    "64517E": "LONG BEN (DONGGUAN) ELECTRONIC TECHNOLOGY CO.,LTD.",
    "D43D7E": "Micro-Star Int'l Co, Ltd",
    "ACD9D6": "tci GmbH",
    "48282F": "zte corporation",
    "88DC96": "EnGenius Technologies, Inc.",
    "20443A": "Schneider Electric Asia Pacific Ltd",
    "ECA29B": "Kemppi Oy",
    "C4BA99": "I+ME Actia Informatik und Mikro-Elektronik GmbH",
    "04CE14": "Wilocity LTD.",
    "543968": "Edgewater Networks Inc",
    "440CFD": "NetMan Co., Ltd.",
    "8CD3A2": "VisSim AS",
    "14358B": "Mediabridge Products, LLC.",
    "00F403": "Orbis Systems Oy",
    "547398": "Toyo Electronics Corporation",
    "C4393A": "SMC Networks Inc",
    "E8D483": "ULTIMATE Europe Transportation Equipment GmbH",
    "1C8464": "FORMOSA WIRELESS COMMUNICATION CORP.",
    "EC9327": "MEMMERT GmbH + Co. KG",
    "A0EF84": "Seine Image Int'l Co., Ltd",
    "E0A30F": "Pevco",
    "5C2479": "Baltech AG",
    "D82DE1": "Tricascade Inc.",
    "D0D212": "K2NET Co.,Ltd.",
    "80D18B": "Hangzhou I'converge Technology Co.,Ltd",
    "4088E0": "Beijing Ereneben Information Technology Limited Shenzhen Branch",
    "E85484": "NEO Information Systems Co., Ltd.",
    "74AE76": "iNovo Broadband, Inc.",
    "709A0B": "Italian Institute of Technology",
    "346E8A": "Ecosense",
    "64F242": "Gerdes Aktiengesellschaft",
    "60F281": "TRANWO TECHNOLOGY CO., LTD.",
    "0CC0C0": "MAGNETI MARELLI SISTEMAS ELECTRONICOS MEXICO",
    "08379C": "Topaz Co. LTD.",
    "D80DE3": "FXI TECHNOLOGIES AS",
    "60CBFB": "AirScape Inc.",
    "7C160D": "Saia-Burgess Controls AG",
    "A497BB": "Hitachi Industrial Equipment Systems Co.,Ltd",
    "4C5427": "Linepro Sp. z o.o.",
    "EC1A59": "Belkin International Inc.",
    "881036": "Panodic(ShenZhen) Electronics Limted",
    "68B6FC": "Hitron Technologies. Inc",
    "942197": "Stalmart Technology Limited",
    "A0C3DE": "Triton Electronic Systems Ltd.",
    "D0699E": "LUMINEX Lighting Control Equipment",
    "B0D2F5": "Vello Systems, Inc.",
    "B0435D": "NuLEDs, Inc.",
    "0808EA": "AMSC",
    "A4934C": "Cisco Systems, Inc",
    "F0FDA0": "Acurix Networks Pty Ltd",
    "B45570": "Borea",
    "5057A8": "Cisco Systems, Inc",
    "00DEFB": "Cisco Systems, Inc",
    "3CA315": "Bless Information & Communications Co., Ltd",
    "100D2F": "Online Security Pty. Ltd.",
    "3C98BF": "Quest Controls, Inc.",
    "D0AEEC": "Alpha Networks Inc.",
    "E477D4": "Minrray Industry Co.,Ltd ",
    "38E08E": "Mitsubishi Electric Corporation",
    "10A932": "Beijing Cyber Cloud Technology Co. ,Ltd.",
    "34FC6F": "ALCEA",
    "C0B357": "Yoshiki Electronics Industry Ltd.",
    "A81758": "Elektronik System i Ume\u00e5 AB",
    "E81324": "GuangZhou Bonsoninfo System CO.,LTD",
    "142DF5": "Amphitech",
    "F83094": "Alcatel-Lucent Telecom Limited",
    "C495A2": "SHENZHEN WEIJIU INDUSTRY AND TRADE DEVELOPMENT CO., LTD",
    "8C6878": "Nortek-AS",
    "202598": "Teleview",
    "38F8B7": "V2COM PARTICIPACOES S.A.",
    "A04CC1": "Helixtech Corp.",
    "34A7BA": "Fischer International Systems Corporation",
    "F8D462": "Pumatronix Equipamentos Eletronicos Ltda.",
    "A0DC04": "Becker-Antriebe GmbH",
    "40605A": "Hawkeye Tech Co. Ltd",
    "5453ED": "Sony Corporation",
    "F473CA": "Conversion Sound Inc.",
    "F8F7FF": "SYN-TECH SYSTEMS INC",
    "E4C806": "Ceiec Electric Technology Inc.",
    "E0F9BE": "Cloudena Corp.",
    "08B4CF": "Abicom International",
    "7C94B2": "Philips Healthcare PCCI",
    "442B03": "Cisco Systems, Inc",
    "B88F14": "Analytica GmbH",
    "60E956": "Ayla Networks, Inc",
    "EC1120": "FloDesign Wind Turbine Corporation",
    "E86D6E": "voestalpine Signaling UK Ltd.",
    "681605": "Systems And Electronic Development FZCO",
    "882012": "LMI Technologies",
    "3C4E47": "Etronic A/S",
    "F48771": "Infoblox",
    "F897CF": "DAESHIN-INFORMATION TECHNOLOGY CO., LTD.",
    "B49EE6": "SHENZHEN TECHNOLOGY CO LTD",
    "BC4B79": "SensingTek",
    "A49005": "CHINA GREATWALL COMPUTER SHENZHEN CO.,LTD",
    "C40ACB": "Cisco Systems, Inc",
    "D4A02A": "Cisco Systems, Inc",
    "88C36E": "Beijing Ereneben lnformation Technology Limited",
    "645422": "Equinox Payments",
    "00376D": "Murata Manufacturing Co., Ltd.",
    "50008C": "Hong Kong Telecommunications (HKT) Limited",
    "34AA99": "Nokia",
    "24B88C": "Crenus Co.,Ltd.",
    "4C9E80": "KYOKKO ELECTRIC Co., Ltd.",
    "5CEB4E": "R. STAHL HMI Systems GmbH",
    "943AF0": "Nokia Corporation",
    "0463E0": "Nome Oy",
    "940B2D": "NetView Technologies(Shenzhen) Co., Ltd",
    "306E5C": "Validus Technologies",
    "C894D2": "Jiangsu Datang  Electronic Products Co., Ltd",
    "BCFE8C": "Altronic, LLC",
    "649EF3": "Cisco Systems, Inc",
    "24BBC1": "Absolute Analysis",
    "9CCAD9": "Nokia Corporation",
    "046D42": "Bryston Ltd.",
    "D8E743": "Wush, Inc",
    "80946C": "TOKYO RADAR CORPORATION",
    "3CE624": "LG Display ",
    "D8F0F2": "Zeebo Inc",
    "644D70": "dSPACE GmbH",
    "D0CF5E": "Energy Micro AS",
    "DCC101": "SOLiD Technologies, Inc.",
    "080D84": "GECO, Inc.",
    "88E712": "Whirlpool Corporation",
    "D412BB": "Quadrant Components Inc. Ltd",
    "806007": "RIM",
    "94E0D0": "HealthStream Taiwan Inc.",
    "DCF858": "Lorent Networks, Inc.",
    "1803FA": "IBT Interfaces",
    "D8052E": "Skyviia Corporation",
    "183825": "Wuhan Lingjiu High-tech Co.,Ltd.",
    "7C4B78": "Red Sun Synthesis Pte Ltd",
    "64A0E7": "Cisco Systems, Inc",
    "EC6264": "Global411 Internet Services, LLC",
    "00F051": "KWB Gmbh",
    "CCEF48": "Cisco Systems, Inc",
    "2437EF": "EMC Electronic Media Communication SA",
    "28B0CC": "Xenya d.o.o.",
    "205B5E": "Shenzhen Wonhe Technology Co., Ltd",
    "B0E50E": "NRG SYSTEMS INC",
    "48C1AC": "PLANTRONICS, INC.",
    "98588A": "SYSGRATION Ltd.",
    "94319B": "Alphatronics BV",
    "E878A1": "BEOVIEW INTERCOM DOO",
    "803F5D": "Winstars Technology Ltd",
    "40BF17": "Digistar Telecom. SA",
    "780738": "Z.U.K. Elzab S.A.",
    "2037BC": "Kuipers Electronic Engineering BV",
    "C8A620": "Nebula, Inc",
    "FC946C": "UBIVELOX",
    "407B1B": "Mettle Networks Inc.",
    "489BE2": "SCI Innovations Ltd",
    "80FFA8": "UNIDIS",
    "40D559": "MICRO S.E.R.I.",
    "306CBE": "Skymotion Technology (HK) Limited",
    "886B76": "CHINA HOPEFUL GROUP HOPEFUL ELECTRIC CO.,LTD",
    "78F7D0": "Silverbrook Research",
    "207600": "Actiontec Electronics, Inc",
    "F04B6A": "Scientific Production Association Siberian Arsenal, Ltd.",
    "64AE0C": "Cisco Systems, Inc",
    "00E175": "AK-Systems Ltd",
    "CC501C": "KVH Industries, Inc.",
    "04D783": "Y&H E&C Co.,LTD.",
    "C83B45": "JRI",
    "C058A7": "Pico Systems Co., Ltd.",
    "EC3F05": "Institute 706, The Second Academy China Aerospace Science & Industry Corp",
    "E8DA96": "Zhuhai Tianrui Electrical Power Tech. Co., Ltd.",
    "F013C3": "SHENZHEN FENDA TECHNOLOGY CO., LTD",
    "04A82A": "Nokia Corporation",
    "E44E18": "Gardasoft VisionLimited",
    "148A70": "ADS GmbH",
    "B4D8DE": "iota Computing, Inc.",
    "C8903E": "Pakton Technologies",
    "DC2E6A": "HCT. Co., Ltd.",
    "2046A1": "VECOW Co., Ltd",
    "FC01CD": "FUNDACION TEKNIKER",
    "9C8BF1": "The Warehouse Limited",
    "00B338": "Kontron Asia Pacific Design Sdn. Bhd",
    "E435FB": "Sabre Technology (Hull) Ltd",
    "F0022B": "Chrontel",
    "B8288B": "Parker Hannifin Manufacturing (UK) Ltd",
    "90D11B": "Palomar Medical Technologies",
    "A06E50": "Nanotek Elektronik Sistemler Ltd. Sti.",
    "84248D": "Zebra Technologies Inc",
    "FCE892": "Hangzhou Lancable Technology Co.,Ltd",
    "34A55D": "TECHNOSOFT INTERNATIONAL SRL",
    "4C774F": "Embedded Wireless Labs ",
    "ECBD09": "FUSION Electronics Ltd",
    "944696": "BaudTec Corporation",
    "54847B": "Digital Devices GmbH",
    "843F4E": "Tri-Tech Manufacturing, Inc.",
    "C83232": "Hunting Innova",
    "D453AF": "VIGO System S.A.",
    "18AD4D": "Polostar Technology Corporation",
    "1071F9": "Cloud Telecomputers, LLC",
    "B8621F": "Cisco Systems, Inc",
    "94C6EB": "NOVA electronics, Inc.",
    "549478": "Silvershore Technology Partners",
    "54CDA7": "Fujian Shenzhou Electronic Co.,Ltd",
    "3C2763": "SLE quality engineering GmbH & Co. KG",
    "B0F1BC": "Dhemax Ingenieros Ltda",
    "508ACB": "SHENZHEN MAXMADE TECHNOLOGY CO., LTD.",
    "D0C282": "Cisco Systems, Inc",
    "147DB3": "JOA TELECOM.CO.,LTD",
    "3CD16E": "Telepower Communication Co., Ltd",
    "40040C": "A&T",
    "948FEE": "Verizon Telematics",
    "50D6D7": "Takahata Precision",
    "00FC70": "Intrepid Control Systems, Inc.",
    "704AAE": "Xstream Flow (Pty) Ltd",
    "40B3FC": "Logital Co. Limited ",
    "DC3C84": "Ticom Geomatics, Inc.",
    "D0131E": "Sunrex Technology Corp",
    "C40F09": "Hermes electronic GmbH",
    "802E14": "azeti Networks AG",
    "D4C1FC": "Nokia Corporation",
    "34BCA6": "Beijing Ding Qing Technology, Ltd.",
    "5835D9": "Cisco Systems, Inc",
    "64D912": "Solidica, Inc.",
    "C47B2F": "Beijing JoinHope Image Technology Ltd.",
    "FC2E2D": "Lorom Industrial Co.LTD.",
    "587521": "CJSC RTSoft",
    "18E288": "STT Condigi",
    "B435F7": "Zhejiang Pearmain Electronics Co.ltd.",
    "9866EA": "Industrial Control Communications, Inc.",
    "983000": "Beijing KEMACOM Technologies Co., Ltd.",
    "90CF15": "Nokia Corporation",
    "F081AF": "IRZ AUTOMATION TECHNOLOGIES LTD",
    "701404": "Limited Liability Company",
    "1C35F1": "NEW Lift Neue Elektronische Wege Steuerungsbau GmbH",
    "803457": "OT Systems Limited",
    "B09928": "FUJITSU LIMITED",
    "04E1C8": "IMS Solu\u00e7\u00f5es em Energia Ltda.",
    "948B03": "EAGET Innovation and Technology Co., Ltd.",
    "48F47D": "TechVision Holding  Internation Limited",
    "703AD8": "Shenzhen Afoundry Electronic Co., Ltd",
    "88F077": "Cisco Systems, Inc",
    "2C0033": "EControls, LLC",
    "AC199F": "SUNGROW POWER SUPPLY CO.,LTD.",
    "7C4A82": "Portsmith LLC",
    "AC5E8C": "Utillink",
    "94E848": "FYLDE MICRO LTD",
    "94AAB8": "Joview(Beijing) Technology Co. Ltd.",
    "28F358": "2C - Trifonov & Co",
    "301A28": "Mako Networks Ltd",
    "3831AC": "WEG",
    "584C19": "Chongqing Guohong Technology Development Company Limited",
    "3CA72B": "MRV Communications (Networks) LTD",
    "285132": "Shenzhen Prayfly Technology Co.,Ltd",
    "E42FF6": "Unicore communication Inc.",
    "84D9C8": "Unipattern Co.,",
    "B80B9D": "ROPEX Industrie-Elektronik GmbH",
    "5070E5": "He Shan World Fair Electronics Technology Limited",
    "802275": "Beijing Beny Wave Technology Co Ltd",
    "24C86E": "Chaney Instrument Co.",
    "F0AE51": "Xi3 Corp",
    "14F0C5": "Xtremio Ltd.",
    "E8C229": "H-Displays (MSC) Bhd",
    "5C0CBB": "CELIZION Inc.",
    "306118": "Paradom Inc.",
    "4C7367": "Genius Bytes Software Solutions GmbH",
    "90EA60": "SPI Lasers Ltd ",
    "C4242E": "Galvanic Applied Sciences Inc",
    "F8EA0A": "Dipl.-Math. Michael Rauch",
    "CCF3A5": "Chi Mei Communication Systems, Inc",
    "14A9E3": "MST CORPORATION",
    "14B73D": "ARCHEAN Technologies",
    "948D50": "Beamex Oy Ab",
    "A433D1": "Fibrlink Communications Co.,Ltd.",
    "6469BC": "Hytera Communications Co .,ltd",
    "B4F323": "PETATEL INC.",
    "14C21D": "Sabtech Industries",
    "C88439": "Sunrise Technologies",
    "F0C27C": "Mianyang Netop Telecom Equipment Co.,Ltd.",
    "283410": "Enigma Diagnostics Limited",
    "0CE82F": "Bonfiglioli Vectron GmbH",
    "40F4EC": "Cisco Systems, Inc",
    "A41BC0": "Fastec Imaging Corporation",
    "205B2A": "Private",
    "F40321": "BeNeXt B.V.",
    "A071A9": "Nokia Corporation",
    "A4E32E": "Silicon & Software Systems Ltd.",
    "5CBD9E": "HONGKONG MIRACLE EAGLE TECHNOLOGY(GROUP) LIMITED",
    "F0BF97": "Sony Corporation",
    "C44AD0": "FIREFLIES SYSTEMS",
    "88E0A0": "Shenzhen VisionSTOR Technologies Co., Ltd",
    "FC10BD": "Control Sistematizado S.A.",
    "F87B8C": "Amped Wireless",
    "241A8C": "Squarehead Technology AS",
    "D44F80": "Kemper Digital GmbH",
    "08E672": "JEBSEE ELECTRONICS CO.,LTD.",
    "B8E589": "Payter BV",
    "EC7D9D": "CPI",
    "C81E8E": "ADV Security (S) Pte Ltd",
    "A88792": "Broadband Antenna Tracking Systems",
    "04E662": "Acroname Inc.",
    "64DC01": "Static Systems Group PLC",
    "607688": "Velodyne",
    "78CD8E": "SMC Networks Inc",
    "C8C126": "ZPM Industria e Comercio Ltda",
    "64DE1C": "Kingnetic Pte Ltd",
    "A862A2": "JIWUMEDIA CO., LTD.",
    "984E97": "Starlight Marketing (H. K.) Ltd.",
    "78A683": "Precidata",
    "FC1FC0": "EURECAM",
    "BC6784": "Environics Oy",
    "B4C44E": "VXL eTech Pvt Ltd",
    "68DCE8": "PacketStorm Communications",
    "488E42": "DIGALOG GmbH",
    "2C8065": "HARTING Inc. of North America",
    "3CC0C6": "d&b audiotechnik GmbH",
    "4468AB": "JUIN COMPANY, LIMITED",
    "F81037": "Atopia Systems, LP",
    "707EDE": "NASTEC LTD.",
    "E05FB9": "Cisco Systems, Inc",
    "F02572": "Cisco Systems, Inc",
    "04FF51": "NOVAMEDIA INNOVISION SP. Z O.O.",
    "4CB4EA": "HRD (S) PTE., LTD.",
    "D44C24": "Vuppalamritha Magnetic Components LTD",
    "8091C0": "AgileMesh, Inc.",
    "084EBF": "Sumitomo Electric Industries, Ltd",
    "AC02CF": "RW Tecnologia Industria e Comercio Ltda",
    "F8C678": "Carefusion",
    "6CAB4D": "Digital Payment Technologies",
    "2CB0DF": "Soliton Technologies Pvt Ltd",
    "ECE555": "Hirschmann Automation",
    "58F98E": "SECUDOS GmbH",
    "B4749F": "ASKEY COMPUTER CORP",
    "7C4AA8": "MindTree Wireless PVT Ltd",
    "E0143E": "Modoosis Inc.",
    "C07E40": "SHENZHEN XDK COMMUNICATION EQUIPMENT CO.,LTD",
    "E44F29": "MA Lighting Technology GmbH",
    "1C7C11": "EID ",
    "48174C": "MicroPower technologies",
    "90507B": "Advanced PANMOBIL Systems GmbH & Co. KG",
    "20AA25": "IP-NET LLC",
    "349A0D": "ZBD Displays Ltd",
    "D41296": "Anobit Technologies Ltd.",
    "0876FF": "Thomson Telecom Belgium",
    "90D852": "Comtec Co., Ltd.",
    "380197": "TSST Global,Inc",
    "588D09": "Cisco Systems, Inc",
    "C0C1C0": "Cisco-Linksys, LLC",
    "0034F1": "Radicom Research, Inc.",
    "6015C7": "IdaTech",
    "DC2008": "ASD Electronics Ltd ",
    "A4D1D1": "ECOtality North America",
    "C49313": "100fio networks technology llc",
    "68597F": "Alcatel Lucent",
    "F065DD": "Primax Electronics Ltd.",
    "706582": "Suzhou Hanming Technologies Co., Ltd.",
    "C4B512": "General Electric Digital Energy",
    "E02538": "Titan Pet Products",
    "CC7A30": "CMAX Wireless Co., Ltd.",
    "B88E3A": "Infinite Technologies JLT",
    "D4CBAF": "Nokia Corporation",
    "10E8EE": "PhaseSpace",
    "A47C1F": "Cobham plc",
    "D46CDA": "CSM GmbH",
    "5CD998": "D-Link Corporation",
    "9433DD": "Taco Inc",
    "E0CF2D": "Gemintek Corporation",
    "C4EEF5": "II-VI Incorporated",
    "D491AF": "Electroacustica General Iberica, S.A.",
    "045D56": "camtron industrial inc.",
    "68234B": "Nihon Dengyo Kousaku",
    "A4BE61": "EutroVision System, Inc.",
    "58BC27": "Cisco Systems, Inc",
    "34D2C4": "RENA GmbH Print Systeme",
    "7C3920": "SSOMA SECURITY",
    "9C77AA": "NADASNV",
    "1C83B0": "Linked IP GmbH",
    "1C3DE7": "Sigma Koki Co.,Ltd.",
    "8818AE": "Tamron Co., Ltd",
    "20D607": "Nokia Corporation",
    "443D21": "Nuvolt",
    "30493B": "Nanjing Z-Com Wireless Co.,Ltd",
    "D07DE5": "Forward Pay Systems, Inc.",
    "04DD4C": "Velocytech",
    "6CE0B0": "SOUND4",
    "9CFFBE": "OTSL Inc.",
    "00F860": "PT. Panggung Electric Citrabuana",
    "EC66D1": "B&W Group LTD",
    "385FC3": "Yu Jeong System, Co.Ltd",
    "888B5D": "Storage Appliance Corporation ",
    "18EF63": "Cisco Systems, Inc",
    "206FEC": "Braemac CA LLC",
    "A45C27": "Nintendo Co., Ltd.",
    "6C0460": "RBH Access Technologies Inc.",
    "706417": "ORBIS TECNOLOGIA ELECTRICA S.A.",
    "B8BA72": "Cynove",
    "78C6BB": "Analog Devices, Inc.",
    "100D32": "Embedian, Inc.",
    "7CED8D": "Microsoft",
    "88ACC1": "Generiton Co., Ltd. ",
    "EC7C74": "Justone Technologies Co., Ltd.",
    "A8B1D4": "Cisco Systems, Inc",
    "CCFCB1": "Wireless Technology, Inc.",
    "3C1A79": "Huayuan Technology CO.,LTD",
    "C802A6": "Beijing Newmine Technology",
    "C84C75": "Cisco Systems, Inc",
    "4C8B55": "Grupo Digicon",
    "04A3F3": "Emicon",
    "1C17D3": "Cisco Systems, Inc",
    "7CE044": "NEON Inc",
    "284C53": "Intune Networks",
    "64D02D": "NEXT GENERATION INTEGRATION LIMITED (NGI)",
    "68784C": "Nortel Networks",
    "9CF61A": "Carrier Fire & Security",
    "7CF098": "Bee Beans Technologies, Inc.",
    "8841C1": "ORBISAT DA AMAZONIA IND E AEROL SA",
    "9C7514": "Wildix srl",
    "4CF737": "SamJi Electronics Co., Ltd",
    "F0D767": "Axema Passagekontroll AB",
    "84A991": "Cyber Trans Japan Co.,Ltd.",
    "F8D756": "Simm Tronic Limited ",
    "A40CC3": "Cisco Systems, Inc",
    "4CBAA3": "Bison Electronics Inc.",
    "90513F": "Elettronica Santerno SpA",
    "00B5D6": "Omnibit Inc.",
    "548922": "Zelfy Inc",
    "6C5CDE": "SunReports, Inc.",
    "34F39B": "WizLAN Ltd.",
    "4C3089": "Thales Transportation Systems GmbH",
    "E86CDA": "Supercomputers and Neurocomputers Research Center",
    "240B2A": "Viettel Group",
    "84F64C": "Cross Point BV",
    "C08B6F": "S I Sistemas Inteligentes Eletr\u00f4nicos Ltda",
    "ECB106": "Acuro Networks, Inc",
    "481249": "Luxcom Technologies Inc.",
    "24A937": "PURE Storage",
    "348302": "iFORCOM Co., Ltd",
    "F86ECF": "Arcx Inc",
    "8C8401": "Private",
    "408493": "Clavister AB",
    "6C7039": "Novar GmbH",
    "B43DB2": "Degreane Horizon",
    "34BA51": "Se-Kure Controls, Inc.",
    "089F97": "LEROY AUTOMATION",
    "78A6BD": "DAEYEON Control&Instrument Co,.Ltd",
    "3C1915": "GFI Chrono Time",
    "147373": "TUBITAK UEKAE",
    "C835B8": "Ericsson, EAB/RWI/K",
    "982D56": "Resolution Audio",
    "FCCF62": "IBM Corp",
    "084E1C": "H2A Systems, LLC",
    "88B627": "Gembird Europe BV",
    "F06853": "Integrated Corporation",
    "A4ADB8": "Vitec Group, Camera Dynamics Ltd",
    "E02630": "Intrigue Technologies, Inc.",
    "104369": "Soundmax Electronic Limited ",
    "C06C0F": "Dobbs Stanford",
    "502DF4": "Phytec Messtechnik GmbH",
    "A4B121": "Arantia 2010 S.L.",
    "E02636": "Nortel Networks",
    "5C57C8": "Nokia Corporation",
    "D46CBF": "Goodrich ISR",
    "F89D0D": "Control Technology Inc.",
    "2C3F3E": "Alge-Timing GmbH",
    "ECC882": "Cisco Systems, Inc",
    "6CFDB9": "Proware Technologies Co Ltd.",
    "B42CBE": "Direct Payment Solutions Limited",
    "CC2218": "InnoDigital Co., Ltd.",
    "C86C1E": "Display Systems Ltd",
    "90A7C1": "Pakedge Device and Software Inc.",
    "80BAAC": "TeleAdapt Ltd",
    "746B82": "MOVEK ",
    "9CC077": "PrintCounts, LLC",
    "3CB17F": "Wattwatchers Pty Ld",
    "CC5459": "OnTime Networks AS",
    "D4F143": "IPROAD.,Inc",
    "C86CB6": "Optcom Co., Ltd.",
    "A01859": "Shenzhen Yidashi Electronics Co Ltd",
    "E8056D": "Nortel Networks",
    "C45976": "Fugoo Coorporation",
    "8C56C5": "Nintendo Co., Ltd.",
    "CCB888": "AnB Securite s.a.",
    "6C5E7A": "Ubiquitous Internet Telecom Co., Ltd",
    "34862A": "Heinz Lackmann GmbH & Co KG",
    "B4ED54": "Wohler Technologies",
    "2CCD27": "Precor Inc",
    "10189E": "Elmo Motion Control",
    "5849BA": "Chitai Electronic Corp.",
    "00D11C": "ACETEL",
    "549A16": "Uzushio Electric Co.,Ltd.",
    "601D0F": "Midnite Solar",
    "A8F94B": "Eltex Enterprise Ltd.",
    "0C8230": "SHENZHEN MAGNUS TECHNOLOGIES CO.,LTD",
    "50A6E3": "David Clark Company",
    "50934F": "Gradual Tecnologia Ltda.",
    "ACE348": "MadgeTech, Inc",
    "9018AE": "Shanghai Meridian Technologies, Co. Ltd.",
    "0494A1": "CATCH THE WIND INC",
    "003A99": "Cisco Systems, Inc",
    "003A9A": "Cisco Systems, Inc",
    "003A98": "Cisco Systems, Inc",
    "F8DC7A": "Variscite LTD",
    "7072CF": "EdgeCore Networks",
    "B8B1C7": "BT&COM CO.,LTD",
    "DC2C26": "Iton Technology Limited",
    "006440": "Cisco Systems, Inc",
    "94C4E9": "PowerLayer Microsystems HongKong Limited",
    "8843E1": "Cisco Systems, Inc",
    "B8F732": "Aryaka Networks Inc",
    "E8DFF2": "PRF Co., Ltd.",
    "A4AD00": "Ragsdale Technology",
    "4C9EE4": "Hanyang Navicom Co.,Ltd.",
    "C47D4F": "Cisco Systems, Inc",
    "3CDF1E": "Cisco Systems, Inc",
    "9C4E8E": "ALT Systems Ltd",
    "042BBB": "PicoCELA, Inc.",
    "FC0877": "Prentke Romich Company",
    "ECD00E": "MiraeRecognition Co., Ltd.",
    "986DC8": "TOSHIBA MITSUBISHI-ELECTRIC INDUSTRIAL SYSTEMS CORPORATION",
    "6CAC60": "Venetex Corp",
    "C87248": "Aplicom Oy",
    "F04BF2": "JTECH Communications, Inc.",
    "1C0FCF": "Sypro Optics GmbH",
    "64F970": "Kenade Electronics Technology Co.,LTD.",
    "D411D6": "ShotSpotter, Inc.",
    "9CAFCA": "Cisco Systems, Inc",
    "C82E94": "Halfa Enterprise Co., Ltd.",
    "7C7BE4": "Z'SEDAI KENKYUSHO CORPORATION",
    "F0DE71": "Shanghai EDO Technologies Co.,Ltd.",
    "60D30A": "Quatius Limited",
    "24CF21": "Shenzhen State Micro Technology Co., Ltd",
    "10BAA5": "GANA I&C CO., LTD",
    "BC9DA5": "DASCOM Europe GmbH",
    "28FBD3": "Ragentek Technology Group",
    "586ED6": "Private",
    "C8D2C1": "Jetlun (Shenzhen) Corporation",
    "E09153": "XAVi Technologies Corp.",
    "64BC11": "CombiQ AB",
    "EC3091": "Cisco Systems, Inc",
    "F4ACC1": "Cisco Systems, Inc",
    "4097D1": "BK Electronics cc",
    "0CE936": "ELIMOS srl",
    "60391F": "ABB Ltd",
    "E8A4C1": "Deep Sea Electronics Ltd",
    "D4C766": "Acentic GmbH",
    "A02EF3": "United Integrated Services Co., Led.",
    "A09805": "OpenVox Communication Co Ltd",
    "00271F": "MIPRO Electronics Co., Ltd",
    "88A5BD": "QPCOM INC.",
    "80177D": "Nortel Networks",
    "0026A9": "Strong Technologies Pty Ltd",
    "0026A5": "MICROROBOT.CO.,LTD",
    "0026A3": "FQ Ingenieria Electronica S.A.",
    "00269D": "M2Mnet Co., Ltd.",
    "0026C5": "Guangdong Gosun Telecommunications Co.,Ltd",
    "0026C4": "Cadmos microsystems S.r.l.",
    "0026C8": "System Sensor",
    "0026C2": "SCDI Co. LTD",
    "002697": "Alpha  Technologies Inc.",
    "00268A": "Terrier SC Ltd",
    "002712": "MaxVision LLC",
    "0026CF": "DEKA R&D",
    "0026F3": "SMC Networks",
    "0026E7": "Shanghai ONLAN Communication Tech. Co., Ltd.",
    "0026E0": "ASITEQ",
    "00270C": "Cisco Systems, Inc",
    "002703": "Testech Electronics Pte Ltd",
    "002612": "Space Exploration Technologies",
    "002616": "Rosemount Inc.",
    "00260B": "Cisco Systems, Inc",
    "0025FD": "OBR Centrum Techniki Morskiej S.A.",
    "002600": "TEAC Australia Pty Ltd.",
    "002689": "General Dynamics Robotic Systems",
    "002685": "Digital Innovation",
    "002623": "JRD Communication Inc",
    "002627": "Truesell",
    "00267A": "wuhan hongxin telecommunication technologies co.,ltd",
    "002664": "Core System Japan",
    "002659": "Nintendo Co., Ltd.",
    "002651": "Cisco Systems, Inc",
    "00264E": "r2p GmbH",
    "00264F": "Kr\u00fcger &Gothe GmbH",
    "002621": "InteliCloud Technology Inc.",
    "00261C": "NEOVIA INC.",
    "002639": "T.M. Electronics, Inc.",
    "002672": "AAMP of America",
    "0025E9": "i-mate Development, Inc.",
    "0025E6": "Belgian Monitoring Systems bvba",
    "0025E0": "CeedTec Sdn Bhd",
    "0025DE": "Probits Co., LTD.",
    "0025B0": "Schmartz Inc",
    "0025AD": "Manufacturing Resources International",
    "0025AC": "I-Tech corporation",
    "0025AB": "AIO LCD PC BU / TPV",
    "0025BA": "Alcatel-Lucent IPD",
    "0025BB": "INNERINT Co., Ltd.",
    "0025B1": "Maya-Creation Corporation",
    "0025B8": "Agile Communications, Inc.",
    "0025CE": "InnerSpace",
    "0025C6": "kasercorp, ltd",
    "0025C5": "Star Link Communication Pvt. Ltd.",
    "0025C7": "altek Corporation",
    "0025FF": "CreNova Multimedia Co., Ltd",
    "0025F3": "Nordwestdeutsche Z\u00e4hlerrevision",
    "0025EC": "Humanware",
    "0025ED": "NuVo Technologies LLC",
    "0025A1": "Enalasys",
    "00259A": "CEStronics GmbH",
    "002591": "NEXTEK, Inc.",
    "00258D": "Haier",
    "0025DD": "SUNNYTEK INFORMATION CO., LTD.",
    "002537": "Runcom Technologies Ltd.",
    "002538": "Samsung Electronics Co., Ltd., Memory Division",
    "002544": "LoJack Corporation",
    "00254A": "RingCube Technologies, Inc.",
    "00254F": "ELETTROLAB Srl",
    "002549": "Jeorich Tech. Co.,Ltd.",
    "002539": "IfTA GmbH",
    "00251C": "EDT",
    "00251A": "Psiber Data Systems Inc.",
    "002514": "PC Worth Int'l Co., Ltd.",
    "002532": "Digital Recorders",
    "002571": "Zhejiang Tianle Digital Electric Co.,Ltd",
    "002583": "Cisco Systems, Inc",
    "00255D": "Morningstar Corporation",
    "002558": "MPEDIA",
    "0024F8": "Technical Solutions Company Ltd.",
    "0024F9": "Cisco Systems, Inc",
    "0024F2": "Uniphone Telecommunication Co., Ltd.",
    "002507": "ASTAK Inc.",
    "002502": "NaturalPoint",
    "00250B": "CENTROFACTOR  INC",
    "002506": "A.I. ANTITACCHEGGIO ITALIA SRL",
    "0024B5": "Nortel Networks",
    "0024B0": "ESAB AB",
    "0024E1": "Convey Computer Corp.",
    "0024D9": "BICOM, Inc.",
    "0024EE": "Wynmax Inc.",
    "0024E3": "CAO Group",
    "0024C3": "Cisco Systems, Inc",
    "0024C4": "Cisco Systems, Inc",
    "002469": "Fasttel - Smart Doorphones",
    "002464": "Bridge Technologies Co AS",
    "002462": "Rayzone Corporation",
    "002494": "Shenzhen Baoxin Tech CO., Ltd.",
    "002489": "Vodafone Omnitel N.V.",
    "00247F": "Nortel Networks",
    "002475": "Compass System(Embedded Dept.)",
    "00246F": "Onda Communication spa",
    "002420": "NetUP Inc.",
    "00241E": "Nintendo Co., Ltd.",
    "00241F": "DCT-Delta GmbH",
    "002460": "Giaval Science Development Co. Ltd.",
    "00245C": "Design-Com Technologies Pty. Ltd.",
    "002444": "Nintendo Co., Ltd.",
    "00243D": "Emerson Appliance Motors and Controls",
    "002437": "Motorola - BSG",
    "00249D": "NES Technology Inc.",
    "00248E": "Infoware ZRt.",
    "00244F": "Asantron Technologies Ltd.",
    "0023FD": "AFT Atlas Fahrzeugtechnik GmbH",
    "0023E3": "Microtronic AG",
    "0023DD": "ELGIN S.A.",
    "0023DE": "Ansync Inc.",
    "0023D9": "Banner Engineering",
    "0023DA": "Industrial Computer Source (Deutschland)GmbH",
    "002413": "Cisco Systems, Inc",
    "00240E": "Inventec Besta Co., Ltd.",
    "002405": "Dilog Nordic AB",
    "0023EF": "Zuend Systemtechnik AG",
    "0023E8": "Demco Corp.",
    "0023AC": "Cisco Systems, Inc",
    "0023B7": "Q-Light Co., Ltd.",
    "0023D8": "Ball-It Oy",
    "00231B": "Danaher Motion - Kollmorgen",
    "00231E": "Cezzer Multimedia Technologies",
    "00231F": "Guangda Electronic & Telecommunication Technology Development Co., Ltd.",
    "002270": "ABK North America, LLC",
    "002313": "Qool Technologies Ltd.",
    "002310": "LNC Technology Co., Ltd.",
    "00235B": "Gulfstream",
    "002359": "Benchmark Electronics ( Thailand ) Public Company Limited",
    "002357": "Pitronot Technologies and Engineering P.T.E. Ltd.",
    "002355": "Kinco Automation(Shanghai) Ltd.",
    "002373": "GridIron Systems, Inc.",
    "002367": "UniControls a.s.",
    "002368": "Zebra Technologies Inc",
    "00236E": "Burster GmbH & Co KG",
    "002366": "Beijing Siasun Electronic System Co.,Ltd.",
    "00239F": "Institut f\u00fcr Pr\u00fcftechnik",
    "00239D": "Mapower Electronics Co., Ltd",
    "002398": "Vutlan sro",
    "00238F": "NIDEC COPAL CORPORATION",
    "002380": "Nanoteq",
    "002334": "Cisco Systems, Inc",
    "00233F": "Purechoice Inc",
    "0022CB": "IONODES Inc.",
    "0022C6": "Sutus Inc",
    "0022C8": "Applied Instruments B.V.",
    "0022C0": "Shenzhen Forcelink Electronic Co, Ltd",
    "002296": "LinoWave Corporation",
    "0022CD": "Ared Technology Co., Ltd.",
    "0022CC": "SciLog, Inc.",
    "0022EB": "Data Respons A/S",
    "0022EF": "iWDL Technologies",
    "0022F2": "SunPower Corp",
    "0022E8": "Applition Co., Ltd.",
    "0022E9": "ProVision Communications",
    "002278": "Shenzhen  Tongfang Multimedia  Technology Co.,Ltd.",
    "00227A": "Telecom Design",
    "0022A7": "Tyco Electronics AMP GmbH",
    "0022A1": "Huawei Symantec Technologies Co.,Ltd.",
    "00229D": "PYUNG-HWA IND.CO.,LTD",
    "00230C": "CLOVER ELECTRONICS CO.,LTD.",
    "002301": "Witron Technology Limited",
    "0022F7": "Conceptronic",
    "0022E6": "Intelligent Data",
    "0022E3": "Amerigon",
    "0022E2": "WABTEC Transit Division",
    "0022A6": "Sony Computer Entertainment America",
    "002220": "Mitac Technology Corp",
    "002227": "uv-electronic GmbH",
    "002225": "Thales Avionics Ltd",
    "00221E": "Media Devices Co., Ltd.",
    "002237": "Shinhint Group",
    "00222F": "Open Grid Computing, Inc.",
    "002244": "Chengdu Linkon Communications Device Co., Ltd",
    "002250": "Point Six Wireless, LLC",
    "00226F": "3onedata Technology Co. Ltd.",
    "002260": "AFREEY Inc.",
    "0021F6": "Oracle Corporation",
    "002206": "Cyberdyne Inc.",
    "002202": "Excito Elektronik i Sk\u00e5ne AB",
    "00220E": "Indigo Security Co., Ltd.",
    "002207": "Inteno Broadband Technology AB",
    "00223E": "IRTrans GmbH",
    "0021CE": "NTC-Metrotek",
    "0021CA": "ART System Co., Ltd.",
    "0021CB": "SMS TECNOLOGIA ELETRONICA LTDA",
    "0021C8": "LOHUIS Networks",
    "0021BF": "Hitachi High-Tech Control Systems Corporation",
    "0021EF": "Kapsys",
    "0021ED": "Telegesis",
    "0021EB": "ESP SYSTEMS, LLC",
    "00219F": "SATEL OY",
    "002196": "Telsey  S.p.A.",
    "00218D": "AP Router Ind. Eletronica LTDA",
    "002190": "Goliath Solutions",
    "002174": "AvaLAN Wireless",
    "002179": "IOGEAR, Inc.",
    "0021DB": "Santachi Video Technology (Shenzhen) Co., Ltd.",
    "0021DF": "Martin Christ GmbH",
    "0021D4": "Vollmer Werke GmbH",
    "0021D6": "LXI Consortium",
    "0021A8": "Telephonics Corporation",
    "0021A9": "Mobilink Telecom Co.,Ltd",
    "0021A6": "Videotec Spa",
    "0021BC": "ZALA COMPUTER",
    "0021B4": "APRO MEDIA CO., LTD",
    "002185": "MICRO-STAR INT'L CO.,LTD.",
    "002182": "SandLinks Systems, Ltd.",
    "002183": "ANDRITZ HYDRO GmbH",
    "002168": "iVeia, LLC",
    "002139": "Escherlogic Inc.",
    "00212C": "SemIndia System Private Limited",
    "00212B": "MSA Auer",
    "00213B": "Berkshire Products, Inc",
    "002137": "Bay Controls, LLC",
    "00215E": "IBM Corp",
    "002155": "Cisco Systems, Inc",
    "002157": "National Datacast, Inc.",
    "00210C": "Cymtec Systems, Inc.",
    "00210B": "GEMINI TRAZE RFID PVT. LTD.",
    "002104": "Gigaset Communications GmbH",
    "001FFB": "Green Packet Bhd",
    "00212A": "Audiovox Corporation",
    "00214D": "Guangzhou Skytone Transmission Technology Com. Ltd.",
    "002150": "EYEVIEW ELECTRONICS",
    "001FF0": "Audio Partnership",
    "001FEA": "Applied Media Technologies Corporation",
    "002110": "Clearbox Systems",
    "001F8B": "Cache IQ",
    "001F85": "Apriva ISS, LLC",
    "001F87": "Skydigital Inc.",
    "001F88": "FMS Force Measuring Systems AG",
    "001F86": "digEcor",
    "001F80": "Lucas Holding bv",
    "001FA5": "Blue-White Industries",
    "001F9D": "Cisco Systems, Inc",
    "001F9F": "Thomson Telecom Belgium",
    "001FA1": "Gtran Inc",
    "001FE9": "Printrex, Inc.",
    "001FDD": "GDI LLC",
    "001FCE": "QTECH LLC",
    "001B58": "ACE CAD Enterprise Co., Ltd.",
    "001F79": "Lodam Electronics A/S",
    "001F71": "xG Technology, Inc.",
    "001FDA": "Nortel Networks",
    "001FD9": "RSD Communications Ltd",
    "001FB0": "TimeIPS, Inc.",
    "001FAE": "Blick South Africa (Pty) Ltd",
    "001F99": "SERONICS co.ltd",
    "001F96": "APROTECH CO.LTD",
    "001F6C": "Cisco Systems, Inc",
    "001F6F": "Fujian Sunnada Communication Co.,Ltd.",
    "001F60": "COMPASS SYSTEMS CORP.",
    "001F6A": "PacketFlux Technologies, Inc.",
    "001F65": "KOREA ELECTRIC TERMINAL CO., LTD.",
    "001F5E": "Dyna Technology Co.,Ltd.",
    "001EE8": "Mytek",
    "001EEE": "ETL Systems Ltd",
    "001EE0": "Urmet SpA",
    "001EDA": "Wesemann Elektrotechniek B.V.",
    "001F17": "IDX Company, Ltd.",
    "001F55": "Honeywell Security (China) Co., Ltd.",
    "001F54": "Lorex Technology Inc.",
    "001F4B": "Lineage Power",
    "001F40": "Speakercraft Inc.",
    "001EFC": "JSC \"MASSA-K\"",
    "001F06": "Integrated Dispatch Solutions",
    "001EFA": "PROTEI Ltd.",
    "001EFB": "Trio Motion Technology Ltd",
    "001EF8": "Emfinity Inc.",
    "001F1B": "RoyalTek Company Ltd.",
    "001F0D": "L3 Communications - Telemetry West",
    "001F2E": "Triangle Research Int'l Pte Ltd",
    "001ECB": "\"RPC \"Energoautomatika\" Ltd",
    "001EBA": "High Density Devices AS",
    "001E9F": "Visioneering Systems, Inc.",
    "001E9B": "San-Eisha, Ltd.",
    "001ED7": "H-Stream Wireless, Inc.",
    "001ED5": "Tekon-Automatics",
    "001EB4": "UNIFAT TECHNOLOGY LTD.",
    "001EA8": "Datang Mobile Communications Equipment CO.,LTD",
    "001EAB": "TeleWell Oy",
    "001E61": "ITEC GmbH",
    "001E59": "Silicon Turnkey Express, LLC",
    "001E71": "MIrcom Group of Companies",
    "001E6B": "Cisco SPVTG",
    "001E70": "Chelton Limited",
    "001E51": "Converter Industry Srl",
    "001E47": "PT. Hariff Daya Tunggal Engineering",
    "001E48": "Wi-Links",
    "001E8A": "eCopy, Inc",
    "001E1B": "Digital Stream Technology, Inc.",
    "001E17": "STN BV",
    "001E18": "Radio Activity srl",
    "001E15": "Beech Hill Electronics",
    "001DC4": "AIOI Systems Co., Ltd.",
    "001DC0": "Enphase Energy",
    "001DBD": "Versamed Inc.",
    "001DF8": "Webpro Vision Technology Corporation",
    "001DF9": "Cybiotronics (Far East) Limited",
    "001DF7": "R. STAHL Schaltger\u00e4te GmbH",
    "001E05": "Xseed Technologies & Computing",
    "001E07": "Winy Technology Co., Ltd.",
    "001E0A": "Syba Tech Limited",
    "001E03": "LiComm Co., Ltd.",
    "001E38": "Bluecard Software Technology Co., Ltd.",
    "001E30": "Shireen Inc",
    "001E2E": "SIRTI S.p.A.",
    "001DEB": "DINEC International",
    "001DDC": "HangZhou DeChangLong Tech&Info Co.,Ltd",
    "001E3E": "KMW Inc.",
    "001DB7": "Tendril Networks, Inc.",
    "001D9D": "ARTJOY INTERNATIONAL LIMITED",
    "001D9E": "AXION TECHNOLOGIES",
    "001D9A": "GODEX INTERNATIONAL CO., LTD",
    "001D97": "Alertus Technologies LLC",
    "001D43": "Shenzhen G-link Digital Technology Co., Ltd.",
    "001D3F": "Mitron Pty Ltd",
    "001D39": "MOOHADIGITAL CO., LTD",
    "001D37": "Thales-Panda Transportation System",
    "001D30": "YX Wireless S.A.",
    "001D7F": "Tekron International Ltd",
    "001D70": "Cisco Systems, Inc",
    "001D79": "SIGNAMAX LLC",
    "001D6F": "Chainzone Technology Co., Ltd",
    "001D78": "Invengo Information Technology Co.,Ltd",
    "001D62": "InPhase Technologies",
    "001D63": "Miele & Cie. KG",
    "001DAE": "CHANG TSENG TECHNOLOGY CO., LTD",
    "001DA6": "Media Numerics Limited",
    "001D91": "Digitize, Inc",
    "001D95": "Flash, Inc.",
    "001D8D": "Fluke Process Instruments GmbH",
    "001D13": "NextGTV",
    "001D10": "LightHaus Logic, Inc.",
    "001D14": "SPERADTONE INFORMATION TECHNOLOGY LIMITED",
    "001CBB": "MusicianLink",
    "001CB2": "BPT SPA",
    "001CB5": "Neihua Network Technology Co.,LTD.(NHN)",
    "001CB4": "Iridium Satellite LLC",
    "001CB6": "Duzon CNT Co., Ltd.",
    "001D1F": "Siauliu Tauro Televizoriai, JSC",
    "001CCD": "Alektrona Corporation",
    "001CC7": "Rembrandt Technologies, LLC d/b/a REMSTREAM",
    "001D04": "Zipit Wireless, Inc.",
    "001CD0": "Circleone Co.,Ltd.",
    "001C9F": "Razorstream, LLC",
    "001CEC": "Mobilesoft (Aust.) Pty Ltd",
    "001CE8": "Cummins Inc",
    "001CF2": "Tenlon Technology Co.,Ltd.",
    "001C1F": "Quest Retail Technology Pty Ltd",
    "001C16": "ThyssenKrupp Elevator",
    "001C19": "secunet Security Networks AG",
    "001C47": "Hangzhou Hollysys Automation Co., Ltd",
    "001C49": "Zoltan Technology Inc.",
    "001C3F": "International Police Technologies, Inc.",
    "001C3B": "AmRoad Technology Inc.",
    "001C32": "Telian Corporation",
    "001C63": "TRUEN",
    "001C5C": "Integrated Medical Systems, Inc.",
    "001C52": "VISIONEE SRL",
    "001C8D": "Mesa Imaging",
    "001C89": "Force Communications, Inc.",
    "001C87": "Uriver Inc.",
    "001C7D": "Excelpoint Manufacturing Pte Ltd",
    "001C77": "Prodys",
    "001C6F": "Emfit Ltd",
    "001C6C": "30805",
    "001C61": "Galaxy  Microsystems LImited",
    "001C2B": "Alertme.com Limited",
    "001C28": "Sphairon Technologies GmbH ",
    "001C09": "SAE Electronic Co.,Ltd.",
    "001C0C": "TANITA Corporation",
    "001B9B": "Hose-McCann Communications",
    "001B9C": "SATEL sp. z o.o.",
    "001B92": "l-acoustics",
    "001B8E": "Hulu Sweden AB",
    "001BE8": "Ultratronik GmbH",
    "001BE1": "ViaLogy",
    "001BDF": "Iskra Sistemi d.d.",
    "001BEC": "Netio Technologies Co., Ltd",
    "001BB4": "Airvod Limited",
    "001BB6": "Bird Electronic Corp.",
    "001BA6": "intotech inc.",
    "001B8A": "2M Electronic A/S",
    "001B80": "LORD Corporation",
    "001B76": "Ripcode, Inc.",
    "001B70": "IRI Ubiteq, INC.",
    "001BC7": "StarVedia Technology Inc.",
    "001BA4": "S.A.E Afikim",
    "001B93": "JC Decaux SA DNT",
    "001BD9": "Edgewater Wireless Systems Inc",
    "001B07": "Mendocino Software",
    "001B08": "Danfoss Drives A/S",
    "001B01": "Applied Radio Technologies",
    "001B02": "ED Co.Ltd",
    "001AFC": "ModusLink Corporation",
    "001B6C": "LookX Digital Media BV",
    "001B6B": "Swyx Solutions AG",
    "001B69": "Equaline Corporation",
    "001B68": "Modnnet Co., Ltd",
    "001B62": "JHT Optoelectronics Co.,Ltd.",
    "001B3F": "ProCurve Networking by HP",
    "001B41": "General Infinity Co.,Ltd.",
    "001B3E": "Curtis, Inc.",
    "001B37": "Computec Oy",
    "001B10": "ShenZhen Kang Hui Technology Co.,ltd",
    "001B0B": "Phidgets Inc.",
    "001B0C": "Cisco Systems, Inc",
    "001B23": "SimpleComTools",
    "001B18": "Tsuken Electric Ind. Co.,Ltd",
    "001AF6": "Woven Systems, Inc.",
    "001AF9": "AeroVIronment (AV Inc)",
    "001AE0": "Mythology Tech Express Inc.",
    "001B50": "Nizhny Novgorod Factory named after M.Frunze, FSUE (NZiF)",
    "001B30": "Solitech Inc.",
    "001B47": "Futarque A/S",
    "001B45": "ABB AS, Division Automation Products",
    "001ABE": "COMPUTER HI-TECH INC.",
    "001AC1": "3Com Ltd",
    "001ABB": "Fontal Technology Incorporation",
    "001ABD": "Impatica Inc.",
    "001AAE": "Savant Systems LLC",
    "001A58": "CCV Deutschland GmbH - Celectronic eHealth Div.",
    "001A5E": "Thincom Technology Co.,Ltd",
    "001A87": "Canhold International Limited",
    "001A88": "Venergy,Co,Ltd",
    "001A74": "Procare International Co",
    "001AB4": "FFEI Ltd.",
    "001AB5": "Home Network System",
    "001A79": "TELECOMUNICATION TECHNOLOGIES LTD.",
    "001AAA": "Analogic Corp.",
    "001A8B": "CHUNIL ELECTRIC IND., CO.",
    "001A8D": "AVECS Bergen GmbH",
    "001A95": "Hisense Mobile Communications Technoligy Co.,Ltd.",
    "001A81": "Zelax",
    "001AD7": "Christie Digital Systems, Inc.",
    "001ACD": "Tidel Engineering LP",
    "001AE2": "Cisco Systems, Inc",
    "001AE7": "Aztek Networks, Inc.",
    "001AC9": "SUZUKEN CO.,LTD",
    "001AA4": "Future University-Hakodate",
    "001A9F": "A-Link Ltd",
    "001A24": "Galaxy Telecom Technologies Ltd",
    "001A20": "CMOTECH Co. Ltd.",
    "001A19": "Computer Engineering Limited",
    "001A18": "Advanced Simulation Technology inc.",
    "001A5C": "Euchner GmbH+Co. KG",
    "001A5B": "NetCare Service Co., Ltd.",
    "001A4E": "NTI AG / LinMot",
    "001A52": "Meshlinx Wireless Inc.",
    "001A13": "Wanlida Group Co., LTD",
    "001A0F": "ARTECHE GROUP",
    "001A43": "Logical Link Communications",
    "001A47": "Agami Systems, Inc.",
    "001A03": "Angel Electronics Co., Ltd.",
    "0019F9": "TDK-Lambda",
    "0019F2": "Teradyne K.K.",
    "001A2F": "Cisco Systems, Inc",
    "001A2D": "The Navvo Group",
    "00198D": "Ocean Optics, Inc.",
    "001985": "IT Watchdogs, Inc",
    "001986": "Cheng Hongjian",
    "0019A9": "Cisco Systems, Inc",
    "0019AE": "Hopling Technologies b.v.",
    "0019AF": "Rigol Technologies, Inc.",
    "0019A7": "ITU-T",
    "0019DE": "MOBITEK",
    "0019E5": "Lynx Studio Technology, Inc.",
    "0019DB": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "00199E": "Nifty",
    "00196A": "MikroM GmbH",
    "0019CE": "Progressive Gaming International",
    "0019BD": "New Media Life",
    "00196B": "Danpex Corporation",
    "00199F": "DKT A/S",
    "0018FF": "PowerQuattro Co.",
    "0018F4": "EO TECHNICS Co., Ltd.",
    "0018F6": "Thomson Telecom Belgium",
    "0018FC": "Altec Electronic AG",
    "0018F5": "Shenzhen Streaming Video Technology Company Limited",
    "0018F9": "VVOND, Inc.",
    "00193B": "LigoWave",
    "001935": "DUERR DENTAL AG",
    "001932": "Gude Systems GmbH",
    "001931": "Balluff GmbH",
    "001914": "Winix Co., Ltd",
    "001901": "F1MEDIA",
    "001906": "Cisco Systems, Inc",
    "00193D": "GMC Guardian Mobility Corp.",
    "001910": "Knick Elektronische Messgeraete GmbH & Co. KG",
    "001913": "Chuang-Yi Network Equipment Co.Ltd.",
    "001951": "NETCONS, s.r.o.",
    "0018FA": "Yushin Precision Equipment Co.,Ltd.",
    "0018EA": "Alltec GmbH",
    "0018E8": "Hacetron Corporation",
    "001957": "Saafnet Canada Inc.",
    "001956": "Cisco Systems, Inc",
    "001958": "Bluetooth SIG, Inc.",
    "0018E3": "Visualgate Systems, Inc.",
    "001899": "ShenZhen jieshun Science&Technology Industry CO,LTD.",
    "00189F": "Lenntek Corporation",
    "0018B7": "D3 LED, LLC",
    "0018C1": "Almitec Inform\u00e1tica e Com\u00e9rcio",
    "0018C4": "Raba Technologies LLC",
    "0018C9": "EOps Technology Limited",
    "0018BC": "ZAO NVP Bolid",
    "00185D": "TAIGUEN TECHNOLOGY (SHEN-ZHEN) CO., LTD.",
    "00185E": "Nexterm Inc.",
    "00186D": "Zhenjiang Sapphire Electronic Industry CO.",
    "00186F": "Setha Industria Eletronica LTDA",
    "001875": "AnaCise Testnology Pte Ltd",
    "0018D8": "ARCH METER Corporation",
    "0018D9": "Santosha Internatonal, Inc",
    "0018CF": "Baldor Electric Company",
    "00188E": "Ekahau, Inc.",
    "001883": "FORMOSA21 INC.",
    "001895": "Hansun Technologies Inc.",
    "00184A": "Catcher, Inc.",
    "00184B": "Las Vegas Gaming, Inc.",
    "0017BB": "Syrinx Industrial Electronics",
    "0017BC": "Touchtunes Music Corporation",
    "0017C1": "CM Precision Technology LTD.",
    "0017B2": "SK Telesys",
    "0017B1": "ACIST Medical Systems, Inc.",
    "0017E1": "DACOS Technologies Co., Ltd.",
    "0017E0": "Cisco Systems, Inc",
    "0017D2": "THINLINX PTY LTD",
    "0017C7": "MARA Systems Consulting AB",
    "001814": "Mitutoyo Corporation",
    "001817": "D. E. Shaw Research, LLC",
    "001811": "Neuros Technology International, LLC.",
    "0017DE": "Advantage Six Ltd",
    "0017D7": "ION Geophysical Corporation Inc.",
    "001837": "Universal ABIT Co., Ltd.",
    "001822": "CEC TELECOM CO.,LTD.",
    "001820": "w5networks",
    "0017B8": "NOVATRON CO., LTD.",
    "001828": "e2v technologies (UK) ltd.",
    "001835": "Thoratec / ITC",
    "00180E": "Avega Systems",
    "001801": "Actiontec Electronics, Inc",
    "0017F3": "Harris Corporation",
    "001750": "GSI Group, MicroE Systems",
    "001755": "GE Security",
    "001751": "Online Corporation",
    "0017A3": "MIX s.r.l.",
    "0017A6": "YOSIN ELECTRONICS CO., LTD.",
    "00179C": "DEPRAG SCHULZ GMBH u. CO.",
    "001796": "Rittmeyer AG",
    "001775": "TTE Germany GmbH",
    "001776": "Meso Scale Diagnostics, LLC",
    "001779": "QuickTel",
    "001767": "Earforce AS",
    "00177C": "Smartlink Network Systems Limited",
    "001781": "Greystone Data System, Inc.",
    "001785": "Sparr Electronics Ltd",
    "00172C": "TAEJIN INFOTECH",
    "00174C": "Millipore",
    "001745": "INNOTZ CO., Ltd",
    "001748": "Neokoros Brasil Ltda",
    "00178D": "Checkpoint Systems, Inc.",
    "00178E": "Gunnebo Cash Automation AB",
    "001763": "Essentia S.p.A.",
    "00175D": "Dongseo system.",
    "001739": "Bright Headphone Electronics Company",
    "00171D": "DIGIT",
    "001718": "Vansco Electronics Oy",
    "001719": "Audiocodes USA, Inc",
    "0016FF": "Wamin Optocomm Mfg Corp",
    "001701": "KDE, Inc.",
    "00170E": "Cisco Systems, Inc",
    "001705": "Methode Electronics",
    "0016DA": "Futronic Technology Co. Ltd.",
    "0016D4": "Compal Communications, Inc.",
    "0016D7": "Sunways AG",
    "0016D1": "ZAT a.s.",
    "001711": "Cytiva Sweden AB",
    "001717": "Leica Geosystems AG",
    "001715": "Qstik",
    "0016C5": "Shenzhen Xing Feng Industry Co.,Ltd",
    "0016CC": "Xcute Mobile Corp.",
    "0016EE": "Royaldigital Inc.",
    "0016ED": "Utility, Inc",
    "0016DE": "FAST Inc",
    "001672": "Zenway enterprise ltd",
    "001671": "Symphox Information Co.",
    "001669": "MRV Communication (Networks) LTD",
    "001668": "Eishin Electronics",
    "001655": "FUHO TECHNOLOGY Co., LTD",
    "001646": "Cisco Systems, Inc",
    "001648": "SSD Company Limited",
    "0016AA": "Kei Communication Technology Inc.",
    "0016A8": "CWT CO., LTD.",
    "0016A6": "Dovado FZ-LLC",
    "0016A2": "CentraLite Systems, Inc.",
    "00167D": "Sky-Line Information Co., Ltd.",
    "001677": "Bihl + Wiedemann GmbH",
    "00165A": "Harman Specialty Group",
    "001659": "Z.M.P. RADWAG",
    "0016C7": "Cisco Systems, Inc",
    "001695": "AVC Technology (International) Limited",
    "0015DF": "Clivet S.p.A.",
    "0015D8": "Interlink Electronics",
    "0015D2": "Xantech Corporation",
    "0015D4": "Emitor AB",
    "0015D6": "OSLiNK Sp. z o.o.",
    "0015D5": "NICEVT",
    "0015F1": "KYLINK Communications Corp.",
    "0015EA": "Tellumat (Pty) Ltd",
    "0015E2": "Dr.Ing. Herbert Knauer GmbH",
    "0015E1": "Picochip Ltd",
    "00160C": "LPL  DEVELOPMENT S.A. DE C.V",
    "001612": "Otsuka Electronics Co., Ltd.",
    "00160B": "TVWorks LLC",
    "001603": "COOLKSKY Co., LTD",
    "001609": "Unitech electronics co., ltd.",
    "001640": "Asmobile Communication Inc.",
    "00163C": "Rebox B.V.",
    "001619": "Lancelan Technologies S.L.",
    "00161D": "Innovative Wireless Technologies, Inc.",
    "001611": "Altecon Srl",
    "0015C7": "Cisco Systems, Inc",
    "001625": "Impinj, Inc.",
    "001623": "Interval Media",
    "0015F5": "Sustainable Energy Systems",
    "00158B": "Park Air Systems Ltd",
    "001586": "Xiamen Overseas Chinese Electronic Co., Ltd.",
    "00157E": "Weidm\u00fcller Interface GmbH & Co. KG",
    "001580": "U-WAY CORPORATION",
    "00157C": "Dave Networks, Inc.",
    "00157F": "ChuanG International Holding CO.,LTD.",
    "001533": "NADAM.CO.,LTD",
    "001537": "Ventus Networks",
    "001534": "A Beltr\u00f3nica-Companhia de Comunica\u00e7\u00f5es, Lda",
    "00159D": "Tripp Lite ",
    "001592": "Facom UK Ltd (Melksham)",
    "00155C": "Dresser Wayne",
    "001559": "Securaplane Technologies, Inc.",
    "001557": "Olivetti",
    "001554": "Atalum Wireless S.A.",
    "0015B3": "Caretech AB",
    "0015AA": "Rextechnik International Co.,",
    "00153F": "Alcatel Alenia Space Italia",
    "001576": "LABiTec - Labor Biomedical Technologies GmbH",
    "0015BE": "Iqua Ltd.",
    "00155E": "Morgan Stanley",
    "001502": "BETA tech",
    "0014FD": "Thecus Technology Corp.",
    "0014FC": "Extandon, Inc.",
    "0014F8": "Scientific Atlanta",
    "0014F7": "CREVIS Co., LTD",
    "00150F": "mingjong",
    "00150B": "SAGE INFOTECH LTD.",
    "001507": "Renaissance Learning Inc",
    "001508": "Global Target Enterprise Inc",
    "0014E5": "Alticast",
    "0014E0": "LET'S Corporation",
    "0014E2": "datacom systems inc.",
    "0014E4": "infinias, LLC",
    "001518": "Shenzhen 10MOONS Technology Development CO.,Ltd",
    "001515": "Leipold+Co.GmbH",
    "001526": "Remote Technologies Inc",
    "0014F1": "Cisco Systems, Inc",
    "0014EA": "S Digm Inc. (Safe Paradigm Inc.)",
    "0014CB": "LifeSync Corporation",
    "0014C6": "Quixant Ltd",
    "0014DB": "Elma Trenew Electronic GmbH",
    "0014CC": "Zetec, Inc.",
    "001474": "K40 Electronics",
    "00146F": "Kohler Co",
    "001466": "Kleinhenz Elektronik GmbH",
    "00146B": "Anagran, Inc.",
    "00149F": "System and Chips, Inc.",
    "001498": "Viking Design Technology",
    "001496": "Phonic Corp.",
    "001493": "Systimax Solutions",
    "00143D": "Aevoe Inc.",
    "00143A": "RAYTALK INTERNATIONAL SRL",
    "001461": "CORONA CORPORATION",
    "001462": "Digiwell Technology, inc",
    "001463": "IDCS N.V.",
    "001465": "Novo Nordisk A/S",
    "00148D": "Cubic Defense Simulation Systems",
    "001486": "Echo Digital Audio Corporation",
    "001482": "Aurora Networks",
    "001436": "Qwerty Elektronik AB",
    "001433": "Empower Technologies(Canada) Inc.",
    "001434": "Keri Systems, Inc",
    "00147F": "Thomson Telecom Belgium",
    "001475": "Wiline Networks, Inc.",
    "001455": "Coder Electronics Corporation",
    "00144E": "SRISA",
    "0014B3": "CoreStar International Corp",
    "0014B1": "Axell Wireless Limited",
    "0013A4": "KeyEye Communications",
    "0013A7": "BATTELLE MEMORIAL INSTITUTE",
    "0013A6": "Extricom Ltd",
    "00139F": "Electronics Design Services, Co., Ltd.",
    "0013A2": "MaxStream, Inc",
    "0013A0": "ALGOSYSTEM Co., Ltd.",
    "001398": "TrafficSim Co.,Ltd",
    "001415": "Intec Automation inc.",
    "001410": "Suzhou Keda Technology CO.,Ltd",
    "001417": "RSE Informations Technologie GmbH",
    "001408": "Eka Systems Inc.",
    "001423": "J-S Co. NEUROCOM",
    "001425": "Galactic Computing Corp.",
    "001419": "SIDSA",
    "0013E2": "GeoVision Inc.",
    "0013DE": "Adapt4, LLC",
    "0013DD": "Abbott Diagnostics",
    "0013D7": "SPIDCOM Technologies SA",
    "0013C7": "IONOS Co.,Ltd.",
    "0013B2": "Carallon Limited",
    "0013AD": "Sendo Ltd",
    "0013AA": "ALS  & TEC Ltd.",
    "0013EE": "JBX Designs Inc.",
    "0013E5": "TENOSYS, INC.",
    "001402": "kk-electronic a/s",
    "0013FF": "Dage-MTI of MC, Inc.",
    "0013BC": "Artimi Ltd",
    "001384": "Advanced Motion Controls",
    "00137B": "Movon Corporation",
    "00134A": "Engim, Inc.",
    "00133A": "VadaTech Inc.",
    "00132A": "Sitronics Telecom Solutions",
    "00139B": "ioIMAGE Ltd.",
    "001396": "Acbel Polytech Inc.",
    "001393": "Panta Systems, Inc.",
    "00138B": "Phantom Technologies LLC",
    "001388": "WiMedia Alliance",
    "001353": "HYDAC Filtertechnik GMBH",
    "00134D": "Inepro BV",
    "00134B": "ToGoldenNet Technology Inc.",
    "001334": "Arkados, Inc.",
    "001337": "Orient Power Home Network Ltd.",
    "001332": "Beijing Topsec Network Security Technology Co., Ltd.",
    "00131F": "NxtPhase T&D, Corp.",
    "00136E": "Techmetro Corp.",
    "00136D": "Tentaculus AB",
    "00136A": "Hach Lange Sarl",
    "00130E": "Focusrite Audio Engineering Limited",
    "001309": "Ocean Broadband Networks",
    "001319": "Cisco Systems, Inc",
    "00131C": "LiteTouch, Inc.",
    "001363": "Verascape, Inc.",
    "0012FF": "Lely Industries N.V.",
    "001303": "GateConnect",
    "0012F9": "URYU SEISAKU, LTD.",
    "001304": "Flaircomm Technologies Co. LTD",
    "0012B3": "Advance Wireless Technology Corp.",
    "0012B0": "Efore Oyj   (Plc)",
    "0012A6": "Dolby Australia",
    "0012A4": "ThingMagic, LLC",
    "0012A9": "3Com Ltd",
    "0012DC": "SunCorp Industrial Limited",
    "0012D7": "Invento Networks, Inc.",
    "0012C4": "Viseon, Inc.",
    "0012D0": "Gossen-Metrawatt-GmbH",
    "0012E5": "Time America, Inc.",
    "0012B8": "G2 Microsystems",
    "001299": "Ktech Telecommunications Inc",
    "001280": "Cisco Systems, Inc",
    "001275": "Sentilla Corporation",
    "001276": "CG Power Systems Ireland Limited",
    "001271": "Measurement Computing Corp",
    "001273": "Stoke Inc",
    "001269": "Value Electronics",
    "001264": "daum electronic gmbh",
    "001261": "Adaptix, Inc",
    "001260": "Stanton Magnetics,inc.",
    "00128C": "Woodward Governor",
    "001293": "ABB Switzerland Ltd.",
    "001294": "SUMITOMO ELECTRIC DEVICE INNOVATIONS, INC",
    "001296": "Addlogix",
    "001258": "TechVoIP Sp z o.o.",
    "001250": "Tokyo Aircaft Instrument Co., Ltd.",
    "001252": "Citronix, LLC",
    "00127F": "Cisco Systems, Inc",
    "00127B": "VIA Networking Technologies, Inc.",
    "001240": "AMOI ELECTRONICS CO.,LTD",
    "00122E": "Signal Technology - AISD",
    "001231": "Motion Control Systems, Inc.",
    "001224": "NexQL Corporation",
    "00124B": "Texas Instruments",
    "00124A": "Dedicated Devices, Inc.",
    "001243": "Cisco Systems, Inc",
    "001203": "ActivNetworks",
    "0011F8": "AIRAYA Corp",
    "0011F4": "woori-net",
    "0011F6": "Asia Pacific Microsystems , Inc.",
    "0011F0": "Wideful Limited",
    "0011F1": "QinetiQ Ltd",
    "0011AA": "Uniclass Technology, Co., LTD",
    "0011B1": "BlueExpert Technology Corp.",
    "0011DB": "Land-Cellular Corporation",
    "0011D2": "Perception Digital Ltd",
    "0011D9": "TiVo",
    "0011CF": "Thrane & Thrane A/S",
    "0011D4": "NetEnrich, Inc",
    "0011D5": "Hangzhou Sunyard System Engineering Co.,Ltd.",
    "0011CC": "Guangzhou Jinpeng Group Co.,Ltd.",
    "0011C7": "Raymarine UK Ltd",
    "0011C9": "MTT Corporation",
    "001211": "Protechna Herbst GmbH & Co. KG",
    "001219": "General Datacomm LLC",
    "001216": "ICP Internet Communication Payment AG",
    "001215": "iStor Networks, Inc.",
    "0011A6": "Sypixx Networks",
    "0011A2": "Manufacturing Technology Inc",
    "00119E": "Solectron Brazil",
    "0011B5": "Shenzhen Powercom Co.,Ltd",
    "0011BB": "Cisco Systems, Inc",
    "0011BD": "Bombardier Transportation",
    "0011ED": "802 Global",
    "00111C": "Pleora Technologies Inc.",
    "001125": "IBM Corp",
    "00111F": "Doremi Labs, Inc.",
    "00111D": "Hectrix Limited",
    "001119": "Solteras, Inc.",
    "001184": "Humo Laboratory,Ltd.",
    "00117E": "Midmark Corp",
    "001152": "Eidsvoll Electronics AS",
    "001150": "Belkin Corporation",
    "001166": "Taelim Electronics Co., Ltd.",
    "001164": "ACARD Technology Corp.",
    "001155": "Sevis Systems",
    "001146": "Telecard-Pribor Ltd",
    "00114C": "caffeina applied research ltd.",
    "001141": "GoodMan Corporation",
    "00118A": "Viewtran Technology Limited",
    "001179": "Singular Technology Co. Ltd.",
    "001173": "SMART Storage Systems",
    "00112C": "IZT GmbH",
    "000FCF": "DataWind Research",
    "000FD2": "EWA Technologies, Inc.",
    "000FCE": "Kikusui Electronics Corp.",
    "000FA1": "Gigabit Systems Inc.",
    "000F99": "APAC opto Electronics Inc.",
    "000FE8": "Lobos, Inc.",
    "000FEB": "Cylon Controls",
    "000FDC": "Ueda Japan  Radio Co., Ltd.",
    "000FF4": "Guntermann & Drunck GmbH",
    "000FF8": "Cisco Systems, Inc",
    "000FF5": "GN&S company",
    "000FD7": "Harman Music Group",
    "000FD4": "Soundcraft",
    "001114": "EverFocus Electronics Corp.",
    "00110E": "Tsurusaki Sealand Transportation Co. Ltd.",
    "00110D": "SANBlaze Technology, Inc.",
    "000FA5": "BWA Technology GmbH",
    "000FB2": "Broadband Pacenet (India) Pvt. Ltd.",
    "001106": "Siemens NV (Belgium)",
    "000FAF": "Dialog Inc.",
    "000F43": "Wasabi Systems Inc.",
    "000F48": "Polypix Inc.",
    "000F50": "StreamScale Limited",
    "000F47": "ROBOX SPA",
    "000F4E": "Cellink",
    "000F31": "Allied Vision Technologies Canada Inc",
    "000F29": "Augmentix Corporation",
    "000F27": "TEAL Electronics, Inc.",
    "000F8E": "DONGYANG TELECOM CO.,LTD.",
    "000F91": "Aerotelecom Co.,Ltd.",
    "000F73": "RS Automation Co., Ltd",
    "000F78": "Datacap Systems Inc",
    "000F70": "Wintec Industries, inc.",
    "000F87": "Maxcess International",
    "000F80": "Trinity Security Systems,Inc.",
    "000F5F": "Nicety Technologies Inc. (NTS)",
    "000F5A": "Peribit Networks",
    "000F74": "Qamcom Technology AB",
    "000F6D": "Midas Engineering",
    "000F3C": "Endeleo Limited",
    "000F18": "Industrial Control Systems",
    "000F1D": "Cosmo Techs Co., Ltd.",
    "000F1B": "Ego Systems Inc.",
    "000F0F": "Real ID Technology Co., Ltd.",
    "000F16": "JAY HOW TECHNOLOGY CO.,",
    "000EF8": "SBC ASI",
    "000EF9": "REA Elektronik GmbH",
    "000EF6": "E-TEN Information Systems Co., Ltd.",
    "000EEA": "Shadong Luneng Jicheng Electronics,Co.,Ltd",
    "000EAD": "Metanoia Technologies, Inc.",
    "000EAF": "CASTEL",
    "000EE6": "Adimos Systems LTD",
    "000EC6": "ASIX ELECTRONICS CORP.",
    "000EBF": "Remsdaq Limited",
    "000EE0": "Mcharge",
    "000EAB": "Cray Inc",
    "000E9F": "TEMIC SDS GmbH",
    "000E92": "Open Telecom",
    "000E96": "Cubic Defense Applications, Inc.",
    "000E8E": "SparkLAN Communications, Inc.",
    "000E91": "Navico Auckland Ltd",
    "000E14": "Visionary Solutions, Inc.",
    "000E1B": "IAV GmbH",
    "000E0F": "ERMME",
    "000E10": "C-guys, Inc.",
    "000E13": "Accu-Sort Systems inc.",
    "000E0A": "SAKUMA DESIGN OFFICE",
    "000E7C": "Televes S.A.",
    "000E73": "Tpack A/S",
    "000E6E": "MAT S.A. (Mircrelec Advanced Technology)",
    "000E56": "4G Systems GmbH & Co. KG",
    "000E55": "AUVITRAN",
    "000E53": "AV TECH CORPORATION",
    "000E66": "Hitachi Industry & Control Solutions, Ltd.",
    "000E68": "E-TOP Network Technology Inc.",
    "000E33": "Shuko Electronics Co.,Ltd",
    "000E3A": "Cirrus Logic",
    "000E3B": "Hawking Technologies, Inc.",
    "000E84": "Cisco Systems, Inc",
    "000E87": "adp Gauselmann GmbH",
    "000E75": "New York Air Brake Corp.",
    "000E48": "Lipman TransAction Solutions",
    "000E3E": "Sun Optronics Inc",
    "000E27": "Crere Networks, Inc.",
    "000E18": "MyA Technology",
    "000D9B": "Heraeus Electro-Nite International N.V.",
    "000D8A": "Winners Electronics Co., Ltd.",
    "000D8E": "Koden Electronics Co., Ltd.",
    "000DCD": "GROUPE TXCOM",
    "000DCF": "Cidra Corp.",
    "000DCA": "Tait Electronics",
    "000DDE": "Joyteck Co., Ltd.",
    "000DE2": "CMZ Sistemi Elettronici",
    "000DDA": "ALLIED TELESIS K.K.",
    "000DD6": "ITI    LTD",
    "000DD5": "O'RITE TECHNOLOGY CO.,LTD",
    "000E0E": "ESA elettronica S.P.A.",
    "000DFD": "Huges Hi-Tech Inc.,",
    "000DF9": "NDS Limited",
    "000DFB": "Komax AG",
    "000E00": "Atrie",
    "000DF4": "Watertek Co.",
    "000DFA": "Micro Control Systems Ltd.",
    "000DFC": "ITFOR Inc.",
    "000DFE": "Hauppauge Computer Works, Inc.",
    "000DB8": "SCHILLER AG",
    "000DC4": "Emcore Corporation",
    "000DEC": "Cisco Systems, Inc",
    "000DF2": "Private",
    "000DA0": "NEDAP N.V.",
    "000D4F": "Kenwood Corporation",
    "000D47": "Collex",
    "000D43": "DRS Tactical Systems Inc.",
    "000D44": "Audio BU - Logitech",
    "000D3D": "Hammerhead Systems, Inc.",
    "000D3E": "APLUX Communications Ltd.",
    "000D5A": "Tiesse SpA",
    "000D61": "Giga-Byte Technology Co., Ltd.",
    "000D36": "Wu Han Routon Electronic Co., Ltd",
    "000D3B": "Microelectronics Technology Inc.",
    "000D1A": "Mustek System Inc.",
    "000D1E": "Control Techniques",
    "000D0D": "ITSupported, LLC",
    "000D06": "Compulogic Limited",
    "000D52": "Comart system",
    "000D4A": "Steag ETA-Optik",
    "000D7C": "Codian Ltd",
    "000D6B": "Mita-Teknik A/S",
    "000D71": "boca systems",
    "000D2D": "NCT Deutschland GmbH",
    "000D7E": "Axiowave Networks, Inc.",
    "000CA7": "Metro (Suzhou) Technologies Co., Ltd.",
    "000CA0": "StorCase Technology, Inc.",
    "000CBE": "Innominate Security Technologies AG",
    "000CBD": "Interface Masters, Inc",
    "000CB2": "UNION co., ltd.",
    "000CB9": "LEA",
    "000CB8": "MEDION AG",
    "000C88": "Apache Micro Peripherals, Inc.",
    "000C82": "NETWORK TECHNOLOGIES INC",
    "000C7C": "Internet Information Image Inc.",
    "000D03": "Matrics, Inc.",
    "000CFF": "MRO-TEK Realty Limited",
    "000CFA": "Digital Systems Corp",
    "000CFD": "Hyundai ImageQuest Co.,Ltd.",
    "000CBB": "ISKRAEMECO",
    "000CC0": "Genera Oy",
    "000CA8": "Garuda Networks Corporation",
    "000CEB": "CNMP Networks, Inc.",
    "000CD3": "Prettl Elektronik Radeberg GmbH",
    "000CD7": "Nallatech Ltd",
    "000CD4": "Positron Public Safety Systems inc.",
    "000C99": "HITEL LINK Co.,Ltd",
    "000C89": "AC Electric Vehicles, Ltd.",
    "000CCC": "Aeroscout Ltd.",
    "000CC7": "Intelligent Computer Solutions Inc.",
    "000C7B": "ALPHA PROJECT Co.,Ltd.",
    "000C77": "Life Racing Ltd",
    "000C64": "X2 MSA Group",
    "000C5A": "IBSmm Embedded Electronics Consulting",
    "000C5E": "Calypso Medical",
    "000C61": "AC Tech corporation DBA Advanced Digital",
    "000C5F": "Avtec, Inc.",
    "000C4B": "Cheops Elektronik",
    "000C05": "RPA Reserch Co., Ltd.",
    "000C27": "Sammy Corporation",
    "000C2A": "OCTTEL Communication Co., Ltd.",
    "000C22": "Double D Electronics Ltd",
    "000C1C": "MicroWeb Co., Ltd.",
    "000C69": "National Radio Astronomy Observatory",
    "000C66": "Pronto Networks Inc",
    "000C45": "Animation Technologies Inc.",
    "000C3C": "MediaChorus, Inc.",
    "000C0F": "Techno-One Co., Ltd",
    "000C38": "TelcoBridges Inc.",
    "000C4E": "Winbest Technology CO,LT",
    "000BC4": "BIOTRONIK GmbH & Co",
    "000BAF": "WOOJU COMMUNICATIONS Co,.Ltd",
    "000BB6": "Metalligence Technology Corp.",
    "000BB7": "Micro Systems Co.,Ltd.",
    "000BB3": "RiT technologies Ltd.",
    "000BE6": "Datel Electronics",
    "000BAE": "Vitals System Inc.",
    "000BD9": "General Hydrogen",
    "000BF2": "Chih-Kan Technology Co., Ltd.",
    "000BEB": "Systegra AG",
    "000BEF": "Code Corporation",
    "000BDF": "Shenzhen RouterD Networks Limited",
    "000BF5": "Shanghai Sibo Telecom Technology Co.,Ltd",
    "000BFE": "CASTEL Broadband Limited",
    "000B7D": "SOLOMON EXTREME INTERNATIONAL LTD.",
    "000B94": "Digital Monitoring Products, Inc.",
    "000BAB": "Advantech Technology (CHINA) Co., Ltd.",
    "000BBA": "Harmonic, Inc",
    "000B57": "Silicon Laboratories",
    "000B51": "Micetek International Inc.",
    "000B53": "INITIUM Co., Ltd.",
    "000B6D": "SOLECTRON JAPAN NAKANIIDA",
    "000B62": "ib-mohnen KG",
    "000B64": "Kieback & Peter GmbH & Co KG",
    "000B67": "Topview Technology Corporation",
    "000B48": "sofrel",
    "000AFB": "Ambri Limited",
    "000AFF": "Kilchherr Elektronik AG",
    "000B4A": "Visimetrics (UK) Ltd",
    "000B37": "MANUFACTURE DES MONTRES ROLEX SA",
    "000B2D": "Danfoss Inc.",
    "000B1E": "KAPPA opto-electronics GmbH",
    "000B1C": "SIBCO bv",
    "000AF8": "American Telecare Inc.",
    "000B17": "MKS Instruments",
    "000A9A": "Aiptek International Inc",
    "000A94": "ShangHai cellink CO., LTD",
    "000A97": "SONICblue, Inc.",
    "000A92": "Presonus Corporation",
    "000A64": "Eracom Technologies",
    "000A62": "Crinis Networks, Inc.",
    "000A6A": "SVM Microwaves s.r.o.",
    "000A66": "MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD.",
    "000AA3": "SHIMAFUJI ELECTRIC CO.,LTD.",
    "000AA7": "FEI Electron Optics",
    "000AA6": "Hochiki Corporation",
    "000AC7": "Unication Group",
    "000ABF": "HIROTA SS",
    "000ABC": "Seabridge Ltd.",
    "000A70": "MPLS Forum",
    "000A72": "STEC, INC.",
    "000A71": "Avrio Technologies, Inc",
    "000ADE": "Happy Communication Co., Ltd.",
    "000AE2": "Binatone Electronics International, Ltd",
    "000AD4": "CoreBell Systems Inc.",
    "000ACA": "YOKOYAMA SHOKAI CO.,Ltd.",
    "000ACE": "RADIANTECH, INC.",
    "000AAC": "TerraTec Electronic GmbH",
    "000AB8": "Cisco Systems, Inc",
    "000A85": "PLAT'C2,Inc",
    "000A83": "SALTO SYSTEMS S.L.",
    "000A86": "Lenze",
    "000A50": "REMOTEK CORPORATION",
    "000A58": "Freyer & Siegel Elektronik GmbH & Co. KG",
    "000A4E": "UNITEK Electronics INC.",
    "000A3D": "Elo Sistemas Eletronicos S.A.",
    "000A46": "ARO WELDING TECHNOLOGIES SAS",
    "0009F6": "Shenzhen Eastern Digital Tech Ltd.",
    "000A01": "SOHOware, Inc.",
    "0009EE": "MEIKYO ELECTRIC CO.,LTD",
    "000A31": "HCV Consulting",
    "004252": "RLX Technologies",
    "000A20": "SVA Networks, Inc.",
    "000A24": "Octave Communications",
    "000A19": "Valere Power, Inc.",
    "0009E5": "Hottinger Br\u00fcel & Kjaer GmbH",
    "0009DE": "Samjin Information & Communications Co., Ltd.",
    "0009E0": "XEMICS S.A.",
    "0009CA": "iMaxNetworks(Shenzhen)Limited.",
    "000A11": "ExPet Technologies, Inc",
    "000A0F": "Ilryung Telesys, Inc",
    "000A0C": "Scientific Research Corporation",
    "0009EC": "Daktronics, Inc.",
    "0009E7": "ADC Techonology",
    "0009CF": "iAd GmbH",
    "000A3F": "Data East Corporation",
    "000956": "Network Systems Group, Ltd. (NSG)",
    "000948": "Vista Control Systems, Corp.",
    "000949": "Glyph Technologies Inc.",
    "0009B1": "Kanematsu Electronics, Ltd.",
    "0009B0": "Onkyo Technology K.K.",
    "0009AB": "Netcontrol Oy",
    "00099D": "Haliplex Communications",
    "000968": "TECHNOVENTURE, INC.",
    "000961": "Switchgear and Instrumentation Ltd",
    "000960": "YOZAN Inc.",
    "00099E": "Testech, Inc.",
    "0009C1": "PROCES-DATA A/S",
    "000999": "CP GEORGES RENAULT",
    "000994": "Cronyx Engineering",
    "000988": "Nudian Electron Co., Ltd.",
    "00098E": "ipcas GmbH",
    "000979": "Advanced Television Systems Committee, Inc.",
    "000963": "Dominion Lasercom Inc.",
    "000966": "TRIMBLE EUROPE BV",
    "0009BB": "MathStar, Inc.",
    "0009B9": "Action Imaging Solutions",
    "0009AC": "LANVOICE",
    "00097C": "Cisco Systems, Inc",
    "00097B": "Cisco Systems, Inc",
    "00093C": "Jacques Technologies P/L",
    "000935": "Sandvine Incorporated",
    "000936": "Ipetronik GmbH & Co. KG",
    "000937": "Inventec Appliance Corp",
    "0008FD": "BlueKorea Co., Ltd.",
    "0008F8": "UTC CCS",
    "0008F3": "WANY",
    "0008D9": "Mitadenshi Co.,LTD",
    "0008D5": "Vanguard Networks Solutions, LLC",
    "0008CD": "With-Net Inc",
    "0008CC": "Remotec, Inc.",
    "0008D1": "KAREL INC.",
    "0008DE": "3UP Systems",
    "0008E0": "ATO Technology Ltd.",
    "0008E4": "Envenergy Inc",
    "0008E3": "Cisco Systems, Inc",
    "0008E5": "IDK Corporation",
    "000913": "SystemK Corporation",
    "00090E": "Helix Technology Inc.",
    "000946": "Cluster Labs GmbH",
    "00093F": "Double-Win Enterpirse CO., LTD",
    "00092A": "MYTECS Co.,Ltd.",
    "000925": "VSN Systemen BV",
    "000900": "TMT",
    "00082C": "Homag AG",
    "000821": "Cisco Systems, Inc",
    "000822": "InPro Comm",
    "00081D": "Ipsil, Incorporated",
    "000823": "Texa Corp.",
    "000828": "Koei Engineering Ltd.",
    "00082D": "Indus Teqsite Private Limited",
    "000820": "Cisco Systems, Inc",
    "000824": "Nuance Document Imaging",
    "00086C": "Plasmon LMS",
    "000868": "PurOptix",
    "000869": "Command-e Technology Co.,Ltd.",
    "000862": "NEC Eluminant Technologies, Inc.",
    "000861": "SoftEnergy Co., Ltd.",
    "000891": "Lyan Inc.",
    "000892": "EM Solutions",
    "00088C": "Quanta Network Systems Inc.",
    "00088A": "Minds@Work",
    "000887": "Maschinenfabrik Reinhausen GmbH",
    "000877": "Liebert-Hiross Spa",
    "00087B": "RTX Telecom A/S",
    "000876": "SDSystem",
    "000870": "Rasvia Systems, Inc.",
    "00086E": "Hyglo AB",
    "000814": "TIL Technologies",
    "0008BD": "TEPG-US",
    "0008AE": "PacketFront Network Products AB",
    "00084F": "Qualstar Corporation",
    "000854": "Netronix, Inc.",
    "00085C": "Shanghai Dare Technologies Co. Ltd.",
    "0008A7": "iLogic Inc.",
    "000899": "Netbind, Inc.",
    "0008A0": "Stotz Feinmesstechnik GmbH",
    "0008C3": "Contex A/S",
    "0008BC": "Ilevo AB",
    "0007BD": "Radionet Ltd.",
    "0007C4": "JEAN Co. Ltd.",
    "0007C3": "Thomson",
    "0007D2": "Logopak Systeme GmbH & Co. KG",
    "0007C9": "Technol Seven Co., Ltd.",
    "0007C7": "Synectics Systems Limited",
    "0007BF": "Armillaire Technologies, Inc.",
    "0007BB": "Candera Inc.",
    "0007A5": "Y.D.K Co. Ltd.",
    "00079C": "Golden Electronics Technology Co., Ltd.",
    "0007B6": "Telecom Technology Ltd.",
    "0007B7": "Samurai Ind. Prods Eletronicos Ltda",
    "0007B0": "Office Details, Inc.",
    "000803": "Cos Tron",
    "000805": "Techno-Holon Corporation",
    "0007FD": "LANergy Ltd.",
    "0007FE": "Rigaku Corporation",
    "0007D9": "Splicecom",
    "0007DA": "Neuro Telecom Co., Ltd.",
    "0007CD": "Kumoh Electronic Co, Ltd",
    "0007CF": "Anoto AB",
    "000777": "Motah Ltd.",
    "000780": "Bluegiga Technologies OY",
    "00078E": "Garz & Fricke GmbH",
    "0007E4": "SoftRadio Co., Ltd.",
    "000754": "Xyterra Computing, Inc.",
    "000757": "Topcall International AG",
    "00074C": "Beicom Inc.",
    "000753": "Beijing Qxcomm Technology Co., Ltd.",
    "00074D": "Zebra Technologies Corp.",
    "00072F": "Intransa, Inc.",
    "000732": "AAEON Technology Inc.",
    "000725": "Bematech International Corp.",
    "000723": "ELCON Systemtechnik GmbH",
    "00071D": "Satelsa Sistemas Y Aplicaciones De Telecomunicaciones, S.A.",
    "000720": "Trutzschler GmbH & Co. KG",
    "00076E": "Sinetica Corporation Limited",
    "00076F": "Synoptics Limited",
    "000773": "Ascom Powerline Communications Ltd.",
    "00076C": "Daehanet, Inc.",
    "00075D": "Celleritas Inc.",
    "000765": "Jade Quantum Technologies, Inc.",
    "000737": "Soriya Co. Ltd.",
    "000734": "ONStor, Inc.",
    "00073F": "Woojyun Systec Co., Ltd.",
    "000728": "Neo Telecom",
    "00072C": "Fabricom",
    "00072D": "CNSystems",
    "000724": "Telemax Co., Ltd.",
    "00071B": "CDVI Americas Ltd",
    "000715": "General Research of Electronics, Inc.",
    "0006FD": "Comjet Information Systems Corp.",
    "0006F9": "Mitsui Zosen Systems Research Inc.",
    "0006F1": "Optillion",
    "0006E0": "MAT Co., Ltd.",
    "0006D7": "Cisco Systems, Inc",
    "0006D5": "Diamond Systems Corp.",
    "00D0B9": "MICROTEK INTERNATIONAL, INC.",
    "00D05F": "VALCOM, INC.",
    "000675": "Banderacom, Inc.",
    "00067B": "Toplink C&C Corporation",
    "000670": "Upponetti Oy",
    "0005EA": "Rednix",
    "0006C9": "Technical Marketing Research, Inc.",
    "0006C8": "Sumitomo Metal Micro Devices, Inc.",
    "0006C0": "United Internetworks, Inc.",
    "000698": "egnite GmbH",
    "00069C": "Transmode Systems AB",
    "0006AF": "Xalted Networks",
    "0006A7": "Primarion",
    "0006A9": "Universal Instruments Corp.",
    "00069E": "UNIQA, Inc.",
    "000668": "Vicon Industries Inc.",
    "00066D": "Compuprint S.P.A.",
    "000658": "Helmut Fischer GmbH Institut f\u00fcr Elektronik und Messtechnik",
    "000649": "3M Deutschland GmbH",
    "000643": "SONO Computer Co., Ltd.",
    "00063E": "Opthos Inc.",
    "00066F": "Korea Data Systems",
    "000686": "ZARDCOM Co., Ltd.",
    "000689": "yLez Technologies Pte Ltd",
    "000681": "Goepel Electronic GmbH",
    "0006B1": "Sonicwall",
    "0006AD": "KB Electronics Ltd.",
    "00063B": "Arcturus Networks Inc.",
    "000626": "MWE GmbH",
    "000606": "RapidWAN, Inc.",
    "000603": "Baker Hughes Inc.",
    "000607": "Omni Directional Control Technology Inc.",
    "0005F7": "Analog Devices, Inc.",
    "000613": "Kawasaki Microelectronics Incorporated",
    "0005D3": "eProduction Solutions, Inc.",
    "000604": "@Track Communications, Inc.",
    "00059C": "Kleinknecht GmbH, Ing. B\u00fcro",
    "0005A2": "CELOX Networks",
    "0005AA": "Moore Industries International Inc.",
    "0005AE": "Mediaport USA",
    "0005B0": "Korea Computer Technology Co., Ltd.",
    "0005B2": "Medison Co., Ltd.",
    "000618": "DigiPower Manufacturing Inc.",
    "000620": "Serial System Ltd.",
    "000617": "Redswitch Inc.",
    "0005C8": "VERYTECH",
    "0005CB": "ROIS Technologies, Inc.",
    "0005CD": "D&M Holdings Inc.",
    "0005E7": "Netrake an AudioCodes Company",
    "0005F4": "System Base Co., Ltd.",
    "0005E1": "Trellis Photonics, Ltd.",
    "0005E2": "Creativ Network Technologies",
    "00054D": "Brans Technologies, Inc.",
    "000547": "Starent Networks",
    "00054E": "Philips",
    "000546": "KDDI Network & Solultions Inc.",
    "000540": "FAST Corporation",
    "00053C": "XIRCOM",
    "000596": "Genotech Co., Ltd.",
    "000579": "Universal Control Solution Corp.",
    "00052E": "Cinta Networks",
    "00052F": "Leviton Network Solutions",
    "00053B": "Harbour Networks Ltd., Co. Beijing",
    "000528": "New Focus, Inc.",
    "000575": "CDS-Electronics BV",
    "00056B": "C.P. Technology Co., Ltd.",
    "000560": "LEADER COMM.CO., LTD",
    "00055E": "Cisco Systems, Inc",
    "000544": "Valley Technologies, Inc.",
    "000597": "Eagle Traffic Control Systems",
    "0005B5": "Broadcom Technologies",
    "00058C": "Opentech Inc.",
    "000556": "360 Systems",
    "000559": "Intracom S.A.",
    "000551": "F & S Elektronik Systeme GmbH",
    "000524": "BTL System (HK) Limited",
    "000522": "LEA*D Corporation, Inc.",
    "000520": "Smartronix, Inc.",
    "00057F": "Acqis Technology",
    "000573": "Cisco Systems, Inc",
    "0004BE": "OptXCon, Inc.",
    "0004C1": "Cisco Systems, Inc",
    "0004C4": "Audiotonix Group Limited",
    "0004B7": "AMB i.t. Holding",
    "0004B9": "S.I. Soubou, Inc.",
    "0004BB": "Bardac Corporation",
    "0004BC": "Giantec, Inc.",
    "0004AF": "Digital Fountain, Inc.",
    "0004D6": "Takagi Industrial Co., Ltd.",
    "0004D1": "Drew Technologies, Inc.",
    "0004D0": "Softlink s.r.o.",
    "008087": "OKI ELECTRIC INDUSTRY CO., LTD",
    "0004D9": "Titan Electronics, Inc.",
    "0004D8": "IPWireless, Inc.",
    "0004B2": "ESSEGI SRL",
    "0004B4": "CIAC",
    "0004A9": "SandStream Technologies, Inc.",
    "0004EB": "Paxonet Communications, Inc.",
    "0004EF": "Polestar Corp.",
    "000514": "KDT Systems Co., Ltd.",
    "00050B": "SICOM Systems, Inc.",
    "000505": "Systems Integration Solutions, Inc.",
    "0004FE": "Pelago Networks",
    "000518": "Jupiters Technology",
    "000461": "EPOX Computer Co., Ltd.",
    "000462": "DAKOS Data & Communication Co., Ltd.",
    "00045F": "Avalue Technology, Inc.",
    "000456": "Cambium Networks Limited",
    "000446": "CYZENTECH Co., Ltd.",
    "00044A": "iPolicy Networks, Inc.",
    "000440": "cyberPIXIE, Inc.",
    "00043C": "SONOS Co., Ltd.",
    "00049C": "Surgient Networks, Inc.",
    "00049D": "Ipanema Technologies",
    "000460": "Knilink Technology, Inc.",
    "00048F": "TD Systems Corporation",
    "000419": "Fibercycle Networks, Inc.",
    "00041C": "ipDialog, Inc.",
    "000418": "Teltronic S.A.U.",
    "000370": "NXTV, Inc.",
    "000468": "Vivity, Inc.",
    "00046F": "Digitel S/A Industria Eletronica",
    "000469": "Innocom, Inc.",
    "000402": "Nexsan Technologies, Ltd.",
    "0003F7": "Plast-Control GmbH",
    "0003FC": "Intertex Data AB",
    "000495": "Tejas Networks India Limited",
    "000487": "Cogency Semiconductor, Inc.",
    "00042C": "Minet, Inc.",
    "00042A": "Wireless Networks, Inc.",
    "000458": "Fusion X Co., Ltd.",
    "00044F": "Schubert System Elektronik Gmbh",
    "0003E1": "Winmate Communication, Inc.",
    "0003E4": "Cisco Systems, Inc",
    "0003DC": "Lexar Media, Inc.",
    "0003D8": "iMPath Networks, Inc.",
    "0003D5": "Advanced Communications Co., Ltd.",
    "0003D6": "RADVision, Ltd.",
    "0003C8": "CML Emergency Services",
    "0003C3": "Micronik Multimedia",
    "0003C0": "RFTNC Co., Ltd.",
    "0003BD": "OmniCluster Technologies, Inc.",
    "0003B8": "NetKit Solutions, LLC",
    "0003B7": "ZACCESS Systems",
    "0003B0": "Xsense Technology Corp.",
    "0003AA": "Watlow",
    "0003A8": "IDOT Computers, Inc.",
    "0003EF": "Oneline AG",
    "0003F1": "Cicada Semiconductor, Inc.",
    "0003ED": "Shinkawa Electric Co., Ltd.",
    "0003A0": "Cisco Systems, Inc",
    "0003A2": "Catapult Communications",
    "00039C": "OptiMight Communications, Inc.",
    "0003D4": "Alloptic, Inc.",
    "0003CE": "ETEN Technologies, Inc.",
    "000385": "Actelis Networks, Inc.",
    "000369": "Nippon Antenna Co., Ltd.",
    "000373": "Aselsan A.S",
    "000377": "Gigabit Wireless",
    "000355": "TeraBeam Internet Systems",
    "000354": "Fiber Logic Communications",
    "000350": "BTICINO SPA",
    "00034E": "Pos Data Company, Ltd.",
    "000348": "Norscan Instruments, Ltd.",
    "000346": "Hitachi Kokusai Electric, Inc.",
    "000344": "Tietech.Co., Ltd.",
    "0002E6": "Gould Instrument Systems, Inc.",
    "0002E4": "JC HYUN Systems, Inc.",
    "0002DE": "Astrodesign, Inc.",
    "0002E2": "NDC Infared Engineering",
    "0002DC": "Fujitsu General Limited",
    "0002E1": "Integrated Network Corporation",
    "0002D7": "EMPEG Ltd",
    "0002F8": "SEAKR Engineering, Inc.",
    "0002F7": "ARM",
    "00D024": "Cognex Corporation",
    "0002F1": "Pinetron Co., Ltd.",
    "0002ED": "DXO Telecom Co., Ltd.",
    "0002EC": "Maschoff Design Engineering",
    "0002D3": "NetBotz, Inc.",
    "0002DA": "ExiO Communications, Inc.",
    "0002D4": "PDA Peripherals, Inc.",
    "0002D6": "NICE Systems",
    "000343": "Martin Professional A/S",
    "000335": "Mirae Technology",
    "000336": "Zetes Technologies",
    "000337": "Vaone, Inc.",
    "000365": "Kira Information & Communications, Ltd.",
    "00033B": "TAMI Tech Co., Ltd.",
    "00032D": "IBASE Technology, Inc.",
    "00032F": "Global Sun Technology, Inc.",
    "00B052": "Atheros Communications",
    "000314": "Teleware Network Systems",
    "00025C": "SCI Systems (Kunshan) Co., Ltd.",
    "000087": "HITACHI, LTD.",
    "000258": "Flying Packets Communications",
    "000255": "IBM Corp",
    "000252": "Carrier Corporation",
    "00024E": "Datacard Group",
    "000242": "Videoframe Systems",
    "000244": "SURECOM Technology Co.",
    "00023E": "Selta Telematica S.p.a",
    "000241": "Amer.com",
    "00027D": "Cisco Systems, Inc",
    "00027C": "Trilithic, Inc.",
    "000275": "SMART Technologies, Inc.",
    "000269": "Nadatel Co., Ltd",
    "000264": "AudioRamp.com",
    "00025F": "Nortel Networks",
    "000299": "Apex, Inc.",
    "000298": "Broadframe Corporation",
    "000297": "C-COR.net",
    "000291": "Open Network Co., Ltd.",
    "0002C6": "Data Track Technology PLC",
    "0002C1": "Innovative Electronic Designs, Inc.",
    "0002C0": "Bencent Tzeng Industry Co., Ltd.",
    "00017A": "Chengdu Maipu Electric Industrial Co., Ltd.",
    "000238": "Serome Technology, Inc.",
    "000229": "Adtec Corporation",
    "00028B": "VDSL Systems OY",
    "00028C": "Micrel-Synergy Semiconductor",
    "00028D": "Movita Technologies, Inc.",
    "0002A6": "Effinet Systems Co., Ltd.",
    "0001E5": "Supernet, Inc.",
    "0001D4": "Leisure Time, Inc.",
    "0001DD": "Avail Networks",
    "0001D5": "HAEDONG INFO & COMM CO., LTD",
    "0001D7": "F5 Networks, Inc.",
    "0001DE": "Trango Systems, Inc.",
    "0001DC": "Activetelco",
    "000207": "VisionGlobal Network Corp.",
    "000208": "Unify Networks, Inc.",
    "000204": "Bodmann Industries Elektronik GmbH",
    "0001F9": "TeraGlobal Communications Corp.",
    "0001FB": "DoTop Technology, Inc.",
    "0001BF": "Teleforce Co., Ltd.",
    "0001B4": "Wayport, Inc.",
    "0001BA": "IC-Net, Inc.",
    "0001B0": "Fulltek Technology Co., Ltd.",
    "0001A9": "BMW AG",
    "0001AA": "Airspan Communications, Ltd.",
    "00019E": "ESS Technology, Inc.",
    "000211": "Nature Worldwide Technology Corp.",
    "000212": "SierraCom",
    "000217": "Cisco Systems, Inc",
    "00022D": "Agere Systems",
    "000226": "XESystems, Inc.",
    "000225": "One Stop Systems",
    "000220": "CANON FINETECH INC.",
    "00020D": "Micronpc.com",
    "0001CF": "Alpha Data Parallel Systems, Ltd.",
    "0001D0": "VitalPoint, Inc.",
    "0001C7": "Cisco Systems, Inc",
    "00017D": "ThermoQuest",
    "000181": "Nortel Networks",
    "000194": "Capital Equipment Corporation",
    "000198": "Darim Vision",
    "0001F8": "TEXIO TECHNOLOGY CORPORATION",
    "0001E8": "Force10 Networks, Inc.",
    "0001E9": "Litton Marine Systems B.V.",
    "0001BE": "Gigalink Co., Ltd.",
    "0001B1": "General Bandwidth",
    "00010A": "CIS TECHNOLOGY INC.",
    "000162": "Cygnet Technologies, Inc.",
    "000167": "HIOKI E.E. CORPORATION",
    "000168": "VITANA CORPORATION",
    "00015F": "DIGITAL DESIGN GmbH",
    "000164": "Cisco Systems, Inc",
    "00B04A": "Cisco Systems, Inc",
    "00B048": "Marconi Communications Inc.",
    "00B0CE": "Viveris Technologies",
    "00B01C": "Westport Technologies",
    "000119": "RTUnet (Australia)",
    "000122": "Trend Communications, Ltd.",
    "00011A": "Hoffmann und Burmeister GbR",
    "000124": "Acer Incorporated",
    "000142": "Cisco Systems, Inc",
    "00012C": "Aravox Technologies, Inc.",
    "000121": "WatchGuard Technologies, Inc.",
    "000129": "DFI Inc.",
    "000154": "G3M Corporation",
    "000152": "CHROMATEK INC.",
    "000150": "GILAT COMMUNICATIONS, LTD.",
    "000151": "Ensemble Communications",
    "000115": "EXTRATECH CORPORATION",
    "000101": "Private",
    "00010D": "Teledyne DALSA Inc.",
    "000105": "Beckhoff Automation GmbH",
    "00B017": "InfoGear Technology Corp.",
    "00012B": "TELENET Co., Ltd.",
    "00013F": "Neighbor World Co., Ltd.",
    "000185": "Hitachi Aloka Medical, Ltd.",
    "0001A2": "Logical Co., Ltd.",
    "00303A": "MAATEL",
    "00302C": "SYLANTRO SYSTEMS CORPORATION",
    "0030DF": "KB/TEL TELECOMUNICACIONES",
    "00304E": "BUSTEC PRODUCTION LTD.",
    "003063": "SANTERA SYSTEMS, INC.",
    "003030": "HARMONIX CORPORATION",
    "0030A3": "Cisco Systems, Inc",
    "0030FD": "INTEGRATED SYSTEMS DESIGN",
    "0030B9": "ECTEL",
    "00307D": "GRE AMERICA, INC.",
    "003021": "HSING TECH. ENTERPRISE CO.,LTD",
    "00B02D": "ViaGate Technologies, Inc.",
    "0030EE": "DSG Technology, Inc.",
    "00309E": "WORKBIT CORPORATION.",
    "0030DE": "WAGO Kontakttechnik GmbH",
    "00303E": "Radcom Ltd.",
    "0030D7": "Innovative Systems, L.L.C.",
    "0030A4": "Woodwind Communications System",
    "00303B": "PowerCom Technology",
    "0030BC": "Optronic AG",
    "0030B8": "RiverDelta Networks",
    "003071": "Cisco Systems, Inc",
    "003088": "Ericsson",
    "0030B5": "Tadiran Microwave Networks",
    "003020": "TSI, Inc..",
    "0030EF": "NEON TECHNOLOGY, INC.",
    "003095": "Procomp Informatics, Ltd.",
    "0030CA": "Discovery Com",
    "0030CE": "Zaffire",
    "003096": "Cisco Systems, Inc",
    "003039": "SOFTBOOK PRESS",
    "00301B": "SHUTTLE, INC.",
    "00307B": "Cisco Systems, Inc",
    "00D03B": "VISION PRODUCTS PTY. LTD.",
    "00D0B3": "DRS Technologies Canada Ltd",
    "00D000": "FERRAN SCIENTIFIC, INC.",
    "00D0D0": "ZHONGXING TELECOM LTD.",
    "00D053": "CONNECTED SYSTEMS",
    "00D097": "Cisco Systems, Inc",
    "00D08E": "Grass Valley, A Belden Brand",
    "00D056": "SOMAT CORPORATION",
    "00D0E0": "DOOIN ELECTRONICS CO.",
    "00D033": "DALIAN DAXIAN NETWORK",
    "00D0D6": "AETHRA TELECOMUNICAZIONI",
    "00D063": "Cisco Systems, Inc",
    "00D0F8": "FUJIAN STAR TERMINAL",
    "00D0ED": "XIOX",
    "003027": "KERBANGO, INC.",
    "003033": "ORIENT TELECOM CO., LTD.",
    "0030DD": "INDIGITA CORPORATION",
    "003099": "BOENIG UND KALLENBACH OHG",
    "0030F2": "Cisco Systems, Inc",
    "003051": "ORBIT AVIONIC & COMMUNICATION",
    "003008": "AVIO DIGITAL, INC.",
    "00301D": "SKYSTREAM, INC.",
    "0030BA": "AC&T SYSTEM CO., LTD.",
    "00D030": "Safetran Systems Corp",
    "00D047": "XN TECHNOLOGIES",
    "00D055": "KATHREIN-WERKE KG",
    "00D0B0": "BITSWITCH LTD.",
    "00302A": "SOUTHERN INFORMATION",
    "0030E1": "Network Equipment Technologies, Inc.",
    "003001": "SMP",
    "00D021": "REGENT ELECTRONICS CORP.",
    "00D09F": "NOVTEK TEST SYSTEMS",
    "00D0FE": "ASTRAL POINT",
    "00D0D4": "V-BITS, INC.",
    "00D0E7": "VCON TELECOMMUNICATION LTD.",
    "00D084": "NEXCOMM SYSTEMS, INC.",
    "00D099": "Elcard Wireless Systems Oy",
    "00D01B": "MIMAKI ENGINEERING CO., LTD.",
    "00D083": "INVERTEX, INC.",
    "00D0BA": "Cisco Systems, Inc",
    "00D0E4": "Cisco Systems, Inc",
    "00D08B": "ADVA Optical Networking Ltd.",
    "00D05A": "SYMBIONICS, LTD.",
    "00D079": "Cisco Systems, Inc",
    "00D098": "Photon Dynamics Canada Inc.",
    "00D0BE": "EMUTEC INC.",
    "00D0F4": "CARINTHIAN TECH INSTITUTE",
    "00D07D": "COSINE COMMUNICATIONS",
    "00D0AF": "CUTLER-HAMMER, INC.",
    "00D052": "ASCEND COMMUNICATIONS, INC.",
    "00D0AD": "TL INDUSTRIES",
    "00D0A4": "ALANTRO COMMUNICATIONS",
    "00D00D": "MICROMERITICS INSTRUMENT",
    "00D04A": "PRESENCE TECHNOLOGY GMBH",
    "00D074": "TAQUA SYSTEMS, INC.",
    "00D0C7": "PATHWAY, INC.",
    "00D07A": "AMAQUEST COMPUTER CORP.",
    "00D0B8": "Iomega Corporation",
    "00509D": "THE INDUSTREE B.V.",
    "00D054": "SAS INSTITUTE INC.",
    "00D009": "HSING TECH. ENTERPRISE CO. LTD",
    "00D092": "GLENAYRE WESTERN MULTIPLEX",
    "005058": "Sangoma Technologies",
    "0050FE": "PCTVnet ASA",
    "0050AB": "NALTEC, Inc.",
    "005037": "KOGA ELECTRONICS CO.",
    "005006": "TAC AB",
    "005009": "PHILIPS BROADBAND NETWORKS",
    "005030": "FUTURE PLUS SYSTEMS",
    "0050D6": "ATLAS COPCO TOOLS AB",
    "005082": "FORESSON CORPORATION",
    "0050DF": "AirFiber, Inc.",
    "0050AA": "KONICA MINOLTA HOLDINGS, INC.",
    "005038": "DAIN TELECOM CO., LTD.",
    "00509E": "Les Technologies SoftAcoustik Inc.",
    "005071": "AIWA CO., LTD.",
    "00505F": "BRAND INNOVATORS",
    "0050B4": "SATCHWELL CONTROL SYSTEMS, LTD",
    "0050D0": "MINERVA SYSTEMS",
    "0050D8": "UNICORN COMPUTER CORP.",
    "0050A8": "OpenCon Systems, Inc.",
    "00509C": "BETA RESEARCH",
    "005023": "PG DESIGN ELECTRONICS, INC.",
    "0050B1": "GIDDINGS & LEWIS",
    "0050B7": "BOSER TECHNOLOGY CO., LTD.",
    "005078": "MEGATON HOUSE, LTD.",
    "005002": "OMNISEC AG",
    "00506A": "EDEVA, INC.",
    "0050C5": "ADS Technologies, Inc",
    "00508E": "OPTIMATION, INC.",
    "005028": "AVAL COMMUNICATIONS",
    "00502F": "TollBridge Technologies, Inc.",
    "005074": "ADVANCED HI-TECH CORP.",
    "00500A": "IRIS TECHNOLOGIES, INC.",
    "005070": "CHAINTECH COMPUTER CO., LTD.",
    "00503F": "ANCHOR GAMES",
    "005032": "PICAZO COMMUNICATIONS, INC.",
    "00504D": "Tokyo Electron Device Limited",
    "00506D": "VIDEOJET SYSTEMS",
    "009008": "HanA Systems Inc.",
    "009088": "BAXALL SECURITY LTD.",
    "00906C": "Sartorius Hamburg GmbH",
    "0090A4": "ALTIGA NETWORKS",
    "0090F9": "Imagine Communications",
    "009089": "SOFTCOM MICROSYSTEMS, INC.",
    "0090EE": "PERSONAL COMMUNICATIONS TECHNOLOGIES",
    "009080": "NOT LIMITED, INC.",
    "0090E8": "MOXA TECHNOLOGIES CORP., LTD.",
    "0090A1": "Flying Pig Systems/High End Systems Inc.",
    "009079": "ClearOne, Inc.",
    "00909A": "ONE WORLD SYSTEMS, INC.",
    "0090C2": "JK microsystems, Inc.",
    "0090AC": "OPTIVISION, INC.",
    "0050B2": "BRODEL GmbH",
    "009076": "FMT AIRCRAFT GATE SUPPORT SYSTEMS AB",
    "009017": "Zypcom, Inc",
    "009049": "ENTRIDIA CORPORATION",
    "009043": "Tattile SRL ",
    "0090AD": "ASPECT ELECTRONICS, INC.",
    "0090E6": "ALi Corporation",
    "009070": "NEO NETWORKS, INC.",
    "009030": "HONEYWELL-DATING",
    "0090C8": "WAVERIDER COMMUNICATIONS (CANADA) INC.",
    "00901B": "DIGITAL CONTROLS",
    "00904E": "DELEM BV",
    "0090ED": "CENTRAL SYSTEM RESEARCH CO., LTD.",
    "009075": "NEC DO BRASIL S.A.",
    "009001": "NISHIMU ELECTRONICS INDUSTRIES CO., LTD.",
    "00901E": "Selesta Ingegneria S.p.A.",
    "0010EC": "Embedded Planet",
    "001059": "DIABLO RESEARCH CO. LLC",
    "001069": "HELIOSS COMMUNICATIONS, INC.",
    "0010BF": "InterAir Wireless",
    "001036": "INTER-TEL INTEGRATED SYSTEMS",
    "001026": "ACCELERATED NETWORKS, INC.",
    "0010AC": "IMCI TECHNOLOGIES",
    "0010FC": "BROADBAND NETWORKS, INC.",
    "001031": "OBJECTIVE COMMUNICATIONS, INC.",
    "00106D": "Axxcelera Broadband Wireless",
    "0010D0": "WITCOM, LTD.",
    "001093": "CMS COMPUTERS, LTD.",
    "00108F": "RAPTOR SYSTEMS",
    "001063": "STARGUIDE DIGITAL NETWORKS",
    "001023": "Network Equipment Technologies",
    "00102B": "UMAX DATA SYSTEMS, INC.",
    "00108D": "Johnson Controls, Inc.",
    "0090F7": "NBASE COMMUNICATIONS LTD.",
    "009012": "GLOBESPAN SEMICONDUCTOR, INC.",
    "00908A": "BAYLY COMMUNICATIONS, INC.",
    "00900E": "HANDLINK TECHNOLOGIES, INC.",
    "001045": "Nortel Networks",
    "00107D": "AURORA COMMUNICATIONS, LTD.",
    "001039": "Vectron Systems AG",
    "0010B6": "ENTRATA COMMUNICATIONS CORP.",
    "0090E4": "NEC AMERICA, INC.",
    "0090C1": "Peco II, Inc.",
    "009040": "Siemens Network Convergence LLC",
    "009047": "GIGA FAST E. LTD.",
    "0090E1": "TELENA S.P.A.",
    "0090CB": "Wireless OnLine, Inc.",
    "0090B7": "DIGITAL LIGHTWAVE, INC.",
    "0090A0": "8X8 INC.",
    "009032": "PELCOMBE GROUP LTD.",
    "00104C": "Teledyne LeCroy, Inc",
    "0010CC": "CLP COMPUTER LOGISTIK PLANUNG GmbH",
    "001030": "EION Inc.",
    "001062": "NX SERVER, ILNC.",
    "0010F0": "RITTAL-WERK RUDOLF LOH GmbH & Co.",
    "001001": "Citel",
    "00105C": "QUANTUM DESIGNS (H.K.) LTD.",
    "001075": "Segate Technology LLC",
    "0010B1": "FOR-A CO., LTD.",
    "0010EE": "CTI PRODUCTS, INC.",
    "001041": "BRISTOL BABCOCK, INC.",
    "0010AA": "MEDIA4, INC.",
    "0010E8": "TELOCITY, INCORPORATED",
    "0010A2": "TNS",
    "001000": "CABLE TELEVISION LABORATORIES, INC.",
    "001009": "HORANET",
    "0010F8": "TEXIO TECHNOLOGY CORPORATION",
    "0010C0": "ARMA, Inc.",
    "001002": "ACTIA",
    "00105B": "NET INSIGHT AB",
    "0010EB": "SELSIUS SYSTEMS, INC.",
    "00101D": "WINBOND ELECTRONICS CORP.",
    "00109F": "PAVO, INC.",
    "001084": "K-BOT COMMUNICATIONS",
    "0010F9": "UNIQUE SYSTEMS, INC.",
    "0010CF": "FIBERLANE COMMUNICATIONS",
    "0010A4": "XIRCOM",
    "0010F1": "I-O CORPORATION",
    "001057": "Rebel.com, Inc.",
    "001065": "RADYNE CORPORATION",
    "001066": "ADVANCED CONTROL SYSTEMS, INC.",
    "00E09F": "PIXEL VISION",
    "00E0CC": "HERO SYSTEMS, LTD.",
    "00E080": "CONTROL RESOURCES CORPORATION",
    "00E004": "PMC-SIERRA, INC.",
    "00E03B": "PROMINET CORPORATION",
    "00E0F5": "TELES AG",
    "00E0D7": "SUNSHINE ELECTRONICS, INC.",
    "00E095": "ADVANCED-VISION TECHNOLGIES CORP.",
    "00E00E": "AVALON IMAGING SYSTEMS, INC.",
    "00E048": "SDL COMMUNICATIONS, INC.",
    "00E0CB": "RESON, INC.",
    "00E0C8": "VIRTUAL ACCESS, LTD.",
    "00E006": "SILICON INTEGRATED SYS. CORP.",
    "00E0AC": "MIDSCO, INC.",
    "00E0FA": "TRL TECHNOLOGY, LTD.",
    "00E02C": "AST COMPUTER",
    "00E067": "eac AUTOMATION-CONSULTING GmbH",
    "00E008": "AMAZING CONTROLS! INC.",
    "00E0AE": "XAQTI CORPORATION",
    "00E0E0": "SI ELECTRONICS, LTD.",
    "00E003": "NOKIA WIRELESS BUSINESS COMMUN",
    "00E0F3": "WebSprint Communications, Inc.",
    "08BBCC": "AK-NORD EDV VERTRIEBSGES. mbH",
    "00E0DB": "ViaVideo Communications, Inc.",
    "00E0A6": "TELOGY NETWORKS, INC.",
    "00604B": "Safe-com GmbH & Co. KG",
    "00606B": "Synclayer Inc.",
    "00603B": "AMTEC spa",
    "00E039": "PARADYNE CORP.",
    "00600B": "LOGWARE GmbH",
    "00E0EF": "DIONEX",
    "00E051": "TALX CORPORATION",
    "00E02D": "InnoMediaLogic, Inc.",
    "00E035": "Artesyn Embedded Technologies",
    "00E090": "BECKMAN LAB. AUTOMATION DIV.",
    "00E085": "GLOBAL MAINTECH, INC.",
    "00E0A0": "WILTRON CO.",
    "00E0C7": "EUROTECH SRL",
    "00E0F1": "THAT CORPORATION",
    "00E0AF": "GENERAL DYNAMICS INFORMATION SYSTEMS",
    "00E054": "KODAI HITEC CO., LTD.",
    "00E0B9": "BYAS SYSTEMS",
    "00E023": "TELRAD",
    "00E02A": "TANDBERG TELEVISION AS",
    "00E04E": "SANYO DENKI CO., LTD.",
    "00E012": "PLUTO TECHNOLOGIES INTERNATIONAL INC.",
    "00E04C": "REALTEK SEMICONDUCTOR CORP.",
    "0060B5": "KEBA GmbH",
    "006001": "InnoSys, Inc.",
    "0060FE": "LYNX SYSTEM DEVELOPERS, INC.",
    "0060BD": "Enginuity Communications",
    "000800": "MULTITECH SYSTEMS, INC.",
    "00E0B5": "ARDENT COMMUNICATIONS CORP.",
    "00E068": "MERRIMAC SYSTEMS INC.",
    "00E049": "MICROWI ELECTRONIC GmbH",
    "00E050": "EXECUTONE INFORMATION SYSTEMS, INC.",
    "00E0BE": "GENROCO INTERNATIONAL, INC.",
    "00E0B6": "Entrada Networks",
    "00E0F4": "INSIDE Technology A/S",
    "00607E": "GIGALABS, INC.",
    "0060D5": "AMADA MIYACHI Co., Ltd",
    "00603F": "PATAPSCO DESIGNS",
    "0060D2": "LUCENT TECHNOLOGIES TAIWAN TELECOMMUNICATIONS CO., LTD.",
    "0060CD": "VideoServer, Inc.",
    "0060AA": "INTELLIGENT DEVICES INC. (IDI)",
    "006065": "B&R Industrial Automation GmbH ",
    "00605D": "SCANIVALVE CORP.",
    "00602C": "LINX Data Terminals, Inc.",
    "0060AC": "RESILIENCE CORPORATION",
    "00604E": "CYCLE COMPUTER CORPORATION, INC.",
    "006014": "EDEC CO., LTD.",
    "0060E1": "ORCKIT COMMUNICATIONS LTD.",
    "0060E6": "SHOMITI SYSTEMS INCORPORATED",
    "0060FF": "QuVis, Inc.",
    "006067": "ACER NETXUS INC.",
    "00609F": "PHAST CORPORATION",
    "006042": "TKS (USA), INC.",
    "006079": "Mainstream Data, Inc.",
    "00609A": "NJK TECHNO CO.",
    "00602B": "PEAK AUDIO",
    "0060F1": "EXP COMPUTER, INC.",
    "0005A8": "WYLE ELECTRONICS",
    "006040": "NETRO CORP.",
    "0060CC": "EMTRAK, INCORPORATED",
    "00606F": "CLARION CORPORATION OF AMERICA",
    "006090": "Artiza Networks Inc",
    "006025": "ACTIVE IMAGING PLC",
    "0060A7": "MICROSENS GmbH & CO. KG",
    "006030": "VILLAGE TRONIC ENTWICKLUNG",
    "00A00D": "THE PANDA PROJECT",
    "00A0E9": "ELECTRONIC RETAILING SYSTEMS INTERNATIONAL",
    "00A0BE": "INTEGRATED CIRCUIT SYSTEMS, INC. COMMUNICATIONS GROUP",
    "00A010": "SYSLOGIC DATENTECHNIK AG",
    "00A016": "MICROPOLIS CORP.",
    "00A048": "QUESTECH, LTD.",
    "00A0BD": "I-TECH CORP.",
    "00A059": "HAMILTON HALLMARK",
    "00A039": "ROSS TECHNOLOGY, INC.",
    "00A0AD": "MARCONI SPA",
    "0060E5": "FUJI AUTOMATION CO., LTD.",
    "00605E": "LIBERTY TECHNOLOGY NETWORKING",
    "0060C6": "DCS AG",
    "00601E": "SOFTLAB, INC.",
    "00600D": "Digital Logic GmbH",
    "00604A": "SAIC IDEAS GROUP",
    "00A0FC": "IMAGE SCIENCES, INC.",
    "00A09C": "Xyplex, Inc.",
    "00A0D6": "SBE, Inc.",
    "00A02E": "BRAND COMMUNICATIONS, LTD.",
    "00A0AF": "WMS INDUSTRIES",
    "00A057": "LANCOM Systems GmbH",
    "00A09F": "COMMVISION CORP.",
    "00A06E": "AUSTRON, INC.",
    "00A0BA": "PATTON ELECTRONICS CO.",
    "00A0B5": "3H TECHNOLOGY",
    "00A04D": "EDA INSTRUMENTS, INC.",
    "00A086": "AMBER WAVE SYSTEMS, INC.",
    "00A01E": "EST CORPORATION",
    "00A062": "AES PRODATA",
    "00A0AE": "NUCOM SYSTEMS, INC.",
    "00A076": "CARDWARE LAB, INC.",
    "00A0A1": "EPIC DATA INC.",
    "00A0CA": "FUJITSU DENSO LTD.",
    "00A022": "CENTRE FOR DEVELOPMENT OF ADVANCED COMPUTING",
    "00A0C0": "DIGITAL LINK CORP.",
    "00A044": "NTT IT CO., LTD.",
    "00A011": "MUTOH INDUSTRIES LTD.",
    "00A0B6": "SANRITZ AUTOMATION CO., LTD.",
    "00A079": "ALPS ELECTRIC (USA), INC.",
    "00A08D": "JACOMO CORPORATION",
    "00A08E": "Check Point Software Technologies",
    "00A0F5": "RADGUARD LTD.",
    "00A084": "Dataplex Pty Ltd",
    "00A049": "DIGITECH INDUSTRIES, INC.",
    "00A09D": "JOHNATHON FREEMAN TECHNOLOGIES",
    "00A0F9": "BINTEC COMMUNICATIONS GMBH",
    "00A003": "Siemens Switzerland Ltd., I B T HVP",
    "000267": "NODE RUNNER, INC.",
    "0020B1": "COMTECH RESEARCH INC.",
    "002032": "ALCATEL TAISEL",
    "0020E9": "DANTEL",
    "002022": "NMS Communications",
    "0020AE": "ORNET DATA COMMUNICATION TECH.",
    "0020AA": "Ericsson Television Limited",
    "00A06B": "DMS DORSCH MIKROSYSTEM GMBH",
    "00A030": "CAPTOR NV/SA",
    "00A0DE": "YAMAHA CORPORATION",
    "00A0F8": "Zebra Technologies Inc",
    "002005": "SIMPLE TECHNOLOGY",
    "0020FA": "GDE SYSTEMS, INC.",
    "002038": "VME MICROSYSTEMS INTERNATIONAL CORPORATION",
    "0020A3": "Harmonic, Inc",
    "002059": "MIRO COMPUTER PRODUCTS AG",
    "002080": "SYNERGY (UK) LTD.",
    "002018": "CIS TECHNOLOGY INC.",
    "002098": "HECTRONIC AB",
    "0020FD": "ITV TECHNOLOGIES, INC.",
    "002034": "ROTEC INDUSTRIEAUTOMATION GMBH",
    "002079": "MIKRON GMBH",
    "0020F5": "PANDATEL AG",
    "002074": "SUNGWOON SYSTEMS",
    "00203C": "EUROTIME AB",
    "002028": "WEST EGG SYSTEMS, INC.",
    "002068": "ISDYNE",
    "0020C8": "LARSCOM INCORPORATED",
    "00C023": "TUTANKHAMON ELECTRONICS",
    "00C08B": "RISQ MODULAR SYSTEMS, INC.",
    "0020C4": "INET,INC.",
    "00200E": "NSSLGlobal Technologies AS",
    "0020E7": "B&W NUCLEAR SERVICE COMPANY",
    "00209D": "LIPPERT AUTOMATIONSTECHNIK",
    "00209C": "PRIMARY ACCESS CORP.",
    "00206D": "DATA RACE, INC.",
    "00203A": "DIGITAL BI0METRICS INC.",
    "0020A4": "MULTIPOINT NETWORKS",
    "002051": "Verilink Corporation",
    "00203B": "WISDM LTD.",
    "0020BA": "CENTER FOR HIGH PERFORMANCE",
    "00C0C3": "ACUSON COMPUTED SONOGRAPHY",
    "00C0B3": "COMSTAT DATACOMM CORPORATION",
    "00C0E5": "GESPAC, S.A.",
    "00C04D": "MITEC, INC.",
    "0020DC": "DENSITRON TAIWAN LTD.",
    "002048": "Marconi Communications",
    "00200C": "ADASTRA SYSTEMS CORP.",
    "002011": "CANOPUS CO., LTD.",
    "0020F0": "UNIVERSAL MICROELECTRONICS CO.",
    "002089": "T3PLUS NETWORKING, INC.",
    "002061": "GarrettCom, Inc.",
    "00C0A1": "TOKYO DENSHI SEKEI CO.",
    "00C02E": "NETWIZ",
    "00C047": "UNIMICRO SYSTEMS, INC.",
    "00C084": "DATA LINK CORP. LTD.",
    "00C041": "DIGITAL TRANSMISSION SYSTEMS",
    "00C00D": "ADVANCED LOGIC RESEARCH, INC.",
    "00C0B4": "MYSON TECHNOLOGY, INC.",
    "00C080": "NETSTAR, INC.",
    "00C045": "ISOLATION SYSTEMS, LTD.",
    "0070B3": "DATA RECALL LTD.",
    "00E6D3": "NIXDORF COMPUTER CORP.",
    "0070B0": "M/A-COM INC. COMPANIES",
    "00C01F": "S.E.R.C.E.L.",
    "006086": "LOGIC REPLACEMENT TECH. LTD.",
    "00C0B6": "HVE, Inc. ",
    "00C07A": "PRIVA B.V.",
    "00C06D": "BOCA RESEARCH, INC.",
    "00C0EA": "ARRAY TECHNOLOGY LTD.",
    "00C009": "KT TECHNOLOGY (S) PTE LTD",
    "00C081": "METRODATA LTD.",
    "00C03B": "MULTIACCESS COMPUTING CORP.",
    "00C082": "MOORE PRODUCTS CO.",
    "00C099": "YOSHIKI INDUSTRIAL CO.,LTD.",
    "00C0DC": "EOS TECHNOLOGIES, INC.",
    "00C03C": "TOWER TECH S.R.L.",
    "00C01D": "GRAND JUNCTION NETWORKS, INC.",
    "00C070": "SECTRA SECURE-TRANSMISSION AB",
    "00C072": "KNX LTD.",
    "00C0AE": "TOWERCOM CO. INC. DBA PC HOUSE",
    "00C0D6": "J1 SYSTEMS, INC.",
    "00C0F6": "CELAN TECHNOLOGY INC.",
    "00C0C2": "INFINITE NETWORKS LTD.",
    "00C0AF": "TEKLOGIX INC.",
    "00C059": "DENSO CORPORATION",
    "00C0F1": "SHINKO ELECTRIC CO., LTD.",
    "00C001": "DIATEK PATIENT MANAGMENT",
    "00C0F4": "INTERLINK SYSTEM CO., LTD.",
    "00C0E2": "CALCOMP, INC.",
    "00C07B": "ASCEND COMMUNICATIONS, INC.",
    "00C098": "CHUNTEX ELECTRONIC CO., LTD.",
    "00C0BE": "ALCATEL - SEL",
    "00C06E": "HAFT TECHNOLOGY, INC.",
    "00C0F8": "ABOUT COMPUTING INC.",
    "00C078": "COMPUTER SYSTEMS ENGINEERING",
    "00C09A": "PHOTONICS CORPORATION",
    "00C01A": "COROMETRICS MEDICAL SYSTEMS",
    "00C068": "HME Clear-Com LTD.",
    "00C0D8": "UNIVERSAL DATA SYSTEMS",
    "004099": "NEWGEN SYSTEMS CORP.",
    "004011": "ANDOVER CONTROLS CORPORATION",
    "0040A1": "ERGO COMPUTING",
    "004081": "MANNESMANN SCANGRAPHIC GMBH",
    "004036": "Minim Inc.",
    "004016": "ADC - Global Connectivity Solutions Division",
    "00406A": "KENTEK INFORMATION SYSTEMS,INC",
    "00400A": "PIVOTAL TECHNOLOGIES, INC.",
    "004082": "LABORATORY EQUIPMENT CORP.",
    "00C08C": "PERFORMANCE TECHNOLOGIES, INC.",
    "00C007": "PINNACLE DATA SYSTEMS, INC.",
    "00C08A": "Lauterbach GmbH",
    "0040B7": "STEALTH COMPUTER SYSTEMS",
    "0040A8": "IMF INTERNATIONAL LTD.",
    "004070": "INTERWARE CO., LTD.",
    "0040AC": "SUPER WORKSTATION, INC.",
    "00C0F7": "ENGAGE COMMUNICATION, INC.",
    "10005A": "IBM Corp",
    "0040E0": "ATOMWIDE LTD.",
    "0040D1": "FUKUDA DENSHI CO., LTD.",
    "004069": "LEMCOM SYSTEMS, INC.",
    "0040F1": "CHUO ELECTRONICS CO., LTD.",
    "0040A9": "DATACOM INC.",
    "0040E3": "QUIN SYSTEMS LTD",
    "004091": "PROCOMP INDUSTRIA ELETRONICA",
    "0040EA": "PLAIN TREE SYSTEMS INC",
    "00408A": "TPS TELEPROCESSING SYS. GMBH",
    "0040FD": "LXE",
    "0040FA": "MICROBOARDS, INC.",
    "00803B": "APT COMMUNICATIONS, INC.",
    "00806A": "ERI (EMPAC RESEARCH INC.)",
    "00C0A8": "GVC CORPORATION",
    "00403B": "SYNERJET INTERNATIONAL CORP.",
    "00403F": "SSANGYONG COMPUTER SYSTEMS",
    "004014": "COMSOFT GMBH",
    "004000": "PCI COMPONENTES DA AMZONIA LTD",
    "00406C": "COPERNIQUE",
    "004075": "Tattile SRL ",
    "004053": "AMPRO COMPUTERS",
    "008038": "DATA RESEARCH & APPLICATIONS",
    "00805E": "LSI LOGIC CORPORATION",
    "008060": "NETWORK INTERFACE CORPORATION",
    "0040A7": "ITAUTEC PHILCO S.A.",
    "004064": "KLA INSTRUMENTS CORPORATION",
    "00405A": "GOLDSTAR INFORMATION & COMM.",
    "004013": "NTT DATA COMM. SYSTEMS CORP.",
    "0080F5": "Quantel Ltd",
    "0080B9": "ARCHE TECHNOLIGIES INC.",
    "0080A7": "Honeywell International Inc",
    "00400C": "GENERAL MICRO SYSTEMS, INC.",
    "004063": "VIA TECHNOLOGIES, INC.",
    "008028": "TRADPOST (HK) LTD",
    "008061": "LITTON SYSTEMS, INC.",
    "0080C3": "BICC INFORMATION SYSTEMS & SVC",
    "008044": "SYSTECH COMPUTER CORP.",
    "008006": "COMPUADD CORPORATION",
    "00808A": "SUMMIT MICROSYSTEMS CORP.",
    "004043": "Nokia Siemens Networks GmbH & Co. KG.",
    "00405E": "NORTH HILLS ISRAEL",
    "00807C": "FIBERCOM, INC.",
    "008091": "TOKYO ELECTRIC CO.,LTD",
    "00809D": "Commscraft Ltd.",
    "0080F4": "TELEMECANIQUE ELECTRIQUE",
    "00808B": "DACOLL LIMITED",
    "0080CB": "FALCO DATA PRODUCTS",
    "008007": "DLOG NC-SYSTEME",
    "008062": "INTERFACE  CO.",
    "00801E": "XINETRON, INC.",
    "0080E2": "T.D.I. CO., LTD.",
    "008036": "REFLEX MANUFACTURING SYSTEMS",
    "008083": "AMDAHL",
    "00804D": "CYCLONE MICROSYSTEMS, INC.",
    "00800B": "CSK CORPORATION",
    "008018": "KOBE STEEL, LTD.",
    "00809B": "JUSTSYSTEM CORPORATION",
    "0080DF": "ADC CODENOLL TECHNOLOGY CORP.",
    "0080D4": "CHASE RESEARCH LTD.",
    "00803D": "SURIGIKEN CO.,  LTD.",
    "008049": "NISSIN ELECTRIC CO., LTD.",
    "0080C1": "LANEX CORPORATION",
    "0080BC": "HITACHI ENGINEERING CO., LTD",
    "0080B2": "NETWORK EQUIPMENT TECHNOLOGIES",
    "008076": "MCNC",
    "008022": "SCAN-OPTICS",
    "0000B2": "TELEVIDEO SYSTEMS, INC.",
    "0000EE": "NETWORK DESIGNERS, LTD.",
    "0000E5": "SIGMEX LTD.",
    "000089": "CAYMAN SYSTEMS INC.",
    "0000FF": "CAMTEC ELECTRONICS LTD.",
    "0000B7": "DOVE COMPUTER CORPORATION",
    "0000F2": "SPIDER COMMUNICATIONS",
    "0000CC": "DENSAN CO., LTD.",
    "0080A8": "VITACOM CORPORATION",
    "008033": "EMS Aviation, Inc.",
    "0080DD": "GMX INC/GIMIX",
    "0080FB": "BVM LIMITED",
    "0000DB": "British Telecommunications plc",
    "0000C1": "Madge Ltd.",
    "0000F6": "APPLIED MICROSYSTEMS CORP.",
    "00003F": "SYNTREX, INC.",
    "00008E": "SOLBOURNE COMPUTER, INC.",
    "0000DC": "HAYES MICROCOMPUTER PRODUCTS",
    "000063": "BARCO CONTROL ROOMS GMBH",
    "0080B4": "SOPHIA SYSTEMS",
    "00807F": "DY-4 INCORPORATED",
    "0000BD": "RYOSEI, Ltd.",
    "00002E": "SOCIETE EVIRA",
    "00004E": "AMPEX CORPORATION",
    "0000C2": "INFORMATION PRESENTATION TECH.",
    "0000FC": "MEIKO",
    "0000CD": "Allied Telesis Labs Ltd",
    "008068": "YAMATECH SCIENTIFIC LTD.",
    "00006D": "CRAY COMMUNICATIONS, LTD.",
    "0000DA": "ATEX",
    "00802D": "XYLOGICS INC",
    "0000A0": "SANYO Electric Co., Ltd.",
    "0000C0": "WESTERN DIGITAL CORPORATION",
    "000033": "EGAN MACHINERY COMPANY",
    "0000EA": "UPNOD AB",
    "000043": "MICRO TECHNOLOGY",
    "000017": "Oracle",
    "000065": "Network General Corporation",
    "000011": "NORMEREL SYSTEMES",
    "0000C4": "WATERS DIV. OF MILLIPORE",
    "0000A2": "Bay Networks",
    "0000EC": "MICROPROCESS",
    "000061": "GATEWAY COMMUNICATIONS",
    "0000A4": "ACORN COMPUTERS LIMITED",
    "0000DD": "TCL INCORPORATED",
    "0000AE": "DASSAULT ELECTRONIQUE",
    "000077": "INTERPHASE CORPORATION",
    "000099": "MTX, INC.",
    "000028": "PRODIGY SYSTEMS CORPORATION",
    "000010": "SYTEK INC.",
    "0000EB": "MATSUSHITA COMM. IND. CO. LTD.",
    "080043": "PIXEL COMPUTER INC.",
    "080045": "CONCURRENT COMPUTER CORP.",
    "08003B": "TORUS SYSTEMS LIMITED",
    "08003C": "SCHLUMBERGER WELL SERVICES",
    "080034": "FILENET CORPORATION",
    "080036": "INTERGRAPH CORPORATION",
    "080033": "BAUSCH & LOMB",
    "080030": "CERN",
    "080031": "LITTLE MACHINES INC.",
    "08007E": "AMALGAMATED WIRELESS(AUS) LTD",
    "08007F": "CARNEGIE-MELLON UNIVERSITY",
    "080029": "Megatek Corporation",
    "080023": "Panasonic Communications Co., Ltd.",
    "08005E": "COUNTERPOINT COMPUTER INC.",
    "080056": "STANFORD LINEAR ACCEL. CENTER",
    "080048": "EUROTHERM GAUGING SYSTEMS",
    "08004F": "CYGNET SYSTEMS",
    "080050": "DAISY SYSTEMS CORP.",
    "08002E": "METAPHOR COMPUTER SYSTEMS",
    "08002B": "DIGITAL EQUIPMENT CORPORATION",
    "080075": "DANSK DATA ELECTRONIK",
    "080078": "ACCELL CORPORATION",
    "08006D": "WHITECHAPEL COMPUTER WORKS",
    "00009D": "LOCUS COMPUTING CORPORATION",
    "0000FD": "HIGH LEVEL HARDWARE",
    "08000F": "MITEL CORPORATION",
    "0270B0": "M/A-COM INC. COMPANIES",
    "000053": "COMPUCORP",
    "08000A": "NESTAR SYSTEMS INCORPORATED",
    "000004": "XEROX CORPORATION",
    "00DD0E": "UNGERMANN-BASS INC.",
    "080018": "PIRELLI FOCOM NETWORKS",
    "0000A6": "NETWORK GENERAL CORPORATION",
    "00BBF0": "UNGERMANN-BASS INC.",
    "00408E": "Tattile SRL ",
    "08001C": "KDD-KOKUSAI DEBNSIN DENWA CO.",
    "080090": "SONOMA SYSTEMS",
    "00800F": "STANDARD MICROSYSTEMS",
    "00406B": "SYSGEN",
    "00DD0C": "UNGERMANN-BASS INC.",
    "0001C8": "CONRAD CORP.",
    "CC45A5": "Texas Instruments",
    "602602": "Texas Instruments",
    "ECBFD0": "Texas Instruments",
    "C4E733": "Clear Align LLC",
    "5CFCE1": "Resideo",
    "F0A731": "TP-Link Corporation Limited",
    "54DED3": "zte corporation",
    "749B89": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C53B4": "EM Microelectronic",
    "50A6D8": "Apple, Inc.",
    "68CAC4": "Apple, Inc.",
    "00308E": "Crossmatch Technologies/HID Global",
    "000633": "Crossmatch Technologies/HID Global",
    "80F5AE": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "9415B2": "Huawei Device Co., Ltd.",
    "E8B722": "GreenTrol Automation ",
    "606134": "Arcade Communications Ltd.",
    "A4978A": "LEAR",
    "4CEA41": "IEEE Registration Authority",
    "74205F": "Shenzhen Zhongruixin Intelligent Technology Co., Ltd.",
    "14064C": "Vogl Electronic GmbH",
    "8C6120": "EM Microelectronic",
    "8C8E4E": "Baylan Olcu Aletleri San. ve Tic.A.S.",
    "F8ACC1": "InnoXings Co., LTD.",
    "4064DC": "X-speed lnformation Technology Co.,Ltd",
    "5CC787": "Huawei Device Co., Ltd.",
    "845075": "Huawei Device Co., Ltd.",
    "5CF838": "IEEE Registration Authority",
    "30D97F": "Tech4home, Lda",
    "4080E1": "FN-LINK TECHNOLOGY Ltd.",
    "C44838": "Satcom Direct, Inc.",
    "9C37CB": "Sony Interactive Entertainment Inc.",
    "3CB0ED": "Nothing Technology Limited",
    "ECCF70": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "04F0E4": "ShenZhen Hosecom Electronic Technology Co.,LTD",
    "D8E23F": "Qingdao Haier Technology Co.,Ltd",
    "D8032A": "COMMTACT LTD",
    "847051": "ALPSALPINE CO,.LTD",
    "50131D": "Motorola Mobility LLC, a Lenovo Company",
    "D47F78": "Dopple B.V.",
    "90A9F7": "IEEE Registration Authority",
    "F0F7FC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30FFFD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0995A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00F7AD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4CBCC": "vivo Mobile Communication Co., Ltd.",
    "3C8427": "Espressif Inc.",
    "740BB0": "Hirschmann Automation and Control GmbH",
    "20ED47": "Juniper Networks",
    "94EF49": "BDR Thermea Group B.V",
    "90B685": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "A0D1B3": "Apple, Inc.",
    "7C4B26": "Apple, Inc.",
    "24D5E4": "Cisco Systems, Inc",
    "24A186": "ARRIS Group, Inc.",
    "CC115A": "Apple, Inc.",
    "0080A3": "Lantronix",
    "90D432": "zte corporation",
    "18E83B": "Citadel Wallet LLC",
    "008066": "Eurotech S.p.A.",
    "EC41F9": "Shenzhen iComm Semiconductor CO.,LTD",
    "785333": "Beijing Xiaomi Electronics Co.,Ltd",
    "9CF8B8": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "9432C1": "Honor Device Co., Ltd.",
    "189578": "DENSO Corporation",
    "147F67": "LG Innotek",
    "EC9A0C": "IEEE Registration Authority",
    "CC4D75": "Beijing Xiaomi Mobile Software Co., Ltd",
    "107C61": "ASUSTek COMPUTER INC.",
    "58FCC8": "LenelS2 Carrier",
    "008DF4": "IEEE Registration Authority",
    "7818EC": "Fortinet, Inc.",
    "6C7F0C": "Cisco Meraki",
    "5CB26D": "Intel Corporate",
    "481CB9": "SZ DJI TECHNOLOGY CO.,LTD",
    "E02EFE": "Zhejiang Dahua Technology Co., Ltd.",
    "CCA08F": "zte corporation",
    "E8B0C5": "Intel Corporate",
    "FCB585": "Shenzhen Water World Information Co.,Ltd.",
    "D424DD": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "6C51E4": "Huawei Device Co., Ltd.",
    "B0FA8B": "Huawei Device Co., Ltd.",
    "ECE660": "Qingdao Hisense Communications Co.,Ltd.",
    "E8288D": "Huawei Device Co., Ltd.",
    "F462DC": "Huawei Device Co., Ltd.",
    "C46237": "sunweit industrial limited",
    "F4D58A": "Palo Alto Networks",
    "C8C6FE": "eero inc.",
    "1889DF": "OMNIVISION",
    "44303F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C07A6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC1E56": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0551F": "Sagemcom Broadband SAS",
    "4CB911": "Raisecom Technology CO., LTD",
    "000E5E": "Raisecom Technology CO., LTD",
    "38C6BD": "Xiaomi Communications Co Ltd",
    "000C8D": "Balluff MV GmbH",
    "908938": "Hefei Linkin Technology Co., Ltd.",
    "A8BA69": "Samsung Electronics Co.,Ltd",
    "A83CA5": "Dell Inc.",
    "80F7A6": "Shenzhen C-Data Technology Co., Ltd.",
    "302FAC": "Zhejiang HuaRay Technology Co.,Ltd",
    "B80BDA": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "342DA3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0C6AC4": "Apple, Inc.",
    "BC37D3": "Apple, Inc.",
    "20CC27": "Cisco Systems, Inc",
    "0C517E": "Apple, Inc.",
    "28D25A": "Infinix mobility limited",
    "80EA0B": "Zyxel Communications Corporation",
    "982F86": "vivo Mobile Communication Co., Ltd.",
    "04E4B6": "Samsung Electronics Co.,Ltd",
    "BCEA9C": "ITEL MOBILE LIMITED",
    "0479FD": "Ciena Corporation",
    "509893": "Texas Instruments",
    "6C8338": "Ubihere",
    "000CEF": "\t ONE Investment Group Limited",
    "503AA0": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "5CDE34": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "842712": "Silicon Laboratories",
    "7CC6B6": "Silicon Laboratories",
    "980C33": "Silicon Laboratories",
    "38398F": "Silicon Laboratories",
    "7CF0E5": "OnePlus Technology (Shenzhen) Co., Ltd",
    "505E5C": "SUNITEC TECHNOLOGY CO.,LIMITED",
    "3802E3": "YICHEN (SHENZHEN) TECHNOLOGY CO.,LTD",
    "E4A7D0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B42BB9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "34C7E9": "Google, Inc.",
    "AC8C46": "Beijing Xiaomi Mobile Software Co., Ltd",
    "482D63": "Wavarts Technologies Co., Ltd",
    "8099E7": "Sony Corporation",
    "209339": "Juniper Networks",
    "E8FF1E": "IEEE Registration Authority",
    "1C54E6": "Shenzhen Yisheng Technology Co.,Ltd",
    "0C4B48": "Nokia",
    "7C7B1C": "Motorola Mobility LLC, a Lenovo Company",
    "7834FD": "vivo Mobile Communication Co., Ltd.",
    "FC3CD7": "Tuya Smart Inc.",
    "B09C63": "Xiaomi Communications Co Ltd",
    "E89847": "Xiaomi Communications Co Ltd",
    "FC386A": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "508811": "Beijing Xiaomi Mobile Software Co., Ltd",
    "F8C4FA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "ACCE92": "New H3C Technologies Co., Ltd",
    "AC81B5": "Accton Technology Corporation",
    "D0460C": "Dell Inc.",
    "84D5A0": "MCOT INC.",
    "84E8CB": "BUFFALO.INC",
    "90A0BE": "Cannice",
    "38E1F4": "Sagemcom Broadband SAS",
    "6453E0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "688BF4": "Arista Network, Inc.",
    "94017D": "SHENZHEN SHLINK.CO.,LIMITED",
    "C0E5DA": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "D8A6FD": "Ghost Autonomy Inc.",
    "202351": "TP-LINK CORPORATION PTE. LTD.",
    "8C8474": "Broadcom Limited",
    "D83D3F": "JOYNED GmbH",
    "D44867": "Silicon Laboratories",
    "D8C6F9": "Tracklab Inc",
    "6C483F": "Xiaomi Communications Co Ltd",
    "3CAFB7": "Xiaomi Communications Co Ltd",
    "540853": "Qingdao Haier Technology Co.,Ltd",
    "BC0866": "Nestle Purina PetCare",
    "DC9B95": "Phyplus Technology (Shanghai) Co., Ltd",
    "4800B3": "Cisco Systems, Inc",
    "2CB68F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C8D1A9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40BB56": "TeraNXT Global India Pvt Ltd.",
    "E05D54": "Ruijie Networks Co.,LTD",
    "18188B": "FCNT LLC",
    "DC71D0": "Apple, Inc.",
    "9CFA76": "Apple, Inc.",
    "20FA85": "Apple, Inc.",
    "9C1A25": "Apple, Inc.",
    "6C6016": "OnePlus Technology (Shenzhen) Co., Ltd",
    "A0782D": "Apple, Inc.",
    "D011E5": "Apple, Inc.",
    "40DA5C": "Apple, Inc.",
    "5C9175": "Apple, Inc.",
    "D84489": "TP-LINK CORPORATION PTE. LTD.",
    "44F770": "Beijing Xiaomi Mobile Software Co., Ltd",
    "A83162": "Hangzhou Huacheng Network Technology Co.,Ltd",
    "64FE15": "Flaircomm Microelectronics,Inc.",
    "D86BFC": "zte corporation",
    "E04824": "Garmin International",
    "08F0B6": "Edifier International",
    "C0555C": "Impulse Labs",
    "A8595F": "Intel Corporate",
    "CC28AA": "ASUSTek COMPUTER INC.",
    "3C9FCD": "Shenzhen Neoway Technology Co.,Ltd.",
    "341453": "Gantner Electronic GmbH",
    "CC641A": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "A89609": "FN-LINK TECHNOLOGY Ltd.",
    "68B41E": "ZEASN TECHNOLOGY PRIVATE LIMITED",
    "841571": "TCT mobile ltd",
    "E8BFB8": "Intel Corporate",
    "DC4BA1": "Wistron Neweb Corporation",
    "34A6EF": "Shenzhen Phaten Tech. LTD",
    "BC3F4E": "Great Talent Technology Limited",
    "540463": "CIG SHANGHAI CO LTD",
    "882067": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E85497": "Samsung Electronics Co.,Ltd",
    "14E01D": "Samsung Electronics Co.,Ltd",
    "DC54AD": "Hangzhou RunZhou Fiber Technologies Co.,Ltd",
    "B0924A": "Sagemcom Broadband SAS",
    "40DE24": "Samsung Electronics Co.,Ltd",
    "1083B4": "Sidora Srl",
    "403059": "Silicon Laboratories",
    "B85C5C": "Microsoft Corporation",
    "F0D805": "Cisco Systems, Inc",
    "086332": "IEEE Registration Authority",
    "B492FE": "Arista Network, Inc.",
    "4471B3": "Logitech Far East",
    "A4FCA1": "Nokia Solutions and Networks GmbH & Co. KG",
    "787E42": "zte corporation",
    "345A18": "Alignment Engine Inc.",
    "0C6825": "Suzhou HYC technology Co., Ltd.",
    "90B4DD": "ZPT R&D",
    "A0FAC8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6429FF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C09B63": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10A145": "nexzo india pvt ltd ",
    "C4E7AE": "Chengdu Meross Technology Co., Ltd.",
    "AC1518": "Espressif Inc.",
    "648FDB": "Huaqin Technology Co.LTD",
    "001F23": "Interacoustics",
    "94E6BA": "Samsung Electronics Co.,Ltd",
    "746DFA": "Samsung Electronics Co.,Ltd",
    "980709": "Huawei Device Co., Ltd.",
    "6083E7": "TP-LINK CORPORATION PTE. LTD.",
    "0441A5": "Apple, Inc.",
    "1C0EC2": "Apple, Inc.",
    "B08BA8": "Amazon Technologies Inc.",
    "38ACDD": "Valenco GmbH",
    "C4CC37": "IEEE Registration Authority",
    "7C7EF9": "eero inc.",
    "C40898": "Dropbeats Technology Co., Ltd.",
    "3C996D": "Marelli Europe s.p.a.",
    "B8CC5F": "Shenzhen iComm Semiconductor CO.,LTD",
    "8850DD": "Infiniband Trade Association",
    "5C35FC": "Actiontec Electronics Inc.",
    "E85FB4": "Xiaomi Communications Co Ltd",
    "ACEAEA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38F195": "HUAWEI TECHNOLOGIES CO.,LTD",
    "982A0A": "Intelbras",
    "081287": "Jiangxi Risound Electronics Co., LTD",
    "1402EC": "Hewlett Packard Enterprise ",
    "1C98EC": "Hewlett Packard Enterprise ",
    "24F27F": "Hewlett Packard Enterprise ",
    "808DB7": "Hewlett Packard Enterprise ",
    "941882": "Hewlett Packard Enterprise ",
    "34FCB9": "Hewlett Packard Enterprise ",
    "484AE9": "Hewlett Packard Enterprise ",
    "4CAEA3": "Hewlett Packard Enterprise ",
    "B8977A": "Nokia Solutions and Networks GmbH & Co. KG",
    "20A6CD": "Hewlett Packard Enterprise ",
    "703A0E": "Hewlett Packard Enterprise",
    "D8C7C8": "Hewlett Packard Enterprise",
    "6CC49F": "Hewlett Packard Enterprise",
    "EC50AA": "Hewlett Packard Enterprise",
    "F061C0": "Hewlett Packard Enterprise",
    "B01F8C": "Hewlett Packard Enterprise",
    "48B4C3": "Hewlett Packard Enterprise",
    "209CB4": "Hewlett Packard Enterprise",
    "4CD587": "Hewlett Packard Enterprise",
    "9C3708": "Hewlett Packard Enterprise",
    "34C515": "Hewlett Packard Enterprise",
    "6026EF": "Hewlett Packard Enterprise",
    "BCD7A5": "Hewlett Packard Enterprise",
    "187A3B": "Hewlett Packard Enterprise",
    "B86CE0": "Hewlett Packard Enterprise ",
    "204C03": "Hewlett Packard Enterprise",
    "F01AA0": "Hewlett Packard Enterprise",
    "A0A001": "Hewlett Packard Enterprise",
    "8C7909": "Hewlett Packard Enterprise",
    "000B86": "Hewlett Packard Enterprise",
    "883A30": "Hewlett Packard Enterprise",
    "7CA62A": "Hewlett Packard Enterprise ",
    "2462CE": "Hewlett Packard Enterprise",
    "D8D8E5": "Vantiva Connected Home - Technologies Telco",
    "4C2338": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "4472AC": "AltoBeam Inc.",
    "A86E84": "TP-LINK CORPORATION PTE. LTD.",
    "888C1B": "EM Microelectronic",
    "3CF083": "Arcadyan Corporation",
    "C0D941": "Shenzhen VMAX Software Co., Ltd.",
    "68B76B": "Fiberhome Telecommunication Technologies Co.,LTD",
    "9C1221": "Sichuan AI-Link Technology Co., Ltd.",
    "103F8C": "New H3C Technologies Co., Ltd",
    "F09E9E": "Espressif Inc.",
    "48B313": "Idesco Oy",
    "68C6AC": "Intel Corporate",
    "BC3898": "Intel Corporate",
    "A04F52": "Intel Corporate",
    "3C94FD": "Mist Systems, Inc.",
    "94CA9A": "Paul Vahle GmbH & Co. KG",
    "742EC1": "Dixon Electro Appliances Pvt Ltd",
    "3023BA": "Accelerated Memory Production Inc.",
    "84F2C1": "zte corporation",
    "40827B": "STMicroelectronics Rousset SAS",
    "D06DC8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4062EE": "Chicony Electronics Co., Ltd.",
    "742972": "Juniper Networks",
    "E45ECC": "Juniper Networks",
    "286B5C": "Cisco Systems, Inc",
    "5879E0": "Samsung Electronics Co.,Ltd",
    "C47764": "Samsung Electronics Co.,Ltd",
    "607FCB": "Samsung Electronics Co.,Ltd",
    "6CB34D": "SharkNinja Operating LLC",
    "5C5A4C": "IEEE Registration Authority",
    "980802": "ORBIS BV",
    "FCB387": "Leapmotor (Jinhua) New Energy Vehicle Parts Technology Co., Ltd.",
    "70357B": "vivo Mobile Communication Co., Ltd.",
    "A8A913": "GN Hearing A/S",
    "BC2978": "Prama Hikvision India Private Limited",
    "1423F3": "Broadcom Limited",
    "9C0E51": "Schneider Electric",
    "1CF64C": "Apple, Inc.",
    "AC5C2C": "Apple, Inc.",
    "50F265": "Apple, Inc.",
    "C06C0C": "Apple, Inc.",
    "EC09C9": "Texas Instruments",
    "6C3A36": "Glowforge Inc",
    "C8EED7": "Lightspeed Technologies Inc.",
    "603192": "OVT India pvt Ltd ",
    "58D15A": "Texas Instruments",
    "9CDAA8": "Apple, Inc.",
    "0C07DF": "Xiaomi Communications Co Ltd",
    "F0ABF3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "7CE712": "Quectel Wireless Solutions Co.,Ltd.",
    "0CEA14": "Ubiquiti Inc",
    "E817FC": "Fujitsu Limited",
    "E82281": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "203B34": "Xiaomi Communications Co Ltd",
    "B43D6B": "NXP Semiconductor (Tianjin) LTD.",
    "4CD7C8": "Guangzhou V-Solution Telecommunication Technology Co.,Ltd.",
    "A42902": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "6CD008": "zte corporation",
    "508CC9": "zte corporation",
    "346E68": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC9096": "Huawei Device Co., Ltd.",
    "2CE2D9": "Huawei Device Co., Ltd.",
    "8492E5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCAE2B": "Titan Products Ltd.",
    "5CEB52": "zte corporation",
    "FCABF5": "zte corporation",
    "083BE9": "New H3C Technologies Co., Ltd",
    "10B65E": "New H3C Technologies Co., Ltd",
    "28EBB7": "ambie corporation",
    "60CF84": "ASUSTek COMPUTER INC.",
    "685134": "Hewlett Packard Enterprise",
    "F0CF4D": "BitRecords GmbH",
    "00153B": "EMH Metering GmbH & Co. KG",
    "B848EC": "Private",
    "F41E57": "Routerboard.com",
    "081031": "Lithiunal Energy",
    "CC6E2A": "Cisco Meraki",
    "30F23C": "TECNO MOBILE LIMITED",
    "1C112F": "vivo Mobile Communication Co., Ltd.",
    "E41E33": "Continental Automotive Technologies GmbH",
    "74AD45": "Valeo Auto- Electric Hungary Ltd",
    "EC308E": "Lierda Science & Technology Group Co., Ltd",
    "70C59C": "Silicon Laboratories",
    "E406BF": "Silicon Laboratories",
    "0CC56C": "Apple, Inc.",
    "983FE8": "Samsung Electronics Co.,Ltd",
    "10ABC9": "Samsung Electronics Co.,Ltd",
    "E86A64": "LCFC(Hefei) Electronics Technology co., ltd",
    "902E16": "LCFC(Hefei) Electronics Technology co., ltd",
    "CC4B04": "Apple, Inc.",
    "4409DA": "Apple, Inc.",
    "809698": "Apple, Inc.",
    "C484FC": "Apple, Inc.",
    "60FF9E": "AzureWave Technology Inc.",
    "001E96": "Sepura Limited",
    "E88088": "LCFC(Hefei) Electronics Technology co., ltd",
    "745D22": "LCFC(Hefei) Electronics Technology co., ltd",
    "408556": "Continental Automotive Romania SLR",
    "9C69ED": "Arista Networks",
    "645563": "Q-Free America, Inc.",
    "C8A913": "Lontium Semiconductor Corporation",
    "101A92": "AKEBONO BRAKE INDUSTRY CO.,LTD.",
    "781F7C": "Nokia",
    "485C2C": "Earda Technologies co Ltd",
    "88F4DA": "Intel Corporate",
    "EC8E77": "Intel Corporate",
    "8407C4": "Walter Kidde Portable Equipment, Inc.",
    "ECFC2F": "Sagemcom Broadband SAS",
    "6045CD": "Sagemcom Broadband SAS",
    "2C5683": "China Mobile Group Device Co.,Ltd.",
    "D823E0": "SPEEDTECH CORP.",
    "30E3A4": "Intel Corporate",
    "A085E3": "Espressif Inc.",
    "084857": "Suteng Innovation Technology Co., Ltd.",
    "74F90F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2014C4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C9515": "Palltronics, Inc.",
    "707DAF": "Plucent AB",
    "64E815": "Arcelik A.S",
    "50056E": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "EC470C": "Universal Electronics, Inc.",
    "3854F5": "AltoBeam Inc.",
    "70F74F": "Robert Bosch JuP1",
    "485E0E": "IEEE Registration Authority",
    "C0BAE6": "Zenitel GB Ltd",
    "E01F34": "HMD Global Oy",
    "4CF475": "Huawei Device Co., Ltd.",
    "3C4AC9": "Huawei Device Co., Ltd.",
    "2C0D27": "Huawei Device Co., Ltd.",
    "5CBE69": "Oraimo Technology Limited",
    "94CFB0": "Huawei Device Co., Ltd.",
    "D8CF61": "Sagemcom Broadband SAS",
    "FC4116": "Google, Inc.",
    "48B424": "eero inc.",
    "A02442": "Shenzhenshi Xinzhongxin Technology Co.Ltd",
    "3C5765": "UNIONMAN TECHNOLOGY CO.,LTD",
    "5805D9": "Seiko Epson Corporation",
    "EC5A31": "BUFFALO.INC",
    "F01EAC": "Rentokil Initial",
    "34E6E6": "LG Innotek",
    "ECA7B1": "IEEE Registration Authority",
    "B48618": "GX India Pvt Ltd",
    "EC1DA9": "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD.",
    "A4A584": "Cisco Systems, Inc",
    "C4A9B8": "XIAMENSHI C-CHIP TECHNOLOGY CO.,LTD",
    "345CF3": "Chipsea Technologies (Shenzhen) Corp.",
    "58ED99": "zte corporation",
    "2441FE": "Wistron Neweb Corporation",
    "F4BDB9": "Ubiqcom India Pvt Ltd",
    "20B5C6": "Mimosa Networks",
    "E8F0A4": "Antonios A. Chariton",
    "EC536F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D801D0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "84EE7F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "788371": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00749C": "Ruijie Networks Co.,LTD",
    "E01C41": "Extreme Networks Headquarters",
    "BCF310": "Extreme Networks Headquarters",
    "489BD5": "Extreme Networks Headquarters",
    "DCB808": "Extreme Networks Headquarters",
    "8C497A": "Extreme Networks Headquarters",
    "B027CF": "Extreme Networks Headquarters",
    "D854A2": "Extreme Networks Headquarters",
    "F46E95": "Extreme Networks Headquarters",
    "000130": "Extreme Networks Headquarters",
    "FC0A81": "Extreme Networks Headquarters",
    "C851FB": "Extreme Networks Headquarters",
    "DC233B": "Extreme Networks Headquarters",
    "C8665D": "Extreme Networks Headquarters",
    "E4DBAE": "Extreme Networks Headquarters",
    "CCCCEA": "PHOENIX CONTACT Electronics GmbH",
    "000FEA": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "48417B": "Nokia Solutions and Networks GmbH & Co. KG",
    "38AD2B": "Hitron Technologies. Inc",
    "E8458B": "MitraStar Technology Corp.",
    "1CAB48": "TECNO MOBILE LIMITED",
    "608CDF": "Beamtrail-Sole Proprietorship",
    "3027CF": "Canopy Growth Corp",
    "D04433": "Clourney Semiconductor",
    "001277": "Beijer Electronics Corp.",
    "944560": "Google, Inc.",
    "5C337B": "Google, Inc.",
    "4086CB": "D-Link Corporation",
    "2047B5": "Sagemcom Broadband SAS",
    "60452E": "Intel Corporate",
    "8CE9EE": "Intel Corporate",
    "CCD342": "Cisco Systems, Inc",
    "40B8C2": "OSMOZIS",
    "88B436": "FUJIFILM Corporation",
    "2C9C58": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "B8F44F": "u-blox AG",
    "540929": "Inventus Power Eletronica do Brasil LTDA",
    "2CBACA": "Cosonic Electroacoustic Technology Co., Ltd.",
    "009CC0": "vivo Mobile Communication Co., Ltd.",
    "606682": "SHENZHEN ATEKO PHOTOELECTRICITY CO.,LTD",
    "788DAF": "Sagemcom Broadband SAS",
    "BC0FFE": "Juniper Networks",
    "BC1896": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2831F8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4AA99": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8F5E1": "Shenzhen Shokz Co., Ltd.",
    "E89E13": "CRESYN",
    "7CFCFD": "Fiberhome Telecommunication Technologies Co.,LTD",
    "4887B8": "TCL King Electrical Appliances(Huizhou)Co.,Ltd",
    "7058A4": "Actiontec Electronics Inc.",
    "A494DC": "Infinite Clouds",
    "E47876": "Arista Networks",
    "A4A930": "Beijing Xiaomi Mobile Software Co., Ltd",
    "4C4341": "Calix Inc.",
    "E46CD1": "Calix Inc.",
    "D0F27F": "BrewLogix, LLC",
    "188025": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "E0D3B4": "Cisco Meraki",
    "FC8C11": "Microsoft Corporation",
    "04FA3F": "OptiCore Inc.",
    "441DB1": "APTIV SERVICES US, LLC",
    "88F00F": "Miraeil",
    "4435D3": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "F8F09D": "Hangzhou Prevail Communication Technology Co., Ltd",
    "7C752D": "Samsung Electronics Co.,Ltd",
    "84EEE4": "Samsung Electronics Co.,Ltd",
    "FC936B": "Samsung Electronics Co.,Ltd",
    "BCB6FB": "P4Q ELECTRONICS, S.L.",
    "A02252": "Astra Wireless Technology FZ-LLC",
    "BC102F": "SJI Industry Company",
    "74C530": "vivo Mobile Communication Co., Ltd.",
    "18A788": "Shenzhen MEK Intellisys Pte Ltd",
    "BC96E5": "SERCOMM PHILIPPINES INC",
    "64DAED": "eero inc.",
    "289401": "NETGEAR",
    "D878C9": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "A85EF2": "TECNO MOBILE LIMITED",
    "789A18": "Routerboard.com",
    "A899AD": "Chaoyue Technology Co., Ltd.",
    "D015BB": "IEEE Registration Authority",
    "64CE6E": "Sierra Wireless, ULC",
    "84DB2F": "Sierra Wireless, ULC",
    "D0484F": "Nokia Solutions and Networks GmbH & Co. KG",
    "78DF72": "Shanghai Imilab Technology Co.Ltd",
    "602B58": "EM Microelectronic",
    "109D9C": "EM Microelectronic",
    "28FF5F": "HG Genuine Intelligent Terminal (Xiaogan) Co.,Ltd.",
    "2C459A": "Dixon Technologies (India) Limited",
    "6C6E07": "CE LINK LIMITED",
    "6C6286": "Nokia",
    "BC8529": "Jiangxi Remote lntelligence Technology Co.,Ltd",
    "7492BA": "Movesense Ltd",
    "607623": "Shenzhen E-Superlink Technology Co., Ltd",
    "B8FC28": "Valeo Vision Systems",
    "0020FC": "Matrox Central Services Inc",
    "888187": "Umeox Innovations Co.,Ltd",
    "50482C": "IEEE Registration Authority",
    "48C461": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "B8B7DB": "GOIP Global Services Pvt. Ltd.",
    "880CE0": "Texas Instruments",
    "B83DF6": "Texas Instruments",
    "B4DDE0": "Shanghai Amphenol Airwave Communication Electronics Co.,Ltd.",
    "C42795": "Vantiva USA LLC",
    "E0885D": "Vantiva USA LLC",
    "802994": "Vantiva USA LLC",
    "9404E3": "Vantiva USA LLC",
    "9064AD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4D892": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30E04F": "Apple, Inc.",
    "64E4A5": "LG Electronics",
    "6CA367": "Avlinkpro",
    "589630": "Vantiva USA LLC",
    "989D5D": "Vantiva USA LLC",
    "484BD4": "Vantiva USA LLC",
    "E0DBD1": "Vantiva USA LLC",
    "ECA81F": "Vantiva USA LLC",
    "54A65C": "Vantiva USA LLC",
    "DCEB69": "Vantiva USA LLC",
    "946A77": "Vantiva USA LLC",
    "D4B92F": "Vantiva USA LLC",
    "D40F9E": "Apple, Inc.",
    "980DAF": "Apple, Inc.",
    "DC6DBC": "Apple, Inc.",
    "A4D23E": "Apple, Inc.",
    "B42A0E": "Vantiva USA LLC",
    "603D26": "Vantiva USA LLC",
    "48BDCE": "Vantiva USA LLC",
    "98D3D7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "886EEB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E40D3B": "Ericsson AB",
    "EC5382": "Honor Device Co., Ltd.",
    "0018AD": "NIDEC INSTRUMENTS CORPORATION",
    "A4CCB3": "Xiaomi Communications Co Ltd",
    "9C0971": "New H3C Technologies Co., Ltd",
    "98AB15": "Fujian Youyike Technology Co.,Ltd",
    "7093C1": "eero inc.",
    "5449FC": "Ubee Interactive Co., Limited",
    "201746": "Paradromics, Inc.",
    "CC4085": "WiZ",
    "DCCD18": "Nintendo Co.,Ltd",
    "18F935": "Cisco Systems, Inc",
    "588B1C": "Cisco Systems, Inc",
    "BC3198": "IEEE Registration Authority",
    "ECA2A0": "Taicang T&W Electronics",
    "0C7043": "Sony Interactive Entertainment Inc.",
    "C8154E": "Intel Corporate",
    "4C496C": "Intel Corporate",
    "0011A9": "Nurivoice Co., Ltd",
    "ECE7C2": "China Mobile Group Device Co.,Ltd.",
    "3C0D2C": "Liquid-Markets GmbH",
    "F4BFBB": "China Mobile Group Device Co.,Ltd.",
    "789F38": "Shenzhen Feasycom Co., Ltd",
    "544C8A": "Microsoft Corporation",
    "84AC60": "Guangxi Hesheng Electronics Co., Ltd.",
    "9CA389": "Nokia",
    "5C101E": "zte corporation",
    "905A08": "Super Micro Computer, Inc.",
    "9070D3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "6C48A6": "Fiberhome Telecommunication Technologies Co.,LTD",
    "FCB9DF": "Motorola Mobility LLC, a Lenovo Company",
    "D081C5": "Juniper Networks",
    "FCF763": "KunGao Micro (JiangSu) Co., LTd",
    "808C97": "Kaon Group Co., Ltd.",
    "C46026": "SKY UK LIMITED",
    "000D39": "Nevion",
    "4C12E8": "VIETNAM POST AND TELECOMMUNICATION INDUSTRY TECHNOLOGY JOIN STOCK COMPANY",
    "2002FE": "Hangzhou Dangbei Network Technology Co., Ltd",
    "74272C": "Advanced Micro Devices, Inc.",
    "C05B44": "Beijing Xiaomi Mobile Software Co., Ltd",
    "208810": "Dell Inc.",
    "A06260": "Private",
    "E44519": "Beijing Xiaomi Electronics Co.,Ltd",
    "E4B224": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A83ED3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6467CD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "34ECB6": "Phyplus Microelectronics Limited",
    "807677": "hangzhou puwell cloud tech co., ltd.",
    "9C1ECF": "Valeo Telematik und Akustik GmbH",
    "F42B8C": "Samsung Electronics Co.,Ltd",
    "4CD2FB": "UNIONMAN TECHNOLOGY CO.,LTD",
    "E41A1D": "NOVEA ENERGIES",
    "F4CAE7": "Arcadyan Corporation",
    "3CEF42": "TCT mobile ltd",
    "B0A3F2": "Huaqin Technology Co. LTD",
    "84F175": "Jiangxi Xunte Intelligent Terminal Co., Ltd",
    "20E46F": "vivo Mobile Communication Co., Ltd.",
    "C89828": "zte corporation",
    "DC3642": "zte corporation",
    "788A86": "China Dragon Technology Limited",
    "E82404": "Quectel Wireless Solutions Co.,Ltd.",
    "F86691": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "6C2ADF": "IEEE Registration Authority",
    "883037": "Juniper Networks",
    "E43819": "Shenzhen Hi-Link Electronic CO.,Ltd.",
    "60DC81": "AltoBeam Inc.",
    "847293": "Texas Instruments",
    "F82E0C": "Texas Instruments",
    "9006F2": "Texas Instruments",
    "E86E3A": "Sony Interactive Entertainment Inc.",
    "BCB1D3": "Cisco Meraki",
    "B87BD4": "Google, Inc.",
    "C8138B": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "34AA31": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "A04C0C": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "6CC242": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "E073E7": "HP Inc.",
    "608246": "Apple, Inc.",
    "98B379": "Apple, Inc.",
    "049D05": "Apple, Inc.",
    "544E45": "Private",
    "54083B": "IEEE Registration Authority",
    "54EF43": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D81BB5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8464DD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68A46A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8DB88": "Dell Inc.",
    "7845C4": "Dell Inc.",
    "5C260A": "Dell Inc.",
    "F48E38": "Dell Inc.",
    "28F10E": "Dell Inc.",
    "109836": "Dell Inc.",
    "001422": "Dell Inc.",
    "0015C5": "Dell Inc.",
    "90B11C": "Dell Inc.",
    "64006A": "Dell Inc.",
    "E4434B": "Dell Inc.",
    "001E4F": "Dell Inc.",
    "0026B9": "Dell Inc.",
    "C81F66": "Dell Inc.",
    "D067E5": "Dell Inc.",
    "209727": "TELTONIKA NETWORKS UAB",
    "001AE8": "Unify Software and Solutions GmbH & Co. KG",
    "2CFE4F": "Xiaomi Communications Co Ltd",
    "907E43": "zte corporation",
    "943EE4": "WiSA Technologies Inc",
    "A8BD3A": "UNION MAN TECHNOLOGY CO.,LTD",
    "48CAC6": "UNION MAN TECHNOLOGY CO.,LTD",
    "D43844": "UNION MAN TECHNOLOGY CO.,LTD",
    "D45347": "Merytronic 2012, S.L.",
    "6C3C8C": "Dell Inc.",
    "C45AB1": "Dell Inc.",
    "A8B028": "CubePilot Pty Ltd",
    "407F5F": "Juniper Networks",
    "2CEA7F": "Dell Inc.",
    "F0D4E2": "Dell Inc.",
    "CC483A": "Dell Inc.",
    "30D042": "Dell Inc.",
    "8C04BA": "Dell Inc.",
    "E454E8": "Dell Inc.",
    "A4BB6D": "Dell Inc.",
    "E01F6A": "Huawei Device Co., Ltd.",
    "00566D": "Huawei Device Co., Ltd.",
    "90CC7A": "Huawei Device Co., Ltd.",
    "8CC58C": "ShenZhen Elsky Technology Co.,LTD",
    "2C704F": "zte corporation",
    "645234": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "5C8816": "Rockwell Automation",
    "381672": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "6C29D2": "Cisco Systems, Inc",
    "08CC81": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "C0DD8A": "Meta Platforms Technologies, LLC",
    "CCA174": "Meta Platforms Technologies, LLC",
    "18F46B": "Telenor Connexion AB",
    "28EA0B": "Microsoft Corporation",
    "0815AE": "China Mobile Group Device Co.,Ltd.",
    "04B4FE": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "F45433": "Rockwell Automation",
    "34C0F9": "Rockwell Automation",
    "D07B6F": "Zhuhai Yunmai Technology Co.,Ltd",
    "04A526": "Nokia",
    "20BA36": "u-blox AG",
    "D80AE6": "zte corporation",
    "489E9D": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "A41437": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "F84DFC": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "849A40": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "C0517E": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "2CA59C": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "40ACBF": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "98F112": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "989DE5": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "3C1BF8": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "784558": "Ubiquiti Inc",
    "AC8BA9": "Ubiquiti Inc",
    "9C65EE": "DZS Inc.",
    "9C5416": "Cisco Systems, Inc",
    "A0AF12": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6096A4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48BDA7": "Honor Device Co., Ltd.",
    "EC2150": "vivo Mobile Communication Co., Ltd.",
    "30B64F": "Juniper Networks",
    "08B258": "Juniper Networks",
    "F4A739": "Juniper Networks",
    "4C16FC": "Juniper Networks",
    "C8E7F0": "Juniper Networks",
    "7C2586": "Juniper Networks",
    "0017CB": "Juniper Networks",
    "001F12": "Juniper Networks",
    "0024DC": "Juniper Networks",
    "5C5EAB": "Juniper Networks",
    "7819F7": "Juniper Networks",
    "9C05D6": "Ubiquiti Inc",
    "28704E": "Ubiquiti Inc",
    "00121E": "Juniper Networks",
    "0010DB": "Juniper Networks",
    "002283": "Juniper Networks",
    "100E7E": "Juniper Networks",
    "44F477": "Juniper Networks",
    "2C2172": "Juniper Networks",
    "4C9614": "Juniper Networks",
    "5C6AEC": "IEEE Registration Authority",
    "CC9F7A": "Chiun Mai Communication System, Inc",
    "5414A7": "Nanjing Qinheng Microelectronics Co., Ltd.",
    "4CE705": "Siemens Industrial Automation Products Ltd., Chengdu",
    "0C7274": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "80DB17": "Juniper Networks",
    "B8F015": "Juniper Networks",
    "E4233C": "Juniper Networks",
    "98868B": "Juniper Networks",
    "005828": "Axon Networks Inc.",
    "FC3342": "Juniper Networks",
    "3C8C93": "Juniper Networks",
    "0C8126": "Juniper Networks",
    "182AD3": "Juniper Networks",
    "94BF94": "Juniper Networks",
    "380716": "FREEBOX SAS",
    "68228E": "Juniper Networks",
    "D8539A": "Juniper Networks",
    "F8C116": "Juniper Networks",
    "880AA3": "Juniper Networks",
    "EC3873": "Juniper Networks",
    "C00380": "Juniper Networks",
    "20D80B": "Juniper Networks",
    "4C6D58": "Juniper Networks",
    "408F9D": "Juniper Networks",
    "AC78D1": "Juniper Networks",
    "A0A3B3": "Espressif Inc.",
    "34987A": "Espressif Inc.",
    "685932": "Sunitec Enterprise Co.,Ltd",
    "B4B9E6": "eero inc.",
    "30B216": "Hitachi Energy Germany AG",
    "7495A7": "Keyence Corporation",
    "88625D": "BITNETWORKS CO.,LTD",
    "C836A3": "GERTEC BRASIL LTDA",
    "F4EE31": "Cisco Systems, Inc",
    "141A97": "Apple, Inc.",
    "4045C4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "74872E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C1EB5": "Apple, Inc.",
    "AC86A3": "Apple, Inc.",
    "001FD6": "Shenzhen Allywll",
    "80D266": "ScaleFlux",
    "E42150": "Shanghai Chint low voltage electrical technology Co.,Ltd.",
    "D404E6": "Broadcom Limited",
    "28EBA6": "Nex-T LLC",
    "C0B3C8": "LLC \"NTC Rotek\"",
    "CC4D74": "Fujian Newland Payment Technology Co., Ltd.",
    "10BE99": "Netberg",
    "60DEF4": "Shenzhen iComm Semiconductor CO.,LTD",
    "84D352": "Tonly Technology Co. Ltd ",
    "F8D758": "Veratron AG",
    "24FE9A": "CyberTAN Technology Inc.",
    "B4CBB8": "Universal Electronics, Inc.",
    "A031EB": "Semikron Elektronik GmbH & Co. KG",
    "BC32B2": "Samsung Electronics Co.,Ltd",
    "EC8A48": "Arista Networks",
    "8C5DB2": "IEEE Registration Authority",
    "88CE3F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ECF8D0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30DF17": "ALPSALPINE CO,.LTD",
    "D8028A": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "B83C28": "Apple, Inc.",
    "3C6D89": "Apple, Inc.",
    "AC4500": "Apple, Inc.",
    "E86538": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "242A04": "Cisco Systems, Inc",
    "141AAA": "Metal Work SpA",
    "CCDEDE": "Nokia",
    "748669": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C80A35": "Qingdao Hisense Smart Life Technology Co., Ltd",
    "24952F": "Google, Inc.",
    "047056": "Arcadyan Corporation",
    "886D2D": "Huawei Device Co., Ltd.",
    "40FF40": "GloquadTech",
    "106838": "AzureWave Technology Inc.",
    "B894D9": "Texas Instruments",
    "7CE269": "Texas Instruments",
    "0804B4": "Texas Instruments",
    "30AF7E": "Texas Instruments",
    "84B1E4": "Apple, Inc.",
    "54EBE9": "Apple, Inc.",
    "AC1615": "Apple, Inc.",
    "EC7379": "Apple, Inc.",
    "FC2A46": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "9CFA3C": "Daeyoung Electronics",
    "80616C": "New H3C Technologies Co., Ltd",
    "40B607": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "0C3526": "Microsoft Corporation",
    "28CF51": "Nintendo Co.,Ltd",
    "FCB0DE": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "F44D5C": "Zyxel Communications Corporation",
    "1CC316": "Xiamen Milesight IoT Co., Ltd.",
    "3C6A48": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "5CB12E": "Cisco Systems, Inc",
    "044A6A": "niliwi nanjing big data Co,.Ltd",
    "64C6D2": "Seiko Epson Corporation",
    "481F66": "China Mobile Group Device Co.,Ltd.",
    "782459": "Alcatel-Lucent Enterprise",
    "A46C24": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78071C": "Green Energy Options Ltd",
    "1C8BEF": "Beijing Xiaomi Electronics Co.,Ltd",
    "28E297": "Shanghai InfoTM Microelectronics Co.,Ltd",
    "58F85C": "LLC Proizvodstvennaya Kompania \"TransService\"",
    "BC6BFF": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "304449": "PLATH Signal Products GmbH & Co. KG",
    "ECC3B0": "zte corporation",
    "087B12": "Sagemcom Broadband SAS",
    "B457E6": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "BCBD84": "zte corporation",
    "50D45C": "Amazon Technologies Inc.",
    "00301A": "SMARTBRIDGES PTE. LTD.",
    "70033F": "Pimax Technology(ShangHai)Co.,Ltd",
    "80F1A4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D0CF0E": "Sagemcom Broadband SAS",
    "08085C": "Luna Products",
    "44D506": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "2C69CC": "Valeo Detection Systems ",
    "D46137": "IEEE Registration Authority",
    "C435D9": "Apple, Inc.",
    "ACC906": "Apple, Inc.",
    "04BC6D": "Apple, Inc.",
    "2C8AC7": "Ubee Interactive Co., Limited",
    "80DECC": "HYBE Co.,LTD",
    "A0ED6D": "Ubee Interactive Co., Limited",
    "B08BBE": "ABL GmbH",
    "C4EB42": "Sagemcom Broadband SAS",
    "F8345A": "Hitron Technologies. Inc",
    "70C932": "Dreame Technology (Suzhou) Limited",
    "744D6D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "301984": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8C249": "AMPERE COMPUTING LLC",
    "648CBB": "Texas Instruments",
    "74B839": "Texas Instruments",
    "C4D36A": "Texas Instruments",
    "988924": "Texas Instruments",
    "341593": "Ruckus Wireless",
    "0CB937": "Ubee Interactive Co., Limited",
    "440CEE": "Robert Bosch Elektronikai Kft.",
    "A017F1": "Allwinner Technology Co., Ltd",
    "2C6F4E": "Hubei Yuan Times Technology Co.,Ltd.",
    "90F82E": "Amazon Technologies Inc.",
    "488F4C": "shenzhen trolink  Technology Co.,Ltd",
    "A8F1B2": "Allwinner Technology Co., Ltd",
    "FCE9D8": "Amazon Technologies Inc.",
    "642943": "D-Link Corporation",
    "585B69": "TVT CO., LTD",
    "B8B2F7": "DRIMAES INC.",
    "C49894": "IEEE Registration Authority",
    "8CFADD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F89A25": "HUAWEI TECHNOLOGIES CO.,LTD",
    "083A8D": "Espressif Inc.",
    "842388": "Ruckus Wireless",
    "386504": "Honor Device Co., Ltd.",
    "F8710C": "Xiaomi Communications Co Ltd",
    "3C135A": "Xiaomi Communications Co Ltd",
    "C084E0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "900117": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E0B668": "zte corporation",
    "A49DDD": "Samsung Electronics Co.,Ltd",
    "6C5563": "Samsung Electronics Co.,Ltd",
    "109F4F": "New H3C Intelligence Terminal Co., Ltd.",
    "389E80": "zte corporation",
    "880FA2": "Sagemcom Broadband SAS",
    "EC96BF": "eSystems MTG GmbH",
    "D42000": "IEEE Registration Authority",
    "704DE7": "TECNO MOBILE LIMITED",
    "74D873": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "30DE4B": "TP-Link Corporation Limited",
    "1C2285": "Serrature Meroni SpA",
    "D492B9": "ORION NOVA, S.L.",
    "001BB5": "Cherry GmbH",
    "68AAC4": "Altice Labs S.A.",
    "DC97E6": "Sagemcom Broadband SAS",
    "68B8BB": "Beijing Xiaomi Electronics Co.,Ltd",
    "14AC60": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "80FD7B": "BLU Products Inc",
    "589351": "Huawei Device Co., Ltd.",
    "2CA7EF": "OnePlus Technology (Shenzhen) Co., Ltd",
    "50FDD5": "SJI Industry Company",
    "D0F4F7": "Huawei Device Co., Ltd.",
    "D4E98A": "Intel Corporate",
    "28808A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "346679": "HUAWEI TECHNOLOGIES CO.,LTD",
    "448CAB": "Beijing Flitlink Vientiane Technology Co., LTD",
    "682624": "Ergatta",
    "A0218B": "ACE Antenna Co., ltd",
    "2084F5": "Yufei Innovation Software(Shenzhen) Co., Ltd.",
    "142103": "Calix Inc.",
    "40B15C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4D496": "Shenzhen Excelsecu Data Technology Co.,Ltd",
    "2C326A": "Apple, Inc.",
    "6CB133": "Apple, Inc.",
    "28E71D": "Arista Networks",
    "88287D": "AltoBeam (China) Inc.",
    "6C97AA": "AI TECHNOLOGY CO.,LTD.",
    "6C6567": "BELIMO Automation AG",
    "4C24CE": "Sichuan AI-Link Technology Co., Ltd.",
    "1C97FB": "CoolBitX Ltd.",
    "445ADF": "MIKAMI & CO., LTD.",
    "086E9C": "Huawei Device Co., Ltd.",
    "D0DAD7": "Apple, Inc.",
    "C4ACAA": "Apple, Inc.",
    "4416FA": "Samsung Electronics Co.,Ltd",
    "FC671F": "Tuya Smart Inc.",
    "78C57D": "Zyxel Communications Corporation",
    "78F1C6": "Cisco Systems, Inc",
    "341B2D": "Cisco Systems, Inc",
    "843C4C": "Robert Bosch SRL",
    "6C72E2": "amitek",
    "38A89B": "Fiberhome Telecommunication Technologies Co.,LTD",
    "6437A4": "TOKYOSHUHA CO.,LTD.",
    "90CAFA": "Google, Inc.",
    "10E840": "ZOWEE TECHNOLOGY(HEYUAN) CO., LTD.",
    "4405E8": "twareLAB",
    "8C3592": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "A0465A": "Motorola Mobility LLC, a Lenovo Company",
    "64B5F2": "Samsung Electronics Co.,Ltd",
    "8C06CB": "Toradex AG",
    "303D51": "IEEE Registration Authority",
    "70B306": "Apple, Inc.",
    "B8496D": "Apple, Inc.",
    "9C924F": "Apple, Inc.",
    "200E2B": "Apple, Inc.",
    "F0D793": "Apple, Inc.",
    "04D9C8": "Hon Hai Precision Industry Co., Ltd.",
    "DCAA43": "Shenzhen Terca Information Technology Co., Ltd.",
    "00FBF9": "Axiado Corporation",
    "DC0B09": "Cisco Systems, Inc",
    "08F3FB": "Cisco Systems, Inc",
    "A036BC": "ASUSTek COMPUTER INC.",
    "840BBB": "MitraStar Technology Corp.",
    "447147": "Beijing Xiaomi Electronics Co.,Ltd",
    "F4BBC7": "vivo Mobile Communication Co., Ltd.",
    "986610": "zte corporation",
    "1C24CD": "ASKEY COMPUTER CORP",
    "A8DC5A": "Digital Watchdog",
    "A0FB83": "Honor Device Co., Ltd.",
    "906560": "EM Microelectronic",
    "28BC05": "BLU Products Inc",
    "1CEF03": "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.",
    "58B03E": "Nintendo Co.,Ltd",
    "AC4E65": "Fiberhome Telecommunication Technologies Co.,LTD",
    "BC5DA3": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "E00871": "Dongguan Liesheng Electronic Co., Ltd.",
    "9C956E": "Microchip Technology Inc.",
    "1073EB": "Infiniti Electro-Optics",
    "2CA774": "Texas Instruments",
    "187A3E": "Silicon Laboratories",
    "300505": "Intel Corporate",
    "B0DCEF": "Intel Corporate",
    "7413EA": "Intel Corporate",
    "A88038": "ShenZhen MovingComm Technology Co., Limited",
    "DCF31C": "Texas Instruments",
    "544538": "Texas Instruments",
    "78C213": "Sagemcom Broadband SAS",
    "4022D8": "Espressif Inc.",
    "184E03": "HMD Global Oy",
    "38F0C8": "Logitech",
    "34FE1C": "CHOUNG HWA TECH CO.,LTD",
    "60CF69": "meerecompany",
    "4C627B": "SmartCow AI Technologies Taiwan Ltd.",
    "BC7B72": "Huawei Device Co., Ltd.",
    "38FDF5": "Renesas Electronics (Penang) Sdn. Bhd.",
    "3C26E4": "Cisco Systems, Inc",
    "3891B7": "Cisco Systems, Inc",
    "345DA8": "Cisco Systems, Inc",
    "38127B": "Crenet Labs Co., Ltd.",
    "B0E45C": "Samsung Electronics Co.,Ltd",
    "D868A0": "Samsung Electronics Co.,Ltd",
    "04292E": "Samsung Electronics Co.,Ltd",
    "F82B7F": "Huawei Device Co., Ltd.",
    "40C3BC": "Huawei Device Co., Ltd.",
    "2853E0": "Sintela Ltd",
    "C412EC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC360C": "Hitron Technologies. Inc",
    "687FF0": "TP-Link Corporation Limited",
    "BC4CA0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "74342B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7891DE": "Guangdong ACIGA Science&Technology Co.,Ltd",
    "88F2BD": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "6C0831": "ANALOG SYSTEMS",
    "A47EFA": "Withings",
    "E0806B": "Xiaomi Communications Co Ltd",
    "7050E7": "IEEE Registration Authority",
    "70AC08": "Silicon Laboratories",
    "200BCF": "Nintendo Co.,Ltd",
    "7070FC": "GOLD&WATER INDUSTRIAL LIMITED",
    "98D93D": "Demant Enterprise A/S",
    "B4A678": "Zhejiang Tmall Technology Co., Ltd.",
    "ACC4BD": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "E4B633": "Wuxi Stars Microsystem Technology Co., Ltd",
    "085104": "Huawei Device Co., Ltd.",
    "785B64": "Huawei Device Co., Ltd.",
    "54E15B": "Huawei Device Co., Ltd.",
    "8C9806": "SHENZHEN SEI ROBOTICS CO.,LTD",
    "200889": "zte corporation",
    "ACA32F": "Solidigm Technology",
    "4C9E6C": "BROADEX TECHNOLOGIES CO.LTD",
    "200B16": "Texas Instruments",
    "8801F9": "Texas Instruments",
    "F85548": "Texas Instruments",
    "68E74A": "Texas Instruments",
    "70A6BD": "Honor Device Co., Ltd.",
    "2426D6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC819C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D0989C": "ConMet",
    "846993": "HP Inc.",
    "C4A10E": "IEEE Registration Authority",
    "542F04": "Shanghai Longcheer Technology Co., Ltd.",
    "746F88": "zte corporation",
    "1CA410": "Amlogic, Inc.",
    "B41974": "Apple, Inc.",
    "6095BD": "Apple, Inc.",
    "EC1127": "Texas Instruments",
    "04E892": "SHENNAN CIRCUITS CO.,LTD",
    "98C81C": "BAYTEC LIMITED",
    "748469": "Nintendo Co.,Ltd",
    "8CCBDF": "FOXCONN INTERCONNECT TECHNOLOGY ",
    "BCE9E2": "Brocade Communications Systems LLC",
    "18A59C": "IEEE Registration Authority",
    "10961A": "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.",
    "74718B": "Apple, Inc.",
    "70317F": "Apple, Inc.",
    "A4CF99": "Apple, Inc.",
    "4C2EB4": "Apple, Inc.",
    "34AD61": "CELESTICA INC.",
    "2C553C": "Vecima Networks Inc.",
    "5443B2": "Espressif Inc.",
    "04B97D": "AiVIS Co., Itd.",
    "C4C063": "New H3C Technologies Co., Ltd",
    "0025CA": "Laird Connectivity",
    "E06CC5": "Huawei Device Co., Ltd.",
    "30963B": "Huawei Device Co., Ltd.",
    "8C6BDB": "Huawei Device Co., Ltd.",
    "10DA49": "Huawei Device Co., Ltd.",
    "60183A": "Huawei Device Co., Ltd.",
    "18C007": "Huawei Device Co., Ltd.",
    "ACBF71": "Bose Corporation",
    "B48351": "Intel Corporate",
    "BCF4D4": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "74563C": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "EC551C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E0798D": "Silicon Laboratories",
    "D89C8E": "Comcast Cable Corporation",
    "ACD31D": "Cisco Meraki",
    "B04A6A": "Samsung Electronics Co.,Ltd",
    "A8798D": "Samsung Electronics Co.,Ltd",
    "5CEDF4": "Samsung Electronics Co.,Ltd",
    "64C269": "eero inc.",
    "BC6E6D": "EM Microelectronic",
    "CCF305": "SHENZHEN TIAN XING CHUANG ZHAN ELECTRONIC CO.,LTD",
    "34BD20": "Hangzhou Hikrobot Technology Co., Ltd.",
    "AC2AA1": "Cisco Systems, Inc",
    "F8E94F": "Cisco Systems, Inc",
    "30894A": "Intel Corporate",
    "98597A": "Intel Corporate",
    "64497D": "Intel Corporate",
    "B8D61A": "Espressif Inc.",
    "283DC2": "Samsung Electronics Co.,Ltd",
    "00D49E": "Intel Corporate",
    "DC9A7D": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "84F1D0": "EHOOME IOT PRIVATE LIMITED",
    "30CB36": "Belden Singapore Pte. Ltd.",
    "30BB7D": "OnePlus Technology (Shenzhen) Co., Ltd",
    "6C67EF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88693D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00991D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "208BD1": "NXP Semiconductor (Tianjin) LTD.",
    "2C07F6": "SKG Health Technologies Co., Ltd. ",
    "3868BE": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "3492C2": "Square Route Co., Ltd.",
    "6CA401": "essensys plc",
    "8C1759": "Intel Corporate",
    "B83FD2": "Mellanox Technologies, Inc.",
    "283E0C": "Preferred Robotics, Inc.",
    "04BC9F": "Calix Inc.",
    "28A53F": "vivo Mobile Communication Co., Ltd.",
    "8C49B6": "vivo Mobile Communication Co., Ltd.",
    "4851D0": "Jiangsu Xinsheng Intelligent Technology Co., Ltd. ",
    "8077A4": "TECNO MOBILE LIMITED",
    "00C30A": "Xiaomi Communications Co Ltd",
    "8852EB": "Xiaomi Communications Co Ltd",
    "B42875": "Futecho Solutions Private Limited",
    "302364": "Nokia Shanghai Bell Co., Ltd.",
    "0C1C1A": "eero inc.",
    "D4FB8E": "Apple, Inc.",
    "B0DE28": "Apple, Inc.",
    "7C131D": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "D49FDD": "Huawei Device Co., Ltd.",
    "00D8A2": "Huawei Device Co., Ltd.",
    "D850A1": "Hunan Danuo Technology Co.,LTD",
    "441AAC": "Elektrik Uretim AS EOS",
    "7C6CF0": "Shenzhen TINNO Mobile Technology Corp.",
    "342840": "Apple, Inc.",
    "18E7B0": "Apple, Inc.",
    "50578A": "Apple, Inc.",
    "D833B7": "Sagemcom Broadband SAS",
    "2C784C": "Iton Technology Corp.",
    "488759": "Xiaomi Communications Co Ltd",
    "AC1E9E": "Xiaomi Communications Co Ltd",
    "E0AEA2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44456F": "SHENZHEN ONEGA TECHNOLOGY CO.,LTD",
    "E8FDF8": "Shanghai High-Flying Electronics  Technology Co., Ltd",
    "DC6B1B": "Huawei Device Co., Ltd.",
    "98CA20": "Shanghai SIMCOM Ltd.",
    "D06DC9": "Sagemcom Broadband SAS",
    "DC152D": "China Mobile Group Device Co.,Ltd.",
    "0857FB": "Amazon Technologies Inc.",
    "00D0FC": "GRANITE MICROSYSTEMS",
    "34976F": "Rootech, Inc.",
    "C01803": "HP Inc.",
    "04CF4B": "Intel Corporate",
    "F897A9": "Ericsson AB",
    "6C6C0F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC9B2D": "China Mobile Group Device Co.,Ltd.",
    "FC38C4": "China Grand Communications Co.,Ltd.",
    "5CF51A": "Zhejiang Dahua Technology Co., Ltd.",
    "101D51": "8Mesh Networks Limited",
    "4CFE2E": "DongGuan Siyoto Electronics Co., Ltd       ",
    "A81306": "vivo Mobile Communication Co., Ltd.",
    "1C34F1": "Silicon Laboratories",
    "AC93C4": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "000688": "Telways Communication Co., Ltd.",
    "D01B1F": "OHSUNG",
    "CC7D5B": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "749552": "Xuzhou WIKA Electronics Control Technology Co., Ltd.",
    "0C4EC0": "Maxlinear Inc",
    "CC8CBF": "Tuya Smart Inc.",
    "901195": "Amazon Technologies Inc.",
    "44EA30": "Samsung Electronics Co.,Ltd",
    "845F04": "Samsung Electronics Co.,Ltd",
    "9C2F9D": "Liteon Technology Corporation",
    "A4F33B": "zte corporation",
    "30E7BC": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "7CB566": "Intel Corporate",
    "50E9DF": "Quectel Wireless Solutions Co.,Ltd.",
    "6C4BB4": "HUMAX Co., Ltd.",
    "589BF7": "Hefei Radio Communication Technology Co., Ltd ",
    "50A030": "IEEE Registration Authority",
    "60109E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40329D": "Union Image Co.,Ltd",
    "E8FA23": "Huawei Device Co., Ltd.",
    "EC3A52": "Huawei Device Co., Ltd.",
    "441B88": "Apple, Inc.",
    "80045F": "Apple, Inc.",
    "9C3E53": "Apple, Inc.",
    "C889F3": "Apple, Inc.",
    "10B9C4": "Apple, Inc.",
    "000E24": "Huwell Technology Inc.",
    "4C7713": "Renesas Electronics (Penang) Sdn. Bhd.",
    "D8AA59": "Tonly Technology Co. Ltd ",
    "64CF13": "Weigao Nikkiso(Weihai)Dialysis Equipment Co.,Ltd",
    "9C7403": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "3CECDE": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "9C6B00": "ASRock Incorporation",
    "54F82A": "u-blox AG",
    "D43538": "Beijing Xiaomi Mobile Software Co., Ltd",
    "10CE02": "Amazon Technologies Inc.",
    "040D84": "Silicon Laboratories",
    "4C195D": "Sagemcom Broadband SAS",
    "0013DC": "IBTEK INC.",
    "D84A2B": "zte corporation",
    "D0F99B": "zte corporation",
    "F41399": "Aerospace new generation communications Co.,Ltd",
    "6026AA": "Cisco Systems, Inc",
    "5C3192": "Cisco Systems, Inc",
    "641ABA": "Dryad Networks GmbH",
    "349454": "Espressif Inc.",
    "A83A48": "Ubiqcom India Pvt Ltd",
    "C85ACF": "HP Inc.",
    "FC29E3": "Infinix mobility limited",
    "9C1C6D": "HEFEI DATANG STORAGE TECHNOLOGY CO.,LTD",
    "88C227": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8054D9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9C8566": "Wingtech Mobile Communications Co.,Ltd.",
    "7070AA": "Amazon Technologies Inc.",
    "D413F8": "Peplink International Ltd.",
    "98672E": "Skullcandy",
    "480286": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "94DEB8": "Silicon Laboratories",
    "A4438C": "ARRIS Group, Inc.",
    "642FC7": "New H3C Technologies Co., Ltd",
    "94A9A8": "Texas Instruments",
    "48B423": "Amazon Technologies Inc.",
    "9C00D3": "SHENZHEN IK WORLD Technology Co., Ltd",
    "282D06": "AMPAK Technology,Inc.",
    "202B20": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "E8CAC8": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "D419F6": "NXP Semiconductor (Tianjin) LTD.",
    "64BF6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30499E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9CEC61": "Huawei Device Co., Ltd.",
    "747069": "Huawei Device Co., Ltd.",
    "14FB70": "Huawei Device Co., Ltd.",
    "84D3D5": "Huawei Device Co., Ltd.",
    "001117": "CESNET",
    "B03366": "vivo Mobile Communication Co., Ltd.",
    "7CC225": "Samsung Electronics Co.,Ltd",
    "FC7692": "Semptian Co.,Ltd.",
    "FCAFBE": "TireCheck GmbH",
    "886EE1": "Erbe Elektromedizin GmbH",
    "C475AB": "Intel Corporate",
    "80B745": "The Silk Technologies ILC LTD",
    "4CABF8": "ASKEY COMPUTER CORP",
    "C8418A": "Samsung Electronics.,LTD",
    "DC973A": "Verana Networks",
    "2066FD": "CONSTELL8 NV",
    "0CB088": "AITelecom",
    "686725": "Espressif Inc.",
    "202027": "Shenzhen Sundray Technologies Company Limited",
    "A0A309": "Apple, Inc.",
    "5C50D9": "Apple, Inc.",
    "884D7C": "Apple, Inc.",
    "A8FE9D": "Apple, Inc.",
    "1848BE": "Amazon Technologies Inc.",
    "F89E94": "Intel Corporate",
    "644212": "Shenzhen Water World Information Co.,Ltd.",
    "C403A8": "Intel Corporate",
    "D463DE": "vivo Mobile Communication Co., Ltd.",
    "0C9043": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "B0F7C4": "Amazon Technologies Inc.",
    "8C5646": "LG Electronics",
    "CC7190": "VIETNAM POST AND TELECOMMUNICATION INDUSTRY TECHNOLOGY JOINT STOCK COMPANY",
    "1400E9": "Mitel Networks Corporation",
    "541F8D": "zte corporation",
    "2CF1BB": "zte corporation",
    "DC2C6E": "Routerboard.com",
    "18A9A6": "Nebra Ltd",
    "547D40": "Powervision Tech Inc.",
    "C0D7AA": "Arcadyan Corporation",
    "6C0F0B": "China Mobile Group Device Co.,Ltd.",
    "D4B7D0": "Ciena Corporation",
    "ACACE2": "CHANGHONG (HONGKONG) TRADING LIMITED",
    "4C8D53": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC2CE6": "Cisco Systems, Inc",
    "CCED4D": "Cisco Systems, Inc",
    "48BD4A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8D4E0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0406F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3429EF": "Qingdao Haier Technology Co.,Ltd",
    "A0B53C": "Technicolor Delivery Technologies Belgium NV",
    "C4BDE5": "Intel Corporate",
    "6CB0FD": "Shenzhen Xinghai Iot Technology Co.,Ltd",
    "FC5F49": "Zhejiang Dahua Technology Co., Ltd.",
    "3035C5": "Huawei Device Co., Ltd.",
    "004F1A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7433A6": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "2C0DA7": "Intel Corporate",
    "649E31": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24CD8D": "Murata Manufacturing Co., Ltd.",
    "541D61": "YEESTOR Microelectronics Co., Ltd",
    "700971": "Samsung Electronics Co.,Ltd",
    "D01B49": "Samsung Electronics Co.,Ltd",
    "F820A9": "Huawei Device Co., Ltd.",
    "A0FF22": "SHENZHEN APICAL TECHNOLOGY CO., LTD",
    "5C0214": "Beijing Xiaomi Mobile Software Co., Ltd",
    "E4936A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "4877BD": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F856C3": "zte corporation",
    "00B0EC": "EACEM",
    "6CD3EE": "ZIMI CORPORATION",
    "04CD15": "Silicon Laboratories",
    "E42805": "Pivotal Optics",
    "381F8D": "Tuya Smart Inc.",
    "208C47": "Tenstorrent Inc",
    "3453D2": "Sagemcom Broadband SAS",
    "18EF3A": "Sichuan AI-Link Technology Co., Ltd.",
    "785EA2": "Sunitec Enterprise Co.,Ltd",
    "DCF4CA": "Apple, Inc.",
    "7CFC16": "Apple, Inc.",
    "88B945": "Apple, Inc.",
    "B0BE83": "Apple, Inc.",
    "10321D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38FDF8": "Cisco Systems, Inc",
    "3C9FC3": "Beijing Sinead Technology Co., Ltd. ",
    "608FA4": "Nokia Solutions and Networks GmbH & Co. KG",
    "7876D9": "EXARA Group",
    "94A408": "Shenzhen Trolink Technology CO, LTD",
    "D0F865": "ITEL MOBILE LIMITED",
    "7C8437": "China Post Communications Equipment Co., Ltd.",
    "B84D43": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "A84A63": "TPV Display Technology(Xiamen) Co.,Ltd.",
    "243FAA": "Huawei Device Co., Ltd.",
    "D867D3": "Huawei Device Co., Ltd.",
    "48474B": "Huawei Device Co., Ltd.",
    "806F1C": "Huawei Device Co., Ltd.",
    "6815D3": "Zaklady Elektroniki i Mechaniki Precyzyjnej R&G S.A.",
    "0080E7": "Leonardo UK Ltd",
    "7CC177": "INGRAM MICRO SERVICES",
    "E0A258": "Wanbang Digital Energy Co.,Ltd",
    "4CBCE9": "LG Innotek",
    "385C76": "PLANTRONICS, INC.",
    "C89F1A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D86852": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC6490": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9C50D1": "Murata Manufacturing Co., Ltd.",
    "140020": " LongSung Technology (Shanghai) Co.,Ltd.   ",
    "18E1DE": "Chengdu ChipIntelli Technology Co., Ltd",
    "8CF681": "Silicon Laboratories",
    "7C66EF": "Hon Hai Precision IND.CO.,LTD",
    "441793": "Espressif Inc.",
    "745CFA": "Shenzhen Shunrui Gaojie Technology Co., Ltd.",
    "5CDFB8": "Shenzhen Unionmemory Information System Limited",
    "E00AF6": "Liteon Technology Corporation",
    "2C1A05": "Cisco Systems, Inc",
    "B40ECF": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "00A388": "SKY UK LIMITED",
    "503DEB": "Zhejiang Tmall Technology Co., Ltd.",
    "F87FA5": "GREATEK",
    "F83869": "LG Electronics",
    "F05ECD": "Texas Instruments",
    "2C0547": "Shenzhen Phaten Tech. LTD",
    "E0C58F": "China Mobile IOT Company Limited",
    "30B930": "zte corporation",
    "949869": "zte corporation",
    "785005": "MOKO TECHNOLOGY Ltd",
    "701AD5": "Openpath Security, Inc.",
    "FC2E19": "China Mobile Group Device Co.,Ltd.",
    "50F908": "Wizardlab Co., Ltd.",
    "AC764C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "5C58E6": "Palo Alto Networks",
    "EC94CB": "Espressif Inc.",
    "EC97E0": "Hangzhou Ezviz Software Co.,Ltd.",
    "D0B66F": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "F0B61E": "Intel Corporate",
    "C84D44": "Shenzhen Jiapeng Huaxiang Technology Co.,Ltd",
    "2C3A91": "Huawei Device Co., Ltd.",
    "64F705": "Huawei Device Co., Ltd.",
    "60CE41": "HUAWEI TECHNOLOGIES CO.,LTD",
    "281709": "HUAWEI TECHNOLOGIES CO.,LTD",
    "606EE8": "Xiaomi Communications Co Ltd",
    "0CE159": "Shenzhen iStartek Technology Co., Ltd.",
    "50C2E8": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "3009C0": "Motorola Mobility LLC, a Lenovo Company",
    "B04692": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "7086CE": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "500F59": "STMicrolectronics International NV",
    "90B57F": "Shenzhen iComm Semiconductor CO.,LTD",
    "E42AAC": "Microsoft Corporation",
    "C0F827": "Rapidmax Technology Corporation",
    "001413": "Trebing & Himstedt Proze\u00dfautomation GmbH & Co. KG",
    "000799": "Tipping Point Technologies, Inc.",
    "DC8D8A": "Nokia Solutions and Networks GmbH & Co. KG",
    "98B177": "LANDIS + GYR",
    "A029BD": "Team Group Inc",
    "64DCDE": "ZheJiang FuChunJiang Information Technology Co.,Ltd",
    "00919E": "Intel Corporate",
    "241D48": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "287777": "zte corporation",
    "6006E3": "Apple, Inc.",
    "D43DF3": "Zyxel Communications Corporation",
    "20579E": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "C884CF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20E7B6": "Universal Electronics, Inc.",
    "808ABD": "Samsung Electronics Co.,Ltd",
    "3CCD57": "Beijing Xiaomi Mobile Software Co., Ltd",
    "245E48": "Apple, Inc.",
    "08C729": "Apple, Inc.",
    "C4C36B": "Apple, Inc.",
    "E8A730": "Apple, Inc.",
    "E023D7": "Sleep Number",
    "806A10": "Whisker Labs - Ting",
    "60E32B": "Intel Corporate",
    "D8BE65": "Amazon Technologies Inc.",
    "106FD9": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "584D42": "Dragos, Inc.",
    "B8DAE8": "Huawei Device Co., Ltd.",
    "B8AE1D": "Guangzhou Xingyi Electronic Technology Co.,Ltd",
    "28DEA8": "zte corporation",
    "ACEC85": "eero inc.",
    "E06781": "Dongguan Liesheng Electronic Co., Ltd.",
    "60A5E2": "Intel Corporate",
    "D4548B": "Intel Corporate",
    "1009F9": "Amazon Technologies Inc.",
    "906976": "Withrobot Inc.",
    "FC9257": "Renesas Electronics (Penang) Sdn. Bhd.",
    "D03E7D": "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.",
    "60B6E1": "Texas Instruments",
    "6C79B8": "Texas Instruments",
    "8CA399": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "60DB15": "New H3C Technologies Co., Ltd",
    "5CA721": "New H3C Technologies Co., Ltd",
    "98F217": "Castlenet Technology Inc.",
    "446752": "Wistron INFOCOMM (Zhongshan) CORPORATION",
    "A82316": "Nokia",
    "38E39F": "Motorola Mobility LLC, a Lenovo Company",
    "D01E1D": "SaiNXT Technologies LLP",
    "78BB88": "Maxio Technology (Hangzhou) Ltd.",
    "608A10": "Microchip Technology Inc.",
    "A864F1": "Intel Corporate",
    "A42A71": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "84EBEF": "Cisco Systems, Inc",
    "908060": "Nilfisk A/S",
    "70A6CC": "Intel Corporate",
    "D48DD9": "Meld Technology, Inc",
    "582FF7": "Sagemcom Broadband SAS",
    "ACE77B": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "6C433C": "TECNO MOBILE LIMITED",
    "48F8FF": "CHENGDU KT ELECTRONIC HI-TECH CO.,LTD",
    "E8C1E8": "Shenzhen Xiao Bi En Culture Education Technology Co.,Ltd.",
    "7C70DB": "Intel Corporate",
    "8C94CC": "SFR",
    "2CDDE9": "Arista Networks",
    "709741": "Arcadyan Corporation",
    "E408E7": "Quectel Wireless Solutions Co.,Ltd.",
    "8CFD15": "Imagine Marketing Private Limited",
    "C4FF22": "Huawei Device Co., Ltd.",
    "A0A0DC": "Huawei Device Co., Ltd.",
    "C4808A": "Cloud Diagnostics Canada ULC",
    "40F420": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "645D92": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "CCA260": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "9C6121": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "E04FBD": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "ACD829": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "5CA176": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "4CD3AF": "HMD Global Oy",
    "C85142": "Samsung Electronics Co.,Ltd",
    "14223B": "Google, Inc.",
    "683A48": "SAMJIN Co., Ltd.",
    "F44588": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0C952": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "10E4C2": "Samsung Electronics Co.,Ltd",
    "0C8DCA": "Samsung Electronics Co.,Ltd",
    "CC0DE7": "B METERS S.R.L.",
    "4456E2": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "3C3712": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "C8BB81": "Huawei Device Co., Ltd.",
    "1C472F": "Huawei Device Co., Ltd.",
    "205E64": "Huawei Device Co., Ltd.",
    "F4419E": "Huawei Device Co., Ltd.",
    "90F9B7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C47469": "BT9",
    "F46FA4": "Physik Instrumente GmbH & Co. KG",
    "78F235": "Sichuan AI-Link Technology Co., Ltd.",
    "54AED2": "CSL Dualcom Ltd",
    "80B655": "Intel Corporate",
    "0CCB0C": "iSYS RTS GmbH",
    "D0E042": "Cisco Systems, Inc",
    "F44EE3": "Intel Corporate",
    "60577D": "eero inc.",
    "CC242E": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "F0016E": "Tianyi Telecom Terminals Company Limited",
    "EC0BAE": "Hangzhou BroadLink Technology Co.,Ltd",
    "802511": "ITEL MOBILE LIMITED",
    "3802DE": "Sercomm Corporation.",
    "9C760E": "Apple, Inc.",
    "94EA32": "Apple, Inc.",
    "50F4EB": "Apple, Inc.",
    "28C709": "Apple, Inc.",
    "D0CFD8": "Huizhou Boshijie Technology Co.,Ltd",
    "5C625A": "CANON INC.",
    "7C0A3F": "Samsung Electronics Co.,Ltd",
    "08AA89": "zte corporation",
    "04D60E": "FUNAI ELECTRIC CO., LTD.",
    "A4352D": "TRIZ Networks corp.",
    "C8BFFE": "Huawei Device Co., Ltd.",
    "082FE9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "984874": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1CA681": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8A660": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B881FA": "Apple, Inc.",
    "082E36": "Huawei Device Co., Ltd.",
    "049F81": "NETSCOUT SYSTEMS INC",
    "00808C": "NETSCOUT SYSTEMS INC",
    "20A7F9": "SHENZHEN OLANBOA TECHNOLOGY CO., LTD",
    "C006C3": "TP-Link Corporation Limited",
    "184859": "Castlenet Technology Inc.",
    "14517E": "New H3C Technologies Co., Ltd",
    "083A38": "New H3C Technologies Co., Ltd",
    "F8ABE5": "shenzhen worldelite electronics co., LTD",
    "08BB3C": "Flextronics Tech.(Ind) Pvt Ltd",
    "04BA1C": "Huawei Device Co., Ltd.",
    "7C3D2B": "Huawei Device Co., Ltd.",
    "18C241": "SonicWall",
    "58454C": "Ericsson AB",
    "2CBEEB": "Nothing Technology Limited",
    "7409AC": "Quext, LLC",
    "F4C7AA": "Marvell Semiconductors",
    "70F088": "Nintendo Co.,Ltd",
    "CCED21": "Nokia Shanghai Bell Co., Ltd.",
    "F04A02": "Cisco Systems, Inc",
    "10394E": "Hisense broadband multimedia technology Co.,Ltd",
    "509A46": "Safetrust Inc",
    "10D561": "Tuya Smart Inc.",
    "F0A3B2": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "FC4DA6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B414E6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC9929": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C1512": "Shenzhen Huanhu Technology Co.,Ltd",
    "4024B2": "Sichuan AI-Link Technology Co., Ltd.",
    "640D22": "LG Electronics (Mobile Communications)",
    "2CFDB3": "Tonly Technology Co. Ltd ",
    "9C9567": "Huawei Device Co., Ltd.",
    "A47B1A": "Huawei Device Co., Ltd.",
    "58AEF1": "Fiberhome Telecommunication Technologies Co.,LTD",
    "787A6F": "Juice Technology AG",
    "C0D46B": "Huawei Device Co., Ltd.",
    "8C31E2": "DAYOUPLUS",
    "E86CC7": "IEEE Registration Authority",
    "641759": "Intellivision Holdings, LLC",
    "504348": "ThingsMatrix Inc.",
    "D85982": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48B25D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A41B34": "China Mobile Group Device Co.,Ltd.",
    "8045DD": "Intel Corporate",
    "147D05": "SERCOMM PHILIPPINES INC",
    "24E853": "LG Innotek",
    "482952": "Sagemcom Broadband SAS",
    "A47E36": "EM Microelectronic",
    "98387D": "ITRONIC  TECHNOLOGY CO . , LTD .",
    "2CCE1E": "Cloudtronics Pty Ltd",
    "ACD618": "OnePlus Technology (Shenzhen) Co., Ltd",
    "6C5640": "BLU Products Inc",
    "C8D884": "Universal Electronics, Inc.",
    "B4A25C": "Cambium Networks Limited",
    "2C71FF": "Amazon Technologies Inc.",
    "48785E": "Amazon Technologies Inc.",
    "20C74F": "SensorPush",
    "DC9A8E": "Nanjing Cocomm electronics co., LTD",
    "DC0077": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B05DD4": "ARRIS Group, Inc.",
    "184CAE": "CONTINENTAL",
    "C4D0E3": "Intel Corporate",
    "F021E0": "eero inc.",
    "307C4A": "Huawei Device Co., Ltd.",
    "145051": "SHARP Corporation",
    "3CC786": "DONGGUAN HUARONG COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "88C3E5": "Betop Techonologies ",
    "E428A4": "Prama India Private Limited",
    "943A91": "Amazon Technologies Inc.",
    "80C501": "OctoGate IT Security Systems GmbH",
    "14D19E": "Apple, Inc.",
    "40C711": "Apple, Inc.",
    "5C7017": "Apple, Inc.",
    "E45E1B": "Google, Inc.",
    "C430CA": "SD Biosensor",
    "28052E": "Dematic Corp",
    "8CEC7B": "Apple, Inc.",
    "0040DD": "HONG TECHNOLOGIES",
    "408C1F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "002DB3": "AMPAK Technology,Inc.",
    "E4FD45": "Intel Corporate",
    "28C87C": "zte corporation",
    "04F03E": "Huawei Device Co., Ltd.",
    "78E22C": "Huawei Device Co., Ltd.",
    "C0D026": "Huawei Device Co., Ltd.",
    "502DFB": "IGShare Co., Ltd.",
    "40EE15": "Zioncom Electronics (Shenzhen) Ltd.",
    "606C63": "Hitron Technologies. Inc",
    "00F361": "Amazon Technologies Inc.",
    "A4423B": "Intel Corporate",
    "ECBE5F": "Vestel Elektronik San ve Tic. A.S.",
    "F42A7D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "04F993": "Infinix mobility limited",
    "BCBD9E": "ITEL MOBILE LIMITED",
    "70CF49": "Intel Corporate",
    "4851C5": "Intel Corporate",
    "6C0309": "Cisco Systems, Inc",
    "BCD295": "Cisco Systems, Inc",
    "F03965": "Samsung Electronics Co.,Ltd",
    "EC7CB6": "Samsung Electronics Co.,Ltd",
    "58A639": "Samsung Electronics Co.,Ltd",
    "A4E57C": "Espressif Inc.",
    "304E1B": "Huawei Device Co., Ltd.",
    "74504E": "New H3C Technologies Co., Ltd",
    "6C0DE1": "Dongguan Cannice Precision Manufacturing Co., Ltd.",
    "8C0FC9": "Huawei Device Co., Ltd.",
    "E8D765": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC208C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "885A85": "Wistron Neweb Corporation",
    "D80093": "Aurender Inc.",
    "20EFBD": "Roku, Inc",
    "BC6D05": "Dusun Electron Co.,Ltd.",
    "C0E018": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CE747": "HUAWEI TECHNOLOGIES CO.,LTD",
    "089AC7": "zte corporation",
    "744CA1": "Liteon Technology Corporation",
    "B4B291": "LG Electronics",
    "A8FFBA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CC2C6": "TP-Link Corporation Limited",
    "A0024A": "IEEE Registration Authority",
    "C86C3D": "Amazon Technologies Inc.",
    "389052": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C0F6EC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70617B": "Cisco Systems, Inc",
    "04E77E": "We Corporation Inc.",
    "74ECB2": "Amazon Technologies Inc.",
    "4C52EC": "SOLARWATT GmbH",
    "146B9A": "zte corporation",
    "30D941": "Raydium Semiconductor Corp.",
    "687627": "Zhuhai Dingzhi Electronic Technology Co., Ltd",
    "948ED3": "Arista Networks",
    "E01FED": "Nokia Shanghai Bell Co., Ltd.",
    "7040FF": "Huawei Device Co., Ltd.",
    "34D693": "Huawei Device Co., Ltd.",
    "24E4C8": "Fiberhome Telecommunication Technologies Co.,LTD",
    "249493": "FibRSol Global Network Limited",
    "28D044": "Shenzhen Xinyin technology company",
    "9C9AC0": "LEGO System A/S",
    "A09F10": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "201B88": "Dongguan Liesheng Electronic Co., Ltd.",
    "74B7B3": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "DC9BD6": "TCT mobile ltd",
    "DC8C1B": "vivo Mobile Communication Co., Ltd.",
    "3CBDC5": "Arcadyan Corporation",
    "A8DA0C": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "5860D8": "ARRIS Group, Inc.",
    "50A5DC": "ARRIS Group, Inc.",
    "A4AAFE": "Huawei Device Co., Ltd.",
    "F83B7E": "Huawei Device Co., Ltd.",
    "8815C5": "Huawei Device Co., Ltd.",
    "9C28B3": "Apple, Inc.",
    "A07817": "Apple, Inc.",
    "5C8730": "Apple, Inc.",
    "B41BB0": "Apple, Inc.",
    "58D349": "Apple, Inc.",
    "F434F0": "Apple, Inc.",
    "F85329": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B08C75": "Apple, Inc.",
    "983F60": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C03FDD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "303235": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "00301F": "OPTICAL NETWORKS, INC.",
    "0858A5": "Beijing Vrv Software Corpoaration Limited.",
    "7CC294": "Beijing Xiaomi Mobile Software Co., Ltd",
    "E0E2E6": "Espressif Inc.",
    "9C7370": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68D48B": "Hailo Technologies Ltd.",
    "40D4BD": "SK Networks Service CO., LTD.",
    "841B77": "Intel Corporate",
    "5C10C5": "Samsung Electronics Co.,Ltd",
    "A8407D": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "FC4B57": "Peerless Instrument Division of Curtiss-Wright ",
    "18EE86": "Novatel Wireless Solutions, Inc.",
    "3CD2E5": "New H3C Technologies Co., Ltd",
    "001391": "OUEN CO.,LTD.",
    "582429": "Google, Inc.",
    "C43772": "Virtuozzo International GmbH",
    "CC3B27": "TECNO MOBILE LIMITED",
    "E80AEC": "Jiangsu Hengtong Optic-Electric Co., LTD",
    "C4DE7B": "Huawei Device Co., Ltd.",
    "6C1A75": "Huawei Device Co., Ltd.",
    "6C7637": "Huawei Device Co., Ltd.",
    "E8EB34": "Cisco Systems, Inc",
    "A4178B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94B271": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78058C": "mMax Communications, Inc.",
    "C4A72B": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "D0C31E": "JUNGJIN Electronics Co.,Ltd",
    "D810CB": "Andrea Informatique",
    "FC1928": "Actions Microelectronics Co., Ltd",
    "24016F": "Huawei Device Co., Ltd.",
    "CCF55F": "E FOCUS INSTRUMENTS INDIA PRIVATE LIMITED",
    "087C39": "Amazon Technologies Inc.",
    "141333": "AzureWave Technology Inc.",
    "C0C9E3": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F88C21": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C4278C": "Huawei Device Co., Ltd.",
    "7C73EB": "Huawei Device Co., Ltd.",
    "E48F1D": "Huawei Device Co., Ltd.",
    "C03C59": "Intel Corporate",
    "4C3BDF": "Microsoft Corporation",
    "84E342": "Tuya Smart Inc.",
    "FCCD2F": "IEEE Registration Authority",
    "808FE8": "Intelbras",
    "18CC18": "Intel Corporate",
    "E0913C": "Kyeungin CNS Co., Ltd.",
    "D05919": "zte corporation",
    "00B881": "New platforms LLC",
    "B098BC": "Huawei Device Co., Ltd.",
    "5C3400": "Hisense Electric Co.,Ltd",
    "B43161": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "10D7B0": "Sagemcom Broadband SAS",
    "341CF0": "Xiaomi Communications Co Ltd",
    "CC4F5C": "IEEE Registration Authority",
    "FC6DD1": "APRESIA Systems, Ltd.",
    "30AFCE": "vivo Mobile Communication Co., Ltd.",
    "B4FBE3": "AltoBeam (China) Inc.",
    "188740": "Xiaomi Communications Co Ltd",
    "6C09BF": "Fiberhome Telecommunication Technologies Co.,LTD",
    "64F947": "Senscomm Semiconductor Co., Ltd.",
    "DC00B0": "FREEBOX SAS",
    "707414": "Murata Manufacturing Co., Ltd.",
    "A0764E": "Espressif Inc.",
    "C8E600": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4D51B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2491BB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "645E10": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ECF22B": "TECNO MOBILE LIMITED",
    "309610": "Huawei Device Co., Ltd.",
    "445943": "zte corporation",
    "343654": "zte corporation",
    "30578E": "eero inc.",
    "FCB3BC": "Intel Corporate",
    "C816DA": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "B04414": "New H3C Technologies Co., Ltd",
    "748F3C": "Apple, Inc.",
    "40F946": "Apple, Inc.",
    "1CFE2B": "Amazon Technologies Inc.",
    "A4AE12": "Hon Hai Precision Industry Co., Ltd.",
    "9C9D7E": "Beijing Xiaomi Mobile Software Co., Ltd",
    "DCA3A2": "Feng mi(Beijing)technology co., LTD",
    "EC570D": "AFE Inc.",
    "7C25DA": "FN-LINK TECHNOLOGY LIMITED",
    "3059B7": "Microsoft",
    "44A54E": "Qorvo International Pte. Ltd.",
    "A8698C": "Oracle Corporation ",
    "A4AC0F": "Huawei Device Co., Ltd.",
    "CCFF90": "Huawei Device Co., Ltd.",
    "003054": "Castlenet Technology Inc.",
    "E45F01": "Raspberry Pi Trading Ltd",
    "001368": "Saab Danmark A/S",
    "A83512": "Huawei Device Co., Ltd.",
    "B030C8": "Teal Drones, Inc.",
    "D4F337": "Xunison Ltd.",
    "48A2B8": "Chengdu Vision-Zenith Tech.Co,.Ltd",
    "6C442A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A47CC9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C40D96": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58E873": "HANGZHOU DANGBEI NETWORK TECH.Co.,Ltd",
    "C0B8E6": "Ruijie Networks Co.,LTD",
    "7412B3": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "9447B0": "BEIJING ESWIN COMPUTING TECHNOLOGY CO., LTD",
    "E45AD4": "Eltex Enterprise Ltd.",
    "84225E": "SHENZHEN TECHNEWCHIP TECHNOLOGY CO.,LTD.",
    "F85C7D": "Shenzhen Honesty Electronics Co.,Ltd.",
    "44AF28": "Intel Corporate",
    "D4A651": "Tuya Smart Inc.",
    "842AFD": "HP Inc.",
    "A0681C": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "90F644": "Huawei Device Co., Ltd.",
    "846B48": "ShenZhen EepuLink Co., Ltd.",
    "B460ED": "Beijing Xiaomi Mobile Software Co., Ltd",
    "00300B": "mPHASE Technologies, Inc.",
    "C01850": "Quanta Computer Inc.",
    "4427F3": "70mai Co.,Ltd.",
    "30CC21": "zte corporation",
    "646C80": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "0026C0": "EnergyHub",
    "80EACA": "Dialog Semiconductor Hellas SA",
    "000896": "Printronix, Inc.",
    "0025DC": "Sumitomo Electric Industries, Ltd",
    "047D7B": "Quanta Computer Inc.",
    "089E01": "Quanta Computer Inc.",
    "A81E84": "Quanta Computer Inc.",
    "DC41A9": "Intel Corporate",
    "8C941F": "Cisco Systems, Inc",
    "687DB4": "Cisco Systems, Inc",
    "BC7F7B": "Huawei Device Co., Ltd.",
    "F0FAC7": "Huawei Device Co., Ltd.",
    "B46F2D": "Wahoo Fitness",
    "5C857E": "IEEE Registration Authority",
    "8803E9": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "DCEF80": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0CDC7E": "Espressif Inc.",
    "2098D8": "Shenzhen Yingdakang Technology CO., LTD",
    "182649": "Intel Corporate",
    "34C93D": "Intel Corporate",
    "1C012D": "Ficer Technology",
    "3CA62F": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "7CC77E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A0CFF5": "zte corporation",
    "98B3EF": "Huawei Device Co., Ltd.",
    "50F958": "Huawei Device Co., Ltd.",
    "C0A36E": "SKY UK LIMITED",
    "6032B1": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "7CFD6B": "Xiaomi Communications Co Ltd",
    "0005C9": "LG Innotek",
    "ACF108": "LG Innotek",
    "345840": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C647A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E41F7B": "Cisco Systems, Inc",
    "E86DCB": "Samsung Electronics Co.,Ltd",
    "304950": "IEEE Registration Authority",
    "189552": "1MORE",
    "98C7A4": "Shenzhen HS Fiber Communication Equipment CO., LTD",
    "00BED5": "New H3C Technologies Co., Ltd",
    "AC5FEA": "OnePlus Technology (Shenzhen) Co., Ltd",
    "044AC6": "Aipon Electronics Co., Ltd",
    "C0FFA8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC1499": "Aimore Acoustics Incorporation",
    "88948F": "Xi'an Zhisensor Technologies Co.,Ltd",
    "5CD5B5": "Shenzhen WiSiYiLink Technology Co.,Ltd",
    "C87EA1": "TCL MOKA International Limited",
    "642C0F": "vivo Mobile Communication Co., Ltd.",
    "08ED9D": "TECNO MOBILE LIMITED",
    "209E79": "Universal Electronics, Inc.",
    "5CFE9E": "Wiwynn Corporation Tainan Branch",
    "881C95": "ITEL MOBILE LIMITED",
    "002003": "PIXEL POWER LTD.",
    "F02E51": "Casa Systems",
    "C8E42F": "Technical Research Design and Development",
    "680AE2": "Silicon Laboratories",
    "F46942": "ASKEY COMPUTER CORP",
    "08E9F6": "AMPAK Technology,Inc.",
    "CCC261": "IEEE Registration Authority",
    "3066D0": "Huawei Device Co., Ltd.",
    "3CB233": "Huawei Device Co., Ltd.",
    "640BD7": "Apple, Inc.",
    "A8913D": "Apple, Inc.",
    "0C3B50": "Apple, Inc.",
    "4CC95E": "Samsung Electronics Co.,Ltd",
    "FCA5D0": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C85BA0": "Shenzhen Qihu Intelligent Technology Company Limited",
    "E0BE03": "Lite-On Network Communication (Dongguan) Limited",
    "A48873": "Cisco Systems, Inc",
    "A4BDC4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C9157": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04CB88": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "B8804F": "Texas Instruments",
    "000F2D": "CHUNG-HSIN ELECTRIC & MACHINERY MFG.CORP.",
    "B46293": "Samsung Electronics Co.,Ltd",
    "000DA1": "MIRAE ITS Co.,LTD.",
    "241407": "Xiamen Sigmastar Technology Ltd.",
    "B47947": "Nutanix",
    "98CBA4": "Benchmark Electronics",
    "183CB7": "Huawei Device Co., Ltd.",
    "5C9012": "Owl Cyber Defense Solutions, LLC",
    "38F7CD": "IEEE Registration Authority",
    "9C2976": "Intel Corporate",
    "04F8F8": "Edgecore Networks Corporation",
    "481693": "Lear Corporation GmbH",
    "18F697": "Axiom Memory Solutions, Inc.",
    "5C6BD7": "Foshan VIOMI Electric Appliance Technology Co. Ltd.",
    "1848CA": "Murata Manufacturing Co., Ltd.",
    "A84025": "Oxide Computer Company",
    "CCD9AC": "Intel Corporate",
    "347DF6": "Intel Corporate",
    "8022A7": "NEC Platforms, Ltd.",
    "5C2316": "Squirrels Research Labs LLC",
    "042144": "Sunitec Enterprise Co.,Ltd",
    "64E172": "Shenzhen Qihoo Intelligent Technology Co.,Ltd",
    "CC0DF2": "Motorola Mobility LLC, a Lenovo Company",
    "94434D": "Ciena Corporation",
    "A027B6": "Samsung Electronics Co.,Ltd",
    "103917": "Samsung Electronics Co.,Ltd",
    "9880EE": "Samsung Electronics Co.,Ltd",
    "90EEC7": "Samsung Electronics Co.,Ltd",
    "1029AB": "Samsung Electronics Co.,Ltd",
    "184ECB": "Samsung Electronics Co.,Ltd",
    "60F43A": "Edifier International",
    "FCB698": "Cambridge Industries(Group) Co.,Ltd.",
    "C4D8F3": "iZotope",
    "0001CD": "ARtem",
    "58A87B": "Fitbit, Inc.",
    "C489ED": "Solid Optics EU N.V.",
    "488F5A": "Routerboard.com",
    "100645": "Sagemcom Broadband SAS",
    "AC67B2": "Espressif Inc.",
    "985949": "LUXOTTICA GROUP S.P.A.",
    "68332C": "KENSTEL NETWORKS LIMITED",
    "D8787F": "Ubee Interactive Co., Limited",
    "9C28BF": "Continental Automotive Czech Republic s.r.o.",
    "782B46": "Intel Corporate",
    "785773": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC6089": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0006B3": "Diagraph Corporation",
    "843E92": "HUAWEI TECHNOLOGIES CO.,LTD",
    "708CB6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50464A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4A402": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9CBD6E": "DERA Co., Ltd",
    "4C3329": "Sweroam",
    "34E5EC": "Palo Alto Networks",
    "00B7A8": "Heinzinger electronic GmbH",
    "34CFF6": "Intel Corporate",
    "D4D2D6": "FN-LINK TECHNOLOGY LIMITED",
    "105072": "Sercomm Corporation.",
    "98FAA7": "INNONET",
    "AC4A67": "Cisco Systems, Inc",
    "900A84": "Mellanox Technologies, Inc.",
    "EC7949": "FUJITSU LIMITED",
    "B8E3B1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "980D51": "Huawei Device Co., Ltd.",
    "DC333D": "Huawei Device Co., Ltd.",
    "285471": "Huawei Device Co., Ltd.",
    "B88E82": "Huawei Device Co., Ltd.",
    "70CA97": "Ruckus Wireless",
    "0887C6": "INGRAM MICRO SERVICES",
    "3CB53D": "HUNAN GOKE MICROELECTRONICS CO.,LTD",
    "AC3A67": "Cisco Systems, Inc",
    "086BD1": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "345180": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "F47335": "Logitech Far East",
    "90ADFC": "Telechips, Inc.",
    "5CA62D": "Cisco Systems, Inc",
    "402B69": "Kumho Electric Inc.",
    "A40801": "Amazon Technologies Inc.",
    "00ADD5": "Huawei Device Co., Ltd.",
    "905D7C": "New H3C Technologies Co., Ltd",
    "A4CFD2": "Ubee Interactive Co., Limited",
    "A8A097": "ScioTeq bvba",
    "BC1695": "zte corporation",
    "E8E98E": "SOLAR controls s.r.o.",
    "842E14": "Silicon Laboratories",
    "0020E1": "ALAMAR ELECTRONICS",
    "08F458": "Huawei Device Co., Ltd.",
    "EC97B2": "SUMEC Machinery & Electric Co.,Ltd.",
    "28FA7A": "Zhejiang Tmall Technology Co., Ltd.",
    "64F6BB": "Fibocom Wireless Inc.",
    "4410FE": "Huizhou Foryou General Electronics Co., Ltd.",
    "EC316D": "Hansgrohe",
    "BC542F": "Intel Corporate",
    "6CD94C": "vivo Mobile Communication Co., Ltd.",
    "AC1ED0": "Temic Automotive Philippines Inc.",
    "0088BA": "NC&C",
    "1005E1": "Nokia",
    "343794": "Hamee Corp.",
    "4CA64D": "Cisco Systems, Inc",
    "CC7F75": "Cisco Systems, Inc",
    "20E874": "Apple, Inc.",
    "D03FAA": "Apple, Inc.",
    "7CAB60": "Apple, Inc.",
    "44C65D": "Apple, Inc.",
    "187EB9": "Apple, Inc.",
    "D4DC09": "Mist Systems, Inc.",
    "380146": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "7894E8": "Radio Bridge",
    "40EC99": "Intel Corporate",
    "98006A": "zte corporation",
    "1C97C5": "Ynomia Pty Ltd",
    "5CC1D7": "Samsung Electronics Co.,Ltd",
    "889655": "Zitte corporation",
    "74B6B6": "eero inc.",
    "7CDFA1": "Espressif Inc.",
    "F4A4D6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCE14F": "BRK Brands, Inc.",
    "A84D4A": "Audiowise Technology Inc.",
    "547FBC": "iodyne",
    "5C27D4": "Shenzhen Qihu Intelligent Technology Company Limited",
    "74D83E": "Intel Corporate",
    "3CBF60": "Apple, Inc.",
    "A87EEA": "Intel Corporate",
    "B00AD5": "zte corporation",
    "08D23E": "Intel Corporate",
    "20114E": "MeteRSit S.R.L.",
    "FCF29F": "China Mobile Iot Limited company",
    "F81F32": "Motorola Mobility LLC, a Lenovo Company",
    "047295": "Apple, Inc.",
    "D446E1": "Apple, Inc.",
    "141459": "Vodafone Italia S.p.A.",
    "5043B9": "OktoInform RUS",
    "88B6EE": "Dish Technologies Corp",
    "484EFC": "ARRIS Group, Inc.",
    "1C919D": "Dongguan Liesheng Electronic Co., Ltd.",
    "2C5741": "Cisco Systems, Inc",
    "B0B353": "IEEE Registration Authority",
    "487B5E": "SMT TELECOMM HK",
    "18300C": "Hisense Electric Co.,Ltd",
    "44A56E": "NETGEAR",
    "AC15F4": "Apple, Inc.",
    "78D162": "Apple, Inc.",
    "08F8BC": "Apple, Inc.",
    "90A25B": "Apple, Inc.",
    "88A479": "Apple, Inc.",
    "00104F": "Oracle Corporation ",
    "F44955": "MIMO TECH Co., Ltd.",
    "3C28A6": "Alcatel-Lucent Enterprise (China)",
    "5050A4": "Samsung Electronics Co.,Ltd",
    "8086D9": "Samsung Electronics Co.,Ltd",
    "386A77": "Samsung Electronics Co.,Ltd",
    "0809C7": "Zhuhai Unitech Power Technology Co., Ltd.",
    "041DC7": "zte corporation",
    "001A4D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "8CAAB5": "Espressif Inc.",
    "58961D": "Intel Corporate",
    "68AFFF": "Shanghai Cambricon Information Technology Co., Ltd.",
    "402C76": "IEEE Registration Authority",
    "80C955": "Redpine Signals, Inc.",
    "7885F4": "Huawei Device Co., Ltd.",
    "88541F": "Google, Inc.",
    "900CC8": "Google, Inc.",
    "68215F": "Edgecore Networks Corporation",
    "44C7FC": "Huawei Device Co., Ltd.",
    "001FD0": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "18C04D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "0025A9": "Shanghai Embedway Information Technologies Co.,Ltd",
    "5C68D0": "Aurora Innovation Inc.",
    "10364A": "Boston Dynamics",
    "000BCC": "JUSAN, S.A.",
    "00E059": "CONTROLLED ENVIRONMENTS, LTD.",
    "00B810": "Yichip\u00a0Microelectronics (Hangzhou) Co.,Ltd",
    "A4B239": "Cisco Systems, Inc",
    "548D5A": "Intel Corporate",
    "D01C3C": "TECNO MOBILE LIMITED",
    "B89A2A": "Intel Corporate",
    "DC21E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000A23": "Parama Networks Inc",
    "3843E5": "Grotech Inc",
    "FC1BD1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "582575": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28DEE5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C8E29": "Arcadyan Corporation",
    "A0224E": "IEEE Registration Authority",
    "18E777": "vivo Mobile Communication Co., Ltd.",
    "0C7A15": "Intel Corporate",
    "94D6DB": "NexFi",
    "C49878": "SHANGHAI MOAAN INTELLIGENT TECHNOLOGY CO.,LTD",
    "B4ECF2": "Shanghai Listent Medical Tech Co., Ltd.",
    "C4954D": "IEEE Registration Authority",
    "A44BD5": "Xiaomi Communications Co Ltd",
    "64956C": "LG Electronics",
    "4077A9": "New H3C Technologies Co., Ltd",
    "64694E": "Texas Instruments",
    "F83331": "Texas Instruments",
    "6C06D6": "Huawei Device Co., Ltd.",
    "6C639C": "ARRIS Group, Inc.",
    "1CBFC0": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "34495B": "Sagemcom Broadband SAS",
    "801609": "Sleep Number",
    "78B46A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6CEBB6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CF55B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "643139": "IEEE Registration Authority",
    "E83F67": "Huawei Device Co., Ltd.",
    "3446EC": "Huawei Device Co., Ltd.",
    "40A6B7": "Intel Corporate",
    "C8F319": "LG Electronics (Mobile Communications)",
    "14876A": "Apple, Inc.",
    "E0B55F": "Apple, Inc.",
    "F8FFC2": "Apple, Inc.",
    "E0EB40": "Apple, Inc.",
    "5C710D": "Cisco Systems, Inc",
    "98AF65": "Intel Corporate",
    "009096": "ASKEY COMPUTER CORP",
    "442295": "China Mobile Iot Limited company",
    "E85A8B": "Xiaomi Communications Co Ltd",
    "80751F": "SKY UK LIMITED",
    "F855CD": "Visteon Corporation",
    "441847": "HUNAN SCROWN ELECTRONIC INFORMATION TECH.CO.,LTD",
    "2C3AFD": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "AC61B9": "WAMA Technology Limited",
    "1CC1BC": "Yichip\u00a0Microelectronics (Hangzhou) Co.,Ltd",
    "AC8B9C": "Primera Technology, Inc.",
    "00AB48": "eero inc.",
    "0030B7": "Teletrol Systems, Inc.",
    "2474F7": "GoPro",
    "7CD566": "Amazon Technologies Inc.",
    "686350": "Hella India Automotive Pvt Ltd",
    "18703B": "Huawei Device Co., Ltd.",
    "D89E61": "Huawei Device Co., Ltd.",
    "347E00": "Huawei Device Co., Ltd.",
    "5CE50C": "Beijing Xiaomi Mobile Software Co., Ltd",
    "F8893C": "Inventec Appliances Corp.",
    "749EF5": "Samsung Electronics Co.,Ltd",
    "68BFC4": "Samsung Electronics Co.,Ltd",
    "B82ADC": "EFR Europ\u00e4ische Funk-Rundsteuerung GmbH",
    "04B1A1": "Samsung Electronics Co.,Ltd",
    "CC464E": "Samsung Electronics Co.,Ltd",
    "306F07": "Nations Technologies Inc.",
    "A0DF15": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4AD34": "Routerboard.com",
    "000733": "DANCONTROL Engineering",
    "A85E45": "ASUSTek COMPUTER INC.",
    "9CFFC2": "AVI Systems GmbH",
    "44D878": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "7897C3": "DINGXIN INFORMATION TECHNOLOGY CO.,LTD",
    "64C901": "INVENTEC Corporation",
    "B43939": "Shenzhen TINNO Mobile Technology Corp.",
    "5CCD5B": "Intel Corporate",
    "84C807": "ADVA Optical Networking Ltd.",
    "10B3C6": "Cisco Systems, Inc",
    "10B3D6": "Cisco Systems, Inc",
    "A0D807": "Huawei Device Co., Ltd.",
    "2C780E": "Huawei Device Co., Ltd.",
    "34B20A": "Huawei Device Co., Ltd.",
    "98F4AB": "Espressif Inc.",
    "D8BFC0": "Espressif Inc.",
    "A0AB51": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "3C894D": "Dr. Ing. h.c. F. Porsche AG",
    "F854B8": "Amazon Technologies Inc.",
    "781735": "Nokia Shanghai Bell Co., Ltd.",
    "4C90DB": "JL Audio",
    "8CF112": "Motorola Mobility LLC, a Lenovo Company",
    "B899AE": "Shenzhen MiaoMing  Intelligent Technology Co.,Ltd",
    "E8D0B9": "Taicang T&W Electronics",
    "F4D620": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "949034": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "3C8F06": "Shenzhen Libtor Technology Co.,Ltd",
    "987A10": "Ericsson AB",
    "202681": "TECNO MOBILE LIMITED",
    "B00875": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64FB92": "PPC Broadband Inc.",
    "141346": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "542BDE": "New H3C Technologies Co., Ltd",
    "98F781": "ARRIS Group, Inc.",
    "5885A2": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "A8C0EA": "Pepwave Limited",
    "00071C": "AT&T",
    "2CF89B": "Cisco Systems, Inc",
    "847637": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC9435": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E02481": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44AEAB": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A4F05E": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "1C687E": "Shenzhen Qihu Intelligent Technology Company Limited",
    "C03656": "Fiberhome Telecommunication Technologies Co.,LTD",
    "B03E51": "SKY UK LIMITED",
    "28167F": "Xiaomi Communications Co Ltd",
    "087190": "Intel Corporate",
    "80B07B": "zte corporation",
    "C85A9F": "zte corporation",
    "500291": "Espressif Inc.",
    "001DDF": "Sunitec Enterprise Co.,Ltd",
    "8C0FFA": "Hutec co.,ltd",
    "ACFE05": "ITEL MOBILE LIMITED",
    "BCC31B": "Kygo Life A",
    "64DF10": "JingLue Semiconductor(SH) Ltd.",
    "100177": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44A191": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24526A": "Zhejiang Dahua Technology Co., Ltd.",
    "20DFB9": "Google, Inc.",
    "5CCAD3": "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.",
    "782A79": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "786559": "Sagemcom Broadband SAS",
    "50D2F5": "Beijing Xiaomi Mobile Software Co., Ltd",
    "D05F64": "IEEE Registration Authority",
    "5CE883": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C463FB": "Neatframe AS",
    "305714": "Apple, Inc.",
    "04A222": "Arcadyan Corporation",
    "04AB6A": "Chun-il Co.,Ltd.",
    "6023A4": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "A4530E": "Cisco Systems, Inc",
    "00403A": "IMPACT TECHNOLOGIES",
    "C8B1CD": "Apple, Inc.",
    "1460CB": "Apple, Inc.",
    "B8F12A": "Apple, Inc.",
    "F887F1": "Apple, Inc.",
    "28FE65": "DongGuan Siyoto Electronics Co., Ltd       ",
    "1806F5": "RAD Data Communications, Ltd.",
    "60447A": "Water-i.d. GmbH",
    "807215": "SKY UK LIMITED",
    "74D637": "Amazon Technologies Inc.",
    "7484E1": "Dongguan Haoyuan Electronics Co.,Ltd",
    "4074E0": "Intel Corporate",
    "44FB5A": "zte corporation",
    "0035FF": "Texas Instruments",
    "DC54D7": "Amazon Technologies Inc.",
    "20968A": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "8C1850": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "D8D4E6": "Hytec Inter Co., Ltd.",
    "840B7C": "Hitron Technologies. Inc",
    "889FAA": "Hella Gutmann Solutions GmbH ",
    "48A73C": "Sichuan tianyi kanghe communications co., LTD",
    "04C807": "Xiaomi Communications Co Ltd",
    "F8E7A0": "vivo Mobile Communication Co., Ltd.",
    "2CFFEE": "vivo Mobile Communication Co., Ltd.",
    "4459E3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "683F1E": "EFFECT Photonics B.V.",
    "44D3CA": "Cisco Systems, Inc",
    "F8A763": "Zhejiang Tmall Technology Co., Ltd.",
    "A49813": "ARRIS Group, Inc.",
    "6C2990": "WiZ Connected Lighting Company Limited",
    "9835ED": "HUAWEI TECHNOLOGIES CO.,LTD",
    "807693": "Newag SA",
    "BC9740": "IEEE Registration Authority",
    "084F0A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8494D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44004D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18CF24": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1449BC": "DrayTek Corp.",
    "20F478": "Xiaomi Communications Co Ltd",
    "18B6F7": "NEW POS TECHNOLOGY LIMITED",
    "C85D38": "HUMAX Co., Ltd.",
    "50F8A5": "eWBM Co., Ltd.",
    "04885F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C850CE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D89B3B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88403B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC8743": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00FA21": "Samsung Electronics Co.,Ltd",
    "7C2302": "Samsung Electronics Co.,Ltd",
    "6CAB05": "Cisco Systems, Inc",
    "5CB15F": "Oceanblue Cloud Technology Limited",
    "000BE4": "Hosiden Corporation",
    "4413D0": "zte corporation",
    "2462AB": "Espressif Inc.",
    "18AACA": "Sichuan tianyi kanghe communications co., LTD",
    "0004DF": "TERACOM TELEMATICA S.A",
    "7438B7": "CANON INC.",
    "90735A": "Motorola Mobility LLC, a Lenovo Company",
    "1C8259": "IEEE Registration Authority",
    "485DEB": "Just Add Power",
    "501395": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "18D9EF": "Shuttle Inc.",
    "88DA33": "Beijing Xiaoyuer Network Technology Co., Ltd",
    "60D248": "ARRIS Group, Inc.",
    "D49DC0": "Samsung Electronics Co.,Ltd",
    "D0196A": "Ciena Corporation",
    "84FDD1": "Intel Corporate",
    "B0700D": "Nokia",
    "5041B9": "I-O DATA DEVICE,INC.",
    "84C78F": "APS Networks GmbH",
    "C09FE1": "zte corporation",
    "80DABC": "Megafone Limited",
    "184644": "Home Control Singapore Pte Ltd",
    "002175": "Pacific Satellite International Ltd.",
    "346B5B": "New H3C Technologies Co., Ltd",
    "84E892": "Actiontec Electronics, Inc",
    "20DA22": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E0B655": "Beijing Xiaomi Electronics Co., Ltd.",
    "C821DA": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "8C0FA0": "di-soric GmbH & Co. KG",
    "78E2BD": "Vodafone Automotive S.p.A.",
    "F848FD": "China Mobile Group Device Co.,Ltd.",
    "F8BBBF": "eero inc.",
    "DCFB48": "Intel Corporate",
    "847C9B": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "3441A8": "ER-Telecom",
    "34DB9C": "Sagemcom Broadband SAS",
    "7440BE": "LG Innotek",
    "20658E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "183D5E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "889746": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "1CDE57": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E0DCFF": "Xiaomi Communications Co Ltd",
    "846FCE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "B0E71D": "Shanghai Maigantech Co.,Ltd",
    "C468D0": "VTech Telecommunications Ltd.",
    "14AEDB": "VTech Telecommunications Ltd.",
    "78DB2F": "Texas Instruments",
    "00778D": "Cisco Systems, Inc",
    "000E8C": "Siemens AG",
    "008764": "Cisco Systems, Inc",
    "FCD2B6": "IEEE Registration Authority",
    "A8E2C3": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "DC7137": "zte corporation",
    "804A14": "Apple, Inc.",
    "703C69": "Apple, Inc.",
    "4C6AF6": "HMD Global Oy",
    "489DD1": "Samsung Electronics Co.,Ltd",
    "7488BB": "Cisco Systems, Inc",
    "A4CF12": "Espressif Inc.",
    "FCB662": "IC Holdings LLC",
    "48049F": "ELECOM CO., LTD",
    "087F98": "vivo Mobile Communication Co., Ltd.",
    "C85261": "ARRIS Group, Inc.",
    "C04121": "Nokia Solutions and Networks GmbH & Co. KG",
    "70BF92": "GN Audio A/S",
    "0CA06C": "Industrial Cyber Sensing Inc.",
    "B06FE0": "Samsung Electronics Co.,Ltd",
    "44B994": "Douglas Lighting Controls",
    "04D4C4": "ASUSTek COMPUTER INC.",
    "A0510B": "Intel Corporate",
    "AC2DA9": "TECNO MOBILE LIMITED",
    "40A93F": "Pivotal Commware, Inc.",
    "002F5C": "Cisco Systems, Inc",
    "F4645D": "Toshiba",
    "C08C71": "Motorola Mobility LLC, a Lenovo Company",
    "F46F4E": "Echowell",
    "08ECF5": "Cisco Systems, Inc",
    "D07650": "IEEE Registration Authority",
    "38F85E": "HUMAX Co., Ltd.",
    "00CB51": "Sagemcom Broadband SAS",
    "C464B7": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2479F8": "KUPSON spol. s r.o.",
    "ACBB61": "YSTen Technology Co.,Ltd",
    "342003": "Shenzhen Feitengyun Technology Co.,LTD",
    "7CFD82": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "689A87": "Amazon Technologies Inc.",
    "64AE88": "Polytec GmbH",
    "00D050": "Iskratel d.o.o.",
    "F07D68": "D-Link Corporation",
    "EC4118": "XIAOMI Electronics,CO.,LTD",
    "042DB4": "First Property (Beijing) Co., Ltd Modern MOMA Branch",
    "04E0B0": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "60D0A9": "Samsung Electronics Co.,Ltd",
    "88CEFA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "002706": "YOISYS",
    "180D2C": "Intelbras",
    "98DAC4": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "D8860B": "IEEE Registration Authority",
    "00177B": "Azalea Networks inc",
    "78DAA2": "Cynosure Technologies Co.,Ltd",
    "38B19E": "IEEE Registration Authority",
    "502B98": "Es-tech International",
    "C82832": "Beijing Xiaomi Electronics Co., Ltd.",
    "38E26E": "ShenZhen Sweet Rain Electronics Co.,Ltd.",
    "70C9C6": "Cisco Systems, Inc",
    "C4346B": "Hewlett Packard",
    "48F17F": "Intel Corporate",
    "8084A9": "oshkosh Corporation",
    "4CC7D6": "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD.",
    "C80873": "Ruckus Wireless",
    "701BFB": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "04766E": "ALPSALPINE CO,.LTD",
    "AC7A4D": "ALPSALPINE CO,.LTD",
    "38C096": "ALPSALPINE CO,.LTD",
    "002643": "ALPSALPINE CO,.LTD",
    "001697": "NEC Corporation",
    "003013": "NEC Corporation",
    "049DFE": "Hivesystem",
    "BC3E07": "Hitron Technologies. Inc",
    "48FDA3": "Xiaomi Communications Co Ltd",
    "288088": "NETGEAR",
    "1C3477": "Innovation Wireless",
    "D05157": "LEAX Arkivator Telecom",
    "9458CB": "Nintendo Co.,Ltd",
    "28EC9A": "Texas Instruments",
    "0CEC84": "Shenzhen TINNO Mobile Technology Corp.",
    "9CDB07": "Yellowtec GmbH",
    "E0189F": "EM Microelectronic",
    "F81308": "Nokia",
    "F8A2D6": "Liteon Technology Corporation",
    "C4E506": "Piper Networks, Inc.",
    "30EB5A": "LANDIS + GYR",
    "F80F6F": "Cisco Systems, Inc",
    "282536": "SHENZHEN HOLATEK CO.,LTD",
    "B8A175": "Roku, Inc.",
    "FCBE7B": "vivo Mobile Communication Co., Ltd.",
    "B40FB3": "vivo Mobile Communication Co., Ltd.",
    "EC5C68": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "FC94CE": "zte corporation",
    "90869B": "zte corporation",
    "182A44": "HIROSE ELECTRONIC SYSTEM",
    "74366D": "Vodafone Italia S.p.A.",
    "CCD3C1": "Vestel Elektronik San ve Tic. A.S.",
    "745F90": "LAM Technologies",
    "A42655": "LTI Motion (Shanghai) Co., Ltd.",
    "60A730": "Shenzhen Yipinfang Internet Technology Co.,Ltd",
    "1804ED": "Texas Instruments",
    "D43260": "GoPro",
    "F4DD9E": "GoPro",
    "D4D919": "GoPro",
    "0080E3": "CORAL NETWORK CORPORATION",
    "D8F2CA": "Intel Corporate",
    "B4C62E": "Molex CMS",
    "0CD0F8": "Cisco Systems, Inc",
    "3C9BD6": "Vizio, Inc",
    "50DB3F": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "1081B4": "Hunan Greatwall Galaxy Science and Technology Co.,Ltd.",
    "00D861": "Micro-Star INTL CO., LTD.",
    "74C17D": "Infinix mobility limited",
    "8871B1": "ARRIS Group, Inc.",
    "F0AF85": "ARRIS Group, Inc.",
    "FCAE34": "ARRIS Group, Inc.",
    "B8259A": "Thalmic Labs ",
    "004279": "Sunitec Enterprise Co.,Ltd",
    "A45046": "Xiaomi Communications Co Ltd",
    "3881D7": "Texas Instruments",
    "F85B9C": "SB SYSTEMS Co.,Ltd",
    "6CA928": "HMD Global Oy",
    "DCDA80": "New H3C Technologies Co., Ltd",
    "F8501C": "Tianjin Geneuo Technology Co.,Ltd",
    "44070B": "Google, Inc.",
    "CC7286": "Xi'an Fengyu Information Technology Co., Ltd.",
    "141114": "TECNO MOBILE LIMITED",
    "00B8B3": "Cisco Systems, Inc",
    "ECF6BD": "SNCF MOBILIT\u00c9S",
    "B831B5": "Microsoft Corporation",
    "007C2D": "Samsung Electronics Co.,Ltd",
    "E89363": "Nokia",
    "7C0CF6": "Guangdong Huiwei High-tech Co., Ltd.",
    "20AD56": "Continental Automotive Systems Inc.",
    "5029F5": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "FC8F7D": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "24BE18": "DADOUTEK COMPANY LIMITED",
    "749D79": "Sercomm Corporation.",
    "00D6FE": "Cisco Systems, Inc",
    "C84782": "Areson Technology Corp.",
    "0CBF74": "Morse Micro",
    "B41D2B": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "14C213": "Apple, Inc.",
    "A4D931": "Apple, Inc.",
    "BCFED9": "Apple, Inc.",
    "808223": "Apple, Inc.",
    "70D313": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9C1D36": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CCBBFE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38B4D3": "BSH Hausgeraete GmbH",
    "283166": "vivo Mobile Communication Co., Ltd.",
    "C04004": "Medicaroid Corporation",
    "A4ED43": "IEEE Registration Authority",
    "300A60": "IEEE Registration Authority",
    "80D065": "CKS Corporation",
    "0C7C28": "Nokia Solutions and Networks GmbH & Co. KG",
    "6843D7": "Agilecom Photonics Solutions Guangdong Limited",
    "007204": "Samsung Electronics Co., Ltd. ARTIK",
    "40C81F": "Shenzhen Xinguodu Technology Co., Ltd.",
    "1459C0": "NETGEAR",
    "CC08FB": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "BCAF91": "TE Connectivity Sensor Solutions",
    "F0D7DC": "Wesine (Wuhan) Technology Co., Ltd.",
    "A81087": "Texas Instruments",
    "8C61A3": "ARRIS Group, Inc.",
    "94298D": "Shanghai AdaptComm Technology Co., Ltd.",
    "00AA6E": "Cisco Systems, Inc",
    "B86A97": "Edgecore Networks Corporation",
    "C8C2F5": "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD.",
    "F05849": "CareView Communications",
    "8CFE74": "Ruckus Wireless",
    "E43493": "HUAWEI TECHNOLOGIES CO.,LTD",
    "342912": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000A5E": "3COM",
    "00105A": "3COM",
    "006097": "3COM",
    "006008": "3COM",
    "000102": "3COM",
    "8C8F8B": "China Mobile Chongqing branch",
    "A02833": "IEEE Registration Authority",
    "001B6E": "Keysight Technologies, Inc.",
    "00040B": "3COM EUROPE LTD",
    "0890BA": "Danlaw Inc",
    "4C364E": "Panasonic Connect Co., Ltd.",
    "BCA58B": "Samsung Electronics Co.,Ltd",
    "80CEB9": "Samsung Electronics Co.,Ltd",
    "D0D3FC": "Mios, Ltd.",
    "6C6CD3": "Cisco Systems, Inc",
    "E049ED": "Audeze LLC",
    "143719": "PT Prakarsa Visi Valutama",
    "582F40": "Nintendo Co.,Ltd",
    "94A3CA": "KonnectONE, LLC",
    "E85D86": "CHANG YOW TECHNOLOGIES INTERNATIONAL CO.,LTD.",
    "604BAA": "Magic Leap, Inc.",
    "4C0143": "eero inc.",
    "A8016D": "Aiwa Corporation",
    "0440A9": "New H3C Technologies Co., Ltd",
    "48A472": "Intel Corporate",
    "B8BEF4": "devolo AG",
    "14D169": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1062E5": "Hewlett Packard",
    "0020B5": "YASKAWA ELECTRIC CORPORATION",
    "286336": "Siemens AG",
    "04C3E6": "IEEE Registration Authority",
    "58FDBE": "Shenzhen Taikaida Technology Co., Ltd",
    "244CE3": "Amazon Technologies Inc.",
    "000157": "SYSWAVE CO., LTD",
    "082525": "Xiaomi Communications Co Ltd",
    "C49500": "Amazon Technologies Inc.",
    "68DD26": "Shanghai Focus Vision Security Technology Co.,Ltd",
    "E06267": "Xiaomi Communications Co Ltd",
    "70B7AA": "vivo Mobile Communication Co., Ltd.",
    "84B31B": "Kinexon GmbH",
    "F8272E": "Mercku",
    "F460E2": "Xiaomi Communications Co Ltd",
    "E4D124": " Mojo Networks, Inc.",
    "0013A3": "Siemens Home & Office Comm. Devices",
    "F4F197": "EMTAKE Inc",
    "6CED51": "NEXCONTROL Co.,Ltd",
    "98BB99": "Phicomm (Sichuan) Co.,Ltd.",
    "002622": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "9CC950": "Baumer Holding",
    "F89910": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "50E0EF": "Nokia",
    "C0B6F9": "Intel Corporate",
    "000EEE": "Muco Industrie BV",
    "7C1C4E": "LG Innotek",
    "D8B6B7": "Comtrend Corporation",
    "2866E3": "AzureWave Technology Inc.",
    "848A8D": "Cisco Systems, Inc",
    "1CC3EB": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "60F18A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C14B4": "zte corporation",
    "3C9872": "Sercomm Corporation.",
    "40C3C6": "SnapRoute",
    "504C7E": "THE 41ST INSTITUTE OF CETC",
    "D01CBB": "Beijing Ctimes Digital Technology Co., Ltd.",
    "7487BB": "Ciena Corporation",
    "CC50E3": "Espressif Inc.",
    "DCE305": "AO ",
    "A4DA32": "Texas Instruments",
    "780473": "Texas Instruments",
    "2C4759": "Beijing MEGA preponderance Science & Technology Co. Ltd",
    "144F8A": "Intel Corporate",
    "002106": "RIM Testing Services",
    "A83E0E": "HMD Global Oy",
    "10C172": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00138A": "Qingdao GoerTek Technology Co., Ltd.",
    "A830AD": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "A41566": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "1C965A": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "401B5F": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "00151E": "ETHERNET Powerlink Standarization Group (EPSG)",
    "00111E": "ETHERNET Powerlink Standarization Group (EPSG)",
    "00409D": "DigiBoard",
    "DCE0EB": "Nanjing Aozheng Information Technology Co.Ltd",
    "BC5FF6": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "5C9656": "AzureWave Technology Inc.",
    "E06066": "Sercomm Corporation.",
    "C8E7D8": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "60D21C": "Sunnovo International Limited",
    "CC51B4": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "00C3F4": "Samsung Electronics Co.,Ltd",
    "B88AEC": "Nintendo Co.,Ltd",
    "AC3B77": "Sagemcom Broadband SAS",
    "78725D": "Cisco Systems, Inc",
    "149346": "PNI sensor corporation",
    "5CCD7C": "MEIZU Technology Co.,Ltd.",
    "EC8C9A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B48655": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D0D783": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8E552": "JUWEL Aquarium AG & Co. KG",
    "FCE66A": "Industrial Software Co",
    "7836CC": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "CCC92C": "Schindler - PORT Technology",
    "001E39": "Comsys Communication Ltd.",
    "8CCF5C": "BEFEGA GmbH",
    "B46BFC": "Intel Corporate",
    "B0FC0D": "Amazon Technologies Inc.",
    "048AE1": "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD.",
    "501D93": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C8D779": "QING DAO HAIER TELECOM CO.,LTD.",
    "70C833": "Wirepas Oy",
    "0C73EB": "IEEE Registration Authority",
    "480BB2": "IEEE Registration Authority",
    "CCC079": "Murata Manufacturing Co., Ltd.",
    "F09CD7": "Guangzhou Blue Cheetah Intelligent Technology Co., Ltd.",
    "BCE143": "Apple, Inc.",
    "10B36F": "Bowei Technology Company Limited ",
    "FC9BC6": "Sumavision Technologies Co.,Ltd",
    "C8292A": "Barun Electronics",
    "0080BA": "SPECIALIX (ASIA) PTE, LTD",
    "F0B5D1": "Texas Instruments",
    "00E000": "FUJITSU LIMITED",
    "90848B": "HDR10+ Technologies, LLC",
    "E482CC": "Jumptronic GmbH",
    "48605F": "LG Electronics (Mobile Communications)",
    "2816A8": "Microsoft Corporation",
    "F8F532": "ARRIS Group, Inc.",
    "B083D6": "ARRIS Group, Inc.",
    "B0416F": "Shenzhen Maxtang Computer Co.,Ltd",
    "0CF5A4": "Cisco Systems, Inc",
    "80C7C5": "Fiberhome Telecommunication Technologies Co.,LTD",
    "BCAB7C": "TRnP KOREA Co Ltd",
    "6C3838": "Marking System Technology Co., Ltd.",
    "9C2EA1": "Xiaomi Communications Co Ltd",
    "0C6ABC": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E019D8": "BH TECHNOLOGIES",
    "30D9D9": "Apple, Inc.",
    "6030D4": "Apple, Inc.",
    "F895EA": "Apple, Inc.",
    "18F1D8": "Apple, Inc.",
    "647033": "Apple, Inc.",
    "846878": "Apple, Inc.",
    "C8D083": "Apple, Inc.",
    "3CCD5D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C7668": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A438CC": "Nintendo Co.,Ltd",
    "74721E": "Edison Labs Inc.",
    "780F77": "HangZhou Gubei Electronics Technology Co.,Ltd",
    "8C4CDC": "PLANEX COMMUNICATIONS INC.",
    "5065F3": "Hewlett Packard",
    "3C9509": "Liteon Technology Corporation",
    "5821E9": "TWPI",
    "30FD38": "Google, Inc.",
    "64CB5D": "SIA \"TeleSet\"",
    "001386": "ABB Inc/Totalflow",
    "003C10": "Cisco Systems, Inc",
    "F041C8": "IEEE Registration Authority",
    "CC9916": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "EC7FC6": "ECCEL CORPORATION SAS",
    "F0E3DC": "Tecon MT, LLC",
    "A8DA01": "Shenzhen NUOLIJIA Digital Technology Co.,Ltd",
    "88E90F": "innomdlelab",
    "0010D8": "CALISTA",
    "002194": "Ping Communication",
    "4CABFC": "zte corporation",
    "7C2A31": "Intel Corporate",
    "0CF346": "Xiaomi Communications Co Ltd",
    "5C5AEA": "FORD",
    "000B7B": "Test-Um Inc.",
    "BCDDC2": "Espressif Inc.",
    "98D863": "Shanghai High-Flying Electronics Technology Co., Ltd",
    "7CFF4D": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "7470FD": "Intel Corporate",
    "C49F4C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C704A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C88F26": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "703A73": "Shenzhen Sundray Technologies Company Limited",
    "10F9EB": "Industria Fueguina de Relojer\u00eda Electr\u00f3nica s.a.",
    "80AD16": "Xiaomi Communications Co Ltd",
    "044EAF": "LG Innotek",
    "003074": "EQUIINET LTD.",
    "B0B3AD": "HUMAX Co., Ltd.",
    "40BD32": "Texas Instruments",
    "CC8E71": "Cisco Systems, Inc",
    "1894C6": "ShenZhen Chenyee Technology Co., Ltd.",
    "70D081": "Beijing Netpower Technologies Inc.",
    "70C94E": "Liteon Technology Corporation",
    "38F554": "HISENSE ELECTRIC CO.,LTD",
    "18A28A": "Essel-T Co., Ltd",
    "3CE824": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54B7E5": "Rayson Technology Co., Ltd.",
    "946372": "vivo Mobile Communication Co., Ltd.",
    "BC0FA7": "Ouster",
    "20365B": "Megafone Limited",
    "E8DE00": "ChongQing GuanFang Technology Co.,LTD",
    "FC643A": "Samsung Electronics Co.,Ltd",
    "F8C120": "Xi'an Link-Science Technology Co.,Ltd",
    "347E5C": "Sonos, Inc.",
    "A8515B": "Samsung Electronics Co.,Ltd",
    "B4FBF9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "506F77": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C41E9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001936": "STERLITE OPTICAL TECHNOLOGIES LIMITED",
    "F0C9D1": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "B4DE31": "Cisco Systems, Inc",
    "A44027": "zte corporation",
    "70F220": "Actiontec Electronics, Inc",
    "1C1161": "Ciena Corporation",
    "B4F7A1": "LG Electronics (Mobile Communications)",
    "70EF00": "Apple, Inc.",
    "28EDE0": "AMPAK Technology, Inc.",
    "C87765": "Tiesse SpA",
    "D0817A": "Apple, Inc.",
    "98CA33": "Apple, Inc.",
    "68AB1E": "Apple, Inc.",
    "345A06": "SHARP Corporation",
    "BCFFEB": "Motorola Mobility LLC, a Lenovo Company",
    "2C37C5": "Qingdao Haier Intelligent Home Appliance Technology Co.,Ltd",
    "CC40D0": "NETGEAR",
    "7C7630": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "9822EF": "Liteon Technology Corporation",
    "3CFB5C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "7440BB": "Hon Hai Precision Ind. Co.,Ltd.",
    "88BD45": "Samsung Electronics Co.,Ltd",
    "54FCF0": "Samsung Electronics Co.,Ltd",
    "306A85": "Samsung Electronics Co.,Ltd",
    "B89F09": "Wistron Neweb Corporation",
    "0402CA": "Shenzhen Vtsonic Co.,ltd",
    "7C7635": "Intel Corporate",
    "4CDD31": "Samsung Electronics Co.,Ltd",
    "788038": "FUNAI ELECTRIC CO., LTD.",
    "4048FD": "IEEE Registration Authority",
    "B80716": "vivo Mobile Communication Co., Ltd.",
    "1CEEC9": "Elo touch solutions",
    "001862": "Seagate Technology",
    "000C50": "Seagate Technology",
    "F045DA": "Texas Instruments",
    "A8EEC6": "Muuselabs NV/SA",
    "E4F042": "Google, Inc.",
    "20B399": "Enterasys",
    "CC2D21": "Tenda Technology Co.,Ltd.Dongguan branch",
    "C8DEC9": "Coriant",
    "D896E0": "Alibaba Cloud Computing Ltd.",
    "207852": "Nokia Solutions and Networks GmbH & Co. KG",
    "F417B8": "AirTies Wireless Networks",
    "38F73D": "Amazon Technologies Inc.",
    "004097": "DATEX DIVISION OF",
    "9C4FCF": "TCT mobile ltd",
    "C0A00D": "ARRIS Group, Inc.",
    "342AF1": "Texas Instruments",
    "44D5A5": "AddOn Computer",
    "58C17A": "Cambium Networks Limited",
    "E0AADB": "Nanjing PANENG Technology Development Co.,Ltd",
    "0005FF": "SNS Solutions, Inc.",
    "F87B20": "Cisco Systems, Inc",
    "1C330E": "PernixData",
    "B8F74A": "RCNTEC",
    "ECF451": "Arcadyan Corporation",
    "645106": "Hewlett Packard",
    "0C1539": "Apple, Inc.",
    "0C6111": "Anda Technologies SAC",
    "581243": "AcSiP Technology Corp.",
    "0022C4": "epro GmbH",
    "002128": "Oracle Corporation",
    "001C73": "Arista Networks",
    "2C8A72": "HTC Corporation",
    "6C5697": "Amazon Technologies Inc.",
    "38019F": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "245CCB": "AXIe Consortium, Inc.",
    "ECFAF4": "SenRa Tech Pvt. Ltd",
    "38AD8E": "New H3C Technologies Co., Ltd",
    "34D0B8": "IEEE Registration Authority",
    "A89FEC": "ARRIS Group, Inc.",
    "00BE9E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "54C57A": "Sunnovo International Limited",
    "D88F76": "Apple, Inc.",
    "409C28": "Apple, Inc.",
    "782D7E": "TRENDnet, Inc.",
    "741AE0": "IEEE Registration Authority",
    "B06EBF": "ASUSTek COMPUTER INC.",
    "24B209": "Avaya Inc",
    "54DF24": "Fiberhome Telecommunication Technologies Co.,LTD",
    "AC1DDF": "IEEE Registration Authority",
    "E8D819": "AzureWave Technology Inc.",
    "609BC8": "Hipad Intelligent Technology Co., Ltd.",
    "406A8E": "Hangzhou Puwell OE Tech Ltd.",
    "1C0FAF": "Lucid Vision Labs",
    "BC5451": "Samsung Electronics Co.,Ltd",
    "74860B": "Cisco Systems, Inc",
    "182D98": "Jinwoo Industrial system",
    "88B4A6": "Motorola Mobility LLC, a Lenovo Company",
    "28CF08": "ESSYS",
    "BC903A": "Robert Bosch GmbH",
    "D0B128": "Samsung Electronics Co.,Ltd",
    "FC65DE": "Amazon Technologies Inc.",
    "F449EF": "EMSTONE",
    "EC51BC": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F079E8": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "583879": "RICOH COMPANY, LTD.",
    "F44C70": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "B0935B": "ARRIS Group, Inc.",
    "D8A534": "Spectronix Corporation",
    "00A096": "MITSUMI ELECTRIC CO.,LTD.",
    "78617C": "MITSUMI ELECTRIC CO.,LTD.",
    "601803": "Daikin Air-conditioning (Shanghai) Co., Ltd.",
    "78321B": "D-Link International",
    "940E6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "08661F": "Palo Alto Networks",
    "74EAC8": "New H3C Technologies Co., Ltd",
    "B4D64E": "Caldero Limited",
    "F89DBB": "Tintri",
    "C4F312": "Texas Instruments",
    "904E91": "IEEE Registration Authority",
    "5CE8B7": "Oraimo Technology Limited",
    "CC66B2": "Nokia",
    "D8A01D": "Espressif Inc.",
    "DC0C2D": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "8CFEB4": "VSOONTECH ELECTRONICS CO., LIMITED",
    "68ECC5": "Intel Corporate",
    "64FB50": "RoomReady/Zdi, Inc.",
    "34298F": "IEEE Registration Authority",
    "5CEA1D": "Hon Hai Precision Ind. Co.,Ltd.",
    "181456": "Nokia Corporation",
    "58B42D": "YSTen Technology Co.,Ltd",
    "3C11B2": "Fraunhofer FIT",
    "104B46": "Mitsubishi Electric Corporation",
    "38E2DD": "zte corporation",
    "885DFB": "zte corporation",
    "78CA04": "Nokia Corporation",
    "B009DA": "Ring Solutions",
    "00054F": "Garmin International",
    "58E28F": "Apple, Inc.",
    "787B8A": "Apple, Inc.",
    "E048D3": "MOBIWIRE MOBILES (NINGBO) CO.,LTD",
    "0017C8": "KYOCERA Display Corporation",
    "78BC1A": "Cisco Systems, Inc",
    "000E59": "Sagemcom Broadband SAS",
    "48BCA6": "\u200bASUNG TECHNO CO.,Ltd",
    "005C86": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "288CB8": "zte corporation",
    "B8DB1C": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "3C10E6": "PHAZR Inc.",
    "904506": "Tokyo Boeki Medisys Inc.",
    "0021A1": "Cisco Systems, Inc",
    "5C546D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30053F": "JTI Co.,Ltd.",
    "F4939F": "Hon Hai Precision Industry Co., Ltd.",
    "000726": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "FC8B97": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "2CAB25": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "1CA532": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "788C4D": "Indyme Solutions, LLC",
    "24B2DE": "Espressif Inc.",
    "EC8AC7": "Fiberhome Telecommunication Technologies Co.,LTD",
    "88365F": "LG Electronics (Mobile Communications)",
    "FC7F56": "CoSyst Control Systems GmbH",
    "2C4053": "Samsung Electronics Co.,Ltd",
    "788102": "Sercomm Corporation.",
    "84AA9C": "MitraStar Technology Corp.",
    "2880A2": "Novatel Wireless Solutions, Inc.",
    "84A1D1": "Sagemcom Broadband SAS",
    "909D7D": "ARRIS Group, Inc.",
    "F0EFD2": "TF PAYMENT SERVICE CO., LTD",
    "DCEB53": "Wuhan QianXiao Elecronic Technology CO.,LTD",
    "7CBACC": "IEEE Registration Authority",
    "540237": "Teltronic AG",
    "001F92": "Motorola Solutions Inc.",
    "000C03": "HDMI Licensing, LLC",
    "0C8FFF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54B121": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A80C63": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CC307": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CB008": "Shenzhen Gwelltimes Technology Co.,Ltd",
    "E86FF2": "Actiontec Electronics, Inc",
    "005018": "AMIT, Inc.",
    "70DEF9": "FAI WAH INTERNATIONAL (HONG KONG) LIMITED",
    "404229": "Layer3TV, Inc",
    "101B54": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4017C": "Ruckus Wireless",
    "0025C4": "Ruckus Wireless",
    "C0C520": "Ruckus Wireless",
    "FC2F6B": "Everspin Technologies, Inc.",
    "287B09": "zte corporation",
    "E0107F": "Ruckus Wireless",
    "B0EABC": "ASKEY COMPUTER CORP",
    "94C691": "EliteGroup Computer Systems Co., LTD",
    "3CF591": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "602101": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "7CEB7F": "Dmet Products Corp.",
    "8C8580": "Smart Innovation LLC",
    "78B28D": "Beijing Tengling Technology CO.Ltd ",
    "F81D90": "Solidwintech",
    "A06A44": "Vizio, Inc",
    "DCBE7A": "Zhejiang Nurotron Biotechnology Co.",
    "3438B7": "HUMAX Co., Ltd.",
    "CC0677": "Fiberhome Telecommunication Technologies Co.,LTD",
    "784501": "Biamp Systems",
    "88B111": "Intel Corporate",
    "54D751": "Proximus",
    "14780B": "Varex Imaging Deutschland AG",
    "ACAFB9": "Samsung Electronics Co.,Ltd",
    "145E45": "Bamboo Systems Group",
    "8C395C": "Bit4id Srl",
    "0080C2": "IEEE 802.1 Chair",
    "ECF342": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "041B6D": "LG Electronics (Mobile Communications)",
    "F44156": "Arrikto Inc.",
    "E45740": "ARRIS Group, Inc.",
    "688DB6": "AETEK INC.",
    "986C5C": "Jiangxi Gosun Guard Security Co.,Ltd",
    "24792A": "Ruckus Wireless",
    "00F82C": "Cisco Systems, Inc",
    "00C1B1": "Cisco Systems, Inc",
    "F4FCB1": "JJ Corp",
    "50FF20": "Keenetic Limited",
    "D4258B": "Intel Corporate",
    "309C23": "Micro-Star INTL CO., LTD.",
    "70D923": "vivo Mobile Communication Co., Ltd.",
    "B83A08": "Tenda Technology Co.,Ltd.Dongguan branch",
    "50D37F": "Yu Fly Mikly Way Science and Technology Co., Ltd.",
    "D8C8E9": "Phicomm (Shanghai) Co., Ltd.",
    "7CB960": "Shanghai X-Cheng telecom LTD",
    "B03D96": "Vision Valley FZ LLC",
    "F894C2": "Intel Corporate",
    "28B448": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70D379": "Cisco Systems, Inc",
    "70DB98": "Cisco Systems, Inc",
    "100501": "PEGATRON CORPORATION",
    "30D386": "zte corporation",
    "181212": "Cepton Technologies",
    "A49BF5": "Hybridserver Tec GmbH",
    "F470AB": "vivo Mobile Communication Co., Ltd.",
    "9CC8AE": "Becton, Dickinson  and Company",
    "B0359F": "Intel Corporate",
    "C0D962": "ASKEY COMPUTER CORP",
    "F80BCB": "Cisco Systems, Inc",
    "B47C9C": "Amazon Technologies Inc.",
    "08028E": "NETGEAR",
    "2C5A0F": "Cisco Systems, Inc",
    "2C3124": "Cisco Systems, Inc",
    "503237": "Apple, Inc.",
    "B0481A": "Apple, Inc.",
    "B49CDF": "Apple, Inc.",
    "48BF6B": "Apple, Inc.",
    "2CFAA2": "Alcatel-Lucent Enterprise",
    "E8E732": "Alcatel-Lucent Enterprise",
    "900628": "Samsung Electronics Co.,Ltd",
    "2C1DB8": "ARRIS Group, Inc.",
    "503DA1": "Samsung Electronics Co.,Ltd",
    "F097E5": "TAMIO, INC",
    "4C1A3D": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "1CA0D3": "IEEE Registration Authority",
    "7C2664": "Sagemcom Broadband SAS",
    "B4A9FE": "GHIA Technology (Shenzhen) LTD",
    "58821D": "H. Schom\u00e4cker GmbH",
    "D8A105": "Syslane, Co., Ltd.",
    "9C84BF": "Apple, Inc.",
    "BCA042": "SHANGHAI FLYCO ELECTRICAL APPLIANCE CO.,LTD",
    "9CFCD1": "Aetheris Technology (Shanghai) Co., Ltd.",
    "3C0518": "Samsung Electronics Co.,Ltd",
    "AC6B0F": "CADENCE DESIGN SYSTEMS INC",
    "7C3866": "Texas Instruments",
    "0C61CF": "Texas Instruments",
    "9C1D58": "Texas Instruments",
    "00111B": "Targa Systems Div L-3 Communications",
    "CCCE1E": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "0CF4D5": "Ruckus Wireless",
    "BC3F8F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "143004": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C750D": "WiFiSONG",
    "3805AC": "Piller Group GmbH",
    "346E9D": "Ericsson AB",
    "38AA3C": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "F4DC41": "YOUNGZONE CULTURE (SHANGHAI) CORP",
    "6854C1": "ColorTokens, Inc.",
    "000F4F": "PCS Systemtechnik GmbH",
    "50A4D0": "IEEE Registration Authority",
    "800010": "AT&T",
    "0C3CCD": "Universal Global Scientific Industrial Co., Ltd.",
    "14ABC5": "Intel Corporate",
    "7CC6C4": "Kolff Computer Supplies b.v.",
    "C4836F": "Ciena Corporation",
    "A80CCA": "Shenzhen Sundray Technologies Company Limited",
    "50D213": "CviLux Corporation",
    "001E29": "Hypertherm Inc",
    "5004B8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000302": "Charles Industries, Ltd.",
    "0024F1": "Shenzhen Fanhai Sanjiang Electronics Co., Ltd.",
    "142FFD": "LT SECURITY INC",
    "C891F9": "Sagemcom Broadband SAS",
    "9C50EE": "Cambridge Industries(Group) Co.,Ltd.",
    "40ED98": "IEEE Registration Authority",
    "78F29E": "PEGATRON CORPORATION",
    "64777D": "Hitron Technologies. Inc",
    "60D262": "Tzukuri Pty Ltd",
    "8404D2": "Kirale Technologies SL",
    "ACDCE5": "Procter & Gamble Company",
    "00B362": "Apple, Inc.",
    "E4E4AB": "Apple, Inc.",
    "00D0B2": "Xiotech Corporation",
    "5CFF35": "Wistron Corporation",
    "64EB8C": "Seiko Epson Corporation",
    "F48C50": "Intel Corporate",
    "001351": "Niles Audio Corporation",
    "000320": "Xpeed, Inc.",
    "54FA96": "Nokia Solutions and Networks GmbH & Co. KG",
    "60334B": "Apple, Inc.",
    "DCD255": "Kinpo Electronics, Inc.",
    "A02C36": "FN-LINK TECHNOLOGY LIMITED",
    "38AFD7": "FUJITSU LIMITED",
    "28993A": "Arista Networks",
    "AC1F6B": "Super Micro Computer, Inc.",
    "508A0F": "SHENZHEN FISE TECHNOLOGY HOLDING CO.,LTD.",
    "7CCBE2": "IEEE Registration Authority",
    "A8A5E2": "MSF-Vathauer Antriebstechnik GmbH & Co KG ",
    "E0508B": "Zhejiang Dahua Technology Co., Ltd.",
    "9C1E95": "Actiontec Electronics, Inc",
    "64DB43": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "60427F": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "BCA8A6": "Intel Corporate",
    "68AF13": "Futura Mobility",
    "681AB2": "zte corporation",
    "7CEBAE": "Ridgeline Instruments",
    "0C4933": "Sichuan Jiuzhou Electronic Technology Co., Ltd.",
    "000064": "Yokogawa Digital Computer Corporation",
    "E89EB4": "Hon Hai Precision Ind. Co.,Ltd.",
    "D46A6A": "Hon Hai Precision Ind. Co.,Ltd.",
    "98FD74": "ACT.CO.LTD",
    "000E58": "Sonos, Inc.",
    "74FF4C": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "002590": "Super Micro Computer, Inc.",
    "D0F73B": "Helmut Mauell GmbH Werk Weida",
    "180675": "Dilax Intelcom GmbH",
    "000FC2": "Uniwell Corporation",
    "CCC5EF": "Co-Comm Servicios Telecomunicaciones S.L.",
    "9002A9": "Zhejiang Dahua Technology Co., Ltd.",
    "C0288D": "Logitech, Inc",
    "6CEC5A": "Hon Hai Precision Ind. CO.,Ltd.",
    "44C346": "HUAWEI TECHNOLOGIES CO.,LTD",
    "307496": "HUAWEI TECHNOLOGIES CO.,LTD",
    "708A09": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4F524": "Motorola Mobility LLC, a Lenovo Company",
    "00BBC1": "CANON INC.",
    "0013A5": "General Solutions, LTD.",
    "000B2E": "Cal-Comp Electronics & Communications Company Ltd.",
    "4865EE": "IEEE Registration Authority",
    "24C1BD": "CRRC DALIAN R&D CO.,LTD.",
    "00A2EE": "Cisco Systems, Inc",
    "0059DC": "Cisco Systems, Inc",
    "C8D3FF": "Hewlett Packard",
    "C4BE84": "Texas Instruments",
    "5098F3": "Rheem Australia Pty Ltd",
    "506B8D": "Nutanix",
    "0038DF": "Cisco Systems, Inc",
    "F4CAE5": "FREEBOX SAS",
    "90004E": "Hon Hai Precision Ind. Co.,Ltd.",
    "7C2634": "ARRIS Group, Inc.",
    "40F413": "Rubezh",
    "E00DB9": "Cree, Inc.",
    "248894": "shenzhen lensun Communication Technology LTD",
    "48DA96": "Eddy Smart Home Solutions Inc.",
    "F074E4": "Thundercomm Technology Co., Ltd",
    "A0722C": "HUMAX Co., Ltd.",
    "B04BBF": "PT HAN SUNG ELECTORONICS INDONESIA",
    "CC2D83": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "0015FF": "Novatel Wireless Solutions, Inc.",
    "B4D135": "Cloudistics",
    "EC107B": "Samsung Electronics Co.,Ltd",
    "1C232C": "Samsung Electronics Co.,Ltd",
    "78888A": "CDR Sp. z o.o. Sp. k.",
    "9C3DCF": "NETGEAR",
    "D46E0E": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "88366C": "EFM Networks",
    "FCD848": "Apple, Inc.",
    "FC83C6": "N-Radio Technologies Co., Ltd.",
    "DC0D30": "Shenzhen Feasycom Technology Co., Ltd.",
    "F0ACD7": "IEEE Registration Authority",
    "9495A0": "Google, Inc.",
    "00A6CA": "Cisco Systems, Inc",
    "D8452B": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "E4186B": "Zyxel Communications Corporation",
    "C83DD4": "CyberTAN Technology Inc.",
    "E0B94D": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "38D547": "ASUSTek COMPUTER INC.",
    "D41D71": "Palo Alto Networks",
    "008731": "Cisco Systems, Inc",
    "88DEA9": "Roku, Inc.",
    "F02FA7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18DED7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60A10A": "Samsung Electronics Co.,Ltd",
    "8C71F8": "Samsung Electronics Co.,Ltd",
    "CC051B": "Samsung Electronics Co.,Ltd",
    "8C7712": "Samsung Electronics Co.,Ltd",
    "9463D1": "Samsung Electronics Co.,Ltd",
    "1C9D3E": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "0021D2": "Samsung Electronics Co.,Ltd",
    "0021D1": "Samsung Electronics Co.,Ltd",
    "001FCC": "Samsung Electronics Co.,Ltd",
    "A42983": "Boeing Defence Australia",
    "EC8892": "Motorola Mobility LLC, a Lenovo Company",
    "004A77": "zte corporation",
    "2C3F0B": "Cisco Meraki",
    "5C497D": "Samsung Electronics Co.,Ltd",
    "98234E": "Micromedia AG",
    "7487A9": "OCT Technology Co., Ltd.",
    "503F98": "CMITECH",
    "782079": "ID Tech",
    "5894CF": "Vertex Standard LMR, Inc.",
    "20F85E": "Delta Electronics",
    "0023E4": "IPnect co. ltd.",
    "70D4F2": "RIM",
    "E4D53D": "Hon Hai Precision Ind. Co.,Ltd.",
    "C0143D": "Hon Hai Precision Ind. Co.,Ltd.",
    "C01885": "Hon Hai Precision Ind. Co.,Ltd.",
    "0C6076": "Hon Hai Precision Ind. Co.,Ltd.",
    "0CEEE6": "Hon Hai Precision Ind. Co.,Ltd.",
    "08D42B": "Samsung Electronics Co.,Ltd",
    "789ED0": "Samsung Electronics Co.,Ltd",
    "B0C4E7": "Samsung Electronics Co.,Ltd",
    "C81479": "Samsung Electronics Co.,Ltd",
    "1CAF05": "Samsung Electronics Co.,Ltd",
    "24C696": "Samsung Electronics Co.,Ltd",
    "94D771": "Samsung Electronics Co.,Ltd",
    "E84E84": "Samsung Electronics Co.,Ltd",
    "B0DF3A": "Samsung Electronics Co.,Ltd",
    "805719": "Samsung Electronics Co.,Ltd",
    "549B12": "Samsung Electronics Co.,Ltd",
    "FCA13E": "Samsung Electronics Co.,Ltd",
    "A00798": "Samsung Electronics Co.,Ltd",
    "001FCD": "Samsung Electronics Co.,Ltd",
    "38ECE4": "Samsung Electronics Co.,Ltd",
    "7825AD": "Samsung Electronics Co.,Ltd",
    "ECE09B": "Samsung Electronics Co.,Ltd",
    "001632": "Samsung Electronics Co.,Ltd",
    "0016DB": "Samsung Electronics Co.,Ltd",
    "001EE2": "Samsung Electronics Co.,Ltd",
    "002490": "Samsung Electronics Co.,Ltd",
    "0023D7": "Samsung Electronics Co.,Ltd",
    "945103": "Samsung Electronics Co.,Ltd",
    "5CE8EB": "Samsung Electronics Co.,Ltd",
    "BC20A4": "Samsung Electronics Co.,Ltd",
    "E4E0C5": "Samsung Electronics Co.,Ltd",
    "20D5BF": "Samsung Electronics Co.,Ltd",
    "34BE00": "Samsung Electronics Co.,Ltd",
    "78521A": "Samsung Electronics Co.,Ltd",
    "78D6F0": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "04FEA1": "Fihonest communication co.,Ltd",
    "1C25E1": "China Mobile IOT Company Limited",
    "C0F636": "Hangzhou Kuaiyue Technologies, Ltd.",
    "7C11CB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001A22": "eQ-3 Entwicklung GmbH",
    "7085C6": "ARRIS Group, Inc.",
    "54E2E0": "ARRIS Group, Inc.",
    "347A60": "ARRIS Group, Inc.",
    "001CC3": "ARRIS Group, Inc.",
    "C0BDD1": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "B479A7": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "240DC2": "TCT mobile ltd",
    "20BBC6": "Jabil Circuit Hungary Ltd.",
    "EC8CA2": "Ruckus Wireless",
    "AC482D": "Ralinwi Nanjing Electronic Technology Co., Ltd.",
    "00549F": "Avaya Inc",
    "049FCA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50016B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "546C0E": "Texas Instruments",
    "EC8EAE": "Nagravision SA",
    "009058": "Ultra Electronics Command & Control Systems",
    "001CFD": "Universal Electronics, Inc.",
    "B80018": "Htel",
    "A043DB": "Sitael S.p.A.",
    "E0E7BB": "Nureva, Inc.",
    "7472B0": "Guangzhou Shiyuan Electronics Co., Ltd. ",
    "DC1A01": "Ecoliv Technology ( Shenzhen ) Ltd.",
    "001087": "XSTREAMIS PLC",
    "00B0B3": "XSTREAMIS PLC",
    "002347": "ProCurve Networking by HP",
    "0024A8": "ProCurve Networking by HP",
    "C09134": "ProCurve Networking by HP",
    "001CEF": "Primax Electronics Ltd.",
    "000276": "Primax Electronics Ltd.",
    "4CB21C": "Maxphotonics Co.,Ltd",
    "205EF7": "Samsung Electronics Co.,Ltd",
    "141F78": "Samsung Electronics Co.,Ltd",
    "D84710": "Sichuan Changhong Electric Ltd.",
    "001972": "Plexus (Xiamen) Co.,ltd.",
    "080087": "Xyplex, Inc.",
    "002363": "Zhuhai Raysharp Technology Co.,Ltd",
    "00001B": "Novell, Inc.",
    "00140E": "Nortel Networks",
    "001E1F": "Nortel Networks",
    "001969": "Nortel Networks",
    "0018B0": "Nortel Networks",
    "0016CA": "Nortel Networks",
    "000FCD": "Nortel Networks",
    "001BBA": "Nortel Networks",
    "0004DC": "Nortel Networks",
    "000CF7": "Nortel Networks",
    "2824FF": "Wistron Neweb Corporation",
    "38256B": "Microsoft Mobile Oy",
    "203AEF": "Sivantos GmbH",
    "005979": "Networked Energy Services",
    "B8BB23": "Guangdong Nufront CSC Co., Ltd",
    "34EA34": "HangZhou Gubei Electronics Technology Co.,Ltd",
    "EC26FB": "TECC CO.,LTD.",
    "0020F4": "SPECTRIX CORPORATION",
    "04EE91": "x-fabric GmbH",
    "207C8F": "Quanta Microsystems,Inc.",
    "000B34": "ShangHai Broadband Technologies CO.LTD",
    "3092F6": "SHANGHAI SUNMON COMMUNICATION TECHNOGY CO.,LTD",
    "A8AD3D": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "44DC91": "PLANEX COMMUNICATIONS INC.",
    "E09DB8": "PLANEX COMMUNICATIONS INC.",
    "000F59": "Phonak AG",
    "74B57E": "zte corporation",
    "24AF4A": "Alcatel-Lucent IPD",
    "7C2064": "Alcatel-Lucent IPD",
    "48F8E1": "Nokia",
    "8C90D3": "Nokia",
    "001478": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "6CB9C5": "Delta Networks, Inc.",
    "0028F8": "Intel Corporate",
    "58BC8F": "Cognitive Systems Corp.",
    "D455BE": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "54D272": "Nuki Home Solutions GmbH",
    "001A6A": "Tranzas, Inc.",
    "B47443": "Samsung Electronics Co.,Ltd",
    "FCF647": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F80CF3": "LG Electronics (Mobile Communications)",
    "C44BD1": "Wallys Communications  Teachnologies Co.,Ltd.",
    "18686A": "zte corporation",
    "DC4427": "IEEE Registration Authority",
    "C49A02": "LG Electronics (Mobile Communications)",
    "001F6B": "LG Electronics (Mobile Communications)",
    "0026E2": "LG Electronics (Mobile Communications)",
    "30766F": "LG Electronics (Mobile Communications)",
    "A8922C": "LG Electronics (Mobile Communications)",
    "BC3400": "IEEE Registration Authority",
    "F4CB52": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B808D7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94611E": "Wata Electronics Co.,Ltd. ",
    "0003B2": "Radware",
    "00A0C6": "Qualcomm Inc.",
    "649C81": "Qualcomm Inc.",
    "0024FF": "QLogic Corporation",
    "001E21": "Qisda Corporation",
    "00039D": "Qisda Corporation",
    "002186": "Universal Global Scientific Industrial Co., Ltd.",
    "183919": "Unicoi Systems",
    "00C0E4": "SIEMENS BUILDING",
    "000D10": "Embedtronics Oy",
    "001FA8": "Smart Energy Instruments Inc.",
    "000FDB": "Westell Technologies Inc.",
    "E8886C": "Shenzhen SC Technologies Co.,LTD",
    "DC35F1": "Positivo Tecnologia S.A.",
    "A47174": "HUAWEI TECHNOLOGIES CO.,LTD",
    "784476": "Zioncom Electronics (Shenzhen) Ltd.",
    "001165": "ZNYX Networks, Inc.",
    "000A68": "Solarflare Communications Inc.",
    "3C0771": "Sony Corporation",
    "80414E": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "249442": "OPEN ROAD SOLUTIONS , INC.",
    "C46413": "Cisco Systems, Inc",
    "00080D": "Toshiba",
    "000E7B": "Toshiba",
    "E8E0B7": "Toshiba",
    "5052D2": "Hangzhou Telin Technologies Co., Limited",
    "90EED9": "UNIVERSAL DE DESARROLLOS ELECTR\u00d3NICOS, SA",
    "340AFF": "Qingdao Hisense Communications Co.,Ltd.",
    "587E61": "Qingdao Hisense Communications Co.,Ltd.",
    "C0A1A2": "MarqMetrix",
    "08D0B7": "Qingdao Hisense Communications Co.,Ltd.",
    "ECD68A": "Shenzhen JMicron Intelligent Technology Developmen",
    "58528A": "Mitsubishi Electric Corporation",
    "680715": "Intel Corporate",
    "3CB6B7": "vivo Mobile Communication Co., Ltd.",
    "0010CA": "Telco Systems, Inc. ",
    "00E09E": "Quantum Corporation",
    "000A08": "Alps Alpine",
    "206A8A": "Wistron Infocomm (Zhongshan) Corporation",
    "2C36A0": "Capisco Limited",
    "B0B2DC": "Zyxel Communications Corporation",
    "CC5D4E": "Zyxel Communications Corporation",
    "404A03": "Zyxel Communications Corporation",
    "C86C87": "Zyxel Communications Corporation",
    "A020A6": "Espressif Inc.",
    "C4F1D1": "BEIJING SOGOU TECHNOLOGY DEVELOPMENT CO., LTD.",
    "38BC1A": "MEIZU Technology Co., Ltd.",
    "001EC0": "Microchip Technology Inc.",
    "00FEC8": "Cisco Systems, Inc",
    "00253E": "Sensus Metering Systems",
    "7C574E": "COBI GmbH",
    "045604": "Gionee Communication Equipment Co.,Ltd.",
    "10BEF5": "D-Link International",
    "606453": "AOD Co.,Ltd.",
    "6C98EB": "Riverbed Technology, Inc.",
    "009E1E": "Cisco Systems, Inc",
    "001848": "Vecima Networks Inc.",
    "0016FB": "SHENZHEN MTC CO LTD",
    "74CC39": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0C8A87": "AgLogica Holdings, Inc",
    "54EDA3": "Navdy, Inc.",
    "945907": "Shanghai HITE-BELDEN Network Technology Co., Ltd.",
    "C8AFE3": "Hefei Radio Communication Technology Co., Ltd ",
    "34A2A2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "749D8F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0415E": "Opsens Solution Inc.",
    "2C5A8D": "SYSTRONIK Elektronik u. Systemtechnik GmbH",
    "E47B3F": "BEIJING CO-CLOUD TECHNOLOGY LTD.",
    "0022E7": "WPS Parking Systems",
    "94E8C5": "ARRIS Group, Inc.",
    "A860B6": "Apple, Inc.",
    "C4B301": "Apple, Inc.",
    "E05F45": "Apple, Inc.",
    "483B38": "Apple, Inc.",
    "1C9148": "Apple, Inc.",
    "4851B7": "Intel Corporate",
    "B8E779": "9Solutions Oy",
    "C864C7": "zte corporation",
    "3822D6": "Hangzhou H3C Technologies Co., Limited",
    "905F2E": "TCT mobile ltd",
    "28F366": "Shenzhen Bilian electronic CO.,LTD",
    "E0A3AC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC7574": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20A680": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8828B3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F823B2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "341290": "Treeview Co.,Ltd.",
    "7CFE4E": "Shenzhen Safe vision Technology Co.,LTD",
    "644FB0": "Hyunjin.com",
    "A4E597": "Gessler GmbH",
    "A86BAD": "Hon Hai Precision Ind. Co.,Ltd.",
    "D80F99": "Hon Hai Precision Ind. Co.,Ltd.",
    "B4B15A": "Siemens AG Energy Management Division",
    "00E0E6": "INCAA Computers",
    "6C3B6B": "Routerboard.com",
    "006CBC": "Cisco Systems, Inc",
    "00A0A4": "Oracle Corporation ",
    "749781": "zte corporation",
    "001D08": "Jiangsu Yinhe  Electronics Co.,Ltd.",
    "5C70A3": "LG Electronics (Mobile Communications)",
    "9CDF03": "Harman/Becker Automotive Systems GmbH",
    "001188": "Enterasys",
    "0001F4": "Enterasys",
    "00109B": "Emulex Corporation",
    "4487FC": "Elitegroup Computer Systems Co.,Ltd.",
    "002465": "Elentec",
    "00089F": "EFM Networks",
    "0050FC": "Edimax Technology Co. Ltd.",
    "00142A": "Elitegroup Computer Systems Co.,Ltd.",
    "00115B": "Elitegroup Computer Systems Co.,Ltd.",
    "C03FD5": "Elitegroup Computer Systems Co.,Ltd.",
    "ECA86B": "Elitegroup Computer Systems Co.,Ltd.",
    "C89CDC": "Elitegroup Computer Systems Co.,Ltd.",
    "002511": "Elitegroup Computer Systems Co.,Ltd.",
    "94513D": "iSmart Alarm, Inc.",
    "001D82": "GN Netcom A/S",
    "001317": "GN Netcom A/S",
    "0016FA": "ECI Telecom Ltd.",
    "7085C2": "ASRock Incorporation",
    "EC93ED": "DDoS-Guard LTD",
    "14D11F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC094C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C6758": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24BCF8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DCEE06": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001174": " Mojo Networks, Inc.",
    "001954": "Leaf Corporation.",
    "9466E7": "WOM Engineering",
    "F8A188": "LED Roadway Lighting",
    "A082AC": "Linear DMS Solutions Sdn. Bhd.",
    "A86AC1": "HanbitEDS Co., Ltd.",
    "D463FE": "Arcadyan Corporation",
    "E85659": "Advanced-Connectek Inc.",
    "30FC68": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "008A96": "Cisco Systems, Inc",
    "BC60A7": "Sony Interactive Entertainment Inc.",
    "C84529": "IMK Networks Co.,Ltd",
    "689361": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "A0043E": "Parker Hannifin Manufacturing Germany GmbH & Co. KG",
    "34BF90": "Fiberhome Telecommunication Technologies Co.,LTD",
    "CCB3F8": "FUJITSU ISOTEC LIMITED",
    "E4A471": "Intel Corporate",
    "10F005": "Intel Corporate",
    "64CC2E": "Xiaomi Communications Co Ltd",
    "8801F2": "Vitec System Engineering Inc.",
    "00BD82": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "4CB8B5": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "BC15AC": "Vodafone Italia S.p.A.",
    "7C477C": "IEEE Registration Authority",
    "F877B8": "Samsung Electronics Co.,Ltd",
    "F0D2F1": "Amazon Technologies Inc.",
    "0452C7": "Bose Corporation",
    "F02745": "F-Secure Corporation",
    "54D0B4": "Xiamen Four-Faith Communication Technology Co.,Ltd",
    "00137C": "Kaicom co., Ltd.",
    "A8E3EE": "Sony Interactive Entertainment Inc.",
    "00248D": "Sony Interactive Entertainment Inc.",
    "00041F": "Sony Interactive Entertainment Inc.",
    "20A90E": "TCT mobile ltd",
    "0CA2F4": "Chameleon Technology (UK) Limited",
    "EC438B": "YAPTV",
    "980CA5": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "441102": "EDMI  Europe Ltd",
    "245EBE": "QNAP Systems, Inc.",
    "0404EA": "Valens Semiconductor Ltd.",
    "800DD7": "Latticework, Inc",
    "A85EE4": "12Sided Technology, LLC",
    "4CCC6A": "Micro-Star INTL CO., LTD.",
    "182195": "Samsung Electronics Co.,Ltd",
    "44783E": "Samsung Electronics Co.,Ltd",
    "546751": "Compal Broadband Networks, Inc.",
    "240B0A": "Palo Alto Networks",
    "D099D5": "Alcatel-Lucent",
    "00247D": "Nokia Danmark A/S",
    "002265": "Nokia Danmark A/S",
    "F4ED5F": "SHENZHEN KTC TECHNOLOGY GROUP ",
    "00E0E4": "FANUC ROBOTICS NORTH AMERICA, Inc.",
    "D0B53D": "SEPRO ROBOTIQUE",
    "00D0EC": "NAKAYO Inc",
    "BC44B0": "Elastifile",
    "74BFB7": "Nusoft Corporation",
    "50DA00": "Hangzhou H3C Technologies Co., Limited",
    "14BB6E": "Samsung Electronics Co.,Ltd",
    "14C3C2": "K.A. Schmersal GmbH & Co. KG",
    "10785B": "Actiontec Electronics, Inc",
    "30636B": "Apple, Inc.",
    "70884D": "JAPAN RADIO CO., LTD.",
    "1886AC": "Nokia Danmark A/S",
    "001F5C": "Nokia Danmark A/S",
    "001F00": "Nokia Danmark A/S",
    "A4F1E8": "Apple, Inc.",
    "00164E": "Nokia Danmark A/S",
    "000D4B": "Roku, Inc.",
    "F88096": "Elsys Equipamentos Eletr\u00f4nicos Ltda",
    "A811FC": "ARRIS Group, Inc.",
    "001DAA": "DrayTek Corp.",
    "E498D1": "Microsoft Mobile Oy",
    "6C2779": "Microsoft Mobile Oy",
    "00CF1C": "Communication Machinery Corporation",
    "28CC01": "Samsung Electronics Co.,Ltd",
    "0018C5": "Nokia Danmark A/S",
    "F4F5A5": "Nokia Corporation",
    "EC9B5B": "Nokia Corporation",
    "2CCC15": "Nokia Corporation",
    "D8FB5E": "ASKEY COMPUTER CORP",
    "6CF373": "Samsung Electronics Co.,Ltd",
    "9C3AAF": "Samsung Electronics Co.,Ltd",
    "781FDB": "Samsung Electronics Co.,Ltd",
    "183F47": "Samsung Electronics Co.,Ltd",
    "0CBF15": "Genetec Inc.",
    "002326": "FUJITSU LIMITED",
    "002668": "Nokia Danmark A/S",
    "002547": "Nokia Danmark A/S",
    "00001D": "Cabletron Systems, Inc.",
    "001BA9": "Brother industries, LTD.",
    "30A220": "ARG Telecom",
    "0040FB": "CASCADE COMMUNICATIONS",
    "D0542D": "Cambridge Industries(Group) Co.,Ltd.",
    "00264D": "Arcadyan Technology Corporation",
    "849CA6": "Arcadyan Technology Corporation",
    "002557": "BlackBerry RTS",
    "001CCC": "BlackBerry RTS",
    "744AA4": "zte corporation",
    "B86CE8": "Samsung Electronics Co.,Ltd",
    "0CB319": "Samsung Electronics Co.,Ltd",
    "1867B0": "Samsung Electronics Co.,Ltd",
    "C0FFD4": "NETGEAR",
    "6CB0CE": "NETGEAR",
    "008EF2": "NETGEAR",
    "9CD36D": "NETGEAR",
    "C40415": "NETGEAR",
    "50A4C8": "Samsung Electronics Co.,Ltd",
    "6C8336": "Samsung Electronics Co.,Ltd",
    "4CA56D": "Samsung Electronics Co.,Ltd",
    "00300A": "Aztech Electronics Pte Ltd",
    "001F3F": "AVM GmbH",
    "246511": "AVM GmbH",
    "E03E44": "Broadcom",
    "DCD87C": "Beijing Jingdong Century Trading Co., LTD.",
    "C4DA7D": "Ivium Technologies B.V.",
    "000B6A": "Asiarock Technology Limited",
    "001B9E": "ASKEY COMPUTER CORP",
    "E0CA94": "ASKEY COMPUTER CORP",
    "0026B6": "ASKEY COMPUTER CORP",
    "145BD1": "ARRIS Group, Inc.",
    "6CC1D2": "ARRIS Group, Inc.",
    "74F612": "ARRIS Group, Inc.",
    "002495": "ARRIS Group, Inc.",
    "0024A0": "ARRIS Group, Inc.",
    "3C754A": "ARRIS Group, Inc.",
    "E48399": "ARRIS Group, Inc.",
    "002143": "ARRIS Group, Inc.",
    "002374": "ARRIS Group, Inc.",
    "E8FCAF": "NETGEAR",
    "841B5E": "NETGEAR",
    "2CB05D": "NETGEAR",
    "A021B7": "NETGEAR",
    "0024B2": "NETGEAR",
    "001B2F": "NETGEAR",
    "001F33": "NETGEAR",
    "1C1448": "ARRIS Group, Inc.",
    "001784": "ARRIS Group, Inc.",
    "001C11": "ARRIS Group, Inc.",
    "001E46": "ARRIS Group, Inc.",
    "0018A4": "ARRIS Group, Inc.",
    "0018C0": "ARRIS Group, Inc.",
    "747548": "Amazon Technologies Inc.",
    "0000B1": "Alpha Micro",
    "001802": "Alpha Networks Inc.",
    "001CA8": "AirTies Wireless Networks",
    "74DE2B": "Liteon Technology Corporation",
    "00225F": "Liteon Technology Corporation",
    "5C93A2": "Liteon Technology Corporation",
    "ACE010": "Liteon Technology Corporation",
    "001CA4": "Sony Corporation",
    "001A75": "Sony Corporation",
    "78843C": "Sony Corporation",
    "0023F1": "Sony Corporation",
    "E86D52": "ARRIS Group, Inc.",
    "0015CF": "ARRIS Group, Inc.",
    "6CFAA7": "AMPAK Technology, Inc.",
    "00080E": "ARRIS Group, Inc.",
    "00909C": "ARRIS Group, Inc.",
    "001225": "ARRIS Group, Inc.",
    "002040": "ARRIS Group, Inc.",
    "386BBB": "ARRIS Group, Inc.",
    "94CE2C": "Sony Corporation",
    "FC0FE6": "Sony Interactive Entertainment Inc.",
    "24FD52": "Liteon Technology Corporation",
    "2CD05A": "Liteon Technology Corporation",
    "74E543": "Liteon Technology Corporation",
    "0013A9": "Sony Corporation",
    "000AD9": "Sony Corporation",
    "000E07": "Sony Corporation",
    "001E45": "Sony Corporation",
    "28C2DD": "AzureWave Technology Inc.",
    "80A589": "AzureWave Technology Inc.",
    "C40938": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "001C50": "TCL Technoly Electronics (Huizhou) Co., Ltd.",
    "0C6AE6": "Stanley Security Solutions",
    "54E4BD": "FN-LINK TECHNOLOGY LIMITED",
    "5414FD": "Orbbec 3D Technology International",
    "485D60": "AzureWave Technology Inc.",
    "74C63B": "AzureWave Technology Inc.",
    "900BC1": "Sprocomm Technologies CO.,Ltd",
    "ACE5F0": "Doppler Labs",
    "00AA02": "Intel Corporation",
    "7C7A91": "Intel Corporate",
    "AC7BA1": "Intel Corporate",
    "6C2995": "Intel Corporate",
    "984FEE": "Intel Corporate",
    "E82AEA": "Intel Corporate",
    "605718": "Intel Corporate",
    "DC85DE": "AzureWave Technology Inc.",
    "B0EE45": "AzureWave Technology Inc.",
    "54271E": "AzureWave Technology Inc.",
    "00238E": "ADB Broadband Italia",
    "001D8B": "ADB Broadband Italia",
    "0013C8": "ADB Broadband Italia",
    "DC0B1A": "ADB Broadband Italia",
    "842615": "ADB Broadband Italia",
    "F0842F": "ADB Broadband Italia",
    "5CE0C5": "Intel Corporate",
    "58A839": "Intel Corporate",
    "7C5CF8": "Intel Corporate",
    "B4E1C4": "Microsoft Mobile Oy",
    "348446": "Ericsson AB",
    "044E06": "Ericsson AB",
    "00D09E": "2Wire Inc",
    "E0757D": "Motorola Mobility LLC, a Lenovo Company",
    "34BB26": "Motorola Mobility LLC, a Lenovo Company",
    "806C1B": "Motorola Mobility LLC, a Lenovo Company",
    "0019E4": "2Wire Inc",
    "001AC4": "2Wire Inc",
    "001B5B": "2Wire Inc",
    "001EC7": "2Wire Inc",
    "002650": "2Wire Inc",
    "002351": "2Wire Inc",
    "001E67": "Intel Corporate",
    "0022FA": "Intel Corporate",
    "001500": "Intel Corporate",
    "0016EB": "Intel Corporate",
    "0018DE": "Intel Corporate",
    "606720": "Intel Corporate",
    "FCF8AE": "Intel Corporate",
    "6036DD": "Intel Corporate",
    "100BA9": "Intel Corporate",
    "648099": "Intel Corporate",
    "D07E35": "Intel Corporate",
    "001E65": "Intel Corporate",
    "C4D987": "Intel Corporate",
    "8C705A": "Intel Corporate",
    "00270E": "Intel Corporate",
    "A088B4": "Intel Corporate",
    "CC3B3E": "Lester Electrical",
    "2082C0": "Xiaomi Communications Co Ltd",
    "00173F": "Belkin International Inc.",
    "001CDF": "Belkin International Inc.",
    "C05627": "Belkin International Inc.",
    "4C17EB": "Sagemcom Broadband SAS",
    "CC33BB": "Sagemcom Broadband SAS",
    "3CA348": "vivo Mobile Communication Co., Ltd.",
    "E45AA2": "vivo Mobile Communication Co., Ltd.",
    "3CEA4F": "2Wire Inc",
    "DC7FA4": "2Wire Inc",
    "B0D5CC": "Texas Instruments",
    "3829DD": "ONvocal Inc",
    "D86CE9": "Sagemcom Broadband SAS",
    "E8F1B0": "Sagemcom Broadband SAS",
    "DC6DCD": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C4282D": "Embedded Intellect Pty Ltd",
    "800A80": "IEEE Registration Authority",
    "5846E1": "Baxter International Inc",
    "1CBA8C": "Texas Instruments",
    "205532": "Gotech International Technology Limited",
    "2CFF65": "Oki Electric Industry Co., Ltd.",
    "0015E9": "D-Link Corporation",
    "001B11": "D-Link Corporation",
    "00265A": "D-Link Corporation",
    "C8BE19": "D-Link International",
    "24BA13": "RISO KAGAKU CORPORATION",
    "5C6B32": "Texas Instruments",
    "84DD20": "Texas Instruments",
    "0017E5": "Texas Instruments",
    "0017EC": "Texas Instruments",
    "0017E7": "Texas Instruments",
    "0017E9": "Texas Instruments",
    "002926": "Applied Optoelectronics, Inc Taiwan Branch",
    "2C27D7": "Hewlett Packard",
    "984BE1": "Hewlett Packard",
    "001831": "Texas Instruments",
    "24FD5B": "SmartThings, Inc.",
    "2876CD": "Funshion Online Technologies Co.,Ltd",
    "001A11": "Google, Inc.",
    "28BC56": "EMAC, Inc.",
    "C8478C": "Beken Corporation",
    "E498D6": "Apple, Inc.",
    "006037": "NXP Semiconductors",
    "DCC0EB": "ASSA ABLOY C\u00d4TE PICARDE",
    "48DB50": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4BA76": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0050C2": "IEEE Registration Authority",
    "A0F895": "Shenzhen TINNO Mobile Technology Corp.",
    "0078CD": "Ignition Design Labs",
    "70CA4D": "Shenzhen lnovance Technology Co.,Ltd.",
    "440010": "Apple, Inc.",
    "0056CD": "Apple, Inc.",
    "00CDFE": "Apple, Inc.",
    "E4FAFD": "Intel Corporate",
    "94659C": "Intel Corporate",
    "484520": "Intel Corporate",
    "C80E77": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "9C3426": "ARRIS Group, Inc.",
    "2C6E85": "Intel Corporate",
    "9049FA": "Intel Corporate",
    "BC0F64": "Intel Corporate",
    "9CEFD5": "Panda Wireless, Inc.",
    "0002B3": "Intel Corporation",
    "000347": "Intel Corporation",
    "000E0C": "Intel Corporation",
    "001320": "Intel Corporate",
    "001DD1": "ARRIS Group, Inc.",
    "001DCF": "ARRIS Group, Inc.",
    "001DD5": "ARRIS Group, Inc.",
    "001DD4": "ARRIS Group, Inc.",
    "CCA462": "ARRIS Group, Inc.",
    "44E137": "ARRIS Group, Inc.",
    "0000C5": "ARRIS Group, Inc.",
    "6455B1": "ARRIS Group, Inc.",
    "106F3F": "BUFFALO.INC",
    "B0C745": "BUFFALO.INC",
    "14CFE2": "ARRIS Group, Inc.",
    "00234D": "Hon Hai Precision Ind. Co.,Ltd.",
    "002556": "Hon Hai Precision Ind. Co.,Ltd.",
    "601888": "zte corporation",
    "D860B0": "bioM\u00e9rieux Italia S.p.A.",
    "F8DB7F": "HTC Corporation",
    "64A769": "HTC Corporation",
    "E899C4": "HTC Corporation",
    "BCCFCC": "HTC Corporation",
    "28565A": "Hon Hai Precision Ind. Co.,Ltd.",
    "000802": "Hewlett Packard",
    "0002A5": "Hewlett Packard",
    "6CC217": "Hewlett Packard",
    "1458D0": "Hewlett Packard",
    "5C8A38": "Hewlett Packard",
    "0080E1": "STMicroelectronics SRL",
    "001871": "Hewlett Packard",
    "000E7F": "Hewlett Packard",
    "001185": "Hewlett Packard",
    "001279": "Hewlett Packard",
    "001321": "Hewlett Packard",
    "0004EA": "Hewlett Packard",
    "D8D385": "Hewlett Packard",
    "18A905": "Hewlett Packard",
    "001B78": "Hewlett Packard",
    "EC9A74": "Hewlett Packard",
    "10604B": "Hewlett Packard",
    "C8CBB8": "Hewlett Packard",
    "843497": "Hewlett Packard",
    "2C59E5": "Hewlett Packard",
    "D8FC38": "Giantec Semiconductor Inc",
    "AC2A0C": "CSR ZHUZHOU INSTITUTE CO.,LTD.",
    "2C6798": "InTalTech Ltd.",
    "6CB56B": "HUMAX Co., Ltd.",
    "E422A5": "PLANTRONICS, INC.",
    "D4C9B2": "Quanergy Systems Inc",
    "6021C0": "Murata Manufacturing Co., Ltd.",
    "88308A": "Murata Manufacturing Co., Ltd.",
    "5CDAD4": "Murata Manufacturing Co., Ltd.",
    "0026E8": "Murata Manufacturing Co., Ltd.",
    "002586": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "54E6FC": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "74EA3A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F81A67": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "EC172F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "002268": "Hon Hai Precision Ind. Co.,Ltd.",
    "BC3AEA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F431C3": "Apple, Inc.",
    "64A5C3": "Apple, Inc.",
    "0019E0": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "847778": "Cochlear Limited",
    "887F03": "Comper Technology Investment Limited",
    "50EB1A": "Brocade Communications Systems LLC",
    "0027F8": "Brocade Communications Systems LLC",
    "748EF8": "Brocade Communications Systems LLC",
    "14E6E4": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "D81FCC": "Brocade Communications Systems LLC",
    "001882": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4EA0E": "Avaya Inc",
    "6CFA58": "Avaya Inc",
    "20F3A3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C37DC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC7670": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24DBAC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C1D67": "HUAWEI TECHNOLOGIES CO.,LTD",
    "84A8E4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCC897": "zte corporation",
    "002512": "zte corporation",
    "001C26": "Hon Hai Precision Ind. Co.,Ltd.",
    "0016CE": "Hon Hai Precision Ind. Co.,Ltd.",
    "D02DB3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8CD2D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2469A5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC233D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78F5FD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C7D5E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F866D1": "Hon Hai Precision Ind. Co.,Ltd.",
    "E8088B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "002438": "Brocade Communications Systems LLC",
    "0014C9": "Brocade Communications Systems LLC",
    "344B50": "zte corporation",
    "300ED5": "Hon Hai Precision Ind. Co.,Ltd.",
    "485AB6": "Hon Hai Precision Ind. Co.,Ltd.",
    "543530": "Hon Hai Precision Ind. Co.,Ltd.",
    "90671C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC25E0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4E3FB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C96BF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60E701": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC53B5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60DE44": "HUAWEI TECHNOLOGIES CO.,LTD",
    "105172": "HUAWEI TECHNOLOGIES CO.,LTD",
    "08E84F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "643E8C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0012D2": "Texas Instruments",
    "F0EBD0": "Shanghai Feixun Communication Co.,Ltd.",
    "888603": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04F938": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC853D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4846FB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70E422": "Cisco Systems, Inc",
    "00500F": "Cisco Systems, Inc",
    "0050A2": "Cisco Systems, Inc",
    "581626": "Avaya Inc",
    "A051C6": "Avaya Inc",
    "A863F2": "Texas Instruments",
    "D0FF50": "Texas Instruments",
    "20C38F": "Texas Instruments",
    "7C669D": "Texas Instruments",
    "D8DDFD": "Texas Instruments",
    "D05FB8": "Texas Instruments",
    "7C1DD9": "Xiaomi Communications Co Ltd",
    "A086C6": "Xiaomi Communications Co Ltd",
    "9C99A0": "Xiaomi Communications Co Ltd",
    "584498": "Xiaomi Communications Co Ltd",
    "E0247F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00464B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "80FB06": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C6097": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6CA849": "Avaya Inc",
    "64C354": "Avaya Inc",
    "50CD22": "Avaya Inc",
    "B4A95A": "Avaya Inc",
    "84EB18": "Texas Instruments",
    "00603E": "Cisco Systems, Inc",
    "00602F": "Cisco Systems, Inc",
    "006047": "Cisco Systems, Inc",
    "00E0B0": "Cisco Systems, Inc",
    "00E0FE": "Cisco Systems, Inc",
    "00E034": "Cisco Systems, Inc",
    "E8EDF3": "Cisco Systems, Inc",
    "E4C722": "Cisco Systems, Inc",
    "64E950": "Cisco Systems, Inc",
    "C07BBC": "Cisco Systems, Inc",
    "24E9B3": "Cisco Systems, Inc",
    "00E0F9": "Cisco Systems, Inc",
    "C8D719": "Cisco-Linksys, LLC",
    "203A07": "Cisco Systems, Inc",
    "F41FC2": "Cisco Systems, Inc",
    "44ADD9": "Cisco Systems, Inc",
    "0C6803": "Cisco Systems, Inc",
    "C08C60": "Cisco Systems, Inc",
    "001079": "Cisco Systems, Inc",
    "001029": "Cisco Systems, Inc",
    "000E08": "Cisco-Linksys, LLC",
    "4403A7": "Cisco Systems, Inc",
    "B0FAEB": "Cisco Systems, Inc",
    "7CAD74": "Cisco Systems, Inc",
    "00023D": "Cisco Systems, Inc",
    "00502A": "Cisco Systems, Inc",
    "0018F3": "ASUSTek COMPUTER INC.",
    "001A92": "ASUSTek COMPUTER INC.",
    "0011D8": "ASUSTek COMPUTER INC.",
    "F07816": "Cisco Systems, Inc",
    "00223A": "Cisco SPVTG",
    "0021BE": "Cisco SPVTG",
    "000C41": "Cisco-Linksys, LLC",
    "0016B6": "Cisco-Linksys, LLC",
    "0018F8": "Cisco-Linksys, LLC",
    "00252E": "Cisco SPVTG",
    "54D46F": "Cisco SPVTG",
    "A4A24A": "Cisco SPVTG",
    "44E08E": "Cisco SPVTG",
    "BCC810": "Cisco SPVTG",
    "7CB21B": "Cisco SPVTG",
    "24767D": "Cisco SPVTG",
    "481D70": "Cisco SPVTG",
    "00E036": "PIONEER CORPORATION",
    "00E04F": "Cisco Systems, Inc",
    "0010FF": "Cisco Systems, Inc",
    "001054": "Cisco Systems, Inc",
    "0010F6": "Cisco Systems, Inc",
    "0010A6": "Cisco Systems, Inc",
    "1CDEA7": "Cisco Systems, Inc",
    "F07F06": "Cisco Systems, Inc",
    "BC16F5": "Cisco Systems, Inc",
    "FC5B39": "Cisco Systems, Inc",
    "346F90": "Cisco Systems, Inc",
    "14DAE9": "ASUSTek COMPUTER INC.",
    "F4CFE2": "Cisco Systems, Inc",
    "A80C0D": "Cisco Systems, Inc",
    "F0B2E5": "Cisco Systems, Inc",
    "5897BD": "Cisco Systems, Inc",
    "5C838F": "Cisco Systems, Inc",
    "ECBD1D": "Cisco Systems, Inc",
    "5CFC66": "Cisco Systems, Inc",
    "D46D50": "Cisco Systems, Inc",
    "74A02F": "Cisco Systems, Inc",
    "88F031": "Cisco Systems, Inc",
    "88908D": "Cisco Systems, Inc",
    "1CABA7": "Apple, Inc.",
    "C0847A": "Apple, Inc.",
    "0019E3": "Apple, Inc.",
    "001B63": "Apple, Inc.",
    "001EC2": "Apple, Inc.",
    "001FF3": "Apple, Inc.",
    "002332": "Apple, Inc.",
    "00236C": "Apple, Inc.",
    "0023DF": "Apple, Inc.",
    "286AB8": "Apple, Inc.",
    "28E02C": "Apple, Inc.",
    "E0B9BA": "Apple, Inc.",
    "00C610": "Apple, Inc.",
    "78A3E4": "Apple, Inc.",
    "E4CE8F": "Apple, Inc.",
    "E8040B": "Apple, Inc.",
    "145A05": "Apple, Inc.",
    "148FC6": "Apple, Inc.",
    "58B035": "Apple, Inc.",
    "5C5948": "Apple, Inc.",
    "C8BCC8": "Apple, Inc.",
    "24AB81": "Apple, Inc.",
    "E0F847": "Apple, Inc.",
    "28E7CF": "Apple, Inc.",
    "002500": "Apple, Inc.",
    "0025BC": "Apple, Inc.",
    "34159E": "Apple, Inc.",
    "0010FA": "Apple, Inc.",
    "0050E4": "Apple, Inc.",
    "000D93": "Apple, Inc.",
    "7CFADF": "Apple, Inc.",
    "0C3021": "Apple, Inc.",
    "DC86D8": "Apple, Inc.",
    "90B931": "Apple, Inc.",
    "D0E140": "Apple, Inc.",
    "24A2E1": "Apple, Inc.",
    "04214C": "Insight Energy Ventures LLC",
    "F832E4": "ASUSTek COMPUTER INC.",
    "80EA96": "Apple, Inc.",
    "600308": "Apple, Inc.",
    "B8F6B1": "Apple, Inc.",
    "8CFABA": "Apple, Inc.",
    "7CD1C3": "Apple, Inc.",
    "F0DCE2": "Apple, Inc.",
    "04F13E": "Apple, Inc.",
    "98F0AB": "Apple, Inc.",
    "7831C1": "Apple, Inc.",
    "A82066": "Apple, Inc.",
    "BC52B7": "Apple, Inc.",
    "881FA1": "Apple, Inc.",
    "C8E0EB": "Apple, Inc.",
    "98B8E3": "Apple, Inc.",
    "885395": "Apple, Inc.",
    "0C3E9F": "Apple, Inc.",
    "783A84": "Apple, Inc.",
    "786C1C": "Apple, Inc.",
    "4C8D79": "Apple, Inc.",
    "1CE62B": "Apple, Inc.",
    "98E0D9": "Apple, Inc.",
    "CC29F5": "Apple, Inc.",
    "285AEB": "Apple, Inc.",
    "FCFC48": "Apple, Inc.",
    "9C293F": "Apple, Inc.",
    "80A1AB": "Intellisis",
    "84285A": "Saffron Solutions Inc",
    "D4B8FF": "Home Control Singapore Pte Ltd",
    "24A074": "Apple, Inc.",
    "F02475": "Apple, Inc.",
    "2C1F23": "Apple, Inc.",
    "549F13": "Apple, Inc.",
    "F0DBE2": "Apple, Inc.",
    "748114": "Apple, Inc.",
    "18F643": "Apple, Inc.",
    "A45E60": "Apple, Inc.",
    "A01828": "Apple, Inc.",
    "D0034B": "Apple, Inc.",
    "10417F": "Apple, Inc.",
    "A8667F": "Apple, Inc.",
    "D02598": "Apple, Inc.",
    "087402": "Apple, Inc.",
    "94F6A3": "Apple, Inc.",
    "5C8D4E": "Apple, Inc.",
    "8863DF": "Apple, Inc.",
    "84788B": "Apple, Inc.",
    "80BE05": "Apple, Inc.",
    "84A423": "Sagemcom Broadband SAS",
    "3C7873": "Airsonics",
    "9C88AD": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C8A2CE": "Oasis Media Systems LLC",
    "88947E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "4054E4": "Wearsafe Labs Inc",
    "58F496": "Source Chain",
    "587F57": "Apple, Inc.",
    "D07C2D": "Leie IOT technology Co., Ltd",
    "EC64E7": "MOCACARE Corporation",
    "40862E": "JDM MOBILE INTERNET SOLUTION CO., LTD.",
    "98F428": "zte corporation",
    "E8343E": "Beijing Infosec Technologies Co., LTD.",
    "346987": "zte corporation",
    "A4CC32": "Inficomm Co., Ltd",
    "C4BBEA": "Pakedge Device and Software Inc",
    "006D52": "Apple, Inc.",
    "305A3A": "ASUSTek COMPUTER INC.",
    "988744": "Wuxi Hongda Science and Technology Co.,LTD",
    "38F557": "JOLATA, INC.",
    "70BF3E": "Charles River Laboratories",
    "A8C87F": "Roqos, Inc.",
    "A03299": "Lenovo (Beijing) Co., Ltd.",
    "246C8A": "YUKAI Engineering",
    "5CCF7F": "Espressif Inc.",
    "3C831E": "CKD Corporation",
    "90DFFB": "HOMERIDER SYSTEMS",
    "54A3FA": "BQT Solutions (Australia)Pty Ltd",
    "2C081C": "OVH",
    "C08488": "Finis Inc",
    "B844D9": "Apple, Inc.",
    "385F66": "Cisco SPVTG",
    "9C7A03": "Ciena Corporation",
    "7011AE": "Music Life LTD",
    "ACC51B": "Zhuhai Pantum Electronics Co., Ltd.",
    "ACEE9E": "Samsung Electronics Co.,Ltd",
    "B857D8": "Samsung Electronics Co.,Ltd",
    "681295": "Lupine Lighting Systems GmbH",
    "041E7A": "DSPWorks",
    "C8A9FC": "Goyoo Networks Inc.",
    "1C497B": "Gemtek Technology Co., Ltd.",
    "2CCF58": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D09380": "Ducere Technologies Pvt. Ltd.",
    "68F956": "Objetivos y Servicio de Valor A\u00f1adido",
    "84A788": "Perples",
    "AC60B6": "Ericsson AB",
    "14B370": "Gigaset Digital Technology (Shenzhen) Co., Ltd.",
    "6889C1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCFEC2": "Invensys Controls UK Limited",
    "689AB7": "Atelier Vision Corporation",
    "444CA8": "Arista Networks",
    "7C2BE1": "Shenzhen Ferex Electrical Co.,Ltd",
    "5031AD": "ABB Global Industries and Services Private Limited",
    "A4C138": "Telink Semiconductor (Taipei) Co. Ltd.",
    "143EBF": "zte corporation",
    "FC2FEF": "UTT Technologies Co., Ltd.",
    "046169": "MEDIA GLOBAL LINKS CO., LTD.",
    "20F510": "Codex Digital Limited",
    "A8741D": "PHOENIX CONTACT Electronics GmbH",
    "1C56FE": "Motorola Mobility LLC, a Lenovo Company",
    "F09A51": "Shanghai Viroyal Electronic Technology Company Limited",
    "BCEB5F": "Fujian Beifeng Telecom Technology Co., Ltd.",
    "B899B0": "Cohere Technologies",
    "A8D828": "Ascensia Diabetes Care",
    "B869C2": "Sunitec Enterprise Co., Ltd.",
    "88CBA5": "Suzhou Torchstar Intelligent Technology Co.,Ltd",
    "4CB82C": "Cambridge Mobile Telematics, Inc.",
    "E4A32F": "Shanghai Artimen Technology Co., Ltd.",
    "7CA23E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "501AA5": "GN Netcom A/S",
    "F4672D": "ShenZhen Topstar Technology Company",
    "A48D3B": "Vizio, Inc",
    "88A2D7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D09DAB": "TCT mobile ltd",
    "887384": "Toshiba",
    "F0AB54": "MITSUMI ELECTRIC CO.,LTD.",
    "14157C": "TOKYO COSMOS ELECTRIC CO.,LTD.",
    "20E407": "Spark srl",
    "809FAB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E00370": "ShenZhen Continental Wireless Technology Co., Ltd.",
    "D85DEF": "Busch-Jaeger Elektro GmbH",
    "3C3178": "Qolsys Inc.",
    "08ECA9": "Samsung Electronics Co.,Ltd",
    "AC5A14": "Samsung Electronics Co.,Ltd",
    "00323A": "so-logic",
    "24B0A9": "Shanghai Mobiletek Communication Ltd.",
    "5CEB68": "Cheerstar Technology Co., Ltd",
    "E04B45": "Hi-P Electronics Pte Ltd",
    "F46A92": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "F0D657": "ECHOSENS",
    "AC562C": "LAVA INTERNATIONAL(H.K) LIMITED",
    "FC9AFA": "Motus Global Inc.",
    "C8E130": "Milkyway Group Ltd",
    "1CC586": "Absolute Acoustics",
    "24693E": "innodisk Corporation",
    "C0DC6A": "Qingdao Eastsoft Communication Technology Co.,LTD",
    "9C37F4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C4711": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1CF03E": "Wearhaus Inc.",
    "486EFB": "Davit System Technology Co., Ltd.",
    "C43ABE": "Sony Corporation",
    "883B8B": "Cheering Connection Co. Ltd.",
    "E4F939": "Minxon Hotel Technology INC.",
    "247656": "Shanghai Net Miles Fiber Optics Technology Co., LTD.",
    "F8CFC5": "Motorola Mobility LLC, a Lenovo Company",
    "407FE0": "Glory Star Technics (ShenZhen) Limited",
    "146B72": "Shenzhen Fortune Ship Technology Co., Ltd.",
    "B8F080": "SPS, INC.",
    "70DA9C": "TECSEN",
    "7840E4": "Samsung Electronics Co.,Ltd",
    "E09971": "Samsung Electronics Co.,Ltd",
    "805067": "W & D TECHNOLOGY CORPORATION",
    "78F944": "Private",
    "5C5B35": "Mist Systems, Inc.",
    "B0966C": "Lanbowan Technology Ltd.",
    "A47B85": "ULTIMEDIA Co Ltd,",
    "241B44": "Hangzhou Tuners Electronics Co., Ltd",
    "80A85D": "Osterhout Design Group",
    "ACCAAB": "Virtual Electric Inc",
    "ECBAFE": "GIROPTIC",
    "3C2C94": "\u676d\u5dde\u5fb7\u6f9c\u79d1\u6280\u6709\u9650\u516c\u53f8\uff08HangZhou Delan Technology Co.,Ltd\uff09",
    "847303": "Letv Mobile and Intelligent Information Technology (Beijing) Corporation Ltd.",
    "3CC2E1": "XINHUA CONTROL ENGINEERING CO.,LTD",
    "E48501": "Geberit International AG",
    "8C873B": "Leica Camera AG",
    "2CA2B4": "Fortify Technologies, LLC",
    "10D38A": "Samsung Electronics Co.,Ltd",
    "206274": "Microsoft Corporation",
    "E8162B": "IDEO Security Co., Ltd.",
    "485415": "NET RULES TECNOLOGIA EIRELI",
    "349E34": "Evervictory Electronic Co.Ltd",
    "BC74D7": "HangZhou JuRu Technology CO.,LTD",
    "28D98A": "Hangzhou Konke Technology Co.,Ltd.",
    "700FC7": "SHENZHEN IKINLOOP TECHNOLOGY CO.,LTD.",
    "142971": "NEMOA ELECTRONICS (HK) CO. LTD",
    "B47356": "Hangzhou Treebear Networking Co., Ltd.",
    "78E980": "RainUs Co.,Ltd",
    "3CB792": "Hitachi Maxell, Ltd., Optronics Division",
    "D89341": "General Electric Global Research",
    "346895": "Hon Hai Precision Ind. Co.,Ltd.",
    "E0FFF7": "Softiron Inc.",
    "3C6A9D": "Dexatek Technology LTD.",
    "1C9ECB": "Beijing Nari Smartchip Microelectronics Company Limited",
    "DCC622": "BUHEUNG SYSTEM",
    "902CC7": "C-MAX Asia Limited",
    "94C038": "Tallac Networks",
    "6836B5": "DriveScale, Inc.",
    "8870EF": "SC Professional Trading Co., Ltd.",
    "3C4937": "ASSMANN Electronic GmbH",
    "844464": "ServerU Inc",
    "003560": "Rosen Aviation",
    "F8BC41": "Rosslare Enterprises Limited",
    "5C966A": "RTNET",
    "BCBC46": "SKS Welding Systems GmbH",
    "C40880": "Shenzhen UTEPO Tech Co., Ltd.",
    "D88039": "Microchip Technology Inc.",
    "F8B2F3": "GUANGZHOU BOSMA TECHNOLOGY CO.,LTD",
    "78EB14": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "84930C": "InCoax Networks Europe AB",
    "1CA2B1": "ruwido austria gmbh",
    "384B76": "AIRTAME ApS",
    "34B7FD": "Guangzhou Younghead Electronic Technology Co.,Ltd",
    "90179B": "Nanomegas",
    "14F893": "Wuhan FiberHome Digital Technology Co.,Ltd.",
    "9816EC": "IC Intracom",
    "34F6D2": "Panasonic Taiwan Co.,Ltd.",
    "307512": "Sony Corporation",
    "D48F33": "Microsoft Corporation",
    "54F876": "ABB AG",
    "582136": "KMB systems, s.r.o.",
    "64002D": "Powerlinq Co., LTD",
    "38B1DB": "Hon Hai Precision Ind. Co.,Ltd.",
    "D062A0": "China Essence Technology (Zhumadian) Co., Ltd.",
    "70FF5C": "Cheerzing Communication(Xiamen)Technology Co.,Ltd",
    "800902": "Keysight Technologies, Inc.",
    "0499E6": "Shenzhen Yoostar Technology Co., Ltd",
    "4C48DA": "Beijing Autelan Technology Co.,Ltd",
    "205CFA": "Yangzhou ChangLian Network Technology Co,ltd.",
    "B47C29": "Shenzhen Guzidi Technology Co.,Ltd",
    "489D18": "Flashbay Limited",
    "20A99B": "Microsoft Corporation",
    "604826": "Newbridge Technologies Int. Ltd.",
    "4CBB58": "Chicony Electronics Co., Ltd.",
    "A41242": "NEC Platforms, Ltd.",
    "C40006": "Lipi Data Systems Ltd.",
    "B41780": "DTI Group Ltd",
    "38F33F": "TATSUNO CORPORATION",
    "D80CCF": "C.G.V. S.A.S.",
    "90203A": "BYD Precision Manufacture Co.,Ltd",
    "38262B": "UTran Technology",
    "480C49": "NAKAYO Inc",
    "3CD9CE": "Eclipse WiFi",
    "6077E2": "Samsung Electronics Co.,Ltd",
    "FC1910": "Samsung Electronics Co.,Ltd",
    "D00AAB": "Yokogawa Digital Computer Corporation",
    "FC790B": "Hitachi High Technologies America, Inc.",
    "6081F9": "Helium Systems, Inc",
    "8401A7": "Greyware Automation Products, Inc",
    "98F170": "Murata Manufacturing Co., Ltd.",
    "04C991": "Phistek INC.",
    "C4C9EC": "Gugaoo   HK Limited",
    "686E48": "Prophet Electronic Technology Corp.,Ltd",
    "14B484": "Samsung Electronics Co.,Ltd",
    "F4C447": "Coagent International Enterprise Limited",
    "A43D78": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A0FC6E": "Telegrafia a.s.",
    "3CA10D": "Samsung Electronics Co.,Ltd",
    "646CB2": "Samsung Electronics Co.,Ltd",
    "680571": "Samsung Electronics Co.,Ltd",
    "209AE9": "Volacomm Co., Ltd",
    "345D10": "Wytek",
    "6C14F7": "Erhardt+Leimer GmbH",
    "B0D59D": "Shenzhen Zowee Technology Co., Ltd",
    "6828BA": "Dejai",
    "78D66F": "Aristocrat Technologies Australia Pty. Ltd.",
    "441E91": "ARVIDA Intelligent Electronics Technology  Co.,Ltd.",
    "50C7BF": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "2053CA": "Risk Technology Ltd",
    "A8A668": "zte corporation",
    "ACA9A0": "Audioengine, Ltd.",
    "A481EE": "Nokia Corporation",
    "78923E": "Nokia Corporation",
    "34E42A": "Automatic Bar Controls Inc.",
    "20A787": "Bointec Taiwan Corporation Limited",
    "04572F": "Sertel Electronics UK Ltd",
    "D8977C": "Grey Innovation",
    "BC8D0E": "Nokia",
    "205A00": "Coval",
    "A49F85": "Lyve Minds, Inc",
    "B05706": "Vallox Oy",
    "48EE07": "Silver Palm Technologies LLC",
    "3C89A6": "KAPELSE",
    "A46CC1": "LTi REEnergy GmbH",
    "C40E45": "ACK Networks,Inc.",
    "C4626B": "ZPT Vigantice",
    "EC1766": "Research Centre Module",
    "A0D12A": "AXPRO Technology Inc.",
    "00EEBD": "HTC Corporation",
    "A824EB": "ZAO NPO Introtest",
    "A8B9B3": "ESSYS",
    "6C09D6": "Digiquest Electronics LTD",
    "481842": "Shanghai Winaas Co. Equipment Co. Ltd.",
    "447098": "MING HONG TECHNOLOGY (SHEN ZHEN) LIMITED",
    "48B977": "PulseOn Oy",
    "2C534A": "Shenzhen Winyao Electronic Limited",
    "A4BBAF": "Lime Instruments",
    "F490CA": "Tensorcom",
    "D4319D": "Sinwatec",
    "B068B6": "Hangzhou OYE Technology Co. Ltd",
    "E44C6C": "Shenzhen Guo Wei Electronic Co,. Ltd.",
    "38F708": "National Resource Management, Inc.",
    "E0DB88": "Open Standard Digital-IF Interface for SATCOM Systems",
    "282246": "Beijing Sinoix Communication Co., LTD",
    "9C65F9": "AcSiP Technology Corp.",
    "487604": "Private",
    "9CBD9D": "SkyDisk, Inc.",
    "74C621": "Zhejiang Hite Renewable Energy Co.,LTD",
    "A8574E": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B843E4": "Vlatacom",
    "C4824E": "Changzhou Uchip Electronics Co., LTD.",
    "B898F7": "Gionee Communication Equipment Co,Ltd.ShenZhen",
    "848336": "Newrun",
    "7071B3": "Brain Corporation",
    "64E625": "Woxu Wireless Co., Ltd",
    "10B713": "Private",
    "208986": "zte corporation",
    "A47760": "Nokia Corporation",
    "C85663": "Sunflex Europe GmbH",
    "88FED6": "ShangHai WangYong Software Co., Ltd.",
    "7C72E4": "Unikey Technologies",
    "7C2048": "KoamTac",
    "8CB7F7": "Shenzhen UniStrong Science & Technology Co., Ltd",
    "B8266C": "ANOV France",
    "182012": "Aztech Associates Inc.",
    "3C300C": "Dewar Electronics Pty Ltd",
    "98FFD0": "Lenovo Mobile Communication Technology Ltd.",
    "38BF2F": "Espec Corp.",
    "A875E2": "Aventura Technologies, Inc.",
    "B87AC9": "Siemens Ltd.",
    "F06130": "Advantage Pharmacy Services, LLC",
    "44C56F": "NGN Easy Satfinder (Tianjin) Electronic Co., Ltd",
    "44C4A9": "Opticom Communication, LLC",
    "6C3C53": "SoundHawk Corp",
    "5C2AEF": "  r2p Asia-Pacific Pty Ltd",
    "084027": "Gridstore Inc.",
    "1C63B7": "OpenProducts 237 AB",
    "B4527E": "Sony Corporation",
    "18AA45": "Fon Technology",
    "94B9B4": "Aptos Technology",
    "407A80": "Nokia Corporation",
    "644214": "Swisscom Energy Solutions AG",
    "0CA694": "Sunitec Enterprise Co.,Ltd",
    "184462": "Riava Networks, Inc.",
    "146080": "zte corporation",
    "9CBB98": "Shen Zhen RND Electronic Co.,LTD",
    "7060DE": "LaVision GmbH",
    "502E5C": "HTC Corporation",
    "FCFE77": "Hitachi Reftechno, Inc.",
    "70533F": "Alfa Instrumentos Eletronicos Ltda.",
    "B07908": "Cummings Engineering",
    "E47723": "zte corporation",
    "9CA9E4": "zte corporation",
    "90F3B7": "Kirisun Communications Co., Ltd.",
    "38B74D": "Fijowave Limited",
    "180C14": "iSonea Limited",
    "C4E92F": "AB Sciex",
    "DC3EF8": "Nokia Corporation",
    "A49F89": "Shanghai Rui Rui Communication Technology Co.Ltd.",
    "50C271": "SECURETECH INC",
    "902083": "General Engine Management Systems Ltd.",
    "14B126": "Industrial Software Co",
    "D850E6": "ASUSTek COMPUTER INC.",
    "A88D7B": "SunDroid Global limited.",
    "3CF748": "Shenzhen Linsn Technology Development Co.,Ltd",
    "6C15F9": "Nautronix Limited",
    "CC720F": "Viscount Systems Inc.",
    "643F5F": "Exablaze",
    "2CCD69": "Aqavi.com",
    "8C2F39": "IBA Dosimetry GmbH",
    "C0A0BB": "D-Link International",
    "F45F69": "Matsufu Electronics distribution Company",
    "28A1EB": "ETEK TECHNOLOGY (SHENZHEN) CO.,LTD",
    "B8F828": "Changshu Gaoshida Optoelectronic Technology Co. Ltd.",
    "3C1A57": "Cardiopulmonary Corp",
    "541B5D": "Techno-Innov",
    "D08A55": "Skullcandy",
    "344F3F": "IO-Power Technology Co., Ltd.",
    "1446E4": "AVISTEL",
    "D095C7": "Pantech Co., Ltd.",
    "9CE7BD": "Winduskorea co., Ltd",
    "3842A6": "Ingenieurbuero Stahlkopf",
    "940BD5": "Himax Technologies, Inc",
    "284D92": "Luminator",
    "248000": "Westcontrol AS",
    "1C4BB9": "SMG ENTERPRISE, LLC",
    "346178": "The Boeing Company",
    "044F8B": "Adapteva, Inc.",
    "2C7B84": "OOO Petr Telegin",
    "A4C0C7": "ShenZhen Hitom Communication Technology Co..LTD",
    "D02C45": "littleBits Electronics, Inc.",
    "B4A82B": "Histar Digital Electronics Co., Ltd.",
    "742B62": "FUJITSU LIMITED",
    "C03580": "A&R TECH",
    "54FB58": "WISEWARE, Lda",
    "E0D1E6": "Aliph dba Jawbone",
    "D82D9B": "Shenzhen G.Credit Communication Technology Co., Ltd",
    "709BFC": "Bryton Inc.",
    "ACE42E": "SK hynix",
    "306112": "PAV GmbH",
    "041B94": "Host Mobility AB",
    "A0CEC8": "CE LINK LIMITED",
    "907A28": "Beijing Morncloud Information And Technology Co. Ltd.",
    "883612": "SRC Computers, LLC",
    "083571": "CASwell INC.",
    "9876B6": "Adafruit",
    "503CC4": "Lenovo Mobile Communication Technology Ltd.",
    "E03E4A": "Cavanagh Group International",
    "DC6F00": "Livescribe, Inc.",
    "54E3B0": "JVL Industri Elektronik",
    "78FE41": "Socus networks",
    "205721": "Salix Technology CO., Ltd.",
    "18104E": "CEDINT-UPM",
    "789F4C": "HOERBIGER Elektronik GmbH",
    "043D98": "ChongQing QingJia Electronics CO.,LTD",
    "FC4BBC": "Sunplus Technology Co., Ltd.",
    "9C1465": "Edata Elektronik San. ve Tic. A.\u015e.",
    "4C55CC": "Zentri Pty Ltd",
    "00C5DB": "Datatech Sistemas Digitales Avanzados SL",
    "8CF945": "Power Automation pte Ltd",
    "F842FB": "Yasuda Joho Co.,ltd.",
    "40BD9E": "Physio-Control, Inc",
    "6C5779": "Aclima, Inc.",
    "C0DA74": "Hangzhou Sunyard Technology Co., Ltd.",
    "887398": "K2E Tekpoint",
    "2C922C": "Kishu Giken Kogyou Company Ltd,.",
    "D8FEE3": "D-Link International",
    "50A0BF": "Alba Fiber Systems Inc.",
    "B836D8": "Videoswitch",
    "58F387": "Airios",
    "3C977E": "IPS Technology Limited",
    "A4FB8D": "Hangzhou Dunchong Technology Co.Ltd",
    "107A86": "U&U ENGINEERING INC.",
    "882364": "Watchnet DVR Inc",
    "581CBD": "Affinegy",
    "284FCE": "Liaoning Wontel Science and Technology Development Co.,Ltd.",
    "F4CD90": "Vispiron Rotec GmbH",
    "542160": "Alula",
    "806C8B": "KAESER KOMPRESSOREN AG",
    "3065EC": "Wistron (ChongQing)",
    "1001CA": "Ashley Butterworth",
    "246AAB": "IT-IS International",
    "542CEA": "PROTECTRON",
    "DC825B": "JANUS, spol. s r.o.",
    "9CA577": "Osorno Enterprises Inc.",
    "C04301": "Epec Oy",
    "E07C62": "Whistle Labs, Inc.",
    "601E02": "EltexAlatau",
    "F07F0C": "Leopold Kostal GmbH &Co. KG",
    "4C6255": "SANMINA-SCI SYSTEM DE MEXICO S.A. DE C.V.",
    "18D6CF": "Kurth Electronic GmbH",
    "F48139": "CANON INC.",
    "1836FC": "Elecsys International Corporation",
    "A4D094": "VIVAVIS AG",
    "2C9464": "Cincoze Co., Ltd.",
    "804B20": "Ventilation Control",
    "287994": "Realplay Digital Technology(Shenzhen) Co.,Ltd",
    "94B8C5": "RuggedCom Inc.",
    "604A1C": "SUYIN Corporation",
    "082719": "APS systems/electronic AG",
    "505AC6": "GUANGDONG SUPER TELECOM CO.,LTD.",
    "B863BC": "ROBOTIS, Co, Ltd",
    "980D2E": "HTC Corporation",
    "9C79AC": "Suntec Software(Shanghai) Co., Ltd.",
    "C419EC": "Qualisys AB",
    "D464F7": "CHENGDU USEE DIGITAL TECHNOLOGY CO., LTD",
    "6472D8": "GooWi Technology Co.,Limited",
    "846223": "Shenzhen Coship Electronics Co., Ltd.",
    "1CFA68": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "DC1DD4": "Microstep-MIS spol. s r.o.",
    "FCDD55": "Shenzhen WeWins wireless Co.,Ltd",
    "B01743": "EDISON GLOBAL CIRCUITS LLC",
    "60E00E": "SHINSEI ELECTRONICS CO LTD",
    "545414": "Digital RF Corea, Inc",
    "24EB65": "SAET I.S. S.r.l.",
    "E894F6": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "50CD32": "NanJing Chaoran Science & Technology Co.,Ltd.",
    "E0EDC7": "Shenzhen Friendcom Technology Development Co., Ltd",
    "D4A499": "InView Technology Corporation",
    "08482C": "Raycore Taiwan Co., LTD.",
    "9498A2": "Shanghai LISTEN TECH.LTD",
    "DC2BCA": "Zera GmbH",
    "5C8486": "Brightsource Industries Israel LTD",
    "BCBAE1": "AREC Inc.",
    "18FA6F": "ISC applied systems corp",
    "B4DD15": "ControlThings Oy Ab",
    "E0C6B3": "MilDef AB",
    "188410": "CoreTrust Inc.",
    "FC229C": "Han Kyung I Net Co.,Ltd.",
    "1832A2": "LAON TECHNOLOGY CO., LTD.",
    "40516C": "Grandex International Corporation",
    "74D02B": "ASUSTek COMPUTER INC.",
    "A01C05": "NIMAX TELECOM CO.,LTD.",
    "D0BE2C": "CNSLink Co., Ltd.",
    "2C26C5": "zte corporation",
    "94C962": "Teseq AG",
    "DC2A14": "Shanghai Longjing Technology Co.",
    "F82EDB": "RTW GmbH & Co. KG",
    "C04A00": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "045FA7": "Shenzhen Yichen Technology Development Co.,LTD",
    "808B5C": "Shenzhen Runhuicheng Technology Co., Ltd",
    "C05E6F": "V. Stonkaus firma \"Kodinis Raktas\"",
    "C0885B": "SnD Tech Co., Ltd.",
    "3CFB96": "Emcraft Systems LLC",
    "081F3F": "WondaLink Inc.",
    "087999": "AIM GmbH",
    "C011A6": "Fort-Telecom ltd.",
    "C0B8B1": "BitBox Ltd",
    "105F06": "Actiontec Electronics, Inc",
    "1853E0": "Hanyang Digitech Co.Ltd",
    "00C14F": "DDL Co,.ltd.",
    "E0CEC3": "ASKEY COMPUTER CORP",
    "F0219D": "Cal-Comp Electronics & Communications Company Ltd.",
    "181725": "Cameo Communications, Inc.",
    "8462A6": "EuroCB (Phils), Inc.",
    "E4F365": "Time-O-Matic, Inc.",
    "DCC0DB": "Shenzhen Kaiboer Technology Co., Ltd.",
    "AC5D10": "Pace Americas",
    "88F490": "Jetmobile Pte Ltd",
    "1C9179": "Integrated System Technologies Ltd",
    "84C8B1": "Incognito Software Systems Inc.",
    "30D357": "Logosol, Inc.",
    "BC39A6": "CSUN System Technology Co.,LTD",
    "ECB541": "SHINANO E and E Co.Ltd.",
    "74372F": "Tongfang Shenzhen Cloudcomputing Technology Co.,Ltd",
    "E8A364": "Signal Path International / Peachtree Audio",
    "BC51FE": "Swann communications Pty Ltd",
    "FC626E": "Beijing MDC Telecom",
    "6886E7": "Orbotix, Inc.",
    "D819CE": "Telesquare",
    "38F597": "home2net GmbH",
    "807B1E": "Corsair Memory, Inc.",
    "B4AB2C": "MtM Technology Corporation",
    "74E424": "APISTE CORPORATION",
    "A0BAB8": "Pixon Imaging",
    "D8D5B9": "Rainforest Automation, Inc.",
    "2411D0": "Chongqing Ehs Science and Technology Development Co.,Ltd.",
    "B461FF": "Lumigon A/S",
    "A0A130": "DLI Taiwan Branch office",
    "30215B": "Shenzhen Ostar Display Electronic Co.,Ltd",
    "CC593E": "Sensium Healthcare Limited",
    "ECE915": "STI Ltd",
    "80D733": "QSR Automations, Inc.",
    "34FA40": "Guangzhou Robustel Technologies Co., Limited",
    "1C5A6B": "Philips Electronics Nederland BV",
    "A875D6": "FreeTek International Co., Ltd.",
    "10A743": "SK Mtek Limited",
    "547FA8": "TELCO systems, s.r.o.",
    "5474E6": "Webtech Wireless",
    "C46DF1": "DataGravity",
    "303D08": "GLINTT TES S.A.",
    "A81FAF": "KRYPTON POLSKA",
    "08E5DA": "NANJING FUJITSU COMPUTER PRODUCTS CO.,LTD. ",
    "E4C146": "Objetivos y Servicios de Valor A",
    "D40057": "MC Technologies GmbH",
    "D410CF": "Huanshun Network Science and Technology Co., Ltd.",
    "6CB311": "Shenzhen Lianrui Electronics Co.,Ltd",
    "94FD2E": "Shanghai Uniscope Technologies Co.,Ltd",
    "5884E4": "IP500 Alliance e.V.",
    "B0358D": "Nokia Corporation",
    "F8E4FB": "Actiontec Electronics, Inc",
    "8C4AEE": "GIGA TMS INC",
    "34C99D": "EIDOLON COMMUNICATIONS TECHNOLOGY CO. LTD.",
    "789F87": "Siemens AG I IA PP PRM",
    "5CE0F6": "NIC.br- Nucleo de Informacao e Coordenacao do Ponto BR",
    "C83D97": "Nokia Corporation",
    "0CF361": "Java Information",
    "C8C791": "Zero1.tv GmbH",
    "ECD925": "RAMI",
    "005D03": "Xilinx, Inc",
    "E8718D": "Elsys Equipamentos Eletronicos Ltda",
    "D0738E": "DONG OH PRECISION CO., LTD. ",
    "64C944": "LARK Technologies, Inc",
    "600F77": "SilverPlus, Inc",
    "0C93FB": "BNS Solutions",
    "E44F5F": "EDS Elektronik Destek San.Tic.Ltd.Sti",
    "E86D54": "Digit Mobile Inc",
    "802FDE": "Zurich Instruments AG",
    "5C38E0": "Shanghai Super Electronics Technology Co.,LTD",
    "08AF78": "Totus Solutions, Inc.",
    "1C9492": "RUAG Schweiz AG",
    "B889CA": "ILJIN ELECTRIC Co., Ltd.",
    "64F50E": "Kinion Technology Company Limited",
    "ACE64B": "Shenzhen Baojia Battery Technology Co., Ltd.",
    "044BFF": "GuangZhou Hedy Digital Technology Co., Ltd",
    "2829D9": "GlobalBeiMing technology (Beijing)Co. Ltd",
    "88615A": "Siano Mobile Silicon Ltd.",
    "70E24C": "SAE IT-systems GmbH & Co. KG",
    "00FD4C": "NEVATEC",
    "144319": "Creative&Link Technology Limited",
    "7CC8D0": "TIANJIN YAAN TECHNOLOGY CO., LTD.",
    "88E917": "Tamaggo",
    "909DE0": "Newland Design + Assoc. Inc.",
    "D8AFF1": "Panasonic Appliances Company",
    "2C6289": "Regenersis (Glenrothes) Ltd",
    "1848D8": "Fastback Networks",
    "F0D3E7": "Sensometrix SA",
    "B01266": "Futaba-Kikaku",
    "58ECE1": "Newport Corporation",
    "4C09B4": "zte corporation",
    "58CF4B": "Lufkin Industries",
    "68B43A": "WaterFurnace International, Inc.",
    "60D2B9": "REALAND BIO CO., LTD.",
    "30FD11": "MACROTECH (USA) INC.",
    "6032F0": "Mplus technology",
    "D8AF3B": "Hangzhou Bigbright Integrated communications system Co.,Ltd",
    "C4DA26": "NOBLEX SA",
    "7CC8AB": "Acro Associates, Inc.",
    "10F3DB": "Gridco Systems, Inc.",
    "601929": "VOLTRONIC POWER TECHNOLOGY(SHENZHEN) CORP.",
    "48B253": "Marketaxess Corporation",
    "305D38": "Beissbarth ",
    "D04CC1": "SINTRONES Technology Corp.",
    "503F56": "Syncmold Enterprise Corp",
    "8CEEC6": "Precepscion Pty. Ltd.",
    "101248": "ITG, Inc.",
    "74943D": "AgJunction",
    "3CF392": "Virtualtek. Co. Ltd",
    "149FE8": "Lenovo Mobile Communication Technology Ltd.",
    "BCD940": "ASR Co,.Ltd.",
    "049C62": "BMT Medical Technology s.r.o.",
    "0C2A69": "electric imp, incorporated",
    "C455C2": "Bach-Simpson",
    "00E8AB": "Meggitt Training Systems, Inc.",
    "B4218A": "Dog Hunter LLC",
    "4C7897": "Arrowhead Alarm Products Ltd",
    "44E8A5": "Myreka Technologies Sdn. Bhd.",
    "B482C5": "Relay2, Inc.",
    "985E1B": "ConversDigital Co., Ltd.",
    "60D1AA": "Vishal Telecommunications Pvt Ltd",
    "709BA5": "Shenzhen Y&D Electronics Co.,LTD.",
    "B48910": "Coster T.E. S.P.A.",
    "B0C83F": "Jiangsu Cynray IOT Co., Ltd.",
    "388AB7": "ITC Networks",
    "BCC23A": "Thomson Video Networks",
    "D48CB5": "Cisco Systems, Inc",
    "ACC2EC": "CLT INT'L IND. CORP.",
    "DC37D2": "Hunan HKT Electronic Technology Co., Ltd",
    "549D85": "EnerAccess inc",
    "B0750C": "QA Cafe",
    "B4E1EB": "Private",
    "A865B2": "DONGGUAN YISHANG ELECTRONIC TECHNOLOGY CO., LIMITED",
    "E8D0FA": "MKS Instruments Deutschland GmbH",
    "98262A": "Applied Research Associates, Inc",
    "3C9174": "ALONG COMMUNICATION TECHNOLOGY",
    "FC5090": "SIMEX Sp. z o.o.",
    "60B982": "RO.VE.R. Laboratories S.p.A.",
    "348137": "UNICARD SA",
    "38B12D": "Sonotronic Nagel GmbH",
    "ACEE3B": "6harmonics Inc",
    "1C6BCA": "Mitsunami Co., Ltd.",
    "E83EFB": "GEODESIC LTD.",
    "642400": "Xorcom Ltd.",
    "9CE10E": "NCTech Ltd",
    "A06D09": "Intelcan Technosystems Inc.",
    "60F3DA": "Logic Way GmbH",
    "B46238": "Exablox",
    "C8BBD3": "Embrane",
    "ECD19A": "Zhuhai Liming Industries Co., Ltd",
    "3C363D": "Nokia Corporation",
    "808698": "Netronics Technologies Inc.",
    "407074": "Life Technology (China) Co., Ltd",
    "20F002": "MTData Developments Pty. Ltd.",
    "1CF4CA": "Private",
    "B4A4B5": "Zen Eye Co.,Ltd",
    "AC1461": "ATAW  Co., Ltd.",
    "E4C6E6": "Mophie, LLC",
    "502D1D": "Nokia Corporation",
    "F48E09": "Nokia Corporation",
    "5848C0": "COFLEC",
    "8C604F": "Cisco Systems, Inc",
    "A4B980": "Parking BOXX Inc.",
    "A47C14": "ChargeStorm AB",
    "3C9F81": "Shenzhen CATIC Bit Communications Technology Co.,Ltd",
    "445F7A": "Shihlin Electric & Engineering Corp.",
    "141A51": "Treetech Sistemas Digitais",
    "587FC8": "S2M",
    "200505": "RADMAX COMMUNICATION PRIVATE LIMITED",
    "F8DB4C": "PNY Technologies, INC.",
    "0C9D56": "Consort Controls Ltd",
    "002D76": "TITECH GmbH",
    "8020AF": "Trade FIDES, a.s.",
    "3CB87A": "Private",
    "088F2C": "Amber Technology Ltd.",
    "441319": "WKK TECHNOLOGY LTD.",
    "980284": "Theobroma Systems GmbH",
    "1CD40C": "Kriwan Industrie-Elektronik GmbH",
    "2C750F": "Shanghai Dongzhou-Lawton Communication Technology Co. Ltd.",
    "5C5015": "Cisco Systems, Inc",
    "C035BD": "Velocytech Aps",
    "287184": "Spire Payments",
    "7CB03E": "OSRAM GmbH",
    "BC8B55": "NPP ELIKS America Inc. DBA T&M Atlantic",
    "C0493D": "MAITRISE TECHNOLOGIQUE",
    "C84544": "Asia Pacific CIS (Wuxi) Co, Ltd",
    "E0EF25": "Lintes Technology Co., Ltd.",
    "8C57FD": "LVX Western",
    "54E63F": "ShenZhen LingKeWeiEr Technology Co., Ltd.",
    "20FABB": "Cambridge Executive Limited",
    "2CBE97": "Ingenieurbuero Bickele und Buehler GmbH",
    "B01C91": "Elim Co",
    "04F17D": "Tarana Wireless",
    "50ED94": "EGATEL SL",
    "48A22D": "Shenzhen Huaxuchang Telecom Technology Co.,Ltd",
    "C86000": "ASUSTek COMPUTER INC.",
    "70A66A": "Prox Dynamics AS",
    "DC3E51": "Solberg & Andersen AS",
    "18B591": "I-Storm",
    "844915": "vArmour Networks, Inc.",
    "D878E5": "KUHN SA",
    "D824BD": "Cisco Systems, Inc",
    "28CD1C": "Espotel Oy",
    "D443A8": "Changzhou Haojie Electric Co., Ltd.",
    "BCE59F": "WATERWORLD Technology Co.,LTD",
    "7041B7": "Edwards Lifesciences LLC",
    "DCA8CF": "New Spin Golf, LLC.",
    "A849A5": "Lisantech Co., Ltd.",
    "A05E6B": "MELPER Co., Ltd.",
    "F83553": "Magenta Research Ltd.",
    "F4044C": "ValenceTech Limited",
    "3497FB": "ADVANCED RF TECHNOLOGIES INC",
    "F03A55": "Omega Elektronik AS",
    "C467B5": "Libratone A/S",
    "4C3910": "Newtek Electronics co., Ltd.",
    "903AA0": "Nokia",
    "B06CBF": "3ality Digital Systems GmbH",
    "98BC57": "SVA TECHNOLOGIES CO.LTD",
    "DC3C2E": "Manufacturing System Insights, Inc.",
    "AC0DFE": "Ekon GmbH - myGEKKO",
    "FC5B26": "MikroBits",
    "40F407": "Nintendo Co., Ltd.",
    "900A3A": "PSG Plastic Service GmbH",
    "54D0ED": "AXIM Communications",
    "644BF0": "CalDigit, Inc",
    "843611": "hyungseul publishing networks",
    "3440B5": "IBM",
    "64ED62": "WOORI SYSTEMS Co., Ltd",
    "5CC9D3": "PALLADIUM ENERGY ELETRONICA DA AMAZONIA LTDA",
    "C81AFE": "DLOGIC GmbH",
    "EC63E5": "ePBoard Design LLC",
    "94DB49": "SITCORP",
    "B4944E": "WeTelecom Co., Ltd.",
    "E00B28": "Inovonics",
    "48022A": "B-Link Electronic Limited",
    "345B11": "EVI HEAT AB",
    "F0620D": "Shenzhen Egreat Tech Corp.,Ltd",
    "2C67FB": "ShenZhen Zhengjili Electronics Co., LTD",
    "D4D748": "Cisco Systems, Inc",
    "344F69": "EKINOPS SAS",
    "F8313E": "endeavour GmbH",
    "143605": "Nokia Corporation",
    "3CE5B4": "KIDASEN INDUSTRIA E COMERCIO DE ANTENAS LTDA",
    "08D09F": "Cisco Systems, Inc",
    "2C002C": "UNOWHY",
    "70CA9B": "Cisco Systems, Inc",
    "C87CBC": "Valink Co., Ltd. ",
    "B81413": "Keen High Holding(HK) Ltd.",
    "CCA374": "Guangdong Guanglian Electronic Technology Co.Ltd",
    "58677F": "Clare Controls Inc.",
    "8C8A6E": "ESTUN AUTOMATION TECHNOLOY CO., LTD",
    "68BC0C": "Cisco Systems, Inc",
    "CC6BF1": "Sound Masking Inc.",
    "18E80F": "Viking Electronics Inc.",
    "2C9717": "I.C.Y. B.V.",
    "78BAD0": "Shinybow Technology Co. Ltd.",
    "2C3F38": "Cisco Systems, Inc",
    "4050E0": "Milton Security Group LLC",
    "000830": "Cisco Systems, Inc",
    "C4EEAE": "VSS Monitoring",
    "F8D3A9": "AXAN Networks",
    "24E6BA": "JSC Zavod im. Kozitsky",
    "0C5A19": "Axtion Sdn Bhd",
    "A8BD1A": "Honey Bee (Hong Kong) Limited",
    "988217": "Disruptive Ltd",
    "9CA3BA": "SAKURA Internet Inc.",
    "A078BA": "Pantech Co., Ltd.",
    "9C5C8D": "FIREMAX IND\u00daSTRIA E COM\u00c9RCIO DE PRODUTOS ELETR\u00d4NICOS  LTDA",
    "D4206D": "HTC Corporation",
    "7C1E52": "Microsoft",
    "DCB4C4": "Microsoft XCG",
    "ACCB09": "Hefcom Metering (Pty) Ltd",
    "1866E3": "Veros Systems, Inc.",
    "74FDA0": "Compupal (Group) Corporation ",
    "CCB8F1": "EAGLE KINGDOM TECHNOLOGIES LIMITED",
    "A429B7": "bluesky",
    "48F317": "Private",
    "D41C1C": "RCF S.P.A.",
    "04888C": "Eifelwerk Butler Systeme GmbH",
    "CCF8F0": "Xi'an HISU Multimedia Technology Co.,Ltd.",
    "BC779F": "SBM Co., Ltd.",
    "406AAB": "RIM",
    "248707": "SEnergy Corporation",
    "D45AB2": "Galleon Systems",
    "C4C19F": "National Oilwell Varco Instrumentation, Monitoring, and Optimization (NOV IMO)",
    "58920D": "Kinetic Avionics Limited",
    "30DE86": "Cedac Software S.r.l.",
    "18C451": "Tucson Embedded Systems",
    "704642": "CHYNG HONG ELECTRONIC CO., LTD.",
    "280CB8": "Mikrosay Yazilim ve Elektronik A.S.",
    "3CC99E": "Huiyang Technology Co., Ltd",
    "AC02EF": "Comsis",
    "B8B42E": "Gionee Communication Equipment Co,Ltd.ShenZhen",
    "443EB2": "DEOTRON Co., LTD.",
    "D059C3": "CeraMicro Technology Corporation",
    "B4FC75": "SEMA Electronics(HK) CO.,LTD",
    "B0BF99": "WIZITDONGDO",
    "FC1794": "InterCreative Co., Ltd",
    "B40B7A": "Brusa Elektronik AG",
    "182C91": "Concept Development, Inc.",
    "1C8E8E": "DB Communication & Systems Co., ltd.",
    "24EC99": "ASKEY COMPUTER CORP",
    "2C1EEA": "AERODEV",
    "78BEB6": "Enhanced Vision",
    "A44B15": "Sun Cupid Technology (HK) LTD",
    "48C862": "Simo Wireless,Inc.",
    "A0165C": "Triteka LTD",
    "449CB5": "Alcomp, Inc",
    "40F14C": "ISE Europe SPRL",
    "E8944C": "Cogent Healthcare Systems Ltd",
    "9067F3": "Alcatel Lucent",
    "9C417C": "Hame  Technology Co.,  Limited ",
    "9C6ABE": "QEES ApS.",
    "9C934E": "Xerox Corporation",
    "2C8BF2": "Hitachi Metals America Ltd",
    "68F895": "Redflow Limited",
    "70B921": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A0E295": "DAT System Co.,Ltd",
    "3826CD": "ANDTEK",
    "D8973B": "Artesyn Embedded Technologies",
    "D4F0B4": "Napco Security Technologies",
    "900917": "Far-sighted mobile",
    "F4A52A": "Hawa Technologies Inc",
    "E8B748": "Cisco Systems, Inc",
    "0C6E4F": "PrimeVOLT Co., Ltd.",
    "BC99BC": "FonSee Technology Inc.",
    "783F15": "EasySYNC Ltd.",
    "147411": "RIM",
    "F8A9DE": "PUISSANCE PLUS",
    "B8D49D": "M Seven System Ltd.",
    "88BFD5": "Simple Audio Ltd",
    "24CBE7": "MYK, Inc.",
    "B0A10A": "Pivotal Systems Corporation",
    "802DE1": "Solarbridge Technologies",
    "986022": "EMW Co., Ltd.",
    "18D071": "DASAN CO., LTD.",
    "3C672C": "Sciovid Inc.",
    "58E476": "CENTRON COMMUNICATIONS TECHNOLOGIES FUJIAN CO.,LTD",
    "447E95": "Alpha and Omega, Inc",
    "044665": "Murata Manufacturing Co., Ltd.",
    "18AEBB": "Siemens Convergence Creators GmbH&Co.KG",
    "50FAAB": "L-tek d.o.o.",
    "3891FB": "Xenox Holding BV",
    "A8E018": "Nokia Corporation",
    "BC5FF4": "ASRock Incorporation",
    "E8B4AE": "Shenzhen C&D Electronics Co.,Ltd",
    "781DFD": "Jabil Inc",
    "CCC62B": "Tri-Systems Corporation",
    "D8C068": "Netgenetech.co.,ltd.",
    "601199": "Siama Systems Inc",
    "A88CEE": "MicroMade Galka i Drozdz sp.j.",
    "DC2B66": "InfoBLOCK S.A. de C.V.",
    "6C81FE": "Mitsuba Corporation",
    "C027B9": "Beijing National Railway Research & Design Institute  of Signal & Communication Co., Ltd.",
    "B0BDA1": "ZAKLAD ELEKTRONICZNY SIMS",
    "70B265": "Hiltron s.r.l.",
    "64D1A3": "Sitecom Europe BV",
    "F43E9D": "Benu Networks, Inc.",
    "DC9B1E": "Intercom, Inc.",
    "B8871E": "Good Mind Industries Co., Ltd.",
    "D4F027": "Trust Power Ltd.",
    "64094C": "Beijing Superbee Wireless Technology Co.,Ltd",
    "04E2F8": "AEP Ticketing solutions srl",
    "EC9ECD": "Artesyn Embedded Technologies",
    "8C5105": "Shenzhen ireadygo Information Technology CO.,LTD.",
    "C8208E": "Storagedata",
    "44D2CA": "Anvia TV Oy",
    "386E21": "Wasion Group Ltd.",
    "609E64": "Vivonic GmbH",
    "BC15A6": "Taiwan Jantek Electronics,Ltd.",
    "1C19DE": "eyevis GmbH",
    "9C807D": "SYSCABLE Korea Inc.",
    "0054AF": "Continental Automotive Systems Inc.",
    "ACCABA": "Midokura Co., Ltd. ",
    "0C8112": "Private",
    "9C95F8": "SmartDoor Systems, LLC",
    "A4856B": "Q Electronics Ltd",
    "20D5AB": "Korea Infocom Co.,Ltd.",
    "743889": "ANNAX Anzeigesysteme GmbH",
    "7C7D41": "Jinmuyu Electronics Co., Ltd.",
    "4C1480": "NOREGON SYSTEMS, INC",
    "0CF3EE": "EM Microelectronic",
    "DCDECA": "Akyllor",
    "A0AAFD": "EraThink Technologies Corp.",
    "2872F0": "ATHENA",
    "0455CA": "BriView (Xiamen) Corp.",
    "AC932F": "Nokia Corporation",
    "30142D": "Piciorgros GmbH",
    "50AF73": "Shenzhen Bitland Information Technology Co., Ltd.",
    "6CA906": "Telefield Ltd",
    "78223D": "Affirmed Networks",
    "3C02B1": "Creation Technologies LP",
    "E441E6": "Ottec Technology GmbH",
    "60DA23": "Estech Co.,Ltd",
    "44DCCB": "SEMINDIA SYSTEMS PVT LTD",
    "F8F014": "RackWare Inc.",
    "BC71C1": "XTrillion, Inc.",
    "E0E8E8": "Olive Telecommunication Pvt. Ltd",
    "6052D0": "FACTS Engineering ",
    "5C9AD8": "FUJITSU LIMITED",
    "A4C0E1": "Nintendo Co., Ltd.",
    "4C3B74": "VOGTEC(H.K.) Co., Ltd",
    "684352": "Bhuu Limited",
    "ECE90B": "SISTEMA SOLUCOES ELETRONICAS LTDA - EASYTECH",
    "A08C9B": "Xtreme Technologies Corp",
    "0CF0B4": "Globalsat International Technology Ltd",
    "48DF1C": "Wuhan NEC Fibre Optic Communications industry Co. Ltd",
    "A83944": "Actiontec Electronics, Inc",
    "74E06E": "Ergophone GmbH",
    "B08991": "LGE ",
    "34B571": "PLDS",
    "3C7437": "RIM",
    "EC9233": "Eddyfi NDT Inc",
    "D49C8E": "University of FUKUI",
    "34A183": "AWare, Inc",
    "9873C4": "Sage Electronic Engineering LLC",
    "B40142": "GCI Science & Technology Co.,LTD",
    "740ABC": "LightwaveRF Technology Ltd",
    "10A13B": "FUJIKURA RUBBER LTD.",
    "CC34D7": "GEWISS S.P.A.",
    "F02A61": "Waldo Networks, Inc.",
    "C8A70A": "Verizon Business",
    "304EC3": "Tianjin Techua Technology Co., Ltd.",
    "B4CFDB": "Shenzhen Jiuzhou Electric Co.,LTD",
    "C01242": "Alpha Security Products",
    "BC20BA": "Inspur (Shandong) Electronic Information Co., Ltd",
    "1CFEA7": "IDentytech Solutins Ltd.",
    "A0DE05": "JSC \"Irbis-T\"",
    "2826A6": "PBR electronics GmbH",
    "B428F1": "E-Prime Co., Ltd.",
    "AC2FA8": "Humannix Co.,Ltd.",
    "1064E2": "ADFweb.com s.r.l.",
    "CCD811": "Aiconn Technology Corporation",
    "44599F": "Criticare Systems, Inc",
    "3C2F3A": "SFORZATO Corp.",
    "74CE56": "Packet Force Technology Limited Company",
    "AC8112": "Gemtek Technology Co., Ltd.",
    "686359": "Advanced Digital Broadcast SA",
    "18922C": "Virtual Instruments",
    "28061E": "NINGBO GLOBAL USEFUL ELECTRIC CO.,LTD",
    "64E8E6": "global moisture management system",
    "5C6A7D": "KENTKART EGE ELEKTRONIK SAN. VE TIC. LTD. STI. ",
    "FCD4F2": "The Coca Cola Company",
    "0817F4": "IBM Corp",
    "F43814": "Shanghai Howell Electronic Co.,Ltd",
    "90610C": "Fida International (S) Pte Ltd",
    "3C5F01": "Synerchip Co., Ltd.",
    "ECBBAE": "Digivoice Tecnologia em Eletronica Ltda",
    "F4E142": "Delta Elektronika BV",
    "E0D10A": "Katoudenkikougyousyo co ltd",
    "C44B44": "Omniprint Inc.",
    "48C8B6": "SysTec GmbH",
    "3C6278": "SHENZHEN JETNET TECHNOLOGY CO.,LTD.",
    "D43D67": "Carma Industries Inc.",
    "C8D5FE": "Shenzhen Zowee Technology Co., Ltd",
    "2C3068": "Pantech Co.,Ltd",
    "00BD27": "Exar Corp.",
    "5C4058": "Jefferson Audio Video Systems, Inc.",
    "58D08F": "IEEE 1904.1 Working Group",
    "CC09C8": "IMAQLIQ LTD",
    "9C4563": "DIMEP Sistemas",
    "E446BD": "C&C TECHNIC TAIWAN CO., LTD.",
    "8CDD8D": "Wifly-City System Inc.",
    "20A2E7": "Lee-Dickens Ltd",
    "FCEDB9": "Arrayent",
    "44ED57": "Longicorn, inc.",
    "EC98C1": "Beijing Risbo Network Technology Co.,Ltd",
    "38A95F": "Actifio Inc",
    "C46354": "U-Raku, Inc.",
    "405FBE": "RIM",
    "58DB8D": "Fast Co., Ltd.",
    "F4DCDA": "Zhuhai Jiahe Communication Technology Co., limited",
    "E80462": "Cisco Systems, Inc",
    "DCD0F7": "Bentek Systems Ltd.",
    "D4A928": "GreenWave Reality Inc",
    "E06290": "Jinan Jovision Science & Technology Co., Ltd.",
    "100E2B": "NEC CASIO Mobile Communications",
    "7CB542": "ACES Technology",
    "905446": "TES ELECTRONIC SOLUTIONS",
    "544A05": "wenglor sensoric gmbh",
    "6C9CE9": "Nimble Storage",
    "A49B13": "Digital Check",
    "C8EE08": "TANGTOP TECHNOLOGY CO.,LTD",
    "70E139": "3view Ltd",
    "18422F": "Alcatel Lucent",
    "6854F5": "enLighted Inc",
    "7472F2": "Chipsip Technology Co., Ltd.",
    "E0A670": "Nokia Corporation",
    "98E165": "Accutome",
    "785712": "Mobile Integration Workgroup",
    "380A0A": "Sky-City Communication and Electronics Limited Company",
    "0CD696": "Amimon Ltd",
    "F4DC4D": "Beijing CCD Digital Technology Co., Ltd",
    "DC4EDE": "SHINYEI TECHNOLOGY CO., LTD.",
    "F0F842": "KEEBOX, Inc.",
    "4083DE": "Zebra Technologies Inc",
    "8897DF": "Entrypass Corporation Sdn. Bhd.",
    "4013D9": "Global ES",
    "AC4FFC": "SVS-VISTEK GmbH",
    "B43741": "Consert, Inc.",
    "94857A": "Evantage Industries Corp",
    "24AF54": "NEXGEN Mediatech Inc.",
    "F0AD4E": "Globalscale Technologies, Inc.",
    "08FAE0": "Fohhn Audio AG",
    "903D5A": "Shenzhen Wision Technology Holding Limited",
    "E087B1": "Nata-Info Ltd.",
    "447C7F": "Innolight Technology Corporation",
    "5C864A": "Secret Labs LLC",
    "D496DF": "SUNGJIN C&T CO.,LTD",
    "58B9E1": "Crystalfontz America, Inc.",
    "20D906": "Iota, Inc.",
    "F45595": "HENGBAO Corporation LTD.",
    "180C77": "Westinghouse Electric Company, LLC",
    "9C4E20": "Cisco Systems, Inc",
    "1C3A4F": "AccuSpec Electronics, LLC",
    "D87533": "Nokia Corporation",
    "4C5DCD": "Oy Finnish Electric Vehicle Technologies Ltd",
    "70D57E": "Scalar Corporation",
    "9835B8": "Assembled Products Corporation",
    "288915": "CashGuard Sverige AB",
    "7CA29B": "D.SignT GmbH & Co. KG",
    "A04041": "SAMWONFA Co.,Ltd.",
    "3018CF": "DEOS control systems GmbH",
    "40406B": "Icomera",
    "6C22AB": "Ainsworth Game Technology",
    "68CA00": "Octopus Systems Limited",
    "E0589E": "Laerdal Medical",
    "98DCD9": "UNITEC Co., Ltd.",
    "C0CFA3": "Creative Electronics & Software, Inc.",
    "6089B7": "KAEL M\u00dcHEND\u0130SL\u0130K ELEKTRON\u0130K T\u0130CARET SANAY\u0130 L\u0130M\u0130TED \u015e\u0130RKET\u0130",
    "30525A": "NST Co., LTD",
    "2CA780": "True Technologies Inc.",
    "B0E39D": "CAT SYSTEM CO.,LTD.",
    "7C2E0D": "Blackmagic Design",
    "5CE286": "Nortel Networks",
    "8C640B": "Beyond Devices d.o.o.",
    "FCE192": "Sichuan Jinwangtong Electronic Science&Technology Co,.Ltd",
    "949C55": "Alta Data Technologies",
    "D479C3": "Cameronet GmbH & Co. KG",
    "0C1DC2": "SeAH Networks",
    "5475D0": "Cisco Systems, Inc",
    "94236E": "Shenzhen Junlan Electronic Ltd",
    "10E6AE": "Source Technologies, LLC",
    "7C6F06": "Caterpillar Trimble Control Technologies",
    "70D5E7": "Wellcore Corporation",
    "3CF72A": "Nokia Corporation",
    "545FA9": "Teracom Limited",
    "408A9A": "TITENG CO., Ltd.",
    "F445ED": "Portable Innovation Technology Ltd.",
    "6C32DE": "Indieon Technologies Pvt. Ltd.",
    "601283": "TSB REAL TIME LOCATION SYSTEMS S.L.",
    "547FEE": "Cisco Systems, Inc",
    "ACEA6A": "GENIX INFOCOMM CO., LTD.",
    "A8F470": "Fujian Newland Communication Science Technologies Co.,Ltd.",
    "8C736E": "FUJITSU LIMITED",
    "04FE7F": "Cisco Systems, Inc",
    "EC4476": "Cisco Systems, Inc",
    "E0BC43": "C2 Microsystems, Inc.",
    "2CA835": "RIM",
    "C41ECE": "HMI Sources Ltd.",
    "14A62C": "S.M. Dezac S.A.",
    "50F003": "Open Stack, Inc.",
    "DC49C9": "CASCO SIGNAL LTD",
    "70D880": "Upos System sp. z o.o.",
    "A05DC1": "TMCT Co., LTD.",
    "B86491": "CK Telecom Ltd",
    "583CC6": "Omneality Ltd.",
    "B0C8AD": "People Power Company",
    "181714": "DAEWOOIS",
    "F0EC39": "Essec",
    "2046F9": "Advanced Network Devices (dba:AND)",
    "487119": "SGB GROUP LTD.",
    "AC583B": "Human Assembler, Inc.",
    "E8E776": "Shenzhen Kootion Technology Co., Ltd",
    "A4B1EE": "H. ZANDER GmbH & Co. KG",
    "842141": "Shenzhen Ginwave Technologies Ltd.",
    "A8995C": "aizo ag",
    "4012E4": "Compass-EOS",
    "446C24": "Reallin Electronic Co.,Ltd",
    "681FD8": "Siemens Industry, Inc.",
    "A0231B": "TeleComp R&D Corp.",
    "B8A3E0": "BenRui Technology Co.,Ltd",
    "3CF52C": "DSPECIALISTS GmbH",
    "5403F5": "EBN Technology Corp.",
    "6C1811": "Decatur Electronics",
    "F8E968": "Egker Kft.",
    "8038FD": "LeapFrog Enterprises, Inc.",
    "ACBEB6": "Visualedge Technology Co., Ltd.",
    "2C9127": "Eintechno Corporation",
    "5C1437": "Thyssenkrupp Aufzugswerke GmbH",
    "9C55B4": "I.S.E. S.r.l.",
    "E4751E": "Getinge Sterilization AB",
    "1065A3": "Panamax LLC",
    "4001C6": "3COM EUROPE LTD",
    "9C5E73": "Calibre UK LTD",
    "04C05B": "Tigo Energy",
    "78B81A": "INTER SALES A/S",
    "B0E97E": "Advanced Micro Peripherals",
    "50252B": "Nethra Imaging Incorporated",
    "F8811A": "OVERKIZ",
    "4C63EB": "Application Solutions (Electronics and Vision) Ltd",
    "702F97": "Aava Mobile Oy",
    "10CA81": "PRECIA",
    "3863F6": "3NOD MULTIMEDIA(SHENZHEN)CO.,LTD",
    "202CB7": "Kong Yue Electronics & Information Industry (Xinhui) Ltd.",
    "74E537": "RADSPIN",
    "CC0080": "BETTINI SRL",
    "644BC3": "Shanghai WOASiS Telecommunications Ltd., Co.",
    "609F9D": "CloudSwitch",
    "CCCC4E": "Sun Fountainhead USA. Corp ",
    "688540": "IGI Mobile, Inc.",
    "A09A5A": "Time Domain",
    "64A837": "Juni Korea Co., Ltd",
    "942E63": "Fins\u00e9cur",
    "AC8317": "Shenzhen Furtunetel Communication Co., Ltd",
    "ACD180": "Crexendo Business Solutions, Inc.",
    "40A6A4": "PassivSystems Ltd",
    "94BA31": "Visiontec da Amaz\u00f4nia Ltda.",
    "9C5B96": "NMR Corporation",
    "60F13D": "JABLOCOM s.r.o.",
    "B894D2": "Retail Innovation HTT AB",
    "F0C24C": "Zhejiang FeiYue Digital Technology Co., Ltd",
    "48343D": "IEP GmbH",
    "D4AAFF": "MICRO WORLD ",
    "002717": "CE Digital(Zhenjiang)Co.,Ltd",
    "002716": "Adachi-Syokai Co., Ltd.",
    "0026E1": "Stanford University, OpenFlow Group",
    "0026DC": "Optical Systems Design",
    "0026C3": "Insightek Corp.",
    "0026C1": "ARTRAY CO., LTD.",
    "0026EC": "Legrand Home Systems, Inc",
    "0026E9": "SP Corp",
    "0026EB": "Advanced Spectrum Technology Co., Ltd.",
    "002700": "Shenzhen Siglent Technology Co., Ltd.",
    "0026D4": "IRCA SpA",
    "002663": "Shenzhen Huitaiwei Tech. Ltd, co.",
    "002661": "Irumtek Co., Ltd.",
    "00265B": "Hitron Technologies. Inc",
    "002656": "Sansonic Electronics USA",
    "002658": "T-Platforms (Cyprus) Limited",
    "00267F": "Oregan Networks Ltd.",
    "00267D": "A-Max Technology Macao Commercial Offshore Company Limited",
    "00267C": "Metz-Werke GmbH & Co KG",
    "002674": "Hunter Douglas",
    "002673": "RICOH COMPANY,LTD.",
    "00266D": "MobileAccess Networks",
    "00264C": "Shanghai DigiVision Technology Co., Ltd.",
    "002646": "SHENYANG TONGFANG MULTIMEDIA TECHNOLOGY COMPANY LIMITED",
    "002644": "Thomson Telecom Belgium",
    "00268B": "Guangzhou Escene Computer Technology Limited",
    "002681": "Interspiro AB",
    "002683": "Ajoho Enterprise Co., Ltd.",
    "00266F": "Coordiwise Technology Corp.",
    "00266E": "Nissho-denki Co.,LTD.",
    "0026BE": "Schoonderbeek Elektronica Systemen B.V.",
    "0026B5": "ICOMM Tele Ltd",
    "00263F": "LIOS Technology GmbH",
    "00263B": "Onbnetech",
    "002634": "Infineta Systems, Inc",
    "002698": "Cisco Systems, Inc",
    "00269B": "SOKRAT Ltd.",
    "00268C": "StarLeaf Ltd.",
    "0026A1": "Megger",
    "0025D4": "General Dynamics Mission Systems",
    "002606": "RAUMFELD GmbH",
    "002607": "Enabling Technology Pty Ltd",
    "002602": "SMART Temps LLC",
    "002605": "CC Systems AB",
    "00262F": "HAMAMATSU TOA ELECTRONICS",
    "002631": "COMMTACT LTD",
    "002624": "Thomson Inc.",
    "0025FB": "Tunstall Healthcare A/S",
    "0025F4": "KoCo Connector AG",
    "0025EF": "I-TEC Co., Ltd.",
    "0025C2": "RingBell Co.,Ltd.",
    "00261A": "Femtocomm System Technology Corp.",
    "0025E3": "Hanshinit Inc.",
    "0025B4": "Cisco Systems, Inc",
    "0025B2": "MBDA Deutschland GmbH",
    "0025A7": "itron",
    "002529": "COMELIT GROUP S.P.A",
    "00252A": "Chengdu GeeYa Technology Co.,LTD",
    "002528": "Daido Signal Co., Ltd.",
    "00259F": "TechnoDigital Technologies GmbH",
    "00259D": "Private",
    "002598": "Zhong Shan City Litai Electronic Industrial Co. Ltd",
    "002599": "Hedon e.d. B.V.",
    "002526": "Genuine Technologies Co., Ltd.",
    "002521": "Logitek Electronic Systems, Inc.",
    "00251F": "ZYNUS VISION INC.",
    "00251E": "ROTEL TECHNOLOGIES",
    "00256B": "ATENIX E.E. s.r.l.",
    "00256E": "Van Breda B.V.",
    "002565": "Vizimax Inc.",
    "00255E": "Shanghai Dare Technologies Co.,Ltd.",
    "00255F": "SenTec AG",
    "002597": "Kalki Communication Technologies",
    "002592": "Guangzhou Shirui Electronic Co., Ltd",
    "002594": "Eurodesign BG LTD",
    "00258A": "Pole/Zero Corporation",
    "00253B": "din Dietmar Nocker Facilitymanagement GmbH",
    "00253D": "DRS Consolidated Controls",
    "002535": "Minimax GmbH & Co KG",
    "002584": "Cisco Systems, Inc",
    "002579": "J & F Labs",
    "00257F": "CallTechSolution Co.,Ltd",
    "002577": "D-BOX Technologies",
    "002572": "Nemo-Q International AB",
    "0024C2": "Asumo Co.,Ltd.",
    "0024BF": "Carrier Culoz SA",
    "0024C0": "NTI COMODO INC",
    "0024BB": "CENTRAL Corporation",
    "0024BC": "HuRob Co.,Ltd",
    "0024B7": "GridPoint, Inc.",
    "0024E4": "Withings",
    "0024DE": "GLOBAL Technology Inc.",
    "0024DB": "Alcohol Monitoring Systems",
    "0024EA": "iris-GmbH infrared & intelligent sensors",
    "0024ED": "YT Elec. Co,.Ltd.",
    "0024EC": "United Information Technology Co.,Ltd.",
    "0024E6": "In Motion Technology Inc.",
    "0024E7": "Plaster Networks",
    "0024A7": "Advanced Video Communications Inc.",
    "0024AB": "A7 Engineering, Inc.",
    "0024A4": "Siklu Communication",
    "00249A": "Beijing Zhongchuang Telecommunication Test Co., Ltd.",
    "0024DD": "Centrak, Inc.",
    "0024D5": "Winward Industrial Limited",
    "0024AE": "IDEMIA",
    "0024AD": "Adolf Thies Gmbh & Co. KG",
    "002519": "Viaas Inc",
    "002513": "CXP DIGITAL BV",
    "002503": "IBM Corp",
    "002504": "Valiant Communications Limited",
    "00245F": "Vine Telecom CO.,Ltd.",
    "002455": "MuLogic BV",
    "00245A": "Nanjing Panda Electronics Company Limited",
    "00245B": "RAIDON TECHNOLOGY, INC.",
    "002459": "ABB Automation products GmbH",
    "00249E": "ADC-Elektronik GmbH",
    "00249F": "RIM Testing Services",
    "002488": "Centre For Development Of Telematics",
    "00248F": "DO-MONIX",
    "002439": "Digital Barriers Advanced Technologies",
    "002434": "Lectrosonics, Inc.",
    "00242F": "Micron",
    "00243A": "Ludl Electronic Products",
    "002479": "Optec Displays, Inc.",
    "002468": "Sumavision Technologies Co.,Ltd",
    "002466": "Unitron nv",
    "002426": "NOHMI BOSAI LTD.",
    "002429": "MK MASTER INC.",
    "00244E": "RadChips, Inc.",
    "00240B": "Virtual Computer Inc.",
    "002402": "Op-Tection GmbH",
    "0023C1": "Securitas Direct AB",
    "0023EB": "Cisco Systems, Inc",
    "0023EC": "Algorithmix GmbH",
    "0023C8": "TEAM-R",
    "002418": "Nextwave Semiconductor",
    "002412": "Benign Technologies Co, Ltd.",
    "00240D": "OnePath Networks LTD.",
    "0023FC": "Ultra Stereo Labs, Inc",
    "0023DB": "saxnet gmbh",
    "0023AA": "HFR, Inc.",
    "0023A5": "SageTV, LLC",
    "0023BB": "Accretech SBS, Inc.",
    "00238D": "Techno Design Co., Ltd.",
    "002387": "ThinkFlood, Inc.",
    "002384": "GGH Engineering s.r.l.",
    "00232D": "SandForce",
    "002323": "Zylin AS",
    "0022F8": "PIMA Electronic Systems Ltd.",
    "00231C": "Fourier Systems Ltd.",
    "00231D": "Deltacom Electronics Ltd",
    "00236F": "DAQ System",
    "002369": "Cisco-Linksys, LLC",
    "00239E": "Jiangsu Lemote Technology Corporation Limited",
    "002391": "Maxian",
    "002392": "Proteus Industries Inc.",
    "002393": "AJINEXTEK",
    "002330": "DIZIPIA, INC.",
    "00230A": "ARBURG GmbH & Co KG",
    "002316": "KISAN ELECTRONICS CO",
    "00230F": "Hirsch Electronics Corporation",
    "00237F": "PLANTRONICS, INC.",
    "002295": "SGM Technology for lighting spa",
    "002287": "Titan Wireless LLC",
    "002288": "Sagrad, Inc.",
    "0022D7": "Nintendo Co., Ltd.",
    "0022D6": "Cypak AB",
    "0022D0": "Polar Electro Oy",
    "002285": "NOMUS COMM SYSTEMS",
    "002281": "Daintree Networks Pty",
    "002273": "Techway",
    "00226B": "Cisco-Linksys, LLC",
    "002267": "Nortel Networks",
    "0022A2": "Xtramus Technologies",
    "00229E": "Social Aid Research Co., Ltd.",
    "0022C3": "Zeeport Technology Inc.",
    "0022DE": "OPPO Digital, Inc.",
    "0022F1": "Private",
    "002242": "Alacron Inc.",
    "002234": "Corventis Inc.",
    "002232": "Design Design Technology Ltd",
    "00222B": "Nucomm, Inc.",
    "002221": "ITOH DENKI CO,LTD.",
    "002226": "Avaak, Inc.",
    "00221D": "Freegene Technology LTD",
    "0021FF": "Cyfrowy Polsat SA",
    "0021F7": "HPN Supply Chain",
    "0021F4": "INRange Systems, Inc",
    "0021F5": "Western Engravers Supply, Inc.",
    "002255": "Cisco Systems, Inc",
    "00224E": "SEEnergy Corp.",
    "002245": "Leine & Linde AB",
    "00220A": "OnLive, Inc",
    "002203": "Glensound Electronics Ltd",
    "002204": "KORATEK",
    "002224": "Good Will Instrument Co., Ltd.",
    "00221B": "Morega Systems",
    "002249": "HOME MULTIENERGY SL",
    "002240": "Universal Telecom S/A",
    "00220F": "MoCA (Multimedia over Coax Alliance)",
    "00225A": "Garde Security AB",
    "00218F": "Avantgarde Acoustic Lautsprechersysteme GmbH",
    "002187": "Imacs GmbH",
    "002181": "Si2 Microsystems Limited",
    "00217E": "Telit Communication s.p.a",
    "0021AC": "Infrared Integrated Systems Ltd",
    "0021A2": "EKE-Electronics Ltd.",
    "00216E": "Function ATI (Huizhou) Telecommunications Co., Ltd.",
    "00216D": "Soltech Co., Ltd.",
    "002165": "Presstek Inc.",
    "0021E4": "I-WIN",
    "0021E5": "Display Solution AG",
    "0021E2": "visago Systems & Controls GmbH & Co. KG",
    "00218E": "MEKICS CO., LTD.",
    "0021C9": "Wavecom Asia Pacific Limited",
    "0021C2": "GL Communications Inc",
    "0021DD": "Northstar Systems Corp",
    "0021D5": "X2E GmbH",
    "0021B2": "Fiberblaze A/S",
    "00214A": "Pixel Velocity, Inc",
    "002146": "Sanmina-SCI",
    "001FF8": "Siemens AG, Sector Industry, Drive Technologies, Motion Control Systems",
    "001FFA": "Coretree, Co, Ltd",
    "001FF5": "Kongsberg Defence & Aerospace",
    "00212E": "dresden-elektronik",
    "002130": "Keico Hightech Inc.",
    "002133": "Building B, Inc",
    "002134": "Brandywine Communications",
    "001FF2": "VIA Technologies, Inc.",
    "001FF1": "Paradox Hellas S.A.",
    "00215B": "SenseAnywhere",
    "00214E": "GS Yuasa Power Supply Ltd.",
    "00211F": "SHINSUNG DELTATECH CO.,LTD.",
    "002124": "Optos Plc",
    "002117": "Tellord",
    "00210F": "Cernium Corp",
    "002101": "Aplicaciones Electronicas Quasar (AEQ)",
    "002103": "GHI Electronics, LLC",
    "002142": "Advanced Control Systems doo",
    "00213A": "Winchester Systems Inc.",
    "001F90": "Actiontec Electronics, Inc",
    "001F91": "DBS Lodging Technologies, LLC",
    "001F98": "DAIICHI-DENTSU LTD.",
    "001F93": "Xiotech Corporation",
    "001FD1": "OPTEX CO.,LTD.",
    "001FC9": "Cisco Systems, Inc",
    "001FE6": "Alphion Corporation",
    "001FDC": "Mobile Safe Track Ltd",
    "001FA9": "Atlanta DTH, Inc.",
    "001FA3": "T&W Electronics(Shenzhen)Co.,Ltd.",
    "001FA2": "Datron World Communications, Inc.",
    "001F70": "Botik Technologies LTD",
    "001FCF": "MSI Technology GmbH",
    "001F2C": "Starbridge Networks",
    "001F31": "Radiocomp",
    "001F2B": "Orange Logic",
    "001F2A": "ACCM",
    "001F30": "Travelping",
    "001F51": "HD Communications Corp",
    "001F53": "GEMAC Chemnitz GmbH",
    "001F4C": "Roseman Engineering Ltd",
    "001F50": "Swissdis AG",
    "001F48": "Mojix Inc.",
    "001F24": "DIGITVIEW TECHNOLOGY CO., LTD.",
    "001F21": "Inner Mongolia Yin An Science & Technology Development Co.,L",
    "001F22": "Source Photonics, Inc.",
    "001F1D": "Atlas Material Testing Technology LLC",
    "001F15": "Bioscrypt Inc",
    "001EDF": "Master Industrialization Center Kista",
    "001EE3": "T&W Electronics (ShenZhen) Co.,Ltd",
    "001ED9": "Mitsubishi Precision Co.,LTd.",
    "001F3E": "RP-Technik e.K.",
    "001F37": "Genesis I&C",
    "001EF0": "Gigafin Networks",
    "001EF2": "Micro Motion Inc",
    "001EEA": "Sensor Switch, Inc.",
    "001F6D": "Cisco Systems, Inc",
    "001F64": "Beijing Autelan Technology Inc.",
    "001F04": "Granch Ltd.",
    "001E83": "LAN/MAN Standards Association (LMSC)",
    "001E7C": "Taiwick Limited",
    "001E78": "Owitek Technology Ltd.,",
    "001E7A": "Cisco Systems, Inc",
    "001E62": "Siemon",
    "001E5C": "RB GeneralEkonomik",
    "001E5D": "Holosys d.o.o.",
    "001E60": "Digital Lighting Systems, Inc",
    "001EA0": "XLN-t",
    "001E98": "GreenLine Communications",
    "001E9A": "HAMILTON Bonaduz AG",
    "001EA6": "Best IT World (India) Pvt. Ltd.",
    "001EA5": "ROBOTOUS, Inc.",
    "001EA7": "Actiontec Electronics, Inc",
    "001E94": "SUPERCOM TECHNOLOGY CORPORATION",
    "001E8F": "CANON INC.",
    "001ED3": "Dot Technology Int'l Co., Ltd.",
    "001ECF": "PHILIPS ELECTRONICS UK LTD",
    "001E6C": "Opaque Systems",
    "001DEC": "Marusys",
    "001DE8": "Nikko Denki Tsushin Corporation(NDTC)",
    "001DDA": "Mikroelektronika spol. s r. o.",
    "001DE3": "Intuicom",
    "001DDE": "Zhejiang Broadcast&Television Technology Co.,Ltd.",
    "001DE5": "Cisco Systems, Inc",
    "001E22": "ARVOO Imaging Products BV",
    "001E1A": "Best Source Taiwan Inc.",
    "001E19": "GTRI",
    "001E14": "Cisco Systems, Inc",
    "001DB8": "Intoto Inc.",
    "001DB0": "FuJian HengTong Information Technology Co.,Ltd",
    "001E0E": "MAXI VIEW HOLDINGS LIMITED",
    "001E0F": "Briot International",
    "001DCB": "Ex\u00e9ns Development Oy",
    "001DCA": "PAV Electronics Limited",
    "001E2F": "DiMoto Pty Ltd",
    "001E36": "IPTE",
    "001E27": "SBN TECH Co.,Ltd.",
    "001DFE": "Palm, Inc",
    "001DF5": "Sunshine Co,LTD",
    "001DF0": "Vidient Systems, Inc.",
    "001DC2": "XORTEC OY",
    "001E44": "SANTEC",
    "001DA7": "Seamless Internet",
    "001DA8": "Takahata Electronics Co.,Ltd",
    "001DA9": "Castles Technology, Co., LTD",
    "001DA5": "WB Electronics",
    "001D96": "WatchGuard Video",
    "001D8F": "PureWave Networks",
    "001D55": "ZANTAZ, Inc",
    "001D52": "Defzone B.V.",
    "001D4A": "Carestream Health, Inc.",
    "001D36": "ELECTRONICS CORPORATION OF INDIA LIMITED",
    "001D33": "Maverick Systems Inc.",
    "001D2C": "Wavetrend Technologies (Pty) Limited",
    "001D71": "Cisco Systems, Inc",
    "001D65": "Microwave Radio Communications",
    "001D64": "Adam Communications Systems Int Ltd",
    "001D5E": "COMING MEDIA CORP.",
    "001DA1": "Cisco Systems, Inc",
    "001D8C": "La Crosse Technology LTD",
    "001D50": "SPINETIX SA",
    "001D45": "Cisco Systems, Inc",
    "001D69": "Knorr-Bremse IT-Services GmbH",
    "001CF8": "Parade Technologies, Ltd.",
    "001CF7": "AudioScience",
    "001CF6": "Cisco Systems, Inc",
    "001CF5": "Wiseblue Technology Limited",
    "001CEE": "SHARP Corporation",
    "001CCA": "Shanghai Gaozhi Science & Technology Development Co.",
    "001CC9": "Kaise Electronic Technology Co., Ltd.",
    "001CC8": "INDUSTRONIC Industrie-Electronic GmbH & Co. KG",
    "001CC6": "ProStor Systems",
    "001CBE": "Nintendo Co., Ltd.",
    "001D27": "NAC-INTERCOM",
    "001D24": "Aclara Power-Line Systems Inc.",
    "001CDE": "Interactive Multimedia eXchange Inc.",
    "001CCF": "LIMETEK",
    "001CAB": "Meyer Sound Laboratories, Inc.",
    "001C9E": "Dualtech IT AB",
    "001C94": "LI-COR Biosciences",
    "001CAD": "Wuhan Telecommunication Devices Co.,Ltd",
    "001CA7": "International Quartz Limited",
    "001D07": "Shenzhen Sang Fei Consumer Communications Co.,Ltd",
    "001D06": "HM Electronics, Inc.",
    "001D01": "Neptune Digital",
    "001D18": "Power Innovation GmbH",
    "001D1B": "Sangean Electronics Inc.",
    "001D17": "Digital Sky Corporation",
    "001CE3": "Optimedical Systems",
    "001C48": "WiDeFi, Inc.",
    "001C46": "QTUM",
    "001C42": "Parallels, Inc.",
    "001C3E": "ECKey Corporation",
    "001C80": "New Business Division/Rhea-Information CO., LTD.",
    "001C83": "New Level Telecom Co., Ltd.",
    "001C76": "The Wandsworth Group Ltd",
    "001C72": "Mayer & Cie GmbH & Co KG",
    "001C8C": "DIAL TECHNOLOGY LTD.",
    "001C93": "ExaDigm Inc",
    "001C85": "Eunicorn",
    "001C0A": "Shenzhen AEE Technology Co.,Ltd.",
    "001C0D": "G-Technology, Inc.",
    "001C03": "Betty TV Technology AG",
    "001C6A": "Weiss Engineering Ltd.",
    "001C67": "Pumpkin Networks, Inc.",
    "001C60": "CSP Frontier Technologies,Inc.",
    "001C2F": "Pfister GmbH",
    "001C27": "Sunell Electronics Co.",
    "001C22": "Aeris Elettronica s.r.l.",
    "001C51": "Celeno Communications",
    "001C54": "Hillstone Networks Inc",
    "001C59": "DEVON IT",
    "001C39": "S Netsystems Inc.",
    "001C37": "Callpod, Inc.",
    "001C33": "Sutron",
    "001C1D": "CHENZHOU GOSPELL DIGITAL TECHNOLOGY CO.,LTD",
    "001C0F": "Cisco Systems, Inc",
    "001BF6": "CONWISE Technology Corporation Ltd.",
    "001BF8": "Digitrax Inc.",
    "001BF1": "Nanjing SilverNet Software Co., Ltd.",
    "001BEF": "Blossoms Digital Technology Co.,Ltd.",
    "001BEB": "DMP Electronics INC.",
    "001BCE": "Measurement Devices Ltd",
    "001BC9": "FSN DISPLAY INC",
    "001BC3": "Mobisolution Co.,Ltd",
    "001BB8": "BLUEWAY ELECTRONIC CO;LTD",
    "001BB2": "Intellect International NV",
    "001BB0": "Bharat Electronics Limited",
    "001BAD": "iControl Incorporated",
    "001BA7": "Lorica Solutions",
    "001B87": "Deepsound Tech. Co., Ltd",
    "001B7F": "TMN Technologies Telecomunicacoes Ltda",
    "001B7E": "Beckmann GmbH",
    "001BA5": "MyungMin Systems, Inc.",
    "001BA2": "IDS Imaging Development Systems GmbH",
    "001B9D": "Novus Security Sp. z o.o.",
    "001B7A": "Nintendo Co., Ltd.",
    "001B72": "Sicep s.p.a.",
    "001B74": "MiraLink Corporation",
    "001B6D": "Midtronics, Inc.",
    "001B6F": "Teletrak Ltd",
    "001BDC": "Vencer Co., Ltd.",
    "001BD5": "Cisco Systems, Inc",
    "001BE7": "Postek Electronics Co., Ltd.",
    "001BE3": "Health Hero Network, Inc.",
    "001B95": "VIDEO SYSTEMS SRL",
    "001B90": "Cisco Systems, Inc",
    "001AF1": "Embedded Artists AB",
    "001AF8": "Copley Controls Corporation",
    "001AF5": "PENTAONE. CO., LTD.",
    "001AED": "INCOTEC GmbH",
    "001AEE": "Shenztech Ltd",
    "001B1D": "Phoenix International Co., Ltd",
    "001B1A": "e-trees Japan, Inc.",
    "001B12": "Apprion",
    "001B5F": "Alien Technology",
    "001B5E": "BPL Limited",
    "001B61": "Digital Acoustics, LLC",
    "001B5C": "Azuretec Co., Ltd.",
    "001B34": "Focus System Inc.",
    "001B3A": "SIMS Corp.",
    "001AFE": "SOFACREAL",
    "001B4B": "SANION Co., Ltd.",
    "001B4D": "Areca Technology Corporation",
    "001B0D": "Cisco Systems, Inc",
    "001B0A": "Intelligent Distributed Controls Ltd",
    "001B2E": "Sinkyo Electron Inc",
    "001B2D": "Med-Eng Systems Inc.",
    "001AE3": "Cisco Systems, Inc",
    "001ADF": "Interactivetv Pty Limited",
    "001AE1": "EDGE ACCESS INC",
    "001AE9": "Nintendo Co., Ltd.",
    "001AE5": "Mvox Technologies Inc.",
    "001AE4": "Medicis Technologies Corporation",
    "001A98": "Asotel Communication Limited Taiwan Branch",
    "001A97": "fitivision technology Inc.",
    "001A90": "Tr\u00f3pico Sistemas e Telecomunica\u00e7\u00f5es da Amaz\u00f4nia LTDA. ",
    "001A94": "Votronic GmbH",
    "001A86": "New Wave Design & Verification",
    "001ACE": "YUPITERU CORPORATION",
    "001ACC": "Celestial Semiconductor, Ltd",
    "001AC7": "UNIPOINT",
    "001AB1": "Asia Pacific Satellite Industries Co., Ltd.",
    "001AB2": "Cyber Solutions Inc.",
    "001AB7": "Ethos Networks LTD.",
    "001AA2": "Cisco Systems, Inc",
    "001AA5": "BRN Phoenix",
    "001A9C": "RightHand Technologies, Inc.",
    "001A9E": "ICON Digital International Limited",
    "001AD2": "Eletronica Nitron Ltda",
    "001AC0": "JOYBIEN TECHNOLOGIES CO., LTD.",
    "001AC2": "YEC Co.,Ltd.",
    "001A7E": "LN Srithai Comm Ltd.",
    "001A72": "Mosart Semiconductor Corp.",
    "001A37": "Lear Corporation",
    "001A38": "Sanmina-SCI",
    "001A2B": "Ayecom Technology Co., Ltd.",
    "001A28": "ASWT Co., LTD. Taiwan Branch H.K.",
    "001A2C": "SATEC Co.,LTD",
    "001A27": "Ubistar",
    "001A68": "Weltec Enterprise Co., Ltd.",
    "001A5A": "Korea Electric Power Data Network  (KDN) Co., Ltd",
    "001A5F": "KitWorks.fi Ltd.",
    "001A2E": "Ziova Coporation",
    "001A32": "ACTIVA MULTIMEDIA",
    "001A35": "BARTEC GmbH",
    "001A48": "Takacom Corporation",
    "001A21": "Brookhuis Applied Technologies BV",
    "001A0B": "BONA TECHNOLOGY INC.",
    "001A06": "OpVista, Inc.",
    "001A5D": "Mobinnova Corp.",
    "001A00": "MATRIX INC.",
    "0019FF": "Finnzymes",
    "0019FA": "Cable Vision Electronics CO., LTD.",
    "0019F1": "Star Communication Network Technology Co.,Ltd",
    "0019EC": "Sagamore Systems, Inc.",
    "0019A5": "RadarFind Corporation",
    "001993": "Changshu Switchgear MFG. Co.,Ltd. (Former Changshu Switchgea",
    "0019C8": "AnyDATA Corporation",
    "0019C4": "Infocrypt Inc.",
    "0019BC": "ELECTRO CHANCE SRL",
    "001973": "Zeugma Systems",
    "001975": "Beijing Huisen networks technology Inc",
    "00197B": "Picotest Corp.",
    "0019D8": "MAXFOR",
    "0019D5": "IP Innovations, Inc.",
    "0019CC": "RCG (HK) Ltd",
    "001981": "Vivox Inc",
    "0019EA": "TeraMage Technologies Co., Ltd.",
    "00199B": "Diversified Technical Systems, Inc.",
    "001990": "ELM DATA Co., Ltd.",
    "00198F": "Nokia Bell N.V.",
    "0019A3": "asteel electronique atlantique",
    "0019AD": "BOBST SA",
    "0019A2": "ORDYN TECHNOLOGIES",
    "001926": "BitsGen Co., Ltd.",
    "001929": "2M2B Montadora de Maquinas Bahia Brasil LTDA",
    "00192A": "Antiope Associates",
    "00190F": "Advansus Corp.",
    "001911": "Just In Mobile Information Technologies (Shanghai) Co., Ltd.",
    "00192E": "Spectral Instruments, Inc.",
    "00192B": "Aclara RF Systems Inc.",
    "00191E": "Beyondwiz Co., Ltd.",
    "00191F": "Microlink communications Inc.",
    "001920": "KUME electric Co.,Ltd.",
    "0018F1": "Chunichi Denshi Co.,LTD.",
    "0018F2": "Beijing Tianyu Communication Equipment Co., Ltd",
    "0018EC": "Welding Technology Corporation",
    "00196E": "Metacom (Pty) Ltd.",
    "001965": "YuHua TelTech (ShangHai) Co., Ltd.",
    "001966": "Asiarock Technology Limited",
    "001917": "Posiflex Inc.",
    "001918": "Interactive Wear AG",
    "00190B": "Southern Vision Systems, Inc.",
    "001903": "Bigfoot Networks Inc",
    "00195C": "Innotech Corporation",
    "00195F": "Valemount Networks Corporation",
    "001900": "Intelliverese - DBA Voicecom",
    "001902": "Cambridge Consultants Ltd",
    "001945": "RF COncepts, LLC",
    "001948": "AireSpider Networks",
    "001943": "Belden",
    "001940": "Rackable Systems",
    "00193C": "HighPoint Technologies Incorporated",
    "00189B": "Thomson Inc.",
    "001894": "NPCore, Inc.",
    "001898": "KINGSTATE ELECTRONICS CORPORATION",
    "001892": "ads-tec GmbH",
    "001891": "Zhongshan General K-mate Electronics Co., Ltd",
    "001889": "WinNet Solutions Limited",
    "0018B2": "ADEUNIS RF",
    "0018B3": "TEC WizHome Co., Ltd.",
    "0018AC": "Shanghai Jiao Da HISYS Technology Co. Ltd.",
    "0018AB": "BEIJING LHWT MICROELECTRONICS INC.",
    "0018ED": "Accutech Ultrasystems Co., Ltd.",
    "0018E0": "ANAVEO",
    "0018CB": "Tecobest Technology Limited",
    "0018BD": "SHENZHEN DVBWORLD TECHNOLOGY CO., LTD.",
    "0018A5": "ADigit Technologies Corp.",
    "0018A6": "Persistent Systems, LLC",
    "001867": "Datalogic ADC",
    "001865": "Siemens Healthcare Diagnostics Manufacturing Ltd",
    "00186A": "Global Link Digital Technology Co,.LTD",
    "001878": "Mackware GmbH",
    "00186E": "3Com Ltd",
    "00186B": "Sambu Communics CO., LTD.",
    "001877": "Amplex A/S",
    "0018CE": "Dreamtech Co., Ltd",
    "0017D3": "Etymotic Research, Inc.",
    "0017CC": "Alcatel-Lucent",
    "0017C4": "Quanta Microsystems, INC.",
    "0017BD": "Tibetsystem",
    "0017BF": "Coherent Research Limited",
    "001847": "AceNet Technology Inc.",
    "001843": "Dawevision Ltd",
    "00182C": "Ascend Networks, Inc.",
    "001836": "REJ Co.,Ltd",
    "0017ED": "WooJooIT Ltd.",
    "0017DD": "Clipsal Australia",
    "0017D9": "AAI Corporation",
    "0017DC": "DAEMYUNG ZERO1",
    "00181B": "TaiJin Metal Co., Ltd.",
    "001805": "Beijing InHand Networking Technology Co.,Ltd.",
    "0017F9": "Forcom Sp. z o.o.",
    "0017F4": "ZERON ALLIANCE",
    "0017F7": "CEM Solutions Pvt Ltd",
    "00181D": "ASIA ELECTRONICS CO.,LTD",
    "00181F": "Palmmicro Communications",
    "00180D": "Terabytes Server Storage Tech Corp",
    "001854": "Argard Co., Ltd",
    "0017AD": "AceNet Corporation",
    "00179E": "Sirit Inc",
    "0017A7": "Mobile Computing Promotion Consortium",
    "00179F": "Apricorn",
    "0017A1": "3soft inc.",
    "00173C": "Extreme Engineering Solutions",
    "001736": "iiTron Inc.",
    "001737": "Industrie Dial Face S.p.A.",
    "001733": "SFR",
    "00174F": "iCatch Inc.",
    "00174A": "SOCOMEC",
    "00174E": "Parama-tech Co.,Ltd.",
    "00175B": "ACS Solutions Switzerland Ltd.",
    "001758": "ThruVision Ltd",
    "001792": "Falcom Wireless Comunications Gmbh",
    "001797": "Telsy Elettronica S.p.A.",
    "00178C": "Independent Witness, Inc",
    "00176F": "PAX Computer Technology(Shenzhen) Ltd.",
    "001771": "APD Communications Ltd",
    "001766": "Accense Technology, Inc.",
    "001764": "ATMedia GmbH",
    "00175E": "Zed-3",
    "0017B4": "Remote Security Systems, LLC",
    "0017AC": "O'Neil Product Development Inc.",
    "001788": "Philips Lighting BV",
    "001729": "Ubicod Co.LTD",
    "001727": "Thermo Ramsey Italia s.r.l.",
    "00172A": "Proware Technology Corp.(By Unifosa)",
    "00171B": "Innovation Lab Corp.",
    "001714": "BR Controls Nederland bv",
    "001709": "Exalt Communications",
    "001703": "MOSDAN Internation Co.,Ltd",
    "0016FC": "TOHKEN CO.,LTD.",
    "0016F8": "AVIQTECH TECHNOLOGY CO., LTD.",
    "0016E8": "Lumissil Microsystems",
    "0016E9": "Tiba Medical Inc",
    "0016DC": "ARCHOS",
    "0016D0": "ATech elektronika d.o.o.",
    "001725": "Liquid Computing",
    "0016F0": "Dell",
    "0016A9": "2EI",
    "0016B0": "VK Corporation",
    "0016B1": "KBS",
    "00167F": "Bluebird Soft Inc.",
    "00167C": "iRex Technologies BV",
    "001673": "Bury GmbH & Co. KG",
    "00168C": "DSL Partner AS",
    "001683": "WEBIO International Co.,.Ltd.",
    "001684": "Donjin Co.,Ltd.",
    "001687": "Chubb CSC-Vendor AP",
    "0016C3": "BA Systems Inc",
    "0016C1": "Eleksen Ltd",
    "00169C": "Cisco Systems, Inc",
    "001694": "Sennheiser Communications A/S",
    "00165F": "Fairmount Automation",
    "001657": "Aegate Ltd",
    "0016AE": "INVENTEL",
    "0016AC": "Toho Technology Corp.",
    "00164C": "PLANET INT Co., Ltd",
    "001649": "SetOne GmbH",
    "001665": "Cellon France",
    "00161B": "Micronet Corporation",
    "001618": "HIVION Co., Ltd.",
    "00161E": "Woojinnet",
    "00161F": "SUNWAVETEC Co., Ltd.",
    "001614": "Picosecond Pulse Labs",
    "00160E": "Optica Technologies Inc.",
    "0015E3": "Dream Technologies Corporation",
    "0015E0": "Ericsson",
    "0015D3": "Pantech&Curitel Communications, Inc.",
    "0015CA": "TeraRecon, Inc.",
    "001647": "Cisco Systems, Inc",
    "001643": "Sunhillo Corporation",
    "001644": "LITE-ON Technology Corp.",
    "00163E": "Xensource, Inc.",
    "001634": "Mathtech, Inc.",
    "0015F9": "Cisco Systems, Inc",
    "0015FD": "Complete Media Systems",
    "0015F6": "SCIENCE AND ENGINEERING SERVICES, INC.",
    "001605": "YORKVILLE SOUND INC.",
    "00162C": "Xanboo",
    "0015F3": "PELTOR AB",
    "0015E7": "Quantec Tontechnik",
    "00154B": "Wonde Proud Technology Co., Ltd",
    "001548": "CUBE TECHNOLOGIES",
    "001570": "Zebra Technologies Inc",
    "00156E": "A. W. Communication Systems Ltd",
    "001568": "Dilithium Networks",
    "00155F": "GreenPeak Technologies",
    "0015AD": "Accedian Networks",
    "0015AC": "Capelon AB",
    "0015A9": "KWANG WOO I&C CO.,LTD",
    "00153C": "Kprotech Co., Ltd.",
    "00158E": "Plustek.INC",
    "001589": "D-MAX Technology Co.,Ltd",
    "001574": "Horizon Semiconductors Ltd.",
    "00155A": "DAINIPPON PHARMACEUTICAL CO., LTD.",
    "00E0A8": "SAT GmbH & Co.",
    "001598": "Kolektor group",
    "00158A": "SURECOM Technology Corp.",
    "0015B4": "Polymap  Wireless LLC",
    "0014FA": "AsGa S.A.",
    "0014FB": "Technical Solutions Inc.",
    "0014F4": "DekTec Digital Video B.V.",
    "0014F5": "OSI Security Devices",
    "0014EC": "Acro Telecom",
    "001512": "Zurich University of Applied Sciences",
    "00150A": "Sonoa Systems, Inc",
    "0014FF": "Precise Automation, Inc.",
    "0014D2": "Kyuden Technosystems Corporation",
    "0014DC": "Communication System Design & Manufacturing (CSDM)",
    "0014EB": "AwarePoint Corporation",
    "00153A": "Shenzhen Syscan Technology Co.,Ltd.",
    "001532": "Consumer Technologies Group, LLC",
    "0014CD": "DigitalZone Co., Ltd.",
    "0014C0": "Symstream Technology Group Ltd",
    "00151D": "M2I CORPORATION",
    "0014AD": "Gassner Wiege- und Me\u00dftechnik GmbH",
    "0014AF": "Datasym POS Inc.",
    "0014A9": "Cisco Systems, Inc",
    "00149B": "Nokota Communications, LLC",
    "0014A1": "Synchronous Communication Corp",
    "00149E": "UbONE Co., Ltd",
    "0014C1": "U.S. Robotics Corporation",
    "0014C4": "Vitelcom Mobile Technology",
    "0014AB": "Senhai Electronic Technology Co., Ltd.",
    "0014B0": "Naeil Community",
    "00148E": "Tele Power Inc.",
    "00148F": "Protronic (Far East) Ltd.",
    "00148C": "General Dynamics Mission Systems",
    "001487": "American Technology Integrators",
    "00144D": "Intelligent Systems",
    "00144A": "Taiwan Thick-Film Ind. Corp.",
    "001445": "Telefon-Gradnja d.o.o.",
    "001444": "Grundfos Holding",
    "001468": "CelPlan International, Inc.",
    "001481": "Multilink Inc",
    "00147C": "3Com Ltd",
    "001464": "Cryptosoft",
    "00145E": "IBM Corp",
    "0014A2": "Core Micro Systems Inc.",
    "001458": "HS Automatic ApS",
    "001437": "GSTeletech Co.,Ltd.",
    "001431": "PDL Electronics Ltd",
    "00142B": "Edata Communication Inc.",
    "00142C": "Koncept International, Inc.",
    "0013BA": "ReadyLinks Inc",
    "0013B8": "RyCo Electronic Systems Limited",
    "0013B6": "Sling Media, Inc.",
    "001424": "Merry Electrics CO., LTD.",
    "001421": "Total Wireless Technologies Pte. Ltd.",
    "001420": "G-Links networking company",
    "001418": "C4Line",
    "00141B": "Cisco Systems, Inc",
    "00140F": "Federal State Unitary Enterprise Leningrad R&D Institute of",
    "001407": "Sperian Protection Instrumentation",
    "001406": "Go Networks",
    "0013F0": "Wavefront Semiconductor",
    "0013EF": "Kingjon Digital Technology Co.,Ltd",
    "0013EB": "Sysmaster Corporation",
    "0013EC": "Netsnapper Technologies SARL",
    "00140A": "WEPIO Co., Ltd.",
    "0013FA": "LifeSize Communications, Inc",
    "0013FB": "RKC INSTRUMENT INC.",
    "0013C3": "Cisco Systems, Inc",
    "0013BD": "HYMATOM SA",
    "0013D0": "t+ Medical Ltd",
    "0013D2": "PAGE IBERICA, S.A.",
    "0013D1": "KIRK telecom A/S",
    "001366": "Neturity Technologies Inc.",
    "00135B": "PanelLink Cinema, LLC",
    "00135E": "EAB/RWI/K",
    "001373": "BLwave Electronics Co., Ltd",
    "00136F": "PacketMotion, Inc.",
    "00134E": "Valox Systems, Inc.",
    "001344": "Fargo Electronics Inc.",
    "001348": "Artila Electronics Co., Ltd.",
    "00137D": "Dynalab, Inc.",
    "001383": "Application Technologies and Engineering Research Laboratory",
    "001387": "27M Technologies AB",
    "0013B4": "Appear AS",
    "0013AE": "Radiance Technologies, Inc.",
    "001397": "Oracle Corporation ",
    "001336": "Tianjin 712 Communication Broadcasting co., ltd.",
    "0012DB": "ZIEHL industrie-elektronik GmbH + Co KG",
    "0012D6": "Jiangsu Yitong High-Tech Co.,Ltd",
    "0012DA": "Cisco Systems, Inc",
    "0012D3": "Zetta Systems, Inc.",
    "0012D5": "Motion Reality Inc.",
    "0012D8": "International Games System Co., Ltd.",
    "001322": "DAQ Electronics, Inc.",
    "001323": "Cap Co., Ltd.",
    "001314": "Asiamajor Inc.",
    "001316": "L-S-B Broadcast Technologies GmbH",
    "0012BA": "FSI Systems, Inc.",
    "0012B2": "AVOLITES LTD.",
    "0012AE": "HLS HARD-LINE Solutions Inc.",
    "0012AF": "ELPRO Technologies",
    "0012E9": "Abbey Systems Ltd",
    "0012E6": "SPECTEC COMPUTER CO., LTD.",
    "0012E3": "Agat Soft LLC",
    "001312": "Amedia Networks Inc.",
    "001300": "IT-FACTORY, INC.",
    "0012EF": "OneAccess SA",
    "00123B": "KeRo Systems ApS",
    "001236": "ConSentry Networks",
    "001235": "Andrew Corporation",
    "001257": "LeapComm Communication Technologies Inc.",
    "001251": "SILINK",
    "001245": "Zellweger Analytics, Inc.",
    "0012A2": "VITA",
    "0012A8": "intec GmbH",
    "0012A5": "Dolphin Interconnect Solutions AS",
    "00129E": "Surf Communications Inc.",
    "00129A": "IRT Electronics Pty Ltd",
    "001282": "Qovia",
    "001287": "Digital Everywhere Unterhaltungselektronik GmbH",
    "001285": "Gizmondo Europe Ltd",
    "001242": "Millennial Net",
    "001241": "a2i marketing center",
    "00126D": "University of California, Berkeley",
    "001267": "Panasonic Corporation",
    "001265": "Enerdyne Technologies, Inc.",
    "00128D": "STB Datenservice GmbH",
    "00128E": "Q-Free ASA",
    "001292": "Griffin Technology",
    "0011F3": "NeoMedia Europe AG",
    "0011E9": "STARNEX CO., LTD.",
    "0011EC": "AVIX INC.",
    "0011E7": "WORLDSAT - Texas de France",
    "0011E8": "Tixi.Com",
    "0011FD": "KORG INC.",
    "0011FA": "Rane Corporation",
    "0011F2": "Institute of Network Technologies",
    "001228": "Data Ltd.",
    "00121F": "Harding Instruments",
    "001220": "Cadco Systems",
    "001210": "WideRay Corp",
    "00120D": "Advanced Telecommunication Technologies, Inc.",
    "00120E": "AboCom",
    "001202": "Decrane Aerospace - Audio International Inc.",
    "0011FE": "Keiyo System Research, Inc.",
    "0011CE": "Ubisense Limited",
    "0011D0": "Tandberg Data ASA",
    "0011C3": "Transceiving System Technology Corporation",
    "0011C2": "United Fiber Optic Communication",
    "0011B7": "Octalix B.V.",
    "0011B9": "Inner Range Pty. Ltd.",
    "0011A7": "Infilco Degremont Inc.",
    "001226": "Japan Direx Corporation",
    "001229": "BroadEasy Technologies Co.,Ltd",
    "001222": "Skardin (UK) Ltd",
    "0011BF": "AESYS S.p.A.",
    "0011E0": "U-MEDIA Communications, Inc.",
    "0011A3": "LanReady Technologies Inc.",
    "0011A4": "JStream Technologies Inc.",
    "001198": "Prism Media Products Limited",
    "001197": "Monitoring Technologies Limited",
    "001199": "2wcom Systems GmbH",
    "001176": "Intellambda Systems, Inc.",
    "001177": "Coaxial Networks, Inc.",
    "001170": "GSC SRL",
    "00116B": "Digital Data Communications Asia Co.,Ltd",
    "001169": "EMS Satcom",
    "001162": "STAR MICRONICS CO.,LTD.",
    "001161": "NetStreams, LLC",
    "001156": "Pharos Systems NZ",
    "001159": "MATISSE NETWORKS INC",
    "00115C": "Cisco Systems, Inc",
    "001181": "InterEnergy Co.Ltd,",
    "001194": "Chi Mei Communication Systems, Inc.",
    "00113D": "KN SOLTEC CO.,LTD.",
    "001134": "MediaCell, Inc.",
    "00115D": "Cisco Systems, Inc",
    "001149": "Proliphix Inc.",
    "00113B": "Micronet Communications Inc.",
    "001135": "Grandeye Ltd",
    "001126": "Venstar Inc.",
    "000FC0": "DELCOMp",
    "000FBA": "Tevebox AB",
    "000FB8": "CallURL Inc.",
    "000FB7": "Cavium",
    "000FD1": "Applied Wireless Identifications Group, Inc.",
    "000FC1": "WAVE Corporation",
    "000FC4": "NST co.,LTD.",
    "000FC9": "Allnet GmbH",
    "000FC6": "Eurocom Industries A/S",
    "000FBE": "e-w/you Inc.",
    "001116": "COTEAU VERT CO., LTD.",
    "001110": "Maxanna Technology Co., Ltd.",
    "001109": "Micro-Star International",
    "001104": "TELEXY",
    "001100": "Schneider Electric",
    "000FFA": "Optinel Systems, Inc.",
    "000FFD": "Glorytek Network Inc.",
    "00112E": "CEICOM",
    "001121": "Cisco Systems, Inc",
    "001122": "CIMSYS Inc",
    "000FD8": "Force, Inc.",
    "000FD3": "Digium",
    "000FF9": "Valcretec, Inc.",
    "000FF7": "Cisco Systems, Inc",
    "000FA9": "PC Fabrik",
    "000F85": "ADDO-Japan Corporation",
    "000F82": "Mortara Instrument, Inc.",
    "000F81": "PAL Pacific Inc.",
    "000F7F": "UBSTORAGE Co.,Ltd.",
    "000F9A": "Synchrony, Inc.",
    "000FA0": "Canon Korea Inc.",
    "000F95": "ELECOM Co.,LTD Laneed Division",
    "000F4B": "Oracle Corporation",
    "000F4A": "Kyushu-kyohan co.,ltd",
    "000F44": "Tivella Inc.",
    "000F40": "Optical Internetworking Forum",
    "000F60": "Lifetron Co.,Ltd",
    "000F5B": "Delta Information Systems, Inc.",
    "000F55": "Datawire Communication Networks Inc.",
    "000F56": "Continuum Photonics Inc",
    "000F49": "Northover Solutions Limited",
    "000F77": "DENTUM CO.,LTD",
    "000F8A": "WideView",
    "000F90": "Cisco Systems, Inc",
    "000F8D": "FAST TV-Server AG",
    "000F7B": "Arce Sistemas, S.A.",
    "000F68": "Vavic Network Technology, Inc.",
    "000F33": "DUALi Inc.",
    "000F2F": "W-LINX TECHNOLOGY CO., LTD.",
    "000EEB": "Sandmartin(zhong shan)Electronics Co.,Ltd",
    "000EEC": "Orban",
    "000EEF": "Private",
    "000EF1": "EZQUEST INC.",
    "000EE7": "AAC ELECTRONICS CORP.",
    "000EE2": "Custom Engineering",
    "000ED0": "Privaris, Inc.",
    "000EC3": "Logic Controls, Inc.",
    "000EC4": "Iskra Transmission d.d.",
    "000EC1": "MYNAH Technologies",
    "000EBD": "Burdick, a Quinton Compny",
    "000EE3": "Chiyu Technology Co.,Ltd",
    "000EE5": "bitWallet, Inc.",
    "000ECF": "PROFIBUS Nutzerorganisation e.V.",
    "000EDA": "C-TECH UNITED CORP.",
    "000EC9": "YOKO Technology Corp.",
    "000F0A": "Clear Edge Networks",
    "000F02": "Digicube Technology Co., Ltd",
    "000F05": "3B SYSTEM INC.",
    "000F1C": "DigitAll World Co., Ltd",
    "000F1A": "Gaming Support B.V.",
    "000EB9": "HASHIMOTO Electronics Industry Co.,Ltd.",
    "000EB2": "Micro-Research Finland Oy",
    "000EB5": "Ecastle Electronics Co., Ltd.",
    "000EA1": "Formosa Teletek Corporation",
    "000E98": "HME Clear-Com LTD.",
    "000E99": "Spectrum Digital, Inc",
    "000EF7": "Vulcan Portals Inc",
    "000EE9": "WayTech Development, Inc.",
    "000EF0": "Festo AG & Co. KG",
    "000EA7": "Endace Technology",
    "000E95": "Fujiya Denki Seisakusho Co.,Ltd.",
    "000E97": "Ultracker Technology CO., Inc",
    "000E45": "Beijing Newtry Electronic Technology Ltd",
    "000E3F": "Soronti, Inc.",
    "000E82": "Infinity Tech",
    "000E89": "CLEMATIC",
    "000E79": "Ample Communications Inc.",
    "000E15": "Tadlys LTD",
    "000E1D": "ARION Technology Inc.",
    "000E12": "Adaptive Micro Systems Inc.",
    "000E4F": "Trajet GmbH",
    "000E50": "Thomson Telecom Belgium",
    "000E4B": "atrium c and i",
    "000E38": "Cisco Systems, Inc",
    "000E36": "HEINESYS, Inc.",
    "000E2B": "Safari Technologies",
    "000E28": "Dynamic Ratings P/L",
    "000E17": "Private",
    "000E6F": "IRIS Corporation Berhad",
    "000DCB": "Petcomkorea Co., Ltd.",
    "000DCC": "NEOSMART Corp.",
    "000DCE": "Dynavac Technology Pte Ltd",
    "000DB9": "PC Engines GmbH",
    "000DB7": "SANKO ELECTRIC CO,.LTD",
    "000DC3": "First Communication, Inc.",
    "000DE6": "YOUNGBO ENGINEERING CO.,LTD",
    "000DDF": "Japan Image & Network Inc.",
    "000E04": "CMA/Microdialysis AB",
    "000DE8": "Nasaco Electronics Pte. Ltd",
    "000DE9": "Napatech Aps",
    "000DDB": "AIRWAVE TECHNOLOGIES INC.",
    "000DAC": "Japan CBM Corporation",
    "000E01": "ASIP Technologies Inc.",
    "000D45": "Tottori SANYO Electric Co., Ltd.",
    "000D46": "Parker SSD Drives",
    "000D41": "Siemens AG ICM MP UC RD IT KLF1",
    "000D42": "Newbest Development Limited",
    "000D3C": "i.Tech Dynamic Ltd",
    "000D2A": "Scanmatic AS",
    "000D29": "Cisco Systems, Inc",
    "000D26": "Primagraphics Limited",
    "000D65": "Cisco Systems, Inc",
    "000D55": "SANYCOM Technology Co.,Ltd",
    "000D57": "Fujitsu I-Network Systems Limited.",
    "000D58": "Private",
    "000D51": "DIVR Systems, Inc.",
    "000D19": "ROBE Show lighting",
    "000D1C": "Amesys Defense",
    "000D13": "Wilhelm Rutenbeck GmbH&Co.KG",
    "000D16": "UHS Systems Pty Ltd",
    "000D1F": "AV Digital",
    "000D0C": "MDI Security Systems",
    "000D01": "P&E Microcomputer Systems, Inc.",
    "000D35": "PAC International Ltd",
    "000D95": "Opti-cell, Inc.",
    "000D91": "Eclipse (HQ Espana) S.L.",
    "000D62": "Funkwerk Dabendorf GmbH",
    "000D80": "Online Development Inc",
    "000CA3": "Rancho Technology, Inc.",
    "000CAA": "Cubic Transportation Systems Inc",
    "000CAC": "Citizen Watch Co., Ltd.",
    "000CAE": "Ailocom Oy",
    "000C9E": "MemoryLink Corp.",
    "000C95": "PrimeNet",
    "000C9C": "Chongho information & communications",
    "000C97": "NV ADB TTV Technologies SA",
    "000C8F": "Nergal s.r.l.",
    "000C92": "WolfVision Gmbh",
    "000C93": "Xeline Co., Ltd.",
    "000C7F": "synertronixx GmbH",
    "000C81": "Schneider Electric (Australia) ",
    "000C80": "Opelcomm Inc.",
    "000C7D": "TEIKOKU ELECTRIC MFG. CO., LTD",
    "000CF0": "M & N GmbH",
    "000CF4": "AKATSUKI ELECTRIC MFG.CO.,LTD.",
    "000CF3": "CALL IMAGE SA",
    "000CE0": "Trek Diagnostics Inc.",
    "000CE2": "Rolls-Royce",
    "000CD1": "SFOM Technology Corp.",
    "000CD8": "M. K. Juchheim GmbH & Co",
    "000CCE": "Cisco Systems, Inc",
    "000CCF": "Cisco Systems, Inc",
    "000D00": "Seaway Networks Inc.",
    "000CF9": "Xylem Water Solutions",
    "000CED": "Real Digital Media",
    "000CC6": "Ka-Ro electronics GmbH",
    "000CC4": "Tiptel AG",
    "000CBA": "Jamex, Inc.",
    "000CB7": "Nanjing Huazhuo Electronics Co., Ltd.",
    "000CAD": "BTU International",
    "000C0A": "Guangdong Province Electronic Technology Research Institute",
    "000C0B": "Broadbus Technologies",
    "000C07": "Iftest AG",
    "000C0C": "APPRO TECHNOLOGY INC.",
    "000C18": "Zenisu Keisoku Inc.",
    "000C0E": "XtremeSpectrum, Inc.",
    "000C12": "Micro-Optronic-Messtechnik GmbH",
    "000C10": "PNI Corporation",
    "000C40": "Altech Controls",
    "000C3E": "Crest Audio",
    "000C3A": "Oxance",
    "000C35": "KaVo Dental GmbH & Co. KG",
    "000C37": "Geomation, Inc.",
    "000C56": "Megatel Computer (1986) Corp.",
    "000C57": "MACKIE Engineering Services Belgium BVBA",
    "000C76": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "000C4A": "Cygnus Microsystems (P) Limited",
    "000C23": "Beijing Lanchuan Tech. Co., Ltd.",
    "000C25": "Allied Telesis Labs, Inc. ",
    "000C54": "Pedestal Networks, Inc",
    "000C68": "SigmaTel, Inc.",
    "000BE5": "HIMS International Corporation",
    "000BE9": "Actel Corporation",
    "000BE3": "Key Stream Co., Ltd.",
    "000BE8": "AOIP",
    "000BDC": "AKCP",
    "000BD8": "Industrial Scientific Corp.",
    "000BD7": "DORMA Time + Access GmbH",
    "000BD3": "cd3o",
    "000BD5": "Nvergence, Inc.",
    "000BD1": "Aeronix, Inc.",
    "000BD2": "Remopro Technology Inc.",
    "000BC7": "ICET S.p.A.",
    "000B9F": "Neue ELSA GmbH",
    "000B95": "eBet Gaming Systems Pty Ltd",
    "000B98": "NiceTechVision",
    "000B9E": "Yasing Technology Corp.",
    "000B88": "Vidisco ltd.",
    "000BB8": "Kihoku Electronic Co.",
    "000BC0": "China IWNComm Co., Ltd.",
    "000BB0": "Sysnet Telematica srl",
    "000BB4": "RDC Semiconductor Inc.,",
    "000B8A": "MITEQ Inc.",
    "000B8C": "Flextronics",
    "000B90": "ADVA Optical Networking Ltd.",
    "000B79": "X-COM, Inc.",
    "000BF9": "Gemstone Communications, Inc.",
    "000BFC": "Cisco Systems, Inc",
    "000BAC": "3Com Ltd",
    "000BEA": "Zultys Technologies",
    "000BDD": "TOHOKU RICOH Co., LTD.",
    "000B4C": "Clarion (M) Sdn Bhd",
    "000B4B": "VISIOWAVE SA",
    "000B36": "Productivity Systems, Inc.",
    "000B35": "Quad Bit System co., Ltd.",
    "000B3F": "Anthology Solutions Inc.",
    "000B15": "Platypus Technology",
    "000B18": "Private",
    "000B0D": "Air2U, Inc.",
    "000B0B": "Corrent Corporation",
    "000B08": "Pillar Data Systems",
    "000B54": "BiTMICRO Networks, Inc.",
    "000B43": "Microscan Systems, Inc.",
    "000B45": "Cisco Systems, Inc",
    "000B21": "G-Star Communications Inc.",
    "000B25": "Aeluros",
    "000B1A": "Industrial Defender, Inc.",
    "000B68": "Addvalue Communications Pte Ltd",
    "000B58": "Astronautics C.A  LTD",
    "000B55": "ADInstruments",
    "000B87": "American Reliance Inc.",
    "000B75": "Iosoft Ltd.",
    "000B70": "Load Technology, Inc.",
    "000B6F": "Media Streaming Networks Inc",
    "000AFA": "Traverse Technologies Australia",
    "000AFC": "Core Tec Communications, LLC",
    "000B3C": "Cygnal Integrated Products, Inc.",
    "000B27": "Scion Corporation",
    "000B63": "Kaleidescape",
    "000ABA": "Arcon Technology Limited",
    "000AAF": "Pipal Systems",
    "000AB2": "Fresnel Wireless Systems",
    "000AB6": "COMPUNETIX, INC",
    "000AAA": "AltiGen Communications Inc.",
    "000ABE": "OPNET Technologies CO., LTD.",
    "000AC0": "Fuyoh Video Industry CO., LTD.",
    "000AAE": "Rosemount Process Analytical",
    "000AB3": "Fa. GIRA",
    "000AC9": "Zambeel Inc",
    "000AC1": "Futuretel",
    "000AC3": "eM Technics Co., Ltd.",
    "000AC4": "Daewoo Teletech Co., Ltd",
    "000AE8": "Cathay Roxus Information Technology Co. LTD",
    "000ADD": "Allworx Corp.",
    "000AE1": "EG Technology",
    "000ADF": "Gennum Corporation",
    "000ADA": "Vindicator Technologies",
    "000A8D": "EUROTHERM LIMITED",
    "000A9F": "Pannaway Technologies, Inc.",
    "000A8F": "Aska International Inc.",
    "000AA0": "Cedar Point Communications",
    "000A8E": "Invacom Ltd",
    "000AF1": "Clarity Design, Inc.",
    "000AF3": "Cisco Systems, Inc",
    "000AEC": "Koatsu Gas Kogyo Co., Ltd.",
    "000AE7": "ELIOP S.A.",
    "000A76": "Beida Jade Bird Huaguang Technology Co.,Ltd",
    "000A99": "Calamp Wireless Networks Inc",
    "000A93": "W2 Networks, Inc.",
    "000A89": "Creval Systems, Inc.",
    "000A80": "Telkonet Inc.",
    "000A79": "corega K.K",
    "000A30": "Visteon Corporation",
    "000A2F": "Artnix Inc.",
    "000A2C": "Active Tchnology Corporation",
    "000A2A": "QSI Systems Inc.",
    "0009D6": "KNC One GmbH",
    "0009C7": "Movistec",
    "0009C9": "BlueWINC Co., Ltd.",
    "0009D4": "Transtech Networks",
    "000A60": "Autostar Technology Pte Ltd",
    "000A5D": "FingerTec Worldwide Sdn Bhd",
    "000A5C": "Carel s.p.a.",
    "000A5A": "GreenNET Technologies Co.,Ltd.",
    "000A04": "3Com Ltd",
    "0009F1": "Yamaki Electric Corporation",
    "0009F4": "Alcon Laboratories, Inc.",
    "000A1F": "ART WARE Telecommunication Co., Ltd.",
    "000A15": "Silicon Data, Inc",
    "000A1B": "Stream Labs",
    "000A56": "HITACHI Maxell Ltd.",
    "000A51": "GyroSignal Technology Co., Ltd.",
    "000A53": "Intronics, Incorporated",
    "000A4A": "Targa Systems Ltd.",
    "000A39": "LoPA Information Technology",
    "000A37": "Procera Networks, Inc.",
    "000A34": "Identicard Systems Incorporated",
    "0009F5": "Emerson Network Power Co.,Ltd",
    "0009EA": "YEM Inc.",
    "0009E3": "Angel Iglesias S.A.",
    "000A1A": "Imerge Ltd",
    "000A0A": "SUNIX Co., Ltd.",
    "000A05": "Widax Corp.",
    "0009CB": "HBrain",
    "0009C5": "KINGENE Technology Corporation",
    "0009CD": "HUDSON SOFT CO.,LTD.",
    "0009C0": "6WIND",
    "0009BF": "Nintendo Co., Ltd.",
    "00097E": "IMI TECHNOLOGY CO., LTD",
    "000971": "Time Management, Inc.",
    "00096A": "Cloverleaf Communications Inc.",
    "00095A": "RACEWOOD TECHNOLOGY",
    "0009B4": "KISAN TELECOM CO., LTD.",
    "0009AE": "OKANO ELECTRIC CO.,LTD",
    "000974": "Innopia Technologies, Inc.",
    "000967": "Tachyon, Inc",
    "000996": "RDI",
    "00098B": "Entropic Communications, Inc.",
    "000991": "Intelligent Platforms, LLC.",
    "0009BA": "MAKU Informationstechik GmbH",
    "00092F": "Akom Technology Corporation",
    "00091F": "A&D Co., Ltd.",
    "000924": "Telebau GmbH",
    "00090D": "LEADER ELECTRONICS CORP.",
    "0008FE": "UNIK C&C Co.,Ltd.",
    "0008FF": "Trilogy Communications Ltd",
    "000904": "MONDIAL electronic",
    "000921": "Planmeca Oy",
    "000919": "MDS Gateways",
    "000918": "SAMSUNG TECHWIN CO.,LTD",
    "000910": "Simple Access Inc.",
    "0008FC": "Gigaphoton Inc.",
    "0008EF": "DIBAL,S.A.",
    "0008F0": "Next Generation Systems, Inc.",
    "000947": "Aztek, Inc.",
    "00093D": "Newisys,Inc.",
    "000951": "Apogee Imaging Systems",
    "00094E": "BARTECH SYSTEMS INTERNATIONAL, INC",
    "0008E9": "NextGig",
    "0008E7": "SHI ControlSystems,Ltd.",
    "000914": "COMPUTROLS INC.",
    "00090C": "Mayekawa Mfg. Co. Ltd.",
    "000938": "Allot Communications",
    "0008C1": "Avistar Communications Corporation",
    "0008C6": "Philips Consumer Communications",
    "0008BF": "Aptus Elektronik AB",
    "0008B8": "E.F. Johnson",
    "0008BB": "NetExcell",
    "0008BE": "XENPAK MSA Group",
    "000865": "JASCOM CO., LTD",
    "000864": "Fasy S.p.A.",
    "000860": "LodgeNet Entertainment Corp.",
    "000897": "Quake Technologies",
    "000890": "AVILINKS SA",
    "00088D": "Sigma-Links Inc.",
    "000884": "Index Braille AB",
    "000851": "Canadian Bank Note Company, Ltd.",
    "000859": "ShenZhen Unitone Electronics Co., Ltd.",
    "00084E": "DivergeNet, Inc.",
    "000825": "Acme Packet",
    "00081F": "Pou Yuen Tech Corp. Ltd.",
    "0008A8": "Systec Co., Ltd.",
    "0008A4": "Cisco Systems, Inc",
    "08006B": "ACCEL TECHNOLOGIES INC.",
    "0008D7": "HOW CORPORATION",
    "000794": "Simple Devices, Inc.",
    "000797": "Netpower Co., Ltd.",
    "00078C": "Elektronikspecialisten i Borlange AB",
    "0007C6": "VDS Vosskuhler GmbH",
    "0007CC": "Kaba Benzing GmbH",
    "0007C0": "NetZerver Inc.",
    "00047E": "Siqura B.V.",
    "000807": "Access Devices Limited",
    "0007FC": "Adept Systems Inc.",
    "0007E8": "EdgeWave",
    "0007A9": "Novasonics",
    "0007A1": "VIASYS Healthcare GmbH",
    "00079E": "Ilinx Co., Ltd.",
    "0007A0": "e-Watch Inc.",
    "000819": "Banksys",
    "00081A": "Sanrad Intelligence Storage Communications (2000) Ltd.",
    "000810": "Key Technology, Inc.",
    "0007BC": "Identix Inc.",
    "0007EB": "Cisco Systems, Inc",
    "0007F5": "Bridgeco Co AG",
    "0007EC": "Cisco Systems, Inc",
    "0007B8": "Corvalent Corporation",
    "0007DC": "Atek Co, Ltd.",
    "000748": "The Imaging Source Europe",
    "000746": "TURCK, Inc.",
    "000741": "Sierra Automated Systems",
    "000745": "Radlan Computer Communications Ltd.",
    "00073E": "China Great-Wall Computer Shenzhen Co., Ltd.",
    "00071E": "Tri-M Engineering / Nupak Dev. Corp.",
    "000717": "Wieland Electric GmbH",
    "000711": "Acterna",
    "00070A": "Unicom Automation Co., Ltd.",
    "000702": "Varex Imaging",
    "000705": "Endress & Hauser GmbH & Co",
    "0006FF": "Sheba Systems Co., Ltd.",
    "00072B": "Jung Myung Telecom Co., Ltd.",
    "000718": "iCanTek Co., Ltd.",
    "000716": "J & S Marine Ltd.",
    "00071A": "Finedigital Inc.",
    "00075C": "Eastman Kodak Company",
    "000756": "Juyoung Telecom",
    "00075A": "Air Products and Chemicals, Inc.",
    "000781": "Itron Inc.",
    "000788": "Clipcomm, Inc.",
    "00077B": "Millimetrix Broadband Networks",
    "000769": "Italiana Macchi SpA",
    "00076B": "Stralfors AB",
    "000768": "Danfoss A/S",
    "00075F": "VCS Video Communication Systems AG",
    "00073B": "Tenovis GmbH & Co KG",
    "000731": "Ophir-Spiricon LLC",
    "000709": "Westerstrand Urfabrik AB",
    "0006D6": "Cisco Systems, Inc",
    "000647": "Etrali S.A.",
    "0006CB": "Jotron Electronics A/S",
    "0006CD": "Leaf Imaging Ltd.",
    "0006C7": "RFNET Technologies Pte Ltd (S)",
    "000674": "Spectrum Control, Inc.",
    "000663": "Human Technology Co., Ltd.",
    "000665": "Sunny Giken, Inc.",
    "000662": "MBM Technology Ltd.",
    "00068B": "AirRunner Technologies, Inc.",
    "000687": "Omnitron Systems Technology, Inc.",
    "0006E4": "Citel Technologies Ltd.",
    "0006D9": "IPM-Net S.p.A.",
    "0006D3": "Alpha Telecom, Inc. U.S.A.",
    "0006A4": "INNOWELL Corp.",
    "000694": "Mobillian Corporation",
    "000695": "Ensure Technologies, Inc.",
    "000691": "PT Inovacao",
    "000692": "Intruvert Networks, Inc.",
    "0006A1": "Celsian Technologies, Inc.",
    "0006AB": "W-Link Systems, Inc.",
    "0006AC": "Intersoft Co.",
    "0006EB": "Global Data",
    "0006E3": "Quantitative Imaging Corporation",
    "0006B9": "A5TEK Corp.",
    "0005C0": "Digital Network Alacarte Co., Ltd.",
    "0005B8": "Electronic Design Associates, Inc.",
    "0005BA": "Area Netwoeks, Inc.",
    "0005BF": "JustEzy Technology, Inc.",
    "0005AC": "Northern Digital, Inc.",
    "0005D1": "Metavector Technologies",
    "0005D5": "Speedcom Wireless",
    "0005D2": "DAP Technologies",
    "0005C5": "Flaga HF",
    "0005CA": "Hitron Technology, Inc.",
    "000669": "Datasound Laboratories Ltd",
    "00066E": "Delta Electronics, Inc.",
    "00065A": "Strix Systems",
    "000652": "Cisco Systems, Inc",
    "000656": "Tactel AB",
    "000641": "ITCN",
    "000648": "Seedsware, Inc.",
    "00064C": "Invicta Networks, Inc.",
    "000638": "Sungjin C&C Co., Ltd.",
    "00060F": "Narad Networks Inc",
    "000610": "Abeona Networks Inc",
    "000611": "Zeus Wireless, Inc.",
    "0005EC": "Mosaic Systems Inc.",
    "00061A": "Zetari Inc.",
    "00061F": "Vision Components GmbH",
    "00061B": "Notebook Development Lab.  Lenovo Japan Ltd.",
    "000602": "Cirkitech Electronics Co.",
    "0005FB": "ShareGate, Inc.",
    "000635": "PacketAir Networks, Inc.",
    "0005F0": "SATEC",
    "0005FE": "Traficon N.V.",
    "00057D": "Sun Communications, Inc.",
    "000581": "Snell",
    "000586": "Lucent Technologies",
    "00057B": "Chung Nam Electronic Co., Ltd.",
    "000571": "Seiwa Electronics Co.",
    "0005AD": "Topspin Communications, Inc.",
    "0005B1": "ASB Technology BV",
    "000599": "DRS Test and Energy Management or DRS-TEM",
    "00059A": "Cisco Systems, Inc",
    "0005AB": "Cyber Fone, Inc.",
    "000592": "Pultek Corp.",
    "00058B": "IPmental, Inc.",
    "000549": "Salira Optical Network Systems",
    "00054C": "RF Innovations Pty Ltd",
    "000543": "IQ Wireless GmbH",
    "000570": "Baydel Ltd.",
    "000577": "SM Information & Communication",
    "00056E": "National Enhance Technology, Inc.",
    "00056D": "Pacific Corporation",
    "000538": "Merilus, Inc.",
    "000530": "Andiamo Systems, Inc.",
    "00052B": "HORIBA, Ltd.",
    "00055C": "Kowa Company, Ltd.",
    "0004E9": "Infiniswitch Corporation",
    "0004E8": "IER, Inc.",
    "0004E0": "Procket Networks",
    "0004DB": "Tellus Group Corp.",
    "0004DD": "Cisco Systems, Inc",
    "008086": "COMPUTER GENERATION INC.",
    "000513": "VTLinx Multimedia Systems, Inc.",
    "00050E": "3ware, Inc.",
    "000510": "Infinite Shanghai Communication Terminals Ltd.",
    "000501": "Cisco Systems, Inc",
    "000508": "Inetcam, Inc.",
    "0004FF": "Acronet Co., Ltd.",
    "000504": "Naray Information & Communication Enterprise",
    "000509": "AVOC Nishimura Ltd.",
    "0004FB": "Commtech, Inc.",
    "000516": "SMART Modular Technologies",
    "000515": "Nuark Co., Ltd.",
    "00051B": "Magic Control Technology Corporation",
    "0004D4": "Proview Electronics Co., Ltd.",
    "0004CD": "Extenway Solutions Inc",
    "0004C0": "Cisco Systems, Inc",
    "0004BA": "KDD Media Will Corporation",
    "0004B6": "Stratex Networks, Inc.",
    "0004D7": "Omitec Instrumentation Ltd.",
    "00051D": "Airocon, Inc.",
    "0004B3": "Videotek, Inc.",
    "0004A4": "NetEnabled, Inc.",
    "000444": "Western Multiplex Corporation",
    "000439": "Rosco Entertainment Technology, Inc.",
    "00043A": "Intelligent Telecommunications, Inc.",
    "00043F": "ESTeem Wireless Modems, Inc",
    "000433": "Cyberboard A/S",
    "000473": "Photonex Corporation",
    "000470": "ipUnplugged AB",
    "00046C": "Cyber Technology Co., Ltd.",
    "000471": "IPrad",
    "00046E": "Cisco Systems, Inc",
    "000474": "LEGRAND",
    "000434": "Accelent Systems, Inc.",
    "00042D": "Sarian Systems, Ltd.",
    "00042E": "Netous Technologies, Ltd.",
    "000425": "Atmel Corporation",
    "00041B": "Bridgeworks Ltd.",
    "000492": "Hive Internet, Ltd.",
    "00048C": "Nayna Networks, Inc.",
    "000491": "Technovision, Inc.",
    "000493": "Tsinghua Unisplendour Co., Ltd.",
    "000494": "Breezecom, Ltd.",
    "000489": "YAFO Networks, Inc.",
    "00048A": "Temia Vertriebs GmbH",
    "000481": "Econolite Control Products, Inc.",
    "000477": "Scalant Systems, Inc.",
    "000416": "Parks S/A Comunicacoes Digitais",
    "00040F": "Asus Network Technologies, Inc.",
    "00040A": "Sage Systems",
    "000404": "Makino Milling Machine Co., Ltd.",
    "000405": "ACN Technologies",
    "000401": "Osaki Electric Co., Ltd.",
    "00045D": "BEKA Elektronik",
    "000459": "Veristar Corporation",
    "00044C": "JENOPTIK",
    "0003A7": "Unixtar Technology, Inc.",
    "0003AE": "Allied Advanced Manufacturing Pte, Ltd.",
    "0003A3": "MAVIX, Ltd.",
    "0003A1": "HIPER Information & Communication, Inc.",
    "000396": "EZ Cast Co., Ltd.",
    "00039A": "SiConnect",
    "000371": "Acomz Networks Corp.",
    "000363": "Miraesys Co., Ltd.",
    "00035F": "Pr\u00fcftechnik Condition Monitoring GmbH & Co. KG",
    "000360": "PAC Interactive Technology, Inc.",
    "000361": "Widcomm, Inc.",
    "000359": "DigitalSis",
    "000352": "Colubris Networks",
    "0003D2": "Crossbeam Systems, Inc.",
    "0003D0": "KOANKEISO Co., Ltd.",
    "0003CF": "Muxcom, Inc.",
    "0003D1": "Takaya Corporation",
    "0003F4": "NetBurner",
    "0003F2": "Seneca Networks",
    "0003F0": "Redfern Broadband Networks",
    "0003EB": "Atrica",
    "0003E7": "Logostek Co. Ltd.",
    "000374": "Control Microsystems",
    "000376": "Graphtec Technology, Inc.",
    "000378": "HUMAX Co., Ltd.",
    "00036D": "Runtop, Inc.",
    "00036E": "Nicon Systems (Pty) Limited",
    "000394": "Connect One",
    "00038A": "America Online, Inc.",
    "00038D": "PCS Revenue Control Systems, Inc.",
    "000388": "Fastfame Technology Co., Ltd.",
    "0003B6": "QSI Corporation",
    "0003B1": "ICU Medical, Inc.",
    "0003B3": "IA Link Systems Co., Ltd.",
    "0003AD": "Emerson Energy Systems AB",
    "00037E": "PORTech Communications, Inc.",
    "0003E3": "Cisco Systems, Inc",
    "0003DB": "Apogee Electronics Corp.",
    "00031F": "Condev Ltd.",
    "00029F": "L-3 Communication Aviation Recorders",
    "00031B": "Cellvision Systems, Inc.",
    "00031C": "Svenska Hardvarufabriken AB",
    "0001A8": "Welltech Computer Co., Ltd.",
    "000317": "Merlin Systems, Inc.",
    "000318": "Cyras Systems, Inc.",
    "00030C": "Telesoft Technologies Ltd.",
    "000308": "AM Communications, Inc.",
    "000307": "Secure Works, Inc.",
    "000306": "Fusion In Tech Co., Ltd.",
    "0002FC": "Cisco Systems, Inc",
    "00032A": "UniData Communication Systems, Inc.",
    "000325": "Arima Computer Corp.",
    "0002FA": "DX Antenna Co., Ltd.",
    "0002FB": "Baumuller Aulugen-Systemtechnik GmbH",
    "0002F6": "Equipe Communications",
    "000223": "ClickTV",
    "0002CB": "TriState Ltd.",
    "0002CA": "EndPoints, Inc.",
    "0002C4": "OPT Machine Vision Tech Co., Ltd",
    "0002BF": "dotRocket, Inc.",
    "0002E3": "LITE-ON Communications, Inc.",
    "0002DD": "Bromax Communications, Ltd.",
    "00034C": "Shanghai DigiVision Technology Co., Ltd.",
    "000349": "Vidicode Datacommunicatie B.V.",
    "00033E": "Tateyama System Laboratory Co., Ltd.",
    "00033C": "Daiden Co., Ltd.",
    "000272": "CC&C Technologies, Inc.",
    "00026D": "Adept Telecom",
    "00026B": "BCM Computers Co., Ltd.",
    "000266": "Thermalogic Corporation",
    "000268": "Harris Government Communications",
    "00025E": "High Technology Ltd",
    "000260": "Accordion Networks, Inc.",
    "00012F": "Twinhead International Corp",
    "000247": "Great Dragon Information Technology (Group) Co., Ltd.",
    "000243": "Raysis Co., Ltd.",
    "000289": "DNE Technologies",
    "000283": "Spectrum Controls, Inc.",
    "000284": "UK Grid Solutions Limited",
    "000280": "Mu Net, Inc.",
    "009064": "Thomson Inc.",
    "00029E": "Information Equipment Co., Ltd.",
    "00029B": "Kreatel Communications AB",
    "000293": "Solid Data Systems",
    "000231": "Ingersoll-Rand",
    "000234": "Imperial Technology, Inc.",
    "000236": "INIT GmbH",
    "00022B": "SAXA, Inc.",
    "00027A": "IOI Technology Corporation",
    "000274": "Tommy Technologies Corp.",
    "000224": "C-COR",
    "00021F": "Aculab PLC",
    "00021A": "Zuma Networks",
    "000222": "Chromisys, Inc.",
    "0002A7": "Vivace Networks",
    "000213": "S.D.E.L.",
    "00020A": "Gefran Spa",
    "0001D1": "CoNet Communications, Inc.",
    "0001B2": "Digital Processing Systems, Inc.",
    "0001B8": "Netsensity, Inc.",
    "0001B9": "SKF (U.K.) Limited",
    "0001BD": "Peterson Electro-Musical Products, Inc.",
    "0001B3": "Precision Electronic Manufacturing",
    "0001B7": "Centos, Inc.",
    "000209": "Shenzhen SED Information Technology Co., Ltd.",
    "000206": "Telital R&D Denmark A/S",
    "000202": "Amino Communications, Ltd.",
    "000201": "IFM Electronic gmbh",
    "000184": "SIEB & MEYER AG",
    "00018E": "Logitec Corporation",
    "000179": "WIRELESS TECHNOLOGY, INC.",
    "000160": "ELMEX Co., LTD.",
    "00014E": "WIN Enterprises, Inc.",
    "0001A4": "Microlink Corporation",
    "000195": "Sena Technologies, Inc.",
    "00017E": "ADTEK System Science Co., Ltd.",
    "000183": "ANITE TELECOMS",
    "0001DB": "Freecom Technologies GmbH",
    "0001DF": "ISDN Communications, Ltd.",
    "0001CA": "Geocast Network Systems, Inc.",
    "0001EF": "Camtel Technology Corp.",
    "0001B5": "Turin Networks, Inc.",
    "00B0F5": "NetWorth Technologies, Inc.",
    "00B0DB": "Nextcell, Inc.",
    "00B0AE": "Symmetricom",
    "00B0E7": "British Federal Ltd.",
    "00B08E": "Cisco Systems, Inc",
    "000118": "EZ Digital Co., Ltd.",
    "000128": "EnjoyWeb, Inc.",
    "00011C": "Universal Talkware Corporation",
    "00010C": "System Talks Inc.",
    "000111": "iDigm Inc.",
    "003073": "International Microsystems, In",
    "00303F": "TurboComm Tech Inc.",
    "00016D": "CarrierComm Inc.",
    "00016A": "ALITEC",
    "00016F": "Inkel Corp.",
    "000170": "ESE Embedded System Engineer'g",
    "000135": "KDC Corp.",
    "000141": "CABLE PRINT",
    "000146": "Tesco Controls, Inc.",
    "000149": "TDT AG",
    "000165": "AirSwitch Corporation",
    "000156": "FIREWIREDIRECT.COM, INC.",
    "000153": "ARCHTEK TELECOM CORPORATION",
    "000114": "KANDA TSUSHIN KOGYO CO., LTD.",
    "000107": "Leiser GmbH",
    "00010B": "Space CyberLink, Inc.",
    "00B0DF": "Starboard Storage Systems",
    "000131": "Bosch Security Systems, Inc.",
    "00304F": "PLANET Technology Corporation",
    "003022": "Fong Kai Industrial Co., Ltd.",
    "003070": "1Net Corporation",
    "0030F8": "Dynapro Systems, Inc.",
    "0030B3": "San Valley Systems, Inc.",
    "003009": "Tachion Networks, Inc.",
    "00307A": "Advanced Technology & Systems",
    "003061": "MobyTEL",
    "003056": "HMS Industrial Networks",
    "0030E7": "CNF MOBILE SOLUTIONS, INC.",
    "003043": "IDREAM TECHNOLOGIES, PTE. LTD.",
    "0030B4": "INTERSIL CORP.",
    "003000": "ALLWELL TECHNOLOGY CORP.",
    "003011": "HMS Industrial Networks   ",
    "00308F": "MICRILOR, Inc.",
    "00309C": "Timing Applications, Inc.",
    "00307E": "Redflex Communication Systems",
    "003004": "LEADTEK RESEARCH INC.",
    "0030F9": "Sollae Systems Co., Ltd.",
    "003002": "Expand Networks",
    "003078": "Cisco Systems, Inc",
    "0030BE": "City-Net Technology, Inc.",
    "003092": "Kontron Electronics AG",
    "00305C": "SMAR Laboratories Corp.",
    "0030B1": "TrunkNet",
    "0030E0": "OXFORD SEMICONDUCTOR LTD.",
    "003064": "ADLINK TECHNOLOGY, INC.",
    "003050": "Versa Technology",
    "0030C0": "Lara Technology, Inc.",
    "003005": "Fujitsu Siemens Computers",
    "0030DB": "Mindready Solutions, Inc.",
    "0030C7": "Macromate Corp.",
    "0030E4": "CHIYODA SYSTEM RIKEN",
    "003066": "RFM",
    "003031": "LIGHTWAVE COMMUNICATIONS, INC.",
    "003060": "Powerfile, Inc.",
    "0030A0": "TYCO SUBMARINE SYSTEMS, LTD.",
    "003015": "CP CLARE CORP.",
    "00304B": "ORBACOM SYSTEMS, INC.",
    "0030FA": "TELICA, INC.",
    "0030A5": "ACTIVE POWER",
    "003084": "ALLIED TELESYN INTERNAIONAL",
    "0030E9": "GMA COMMUNICATION MANUFACT'G",
    "00D03E": "ROCKETCHIPS, INC.",
    "00D093": "TQ - COMPONENTS GMBH",
    "00D03F": "AMERICAN COMMUNICATION",
    "00D0CE": "iSystem Labs ",
    "00D0F9": "ACUTE COMMUNICATIONS CORP.",
    "00D0F7": "NEXT NETS CORPORATION",
    "00D003": "COMDA ENTERPRISES CORP.",
    "00D0D2": "EPILOG CORPORATION",
    "003068": "CYBERNETICS TECH. CO., LTD.",
    "003091": "TAIWAN FIRST LINE ELEC. CORP.",
    "0030CD": "CONEXANT SYSTEMS, INC.",
    "00305B": "Toko Inc.",
    "003029": "OPICOM",
    "003083": "Ivron Systems",
    "0030B6": "Cisco Systems, Inc",
    "00D028": "Harmonic, Inc",
    "00D025": "XROSSTECH, INC.",
    "00D044": "ALIDIAN NETWORKS, INC.",
    "00D018": "QWES. COM, INC.",
    "00D007": "MIC ASSOCIATES, INC.",
    "00D0FF": "Cisco Systems, Inc",
    "00D077": "LUCENT TECHNOLOGIES",
    "00D08C": "GENOA TECHNOLOGY, INC.",
    "00D059": "AMBIT MICROSYSTEMS CORP.",
    "00D0FD": "OPTIMA TELE.COM, INC.",
    "00D0E6": "IBOND INC.",
    "00D0D1": "Sycamore Networks",
    "00D087": "MICROFIRST INC.",
    "00D091": "SMARTSAN SYSTEMS, INC.",
    "00D080": "EXABYTE CORPORATION",
    "00D04E": "LOGIBAG",
    "00D027": "APPLIED AUTOMATION, INC.",
    "00D0A9": "SHINANO KENSHI CO., LTD.",
    "00D0DD": "SUNRISE TELECOM, INC.",
    "00D031": "INDUSTRIAL LOGIC CORPORATION",
    "00D038": "FIVEMERE, LTD.",
    "00D0C6": "THOMAS & BETTS CORP.",
    "0001A7": "UNEX TECHNOLOGY CORPORATION",
    "00D089": "DYNACOLOR, INC.",
    "00D072": "BROADLOGIC",
    "00D0E2": "MRT MICRO, INC.",
    "00D0A6": "LANBIRD TECHNOLOGY CO., LTD.",
    "00D049": "IMPRESSTEK CO., LTD.",
    "00D05B": "ACROLOOP MOTION CONTROL",
    "00D042": "MAHLO GMBH & CO. UG",
    "00D02C": "CAMPBELL SCIENTIFIC, INC.",
    "00D0CD": "ATAN TECHNOLOGY INC.",
    "00D040": "SYSMATE CO., LTD.",
    "00D06F": "KMC CONTROLS",
    "00D01A": "URMET  TLC S.P.A.",
    "00D06A": "LINKUP SYSTEMS CORPORATION",
    "00504E": "SIERRA MONITOR CORP.",
    "0050D7": "TELSTRAT",
    "0050F6": "PAN-INTERNATIONAL INDUSTRIAL CORP.",
    "00506C": "Beijer Electronics Products AB",
    "005044": "ASACA CORPORATION",
    "00500E": "CHROMATIS NETWORKS, INC.",
    "005060": "TANDBERG TELECOM AS",
    "0050EE": "TEK DIGITEL CORPORATION",
    "0050FF": "HAKKO ELECTRONICS CO., LTD.",
    "0050D2": "CMC Electronics Inc",
    "0050F9": "Sensormatic Electronics LLC",
    "005048": "INFOLIBRIA",
    "00501C": "JATOM SYSTEMS, INC.",
    "00507A": "XPEED, INC.",
    "00504F": "OLENCOM ELECTRONICS",
    "00506E": "CORDER ENGINEERING CORPORATION",
    "00502C": "SOYO COMPUTER, INC.",
    "0050E6": "HAKUSAN CORPORATION",
    "00503C": "TSINGHUA NOVEL ELECTRONICS",
    "005033": "MAYAN NETWORKS",
    "005045": "RIOWORKS SOLUTIONS, INC.",
    "00502B": "GENRAD LTD.",
    "00502E": "CAMBEX CORPORATION",
    "005007": "SIEMENS TELECOMMUNICATION SYSTEMS LIMITED",
    "005022": "ZONET TECHNOLOGY, INC.",
    "005040": "Panasonic Electric Works Co., Ltd.",
    "0050C9": "MASPRO DENKOH CORP.",
    "005069": "PixStream Incorporated",
    "005068": "ELECTRONIC INDUSTRIES ASSOCIATION",
    "00D0CC": "TECHNOLOGIES LYRE INC.",
    "005077": "PROLIFIC TECHNOLOGY, INC.",
    "0050EC": "OLICOM A/S",
    "0050F8": "ENTREGA TECHNOLOGIES, INC.",
    "005042": "SCI MANUFACTURING SINGAPORE PTE, LTD.",
    "0050C0": "GATAN, INC.",
    "005051": "IWATSU ELECTRIC CO., LTD.",
    "0050BB": "CMS TECHNOLOGIES",
    "005062": "KOUWELL ELECTRONICS CORP.  **",
    "005097": "MMC-EMBEDDED COMPUTERTECHNIK GmbH",
    "005010": "NovaNET Learning, Inc.",
    "00509A": "TAG ELECTRONIC SYSTEMS",
    "0050EB": "ALPHA-TOP CORPORATION",
    "0050EF": "SPE Systemhaus GmbH",
    "005098": "GLOBALOOP, LTD.",
    "0050BC": "HAMMER STORAGE SOLUTIONS",
    "009022": "IVEX",
    "009016": "ZAC",
    "0090FF": "TELLUS TECHNOLOGY INC.",
    "00903E": "N.V. PHILIPS INDUSTRIAL ACTIVITIES",
    "0090BA": "VALID NETWORKS, INC.",
    "009018": "ITO ELECTRIC INDUSTRY CO, LTD.",
    "0050D5": "AD SYSTEMS CORP.",
    "0050F3": "GLOBAL NET INFORMATION CO., Ltd.",
    "0050BE": "FAST MULTIMEDIA AG",
    "00506F": "G-CONNECT",
    "00507D": "IFP",
    "0090B3": "AGRANAT SYSTEMS",
    "00905D": "NETCOM SICHERHEITSTECHNIK GMBH",
    "0090D1": "LEICHU ENTERPRISE CO., LTD.",
    "009046": "DEXDYNE, LTD.",
    "0090CD": "ENT-EMPRESA NACIONAL DE TELECOMMUNICACOES, S.A.",
    "0090D0": "Thomson Telecom Belgium",
    "00909B": "MARKEM-IMAJE",
    "0090FB": "PORTWELL, INC.",
    "009094": "OSPREY TECHNOLOGIES, INC.",
    "0090DA": "DYNARC, INC.",
    "0090E0": "SYSTRAN CORP.",
    "009071": "Applied Innovation Inc.",
    "0090DC": "TECO INFORMATION SYSTEMS",
    "0090E2": "DISTRIBUTED PROCESSING TECHNOLOGY",
    "0090C7": "ICOM INC.",
    "009035": "ALPHA TELECOM, INC.",
    "0090F0": "Harmonic Video Systems Ltd.",
    "00907A": "Spectralink, Inc",
    "009020": "PHILIPS ANALYTICAL X-RAY B.V.",
    "0010A3": "OMNITRONIX, INC.",
    "0010AD": "SOFTRONICS USB, INC.",
    "0010A7": "UNEX TECHNOLOGY CORPORATION",
    "0010D5": "IMASDE CANARIAS, S.A.",
    "001055": "FUJITSU MICROELECTRONICS, INC.",
    "00904F": "ABB POWER T&D COMPANY, INC.",
    "009060": "SYSTEM CREATE CORP.",
    "009013": "SAMSAN CORP.",
    "009085": "GOLDEN ENTERPRISES, INC.",
    "009053": "DAEWOO ELECTRONICS CO., LTD.",
    "00903C": "ATLANTIC NETWORK SYSTEMS",
    "009077": "ADVANCED FIBRE COMMUNICATIONS",
    "009099": "ALLIED TELESIS, K.K.",
    "009055": "PARKER HANNIFIN CORPORATION COMPUMOTOR DIVISION",
    "00905C": "EDMI",
    "0090E3": "AVEX ELECTRONICS INC.",
    "0090A9": "WESTERN DIGITAL",
    "0090F3": "ASPECT COMMUNICATIONS",
    "001052": "METTLER-TOLEDO (ALBSTADT) GMBH",
    "00106B": "SONUS NETWORKS, INC.",
    "0010C3": "CSI-CONTROL SYSTEMS",
    "00900F": "KAWASAKI HEAVY INDUSTRIES, LTD",
    "0090EA": "ALPHA TECHNOLOGIES, INC.",
    "0090A7": "CLIENTEC CORPORATION",
    "001051": "CMICRO CORPORATION",
    "001037": "CYQ've Technology Co., Ltd.",
    "00101B": "CORNET TECHNOLOGY, INC.",
    "0010DC": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "00100A": "WILLIAMS COMMUNICATIONS GROUP",
    "001032": "ALTA TECHNOLOGY",
    "0010F4": "Vertical Communications",
    "001077": "SAF DRIVE SYSTEMS, LTD.",
    "0010B3": "NOKIA MULTIMEDIA TERMINALS",
    "00107A": "AmbiCom, Inc.",
    "00102D": "HITACHI SOFTWARE ENGINEERING",
    "001033": "ACCESSLAN COMMUNICATIONS, INC.",
    "0010DD": "ENABLE SEMICONDUCTOR, INC.",
    "001078": "NUERA COMMUNICATIONS, INC.",
    "001015": "OOmon Inc.",
    "0010B9": "MAXTOR CORP.",
    "00105D": "Draeger Medical",
    "001012": "PROCESSOR SYSTEMS (I) PVT LTD",
    "001091": "NO WIRES NEEDED BV",
    "001080": "METAWAVE COMMUNICATIONS",
    "00101E": "MATSUSHITA ELECTRONIC INSTRUMENTS CORP.",
    "00104D": "SURTEC INDUSTRIES, INC.",
    "00E0BB": "NBX CORPORATION",
    "00E08A": "GEC AVERY, LTD.",
    "00E086": "Emerson Network Power, Avocent Division",
    "00E01B": "SPHERE COMMUNICATIONS, INC.",
    "00E07F": "LOGISTISTEM s.r.l.",
    "00E013": "EASTERN ELECTRONIC CO., LTD.",
    "00E0FD": "A-TREND TECHNOLOGY CO., LTD.",
    "00E0BD": "INTERFACE SYSTEMS, INC.",
    "00E0C5": "BCOM ELECTRONICS INC.",
    "00E0EE": "MAREL HF",
    "00E08E": "UTSTARCOM",
    "00E03F": "JATON CORPORATION",
    "00E0D4": "EXCELLENT COMPUTER",
    "00E005": "TECHNICAL CORP.",
    "00E0C1": "MEMOREX TELEX JAPAN, LTD.",
    "00E084": "COMPULITE R&D",
    "00E06E": "FAR SYSTEMS S.p.A.",
    "00E06D": "COMPUWARE CORPORATION",
    "00607F": "AURORA TECHNOLOGIES, INC.",
    "00E029": "STANDARD MICROSYSTEMS CORP.",
    "0060A5": "PERFORMANCE TELECOM CORP.",
    "00E038": "PROXIMA CORPORATION",
    "00E09C": "MII",
    "00E0E9": "DATA LABS, INC.",
    "00E00C": "MOTOROLA",
    "00E00A": "DIBA, INC.",
    "00E0C4": "HORNER ELECTRIC, INC.",
    "00E096": "SHIMADZU CORPORATION",
    "00E017": "EXXACT GmbH",
    "0060A1": "VPNet, Inc.",
    "006027": "Superior Modular Products",
    "0060BC": "KeunYoung Electronics & Communication Co., Ltd.",
    "00E015": "HEIWA CORPORATION",
    "00E069": "JAYCOR",
    "00E0A4": "ESAOTE S.p.A.",
    "00E0DE": "DATAX NV",
    "00E0EA": "INNOVAT COMMUNICATIONS, INC.",
    "00E064": "SAMSUNG ELECTRONICS",
    "00E0C9": "AutomatedLogic Corporation",
    "00E0E8": "GRETACODER Data Systems AG",
    "00E016": "RAPID CITY COMMUNICATIONS",
    "00E0A5": "ComCore Semiconductor, Inc.",
    "00E0A9": "FUNAI ELECTRIC CO., LTD.",
    "006029": "CARY PERIPHERALS INC.",
    "0060A8": "TIDOMAT AB",
    "0060D4": "ELDAT COMMUNICATION LTD.",
    "0060FC": "CONSERVATION THROUGH INNOVATION LTD.",
    "006018": "STELLAR ONE CORPORATION",
    "0060B6": "LAND COMPUTER CO., LTD.",
    "00606C": "ARESCOM",
    "00602E": "CYCLADES CORPORATION",
    "006074": "QSC LLC",
    "006076": "SCHLUMBERGER TECHNOLOGIES RETAIL PETROLEUM SYSTEMS",
    "006061": "WHISTLE COMMUNICATIONS CORP.",
    "0060F9": "DIAMOND LANE COMMUNICATIONS",
    "0060EA": "StreamLogic",
    "0060E3": "ARBIN INSTRUMENTS",
    "006071": "MIDAS LAB, INC.",
    "0060D9": "TRANSYS NETWORKS INC.",
    "00601F": "STALLION TECHNOLOGIES",
    "00601B": "MESA ELECTRONICS",
    "00600A": "SORD COMPUTER CORPORATION",
    "0060A4": "GEW Technologies (PTY)Ltd",
    "006064": "NETCOMM LIMITED",
    "0060C5": "ANCOT CORP.",
    "0060A9": "GESYTEC MBH",
    "0060F2": "LASERGRAPHICS, INC.",
    "00A07A": "ADVANCED PERIPHERALS TECHNOLOGIES, INC.",
    "00A04E": "VOELKER TECHNOLOGIES, INC.",
    "00A05A": "KOFAX IMAGE PRODUCTS",
    "00A052": "STANILITE ELECTRONICS PTY. LTD",
    "00A05E": "MYRIAD LOGIC INC.",
    "00A095": "ACACIA NETWORKS, INC.",
    "00A0F2": "INFOTEK COMMUNICATIONS, INC.",
    "00A0DF": "STS TECHNOLOGIES, INC.",
    "00A094": "COMSAT CORPORATION",
    "006054": "CONTROLWARE GMBH",
    "0060C2": "MPL AG",
    "00609B": "AstroNova, Inc",
    "0060DB": "NTP ELEKTRONIK A/S",
    "006052": "PERIPHERALS ENTERPRISE CO., Ltd.",
    "0060B2": "PROCESS CONTROL CORP.",
    "006081": "TV/COM INTERNATIONAL",
    "00A005": "DANIEL INSTRUMENTS, LTD.",
    "00A053": "COMPACT DEVICES, INC.",
    "00A069": "Symmetricom, Inc.",
    "0060C3": "NETVISION CORPORATION",
    "006082": "NOVALINK TECHNOLOGIES, INC.",
    "0060E7": "RANDATA",
    "00A0EF": "LUCIDATA LTD.",
    "00A0CE": "Ecessa",
    "00A020": "CITICORP/TTI",
    "00A067": "NETWORK SERVICES GROUP",
    "00A0E0": "TENNYSON TECHNOLOGIES PTY LTD",
    "00A099": "K-NET LTD.",
    "00A0EC": "TRANSMITTON LTD.",
    "00A0AB": "NETCS INFORMATIONSTECHNIK GMBH",
    "00A0D8": "SPECTRA - TEK",
    "00A080": "Tattile SRL ",
    "00A02B": "TRANSITIONS RESEARCH CORP.",
    "00A0E8": "REUTERS HOLDINGS PLC",
    "00A008": "NETCORP",
    "00A0C3": "UNICOMPUTER GMBH",
    "00A00A": "Airspan",
    "00A0E7": "CENTRAL DATA CORPORATION",
    "00A0FA": "Marconi Communication GmbH",
    "00A0CB": "ARK TELECOMMUNICATIONS, INC.",
    "00A050": "CYPRESS SEMICONDUCTOR",
    "00A0DD": "AZONIX CORPORATION",
    "00A028": "CONNER PERIPHERALS",
    "00A09E": "ICTV",
    "00A0FD": "SCITEX DIGITAL PRINTING, INC.",
    "00A00F": "Broadband Technologies",
    "00A002": "LEEDS & NORTHRUP AUSTRALIA PTY LTD",
    "00A0E4": "OPTIQUEST",
    "00A0EE": "NASHOBA NETWORKS",
    "00A066": "ISA CO., LTD.",
    "00A034": "AXEL",
    "00A001": "DRS Signal Solutions",
    "00A075": "MICRON TECHNOLOGY, INC.",
    "00A009": "WHITETREE NETWORK",
    "002023": "T.C. TECHNOLOGIES PTY. LTD",
    "0020B2": "GKD Gesellschaft Fur Kommunikation Und Datentechnik",
    "002052": "RAGULA SYSTEMS",
    "0020FE": "TOPWARE INC. / GRAND COMPUTER",
    "002073": "FUSION SYSTEMS CORPORATION",
    "002035": "IBM Corp",
    "00207A": "WiSE Communications, Inc.",
    "00203E": "LogiCan Technologies, Inc.",
    "002058": "ALLIED SIGNAL INC.",
    "00205A": "COMPUTER IDENTICS",
    "002000": "LEXMARK INTERNATIONAL, INC.",
    "00201D": "KATANA PRODUCTS",
    "00A00C": "KINGMAX TECHNOLOGY, INC.",
    "00A0BB": "HILAN GMBH",
    "00A091": "APPLICOM INTERNATIONAL",
    "00A0A5": "TEKNOR MICROSYSTEME, INC.",
    "00A017": "J B M CORPORATION",
    "00A025": "REDCOM LABS INC.",
    "00A0A2": "B810 S.R.L.",
    "0020B7": "NAMAQUA COMPUTERWARE",
    "0020E3": "MCD KENCOM CORPORATION",
    "002013": "DIVERSIFIED TECHNOLOGY, INC.",
    "0020AB": "MICRO INDUSTRIES CORP.",
    "00208D": "CMD TECHNOLOGY",
    "0020DD": "Cybertec Pty Ltd",
    "0020E6": "LIDKOPING MACHINE TOOLS AB",
    "002086": "MICROTECH ELECTRONICS LIMITED",
    "002046": "CIPRICO, INC.",
    "002026": "AMKLY SYSTEMS, INC.",
    "00A054": "Private",
    "00208E": "CHEVIN SOFTWARE ENG. LTD.",
    "00209B": "ERSAT ELECTRONIC GMBH",
    "00201C": "EXCEL, INC.",
    "00207F": "KYOEI SANGYO CO., LTD.",
    "002008": "CABLE & COMPUTER TECHNOLOGY",
    "00C00B": "NORCONTROL A.S.",
    "0020B0": "GATEWAY DEVICES, INC.",
    "00205B": "Kentrox, LLC",
    "002065": "SUPERNET NETWORKING INC.",
    "002019": "OHLER GMBH",
    "00209E": "BROWN'S OPERATING SYSTEM SERVICES, LTD.",
    "00202A": "N.V. DZINE",
    "002083": "PRESTICOM INCORPORATED",
    "0020D3": "OST (OUEST STANDARD TELEMATIQU",
    "0020F6": "NET TEK  AND KARLNET, INC.",
    "0020C9": "VICTRON BV",
    "002077": "KARDIOS SYSTEMS CORP.",
    "002097": "APPLIED SIGNAL TECHNOLOGY",
    "0020C6": "NECTEC",
    "00C048": "BAY TECHNICAL ASSOCIATES",
    "00C00E": "PSITECH, INC.",
    "00C031": "DESIGN RESEARCH SYSTEMS, INC.",
    "000701": "RACAL-DATACOM",
    "00C09C": "HIOKI E.E. CORPORATION",
    "00C097": "ARCHIPEL SA",
    "00C004": "JAPAN BUSINESS COMPUTER CO.LTD",
    "00C02D": "FUJI PHOTO FILM CO., LTD.",
    "00C0BD": "INEX TECHNOLOGIES, INC.",
    "00C088": "EKF ELEKTRONIK GMBH",
    "00C0FD": "PROSUM",
    "00C076": "I-DATA INTERNATIONAL A-S",
    "00C046": "Blue Chip Technology Ltd",
    "00C014": "TELEMATICS CALABASAS INT'L,INC",
    "00AA3C": "OLIVETTI TELECOM SPA (OLTECO)",
    "00C011": "INTERACTIVE COMPUTING DEVICES",
    "00C03E": "FA. GEBR. HELLER GMBH",
    "00C0AA": "SILICON VALLEY COMPUTER",
    "00C066": "DOCUPOINT, INC.",
    "00C060": "ID SCANDINAVIA AS",
    "00C0C9": "ELSAG BAILEY PROCESS",
    "00C054": "NETWORK PERIPHERALS, LTD.",
    "00C022": "LASERMASTER TECHNOLOGIES, INC.",
    "00C025": "DATAPRODUCTS CORPORATION",
    "00C0DF": "KYE Systems Corp.",
    "00C0F5": "METACOMP, INC.",
    "00C091": "JABIL CIRCUIT, INC.",
    "00C049": "U.S. ROBOTICS, INC.",
    "00C09D": "DISTRIBUTED SYSTEMS INT'L, INC",
    "00407B": "SCIENTIFIC ATLANTA",
    "00402C": "ISIS DISTRIBUTED SYSTEMS, INC.",
    "0040CC": "SILCOM MANUF'G TECHNOLOGY INC.",
    "00C09E": "CACHE COMPUTERS, INC.",
    "00C0AC": "GAMBIT COMPUTER COMMUNICATIONS",
    "00C034": "TRANSACTION NETWORK",
    "00C093": "ALTA RESEARCH CORP.",
    "0040CF": "STRAWBERRY TREE, INC.",
    "004077": "MAXTON TECHNOLOGY CORPORATION",
    "0040E7": "ARNOS INSTRUMENTS & COMPUTER",
    "004087": "UBITREX CORPORATION",
    "004007": "TELMAT INFORMATIQUE",
    "00C0E9": "OAK SOLUTIONS, LTD.",
    "00C0C5": "SID INFORMATICA",
    "00C051": "ADVANCED INTEGRATION RESEARCH",
    "00C085": "ELECTRONICS FOR IMAGING, INC.",
    "00C0B2": "NORAND CORPORATION",
    "00C02C": "CENTRUM COMMUNICATIONS, INC.",
    "00C02B": "GERLOFF GESELLSCHAFT FUR",
    "004073": "BASS ASSOCIATES",
    "00407D": "EXTENSION TECHNOLOGY CORP.",
    "00404D": "TELECOMMUNICATIONS TECHNIQUES",
    "00400D": "LANNET DATA COMMUNICATIONS,LTD",
    "004019": "AEON SYSTEMS, INC.",
    "0040BE": "BOEING DEFENSE & SPACE",
    "00406E": "COROLLARY, INC.",
    "00C0FB": "ADVANCED TECHNOLOGY LABS",
    "00405B": "FUNASSET LIMITED",
    "00408B": "RAYLAN CORPORATION",
    "0040EF": "HYPERCOM, INC.",
    "004093": "PAXDATA NETWORKS LTD.",
    "004085": "SAAB INSTRUMENTS AB",
    "004023": "LOGIC CORPORATION",
    "0040A4": "ROSE ELECTRONICS",
    "004022": "KLEVER COMPUTERS, INC.",
    "004074": "CABLE AND WIRELESS",
    "0040B8": "IDEA ASSOCIATES",
    "0040E8": "CHARLES RIVER DATA SYSTEMS,INC",
    "0040C0": "VISTA CONTROLS CORPORATION",
    "00405D": "STAR-TEK, INC.",
    "004008": "A PLUS INFO CORPORATION",
    "0040B5": "VIDEO TECHNOLOGY COMPUTERS LTD",
    "004012": "WINDATA, INC.",
    "0040D5": "Sartorius Mechatronics T&H GmbH ",
    "0080D7": "Fantum Engineering",
    "00807A": "AITECH SYSTEMS LTD.",
    "0080DC": "PICKER INTERNATIONAL",
    "00C0A0": "ADVANCE MICRO RESEARCH, INC.",
    "00C010": "HIRAKAWA HEWTECH CORP.",
    "0040BF": "CHANNEL SYSTEMS INTERN'L INC.",
    "00401E": "ICC",
    "00409A": "NETWORK EXPRESS, INC.",
    "004094": "SHOGRAPHICS, INC.",
    "004055": "METRONIX GMBH",
    "0040E2": "MESA RIDGE TECHNOLOGIES, INC.",
    "0040C4": "KINKEI SYSTEM CORPORATION",
    "00C037": "DYNATEM",
    "004083": "TDA INDUSTRIA DE PRODUTOS",
    "004027": "SMC MASSACHUSETTS, INC.",
    "004045": "TWINHEAD CORPORATION",
    "004076": "Sun Conversion Technologies",
    "0040F0": "MicroBrain,Inc.",
    "004089": "MEIDENSHA CORPORATION",
    "004028": "NETCOMM LIMITED",
    "0040CB": "LANWAN TECHNOLOGIES",
    "0040B2": "SYSTEMFORSCHUNG",
    "0040E6": "C.A.E.N.",
    "00801C": "NEWPORT SYSTEMS SOLUTIONS",
    "008095": "BASIC MERTON HANDELSGES.M.B.H.",
    "0080AE": "HUGHES NETWORK SYSTEMS",
    "004010": "SONIC SYSTEMS, INC.",
    "0040CA": "FIRST INTERNAT'L COMPUTER, INC",
    "008031": "BASYS, CORP.",
    "00803A": "VARITYPER, INC.",
    "004015": "ASCOM INFRASYS AG",
    "008056": "SPHINX Electronics GmbH & Co KG",
    "008013": "THOMAS-CONRAD CORPORATION",
    "00806E": "NIPPON STEEL CORPORATION",
    "008010": "COMMODORE INTERNATIONAL",
    "0080D5": "CADRE TECHNOLOGIES",
    "00801B": "KODIAK TECHNOLOGY",
    "0080D3": "SHIVA CORP.",
    "0080B3": "AVAL DATA CORPORATION",
    "0080A1": "MICROTEST, INC.",
    "008082": "PEP MODULAR COMPUTERS GMBH",
    "008039": "ALCATEL STC AUSTRALIA",
    "008023": "INTEGRATED BUSINESS NETWORKS",
    "00806B": "SCHMID TELECOMMUNICATION",
    "008059": "STANLEY ELECTRIC CO., LTD",
    "00802B": "INTEGRATED MARKETING CO",
    "008001": "PERIPHONICS CORPORATION",
    "008097": "CENTRALP AUTOMATISMES",
    "008088": "VICTOR COMPANY OF JAPAN, LTD.",
    "0080D8": "NETWORK PERIPHERALS INC.",
    "00809E": "DATUS GMBH",
    "008041": "VEB KOMBINAT ROBOTRON",
    "008080": "DATAMEDIA CORPORATION",
    "008098": "TDK CORPORATION",
    "00803F": "TATUNG COMPANY",
    "0080E6": "PEER NETWORKS, INC.",
    "0080E0": "XTP SYSTEMS, INC.",
    "0080DB": "GRAPHON CORPORATION",
    "0080CA": "NETCOM RESEARCH INCORPORATED",
    "008071": "SAI TECHNOLOGY",
    "008020": "NETWORK PRODUCTS",
    "008070": "COMPUTADORAS MICRON",
    "008008": "DYNATECH COMPUTER SYSTEMS",
    "0000E4": "IN2 GROUPE INTERTECHNIQUE",
    "000094": "ASANTE TECHNOLOGIES",
    "000090": "MICROCOM",
    "0000B6": "MICRO-MATIC RESEARCH",
    "000082": "LECTRA SYSTEMES SA",
    "00002B": "CRISP AUTOMATION, INC",
    "000014": "NETRONIX",
    "000072": "MINIWARE TECHNOLOGY",
    "0000A1": "MARQUETTE ELECTRIC CO.",
    "0000F5": "DIAMOND SALES LIMITED",
    "008047": "IN-NET CORP.",
    "008067": "SQUARE D COMPANY",
    "008045": "MATSUSHITA ELECTRIC IND. CO",
    "0080BF": "TAKAOKA ELECTRIC MFG. CO. LTD.",
    "008017": "PFU LIMITED",
    "0080F8": "MIZAR, INC.",
    "000051": "HOB ELECTRONIC GMBH & CO. KG",
    "0000A7": "NETWORK COMPUTING DEVICES INC.",
    "000026": "SHA-KEN CO., LTD.",
    "0080A9": "CLEARPOINT RESEARCH",
    "0080F9": "HEURIKON CORPORATION",
    "000047": "NICOLET INSTRUMENTS CORP.",
    "000070": "HCL LIMITED",
    "00008F": "Raytheon",
    "000045": "FORD AEROSPACE & COMM. CORP.",
    "00009C": "ROLM MIL-SPEC COMPUTERS",
    "00007C": "AMPERE INCORPORATED",
    "000068": "ROSEMOUNT CONTROLS",
    "000046": "OLIVETTI NORTH AMERICA",
    "00008D": "Cryptek Inc.",
    "00003B": "i Controls, Inc.",
    "000021": "SUREMAN COMP. & COMMUN. CORP.",
    "000074": "RICOH COMPANY LTD.",
    "00005C": "TELEMATICS INTERNATIONAL INC.",
    "0000AC": "CONWARE COMPUTER CONSULTING",
    "0000F1": "MAGNA COMPUTER CORPORATION",
    "000054": "Schneider Electric",
    "000020": "DATAINDUSTRIER DIAB AB",
    "00007A": "DANA COMPUTER INC.",
    "000098": "CROSSCOMM CORPORATION",
    "0000C6": "EON SYSTEMS",
    "00008B": "INFOTRON",
    "000030": "VG LABORATORY SYSTEMS LTD",
    "000035": "SPECTRAGRAPHICS CORPORATION",
    "0000E9": "ISICAD, INC.",
    "00009F": "AMERISTAR TECHNOLOGIES INC.",
    "0000E3": "INTEGRATED MICRO PRODUCTS LTD",
    "0000AD": "BRUKER INSTRUMENTS INC.",
    "0000D3": "WANG LABORATORIES INC.",
    "0000D0": "DEVELCON ELECTRONICS LTD.",
    "000093": "PROTEON INC.",
    "080067": "ComDesign",
    "08005D": "GOULD INC.",
    "08005B": "VTA TECHNOLOGIES INC.",
    "080057": "Evans & Sutherland",
    "080071": "MATRA (DSIE)",
    "08006C": "SUNTEK TECHNOLOGY INT'L",
    "0000B3": "CIMLINC INCORPORATED",
    "08002D": "LAN-TEC INC.",
    "08008C": "NETWORK RESEARCH CORPORATION",
    "080081": "ASTECH INC.",
    "00DD00": "UNGERMANN-BASS INC.",
    "0000AA": "XEROX CORPORATION",
    "100000": "Private",
    "080011": "TEKTRONIX INC.",
    "08001D": "ABLE COMMUNICATIONS INC.",
    "00DD0B": "UNGERMANN-BASS INC.",
    "AA0004": "DIGITAL EQUIPMENT CORPORATION",
    "08000C": "MIKLYN DEVELOPMENT CO.",
    "00DD05": "UNGERMANN-BASS INC.",
    "080003": "ADVANCED COMPUTER COMM.",
    "00DD0F": "UNGERMANN-BASS INC.",
    "000001": "XEROX CORPORATION",
    "00DD03": "UNGERMANN-BASS INC.",
    "0000D7": "DARTMOUTH COLLEGE",
    "080017": "NATIONAL SEMICONDUCTOR",
    "040AE0": "XMIT AG COMPUTER NETWORKS",
    "080026": "NORSK DATA A.S.",
    "080025": "CONTROL DATA",
    "48D017": "Telecom Infra Project",
    "F809A4": "Henan Thinker Rail Transportation Research Inc.",
    "A842A1": "TP-Link Corporation Limited",
    "9CF55F": "Harman/Becker Automotive Systems GmbH",
    "902C09": "Apple, Inc.",
    "A0B40F": "Apple, Inc.",
    "807C0A": "zte corporation",
    "EC0D51": "Apple, Inc.",
    "ACDFA1": "Apple, Inc.",
    "C05F87": "Legrand INTELLIGENT ELECTRICAL(HUIZHOU)CO.,LTD.",
    "685210": "MCS Logic",
    "C8CCB5": "Hunter Douglas",
    "14DAB9": "Huawei Device Co., Ltd.",
    "B0CAE7": "Huawei Device Co., Ltd.",
    "A87C45": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2CEDB0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C693E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC621F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48836F": "EM Microelectronic",
    "F4832C": "OHSUNG",
    "48EDE6": "Zyxel Communications Corporation",
    "A410B6": "Cisco Systems, Inc",
    "C84709": "Cisco Systems, Inc",
    "3C25F8": "Dell Inc.",
    "000324": "LIMNO Co., Ltd.",
    "70BDD2": "Adva Network Security GmbH",
    "D00B27": "Murata Manufacturing Co., Ltd.",
    "4C5739": "Samsung Electronics Co.,Ltd",
    "48E533": "Sunnovo International Limited",
    "F0FDDD": "Foxtron Vehicle Technologies Co., Ltd.",
    "00D598": "BOPEL MOBILE TECHNOLOGY CO.,LIMITED",
    "904528": "Nintendo Co.,Ltd",
    "F45246": "ASKEY COMPUTER CORP",
    "6C1629": "Edifier International",
    "F037CF": "Huawei Device Co., Ltd.",
    "2CC8F5": "Huawei Device Co., Ltd.",
    "AC361B": "Hon Hai Precision Industry Co.,LTD",
    "D8132A": "Espressif Inc.",
    "9C1ECE": "ALT Co., Ltd.",
    "B8060D": "Tuya Smart Inc.",
    "001F9C": "Havis Inc.",
    "C8E193": "ITEL MOBILE LIMITED",
    "D4A23D": "New H3C Technologies Co., Ltd",
    "28CB5C": "Shenzhen CPETEK Technology Co.,Ltd.",
    "D05BCB": "Tianyi Telecom Terminals Company Limited",
    "BC9A8E": "HUMAX NETWORKS",
    "502065": "Quectel Wireless Solutions Co.,Ltd.",
    "102CB1": "Smart Innovation LLC",
    "FC478D": "SHENZHEN TOPWELL TECHNOLOGY CO., LTD.",
    "806599": "Espressif Inc.",
    "04AEC7": "Marquardt",
    "E4F8BE": "TECNO MOBILE LIMITED",
    "00C0F2": "Lantronix",
    "E4AAE4": "Xiaomi Communications Co Ltd",
    "68C44C": "Xiaomi Communications Co Ltd",
    "A0004C": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "D8E593": "Apple, Inc.",
    "CC68E0": "Apple, Inc.",
    "70DA48": "Cisco Systems, Inc",
    "F4B62D": "Dongguan Huayin Electronic Technology Co., Ltd.",
    "20108A": "zte corporation",
    "58FFA1": "zte corporation",
    "B08B9E": "Telechips, Inc.",
    "BC1FE1": "Ascendent Technology Group",
    "ACDB22": "Marquardt Schaltsysteme SCS",
    "B8B1EA": "Honor Device Co., Ltd.",
    "2C58B9": "HP Inc.",
    "3050F1": "Ennoconn Corporation.",
    "B0FA91": "EM Microelectronic",
    "D4F921": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "E47A2C": "SZ DJI TECHNOLOGY CO.,LTD",
    "60601F": "SZ DJI TECHNOLOGY CO.,LTD",
    "488002": "Cisco Systems, Inc",
    "B46AD4": "Edgecore Americas Networking Corporation",
    "AC712E": "Fortinet, Inc.",
    "CCD843": "Beijing Xiaomi Mobile Software Co., Ltd",
    "C03532": "Liteon Technology Corporation",
    "740635": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "F801B4": "LG Electronics",
    "5421A9": "ITEL MOBILE LIMITED",
    "7CC0AA": "Microsoft Corporation",
    "94AB18": "cellXica ltd",
    "1073C6": "August Internet Limited",
    "F03F03": "Private",
    "C421B9": "zte corporation",
    "BCB2CC": "Samsung Electronics Co.,Ltd",
    "08A5DF": "Samsung Electronics Co.,Ltd",
    "6CD63F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C184E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4F7D5": "Sony Interactive Entertainment Inc.",
    "98288B": "zhejiang Dusun Electron Co.,Ltd",
    "E0E8E6": "Shenzhen C-Data Technology Co., Ltd.",
    "6C1FF7": "Ugreen Group Limited",
    "045EA4": "Netis Technology Co., Ltd.",
    "64EEB7": "Netis Technology Co., Ltd.",
    "048D38": "Netis Technology Co., Ltd.",
    "E82587": "Shenzhen Chilink IoT Technology Co., Ltd.",
    "507B91": "Sigmastar Technology Ltd.",
    "8C44A5": "Cisco Systems, Inc",
    "08E342": "Cear, Inc.",
    "A0E025": "Provision-ISR",
    "F4FE3E": "Apple, Inc.",
    "1455B9": "Nokia Solutions and Networks GmbH & Co. KG",
    "9C2472": "Sagemcom Broadband SAS",
    "F439A6": "Apple, Inc.",
    "FC5557": "Apple, Inc.",
    "342B6E": "Apple, Inc.",
    "606525": "Apple, Inc.",
    "8C5570": "IEEE Registration Authority",
    "205A1D": "zte corporation",
    "8C0879": "Texas Instruments",
    "6823B0": "Texas Instruments",
    "78CD55": "Texas Instruments",
    "F8F519": "Rulogic Inc.",
    "808544": "Intelbras",
    "24698E": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "640DCE": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "6C5CB1": "Silicon Laboratories",
    "5CC7C1": "Silicon Laboratories",
    "60B647": "Silicon Laboratories",
    "A46DD4": "Silicon Laboratories",
    "0CAE5F": "Silicon Laboratories",
    "28DBA7": "Silicon Laboratories",
    "583BC2": "Silicon Laboratories",
    "3425B4": "Silicon Laboratories",
    "006005": "Touchstar ATC Limited",
    "4CB7E0": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "BC0FB7": "sywinkey HongKong Co,. Limited?",
    "601D56": "Extreme Networks Headquarters",
    "9CF1D4": "Roku, Inc",
    "00408C": "Axis Communications AB",
    "00CAE0": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "44D465": "NXP Semiconductors Taiwan Ltd.",
    "54606D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A486DB": "Guangdong Juan Intelligent Technology Joint Stock Co., Ltd.",
    "9C63C0": "Mellanox Technologies, Inc.",
    "5C2573": "Mellanox Technologies, Inc.",
    "D4F04A": "SKY UK LIMITED",
    "ACBAC0": "Intertech Services AG",
    "1C7ACF": "vivo Mobile Communication Co., Ltd.",
    "A86A86": "Xiaomi Communications Co Ltd",
    "3C450B": "Sentry Equipment Corp.",
    "70D51E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "B87E39": "Motorola Mobility LLC, a Lenovo Company",
    "646772": "SERCOMM PHILIPPINES INC",
    "54ECB0": "Sercomm France Sarl",
    "DCDA4D": "CELESTICA INC.",
    "505E24": "zte corporation",
    "84BA59": "Wistron InfoComm(Chongqing)Co.,Ltd.",
    "1090FA": "New H3C Technologies Co., Ltd",
    "7473E2": "Hillstone Networks Corp.",
    "D401C3": "Routerboard.com",
    "74077E": "Quectel Wireless Solutions Co.,Ltd.",
    "30EC7C": "Shenzhen Along Electronics Co., Ltd",
    "382656": "TCL King Electrical Appliances(Huizhou)Co.,Ltd",
    "54C6A6": "Hubei Yangtze Mason Semiconductor Technology  Co., Ltd.",
    "B09E1B": "Butlr Technologies, Inc.",
    "14A1DF": "China Mobile Group Device Co.,Ltd.",
    "D0A011": "IEEE Registration Authority",
    "B0B369": "Shenzhen SDMC Technology CO.,Ltd.",
    "A8CFE0": "GDN Enterprises Private Limited",
    "D0B3C2": "Meta Platforms Technologies, LLC",
    "D8F760": "EM Microelectronic",
    "40B82D": "Ruckus Wireless",
    "005092": "Rigaku Corporation Osaka Plant",
    "B47748": "Shenzhen Neoway Technology Co.,Ltd.",
    "C43EAB": "Huawei Device Co., Ltd.",
    "CC4460": "Huawei Device Co., Ltd.",
    "24506F": "THINKCAR TECH CO.,LTD.",
    "34EA10": "Orbic North America",
    "6C92CF": "Broadcom Limited",
    "F43149": "Pixel FX",
    "78E996": "IEEE Registration Authority",
    "3C3576": "ITEL MOBILE LIMITED",
    "A021AA": "China Mobile Group Device Co.,Ltd.",
    "F42015": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "90212E": "Apption Labs Ltd",
    "50D448": "Private",
    "1C7A43": "vivo Mobile Communication Co., Ltd.",
    "084473": "zte corporation",
    "943589": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94261D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A45C25": "Texas Instruments",
    "98F487": "Texas Instruments",
    "FC599F": "Ruijie Networks Co.,LTD",
    "84A329": "Arcadyan Corporation",
    "9C098B": "Cisco Systems, Inc",
    "9C5884": "Apple, Inc.",
    "24B339": "Apple, Inc.",
    "E04E7A": "Nanjing Qinheng Microelectronics Co., Ltd.",
    "D07602": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "94BDBE": "TPV Display Technology (Xiamen) Co.,Ltd.",
    "940BFA": "EM Microelectronic",
    "84821B": "PROX SG Pte Ltd",
    "80A579": "IEEE Registration Authority",
    "6CEFBD": "Cisco Meraki",
    "743989": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "283984": "Qidi Technology (shanghai) Co.,Ltd.",
    "5CFA5A": "Sinepower Lda",
    "F49D8A": "Fantasia Trading LLC",
    "7C992E": "Shanghai Notion lnformatio Technology Co.,Ltd.",
    "8CEA88": "Chengdu Yocto Communication Technology Co.Ltd.",
    "105FAD": "Intel Corporate",
    "4CB04A": "Intel Corporate",
    "40EEBE": "NXP Semiconductor (Tianjin) LTD.",
    "FC9816": "ALPSALPINE CO,.LTD",
    "E08F4C": "Intel Corporate",
    "54726E": "Daimler Truck AG",
    "88659F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "3C0A7A": "Samsung Electronics Co.,Ltd",
    "2424B7": "Samsung Electronics Co.,Ltd",
    "F0051B": "Samsung Electronics Co.,Ltd",
    "BCA080": "Samsung Electronics Co.,Ltd",
    "002B70": "Samsung Electronics Co.,Ltd",
    "C44606": "Cisco Systems, Inc",
    "487410": "Cisco Systems, Inc",
    "B436A9": "Fibocom Wireless Inc.",
    "942770": "BSH Hausger\u00e4te GmbH",
    "AC1EA9": "Intelbras",
    "80E94A": "LEAPS s.r.o.",
    "20185B": "Shenzhen Jingxun Technology Co., Ltd.",
    "58FE7E": "zte corporation",
    "34194D": "Arcadyan Corporation",
    "9C5A80": "Juniper Networks",
    "6C80AB": "ifanr Inc",
    "588670": "Juniper Networks",
    "7C0CFA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3483D5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C40826": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "2817CB": "Software Freedom Conservancy",
    "7C5E98": "eero inc.",
    "C8586A": "Beijing Winner Microelectronics Co.,Ltd. ",
    "1C64F0": "Motorola Mobility LLC, a Lenovo Company",
    "74BEF3": "Motorola Mobility LLC, a Lenovo Company",
    "AC8866": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "5C4842": "Hangzhou Anysoft Information Technology Co. , Ltd",
    "8CC9E9": "Huawei Device Co., Ltd.",
    "D854F2": "Huawei Device Co., Ltd.",
    "E454E5": "Huawei Device Co., Ltd.",
    "886BDB": "Apple, Inc.",
    "1435B7": "Apple, Inc.",
    "7CC025": "Palo Alto Networks",
    "C4AB4D": "Cisco Systems, Inc",
    "10E376": "Cisco Systems, Inc",
    "F45293": "Apple, Inc.",
    "942157": "Apple, Inc.",
    "9C9E6E": "Espressif Inc.",
    "74C17E": "LG Electronics",
    "B4174D": "PROJECT MONITOR INC",
    "644842": "Apple, Inc.",
    "109819": "Dell Inc.",
    "FC915D": "Google, Inc.",
    "CCB777": "zte corporation",
    "3089A6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60F419": "eero inc.",
    "DC9272": "Sagemcom Broadband SAS",
    "180C7A": "Sagemcom Broadband SAS",
    "F84D8B": "ecamtek",
    "1CE89E": "SJIT",
    "ACA31E": "Hewlett Packard Enterprise",
    "D06726": "Hewlett Packard Enterprise ",
    "EC9B8B": "Hewlett Packard Enterprise ",
    "089734": "Hewlett Packard Enterprise ",
    "B0B867": "Hewlett Packard Enterprise ",
    "8030E0": "Hewlett Packard Enterprise ",
    "004E35": "Hewlett Packard Enterprise ",
    "B45D50": "Hewlett Packard Enterprise",
    "40E3D6": "Hewlett Packard Enterprise",
    "00FD45": "Hewlett Packard Enterprise ",
    "C8B5AD": "Hewlett Packard Enterprise ",
    "5CBA2C": "Hewlett Packard Enterprise ",
    "5CED8C": "Hewlett Packard Enterprise ",
    "9020C2": "Hewlett Packard Enterprise",
    "94F128": "Hewlett Packard Enterprise ",
    "0CC3B8": "Shenzhen Jiahua Zhongli Technology Co., LTD",
    "EC6794": "Hewlett Packard Enterprise",
    "A025D7": "Hewlett Packard Enterprise",
    "988F00": "Hewlett Packard Enterprise",
    "A40E75": "Hewlett Packard Enterprise",
    "38BD7A": "Hewlett Packard Enterprise",
    "E81098": "Hewlett Packard Enterprise",
    "445BED": "Hewlett Packard Enterprise",
    "A85BF7": "Hewlett Packard Enterprise",
    "ECFCC6": "Hewlett Packard Enterprise",
    "64E881": "Hewlett Packard Enterprise",
    "EC3BAF": "EM Microelectronic",
    "5C241F": "Qingdao Haier Technology Co.,Ltd",
    "48E2AD": "HUMAX NETWORKS",
    "5038AB": "PROVE",
    "AC8EBD": "Microsoft Corporation",
    "ACFAE4": "Nintendo Co.,Ltd",
    "98B71E": "Infinix mobility limited",
    "782E03": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "4438E8": "Intel Corporate",
    "7829AD": "NINGBO QIXIANG INFORMATION TECHNOLOGY CO., LTD",
    "381B9E": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "64078C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "480234": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8DF24": "OHSUNG",
    "7CBAC6": "Solar Manager AG",
    "0401A1": "Fortinet, Inc.",
    "D056FB": "Samsung Electronics Co.,Ltd",
    "C07AD6": "Samsung Electronics Co.,Ltd",
    "5C15C5": "Shenzhen SSC Technology Co. Ltd",
    "28AF21": "zte corporation",
    "0C323A": "Samsung Electronics Co.,Ltd",
    "000295": "MAVENIR IPA UK LTD",
    "F87A39": "IEEE Registration Authority",
    "78E61C": "Honor Device Co., Ltd.",
    "04137A": "Apple, Inc.",
    "70F94A": "Apple, Inc.",
    "90B790": "Apple, Inc.",
    "044707": "Texas Instruments",
    "945044": "Texas Instruments",
    "C8844E": "Flextronics International Kft",
    "78EE4C": "Espressif Inc.",
    "F49ECE": "Sena Technologies Co., Ltd.",
    "34105D": "Texas Instruments",
    "F0D635": "Apple, Inc.",
    "9CDBCB": "Funshion Online Technologies Co.,Ltd",
    "382CE5": "Tuya Smart Inc.",
    "847848": "Ubiquiti Inc",
    "C418FC": "Cisco Systems, Inc",
    "EC5BCD": "IEEE Registration Authority",
    "CC96A2": "China Mobile Group Device Co.,Ltd.",
    "543D60": "Arcadyan Corporation",
    "6C1D2C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20C2B0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C05234": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0C2C7": "Motorola Mobility LLC, a Lenovo Company",
    "648B9B": "ALWAYS ON TECH PTE.LTD.",
    "382835": "zte corporation",
    "68BF6C": "Arista Networks",
    "74A5C2": "KYOCERA Corporation ",
    "741213": "Linksys USA, Inc",
    "343D7F": "Klipsch Group, Inc.",
    "38DE35": "GUANGZHOU YUANDIANHE COMMUNICATION TECHNOLOGY CO.,LTD",
    "94B3F7": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "A00A9A": "Huawei Device Co., Ltd.",
    "1C83EC": "Ubee Interactive co, Limited.",
    "280708": "Samsung Electronics Co.,Ltd",
    "581F19": "EMH Metering GmbH & Co. KG",
    "08DA33": "IEEE Registration Authority",
    "B8B4C9": "Cisco Meraki",
    "C01C6A": "Google, Inc.",
    "8C1ECF": "UNIONMAN TECHNOLOGY CO.,LTD",
    "7C3E82": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "F024F9": "Espressif Inc.",
    "F8160C": "Shenzhen iComm Semiconductor CO.,LTD",
    "68CCBA": "Dense Air Networks US LLC",
    "4C6BB8": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "0056F1": "zte corporation",
    "744218": "Apple, Inc.",
    "904CC5": "Apple, Inc.",
    "EC4654": "Apple, Inc.",
    "5405DB": "LCFC(Hefei) Electronics Technology co., ltd",
    "50F351": "Apple, Inc.",
    "202DF6": "Apple, Inc.",
    "545B86": "Panasonic Automotive Systems Co.,Ltd",
    "C4EFBB": "LCFC(Hefei) Electronics Technology co., ltd",
    "84A938": "LCFC(Hefei) Electronics Technology co., ltd",
    "FC5CEE": "LCFC(Hefei) Electronics Technology co., ltd",
    "40F4C9": "FN-LINK TECHNOLOGY Ltd.",
    "38A5C9": "Tuya Smart Inc.",
    "F8172D": "Tuya Smart Inc.",
    "8C5C20": "Vantiva USA LLC",
    "987596": "EM Microelectronic",
    "00A159": "LG Electronics",
    "4C22C9": "zte corporation",
    "C41375": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "8CBFEA": "Espressif Inc.",
    "B41DC4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "105F02": "Ruijie Networks Co.,LTD",
    "2824C9": "Amazon Technologies Inc.",
    "CC6C52": "DZS Inc.",
    "341736": "Shenzhen iComm Semiconductor CO.,LTD",
    "1C627E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6056B1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5000E0": "Cisco Systems, Inc",
    "74E2E7": "Cisco Systems, Inc",
    "C4600A": "Huaqin Technology Co.LTD",
    "147E19": "Hewlett Packard Enterprise",
    "3806E6": "Arcadyan Corporation",
    "1074C5": "Calix Inc.",
    "BCCD7F": "Huawei Device Co., Ltd.",
    "742869": "Huawei Device Co., Ltd.",
    "44896D": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "E451A9": "Nanjing Xinlian Electronics Co., Ltd",
    "00F8CC": "Sagemcom Broadband SAS",
    "145A41": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "AC45CA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "D0C1B5": "Dell Inc.",
    "8876B9": "D-Link Corporation",
    "ECE78E": "AsiaTelco Technologies Co.",
    "C48B66": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "14E289": "Abietec Inc.",
    "502FBB": "Motorola Mobility LLC, a Lenovo Company",
    "6C0B5E": "HP Inc.",
    "2C195C": "Beijing Xiaomi Mobile Software Co., Ltd",
    "50695A": "AiFamous\uff08shenzhen\uff09Technology Co.,Ltd",
    "C463C4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54A245": "Digisol Systems Limited",
    "90A196": "Private",
    "1435B3": "Future Designs, Inc.",
    "6C8720": "New H3C Technologies Co., Ltd",
    "0C8D7A": "RADiflow",
    "FCA9F5": "Xiaomi Communications Co Ltd",
    "6C0370": "Extreme Networks Headquarters",
    "7896A3": "Extreme Networks Headquarters",
    "DCE650": "Extreme Networks Headquarters",
    "241FBD": "Extreme Networks Headquarters",
    "E444E5": "Extreme Networks Headquarters",
    "40882F": "Extreme Networks Headquarters",
    "4018B1": "Extreme Networks Headquarters",
    "206C8A": "Extreme Networks Headquarters",
    "885BDD": "Extreme Networks Headquarters",
    "348584": "Extreme Networks Headquarters",
    "949B2C": "Extreme Networks Headquarters",
    "A4EA8E": "Extreme Networks Headquarters",
    "B85001": "Extreme Networks Headquarters",
    "14144B": "Ruijie Networks Co.,LTD",
    "0004A5": "Barco NV",
    "ACED32": "Extreme Networks Headquarters",
    "00E60E": "Extreme Networks Headquarters",
    "44D980": "EVERYBOT INC.",
    "0016E6": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "D0DC2C": "Cisco Systems, Inc",
    "F83C44": "SHENZHEN TRANSCHAN TECHNOLOGY LIMITED",
    "2015DE": "Samsung Electronics Co.,Ltd",
    "18523D": "Xiamen Jiwu Technology CO.,Ltd",
    "D843EA": "SY Electronics Ltd",
    "D09168": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "D4A38B": "ELE(GROUP)CO.,LTD",
    "5413CA": "ITEL MOBILE LIMITED",
    "10E83A": "FIBERX DISTRIBUIDORA DE PRODUTOS DE TELECOMUNICACAO LTDA",
    "E8C829": "Intel Corporate",
    "E04C05": "EverCharge",
    "58687A": "Sagemcom Broadband SAS",
    "80482C": "Wyze Labs Inc",
    "885EBD": "NCKOREA Co.,Ltd.",
    "1C6066": "TEJAS NETWORKS LTD",
    "9C891E": "FireBrick Ltd",
    "D0F405": "Hon Hai Precision Industry Co., Ltd.",
    "58A2E1": "Mellanox Technologies, Inc.",
    "B0CF0E": "Mellanox Technologies, Inc.",
    "8809AF": "Masimo Corporation",
    "B04AB4": "Motorola Mobility LLC, a Lenovo Company",
    "C414A2": "Cisco Meraki",
    "64E738": "Zhejiang SUPCON Technology Co., Ltd.",
    "5C7DF3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "EC0482": "STL Systems AG",
    "E88DA6": "Quectel Wireless Solutions Co.,Ltd.",
    "3C0AF3": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "CCE9FA": "Samsung Electronics Co.,Ltd",
    "6466D8": "Samsung Electronics Co.,Ltd",
    "489A5B": "Shenzhen iComm Semiconductor CO.,LTD",
    "9C9613": "Lenovo Future Communication Technology (Chongqing) Company Limited",
    "DCF51B": "Arcadyan Corporation",
    "087B87": "Cisco Systems, Inc",
    "000CEC": "Safran Trusted 4D Inc.",
    "58D312": "zte corporation",
    "D8097F": "zte corporation",
    "848687": "weiyuantechnology",
    "00E63A": "Ruckus Wireless",
    "44E761": "Infinix mobility limited",
    "6C68A4": "Guangzhou V-Solution Telecommunication Technology Co.,Ltd.",
    "44B59C": "Tenet Networks Private Limited",
    "A436C7": "LG Innotek",
    "38F6ED": "EVK DI Kerschhaggl GmbH",
    "D82D40": "Janz - Contagem e Gest\u00e3o de Flu\u00eddos S.A.",
    "003969": "Air-Weigh Incorporated",
    "C07C90": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "94F929": "Meta Platforms Technologies, LLC",
    "B8C065": "Universal Electronics, Inc.",
    "2091DF": "Apple, Inc.",
    "A89C78": "Apple, Inc.",
    "7C6130": "Apple, Inc.",
    "80B989": "Apple, Inc.",
    "C4524F": "Apple, Inc.",
    "603E5F": "Apple, Inc.",
    "D0CEC9": "HAN CHANG",
    "A4BD7E": "HMD Global Oy",
    "68DA73": "IEEE Registration Authority",
    "8CD08B": "WuXi Rigosys Technology Co.,LTD",
    "4432C8": "Vantiva USA LLC",
    "08952A": "Vantiva USA LLC",
    "FC528D": "Vantiva USA LLC",
    "6C55E8": "Vantiva USA LLC",
    "889E68": "Vantiva USA LLC",
    "F85E42": "Vantiva USA LLC",
    "EC937D": "Vantiva USA LLC",
    "7C9A54": "Vantiva USA LLC",
    "E03717": "Vantiva USA LLC",
    "905851": "Vantiva USA LLC",
    "087E64": "Vantiva USA LLC",
    "1033BF": "Vantiva USA LLC",
    "00651E": "Amcrest Technologies",
    "087073": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28BE9B": "Vantiva USA LLC",
    "D05A00": "Vantiva USA LLC",
    "3C9A77": "Vantiva USA LLC",
    "D4E2CB": "Vantiva USA LLC",
    "5C22DA": "Vantiva USA LLC",
    "FCC233": "ASUSTek COMPUTER INC.",
    "00EE01": "Enablers Solucoes e Consultoria em Dispositivos",
    "80F0CF": "Ruckus Wireless",
    "68DECE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "FCB467": "Espressif Inc.",
    "D8BC38": "Espressif Inc.",
    "38315A": "Rinnai",
    "3C19CB": "TECNO MOBILE LIMITED",
    "FCFA21": "zte corporation",
    "D07CB2": "Sigmastar Technology Ltd.",
    "0438DC": "China Unicom Online Information Technology Co.,Ltd",
    "DCBDCC": "Quectel Wireless Solutions Co.,Ltd.",
    "A800E3": "Starkey Labs Inc.",
    "B096EA": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "64009C": "Insulet Corporation",
    "605699": "MAGNETI MARELLI   S.E. S.p.A.",
    "A82C3E": "Shenzhen Cultraview Digital Technology Co., Ltd",
    "D0F928": "zte corporation",
    "38922E": "ArrayComm",
    "186A81": "Sagemcom Broadband SAS",
    "A07F8A": "Sagemcom Broadband SAS",
    "2C9975": "Samsung Electronics Co.,Ltd",
    "B0F00C": "Dongguan Wecxw CO.,Ltd.",
    "A8FB40": "Nokia Solutions and Networks GmbH & Co. KG",
    "A8E539": "Nurivoice Co., Ltd",
    "A8584E": "PK VEGA",
    "C47EE0": "Cisco Systems, Inc",
    "1091D1": "Intel Corporate",
    "6C4CE2": "Intel Corporate",
    "1C7C98": "NEC Platforms, Ltd.",
    "00E04D": "INTERNET INITIATIVE JAPAN, INC",
    "743AEF": "Kaon Group Co., Ltd.",
    "F854F6": "AzureWave Technology Inc.",
    "087458": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C482E1": "Tuya Smart Inc.",
    "24E4CE": "Kaon Group Co., Ltd.",
    "F0C558": "U.D.Electronic Corp.",
    "60C9AA": "Nokia",
    "E068EE": "Phyplus Microelectronics Limited",
    "6879DD": "Omnipless Manufacturing (PTY) Ltd",
    "48062B": "Private",
    "E87640": "SKY UK LIMITED",
    "743357": "vivo Mobile Communication Co., Ltd.",
    "AC00F9": "BizLink Technology (S.E.A) Sdn. Bhd.",
    "681A7C": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "00181C": "VITEC",
    "989F1E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "944788": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5412CB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98FBF5": "ATRALTECH",
    "90395F": "Amazon Technologies Inc.",
    "301D49": "Firmus Technologies Pty Ltd",
    "401482": "Cisco Systems, Inc",
    "800794": "Samsung Electronics Co.,Ltd",
    "F43C3B": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "74D7CA": "Panasonic Automotive Systems Co.,Ltd",
    "D4B680": "Shanghai Linkyum Microeletronics Co.,Ltd",
    "C81337": "Juniper Networks",
    "AC416A": "Amazon Technologies Inc.",
    "8CEEFD": "zte corporation",
    "C04943": "zte corporation",
    "58CDC9": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "BCF499": "Rockwell Automation",
    "903C1D": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "246C84": "Cisco Systems, Inc",
    "90EB50": "Cisco Systems, Inc",
    "609532": "Zebra Technologies Inc.",
    "D4E95E": "Texas Instruments",
    "1015C1": "Zhanzuo (Beijing) Technology Co., Ltd.",
    "8CF3E7": "solidotech",
    "90A7BF": "EM Microelectronic",
    "80EE25": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "1055E4": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "28EF01": "Amazon Technologies Inc.",
    "D452C7": "Beijing L&S Lancom Platform Tech. Co., Ltd.",
    "8488E1": "Apple, Inc.",
    "10BD3A": "Apple, Inc.",
    "184A53": "Apple, Inc.",
    "109F41": "Apple, Inc.",
    "7072FE": "Apple, Inc.",
    "2CC253": "Apple, Inc.",
    "B0FB15": "Laird Connectivity",
    "80C45D": "IPG Laser GmbH",
    "68F63B": "Amazon Technologies Inc.",
    "6885A4": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "24E8E5": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "04AB08": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "E8D52B": "Google, Inc.",
    "CC3ADF": "Neptune Technology Group Inc.",
    "68EC8A": "IKEA of Sweden AB",
    "111111": "Private",
    "B456FA": "IOPSYS Software Solutions",
    "E8F8D0": "Nokia Shanghai Bell Co., Ltd.",
    "408E2C": "Microsoft Corporation",
    "04CE09": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "40679B": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "C068CC": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "28BEF3": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "E0CB19": "Nokia",
    "848F69": "Dell Inc.",
    "549F35": "Dell Inc.",
    "845A3E": "Cisco Systems, Inc",
    "B0216F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94DF34": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7430AF": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A44CC8": "Dell Inc.",
    "E4F004": "Dell Inc.",
    "20040F": "Dell Inc.",
    "D4BED9": "Dell Inc.",
    "ECF4BB": "Dell Inc.",
    "B8CA3A": "Dell Inc.",
    "B82A72": "Dell Inc.",
    "BC305B": "Dell Inc.",
    "000D56": "Dell Inc.",
    "001D09": "Dell Inc.",
    "0023AE": "Dell Inc.",
    "14C35E": "FibRSol Global Network Limited",
    "880894": "Skullcandy",
    "902AEE": "Xiaomi Communications Co Ltd",
    "40FAFE": "Motorola Mobility LLC, a Lenovo Company",
    "201C3A": "Nintendo Co.,Ltd",
    "249038": "Universal Biosensors Pty Ltd",
    "F8EDAE": "MOBIWIRE MOBILES(NINGBO) CO.,LTD",
    "D8478F": "Microchip Technology Inc.",
    "3C0CDB": "UNION MAN TECHNOLOGY CO.,LTD",
    "F4D9C6": "UNION MAN TECHNOLOGY CO.,LTD",
    "94988F": "Sagemcom Broadband SAS",
    "F07B65": "Sagemcom Broadband SAS",
    "C025A5": "Dell Inc.",
    "E8B5D0": "Dell Inc.",
    "089204": "Dell Inc.",
    "70B5E8": "Dell Inc.",
    "B8CB29": "Dell Inc.",
    "34735A": "Dell Inc.",
    "8CEC4B": "Dell Inc.",
    "544810": "Dell Inc.",
    "A89969": "Dell Inc.",
    "F40270": "Dell Inc.",
    "3448ED": "Dell Inc.",
    "A8C56F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "AC7A94": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "04DA28": "Chongqing Zhouhai Intelligent Technology Co., Ltd",
    "0C014B": "zte corporation",
    "74D6E5": "Huawei Device Co., Ltd.",
    "5C2167": "Rockwell Automation",
    "20F597": "Maasiv, LLC",
    "988CB3": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "D8E35E": "LG Innotek",
    "786829": "eero inc.",
    "14A417": "Shenzhen Belon Technology CO.,LTD",
    "60489C": "YIPPEE ELECTRONICS CO.,LIMITED",
    "34AFA3": "Recogni Inc",
    "DC9EAB": "Chongqing Yipingfang Technology Co., Ltd.",
    "4891D5": "Cisco Systems, Inc",
    "6C8F4E": "Chongqing Yipingfang Technology Co., Ltd.",
    "E01FFC": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "CC4740": "AzureWave Technology Inc.",
    "001491": "Daniels Electronics Ltd. dba Codan Radio Communications",
    "200BC5": "Cisco Systems, Inc",
    "90E95E": "Cisco Systems, Inc",
    "2857BE": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "5850ED": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "1012FB": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "4CF5DC": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "988B0A": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "807C62": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "BC9B5E": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "2029B9": "Ikotek technology SH Co., Ltd",
    "304F75": "DZS Inc.",
    "D096FB": "DZS Inc.",
    "000147": "DZS Inc.",
    "000271": "DZS Inc.",
    "1868CB": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "BCBAC2": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "ACCB51": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "9C65FA": "AcSiP",
    "882508": "Meta Platforms Technologies, LLC",
    "80BEAF": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "040312": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "4881D4": "Ruijie Networks Co.,LTD",
    "9C84B6": "Shenzhen iComm Semiconductor CO.,LTD",
    "34CA81": "New H3C Intelligence Terminal Co., Ltd.",
    "280C2D": "QUALVISION TECHNOLOGY CO.,LTD",
    "8867DC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4678B": "Alphabet Capital Sdn Bhd",
    "60F04D": "Honor Device Co., Ltd.",
    "5C628B": "TP-Link Corporation Limited",
    "D43A2F": "SHENZHEN MTC CO LTD",
    "60030C": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "40410D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C8B6B": "Amazon Technologies Inc.",
    "C0613D": "BioIntelliSense, Inc.",
    "0418D6": "Ubiquiti Inc",
    "24A43C": "Ubiquiti Inc",
    "0019E2": "Juniper Networks",
    "002688": "Juniper Networks",
    "841888": "Juniper Networks",
    "C82B6B": "shenzhen worldelite electronics co., LTD",
    "1814AE": "Nokia",
    "44D9E7": "Ubiquiti Inc",
    "D021F9": "Ubiquiti Inc",
    "70A741": "Ubiquiti Inc",
    "B0A86E": "Juniper Networks",
    "0881F4": "Juniper Networks",
    "E4BCAA": "Xiaomi Communications Co Ltd",
    "545DD9": "EDISTEC",
    "404CCA": "Espressif Inc.",
    "307C5E": "Juniper Networks",
    "F01C2D": "Juniper Networks",
    "64649B": "Juniper Networks",
    "009069": "Juniper Networks",
    "80ACAC": "Juniper Networks",
    "0C8610": "Juniper Networks",
    "544B8C": "Juniper Networks",
    "2C2131": "Juniper Networks",
    "9CCC83": "Juniper Networks",
    "44AA50": "Juniper Networks",
    "E8B6C2": "Juniper Networks",
    "C409B7": "Juniper Networks",
    "74E798": "Juniper Networks",
    "487310": "Juniper Networks",
    "E4FC82": "Juniper Networks",
    "B8C253": "Juniper Networks",
    "14B3A1": "Juniper Networks",
    "D4996C": "Juniper Networks",
    "045C6C": "Juniper Networks",
    "40DEAD": "Juniper Networks",
    "0C599C": "Juniper Networks",
    "9CD1D0": "Guangzhou Ronsuo Electronic Technology Co.,Ltd",
    "3C792B": "Dongguan Auklink TechnologyCo.,Ltd",
    "000E51": "TECNA SpA",
    "C0C70A": "Ruckus Wireless",
    "001616": "BROWAN COMMUNICATIONS INCORPORATION",
    "B461E9": "Sichuan AI-Link Technology Co., Ltd.",
    "64E204": "NTN Technical Service Corporation",
    "2C9811": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "90D689": "Huahao Fangzhou Technology Co.,Ltd",
    "C08B2A": "Cisco Systems, Inc",
    "2CCA75": "Robert Bosch GmbH AnP",
    "CCD81F": "Maipu Communication Technology Co.,Ltd.",
    "E0D8C4": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "38141B": "Secure Letter Inc.",
    "BC9E2C": "China Mobile Group Device Co.,Ltd.",
    "88C6E8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "707CE3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C6F9B": "zte corporation",
    "5C9462": "Shenzhen Jiuzhou Electric Co.,LTD",
    "386F6B": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "541149": "vivo Mobile Communication Co., Ltd.",
    "9052BF": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "881E59": "Onion Corporation",
    "2C002A": "Shenzhen TINNO Mobile Technology Corp.",
    "FC9CA7": "Apple, Inc.",
    "48E15C": "Apple, Inc.",
    "7415F5": "Apple, Inc.",
    "48A98A": "Routerboard.com",
    "E88843": "Xiaomi Communications Co Ltd",
    "28022E": "Apple, Inc.",
    "ACFC82": "Shenzhen Sundray Technologies Company Limited",
    "705A6F": "IEEE Registration Authority",
    "DCA313": "Shenzhen Changjin Communication Technology Co.,Ltd",
    "182654": "Samsung Electronics Co.,Ltd",
    "1C6760": "Phonesuite",
    "043201": "Broadcom Limited",
    "7C1779": "EM Microelectronic",
    "B04B68": "NAKAYO Inc",
    "F86B14": "Barrot Technology Co.,LTD",
    "704698": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78084D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ECAA8F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "689A21": "Fiberhome Telecommunication Technologies Co.,LTD",
    "38396C": "Huawei Device Co., Ltd.",
    "5066E5": "Huawei Device Co., Ltd.",
    "DC7794": "Huawei Device Co., Ltd.",
    "D8341C": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "00187F": "ZODIANET",
    "CCF826": "Samsung Electronics Co.,Ltd",
    "70B651": "Eight Sleep",
    "90CEB8": "Texas Instruments",
    "B4BC7C": "Texas Instruments",
    "F88A5E": "Texas Instruments",
    "F4331C": "Toast, Inc.",
    "5451DE": "Cisco Systems, Inc",
    "4C0617": "Taicang T&W Electronics",
    "D49AF6": "AzureWave Technology Inc.",
    "AC81F3": "Nokia Corporation",
    "606E41": "Barrot Technology Co.,LTD",
    "2C1809": "Apple, Inc.",
    "FC47D8": "Apple, Inc.",
    "E49D73": "Edgecore Networks Corporation",
    "44EB2E": "ALPSALPINE CO,.LTD",
    "047F0E": "Barrot Technology Co.,LTD",
    "D40145": "ATW TECHNOLOGY, INC.",
    "A8F8C9": "NXP Semiconductor (Tianjin) LTD.",
    "94720F": "Guangdong Nanguang Photo&Video Systems Co., Ltd.",
    "646876": "Edifier International",
    "7C646C": "LG Electronics",
    "48C1EE": "Honor Device Co., Ltd.",
    "88948E": "Max Weishaupt GmbH",
    "DCB347": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "60292B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A486AE": "Quectel Wireless Solutions Co.,Ltd.",
    "58D391": "Quectel Wireless Solutions Co.,Ltd.",
    "0024AC": "Hangzhou DPtech Technologies Co., Ltd.",
    "CC087B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C64F3": "sywinkey HongKong Co,. Limited?",
    "28E424": "New H3C Technologies Co., Ltd",
    "50804A": "Quectel Wireless Solutions Co.,Ltd.",
    "601521": "Redarc Electronics",
    "E426D5": "Motorola Mobility LLC, a Lenovo Company",
    "FC3FA6": "eero inc.",
    "E04B41": "Hangzhou Beilian Low Carbon Technology Co., Ltd.",
    "8C8E0D": "zte corporation",
    "9C8824": "PetroCloud LLC",
    "5C2FAF": "HomeWizard B.V.",
    "704777": "Ruckus Wireless",
    "00068F": "Telemonitor, Inc.",
    "040067": "Stanley Black & Decker",
    "7830F5": "TBT Inc.",
    "88AF7B": "Nanjing Powercore Tech Co.,Ltd",
    "C47B80": "Protempis, LLC",
    "00090F": "Fortinet, Inc.",
    "000CE6": "Fortinet, Inc.",
    "A024F9": "Chengdu InnovaTest Technology Co., Ltd",
    "E0B98A": "Shenzhen Taike industrial automation company,Ltd",
    "A8EE6D": "Fine Point-High Export",
    "54C6FF": "New H3C Technologies Co., Ltd",
    "F4E975": "New H3C Technologies Co., Ltd",
    "D4D4DA": "Espressif Inc.",
    "386407": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "1C63A5": "securityplatform",
    "C4E5B1": "Suzhou PanKore Integrated Circuit Technology Co. Ltd.",
    "ACC4A9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "7C949F": "Shenzhen iComm Semiconductor CO.,LTD",
    "3C2CA6": "Beijing Xiaomi Electronics Co.,Ltd",
    "34F223": "Fujian Newland Communication Science Technology Co.,Ltd.",
    "280AEE": "Renesas Electronics (Penang) Sdn. Bhd.",
    "70AED5": "Apple, Inc.",
    "4C22F3": "Arcadyan Corporation",
    "200C86": "GX India Pvt Ltd",
    "88B86F": "Infinix mobility limited",
    "C4EB43": "Sagemcom Broadband SAS",
    "C4EB41": "Sagemcom Broadband SAS",
    "743E39": "YUSUR Technology Co., Ltd.",
    "4C74A7": "IEEE Registration Authority",
    "B8D4BC": "zte corporation",
    "385319": "34ED LLC DBA Centegix",
    "846082": "Hyperloop Technologies, Inc dba Virgin Hyperloop",
    "F04F7C": "Amazon Technologies Inc.",
    "F0A225": "Amazon Technologies Inc.",
    "D43A2C": "Google, Inc.",
    "E8AACB": "Samsung Electronics Co.,Ltd",
    "C4AAC4": "Zhejiang Dahua Technology Co., Ltd.",
    "BCD0EB": "New H3C Technologies Co., Ltd",
    "605801": "Shandong ZTop Microelectronics Co., Ltd.",
    "D8630D": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "9C635B": "zte corporation",
    "1461A4": "Honor Device Co., Ltd.",
    "D09686": "IEEE Registration Authority",
    "94D86B": "nass magnet Hung\u00e1ria Kft.",
    "38A9EA": "HK DAPU ELECTRONIC TECHNOLOGY CO., LIMITED",
    "E8EF22": "Siemens Numerical Control Ltd., Nanjing",
    "786C84": "Amazon Technologies Inc.",
    "94BE09": "China Mobile Group Device Co.,Ltd.",
    "E07526": "China Dragon Technology Limited",
    "E48E10": "CIG SHANGHAI CO LTD",
    "64DBF7": "Nokia Shanghai Bell Co., Ltd.",
    "3CA2C3": "vivo Mobile Communication Co., Ltd.",
    "5C475E": "Ring LLC",
    "54102E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94D54D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24E50F": "Google, Inc.",
    "C091B9": "Amazon Technologies Inc.",
    "702804": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "F8075D": "Huawei Device Co., Ltd.",
    "98226E": "Amazon Technologies Inc.",
    "F8A4FB": "Nanjing Decowell Automation Co.,LTD",
    "24F0D3": "Samsung Electronics Co.,Ltd",
    "582071": "Samsung Electronics Co.,Ltd",
    "684992": "Cisco Meraki",
    "F89B6E": "Nokia Solutions and Networks GmbH & Co. KG",
    "14DC51": "Xiamen Cheerzing IOT Technology Co.,Ltd.",
    "04331F": "Huawei Device Co., Ltd.",
    "D4BABA": "IEEE Registration Authority",
    "9CD8E3": "Wuhan Huazhong Numerical Control Co., Ltd",
    "E0C264": "Intel Corporate",
    "24DC0F": "Phytium Technology Co.,Ltd.",
    "7C2F80": "Gigaset Communications GmbH",
    "60C7BE": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "0404B8": "China Hualu Panasonic AVC Networks Co., LTD.",
    "40C2BA": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "18D6DD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC1803": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68962E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC3DD1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54C45B": "Arcadyan Corporation",
    "80BC37": "Ruckus Wireless",
    "E001A6": "Edgecore Networks Corporation",
    "203626": "TP-Link Corporation Limited",
    "C0515C": "zte corporation",
    "BC89A7": "Apple, Inc.",
    "30B0EA": "Shenzhen Chuangxin Internet Communication Technology Co., Ltd",
    "4CFC22": "SHANGHAI HI-TECH CONTROL SYSTEM CO.,LTD.",
    "E4ECE8": "Samsung Electronics Co.,Ltd",
    "C8908A": "Samsung Electronics Co.,Ltd",
    "403306": "Taicang T&W Electronics",
    "000BF8": "Infinera, Inc.",
    "783EA1": "Nokia Shanghai Bell Co., Ltd.",
    "7C50DA": "E.J Ward",
    "A41CB4": "DFI Inc",
    "401A58": "Wistron Neweb Corporation",
    "2CF2A5": "Sagemcom Broadband SAS",
    "7408DE": "Fujian Landi Commercial Technology Co., Ltd.",
    "10FC33": "Huawei Device Co., Ltd.",
    "943C96": "Sagemcom Broadband SAS",
    "70B7E4": "Broadcom Limited",
    "1423F2": "Broadcom Limited",
    "B08101": "Honor Device Co., Ltd.",
    "485DED": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "5CFC6E": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "7CDCCC": "BEIJING STARBLAZE TECHNOLOGY CO.,LTD",
    "34479A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "94AD23": "Apple, Inc.",
    "20A5CB": "Apple, Inc.",
    "F421CA": "Apple, Inc.",
    "3C0268": "Infinera, Inc.",
    "CC5EF8": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "FC1263": "ASKEY COMPUTER CORP",
    "58305B": "Shanghai Junqian Sensing Technology Co.,LTD",
    "A03131": "Procenne Digital Security",
    "C071AA": "ShenZhen OnMicro Electronics Co.,Ltd.",
    "709C45": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC0736": "Huawei Device Co., Ltd.",
    "94A07D": "Huawei Device Co., Ltd.",
    "5C0758": "Ufispace Co., LTD.",
    "142D79": "Fiberhome Telecommunication Technologies Co.,LTD",
    "84F44C": "International Integrated Systems., Inc.",
    "589A3E": "Amazon Technologies Inc.",
    "E838A0": "Vizio, Inc",
    "ECC07A": "Laird Connectivity",
    "C81EC2": "ITEL MOBILE LIMITED",
    "6883CB": "Apple, Inc.",
    "68B9C2": "Earda Technologies co Ltd",
    "88B863": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "1C46D1": "SKY UK LIMITED",
    "003F10": "Shenzhen GainStrong Technology Co., Ltd.",
    "8822B2": "Chipsea Technologies (Shenzhen) Corp.",
    "4802AF": "Telit Communication s.p.a",
    "2004F3": "Honor Device Co., Ltd.",
    "38E7C0": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "F0C1CE": "GoodWe Technologies CO., Ltd",
    "906AEB": "Microsoft Corporation",
    "C4EB39": "Sagemcom Broadband SAS",
    "0052C8": "Made Studio Design Ltd.",
    "582B0A": "Texas Instruments",
    "089DF4": "Intel Corporate",
    "DC4628": "Intel Corporate",
    "0C9192": "Intel Corporate",
    "48AD9A": "Intel Corporate",
    "D8E844": "zte corporation",
    "E46A35": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "6865B7": "Zhishang Chuanglian Technology Co., Ltd",
    "9C0591": "Mellanox Technologies, Inc.",
    "E4B555": "Huawei Device Co., Ltd.",
    "2877B1": "Tri plus grupa d.o.o.",
    "405EF6": "Samsung Electronics Co.,Ltd",
    "945244": "Samsung Electronics Co.,Ltd",
    "9C2E7A": "Samsung Electronics Co.,Ltd",
    "7C6305": "Amazon Technologies Inc.",
    "D87E6F": "CASCINATION AG",
    "044F7A": "China Mobile Group Device Co.,Ltd.",
    "FCC737": "Shaanxi Gangsion Electronic Technology Co., Ltd",
    "94286F": "zte corporation",
    "400EF3": "zte corporation",
    "743AF4": "Intel Corporate",
    "687A64": "Intel Corporate",
    "BC0358": "Intel Corporate",
    "5431D4": "TGW Mechanics GmbH",
    "E016B1": "Advanced Design Technology co.,ltd.",
    "AC606F": "Nokia Shanghai Bell Co., Ltd.",
    "C43CB0": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "C06911": "Arista Networks",
    "9401AC": "Wuhan Qianyang Iotian Technology Co., Ltd",
    "2C93FB": "Sercomm France Sarl",
    "7844FD": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B4466B": "REALTIMEID AS",
    "58C57E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2418C0": "E. Wehrle GmbH",
    "0021D0": "Global Display Solutions Spa",
    "4C312D": "Sichuan AI-Link Technology Co., Ltd.",
    "C04E30": "Espressif Inc.",
    "AC15A2": "TP-Link Corporation Limited",
    "B8B409": "Samsung Electronics Co.,Ltd",
    "34CF6C": "Hangzhou Taili wireless communication equipment Co.,Ltd",
    "489BE0": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "ACDF9F": "Arcadyan Corporation",
    "B859C8": "70mai Co.,Ltd.",
    "C82496": "Jiangsu Yinhe  Electronics Co.,Ltd.",
    "E0F678": "Fiberhome Telecommunication Technologies Co.,LTD",
    "28F7D6": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A8E207": "GOIP Global Services Pvt. Ltd.",
    "D4E22F": "Roku, Inc",
    "5CFA25": "Sagemcom Broadband SAS",
    "F8CDC8": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "402230": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "145BB9": "ConMet",
    "0830CE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "14130B": "Garmin International",
    "90235B": "Amazon Technologies Inc.",
    "1CBCEC": "silex technology, Inc.",
    "64D814": "Cisco Systems, Inc",
    "7C11CD": "QianTang Technology",
    "447F77": "Connected Home",
    "B0285B": "JUHUA Technology Inc.",
    "E06A05": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "90CD1F": "Quectel Wireless Solutions Co.,Ltd.",
    "749779": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "403B7B": "Huawei Device Co., Ltd.",
    "F0B661": "eero inc.",
    "0425F0": "Nokia",
    "4857D2": "Broadcom Limited",
    "9C2183": "Broadcom Limited",
    "D834EE": "SHURE INCORPORATED",
    "94C5A6": "ITEL MOBILE LIMITED",
    "5CE91E": "Apple, Inc.",
    "88034C": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "047C16": "Micro-Star INTL CO., LTD.",
    "001794": "Cisco Systems, Inc",
    "F4CE46": "Hewlett Packard",
    "E0D738": "WireStar Networks",
    "C8EBEC": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "404244": "Cisco Systems, Inc",
    "04B6BE": "CIG SHANGHAI CO LTD",
    "7CECB1": "Apple, Inc.",
    "C0EDE5": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "10A562": "Iton Technology Corp.",
    "D4A3EB": "Shenzhen iComm Semiconductor CO.,LTD",
    "FCB97E": "GE Appliances",
    "30E8E4": "Qorvo International Pte. Ltd.",
    "281293": "Honor Device Co., Ltd.",
    "C0A938": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48DC9D": "Grandprint(Beijing) Technology Co., LTD.",
    "64FD96": "Sagemcom Broadband SAS",
    "64989E": "TRINNOV AUDIO",
    "BCC746": "Hon Hai Precision IND.CO.,LTD",
    "8C763F": "ARRIS Group, Inc.",
    "F0D415": "Intel Corporate",
    "D4D853": "Intel Corporate",
    "643172": "ZHEJIANG HISING TECHNOLOGY CO.,LTD",
    "D0FCD0": "HUMAX Co., Ltd.",
    "20FADB": "Huahao Kunpeng Technology (chengDu)  Co.,Ltd.",
    "0C8B95": "Espressif Inc.",
    "9CBFCD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B89FCC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3425BE": "Amazon Technologies Inc.",
    "9CE041": "Nokia",
    "783486": "Nokia",
    "685E1C": "Texas Instruments",
    "38AB41": "Texas Instruments",
    "CC4792": "ASIX Electronics Corporation",
    "E046EE": "NETGEAR",
    "5876AC": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "9C4F5F": "Google, Inc.",
    "A02942": "Intel Corporate",
    "1071B3": "Zyxel Communications Corporation",
    "DCBE49": "ITEL MOBILE LIMITED",
    "9C9561": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "E0036B": "Samsung Electronics Co.,Ltd",
    "1869D4": "Samsung Electronics Co.,Ltd",
    "0463D0": "Huawei Device Co., Ltd.",
    "14448F": "Edgecore Networks Corporation",
    "50C1F0": "NXP Semiconductor (Tianjin) LTD.",
    "F4848D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "581122": "ASUSTek COMPUTER INC.",
    "74694A": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "00B0EE": "Ajile Systems, Inc.",
    "50DCD0": "Observint Technologies, Inc.",
    "78152D": "UNION CHIP TECHNOLOGY LIMITED",
    "00A554": "Intel Corporate",
    "D4F0EA": "Beijing Xiaomi Mobile Software Co., Ltd",
    "0C8629": "IEEE Registration Authority",
    "98A92D": "New H3C Technologies Co., Ltd",
    "94ABFE": "Nokia",
    "5C24E2": "Suzhou Denbom Electronic S&T Co., Ltd",
    "90DF7D": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "BC1D89": "Motorola Mobility LLC, a Lenovo Company",
    "E47305": "Shenzhen INVT Electric CO.,Ltd",
    "A854A2": "Heimgard Technologies AS",
    "B87EE5": "Intelbras",
    "60BEB4": "S-Bluetech co., limited",
    "B4B742": "Amazon Technologies Inc.",
    "78A03F": "Amazon Technologies Inc.",
    "5046AE": "MERCURY CORPORATION",
    "B099D7": "Samsung Electronics Co.,Ltd",
    "30F70D": "Cisco Systems, Inc",
    "742A8A": "shenzhen worldelite electronics co., LTD",
    "A069D9": "New H3C Technologies Co., Ltd",
    "20FE00": "Amazon Technologies Inc.",
    "F421AE": "Shanghai Xiaodu Technology Limited",
    "747446": "Google, Inc.",
    "7820A5": "Nintendo Co.,Ltd",
    "2078CD": "Apple, Inc.",
    "30D53E": "Apple, Inc.",
    "F412FA": "Espressif Inc.",
    "B0F208": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "F0B2B9": "Intel Corporate",
    "D0880C": "Apple, Inc.",
    "1C57DC": "Apple, Inc.",
    "604F5B": "Huawei Device Co., Ltd.",
    "DC99FE": "Armatura LLC",
    "9CF155": "Nokia",
    "5023A2": "Apple, Inc.",
    "98698A": "Apple, Inc.",
    "78FBD8": "Apple, Inc.",
    "A4C337": "Apple, Inc.",
    "B0F1D8": "Apple, Inc.",
    "B01B4B": "Invisible Fun Studio Limited",
    "487412": "OnePlus Technology (Shenzhen) Co., Ltd",
    "A8E621": "Amazon Technologies Inc.",
    "3CCFB4": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "043CE8": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "24813B": "Cisco Systems, Inc",
    "0CB8E8": "Renesas Electronics (Penang) Sdn. Bhd.",
    "F0A654": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "E8F375": "Nokia",
    "E831CD": "Espressif Inc.",
    "381EC7": " Chipsea Technologies(Shenzhen) Corp.",
    "C0886D": "Securosys SA",
    "14DCE2": "THALES AVS France",
    "EC5C84": "Murata Manufacturing Co., Ltd.",
    "C89BD7": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "28B133": "SHINEMAN(SHENZHEN) Tech. Cor., Ltd.",
    "74E6B8": "LG Electronics",
    "28D0F5": "Ruijie Networks Co.,LTD",
    "941865": "NETGEAR",
    "A8587C": "Shoogee GmbH & Co. KG",
    "985F4F": "Tongfang Computer Co.,Ltd.",
    "34F86E": "Parker Hannifin Corporation",
    "0CAEBD": "Edifier International",
    "90F260": "Shenzhen Honesty Electronics Co.,Ltd.",
    "1889CF": "TECNO MOBILE LIMITED",
    "84160C": "Broadcom Limited",
    "6CD869": "Guangzhou Sat Infrared Co.,LTD",
    "38F7F1": "Huawei Device Co., Ltd.",
    "4076A9": "Huawei Device Co., Ltd.",
    "F438C1": "Huawei Device Co., Ltd.",
    "28EA2D": "Apple, Inc.",
    "D004B0": "Samsung Electronics Co.,Ltd",
    "98B08B": "Samsung Electronics Co.,Ltd",
    "9C2595": "Samsung Electronics Co.,Ltd",
    "E8EECC": "Fantasia Trading LLC",
    "04BD97": "Cisco Systems, Inc",
    "541651": "Ruijie Networks Co.,LTD",
    "A06720": "China Dragon Technology Limited",
    "94C9B7": "IEEE Registration Authority",
    "60BC4C": "EWM Hightec Welding GmbH",
    "0C2369": "Honeywell SPS",
    "AC0425": "ball-b GmbH Co KG",
    "A08069": "Intel Corporate",
    "2072A9": "Beijing Xiaomi Electronics Co.,Ltd",
    "D0622C": "Xi'an Yipu Telecom Technology Co.,Ltd.",
    "E05689": "Lootom Telcovideo Network (Wuxi) Co Ltd",
    "A81710": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "086AE5": "Amazon Technologies Inc.",
    "209BE6": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "7087A7": "Murata Manufacturing Co., Ltd.",
    "4CE0DB": "Xiaomi Communications Co Ltd",
    "7C33F9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8E60C": "Apple, Inc.",
    "48352B": "Apple, Inc.",
    "4CE6C0": "Apple, Inc.",
    "3888A4": "Apple, Inc.",
    "44DA30": "Apple, Inc.",
    "A02BB8": "Hewlett Packard",
    "8CB094": "Airtech I&C Co., Ltd",
    "70A8D3": "Intel Corporate",
    "D8FBD6": "Amazon Technologies Inc.",
    "9C47F9": "LJU Automatisierungstechnik GmbH",
    "206296": "Shenzhen Malio Technology Co.,Ltd",
    "2C3358": "Intel Corporate",
    "00064D": "Sencore",
    "1C869A": "Samsung Electronics Co.,Ltd",
    "EC6488": "Honor Device Co., Ltd.",
    "98BEDC": "Honor Device Co., Ltd.",
    "E42761": "Honor Device Co., Ltd.",
    "A06974": "Honor Device Co., Ltd.",
    "0CC6FD": "Xiaomi Communications Co Ltd",
    "50DAD6": "Xiaomi Communications Co Ltd",
    "3408E1": "Texas Instruments",
    "18B96E": "Dongguan Liesheng Electronic Co., Ltd.",
    "64BE63": "STORDIS GmbH",
    "D46624": "Cisco Systems, Inc",
    "5058B0": "Hunan Greatwall Computer System Co., Ltd.",
    "14CB65": "Microsoft Corporation",
    "7813E0": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "4861EE": "Samsung Electronics Co.,Ltd",
    "A84B4D": "Samsung Electronics Co.,Ltd",
    "F4F309": "Samsung Electronics Co.,Ltd",
    "D07880": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E06CF6": "ESSENCORE limited",
    "78C62B": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "3C869A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C2768": "HUAWEI TECHNOLOGIES CO.,LTD",
    "546990": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D80A60": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4F98D": "Espressif Inc.",
    "9C9019": "Beyless",
    "0434F6": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "546925": "PS INODIC CO., LTD.",
    "F89522": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8B132": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E84727": "Quectel Wireless Solutions Co.,Ltd.",
    "28C538": "Apple, Inc.",
    "7857B0": "GERTEC BRASIL LTDA",
    "14019C": "Ubyon Inc.",
    "E8CBED": " Chipsea Technologies(Shenzhen) Corp.",
    "C4F122": "Nexar Ltd.",
    "6007C4": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "BCDB09": "Cisco Meraki",
    "0499B9": "Apple, Inc.",
    "78028B": "Apple, Inc.",
    "F84D89": "Apple, Inc.",
    "086F48": "Shenzhen iComm Semiconductor CO.,LTD",
    "0816D5": "GOERTEK INC.",
    "1C568E": "Zioncom Electronics (Shenzhen) Ltd.",
    "28563A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "540F57": "Silicon Laboratories",
    "4448FF": "Qingdao Haier Technology Co.,Ltd",
    "D4D7CF": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "F4E4D7": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "F84E17": "Sony Corporation",
    "04C461": "Murata Manufacturing Co., Ltd.",
    "A08222": "Qingdao Haier Technology Co.,Ltd",
    "E01F2B": "Nokia Solutions and Networks GmbH & Co. KG",
    "AC8226": "Qingdao Haier Technology Co.,Ltd",
    "C8CB9E": "Intel Corporate",
    "3C5576": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "90EB48": "Shanghai XinMiaoLink Technology Co., Ltd",
    "A8C266": "HUMAX Co., Ltd.",
    "185345": "Nokia",
    "E48210": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8F7B9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9C2BA6": "Ruijie Networks Co.,LTD",
    "941457": "Shenzhen Sundray Technologies Company Limited",
    "BC5729": "Shenzhen KKM Co., Ltd",
    "64B94E": "Dell Technologies",
    "5447E8": "Syrotech Networks. Ltd.",
    "EC2BEB": "Amazon Technologies Inc.",
    "B898AD": "Motorola Mobility LLC, a Lenovo Company",
    "7052D8": "ITEL MOBILE LIMITED",
    "249745": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D86C5A": "HUMAX Co., Ltd.",
    "E4C0E2": "Sagemcom Broadband SAS",
    "4C3FA7": "uGrid Network Inc.",
    "001D5B": "Tecvan Informatica Ltda",
    "1CAE3E": "IEEE Registration Authority",
    "D0C35A": "Jabil Circuit de Chihuahua",
    "84B630": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "58C7AC": "New H3C Technologies Co., Ltd",
    "0CCAFB": "TPVision Europe B.V",
    "687724": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "182DF7": "JY COMPANY",
    "EC2C11": "CWD INNOVATION LIMITED",
    "F03575": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "2067E0": "Shenzhen iComm Semiconductor CO.,LTD",
    "F8B54D": "Intel Corporate",
    "38A8CD": "IEEE Registration Authority",
    "546706": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C0F535": "AMPAK Technology,Inc.",
    "908175": "Samsung Electronics Co.,Ltd",
    "307467": "Samsung Electronics Co.,Ltd",
    "F0CD31": "Samsung Electronics Co.,Ltd",
    "6401FB": "Landis+Gyr GmbH",
    "545284": "Huawei Device Co., Ltd.",
    "646140": "Huawei Device Co., Ltd.",
    "C4170E": "Huawei Device Co., Ltd.",
    "7C8334": "IEEE Registration Authority",
    "207BD2": "ASIX Electronics Corporation",
    "C4E532": "Arcadyan Corporation",
    "0CAAEE": "Ansjer Electronics Co., Ltd.",
    "906A94": "hangzhou huacheng network technology co., ltd",
    "100C29": "Shenzhen NORCO lntelligent Technology Co.,Ltd",
    "8C3DB1": "Beijing H-IoT Technology Co., Ltd.",
    "D8A6F0": "Wu Qi Technologies,Inc.",
    "08E4DF": "Shenzhen Sande Dacom Electronics Co., Ltd",
    "105A17": "Tuya Smart Inc.",
    "008A76": "Apple, Inc.",
    "FCA64C": "Alibaba cloud computing Co., Ltd",
    "E069BA": "Cisco Systems, Inc",
    "2436DA": "Cisco Systems, Inc",
    "2037A5": "Apple, Inc.",
    "DCB54F": "Apple, Inc.",
    "B8374A": "Apple, Inc.",
    "EC7C2C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C8382": "Nokia",
    "283613": "IEEE Registration Authority",
    "145808": "Taicang T&W Electronics",
    "F4239C": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "4CEBD6": "Espressif Inc.",
    "F4B19C": "AltoBeam (China) Inc.",
    "A8D081": "Huawei Device Co., Ltd.",
    "740CEE": "Huawei Device Co., Ltd.",
    "6CB4FD": "Huawei Device Co., Ltd.",
    "7432C2": "KYOLIS",
    "00CE30": "Express LUCK Industrial Ltd.",
    "1C9DC2": "Espressif Inc.",
    "640E9B": "ISHIDA MEDICAL CO., LTD.",
    "24F603": "HUAWEI TECHNOLOGIES CO.,LTD",
    "08FA28": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8CA85": "NEC Corporation",
    "1C2CE0": "Shanghai  Mountain View Silicon",
    "2442BC": "Alinco,incorporated",
    "18FC26": "Qorvo International Pte. Ltd.",
    "ECC1AB": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "0000C8": "ALTOS COMPUTER SYSTEMS",
    "A0941A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "300EB8": "LG Electronics",
    "BC3329": "Sony Interactive Entertainment Inc.",
    "A88940": "Huawei Device Co., Ltd.",
    "0C0E76": "D-Link International",
    "703E97": "Iton Technology Corp.",
    "6C11B3": "Wu Qi Technologies,Inc.",
    "104738": "Nokia Shanghai Bell Co., Ltd.",
    "38B800": "Wistron Neweb Corporation",
    "CC312A": "HUIZHOU TCL COMMUNICATION ELECTRON CO.,LTD",
    "30B037": "New H3C Technologies Co., Ltd",
    "FC122C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6CD1E5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3039A9": "Hongshan Information Science and Technology (HangZhou) Co.,Ltd.",
    "FCD908": "Xiaomi Communications Co Ltd",
    "1C573E": "Altice Labs S.A.",
    "D0167C": "eero inc.",
    "84144D": "Intel Corporate",
    "9C6BF0": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "44953B": "RLTech India Private Limited",
    "945AFC": "Amazon Technologies Inc.",
    "00D76D": "Intel Corporate",
    "B41CAB": "ICR, inc.",
    "A85B36": "IEEE Registration Authority",
    "2C1165": "Silicon Laboratories",
    "DCED83": "Beijing Xiaomi Mobile Software Co., Ltd",
    "BCD7D4": "Roku, Inc",
    "AC0BFB": "Espressif Inc.",
    "20B868": "Motorola Mobility LLC, a Lenovo Company",
    "289C6E": "Shanghai High-Flying Electronics  Technology Co., Ltd",
    "1C2A8B": "Nokia",
    "84F703": "Espressif Inc.",
    "485A67": "Shaanxi Ruixun Electronic Information Technology Co., Ltd",
    "945C9A": "Apple, Inc.",
    "8C1F64": "IEEE Registration Authority",
    "F47B09": "Intel Corporate",
    "E87829": "IEEE Registration Authority",
    "50325F": "Silicon Laboratories",
    "1C4190": "Universal Electronics, Inc.",
    "FC8A3D": "zte corporation",
    "085A11": "D-Link International",
    "64C403": "Quectel Wireless Solutions Co.,Ltd.",
    "EC9468": "META SYSTEM SPA ",
    "00F39F": "Apple, Inc.",
    "64C394": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C77CB": "Intel Corporate",
    "44D5CC": "Amazon Technologies Inc.",
    "00005B": "ELTEC ELEKTRONIK AG",
    "5CE42A": "Intel Corporate",
    "F0C814": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "705FA3": "Xiaomi Communications Co Ltd",
    "3C62F0": "Sercomm Corporation.",
    "983B67": "DWnet Technologies(Suzhou) Corporation",
    "6C9392": "BEKO Technologies GmbH",
    "58355D": "Huawei Device Co., Ltd.",
    "AC7E01": "Huawei Device Co., Ltd.",
    "1874E2": "IEEE Registration Authority",
    "A0B086": "Hirschmann Automation and Control GmbH",
    "30B346": "CJSC NORSI-TRANS",
    "0004B8": "Kumahira Co., Ltd.",
    "000B3A": "PESA Inc.",
    "04B9E3": "Samsung Electronics Co.,Ltd",
    "94AA0A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E85C0A": "Cisco Systems, Inc",
    "9843FA": "Intel Corporate",
    "20D276": "ITEL MOBILE LIMITED",
    "3CBDD8": "LG ELECTRONICS INC",
    "54C250": "Iskratel d.o.o.",
    "00FAB6": "Kontakt Micro-Location Sp z o.o.",
    "7CF462": "BEIJING HUAWOO TECHNOLOGIES CO.LTD",
    "70B950": "Texas Instruments",
    "781C5A": "SHARP Corporation",
    "C45E5C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D01769": "Murata Manufacturing Co., Ltd.",
    "20C19B": "Intel Corporate",
    "2C6DC1": "Intel Corporate",
    "D09FD9": "IEEE Registration Authority",
    "A0D722": "Samsung Electronics Co.,Ltd",
    "5049B0": "Samsung Electronics Co.,Ltd",
    "E805DC": "Verifone Inc.",
    "000CDF": "JAI Manufacturing",
    "C4A151": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "201F3B": "Google, Inc.",
    "C8059E": "Hefei Symboltek Co.,Ltd",
    "34FCA1": "Micronet union Technology(Chengdu)Co., Ltd.",
    "107BCE": "Nokia",
    "B825B5": "Trakm8 Ltd",
    "20415A": "Smarteh d.o.o.",
    "B0A7B9": "TP-Link Corporation Limited",
    "6C5AB0": "TP-Link Corporation Limited",
    "60E6F0": "Wistron Neweb Corporation",
    "C8F5D6": "IEEE Registration Authority",
    "2897B8": "myenergi Ltd",
    "F44E38": "Olibra LLC",
    "F092B4": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "ECF8EB": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "405899": "Logitech Far East",
    "08A842": "Huawei Device Co., Ltd.",
    "2C6373": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "88E056": "HUAWEI TECHNOLOGIES CO.,LTD",
    "540E58": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "5C648E": "Zyxel Communications Corporation",
    "0000B8": "SEIKOSHA CO., LTD.",
    "D035E5": "EM Microelectronic",
    "9096F3": "BUFFALO.INC",
    "082CED": "Technity Solutions Inc.",
    "145AFC": "Liteon Technology Corporation",
    "EC153D": "Beijing Yaxunhongda Technology Co., Ltd.",
    "2066CF": "FREEBOX SAS",
    "2C8DB1": "Intel Corporate",
    "586C25": "Intel Corporate",
    "2CC81B": "Routerboard.com",
    "7847E3": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "30E396": "Huawei Device Co., Ltd.",
    "D07D33": "Huawei Device Co., Ltd.",
    "C0E1BE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "042728": "Microsoft Corporation",
    "18BFB3": "Samsung Electronics Co., Ltd., Memory Division",
    "A49BCD": "Cisco Systems, Inc",
    "B4E3F9": "Silicon Laboratories",
    "1842D4": "Wuhan Hosan Telecommunication Technology Co.,Ltd",
    "A8637D": "D-Link International",
    "6C1B3F": "MiraeSignal Co., Ltd",
    "B03DC2": "Wasp artificial intelligence(Shenzhen) Co.,ltd",
    "C84D34": "LIONS Taiwan Technology Inc.",
    "2025D2": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2C43BE": "Sunnovo International Limited",
    "5810B7": "Infinix mobility limited",
    "E0925C": "Apple, Inc.",
    "00116C": "Nanwang Multimedia Inc.,Ltd",
    "28F49B": "LEETEK",
    "D8A35C": "Samsung Electronics Co.,Ltd",
    "C09296": "zte corporation",
    "50E24E": "zte corporation",
    "D8E72B": "NETSCOUT SYSTEMS INC",
    "78E36D": "Espressif Inc.",
    "000139": "Point Multimedia Systems",
    "B02491": "Huawei Device Co., Ltd.",
    "98751A": "Huawei Device Co., Ltd.",
    "B48901": "HUAWEI TECHNOLOGIES CO.,LTD",
    "449BC1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C9180": "Apple, Inc.",
    "4CB910": "Apple, Inc.",
    "34865D": "Espressif Inc.",
    "1CA0EF": "IEEE Registration Authority",
    "446FF8": "Dyson Limited",
    "28AFFD": "Cisco Systems, Inc",
    "D097FE": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "843A5B": "Inventec(Chongqing) Corporation",
    "98CDAC": "Espressif Inc.",
    "F845C4": "Shenzhen Netforward Micro-Electronic Co., Ltd.",
    "EC9365": "Mapper.ai, Inc.",
    "9C7613": "Ring LLC",
    "808AF7": "Nanoleaf",
    "182A57": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F83E95": "HUAWEI TECHNOLOGIES CO.,LTD",
    "481258": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC3296": "Huawei Device Co., Ltd.",
    "D8EB46": "Google, Inc.",
    "2C17E0": "SYSTEMES ET TECHNOLOGIES IDENTIFICATION (STid)",
    "5449DF": "Peloton Interactive, Inc",
    "781053": "China Mobile Group Device Co.,Ltd.",
    "A84397": "Innogrit Corporation",
    "08474C": "Nokia",
    "90C119": "Nokia",
    "8CAACE": "Xiaomi Communications Co Ltd",
    "E89F39": "Nokia",
    "B0227A": "HP Inc.",
    "60A751": "Huawei Device Co., Ltd.",
    "102B41": "Samsung Electronics Co.,Ltd",
    "344AC3": "HuNan ZiKun Information Technology CO., Ltd",
    "803428": "Microchip Technology Inc.",
    "00E421": "Sony Interactive Entertainment Inc.",
    "D85ED3": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "249AD8": "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD.",
    "D8CD2C": "WUXI NEIHUA NETWORK TECHNOLOGY CO., LTD",
    "7CF880": "Cisco Systems, Inc",
    "981A35": "HUAWEI TECHNOLOGIES CO.,LTD",
    "682D83": "SHENZHEN DINGHE COMMUNICATION COMPANY",
    "403F8C": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "CCDB04": "DataRemote Inc.",
    "04D6F4": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "D8BBC1": "Micro-Star INTL CO., LTD.",
    "58FC20": "Altice Labs S.A.",
    "B844AE": "TCT mobile ltd",
    "F8AA3F": "DWnet Technologies(Suzhou) Corporation",
    "0CCF89": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "E4F75B": "ARRIS Group, Inc.",
    "F8790A": "ARRIS Group, Inc.",
    "1C937C": "ARRIS Group, Inc.",
    "AC5AFC": "Intel Corporate",
    "D4351D": "Technicolor Delivery Technologies Belgium NV",
    "F077C3": "Intel Corporate",
    "049081": "Pensando Systems, Inc.",
    "4C796E": "Intel Corporate",
    "F09E4A": "Intel Corporate",
    "C89402": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "7CB94C": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "0C8C69": "Shenzhen elink smart Co., ltd",
    "00682B": "Huawei Device Co., Ltd.",
    "98AD1D": "Huawei Device Co., Ltd.",
    "446FD8": "IEEE Registration Authority",
    "84D343": "Calix Inc.",
    "38B5D3": "SecuWorks",
    "DC15C8": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "6CD630": "Rootous System Co.,Ltd",
    "B068E6": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "BCD7CE": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "34318F": "Apple, Inc.",
    "C41411": "Apple, Inc.",
    "CCC95D": "Apple, Inc.",
    "7061BE": "Wistron Neweb Corporation",
    "C833E5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E86DE9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "144658": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58BE72": "HUAWEI TECHNOLOGIES CO.,LTD",
    "108FFE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D847BB": "Huawei Device Co., Ltd.",
    "2CDD0C": "Discovergy GmbH",
    "BC9D42": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "44B6BE": "Cisco Systems, Inc",
    "8CDEE6": "Samsung Electronics Co.,Ltd",
    "10EC81": "Samsung Electronics Co.,Ltd",
    "C45D83": "Samsung Electronics Co.,Ltd",
    "9C9C1F": "Espressif Inc.",
    "240B88": "Taicang T&W Electronics",
    "0C938F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A0412D": "Lansen Systems AB",
    "C43960": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "5CBD9A": "Huawei Device Co., Ltd.",
    "14A3B4": "Huawei Device Co., Ltd.",
    "5CA4A4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D809D6": "ZEXELON CO., LTD.",
    "9CA570": "eero inc.",
    "7404F0": "Mobiwire Mobiles (NingBo) Co., LTD",
    "B4B5B6": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "047A0B": "Beijing Xiaomi Electronics Co., Ltd.",
    "900F0C": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "4C1154": "Mobiwire Mobiles (NingBo) Co., LTD",
    "C4BF60": "TECNO MOBILE LIMITED",
    "D040EF": "Murata Manufacturing Co., Ltd.",
    "D056BF": "AMOSENSE",
    "2C9AA4": "Eolo SpA",
    "18D5B6": "SMG Holdings LLC",
    "70F096": "Cisco Systems, Inc",
    "BC9A53": "Huawei Device Co., Ltd.",
    "DC2727": "Huawei Device Co., Ltd.",
    "F042F5": "Huawei Device Co., Ltd.",
    "E02E3F": "Huawei Device Co., Ltd.",
    "EC75ED": "Citrix Systems, Inc.",
    "4C7525": "Espressif Inc.",
    "001340": "AD.EL s.r.l.",
    "002236": "VECTOR SP. Z O.O.",
    "80B97A": "eero inc.",
    "A453EE": "IEEE Registration Authority",
    "C8A40D": "Cooler Master Technology Inc",
    "38420B": "Sonos, Inc.",
    "608D26": "Arcadyan Corporation",
    "1C3929": "OHSUNG",
    "48701E": "Texas Instruments",
    "508E49": "Xiaomi Communications Co Ltd",
    "AC3328": "Huawei Device Co., Ltd.",
    "8CF319": "Siemens Industrial Automation Products Ltd., Chengdu",
    "B00CD1": "Hewlett Packard",
    "2CF05D": "Micro-Star INTL CO., LTD.",
    "8C476E": "IEEE Registration Authority",
    "0002F4": "PCTEL, Inc.",
    "309E1D": "OHSUNG",
    "687912": "IEEE Registration Authority",
    "74D6CB": "New H3C Technologies Co., Ltd",
    "0C3AFA": "New H3C Technologies Co., Ltd",
    "A8817E": "Apple, Inc.",
    "18B6CC": "We Corporation Inc.",
    "30A023": "ROCK PATH S.R.L",
    "E848B8": "TP-Link Corporation Limited",
    "78CB2C": "Join Digital, Inc.",
    "F8665A": "Apple, Inc.",
    "60BEC4": "Apple, Inc.",
    "F8B1DD": "Apple, Inc.",
    "588694": "EFM Networks",
    "C09435": "ARRIS Group, Inc.",
    "DCB72E": "Xiaomi Communications Co Ltd",
    "E0C377": "Samsung Electronics Co.,Ltd",
    "4CFBF4": "Optimal Audio Ltd",
    "E02BE9": "Intel Corporate",
    "F86D73": "Zengge Co., Limited",
    "AC3728": "Taicang T&W Electronics",
    "6CE874": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C469F0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC9930": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E868E7": "Espressif Inc.",
    "647924": "Huawei Device Co., Ltd.",
    "D8CC98": "Huawei Device Co., Ltd.",
    "20CD6E": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "64037F": "Samsung Electronics Co.,Ltd",
    "B49D02": "Samsung Electronics Co.,Ltd",
    "809FF5": "Samsung Electronics Co.,Ltd",
    "D0BCC1": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "DC4A9E": "IEEE Registration Authority",
    "FC8D3D": "Leapfive Tech. Ltd.",
    "4014AD": "Huawei Device Co., Ltd.",
    "78C95E": "Midmark RTLS ",
    "C033DA": "Shenzhen JRUN Technologies CO., LTD",
    "A87484": "zte corporation",
    "BC69CB": "Panasonic Electric Works Networks Co., Ltd.",
    "089356": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DCB7FC": "Alps Electric (Ireland) Ltd",
    "ACFAA5": "digitron",
    "B40EDE": "Intel Corporate",
    "6C146E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44E968": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CEF56": "Shenzhen Sundray Technologies Company Limited",
    "E84F4B": "Shenzhen Delos Electronic Co., Ltd",
    "B04A39": "Beijing Roborock Technology Co., Ltd.",
    "A4F9E4": "AirVine Scientific, Inc.",
    "F44FD3": "shenzhen hemuwei technology co.,ltd",
    "AC122F": "Fantasia Trading LLC",
    "FC45C3": "Texas Instruments",
    "AC6AA3": "Shenzhen Kertong Technology Co.,Ltd",
    "E4C32A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "909A4A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "BC9789": "Huawei Device Co., Ltd.",
    "684571": "Huawei Device Co., Ltd.",
    "8493A0": "Huawei Device Co., Ltd.",
    "9C8E9C": "Huawei Device Co., Ltd.",
    "FC698C": "ANDREAS STIHL AG & Co. KG",
    "3CE038": "Omnifi Inc.",
    "44CE3A": "Jiangsu Huacun Electronic Technology Co., Ltd.",
    "9C1EA4": "Renesas Electronics (Penang) Sdn. Bhd.",
    "001FB8": "Universal Remote Control, Inc.",
    "B8AE1C": "Smart Cube., Ltd",
    "607072": "SHENZHEN HONGDE SMART LINK TECHNOLOGY CO., LTD",
    "6C2D24": "Zhen Shi Information Technology (Shanghai) Co., Ltd.",
    "C8B6D3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E04BA6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "800384": "Ruckus Wireless",
    "D8373B": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "A8301C": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "7C78B2": "Wyze Labs Inc",
    "B8477A": "Dasan Electron Co., Ltd.",
    "6CADAD": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "0060E9": "ATOP TECHNOLOGIES, INC.",
    "D8F3BC": "Liteon Technology Corporation",
    "D4482D": "Shenzhen Deejoy Lighting Technology Co.,Ltd.",
    "78E3DE": "Apple, Inc.",
    "F46FED": "Fiberhome Telecommunication Technologies Co.,LTD",
    "94B97E": "Espressif Inc.",
    "CCE0DA": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "408C4C": "Shenzhen MiaoMing  Intelligent Technology Co.,Ltd",
    "141B30": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "D0D23C": "Apple, Inc.",
    "64D2C4": "Apple, Inc.",
    "DC5285": "Apple, Inc.",
    "E88152": "Apple, Inc.",
    "908158": "Apple, Inc.",
    "D8F8AF": "DAONTEC",
    "E4842B": "HANGZHOU SOFTEL OPTIC CO., LTD",
    "94026B": "Optictimes Co.,Ltd",
    "084296": "Mobile Technology Solutions LLC",
    "34EFB6": "Edgecore Networks Corporation",
    "E874C7": "Sentinhealth",
    "B4ADA3": "Guangzhou\u00a0Shiyuan\u00a0Electronic\u00a0Technology\u00a0Company\u00a0Limited",
    "18FDCB": "IEEE Registration Authority",
    "009023": "ZILOG INC.",
    "B0FEE5": "Huawei Device Co., Ltd.",
    "4487DB": "Tymphany Acoustic Technology (Huizhou) Co.,  Ltd.",
    "642315": "Huawei Device Co., Ltd.",
    "A4C74B": "Huawei Device Co., Ltd.",
    "085BD6": "Intel Corporate",
    "C81739": "ITEL MOBILE LIMITED",
    "58D061": "HUAWEI TECHNOLOGIES CO.,LTD",
    "90CCDF": "Intel Corporate",
    "B8D309": "Cox Communications, Inc",
    "689320": "New H3C Technologies Co., Ltd",
    "9454CE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "388ABE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "047E23": "China Mobile IOT Company Limited",
    "14CCB3": "AO \"GK NATEKS\"",
    "001C7B": "Castlenet Technology Inc.",
    "5484DC": "zte corporation",
    "38549B": "zte corporation",
    "DCCD2F": "Seiko Epson Corporation",
    "343EA4": "Ring LLC",
    "04C1D8": "Huawei Device Co., Ltd.",
    "C8BC9C": "Huawei Device Co., Ltd.",
    "8C8D28": "Intel Corporate",
    "6CBAB8": "Sagemcom Broadband SAS",
    "E0D464": "Intel Corporate",
    "C0F6C2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "007E56": "China Dragon Technology Limited",
    "083AF2": "Espressif Inc.",
    "743A20": "New H3C Technologies Co., Ltd",
    "3C1A9E": "VitalThings AS",
    "C884A1": "Cisco Systems, Inc",
    "D8778B": "Intelbras",
    "30C9AB": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "D4AFF7": "Arista Networks",
    "C4D738": "Huawei Device Co., Ltd.",
    "60077C": "Jala Group",
    "609866": "Texas Instruments",
    "1CD6BE": "Wistron Neweb Corporation",
    "28D127": "Beijing Xiaomi Mobile Software Co., Ltd",
    "30A452": "Arrival Elements BV",
    "F469D5": "IEEE Registration Authority",
    "8CC5B4": "Sagemcom Broadband SAS",
    "F81B04": "Zhong Shan City Richsound Electronic Industrial Ltd",
    "D88C79": "Google, Inc.",
    "10341B": "Spacelink",
    "F86465": "Anova Applied Electronics, Inc.",
    "90A935": "JWEntertainment",
    "60D4E9": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C868DE": "Huawei Device Co., Ltd.",
    "D00DF7": "Huawei Device Co., Ltd.",
    "4805E2": "Huawei Device Co., Ltd.",
    "001E68": "Quanta Computer Inc.",
    "2C600C": "Quanta Computer Inc.",
    "00238B": "Quanta Computer Inc.",
    "0008F6": "Sumitomo Electric Industries, Ltd",
    "000BA2": "Sumitomo Electric Industries, Ltd",
    "00005F": "Sumitomo Electric Industries, Ltd",
    "18ECE7": "BUFFALO.INC",
    "BC03A7": "MFP MICHELIN",
    "CCDB93": "Cisco Systems, Inc",
    "C8E265": "Intel Corporate",
    "18D61C": "Shenzhen TINNO Mobile Technology Corp.",
    "5448E6": "Beijing Xiaomi Mobile Software Co., Ltd",
    "2C6F51": "Herospeed Digital Technology Limited",
    "80EA07": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "8012DF": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "04A2F3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "982FF8": "Huawei Device Co., Ltd.",
    "40DCA5": "Huawei Device Co., Ltd.",
    "30C50F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2868D2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00D0B6": "CRESCENT NETWORKS, INC.",
    "C01C30": "Shenzhen WIFI-3L Technology Co.,Ltd",
    "1CE61D": "Samsung Electronics Co.,Ltd",
    "441C7F": "Motorola Mobility LLC, a Lenovo Company",
    "00B8B6": "Motorola Mobility LLC, a Lenovo Company",
    "BC17B8": "Intel Corporate",
    "08FA79": "vivo Mobile Communication Co., Ltd.",
    "D8C678": "MitraStar Technology Corp.",
    "ECC01B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9844CE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F09BB8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44BB3B": "Google, Inc.",
    "14C14E": "Google, Inc.",
    "D8C0A6": "AzureWave Technology Inc.",
    "10E77A": "STMicrolectronics International NV",
    "E43D1A": "Broadcom Limited",
    "7018A7": "Cisco Systems, Inc",
    "24F150": "Guangzhou Qi'an Technology Co., Ltd.",
    "304240": "zte corporation",
    "98A942": "Guangzhou Tozed Kangwei Intelligent Technology Co., LTD",
    "4C2113": "Nokia Shanghai Bell Co., Ltd.",
    "183EEF": "Apple, Inc.",
    "607EC9": "Apple, Inc.",
    "982CBC": "Intel Corporate",
    "2050E7": "AMPAK Technology,Inc.",
    "4CC53E": "Zyxel Communications Corporation",
    "108B6A": "Antailiye Technology Co.,Ltd",
    "802DBF": "Cisco Systems, Inc",
    "8C5EBD": "Huawei Device Co., Ltd.",
    "2CA042": "Huawei Device Co., Ltd.",
    "C083C9": "Huawei Device Co., Ltd.",
    "FC609B": "New H3C Technologies Co., Ltd",
    "E8A1F8": "zte corporation",
    "14C88B": "Apple, Inc.",
    "EC2651": "Apple, Inc.",
    "08ACC4": "FMTech",
    "28B371": "Ruckus Wireless",
    "D4F829": "Sagemcom Broadband SAS",
    "DC503A": "Nanjing Ticom Tech Co., Ltd.",
    "18473D": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "D477B2": "Netix Global B.V.",
    "F86BD9": "Cisco Systems, Inc",
    "C014FE": "Cisco Systems, Inc",
    "7CAD4F": "Cisco Systems, Inc",
    "605661": "IXECLOUD Tech",
    "706979": "IEEE Registration Authority",
    "244BFE": "ASUSTek COMPUTER INC.",
    "7CDDE9": "ATOM tech Inc.",
    "942533": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44F4E7": "Cohesity Inc",
    "F82F65": "Huawei Device Co., Ltd.",
    "0831A4": "Huawei Device Co., Ltd.",
    "A8E978": "Huawei Device Co., Ltd.",
    "E06234": "Texas Instruments",
    "50382F": "ASE Group Chung-Li",
    "708CBB": "MIMODISPLAYKOREA",
    "B848AA": "EM Microelectronic",
    "E0D462": "Huawei Device Co., Ltd.",
    "909838": "Huawei Device Co., Ltd.",
    "F8AC65": "Intel Corporate",
    "0019F4": "Convergens Oy Ltd",
    "F0B022": "TOHO Electronics INC.",
    "0C14D2": "China Mobile Group Device Co.,Ltd.",
    "987ECA": "Inventus Power Eletronica do Brasil LTDA",
    "6C2F8A": "Samsung Electronics Co.,Ltd",
    "2C9FFB": "Wistron Neweb Corporation",
    "84CCA8": "Espressif Inc.",
    "002060": "ALCATEL ITALIA S.p.A.",
    "948AC6": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "448DBF": "Rhino Mobility LLC",
    "C0A66D": "Inspur Group Co., Ltd.",
    "AC7A56": "Cisco Systems, Inc",
    "D89136": "Dover Fueling Solutions",
    "E0F6B5": "Nintendo Co.,Ltd",
    "40B31E": "Universal Electronics, Inc.",
    "884033": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8986D": "Palo Alto Networks",
    "FC956A": "OCTAGON SYSTEMS CORP.",
    "24C8D3": "McWane India Pvt Ltd",
    "B8E3EE": "Universal Electronics, Inc.",
    "C88314": "Tempo Communications",
    "68B9D3": "Shenzhen Trolink Technology CO, LTD",
    "9056FC": "TECNO MOBILE LIMITED",
    "98ED5C": "Tesla,Inc.",
    "C44268": "CRESTRON ELECTRONICS, INC.",
    "40F520": "Espressif Inc.",
    "78AA82": "New H3C Technologies Co., Ltd",
    "7C89C1": "Palo Alto Networks",
    "84D6C5": "SolarEdge Technologies",
    "38F601": "Solid State Storage Technology Corporation",
    "1070FD": "Mellanox Technologies, Inc.",
    "B8C6AA": "Earda Technologies co Ltd",
    "54AED0": "DASAN Networks, Inc. ",
    "B86392": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "5471DD": "Huawei Device Co., Ltd.",
    "00062A": "Cisco Systems, Inc",
    "6CF712": "Nokia Solutions and Networks GmbH & Co. KG",
    "20826A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "487AF6": "NCS ELECTRICAL SDN BHD",
    "486E70": "Zhejiang Tmall Technology Co., Ltd.",
    "601D9D": "Sichuan AI-Link Technology Co., Ltd.",
    "6409AC": "TCT mobile ltd",
    "24B657": "Cisco Systems, Inc",
    "5C5578": "iryx corp",
    "08C0EB": "Mellanox Technologies, Inc.",
    "E4E112": "Texas Instruments",
    "3414B5": "Texas Instruments",
    "D003EB": "Texas Instruments",
    "08BFA0": "Samsung Electronics Co.,Ltd",
    "701F3C": "Samsung Electronics Co.,Ltd",
    "D48A39": "Samsung Electronics Co.,Ltd",
    "E4F3C4": "Samsung Electronics Co.,Ltd",
    "9C2F4E": "zte corporation",
    "D8A8C8": "zte corporation",
    "189E2C": "Huawei Device Co., Ltd.",
    "94DB56": "Sony Home Entertainment&Sound Products Inc",
    "3822E2": "HP Inc.",
    "F01090": "New H3C Technologies Co., Ltd",
    "646266": "IEEE Registration Authority",
    "D85F77": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "2C97ED": "Sony Imaging Products & Solutions Inc.",
    "188A6A": "AVPro Global Hldgs",
    "3C7D0A": "Apple, Inc.",
    "B4157E": "Celona Inc.",
    "645E2C": "IRay Technology Co., Ltd.",
    "147740": "Huawei Device Co., Ltd.",
    "B89047": "Apple, Inc.",
    "909C4A": "Apple, Inc.",
    "908C43": "Apple, Inc.",
    "18D98F": "Huawei Device Co., Ltd.",
    "2811EC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00692D": "Sunnovo International Limited",
    "F4032A": "Amazon Technologies Inc.",
    "9C28F7": "Xiaomi Communications Co Ltd",
    "10521C": "Espressif Inc.",
    "803049": "Liteon Technology Corporation",
    "E42686": "DWnet Technologies(Suzhou) Corporation",
    "0C3796": "BIZLINK TECHNOLOGY, INC.",
    "38EB47": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10C3AB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "943BB0": "New H3C Technologies Co., Ltd",
    "289AF7": "ADVA Optical Networking Ltd.",
    "E84943": "YUGE Information technology Co. Ltd",
    "9043E2": "Cornami, Inc",
    "501408": "AiNET",
    "F0F0A4": "Amazon Technologies Inc.",
    "CC2D1B": "SFR",
    "B0B194": "zte corporation",
    "A02D13": "AirTies Wireless Networks",
    "3C5731": "Cisco Systems, Inc",
    "342FBD": "Nintendo Co.,Ltd",
    "6C42AB": "Subscriber Networks, Inc.",
    "64F6F7": "Anhui Dynamic Power Co., Ltd.",
    "941C56": "Actiontec Electronics, Inc",
    "80E540": "ARRIS Group, Inc.",
    "C0D682": "Arista Networks",
    "D80BCB": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "9CFCE8": "Intel Corporate",
    "8468C8": "TOTOLINK TECHNOLOGY INT\u2018L LIMITED",
    "2C91AB": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "807B3E": "Samsung Electronics Co.,Ltd",
    "F8F1E6": "Samsung Electronics Co.,Ltd",
    "88A9B7": "Apple, Inc.",
    "ECCED7": "Apple, Inc.",
    "AC9085": "Apple, Inc.",
    "A824B8": "Nokia",
    "6490C1": "Beijing Xiaomi Mobile Software Co., Ltd",
    "CC5CDE": "China Mobile Group Device Co.,Ltd.",
    "4C4FEE": "OnePlus Technology (Shenzhen) Co., Ltd",
    "60CE86": "Sercomm Corporation.",
    "0009FC": "IPFLEX Inc.",
    "48EB62": "Murata Manufacturing Co., Ltd.",
    "50DE19": "IEEE Registration Authority",
    "D005E4": "Huawei Device Co., Ltd.",
    "30AAE4": "Huawei Device Co., Ltd.",
    "5CE176": "Cisco Systems, Inc",
    "28CDC4": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "E8D8D1": "HP Inc.",
    "9055DE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E8910F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "14AB56": "WUXI FUNIDE DIGITAL CO.,LTD",
    "4CE175": "Cisco Systems, Inc",
    "50E549": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "CC5289": "SHENZHEN OPTFOCUS TECHNOLOGY.,LTD",
    "CC5D78": "JTD Consulting",
    "B0A454": "Tripwire Inc.",
    "ACEB51": "Universal Electronics, Inc.",
    "C4B239": "Cisco Systems, Inc",
    "A82BCD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48DC2D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "809F9B": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "BCF9F2": "TEKO",
    "6010A2": "Crompton Instruments",
    "349F7B": "CANON INC.",
    "34F150": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "D03110": "Ingenic Semiconductor Co.,Ltd",
    "A8E77D": "Texas Instruments",
    "DC543D": "ITEL MOBILE LIMITED",
    "08CC27": "Motorola Mobility LLC, a Lenovo Company",
    "04D395": "Motorola Mobility LLC, a Lenovo Company",
    "E09806": "Espressif Inc.",
    "F4CFA2": "Espressif Inc.",
    "2877F1": "Apple, Inc.",
    "3C510E": "Cisco Systems, Inc",
    "D8AED0": "Shanghai Engineering Science & Technology Co.,LTD CGNPC",
    "0C42A1": "Mellanox Technologies, Inc.",
    "E0859A": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "748A28": "HMD Global Oy",
    "ECA940": "ARRIS Group, Inc.",
    "FC8596": "Axonne Inc.",
    "50C6AD": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0C8447": "Fiberhome Telecommunication Technologies Co.,LTD",
    "9C6B72": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "5CB4E2": "Inspur Software Group Ltd.",
    "D42493": "GW Technologies Co.,Ltd",
    "6C410E": "Cisco Systems, Inc",
    "149138": "Amazon Technologies Inc.",
    "F81E6F": "EBG compleo GmbH",
    "6C310E": "Cisco Systems, Inc",
    "58D50A": "Murata Manufacturing Co., Ltd.",
    "14A32F": "Huawei Device Co., Ltd.",
    "04D3B5": "Huawei Device Co., Ltd.",
    "00BB1C": "Huawei Device Co., Ltd.",
    "DC4BFE": "Shenzhen Belon Technology CO.,LTD",
    "506255": "IEEE Registration Authority",
    "80AC7C": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "88A303": "Samsung Electronics Co.,Ltd",
    "FCDE90": "Samsung Electronics Co.,Ltd",
    "1854CF": "Samsung Electronics Co.,Ltd",
    "80795D": "Infinix mobility limited",
    "F0A7B2": "FUTABA CORPORATION",
    "609B2D": "JMACS Japan Co., Ltd.",
    "F0A35A": "Apple, Inc.",
    "608373": "Apple, Inc.",
    "84AD8D": "Apple, Inc.",
    "74428B": "Apple, Inc.",
    "642CAC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E884C6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C710D": "Cisco Systems, Inc",
    "00D2B1": "TPV Display Technology (Xiamen) Co.,Ltd.",
    "1C4455": "Sieb & Meyer AG",
    "E01F88": "Xiaomi Communications Co Ltd",
    "803253": "Intel Corporate",
    "C0E434": "AzureWave Technology Inc.",
    "5CE7A0": "Nokia",
    "8CDC02": "zte corporation",
    "D03D52": "Ava Security Limited",
    "246F8C": "Huawei Device Co., Ltd.",
    "1C1386": "Huawei Device Co., Ltd.",
    "BC2EF6": "Huawei Device Co., Ltd.",
    "4455C4": "Huawei Device Co., Ltd.",
    "10C65E": "Adapt-IP",
    "20311C": "vivo Mobile Communication Co., Ltd.",
    "B4E842": "Hong Kong Bouffalo Lab Limited",
    "7CA7B0": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "683943": "ittim",
    "2852F9": "Zhongxin Intelligent Times (Shenzhen) Co., Ltd.",
    "B8F853": "Arcadyan Corporation",
    "E0D083": "Samsung Electronics Co.,Ltd",
    "D80B9A": "Samsung Electronics Co.,Ltd",
    "6C1DEB": "u-blox AG",
    "287FCF": "Intel Corporate",
    "8031F0": "Samsung Electronics Co.,Ltd",
    "743C18": "Taicang T&W Electronics",
    "1088CE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8C02FA": "COMMANDO Networks Limited",
    "4C80BA": "Wuhan Tianyu Information Industry Co., Ltd.",
    "34B5A3": "CIG SHANGHAI CO LTD",
    "AC8D34": "HUAWEI TECHNOLOGIES CO.,LTD",
    "743170": "Arcadyan Technology Corporation",
    "401175": "IEEE Registration Authority",
    "84A9EA": "Career Technologies USA",
    "E405F8": "Bytedance",
    "583526": "DEEPLET TECHNOLOGY CORP",
    "283334": "Huawei Device Co., Ltd.",
    "30A2C2": "Huawei Device Co., Ltd.",
    "F47488": "New H3C Technologies Co., Ltd",
    "DCDFD6": "zte corporation",
    "ACA88E": "SHARP Corporation",
    "98415C": "Nintendo Co.,Ltd",
    "848094": "Meter, Inc.",
    "10B3D5": "Cisco Systems, Inc",
    "000084": "SUPERNET",
    "50A132": "Shenzhen MiaoMing  Intelligent Technology Co.,Ltd",
    "807871": "ASKEY COMPUTER CORP",
    "4C7A48": "Nippon Seiki (Europe) B.V.",
    "705425": "ARRIS Group, Inc.",
    "5C0BCA": "Tunstall Nordic AB",
    "F4B688": "PLANTRONICS, INC.",
    "000C86": "Cisco Systems, Inc",
    "083A88": "Universal Global Scientific Industrial Co., Ltd.",
    "08318B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CB1CD": "Ruckus Wireless",
    "F83CBF": "BOTATO ELECTRONICS SDN BHD",
    "FC589A": "Cisco Systems, Inc",
    "88517A": "HMD Global Oy",
    "ACB3B5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F49C12": "Structab AB",
    "60AB14": "LG Innotek",
    "BC62D2": "Genexis International B.V.",
    "6C9E7C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "84D15A": "TCT mobile ltd",
    "F08620": "Arcadyan Corporation",
    "DCCC8D": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "B8F653": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "446747": "HUAWEI TECHNOLOGIES CO.,LTD",
    "884A70": "Wacom Co.,Ltd.",
    "A463A1": "Inventus Power Eletronica do Brasil LTDA",
    "3C9D56": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70FD45": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44D5F2": "IEEE Registration Authority",
    "0CDD24": "Intel Corporate",
    "F05C77": "Google, Inc.",
    "001D29": "Doro AB",
    "ECA5DE": "ONYX WIFI Inc",
    "8C4962": "Roku, Inc",
    "D44BB6": "Zhejiang Tmall Technology Co., Ltd.",
    "38184C": "Sony Home Entertainment&Sound Products Inc",
    "347563": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "142E5E": "Sercomm Corporation.",
    "0025CB": "Reiner SCT",
    "44237C": "Beijing Xiaomi Mobile Software Co., Ltd",
    "309435": "vivo Mobile Communication Co., Ltd.",
    "50EB71": "Intel Corporate",
    "C4AC59": "Murata Manufacturing Co., Ltd.",
    "5816D7": "ALPSALPINE CO,.LTD",
    "FCA47A": "IEEE Registration Authority",
    "6CD71F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F06865": "Taicang T&W Electronics",
    "F4B5BB": "CERAGON NETWORKS",
    "507AC5": "Apple, Inc.",
    "4C6BE8": "Apple, Inc.",
    "8C861E": "Apple, Inc.",
    "542B8D": "Apple, Inc.",
    "000A17": "NESTAR COMMUNICATIONS, INC",
    "381A52": "Seiko Epson Corporation",
    "C064E4": "Cisco Systems, Inc",
    "D82FE6": "Zhejiang Tmall Technology Co., Ltd.",
    "E4FDA1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E419C1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B86685": "Sagemcom Broadband SAS",
    "B452A9": "Texas Instruments",
    "140F42": "Nokia",
    "006D61": "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.",
    "5C879C": "Intel Corporate",
    "24EE9A": "Intel Corporate",
    "D8AF81": "AO ",
    "D0C65B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9078B2": "Xiaomi Communications Co Ltd",
    "78C313": "China Mobile Group Device Co.,Ltd.",
    "7434AE": "this is engineering Inc.",
    "74ADB7": "China Mobile Group Device Co.,Ltd.",
    "78CC2B": "SINEWY TECHNOLOGY CO., LTD",
    "D0C857": "IEEE Registration Authority",
    "94D00D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C48A5A": "JFCONTROL",
    "B49A95": "Shenzhen Boomtech Industrial Corporation",
    "000970": "Vibration Research Corporation",
    "14A2A0": "Cisco Systems, Inc",
    "54EF44": "Lumi United Technology Co., Ltd",
    "E4AB89": "MitraStar Technology Corp.",
    "1C20DB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CFD18": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCBCD1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7460FA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C40683": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50D4F7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "402B50": "ARRIS Group, Inc.",
    "001F47": "MCS Logic Inc.",
    "B45459": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "FC492D": "Amazon Technologies Inc.",
    "74EE2A": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "080039": "SPIDER SYSTEMS LIMITED",
    "889E33": "TCT mobile ltd",
    "6C8AEC": "Nantong Coship Electronics Co., Ltd.",
    "84C2E4": "Jiangsu Qinheng Co., Ltd.",
    "8CE5C0": "Samsung Electronics Co.,Ltd",
    "F08A76": "Samsung Electronics Co.,Ltd",
    "ECAA25": "Samsung Electronics Co.,Ltd",
    "687D6B": "Samsung Electronics Co.,Ltd",
    "485169": "Samsung Electronics Co.,Ltd",
    "B4CFE0": "Sichuan tianyi kanghe communications co., LTD",
    "BC7FA4": "Xiaomi Communications Co Ltd",
    "AC83E9": "Beijing Zile Technology Co., Ltd",
    "D8CA06": "Titan DataCenters France",
    "6095CE": "IEEE Registration Authority",
    "D8F15B": "Espressif Inc.",
    "6CF17E": "Zhejiang Uniview Technologies Co.,Ltd.",
    "90473C": "China Mobile Group Device Co.,Ltd.",
    "7C21D8": "Shenzhen Think Will Communication Technology co., LTD.",
    "FCEA50": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "00E06B": "W&G SPECIAL PRODUCTS",
    "D4F057": "Nintendo Co.,Ltd",
    "488764": "vivo Mobile Communication Co., Ltd.",
    "5C1CB9": "vivo Mobile Communication Co., Ltd.",
    "C0FD84": "zte corporation",
    "444B7E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E8D0FC": "Liteon Technology Corporation",
    "E8E8B7": "Murata Manufacturing Co., Ltd.",
    "083A2F": "Guangzhou Juan Intelligent Tech Joint Stock Co.,Ltd",
    "1C3A60": "Ruckus Wireless",
    "6009C3": "u-blox AG",
    "DC8C37": "Cisco Systems, Inc",
    "103D3E": "China Mobile Group Device Co.,Ltd.",
    "64CC22": "Arcadyan Corporation",
    "4C9157": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "9C25BE": "Wildlife Acoustics, Inc.",
    "D039EA": "NetApp",
    "F8DFE1": "MyLight Systems",
    "788C77": "LEXMARK INTERNATIONAL, INC.",
    "3C0C7D": "Tiny Mesh AS",
    "3476C5": "I-O DATA DEVICE,INC.",
    "60D2DD": "Shenzhen Baitong Putian Technology Co.,Ltd.",
    "5C32C5": "Teracom Ltd.",
    "24DA33": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCAB90": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4975C": "VTech Telecommunications Ltd.",
    "1C2704": "zte corporation",
    "5078B3": "zte corporation",
    "F0D4F7": "varram system",
    "E0CC7A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B02A1F": " Wingtech Group (HongKong\uff09Limited",
    "5893D8": "Texas Instruments",
    "5051A9": "Texas Instruments",
    "F47960": "HUAWEI TECHNOLOGIES CO.,LTD",
    "145290": "KNS Group LLC (YADRO Company)",
    "6C23CB": "Wattty Corporation",
    "60AB67": "Xiaomi Communications Co Ltd",
    "AC710C": "China Mobile Group Device Co.,Ltd.",
    "A8DB03": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "308944": "DEVA Broadcast Ltd.",
    "00B600": "VOIM Co., Ltd.",
    "002615": "Teracom Limited",
    "9C8EDC": "Teracom Limited",
    "000191": "SYRED Data Systems",
    "4889E7": "Intel Corporate",
    "A0BD1D": "Zhejiang Dahua Technology Co., Ltd.",
    "B4D0A9": "China Mobile Group Device Co.,Ltd.",
    "4846C1": "FN-LINK TECHNOLOGY LIMITED",
    "ACE2D3": "Hewlett Packard",
    "00FD22": "Cisco Systems, Inc",
    "ACEE70": "Fontem Ventures BV",
    "4418FD": "Apple, Inc.",
    "005B94": "Apple, Inc.",
    "E0897E": "Apple, Inc.",
    "2CCCE6": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "CC2C83": "DarkMatter L.L.C",
    "DCED84": "Haverford Systems Inc",
    "644C36": "Intel Corporate",
    "ACD564": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "94D075": "CIS Crypto",
    "28B4FB": "Sprocomm Technologies CO.,LTD.",
    "C05336": "Beijing National Railway Research & Design Institute of Signal & Communication Group Co..Ltd.",
    "606ED0": "SEAL AG",
    "40F9D5": "Tecore Networks",
    "E44CC7": "IEEE Registration Authority",
    "D4E880": "Cisco Systems, Inc",
    "3C20F6": "Samsung Electronics Co.,Ltd",
    "7C38AD": "Samsung Electronics Co.,Ltd",
    "2C01B5": "Cisco Systems, Inc",
    "A8346A": "Samsung Electronics Co.,Ltd",
    "FC29F3": "McPay Co.,LTD.",
    "F8AFDB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "688B0F": "China Mobile IOT Company Limited",
    "E49F1E": "ARRIS Group, Inc.",
    "3CF011": "Intel Corporate",
    "D49CDD": "AMPAK Technology,Inc.",
    "50F722": "Cisco Systems, Inc",
    "9809CF": "OnePlus Technology (Shenzhen) Co., Ltd",
    "B8DE5E": "LONGCHEER TELECOMMUNICATION LIMITED",
    "9849E1": "Boeing Defence Australia",
    "04D7A5": "New H3C Technologies Co., Ltd",
    "885A06": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "5447D3": "TSAT AS",
    "4C4D66": "Nanjing Jiahao Technology Co., Ltd.",
    "A4817A": "CIG SHANGHAI CO LTD",
    "CCEDDC": "MitraStar Technology Corp.",
    "20B780": "Toshiba Visual Solutions Corporation Co.,Ltd",
    "A4E7E4": "Connex GmbH",
    "F0B31E": "Universal Electronics, Inc.",
    "ECA9FA": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "0003A5": "Medea Corporation",
    "BCE67C": "Cambium Networks Limited",
    "7C1E06": "New H3C Technologies Co., Ltd",
    "18DFB4": "BOSUNG POWERTEC CO.,LTD.",
    "F89173": "AEDLE SAS",
    "B8EF8B": "SHENZHEN CANNICE TECHNOLOGY CO.,LTD",
    "B8186F": "ORIENTAL MOTOR CO., LTD.",
    "001A3F": "Intelbras",
    "544741": "XCHENG HOLDING",
    "CCF735": "Amazon Technologies Inc.",
    "C4F839": "Actia Automotive",
    "C84F86": "Sophos Ltd",
    "6429ED": "AO \"PKK Milandr\"",
    "F82F6A": "ITEL MOBILE LIMITED",
    "C0D834": "xvtec ltd",
    "14D4FE": "ARRIS Group, Inc.",
    "443C88": "FICOSA MAROC INTERNATIONAL",
    "841C70": "zte corporation",
    "78DD12": "Arcadyan Corporation",
    "D425CC": "IEEE Registration Authority",
    "8C6DC4": "Megapixel VR",
    "C8F742": "HangZhou Gubei Electronics Technology Co.,Ltd",
    "006FF2": "MITSUMI ELECTRIC CO.,LTD.",
    "30DF8D": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "D4C93C": "Cisco Systems, Inc",
    "2C5D34": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "9C1463": "Zhejiang Dahua Technology Co., Ltd.",
    "E0750A": "ALPSALPINE CO,.LTD",
    "0019C1": "ALPSALPINE CO,.LTD",
    "0016FE": "ALPSALPINE CO,.LTD",
    "9C8D7C": "ALPSALPINE CO,.LTD",
    "002433": "ALPSALPINE CO,.LTD",
    "002306": "ALPSALPINE CO,.LTD",
    "B4EC02": "ALPSALPINE CO,.LTD",
    "CCD39D": "IEEE Registration Authority",
    "BC7536": "ALPSALPINE CO,.LTD",
    "E0AE5E": "ALPSALPINE CO,.LTD",
    "D4B761": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "7C035E": "Xiaomi Communications Co Ltd",
    "44FE3B": "Arcadyan Corporation",
    "D83AF5": "Wideband Labs LLC",
    "38D9A5": "Mikotek Information Inc. ",
    "646038": "Hirschmann Automation and Control GmbH",
    "649D99": "FS COM INC",
    "00169D": "Cisco Systems, Inc",
    "4C875D": "Bose Corporation",
    "B0E7DE": "Homa Technologies JSC",
    "00D279": "VINGROUP JOINT STOCK COMPANY",
    "00004C": "NEC Corporation",
    "8CCF8F": "ITC Systems",
    "4C962D": "Fresh AB",
    "484A30": "George Robotics Limited",
    "4861A3": "Concern \"Axion\" JSC",
    "4CE5AE": "Tianjin Beebox Intelligent Technology Co.,Ltd.",
    "F4C7C8": "Kelvin Inc.",
    "304A26": "Shenzhen Trolink Technology CO, LTD",
    "D467D3": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A41232": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "48E3C3": "JENOPTIK Advanced Systems GmbH",
    "CC355A": "SecuGen Corporation",
    "80546A": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "B447F5": "Earda Technologies co Ltd",
    "1CF29A": "Google, Inc.",
    "748A0D": "ARRIS Group, Inc.",
    "CC75E2": "ARRIS Group, Inc.",
    "A89CA4": "Furrion Limited",
    "2875D8": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "90E202": "Texas Instruments",
    "302478": "Sagemcom Broadband SAS",
    "184BDF": "Caavo Inc",
    "FC7774": "Intel Corporate",
    "700B4F": "Cisco Systems, Inc",
    "E4388C": "Digital Products Limited",
    "4455B1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98F9C7": "IEEE Registration Authority",
    "A40C66": "Shenzhen Colorful Yugong Technology and Development Co., Ltd.",
    "84A93E": "Hewlett Packard",
    "349342": "TTE Corporation",
    "589EC6": "Gigaset Communications GmbH",
    "B0C387": "GOEFER, Inc.",
    "086BD7": "Silicon Laboratories",
    "6458AD": "China Mobile IOT Company Limited",
    "A0A3B8": "WISCLOUD",
    "38F9D3": "Apple, Inc.",
    "FC183C": "Apple, Inc.",
    "64C753": "Apple, Inc.",
    "201F31": "Inteno Broadband Technology AB",
    "6C2CDC": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "7835A0": "Zurn Industries LLC",
    "8C7BF0": "Xufeng Development Limited",
    "E0A509": "Bitmain Technologies Inc",
    "3C5CC4": "Amazon Technologies Inc.",
    "D8A756": "Sagemcom Broadband SAS",
    "54833A": "Zyxel Communications Corporation",
    "F43909": "Hewlett Packard",
    "B89A9A": "Xin Shi Jia Technology (Beijing) Co.,Ltd",
    "2CCC44": "Sony Interactive Entertainment Inc.",
    "F47DEF": "Samsung Electronics Co.,Ltd",
    "7C8BB5": "Samsung Electronics Co.,Ltd",
    "D8A98B": "Texas Instruments",
    "D8D6F3": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "787052": "Welotec GmbH",
    "14EFCF": "SCHREDER",
    "3830F9": "LG Electronics (Mobile Communications)",
    "00F48D": "Liteon Technology Corporation",
    "702ED9": "Guangzhou Shiyuan Electronics Co., Ltd.",
    "70192F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A83FA1": "IEEE Registration Authority",
    "F4DBE6": "Cisco Systems, Inc",
    "10B9F7": "Niko-Servodan",
    "18A7F1": "Qingdao Haier Technology Co.,Ltd",
    "90E17B": "Apple, Inc.",
    "D81C79": "Apple, Inc.",
    "58E6BA": "Apple, Inc.",
    "248498": "Beijing Jiaoda Microunion Tech.Co.,Ltd.",
    "6C9BC0": "Chemoptics Inc.",
    "04EB40": "Cisco Systems, Inc",
    "1801F1": "Xiaomi Communications Co Ltd",
    "C074AD": "Grandstream Networks, Inc.",
    "F095F1": "Carl Zeiss AG",
    "C0847D": "AMPAK Technology, Inc.",
    "44E4EE": "Wistron Neweb Corporation",
    "DC41E5": "Shenzhen Zhixin Data Service Co., Ltd.",
    "10C22F": "China Entropy Co., Ltd.",
    "BC3865": "JWCNETWORKS",
    "0409A5": "HFR, Inc.",
    "94917F": "ASKEY COMPUTER CORP",
    "9C0CDF": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "7CD30A": "INVENTEC CORPORATION",
    "001E33": "INVENTEC CORPORATION",
    "FC1D84": "Autobase",
    "00A5BF": "Cisco Systems, Inc",
    "C8BAE9": "QDIS",
    "C44F33": "Espressif Inc.",
    "546AD8": "Elster Water Metering",
    "002654": "3COM",
    "0050DA": "3COM",
    "000476": "3COM",
    "000475": "3COM",
    "00D096": "3COM EUROPE LTD",
    "3009F9": "IEEE Registration Authority",
    "B4DDD0": "Continental Automotive Hungary Kft",
    "48F027": "Chengdu newifi Co.,Ltd",
    "B42E99": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "342CC4": "Compal Broadband Networks, Inc.",
    "DC16B2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24FB65": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0CB527": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C03AB": "Xiaomi Communications Co Ltd",
    "C8544B": "Zyxel Communications Corporation",
    "14C697": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "242124": "Nokia",
    "18AC9E": "ITEL MOBILE LIMITED",
    "EC84B4": "CIG SHANGHAI CO LTD",
    "4422F1": "S.FAC, INC",
    "78524A": "Optonic GmbH",
    "C46E7B": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "C048FB": "Shenzhen JingHanDa Electronics Co.Ltd",
    "20E882": "zte corporation",
    "4C1159": "Vision Information & Communications",
    "00B5D0": "Samsung Electronics Co.,Ltd",
    "1496E5": "Samsung Electronics Co.,Ltd",
    "D07FA0": "Samsung Electronics Co.,Ltd",
    "009093": "EIZO Corporation",
    "F09FFC": "SHARP Corporation",
    "0CB5DE": "Alcatel Lucent",
    "50DCFC": "ECOCOM",
    "000B3B": "devolo AG",
    "240588": "Google, Inc.",
    "00049F": "Freescale Semiconductor",
    "00D07B": "COMCAM INTERNATIONAL INC",
    "14E9B2": "Fiberhome Telecommunication Technologies Co.,LTD",
    "700B01": "Sagemcom Broadband SAS",
    "5C2623": "WaveLynx Technologies Corporation",
    "303855": "Nokia Corporation",
    "00B670": "Cisco Systems, Inc",
    "AC6417": "Siemens AG",
    "EC6F0B": "FADU, Inc.",
    "3466EA": "VERTU INTERNATIONAL CORPORATION LIMITED",
    "28385C": "FLEXTRONICS",
    "00BB60": "Intel Corporate",
    "7C6DA6": "Superwave Group LLC",
    "B870F4": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "1C3947": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "342792": "FREEBOX SAS",
    "883F99": "Siemens AG",
    "705AB6": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "201A06": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "F8A963": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "DC0EA1": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "0006EC": "Harris Corporation",
    "0C1C57": "Texas Instruments",
    "806FB0": "Texas Instruments",
    "347916": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D016B4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0F08F": "Nextek Solutions Pte Ltd",
    "8CB0E9": "Samsung Electronics.,LTD",
    "A46191": "NamJunSa",
    "40A108": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "745933": "Danal Entertainment",
    "EC58EA": "Ruckus Wireless",
    "84A24D": "Birds Eye Systems Private Limited",
    "7C6B9C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "009D6B": "Murata Manufacturing Co., Ltd.",
    "D45251": "IBT Ingenieurbureau Broennimann Thun",
    "0018B5": "Magna Carta",
    "D87EB1": "x.o.ware, inc.",
    "20A8B9": "SIEMENS AG",
    "D0EFC1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "485702": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0017B6": "Aquantia Corporation",
    "105917": "Tonal",
    "388B59": "Google, Inc.",
    "880118": "BLT Co",
    "A42618": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "34E12D": "Intel Corporate",
    "DCAF68": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "882D53": "Baidu Online Network Technology (Beijing) Co., Ltd.",
    "644F42": "JETTER CO., Ltd.",
    "2C97B1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24EC51": "ADF Technologies Sdn Bhd",
    "7089CC": "China Mobile Group Device Co.,Ltd.",
    "2C5BE1": "Centripetal Networks, Inc",
    "DCEFCA": "Murata Manufacturing Co., Ltd.",
    "00BC60": "Cisco Systems, Inc",
    "00D0B5": "IPricot formerly DotCom",
    "CC7B61": "NIKKISO CO., LTD.",
    "9C713A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8B7F1": "Wistron Neweb Corporation",
    "8050F6": "ITEL MOBILE LIMITED",
    "A8CAB9": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "5061BF": "Cisco Systems, Inc",
    "0009DF": "Vestel Elektronik San ve Tic. A.S.",
    "000FA2": "2xWireless",
    "746BAB": "GUANGDONG ENOK COMMUNICATION CO., LTD",
    "0CB6D2": "D-Link International",
    "7829ED": "ASKEY COMPUTER CORP",
    "000BB2": "SMALLBIG TECHNOLOGY",
    "F4032F": "Reduxio Systems",
    "944A0C": "Sercomm Corporation.",
    "700F6A": "Cisco Systems, Inc",
    "108EE0": "Samsung Electronics Co.,Ltd",
    "FCA621": "Samsung Electronics Co.,Ltd",
    "8CF228": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "80029C": "Gemtek Technology Co., Ltd.",
    "E8D099": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00305E": "Abelko Innovation",
    "FC6BF0": "TOPWELL INTERNATIONAL HOLDINDS LIMITED",
    "001477": "Trilliant",
    "00079B": "Aurora Networks",
    "54B203": "PEGATRON CORPORATION",
    "3868DD": "INVENTEC CORPORATION",
    "3C6AA7": "Intel Corporate",
    "A028ED": "HMD Global Oy",
    "D0C5D3": "AzureWave Technology Inc.",
    "14169E": " Wingtech Group (HongKong\uff09Limited",
    "107BA4": "Olive & Dove Co.,Ltd.",
    "203956": "HMD Global Oy",
    "78AFE4": "Comau S.p.A",
    "90A137": "Beijing Splendidtel Communication Technology Co,. Ltd",
    "BC325F": "Zhejiang Dahua Technology Co., Ltd.",
    "AC5474": "China Mobile IOT Company Limited",
    "8C1CDA": "IEEE Registration Authority",
    "7C41A2": "Nokia",
    "F8C39E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "505BC2": "Liteon Technology Corporation",
    "6C21A2": "AMPAK Technology, Inc.",
    "9C2F73": "Universal Tiancheng Technology (Beijing) Co., Ltd.",
    "D832E3": "Xiaomi Communications Co Ltd",
    "9487E0": "Xiaomi Communications Co Ltd",
    "38AF29": "Zhejiang Dahua Technology Co., Ltd.",
    "300AC5": "Ruio telecommunication technologies Co., Limited",
    "00E065": "OPTICAL ACCESS INTERNATIONAL",
    "C88629": "Shenzhen Duubee Intelligent Technologies Co.,LTD.",
    "CCC2E0": "Raisecom Technology CO., LTD",
    "883F4A": "Texas Instruments",
    "D88A3B": "UNIT-EM",
    "EC5A86": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "C4BAA3": "Beijing Winicssec Technologies Co., Ltd.",
    "9CFEA1": "Fiberhome Telecommunication Technologies Co.,LTD",
    "4466FC": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "50A009": "Xiaomi Communications Co Ltd",
    "001DFA": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "9814D2": "Avonic",
    "08F69C": "Apple, Inc.",
    "04FA83": "Qingdao Haier Technology Co.,Ltd",
    "78F9B4": "Nokia Solutions and Networks GmbH & Co. KG",
    "2016B9": "Intel Corporate",
    "00165C": "Trackflow Ltd.",
    "50A67F": "Apple, Inc.",
    "60DEF3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1409DC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38BAF8": "Intel Corporate",
    "0007A8": "Haier Group Technologies Ltd",
    "14205E": "Apple, Inc.",
    "B841A4": "Apple, Inc.",
    "9CE65E": "Apple, Inc.",
    "C49880": "Apple, Inc.",
    "E0338E": "Apple, Inc.",
    "88964E": "ARRIS Group, Inc.",
    "C0EEB5": "Enice Network.",
    "D461DA": "Apple, Inc.",
    "F01898": "Apple, Inc.",
    "881908": "Apple, Inc.",
    "5C0947": "Apple, Inc.",
    "641CAE": "Samsung Electronics Co.,Ltd",
    "F8E44E": "MCOT INC.",
    "40CD7A": "Qingdao Hisense Communications Co.,Ltd.",
    "004098": "DRESSLER GMBH & CO.",
    "949990": "VTC Telecommunications",
    "F4BC97": "Shenzhen Crave Communication Co., LTD",
    "28FEDE": "COMESTA, Inc.",
    "907910": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "003DE8": "LG Electronics (Mobile Communications)",
    "68FEDA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D076E7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "9CA615": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "E44E76": "CHAMPIONTECH  ENTERPRISE (SHENZHEN) INC",
    "DC4EF4": "Shenzhen MTN Electronics CO., Ltd",
    "F08173": "Amazon Technologies Inc.",
    "00508B": "Hewlett Packard",
    "146B9C": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "948DEF": "Oetiker Schweiz AG",
    "144E34": "Remote Solution",
    "3403DE": "Texas Instruments",
    "34D712": "Smartisan Digital Co., Ltd",
    "A06610": "FUJITSU LIMITED",
    "44FFBA": "zte corporation",
    "2CD974": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "D4F786": "Fiberhome Telecommunication Technologies Co.,LTD",
    "94B86D": "Intel Corporate",
    "240A63": "ARRIS Group, Inc.",
    "8817A3": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "88A9A7": "IEEE Registration Authority",
    "F88B37": "ARRIS Group, Inc.",
    "6084BD": "BUFFALO.INC",
    "347ECA": "NEXTWILL",
    "EC8914": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B89436": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E0E62E": "TCT mobile ltd",
    "B42EF8": "Eline Technology co.Ltd",
    "A4D4B2": "Shenzhen MeiG Smart Technology Co.,Ltd",
    "8CF773": "Nokia",
    "F4C248": "Samsung Electronics Co.,Ltd",
    "F47190": "Samsung Electronics Co.,Ltd",
    "2802D8": "Samsung Electronics Co.,Ltd",
    "DCE533": "IEEE Registration Authority",
    "D8445C": "DEV Tecnologia Ind Com Man Eq LTDA",
    "7C3953": "zte corporation",
    "38E1AA": "zte corporation",
    "48C796": "Samsung Electronics Co.,Ltd",
    "501479": "iRobot Corporation ",
    "E42D7B": "China Mobile IOT Company Limited",
    "387862": "Sony Corporation",
    "4CEFC0": "Amazon Technologies Inc.",
    "C464E3": "Texas Instruments",
    "CC3B58": "Curiouser Products Inc",
    "DCDD24": "Energica Motor Company SpA",
    "641CB0": "Samsung Electronics Co.,Ltd",
    "903A72": "Ruckus Wireless",
    "C4FFBC": "IEEE Registration Authority",
    "283B82": "D-Link International",
    "509551": "ARRIS Group, Inc.",
    "804126": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ACF970": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58D759": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C5973": "Zyxel Communications Corporation",
    "24181D": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "F89066": "Nain Inc.",
    "7006AC": "Eastcompeace Technology Co., Ltd",
    "6C54CD": "LAMPEX ELECTRONICS LIMITED",
    "94290C": "Shenyang wisdom Foundation Technology Development Co., Ltd.",
    "000889": "Dish Technologies Corp",
    "04C9D9": "Dish Technologies Corp",
    "7055F8": "Cerebras Systems Inc",
    "580454": "ICOMM HK LIMITED",
    "C477AF": "Advanced Digital Broadcast SA",
    "D4909C": "Apple, Inc.",
    "E4E0A6": "Apple, Inc.",
    "5C0038": "Viasat Group S.p.A.",
    "9C32CE": "CANON INC.",
    "802BF9": "Hon Hai Precision Ind. Co.,Ltd.",
    "54B802": "Samsung Electronics Co.,Ltd",
    "F0B5B7": "Disruptive Technologies Research AS",
    "B4DEDF": "zte corporation",
    "805E0C": "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD.",
    "6C49C1": "o2ones Co., Ltd.",
    "34415D": "Intel Corporate",
    "005091": "NETACCESS, INC.",
    "70EEA3": "Eoptolink Technology Inc. Ltd,",
    "7047E9": "vivo Mobile Communication Co., Ltd.",
    "5C521E": "Nintendo Co.,Ltd",
    "3C2C99": "Edgecore Networks Corporation",
    "10CEA9": "Texas Instruments",
    "88D039": "Tonly Technology Co. Ltd ",
    "20E09C": "Nokia",
    "2CFDA1": "ASUSTek COMPUTER INC.",
    "3807D4": "Zeppelin Systems GmbH",
    "04197F": "Grasphere Japan",
    "14444A": "Apollo Seiko Ltd.",
    "5C81A7": "Network Devices Pty Ltd",
    "5C0C0E": "Guizhou Huaxintong Semiconductor Technology Co Ltd",
    "5C86C1": "DONGGUAN SOLUM ELECTRONICS CO.,LTD",
    "38437D": "Compal Broadband Networks, Inc.",
    "506F98": "Sehaj Synergy Technologies Private Limited",
    "4CAE1C": "SaiNXT Technologies LLP",
    "142882": "MIDICOM ELECTRONICS CO.LTD",
    "30E48E": "Vodafone UK",
    "683E02": "SIEMENS AG, Digital Factory, Motion Control System",
    "000261": "Tilgin AB",
    "0014C3": "Seagate Technology",
    "0004CF": "Seagate Technology",
    "002037": "Seagate Technology",
    "EC8193": "Logitech, Inc",
    "6CDD30": "Cisco Systems, Inc",
    "6C4E86": "Third Millennium Systems Ltd.",
    "FC9DD8": "Beijing TongTongYiLian Science and Technology Ltd.",
    "503CEA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "DC2834": "HAKKO Corporation",
    "001730": "Automation Electronics",
    "00E091": "LG Electronics",
    "DCE1AD": "Shenzhen Wintop Photoelectric Technology Co., Ltd",
    "948854": "Texas Instruments",
    "001D0D": "Sony Interactive Entertainment Inc.",
    "ECB0E1": "Ciena Corporation",
    "449160": "Murata Manufacturing Co., Ltd.",
    "B4F1DA": "LG Electronics (Mobile Communications)",
    "C863F1": "Sony Interactive Entertainment Inc.",
    "5C7776": "TCT mobile ltd",
    "70E56E": "Texas Instruments",
    "547DCD": "Texas Instruments",
    "0019C2": "Equustek Solutions, Inc.",
    "80000B": "Intel Corporate",
    "00AECD": "Pensando Systems",
    "84509A": "Easy Soft TV Co., Ltd",
    "6C05D5": "Ethertronics Inc",
    "F8B7E2": "Cisco Systems, Inc",
    "F82055": "Green Information System",
    "ACA667": "Electronic Systems Protection, Inc.",
    "000097": "Dell EMC",
    "8CCF09": "Dell EMC",
    "8C839D": "SHENZHEN XINYUPENG ELECTRONIC TECHNOLOGY CO., LTD",
    "B0FC36": "CyberTAN Technology Inc.",
    "001DF4": "Magellan Technology Pty Limited",
    "0081F9": "Texas Instruments",
    "ECB5FA": "Philips Lighting BV",
    "44CD0E": "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD.",
    "E8825B": "ARRIS Group, Inc.",
    "74E19A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "001CAE": "WiChorus, Inc.",
    "80B03D": "Apple, Inc.",
    "E49ADC": "Apple, Inc.",
    "ACE4B5": "Apple, Inc.",
    "D0D2B0": "Apple, Inc.",
    "B0C19E": "zte corporation",
    "0C3747": "zte corporation",
    "78DDD9": "Guangzhou Shiyuan Electronics Co., Ltd. ",
    "E8DF70": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "D05995": "Fiberhome Telecommunication Technologies Co.,LTD",
    "18CC88": "Hitachi Johnson Controls Air ",
    "7CDD76": "Suzhou Hanming Technologies Co., Ltd.",
    "246880": "Braveridge.co.,ltd.",
    "A8D3C8": "Topcon Electronics GmbH & Co. KG",
    "F06D78": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "707D95": "Shenzhen City LinwlanTechnology Co. Ltd.",
    "28AD3E": "Shenzhen TONG BO WEI Technology CO.,LTD",
    "70991C": "Shenzhen Honesty Electronics Co.,Ltd",
    "001C56": "Pado Systems, Inc.",
    "80C755": "Panasonic Appliances Company",
    "1878D4": "Verizon ",
    "F0BD2E": "H+S Polatis Ltd",
    "746EE4": "Asia Vital Components Co.,Ltd.",
    "2C431A": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "70708B": "Cisco Systems, Inc",
    "389F5A": "C-Kur TV Inc.",
    "D843ED": "Suzuken",
    "38A6CE": "SKY UK LIMITED",
    "0040E4": "E-M TECHNOLOGY, INC.",
    "984B4A": "ARRIS Group, Inc.",
    "E084F3": "High Grade Controls Corporation",
    "0C62A6": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "18132D": "zte corporation",
    "BC4101": "Shenzhen TINNO Mobile Technology Corp.",
    "043A0D": "SM Optics S.r.l.",
    "44EAD8": "Texas Instruments",
    "448F17": "Samsung Electronics Co., Ltd. ARTIK",
    "00FC8B": "Amazon Technologies Inc.",
    "0076B1": "Somfy-Protect By Myfox SAS",
    "C0742B": "SHENZHEN XUNLONG SOFTWARE CO.,LIMITED",
    "74D21D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3890A5": "Cisco Systems, Inc",
    "6CC147": "Xiamen Hanin Electronic Technology Co., Ltd",
    "A072E4": "NJ SYSTEM CO.,LTD",
    "4C1365": "Emplus Technologies",
    "CCF957": "u-blox AG",
    "5C6776": "IDS Imaging Development Systems GmbH",
    "1CDDEA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "B8D94D": "Sagemcom Broadband SAS",
    "189BA5": "IEEE Registration Authority",
    "A491B1": "Technicolor Delivery Technologies Belgium NV",
    "1C7022": "Murata Manufacturing Co., Ltd.",
    "CC9891": "Cisco Systems, Inc",
    "28BF89": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8C4500": "Murata Manufacturing Co., Ltd.",
    "A43412": "Thales Alenia Space",
    "681F40": "Blu Wireless Technology Ltd",
    "90ADF7": "vivo Mobile Communication Co., Ltd.",
    "ECFA03": "FCA",
    "002294": "KYOCERA CORPORATION ",
    "3889DC": "Opticon Sensors Europe B.V.",
    "903DBD": "SECURE METERS LIMITED",
    "FC017C": "Hon Hai Precision Ind. Co.,Ltd.",
    "90324B": "Hon Hai Precision Ind. Co.,Ltd.",
    "940006": "jinyoung",
    "6C96CF": "Apple, Inc.",
    "78886D": "Apple, Inc.",
    "74E5F9": "Intel Corporate",
    "40CE24": "Cisco Systems, Inc",
    "3432E6": "Panasonic Industrial Devices Europe GmbH",
    "40017A": "Cisco Systems, Inc",
    "38CD07": "Beijing FaceCam Technology Co., Ltd.",
    "20EE28": "Apple, Inc.",
    "B4F61C": "Apple, Inc.",
    "08F4AB": "Apple, Inc.",
    "8C8590": "Apple, Inc.",
    "BC88C3": "Ningbo Dooya Mechanic & Electronic Technology Co., Ltd",
    "00127D": "MobileAria",
    "00D060": "Panasonic Europe Ltd.",
    "A08869": "Intel Corporate",
    "58A0CB": "TrackNet, Inc",
    "0C4B54": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "74819A": "PT. Hartono Istana Teknologi",
    "A82BB5": "Edgecore Networks Corporation",
    "283545": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "602E20": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E472E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18B81F": "ARRIS Group, Inc.",
    "94E36D": "Texas Instruments",
    "F0F8F2": "Texas Instruments",
    "341513": "Texas Instruments",
    "E06089": "Cloudleaf, Inc.",
    "783690": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "508F4C": "Xiaomi Communications Co Ltd",
    "A47758": "Ningbo Freewings Technologies Co.,Ltd",
    "60F677": "Intel Corporate",
    "E8E1E2": "Energotest",
    "7811DC": "XIAOMI Electronics,CO.,LTD",
    "D463C6": "Motorola Mobility LLC, a Lenovo Company",
    "C444A0": "Cisco Systems, Inc",
    "18742E": "Amazon Technologies Inc.",
    "F844E3": "Taicang T&W Electronics",
    "24A534": "SynTrust Tech International Ltd. ",
    "001D2E": "Ruckus Wireless",
    "044F4C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C151F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "008BFC": "mixi,Inc.",
    "90A365": "HMD Global Oy",
    "DC44B6": "Samsung Electronics Co.,Ltd",
    "1007B6": "Samsung Electronics Co.,Ltd",
    "342D0D": "Samsung Electronics Co.,Ltd",
    "D837BE": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "001392": "Ruckus Wireless",
    "D4684D": "Ruckus Wireless",
    "8C0C90": "Ruckus Wireless",
    "6CAAB3": "Ruckus Wireless",
    "C08ADE": "Ruckus Wireless",
    "085114": "QINGDAO TOPSCOMM COMMUNICATION CO., LTD",
    "70788B": "vivo Mobile Communication Co., Ltd.",
    "4859A4": "zte corporation",
    "70F11C": "Shenzhen Ogemray Technology Co.,Ltd",
    "7065A3": "Kandao lightforge Co., Ltd.",
    "74D0DC": "Ericsson AB",
    "54BD79": "Samsung Electronics Co.,Ltd",
    "AC9E17": "ASUSTek COMPUTER INC.",
    "641666": "Nest Labs Inc.",
    "EC42B4": "ADC Corporation",
    "60DA83": "Hangzhou H3C Technologies Co., Limited",
    "2C5731": " Wingtech Group (HongKong\uff09Limited",
    "CC4639": "WAAV, Inc.",
    "A0423F": "Tyan Computer Corp",
    "D8DF7A": "Quest Software, Inc.",
    "E4A749": "Palo Alto Networks",
    "A86B7C": "SHENZHEN FENGLIAN TECHNOLOGY CO., LTD.",
    "B03956": "NETGEAR",
    "B4E62A": "LG Innotek",
    "A0C5F2": "IEEE Registration Authority",
    "8C147D": "IEEE Registration Authority",
    "A0AFBD": "Intel Corporate",
    "A0341B": "Adero Inc",
    "A0239F": "Cisco Systems, Inc",
    "70F35A": "Cisco Systems, Inc",
    "FC4DD4": "Universal Global Scientific Industrial Co., Ltd.",
    "145BE1": "nyantec GmbH",
    "30074D": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "00180A": "Cisco Meraki",
    "88D50C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "D428D5": "TCT mobile ltd",
    "7C8BCA": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B04E26": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B089C2": "Zyptonite",
    "F023B9": "IEEE Registration Authority",
    "1C1FD4": "LifeBEAM Technologies LTD",
    "E83935": "Hewlett Packard",
    "A4F4C2": "VNPT TECHNOLOGY",
    "009AD2": "Cisco Systems, Inc",
    "947BE7": "Samsung Electronics Co.,Ltd",
    "5092B9": "Samsung Electronics Co.,Ltd",
    "DC74A8": "Samsung Electronics Co.,Ltd",
    "5C6A80": "Zyxel Communications Corporation",
    "D860B3": "Guangdong Global Electronic Technology CO.\uff0cLTD",
    "64351C": "e-CON SYSTEMS INDIA PVT LTD",
    "605317": "Sandstone Technologies",
    "9CAF6F": "ITEL MOBILE LIMITED",
    "FC539E": "Shanghai Wind Technologies Co.,Ltd",
    "A8D579": "Beijing Chushang Science and Technology Co.,Ltd",
    "84CD62": "ShenZhen IDWELL Technology CO.,Ltd",
    "907065": "Texas Instruments",
    "481063": "NTT Innovation Institute, Inc.",
    "A08E78": "Sagemcom Broadband SAS",
    "60BA18": "nextLAP GmbH",
    "001FA4": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "C4AE12": "Samsung Electronics Co.,Ltd",
    "18A958": "PROVISION THAI CO., LTD.",
    "94D299": "Techmation Co.,Ltd.",
    "488803": "ManTechnology Inc.",
    "B436E3": "KBVISION GROUP",
    "6C4B90": "LiteON",
    "C83A6B": "Roku, Inc",
    "B4C6F8": "Axilspot Communication",
    "9CE951": "Shenzhen Sang Fei Consumer Communications Ltd., Co.",
    "74C9A3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "EC8A4C": "zte corporation",
    "B8D50B": "Sunitec Enterprise Co.,Ltd",
    "BC66DE": "Shadow Creator Information Technology Co.,Ltd.",
    "2CABEB": "Cisco Systems, Inc",
    "58D9D5": "Tenda Technology Co.,Ltd.Dongguan branch",
    "D45F25": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "40C8CB": "AM Telecom co., Ltd.",
    "64D154": "Routerboard.com",
    "28FF3E": "zte corporation",
    "B8D7AF": "Murata Manufacturing Co., Ltd.",
    "D4AE05": "Samsung Electronics Co.,Ltd",
    "F0EE10": "Samsung Electronics Co.,Ltd",
    "E048AF": "Premietech Limited",
    "2C3311": "Cisco Systems, Inc",
    "5082D5": "Apple, Inc.",
    "341A35": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2C029F": "3ALogics",
    "00050F": "Tanaka S/S Ltd.",
    "989E63": "Apple, Inc.",
    "886B6E": "Apple, Inc.",
    "D4DCCD": "Apple, Inc.",
    "484BAA": "Apple, Inc.",
    "DCA904": "Apple, Inc.",
    "6CAB31": "Apple, Inc.",
    "4C74BF": "Apple, Inc.",
    "04946B": "TECNO MOBILE LIMITED",
    "A04C5B": "Shenzhen TINNO Mobile Technology Corp.",
    "F85971": "Intel Corporate",
    "F4E4AD": "zte corporation",
    "BC024A": "HMD Global Oy",
    "949901": "Shenzhen YITOA Digital Appliance CO.,LTD",
    "1005CA": "Cisco Systems, Inc",
    "7894B4": "Sercomm Corporation.",
    "C4700B": "GUANGZHOU CHIP TECHNOLOGIES CO.,LTD",
    "443708": "MRV Comunications",
    "285F2F": "RNware Co.,Ltd.",
    "3CA067": "Liteon Technology Corporation",
    "1C1EE3": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "2C55D3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DCC64B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "043389": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B81DAA": "LG Electronics (Mobile Communications)",
    "00E400": "Sichuan Changhong Electric Ltd.",
    "542F8A": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "6014B3": "CyberTAN Technology Inc.",
    "105611": "ARRIS Group, Inc.",
    "44032C": "Intel Corporate",
    "500FF5": "Tenda Technology Co.,Ltd.Dongguan branch",
    "BC452E": "Knowledge Development for POF S.L.",
    "706DEC": "Wifi-soft LLC",
    "B0C205": "BIONIME",
    "94F551": "Cadi Scientific Pte Ltd",
    "00A068": "BHP LIMITED",
    "703ACB": "Google, Inc.",
    "105AF7": "ADB Italia ",
    "00C024": "EDEN SISTEMAS DE COMPUTACAO SA",
    "7C4685": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "1C398A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00179B": "CHANT SINCERE CO.,LTD",
    "0823B2": "vivo Mobile Communication Co., Ltd.",
    "88C3B3": "SOVICO",
    "E05124": "NXP Semiconductors",
    "6C160E": "ShotTracker",
    "803A0A": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "A462DF": "DS Global. Co., LTD",
    "08CCA7": "Cisco Systems, Inc",
    "0896AD": "Cisco Systems, Inc",
    "347877": "O-Net Communications (Shenzhen) Limited",
    "0020CC": "DIGITAL SERVICES, LTD.",
    "285261": "Cisco Systems, Inc",
    "286F7F": "Cisco Systems, Inc",
    "F0C850": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001DA3": "SabiOso",
    "689FF0": "zte corporation",
    "5CAF06": "LG Electronics (Mobile Communications)",
    "4C1694": "shenzhen sibituo Technology Co., Ltd",
    "00014F": "Adtran Inc",
    "7868F7": "YSTen Technology Co.,Ltd",
    "C81451": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44D437": "Inteno Broadband Technology AB",
    "ECE154": "Beijing Unisound Information Technology Co.,Ltd.",
    "E865D4": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24D51C": "Zhongtian broadband technology co., LTD ",
    "4CECEF": "Soraa, Inc.",
    "1CEFCE": "bebro electronic GmbH",
    "64DBA0": "Select Comfort",
    "EC43F6": "Zyxel Communications Corporation",
    "60C658": "PHYTRONIX Co.,Ltd.",
    "FCB58A": "Wapice Ltd.",
    "20780B": "Delta Faucet Company",
    "2CD02D": "Cisco Systems, Inc",
    "24A7DC": "SKY UK LIMITED",
    "0C73BE": "Dongguan Haimai Electronie Technology Co.,Ltd",
    "C8AA55": "Hunan Comtom Electronic Incorporated Co.,Ltd",
    "98B6E9": "Nintendo Co.,Ltd",
    "186590": "Apple, Inc.",
    "F86214": "Apple, Inc.",
    "784F43": "Apple, Inc.",
    "404D7F": "Apple, Inc.",
    "64B0A6": "Apple, Inc.",
    "7C04D0": "Apple, Inc.",
    "84FCAC": "Apple, Inc.",
    "DC0C5C": "Apple, Inc.",
    "F8983A": "Leeman International (HongKong) Limited",
    "001D72": "Wistron Corporation",
    "30E171": "Hewlett Packard",
    "F015B9": "PlayFusion Limited",
    "64136C": "zte corporation",
    "04B648": "ZENNER",
    "98F199": "NEC Platforms, Ltd.",
    "1840A4": "Shenzhen Trylong Smart Science and Technology Co., Ltd.",
    "4C7487": "Leader Phone Communication Technology Co., Ltd.",
    "4C38D5": "MITAC COMPUTING TECHNOLOGY CORPORATION",
    "54B56C": "Xi'an NovaStar Tech Co., Ltd",
    "1C48CE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "344CC8": "Echodyne Corp",
    "603E7B": "Gafachi, Inc.",
    "D8197A": "Nuheara Ltd",
    "70700D": "Apple, Inc.",
    "6C5C14": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "AC83F3": "AMPAK Technology, Inc.",
    "CC8CDA": "Shenzhen Wei Da Intelligent Technology Go.,Ltd",
    "D436DB": "Jiangsu Toppower Automotive Electronics Co., Ltd",
    "E80945": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "C80CC8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0425C5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2CDCAD": "Wistron Neweb Corporation",
    "B0A2E7": "Shenzhen TINNO Mobile Technology Corp.",
    "7C2587": "chaowifi.com",
    "002144": "SS Telecoms",
    "00501E": "Grass Valley, A Belden Brand",
    "EC0D9A": "Mellanox Technologies, Inc.",
    "3CFA43": "HUAWEI TECHNOLOGIES CO.,LTD",
    "145F94": "HUAWEI TECHNOLOGIES CO.,LTD",
    "90D7BE": "Wavelab Global Inc.",
    "244E7B": "IEEE Registration Authority",
    "CC9470": "Kinestral Technologies, Inc.",
    "085DDD": "MERCURY CORPORATION",
    "001F82": "Cal-Comp Electronics & Communications Company Ltd.",
    "883C1C": "MERCURY CORPORATION",
    "2834A2": "Cisco Systems, Inc",
    "7823AE": "ARRIS Group, Inc.",
    "006BF1": "Cisco Systems, Inc",
    "30AEA4": "Espressif Inc.",
    "CC61E5": "Motorola Mobility LLC, a Lenovo Company",
    "20719E": "SF Technology Co.,Ltd",
    "2CC260": "Oracle Corporation ",
    "3C3F51": "2CRSI",
    "50584F": "waytotec,Inc.",
    "B439D6": "ProCurve Networking by HP",
    "34F39A": "Intel Corporate",
    "8C60E7": "MPGIO CO.,LTD",
    "8C8ABB": "Beijing Orient View Technology Co., Ltd.",
    "00039B": "NetChip Technology, Inc.",
    "609AC1": "Apple, Inc.",
    "748D08": "Apple, Inc.",
    "9C8BA0": "Apple, Inc.",
    "CC088D": "Apple, Inc.",
    "38A4ED": "Xiaomi Communications Co Ltd",
    "B89919": "7signal Solutions, Inc",
    "40FE0D": "MAXIO",
    "AC64DD": "IEEE Registration Authority",
    "BC39D9": "Z-TEC",
    "3C2AF4": "Brother Industries, LTD.",
    "C0854C": "Ragentek Technology Group",
    "D816C1": "DEWAV (HK) ELECTRONICS LIMITED",
    "94B819": "Nokia",
    "787D48": "ITEL MOBILE LIMITED",
    "8871E5": "Amazon Technologies Inc.",
    "60EFC6": "Shenzhen Chima Technologies Co Limited",
    "20DBAB": "Samsung Electronics Co., Ltd.",
    "383A21": "IEEE Registration Authority",
    "D8380D": "SHENZHEN IP-COM Network Co.,Ltd",
    "FCCAC4": "LifeHealth, LLC",
    "88AD43": "PEGATRON CORPORATION",
    "B4EFFA": "Lemobile Information Technology (Beijing) Co., Ltd.",
    "B0C128": "Adler ELREHA GmbH",
    "F81D78": "IEEE Registration Authority",
    "38F7B2": "SEOJUN ELECTRIC",
    "7802B7": "ShenZhen Ultra Easy Technology CO.,LTD",
    "646184": "VELUX",
    "3087D9": "Ruckus Wireless",
    "F09838": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0D9B2": "EXO S.A.",
    "E4C801": "BLU Products Inc",
    "001FC6": "ASUSTek COMPUTER INC.",
    "6C71BD": "EZELINK TELECOM",
    "C87E75": "Samsung Electronics Co.,Ltd",
    "00265F": "Samsung Electronics Co.,Ltd",
    "00233A": "Samsung Electronics Co.,Ltd",
    "101250": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "0007AB": "Samsung Electronics Co.,Ltd",
    "E8E5D6": "Samsung Electronics Co.,Ltd",
    "80691A": "Belkin International Inc.",
    "B80756": "Cisco Meraki",
    "001427": "JazzMutant",
    "001E84": "Pika Technologies Inc.",
    "10DDB1": "Apple, Inc.",
    "002329": "DDRdrive LLC",
    "C80E14": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "AC63BE": "Amazon Technologies Inc.",
    "086A0A": "ASKEY COMPUTER CORP",
    "98E7F4": "Hewlett Packard",
    "EC65CC": "Panasonic Automotive Systems Company of America",
    "3456FE": "Cisco Meraki",
    "0026AD": "Arada Systems, Inc.",
    "002486": "DesignArt Networks",
    "002478": "Mag Tech Electronics Co Limited",
    "382DD1": "Samsung Electronics Co.,Ltd",
    "001B2C": "ATRON electronic GmbH",
    "9034FC": "Hon Hai Precision Ind. Co.,Ltd.",
    "0C84DC": "Hon Hai Precision Ind. Co.,Ltd.",
    "FC1F19": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "840B2D": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "206432": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "B407F9": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "889FFA": "Hon Hai Precision Ind. Co.,Ltd.",
    "8C7CB5": "Hon Hai Precision Ind. Co.,Ltd.",
    "C44619": "Hon Hai Precision Ind. Co.,Ltd.",
    "506313": "Hon Hai Precision Ind. Co.,Ltd.",
    "D0667B": "Samsung Electronics Co.,Ltd",
    "E8039A": "Samsung Electronics Co.,Ltd",
    "30CDA7": "Samsung Electronics Co.,Ltd",
    "3C8BFE": "Samsung Electronics Co.,Ltd",
    "D4E8B2": "Samsung Electronics Co.,Ltd",
    "002339": "Samsung Electronics Co.,Ltd",
    "5001BB": "Samsung Electronics Co.,Ltd",
    "00166C": "Samsung Electronics Co.,Ltd",
    "001599": "Samsung Electronics Co.,Ltd",
    "0012FB": "Samsung Electronics Co.,Ltd",
    "001247": "Samsung Electronics Co.,Ltd",
    "0015B9": "Samsung Electronics Co.,Ltd",
    "002491": "Samsung Electronics Co.,Ltd",
    "60D819": "Hon Hai Precision Ind. Co.,Ltd.",
    "F82FA8": "Hon Hai Precision Ind. Co.,Ltd.",
    "C40142": "MaxMedia Technology Limited",
    "8430E5": "SkyHawke Technologies, LLC",
    "1C77F6": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "58E326": "Compass Technologies Inc.",
    "001B2A": "Cisco Systems, Inc",
    "749DDC": "2Wire Inc",
    "C8BA94": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "843838": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "54880E": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "F025B7": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "F04347": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9CB2B2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8C83A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14DDE5": "MPMKVVCL",
    "001A09": "Wayfarer Transit Systems Ltd",
    "742344": "Xiaomi Communications Co Ltd",
    "E492FB": "Samsung Electronics Co.,Ltd",
    "6CB7F4": "Samsung Electronics Co.,Ltd",
    "181EB0": "Samsung Electronics Co.,Ltd",
    "247F20": "Sagemcom Broadband SAS",
    "684898": "Samsung Electronics Co.,Ltd",
    "3423BA": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "400E85": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "E09DFA": "Wanan Hongsheng Electronic Co.Ltd",
    "1489FD": "Samsung Electronics Co.,Ltd",
    "BC851F": "Samsung Electronics Co.,Ltd",
    "B85E7B": "Samsung Electronics Co.,Ltd",
    "2C4401": "Samsung Electronics Co.,Ltd",
    "B8D9CE": "Samsung Electronics Co.,Ltd",
    "1C66AA": "Samsung Electronics Co.,Ltd",
    "5C3C27": "Samsung Electronics Co.,Ltd",
    "BC72B1": "Samsung Electronics Co.,Ltd",
    "78F7BE": "Samsung Electronics Co.,Ltd",
    "70A84C": "MONAD., Inc.",
    "84C7EA": "Sony Corporation",
    "24E43F": "Wenzhou Kunmei Communication Technology Co.,Ltd.",
    "D013FD": "LG Electronics (Mobile Communications)",
    "287AEE": "ARRIS Group, Inc.",
    "00258B": "Mellanox Technologies, Inc.",
    "BC644B": "ARRIS Group, Inc.",
    "88797E": "Motorola Mobility LLC, a Lenovo Company",
    "305890": "Frontier Silicon Ltd",
    "708BCD": "ASUSTek COMPUTER INC.",
    "2CAC44": "CONEXTOP",
    "003676": "ARRIS Group, Inc.",
    "FC8E7E": "ARRIS Group, Inc.",
    "FC6FB7": "ARRIS Group, Inc.",
    "D42C0F": "ARRIS Group, Inc.",
    "A055DE": "ARRIS Group, Inc.",
    "80F503": "ARRIS Group, Inc.",
    "44AAF5": "ARRIS Group, Inc.",
    "606405": "Texas Instruments",
    "00562B": "Cisco Systems, Inc",
    "E8FD90": "Turbostor",
    "1899F5": "Sichuan Changhong Electric Ltd.",
    "18ABF5": "Ultra Electronics Electrics",
    "B03EB0": "MICRODIA Ltd.",
    "0025C3": "21168",
    "000F57": "CABLELOGIC Co., Ltd.",
    "000342": "Nortel Networks",
    "001283": "Nortel Networks",
    "0011F9": "Nortel Networks",
    "10E68F": "KWANGSUNG ELECTRONICS KOREA CO.,LTD.",
    "4CFACA": "Cambridge Industries(Group) Co.,Ltd.",
    "A48269": "Datrium, Inc.",
    "001158": "Nortel Networks",
    "000F6A": "Nortel Networks",
    "000E62": "Nortel Networks",
    "000CF8": "Nortel Networks",
    "000997": "Nortel Networks",
    "001CEB": "Nortel Networks",
    "001C17": "Nortel Networks",
    "001A8F": "Nortel Networks",
    "001591": "RLW Inc.",
    "00182E": "XStreamHD",
    "0026F1": "ProCurve Networking by HP",
    "380DD4": "Primax Electronics Ltd.",
    "98FDB4": "Primax Electronics Ltd.",
    "D8C46A": "Murata Manufacturing Co., Ltd.",
    "D8FB68": "Cloud Corner Ltd.",
    "685388": "P&S Technology",
    "14C1FF": "ShenZhen QianHai Comlan communication Co.,LTD",
    "ECFAAA": "The IMS Company",
    "0014C7": "Nortel Networks",
    "001DAF": "Nortel Networks",
    "888322": "Samsung Electronics Co.,Ltd",
    "E89309": "Samsung Electronics Co.,Ltd",
    "982F3C": "Sichuan Changhong Electric Ltd.",
    "F00786": "Shandong Bittel Electronics Co., Ltd",
    "00D0F6": "Nokia",
    "54A619": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "38F8CA": "OWIN Inc.",
    "002105": "Alcatel-Lucent IPD",
    "0017D1": "Nortel Networks",
    "2057AF": "Shenzhen FH-NET OPTOELECTRONICS CO.,LTD",
    "54DC1D": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "6CD032": "LG Electronics",
    "88A6C6": "Sagemcom Broadband SAS",
    "94D469": "Cisco Systems, Inc",
    "882BD7": "ADD\u00c9NERGIE  TECHNOLOGIES",
    "C0E42D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "8CA6DF": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "8416F9": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "18D6C7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "44334C": "Shenzhen Bilian electronic CO.,LTD",
    "ACA213": "Shenzhen Bilian electronic CO.,LTD",
    "3C3300": "Shenzhen Bilian electronic CO.,LTD",
    "0090CC": "PLANEX COMMUNICATIONS INC.",
    "0022CF": "PLANEX COMMUNICATIONS INC.",
    "E417D8": "8BITDO TECHNOLOGY HK LIMITED",
    "9CD332": "PLC Technology Ltd",
    "64899A": "LG Electronics (Mobile Communications)",
    "F8A9D0": "LG Electronics (Mobile Communications)",
    "CCFA00": "LG Electronics (Mobile Communications)",
    "74A722": "LG Electronics (Mobile Communications)",
    "F01C13": "LG Electronics (Mobile Communications)",
    "A816B2": "LG Electronics (Mobile Communications)",
    "64BC0C": "LG Electronics (Mobile Communications)",
    "344DF7": "LG Electronics (Mobile Communications)",
    "583F54": "LG Electronics (Mobile Communications)",
    "B0C5CA": "IEEE Registration Authority",
    "7419F8": "IEEE Registration Authority",
    "001BC5": "IEEE Registration Authority",
    "78C3E9": "Samsung Electronics Co.,Ltd",
    "8C1ABF": "Samsung Electronics Co.,Ltd",
    "30CBF8": "Samsung Electronics Co.,Ltd",
    "A0CBFD": "Samsung Electronics Co.,Ltd",
    "E45D75": "Samsung Electronics Co.,Ltd",
    "000031": "QPSX COMMUNICATIONS, LTD.",
    "000E1E": "QLogic Corporation",
    "0014D1": "TRENDnet, Inc.",
    "C01ADA": "Apple, Inc.",
    "00121C": "PARROT SA",
    "9003B7": "PARROT SA",
    "90C682": "IEEE Registration Authority",
    "58FCDB": "IEEE Registration Authority",
    "001C14": "VMware, Inc.",
    "005056": "VMware, Inc.",
    "0010C1": "OI ELECTRIC CO.,LTD",
    "38A28C": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "A09E1A": "Polar Electro Oy",
    "B4A5EF": "Sercomm Corporation.",
    "849D64": "SMC Corporation",
    "0080F7": "Zenith Electronics Corporation",
    "483C0C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "208756": "SIEMENS AG",
    "BC307D": "Wistron Neweb Corporation",
    "48A9D2": "Wistron Neweb Corporation",
    "80EA23": "Wistron Neweb Corporation",
    "74B472": "CIESSE",
    "FCF152": "Sony Corporation",
    "309BAD": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "CC52AF": "Universal Global Scientific Industrial Co., Ltd.",
    "002713": "Universal Global Scientific Industrial Co., Ltd.",
    "001BB1": "Wistron Neweb Corporation",
    "BC307E": "Wistron Neweb Corporation",
    "68B35E": "Shenzhen Neostra Technology Co.Ltd",
    "1CD6BD": "LEEDARSON LIGHTING CO., LTD.",
    "D0D94F": "IEEE Registration Authority",
    "506583": "Texas Instruments",
    "B09122": "Texas Instruments",
    "FC51A4": "ARRIS Group, Inc.",
    "9857D3": "HON HAI-CCPBG  PRECISION IND.CO.,LTD.",
    "AC0481": "Jiangsu Huaxing Electronics Co., Ltd.",
    "FCF528": "Zyxel Communications Corporation",
    "00A0C5": "Zyxel Communications Corporation",
    "408805": "Motorola Mobility LLC, a Lenovo Company",
    "60C0BF": "ON Semiconductor",
    "001E04": "Hanson Research Corporation",
    "98398E": "Samsung Electronics Co.,Ltd",
    "D0FCCC": "Samsung Electronics Co.,Ltd",
    "24E271": "Qingdao Hisense Communications Co.,Ltd.",
    "BC6010": "Qingdao Hisense Communications Co.,Ltd.",
    "F0F644": "Whitesky Science & Technology Co.,Ltd.",
    "7C6AF3": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20F17C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "346AC2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C41CFF": "Vizio, Inc",
    "44D1FA": "Shenzhen Yunlink Technology Co., Ltd",
    "C09727": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "DC293A": "Shenzhen Nuoshi Technology Co., LTD.",
    "40562D": "Smartron India Pvt ltd",
    "E46251": "HAO CHENG GROUP LIMITED",
    "A0D385": "AUMA Riester GmbH & Co. KG",
    "1414E6": "Ningbo Sanhe Digital Co.,Ltd",
    "3876D1": "Euronda SpA",
    "0C5101": "Apple, Inc.",
    "2CF0A2": "Apple, Inc.",
    "68FB7E": "Apple, Inc.",
    "84A134": "Apple, Inc.",
    "001A34": "Konka Group Co., Ltd.",
    "0011FC": "HARTING Electronics GmbH",
    "002389": "Hangzhou H3C Technologies Co., Limited",
    "3CE5A6": "Hangzhou H3C Technologies Co., Limited",
    "D8209F": "Cubro Acronet GesmbH",
    "8C7716": "LONGCHEER TELECOMMUNICATION LIMITED",
    "C4693E": "Turbulence Design Inc.",
    "009569": "LSD Science and Technology Co.,Ltd.",
    "B0CF4D": "MI-Zone Technology Ireland",
    "289AFA": "TCT mobile ltd",
    "904D4A": "Sagemcom Broadband SAS",
    "044E5A": "ARRIS Group, Inc.",
    "5CDD70": "Hangzhou H3C Technologies Co., Limited",
    "3C8C40": "Hangzhou H3C Technologies Co., Limited",
    "A067BE": "Sicon srl",
    "E04F43": "Universal Global Scientific Industrial Co., Ltd.",
    "38700C": "ARRIS Group, Inc.",
    "6C2483": "Microsoft Mobile Oy",
    "6891D0": "IEEE Registration Authority",
    "002582": "Maksat Technologies (P) Ltd",
    "48C049": "Broad Telecom SA",
    "AC6FBB": "TATUNG Technology Inc.",
    "001C41": "scemtec Transponder Technology GmbH",
    "146308": "JABIL CIRCUIT (SHANGHAI) LTD.",
    "001E25": "INTEK DIGITAL",
    "00E0CF": "INTEGRATED DEVICE",
    "0060B1": "Input/Output, Inc.",
    "547F54": "INGENICO",
    "90C7D8": "zte corporation",
    "003A7D": "Cisco Systems, Inc",
    "001A45": "GN Netcom A/S",
    "002088": "GLOBAL VILLAGE COMMUNICATION",
    "001CD7": "Harman/Becker Automotive Systems GmbH",
    "001921": "Elitegroup Computer Systems Co.,Ltd.",
    "0016EC": "Elitegroup Computer Systems Co.,Ltd.",
    "000795": "Elitegroup Computer Systems Co.,Ltd.",
    "541379": "Hon Hai Precision Ind. Co.,Ltd.",
    "844076": "Drivenets",
    "00065F": "ECI Telecom Ltd.",
    "00208F": "ECI Telecom Ltd.",
    "FC0F4B": "Texas Instruments",
    "D4883F": "HDPRO CO., LTD.",
    "60B617": "Fiberhome Telecommunication Technologies Co.,LTD",
    "18A3E8": "Fiberhome Telecommunication Technologies Co.,LTD",
    "DC9C9F": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "74DFBF": "Liteon Technology Corporation",
    "F03E90": "Ruckus Wireless",
    "00185C": "EDSLAB Technologies",
    "000E2E": "Edimax Technology Co. Ltd.",
    "84AD58": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58605F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D8D723": "IDS, Inc",
    "00A0F4": "GE",
    "AC0D1B": "LG Electronics (Mobile Communications)",
    "F0D1B8": "LEDVANCE",
    "986D35": "IEEE Registration Authority",
    "88795B": "Konka Group Co., Ltd.",
    "B07E70": "Zadara Storage Ltd.",
    "0080B1": "SOFTCOM A/S",
    "202DF8": "Digital Media Cartridge Ltd.",
    "FC2FAA": "Nokia",
    "081F71": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "5CCA1A": "Microsoft Mobile Oy",
    "741E93": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0C5A9E": "Wi-SUN Alliance",
    "D8803C": "Anhui Huami Information Technology Company Limited",
    "10D0AB": "zte corporation",
    "0004C6": "YAMAHA MOTOR CO.,LTD",
    "981FB1": "Shenzhen Lemon Network Technology Co.,Ltd",
    "202D07": "Samsung Electronics Co.,Ltd",
    "B44BD2": "Apple, Inc.",
    "DC415F": "Apple, Inc.",
    "641225": "Cisco Systems, Inc",
    "7864E6": "Green Motive Technology Limited",
    "3CBEE1": "NIKON CORPORATION",
    "18C501": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "00A0B8": "NetApp",
    "102AB3": "Xiaomi Communications Co Ltd",
    "40D357": "Ison Technology Co., Ltd.",
    "A0B9ED": "Skytap",
    "0034DA": "LG Electronics (Mobile Communications)",
    "3810D5": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "006016": "CLARIION",
    "00C88B": "Cisco Systems, Inc",
    "0009D2": "Mai Logic Inc.",
    "24C3F9": "Securitas Direct AB",
    "2C21D7": "IMAX Corporation",
    "E8B2AC": "Apple, Inc.",
    "E49A79": "Apple, Inc.",
    "30A9DE": "LG Innotek",
    "F01B6C": "vivo Mobile Communication Co., Ltd.",
    "4C2578": "Nokia Corporation",
    "BCC6DB": "Nokia Corporation",
    "60A8FE": "Nokia Solutions and Networks GmbH & Co. KG",
    "94C960": "Zhongshan B&T technology.co.,ltd",
    "A41588": "ARRIS Group, Inc.",
    "F45C89": "Apple, Inc.",
    "20768F": "Apple, Inc.",
    "9C5CF9": "Sony Corporation",
    "0011D1": "Soft Imaging System GmbH",
    "98D686": "Chyi Lee industry Co., ltd.",
    "8CC661": "Current, powered by GE",
    "88A084": "Formation Data Systems",
    "74C330": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "001D3B": "Nokia Danmark A/S",
    "001DFD": "Nokia Danmark A/S",
    "001E3B": "Nokia Danmark A/S",
    "001EA4": "Nokia Danmark A/S",
    "0026CC": "Nokia Danmark A/S",
    "000EED": "Nokia Danmark A/S",
    "00119F": "Nokia Danmark A/S",
    "001A16": "Nokia Danmark A/S",
    "DCB3B4": "Honeywell Environmental & Combustion Controls (Tianjin) Co., Ltd.",
    "00247C": "Nokia Danmark A/S",
    "001A89": "Nokia Danmark A/S",
    "001ADC": "Nokia Danmark A/S",
    "0025CF": "Nokia Danmark A/S",
    "0021AB": "Nokia Danmark A/S",
    "001FDE": "Nokia Danmark A/S",
    "001FDF": "Nokia Danmark A/S",
    "547975": "Nokia Corporation",
    "A87B39": "Nokia Corporation",
    "002266": "Nokia Danmark A/S",
    "0021FE": "Nokia Danmark A/S",
    "702559": "CyberTAN Technology Inc.",
    "001D20": "Comtrend Corporation",
    "08373D": "Samsung Electronics Co.,Ltd",
    "C488E5": "Samsung Electronics Co.,Ltd",
    "607EDD": "Microsoft Mobile Oy",
    "A8A089": "Tactical Communications",
    "48365F": "Wintecronics Ltd.",
    "000BCA": "DATAVAN TC",
    "C477AB": "Beijing ASU Tech Co.,Ltd",
    "0C75BD": "Cisco Systems, Inc",
    "300D43": "Microsoft Mobile Oy",
    "5056BF": "Samsung Electronics Co.,Ltd",
    "90F1AA": "Samsung Electronics Co.,Ltd",
    "1077B1": "Samsung Electronics Co.,Ltd",
    "001FC7": "Casio Hitachi Mobile Communications Co., Ltd.",
    "FCC734": "Samsung Electronics Co.,Ltd",
    "8425DB": "Samsung Electronics Co.,Ltd",
    "B0EC71": "Samsung Electronics Co.,Ltd",
    "E458B8": "Samsung Electronics Co.,Ltd",
    "088C2C": "Samsung Electronics Co.,Ltd",
    "A49A58": "Samsung Electronics Co.,Ltd",
    "08EE8B": "Samsung Electronics Co.,Ltd",
    "64B853": "Samsung Electronics Co.,Ltd",
    "389496": "Samsung Electronics Co.,Ltd",
    "00000E": "FUJITSU LIMITED",
    "000B5D": "FUJITSU LIMITED",
    "080581": "Roku, Inc.",
    "74458A": "Samsung Electronics Co.,Ltd",
    "C4473F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CDC96": "Arcadyan Technology Corporation",
    "001A2A": "Arcadyan Technology Corporation",
    "88252C": "Arcadyan Technology Corporation",
    "00E063": "Cabletron Systems, Inc.",
    "000DF3": "Asmax Solutions",
    "000DB6": "Broadcom",
    "000AF7": "Broadcom",
    "D40129": "Broadcom",
    "001D00": "Brivo Systems, LLC",
    "0020D6": "Breezecom, Ltd.",
    "1CC63C": "Arcadyan Technology Corporation",
    "1883BF": "Arcadyan Technology Corporation",
    "68ED43": "BlackBerry RTS",
    "70AAB2": "BlackBerry RTS",
    "000F86": "BlackBerry RTS",
    "00146C": "NETGEAR",
    "001E2A": "NETGEAR",
    "00184D": "NETGEAR",
    "00040E": "AVM GmbH",
    "9CC7A6": "AVM GmbH",
    "A06391": "NETGEAR",
    "20E52A": "NETGEAR",
    "4494FC": "NETGEAR",
    "200CC8": "NETGEAR",
    "0CA42A": "OB Telecom Electronic Technology Co., Ltd",
    "744401": "NETGEAR",
    "E091F5": "NETGEAR",
    "40BA61": "ARIMA Communications Corp.",
    "0011F5": "ASKEY COMPUTER CORP",
    "0016E3": "ASKEY COMPUTER CORP",
    "E839DF": "ASKEY COMPUTER CORP",
    "0024D2": "ASKEY COMPUTER CORP",
    "B4EEB4": "ASKEY COMPUTER CORP",
    "745612": "ARRIS Group, Inc.",
    "E46449": "ARRIS Group, Inc.",
    "40FC89": "ARRIS Group, Inc.",
    "2C9E5F": "ARRIS Group, Inc.",
    "002636": "ARRIS Group, Inc.",
    "001311": "ARRIS Group, Inc.",
    "0023EE": "ARRIS Group, Inc.",
    "001ADE": "ARRIS Group, Inc.",
    "001CC1": "ARRIS Group, Inc.",
    "001E5A": "ARRIS Group, Inc.",
    "0050E3": "ARRIS Group, Inc.",
    "7CBFB1": "ARRIS Group, Inc.",
    "001371": "ARRIS Group, Inc.",
    "0019A6": "ARRIS Group, Inc.",
    "002136": "ARRIS Group, Inc.",
    "001626": "ARRIS Group, Inc.",
    "D0DF9A": "Liteon Technology Corporation",
    "1C659D": "Liteon Technology Corporation",
    "3010B3": "Liteon Technology Corporation",
    "E8C74F": "Liteon Technology Corporation",
    "D05349": "Liteon Technology Corporation",
    "E874E6": "ADB Broadband Italia",
    "0020E0": "Actiontec Electronics, Inc",
    "002662": "Actiontec Electronics, Inc",
    "948815": "Infinique Worldwide Inc",
    "5CB524": "Sony Corporation",
    "90C115": "Sony Corporation",
    "D05162": "Sony Corporation",
    "18002D": "Sony Corporation",
    "280DFC": "Sony Interactive Entertainment Inc.",
    "0016B8": "Sony Corporation",
    "0024EF": "Sony Corporation",
    "0025E7": "Sony Corporation",
    "58170C": "Sony Corporation",
    "701A04": "Liteon Technology Corporation",
    "48D224": "Liteon Technology Corporation",
    "20689D": "Liteon Technology Corporation",
    "002553": "ADB Broadband Italia",
    "00193E": "ADB Broadband Italia",
    "000827": "ADB Broadband Italia",
    "D0E44A": "Murata Manufacturing Co., Ltd.",
    "080046": "Sony Corporation",
    "4C14A3": "TCL Technoly Electronics (Huizhou) Co., Ltd.",
    "4CB0E8": "Beijing RongZhi xinghua technology co., LTD",
    "74DAEA": "Texas Instruments",
    "D887D5": "Leadcore Technology CO.,LTD",
    "00F28B": "Cisco Systems, Inc",
    "ECF00E": "AboCom",
    "00E098": "AboCom",
    "742F68": "AzureWave Technology Inc.",
    "384FF0": "AzureWave Technology Inc.",
    "3413E8": "Intel Corporate",
    "34E6AD": "Intel Corporate",
    "081196": "Intel Corporate",
    "183DA2": "Intel Corporate",
    "809B20": "Intel Corporate",
    "B4B676": "Intel Corporate",
    "3CA9F4": "Intel Corporate",
    "9C4E36": "Intel Corporate",
    "B88A60": "Intel Corporate",
    "78FF57": "Intel Corporate",
    "002710": "Intel Corporate",
    "001B21": "Intel Corporate",
    "18FF0F": "Intel Corporate",
    "A48E0A": "DeLaval International AB",
    "AC2B6E": "Intel Corporate",
    "000A8A": "Cisco Systems, Inc",
    "9C3583": "Nipro Diagnostics, Inc",
    "C06118": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "002314": "Intel Corporate",
    "340286": "Intel Corporate",
    "001CBF": "Intel Corporate",
    "34DE1A": "Intel Corporate",
    "E8B1FC": "Intel Corporate",
    "CC3D82": "Intel Corporate",
    "F8E079": "Motorola Mobility LLC, a Lenovo Company",
    "CCC3EA": "Motorola Mobility LLC, a Lenovo Company",
    "40786A": "Motorola Mobility LLC, a Lenovo Company",
    "D02212": "IEEE Registration Authority",
    "100723": "IEEE Registration Authority",
    "A44F29": "IEEE Registration Authority",
    "74F8DB": "IEEE Registration Authority",
    "A43BFA": "IEEE Registration Authority",
    "001F3C": "Intel Corporate",
    "002315": "Intel Corporate",
    "00166F": "Intel Corporate",
    "0019D1": "Intel Corporate",
    "0019D2": "Intel Corporate",
    "C0830A": "2Wire Inc",
    "383BC8": "2Wire Inc",
    "60FE20": "2Wire Inc",
    "00183F": "2Wire Inc",
    "002456": "2Wire Inc",
    "18622C": "Sagemcom Broadband SAS",
    "3C81D8": "Sagemcom Broadband SAS",
    "40F201": "Sagemcom Broadband SAS",
    "D084B0": "Sagemcom Broadband SAS",
    "181E78": "Sagemcom Broadband SAS",
    "0037B7": "Sagemcom Broadband SAS",
    "0054BD": "Swelaser AB",
    "20BB76": "COL GIOVANNI PAOLO SpA",
    "3CDD89": "SOMO HOLDINGS & TECH. CO.,LTD.",
    "1801E3": "Bittium Wireless Ltd",
    "001E4C": "Hon Hai Precision Ind. Co.,Ltd.",
    "D8543A": "Texas Instruments",
    "649C8E": "Texas Instruments",
    "102EAF": "Texas Instruments",
    "7C8EE4": "Texas Instruments",
    "B4EED4": "Texas Instruments",
    "388602": "Flexoptix GmbH",
    "4065A3": "Sagemcom Broadband SAS",
    "00194B": "Sagemcom Broadband SAS",
    "001E74": "Sagemcom Broadband SAS",
    "149182": "Belkin International Inc.",
    "C4EDBA": "Texas Instruments",
    "9059AF": "Texas Instruments",
    "BC6A29": "Texas Instruments",
    "847E40": "Texas Instruments",
    "C83E99": "Texas Instruments",
    "40984E": "Texas Instruments",
    "0017EB": "Texas Instruments",
    "0017E6": "Texas Instruments",
    "001832": "Texas Instruments",
    "3C2DB7": "Texas Instruments",
    "D03761": "Texas Instruments",
    "5464D9": "Sagemcom Broadband SAS",
    "9C8E99": "Hewlett Packard",
    "00195B": "D-Link Corporation",
    "000F3D": "D-Link Corporation",
    "24DA11": "NO NDA Inc",
    "D0B33F": "Shenzhen TINNO Mobile Technology Corp.",
    "BCD1D3": "Shenzhen TINNO Mobile Technology Corp.",
    "EC2280": "D-Link International",
    "2400BA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24DF6A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40D855": "IEEE Registration Authority",
    "F4F5D8": "Google, Inc.",
    "D83C69": "Shenzhen TINNO Mobile Technology Corp.",
    "B0C090": "Chicony Electronics Co., Ltd.",
    "907F61": "Chicony Electronics Co., Ltd.",
    "AC06C7": "ServerNet S.r.l.",
    "0059AC": "KPN. B.V.",
    "001735": "Intel Wireless Network Group",
    "74AC5F": "Qiku Internet Network Scientific (Shenzhen) Co., Ltd.",
    "18AF61": "Apple, Inc.",
    "5CF938": "Apple, Inc.",
    "0C0535": "Juniper Systems",
    "BC83A7": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "BCEC23": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "788B77": "Standar Telecom",
    "001DCD": "ARRIS Group, Inc.",
    "001DD2": "ARRIS Group, Inc.",
    "E83EFC": "ARRIS Group, Inc.",
    "8C579B": "Wistron Neweb Corporation",
    "34AB37": "Apple, Inc.",
    "38CADA": "Apple, Inc.",
    "900DCB": "ARRIS Group, Inc.",
    "8C09F4": "ARRIS Group, Inc.",
    "101F74": "Hewlett Packard",
    "009C02": "Hewlett Packard",
    "0019BB": "Hewlett Packard",
    "001F29": "Hewlett Packard",
    "780CB8": "Intel Corporate",
    "185E0F": "Intel Corporate",
    "001635": "Hewlett Packard",
    "0008C7": "Hewlett Packard",
    "0010E3": "Hewlett Packard",
    "000883": "Hewlett Packard",
    "8CDCD4": "Hewlett Packard",
    "DCFB02": "BUFFALO.INC",
    "8857EE": "BUFFALO.INC",
    "00215A": "Hewlett Packard",
    "00237D": "Hewlett Packard",
    "002655": "Hewlett Packard",
    "000D9D": "Hewlett Packard",
    "001560": "Hewlett Packard",
    "E8ED05": "ARRIS Group, Inc.",
    "789684": "ARRIS Group, Inc.",
    "CC65AD": "ARRIS Group, Inc.",
    "00207B": "Intel Corporation",
    "001175": "Intel Corporation",
    "D4C9EF": "Hewlett Packard",
    "FC15B4": "Hewlett Packard",
    "940937": "HUMAX Co., Ltd.",
    "E84DD0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C45BA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20906F": "Shenzhen Tencent Computer System Co., Ltd.",
    "6CE3B6": "Nera Telecommunications Ltd.",
    "EC5F23": "Qinghai Kimascend Electronics Technology Co. Ltd.",
    "047D50": "Shenzhen Kang Ying Technology Co.Ltd.",
    "54EFFE": "Fullpower Technologies, Inc.",
    "DCD321": "HUMAX Co., Ltd.",
    "4482E5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00234E": "Hon Hai Precision Ind. Co.,Ltd.",
    "2C8158": "Hon Hai Precision Ind. Co.,Ltd.",
    "8002DF": "ORA Inc.",
    "2C233A": "Hewlett Packard",
    "000A57": "Hewlett Packard",
    "0001E7": "Hewlett Packard",
    "0001E6": "Hewlett Packard",
    "00306E": "Hewlett Packard",
    "CC4463": "Apple, Inc.",
    "6C72E7": "Apple, Inc.",
    "741BB2": "Apple, Inc.",
    "001D0F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "3C4A92": "Hewlett Packard",
    "002376": "HTC Corporation",
    "38E7D8": "HTC Corporation",
    "188796": "HTC Corporation",
    "B4CEF6": "HTC Corporation",
    "7C7D3D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3CA82A": "Hewlett Packard",
    "90489A": "Hon Hai Precision Ind. Co.,Ltd.",
    "0071CC": "Hon Hai Precision Ind. Co.,Ltd.",
    "B05B67": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CCA223": "HUAWEI TECHNOLOGIES CO.,LTD",
    "786A89": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14B968": "HUAWEI TECHNOLOGIES CO.,LTD",
    "882593": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "001FE1": "Hon Hai Precision Ind. Co.,Ltd.",
    "10A5D0": "Murata Manufacturing Co., Ltd.",
    "0012F2": "Brocade Communications Systems LLC",
    "00051E": "Brocade Communications Systems LLC",
    "384608": "zte corporation",
    "4CAC0A": "zte corporation",
    "B4B362": "zte corporation",
    "B075D5": "zte corporation",
    "6CE873": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C46E1F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "50FA84": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "44B32D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "0023CD": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "D85D4C": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A0F3C1": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "783E53": "SKY UK LIMITED",
    "0019FB": "SKY UK LIMITED",
    "C4F57C": "Brocade Communications Systems LLC",
    "D0154A": "zte corporation",
    "0026ED": "zte corporation",
    "002293": "zte corporation",
    "083E8E": "Hon Hai Precision Ind. Co.,Ltd.",
    "006057": "Murata Manufacturing Co., Ltd.",
    "70A8E3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8E811": "HUAWEI TECHNOLOGIES CO.,LTD",
    "200BC7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F84ABF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78D752": "HUAWEI TECHNOLOGIES CO.,LTD",
    "104780": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94772B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D440F0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04021F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50A72B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0CD6BD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00F81C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "404D8E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C5499": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F81547": "Avaya Inc",
    "087A4C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ACE215": "HUAWEI TECHNOLOGIES CO.,LTD",
    "346BD3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "70723C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ACE87B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F83DFF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "285FDB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CF96A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F49FF3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "240995": "HUAWEI TECHNOLOGIES CO.,LTD",
    "84DBAC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "506184": "Avaya Inc",
    "10CDAE": "Avaya Inc",
    "048A15": "Avaya Inc",
    "B4B017": "Avaya Inc",
    "90FB5B": "Avaya Inc",
    "C8F406": "Avaya Inc",
    "7052C5": "Avaya Inc",
    "00040D": "Avaya Inc",
    "548998": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30D17E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "640980": "Xiaomi Communications Co Ltd",
    "185936": "Xiaomi Communications Co Ltd",
    "20A783": "miControl GmbH",
    "0090F2": "Cisco Systems, Inc",
    "74D6EA": "Texas Instruments",
    "209148": "Texas Instruments",
    "F8A45F": "Xiaomi Communications Co Ltd",
    "60735C": "Cisco Systems, Inc",
    "34A84E": "Cisco Systems, Inc",
    "54781A": "Cisco Systems, Inc",
    "E02F6D": "Cisco Systems, Inc",
    "94049C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "688F84": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00605C": "Cisco Systems, Inc",
    "0006C1": "Cisco Systems, Inc",
    "00E014": "Cisco Systems, Inc",
    "0050F0": "Cisco Systems, Inc",
    "005014": "Cisco Systems, Inc",
    "0050BD": "Cisco Systems, Inc",
    "00906F": "Cisco Systems, Inc",
    "544A16": "Texas Instruments",
    "58971E": "Cisco Systems, Inc",
    "B4E9B0": "Cisco Systems, Inc",
    "000832": "Cisco Systems, Inc",
    "70105C": "Cisco Systems, Inc",
    "10F311": "Cisco Systems, Inc",
    "3085A9": "ASUSTek COMPUTER INC.",
    "B83861": "Cisco Systems, Inc",
    "580A20": "Cisco Systems, Inc",
    "001731": "ASUSTek COMPUTER INC.",
    "002215": "ASUSTek COMPUTER INC.",
    "E0CB4E": "ASUSTek COMPUTER INC.",
    "2C3ECF": "Cisco Systems, Inc",
    "508789": "Cisco Systems, Inc",
    "381C1A": "Cisco Systems, Inc",
    "BC671C": "Cisco Systems, Inc",
    "346288": "Cisco Systems, Inc",
    "CCD8C1": "Cisco Systems, Inc",
    "7C0ECE": "Cisco Systems, Inc",
    "A0ECF9": "Cisco Systems, Inc",
    "5CA48A": "Cisco Systems, Inc",
    "1C1D86": "Cisco Systems, Inc",
    "5017FF": "Cisco Systems, Inc",
    "189C5D": "Cisco Systems, Inc",
    "DCA5F4": "Cisco Systems, Inc",
    "547C69": "Cisco Systems, Inc",
    "9C57AD": "Cisco Systems, Inc",
    "B05947": "Shenzhen Qihu Intelligent Technology Company Limited",
    "004096": "Cisco Systems, Inc",
    "0CE0E4": "PLANTRONICS, INC.",
    "74A2E6": "Cisco Systems, Inc",
    "BCF1F2": "Cisco Systems, Inc",
    "C80084": "Cisco Systems, Inc",
    "40A6E8": "Cisco Systems, Inc",
    "E86549": "Cisco Systems, Inc",
    "B07D47": "Cisco Systems, Inc",
    "38ED18": "Cisco Systems, Inc",
    "382056": "Cisco Systems, Inc",
    "DCCEC1": "Cisco Systems, Inc",
    "60FEC5": "Apple, Inc.",
    "E425E7": "Apple, Inc.",
    "BC926B": "Apple, Inc.",
    "101C0C": "Apple, Inc.",
    "080007": "Apple, Inc.",
    "40D32D": "Apple, Inc.",
    "C42C03": "Apple, Inc.",
    "9027E4": "Apple, Inc.",
    "109ADD": "Apple, Inc.",
    "0016CB": "Apple, Inc.",
    "0017F2": "Apple, Inc.",
    "001F5B": "Apple, Inc.",
    "002436": "Apple, Inc.",
    "001947": "Cisco SPVTG",
    "001839": "Cisco-Linksys, LLC",
    "00254B": "Apple, Inc.",
    "7C6D62": "Apple, Inc.",
    "6C3E6D": "Apple, Inc.",
    "BC6778": "Apple, Inc.",
    "20C9D0": "Apple, Inc.",
    "68967B": "Apple, Inc.",
    "84FCFE": "Apple, Inc.",
    "E48B7F": "Apple, Inc.",
    "581FAA": "Apple, Inc.",
    "88C663": "Apple, Inc.",
    "A46706": "Apple, Inc.",
    "8C5877": "Apple, Inc.",
    "283737": "Apple, Inc.",
    "50EAD6": "Apple, Inc.",
    "B817C2": "Apple, Inc.",
    "7C11BE": "Apple, Inc.",
    "BC3BAF": "Apple, Inc.",
    "3CE072": "Apple, Inc.",
    "38484C": "Apple, Inc.",
    "D8D1CB": "Apple, Inc.",
    "A8FAD8": "Apple, Inc.",
    "008865": "Apple, Inc.",
    "E0F5C6": "Apple, Inc.",
    "A0EDCD": "Apple, Inc.",
    "189EFC": "Apple, Inc.",
    "804971": "Apple, Inc.",
    "98D6BB": "Apple, Inc.",
    "7CF05F": "Apple, Inc.",
    "5C97F3": "Apple, Inc.",
    "D4F46F": "Apple, Inc.",
    "48437C": "Apple, Inc.",
    "34A395": "Apple, Inc.",
    "787E61": "Apple, Inc.",
    "C0F2FB": "Apple, Inc.",
    "24E314": "Apple, Inc.",
    "80E650": "Apple, Inc.",
    "90FD61": "Apple, Inc.",
    "2CF0EE": "Apple, Inc.",
    "3C15C2": "Apple, Inc.",
    "6C709F": "Apple, Inc.",
    "6476BA": "Apple, Inc.",
    "34E2FD": "Apple, Inc.",
    "04489A": "Apple, Inc.",
    "087045": "Apple, Inc.",
    "A88808": "Apple, Inc.",
    "A4C361": "Apple, Inc.",
    "B09FBA": "Apple, Inc.",
    "8C2937": "Apple, Inc.",
    "ACCF5C": "Apple, Inc.",
    "80006E": "Apple, Inc.",
    "848E0C": "Apple, Inc.",
    "F099BF": "Apple, Inc.",
    "94E96A": "Apple, Inc.",
    "AC293A": "Apple, Inc.",
    "9CFC01": "Apple, Inc.",
    "60F81D": "Apple, Inc.",
    "5CF5DA": "Apple, Inc.",
    "18EE69": "Apple, Inc.",
    "649ABE": "Apple, Inc.",
    "9C35EB": "Apple, Inc.",
    "507A55": "Apple, Inc.",
    "0C4DE9": "Apple, Inc.",
    "F0F61C": "Apple, Inc.",
    "F0F249": "Hitron Technologies. Inc",
    "38C986": "Apple, Inc.",
    "D03311": "Apple, Inc.",
    "5882A8": "Microsoft",
    "C8C2C6": "Shanghai Airm2m Communication Technology Co., Ltd",
    "789C85": "August Home, Inc.",
    "DCFE07": "PEGATRON CORPORATION",
    "707938": "Wuxi Zhanrui Electronic Technology Co.,LTD",
    "B813E9": "Trace Live Network",
    "58685D": "Tempo Australia Pty Ltd",
    "243184": "SHARP Corporation",
    "582BDB": "Pax AB",
    "24DA9B": "Motorola Mobility LLC, a Lenovo Company",
    "30E090": "Genevisio Ltd.",
    "F44D30": "Elitegroup Computer Systems Co.,Ltd.",
    "E03676": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC388F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C49E41": "G24 Power Limited",
    "D03E5C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F00D5C": "JinQianMao  Technology Co.,Ltd.",
    "54BE53": "zte corporation",
    "280E8B": "Beijing Spirit Technology Development Co., Ltd.",
    "80B709": "Viptela, Inc",
    "A4DCBE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D494E8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B078F0": "Beijing HuaqinWorld Technology Co.,Ltd.",
    "38D40B": "Samsung Electronics Co.,Ltd",
    "209BCD": "Apple, Inc.",
    "94BBAE": "Husqvarna AB",
    "D40AA9": "ARRIS Group, Inc.",
    "203D66": "ARRIS Group, Inc.",
    "ECB870": "Beijing Heweinet Technology Co.,Ltd.",
    "3095E3": "SHANGHAI SIMCOM LIMITED",
    "E83A12": "Samsung Electronics Co.,Ltd",
    "80656D": "Samsung Electronics Co.,Ltd",
    "FCF136": "Samsung Electronics Co.,Ltd",
    "B88687": "Liteon Technology Corporation",
    "18895B": "Samsung Electronics Co.,Ltd",
    "584925": "E3 Enterprise",
    "94F278": "Elma Electronic",
    "0894EF": "Wistron Infocomm (Zhongshan) Corporation",
    "283713": "Shenzhen 3Nod Digital Technology Co., Ltd.",
    "E0319E": "Valve Corporation",
    "7CAB25": "MESMO TECHNOLOGY INC.",
    "584822": "Sony Corporation",
    "747336": "MICRODIGTAL Inc",
    "B0411D": "ITTIM Technologies",
    "7CA237": "King Slide Technology CO., LTD.",
    "D404CD": "ARRIS Group, Inc.",
    "3C5CC3": "Shenzhen First Blue Chip Technology Ltd",
    "ECEED8": "ZTLX Network Technology Co.,Ltd",
    "80EB77": "Wistron Corporation",
    "483974": "Proware Technologies Co., Ltd.",
    "30FFF6": "HangZhou KuoHeng Technology Co.,ltd",
    "48E244": "Hon Hai Precision Ind. Co.,Ltd.",
    "D8EFCD": "Nokia Solutions and Networks GmbH & Co. KG",
    "4CC681": "Shenzhen Aisat Electronic Co., Ltd.",
    "C49FF3": "Mciao Technologies, Inc.",
    "788E33": "Jiangsu SEUIC Technology Co.,Ltd",
    "884157": "Shenzhen Atsmart Technology Co.,Ltd.",
    "D89A34": "Beijing SHENQI Technology Co., Ltd.",
    "C4EA1D": "Technicolor Delivery Technologies Belgium NV",
    "7CF90E": "Samsung Electronics Co.,Ltd",
    "382B78": "ECO PLUGS ENTERPRISE CO., LTD",
    "A47B2C": "Nokia",
    "DCDB70": "Tonfunk Systementwicklung und Service GmbH",
    "800B51": "Chengdu XGimi Technology Co.,Ltd",
    "3481F4": "SST Taiwan Ltd.",
    "F8BF09": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CB25C": "Acacia Communications",
    "803B2A": "ABB Xiamen Low Voltage Equipment Co.,Ltd.",
    "A0A65C": "Supercomputing Systems AG",
    "F80D60": "CANON INC.",
    "F0182B": "LG Chem",
    "E8377A": "Zyxel Communications Corporation",
    "00A784": "ITX security",
    "5CB43E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54E140": "INGENICO",
    "E4907E": "Motorola Mobility LLC, a Lenovo Company",
    "3CCE15": "Mercedes-Benz USA, LLC",
    "38FACA": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "149A10": "Microsoft Corporation",
    "707781": "Hon Hai Precision Ind. Co.,Ltd.",
    "50F0D3": "Samsung Electronics Co.,Ltd",
    "24E5AA": "Philips Oral Healthcare, Inc.",
    "78BDBC": "Samsung Electronics Co.,Ltd",
    "287610": "IgniteNet",
    "746A3A": "Aperi Corporation",
    "94A7B7": "zte corporation",
    "1844E6": "zte corporation",
    "485073": "Microsoft Corporation",
    "349B5B": "Maquet GmbH",
    "84119E": "Samsung Electronics Co.,Ltd",
    "54E2C8": "Dongguan Aoyuan Electronics Technology Co., Ltd",
    "20D75A": "Posh Mobile Limited",
    "F41563": "F5 Networks, Inc.",
    "8C8B83": "Texas Instruments",
    "4011DC": "Sonance",
    "1C8341": "Hefei Bitland Information Technology Co.Ltd",
    "706879": "Saijo Denki International Co., Ltd.",
    "081FEB": "BinCube",
    "785F4C": "Argox Information Co., Ltd.",
    "6C1E70": "Guangzhou YBDS IT Co.,Ltd",
    "D8ADDD": "Sonavation, Inc.",
    "8833BE": "Ivenix, Inc.",
    "54B80A": "D-Link International",
    "34CC28": "Nexpring Co. LTD.,",
    "249EAB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6CA75F": "zte corporation",
    "10AF78": "Shenzhen ATUE Technology Co., Ltd",
    "E48D8C": "Routerboard.com",
    "8C7967": "zte corporation",
    "CCA4AF": "Shenzhen Sowell Technology Co., LTD",
    "CC19A8": "PT Inova\u00e7\u00e3o e Sistemas SA",
    "B4B265": "DAEHO I&T",
    "74E28C": "Microsoft Corporation",
    "0071C2": "PEGATRON CORPORATION",
    "7C8274": "Shenzhen Hikeen Technology CO.,LTD",
    "94D417": "GPI KOREA INC.",
    "E03560": "Challenger Supply Holdings, LLC",
    "244B81": "Samsung Electronics Co.,Ltd",
    "D855A3": "zte corporation",
    "38D82F": "zte corporation",
    "7858F3": "Vachen Co.,Ltd",
    "244B03": "Samsung Electronics Co.,Ltd",
    "3CCB7C": "TCT mobile ltd",
    "E4CE70": "Health & Life co., Ltd.",
    "704E66": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "102C83": "XIMEA",
    "709C8F": "Nero AG",
    "844BB7": "Beijing Sankuai Online Technology Co.,Ltd",
    "68F0BC": "Shenzhen LiWiFi Technology Co., Ltd",
    "2884FA": "SHARP Corporation",
    "60D9A0": "Lenovo Mobile Communication Technology Ltd.",
    "F07959": "ASUSTek COMPUTER INC.",
    "E08E3C": "Aztech Electronics Pte Ltd",
    "3C1E04": "D-Link International",
    "300EE3": "Aquantia Corporation",
    "60AF6D": "Samsung Electronics Co.,Ltd",
    "B85A73": "Samsung Electronics Co.,Ltd",
    "68B983": "b-plus GmbH",
    "18F145": "NetComm Wireless Limited",
    "ACABBF": "AthenTek Inc.",
    "981DFA": "Samsung Electronics Co.,Ltd",
    "186882": "Beward R&D Co., Ltd.",
    "78B3B9": "ShangHai sunup lighting CO.,LTD",
    "04C09C": "Tellabs Inc.",
    "7429AF": "Hon Hai Precision Ind. Co.,Ltd.",
    "40EACE": "FOUNDER BROADBAND NETWORK SERVICE CO.,LTD",
    "848EDF": "Sony Corporation",
    "EC8009": "NovaSparks",
    "50ADD5": "Dynalec Corporation",
    "B04519": "TCT mobile ltd",
    "D88D5C": "Elentec",
    "3C1A0F": "ClearSky Data",
    "E8CC18": "D-Link International",
    "B09137": "ISis ImageStream Internet Solutions, Inc",
    "8C0551": "Koubachi AG",
    "D897BA": "PEGATRON CORPORATION",
    "A8D88A": "Wyconn",
    "207693": "Lenovo (Beijing) Limited.",
    "600417": "POSBANK CO.,LTD",
    "2497ED": "Techvision Intelligent Technology Limited",
    "A49D49": "Ketra, Inc.",
    "C03896": "Hon Hai Precision Ind. Co.,Ltd.",
    "2C5089": "Shenzhen Kaixuan Visual Technology Co.,Limited",
    "1C7E51": "3bumen.com",
    "948E89": "INDUSTRIAS UNIDAS SA DE CV",
    "084656": "VEO-LABS",
    "EC3C5A": "SHEN ZHEN HENG SHENG HUI DIGITAL TECHNOLOGY CO.,LTD",
    "4488CB": "Camco Technologies NV",
    "6872DC": "CETORY.TV Company Limited",
    "00AEFA": "Murata Manufacturing Co., Ltd.",
    "841826": "Osram GmbH",
    "50294D": "NANJING IOT SENSOR TECHNOLOGY CO,LTD",
    "0CCFD1": "SPRINGWAVE Co., Ltd",
    "74BADB": "Longconn Electornics(shenzhen)Co.,Ltd",
    "909F33": "EFM Networks",
    "3077CB": "Maike Industry(Shenzhen)CO.,LTD",
    "102F6B": "Microsoft Corporation",
    "945493": "Rigado, LLC",
    "6CBFB5": "Noon Technology Co., Ltd",
    "B8F317": "iSun Smasher Communications Private Limited",
    "8CF813": "ORANGE POLSKA",
    "68F06D": "ALONG INDUSTRIAL CO., LIMITED",
    "F82441": "Yeelink",
    "108A1B": "RAONIX Inc.",
    "F8E903": "D-Link International",
    "E89606": "testo Instruments (Shenzhen) Co., Ltd.",
    "F42853": "Zioncom Electronics (Shenzhen) Ltd.",
    "1C9C26": "Zoovel Technologies",
    "D4EC86": "LinkedHope Intelligent Technologies Co., Ltd",
    "046785": "scemtec Hard- und Software fuer Mess- und Steuerungstechnik GmbH",
    "D0FA1D": "Qihoo  360  Technology Co.,Ltd",
    "B89BE4": "ABB Power Systems Power Generation",
    "505065": "TAKT Corporation",
    "40C62A": "Shanghai Jing Ren Electronic Technology Co., Ltd.",
    "E8150E": "Nokia Corporation",
    "DC537C": "Compal Broadband Networks, Inc.",
    "AC11D3": "Suzhou HOTEK  Video Technology Co. Ltd",
    "8432EA": "ANHUI WANZTEN P&T CO., LTD",
    "E01D38": "Beijing HuaqinWorld Technology Co.,Ltd",
    "E47FB2": "FUJITSU LIMITED",
    "FC6DC0": "BME CORPORATION",
    "24D13F": "MEXUS CO.,LTD",
    "7824AF": "ASUSTek COMPUTER INC.",
    "0CAC05": "Unitend Technologies Inc.",
    "FC9FE1": "CONWIN.Tech. Ltd",
    "A81B5D": "Foxtel Management Pty Ltd",
    "44A6E5": "THINKING TECHNOLOGY CO.,LTD",
    "3CAA3F": "iKey, Ltd.",
    "0C383E": "Fanvil Technology Co., Ltd.",
    "60CDA9": "Abloomy",
    "B8AD3E": "BLUECOM",
    "183009": "Woojin Industrial Systems Co., Ltd.",
    "74DBD1": "Ebay Inc",
    "80AD67": "Kasda Networks Inc",
    "30B5F1": "Aitexin Technology Co., Ltd",
    "B01041": "Hon Hai Precision Ind. Co.,Ltd.",
    "C44202": "Samsung Electronics Co.,Ltd",
    "B4AE6F": "Circle Reliance, Inc DBA Cranberry Networks",
    "90DA6A": "FOCUS H&S Co., Ltd.",
    "103047": "Samsung Electronics Co.,Ltd",
    "5C2E59": "Samsung Electronics Co.,Ltd",
    "4C6E6E": "Comnect Technology CO.,LTD",
    "8C3357": "HiteVision Digital Media Technology Co.,Ltd.",
    "F884F2": "Samsung Electronics Co.,Ltd",
    "A45DA1": "ADB Broadband Italia",
    "C46BB4": "myIDkey",
    "84948C": "Hitron Technologies. Inc",
    "A8F7E0": "PLANET Technology Corporation",
    "4486C1": "Siemens Low Voltage & Products",
    "E0CBEE": "Samsung Electronics Co.,Ltd",
    "4C3909": "HPL Electric & Power Private Limited",
    "907EBA": "UTEK TECHNOLOGY (SHENZHEN) CO.,LTD",
    "A002DC": "Amazon Technologies Inc.",
    "4045DA": "Spreadtrum Communications (Shanghai) Co., Ltd.",
    "3451AA": "JID GLOBAL",
    "98BE94": "IBM",
    "6C198F": "D-Link International",
    "C8FF77": "Dyson Limited",
    "542AA2": "Alpha Networks Inc.",
    "B49EAC": "Imagik Int'l Corp",
    "54B753": "Hunan Fenghui Yinjia Science And Technology Co.,Ltd",
    "B0754D": "Nokia",
    "CC07E4": "Lenovo Mobile Communication Technology Ltd.",
    "ACB859": "Uniband Electronic Corp,",
    "88B1E1": " Mojo Networks, Inc.",
    "54EE75": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "0C63FC": "Nanjing Signway Technology Co., Ltd",
    "D4E08E": "ValueHD Corporation",
    "C89F1D": "SHENZHEN COMMUNICATION TECHNOLOGIES CO.,LTD",
    "0874F6": "Winterhalter Gastronom GmbH",
    "143DF2": "Beijing Shidai Hongyuan Network Communication Co.,Ltd",
    "6047D4": "FORICS Electronic Technology Co., Ltd.",
    "D8492F": "CANON INC.",
    "2C39C1": "Ciena Corporation",
    "800E24": "ForgetBox",
    "3C25D7": "Nokia Corporation",
    "30A8DB": "Sony Corporation",
    "54D163": "MAX-TECH,INC",
    "083F3E": "WSH GmbH",
    "1820A6": "Sage Co., Ltd.",
    "20EAC7": "SHENZHEN RIOPINE ELECTRONICS CO., LTD",
    "64B370": "PowerComm Solutions LLC",
    "5CF50D": "Institute of microelectronic applications",
    "749C52": "Huizhou Desay SV Automotive Co., Ltd.",
    "48B5A7": "Glory Horse Industries Ltd.",
    "0C4F5A": "ASA-RT s.r.l.",
    "CCA614": "AIFA TECHNOLOGY CORP.",
    "4C8B30": "Actiontec Electronics, Inc",
    "0805CD": "DongGuang EnMai Electronic Product Co.Ltd.",
    "18FF2E": "Shenzhen Rui Ying Da Technology Co., Ltd",
    "847207": "I&C Technology",
    "D4224E": "Alcatel Lucent",
    "9C86DA": "Phoenix Geophysics Ltd.",
    "2C073C": "DEVLINE LIMITED",
    "7C1A03": "8Locations Co., Ltd.",
    "10DEE4": "automationNEXT GmbH",
    "F03A4B": "Bloombase, Inc.",
    "90F1B0": "Hangzhou Anheng Info&Tech CO.,LTD",
    "90DB46": "E-LEAD ELECTRONIC CO., LTD",
    "344F5C": "R&amp;M AG",
    "7CE4AA": "Private",
    "C4291D": "KLEMSAN ELEKTRIK ELEKTRONIK SAN.VE TIC.AS.",
    "FCF8B7": "TRONTEQ Electronic",
    "30F42F": "ESP",
    "704E01": "KWANGWON TECH CO., LTD.",
    "746A8F": "VS Vision Systems GmbH",
    "54A31B": "Shenzhen Linkworld Technology Co,.LTD",
    "CC398C": "Shiningtek",
    "6C5F1C": "Lenovo Mobile Communication Technology Ltd.",
    "404A18": "Addrek Smart Solutions",
    "C0C569": "SHANGHAI LYNUC CNC TECHNOLOGY CO.,LTD",
    "C4C0AE": "MIDORI ELECTRONIC CO., LTD.",
    "ACC595": "Graphite Systems",
    "D8EE78": "Moog Protokraft",
    "7CE1FF": "Computer Performance, Inc. DBA Digital Loggers, Inc.",
    "D8150D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "4CD7B6": "Helmer Scientific",
    "8425A4": "Tariox Limited",
    "483D32": "Syscor Controls &amp; Automation",
    "A0E453": "Sony Corporation",
    "3C0C48": "Servergy, Inc.",
    "CC856C": "SHENZHEN MDK DIGITAL TECHNOLOGY CO.,LTD",
    "2C957F": "zte corporation",
    "98FF6A": "OTEC(Shanghai)Technology Co.,Ltd.",
    "7CCD11": "MS-Magnet",
    "5850AB": "TLS Corporation",
    "241148": "Entropix, LLC",
    "CC7498": "Filmetrics Inc.",
    "64BABD": "SDJ Technologies, Inc.",
    "24C848": "mywerk Portal GmbH",
    "50ED78": "Changzhou Yongse Infotech Co.,Ltd",
    "1CC11A": "Wavetronix",
    "90028A": "Shenzhen Shidean Legrand Electronic Products Co.,Ltd",
    "085AE0": "Recovision Technology Co., Ltd.",
    "E0C86A": "SHENZHEN TW-SCIE Co., Ltd",
    "1879A2": "GMJ ELECTRIC LIMITED",
    "C445EC": "Shanghai Yali Electron Co.,LTD",
    "E0E631": "SNB TECHNOLOGIES LIMITED",
    "F8572E": "Core Brands, LLC",
    "78B5D2": "Ever Treasure Industrial Limited",
    "FC09D8": "ACTEON Group",
    "743ECB": "Gentrice tech",
    "7C444C": "Entertainment Solutions, S.L.",
    "0444A1": "TELECON GALICIA,S.A.",
    "20C60D": "Shanghai annijie Information technology Co.,LTD",
    "38CA97": "Contour Design LLC",
    "BC2D98": "ThinGlobal LLC",
    "BC1A67": "YF Technology Co., Ltd",
    "AC6BAC": "Jenny Science AG",
    "7C9763": "Openmatics s.r.o.",
    "48A2B7": "Kodofon JSC",
    "BCEE7B": "ASUSTek COMPUTER INC.",
    "3413A8": "Mediplan Limited",
    "8CAE89": "Y-cam Solutions Ltd",
    "FCE1D9": "Stable Imaging Solutions LLC",
    "B04545": "YACOUB Automation GmbH",
    "C8EE75": "Pishion International Co. Ltd",
    "682DDC": "Wuhan Changjiang Electro-Communication Equipment CO.,LTD",
    "E8611F": "Dawning Information Industry Co.,Ltd",
    "705957": "Medallion Instrumentation Systems",
    "9CD643": "D-Link International",
    "3C18A0": "Luxshare Precision Industry Company Limited",
    "94E98C": "Nokia",
    "94C3E4": "Atlas Copco IAS GmbH",
    "20E791": "Siemens Healthcare Diagnostics, Inc",
    "089758": "Shenzhen Strong Rising Electronics Co.,Ltd DongGuan Subsidiary",
    "FC19D0": "Cloud Vision Networks Technology Co.,Ltd.",
    "CC3429": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "7CBF88": "Mobilicom LTD",
    "60DB2A": "HNS",
    "9486D4": "Surveillance Pro Corporation",
    "B424E7": "Codetek Technology Co.,Ltd",
    "A0A23C": "GPMS",
    "68FCB3": "Next Level Security Systems, Inc.",
    "542F89": "Euclid Laboratories, Inc.",
    "2847AA": "Nokia Corporation",
    "5CD61F": "Qardio, Inc",
    "CCFB65": "Nintendo Co., Ltd.",
    "FCD817": "Beijing Hesun Technologies Co.Ltd.",
    "34885D": "Logitech Far East",
    "88576D": "XTA Electronics Ltd",
    "BC4100": "CODACO ELECTRONIC s.r.o.",
    "5027C7": "TECHNART Co.,Ltd",
    "6C5AB5": "TCL Technoly Electronics (Huizhou) Co., Ltd.",
    "B43E3B": "Viableware, Inc",
    "0C5CD8": "DOLI Elektronik GmbH",
    "EC3E09": "PERFORMANCE DESIGNED PRODUCTS, LLC",
    "947C3E": "Polewall Norge AS",
    "385AA8": "Beijing Zhongdun Security Technology Development Co.",
    "F4A294": "EAGLE WORLD DEVELOPMENT CO., LIMITED",
    "9C443D": "CHENGDU XUGUANG TECHNOLOGY CO, LTD",
    "3C15EA": "TESCOM CO., LTD.",
    "E80410": "Private",
    "909916": "ELVEES NeoTek OJSC",
    "A0143D": "PARROT SA",
    "6024C1": "Jiangsu Zhongxun Electronic Technology Co., Ltd",
    "60A9B0": "Merchandising Technologies, Inc",
    "34A3BF": "Terewave. Inc.",
    "8C088B": "Remote Solution",
    "FC1BFF": "V-ZUG AG",
    "00A2FF": "abatec group AG",
    "F4BD7C": "Chengdu jinshi communication Co., LTD",
    "DCC422": "Systembase Limited",
    "C8F36B": "Yamato Scale Co.,Ltd.",
    "6CD1B0": "WING SING ELECTRONICS HONG KONG LIMITED",
    "98F8C1": "IDT Technology Limited",
    "A4F522": "CHOFU SEISAKUSHO CO.,LTD",
    "845C93": "Chabrier Services",
    "68E166": "Private",
    "60FEF9": "Thomas & Betts",
    "B8DC87": "IAI Corporation",
    "78CB33": "DHC Software Co.,Ltd",
    "DCF755": "SITRONIK",
    "BC2BD7": "Revogi Innovation Co., Ltd.",
    "7C6FF8": "ShenZhen ACTO Digital Video Technology Co.,Ltd.",
    "286D97": "SAMJIN Co., Ltd.",
    "24ECD6": "CSG Science & Technology Co.,Ltd.Hefei",
    "CC2A80": "Micro-Biz intelligence solutions Co.,Ltd",
    "F42896": "SPECTO PAINEIS ELETRONICOS LTDA",
    "1C48F9": "GN Netcom A/S",
    "9C4EBF": "BoxCast",
    "34A843": "KYOCERA Display Corporation",
    "80BBEB": "Satmap Systems Ltd",
    "00B78D": "Nanjing Shining Electric Automation Co., Ltd",
    "5C026A": "Applied Vision Corporation",
    "0C9301": "PT. Prasimax Inovasi Teknologi",
    "746630": "T:mi Ytti",
    "6CB350": "Anhui comhigher tech co.,ltd",
    "E89218": "Arcontia International AB",
    "0075E1": "Ampt, LLC",
    "D46A91": "SnapAV",
    "B0793C": "Revolv Inc",
    "3859F8": "MindMade Sp. z o.o.",
    "4CDF3D": "TEAM ENGINEERS ADVANCE TECHNOLOGIES INDIA PVT LTD",
    "98CDB4": "Virident Systems, Inc.",
    "A42305": "Open Networking Laboratory",
    "A05B21": "ENVINET GmbH",
    "50B8A2": "ImTech Technologies LLC,",
    "B04C05": "Fresenius Medical Care Deutschland GmbH",
    "34CD6D": "CommSky Technologies",
    "B0FEBD": "Private",
    "60699B": "isepos GmbH",
    "D4D50D": "Southwest Microwave, Inc",
    "E438F2": "Advantage Controls",
    "C4C755": "Beijing HuaqinWorld Technology Co.,Ltd",
    "0C2D89": "QiiQ Communications Inc.",
    "A8D236": "Lightware Visual Engineering",
    "788DF7": "Hitron Technologies. Inc",
    "945047": "Rechnerbetriebsgruppe",
    "E031D0": "SZ Telstar CO., LTD",
    "54112F": "Sulzer Pump Solutions Finland Oy",
    "4C55B8": "Turkcell Teknoloji",
    "088039": "Cisco SPVTG",
    "981094": "Shenzhen Vsun communication technology Co.,ltd",
    "A4F3C1": "Open Source Robotics Foundation, Inc.",
    "141330": "Anakreon UK LLP",
    "74CA25": "Calxeda, Inc.",
    "B4346C": "MATSUNICHI DIGITAL TECHNOLOGY (HONG KONG) LIMITED",
    "9046B7": "Vadaro Pte Ltd",
    "04CF25": "MANYCOLORS, INC.",
    "B050BC": "SHENZHEN BASICOM ELECTRONIC CO.,LTD.",
    "841E26": "KERNEL-I Co.,LTD",
    "DC7014": "Private",
    "E4F3E3": "Shanghai iComhome Co.,Ltd.",
    "0086A0": "Private",
    "60FE1E": "China Palms Telecom.Ltd",
    "0CF405": "Beijing Signalway Technologies Co.,Ltd",
    "5061D6": "Indu-Sol GmbH",
    "04BFA8": "ISB Corporation",
    "8CC7D0": "zhejiang ebang communication co.,ltd",
    "B8AE6E": "Nintendo Co., Ltd.",
    "D0EB03": "Zhehua technology limited",
    "683EEC": "ERECA",
    "0C2AE7": "Beijing General Research Institute of Mining and Metallurgy",
    "983071": "DAIKYUNG VASCOM",
    "D49524": "Clover Network, Inc.",
    "0C0400": "Jantar d.o.o.",
    "687CD5": "Y Soft Corporation, a.s.",
    "C04DF7": "SERELEC",
    "50A715": "Aboundi, Inc.",
    "C42628": "Airo Wireless",
    "30AABD": "Shanghai Reallytek Information Technology Co.,Ltd",
    "A4B818": "PENTA Gesellschaft f\u00fcr elektronische Industriedatenverarbeitung mbH",
    "907AF1": "Wally",
    "2CB693": "Radware",
    "AC4122": "Eclipse Electronic Systems Inc.",
    "0C8484": "Zenovia Electronics Inc.",
    "005907": "LenovoEMC Products USA, LLC",
    "A861AA": "Cloudview Limited",
    "C8B373": "Cisco-Linksys, LLC",
    "2C245F": "Babolat VS",
    "905692": "Autotalks Ltd.",
    "FC1186": "Logic3 plc",
    "C8EEA6": "Shenzhen SHX Technology Co., Ltd",
    "2481AA": "KSH International Co., Ltd.",
    "E01877": "FUJITSU LIMITED",
    "E457A8": "Stuart Manufacturing, Inc.",
    "789966": "Musilab Electronics (DongGuan)Co.,Ltd.",
    "841715": "GP Electronics (HK) Ltd.",
    "58EB14": "Proteus Digital Health",
    "C458C2": "Shenzhen TATFOOK Technology Co., Ltd.",
    "D0CDE1": "Scientech Electronics",
    "104D77": "Innovative Computer Engineering",
    "5CE0CA": "FeiTian United (Beijing) System Technology Co., Ltd.",
    "E08177": "GreenBytes, Inc.",
    "9C9811": "Guangzhou Sunrise Electronics Development Co., Ltd",
    "B86091": "Onnet Technologies and Innovations LLC",
    "301518": "Ubiquitous Communication Co. ltd.",
    "D0D471": "MVTECH co., Ltd",
    "0868D0": "Japan System Design",
    "D4223F": "Lenovo Mobile Communication Technology Ltd.",
    "E8E875": "iS5 Communications Inc.",
    "C80E95": "OmniLync Inc.",
    "6897E8": "Society of Motion Picture &amp; Television Engineers",
    "7CA15D": "GN ReSound A/S",
    "3C081E": "Beijing Yupont Electric Power Technology Co.,Ltd",
    "FC58FA": "Shen Zhen Shi Xin Zhong Xin Technology Co.,Ltd.",
    "60BB0C": "Beijing HuaqinWorld Technology Co,Ltd",
    "10B9FE": "Lika srl",
    "A42C08": "Masterwork Automodules",
    "8C078C": "FLOW DATA INC",
    "F8DFA8": "zte corporation",
    "A895B0": "Aker Subsea Ltd",
    "28CBEB": "One",
    "B8C46F": "PRIMMCON INDUSTRIES INC",
    "D8B02E": "Guangzhou Zonerich Business Machine Co., LTD.",
    "4CCC34": "Motorola Solutions Inc.",
    "30055C": "Brother industries, LTD.",
    "080EA8": "Velex s.r.l.",
    "C4E032": "IEEE 1904.1 Working Group",
    "8C76C1": "Goden Tech Limited",
    "08F1B7": "Towerstream Corpration",
    "C044E3": "Shenzhen Sinkna Electronics Co., LTD",
    "18550F": "Cisco SPVTG",
    "187A93": "AMICCOM Electronics Corporation",
    "8887DD": "DarbeeVision Inc.",
    "30C82A": "WI-BIZ srl",
    "C45DD8": "HDMI Forum",
    "C4EBE3": "RRCN SAS",
    "94756E": "QinetiQ North America",
    "4C1A95": "Novakon Co., Ltd.",
    "A00363": "Robert Bosch Healthcare GmbH",
    "6499A0": "AG Elektronik AB",
    "848E96": "Embertec Pty Ltd",
    "449B78": "The Now Factory",
    "DCA989": "MACANDC",
    "7C438F": "E-Band Communications Corp.",
    "F0F669": "Motion Analysis Corporation",
    "78995C": "Nationz Technologies Inc",
    "54115F": "Atamo Pty Ltd",
    "8CAE4C": "Plugable Technologies",
    "0CC655": "Wuxi YSTen Technology Co.,Ltd.",
    "5C89D4": "Beijing Banner Electric Co.,Ltd",
    "182A7B": "Nintendo Co., Ltd.",
    "68FB95": "Generalplus Technology Inc.",
    "D0B498": "Robert Bosch LLC Automotive Electronics",
    "E05597": "Emergent Vision Technologies Inc.",
    "242FFA": "Toshiba Global Commerce Solutions",
    "849DC5": "Centera Photonics Inc.",
    "580943": "Private",
    "ECFC55": "A. Eberle GmbH & Co. KG",
    "88A3CC": "Amatis Controls",
    "C0A0C7": "FAIRFIELD INDUSTRIES",
    "A0E25A": "Amicus SK, s.r.o.",
    "D40FB2": "Applied Micro Electronics AME bv",
    "485A3F": "WISOL",
    "70F1E5": "Xetawave LLC",
    "E496AE": "ALTOGRAPHICS Inc.",
    "F8F082": "NAGTECH LLC",
    "4C2258": "cozybit, Inc.",
    "F45214": "Mellanox Technologies, Inc.",
    "10F49A": "T3 Innovation",
    "3C57BD": "Kessler Crane Inc.",
    "04E9E5": "PJRC.COM, LLC",
    "50465D": "ASUSTek COMPUTER INC.",
    "74BFA1": "HYUNTECK",
    "08B738": "Lite-On Technogy Corp.",
    "F8AA8A": "Axview Technology (Shenzhen) Co.,Ltd",
    "7C0187": "Curtis Instruments, Inc.",
    "54F666": "Berthold Technologies GmbH and Co.KG",
    "1C11E1": "Wartsila Finland Oy",
    "703811": "Siemens Mobility Limited",
    "CC4BFB": "Hellberg Safety AB",
    "6CADEF": "KZ Broadband Technologies, Ltd. ",
    "745FAE": "TSL PPL",
    "60BD91": "Move Innovation",
    "6851B7": "PowerCloud Systems, Inc.",
    "1C959F": "Veethree Electronics And Marine LLC",
    "B4009C": "CableWorld Ltd.",
    "2C5AA3": "PROMATE ELECTRONIC CO.LTD",
    "34E0CF": "zte corporation",
    "F05F5A": "Getriebebau NORD GmbH and Co. KG",
    "CC262D": "Verifi, LLC",
    "3C8AE5": "Tensun Information Technology(Hangzhou) Co.,LTD",
    "C0AA68": "OSASI Technos Inc.",
    "88D7BC": "DEP Company",
    "F49466": "CountMax,  ltd",
    "742D0A": "Norfolk Elektronik AG",
    "34C803": "Nokia Corporation",
    "80AAA4": "USAG",
    "A40BED": "Carry Technology Co.,Ltd",
    "702393": "fos4X GmbH",
    "F85F2A": "Nokia Corporation",
    "C438D3": "TAGATEC CO.,LTD",
    "6CE4CE": "Villiger Security Solutions AG",
    "803FD6": "bytes at work AG",
    "E85BF0": "Imaging Diagnostics",
    "F82285": "Cypress Technology CO., LTD.",
    "24EE3A": "Chengdu Yingji Electronic Hi-tech Co Ltd",
    "0CC66A": "Nokia Corporation",
    "74273C": "ChangYang Technology (Nanjing) Co., LTD",
    "087CBE": "Quintic Corp.",
    "044A50": "Ramaxel Technology (Shenzhen) limited company",
    "38A5B6": "SHENZHEN MEGMEET ELECTRICAL CO.,LTD",
    "0CD9C1": "Visteon Corporation",
    "68AB8A": "RF IDeas",
    "E804F3": "Throughtek Co., Ltd.",
    "289EDF": "Danfoss Turbocor Compressors, Inc",
    "784405": "FUJITU(HONG KONG) ELECTRONIC Co.,LTD.",
    "C4AD21": "MEDIAEDGE Corporation",
    "0868EA": "EITO ELECTRONICS CO., LTD.",
    "801DAA": "Avaya Inc",
    "7C092B": "Bekey A/S",
    "842BBC": "Modelleisenbahn GmbH",
    "C401B1": "SeekTech INC",
    "C0C946": "MITSUYA LABORATORIES INC.",
    "E8CBA1": "Nokia Corporation",
    "E0AAB0": "SUNTAILI ENTERPRISE CO. LTD,",
    "649FF7": "Kone OYj",
    "20C1AF": "i Wit Digital Co., Limited",
    "085B0E": "Fortinet, Inc.",
    "EC42F0": "ADL Embedded Solutions, Inc.",
    "502ECE": "Asahi Electronics Co.,Ltd",
    "AC14D2": "wi-daq, inc.",
    "9C4CAE": "Mesa Labs",
    "18421D": "Private",
    "28C914": "Taimag Corporation",
    "7493A4": "Zebra Technologies Corp.",
    "E47185": "Securifi Ltd",
    "802AFA": "Germaneers GmbH",
    "30AEF6": "Radio Mobile Access",
    "F4600D": "Panoptic Technology, Inc",
    "A82BD6": "Shina System Co., Ltd",
    "ACCF23": "Hi-flying electronics technology Co.,Ltd",
    "CC912B": "TE Connectivity Touch Solutions",
    "C05E79": "SHENZHEN HUAXUN ARK TECHNOLOGIES CO.,LTD",
    "C8AE9C": "Shanghai TYD Elecronic Technology Co. Ltd",
    "080CC9": "Mission Technology Group, dba Magma",
    "640E94": "Pluribus Networks, Inc.",
    "0CB4EF": "Digience Co.,Ltd.",
    "AC40EA": "C&T Solution Inc. ",
    "002AAF": "LARsys-Automation GmbH",
    "1CE165": "Marshal Corporation",
    "4016FA": "EKM Metering",
    "0C130B": "Uniqoteq Ltd.",
    "58BFEA": "Cisco Systems, Inc",
    "FC1D59": "I Smart Cities HK Ltd",
    "78C4AB": "Shenzhen Runsil Technology Co.,Ltd",
    "146A0B": "Cypress Electronics Limited",
    "F490EA": "Deciso B.V.",
    "2C542D": "Cisco Systems, Inc",
    "AC3FA4": "TAIYO YUDEN CO.,LTD",
    "6CAE8B": "IBM Corporation",
    "BC1401": "Hitron Technologies. Inc",
    "5CEE79": "Global Digitech Co LTD",
    "80CEB1": "Theissen Training Systems GmbH",
    "FC2A54": "Connected Data, Inc.",
    "40AC8D": "Data Management, Inc.",
    "4CAA16": "AzureWave Technologies (Shanghai) Inc.",
    "609084": "DSSD Inc",
    "782544": "Omnima Limited",
    "A41875": "Cisco Systems, Inc",
    "045C06": "Zmodo Technology Corporation",
    "747B7A": "ETH Inc.",
    "48EA63": "Zhejiang Uniview Technologies Co., Ltd.",
    "E88DF5": "ZNYX Networks, Inc.",
    "642DB7": "SEUNGIL ELECTRONICS",
    "28BA18": "NextNav, LLC",
    "AC3D75": "HANGZHOU ZHIWAY TECHNOLOGIES CO.,LTD.",
    "A090DE": "VEEDIMS,LLC",
    "F436E1": "Abilis Systems SARL",
    "94CA0F": "Honeywell Analytics",
    "2C2D48": "bct electronic GesmbH",
    "284121": "OptiSense Network, LLC",
    "38458C": "MyCloud Technology corporation",
    "10E4AF": "APR, LLC",
    "D05785": "Pantech Co., Ltd.",
    "408B07": "Actiontec Electronics, Inc",
    "90F72F": "Phillips Machine & Welding Co., Inc. ",
    "F4EA67": "Cisco Systems, Inc",
    "04F021": "Compex Systems Pte Ltd",
    "342F6E": "Anywire corporation",
    "BC2C55": "Bear Flag Design, Inc.",
    "0C7523": "BEIJING GEHUA CATV NETWORK CO.,LTD",
    "B08E1A": "URadio Systems Co., Ltd",
    "D8E952": "KEOPSYS",
    "940070": "Nokia Corporation",
    "E80C75": "Syncbak, Inc.",
    "CCEED9": "VAHLE Automation GmbH",
    "608C2B": "Hanson Technology",
    "645299": "The Chamberlain Group, Inc",
    "800A06": "COMTEC co.,ltd",
    "002A6A": "Cisco Systems, Inc",
    "40E793": "Shenzhen Siviton Technology Co.,Ltd",
    "000831": "Cisco Systems, Inc",
    "34D09B": "MobilMAX Technology Inc.",
    "BCA4E1": "Nabto",
    "2818FD": "Aditya Infotech Ltd.",
    "D8B90E": "Triple Domain Vision Co.,Ltd.",
    "34BA9A": "Asiatelco Technologies Co.",
    "F0007F": "Janz - Contadores de Energia, SA",
    "30B3A2": "Shenzhen Heguang Measurement & Control Technology Co.,Ltd",
    "506028": "Xirrus Inc.",
    "0091FA": "Synapse Product Development",
    "A05AA4": "Grand Products Nevada, Inc.",
    "F0EEBB": "VIPAR GmbH",
    "E4FA1D": "PAD Peripheral Advanced Design Inc.",
    "005CB1": "Gospell DIGITAL TECHNOLOGY CO., LTD",
    "1C5C55": "PRIMA Cinema, Inc",
    "908FCF": "UNO System Co., Ltd",
    "6CE907": "Nokia Corporation",
    "984A47": "CHG Hospital Beds",
    "144978": "Digital Control Incorporated",
    "2C10C1": "Nintendo Co., Ltd.",
    "8CD17B": "CG Mobile",
    "3C6A7D": "Niigata Power Systems Co., Ltd.",
    "502267": "PixeLINK",
    "506441": "Greenlee",
    "9C1FDD": "Accupix Inc.",
    "7CDD11": "Chongqing MAS SCI&TECH.Co.,Ltd",
    "B8FD32": "Zhejiang ROICX Microelectronics",
    "0064A6": "Maquet CardioVascular",
    "988BAD": "Corintech Ltd.",
    "D44B5E": "TAIYO YUDEN CO., LTD.",
    "640E36": "TAZTAG",
    "C8AF40": "marco Systemanalyse und Entwicklung GmbH",
    "40984C": "Casacom Solutions AG",
    "3C7059": "MakerBot Industries",
    "502690": "FUJITSU LIMITED",
    "5C18B5": "Talon Communications",
    "64E161": "DEP Corp.",
    "8823FE": "TTTech Computertechnik AG",
    "70EE50": "Netatmo",
    "D4F63F": "IEA S.R.L.",
    "58B0D4": "ZuniData Systems Inc.",
    "64557F": "NSFOCUS Information Technology Co., Ltd.",
    "00082F": "Cisco Systems, Inc",
    "386077": "PEGATRON CORPORATION",
    "708105": "Cisco Systems, Inc",
    "941D1C": "TLab West Systems AB",
    "94AE61": "Alcatel Lucent",
    "5CCEAD": "CDYNE Corporation",
    "AC54EC": "IEEE P1823 Standards Working Group",
    "709756": "Happyelectronics Co.,Ltd",
    "E455EA": "Dedicated Computing",
    "B89AED": "OceanServer Technology, Inc",
    "C87D77": "Shenzhen Kingtech Communication Equipment Co.,Ltd",
    "B820E7": "Guangzhou Horizontal Information & Network Integration Co. Ltd",
    "00CD90": "MAS Elektronik AG",
    "7C6B52": "Tigaro Wireless",
    "E0ED1A": "vastriver Technology Co., Ltd",
    "685E6B": "PowerRay Co., Ltd.",
    "B05CE5": "Nokia Corporation",
    "3482DE": "Kiio Inc",
    "20C8B3": "SHENZHEN BUL-TECH CO.,LTD.",
    "4C5FD2": "Alcatel-Lucent",
    "B8F5E7": "WayTools, LLC",
    "B81999": "Nesys",
    "34255D": "Shenzhen Loadcom Technology Co.,Ltd",
    "FC0012": "Toshiba Samsung Storage Technolgoy Korea Corporation ",
    "C029F3": "XySystem",
    "94DE0E": "SmartOptics AS",
    "807A7F": "ABB Genway Xiamen Electrical Equipment CO., LTD",
    "24DAB6": "Sistemas de Gesti\u00f3n Energ\u00e9tica S.A. de C.V",
    "B07D62": "Dipl.-Ing. H. Horstmann GmbH",
    "9CC7D1": "SHARP Corporation",
    "149090": "KongTop industrial(shen zhen)CO.,LTD",
    "28C718": "Altierre",
    "7C4C58": "Scale Computing, Inc.",
    "1013EE": "Justec International Technology INC.",
    "8C271D": "QuantHouse",
    "38DE60": "Mohlenhoff GmbH",
    "2839E7": "Preceno Technology Pte.Ltd.",
    "D4CEB8": "Enatel LTD",
    "30688C": "Reach Technology Inc.",
    "F8E7B5": "\u00b5Tech Tecnologia LTDA",
    "10EED9": "Canoga Perkins Corporation",
    "181420": "TEB SAS",
    "4CA74B": "Alcatel Lucent",
    "1CE192": "Qisda Corporation",
    "706F81": "Private",
    "DCF05D": "Letta Teknoloji",
    "8CB82C": "IPitomy Communications",
    "807DE3": "Chongqing Sichuan Instrument Microcircuit Co.LTD.",
    "DC175A": "Hitachi High-Technologies Corporation",
    "900D66": "Digimore Electronics Co., Ltd",
    "AC4AFE": "Hisense Broadband Multimedia Technology Co.,Ltd.",
    "54F5B6": "ORIENTAL PACIFIC INTERNATIONAL LIMITED",
    "90342B": "Gatekeeper Systems, Inc.",
    "5C16C7": "Arista Networks",
    "3C096D": "Powerhouse Dynamics",
    "C8A1BA": "Neul Ltd",
    "C43A9F": "Siconix Inc.",
    "686E23": "Wi3 Inc.",
    "F49461": "NexGen Storage",
    "0C924E": "Rice Lake Weighing Systems",
    "B8CDA7": "Maxeler Technologies Ltd.",
    "A4B36A": "JSC SDO Chromatec",
    "F43D80": "FAG Industrial Services GmbH",
    "5435DF": "Symeo GmbH",
    "48DCFB": "Nokia Corporation",
    "F0DB30": "Yottabyte",
    "9C31B6": "Kulite Semiconductor Products Inc",
    "10C586": "BIO SOUND LAB CO., LTD.",
    "10768A": "EoCell",
    "0C3956": "Observator instruments",
    "DCA6BD": "Beijing Lanbo Technology Co., Ltd.",
    "A45A1C": "smart-electronic GmbH",
    "806459": "Nimbus Inc.",
    "8C89A5": "Micro-Star INT'L CO., LTD",
    "B4A5A9": "MODI GmbH",
    "E8C320": "Austco Marketing & Service (USA) ltd.",
    "C436DA": "Rusteletech Ltd.",
    "688470": "eSSys Co.,Ltd",
    "8C5CA1": "d-broad,INC",
    "D43AE9": "DONGGUAN ipt INDUSTRIAL CO., LTD",
    "589835": "Technicolor Delivery Technologies Belgium NV",
    "E8CC32": "Micronet  LTD",
    "E4DD79": "En-Vision America, Inc.",
    "18F650": "Multimedia Pacific Limited",
    "20B7C0": "OMICRON electronics GmbH",
    "8058C5": "NovaTec Kommunikationstechnik GmbH",
    "B8C716": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D42C3D": "Sky Light Digital Limited",
    "1C184A": "ShenZhen RicherLink Technologies Co.,LTD",
    "F44EFD": "Actions Semiconductor Co.,Ltd.(Cayman Islands)",
    "24B8D2": "Opzoon Technology Co.,Ltd.",
    "A49981": "FuJian Elite Power Tech CO.,LTD.",
    "B83A7B": "Worldplay (Canada) Inc.",
    "D0EB9E": "Seowoo Inc.",
    "78028F": "Adaptive Spectrum and Signal Alignment (ASSIA), Inc.",
    "24C9DE": "Genoray",
    "54055F": "Alcatel Lucent",
    "6C5D63": "ShenZhen Rapoo Technology Co., Ltd.",
    "0432F4": "Partron",
    "1407E0": "Abrantix AG",
    "DCCF94": "Beijing Rongcheng Hutong Technology Co., Ltd.",
    "A4DB2E": "Kingspan Environmental Ltd",
    "DC16A2": "Medtronic Diabetes",
    "308CFB": "Dropcam",
    "BCCD45": "VOISMART",
    "143E60": "Nokia",
    "5C56ED": "3pleplay Electronics Private Limited",
    "941673": "Point Core SARL",
    "C8FE30": "Bejing DAYO Mobile Communication Technology Ltd.",
    "E4D71D": "Oraya Therapeutics",
    "8427CE": "Corporation of the Presiding Bishop of The Church of Jesus Christ of Latter-day Saints",
    "48D8FE": "ClarIDy Solutions, Inc.",
    "70A41C": "Advanced Wireless Dynamics S.L.",
    "7032D5": "Athena Wireless Communications Inc",
    "78510C": "LiveU Ltd.",
    "44AAE8": "Nanotec Electronic GmbH & Co. KG",
    "70E843": "Beijing C&W Optical Communication Technology Co.,Ltd.",
    "2C7ECF": "Onzo Ltd",
    "74B00C": "Network Video Technologies, Inc",
    "BCBBC9": "Kellendonk Elektronik GmbH",
    "B42A39": "ORBIT MERRET, spol. s r. o.",
    "D4945A": "COSMO CO., LTD",
    "304C7E": "Panasonic Electric Works Automation Controls Techno Co.,Ltd.",
    "5CF207": "Speco Technologies",
    "E84040": "Cisco Systems, Inc",
    "D89DB9": "eMegatech International Corp.",
    "405A9B": "ANOVO",
    "103711": "NORBIT ITS",
    "B4B88D": "Thuh Company",
    "4C73A5": "KOVE",
    "D428B2": "ioBridge, Inc.",
    "E83EB6": "RIM",
    "BC35E5": "Hydro Systems Company",
    "28CCFF": "Corporacion Empresarial Altra SL",
    "94FD1D": "WhereWhen Corp",
    "4C07C9": "COMPUTER OFFICE Co.,Ltd.",
    "00B342": "MacroSAN Technologies Co., Ltd.",
    "9C5D95": "VTC Electronics Corp.",
    "B8A8AF": "Logic S.p.A.",
    "60F673": "TERUMO CORPORATION",
    "E42AD3": "Magneti Marelli S.p.A. Powertrain",
    "1CF5E7": "Turtle Industry Co., Ltd.",
    "980EE4": "Private",
    "447DA5": "VTION INFORMATION TECHNOLOGY (FUJIAN) CO.,LTD",
    "0CCDD3": "EASTRIVER TECHNOLOGY CO., LTD.",
    "84DE3D": "Crystal Vision Ltd",
    "E06995": "PEGATRON CORPORATION",
    "F8769B": "Neopis Co., Ltd.",
    "C89C1D": "Cisco Systems, Inc",
    "D075BE": "Reno A&E",
    "BC6E76": "Green Energy Options Ltd",
    "E828D5": "Cots Technology",
    "F8DAF4": "Taishan Online Technology Co., Ltd.",
    "08D5C0": "Seers Technology Co., Ltd",
    "6C33A9": "Magicjack LP",
    "108CCF": "Cisco Systems, Inc",
    "D8E3AE": "CIRTEC MEDICAL SYSTEMS",
    "CC7669": "SEETECH",
    "AC20AA": "DMATEK Co., Ltd.",
    "E437D7": "HENRI DEPAEPE S.A.S.",
    "E0A1D7": "SFR",
    "9481A4": "Azuray Technologies",
    "BCE09D": "Eoslink",
    "BC4377": "Hang Zhou Huite Technology Co.,ltd.",
    "7CDD90": "Shenzhen Ogemray Technology Co., Ltd.",
    "9C220E": "TASCAN Systems GmbH",
    "0C3C65": "Dome Imaging Inc",
    "C8DF7C": "Nokia Corporation",
    "FCAF6A": "Qulsar Inc",
    "081735": "Cisco Systems, Inc",
    "CCBE71": "OptiLogix BV",
    "0C469D": "MS Sedco",
    "00B033": "OAO \"Izhevskiy radiozavod\"",
    "08B7EC": "Wireless Seismic",
    "18AF9F": "DIGITRONIC Automationsanlagen GmbH",
    "E46C21": "messMa GmbH",
    "E0F379": "Vaddio",
    "78B6C1": "AOBO Telecom Co.,Ltd",
    "B09AE2": "STEMMER IMAGING GmbH",
    "14EE9D": "AirNav Systems LLC",
    "B44CC2": "NR ELECTRIC CO., LTD",
    "48CB6E": "Cello Electronics (UK) Ltd",
    "006DFB": "Vutrix Technologies Ltd",
    "78D004": "Neousys Technology Inc.",
    "8895B9": "Unified Packet Systems Crop",
    "78A051": "iiNet Labs Pty Ltd ",
    "804F58": "ThinkEco, Inc.",
    "448C52": "KTIS CO., Ltd",
    "346F92": "White Rodgers Division",
    "34BDF9": "Shanghai WDK Industrial Co.,Ltd.",
    "D8FE8F": "IDFone Co., Ltd.",
    "888C19": "Brady Corp Asia Pacific Ltd",
    "08D29A": "Proformatique",
    "C89383": "Embedded Automation, Inc.",
    "9067B5": "Alcatel-Lucent",
    "0475F5": "CSST",
    "E80C38": "DAEYOUNG INFORMATION SYSTEM CO., LTD",
    "E08A7E": "Exponent",
    "8C4DEA": "Cerio Corporation",
    "34DF2A": "Fujikon Industrial Co.,Limited",
    "70A191": "Trendsetter Medical, LLC",
    "24BA30": "Technical Consumer Products, Inc.",
    "188ED5": "TP Vision Belgium N.V. - innovation site Brugge",
    "40B2C8": "Nortel Networks",
    "708B78": "citygrow technology co., ltd",
    "A8B0AE": "BizLink Special Cables Germany GmbH",
    "E42771": "Smartlabs",
    "70B08C": "Shenou Communication Equipment Co.,Ltd",
    "C03B8F": "Minicom Digital Signage",
    "20FEDB": "M2M Solution S.A.S.",
    "C8A729": "SYStronics Co., Ltd.",
    "4454C0": "Thompson Aerospace",
    "64317E": "Dexin Corporation",
    "3C99F7": "Lansentechnology AB",
    "507D02": "BIODIT",
    "B4A4E3": "Cisco Systems, Inc",
    "94DD3F": "A+V Link Technologies, Corp.",
    "F44227": "S & S Research Inc.",
    "8C1F94": "RF Surgical System Inc. ",
    "4491DB": "Shanghai Huaqin Telecom Technology Co.,Ltd",
    "A8556A": "3S System Technology Inc.",
    "D0574C": "Cisco Systems, Inc",
    "F8DAE2": "NDC Technologies",
    "AC83F0": "Cobalt Digital Inc.",
    "CC6B98": "Minetec Wireless Technologies",
    "3C04BF": "PRAVIS SYSTEMS Co.Ltd.,",
    "7C55E7": "YSI, Inc.",
    "C4F464": "Spica international",
    "602A54": "CardioTek B.V.",
    "BCFFAC": "TOPCON CORPORATION",
    "14D76E": "CONCH ELECTRONIC Co.,Ltd",
    "445EF3": "Tonalite Holding B.V.",
    "68DB96": "OPWILL Technologies CO .,LTD",
    "94A7BC": "BodyMedia, Inc.",
    "C8A1B6": "Shenzhen Longway Technologies Co., Ltd",
    "38580C": "Panaccess Systems GmbH",
    "4451DB": "Raytheon BBN Technologies",
    "585076": "Linear Equipamentos Eletronicos SA",
    "F0F9F7": "IES GmbH & Co. KG",
    "64A232": "OOO Samlight",
    "64FC8C": "Zonar Systems",
    "0C8D98": "TOP EIGHT IND CORP",
    "40C7C9": "Naviit Inc.",
    "9803A0": "ABB n.v. Power Quality Products",
    "DCFAD5": "STRONG Ges.m.b.H.",
    "6C8D65": "Wireless Glue Networks, Inc.",
    "7CBB6F": "Cosco Electronics Co., Ltd.",
    "20B0F7": "Enclustra GmbH",
    "B4C810": "Umpi srl",
    "543131": "Raster Vision Ltd",
    "D0E347": "Yoga",
    "705EAA": "Action Target, Inc.",
    "34F968": "ATEK Products, LLC",
    "F8C091": "Highgates Technology",
    "AC9B84": "Smak Tecnologia e Automacao",
    "90F278": "Radius Gateway",
    "806629": "Prescope Technologies CO.,LTD.",
    "4C60D5": "airPointe of New Hampshire",
    "842914": "EMPORIA TELECOM Produktions- und VertriebsgesmbH & Co KG",
    "BC7DD1": "Radio Data Comms",
    "646707": "Beijing Omnific Technology, Ltd.",
    "58FD20": "Systemhouse Solutions AB",
    "F0ED1E": "Bilkon Bilgisayar Kontrollu Cih. Im.Ltd.",
    "C416FA": "Prysm Inc",
    "506F9A": "Wi-Fi Alliance",
    "241F2C": "Calsys, Inc.",
    "F0BDF1": "Sipod Inc.",
    "CC43E3": "Trump s.a.",
    "ACA016": "Cisco Systems, Inc",
    "58E747": "Deltanet AG",
    "40618E": "Stella-Green Co",
    "68E41F": "Unglaube Identech GmbH",
    "389F83": "OTN Systems N.V.",
    "BC6A16": "tdvine",
    "003A9D": "NEC Platforms, Ltd.",
    "009363": "Uni-Link Technology Co., Ltd.",
    "7C7673": "ENMAS GmbH",
    "003532": "Electro-Metrics Corporation",
    "081FF3": "Cisco Systems, Inc",
    "44376F": "Young Electric Sign Co",
    "6CDC6A": "Promethean Limited",
    "9055AE": "Ericsson, EAB/RWI/K",
    "84C727": "Gnodal Ltd",
    "28CD4C": "Individual Computers GmbH",
    "8C53F7": "A&D ENGINEERING CO., LTD.",
    "404022": "ZIV",
    "A85BB0": "Shenzhen Dehoo Technology Co.,Ltd",
    "888717": "CANON INC.",
    "1010B6": "McCain Inc",
    "D4823E": "Argosy Technologies, Ltd.",
    "44A689": "PROMAX ELECTRONICA SA",
    "087695": "Auto Industrial Co., Ltd.",
    "ACCE8F": "HWA YAO TECHNOLOGIES CO., LTD",
    "8C9236": "Aus.Linx Technology Co., Ltd.",
    "F8912A": "GLP German Light Products GmbH",
    "10C73F": "Midas Klark Teknik Ltd",
    "44E49A": "OMNITRONICS PTY LTD",
    "08F2F4": "Net One Partners Co.,Ltd.",
    "0C7D7C": "Kexiang Information Technology Co, Ltd.",
    "B482FE": "ASKEY COMPUTER CORP",
    "307C30": "RIM",
    "BC4E3C": "CORE STAFF CO., LTD.",
    "3037A6": "Cisco Systems, Inc",
    "DC1D9F": "U & B tech",
    "B09074": "Fulan Electronics Limited",
    "94F692": "Geminico co.,Ltd.",
    "785C72": "Hioso Technology Co., Ltd.",
    "580556": "Elettronica GF S.r.L.",
    "10445A": "Shaanxi Hitech Electronic Co., LTD",
    "F47626": "Viltechmeda UAB ",
    "68EFBD": "Cisco Systems, Inc",
    "F02FD8": "Bi2-Vision",
    "502A8B": "Telekom Research and Development Sdn Bhd",
    "EC43E6": "AWCER Ltd.",
    "F02408": "Talaris (Sweden) AB",
    "8081A5": "TONGQING COMMUNICATION EQUIPMENT (SHENZHEN) Co.,Ltd",
    "8894F9": "Gemicom Technology, Inc.",
    "502A7E": "Smart electronic GmbH",
    "F0264C": "Sigrist-Photometer AG",
    "544249": "Sony Corporation",
    "0C17F1": "TELECSYS",
    "7812B8": "ORANTEK LIMITED",
    "10B7F6": "Plastoform Industries Ltd.",
    "448E81": "VIG",
    "0C8411": "A.O. Smith Water Products",
    "98BC99": "Edeltech Co.,Ltd.",
    "904716": "RORZE CORPORATION",
    "A4DA3F": "Bionics Corp.",
    "A8C222": "TM-Research Inc.",
    "003D41": "Hatteland Computer AS",
    "CC5076": "Ocom Communications, Inc.",
    "5C8778": "Cybertelbridge co.,ltd",
    "38BB23": "OzVision America LLC",
    "003A9B": "Cisco Systems, Inc",
    "2C3427": "ERCO & GENER",
    "80912A": "Lih Rong electronic Enterprise Co., Ltd.",
    "803B9A": "ghe-ces electronic ag",
    "743256": "NT-ware Systemprg GmbH",
    "4CC452": "Shang Hai Tyd. Electon Technology Ltd.",
    "7CCB0D": "Antaira Technologies, LLC",
    "C01E9B": "Pixavi AS",
    "C4E17C": "U2S co.",
    "20BFDB": "DVL",
    "448312": "Star-Net",
    "E0ABFE": "Orb Networks, Inc.",
    "A05DE7": "DIRECTV, Inc.",
    "087618": "ViE Technologies Sdn. Bhd.",
    "D0E40B": "Wearable Inc.",
    "747E1A": "Red Embedded Design Limited",
    "14A86B": "ShenZhen Telacom Science&Technology Co., Ltd",
    "0CC3A7": "Meritec",
    "DCE2AC": "Lumens Digital Optics Inc.",
    "98D88C": "Nortel Networks",
    "78192E": "NASCENT Technology",
    "48EB30": "ETERNA TECHNOLOGY, INC.",
    "4C322D": "TELEDATA NETWORKS",
    "AC867E": "Create New Technology (HK) Limited Company",
    "8C598B": "C Technologies AB",
    "D44CA7": "Informtekhnika & Communication, LLC",
    "A04025": "Actioncable, Inc.",
    "4C4B68": "Mobile Device, Inc. ",
    "201257": "Most Lucky Trading Ltd",
    "E8DAAA": "VideoHome Technology Corp.",
    "647D81": "YOKOTA INDUSTRIAL CO,.LTD",
    "7CCFCF": "Shanghai SEARI Intelligent System Co., Ltd",
    "DC3350": "TechSAT GmbH",
    "E064BB": "DigiView S.r.l.",
    "68AAD2": "DATECS LTD.,",
    "A4DE50": "Total Walther GmbH",
    "D8D67E": "GSK CNC EQUIPMENT CO.,LTD",
    "C4AAA1": "SUMMIT DEVELOPMENT, spol.s r.o.",
    "1CF061": "SCAPS GmbH",
    "A893E6": "JIANGXI JINGGANGSHAN CKING COMMUNICATION TECHNOLOGY CO.,LTD",
    "3032D4": "Hanilstm Co., Ltd.",
    "04B466": "BSP Co., Ltd.",
    "68CC9C": "Mine Site Technologies",
    "146E0A": "Private",
    "0CE709": "Fox Crypto B.V.",
    "6C0F6A": "JDC Tech Co., Ltd.",
    "F0BCC8": "MaxID (Pty) Ltd",
    "24828A": "Prowave Technologies Ltd.",
    "00271B": "Alec Sicherheitssysteme GmbH",
    "002718": "Suzhou NEW SEAUNION Video Technology Co.,Ltd",
    "00270B": "Adura Technologies",
    "00270D": "Cisco Systems, Inc",
    "002707": "Lift Complex DS, JSC",
    "0026FE": "MKD Technology Inc.",
    "B4B5AF": "Minsung Electronics",
    "04B3B6": "Seamap (UK) Ltd",
    "0026E5": "AEG Power Solutions",
    "0026BC": "General Jack Technology Ltd.",
    "0026AE": "Wireless Measurement Ltd",
    "0026B1": "Navis Auto Motive Systems, Inc.",
    "0026AA": "Kenmec Mechanical Engineering Co., Ltd.",
    "002670": "Cinch Connectors",
    "002671": "AUTOVISION Co., Ltd",
    "0026D7": "KM Electornic Technology Co., Ltd.",
    "0026D0": "Semihalf",
    "0026D2": "Pcube Systems, Inc.",
    "0026A0": "moblic",
    "002696": "NOOLIX Co., Ltd",
    "00269A": "Carina System Co., Ltd.",
    "002695": "ZT Group Int'l Inc",
    "002693": "QVidium Technologies, Inc.",
    "0026CD": "PurpleComm, Inc.",
    "00268E": "Alta Solutions, Inc.",
    "002678": "Logic Instrument SA",
    "002677": "DEIF A/S",
    "0026E3": "DTI",
    "002665": "ProtectedLogic Corporation",
    "002660": "Logiways",
    "002614": "KTNF",
    "002610": "Apacewave Technologies",
    "00260E": "Ablaze Systems, LLC",
    "00260D": "Mercury Systems, Inc.",
    "00260A": "Cisco Systems, Inc",
    "00262C": "IKT Advanced Technologies s.r.o.",
    "00262E": "Chengdu Jiuzhou Electronic Technology Inc",
    "002629": "Juphoon System Software Inc.",
    "002648": "Emitech Corp.",
    "002645": "Circontrol S.A.",
    "00263E": "Trapeze Networks",
    "00263C": "Bachmann Technology GmbH & Co. KG",
    "00263D": "MIA Corporation",
    "002632": "Instrumentation Technologies d.d.",
    "0025E2": "Everspring Industry Co., Ltd.",
    "0025E1": "SHANGHAI SEEYOO ELECTRONIC & TECHNOLOGY CO., LTD",
    "002625": "MediaSputnik",
    "002626": "Geophysical Survey Systems, Inc.",
    "00261B": "LAUREL BANK MACHINES CO., LTD.",
    "0025F9": "GMK electronic design GmbH",
    "0025F7": "Ansaldo STS USA",
    "002603": "Shenzhen Wistar Technology Co., Ltd",
    "0025A6": "Central Network Solution Co., Ltd.",
    "0025AA": "Beijing Soul Technology Co.,Ltd.",
    "0025A3": "Trimax Wireless, Inc.",
    "00259C": "Cisco-Linksys, LLC",
    "0025A2": "Alta Definicion LINCEO S.L.",
    "0025DA": "Secura Key",
    "0025DB": "ATI Electronics(Shenzhen) Co., LTD",
    "0025D5": "Robonica (Pty) Ltd",
    "0025CC": "Mobile Communications Korea Incorporated",
    "0025C8": "S-Access GmbH",
    "0025C0": "ZillionTV Corporation",
    "0025BD": "Italdata Ingegneria dell'Idea S.p.A.",
    "0025B7": "Costar  electronics, inc.,",
    "00257D": "PointRed Telecom Private Ltd.",
    "002588": "Genie Industries, Inc.",
    "002580": "Equipson S.A.",
    "00256D": "Broadband Forum",
    "00256C": "\"Azimut\" Production Association JSC",
    "002563": "Luxtera Inc",
    "002593": "DatNet Informatikai Kft.",
    "00258E": "The Weather Channel",
    "00250D": "GZT Telkom-Telmor sp. z o.o.",
    "00250E": "gt german telematics gmbh",
    "002508": "Maquet Cardiopulmonary AG",
    "002524": "Lightcomm Technology Co., Ltd",
    "002522": "ASRock Incorporation",
    "00251B": "Philips CareServant",
    "002518": "Power PLUS Communications AG",
    "002515": "SFR",
    "002536": "Oki Electric Industry Co., Ltd.",
    "002541": "Maquet Critical Care AB",
    "00254E": "Vertex Wireless Co., Ltd.",
    "002546": "Cisco Systems, Inc",
    "002545": "Cisco Systems, Inc",
    "002542": "Pittasoft",
    "002531": "Cloud Engines, Inc.",
    "00252D": "Kiryung Electronics",
    "00252B": "Stirling Energy Systems",
    "0024FC": "QuoPin Co., Ltd.",
    "002560": "Ibridge Networks & Communications Ltd.",
    "0024B3": "Graf-Syteco GmbH & Co. KG",
    "0024AA": "Dycor Technologies Ltd.",
    "0024D0": "Shenzhen SOGOOD Industry CO.,LTD.",
    "0024CC": "Fascinations Toys and Gifts, Inc.",
    "0024CB": "Autonet Mobile",
    "0024D1": "Thomson Inc.",
    "0024C9": "Broadband Solutions Group",
    "0024CA": "Tobii Technology AB",
    "0024A9": "Ag Leader Technology",
    "0024A6": "TELESTAR DIGITAL GmbH",
    "0024A3": "Sonim Technologies Inc",
    "0024FB": "Private",
    "0024FA": "Hilger u. Kern GMBH",
    "0024F6": "MIYOSHI ELECTRONICS CORPORATION",
    "0024F0": "Seanodes",
    "0024C7": "Mobilarm Ltd",
    "0024B8": "free alliance sdn bhd",
    "00249B": "Action Star Enterprise Co., Ltd.",
    "002497": "Cisco Systems, Inc",
    "0024DF": "Digitalbox Europe GmbH",
    "002425": "Shenzhenshi chuangzhicheng Technology Co.,Ltd",
    "002427": "SSI COMPUTER CORP",
    "00241C": "FuGang Electronic (DG) Co.,Ltd",
    "002419": "Private",
    "002415": "Magnetic Autocontrol GmbH",
    "002411": "PharmaSmart LLC",
    "00240F": "Ishii Tool & Engineering Corporation",
    "00243F": "Storwize, Inc.",
    "002440": "Halo Monitoring, Inc.",
    "002446": "MMB Research Inc.",
    "002445": "Adtran Inc",
    "002472": "ReDriven Power Inc.",
    "002471": "Fusion MultiSystems dba Fusion-io",
    "002474": "Autronica Fire And Securirty",
    "0023FA": "RG Nets, Inc.",
    "0023F2": "TVLogic",
    "00240A": "US Beverage Net",
    "002407": "TELEM SAS",
    "002463": "Phybridge Inc",
    "0023C2": "SAMSUNG Electronics. Co. LTD",
    "0023C4": "Lux Lumen",
    "0023C5": "Radiation Safety and Control Services Inc",
    "0023C6": "SMC Corporation",
    "0023B9": " Airbus Defence and Space Deutschland GmbH",
    "0023BD": "Digital Ally, Inc.",
    "0023BF": "Mainpine, Inc.",
    "0023B2": "Intelligent Mechatronic Systems Inc",
    "0023B5": "ORTANA LTD",
    "002390": "Algolware Corporation",
    "002388": "V.T. Telematica S.p.a.",
    "002386": "IMI Hydronic Engineering international SA",
    "002383": "InMage Systems Inc",
    "002381": "Lengda Technology(Xiamen) Co.,Ltd.",
    "00237B": "WHDI LLC",
    "002324": "G-PRO COMPUTER",
    "0023E0": "INO Therapeutics LLC",
    "0023D5": " WAREMA Renkhoff SE",
    "0023E7": "Hinke A/S",
    "0023E6": "Innovation Farm, Inc.",
    "0023E2": "SEA Signalisation",
    "0023C9": "Sichuan Tianyi Information Science & Technology Stock CO.,LTD",
    "0023CE": "KITA DENSHI CORPORATION",
    "0023CF": "CUMMINS-ALLISON CORP.",
    "0023A9": "Beijing Detianquan Electromechanical Equipment Co., Ltd",
    "0023A7": "Redpine Signals, Inc.",
    "00239B": "Elster Solutions, LLC",
    "0022EE": "Algo Communication Products Ltd",
    "0022EA": "Rustelcom Inc.",
    "0022F0": "3 Greens Aviation Limited",
    "0022EC": "IDEALBT TECHNOLOGY CORPORATION",
    "002331": "Nintendo Co., Ltd.",
    "002335": "Linkflex Co.,Ltd",
    "00232F": "Advanced Card Systems Ltd.",
    "002319": "Sielox LLC",
    "002311": "Gloscom Co., Ltd.",
    "00235D": "Cisco Systems, Inc",
    "00235C": "Aprius, Inc.",
    "002352": "DATASENSOR S.p.A.",
    "002309": "Janam Technologies LLC",
    "002304": "Cisco Systems, Inc",
    "0022F9": "Pollin Electronic GmbH",
    "002353": "F E T Elettronica snc",
    "002342": "Coffee Equipment Company",
    "002337": "Global Star Solutions ULC",
    "0022DD": "Protecta Electronics Ltd",
    "0022DB": "Translogic Corporation",
    "00236A": "SmartRG Inc",
    "002370": "Snell",
    "002325": "IOLAN Holding",
    "002321": "Avitech International Corp",
    "002274": "FamilyPhone AB",
    "00226A": "Honeywell",
    "002262": "BEP Marine",
    "002263": "Koos Technical Services, Inc.",
    "00226C": "LinkSprite Technologies, Inc.",
    "00225C": "Multimedia & Communication Technology",
    "0022B9": "Analogix Seminconductor, Inc",
    "0022B8": "Norcott",
    "0022B7": "GSS Grundig SAT-Systems GmbH",
    "0022B3": "Sei S.p.A.",
    "0022AB": "Shenzhen Turbosight Technology Ltd",
    "00229B": "AverLogic Technologies, Inc.",
    "00229C": "Verismo Networks Inc",
    "00229A": "Lastar, Inc.",
    "002284": "DESAY A&V SCIENCE AND TECHNOLOGY CO.,LTD",
    "002286": "ASTRON",
    "002282": "8086 Consultancy",
    "002276": "Triple EYE B.V.",
    "002254": "Bigelow Aerospace",
    "002257": "3COM EUROPE LTD",
    "002246": "Evoc Intelligent Technology Co.,Ltd.",
    "002248": "Microsoft Corporation",
    "0022C5": "INFORSON Co,Ltd.",
    "0022BF": "SieAmp Group of Companies",
    "0022BE": "Cisco Systems, Inc",
    "002290": "Cisco Systems, Inc",
    "00228A": "Teratronik elektronische systeme gmbh",
    "00228E": "TV-NUMERIC",
    "0022DA": "ANATEK, LLC",
    "002214": "RINNAI KOREA",
    "00220B": "National Source Coding Center",
    "00220C": "Cisco Systems, Inc",
    "0021EA": "Bystronic Laser AG",
    "0021FD": "LACROIX TRAFFIC S.A.U",
    "002228": "Breeze Innovations Ltd.",
    "002229": "Compumedics Ltd",
    "002200": "IBM Corp",
    "002230": "FutureLogic Inc.",
    "00222E": "maintech GmbH",
    "0021E1": "Nortel Networks",
    "0021CD": "LiveTV",
    "002216": "SHIBAURA VENDING MACHINE CORPORATION",
    "00216F": "SymCom, Inc.",
    "002166": "NovAtel Inc.",
    "002164": "Special Design Bureau for Seismic Instrumentation",
    "002160": "Hidea Solutions Co. Ltd.",
    "0021C0": "Mobile Appliance, Inc.",
    "0021BB": "Riken Keiki Co., Ltd.",
    "0021B1": "DIGITAL SOLUTIONS LTD",
    "0021B0": "Tyco Telecommunications",
    "002152": "General Satellite Research & Development Limited",
    "002158": "Style Flying Technology Co.",
    "002188": "EMC Corporation",
    "00217F": "Intraco Technology Pte Ltd",
    "00217D": "PYXIS S.R.L.",
    "0021AD": "Nordic ID Oy",
    "0021A3": "Micromint",
    "0021A5": "ERLPhase Power Technologies Ltd.",
    "0021C6": "CSJ Global, Inc.",
    "0021C3": "CORNELL Communications, Inc.",
    "0021C7": "Russound",
    "0021C1": "ABB Oy / Medium Voltage Products",
    "002148": "Kaco Solar Korea",
    "00219D": "Adesys BV",
    "002195": "GWD Media Limited",
    "002116": "Transcon Electronic Systems, spol. s r. o.",
    "002115": "PHYWE Systeme GmbH & Co. KG",
    "002111": "Uniphone Inc.",
    "002114": "Hylab Technology Inc.",
    "002132": "Masterclock, Inc.",
    "002131": "Blynke Inc.",
    "002129": "Cisco-Linksys, LLC",
    "00211D": "Dataline AB",
    "002120": "Sequel Technologies",
    "00211A": "LInTech Corporation",
    "002141": "RADLIVE",
    "00213D": "Cermetek Microelectronics, Inc.",
    "002140": "EN Technologies Inc.",
    "00213C": "AliphCom",
    "001FE7": "Simet",
    "001FDB": "Network Supply Corp.,",
    "001FD4": "4IPNET, INC.",
    "001FCB": "NIW Solutions",
    "001FF7": "Nakajima All Precision Co., Ltd.",
    "001FEB": "Trio Datacom Pty Ltd",
    "001FB5": "I/O Interconnect Inc.",
    "001FAD": "Brown Innovations, Inc",
    "001FBF": "Fulhua Microelectronics Corp. Taiwan Branch",
    "001FBE": "Shenzhen Mopnet Industrial Co.,Ltd",
    "001FC2": "Jow Tong Technology Co Ltd",
    "001F9B": "POSBRO",
    "001F97": "BERTANA srl",
    "001F8C": "CCS Inc.",
    "001F81": "Accel Semiconductor Corp",
    "001F83": "Teleplan Technology Services Sdn Bhd",
    "001F68": "Martinsson Elektronik AB",
    "001F63": "JSC Goodwin-Europa",
    "001F69": "Pingood Technology Co., Ltd.",
    "001FD2": "COMMTECH TECHNOLOGY MACAO COMMERCIAL OFFSHORE LTD.",
    "001F78": "Blue Fox Porini Textile",
    "001F4D": "Segnetics LLC",
    "001FA6": "Stilo srl",
    "001F13": "S.& A.S. Ltd.",
    "001F10": "TOLEDO DO BRASIL INDUSTRIA DE BALANCAS  LTDA",
    "001F0F": "Select Engineered Systems",
    "001F02": "Pixelmetrix Corporation Pte Ltd",
    "001EFE": "LEVEL s.r.o.",
    "001F34": "Lung Hwa Electronics Co., Ltd.",
    "001F25": "MBS GmbH",
    "001F27": "Cisco Systems, Inc",
    "001EDB": "Giken Trastem Co., Ltd.",
    "001ED1": "Keyprocessor B.V.",
    "001ED0": "Ingespace",
    "001EEF": "Cantronic International Limited",
    "001EDE": "BYD COMPANY LIMITED",
    "001EDD": "WASKO S.A.",
    "001F42": "Etherstack plc",
    "001F35": "AIR802 LLC",
    "001F26": "Cisco Systems, Inc",
    "001F1A": "Prominvest",
    "001F18": "Hakusan.Mfg.Co,.Ltd",
    "001ECD": "KYLAND Technology Co. LTD",
    "001EBF": "Haas Automation Inc.",
    "001EEB": "Talk-A-Phone Co.",
    "001EBC": "WINTECH AUTOMATION CO.,LTD.",
    "001EB1": "Cryptsoft Pty Ltd",
    "001EAF": "Ophir Optronics Ltd",
    "001EAD": "Wingtech Group Limited",
    "001E32": "Zensys",
    "001E35": "Nintendo Co., Ltd.",
    "001E2B": "Radio Systems Design, Inc.",
    "001E28": "Lumexis Corporation",
    "001E43": "AISIN CORPORATION",
    "001E6F": "Magna-Power Electronics, Inc.",
    "001E6A": "Beijing Bluexon Technology Co.,Ltd",
    "001E66": "RESOL Elektronische Regelungen GmbH",
    "001E63": "Vibro-Meter SA",
    "001E53": "Further Tech Co., LTD",
    "001E4E": "DAKO EDV-Ingenieur- und Systemhaus GmbH",
    "001E49": "Cisco Systems, Inc",
    "001E42": "Teltonika",
    "001E87": "Realease Limited",
    "001E7F": "CBM of America",
    "001E82": "SanDisk Corporation",
    "001E8E": "Hunkeler AG",
    "001EA1": "Brunata a/s",
    "001DB9": "Wellspring Wireless",
    "001DBB": "Dynamic System Electronics Corp.",
    "001DB3": "HPN Supply Chain",
    "001DB1": "Crescendo Networks",
    "001DB4": "KUMHO ENG CO.,LTD",
    "001E24": "Zhejiang Bell Technology Co.,ltd",
    "001E20": "Intertain Inc.",
    "001E1C": "SWS Australia Pty Limited",
    "001E12": "Ecolab",
    "001DDB": "C-BEL Corporation",
    "001DE6": "Cisco Systems, Inc",
    "001DE7": "Marine Sonic Technology, Ltd.",
    "001E16": "Keytronix",
    "001E08": "Centec Networks Inc",
    "001DC5": "Beijing Jiaxun Feihong Electricial Co., Ltd.",
    "001DC6": "SNR Inc.",
    "001DA4": "Hangzhou System Technology CO., LTD",
    "001D9F": "MATT   R.P.Traczynscy Sp.J.",
    "001DBF": "Radiient Technologies, Inc.",
    "001DFB": "NETCLEUS Systems Corporation",
    "001D3E": "SAKA TECHNO SCIENCE CO.,LTD",
    "001D40": " Intel \u2013 GE Care Innovations LLC",
    "001D34": "SYRIS Technology Corp",
    "001D32": "Longkay Communication & Technology (Shanghai) Co. Ltd",
    "001D2A": "SHENZHEN BUL-TECH CO.,LTD.",
    "001D2D": "Pylone, Inc.",
    "001D90": "EMCO Flow Systems",
    "001D93": "Modacom",
    "001D94": "Climax Technology Co., Ltd",
    "001D8E": "Alereon, Inc.",
    "001D5D": "Control Dynamics Pty. Ltd.",
    "001D59": "Mitra Energy & Infrastructure",
    "001D57": "CAETEC Messtechnik",
    "001D51": "Babcock & Wilcox Power Generation Group, Inc",
    "001D2B": "Wuhan Pont Technology CO. , LTD",
    "001D22": "Foss Analytical A/S",
    "001D23": "SENSUS ",
    "001D7B": "Ice Energy, Inc.",
    "001D6C": "ClariPhy Communications, Inc.",
    "001D4C": "Alcatel-Lucent",
    "001D84": "Gateway, Inc.",
    "001D85": "Call Direct Cellular Solutions",
    "001D1A": "OvisLink S.A.",
    "001D0B": "Power Standards Lab",
    "001D02": "Cybertech Telecom Development",
    "001CFE": "Quartics Inc",
    "001C81": "NextGen Venturi LTD",
    "001C7A": "Perfectone Netware Company Ltd",
    "001C75": "Segnet Ltd.",
    "001C74": "Syswan Technologies Inc.",
    "001CB7": "USC DigiArk Corporation",
    "001CB1": "Cisco Systems, Inc",
    "001CAF": "Plato Networks Inc.",
    "001CE9": "Galaxy Technology Limited",
    "001CEA": "Scientific-Atlanta, Inc",
    "001CCB": "Forth Corporation Public Company Limited",
    "001CBC": "CastGrabber, LLC",
    "001CDB": "CARPOINT CO.,LTD",
    "001CD5": "ZeeVee, Inc.",
    "001CD3": "ZP Engineering SEL",
    "001CE7": "Rocon PLC Research Centre",
    "001CE4": "EleSy JSC",
    "001CE2": "Attero Tech, LLC.",
    "001CAA": "Bellon Pty Ltd",
    "001CA0": "Production Resource Group, LLC",
    "001C98": "LUCKY TECHNOLOGY (HK) COMPANY LIMITED",
    "001C91": "Gefen LLC",
    "001C92": "Tervela",
    "001C8A": "Cirrascale Corporation",
    "001C02": "Pano Logic",
    "001C05": "Nonin Medical Inc.",
    "001C06": "Siemens Numerical Control Ltd., Nanjing",
    "001C04": "Airgain, Inc.",
    "001C01": "ABB Oy Drives",
    "001BFF": "Millennia Media inc.",
    "001BEA": "Nintendo Co., Ltd.",
    "001BE5": "802automation Limited",
    "001BE4": "TOWNET SRL",
    "001C68": "Anhui Sun Create Electronics Co., Ltd",
    "001C66": "UCAMP CO.,LTD",
    "001C2A": "Envisacor Technologies Inc.",
    "001BF2": "KWORLD COMPUTER CO., LTD",
    "001BF0": "Value Platforms Limited",
    "001C4D": "Aplix IP Holdings Corporation",
    "001C38": "Bio-Rad Laboratories, Inc.",
    "001C30": "Mode Lighting (UK ) Ltd.",
    "001C2E": "HPN Supply Chain",
    "001C53": "Synergy Lighting Controls",
    "001C1B": "Hyperstone GmbH",
    "001C10": "Cisco-Linksys, LLC",
    "001BC6": "Strato Rechenzentrum AG",
    "001BC4": "Ultratec, Inc.",
    "001BC2": "Integrated Control Technology Limitied",
    "001BBB": "RFTech Co.,Ltd",
    "001B5D": "Vololink Pty Ltd",
    "001B5A": "Apollo Imaging Technologies, Inc.",
    "001B56": "Tehuti Networks Ltd.",
    "001B86": "Bosch Access Systems GmbH",
    "001B7C": "A & R Cambridge",
    "001BD2": "ULTRA-X ASIA PACIFIC Inc.",
    "001B8D": "Electronic Computer Systems, Inc.",
    "001BAA": "XenICs nv",
    "001BA1": "\u00c5mic AB",
    "001B96": "General Sensing",
    "001AD5": "KMC CHAIN INDUSTRIAL CO., LTD.",
    "001AD0": "Albis Technologies AG",
    "001AD3": "Vamp Ltd.",
    "001AD8": "AlsterAero GmbH",
    "001ADA": "Biz-2-Me Inc.",
    "001ACB": "Autocom Products Ltd",
    "001ACF": "C.T. ELETTRONICA",
    "001B15": "Voxtel, Inc.",
    "001AEA": "Radio Terminal Systems Pty Ltd",
    "001ADD": "PePWave Ltd",
    "001AD4": "iPOX Technology Co., Ltd.",
    "001AD6": "JIAGNSU AETNA ELECTRIC CO.,LTD",
    "001AFB": "Joby Inc.",
    "001AFD": "EVOLIS",
    "001B03": "Action Technology (SZ) Co., Ltd",
    "001B1E": "HART Communication Foundation",
    "001B14": "Carex Lighting Equipment Factory",
    "001B4C": "Signtech",
    "001A6F": "MI.TEL s.r.l.",
    "001A71": "Diostech Co., Ltd.",
    "001A69": "Wuhan Yangtze Optical Technology CO.,Ltd.",
    "001A67": "Infinite QL Sdn Bhd",
    "001AB8": "Anseri Corporation",
    "001ABC": "U4EA Technologies Ltd",
    "001AA3": "DELORME",
    "001A9B": "ADEC & Parter AG",
    "001A9D": "Skipper Wireless, Inc.",
    "001AC3": "Scientific-Atlanta, Inc",
    "001ABF": "TRUMPF Laser Marking Systems AG",
    "001A8E": "3Way Networks Ltd",
    "001A85": "NV Michel Van de Wiele",
    "001A64": "IBM Corp",
    "001A55": "ACA-Digital Corporation",
    "001A1C": "GT&T Engineering Pte Ltd",
    "001A1F": "Coastal Environmental Systems",
    "001A23": "Ice Qube, Inc",
    "001A1D": "PChome Online Inc.",
    "001A17": "Teak Technologies, Inc.",
    "001A51": "Alfred Mann Foundation",
    "001A44": "JWTrading Co., Ltd",
    "0019E6": "TOYO MEDIC CO.,LTD.",
    "0019E8": "Cisco Systems, Inc",
    "0019DF": "Thomson Inc.",
    "0019DD": "FEI-Zyfer, Inc.",
    "001A05": "OPTIBASE LTD",
    "0019FC": "PT. Ufoakses Sukses Luarbiasa",
    "0019D4": "ICX Technologies",
    "0019CF": "SALICRU, S.A.",
    "001A49": "Micro Vision Co.,LTD",
    "001A3D": "Ajin Vision Co.,Ltd",
    "001A41": "INOCOVA Co.,Ltd",
    "001A0A": "Adaptive Micro-Ware Inc.",
    "001A33": "ASI Communications, Inc.",
    "0019B2": "XYnetsoft Co.,Ltd",
    "0019A4": "Austar Technology (hang zhou) Co.,Ltd",
    "0019AA": "Cisco Systems, Inc",
    "0019B1": "Arrow7 Corporation",
    "001996": "TurboChef Technologies Inc.",
    "001997": "Soft Device Sdn Bhd",
    "001998": "SATO CORPORATION",
    "00199C": "CTRING",
    "001952": "ACOGITO Co., Ltd",
    "001946": "Cianet Industria e Comercio S/A",
    "001949": "TENTEL  COMTECH CO., LTD.",
    "001967": "TELDAT Sp.J.",
    "001971": "Guangzhou Unicomp Technology Co.,Ltd",
    "001964": "Doorking Inc.",
    "001976": "Xipher Technologies, LLC",
    "00196C": "ETROVISION TECHNOLOGY",
    "0019B3": "Stanford Research Systems",
    "00198B": "Novera Optics Korea, Inc.",
    "001961": "Blaupunkt  Embedded Systems GmbH",
    "001944": "Fossil Partners, L.P.",
    "001942": "ON SOFTWARE INTERNATIONAL LIMITED",
    "00193F": "RDI technology(Shenzhen) Co.,LTD",
    "001941": "Pitney Bowes, Inc",
    "00192D": "Nokia Corporation",
    "001930": "Cisco Systems, Inc",
    "001927": "ImCoSys Ltd",
    "001908": "Duaxes Corporation",
    "00190C": "Encore Electronics, Inc.",
    "0018FD": "Optimal Technologies International Inc.",
    "0018F0": "JOYTOTO Co., Ltd.",
    "0018E9": "Numata Corporation",
    "0018EF": "Escape Communications, Inc.",
    "0018E6": "Computer Hardware Design SIA",
    "0018DB": "EPL Technology Ltd",
    "0018D6": "Swirlnet A/S",
    "0018CD": "Erae Electronics Industry Co., Ltd",
    "0018C8": "ISONAS Inc.",
    "0018BE": "ANSA Corporation",
    "0018BA": "Cisco Systems, Inc",
    "001919": "ASTEL Inc.",
    "0018F7": "Kameleon Technologies",
    "00189D": "Navcast Inc.",
    "001885": "Motorola Solutions Inc.",
    "001888": "GOTIVE a.s.",
    "00188A": "Infinova LLC",
    "001849": "nVent, Schroff GmbH",
    "001846": "Crypto S.A.",
    "001845": "Pulsar-Telecom LLC.",
    "00183D": "Vertex Link Corporation",
    "0018A3": "ZIPPY TECHNOLOGY CORP.",
    "001893": "SHENZHEN PHOTON BROADBAND TECHNOLOGY CO.,LTD",
    "0018A0": "Cierma Ascenseurs",
    "001863": "Veritech Electronics Limited",
    "00185A": "uControl, Inc.",
    "0018B4": "Dawon Media Inc.",
    "0018B6": "S3C, Inc.",
    "0018B1": "IBM Corp",
    "001886": "EL-TECH, INC.",
    "001887": "Metasystem SpA",
    "00187B": "4NSYS Co. Ltd.",
    "00187E": "RGB Spectrum",
    "001852": "StorLink Semiconductors, Inc.",
    "001850": "Secfone Kft",
    "001858": "TagMaster AB",
    "001825": "Private",
    "001879": "dSys",
    "0017CE": "Screen Service Spa",
    "0017CD": "CEC Wireless R&D Ltd.",
    "0017D0": "Opticom Communications, LLC",
    "0017EF": "IBM Corp",
    "0017F5": "LIG NEOPTEK",
    "0017FE": "TALOS SYSTEM INC.",
    "0017F8": "Motech Industries Inc.",
    "0017DB": "CANKO TECHNOLOGIES INC.",
    "0017D6": "Bluechips Microhouse Co.,Ltd.",
    "00181E": "GDX Technologies Ltd.",
    "0017C6": "Cross Match Technologies Inc",
    "0017C3": "KTF Technologies Inc.",
    "0017B7": "Tonze Technology Co.",
    "001807": "Fanstel Corp.",
    "001808": "SightLogix, Inc.",
    "001803": "ArcSoft Shanghai Co. LTD",
    "0017AB": "Nintendo Co., Ltd.",
    "001824": "Kimaldi Electronics, S.L.",
    "001798": "Azonic Technology Co., LTD",
    "00178F": "NINGBO YIDONG ELECTRONIC CO.,LTD.",
    "00178A": "DARTS TECHNOLOGIES CORP.",
    "001721": "FITRE S.p.A.",
    "001720": "Image Sensing Systems, Inc.",
    "00171A": "Winegard Company",
    "001789": "Zenitron Corporation",
    "001787": "Brother, Brother & Sons ApS",
    "001760": "Naito Densei Machida MFG.CO.,LTD",
    "001761": "Private",
    "001768": "Zinwave Ltd",
    "001769": "Cymphonix Corp",
    "001762": "Solar Technology, Inc.",
    "00173E": "LeucotronEquipamentos Ltda.",
    "001734": "ADC Telecommunications",
    "00172E": "FXC Inc.",
    "00172B": "Global Technologies Inc.",
    "001726": "m2c Electronic Technology Ltd.",
    "001772": "ASTRO Strobel Kommunikationssysteme GmbH",
    "00177A": "ASSA ABLOY AB",
    "00171C": "NT MicroSystems, Inc.",
    "001716": "Qno Technology Inc.",
    "001747": "Trimble",
    "0016BD": "ATI Industrial Automation",
    "0016C0": "Semtech Corporation",
    "0016C2": "Avtec Systems Inc",
    "0016F1": "OmniSense, LLC",
    "0016F4": "Eidicom Co., Ltd.",
    "0016E7": "Dynamix Promotions Limited",
    "0016BA": "WEATHERNEWS INC.",
    "0016B2": "DriveCam Inc",
    "0016B3": "Photonicbridges (China) Co., Ltd.",
    "0016AD": "BT-Links Company Limited",
    "0016E5": "FORDLEY DEVELOPMENT LIMITED",
    "0016DD": "Gigabeam Corporation",
    "0016C8": "Cisco Systems, Inc",
    "0016C4": "SiRF Technology, Inc.",
    "0016F3": "CAST Information Co., Ltd",
    "0016F5": "Dalian Golden Hualu Digital Technology Co.,Ltd",
    "0016F9": "CETRTA POT, d.o.o., Kranj",
    "00170A": "INEW DIGITAL COMPANY",
    "00162A": "Antik computers & communications s.r.o.",
    "001629": "Nivus GmbH",
    "001621": "Colorado Vnet",
    "00161A": "Dametric AB",
    "001631": "Xteam",
    "00162E": "Space Shuttle Hi-Tech Co., Ltd.",
    "00166E": "Arbitron Inc.",
    "00166A": "TPS",
    "001663": "KBT Mobile",
    "001654": "Flex-P Industries Sdn. Bhd.",
    "001656": "Nintendo Co., Ltd.",
    "001651": "Exeo Systems",
    "00164B": "Quorion Data Systems GmbH",
    "001688": "ServerEngines LLC",
    "00168B": "Paralan Corporation",
    "001682": "OMS Motion",
    "00169F": "Vimtron Electronics Co., Ltd.",
    "00169A": "Quadrics Ltd",
    "001692": "Scientific-Atlanta, Inc.",
    "001691": "Moser-Baer AG",
    "0016AF": "Shenzhen Union Networks Equipment Co.,Ltd.",
    "00169E": "TV One Ltd",
    "0015ED": "Fulcrum Microsystems, Inc.",
    "0015F0": "EGO BV",
    "0015EE": "Omnex Control Systems",
    "00160A": "SWEEX Europe BV",
    "001602": "CEYON TECHNOLOGY CO.,LTD.",
    "001600": "CelleBrite Mobile Synchronization",
    "0015F4": "Eventide",
    "0015C0": "DIGITAL TELEMEDIA CO.,LTD.",
    "0015C2": "3M Germany",
    "0015B2": "Advanced Industrial Computer, Inc.",
    "001615": "Nittan Company, Limited",
    "001617": "MSI",
    "001594": "BIXOLON CO.,LTD",
    "001590": "Hectronic GmbH",
    "001610": "Carina Technology",
    "001606": "Ideal Industries",
    "001607": "Curves International Inc.",
    "0015A5": "DCI Co., Ltd.",
    "0015C8": "FlexiPanel Ltd",
    "0015DA": "IRITEL A.D.",
    "00154A": "WANSHIH ELECTRONIC CO., LTD",
    "00154C": "Saunders Electronics",
    "00154D": "Netronome Systems, Inc.",
    "001549": "Dixtal Biomedica Ind. Com. Ltda",
    "001511": "Data Center Systems",
    "00150E": "OPENBRAIN TECHNOLOGIES CO., LTD.",
    "00150D": "Hoana Medical, Inc.",
    "001506": "Neo Photonics",
    "001504": "GAME PLUS CO., LTD.",
    "001505": "Actiontec Electronics, Inc",
    "00158C": "Liab ApS",
    "00158F": "NTT Advanced Technology Corporation",
    "001583": "IVT corporation",
    "001588": "Salutica Allied Solutions Sdn Bhd",
    "001585": "Aonvision Technolopy Corp.",
    "00152B": "Cisco Systems, Inc",
    "00152C": "Cisco Systems, Inc",
    "001528": "Beacon Medical Products LLC d.b.a. BeaconMedaes",
    "001527": "Balboa Instruments",
    "001521": "Horoquartz",
    "001520": "Radiocrafts AS",
    "001539": "Technodrive srl",
    "001531": "KOCOM",
    "001535": "OTE Spa",
    "001536": "Powertech co.,Ltd",
    "00151C": "LENECO",
    "001519": "StoreAge Networking Technologies",
    "001547": "AiZen Solutions Inc.",
    "00153D": "ELIM PRODUCT CO.",
    "001544": "coM.s.a.t. AG",
    "00156B": "Perfisans Networks Corp.",
    "001566": "A-First Technology Co., Ltd.",
    "001579": "Lunatone Industrielle Elektronik GmbH",
    "0014CF": "INVISIO Communications",
    "0014BE": "Wink communication technology CO.LTD",
    "0014B2": "mCubelogics Corporation",
    "0014AE": "Wizlogics Co., Ltd.",
    "0014A6": "Teranetics, Inc.",
    "0014AA": "Ashly Audio, Inc.",
    "001489": "B15402100 - JANDEI, S.L.",
    "0014BA": "Carvers SA de CV",
    "0014B6": "Enswer Technology Inc.",
    "0014D7": "Datastore Technology Corp",
    "0014DD": "Covergence Inc.",
    "0014D4": "K Technology Corporation",
    "0014E6": "AIM Infrarotmodule GmbH",
    "0014DE": "Sage Instruments Inc.",
    "0014DF": "HI-P Tech Corporation",
    "0014FE": "Artech Electronics",
    "0014F3": "ViXS Systems Inc",
    "00148A": "Elin Ebg Traction Gmbh",
    "001409": "MAGNETI MARELLI   S.E. S.p.A.",
    "00140B": "FIRST INTERNATIONAL COMPUTER, INC.",
    "0013FD": "Nokia Danmark A/S",
    "001400": "MINERVA KOREA CO., LTD",
    "0013FC": "SiCortex, Inc",
    "001480": "Hitachi-LG Data Storage Korea, Inc",
    "00147E": "InnerWireless",
    "00147D": "Aeon Digital International",
    "001476": "MultiCom Industries Limited",
    "001473": "Bookham Inc",
    "00144F": "Oracle Corporation ",
    "001456": "Edge Products",
    "001450": "Heim Systems GmbH",
    "001452": "CALCULEX,INC.",
    "00145D": "WJ Communications, Inc.",
    "001442": "ATTO CORPORATION",
    "001447": "BOAZ Inc.",
    "00143E": "AirLink Communications, Inc.",
    "00143B": "Sensovation AG",
    "00142D": "Toradex AG",
    "001414": "Jumpnode Systems LLC.",
    "001429": "V Center Technologies Co., Ltd.",
    "00141E": "P.A. Semi, Inc.",
    "00146E": "H. Stoll GmbH & Co. KG",
    "001469": "Cisco Systems, Inc",
    "0013D8": "Princeton Instruments",
    "0013CB": "Zenitel Norway AS",
    "0013CF": "4Access Communications",
    "0013C9": "Beyond Achieve Enterprises Ltd.",
    "001378": "Qsan Technology, Inc.",
    "00137A": "Netvox Technology Co., Ltd.",
    "001381": "CHIPS & Systems, Inc.",
    "0013F6": "Cintech",
    "0013F3": "Giga-byte Communications Inc.",
    "0013F4": "Psitek (Pty) Ltd",
    "0013E9": "VeriWave, Inc.",
    "0013BE": "Virtual Conexions",
    "0013B9": "BM SPA",
    "0013AB": "Telemotive AG",
    "001395": "congatec GmbH",
    "0013A8": "Tanisys Technology",
    "001399": "STAC Corporation.",
    "0013AC": "Sunmyung Electronics Co., LTD",
    "0013C6": "OpenGear, Inc",
    "0013DA": "Diskware Co., Ltd",
    "001367": "Narayon. Co., Ltd.",
    "00135C": "OnSite Systems, Inc.",
    "00135F": "Cisco Systems, Inc",
    "001356": "FLIR Radiation Inc",
    "00135A": "Project T&E Limited",
    "001361": "Biospace Co., Ltd.",
    "001362": "ShinHeung Precision Co., Ltd.",
    "00134F": "Rapidus Wireless Networks Inc.",
    "001338": "FRESENIUS-VIAL",
    "00132D": "iWise Communications",
    "00132C": "MAZ Brandenburg GmbH",
    "001324": "Schneider Electric Ultra Terminal",
    "001326": "ECM Systems Ltd",
    "001327": "Data Acquisitions limited",
    "0012F6": "MDK CO.,LTD.",
    "0012F4": "Belco International Co.,Ltd.",
    "0012F1": "IFOTEC",
    "0012F5": "Imarda New Zealand Limited",
    "00130F": "EGEMEN Bilgisayar Muh San ve Tic LTD STI",
    "001313": "GuangZhou Post & Telecom Equipment ltd",
    "0012F8": "WNI Resources, LLC",
    "00131D": "Scanvaegt International A/S",
    "001318": "DGSTATION Co., Ltd.",
    "00131A": "Cisco Systems, Inc",
    "00130C": "HF System Corporation",
    "001379": "PONDER INFORMATION INDUSTRIES LTD.",
    "001374": "Atheros Communications, Inc.",
    "001369": "Honda Electron Co., LED.",
    "001342": "Vision Research, Inc.",
    "00133B": "Speed Dragon Multimedia Limited",
    "001301": "IronGate S.L.",
    "0012CB": "CSS Inc.",
    "0012CE": "Advanced Cybernetics Group",
    "0012CA": "Mechatronic Brick Aps",
    "0012C7": "SECURAY Technologies Ltd.Co.",
    "0012E8": "Fraunhofer IMS",
    "0012DD": "Shengqu Information Technology (Shanghai) Co., Ltd.",
    "0012E0": "Codan Limited",
    "0012A1": "BluePacket Communications Co., Ltd.",
    "00129B": "E2S Electronic Engineering Solutions, S.L.",
    "001298": "MICO ELECTRIC(SHENZHEN) LIMITED",
    "0012AC": "ONTIMETEK INC.",
    "0012BC": "Echolab LLC",
    "0012BD": "Avantec Manufacturing Limited",
    "0012AB": "WiLife, Inc.",
    "0012CD": "ASEM SpA",
    "0012C0": "HotLava Systems, Inc.",
    "0012B7": "PTW Freiburg",
    "0012DE": "Radio Components Sweden AB",
    "00127C": "SWEGON AB",
    "001278": "International Bar Code",
    "00127A": "Sanyu Industry Co.,Ltd.",
    "001206": "iQuest (NZ) Ltd",
    "001207": "Head Strong International Limited",
    "001209": "Fastrax Ltd",
    "00120B": "Chinasys Technologies Limited",
    "001205": "Terrasat Communications, Inc.",
    "001254": "Spectra Technologies Holdings Company Ltd",
    "00124F": "nVent",
    "001249": "Delta Elettronica S.p.A.",
    "001221": "B.Braun Melsungen AG",
    "001212": "PLUS  Corporation",
    "001213": "Metrohm AG",
    "001218": "ARUZE Corporation",
    "001266": "Swisscom Hospitality Services SA",
    "00125E": "CAEN",
    "00125D": "CyberNet Inc.",
    "001272": "Redux Communications Ltd.",
    "00126A": "OPTOELECTRONICS Co., Ltd.",
    "001268": "IPS d.o.o.",
    "00123E": "ERUNE technology Co., Ltd.",
    "00123A": "Posystech Inc., Co.",
    "001234": "Camille Bauer",
    "00124D": "Inducon BV",
    "001238": "SetaBox Technology Co., Ltd.",
    "00123C": "Second Rule LLC",
    "001223": "Pixim",
    "001190": "Digital Design Corporation",
    "001196": "Actuality Systems, Inc.",
    "001187": "Category Solutions, Inc",
    "00116F": "Netforyou Co., LTD.",
    "001171": "DEXTER Communications, Inc.",
    "001167": "Integrated System Solution Corp.",
    "0011D3": "NextGenTel Holding ASA",
    "0011C4": "Terminales de Telecomunicacion Terrestre, S.L.",
    "0011CB": "Jacobsons AB",
    "00117B": "B\u00fcchi  Labortechnik AG",
    "0011E3": "Thomson, Inc.",
    "0011DC": "Glunz & Jensen",
    "0011F7": "Shenzhen Forward Industry Co., Ltd",
    "0011AB": "TRUSTABLE TECHNOLOGY CO.,LTD.",
    "0011A5": "Fortuna Electronic Corp.",
    "000FFE": "G-PRO COMPUTER",
    "000FDA": "YAZAKI CORPORATION",
    "000FEF": "Thales e-Transactions GmbH",
    "001105": "Sunplus Technology Co., Ltd.",
    "001102": "Aurora Multimedia Corp.",
    "00113C": "Micronas GmbH",
    "001131": "UNATECH. CO.,LTD",
    "00114E": "690885 Ontario Inc.",
    "001145": "ValuePoint Networks",
    "001127": "TASI, Inc",
    "00112A": "Niko NV",
    "00112B": "NetModule AG",
    "00112D": "iPulse Systems",
    "001154": "Webpro Technologies Inc.",
    "001151": "Mykotronx",
    "001123": "Appointech, Inc.",
    "00110F": "netplat,Inc.",
    "00116D": "American Time and Signal",
    "001160": "ARTDIO Company Co., LTD",
    "001115": "EPIN Technologies, Inc.",
    "00110B": "Franklin Technology Systems",
    "000FEE": "XTec, Incorporated",
    "000FE4": "Pantech Co.,Ltd",
    "000FE7": "Lutron Electronics Co., Inc.",
    "000FE6": "MBTech Systems, Inc.",
    "000FE9": "GW TECHNOLOGIES CO.,LTD.",
    "000FE1": "ID DIGITAL CORPORATION",
    "000FAB": "Kyushu Electronics Systems Inc.",
    "000FAD": "FMN communications GmbH",
    "000FAC": "IEEE 802.11",
    "000FB3": "Actiontec Electronics, Inc",
    "000F9C": "Panduit Corp",
    "000F7A": "BeiJing NuQX Technology CO.,LTD",
    "000F64": "D&R Electronica Weesp BV",
    "000F72": "Sandburst",
    "000F75": "First Silicon Solutions",
    "000F7C": "ACTi Corporation",
    "000FC7": "Dionica R&D Ltd.",
    "000FC8": "Chantry Networks",
    "000FBB": "Nokia Siemens Networks GmbH & Co. KG.",
    "000FBC": "Onkey Technologies, Inc.",
    "000FDF": "SOLOMON Technology Corp.",
    "000FD6": "Sarotech Co., Ltd",
    "000F98": "Avamax Co. Ltd.",
    "000F97": "Avanex Corporation",
    "000F96": "Telco Systems, Inc. ",
    "000FB6": "Europlex Technologies",
    "000FB9": "Adaptive Instruments",
    "000F8B": "Orion MultiSystems Inc",
    "000F8C": "Gigawavetech Pte Ltd",
    "000F45": "Stretch, Inc.",
    "000F3B": "Fuji System Machines Co., Ltd.",
    "000F37": "Xambala Incorporated",
    "000F38": "Netstar",
    "000F3A": "HISHARP",
    "000F34": "Cisco Systems, Inc",
    "000F30": "Raza Microelectronics Inc",
    "000F2E": "Megapower International Corp.",
    "000F26": "WorldAccxx  LLC",
    "000F53": "Solarflare Communications Inc.",
    "000F51": "Azul Systems, Inc.",
    "000F07": "Mangrove Systems, Inc.",
    "000F00": "Legra Systems, Inc.",
    "000F01": "DIGITALKS INC",
    "000F03": "COM&C CO., LTD",
    "000F54": "Entrelogic Corporation",
    "000EDF": "PLX Technology",
    "000EE1": "ExtremeSpeed Inc.",
    "000F25": "AimValley B.V.",
    "000F19": "Boston Scientific",
    "000ED3": "Epicenter, Inc.",
    "000ED7": "Cisco Systems, Inc",
    "000F10": "RDM Corporation",
    "000EC5": "Digital Multitools Inc",
    "000EC7": "Motorola Korea",
    "000EBA": "HANMI SEMICONDUCTOR CO., LTD.",
    "000EBC": "Paragon Fidelity GmbH",
    "000EA8": "United Technologists Europe Limited",
    "000E8B": "Astarte Technology Co, Ltd.",
    "000E80": "Thomson Technology Inc",
    "000EAA": "Scalent Systems, Inc.",
    "000EAC": "MINTRON ENTERPRISE CO., LTD.",
    "000EAE": "GAWELL TECHNOLOGIES CORP.",
    "000EA4": "Quantum Corp.",
    "000E6B": "Janitza electronics GmbH",
    "000E67": "Eltis Microelectronics Ltd.",
    "000E65": "TransCore",
    "000E7A": "GemWon Communications Co., Ltd.",
    "000E6C": "Device Drivers Limited",
    "000E9D": "Tiscali UK Ltd",
    "000E5F": "activ-net GmbH & Co. KG",
    "000E57": "Iworld Networking, Inc.",
    "000E47": "NCI System Co.,Ltd.",
    "000E85": "Catalyst Enterprises, Inc.",
    "000E74": "Solar Telecom. Tech",
    "000DE7": "Snap-on OEM Group",
    "000DF0": "QCOM TECHNOLOGY INC.",
    "000DE5": "Samsung Thales",
    "000DDD": "Profilo Telra Elektronik Sanayi ve Ticaret. A.\u015e",
    "000DF5": "Teletronics International Inc.",
    "000DF7": "Space Dynamics Lab",
    "000DEB": "CompXs Limited",
    "000DEE": "Andrew RF Power Amplifier Group",
    "000DEF": "Soc. Coop. Bilanciai",
    "000E11": "BDT B\u00fcro und Datentechnik GmbH & Co.KG ",
    "000DF8": "ORGA Kartensysteme GmbH",
    "000E46": "Niigata Seimitsu Co.,Ltd.",
    "000E43": "G-Tek Electronics Sdn. Bhd.",
    "000E44": "Digital 5, Inc.",
    "000E1F": "TCL Networks Equipment Co., Ltd.",
    "000E26": "Gincom Technology Corp.",
    "000E16": "SouthWing S.L.",
    "000E0B": "Netac Technology Co., Ltd.",
    "000E32": "Kontron Medical",
    "000DAF": "Plexus Corp (UK) Ltd",
    "000DB1": "Japan Network Service Co., Ltd.",
    "000DA7": "Private",
    "000DAD": "Dataprobe, Inc.",
    "000DA9": "INGETEAM",
    "000DAB": "Parker Hannifin GmbH Electromechanical Division Europe",
    "000D78": "Engineering & Security",
    "000D79": "Dynamic Solutions Co,.Ltd.",
    "000D6D": "K-Tech Devices Corp.",
    "000D6E": "K-Patents Oy",
    "000DA8": "Teletronics Technology Corporation",
    "000D9E": "TOKUDEN OHIZUMI SEISAKUSYO Co.,Ltd.",
    "000DA3": "Emerging Technologies Limited",
    "000DA4": "DOSCH & AMAND SYSTEMS AG",
    "000D9A": "INFOTEC LTD",
    "000DC0": "Spagat AS",
    "000DC6": "DigiRose Technology Co., Ltd.",
    "000DC1": "SafeWeb Inc",
    "000D85": "Tapwave, Inc.",
    "000D82": "PHSNET",
    "000D83": "Sanmina-SCI Hungary  Ltd.",
    "000D7F": "MIDAS  COMMUNICATION TECHNOLOGIES PTE LTD ( Foreign Branch)",
    "000D75": "Kobian Pte Ltd - Taiwan Branch",
    "000D6A": "Redwood Technologies LTD",
    "000D5D": "Raritan Computer, Inc",
    "000D63": "DENT Instruments, Inc.",
    "000D66": "Cisco Systems, Inc",
    "000D96": "Vtera Technology Inc.",
    "000D8B": "T&D Corporation",
    "000D90": "Factum Electronics AB",
    "000DBA": "Oc\u00e9 Document Technologies GmbH",
    "000DBD": "Cisco Systems, Inc",
    "000DB5": "GLOBALSAT TECHNOLOGY CORPORATION",
    "000DB4": "Stormshield",
    "000CCB": "Design Combus Ltd",
    "000CC9": "ILWOO DATA & TECHNOLOGY CO.,LTD",
    "000D02": "NEC Platforms, Ltd.",
    "000D07": "Calrec Audio Ltd",
    "000D2F": "AIN Comm.Tech.Co., LTD",
    "000D21": "WISCORE Inc.",
    "000D23": "Smart Solution, Inc",
    "000D27": "MICROPLEX Printware AG",
    "000D1B": "Kyoto Electronics Manufacturing Co., Ltd.",
    "000D1D": "HIGH-TEK HARNESS ENT. CO., LTD.",
    "000D09": "Yuehua(Zhuhai) Electronic CO. LTD",
    "000CEE": "jp-embedded",
    "000CE9": "BLOOMBERG L.P.",
    "000CEA": "aphona Kommunikationssysteme",
    "000D40": "Verint Loronix Video Solutions",
    "000D37": "WIPLUG",
    "000D33": "Prediwave Corp.",
    "000CDA": "FreeHand Systems, Inc.",
    "000CDD": "AOS technologies AG",
    "000CCD": "IEC - TC57",
    "000CFB": "Korea Network Systems",
    "000CF5": "InfoExpress",
    "000C85": "Cisco Systems, Inc",
    "000C84": "Eazix, Inc.",
    "000C7A": "DaTARIUS Technologies GmbH",
    "000C79": "Extel Communications P/L",
    "000C6D": "Edwards Ltd.",
    "000C70": "ACC GmbH",
    "000C6A": "MBARI",
    "000C6B": "Kurz Industrie-Elektronik GmbH",
    "000C9D": "UbeeAirWalk, Inc.",
    "000C9F": "NKE Corporation",
    "000C9A": "Hitech Electronics Corp.",
    "000C91": "Riverhead Networks Inc.",
    "000C5D": "CHIC TECHNOLOGY (CHINA) CORP.",
    "000C4F": "UDTech Japan Corporation",
    "000C62": "ABB AB, Cewe-Control ",
    "000C4C": "Arcor AG&Co.",
    "000C47": "SK Teletech(R&D Planning Team)",
    "000CAF": "TRI TERM CO.,LTD.",
    "000CB3": "ROUND Co.,Ltd.",
    "000CA5": "Naman NZ LTd",
    "000CA9": "Ebtron Inc.",
    "000C75": "Oriental integrated electronics. LTD",
    "000C20": "Fi WIn, Inc.",
    "000C15": "CyberPower Systems, Inc.",
    "000BD6": "Paxton Access Ltd",
    "000BD0": "XiMeta Technology Americas Inc.",
    "000BD4": "Beijing Wise Technology & Science Development Co.Ltd",
    "000BE7": "COMFLUX TECHNOLOGY INC.",
    "000BDA": "EyeCross Co.,Inc.",
    "000C16": "Concorde Microsystems Inc.",
    "000C09": "Hitachi IE Systems Co., Ltd",
    "000BEC": "NIPPON ELECTRIC INSTRUMENT, INC.",
    "000C3F": "Cogent Defence & Security Networks,",
    "000C30": "Cisco Systems, Inc",
    "000C1F": "Glimmerglass Networks",
    "000C02": "ABB Oy",
    "000BFF": "Berkeley Camera Engineering",
    "000B73": "Kodeos Communications",
    "000B76": "ET&T Technology Co. Ltd.",
    "000B5E": "Audio Engineering Society Inc.",
    "000B5F": "Cisco Systems, Inc",
    "000B60": "Cisco Systems, Inc",
    "000B65": "Sy.A.C. srl",
    "000B61": "Friedrich L\u00fctze GmbH & Co. KG",
    "000B99": "SensAble Technologies, Inc.",
    "000B9C": "TriBeam Technologies, Inc.",
    "000B89": "Top Global Technology, Ltd.",
    "000B8B": "KERAJET, S.A.",
    "000B59": "ScriptPro, LLC",
    "000B5C": "Newtech Co.,Ltd",
    "000B5B": "Rincon Research Corporation",
    "000B7C": "Telex Communications",
    "000B83": "DATAWATT B.V.",
    "000B71": "Litchfield Communications Inc.",
    "000B74": "Kingwave Technology Co., Ltd.",
    "000B7E": "SAGINOMIYA Seisakusho Inc.",
    "000B80": "Lycium Networks",
    "000BA7": "Maranti Networks",
    "000BAA": "Aiphone co.,Ltd",
    "000BA4": "Shiron Satellite Communications Ltd. (1996)",
    "000BC1": "Bay Microsystems, Inc.",
    "000BC5": "SMC Networks, Inc.",
    "000BC6": "ISAC, Inc.",
    "000B41": "Ing. B\u00fcro Dr. Beutlhauser",
    "000B05": "Pacific Broadband Networks",
    "000B00": "FUJIAN START COMPUTER EQUIPMENT CO.,LTD",
    "000B03": "Taekwang Industrial Co., Ltd",
    "000B01": "DAIICHI ELECTRONICS CO., LTD.",
    "000AEA": "ADAM ELEKTRONIK LTD. \u015eTI",
    "000AE3": "YANG MEI TECHNOLOGY CO., LTD",
    "000ADC": "RuggedCom Inc.",
    "000AE0": "Fujitsu Softek",
    "000B20": "Hirata corporation",
    "000B22": "Environmental Systems and Services",
    "000B1B": "Systronix, Inc.",
    "000B3E": "BittWare, Inc",
    "000B29": "LS(LG) Industrial Systems co.,Ltd",
    "000ABB": "Taiwan Secom Co,. Ltd",
    "000AAD": "Stargames Corporation",
    "000B39": "Keisoku Giken Co.,Ltd.",
    "000B33": "Vivato Technologies",
    "000B11": "HIMEJI ABC TRADING CO.,LTD.",
    "000A6E": "Harmonic, Inc",
    "000A6D": "EKS Elektronikservice GmbH",
    "000A74": "Manticom Networks Inc.",
    "000A6F": "ZyFLEX Technologies Inc",
    "000AA4": "SHANGHAI SURVEILLANCE TECHNOLOGY CO,LTD",
    "000A9E": "BroadWeb Corportation",
    "000A90": "Bayside Interactive, Inc.",
    "000A9D": "King Young Technology Co. Ltd.",
    "000AB1": "GENETEC Corporation",
    "000AB7": "Cisco Systems, Inc",
    "000AB9": "Astera Technologies Corp.",
    "000AA1": "V V S Limited",
    "000A25": "CERAGON NETWORKS",
    "000A21": "Integra Telecom Co. Ltd",
    "000A1E": "Red-M Products Limited",
    "000A4B": "DataPower Technology, Inc.",
    "000A43": "Chunghwa Telecom Co., Ltd.",
    "000A45": "Audio-Technica Corp.",
    "000A33": "Emulex Corporation",
    "000A2E": "MAPLE NETWORKS CO., LTD",
    "000A2D": "Cabot Communications Limited",
    "000A35": "Xilinx",
    "000A3B": "GCT Semiconductor, Inc",
    "000A8B": "Cisco Systems, Inc",
    "000A88": "InCypher S.A.",
    "000A7C": "Tecton Ltd",
    "000A63": "DHD GmbH",
    "000A67": "OngCorp",
    "0009DA": "Control Module Inc.",
    "0009D7": "DC Security Products",
    "0009D8": "F\u00e4lt Communications AB",
    "0009DB": "eSpace",
    "0009D5": "Signal Communication, Inc.",
    "0009FB": "Philips Patient Monitoring",
    "0009F9": "ART JAPAN CO., LTD.",
    "0009FD": "Ubinetics Limited",
    "0009B6": "Cisco Systems, Inc",
    "0009B3": "MCM Systems Ltd",
    "0009A3": "Leadfly Techologies Corp. Ltd.",
    "0009A5": "HANSUNG ELETRONIC INDUSTRIES DEVELOPMENT CO., LTD",
    "0009D3": "Western DataCom Co., Inc.",
    "000A14": "TECO a.s.",
    "000A0B": "Sealevel Systems, Inc.",
    "000A10": "FAST media integrations AG",
    "0009E6": "Cyber Switching Inc.",
    "0009E2": "Sinbon Electronics Co., Ltd.",
    "00099A": "ELMO COMPANY, LIMITED",
    "00099C": "Naval Research Laboratory",
    "0009BC": "Utility, Inc",
    "0009BE": "Mamiya-OP Co.,Ltd.",
    "000992": "InterEpoch Technology,INC.",
    "000995": "Castle Technology Ltd",
    "000998": "Capinfo Company Limited",
    "00098F": "Cetacean Networks",
    "000986": "Metalink LTD.",
    "000987": "NISHI NIPPON ELECTRIC WIRE & CABLE CO.,LTD.",
    "000989": "VividLogic Inc.",
    "00098C": "Option Wireless Sweden",
    "000985": "Auto Telecom Company",
    "00098D": "Velocity Semiconductor",
    "000981": "Newport Networks",
    "00092C": "Hitpoint Inc.",
    "00092B": "iQstor Networks, Inc.",
    "000926": "YODA COMMUNICATIONS, INC.",
    "000927": "TOYOKEIKI CO.,LTD.",
    "000923": "Heaman System Co., Ltd",
    "00091D": "Proteam Computer Corporation",
    "000955": "Young Generation International Corp.",
    "00094A": "Homenet Communications",
    "00094B": "FillFactory NV",
    "00094D": "Braintree Communications Pty Ltd",
    "000950": "Independent Storage Corporation",
    "000954": "AMiT spol. s. r. o.",
    "000952": "Auerswald GmbH & Co. KG",
    "000944": "Cisco Systems, Inc",
    "000962": "Sonitor Technologies AS",
    "00095D": "Dialogue Technology Corp.",
    "00095C": "Philips Medical Systems - Cardiac and Monitoring Systems (CM",
    "000958": "INTELNET S.A.",
    "000909": "Telenor Connect A/S",
    "00090A": "SnedFar Technology Co., Ltd.",
    "000984": "MyCasa Network Inc.",
    "000917": "WEM Technology Inc",
    "000873": "DapTechnology B.V.",
    "00087A": "Wipotec GmbH",
    "000871": "NORTHDATA Co., Ltd.",
    "00087E": "Bon Electro-Telecom Inc.",
    "000880": "BroadTel Canada Communications inc.",
    "0008B4": "SYSPOL",
    "0008B3": "Fastwel",
    "0008B2": "SHENZHEN COMPASS TECHNOLOGY DEVELOPMENT CO.,LTD",
    "0008B1": "ProQuent Systems",
    "0008AF": "Novatec Corporation",
    "0008A6": "Multiware & Image Co., Ltd.",
    "0008EB": "ROMWin Co.,Ltd.",
    "0008E8": "Excel Master Ltd.",
    "0008DC": "Wiznet",
    "0008DD": "Telena Communications, Inc.",
    "0008E1": "Barix AG",
    "0008DA": "SofaWare Technologies Ltd.",
    "0008D4": "IneoQuest Technologies, Inc",
    "0008D6": "HASSNET Inc.",
    "0008CE": "IPMobileNet Inc.",
    "00088E": "Nihon Computer Co., Ltd.",
    "000881": "DIGITAL HANDS CO.,LTD.",
    "000882": "SIGMA CORPORATION",
    "000903": "Panasas, Inc",
    "000907": "Chrysalis Development",
    "000906": "Esteem Networks",
    "0008FB": "SonoSite, Inc.",
    "0008EE": "Logic Product Development",
    "0008C2": "Cisco Systems, Inc",
    "0008C0": "ASA SYSTEMS",
    "0008A3": "Cisco Systems, Inc",
    "00089E": "Beijing Enter-Net co.LTD",
    "0007DD": "Cradle Technologies",
    "0007D5": "3e Technologies Int;., Inc.",
    "0007DB": "Kirana Networks, Inc.",
    "00086D": "Missouri FreeNet",
    "00086A": "Securiton Gmbh",
    "000863": "Entrisphere Inc.",
    "000866": "DSX Access Systems, Inc.",
    "00085F": "Picanol N.V.",
    "00047B": "Schlumberger",
    "0007D1": "Spectrum Signal Processing Inc.",
    "0007B9": "Ginganet Corporation",
    "00047F": "Chr. Mayr GmbH & Co. KG",
    "00080C": "VDA Group S.p.a.",
    "000804": "ICA Inc.",
    "0007FF": "Gluon Networks",
    "0007F7": "Galtronics",
    "000829": "TOKYO ELECTRON DEVICE NAGASAKI LIMITED",
    "00081B": "Windigo Systems",
    "0007B4": "Cisco Systems, Inc",
    "0007B3": "Cisco Systems, Inc",
    "0007B5": "Any One Wireless Ltd.",
    "0007EF": "Lockheed Martin Tactical Systems",
    "0007F4": "Eletex Co., Ltd.",
    "000852": "Davolink Co. Inc.",
    "000857": "Polaris Networks, Inc.",
    "000736": "Data Video Technologies Co., Ltd.",
    "000738": "Young Technology Co., Ltd.",
    "00077F": "J Communications Co., Ltd.",
    "000771": "Embedded System Corporation",
    "000770": "Ubiquoss Inc",
    "000764": "YoungWoo Telecom Co. Ltd.",
    "000766": "Chou Chin Industrial Co., Ltd.",
    "000761": "29530",
    "000755": "Lafon",
    "000759": "Boris Manufacturing Corp.",
    "000751": "m-u-t AG",
    "000750": "Cisco Systems, Inc",
    "000789": "Allradio Co., Ltd",
    "000787": "Idea System Co., Ltd.",
    "000785": "Cisco Systems, Inc",
    "000775": "Valence Semiconductor, Inc.",
    "000793": "Shin Satellite Public Company Limited",
    "000796": "LSI Systems, Inc.",
    "00078B": "Wegener Communications, Inc.",
    "000783": "SynCom Network, Inc.",
    "000742": "Ormazabal",
    "00074B": "Daihen Corporation",
    "00074A": "Carl Valentin GmbH",
    "00073C": "Telecom Design",
    "000719": "Mobiis Co., Ltd.",
    "00070D": "Cisco Systems, Inc",
    "00070E": "Cisco Systems, Inc",
    "00070B": "Novabase SGPS, SA",
    "0007A2": "Opteon Corporation",
    "000729": "Kistler Instrumente AG",
    "00072A": "Innovance Networks",
    "0006EE": "Shenyang Neu-era Information & Technology Stock Co., Ltd",
    "0006E8": "Optical Network Testing, Inc.",
    "0006E2": "Ceemax Technology Co., Ltd.",
    "0006E1": "Techno Trade s.a",
    "000710": "Adax, Inc.",
    "0006F8": "The Boeing Company",
    "0006FB": "Hitachi Printing Solutions, Ltd.",
    "0006FC": "Fnet Co., Ltd.",
    "0006B5": "Source Photonics, Inc.",
    "0006A5": "PINON Corp.",
    "00069D": "Petards Ltd",
    "0006A8": "KC Technology, Inc.",
    "0006A0": "Mx Imaging",
    "0006BA": "Westwave Communications",
    "0006BE": "Baumer Optronic GmbH",
    "0006C4": "Piolink Inc.",
    "0006F4": "Prime Electronics & Satellitics Inc.",
    "000690": "Euracom Communication GmbH",
    "0006DA": "ITRAN Communications Ltd.",
    "0006D0": "Elgar Electronics Corp.",
    "0006CC": "JMI Electronics Co., Ltd.",
    "0006BF": "Accella Technologies Co., Ltd.",
    "00067E": "WinCom Systems, Inc.",
    "000677": "SICK AG",
    "00062E": "Aristos Logic Corp.",
    "000625": "The Linksys Group, Inc.",
    "000627": "Uniwide Technologies, Inc.",
    "000624": "Gentner Communications Corp.",
    "00062C": "Bivio Networks",
    "000621": "Hinox, Co., Ltd.",
    "00061C": "Hoshino Metal Industries, Ltd.",
    "0005F5": "Geospace Technologies",
    "0005FD": "PacketLight Networks Ltd.",
    "0005E4": "Red Lion Controls Inc.",
    "0005F2": "Power R, Inc.",
    "0005F3": "Webyn",
    "000601": "Otanikeiki Co., Ltd.",
    "000667": "Tripp Lite",
    "000660": "NADEX Co., Ltd.",
    "00064E": "Broad Net Technology Inc.",
    "00062D": "TouchStar Technologies, L.L.C.",
    "00063C": "Intrinsyc Software International Inc.",
    "000630": "Adtranz Sweden",
    "000637": "Toptrend-Meta Information (ShenZhen) Inc.",
    "0005DF": "Electronic Innovation, Inc.",
    "0005E0": "Empirix Corp.",
    "0005D8": "Arescom, Inc.",
    "0005DE": "Gi Fone Korea, Inc.",
    "0005C4": "Telect, Inc.",
    "000605": "Inncom International, Inc.",
    "0005FA": "IPOptical, Inc.",
    "0005E5": "Renishaw PLC",
    "000623": "MGE UPS Systems France",
    "00060B": "Artesyn Embedded Technologies",
    "000615": "Kimoto Electric Co., Ltd.",
    "00060A": "Blue2space",
    "000679": "Konami Corporation",
    "00066C": "Robinson Corporation",
    "000666": "Roving Networks",
    "000646": "ShenZhen XunBao Network Technology Co Ltd",
    "00064B": "Alexon Co., Ltd.",
    "000563": "J-Works, Inc.",
    "000557": "Agile TV Corporation",
    "00055B": "Charles Industries, Ltd.",
    "000554": "Rangestar Wireless",
    "000553": "DVC Company, Inc.",
    "00054B": "Eaton Automation AG",
    "000567": "Etymonic Design, Inc.",
    "000565": "Tailyn Communication Company Ltd.",
    "000566": "Secui.com Corporation",
    "00056C": "Hung Chang Co., Ltd.",
    "00055F": "Cisco Systems, Inc",
    "00055D": "D-LINK SYSTEMS, INC.",
    "000561": "nac Image Technology, Inc.",
    "0005D4": "FutureSmart Networks, Inc.",
    "0005CE": "Prolink Microsystems Corporation",
    "000594": "HMS Industrial Networks",
    "00057C": "RCO Security AB",
    "000583": "ImageCom Limited",
    "00056F": "Innomedia Technologies Pvt. Ltd.",
    "000574": "Cisco Systems, Inc",
    "00059E": "Zinwell Corporation",
    "00058E": "Flextronics International GmbH & Co. Nfg. KG",
    "0005C2": "Soronti, Inc.",
    "00059D": "Daniel Computing Systems, Inc.",
    "0005A4": "Lucid Voice Ltd.",
    "0005B3": "Asahi-Engineering Co., Ltd.",
    "0005A5": "KOTT",
    "0005A3": "QEI, Inc.",
    "000542": "Otari, Inc.",
    "0004F6": "Amphus",
    "0004F4": "Infinite Electronics Inc.",
    "0004F1": "WhereNet",
    "000521": "Control Microsystems",
    "00051F": "Taijin Media Co., Ltd.",
    "000523": "AVL List GmbH",
    "000537": "Nets Technology Co., Ltd.",
    "000532": "Cisco Systems, Inc",
    "000511": "Complementary Technologies Ltd",
    "000506": "Reddo Networks AB",
    "00050A": "ICS Spa",
    "0004EC": "Memobox SA",
    "0004E4": "Daeryung Ind., Inc.",
    "00050C": "Network Photonics, Inc.",
    "0004E2": "SMC Networks, Inc.",
    "000536": "Danam Communications, Inc.",
    "000526": "IPAS GmbH",
    "00042F": "International Communications Products, Inc.",
    "000429": "Pixord Corporation",
    "000426": "Autosys",
    "000421": "Ocular Networks",
    "000424": "TMC s.r.l.",
    "00041D": "Corega of America",
    "00041A": "Ines Test and Measurement GmbH & CoKG",
    "00041E": "Shikoku Instrumentation Co., Ltd.",
    "000467": "Wuhan Research Institute of MII",
    "00045A": "The Linksys Group, Inc.",
    "000463": "Bosch Security Systems",
    "00045C": "Mobiwave Pte Ltd",
    "000451": "Medrad, Inc.",
    "000453": "YottaYotta, Inc.",
    "000450": "DMD Computers SRL",
    "000443": "Agilent Technologies, Inc.",
    "000449": "Mapletree Networks",
    "0004CB": "Tdsoft Communication, Ltd.",
    "0004CC": "Peek Traffic B.V.",
    "0004C8": "LIBA Maschinenfabrik GmbH",
    "0004BF": "VersaLogic Corp.",
    "0004C3": "CASTOR Informatique",
    "0004B1": "Signal Technology, Inc.",
    "000483": "Deltron Technology, Inc.",
    "000482": "Medialogic Corp.",
    "000415": "Rasteme Systems Co., Ltd.",
    "000441": "Half Dome Systems, Inc.",
    "0004B5": "Equitrac Corporation",
    "0003C7": "hopf Elektronik GmbH",
    "0003C2": "Solphone K.K.",
    "0003C6": "ICUE Systems, Inc.",
    "0003BB": "Signal Communications Limited",
    "0003BE": "Netility",
    "000403": "Nexsi Corporation",
    "000406": "Fa. Metabox AG",
    "0003F8": "SanCastle Technologies, Inc.",
    "0003FA": "TiMetra Networks",
    "0003FB": "ENEGATE Co.,Ltd.",
    "0003F6": "Allegro Networks, Inc.",
    "000397": "FireBrick Limited",
    "000398": "WISI",
    "00039E": "Tera System Co., Ltd.",
    "000392": "Hyundai Teletek Co., Ltd.",
    "000395": "California Amplifier",
    "0003E5": "Hermstedt SG",
    "0003E8": "Wavesight Limited",
    "0003DF": "Desana Systems",
    "0003DA": "Takamisawa Cybernetics Co., Ltd.",
    "0003D9": "Secheron SA",
    "0003F3": "Dazzle Multimedia, Inc.",
    "0003EC": "ICG Research, Inc.",
    "0003E9": "Akara Canada, Inc.",
    "0003B4": "Macrotek International Corp.",
    "0003A6": "Traxit Technology, Inc.",
    "0003A4": "Imation Corp.",
    "0003AB": "Bridge Information Systems",
    "0003CA": "MTS Systems Corp.",
    "0003CB": "SystemGear Co., Ltd.",
    "00037C": "Coax Media",
    "00037F": "Atheros Communications, Inc.",
    "0002F0": "AME Optimedia Technology Co., Ltd.",
    "00038E": "Atoga Systems, Inc.",
    "00030D": "Uniwill Computer Corp.",
    "000309": "Texcel Technology PLC",
    "000303": "JAMA Electronics Co., Ltd.",
    "000305": "MSC Vertriebs GmbH",
    "0002FE": "Viditec, Inc.",
    "00019F": "ReadyNet",
    "0002FF": "Handan BroadInfoCom",
    "000321": "Reco Research Co., Ltd.",
    "000316": "Nobell Communications, Inc.",
    "00030F": "Digital China (Shanghai) Networks Ltd.",
    "000311": "Micro Technology Co., Ltd.",
    "000327": "HMS Industrial Networks",
    "00032E": "Scope Information Management, Ltd.",
    "000329": "F3, Inc.",
    "00031A": "Beijing Broad Telecom Ltd., China",
    "00035B": "BridgeWave Communications",
    "000357": "Intervoice-Brite, Inc.",
    "000345": "Routrek Networks Corporation",
    "0002E9": "CS Systemes De Securite - C3S",
    "0002E5": "Timeware Ltd.",
    "0002E0": "ETAS GmbH",
    "00033F": "BigBand Networks, Ltd.",
    "000368": "Embedone Co., Ltd.",
    "0002F5": "VIVE Synergies, Inc.",
    "0002B7": "Watanabe Electric Industry Co., Ltd.",
    "0002AF": "TeleCruz Technology, Inc.",
    "0002A8": "Air Link Technology",
    "0002AB": "CTC Union Technologies Co., Ltd.",
    "0002A4": "AddPac Technology Co., Ltd.",
    "0002A3": "Hitachi Energy Switzerland Ltd",
    "0002A0": "Flatstack Ltd.",
    "0002C8": "Technocom Communications Technology (pte) Ltd",
    "0002B8": "WHI KONSULT AB",
    "0002BB": "Continuous Computing Corp",
    "0002BC": "LVL 7 Systems, Inc.",
    "0002A9": "RACOM, s.r.o.",
    "0002B2": "Cablevision",
    "00025B": "Cambridge Silicon Radio",
    "000256": "Alpha Processor, Inc.",
    "000259": "Tsann Kuen China (Shanghai)Enterprise Co., Ltd. IT Group",
    "000251": "Soma Networks, Inc.",
    "00026A": "Cocess Telecom Co., Ltd.",
    "00026C": "Philips CFT",
    "000262": "Soyo Group Soyo Com Tech Co., Ltd",
    "000265": "Virditech Co. Ltd.",
    "00028A": "Ambit Microsystems Corporation",
    "0001FA": "HOROSCAS",
    "000282": "ViaClix, Inc.",
    "000285": "Riverstone Networks",
    "000294": "Tokyo Sokushin Co., Ltd.",
    "000296": "Lectron Co,. Ltd.",
    "00028E": "Rapid 5 Networks, Inc.",
    "000279": "Control Applications, Ltd.",
    "00024F": "IPM Datacom S.R.L.",
    "0002D5": "ACR",
    "0002CE": "FoxJet, Inc.",
    "0002C3": "Arelnet Ltd.",
    "00023C": "Creative Technology, Ltd.",
    "00022C": "ABB Bomem, Inc.",
    "0001FF": "Data Direct Networks, Inc.",
    "0001FC": "Keyence Corporation",
    "0001FD": "Digital Voice Systems, Inc.",
    "0001EE": "Comtrol Europe, Ltd.",
    "0001F0": "Tridium, Inc.",
    "0001F1": "Innovative Concepts, Inc.",
    "0001E2": "Ando Electric Corporation",
    "00022F": "P-Cube, Ltd.",
    "000227": "ESD Electronic System Design GmbH",
    "00021D": "Data General Communication Ltd.",
    "000219": "Paralon Technologies",
    "0001DA": "WINCOMM Corporation",
    "0001D2": "inXtron, Inc. ",
    "0001C6": "Quarry Technologies",
    "000210": "Fenecom",
    "00020B": "Native Networks, Inc.",
    "000218": "Advanced Scientific Corp",
    "000203": "Woonsang Telecom, Inc.",
    "0001F5": "ERIM S.A.",
    "0001CE": "Custom Micro Products, Ltd.",
    "0001BB": "Frequentis",
    "0001BC": "Brains Corporation",
    "0001C0": "CompuLab, Ltd.",
    "0001A0": "Infinilink Corporation",
    "000196": "Cisco Systems, Inc",
    "00017C": "AG-E GmbH",
    "0001D3": "PAXCOMM, Inc.",
    "0001E1": "Kinpo Electronics, Inc.",
    "000199": "HeiSei Electronics",
    "000110": "Gotham Networks",
    "000112": "Shark Multimedia Inc.",
    "000116": "Netspect Technologies, Inc.",
    "000108": "AVLAB Technology, Inc.",
    "00306C": "Hitex Holding GmbH",
    "00308B": "Brix Networks",
    "000177": "EDSL",
    "00014D": "Shin Kin Enterprises Co., Ltd",
    "000174": "CyberOptics Corporation",
    "00015E": "BEST TECHNOLOGY CO., LTD.",
    "000161": "Meta Machine Technology",
    "000155": "Promise Technology, Inc.",
    "0001A1": "Mag-Tek, Inc.",
    "000186": "Uwe Disch",
    "0001A6": "Scientific-Atlanta Arcodan A/S",
    "000172": "TechnoLand Co., LTD.",
    "00012E": "PC Partner Ltd.",
    "00013E": "Ascom Tateco AB",
    "000132": "Dranetz - BMI",
    "00013B": "BNA SYSTEMS",
    "00019D": "E-Control Systems, Inc.",
    "00018B": "NetLinks Co., Ltd.",
    "00018D": "AudeSi Technologies",
    "000113": "OLYMPUS CORPORATION",
    "000134": "Selectron Systems AG",
    "00011E": "Precidia Technologies, Inc.",
    "00016E": "Conklin Corporation",
    "003032": "MagicRam, Inc.",
    "0030EA": "TeraForce Technology Corporation",
    "003069": "IMPACCT TECHNOLOGY CORP.",
    "0030C3": "FLUECKIGER ELEKTRONIK AG",
    "00305A": "TELGEN CORPORATION",
    "00309B": "Smartware",
    "003045": "Village Networks, Inc. (VNI)",
    "0030E5": "Amper Datos S.A.",
    "003094": "Cisco Systems, Inc",
    "003040": "Cisco Systems, Inc",
    "00308A": "NICOTRA SISTEMI S.P.A",
    "00062B": "INTRASERVER TECHNOLOGY",
    "000100": "EQUIP'TRANS",
    "00B09D": "Point Grey Research Inc.",
    "00B06D": "Jones Futurex Inc.",
    "00B094": "Alaris, Inc.",
    "003072": "Intellibyte Inc.",
    "003006": "SUPERPOWER COMPUTER",
    "003038": "XCP, INC.",
    "003079": "CQOS, INC.",
    "0030F0": "Uniform Industrial Corp.",
    "0030C9": "LuxN, N",
    "00300C": "CONGRUENCY, LTD.",
    "00304C": "APPIAN COMMUNICATIONS, INC.",
    "0030E8": "ENSIM CORP.",
    "003028": "FASE Saldatura srl",
    "0030D9": "DATACORE SOFTWARE CORP.",
    "003026": "HeiTel Digital Video GmbH",
    "003047": "NISSEI ELECTRIC CO., LTD.",
    "003077": "ONPREM NETWORKS",
    "0030D4": "AAE Systems, Inc.",
    "003010": "VISIONETICS INTERNATIONAL",
    "003017": "BlueArc UK Ltd",
    "0030F7": "RAMIX INC.",
    "00D0D7": "B2C2, INC.",
    "00D073": "ACN ADVANCED COMMUNICATIONS",
    "00D057": "ULTRAK, INC.",
    "00D0E5": "SOLIDUM SYSTEMS CORP.",
    "00D0D3": "Cisco Systems, Inc",
    "00D017": "SYNTECH INFORMATION CO., LTD.",
    "00D036": "TECHNOLOGY ATLANTA CORP.",
    "0030AB": "DELTA NETWORKS, INC.",
    "003049": "BRYANT TECHNOLOGY, LTD.",
    "00306D": "LUCENT TECHNOLOGIES",
    "00D045": "KVASER AB",
    "00D004": "PENTACOM LTD.",
    "00D005": "ZHS ZEITMANAGEMENTSYSTEME",
    "003080": "Cisco Systems, Inc",
    "003081": "ALTOS C&C",
    "0030D0": "Tellabs",
    "003014": "DIVIO, INC.",
    "00301C": "ALTVATER AIRDATA SYSTEMS",
    "00D01E": "PINGTEL CORP.",
    "00D065": "TOKO ELECTRIC",
    "00D014": "ROOT, INC.",
    "00D0CA": "Intrinsyc Software International Inc.",
    "00D023": "INFORTREND TECHNOLOGY, INC.",
    "00D0A2": "INTEGRATED DEVICE",
    "00D010": "CONVERGENT NETWORKS, INC.",
    "00D04B": "LA CIE GROUP S.A.",
    "00D00E": "PLURIS, INC.",
    "00D012": "GATEWORKS CORP.",
    "00D04D": "DIV OF RESEARCH & STATISTICS",
    "00D02E": "COMMUNICATION AUTOMATION CORP.",
    "00D0C5": "COMPUTATIONAL SYSTEMS, INC.",
    "00D046": "DOLBY LABORATORIES, INC.",
    "00D0DE": "PHILIPS MULTIMEDIA NETWORK",
    "00D00C": "SNIJDER MICRO SYSTEMS",
    "00D0DA": "TAICOM DATA SYSTEMS CO., LTD.",
    "00D03C": "Vieo, Inc.",
    "00D09A": "FILANET CORPORATION",
    "00D0AE": "ORESIS COMMUNICATIONS, INC.",
    "00D0F2": "MONTEREY NETWORKS",
    "00D0A8": "NETWORK ENGINES, INC.",
    "00D0B4": "KATSUJIMA CO., LTD.",
    "00D086": "FOVEON, INC.",
    "00D0E8": "MAC SYSTEM CO., LTD.",
    "00D06B": "SR TELECOM INC.",
    "00D0DC": "MODULAR MINING SYSTEMS, INC.",
    "00D026": "HIRSCHMANN AUSTRIA GMBH",
    "00D0F0": "CONVISION TECHNOLOGY GMBH",
    "00D0E3": "ELE-CHEM ENGINEERING CO., LTD.",
    "00D0C4": "TERATECH CORPORATION",
    "00D061": "TREMON ENTERPRISES CO., LTD.",
    "00D0AB": "DELTAKABEL TELECOM CV",
    "00D01D": "FURUNO ELECTRIC CO., LTD.",
    "00D0A7": "TOKYO SOKKI KENKYUJO CO., LTD.",
    "005076": "IBM Corp",
    "0050D4": "JOOHONG INFORMATION &",
    "0050A6": "OPTRONICS",
    "0050DB": "CONTEMPORARY CONTROL",
    "00507C": "VIDEOCON AG",
    "005047": "Private",
    "005084": "Quantum Corp.",
    "0050A9": "MOLDAT WIRELESS TECHNOLGIES",
    "00509B": "SWITCHCORE AB",
    "00507E": "NEWER TECHNOLOGY",
    "0050CE": "LG INTERNATIONAL CORP.",
    "0050F7": "VENTURE MANUFACTURING (SINGAPORE) LTD.",
    "005019": "SPRING TIDE NETWORKS, INC.",
    "00501B": "ABL CANADA, INC.",
    "0050DC": "TAS TELEFONBAU A. SCHWABE GMBH & CO. KG",
    "005008": "TIVA MICROCOMPUTER CORP. (TMC)",
    "00501F": "MRG SYSTEMS, LTD.",
    "0050FD": "VISIONCOMM CO., LTD.",
    "0050BF": "Metalligence Technology Corp.",
    "005036": "NETCAM, LTD.",
    "005083": "GILBARCO, INC.",
    "00D06C": "SHAREWAVE, INC.",
    "00D0EE": "DICTAPHONE CORPORATION",
    "00504C": "Galil Motion Control",
    "00D034": "ORMEC SYSTEMS CORP.",
    "00D08A": "PHOTRON USA",
    "0050A7": "Cisco Systems, Inc",
    "005043": "MARVELL SEMICONDUCTOR, INC.",
    "0050FA": "OXTEL, LTD.",
    "005055": "DOMS A/S",
    "005072": "CORVIS CORPORATION",
    "0090EF": "INTEGRIX, INC.",
    "0090C5": "INTERNET MAGIC, INC.",
    "00908C": "ETREND ELECTRONICS, INC.",
    "009048": "ZEAL CORPORATION",
    "009007": "DOMEX TECHNOLOGY CORP.",
    "00902D": "DATA ELECTRONICS (AUST.) PTY, LTD.",
    "005016": "Molex Canada Ltd",
    "005052": "TIARA NETWORKS, INC.",
    "005064": "CAE ELECTRONICS",
    "00903D": "BIOPAC SYSTEMS, INC.",
    "009057": "AANetcom, Inc.",
    "009083": "TURBO COMMUNICATION, INC.",
    "0090D7": "NetBoost Corp.",
    "005027": "GENICOM CORPORATION",
    "00505A": "NETWORK ALCHEMY, INC.",
    "005039": "MARINER NETWORKS",
    "005095": "PERACOM NETWORKS",
    "0090B9": "BERAN INSTRUMENTS LTD.",
    "0090A5": "SPECTRA LOGIC",
    "0090A3": "Corecess Inc.",
    "00901F": "ADTEC PRODUCTIONS, INC.",
    "009038": "FOUNTAIN TECHNOLOGIES, INC.",
    "0090B0": "VADEM",
    "00505B": "KAWASAKI LSI U.S.A., INC.",
    "0050CC": "Seagate Cloud Systems Inc",
    "0090D4": "BindView Development Corp.",
    "0090C4": "JAVELIN SYSTEMS, INC.",
    "0050B8": "INOVA COMPUTERS GMBH & CO. KG",
    "0090B6": "FIBEX SYSTEMS",
    "009063": "COHERENT COMMUNICATIONS SYSTEMS CORPORATION",
    "009062": "ICP VORTEX COMPUTERSYSTEME GmbH",
    "0090C3": "TOPIC SEMICONDUCTOR CORP.",
    "00905A": "DEARBORN GROUP, INC.",
    "0090EC": "PYRESCOM",
    "0010FB": "ZIDA TECHNOLOGIES LIMITED",
    "001053": "COMPUTER TECHNOLOGY CORP.",
    "0010ED": "SUNDANCE TECHNOLOGY, INC.",
    "00109D": "CLARINET SYSTEMS, INC.",
    "00100E": "MICRO LINEAR COPORATION",
    "009095": "UNIVERSAL AVIONICS",
    "009041": "APPLIED DIGITAL ACCESS",
    "009024": "PIPELINKS, INC.",
    "00903A": "NIHON MEDIA TOOL INC.",
    "0090B2": "AVICI SYSTEMS INC.",
    "009082": "FORCE INSTITUTE",
    "009000": "DIAMOND MULTIMEDIA",
    "009054": "INNOVATIVE SEMICONDUCTORS, INC",
    "00906E": "PRAXON, INC.",
    "0090F6": "ESCALATE NETWORKS, INC.",
    "0090A8": "NineTiles Networks, Ltd.",
    "009015": "CENTIGRAM COMMUNICATIONS CORP.",
    "009029": "CRYPTO AG",
    "00900B": "LANNER ELECTRONICS, INC.",
    "009061": "PACIFIC RESEARCH & ENGINEERING CORPORATION",
    "009011": "WAVTrace, Inc.",
    "009065": "FINISAR CORPORATION",
    "0010D3": "GRIPS ELECTRONIC GMBH",
    "0090C0": "K.J. LAW ENGINEERS, INC.",
    "0090CE": " avateramedical Mechatronics GmbH",
    "0090DF": "MITSUBISHI CHEMICAL AMERICA, INC.",
    "0010C8": "COMMUNICATIONS ELECTRONICS SECURITY GROUP",
    "001086": "ATTO Technology, Inc.",
    "0010F3": "Nexcom International Co., Ltd.",
    "0010DF": "RISE COMPUTER INC.",
    "001072": "GVN TECHNOLOGIES, INC.",
    "0010DA": "Kollmorgen Corp",
    "0010E4": "NSI CORPORATION",
    "001088": "AMERICAN NETWORKS INC.",
    "00107E": "BACHMANN ELECTRONIC GmbH",
    "0010A0": "INNOVEX TECHNOLOGIES, INC.",
    "001016": "T.SQWARE",
    "001090": "CIMETRICS, INC.",
    "0010F5": "AMHERST SYSTEMS, INC.",
    "00103D": "PHASECOM, LTD.",
    "0010EA": "ADEPT TECHNOLOGY",
    "001022": "SatCom Media Corporation",
    "001096": "TRACEWELL SYSTEMS, INC.",
    "001082": "JNA TELECOMMUNICATIONS LIMITED",
    "001098": "STARNET TECHNOLOGIES, INC.",
    "001068": "COMOS TELECOM",
    "00103F": "TOLLGRADE COMMUNICATIONS, INC.",
    "0010CD": "INTERFACE CONCEPT",
    "001056": "SODICK CO., LTD.",
    "001061": "HOSTLINK CORP.",
    "001099": "InnoMedia, Inc.",
    "001042": "Alacritech, Inc.",
    "0010E1": "S.I. TECH, INC.",
    "0010BB": "DATA & INFORMATION TECHNOLOGY",
    "001020": "Hand Held Products Inc",
    "00103A": "DIAMOND NETWORK TECH",
    "001004": "THE BRANTLEY COILE COMPANY,INC",
    "0010EF": "DBTEL INCORPORATED",
    "001076": "EUREM GmbH",
    "0010AE": "SHINKO ELECTRIC INDUSTRIES CO.",
    "0010C4": "MEDIA GLOBAL LINKS CO., LTD.",
    "0010FE": "DIGITAL EQUIPMENT CORPORATION",
    "00106C": "EDNT GmbH",
    "0010E9": "RAIDTEC LTD.",
    "001003": "IMATRON, INC.",
    "001071": "ADVANET INC.",
    "00102A": "ZF MICROSYSTEMS, INC.",
    "0010E5": "SOLECTRON TEXAS",
    "0010F7": "IRIICHI TECHNOLOGIES Inc.",
    "0010AB": "KOITO ELECTRIC INDUSTRIES, LTD.",
    "001010": "INITIO CORPORATION",
    "0010F2": "ANTEC",
    "00E007": "Avaya ECS Ltd",
    "0010BE": "MARCH NETWORKS CORPORATION",
    "0010AF": "TAC SYSTEMS, INC.",
    "00108C": "Fujitsu Services Ltd",
    "001058": "ArrowPoint Communications",
    "00100F": "INDUSTRIAL CPU SYSTEMS",
    "0010BC": "Aastra Telecom",
    "001049": "ShoreTel, Inc",
    "00105E": "Spirent plc, Service Assurance Broadband",
    "00E0BF": "TORRENT NETWORKING TECHNOLOGIES CORP.",
    "00E0E3": "SK-ELEKTRONIK GMBH",
    "00E0C6": "LINK2IT, L.L.C.",
    "00E0E5": "CINCO NETWORKS, INC.",
    "00E061": "EdgePoint Networks, Inc.",
    "00E0B8": "GATEWAY 2000",
    "00E07C": "METTLER-TOLEDO, INC.",
    "00E026": "Redlake MASD LLC",
    "00E020": "TECNOMEN OY",
    "00E00D": "RADIANT SYSTEMS",
    "00E0DC": "NEXWARE CORP.",
    "00E037": "CENTURY CORPORATION",
    "00E053": "CELLPORT LABS, INC.",
    "00E0D3": "DATENTECHNIK GmbH",
    "00E043": "VitalCom",
    "00E0B3": "EtherWAN Systems, Inc.",
    "00E0ED": "SILICOM, LTD.",
    "00E0FB": "LEIGHTRONIX, INC.",
    "00E0C2": "NECSY S.p.A.",
    "00E09B": "ENGAGE NETWORKS, INC.",
    "00E045": "TOUCHWAVE, INC.",
    "00E040": "DeskStation Technology, Inc.",
    "00E01A": "COMTEC SYSTEMS. CO., LTD.",
    "00E081": "TYAN COMPUTER CORP.",
    "00E057": "HAN MICROTELECOM. CO., LTD.",
    "00E0BC": "SYMON COMMUNICATIONS, INC.",
    "00E07E": "WALT DISNEY IMAGINEERING",
    "00E078": "BERKELEY NETWORKS",
    "00E087": "LeCroy - Networking Productions Division",
    "00E041": "CSPI",
    "00E0E2": "INNOVA CORP.",
    "00E082": "ANERMA",
    "00E077": "WEBGEAR, INC.",
    "00E056": "HOLONTECH CORPORATION",
    "00E02F": "MCNS HOLDINGS, L.P.",
    "00E06C": "Ultra Electronics Command & Control Systems",
    "00E04A": "ZX Technologies, Inc",
    "00E031": "HAGIWARA ELECTRIC CO., LTD.",
    "00E00B": "ROOFTOP COMMUNICATIONS CORP.",
    "00E0B2": "TELMAX COMMUNICATIONS CORP.",
    "00E0EC": "CELESTICA INC.",
    "00E07A": "MIKRODIDAKT AB",
    "00E099": "SAMSON AG",
    "006022": "VICOM SYSTEMS, INC.",
    "0060EE": "APOLLO",
    "0060D8": "ELMIC SYSTEMS, INC.",
    "0060EF": "FLYTECH TECHNOLOGY CO., LTD.",
    "006085": "Storage Concepts",
    "006053": "TOYODA MACHINE WORKS, LTD.",
    "006056": "NETWORK TOOLS, INC.",
    "00600C": "Eurotech Inc.",
    "006032": "I-CUBE, INC.",
    "006033": "ACUITY IMAGING, INC.",
    "006013": "NETSTAL MASCHINEN AG",
    "000288": "GLOBAL VILLAGE COMMUNICATION",
    "006034": "ROBERT BOSCH GmbH",
    "006050": "INTERNIX INC.",
    "0060E0": "AXIOM TECHNOLOGY CO., LTD.",
    "006096": "T.S. MICROTECH INC.",
    "00601D": "LUCENT TECHNOLOGIES",
    "00607B": "FORE SYSTEMS, INC.",
    "006011": "CRYSTAL SEMICONDUCTOR CORP.",
    "0060FA": "EDUCATIONAL TECHNOLOGY RESOURCES, INC.",
    "0060DA": "Red Lion Controls, LP",
    "0060AE": "TRIO INFORMATION SYSTEMS AB",
    "006043": "iDirect, INC.",
    "00603A": "QUICK CONTROLS LTD.",
    "006028": "MACROVISION CORPORATION",
    "00601C": "TELXON CORPORATION",
    "0060C9": "ControlNet, Inc.",
    "00605F": "NIPPON UNISOFT CORPORATION",
    "006091": "FIRST PACIFIC NETWORKS, INC.",
    "00A061": "PURITAN BENNETT",
    "00A03C": "EG&G NUCLEAR INSTRUMENTS",
    "00A0C4": "CRISTIE ELECTRONICS LTD.",
    "00A063": "JRL SYSTEMS, INC.",
    "006045": "PATHLIGHT TECHNOLOGIES",
    "00A05D": "CS COMPUTER SYSTEME GmbH",
    "00A033": "imc MeBsysteme GmbH",
    "00A0A9": "NAVTEL COMMUNICATIONS INC.",
    "00A071": "VIDEO LOTTERY TECHNOLOGIES,INC",
    "00A0EA": "ETHERCOM CORP.",
    "00A0DC": "O.N. ELECTRONIC CO., LTD.",
    "00A00B": "COMPUTEX CO., LTD.",
    "00A0E2": "Keisokugiken Corporation",
    "0060F0": "JOHNSON & JOHNSON MEDICAL, INC",
    "0060F5": "ICON WEST, INC.",
    "006062": "TELESYNC, INC.",
    "0060E4": "COMPUSERVE, INC.",
    "0060BE": "WEBTRONICS",
    "0060BF": "MACRAIGOR SYSTEMS, INC.",
    "006000": "XYCOM INC.",
    "0060A6": "PARTICLE MEASURING SYSTEMS",
    "00602A": "SYMICRON COMPUTER COMMUNICATIONS, LTD.",
    "00A06D": "MANNESMANN TALLY CORPORATION",
    "00A0F6": "AutoGas Systems Inc.",
    "00608F": "TEKRAM TECHNOLOGY CO., LTD.",
    "0060C4": "SOLITON SYSTEMS K.K.",
    "006080": "MICROTRONIX DATACOM LTD.",
    "00A037": "Mindray DS USA, Inc.",
    "00A04C": "INNOVATIVE SYSTEMS & TECHNOLOGIES, INC.",
    "00A031": "HAZELTINE CORPORATION, MS 1-17",
    "00A041": "INFICON",
    "00A01A": "BINAR ELEKTRONIK AB",
    "00A088": "ESSENTIAL COMMUNICATIONS",
    "00A0C2": "R.A. SYSTEMS CO., LTD.",
    "00A04B": "TFL LAN INC.",
    "00A064": "KVB/ANALECT",
    "00A03E": "ATM FORUM",
    "00A098": "NetApp",
    "00A06C": "SHINDENGEN ELECTRIC MFG. CO., LTD.",
    "00A01F": "TRICORD SYSTEMS, INC.",
    "00A0FB": "Toray Engineering D Solutions Co., Ltd.",
    "00A07E": "AVID TECHNOLOGY, INC.",
    "00A06F": "Color Sentinel Systems, LLC",
    "00A0C7": "TADIRAN TELECOMMUNICATIONS",
    "00A02C": "interWAVE Communications",
    "00A0F7": "V.I COMPUTER CORP.",
    "00A090": "TimeStep Corporation",
    "00A0A7": "VORAX CORPORATION",
    "00A08B": "ASTON ELECTRONIC DESIGNS LTD.",
    "00A0B1": "FIRST VIRTUAL CORPORATION",
    "002010": "JEOL SYSTEM TECHNOLOGY CO. LTD",
    "00209F": "MERCURY COMPUTER SYSTEMS, INC.",
    "002091": "J125, NATIONAL SECURITY AGENCY",
    "0020BD": "NIOBRARA R & D CORPORATION",
    "002054": "Sycamore Networks",
    "0020A7": "PAIRGAIN TECHNOLOGIES, INC.",
    "002072": "WORKLINK INNOVATIONS",
    "0020CB": "PRETEC ELECTRONICS CORP.",
    "0020EB": "CINCINNATI MICROWAVE, INC.",
    "0020A0": "OA LABORATORY CO., LTD.",
    "00200D": "CARL ZEISS",
    "00202D": "TAIYO CORPORATION",
    "00A0DB": "FISHER & PAYKEL PRODUCTION",
    "00A097": "JC INFORMATION SYSTEMS",
    "00A073": "COM21, INC.",
    "00A03A": "KUBOTEK CORPORATION",
    "00A0B2": "SHIMA SEIKI",
    "00A027": "FIREPOWER SYSTEMS, INC.",
    "00A081": "ALCATEL DATA NETWORKS",
    "00A0D4": "RADIOLAN,  INC.",
    "0020B9": "METRICOM, INC.",
    "002039": "SCINETS",
    "00A046": "SCITEX CORP. LTD.",
    "00A092": "H. BOLLMANN MANUFACTURERS, LTD",
    "0020EC": "TECHWARE SYSTEMS CORP.",
    "00206E": "XACT, INC.",
    "0020F1": "ALTOS INDIA LIMITED",
    "002041": "DATA NET",
    "002076": "REUDO CORPORATION",
    "0020E8": "DATATREK CORPORATION",
    "0020C5": "EAGLE TECHNOLOGY",
    "0020E2": "INFORMATION RESOURCE ENGINEERING",
    "002007": "SFA, INC.",
    "00205C": "InterNet Systems of Florida, Inc.",
    "002055": "ALTECH CO., LTD.",
    "00200A": "SOURCE-COMM CORP.",
    "0020CF": "TEST & MEASUREMENT SYSTEMS INC",
    "0020A5": "API ENGINEERING",
    "002064": "PROTEC MICROSYSTEMS, INC.",
    "002033": "SYNAPSE TECHNOLOGIES, INC.",
    "0020EA": "EFFICIENT NETWORKS, INC.",
    "00206A": "OSAKA COMPUTER CORP.",
    "0020B4": "TERMA ELEKTRONIK AS",
    "0020E4": "HSING TECH ENTERPRISE CO., LTD",
    "0020A2": "GALCOM NETWORKING LTD.",
    "002031": "Tattile SRL ",
    "0020D0": "VERSALYNX CORPORATION",
    "00206C": "EVERGREEN TECHNOLOGY CORP.",
    "00205E": "CASTLE ROCK, INC.",
    "002012": "CAMTRONICS MEDICAL SYSTEMS",
    "002075": "MOTOROLA COMMUNICATION ISRAEL",
    "0020BB": "ZAX CORPORATION",
    "0020A8": "SAST TECHNOLOGY CORP.",
    "002045": "ION Networks, Inc.",
    "00208A": "SONIX COMMUNICATIONS, LTD.",
    "002021": "ALGORITHMS SOFTWARE PVT. LTD.",
    "002049": "COMTRON, INC.",
    "002050": "KOREA COMPUTER INC.",
    "002084": "OCE PRINTING SYSTEMS, GMBH",
    "002009": "PACKARD BELL ELEC., INC.",
    "002027": "MING FORTUNE INDUSTRY CO., LTD",
    "00202C": "WELLTRONIX CO., LTD.",
    "0020DB": "XNET TECHNOLOGY, INC.",
    "001C7C": "PERQ SYSTEMS CORPORATION",
    "00C039": "Teridian Semiconductor Corporation",
    "00C0A9": "BARRON MCCANN LTD.",
    "00C019": "LEAP TECHNOLOGY, INC.",
    "00C0CF": "IMATRAN VOIMA OY",
    "00C07D": "RISC DEVELOPMENTS LTD.",
    "00C0B5": "CORPORATE NETWORK SYSTEMS,INC.",
    "00C04B": "CREATIVE MICROSYSTEMS",
    "00C0B9": "FUNK SOFTWARE, INC.",
    "00C015": "NEW MEDIA CORPORATION",
    "00C083": "TRACE MOUNTAIN PRODUCTS, INC.",
    "00C094": "VMX INC.",
    "00C0F9": "Artesyn Embedded Technologies",
    "00C075": "XANTE CORPORATION",
    "00C05B": "NETWORKS NORTHWEST, INC.",
    "00C008": "SECO SRL",
    "00C0B7": "AMERICAN POWER CONVERSION CORP",
    "00C0FC": "ELASTIC REALITY, INC.",
    "00C0BB": "FORVAL CREATIVE, INC.",
    "0020D2": "RAD DATA COMMUNICATIONS, LTD.",
    "002002": "SERITECH ENTERPRISE CO., LTD.",
    "00204B": "AUTOCOMPUTER CO., LTD.",
    "00208C": "GALAXY NETWORKS, INC.",
    "0020A6": "Proxim Wireless",
    "00C043": "STRATACOM",
    "00C028": "JASCO CORPORATION",
    "00C08D": "TRONIX PRODUCT DEVELOPMENT",
    "00C02A": "OHKURA ELECTRIC CO., LTD.",
    "00C0EF": "ABIT CORPORATION",
    "00C061": "SOLECTEK CORPORATION",
    "00C0AD": "MARBEN COMMUNICATION SYSTEMS",
    "00C07F": "NUPON COMPUTING CORP.",
    "00C057": "MYCO ELECTRONICS",
    "00C056": "SOMELEC",
    "00C027": "CIPHER SYSTEMS, INC.",
    "00C05C": "ELONEX PLC",
    "00C0ED": "US ARMY ELECTRONIC",
    "0040BD": "STARLIGHT NETWORKS, INC.",
    "0040ED": "NETWORK CONTROLS INT'NATL INC.",
    "0040E5": "SYBUS CORPORATION",
    "0040A5": "CLINICOMP INTL.",
    "004005": "ANI COMMUNICATIONS INC.",
    "00C030": "INTEGRATED ENGINEERING B. V.",
    "00C0A6": "EXICOM AUSTRALIA PTY. LTD",
    "00C0CB": "CONTROL TECHNOLOGY CORPORATION",
    "00C0D1": "COMTREE TECHNOLOGY CORPORATION",
    "00C038": "RASTER IMAGE PROCESSING SYSTEM",
    "00C092": "MENNEN MEDICAL INC.",
    "00C052": "BURR-BROWN",
    "00C0EB": "SEH COMPUTERTECHNIK GMBH",
    "0040DB": "ADVANCED TECHNICAL SOLUTIONS",
    "00409B": "HAL COMPUTER SYSTEMS INC.",
    "0040EB": "MARTIN MARIETTA CORPORATION",
    "00C032": "I-CUBED LIMITED",
    "00C0A5": "DICKENS DATA SYSTEMS",
    "00C0D3": "OLYMPUS IMAGE SYSTEMS, INC.",
    "00C0E8": "PLEXCOM, INC.",
    "00400E": "MEMOTEC, INC.",
    "00C03D": "WIESEMANN & THEIS GMBH",
    "00404C": "HYPERTEC PTY LTD.",
    "00C0E0": "DSC COMMUNICATION CORP.",
    "00C0DA": "NICE SYSTEMS LTD.",
    "0040C8": "MILAN TECHNOLOGY CORPORATION",
    "0040BA": "ALLIANT COMPUTER SYSTEMS CORP.",
    "0040CE": "NET-SOURCE, INC.",
    "004062": "E-SYSTEMS, INC./GARLAND DIV.",
    "0040D9": "AMERICAN MEGATRENDS INC.",
    "004021": "RASTER GRAPHICS",
    "0040C1": "BIZERBA-WERKE WILHEIM KRAUT",
    "0040E1": "MARNER INTERNATIONAL, INC.",
    "0040FE": "SYMPLEX COMMUNICATIONS",
    "0040D4": "GAGE TALKER CORP.",
    "004038": "TALENT ELECTRIC INCORPORATED",
    "0040D8": "OCEAN OFFICE AUTOMATION LTD.",
    "0040C6": "FIBERNET RESEARCH, INC.",
    "004032": "DIGITAL COMMUNICATIONS",
    "0040C2": "APPLIED COMPUTING DEVICES",
    "004088": "MOBIUS TECHNOLOGIES, INC.",
    "0080AA": "MAXPEED",
    "00C050": "TOYO DENKI SEIZO K.K.",
    "00401C": "AST RESEARCH, INC.",
    "00400F": "DATACOM TECHNOLOGIES",
    "004006": "SAMPO TECHNOLOGY CORPORATION",
    "004034": "BUSTEK CORPORATION",
    "008093": "XYRON CORPORATION",
    "008092": "Silex Technology, Inc.",
    "00805A": "TULIP COMPUTERS INTERNAT'L B.V",
    "00807E": "SOUTHERN PACIFIC LTD.",
    "0080EF": "RATIONAL",
    "0080F0": "Panasonic Communications Co., Ltd.",
    "008051": "FIBERMUX",
    "0080C6": "NATIONAL DATACOMM CORPORATION",
    "004050": "IRONICS, INCORPORATED",
    "004047": "WIND RIVER SYSTEMS",
    "004041": "FUJIKURA LTD.",
    "008000": "MULTITECH SYSTEMS, INC.",
    "008069": "COMPUTONE SYSTEMS",
    "008035": "TECHNOLOGY WORKS, INC.",
    "00804E": "APEX COMPUTER COMPANY",
    "008055": "FERMILAB",
    "00802A": "TEST SYSTEMS & SIMULATIONS INC",
    "00801D": "INTEGRATED INFERENCE MACHINES",
    "0080C0": "PENRIL DATACOMM",
    "008075": "PARSYTEC GMBH",
    "0080ED": "IQ TECHNOLOGIES, INC.",
    "00809A": "NOVUS NETWORKS LTD",
    "00804A": "PRO-LOG",
    "008004": "ANTLOW COMMUNICATIONS, LTD.",
    "0080D0": "COMPUTER PERIPHERALS, INC.",
    "008024": "KALPANA, INC.",
    "008040": "JOHN FLUKE MANUFACTURING CO.",
    "008021": "Alcatel Canada Inc.",
    "0080E8": "CUMULUS CORPORATIION",
    "000066": "TALARIS SYSTEMS, INC.",
    "000049": "APRICOT COMPUTERS, LTD",
    "0000FA": "MICROSAGE COMPUTER SYSTEMS INC",
    "0000D4": "PURE DATA LTD.",
    "000019": "APPLIED DYNAMICS INTERNATIONAL",
    "0000AB": "LOGIC MODELING CORPORATION",
    "0080F2": "RAYCOM SYSTEMS INC",
    "0080BD": "THE FURUKAWA ELECTRIC CO., LTD",
    "008025": "Telit Wireless Solutions GmbH",
    "0080EA": "ADVA Optical Networking Ltd.",
    "00800D": "VOSSWINKEL F.U.",
    "0080D1": "KIMTRON CORPORATION",
    "00001E": "TELSIST INDUSTRIA ELECTRONICA",
    "000050": "RADISYS CORPORATION",
    "00802E": "CASTLE ROCK COMPUTING",
    "00004F": "LOGICRAFT, INC.",
    "000015": "DATAPOINT CORPORATION",
    "00001C": "BELL TECHNOLOGIES",
    "000034": "NETWORK RESOURCES CORPORATION",
    "000022": "VISUAL TECHNOLOGY INC.",
    "0000B5": "DATABILITY SOFTWARE SYS. INC.",
    "0000E0": "QUADRAM CORP.",
    "000027": "JAPAN RADIO COMPANY",
    "0000E8": "ACCTON TECHNOLOGY CORP.",
    "00002F": "TIMEPLEX INC.",
    "0000E6": "APTOR PRODUITS DE COMM INDUST",
    "00009A": "RC COMPUTER A/S",
    "00004B": "ICL DATA OY",
    "008042": "Artesyn Embedded Technologies",
    "0080AC": "IMLOGIX, DIVISION OF GENESYS",
    "000080": "CRAY COMMUNICATIONS A/S",
    "080089": "Kinetics",
    "080086": "KONICA MINOLTA HOLDINGS, INC.",
    "080083": "Seiko Instruments Inc.",
    "000073": "SIECOR CORPORATION",
    "0000B9": "MCDONNELL DOUGLAS COMPUTER SYS",
    "0000BF": "SYMMETRIC COMPUTER SYSTEMS",
    "00002D": "CHROMATICS INC",
    "000059": "Hellige GMBH",
    "000069": "CONCORD COMMUNICATIONS INC",
    "0000E7": "Star Gate Technologies",
    "00004D": "DCI CORPORATION",
    "00006F": "Madge Ltd.",
    "000078": "LABTAM LIMITED",
    "00005A": "SysKonnect GmbH",
    "000071": "ADRA SYSTEMS INC.",
    "000023": "ABB INDUSTRIAL SYSTEMS AB",
    "000018": "WEBSTER COMPUTER CORPORATION",
    "0000D5": "MICROGNOSIS INTERNATIONAL",
    "00003A": "CHYRON CORPORATION",
    "0000BE": "THE NTI GROUP",
    "0000D9": "NIPPON TELEGRAPH & TELEPHONE",
    "080024": "10NET COMMUNICATIONS/DCA",
    "080022": "NBI INC.",
    "080020": "Oracle Corporation",
    "08001F": "SHARP CORPORATION",
    "020701": "RACAL-DATACOM",
    "080006": "SIEMENS AG",
    "08002A": "MOSAIC TECHNOLOGIES INC.",
    "080013": "Exxon",
    "021C7C": "PERQ SYSTEMS CORPORATION",
    "080061": "JAROGATE LTD.",
    "08005F": "SABER TECHNOLOGY CORP.",
    "080058": "SYSTEMS CONCEPTS",
    "04E0C4": "TRIUMPH-ADLER AG",
    "080049": "UNIVATION",
    "000005": "XEROX CORPORATION",
    "00DD08": "UNGERMANN-BASS INC.",
    "AA0000": "DIGITAL EQUIPMENT CORPORATION",
    "AA0001": "DIGITAL EQUIPMENT CORPORATION",
    "AA0002": "DIGITAL EQUIPMENT CORPORATION",
    "080014": "EXCELAN",
    "080065": "GENRAD INC.",
    "000007": "XEROX CORPORATION",
    "00801F": "KRUPP ATLAS ELECTRONIK GMBH",
    "02AA3C": "OLIVETTI TELECOMM SPA (OLTECO)",
    "080059": "A/S MYCRON",
    "080008": "BOLT BERANEK AND NEWMAN INC.",
    "8487FF": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "645725": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "0080E9": "Madge Ltd.",
    "0040D6": "LOCAMATION B.V.",
    "08004B": "Planning Research Corp.",
    "0425E8": "Texas Instruments",
    "283C90": "Texas Instruments",
    "E4FA5B": "Texas Instruments",
    "000009": "XEROX CORPORATION",
    "44C3B6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8F72F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50A1F3": "Huawei Device Co., Ltd.",
    "DC1057": "Apple, Inc.",
    "30D875": "Apple, Inc.",
    "485FDF": "zte corporation",
    "6C1544": "Microsoft Corporation",
    "683EC0": "Apple, Inc.",
    "3C55DB": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "F0F84A": "BUFFALO.INC",
    "EC6E79": "InHand Networks, INC.",
    "C80C53": "China Mobile Group Device Co.,Ltd.",
    "A4004E": "Cisco Systems, Inc",
    "2C9EE0": "Cavli Inc.",
    "64C17E": "cheilelectric",
    "B02EE0": "Huawei Device Co., Ltd.",
    "A4373E": "Huawei Device Co., Ltd.",
    "C44F5F": "Huawei Device Co., Ltd.",
    "785F28": "EM Microelectronic",
    "38EFE3": " INGENICO TERMINALS SAS",
    "FC8D13": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "9CB1DC": "Earda Technologies co Ltd",
    "240462": "Siemens Energy Global GmbH & Co.KG - GT PRM",
    "388871": "ASKEY COMPUTER CORP",
    "BCF87E": "Arcadyan Corporation",
    "284EE9": "mercury corperation",
    "58E359": "Interroll Software & Electronics GmbH",
    "8C3223": "JWIPC Technology Co.,Ltd.",
    "00C896": "CIG SHANGHAI CO LTD",
    "001B09": "MATRIX COMSEC PRIVATE LIMITED",
    "2800AF": "Dell Inc.",
    "000413": "snom technology GmbH",
    "74B059": "Motorola Mobility LLC, a Lenovo Company",
    "E48429": "New H3C Technologies Co., Ltd",
    "F41A79": "IEEE Registration Authority",
    "8C0734": "Private",
    "987A9B": "TCL MOKA International Limited",
    "781699": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E40A16": "HUAWEI TECHNOLOGIES CO.,LTD",
    "409CA7": "CHINA DRAGON TECHNOLOGY LIMITED",
    "D0AD08": "HP Inc.",
    "60045C": "NXP Semiconductor (Tianjin) LTD.",
    "849C02": "Druid Software",
    "CC40B2": "ECI Telecom Ltd.",
    "A4E287": "Xiaomi Communications Co Ltd",
    "80953A": "Apple, Inc.",
    "68AE04": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "CC6023": "Apple, Inc.",
    "0CDBEA": "Apple, Inc.",
    "6845CC": "Apple, Inc.",
    "AC9738": "Apple, Inc.",
    "08C224": "Amazon Technologies Inc.",
    "68932E": "Habana Labs LTD.",
    "2CC44F": "IEEE Registration Authority",
    "981223": "Tarmoc Network LTD",
    "1C4C27": "World WLAN Application Alliance",
    "D0C901": "GLA ELECTRONICS PVT LTD",
    "4099E3": "Guangzhou Mudi Information Technology Co., Ltd",
    "7C8899": "FN-LINK TECHNOLOGY Ltd.",
    "FC702E": "Sichuan AI-Link Technology Co., Ltd.",
    "B4F95D": "Juniper Networks",
    "046874": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "7490BC": "Arcadyan Corporation",
    "B047E9": "Intel Corporate",
    "6C2F80": "Intel Corporate",
    "D06578": "Intel Corporate",
    "98BD80": "Intel Corporate",
    "D476A0": "Fortinet, Inc.",
    "94FF3C": "Fortinet, Inc.",
    "A0F509": "IEI Integration Corp.",
    "6C45C4": "Cloudflare, Inc.",
    "000D97": "Hitachi Energy USA Inc.",
    "187F88": "Ring LLC",
    "B87E40": "Huawei Device Co., Ltd.",
    "1C2FA2": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "A43F68": "Arista Network, Inc.",
    "BC9D4E": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "1C3B01": "Shanghai Xiaodu Technology Limited",
    "94706C": "Quectel Wireless Solutions Co.,Ltd.",
    "64D81B": "Vestel Elektronik San ve Tic. A.S.",
    "30DCE7": "zte corporation",
    "D01255": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "3C2D9E": "Vantiva - Connected Home",
    "08C7F5": "Vantiva Connected Home - Technologies Telco",
    "C01693": "Xiaomi Communications Co Ltd",
    "FC5B8C": "Xiaomi Communications Co Ltd",
    "284E44": "HUAWEI TECHNOLOGIES CO.,LTD",
    "802EC3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3CB8D6": "Bluebank Communication Technology Co.,Ltd.",
    "9C9E03": "awayfrom",
    "D4D659": "Meta Platforms Technologies, LLC",
    "80053A": "CHeKT Inc.",
    "D8B32F": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "54104F": "Samsung Electronics Co.,Ltd",
    "B05476": "Samsung Electronics Co.,Ltd",
    "EC90C1": "Samsung Electronics Co.,Ltd",
    "74CA60": "Sonos, Inc.",
    "30D959": "Shanghai Longcheer Technology Co., Ltd.",
    "007839": "Nokia",
    "108A7B": "Nokia",
    "A052AB": "AVM ELECTRONICS PTE LTD",
    "BC744B": "Nintendo Co.,Ltd",
    "6C6F18": "Stereotaxis, Inc.",
    "F0866F": "EM Microelectronic",
    "5CB260": "EM Microelectronic",
    "A8BB56": "Apple, Inc.",
    "282D7F": "Apple, Inc.",
    "8C26AA": "Apple, Inc.",
    "90623F": "Apple, Inc.",
    "5C07A4": "Ciena Corporation",
    "004BF3": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "BC54FC": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "90769F": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "4C7766": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "005CC2": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "FCE8C0": "Espressif Inc.",
    "88548E": "vivo Mobile Communication Co., Ltd.",
    "581DC9": "MSE CO.,LTD.",
    "CC7B5C": "Espressif Inc.",
    "505B1D": "Shenzhen C-Data Technology Co., Ltd.",
    "88010C": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "A49E69": "Silicon Laboratories",
    "F082C0": "Silicon Laboratories",
    "60EFAB": "Silicon Laboratories",
    "D87A3B": "Silicon Laboratories",
    "3C2EF5": "Silicon Laboratories",
    "3410F4": "Silicon Laboratories",
    "0CEFF6": "Silicon Laboratories",
    "ECF64C": "Silicon Laboratories",
    "70C912": "Sichuan AI-Link Technology Co., Ltd.",
    "488EB7": "Zebra Technologies Inc.",
    "90AB96": "Silicon Laboratories",
    "0C82D5": "Maxio Technology Hangzhou Co., Ltd.",
    "0401BB": "TECNO MOBILE LIMITED",
    "AC72DD": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "64BB1E": "Earda Technologies co Ltd",
    "648214": "FN-LINK TECHNOLOGY Ltd.",
    "E09CE5": "Shanghai Tricheer Technology Co.,Ltd.",
    "FCD290": "SKY UK LIMITED",
    "78305D": "zte corporation",
    "940B83": "zte corporation",
    "E47450": "Shenzhen Grandsun Electronic Co.,Ltd.",
    "B4A3BD": "Extreme Networks Headquarters",
    "7CD3E5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8876E": "Intertech Services AG",
    "3C0B4F": "Intertech Services AG",
    "6C62FE": "Juniper Networks",
    "C8A23B": "Shenzhen Sundray Technologies Company Limited",
    "9C9ED5": "Xiaomi Communications Co Ltd",
    "743822": "Xiaomi Communications Co Ltd",
    "B83BAB": "Arcadyan Corporation",
    "40A63D": "SignalFire Telemetry",
    "605464": "Eyedro Green Solutions Inc.",
    "CCE536": "ittim",
    "BC0004": "Fiberhome Telecommunication Technologies Co.,LTD",
    "98A829": "AltoBeam Inc.",
    "84E9C1": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "B4622E": "Zhong Ge Smart Technology Co., Ltd.",
    "C46E33": "Zhong Ge Smart Technology Co., Ltd.",
    "2C67BE": "DWnet Technologies(Suzhou) Corporation",
    "EC192E": "Cisco Systems, Inc",
    "F896FE": "LG Innotek",
    "A4C40D": "WAC Lighting",
    "58306E": "Nokia",
    "A8CA77": "Amazon Technologies Inc.",
    "D0C907": "Private",
    "142B2F": "Espressif Inc.",
    "C4A451": "TECNO MOBILE LIMITED",
    "F89D9D": "Shenzhen MinewSemi Co.,LTD.",
    "F061F3": "Comcast Cable Corporation",
    "D48D26": "LG Innotek",
    "F4F28A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94E300": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0CE5B5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCF738": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C862A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C03379": "HUAWEI TECHNOLOGIES CO.,LTD",
    "241E2B": "Zhejiang Cainiao Supply Chain Management Co., Ltd",
    "AC7F8D": "Extreme Networks Headquarters",
    "24ACAC": "Polar Electro Oy",
    "50206B": "Copeland - Transportation Solutions ApS",
    "80D10A": "Sichuan AI-Link Technology Co., Ltd.",
    "1047E7": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "94F6F2": "Honor Device Co., Ltd.",
    "449046": "Honor Device Co., Ltd.",
    "B0C38E": "Huawei Device Co., Ltd.",
    "143B51": "Huawei Device Co., Ltd.",
    "D07380": "Huawei Device Co., Ltd.",
    "ACFCE3": "EM Microelectronic",
    "042DAD": "Areus GmbH",
    "781C1E": "Chongqing Yipingfang Technology Co., Ltd.",
    "78A13E": "New H3C Technologies Co., Ltd",
    "48CA43": "Espressif Inc.",
    "7CBAC0": "EVBox BV",
    "98256E": "Private",
    "B0A7D2": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8C87D0": "Shenzhen Uascent Technology Co.,Ltd",
    "5C1783": "Edgecore Americas Networking Corporation",
    "647060": "Texas Instruments",
    "00AAFD": "Texas Instruments",
    "0C4BEE": "Texas Instruments",
    "D413B3": "Wu Qi Technologies,Inc.",
    "60E5D8": "zte corporation",
    "5435E9": "Feitian Technologies Co., Ltd",
    "7CB59F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "505C88": "Cisco Systems, Inc",
    "30138B": "HP Inc.",
    "606832": "Guangdong Seneasy Intelligent Technology Co., Ltd.",
    "8433F2": "Shenzhen Stellamore Technology Co.,Ltd",
    "28D043": "AzureWave Technology Inc.",
    "E4D3AA": "FCNT LLC",
    "B0D576": "Apple, Inc.",
    "142876": "Apple, Inc.",
    "147FCE": "Apple, Inc.",
    "6CC3B2": "Cisco Meraki",
    "F8CE07": "ZHEJIANG DAHUA TECHNOLOGYCO.,LTD",
    "2098ED": "AltoBeam Inc.",
    "983910": "Kaon Group Co., Ltd.",
    "804C5D": "NXP Semiconductor (Tianjin) LTD.",
    "F88FC8": "Chipsea Technologies (Shenzhen) Corp.",
    "4C10D5": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "289176": "Indyme Solutions, LLC",
    "28A06B": "Intel Corporate",
    "7C8BC1": "Infinix mobility limited",
    "3CCA61": "TECNO MOBILE LIMITED",
    "ECA971": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "68A2AA": "Acres Manufacturing",
    "88AE35": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "7049A2": "Zyxel Communications Corporation",
    "A07D9C": "Samsung Electronics Co.,Ltd",
    "F8C3F1": "Raytron Photonics Co.,Ltd.",
    "E00855": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "D89563": "Taiwan Digital Streaming Co.",
    "E8A848": "Wacom Co.,Ltd.",
    "5CA3EB": "SKODA DIGITAL s.r.o.",
    "5CBE05": "ISPEC",
    "08524E": "Shenzhen Fangcheng Baiyi Technology Co., Ltd.",
    "ACF42C": "Earda Technologies co Ltd",
    "C0AB2B": "Huawei Device Co., Ltd.",
    "3877CD": "KOKUSAI ELECTRIC CORPORATION",
    "508A7F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4CBBE": "Great Talent Technology Limited",
    "E8B527": "Phyplus Technology (Shanghai) Co., Ltd",
    "98A878": "Agnigate Technologies Private Limited",
    "305223": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "DC4965": "DASAN Newtork Solutions",
    "189E2D": "Allwinner Technology Co., Ltd",
    "9C6076": "Apple, Inc.",
    "38E13D": "Apple, Inc.",
    "D0D49F": "Apple, Inc.",
    "9C2D49": "Nanowell Info Tech Co., Limited",
    "887FD5": "zte corporation",
    "F47470": "Cisco Systems, Inc",
    "60A434": "IEEE Registration Authority",
    "387B01": "Shenzhen MiaoMing Intelligent Technology Co.,Ltd",
    "0CEDC8": "Xiaomi Communications Co Ltd",
    "88BCAC": "Zebra Technologies Inc.",
    "C88DD4": "Markone technology Co., Ltd.",
    "2C36F2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4995F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8F059": "HUAWEI TECHNOLOGIES CO.,LTD",
    "283DE8": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "180403": "vivo Mobile Communication Co., Ltd.",
    "206BD5": "vivo Mobile Communication Co., Ltd.",
    "B47AF1": "Hewlett Packard Enterprise ",
    "489ECB": "Hewlett Packard Enterprise ",
    "9C1C12": "Hewlett Packard Enterprise",
    "04BD88": "Hewlett Packard Enterprise",
    "E8F724": "Hewlett Packard Enterprise ",
    "DC680C": "Hewlett Packard Enterprise ",
    "48DF37": "Hewlett Packard Enterprise ",
    "20677C": "Hewlett Packard Enterprise ",
    "4448C1": "Hewlett Packard Enterprise ",
    "6828CF": "Hewlett Packard Enterprise",
    "E4DE40": "Hewlett Packard Enterprise",
    "480020": "Hewlett Packard Enterprise",
    "5CA47D": "Hewlett Packard Enterprise",
    "E89505": "Shenzhen MiaoMing Intelligent Technology Co.,Ltd",
    "DCB7AC": "Hewlett Packard Enterprise",
    "54D7E3": "Hewlett Packard Enterprise",
    "14ABEC": "Hewlett Packard Enterprise",
    "104F58": "Hewlett Packard Enterprise",
    "CCD083": "Hewlett Packard Enterprise",
    "BC4632": "Fiberhome Telecommunication Technologies Co.,LTD",
    "685377": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "F05C19": "Hewlett Packard Enterprise",
    "7C573C": "Hewlett Packard Enterprise",
    "348A12": "Hewlett Packard Enterprise",
    "1063A3": "IEEE Registration Authority",
    "E80690": "Espressif Inc.",
    "DCA706": "CHENGDU KT ELECTRONIC HI-TECH CO.,LTD",
    "E0604A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "5006F5": "Roku, Inc",
    "9816CD": "leapio",
    "34516F": "Skychers Creations ShenZhen Limited",
    "C86E08": "Intel Corporate",
    "B8374B": "Hewlett Packard Enterprise ",
    "188637": "INGRAM MICRO SERVICES",
    "78078F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "309E62": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10C0D5": "HOLOEYE Photonics AG",
    "54FA89": "Medtronic CRM",
    "241651": "Chipsea Technologies (Shenzhen) Corp.",
    "58769C": "Palo Alto Networks",
    "B4C556": "Shanghai Kenmyond Industrial Network Equipment Co., Ltd",
    "EC725B": "zte corporation",
    "E49282": "Samsung Electronics Co.,Ltd",
    "F0090D": "TP-LINK CORPORATION PTE. LTD.",
    "0009F7": "Calian Advanced Technologies",
    "0494E9": "FAXedge Technologies, LLC",
    "387ACC": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "7C72E7": "Texas Instruments",
    "3030D0": "Texas Instruments",
    "54FEEB": "Texas Instruments",
    "0884FB": "Honor Device Co., Ltd.",
    "14BEFC": "Nanjing Jiahao Technology Co., Ltd.",
    "905F7A": "Apple, Inc.",
    "F8F58C": "Apple, Inc.",
    "0C85E1": "Apple, Inc.",
    "282E89": "Wistron Neweb Corporation",
    "90FB5D": "Beijing Xiaomi Mobile Software Co., Ltd",
    "441690": "Wuxi Ranke Technology Co., Ltd.",
    "9CA6D8": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0C2A6F": "Silicon Laboratories",
    "20DBEA": "Cisco Systems, Inc",
    "E01CA7": "Arista Networks, Inc.",
    "A0479B": "PROCITEC GmbH",
    "74C90F": "Microchip Technologies Inc",
    "34CDB0": "Espressif Inc.",
    "18E204": "BEIJING COOLSHARK TECHNOLOGY CO.,LTD.",
    "2CB7A1": "Huawei Device Co., Ltd.",
    "6C8243": "Huawei Device Co., Ltd.",
    "B476A4": "Huawei Device Co., Ltd.",
    "5026D2": "AVIRE Trading Limited",
    "942453": "HUAWEI TECHNOLOGIES CO.,LTD",
    "184F43": "UNIONMAN TECHNOLOGY CO.,LTD",
    "F86BFA": "Infinix mobility limited",
    "B03226": "Keheng Information Industry Co., Ltd.",
    "3C5836": "Sagemcom Broadband SAS",
    "84CB85": "EM Microelectronic",
    "ACF23C": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "E45768": "vivo Mobile Communication Co., Ltd.",
    "48E6C6": "IEEE Registration Authority",
    "000417": "Schneider Electric",
    "D01BBE": "Onward Brands",
    "246A0E": "HP Inc.",
    "B07C51": "Ruckus Wireless",
    "884F59": "Cisco Systems, Inc",
    "7CFA80": "JiangSu Fulian Communication Technology Co., Ltd",
    "C08706": "Shenzhen Qianfenyi Intelligent Technology Co.,LTD",
    "C4B349": "Apple, Inc.",
    "34F68D": "Apple, Inc.",
    "9454C5": "Espressif Inc.",
    "ACB722": "Qingdao Haier Technology Co.,Ltd",
    "CC2746": "Apple, Inc.",
    "285923": "Xiaomi Communications Co Ltd",
    "D0CEC0": "Xiaomi Communications Co Ltd",
    "0C01A5": "zte corporation",
    "98FA9B": "LCFC(Hefei) Electronics Technology co., ltd",
    "F875A4": "LCFC(Hefei) Electronics Technology co., ltd",
    "8C8CAA": "LCFC(Hefei) Electronics Technology co., ltd",
    "701988": "Nanjing Qinheng Microelectronics Co., Ltd.",
    "FC1A46": "Samsung Electronics Co.,Ltd",
    "983DAE": "Espressif Inc.",
    "5C5E0A": "Samsung Electronics Co.,Ltd",
    "CCB0B3": "Microsoft Corporation",
    "80D52C": "Beijing Cheering Networks Technology Co.,Ltd.",
    "88B2AB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24E3A4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "3000FC": "Nokia",
    "64DE6D": "Intel Corporate",
    "EC4C8C": "Intel Corporate",
    "EC6652": "Info Fiber Solutions Pvt Ltd",
    "CC763A": "zte corporation",
    "300475": "QBIC COMMUNICATIONS DMCC",
    "000AF6": "Copeland LP",
    "5C18DD": "CIG SHANGHAI CO LTD",
    "14360E": "Zyxel Communications Corporation",
    "5C4EEE": "AltoBeam Inc.",
    "5CC1F2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C805A4": "Motorola(Wuhan) Mobility Technologies Communication Co.,Ltd",
    "0072EE": "Intel Corporate",
    "C858B3": "Intel Corporate",
    "E41FD5": "Intel Corporate",
    "EC1B5F": "Hewlett Packard Enterprise",
    "1CB46C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8F673": "Microsoft Corporation",
    "7864A0": "Cisco Systems, Inc",
    "5CDB36": "Calix Inc.",
    "4C7B35": "UNIONMAN TECHNOLOGY CO.,LTD",
    "9C45F0": "SKYLARK ELECTRONICS PVT LTD",
    "409249": "Shanghai Baud Data Communication Co.,Ltd.",
    "0C7E24": "Garmin International",
    "7475DF": "TECLINK",
    "686B6A": "Phytium Technology Co.,Ltd.",
    "3407AC": "PRONYX TRADING LLC",
    "58B858": "SZ DJI TECHNOLOGY CO.,LTD",
    "D0CBDD": "eero inc.",
    "44B423": "HANWHA VISION VIETNAM COMPANY LIMITED",
    "6887BD": "zte corporation",
    "3058EB": "zte corporation",
    "4405B8": "Huawei Device Co., Ltd.",
    "841A24": "UNIONMAN TECHNOLOGY CO.,LTD",
    "786CAB": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "007D3B": "Samsung Electronics Co.,Ltd",
    "5C40E3": "NOVAON",
    "986297": "Shenzhen Techwinsemi Technology Co., Ltd.",
    "E8BCE4": "Cisco Systems, Inc",
    "8C8726": "VAST Data Inc",
    "F8EF5D": "Motorola Mobility LLC, a Lenovo Company",
    "18E8EC": "STMicrolectronics International NV",
    "A409B3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28F52B": "FN-LINK TECHNOLOGY Ltd.",
    "68628A": "vivo Mobile Communication Co., Ltd.",
    "2844F4": "Honor Device Co., Ltd.",
    "BC31E2": "New H3C Technologies Co., Ltd",
    "B4C799": "Extreme Networks Headquarters",
    "58696C": "Ruijie Networks Co.,LTD",
    "8C8394": "Arcadyan Corporation",
    "F0AB1F": "zte corporation",
    "7CB30A": "zte corporation",
    "4C62DF": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "B00C9D": "Quectel Wireless Solutions Co.,Ltd.",
    "C8675E": "Extreme Networks Headquarters",
    "C413E2": "Extreme Networks Headquarters",
    "90B832": "Extreme Networks Headquarters",
    "40E317": "Extreme Networks Headquarters",
    "AC4DD9": "Extreme Networks Headquarters",
    "F4CE48": "Extreme Networks Headquarters",
    "00DCB2": "Extreme Networks Headquarters",
    "9C5D12": "Extreme Networks Headquarters",
    "F09CE9": "Extreme Networks Headquarters",
    "CC14BC": "Edifier International",
    "E8D3EB": "eero inc.",
    "200B74": "AzureWave Technology Inc.",
    "0025DF": "Taser International Inc.",
    "B0104B": "Fiberhome Telecommunication Technologies Co.,LTD",
    "44365D": "Shenzhen HippStor Technology Co., Ltd",
    "F8FE5E": "Intel Corporate",
    "54C078": "Infinix mobility limited",
    "88F916": "Qingdao Dayu Dance Digital Technology Co.,Ltd",
    "E81711": "Shenzhen Vipstech Co., Ltd",
    "7CBF77": "SPEEDTECH CORP.",
    "18C009": "New H3C Technologies Co., Ltd",
    "7C7398": "Espressif Inc.",
    "6CB456": "Espressif Inc.",
    "E4E26C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "3030F9": "Espressif Inc.",
    "BC2411": "Proxmox Server Solutions GmbH",
    "706173": "Calantec GmbH",
    "9C4952": "Dongguan Liesheng Electronic Co., Ltd.",
    "C022F1": "IEEE Registration Authority",
    "041471": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C816A5": "Masimo Corporation",
    "446D05": "NoTraffic",
    "38C0EA": "Fortinet, Inc.",
    "D094CF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "542259": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC6880": "zte corporation",
    "102874": "Shenzhen Jingxun Technology Co., Ltd.",
    "043855": "Scopus International Pvt. Ltd.",
    "A8B483": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "900E9E": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "54F29F": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "C82E18": "Espressif Inc.",
    "E86BEA": "Espressif Inc.",
    "288EB9": "Wacom Co.,Ltd.",
    "C89D6D": "ITEL MOBILE LIMITED",
    "ECB0D2": "EM Microelectronic",
    "4CEB76": "Murrelektronik GmbH",
    "6855D4": "Seiko Epson Corporation",
    "D422CD": "Movella Technologies B.V.",
    "28BBED": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "344E2F": "LEAR",
    "4CA0D4": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "98EDCA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0C659A": "Panasonic Automotive Systems Company of America",
    "00409E": "Concurrent Technologies Ltd.",
    "ECAB3E": "ESSYS",
    "384A80": "Samsung Electronics Co.,Ltd",
    "E41088": "Samsung Electronics Co.,Ltd",
    "BC8D1F": "Cisco Systems, Inc",
    "045791": "Shenzhenshi Xinzhongxin Technology Co.Ltd",
    "88A6EF": "IEEE Registration Authority",
    "54DF1B": "Vestel Elektronik San ve Tic. A.S.",
    "38F9F5": "Garmin International",
    "60BD2C": "Taicang T&W Electronics",
    "C493BB": "Beijing Xiaomi Mobile Software Co., Ltd",
    "E89C25": "ASUSTek COMPUTER INC.",
    "ECC38A": "Accuenergy (CANADA) Inc",
    "14993E": "Xiaomi Communications Co Ltd",
    "2C4C15": "Juniper Networks",
    "78467D": "SKAIChips",
    "EC748C": "Sony Interactive Entertainment Inc.",
    "B04FA6": "DongGuan Ramaxel Memory Technology",
    "4C9992": "vivo Mobile Communication Co., Ltd.",
    "000CC8": "Xytronix Research & Design, Inc.",
    "84F5EB": "zte corporation",
    "D8E72F": "Chipsea Technologies (Shenzhen) Corp.",
    "74C76E": "RTK-TECHNOLOGIES, LLC",
    "14C050": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "441A4C": "xFusion Digital Technologies Co.,Ltd.",
    "4C62CD": "Nokia",
    "3C52A1": "TP-Link Corporation Limited",
    "98DA92": "Vuzix Corporation",
    "78465C": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "D0DD7C": "zte corporation",
    "843C99": "zte corporation",
    "9CF86B": "AgiTech Distribution Limited - Linki",
    "F8E5CE": "Apple, Inc.",
    "68E59E": "Cisco Systems, Inc",
    "1CC992": "Honor Device Co., Ltd.",
    "400FC1": "Vantiva USA LLC",
    "74978E": "Nova Labs",
    "0823C6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "107B93": "Zhen Shi Information Technology (Shanghai) Co., Ltd.",
    "705A9E": "Vantiva USA LLC",
    "88F7C7": "Vantiva USA LLC",
    "1C9D72": "Vantiva USA LLC",
    "3C82C0": "Vantiva USA LLC",
    "80DAC2": "Vantiva USA LLC",
    "481B40": "Vantiva USA LLC",
    "B85E71": "Vantiva USA LLC",
    "0C0227": "Vantiva USA LLC",
    "14B7F8": "Vantiva USA LLC",
    "1062D0": "Vantiva USA LLC",
    "10C25A": "Vantiva USA LLC",
    "F4C114": "Vantiva USA LLC",
    "F83B1D": "Vantiva USA LLC",
    "E4BFFA": "Vantiva USA LLC",
    "08D1F9": "Espressif Inc.",
    "34B7DA": "Espressif Inc.",
    "1071FA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "D08A91": "Vantiva USA LLC",
    "C87F54": "ASUSTek COMPUTER INC.",
    "7036B2": "Focusai Corp",
    "4CD717": "Dell Inc.",
    "00C711": "ITEL MOBILE LIMITED",
    "C84C78": "zte corporation",
    "802D1A": "zte corporation",
    "6823F4": "Shenzhen Jinlangxin Technology Co., Ltd",
    "90837E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "CCB071": "Fiberhome Telecommunication Technologies Co.,LTD",
    "485541": "Iskratel d.o.o.",
    "F09008": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "348D52": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "1C860B": "Guangdong Taiying Technology Co.,Ltd",
    "74D713": "Huaqin Technology Co. LTD",
    "2CC3E6": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "F020FF": "Intel Corporate",
    "C88A9A": "Intel Corporate",
    "A0B339": "Intel Corporate",
    "6CF6DA": "Intel Corporate",
    "C86BBC": "IEEE Registration Authority",
    "30600A": "CIG SHANGHAI CO LTD",
    "6813F3": "Amazon Technologies Inc.",
    "806D97": "Private",
    "4C70CC": "Blyott NV",
    "302F1E": "SIEMENS AG",
    "28EB0A": "Rolling Wireless S.a.r.l. Luxembourg",
    "848A59": "Hisilicon Technologies Co., Ltd",
    "586861": "VIASAT, INCORPORATED",
    "14C9CF": "Sigmastar Technology Ltd.",
    "847ADF": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "DCCD66": "NXP Semiconductor (Tianjin) LTD.",
    "6074F4": "Private",
    "A03768": "Shenzhen E-Life Intelligence Technology Co.,Ltd.",
    "44FA66": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "FCB2D6": "CIG SHANGHAI CO LTD",
    "60FCF1": "Private",
    "5476B2": "Raisecom Technology CO., LTD",
    "143375": "Zyxel Communications Corporation",
    "2CFEE2": "Qingdao Hisense Communications Co.,Ltd.",
    "B0FF72": "IEEE Registration Authority",
    "649A63": "Ring LLC",
    "102834": "SALZ Automation GmbH",
    "60C01E": "V&G Information System Co.,Ltd",
    "2C3C05": "Marinesync Corp",
    "148554": "Earda Technologies co Ltd",
    "0008B9": "Kaon Group Co., Ltd.",
    "C4D666": "Cisco Meraki",
    "18DE50": "Tuya Smart Inc.",
    "98CCF3": "Amazon Technologies Inc.",
    "5091E3": "TP-Link Corporation Limited",
    "F02178": "UNIONMAN TECHNOLOGY CO.,LTD",
    "947918": "ITEL MOBILE LIMITED",
    "68F0B5": "Honor Device Co., Ltd.",
    "001457": "Nevion",
    "005001": "YAMASHITA SYSTEMS CORP.",
    "6CEEF7": "shenzhen scodeno technology co., Ltd.",
    "2C3EBF": "HOSIN Global Electronics Co., Ltd.",
    "40D563": "HANA Electronics",
    "3C3332": "D-Link Corporation",
    "B440DC": "Samsung Electronics Co.,Ltd",
    "6417CD": "Samsung Electronics Co.,Ltd",
    "DC69E2": "Samsung Electronics Co.,Ltd",
    "C05064": "SHENNAN CIRCUITS CO.,LTD",
    "9CFB77": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "643E0A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "348818": "Cisco Systems, Inc",
    "402A8F": "Shanghai High-Flying Electronics  Technology Co., Ltd",
    "DC2DDE": "Ledworks SRL",
    "101849": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "08F9E0": "Espressif Inc.",
    "0036D7": "Keltron IOT Corp.",
    "34DC99": "New H3C Technologies Co., Ltd",
    "5CE8D3": "Signalinks Communication Technology Co., Ltd",
    "44053F": "Sagemcom Broadband SAS",
    "80802C": "Fortinet, Inc.",
    "4831B7": "Espressif Inc.",
    "543204": "Espressif Inc.",
    "CCB54C": "Texas Instruments",
    "74A58C": "Texas Instruments",
    "3CE002": "Texas Instruments",
    "98038A": "Texas Instruments",
    "B86061": "China Mobile Group Device Co.,Ltd.",
    "B03893": "Onda TLC Italia S.r.l.",
    "8CDF2C": "vivo Mobile Communication Co., Ltd.",
    "F87928": "zte corporation",
    "D88ED4": "eero inc.",
    "943FD6": "Apple, Inc.",
    "002067": "Private",
    "2CD7FF": "LANCOM Systems GmbH",
    "38AB16": "NPO RTT LLC",
    "348511": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "F8B8B4": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "681AA4": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "48555E": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "28C1A0": "Apple, Inc.",
    "EC2C73": "Apple, Inc.",
    "7CC06F": "Apple, Inc.",
    "EC8150": "Apple, Inc.",
    "D42FCA": "Apple, Inc.",
    "D058A5": "Apple, Inc.",
    "2C67AB": "EZELINK TELECOM",
    "B09738": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "1C880C": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "700692": "IEEE Registration Authority",
    "38B5C9": "INGRAM MICRO SERVICES",
    "984B06": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ACFF6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "844DBE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "80B946": "Nokia",
    "F8CAB8": "Dell Inc.",
    "44A842": "Dell Inc.",
    "4C7625": "Dell Inc.",
    "001143": "Dell Inc.",
    "001372": "Dell Inc.",
    "F8BC12": "Dell Inc.",
    "3417EB": "Dell Inc.",
    "B083FE": "Dell Inc.",
    "000874": "Dell Inc.",
    "0024E8": "Dell Inc.",
    "002219": "Dell Inc.",
    "00188B": "Dell Inc.",
    "F01FAF": "Dell Inc.",
    "18A99B": "Dell Inc.",
    "ACDE48": "Private",
    "0050C7": "Private",
    "5488FE": "Xiaoniu network technology (Shanghai) Co., Ltd.",
    "F4D580": "YAMAHA CORPORATION",
    "5030F4": "Exascend, Inc.",
    "601895": "Dell Inc.",
    "B04F13": "Dell Inc.",
    "381428": "Dell Inc.",
    "F4EE08": "Dell Inc.",
    "908D6E": "Dell Inc.",
    "EC2A72": "Dell Inc.",
    "7486E2": "Dell Inc.",
    "00BE43": "Dell Inc.",
    "605B30": "Dell Inc.",
    "C84BD6": "Dell Inc.",
    "E8655F": "Dell Inc.",
    "E8B265": "Dell Inc.",
    "AC91A1": "Dell Inc.",
    "C4CBE1": "Dell Inc.",
    "107D1A": "Dell Inc.",
    "509A4C": "Dell Inc.",
    "405CFD": "Dell Inc.",
    "D09466": "Dell Inc.",
    "D89EF3": "Dell Inc.",
    "B4E10F": "Dell Inc.",
    "1866DA": "Dell Inc.",
    "18DBF2": "Dell Inc.",
    "14B31F": "Dell Inc.",
    "886FD4": "Dell Inc.",
    "5CF9DD": "Dell Inc.",
    "004E01": "Dell Inc.",
    "106530": "Dell Inc.",
    "3C2C30": "Dell Inc.",
    "247152": "Dell Inc.",
    "8C47BE": "Dell Inc.",
    "94CBCD": "zte corporation",
    "805F8E": "Huizhou BYD Electronic Co., Ltd.",
    "A83A79": "Mist Systems, Inc.",
    "E49069": "Rockwell Automation",
    "184C08": "Rockwell Automation",
    "FC35E6": "Visteon Corporation",
    "8CBA25": "UNION MAN TECHNOLOGY CO.,LTD",
    "54725E": "UNION MAN TECHNOLOGY CO.,LTD",
    "F814FE": "UNION MAN TECHNOLOGY CO.,LTD",
    "CC62FE": "UNION MAN TECHNOLOGY CO.,LTD",
    "C09573": "AIxLink",
    "404101": "Rockwell Automation",
    "68C8EB": "Rockwell Automation",
    "40F21C": "DZS Inc.",
    "00180C": "DZS Inc.",
    "00E0DF": "DZS Inc.",
    "00A01B": "DZS Inc.",
    "D09395": "IEEE Registration Authority",
    "7CBFAE": "Renesas Electronics (Penang) Sdn. Bhd.",
    "80AFCA": "Shenzhen Cudy Technology Co., Ltd.",
    "1C6E74": "EnOcean Edge Inc.",
    "04472A": "Palo Alto Networks",
    "C02C17": "Cisco Systems, Inc",
    "984925": "Juniper Networks",
    "4419B6": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "B417A8": "Meta Platforms Technologies, LLC",
    "C04884": "Sigma Bilisim Sist. Tekn. Elk. Enj. ve San. D??. Tic. Ltd. ?ti.",
    "7C8931": "Huawei Device Co., Ltd.",
    "BC5E33": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "FC9FFD": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "E8A0ED": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "5C345B": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "7CEDC6": "Amazon Technologies Inc.",
    "4825F3": "Huawei Device Co., Ltd.",
    "4CBD8F": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "F412DA": "zte corporation",
    "540295": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C8787D": "D-Link Corporation",
    "D8B370": "Ubiquiti Inc",
    "B0A732": "Espressif Inc.",
    "B0B21C": "Espressif Inc.",
    "E09B27": "Ciena Corporation",
    "8CE042": "vivo Mobile Communication Co., Ltd.",
    "FCECDA": "Ubiquiti Inc",
    "942A6F": "Ubiquiti Inc",
    "F4E2C6": "Ubiquiti Inc",
    "78FE3D": "Juniper Networks",
    "648788": "Juniper Networks",
    "AC4BC8": "Juniper Networks",
    "3C94D5": "Juniper Networks",
    "B4FBE4": "Ubiquiti Inc",
    "687251": "Ubiquiti Inc",
    "DC38E1": "Juniper Networks",
    "CCE17F": "Juniper Networks",
    "9CC893": "Juniper Networks",
    "58E434": "Juniper Networks",
    "B0EB7F": "Juniper Networks",
    "28B829": "Juniper Networks",
    "E897B8": "Chiun Mai Communication System, Inc",
    "94F524": "Chengdu BeiZhongWangXin Technology Co.Ltd",
    "60FAB1": "Kempower Oyj",
    "407183": "Juniper Networks",
    "40B4F0": "Juniper Networks",
    "384F49": "Juniper Networks",
    "5800BB": "Juniper Networks",
    "D007CA": "Juniper Networks",
    "D8B122": "Juniper Networks",
    "FC5703": "Hisense broadband multimedia technology Co.,Ltd",
    "64C3D6": "Juniper Networks",
    "C0BFA7": "Juniper Networks",
    "B033A6": "Juniper Networks",
    "201BC9": "Juniper Networks",
    "541E56": "Juniper Networks",
    "88A25E": "Juniper Networks",
    "003146": "Juniper Networks",
    "50C709": "Juniper Networks",
    "00C52C": "Juniper Networks",
    "B48A5F": "Juniper Networks",
    "E0F62D": "Juniper Networks",
    "EC7C5C": "Juniper Networks",
    "807FF8": "Juniper Networks",
    "C0EAC3": "IEEE Registration Authority",
    "FCA0F3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04A81C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "304074": "zte corporation",
    "A01077": "zte corporation",
    "54B7BD": "Arcadyan Corporation",
    "BC73A4": "ANDA TELECOM PVT LTD",
    "14F5F9": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "140FA6": "Renesas Electronics (Penang) Sdn. Bhd.",
    "148473": "Cisco Systems, Inc",
    "AC1754": "tiko Energy Solutions AG",
    "ACE0D6": "koreabts",
    "A4C23E": "Huizhou Speed Wireless Technology Co.,Ltd",
    "0050C4": "IMD",
    "001AA6": "Elbit Systems Deutschland GmbH & Co. KG",
    "7C45F9": "IEEE Registration Authority",
    "10CF0F": "Apple, Inc.",
    "90314B": "AltoBeam Inc.",
    "D8638C": "Shenzhen Dttek Technology Co., Ltd.",
    "1CCA41": "AO ",
    "347DE4": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "CCCF83": "CIG SHANGHAI CO LTD",
    "04F778": "Sony Interactive Entertainment Inc.",
    "B4636F": "Nokia Solutions and Networks GmbH & Co. KG",
    "24D337": "Xiaomi Communications Co Ltd",
    "7C7A3C": "New H3C Technologies Co., Ltd",
    "483177": "Nintendo Co.,Ltd",
    "1C76F2": "Samsung Electronics Co.,Ltd",
    "0092A5": "LG Innotek",
    "205F3D": "Adtran Inc",
    "3C3B4D": "Toyo Seisakusho Kaisha, Limited",
    "3CBCD0": "zte corporation",
    "D0A9D3": "EM Microelectronic",
    "14DD02": "Liangang Optoelectronic Technology CO., Ltd.",
    "60C727": "Digiboard Eletronica da Amazonia Ltda",
    "408EF6": "Infinix mobility limited",
    "E02DF0": "ALPSALPINE CO,.LTD",
    "646E60": "zte corporation",
    "4C421E": "Cisco Systems, Inc",
    "64B2B4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "4831DB": "Huawei Device Co., Ltd.",
    "58707F": "Ericsson AB",
    "887477": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC315D": "Apple, Inc.",
    "D039FA": "Samsung Electronics Co.,Ltd",
    "B40B1D": "Samsung Electronics Co.,Ltd",
    "AC80FB": "Samsung Electronics Co.,Ltd",
    "147F0F": "Texas Instruments",
    "444AD6": "Shenzhen Rinocloud Technology Co.,Ltd.",
    "189EAD": "Shenzhen Chengqian Information Technology Co., Ltd",
    "94DDF8": "Brother Industries, LTD.",
    "84B386": "IEEE Registration Authority",
    "A45D5E": "Wilk Elektronik S.A.",
    "98BFF4": "MARKIN co., Ltd.",
    "78605B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "04F9F8": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "74A6CD": "Apple, Inc.",
    "2C7CF2": "Apple, Inc.",
    "30D7A1": "Apple, Inc.",
    "E0382D": "IEEE Registration Authority",
    "68FCB6": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A8727E": "WISDRI (wuhan) Automation Company Limited",
    "DCD2FD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5873D1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F42D06": "zte corporation",
    "1C674A": "zte corporation",
    "7C5758": "HP Inc.",
    "C829C8": "Palo Alto Networks",
    "9C5322": "TP-Link Corporation Limited",
    "482254": "TP-Link Corporation Limited",
    "4CB087": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88B4BE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "947BAE": "Xiaomi Communications Co Ltd",
    "38F8F6": "Adtran Inc",
    "B4E265": "Shenzhen SDMC Technology CO.,Ltd.",
    "00C28F": "Allied Telesis K.K.",
    "3847F2": "Recogni Inc",
    "D8AD49": "Honor Device Co., Ltd.",
    "B484D5": "GooWi Wireless Technology Co., Limited",
    "54077D": "NETGEAR",
    "C45379": "Micronview Limited Liability Company",
    "804AF2": "Sonos, Inc.",
    "D8312C": "zte corporation",
    "049F15": "Humane",
    "3838A6": "Arista Networks",
    "50236D": "Nintendo Co.,Ltd",
    "A03975": "Leo Bodnar Electronics Ltd",
    "7493DA": "ASKEY COMPUTER CORP",
    "C84AA0": "Sony Interactive Entertainment Inc.",
    "4C968A": "Wacom Co.,Ltd.",
    "EC83B7": "PUWELL CLOUD TECH LIMITED",
    "0C6422": "Beijing Wiseasy Technology Co.,Ltd.",
    "C8F2B4": "Guizhou Huaxin Information Technology Co., Ltd.",
    "F42756": "DASAN Newtork Solutions",
    "E0A1CE": "zte corporation",
    "24A6FA": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "2C75CB": "Novitec Co., Ltd.",
    "9C558F": "Lockin Technology(Beijing) Co.,Ltd.",
    "E8268D": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "6C9106": "Katena Computing Technologies",
    "6C70CB": "Samsung Electronics Co.,Ltd",
    "907BC6": "Texas Instruments",
    "904846": "Texas Instruments",
    "C47905": "Zhejiang Uniview Technologies Co.,Ltd.",
    "5027A9": "eero inc.",
    "A411BB": "Cisco Systems, Inc",
    "504921": "Cisco Systems, Inc",
    "10BBF3": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "D04E99": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20A766": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C058E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0C725": "Apple, Inc.",
    "201A94": "Apple, Inc.",
    "B0E5EF": "Apple, Inc.",
    "288FF6": "Apple, Inc.",
    "44B7D0": "Microchip Technology Inc.",
    "D4DA21": "Beijing Xiaomi Mobile Software Co., Ltd",
    "6444D5": "TD Tech",
    "E88F6F": "TCT mobile ltd",
    "84FCE6": "Espressif Inc.",
    "58B965": "Apple, Inc.",
    "743174": "Apple, Inc.",
    "CCCC77": "Zaram Technology. Inc.",
    "BC458C": "Shenzhen Topwise Communication Co.,Ltd",
    "947806": "NINGBO SUNVOT TECHNOLOGY CO.,LTD",
    "784F24": "Taicang T&W Electronics",
    "F8AAB3": "DESSMANN (China) Machinery & Electronic  Co., Ltd",
    "ECDA3B": "Espressif Inc.",
    "6813E2": "Eltex Enterprise LTD",
    "7C669A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC1E97": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B4E46B": "China Mobile IOT Company Limited",
    "BCB923": "Alta Networks",
    "9C5467": "Nokia",
    "B46142": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C8442": "Cisco Systems, Inc",
    "84F117": "Newseason",
    "C8EFBC": "Inspur Communication Technology Co.,Ltd.",
    "98CCE4": "Shenzhen Mindray Animal Medical Technology Co.,LTD",
    "3C0664": "Beijing Leagrid Technology Co.,Ltd.",
    "BC5274": "Samsung Electronics Co.,Ltd",
    "9009DF": "Intel Corporate",
    "E40D36": "Intel Corporate",
    "149F43": "Cisco Meraki",
    "64DB38": "zte corporation",
    "2C08B4": "Huawei Device Co., Ltd.",
    "B40421": "zte corporation",
    "E84368": "zte corporation",
    "5007C3": "Amazon Technologies Inc.",
    "848DCE": "Ciena Corporation",
    "E466AB": "zte corporation",
    "A0957F": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "64135A": "Itectra A/S",
    "54E1B6": "Renesas Electronics (Penang) Sdn. Bhd.",
    "CC77C9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2C572C": "Allwinner Technology Co., Ltd",
    "ECDFC9": "Hangzhou Microimage Software Co., Ltd",
    "3CE441": "Amazon Technologies Inc.",
    "444201": "Amazon Technologies Inc.",
    "E46D7F": "Ciena Corporation",
    "80276C": "Cisco Systems, Inc",
    "6C4EF6": "Cisco Systems, Inc",
    "5C2763": "Itibia Technologies",
    "40C1F6": "Shenzhen Jingxun Technology Co., Ltd.",
    "FC22D3": "FDSYS",
    "4C5CDF": "ITEL MOBILE LIMITED",
    "C02E26": "iRhythm Technologies, Inc.",
    "0030AF": "Honeywell GmbH",
    "1CB8BA": "XIAMEN LEELEN TECHNOLOGY CO., LTD",
    "F0CCE0": "Shenzhen All-Smartlink Technology Co.,Ltd.",
    "F85C7E": "Shenzhen Honesty Electronics Co.,Ltd.",
    "F824E4": "Beyonics Technology Electronic (Changshu) Co., Ltd",
    "44E2F1": "NewRadio Technologies Co. , Ltd.",
    "504877": "Honor Device Co., Ltd.",
    "78C1AE": "Hangzhou Ezviz Software Co.,Ltd.",
    "30C6D7": "New H3C Technologies Co., Ltd",
    "F40595": "Sagemcom Broadband SAS",
    "00234B": "Inyuan Technology Inc.",
    "00176E": "DUCATI SISTEMI",
    "1C6A76": "Apple, Inc.",
    "6C7E67": "Apple, Inc.",
    "A4C6F0": "Apple, Inc.",
    "A88FD9": "Apple, Inc.",
    "089542": "Apple, Inc.",
    "E49C67": "Apple, Inc.",
    "64E220": "Qisda Corporation",
    "D4F242": "Huawei Device Co., Ltd.",
    "B07839": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "F41C71": "SHENZHEN SANMU COMMUNICATION TECHNOLOGY CO., LTD",
    "BC107B": "Samsung Electronics Co.,Ltd",
    "D4AD20": "Jinan USR IOT Technology Limited",
    "1CFC17": "Cisco Systems, Inc",
    "10AE60": "Amazon Technologies Inc.",
    "1C3283": "COMTTI Intelligent Technology(Shenzhen) Co., Ltd.",
    "C8848C": "Ruckus Wireless",
    "D85482": "Oxit, LLC",
    "C4EFDA": "Honeywell",
    "D8F507": "Fiberhome Telecommunication Technologies Co.,LTD",
    "70110E": "zte corporation",
    "98818A": "Huawei Device Co., Ltd.",
    "A8AA7C": "Huawei Device Co., Ltd.",
    "B4C2F7": "Huawei Device Co., Ltd.",
    "E05A1B": "Espressif Inc.",
    "488AE8": "vivo Mobile Communication Co., Ltd.",
    "CCBA6F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "785C5E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E886CF": "Nokia",
    "08B61F": "Espressif Inc.",
    "3C4E56": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "A08CF2": "YINUOLINK CO.,LTD",
    "40475E": "eero inc.",
    "5C3E1B": "Apple, Inc.",
    "7C2ACA": "Apple, Inc.",
    "288EEC": "Apple, Inc.",
    "8812AC": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "306371": "Shenzhenshi Xinzhongxin Technology Co.Ltd",
    "98A2C0": "Cisco Systems, Inc",
    "00410E": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "FC6179": "IEEE Registration Authority",
    "EC74D7": "Grandstream Networks Inc",
    "28011C": "zte corporation",
    "20E6DF": "eero inc.",
    "F4931C": "Universal Electronics, Inc.",
    "6C302A": "Texas Instruments",
    "7446B3": "Texas Instruments",
    "74375F": "SERCOMM PHILIPPINES INC",
    "C82AF1": "TCT mobile ltd",
    "3CE90E": "Espressif Inc.",
    "A842E3": "Espressif Inc.",
    "D49B74": "Kinetic Technologies",
    "480EEC": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "503EAA": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "50F261": "Photon Sail Technologies",
    "944E5B": "Ubee Interactive Co., Limited",
    "B4BA9D": "SKY UK LIMITED",
    "E08614": "Novatel Wireless Solutions, Inc.",
    "40F8DF": "CANON INC.",
    "0C7FED": "IEEE Registration Authority",
    "7C67AB": "Roku, Inc",
    "A8DE68": "Beijing Wide Technology Co.,Ltd",
    "F43BD8": "Intel Corporate",
    "A0889D": "Huawei Device Co., Ltd.",
    "3CFEAC": "Cisco Systems, Inc",
    "04A741": "Cisco Systems, Inc",
    "7C0C92": "Suzhou Mobydata Smart System Co.,Ltd.",
    "98D742": "Samsung Electronics Co.,Ltd",
    "F46ADD": "Liteon Technology Corporation",
    "18E91D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48706F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18B185": "Qiao Information Technology (Zhengzhou) Co., Ltd.",
    "04BAD6": "D-Link Corporation",
    "303F5D": "PT HAN SUNG ELECTORONICS INDONESIA",
    "B0FBDD": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "E09C8D": "Seakeeper, Inc.",
    "307F10": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A490CE": "vivo Mobile Communication Co., Ltd.",
    "3C69D1": "ADC Automotive Distance Control System GmbH",
    "54A9C8": "Home Control Singapore Pte Ltd",
    "A0B765": "Espressif Inc.",
    "CCDBA7": "Espressif Inc.",
    "C86C20": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "18863A": "DIGITAL ART SYSTEM",
    "C8D6B7": "Solidigm Technology",
    "F0877F": "Magnetar Technology Shenzhen Co., LTD.",
    "34AC11": "China Mobile Group Device Co.,Ltd.",
    "4432C2": "GOAL Co., Ltd.",
    "70662A": "Sony Interactive Entertainment Inc.",
    "DC71DD": "AX Technologies",
    "2064DE": "Sunitec Enterprise Co.,Ltd",
    "A40F98": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "74B725": "Huawei Device Co., Ltd.",
    "408EDF": "Huawei Device Co., Ltd.",
    "D880DC": "Huawei Device Co., Ltd.",
    "E8B3EF": "Fiberhome Telecommunication Technologies Co.,LTD",
    "5CFB3A": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "D04E50": "Mobiwire Mobiles (NingBo) Co., LTD",
    "F46C68": "Wistron Neweb Corporation",
    "8493B2": "zte corporation",
    "E8F791": "Xiaomi Communications Co Ltd",
    "B49F4D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E83A4B": "China Mobile Group Device Co.,Ltd.",
    "60E9AA": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "240F5E": "Shenzhen z-router Technology Co., Ltd",
    "000C06": "Nixvue Systems  Pte Ltd",
    "AC2929": "Infinix mobility limited",
    "18FD74": "Routerboard.com",
    "6C976D": "Motorola Mobility LLC, a Lenovo Company",
    "6411A4": "Motorola Mobility LLC, a Lenovo Company",
    "78669D": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "48468D": "Zepcam B.V.",
    "904992": "YSTen Technology Co.,Ltd",
    "3CCE0D": "Shenzhen juduoping Technology Co.,Ltd",
    "000EDB": "XiNCOM Corp.",
    "40D95A": "AMPAK Technology,Inc.",
    "D0497C": "OnePlus Technology (Shenzhen) Co., Ltd",
    "74D4DD": "Quanta Computer Inc.",
    "4CD0DD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4902A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "905E44": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10F068": "Ruckus Wireless",
    "7820BD": "Polysense (Beijing) Technologies Co. Ltd",
    "50E636": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "7CE152": "THE GOODYEAR TIRE & RUBBER COMPANY",
    "EC2125": "Toshiba Corp.",
    "28CDC1": "Raspberry Pi Trading Ltd",
    "68CE4E": "L-3 Communications Infrared Products",
    "00E5F1": "BUFFALO.INC",
    "34EE2A": "ConMet",
    "684E05": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "0499BB": "Apple, Inc.",
    "F04DD4": "Sagemcom Broadband SAS",
    "5C1BF4": "Apple, Inc.",
    "A851AB": "Apple, Inc.",
    "50392F": "INGRAM MICRO SERVICES",
    "FC8417": "Honor Device Co., Ltd.",
    "2CA79E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ACB566": "Renesas Electronics (Penang) Sdn. Bhd.",
    "381F26": "IEEE Registration Authority",
    "30045C": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "9079CF": "zte corporation",
    "8C1E80": "Cisco Systems, Inc",
    "C0EE40": "Laird Connectivity",
    "888FA4": "Huawei Device Co., Ltd.",
    "5068AC": "Huawei Device Co., Ltd.",
    "A4F933": "Intel Corporate",
    "10F60A": "Intel Corporate",
    "70D823": "Intel Corporate",
    "3C4645": "Shanghai Infinity Wireless Technologies Co.,Ltd.",
    "C4DEE2": "Espressif Inc.",
    "68B6B3": "Espressif Inc.",
    "FCA05A": "Oray.com co., LTD.",
    "90486C": "Ring LLC",
    "A41EE1": "Taicang T&W Electronics",
    "E8FB1C": "AzureWave Technology Inc.",
    "0C7FB2": "ARRIS Group, Inc.",
    "102407": "HUAWEI TECHNOLOGIES CO.,LTD",
    "74D9EB": "Petabit Scale, Inc.",
    "B4A7C6": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "80C3BA": "Sonova Consumer Hearing GmbH",
    "487E48": "Earda Technologies co Ltd",
    "286B35": "Intel Corporate",
    "4035E6": "Samsung Electronics Co.,Ltd",
    "D0A46F": "China Dragon Technology Limited",
    "5478C9": "AMPAK Technology,Inc.",
    "3043D7": "IEEE Registration Authority",
    "E84DEC": "Xerox Corporation",
    "C8B82F": "eero inc.",
    "B85DC3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C60CD": "NR ELECTRIC CO., LTD",
    "C4DF39": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "CC60C8": "Microsoft Corporation",
    "00405F": "AFE COMPUTERS LTD.",
    "B43AE2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0C8B5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC101A": "Palo Alto Networks",
    "6CAEE3": "Nokia",
    "0CAC8A": "Sagemcom Broadband SAS",
    "D8365F": "Intelbras",
    "0060E2": "QUEST ENGINEERING & DEVELOPMENT",
    "7404F1": "Intel Corporate",
    "00A265": "M2Motive Technology Inc.",
    "809733": " Shenzhen Elebao Technology Co., Ltd",
    "1054D2": "IEEE Registration Authority",
    "F46D2F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "00EBD8": "MERCUSYS TECHNOLOGIES CO., LTD.",
    "BC606B": "Shanghai Baud Data Communication Co.,Ltd.",
    "DCFE23": "Murata Manufacturing Co., Ltd.",
    "347379": "xFusion Digital Technologies Co., Limited",
    "D83DCC": "shenzhen UDD Technologies,co.,Ltd",
    "6C51BF": "Huawei Device Co., Ltd.",
    "C814B4": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "6C8D77": "Cisco Systems, Inc",
    "7411B2": "Cisco Systems, Inc",
    "04E69E": "ZHONGGUANCUN XINHAIZEYOU TECHNOLOGY CO.,LTD",
    "0417B6": "Smart Innovation LLC",
    "AC80AE": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E85F02": "Apple, Inc.",
    "E8C7CF": "Wistron Neweb Corporation",
    "CC5C61": "Huawei Device Co., Ltd.",
    "1CF42B": "Huawei Device Co., Ltd.",
    "E893F3": "Graphiant Inc",
    "00198E": "Demant A/S",
    "60FB00": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "D09200": "FiRa Consortium",
    "7CC180": "Apple, Inc.",
    "280244": "Apple, Inc.",
    "BCDF58": "Google, Inc.",
    "2C301A": "Technicolor CH USA Inc for Telus",
    "E42B79": "Nokia",
    "5C843C": "Sony Interactive Entertainment Inc.",
    "E8AEC5": "Arista Networks",
    "4017F6": "TKH SECURITY,S.L.U.",
    "60A6C5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "208C86": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0A951": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C049EF": "Espressif Inc.",
    "94B555": "Espressif Inc.",
    "74F2FA": "Xiaomi Communications Co Ltd",
    "ECE7A7": "Intel Corporate",
    "004058": "UKG",
    "185E0B": "zte corporation",
    "D0BB61": "zte corporation",
    "588FCF": "Hangzhou Ezviz Software Co.,Ltd.",
    "64D69A": "Intel Corporate",
    "3CE9F7": "Intel Corporate",
    "F4CE23": "Intel Corporate",
    "A84FB1": "Cisco Systems, Inc",
    "AC7352": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "50C275": "GN Audio A/S",
    "C8727E": "Nokia",
    "0CB815": "Espressif Inc.",
    "6CFFCE": "Sagemcom Broadband SAS",
    "C899B2": "Arcadyan Corporation",
    "F4B898": "Texas Instruments",
    "B0D278": "Texas Instruments",
    "6425EC": "guangdong kesheng zhixun technology ",
    "00E0CD": "SAAB SENSIS CORPORATION",
    "54ACFC": "LIZN ApS",
    "4C7274": "Shenzhenshi Xinzhongxin Technology Co.Ltd",
    "0024B6": "Seagate Technology",
    "A439B3": "Beijing Xiaomi Mobile Software Co., Ltd",
    "800CF9": "Amazon Technologies Inc.",
    "A09208": "Tuya Smart Inc.",
    "E484D3": "Xiaomi Communications Co Ltd",
    "04106B": "Xiaomi Communications Co Ltd",
    "04CCBC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1CE504": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C0BAB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94D2BC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E0D045": "Intel Corporate",
    "201F54": "Raisecom Technology CO., LTD",
    "7C97E1": "Huawei Device Co., Ltd.",
    "147EA1": "Britania Eletr\u00f4nicos S.A.",
    "E81656": "Hangzhou BroadLink Technology Co.,Ltd",
    "E0A25A": "Shanghai Mo xiang Network Technology CO.,ltd",
    "2887BA": "TP-Link Corporation Limited",
    "3460F9": "TP-Link Corporation Limited",
    "684AE9": "Samsung Electronics Co.,Ltd",
    "C83E9E": "Huawei Device Co., Ltd.",
    "60DD70": "Apple, Inc.",
    "98A5F9": "Apple, Inc.",
    "ECA907": "Apple, Inc.",
    "A88C3E": "Microsoft Corporation",
    "F82551": "Seiko Epson Corporation",
    "7453A8": "ACL Airshop BV",
    "4C06B7": "ProDVX Europe B.V.",
    "605747": "CIG SHANGHAI CO LTD",
    "149E5D": "JSC \"IB Reform\"",
    "940D2D": "Universal Electronics, Inc.",
    "14EBB6": "TP-Link Corporation Limited",
    "54AF97": "TP-Link Corporation Limited",
    "54CF8D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D48866": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D48457": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "BC4760": "Samsung Electronics Co.,Ltd",
    "2C4DDE": "TECNO MOBILE LIMITED",
    "303F7B": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "F863D9": "ARRIS Group, Inc.",
    "A405D6": "ARRIS Group, Inc.",
    "DCA633": "ARRIS Group, Inc.",
    "D8CFBF": "Motorola Mobility LLC, a Lenovo Company",
    "1845B3": "IEEE Registration Authority",
    "68BE49": "Nebula Matrix",
    "94F827": "Shanghai Imilab Technology Co.Ltd",
    "0CB789": "Honor Device Co., Ltd.",
    "F0B040": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "70B8F6": "Espressif Inc.",
    "34CE69": "Nokia Solutions and Networks GmbH & Co. KG",
    "E0720A": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "24D904": "Sichuan Changhong Network Technologies Co., Ltd.",
    "3C457A": "SKY UK LIMITED",
    "483E5E": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "50EBF6": "ASUSTek COMPUTER INC.",
    "C448FA": "Taicang T&W Electronics",
    "A89892": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "2804C6": "Wanan Hongsheng Electronic Co.Ltd",
    "D47954": "Huawei Device Co., Ltd.",
    "C839AC": "Huawei Device Co., Ltd.",
    "145594": "Huawei Device Co., Ltd.",
    "08C06C": "Huawei Device Co., Ltd.",
    "FC0296": "Xiaomi Communications Co Ltd",
    "60E85B": "Texas Instruments",
    "B4BA02": "Agatel Ltd",
    "60FF12": "Samsung Electronics Co.,Ltd",
    "24C613": "Samsung Electronics Co.,Ltd",
    "94E129": "Samsung Electronics Co.,Ltd",
    "C44F96": "Alps Alpine",
    "685811": "Fiberhome Telecommunication Technologies Co.,LTD",
    "902759": "Nanjing Jiahao Technology Co., Ltd.",
    "5C46B0": "SIMCom Wireless Solutions Limited",
    "3C219C": "Intel Corporate",
    "DC6294": "Guangzhou Lango Electronics Technology Co.,Ltd.",
    "84BA20": "Silicon Laboratories",
    "003C84": "Silicon Laboratories",
    "281D21": "IN ONE SMART TECHNOLOGY(H,K,)LIMITED",
    "74E336": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "C4345B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "703217": "Intel Corporate",
    "3498B5": "NETGEAR",
    "B0739C": "Amazon Technologies Inc.",
    "1859F5": "Cisco Systems, Inc",
    "D45763": "Apple, Inc.",
    "C02C5C": "Apple, Inc.",
    "EC4269": "HMD Global Oy",
    "24B72A": "China Dragon Technology Limited",
    "1C73E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C83E8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F829C0": "Availink, Inc.",
    "34A5B4": "NAVTECH PTE LTD",
    "E89F6D": "Espressif Inc.",
    "6457E5": "Beijing Royaltech Co.,Ltd",
    "FC9BD4": "EdgeQ",
    "145790": "Qingdao Haier Technology Co.,Ltd",
    "9877CB": "Vorteks ED",
    "389461": "Renesas Electronics (Penang) Sdn. Bhd.",
    "485519": "Espressif Inc.",
    "F8FCE1": "Amazon Technologies Inc.",
    "942957": "Airpo Networks Technology Co.,Ltd.",
    "986EE8": "IEEE Registration Authority",
    "405F7D": "TCT mobile ltd",
    "4C034F": "Intel Corporate",
    "D4C3B0": "Gearlinx Pty Ltd",
    "E070EA": "HP Inc.",
    "B88A72": "Renesas Electronics (Penang) Sdn. Bhd.",
    "2C53D7": "Sonova AG",
    "E8979A": "Quectel Wireless Solutions Co.,Ltd.",
    "5CA6E6": "TP-Link Corporation Limited",
    "B4B024": "TP-Link Corporation Limited",
    "18F22C": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "DCA782": "HUAWEI TECHNOLOGIES CO.,LTD",
    "045170": "Zhongshan K-mate General Electronics Co.,Ltd",
    "384554": "Harman/Becker Automotive Systems GmbH",
    "88D82E": "Intel Corporate",
    "50547B": "Nanjing Qinheng Microelectronics Co., Ltd.",
    "90F3B8": "China Mobile Group Device Co.,Ltd.",
    "38563D": "Microsoft Corporation",
    "28937D": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "D41AD1": "Zyxel Communications Corporation",
    "B845F4": "New H3C Technologies Co., Ltd",
    "EC01D5": "Cisco Systems, Inc",
    "180EAC": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "144920": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04D921": "Occuspace",
    "48B9C2": "Teletics Inc.",
    "7463C2": "Huawei Device Co., Ltd.",
    "D48FA2": "Huawei Device Co., Ltd.",
    "8881B9": "Huawei Device Co., Ltd.",
    "3C38F4": "Sony Corporation",
    "0CC413": "Google, Inc.",
    "4CEAAE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C436C0": "BUFFALO.INC",
    "08B49D": "TECNO MOBILE LIMITED",
    "081605": "Vodafone Italia S.p.A.",
    "2CE032": "TCL King Electrical Appliances(Huizhou)Co.,Ltd",
    "A0D2B1": "Amazon Technologies Inc.",
    "806D71": "Amazon Technologies Inc.",
    "9897CC": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "3CE4B0": "Texas Instruments",
    "E80115": "COOCAA Network Technology CO.,TD.",
    "2CB0FD": "Shenzhen MiaoMing  Intelligent Technology Co.,Ltd",
    "88AC9E": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "9C36F8": "Hyundai Kefico",
    "E0B72E": "ShenZhen Qualmesh Technology Co.,Ltd.",
    "B859CE": "Earda Technologies co Ltd",
    "8C7AAA": "Apple, Inc.",
    "7C2499": "Apple, Inc.",
    "C4FBC8": "Shenzhen Candour Co., Ltd.",
    "000D4E": "NDR Co.,LTD.",
    "100020": "Apple, Inc.",
    "BC1E85": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CEA12": "Shenzhen MiaoMing  Intelligent Technology Co.,Ltd",
    "C8BD4D": "Samsung Electronics Co.,Ltd",
    "2C15BF": "Samsung Electronics Co.,Ltd",
    "1C53F9": "Google, Inc.",
    "04421A": "ASUSTek COMPUTER INC.",
    "207C14": "Qotom",
    "CC3F8A": "KOMATSU LTD.",
    "305696": "Infinix mobility limited",
    "7CB073": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "0CE5A3": "SharkNinja",
    "409151": "Espressif Inc.",
    "B0E9FE": "Woan Technology (Shenzhen) Co., Ltd.",
    "C89665": "Microsoft Corporation",
    "305F77": "New H3C Technologies Co., Ltd",
    "8C7A3D": "Xiaomi Communications Co Ltd",
    "987EE3": "vivo Mobile Communication Co., Ltd.",
    "34317F": "Panasonic Appliances Company",
    "00BD3E": "Vizio, Inc",
    "B85600": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04B86A": "SKY UK LIMITED",
    "18F87F": "Wha Yu Industrial Co., Ltd.",
    "0899E8": "KEMAS GmbH",
    "E446B0": "Fujitsu Client Computing Limited",
    "54CE82": "zte corporation",
    "584849": "IEEE Registration Authority",
    "E075AA": "Beijing Jingling Information System Technology Co., Ltd.",
    "80CBBC": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "EC354D": "Wingtech Mobile Communications Co.,Ltd",
    "7C3985": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44AE44": "Huawei Device Co., Ltd.",
    "444F8E": "WiZ",
    "000534": "Northstar Engineering Ltd.",
    "BC091B": "Intel Corporate",
    "0026A7": "CONNECT SRL",
    "180712": "Shenzhen Dazoo Technologies CO.,Ltd",
    "F02A2B": "IEEE Registration Authority",
    "10F605": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "5C8F40": "TECNO MOBILE LIMITED",
    "90F157": "Garmin International",
    "FC5C45": "Ruckus Wireless",
    "20D778": "Texas Instruments",
    "4C2FD7": "Huawei Device Co., Ltd.",
    "D47415": "Huawei Device Co., Ltd.",
    "001441": "Innovation Sound Technology Co., LTD.",
    "B4ECFF": "Wuhan IPG Technologies Co., Ltd.",
    "F057A6": "Intel Corporate",
    "20BECD": "eero inc.",
    "04ECD8": "Intel Corporate",
    "7CC255": "Super Micro Computer, Inc.",
    "50586F": "Huawei Device Co., Ltd.",
    "F0B13F": "Huawei Device Co., Ltd.",
    "1CE639": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C6794": "vivo Mobile Communication Co., Ltd.",
    "642677": "BKM-Micronic Richtfunkanlagen GmbH",
    "700777": "OnTarget Technologies, Inc",
    "0064AF": "Dish Technologies Corp",
    "00BFAF": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "E0CB56": "Shenzhen iComm Semiconductor CO.,LTD",
    "40E1E4": "Nokia Solutions and Networks GmbH & Co. KG",
    "2864EF": "Shenzhen Fsan Intelligent Technology Co.,Ltd",
    "38FF13": "Joint Stock Company \"Research Instinite \"Masshtab\"",
    "D0DBB7": "Casa Systems",
    "E4B503": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "ECF40C": "Cisco Systems, Inc",
    "1006ED": "Cisco Systems, Inc",
    "D4EB68": "Cisco Systems, Inc",
    "C44D84": "Cisco Systems, Inc",
    "102CEF": "EMU Electronic AG",
    "8C1D96": "Intel Corporate",
    "60DD8E": "Intel Corporate",
    "B4CDF5": "CUB ELECPARTS INC.",
    "6CCF39": "Guangdong Starfive Technology Co., Ltd.",
    "D88C73": "zte corporation",
    "3865B2": "Apple, Inc.",
    "D8DE3A": "Apple, Inc.",
    "205383": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8857B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50D065": "ESYLUX GmbH",
    "90593C": "AZ-TECHNOLOGY SDN BHD",
    "1CC10C": "Intel Corporate",
    "F4A475": "Intel Corporate",
    "C89E43": "NETGEAR",
    "E87865": "Apple, Inc.",
    "A04ECF": "Apple, Inc.",
    "0887C7": "Apple, Inc.",
    "C0D063": "EM Microelectronic",
    "7C10C9": "ASUSTek COMPUTER INC.",
    "DCF56E": "Wellysis Corp.",
    "844709": "Shenzhen IP3 Century Intelligent Technology CO.,Ltd",
    "5CD06E": "Xiaomi Communications Co Ltd",
    "0091EB": "Renesas Electronics (Penang) Sdn. Bhd.",
    "A8671E": "RATP",
    "8CE9B4": "Zhejiang Dahua Technology Co., Ltd.",
    "08855B": "Kontron Europe GmbH",
    "E84FA7": "Huawei Device Co., Ltd.",
    "449F46": "Huawei Device Co., Ltd.",
    "345184": "Huawei Device Co., Ltd.",
    "FCF77B": "Huawei Device Co., Ltd.",
    "64ED57": "ARRIS Group, Inc.",
    "EC08E5": "Motorola Mobility LLC, a Lenovo Company",
    "80CC9C": "NETGEAR",
    "74E20C": "Amazon Technologies Inc.",
    "4CD577": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "507097": "China Mobile Group Device Co.,Ltd.",
    "0C6046": "vivo Mobile Communication Co., Ltd.",
    "CC1531": "Intel Corporate",
    "C0F9B0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "280FEB": "LG Innotek",
    "CC2D8C": "LG ELECTRONICS INC",
    "105403": "INTARSO GmbH",
    "B81332": "AMPAK Technology,Inc.",
    "589153": "China Mobile IOT Company Limited",
    "987DDD": "China Mobile Group Device Co.,Ltd.",
    "44DB60": "Nanjing Baihezhengliu Technology Co., Ltd",
    "B8B77D": "Guangdong Transtek Medical Electronics CO.,Ltd",
    "C478A2": "Huawei Device Co., Ltd.",
    "9C9E71": "Huawei Device Co., Ltd.",
    "C0238D": "Samsung Electronics Co.,Ltd",
    "745D68": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C0FBF9": "IEEE Registration Authority",
    "2811A8": "Intel Corporate",
    "74CF00": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "0C9A3C": "Intel Corporate",
    "DC2148": "Intel Corporate",
    "9C5A81": "Xiaomi Communications Co Ltd",
    "20CFAE": "Cisco Systems, Inc",
    "98499F": "Domo Tactical Communications",
    "148C4A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "609BB4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C27BC": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "ACE14F": "Autonomic Controls, Inc.",
    "AC976C": "Greenliant",
    "2CEADC": "ASKEY COMPUTER CORP",
    "1469A2": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "046B25": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "1012B4": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "88C9B3": "IEEE Registration Authority",
    "5813D3": "Gemtek Technology Co., Ltd.",
    "201E88": "Intel Corporate",
    "5CB00A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "206A94": "Hitron Technologies. Inc",
    "309587": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "20CE2A": "IEEE Registration Authority",
    "9C32A9": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "908674": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "68262A": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "B8224F": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "C8BD69": "Samsung Electronics Co.,Ltd",
    "C04B13": "WonderSound Technology Co., Ltd",
    "9C7F81": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "642753": "Huawei Device Co., Ltd.",
    "28534E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A87650": "Samsung Electronics Co.,Ltd",
    "54D17D": "Samsung Electronics Co.,Ltd",
    "603AAF": "Samsung Electronics Co.,Ltd",
    "00E5E4": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "643AB1": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "8048A5": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "44BA46": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "F40228": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "0C83CC": "Alpha Networks Inc.",
    "14CB19": "HP Inc.",
    "001CC5": "3Com Ltd",
    "E84F25": "Murata Manufacturing Co., Ltd.",
    "0425E0": "Taicang T&W Electronics",
    "C8B6FE": "Fitbit, Inc.",
    "187CAA": "China Mobile Group Device Co.,Ltd.",
    "C4BCD7": "New Ryatek",
    "C0AEFD": "Shenzhen HC-WLAN Technology Co.,Ltd",
    "1027F5": "TP-Link Corporation Limited",
    "ECB970": "Ruijie Networks Co.,LTD",
    "308398": "Espressif Inc.",
    "F889D2": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "DC87CB": "Beijing Perfectek Technologies Co., Ltd.",
    "385247": "Huawei Device Co., Ltd.",
    "C0DCD7": "Huawei Device Co., Ltd.",
    "3037B3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CF666": "Tuya Smart Inc.",
    "102D41": "Sichuan AI-Link Technology Co., Ltd.",
    "A44C62": "Hangzhou Microimage Software Co., Ltd",
    "40BEEE": "Shenzhen Yunding Information Technology Co.,Ltd",
    "7C55A7": "Kastle Systems",
    "FCE806": "Edifier International",
    "3078D3": "Virgilant Technologies Ltd.",
    "3085EB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8406FA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "B8D43E": "vivo Mobile Communication Co., Ltd.",
    "741575": "Xiaomi Communications Co Ltd",
    "90A822": "Amazon Technologies Inc.",
    "0C5CB5": "IEEE Registration Authority",
    "74AD98": "Cisco Systems, Inc",
    "0017E8": "Texas Instruments",
    "E8EA4D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3CFFD8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50C2ED": "GN Audio A/S",
    "7864C0": "Apple, Inc.",
    "E81CD8": "Apple, Inc.",
    "3C0630": "Apple, Inc.",
    "848C8D": "Apple, Inc.",
    "0CE441": "Apple, Inc.",
    "B82AA9": "Apple, Inc.",
    "00C06A": "Zahner-Elektrik Ingeborg Zahner-Schiller GmbH & Co. KG.",
    "8C4361": "Hailo Digital Hub GmbH & Co. KG",
    "4851CF": "Intelbras",
    "4C5D3C": "Cisco Systems, Inc",
    "34732D": "Cisco Systems, Inc",
    "8C3446": "Huawei Device Co., Ltd.",
    "F46B8C": "Hon Hai Precision Industry Co., Ltd.",
    "FC3497": "ASUSTek COMPUTER INC.",
    "847AB6": "AltoBeam (China) Inc.",
    "245DFC": "IEEE Registration Authority",
    "A4CEDA": "Arcadyan Corporation",
    "34E9FE": "Metis Co., Ltd.",
    "4C617E": "Huawei Device Co., Ltd.",
    "E0E37C": "Huawei Device Co., Ltd.",
    "2418C6": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "649714": "eero inc.",
    "B4107B": "Texas Instruments",
    "9C8281": "vivo Mobile Communication Co., Ltd.",
    "AC139C": "Adtran Inc",
    "804786": "Samsung Electronics Co.,Ltd",
    "D4475A": "ScreenBeam, Inc.",
    "78A6A0": "Hangzhou Ezviz Software Co.,Ltd.",
    "98C3D2": "Ningbo Sanxing Medical Electric Co.,Ltd",
    "CC86EC": "Silicon Laboratories",
    "047BCB": "Universal Global Scientific Industrial Co., Ltd.",
    "E44164": "Nokia",
    "0881B2": "Logitech (China) Technology Co., Ltd",
    "C4F174": "eero inc.",
    "28D0EA": "Intel Corporate",
    "CC812A": "vivo Mobile Communication Co., Ltd.",
    "7C1B93": "Huawei Device Co., Ltd.",
    "DC2D3C": "Huawei Device Co., Ltd.",
    "E455A8": "Cisco Meraki",
    "041119": "IEEE Registration Authority",
    "E00CE5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2841EC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C004D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C9957": "Intel Corporate",
    "38A067": "Nokia Solutions and Networks GmbH & Co. KG",
    "18A6F7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "04D320": "ITEL MOBILE LIMITED",
    "30F94B": "Universal Electronics, Inc.",
    "D8ECE5": "Zyxel Communications Corporation",
    "C470AB": "Ruijie Networks Co.,LTD",
    "CC6B1E": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "F0D08C": "TCT mobile ltd",
    "982782": "IEEE Registration Authority",
    "FC4009": "zte corporation",
    "24A65E": "zte corporation",
    "509839": "Xiaomi Communications Co Ltd",
    "6407F6": "Samsung Electronics Co.,Ltd",
    "48007D": "DTS ELEKTRONIK SAN. TIC. LTD. STI.",
    "30B1B5": "Arcadyan Corporation",
    "E4C90B": "Radwin",
    "70DFF7": "ARRIS Group, Inc.",
    "003DE1": "Huawei Device Co., Ltd.",
    "5CC336": "ittim",
    "50ED3C": "Apple, Inc.",
    "FC041C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F4D488": "Apple, Inc.",
    "682F67": "Apple, Inc.",
    "E0E8BB": "Unicom Vsens Telecommunications Co., Ltd.",
    "708976": "Tuya Smart Inc.",
    "245B83": "Renesas Electronics (Penang) Sdn. Bhd.",
    "584120": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "3C06A7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "24E927": "TomTom International BV",
    "68ABBC": "Beijing Xiaomi Mobile Software Co., Ltd",
    "34C103": "Hangzhou Huamu Technology Co.,Ltd.",
    "0C7329": "Sercomm Corporation.",
    "B06088": "Intel Corporate",
    "6413AB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C13D5": "Cisco Systems, Inc",
    "F0F564": "Samsung Electronics Co.,Ltd",
    "F06F46": "Ubiik",
    "6C02E0": "HP Inc.",
    "F0ACA4": "HBC-radiomatic",
    "802278": "China Mobile IOT Company Limited",
    "14230A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58AEA8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64D7C0": "Huawei Device Co., Ltd.",
    "946010": "Huawei Device Co., Ltd.",
    "D814DF": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "90027A": "Shenzhen Sworix Techonlogy Co., Ltd",
    "D05AFD": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "6CD704": "HUAWEI TECHNOLOGIES CO.,LTD",
    "543AD6": "Samsung Electronics Co.,Ltd",
    "845CF3": "Intel Corporate",
    "70E46E": "Lytx",
    "58AE2B": "Huawei Device Co., Ltd.",
    "E43C80": "University of Oklahoma",
    "0C4885": "LG Electronics (Mobile Communications)",
    "544617": "zte corporation",
    "5C85F8": "SHENZHEN KAIFA TECHNOLOGY CO.,LTD.",
    "2C793D": "Boditech Med",
    "903CB3": "Edgecore Networks Corporation",
    "9CBCF0": "Xiaomi Communications Co Ltd",
    "A46BB6": "Intel Corporate",
    "FC0C45": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "5467E6": "SHENZHEN MTC CO LTD",
    "34FEC5": "Shenzhen Sunwoda intelligent hardware Co.,Ltd",
    "5466F9": "ConMet",
    "8C64D4": "Hyeco Smart Tech Co.,Ltd",
    "8444AF": "Zhejiang Tmall Technology Co., Ltd.",
    "14B2E5": "Shenzhen iComm Semiconductor CO.,LTD",
    "A8F766": "ITE Tech Inc",
    "7C210D": "Cisco Systems, Inc",
    "D03C1F": "Intel Corporate",
    "A03D6E": "Cisco Systems, Inc",
    "B08BD0": "Cisco Systems, Inc",
    "000BDE": "TELDIX GmbH",
    "F89725": "OPPLE LIGHTING CO., LTD",
    "3C5447": "HUAWEI TECHNOLOGIES CO.,LTD",
    "006B6F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CDEF9": "Beijing Xiaomi Mobile Software Co., Ltd",
    "74E9BF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D8109F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "482567": "Poly",
    "A89AD7": "Nokia",
    "181171": "Guangzhou Doctorpai Education & Technology Co.,Ltd",
    "603573": "Earda Technologies co Ltd",
    "A49733": "ASKEY COMPUTER CORP",
    "F02F74": "ASUSTek COMPUTER INC.",
    "006E02": "Xovis AG",
    "FCD436": "Motorola Mobility LLC, a Lenovo Company",
    "0CEC8D": "Motorola Mobility LLC, a Lenovo Company",
    "184F5D": "JRC Mobility Inc.",
    "103F44": "Xiaomi Communications Co Ltd",
    "E07726": "Huawei Device Co., Ltd.",
    "8C3401": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "249F89": "Texas Instruments",
    "247625": "Texas Instruments",
    "2C1875": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "D06EDE": "Sagemcom Broadband SAS",
    "F87A41": "Cisco Systems, Inc",
    "84F147": "Cisco Systems, Inc",
    "94A67E": "NETGEAR",
    "D439B8": "Ciena Corporation",
    "F023AE": "AMPAK Technology,Inc.",
    "ACF85C": "Chengdu Higon Integrated Circuit Design Co,. Ltd.",
    "08CBE5": "R3 Solutions GmbH",
    "249494": "Hong Kong Bouffalo Lab Limited",
    "30BE3B": "Mitsubishi Electric Corporation",
    "3CA37E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7898E8": "D-Link International",
    "0034A1": "RF-LAMBDA USA INC.",
    "78F09B": "Huawei Device Co., Ltd.",
    "48EF61": "Huawei Device Co., Ltd.",
    "F0B3EC": "Apple, Inc.",
    "F465A6": "Apple, Inc.",
    "00E93A": "AzureWave Technology Inc.",
    "44F21B": "Apple, Inc.",
    "74650C": "Apple, Inc.",
    "E06D17": "Apple, Inc.",
    "F85EA0": "Intel Corporate",
    "5C6199": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "E8DB84": "Espressif Inc.",
    "D05509": "Nintendo Co.,Ltd",
    "00081E": "Repeatit AB",
    "A07751": "ASMedia Technology Inc.",
    "305684": "SHENZHEN YUNJI INTELLIGENT TECHNOLOGY CO.,LTD",
    "38FC98": "Intel Corporate",
    "9CB2E8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60DB98": "Calix Inc.",
    "6872C3": "Samsung Electronics Co.,Ltd",
    "70B13D": "Samsung Electronics Co.,Ltd",
    "502F9B": "Intel Corporate",
    "14563A": "Huawei Device Co., Ltd.",
    "7090B7": "Huawei Device Co., Ltd.",
    "F8E43B": "ASIX Electronics Corporation",
    "342B70": "Arris",
    "4C0220": "Xiaomi Communications Co Ltd",
    "D41B81": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "F40B9F": "CIG SHANGHAI CO LTD",
    "D021AC": "Yohana",
    "7845B3": "Huawei Device Co., Ltd.",
    "20DCFD": "Huawei Device Co., Ltd.",
    "FC65B3": "Huawei Device Co., Ltd.",
    "C094AD": "zte corporation",
    "00188C": "Mobile Action Technology Inc.",
    "1C90BE": "Ericsson AB",
    "787DF3": "Sterlite Technologies Limited",
    "74731D": "ifm electronic gmbh",
    "001F94": "Lascar Electronics Ltd",
    "109D7A": "Huawei Device Co., Ltd.",
    "DC6373": "OBARA KOREA",
    "D47EE4": "China Mobile IOT Company Limited",
    "1CA852": "SENSAIO PTE LTD",
    "401C83": "Intel Corporate",
    "443B32": "Intelbras",
    "88892F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28E5B0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C440F6": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "84EA97": "Shenzhen iComm Semiconductor CO.,LTD",
    "A47D9F": "Shenzhen iComm Semiconductor CO.,LTD",
    "38CA73": "Shenzhen MiaoMing  Intelligent Technology Co.,Ltd",
    "6C0DC4": "Beijing Xiaomi Electronics Co., Ltd.",
    "E01CFC": "D-Link International",
    "683E26": "Intel Corporate",
    "8C554A": "Intel Corporate",
    "549FC6": "Cisco Systems, Inc",
    "F01D2D": "Cisco Systems, Inc",
    "0055B1": "Shanghai Baud Data Communication Co.,Ltd.",
    "74901F": "Ragile Networks Inc.",
    "204EF6": "AzureWave Technology Inc.",
    "088FC3": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "9CBF0D": "Framework Computer LLC",
    "0C8B7D": "Vizio, Inc",
    "0023E9": "F5 Networks, Inc.",
    "3C7C3F": "ASUSTek COMPUTER INC.",
    "24470E": "PentronicAB",
    "8CD67F": "EM Microelectronic",
    "34916F": "UserGate Ltd.",
    "EC4D3E": "Beijing Xiaomi Mobile Software Co., Ltd",
    "5C91FD": "Jaewoncnc",
    "EC3EB3": "Zyxel Communications Corporation",
    "FC449F": "zte corporation",
    "9860CA": "Apple, Inc.",
    "4490BB": "Apple, Inc.",
    "34FD6A": "Apple, Inc.",
    "443583": "Apple, Inc.",
    "D8A491": "Huawei Device Co., Ltd.",
    "681324": "Huawei Device Co., Ltd.",
    "A8C092": "Huawei Device Co., Ltd.",
    "24B105": "Prama Hikvision India Private Limited",
    "709CD1": "Intel Corporate",
    "142C78": "GooWi Wireless Technology Co., Limited",
    "98FC84": "IEEE Registration Authority",
    "DC7223": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "90AAC3": "Hitron Technologies. Inc",
    "44917C": "HMD Global Oy",
    "245F9F": "Huawei Device Co., Ltd.",
    "CCB0A8": "Huawei Device Co., Ltd.",
    "502873": "Huawei Device Co., Ltd.",
    "F82E3F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "90A5AF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0476B0": "Cisco Systems, Inc",
    "40F078": "Cisco Systems, Inc",
    "78D71A": "Ciena Corporation",
    "20F44F": "Nokia",
    "4CB99B": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "BC7E8B": "Samsung Electronics Co.,Ltd",
    "4CAEEC": "Guangzhou limee technology co.,LTD",
    "9C6B37": "Renesas Electronics (Penang) Sdn. Bhd.",
    "A83759": "Huawei Device Co., Ltd.",
    "50F7ED": "Huawei Device Co., Ltd.",
    "6433DB": "Texas Instruments",
    "A406E9": "Texas Instruments",
    "B0B113": "Texas Instruments",
    "3420E3": "Ruckus Wireless",
    "78719C": "ARRIS Group, Inc.",
    "0C31DC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40DDD1": "Beautiful Card Corporation",
    "C0E7BF": "Sichuan AI-Link Technology Co., Ltd.",
    "8060B7": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "6CA4D1": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A446B4": "Huawei Device Co., Ltd.",
    "DCD444": "Huawei Device Co., Ltd.",
    "70828E": "OleumTech Corporation",
    "20A171": "Amazon Technologies Inc.",
    "346D9C": "Carrier Corporation",
    "7CF9A0": "Fiberhome Telecommunication Technologies Co.,LTD",
    "345594": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "4C93A6": "IEEE Registration Authority",
    "CC47BD": "Rhombus Systems",
    "40AA56": "China Dragon Technology Limited",
    "68545A": "Intel Corporate",
    "001E31": "infomark",
    "3CE3E7": "China Mobile Group Device Co.,Ltd.",
    "24FD0D": "Intelbras",
    "E0693A": "Innophase Inc.",
    "0002AC": "3PAR data",
    "5488DE": "Cisco Systems, Inc",
    "782B64": "Bose Corporation",
    "D4F756": "zte corporation",
    "54AB3A": "Quanta Computer Inc.",
    "E89A8F": "Quanta Computer Inc.",
    "EC6C9A": "Arcadyan Corporation",
    "0CC844": "Cambridge Mobile Telematics, Inc.",
    "E0B260": "TENO NETWORK TECHNOLOGIES COMPANY LIMITED",
    "482335": "Dialog Semiconductor Hellas SA",
    "E8DA20": "Nintendo Co.,Ltd",
    "E8136E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CAE13": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C2EFE": "Shenzhen Comnect Technology Co.,LTD",
    "643AEA": "Cisco Systems, Inc",
    "3C53D7": "CEDES AG",
    "A0D83D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "58F2FC": "Huawei Device Co., Ltd.",
    "10BC97": "vivo Mobile Communication Co., Ltd.",
    "D01411": "IEEE Registration Authority",
    "507043": "SKY UK LIMITED",
    "4401BB": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "C80210": "LG Innotek",
    "E06CA6": "Creotech Instruments S.A.",
    "44680C": "Wacom Co.,Ltd.",
    "980E24": "Phytium Technology Co.,Ltd.",
    "A830BC": "Samsung Electronics Co.,Ltd",
    "001EB2": "LG Innotek",
    "C086B3": "Shenzhen Voxtech Co., Ltd.",
    "44ADB1": "Sagemcom Broadband SAS",
    "1C98C1": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "A09B17": "Taicang T&W Electronics",
    "0063DE": "CLOUDWALK TECHNOLOGY CO.,LTD",
    "60A423": "Silicon Laboratories",
    "7846D4": "Samsung Electronics Co.,Ltd",
    "6888A1": "Universal Electronics, Inc.",
    "E43EC6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38881E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2CDB07": "Intel Corporate",
    "988D46": "Intel Corporate",
    "001693": "PowerLink Technology Inc.",
    "AC1F09": "shenzhen RAKwireless technology  Co.,Ltd",
    "10F920": "Cisco Systems, Inc",
    "9077EE": "Cisco Systems, Inc",
    "3C13CC": "Cisco Systems, Inc",
    "D8DC40": "Apple, Inc.",
    "805FC5": "Apple, Inc.",
    "3C4DBE": "Apple, Inc.",
    "48262C": "Apple, Inc.",
    "147DDA": "Apple, Inc.",
    "C4910C": "Apple, Inc.",
    "2848E7": "Huawei Device Co., Ltd.",
    "E4268B": "Huawei Device Co., Ltd.",
    "6C9961": "Sagemcom Broadband SAS",
    "206980": "Apple, Inc.",
    "BC13A8": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "94E3EE": "zte corporation",
    "8CE468": "Guangzhou Sageran Technology Co., Ltd.",
    "C0E3A0": "Renesas Electronics (Penang) Sdn. Bhd.",
    "AC2334": "Infinix mobility limited",
    "F8BC0E": "eero inc.",
    "E01995": "Nutanix",
    "2479EF": "Greenpacket Berhad, Taiwan",
    "B05CDA": "HP Inc.",
    "FCBC0E": "Zhejiang Cainiao Supply Chain Management Co., Ltd",
    "2CD066": "Xiaomi Communications Co Ltd",
    "50E039": "Zyxel Communications Corporation",
    "B85776": "lignex1",
    "F85128": "SimpliSafe",
    "2400FA": "China Mobile (Hangzhou) Information Technology Co., Ltd",
    "DCBD7A": "Guangzhou\u00a0Shiyuan\u00a0Electronic\u00a0Technology\u00a0Company\u00a0Limited",
    "342EB7": "Intel Corporate",
    "60AAEF": "Huawei Device Co., Ltd.",
    "D0F3F5": "Huawei Device Co., Ltd.",
    "9016BA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D44649": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9400B0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC26A1": "FACTORY FIVE Corporation",
    "18DFC1": "Aetheros",
    "E44122": "OnePlus Technology (Shenzhen) Co., Ltd",
    "9C19C2": "Dongguan Liesheng Electronic Co., Ltd.",
    "B01B7C": "Ontrol A.S.",
    "001C70": "NOVACOMM LTDA",
    "000FA4": "Sprecher Automation GmbH",
    "0024A2": "Hong Kong Middleware Technology Limited",
    "74CBF3": "Lava international limited",
    "84A3B5": "Propulsion systems",
    "9CEDFA": "EVUlution AG",
    "EC63ED": "Hyundai Autoever Corp.",
    "1C1338": "Kimball Electronics Group, LLC",
    "2468B0": "Samsung Electronics Co.,Ltd",
    "30FCEB": "LG Electronics (Mobile Communications)",
    "FC71FA": "Trane Technologies",
    "0002D8": "BRECIS Communications Corporation",
    "B4EF1C": "360 AI Technology Co.Ltd",
    "B8F009": "Espressif Inc.",
    "0001CC": "Japan Total Design Communication Co., Ltd.",
    "04F5F4": "Proxim Wireless",
    "008016": "WANDEL AND GOLTERMANN",
    "E8B470": "IEEE Registration Authority",
    "A4B1C1": "Intel Corporate",
    "00092E": "B&Tech System Inc.",
    "D41AC8": "Nippon Printer Engineering",
    "5061F6": "Universal Electronics, Inc.",
    "F4EB9F": "Ellu Company 2019 SL",
    "E898C2": "ZETLAB Company",
    "64DDE9": "Xiaomi Communications Co Ltd",
    "4C4088": "SANSHIN ELECTRONICS CO.,LTD.",
    "B04502": "Huawei Device Co., Ltd.",
    "1C1FF1": "Huawei Device Co., Ltd.",
    "14DE39": "Huawei Device Co., Ltd.",
    "F45420": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "0C817D": "EEP Elektro-Elektronik Pranjic GmbH",
    "C8BCE5": "Sense Things Japan INC.",
    "9C611D": "Panasonic Corporation of North America",
    "C095DA": "NXP India Private Limited",
    "6849B2": "CARLO GAVAZZI LTD",
    "487AFF": "ESSYS",
    "C8D778": "BSH Hausgeraete GmbH",
    "CCA7C1": "Google, Inc.",
    "A4C54E": "Huawei Device Co., Ltd.",
    "D4BBE6": "Huawei Device Co., Ltd.",
    "40B6E7": "Huawei Device Co., Ltd.",
    "D0B45D": "Huawei Device Co., Ltd.",
    "8836CF": "Huawei Device Co., Ltd.",
    "009EEE": "Positivo Tecnologia S.A.",
    "8C5FAD": "Fiberhome Telecommunication Technologies Co.,LTD",
    "ACC25D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8C0C87": "Nokia",
    "F0D5BF": "Intel Corporate",
    "DCD2FC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60F262": "Intel Corporate",
    "00A058": "GLORY, LTD.",
    "7C9EBD": "Espressif Inc.",
    "1C0219": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "6C6A77": "Intel Corporate",
    "1869D8": "Tuya Smart Inc.",
    "C8B29B": "Intel Corporate",
    "CC418E": "MSA Innovation",
    "B42200": "Brother Industries, LTD.",
    "402F86": "LG Innotek",
    "186F2D": "Shenzhen Sundray Technologies Company Limited",
    "F84FAD": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "4C0A3D": "ADNACOM INC.",
    "CC7F76": "Cisco Systems, Inc",
    "9405BB": "IEEE Registration Authority",
    "B802A4": "Aeonsemi, Inc.",
    "A84122": "China Mobile (Hangzhou) Information Technology Co.,Ltd.",
    "206D31": "FIREWALLA INC",
    "E48326": "HUAWEI TECHNOLOGIES CO.,LTD",
    "447654": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CD9A0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F033E5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C806B": "Hunan Voc Acoustics Technology Co., Ltd.",
    "60DE35": "GITSN, Inc.",
    "28317E": "Hongkong Nano IC Technologies Co., Ltd",
    "B4F18C": "Huawei Device Co., Ltd.",
    "C432D1": "Farlink Technology Limited",
    "DC9840": "Microsoft Corporation",
    "B8CEF6": "Mellanox Technologies, Inc.",
    "6CDDBC": "Samsung Electronics Co.,Ltd",
    "8C3B32": "Microfan B.V.",
    "3C58C2": "Intel Corporate",
    "CCF9E4": "Intel Corporate",
    "645CF3": "ParanTek Inc.",
    "90749D": "IRay Technology Co., Ltd.",
    "28401A": "C8 MediSensors, Inc.",
    "D06544": "Apple, Inc.",
    "E490FD": "Apple, Inc.",
    "84AB1A": "Apple, Inc.",
    "E45E37": "Intel Corporate",
    "B0CCFE": "Huawei Device Co., Ltd.",
    "540DF9": "Huawei Device Co., Ltd.",
    "006619": "Huawei Device Co., Ltd.",
    "FC3964": "ITEL MOBILE LIMITED",
    "E8ABFA": "Shenzhen Reecam Tech.Ltd.",
    "14472D": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "000E9E": "Topfield Co., Ltd",
    "5894B2": "BrainCo",
    "B09575": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "000C1E": "Global Cache",
    "F008D1": "Espressif Inc.",
    "90E2FC": "IEEE Registration Authority",
    "C853E1": "Beijing Bytedance Network Technology Co., Ltd",
    "14169D": "Cisco Systems, Inc",
    "48A2E6": "Resideo",
    "CCD42E": "Arcadyan Corporation",
    "14AE85": "IEEE Registration Authority",
    "048C16": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98DD5B": "TAKUMI JAPAN LTD",
    "B4B055": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C5CF1": "eero inc.",
    "48D24F": "Sagemcom Broadband SAS",
    "E4AAEC": "Tianjin Hualai Technology Co., Ltd",
    "94BE46": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "6C5D3A": "Microsoft Corporation",
    "8CC681": "Intel Corporate",
    "C8C750": "Motorola Mobility LLC, a Lenovo Company",
    "ACF8CC": "ARRIS Group, Inc.",
    "8C5A25": "ARRIS Group, Inc.",
    "2CDCD7": "AzureWave Technology Inc.",
    "483FDA": "Espressif Inc.",
    "E0BB9E": "Seiko Epson Corporation",
    "48DD0C": "eero inc.",
    "58278C": "BUFFALO.INC",
    "140AC5": "Amazon Technologies Inc.",
    "2083F8": "Advanced Digital Broadcast SA",
    "940C98": "Apple, Inc.",
    "E8FBE9": "Apple, Inc.",
    "38EC0D": "Apple, Inc.",
    "9C54DA": "SkyBell Technologies Inc.",
    "4C494F": "zte corporation",
    "C4741E": "zte corporation",
    "D46075": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "78C5F8": "Huawei Device Co., Ltd.",
    "DC8983": "Samsung Electronics Co.,Ltd",
    "5CCB99": "Samsung Electronics Co.,Ltd",
    "90B144": "Samsung Electronics Co.,Ltd",
    "001D7D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "1C6F65": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "00241D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "E47C65": "Sunstar Communication Technology  Co., Ltd",
    "5C78F8": "Huawei Device Co., Ltd.",
    "B827C5": "Huawei Device Co., Ltd.",
    "D45D64": "ASUSTek COMPUTER INC.",
    "142A14": "ShenZhen Selenview Digital Technology Co.,Ltd",
    "B86142": "Beijing Tricolor Technology Co., Ltd",
    "FC8E5B": "China Mobile Iot Limited company",
    "A4307A": "Samsung Electronics Co.,Ltd",
    "00A0B3": "ZYKRONIX",
    "384B5B": "ZTRON TECHNOLOGY LIMITED",
    "34E3DA": "Hoval Aktiengesellschaft",
    "D87E76": "ITEL MOBILE LIMITED",
    "200A0D": "IEEE Registration Authority",
    "2CE310": "Stratacache",
    "6819AC": "Guangzhou Xianyou Intelligent Technogoly CO., LTD",
    "E82E0C": "NETINT Technologies Inc.",
    "1892A4": "Ciena Corporation",
    "14115D": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "B0735D": "Huawei Device Co., Ltd.",
    "F0B4D2": "D-Link International",
    "5C3A45": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "A03C31": "Shenzhen Belon Technology CO.,LTD",
    "E4F327": "ATOL LLC",
    "404C77": "ARRIS Group, Inc.",
    "A897CD": "ARRIS Group, Inc.",
    "10082C": "Texas Instruments",
    "6802B8": "Compal Broadband Networks, Inc.",
    "3463D4": "BIONIX SUPPLYCHAIN TECHNOLOGIES SLU",
    "08F7E9": "HRCP Research and Development Partnership",
    "502DBB": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "3C22FB": "Apple, Inc.",
    "D49E3B": "Guangzhou\u00a0Shiyuan\u00a0Electronic\u00a0Technology\u00a0Company\u00a0Limited",
    "DC4BDD": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "C0B5CD": "Huawei Device Co., Ltd.",
    "4C5077": "Huawei Device Co., Ltd.",
    "30FD65": "HUAWEI TECHNOLOGIES CO.,LTD",
    "58EAFC": "ELL-IoT Inc",
    "9013DA": "Athom B.V.",
    "DCDCE2": "Samsung Electronics Co.,Ltd",
    "A0AC69": "Samsung Electronics Co.,Ltd",
    "1089FB": "Samsung Electronics Co.,Ltd",
    "5C3A3D": "zte corporation",
    "7CA1AE": "Apple, Inc.",
    "80E455": "New H3C Technologies Co., Ltd",
    "00909E": "Critical IO, LLC",
    "ACBD70": "Huawei Device Co., Ltd.",
    "90B8E0": "SHENZHEN YANRAY TECHNOLOGY CO.,LTD",
    "2C4CC6": "Murata Manufacturing Co., Ltd.",
    "38E8EE": "Nanjing Youkuo Electric Technology Co., Ltd",
    "440377": "IEEE Registration Authority",
    "0812A5": "Amazon Technologies Inc.",
    "9CF531": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C809A8": "Intel Corporate",
    "748B34": "Shanghai Smart System Technology Co., Ltd",
    "4CBC72": "Primex Wireless",
    "D4772B": "Nanjing Ztlink Network Technology Co.,Ltd",
    "64F9C0": "ANALOG DEVICES",
    "BCA511": "NETGEAR",
    "F8C4F3": "Shanghai Infinity Wireless Technologies Co.,Ltd.",
    "C4E90A": "D-Link International",
    "18D0C5": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "30E98E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4447D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "34ED1B": "Cisco Systems, Inc",
    "142475": "4DReplay, Inc",
    "80647A": "Ola Sense Inc",
    "C0B883": "Intel Corporate",
    "70F82B": "DWnet Technologies(Suzhou) Corporation",
    "44CB8B": "LG Innotek",
    "EC1BBD": "Silicon Laboratories",
    "10DCB6": "IEEE Registration Authority",
    "D8A315": "vivo Mobile Communication Co., Ltd.",
    "14A1BF": "ASSA ABLOY Korea Co., Ltd Unilock",
    "9483C4": "GL Technologies (Hong Kong) Limited",
    "1CEA0B": "Edgecore Networks Corporation",
    "24418C": "Intel Corporate",
    "44EFBF": "China Dragon Technology Limited",
    "4CB44A": "NANOWAVE Technologies Inc.",
    "F8D027": "Seiko Epson Corporation",
    "5C666C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "4C4BF9": "IEEE Registration Authority",
    "9C93B0": "Megatronix (Beijing) Technology Co., Ltd.",
    "64AEF1": "Qingdao Hisense Electronics Co.,Ltd.",
    "048C9A": "Huawei Device Co., Ltd.",
    "EC3CBB": "Huawei Device Co., Ltd.",
    "28BD89": "Google, Inc.",
    "984827": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "70441C": "SHENZHEN KAIFA TECHNOLOGY CO.,LTD.",
    "ACE342": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9017C8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88D5A8": "ITEL MOBILE LIMITED",
    "B81F5E": "Apption Labs Limited",
    "D81265": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "8C3A7E": "Universal Electronics, Inc.",
    "208593": "IEEE Registration Authority",
    "E4922A": "DBG HOLDINGS LIMITED",
    "2C641F": "Vizio, Inc",
    "54E7D5": "Sun Cupid Technology (HK) LTD",
    "B47C59": "Jiangsu Hengxin Technology Co.,Ltd.",
    "300D9E": "Ruijie Networks Co.,LTD",
    "BCB0E7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5434EF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ECFA5C": "Beijing Xiaomi Electronics Co., Ltd.",
    "F8B46A": "Hewlett Packard",
    "4801C5": "OnePlus Technology (Shenzhen) Co., Ltd",
    "18BF1C": "Jiangsu Huitong Group Co.,Ltd.",
    "207759": "OPTICAL NETWORK VIDEO TECHNOLOGIES (SHENZHEN) CO., LTD.",
    "889D98": "Allied-telesisK.K.",
    "4C56DF": "Targus US LLC",
    "B4EE25": "Shenzhen Belon Technology CO.,LTD",
    "C82B96": "Espressif Inc.",
    "DCF8B9": "zte corporation",
    "189088": "eero inc.",
    "241510": "IEEE Registration Authority",
    "6C4D51": "Shenzhen Ceres Technology Co., Ltd.",
    "98523D": "Sunitec Enterprise Co.,Ltd",
    "000163": "Cisco Systems, Inc",
    "E00084": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04819B": "SKY UK LIMITED",
    "2CA89C": "Creatz inc.",
    "4CDC0D": "Coral Telecom Limited",
    "C4E1A1": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "ACC358": "Continental Automotive Czech Republic s.r.o.",
    "F09919": "Garmin International",
    "1855E3": "Apple, Inc.",
    "E450EB": "Apple, Inc.",
    "886440": "Apple, Inc.",
    "6070C0": "Apple, Inc.",
    "F0C371": "Apple, Inc.",
    "60634C": "D-Link International",
    "683489": "LEA Professional",
    "94BF80": "zte corporation",
    "0000DE": "CETIA",
    "F43E66": "Bee Computing (HK) Limited",
    "DC396F": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "B4C476": "Wuhan Maritime Communication Research Institute",
    "9C3A9A": "Shenzhen Sundray Technologies Company Limited",
    "B46C47": "Panasonic Appliances Company",
    "64FF0A": "Wistron Neweb Corporation",
    "44422F": "TESTOP CO.,LTD.",
    "549C27": "Plasma Cloud Limited",
    "541589": "MCS Logic Inc.",
    "845733": "Microsoft Corporation",
    "3CECEF": "Super Micro Computer, Inc.",
    "987A14": "Microsoft Corporation",
    "C83DDC": "Xiaomi Communications Co Ltd",
    "B0B5E8": "Ruroc LTD",
    "58F39C": "Cisco Systems, Inc",
    "002423": "AzureWave Technologies (Shanghai) Inc.",
    "8C593C": "IEEE Registration Authority",
    "6029D5": "DAVOLINK Inc.",
    "509744": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "0077E4": "Nokia Solutions and Networks GmbH & Co. KG",
    "380118": "ULVAC,Inc.",
    "14ADCA": "China Mobile Iot Limited company",
    "00AD63": "Dedicated Micros Malta LTD",
    "F41D6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CEC9B": "Fuzhou Teraway Information Technology Co.,Ltd",
    "CC9070": "Cisco Systems, Inc",
    "2841C6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E415F6": "Texas Instruments",
    "E828C1": "Eltex Enterprise Ltd.",
    "78D347": "Ericsson AB",
    "F82387": "Shenzhen Horn Audio Co.,Ltd.",
    "809133": "AzureWave Technology Inc.",
    "BC98DF": "Motorola Mobility LLC, a Lenovo Company",
    "70FC8F": "FREEBOX SAS",
    "501B32": "Taicang T&W Electronics",
    "1819D6": "Samsung Electronics Co.,Ltd",
    "AC4228": "Parta Networks",
    "B4F58E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C48FC1": "DEEPTRACK S.L.U.",
    "B0A6F5": "Xaptum, Inc.",
    "ACF5E6": "Cisco Systems, Inc",
    "D4D252": "Intel Corporate",
    "58A023": "Intel Corporate",
    "DCB082": "Nokia",
    "F8C397": "NZXT Corp. Ltd.",
    "70DDA8": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "4C6F9C": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "782C29": "New H3C Technologies Co., Ltd",
    "A4A179": "Nanjing dianyan electric power automation co. LTD",
    "68DB67": "Nantong Coship Electronics Co., Ltd.",
    "980D67": "Zyxel Communications Corporation",
    "702E80": "DIEHL Connectivity Solutions",
    "ACA46E": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "C8B422": "ASKEY COMPUTER CORP",
    "94EE9F": "HMD Global Oy",
    "F4323D": "Sichuan tianyi kanghe communications co., LTD",
    "109397": "ARRIS Group, Inc.",
    "5075F1": "ARRIS Group, Inc.",
    "001EA3": "Nokia Danmark A/S",
    "38F32E": "Skullcandy",
    "DC2AA1": "MedHab LLC",
    "E4F3E8": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "F8B797": "NEC Platforms, Ltd.",
    "B0AAD2": "Sichuan tianyi kanghe communications co., LTD",
    "CCA12B": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "C46516": "Hewlett Packard",
    "E41E0A": "IEEE Registration Authority",
    "AC00D0": "zte corporation",
    "E8C417": "Fiberhome Telecommunication Technologies Co.,LTD",
    "243154": "HUAWEI TECHNOLOGIES CO.,LTD",
    "981E19": "Sagemcom Broadband SAS",
    "84B866": "Beijing XiaoLu technology co. LTD",
    "1422DB": "eero inc.",
    "E8ECA3": "Dongguan Liesheng Electronic Co.Ltd",
    "08A6BC": "Amazon Technologies Inc.",
    "2C58E8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18BC5A": "Zhejiang Tmall Technology Co., Ltd.",
    "C4C138": "OWLink Technology Inc",
    "AC37C9": "RAID Incorporated",
    "205869": "Ruckus Wireless",
    "CC37AB": "Edgecore Networks Corporation",
    "907841": "Intel Corporate",
    "C86314": "IEEE Registration Authority",
    "1CB3E9": " Shenzhen Zhongke United Communication Technology ",
    "84B8B8": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "D041C9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E8018D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "18399C": "Skorpios Technologies",
    "94C2BD": "TECNOBIT",
    "4883B4": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "701E68": "Hanna Instruments, Inc.",
    "C0074A": "Brita GmbH",
    "E8B2FE": "HUMAX Co., Ltd.",
    "0017FA": "Microsoft Corporation",
    "10A3B8": "Iskratel d.o.o.",
    "70CD91": "TERACOM TELEMATICA S.A",
    "941625": "Apple, Inc.",
    "543B30": "duagon AG",
    "8C965F": "Shandong Zhongan Technology Co., Ltd.",
    "B0BB8B": "WAVETEL TECHNOLOGY LIMITED",
    "34A8EB": "Apple, Inc.",
    "A483E7": "Apple, Inc.",
    "F4AFE7": "Apple, Inc.",
    "AC88FD": "Apple, Inc.",
    "6489F1": "Samsung Electronics Co.,Ltd",
    "503E7C": "LeiShen Intelligent  System Co.Ltd",
    "243F30": "Oxygen Broadband s.a.",
    "3C9180": "Liteon Technology Corporation",
    "20326C": "Samsung Electronics Co.,Ltd",
    "80FD7A": "BLU Products Inc",
    "B4A305": "XIAMEN YAXON NETWORK CO., LTD.",
    "803E48": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "24586E": "zte corporation",
    "94BFC4": "Ruckus Wireless",
    "2034FB": "Xiaomi Communications Co Ltd",
    "A89CED": "Xiaomi Communications Co Ltd",
    "6CA604": "ARRIS Group, Inc.",
    "D0EC35": "Cisco Systems, Inc",
    "00124E": "XAC AUTOMATION CORP.",
    "88E034": "Shinwa industries(China) ltd.",
    "BCCF4F": "Zyxel Communications Corporation",
    "0CE041": "iDruide",
    "B88FB4": "JABIL CIRCUIT ITALIA S.R.L",
    "C010B1": "HMD Global Oy",
    "0052C2": "peiker acustic GmbH",
    "90895F": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "48D845": "Shenzhen Mainuoke Electronics Co., Ltd",
    "E458E7": "Samsung Electronics Co.,Ltd",
    "00104A": "The Parvus Corporation",
    "380025": "Intel Corporate",
    "48C3B0": "Pharos Co.Ltd",
    "DC58BC": "Thomas-Krenn.AG",
    "001025": "Grayhill, Inc",
    "70EA1A": "Cisco Systems, Inc",
    "808A8B": "vivo Mobile Communication Co., Ltd.",
    "0CF475": "Zliide Technologies ApS",
    "68FF7B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "808F1D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "48BD0E": "Quanta Storage Inc.",
    "000F69": "SEW Eurodrive GmbH & Co. KG",
    "8C53D2": "China Mobile Group Device Co.,Ltd.",
    "D058C0": "Qingdao Haier Multimedia Limited. ",
    "F8D478": "Flextronics Tech.(Ind) Pvt Ltd",
    "D45383": "Murata Manufacturing Co., Ltd.",
    "A04246": "IT Telecom Co., Ltd.",
    "DC6723": "barox Kommunikation GmbH",
    "44B462": "Flextronics Tech.(Ind) Pvt Ltd",
    "A45F9B": "Nexell",
    "1C3B8F": "Selve GmbH & Co. KG",
    "E4E749": "Hewlett Packard",
    "9844B6": "INFRANOR SAS",
    "38839A": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "C8F6C8": "Fiberhome Telecommunication Technologies Co.,LTD",
    "ECC57F": "Suzhou Pairlink Network Technology",
    "A86DAA": "Intel Corporate",
    "38C2BA": "CCTV NEOTECH",
    "A0F9B7": "Ademco Smart Homes Technology(Tianjin)Co.,Ltd.",
    "A83CCB": "ROSSMA",
    "CC3FEA": "BAE Systems, Inc",
    "E85BB7": "Ample Systems Inc.",
    "94677E": "Belden India Private Limited",
    "AC4330": "Versa Networks",
    "D43A2E": "SHENZHEN MTC CO LTD",
    "30C3D9": "ALPSALPINE CO,.LTD",
    "AC5775": "HMD Global Oy",
    "50AD92": "NX Technologies",
    "4CF2BF": "Cambridge Industries(Group) Co.,Ltd.",
    "CC9EA2": "Amazon Technologies Inc.",
    "003217": "Cisco Systems, Inc",
    "001BFB": "ALPSALPINE CO,.LTD",
    "8CAEDB": "NAGTECH LLC",
    "78B213": "DWnet Technologies(Suzhou) Corporation",
    "7CDB98": "ASKEY COMPUTER CORP",
    "F00DF5": "ACOMA Medical Industry Co,. Ltd.",
    "58C232": "NEC Corporation",
    "381D14": "Skydio Inc.",
    "74F737": "KCE",
    "48A493": "TAIYO YUDEN CO.,LTD",
    "88D211": "Eko Devices, Inc.",
    "B8C227": "PSTec",
    "3C286D": "Google, Inc.",
    "00093A": "Molex CMS",
    "6CA936": "DisplayLink (UK) Ltd",
    "708540": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "94E0D6": "China Dragon Technology Limited",
    "B4A9FC": "Quanta Computer Inc.",
    "380B3C": "Texas Instruments",
    "6845F1": "TOSHIBA CLIENT SOLUTIONS CO., LTD.",
    "B40B78": "Brusa Elektronik AG",
    "207918": "Intel Corporate",
    "A48CC0": "JLG Industries, Inc.",
    "DC48B2": "Baraja Pty. Ltd.",
    "ACAE19": "Roku, Inc",
    "0C9541": "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.",
    "14D00D": "Apple, Inc.",
    "C03DD9": "MitraStar Technology Corp.",
    "2CAA8E": "Wyze Labs Inc",
    "703A51": "Xiaomi Communications Co Ltd",
    "9C6937": "Qorvo International Pte. Ltd.",
    "E82C6D": "SmartRG, Inc.",
    "04F9D9": "Speaker Electronic(Jiashan) Co.,Ltd",
    "181E95": "AuVerte",
    "DC080F": "Apple, Inc.",
    "F8E94E": "Apple, Inc.",
    "EC2CE2": "Apple, Inc.",
    "40BC60": "Apple, Inc.",
    "E83617": "Apple, Inc.",
    "9C648B": "Apple, Inc.",
    "344262": "Apple, Inc.",
    "48E695": "Insigma Inc",
    "B479C8": "Ruckus Wireless",
    "706D15": "Cisco Systems, Inc",
    "A4A1E4": "Innotube, Inc.",
    "001060": "BILLIONTON SYSTEMS, INC.",
    "C4D489": "JiangSu Joyque Information Industry Co.,Ltd",
    "B82CA0": "Resideo",
    "B4F949": "optilink networks pvt ltd",
    "48352E": "Shenzhen Wolck Network Product Co.,LTD",
    "6CC7EC": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "D89685": "GoPro",
    "C0BDC8": "Samsung Electronics Co.,Ltd",
    "647BCE": "Samsung Electronics Co.,Ltd",
    "A887B3": "Samsung Electronics Co.,Ltd",
    "6C006B": "Samsung Electronics Co.,Ltd",
    "3C01EF": "Sony Corporation",
    "04E598": "Xiaomi Communications Co Ltd",
    "3C3786": "NETGEAR",
    "98D3E7": "Netafim L",
    "F063F9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CC385": "HUAWEI TECHNOLOGIES CO.,LTD",
    "900EB3": "Shenzhen Amediatech Technology Co., Ltd.",
    "00AD24": "D-Link International",
    "1CFD08": "IEEE Registration Authority",
    "B8599F": "Mellanox Technologies, Inc.",
    "301389": "Siemens AG, Automations & Drives,",
    "A0A4C5": "Intel Corporate",
    "105BAD": "Mega Well Limited",
    "94EAEA": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "F4D108": "Intel Corporate",
    "60CE92": "The Refined Industry Company Limited",
    "74BFC0": "CANON INC.",
    "C8D9D2": "Hewlett Packard",
    "24FCE5": "Samsung Electronics Co.,Ltd",
    "809621": "Lenovo",
    "78055F": "Shenzhen WYC Technology Co., Ltd.",
    "1862E4": "Texas Instruments",
    "20B001": "Technicolor Delivery Technologies Belgium NV",
    "F05494": "Honeywell Connected Building",
    "54068B": "Ningbo Deli Kebei Technology Co.LTD",
    "549FAE": "iBASE Gaming Inc",
    "B02A43": "Google, Inc.",
    "181DEA": "Intel Corporate",
    "185680": "Intel Corporate",
    "C474F8": "Hot Pepper, Inc.",
    "48872D": "SHEN ZHEN DA XIA LONG QUE TECHNOLOGY CO.,LTD",
    "E81A58": "TECHNOLOGIC SYSTEMS",
    "00EABD": "Cisco Systems, Inc",
    "5CC999": "New H3C Technologies Co., Ltd",
    "EC79F2": "Startel",
    "04BC87": "Shenzhen JustLink Technology Co., LTD",
    "54C33E": "Ciena Corporation",
    "142233": "Fiberhome Telecommunication Technologies Co.,LTD",
    "BCB22B": "EM-Tech",
    "DC3757": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "005099": "3COM EUROPE LTD",
    "D4AB82": "ARRIS Group, Inc.",
    "704FB8": "ARRIS Group, Inc.",
    "0060EB": "FOURTHTRACK SYSTEMS",
    "743400": "MTG Co., Ltd.",
    "6035C0": "SFR",
    "001AC5": "Keysight Technologies, Inc.",
    "00201E": "NETQUEST CORPORATION",
    "00608C": "3COM",
    "00A024": "3COM",
    "0020AF": "3COM",
    "00104B": "3COM",
    "B08BCF": "Cisco Systems, Inc",
    "C4985C": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "30A1FA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "645D86": "Intel Corporate",
    "64628A": "evon GmbH",
    "0415D9": "Viwone",
    "9CAA1B": "Microsoft Corporation",
    "A89A93": "Sagemcom Broadband SAS",
    "8C9246": "Oerlikon Textile Gmbh&Co.KG",
    "ECB313": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "242E90": "PALIT MICROSYSTEMS, LTD",
    "F4068D": "devolo AG",
    "988ED4": "ITEL MOBILE LIMITED",
    "E8A788": "XIAMEN LEELEN TECHNOLOGY CO., LTD",
    "0C9D92": "ASUSTek COMPUTER INC.",
    "0CCB85": "Motorola Mobility LLC, a Lenovo Company",
    "4017E2": "INTAI TECHNOLOGY CORP.",
    "4898CA": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "247E51": "zte corporation",
    "E8B541": "zte corporation",
    "0C96E6": "Cloud Network Technology (Samoa) Limited",
    "3C8994": "SKY UK LIMITED",
    "582D34": "Qingping Electronics (Suzhou) Co., Ltd",
    "20DE88": "IC Realtime LLC",
    "A85AF3": "Shanghai Siflower Communication Technology Co., Ltd",
    "70FD46": "Samsung Electronics Co.,Ltd",
    "8C83E1": "Samsung Electronics Co.,Ltd",
    "889F6F": "Samsung Electronics Co.,Ltd",
    "5C63C9": "Intellithings Ltd.",
    "000E94": "Maas International BV",
    "000C43": "Ralink Technology, Corp.",
    "001A31": "SCAN COIN AB",
    "001B84": "Scan Engineering Telecom",
    "E00EE1": "We Corporation Inc.",
    "482CA0": "Xiaomi Communications Co Ltd",
    "A4E615": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "A09351": "Cisco Systems, Inc",
    "88AE1D": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "B888E3": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "3412F9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BCE265": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CD1A1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88BFE4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "741F79": "YOUNGKOOK ELECTRONICS CO.,LTD",
    "208984": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "B4CEFE": "James Czekaj",
    "F8CC6E": "DEPO Electronics Ltd",
    "F8369B": "Texas Instruments",
    "00C055": "MODULAR COMPUTING TECHNOLOGIES",
    "E41FE9": "Dunkermotoren GmbH",
    "D8760A": "Escort, Inc.",
    "5C2ED2": "ABC(XiSheng) Electronics Co.,Ltd",
    "20163D": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "641331": "Bosch Car Multimedia (Wuhu) Co. Ltd.",
    "183A48": "VostroNet",
    "782F17": "Xlab Co.,Ltd",
    "B0027E": "MULLER SERVICES",
    "24FAF3": "Shanghai Flexem Technology Co.,Ltd.",
    "C4518D": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "486834": "Silicon Motion, Inc.",
    "001133": "Siemens AG Austria",
    "000B23": "Siemens Home & Office Comm. Devices",
    "340A98": "HUAWEI TECHNOLOGIES CO.,LTD",
    "646D6C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4B8B4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "289EFC": "Sagemcom Broadband SAS",
    "E0735F": "NUCOM",
    "0051ED": "LG Innotek",
    "98039B": "Mellanox Technologies, Inc.",
    "40DC9D": "HAJEN",
    "F0BCC9": "PFU LIMITED",
    "88D2BF": "German Autolabs",
    "8C3579": "QDIQO Sp. z o.o.",
    "38C70A": "WiFiSong",
    "D82477": "Universal Electric Corporation",
    "00907F": "WatchGuard Technologies, Inc.",
    "4C5E0C": "Routerboard.com",
    "D4CA6D": "Routerboard.com",
    "FCFBFB": "Cisco Systems, Inc",
    "007E95": "Cisco Systems, Inc",
    "08D46A": "LG Electronics (Mobile Communications)",
    "4006A0": "Texas Instruments",
    "487583": "Intellion AG",
    "9C5A44": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "14CAA0": "Hu&Co",
    "80B575": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4BE2B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2811A5": "Bose Corporation",
    "D8F3DB": "Post CH AG",
    "DCB4AC": "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD.",
    "F4EE14": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "6C5940": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "D02516": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "1C60DE": "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "001472": "China Broadband Wireless IP Standard group(ChinaBWIPS)",
    "A45385": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "00402F": "XLNT DESIGNS INC.",
    "04ECBB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "001017": "Bosch Access Systems GmbH",
    "D42122": "Sercomm Corporation.",
    "00C002": "Sercomm Corporation.",
    "68E7C2": "Samsung Electronics Co.,Ltd",
    "58B10F": "Samsung Electronics Co.,Ltd",
    "203DBD": "LG Innotek",
    "280245": "Konze System Technology Co.,Ltd.",
    "E48F65": "Yelatma Instrument Making Enterprise, JSC",
    "840D8E": "Espressif Inc.",
    "A492CB": "Nokia",
    "C0D2F3": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "94193A": "Elvaco AB",
    "BC9911": "Zyxel Communications Corporation",
    "3CF5CC": "New H3C Technologies Co., Ltd",
    "00BB3A": "Amazon Technologies Inc.",
    "C08135": "Ningbo Forfan technology Co., LTD",
    "3CF4F9": "Moda-InnoChips",
    "0CB34F": "Shenzhen Xiaoqi Intelligent Technology Co., Ltd.",
    "64A2F9": "OnePlus Technology (Shenzhen) Co., Ltd",
    "A87D12": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F82DC0": "ARRIS Group, Inc.",
    "189C27": "ARRIS Group, Inc.",
    "88D37B": "FirmTek, LLC",
    "B4C0F5": "Shenzhen TINNO Mobile Technology Corp.",
    "406231": "GIFA",
    "FCB7F0": "Idaho National Laboratory",
    "5050CE": "Hangzhou Dianyixia Communication Technology Co. Ltd.",
    "2C28B7": "Hangzhou Ruiying technology co., LTD",
    "046B1B": "SYSDINE Co., Ltd.",
    "E8FAF7": "Guangdong Uniteddata Holding Group Co., Ltd.",
    "949D57": "Panasonic do Brasil Limitada",
    "FC6947": "Texas Instruments",
    "1C666D": "Hon Hai Precision Ind. Co.,Ltd.",
    "DCDE4F": "Gionee Communication Equipment Co Ltd ",
    "4CD0CB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C24F0": "IEEE Registration Authority",
    "58DB15": "TECNO MOBILE LIMITED",
    "002082": "ONEAC CORPORATION",
    "0000A8": "Stratus Technologies",
    "0004FC": "Stratus Technologies",
    "E07DEA": "Texas Instruments",
    "505DAC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0A37E": "QING DAO HAIER TELECOM CO.,LTD.",
    "04D3B0": "Intel Corporate",
    "3CEAF9": "JUBIXCOLTD",
    "682C7B": "Cisco Systems, Inc",
    "441E98": "Ruckus Wireless",
    "00250C": "Senet Inc",
    "0C8063": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "007278": "Cisco Systems, Inc",
    "00A021": "General Dynamics Mission Systems",
    "90DD5D": "Apple, Inc.",
    "DC2919": "AltoBeam (Xiamen) Technology Ltd, Co.",
    "645AED": "Apple, Inc.",
    "84F3EB": "Espressif Inc.",
    "001B48": "Shenzhen Lantech Electronics Co., Ltd.",
    "50BC96": "Apple, Inc.",
    "FC2A9C": "Apple, Inc.",
    "A056F3": "Apple, Inc.",
    "549963": "Apple, Inc.",
    "C0B658": "Apple, Inc.",
    "48A91C": "Apple, Inc.",
    "002FD9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "885FE8": "IEEE Registration Authority",
    "F0AF50": "Phantom Intelligence",
    "B4CD27": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C819F7": "Samsung Electronics Co.,Ltd",
    "C42C4F": "Qingdao Hisense Mobile Communication Technology Co,Ltd",
    "24CACB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "543E64": "Fiberhome Telecommunication Technologies Co.,LTD",
    "6402CB": "ARRIS Group, Inc.",
    "3880DF": "Motorola Mobility LLC, a Lenovo Company",
    "BC6A2F": "Henge Docks LLC",
    "48BD3D": "New H3C Technologies Co., Ltd",
    "0C08B4": "HUMAX Co., Ltd.",
    "F4E11E": "Texas Instruments",
    "002705": "Sectronic",
    "180F76": "D-Link International",
    "DC0265": "Meditech Kft",
    "909497": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC729B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14A72B": "currentoptronics Pvt.Ltd",
    "3C1710": "Sagemcom Broadband SAS",
    "34029B": "Plexonics Technologies LImited",
    "84DB9E": "Pink Nectarine Health AB",
    "900372": "Longnan Junya Digital Technology Co. Ltd. ",
    "A4DA22": "IEEE Registration Authority",
    "8C4CAD": "Evoluzn Inc.",
    "D4E6B7": "Samsung Electronics Co.,Ltd",
    "4C776D": "Cisco Systems, Inc",
    "3CDCBC": "Samsung Electronics Co.,Ltd",
    "804E70": "Samsung Electronics Co.,Ltd",
    "74E182": "Texas Instruments",
    "E8DEFB": "MESOTIC SAS",
    "94FE9D": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "6CB6CA": "DIVUS GmbH",
    "04D13A": "Xiaomi Communications Co Ltd",
    "8CF957": "RuiXingHengFang Network (Shenzhen) Co.,Ltd",
    "0CC6CC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6CC4D5": "HMD Global Oy",
    "80C548": "Shenzhen Zowee Technology Co.,Ltd",
    "C400AD": "Advantech Technology (CHINA) Co., Ltd.",
    "3499D7": "Universal Flow Monitors, Inc.",
    "0C8BD3": "ITEL MOBILE LIMITED",
    "0024AF": "Dish Technologies Corp",
    "C0A8F0": "Adamson Systems Engineering",
    "9C431E": "IEEE Registration Authority",
    "4CC206": "Somfy",
    "785860": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8ABF3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "449EF9": "vivo Mobile Communication Co., Ltd.",
    "B4E9A3": "port industrial automation GmbH",
    "6C2ACB": "Paxton Access Ltd",
    "980074": "Raisecom Technology CO., LTD",
    "18C19D": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "0C9838": "Xiaomi Communications Co Ltd",
    "282C02": "IEEE Registration Authority",
    "583BD9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "DCA266": "Hon Hai Precision Ind. Co.,Ltd.",
    "C48466": "Apple, Inc.",
    "347C25": "Apple, Inc.",
    "CC2DB7": "Apple, Inc.",
    "A0BDCD": "SKY UK LIMITED",
    "FCA183": "Amazon Technologies Inc.",
    "74EACB": "New H3C Technologies Co., Ltd",
    "BC91B5": "Infinix mobility limited",
    "D41A3F": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "30C507": "ECI Telecom Ltd.",
    "282FC2": "Automotive Data Solutions",
    "E88E60": "NSD Corporation",
    "000F9B": "Ross Video Limited",
    "0024BA": "Texas Instruments",
    "60512C": "TCT mobile ltd",
    "64DB81": "Syszone Co., Ltd.",
    "44AD19": "XINGFEI \uff08H.K\uff09LIMITED ",
    "38ADBE": "New H3C Technologies Co., Ltd",
    "04B167": "Xiaomi Communications Co Ltd",
    "EC8350": "Microsoft Corporation",
    "5C865C": "Samsung Electronics Co.,Ltd",
    "5CAD76": "Shenzhen TCL New Technology Co., Ltd",
    "646E6C": "Radio Datacom LLC",
    "04F128": "HMD Global Oy",
    "0C5203": "AGM GROUP LIMITED",
    "2C5491": "Microsoft Corporation",
    "E4A7A0": "Intel Corporate",
    "C8458F": "Wyler AG",
    "001BB9": "Elitegroup Computer Systems Co.,Ltd.",
    "002461": "Shin Wang Tech.",
    "1CDF52": "Texas Instruments",
    "001E1D": "East Coast Datacom, Inc.",
    "5CE28C": "Zyxel Communications Corporation",
    "E4BD4B": "zte corporation",
    "3C2EF9": "Apple, Inc.",
    "7C010A": "Texas Instruments",
    "000144": "Dell EMC",
    "08001B": "Dell EMC",
    "38D7CA": "7HUGS LABS",
    "A04EA7": "Apple, Inc.",
    "F0989D": "Apple, Inc.",
    "E42B34": "Apple, Inc.",
    "7846C4": "DAEHAP HYPER-TECH",
    "FCD6BD": "Robert Bosch GmbH",
    "18396E": "SUNSEA TELECOMMUNICATIONS CO.,LTD.",
    "EC7D11": "vivo Mobile Communication Co., Ltd.",
    "D80831": "Samsung Electronics Co.,Ltd",
    "701F53": "Cisco Systems, Inc",
    "9441C1": "Mini-Cam Limited",
    "48BA4E": "Hewlett Packard",
    "500F80": "Cisco Systems, Inc",
    "504EDC": "Ping Communication",
    "08674E": "Hisense broadband multimedia technology Co.,Ltd",
    "8C68C8": "zte corporation",
    "EC8263": "zte corporation",
    "74BBD3": "Shenzhen xeme Communication Co., Ltd.",
    "683C7D": "Magic Intelligence Technology Limited",
    "0C1C20": "Kakao Corp",
    "94282E": "New H3C Technologies Co., Ltd",
    "10F1F2": "LG Electronics (Mobile Communications)",
    "8C3C4A": "NAKAYO Inc",
    "6CB749": "HUAWEI TECHNOLOGIES CO.,LTD",
    "989C57": "HUAWEI TECHNOLOGIES CO.,LTD",
    "185282": "Fiberhome Telecommunication Technologies Co.,LTD",
    "7086C1": "Texas Instruments",
    "DC6AEA": "Infinix mobility limited",
    "98F5A9": "OHSUNG",
    "D86162": "Wistron Neweb Corporation",
    "F4D7B2": "LGS Innovations, LLC",
    "00152A": "Nokia Corporation",
    "A41115": "Robert Bosch Engineering and Business Solutions pvt. Ltd.",
    "28840E": "silicon valley immigration service ",
    "80615F": "Beijing Sinead Technology Co., Ltd. ",
    "444AB0": "Zhejiang Moorgen Intelligence Technology Co., Ltd",
    "B019C6": "Apple, Inc.",
    "9C4A7B": "Nokia Corporation",
    "2CD2E7": "Nokia Corporation",
    "386EA2": "vivo Mobile Communication Co., Ltd.",
    "982D68": "Samsung Electronics Co., Ltd",
    "BC2E48": "ARRIS Group, Inc.",
    "608CE6": "ARRIS Group, Inc.",
    "080070": "Mitsubishi Precision Co.,LTd.",
    "48EC5B": "Nokia Solutions and Networks GmbH & Co. KG",
    "C421C8": "KYOCERA CORPORATION ",
    "80739F": "KYOCERA CORPORATION ",
    "3866F0": "Apple, Inc.",
    "705812": "Panasonic Corporation AVC Networks Company",
    "04209A": "Panasonic Corporation AVC Networks Company",
    "34008A": "IEEE Registration Authority",
    "9050CA": "Hitron Technologies. Inc",
    "409922": "AzureWave Technology Inc.",
    "C06D1A": "Tianjin Henxinhuifeng Technology Co.,Ltd.",
    "107B44": "ASUSTek COMPUTER INC.",
    "1CAB34": "New H3C Technologies Co., Ltd",
    "3C7843": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C0979": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4FB5D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E86819": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC512C": "Infinix mobility limited",
    "0030C8": "GAD LINE, LTD.",
    "0016E0": "3Com Ltd",
    "40D63C": "Equitech Industrial(DongGuan)Co.,Ltd",
    "F4F3AA": "JBL GmbH & Co. KG",
    "40A3CC": "Intel Corporate",
    "D8DECE": "ISUNG CO.,LTD",
    "801934": "Intel Corporate",
    "703EAC": "Apple, Inc.",
    "0011C0": "Aday Technology Inc",
    "0005F1": "Vrcom, Inc.",
    "84253F": "silex technology, Inc.",
    "0008C9": "TechniSat Digital GmbH Daun",
    "D8B12A": "Panasonic Mobile Communications Co.,Ltd.",
    "8C2505": "HUAWEI TECHNOLOGIES CO.,LTD",
    "08A8A1": "Cyclotronics Power Concepts, Inc",
    "F4B520": "Biostar Microtech international corp.",
    "008009": "JUPITER SYSTEMS, INC.",
    "00C064": "General Datacomm LLC",
    "30C01B": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "8C3BAD": "NETGEAR",
    "64CFD9": "Texas Instruments",
    "747D24": "Phicomm (Shanghai) Co., Ltd.",
    "50642B": "XIAOMI Electronics,CO.,LTD",
    "887A31": "Velankani Electronics Pvt. Ltd.",
    "8C0F6F": "PEGATRON CORPORATION",
    "309935": "zte corporation",
    "0C72D9": "zte corporation",
    "6432A8": "Intel Corporate",
    "8886C2": "STABILO International GmbH",
    "CC2F71": "Intel Corporate",
    "B8F8BE": "BLUECOM",
    "2C5D93": "Ruckus Wireless",
    "38FF36": "Ruckus Wireless",
    "84183A": "Ruckus Wireless",
    "A47B9D": "Espressif Inc.",
    "7C2EDD": "Samsung Electronics Co.,Ltd",
    "3CF7A4": "Samsung Electronics Co.,Ltd",
    "0000FE": "Annapolis Micro Systems, Inc.",
    "38E595": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "BC9680": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "24C9A1": "Ruckus Wireless",
    "002482": "Ruckus Wireless",
    "689234": "Ruckus Wireless",
    "50A733": "Ruckus Wireless",
    "103034": "Cara Systems",
    "00D01F": "Senetas Corporation Ltd",
    "4C65A8": "IEEE Registration Authority",
    "D822F4": "Avnet Silica",
    "348F27": "Ruckus Wireless",
    "2C9EEC": "Jabil Circuit Penang",
    "706E6D": "Cisco Systems, Inc",
    "E86D65": "AUDIO MOBIL Elektronik GmbH",
    "6447E0": "Feitian Technologies Co., Ltd",
    "B44F96": "Zhejiang Xinzailing Technology co., ltd",
    "B0DFC1": "Tenda Technology Co.,Ltd.Dongguan branch",
    "9C6F52": "zte corporation",
    "604762": "Beijing Sensoro Technology Co.,Ltd.",
    "986F60": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "BC1C81": "Sichuan iLink Technology Co., Ltd.",
    "900A1A": "Taicang T&W Electronics",
    "506E92": "Innocent Technology Co., Ltd.",
    "30FE31": "Nokia",
    "C4571F": "June Life Inc",
    "886AE3": "Alpha Networks Inc.",
    "1C4D70": "Intel Corporate",
    "001CFA": "Alarm.com",
    "60313B": "Sunnovo International Limited",
    "6CB227": "Sony Video & Sound Products Inc.",
    "00A3D1": "Cisco Systems, Inc",
    "488D36": "Arcadyan Corporation",
    "E8E1E1": "Gemtek Technology Co., Ltd.",
    "28070D": "GUANGZHOU WINSOUND INFORMATION TECHNOLOGY CO.,LTD.",
    "FC4D8C": "SHENZHEN PANTE ELECTRONICS TECHNOLOGY CO., LTD",
    "FC06ED": "M2Motive Technology Inc.",
    "BCD713": "Owl Labs",
    "000CAB": "Commend International GmbH",
    "745427": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "60720B": "BLU Products Inc",
    "308976": "DALIAN LAMBA TECHNOLOGY CO.,LTD",
    "2C2617": "Oculus VR, LLC",
    "10C6FC": "Garmin International",
    "AC2205": "Compal Broadband Networks, Inc.",
    "80A036": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "F07485": "NGD Systems, Inc.",
    "34D954": "WiBotic Inc.",
    "4857DD": "Facebook Inc",
    "487D2E": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F0D4F6": "Lars Thrane A/S",
    "F4A997": "CANON INC.",
    "B0DAF9": "ARRIS Group, Inc.",
    "1835D1": "ARRIS Group, Inc.",
    "64DFE9": "ATEME",
    "A0094C": "CenturyLink",
    "20F452": "Shanghai IUV Software Development Co. Ltd",
    "B43934": "Pen Generations, Inc.",
    "7426AC": "Cisco Systems, Inc",
    "143F27": "Noccela Oy",
    "105887": "Fiberhome Telecommunication Technologies Co.,LTD",
    "B02628": "Broadcom Limited",
    "900E83": "Monico Monitoring, Inc.",
    "601466": "zte corporation",
    "38AC3D": "Nephos Inc",
    "9874DA": "Infinix mobility limited",
    "1C5A0B": "Tegile Systems",
    "046E02": "OpenRTLS Group",
    "9C061B": "Hangzhou H3C Technologies Co., Limited",
    "50338B": "Texas Instruments",
    "6854ED": "Alcatel-Lucent",
    "E037BF": "Wistron Neweb Corporation",
    "AC7409": "Hangzhou H3C Technologies Co., Limited",
    "680235": "Konten Networks Inc.",
    "D8D866": "SHENZHEN TOZED TECHNOLOGIES CO.,LTD.",
    "D8C06A": "Hunantv.com Interactive Entertainment Media Co.,Ltd.",
    "001192": "Cisco Systems, Inc",
    "38F135": "SensorTec-Canada",
    "E8DE8E": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "E81367": "AIRSOUND Inc.",
    "AC202E": "Hitron Technologies. Inc",
    "70AF24": "TP Vision Belgium NV",
    "A41163": "IEEE Registration Authority",
    "7CE97C": "ITEL MOBILE LIMITED",
    "C4D197": "Ventia Utility Services",
    "F49634": "Intel Corporate",
    "F093C5": "Garland Technology",
    "9810E8": "Apple, Inc.",
    "2C86D2": "Cisco Systems, Inc",
    "DCA4CA": "Apple, Inc.",
    "8C8FE9": "Apple, Inc.",
    "C0D012": "Apple, Inc.",
    "BCA920": "Apple, Inc.",
    "48A195": "Apple, Inc.",
    "F80377": "Apple, Inc.",
    "C49DED": "Microsoft Corporation",
    "98A40E": "Snap, Inc.",
    "8058F8": "Motorola Mobility LLC, a Lenovo Company",
    "70AF25": "Nishiyama Industry Co.,LTD.",
    "E8D11B": "ASKEY COMPUTER CORP",
    "0C5F35": "Niagara Video Corporation",
    "9800C1": "GuangZhou CREATOR Technology Co.,Ltd.(CHINA)",
    "CCBE59": "Calix Inc.",
    "903D6B": "Zicon Technology Corp.",
    "B0C46C": "Senseit",
    "98D3D2": "MEKRA Lang GmbH & Co. KG",
    "001912": "Welcat Inc",
    "B8F883": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "DCFE18": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "704F57": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "8C78D7": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "B8EAAA": "ICG NETWORKS CO.,ltd",
    "5CBA37": "Microsoft Corporation",
    "00C0C6": "PERSONAL MEDIA CORP.",
    "3C7F6F": "Telechips, Inc.",
    "F8A34F": "zte corporation",
    "C87324": " Sow Cheng Technology Co. Ltd.",
    "00210D": "SAMSIN INNOTEC",
    "0002A1": "World Wide Packets",
    "00E022": "Analog Devices, Inc.",
    "50E666": "Shenzhen Techtion Electronics Co., Ltd.",
    "0016D3": "Wistron Corporation",
    "001F16": "Wistron Corporation",
    "00262D": "Wistron Corporation",
    "B04089": "Senient Systems LTD",
    "5425EA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C894BB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10B1F8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "089E08": "Google, Inc.",
    "28FECD": "Lemobile Information Technology (Beijing) Co., Ltd.",
    "4C4E03": "TCT mobile ltd",
    "0495E6": "Tenda Technology Co.,Ltd.Dongguan branch",
    "702D84": "i4C Innovations",
    "C0D9F7": "ShanDong Domor Intelligent S&T CO.,Ltd",
    "00608B": "ConferTech International",
    "CCA219": "SHENZHEN ALONG INVESTMENT CO.,LTD",
    "4C1A3A": "PRIMA Research And Production Enterprise Ltd.",
    "2C200B": "Apple, Inc.",
    "8866A5": "Apple, Inc.",
    "901711": "Hagenuk Marinekommunikation GmbH",
    "0010DE": "INTERNATIONAL DATACASTING CORPORATION",
    "38BC01": "HUAWEI TECHNOLOGIES CO.,LTD",
    "341E6B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "886639": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000048": "Seiko Epson Corporation",
    "8CA5A1": "Oregano Systems - Design & Consulting GmbH",
    "B8ECA3": "Zyxel Communications Corporation",
    "B0B98A": "NETGEAR",
    "805A04": "LG Electronics (Mobile Communications)",
    "ACC1EE": "Xiaomi Communications Co Ltd",
    "5419C8": "vivo Mobile Communication Co., Ltd.",
    "000277": "Cash Systemes Industrie",
    "BC8385": "Microsoft Corporation",
    "E4B005": "Beijing IQIYI Science & Technology Co., Ltd.",
    "B05216": "Hon Hai Precision Ind. Co.,Ltd.",
    "B0E892": "Seiko Epson Corporation",
    "AC1826": "Seiko Epson Corporation",
    "A4EE57": "Seiko Epson Corporation",
    "9CAED3": "Seiko Epson Corporation",
    "707C69": "Avaya Inc",
    "500B91": "IEEE Registration Authority",
    "F8461C": "Sony Interactive Entertainment Inc.",
    "14A78B": "Zhejiang Dahua Technology Co., Ltd.",
    "2C598A": "LG Electronics (Mobile Communications)",
    "686975": "Angler Labs Inc",
    "18F87A": "i3 International Inc.",
    "A0E4CB": "Zyxel Communications Corporation",
    "284ED7": "OutSmart Power Systems, Inc.",
    "64A68F": "Zhongshan Readboy Electronics Co.,Ltd",
    "00425A": "Cisco Systems, Inc",
    "704D7B": "ASUSTek COMPUTER INC.",
    "9CFBD5": "vivo Mobile Communication Co., Ltd.",
    "18F76B": "Zhejiang Winsight Technology CO.,LTD",
    "5CE30E": "ARRIS Group, Inc.",
    "4C26E7": "Welgate Co., Ltd.",
    "006041": "Yokogawa Digital Computer Corporation",
    "583112": "DRUST",
    "9C83BF": "PRO-VISION, Inc.",
    "78EF4C": "Unetconvergence Co., Ltd.",
    "00C05A": "SEMAPHORE COMMUNICATIONS CORP.",
    "0007F9": "Sensaphone",
    "001CB3": "Apple, Inc.",
    "20D25F": "SmartCap Technologies",
    "E47DBD": "Samsung Electronics Co.,Ltd",
    "48D343": "ARRIS Group, Inc.",
    "A0B8F8": "Amgen U.S.A. Inc.",
    "884477": "HUAWEI TECHNOLOGIES CO.,LTD",
    "149D09": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C11BF": "Zhejiang Dahua Technology Co., Ltd.",
    "1CC0E1": "IEEE Registration Authority",
    "00B0E1": "Cisco Systems, Inc",
    "005093": "BOEING",
    "C81B5C": "BCTech",
    "28EE52": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "001A39": "Merten GmbH&CoKG",
    "E07C13": "zte corporation",
    "F41F88": "zte corporation",
    "E43ED7": "Arcadyan Corporation",
    "14EDBB": "2Wire Inc",
    "18E29F": "vivo Mobile Communication Co., Ltd.",
    "F07960": "Apple, Inc.",
    "A0D795": "Apple, Inc.",
    "0090E7": "HORSCH ELEKTRONIK AG",
    "905C44": "Compal Broadband Networks, Inc.",
    "00F22C": "Shanghai B-star Technology Co.,Ltd.",
    "842519": "Samsung Electronics",
    "5C2443": "O-Sung Telecom Co., Ltd.",
    "707990": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A04E01": "CENTRAL ENGINEERING co.,ltd.",
    "28CA09": "ThyssenKrupp Elevators (Shanghai) Co.,Ltd",
    "2047ED": "SKY UK LIMITED",
    "748A69": "Korea Image Technology Co., Ltd",
    "24920E": "Samsung Electronics Co.,Ltd",
    "FC4203": "Samsung Electronics Co.,Ltd",
    "A01081": "Samsung Electronics Co.,Ltd",
    "5454CF": "PROBEDIGITAL CO.,LTD",
    "6474F6": "Shooter Detection Systems",
    "3C8BCD": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "4CF95D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8421F1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0005EE": "Vanderbilt International (SWE) AB ",
    "F8633F": "Intel Corporate",
    "088620": "TECNO MOBILE LIMITED",
    "002566": "Samsung Electronics Co.,Ltd",
    "981333": "zte corporation",
    "D0DB32": "Nokia Corporation",
    "E80036": "Befs co,. ltd",
    "C09F05": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "04180F": "Samsung Electronics Co.,Ltd",
    "2013E0": "Samsung Electronics Co.,Ltd",
    "F43C96": "Ericsson AB",
    "388479": "Cisco Meraki",
    "C4411E": "Belkin International Inc.",
    "24F5A2": "Belkin International Inc.",
    "F89E28": "Cisco Meraki",
    "683A1E": "Cisco Meraki",
    "C0F945": "Toshiba Toko Meter Systems Co., LTD.",
    "70F8E7": "IEEE Registration Authority",
    "D42C44": "Cisco Systems, Inc",
    "843DC6": "Cisco Systems, Inc",
    "002485": "ConteXtream Ltd",
    "28FCF6": "Shenzhen Xin KingBrand enterprises Co.,Ltd",
    "001F58": "EMH Energiemesstechnik GmbH",
    "5C4979": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "689423": "Hon Hai Precision Ind. Co.,Ltd.",
    "844BF5": "Hon Hai Precision Ind. Co.,Ltd.",
    "08EDB9": "Hon Hai Precision Ind. Co.,Ltd.",
    "5C6D20": "Hon Hai Precision Ind. Co.,Ltd.",
    "5CAC4C": "Hon Hai Precision Ind. Co.,Ltd.",
    "0016DF": "Lundinova AB",
    "001D0C": "MobileCompia",
    "E0CBBC": "Cisco Meraki",
    "B88EDF": "Zencheer Communication Technology Co., Ltd.",
    "DC7144": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "980C82": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "A00BBA": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "3C77E6": "Hon Hai Precision Ind. Co.,Ltd.",
    "70188B": "Hon Hai Precision Ind. Co.,Ltd.",
    "8CC8CD": "Samsung Electronics Co.,Ltd",
    "8018A7": "Samsung Electronics Co.,Ltd",
    "F47B5E": "Samsung Electronics Co.,Ltd",
    "08C6B3": "QTECH LLC",
    "606BBD": "Samsung Electronics Co.,Ltd",
    "00214C": "Samsung Electronics Co.,Ltd",
    "00166B": "Samsung Electronics Co.,Ltd",
    "0000F0": "Samsung Electronics Co.,Ltd",
    "184617": "Samsung Electronics Co.,Ltd",
    "380A94": "Samsung Electronics Co.,Ltd",
    "D0DFC7": "Samsung Electronics Co.,Ltd",
    "D0C1B1": "Samsung Electronics Co.,Ltd",
    "70F927": "Samsung Electronics Co.,Ltd",
    "A8F274": "Samsung Electronics Co.,Ltd",
    "D487D8": "Samsung Electronics Co.,Ltd",
    "F0728C": "Samsung Electronics Co.,Ltd",
    "34AA8B": "Samsung Electronics Co.,Ltd",
    "BC4486": "Samsung Electronics Co.,Ltd",
    "20D390": "Samsung Electronics Co.,Ltd",
    "0018AF": "Samsung Electronics Co.,Ltd",
    "001EE1": "Samsung Electronics Co.,Ltd",
    "34E71C": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "886AB1": "vivo Mobile Communication Co., Ltd.",
    "6C1E90": "Hansol Technics Co., Ltd.",
    "C8DE51": " IntegraOptics",
    "24DBED": "Samsung Electronics Co.,Ltd",
    "C45006": "Samsung Electronics Co.,Ltd",
    "88329B": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "1449E0": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "D02544": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "A00460": "NETGEAR",
    "9401C2": "Samsung Electronics Co.,Ltd",
    "50FC9F": "Samsung Electronics Co.,Ltd",
    "380B40": "Samsung Electronics Co.,Ltd",
    "005A13": "HUAWEI TECHNOLOGIES CO.,LTD",
    "946124": "Pason Systems",
    "70B14E": "ARRIS Group, Inc.",
    "B8FF61": "Apple, Inc.",
    "304487": "Hefei Radio Communication Technology Co., Ltd ",
    "2C9D1E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "400D10": "ARRIS Group, Inc.",
    "943DC9": "Asahi Net, Inc.",
    "9884E3": "Texas Instruments",
    "38D269": "Texas Instruments",
    "C8FD19": "Texas Instruments",
    "508CB1": "Texas Instruments",
    "440444": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "98F058": "Lynxspring, Incl.",
    "68C44D": "Motorola Mobility LLC, a Lenovo Company",
    "0081C4": "Cisco Systems, Inc",
    "58E876": "IEEE Registration Authority",
    "00177E": "Meshcom Technologies Inc.",
    "C4F5A5": "Kumalift Co., Ltd.",
    "F8E61A": "Samsung Electronics Co.,Ltd",
    "84B541": "Samsung Electronics Co.,Ltd",
    "006F64": "Samsung Electronics Co.,Ltd",
    "DC6672": "Samsung Electronics Co.,Ltd",
    "EC8EB5": "Hewlett Packard",
    "70AF6A": "SHENZHEN FENGLIAN TECHNOLOGY CO., LTD.",
    "E0DDC0": "vivo Mobile Communication Co., Ltd.",
    "AC9CE4": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "00233E": "Alcatel-Lucent IPD",
    "6CBEE9": "Alcatel-Lucent IPD",
    "D03742": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "20F543": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "001765": "Nortel Networks",
    "0015E8": "Nortel Networks",
    "00159B": "Nortel Networks",
    "001540": "Nortel Networks",
    "001ECA": "Nortel Networks",
    "00130A": "Nortel Networks",
    "001F0A": "Nortel Networks",
    "000F06": "Nortel Networks",
    "C4047B": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "20F41B": "Shenzhen Bilian electronic CO.,LTD",
    "6CA858": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A84E3F": "Hitron Technologies. Inc",
    "00164D": "Alcatel-Lucent IPD",
    "001AF0": "Alcatel-Lucent IPD",
    "38521A": "Nokia",
    "001E40": "Shanghai DareGlobal Technologies Co.,Ltd",
    "94D723": "Shanghai DareGlobal Technologies Co.,Ltd",
    "FCFAF7": "Shanghai Baud Data Communication Co.,Ltd.",
    "D826B9": "Guangdong Coagent Electronics S&amp;T Co.,Ltd.",
    "0022A9": "LG Electronics (Mobile Communications)",
    "0025E5": "LG Electronics (Mobile Communications)",
    "0021FB": "LG Electronics (Mobile Communications)",
    "A091C8": "zte corporation",
    "E4956E": "IEEE Registration Authority",
    "B437D1": "IEEE Registration Authority",
    "2C6A6F": "IEEE Registration Authority",
    "A46032": "MRV Communications (Networks) LTD",
    "40667A": "mediola - connected living AG",
    "9C2A83": "Samsung Electronics Co.,Ltd",
    "68A0F6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000E5C": "ARRIS Group, Inc.",
    "845DD7": "Shenzhen Netcom Electronics Co.,Ltd",
    "E892A4": "LG Electronics (Mobile Communications)",
    "10683F": "LG Electronics (Mobile Communications)",
    "40B0FA": "LG Electronics (Mobile Communications)",
    "A039F7": "LG Electronics (Mobile Communications)",
    "00B064": "Cisco Systems, Inc",
    "34FCEF": "LG Electronics (Mobile Communications)",
    "BCF5AC": "LG Electronics (Mobile Communications)",
    "98D6F7": "LG Electronics (Mobile Communications)",
    "700514": "LG Electronics (Mobile Communications)",
    "0055DA": "IEEE Registration Authority",
    "78C2C0": "IEEE Registration Authority",
    "1CCAE3": "IEEE Registration Authority",
    "08D833": "Shenzhen RF Technology Co., Ltd",
    "00199D": "Vizio, Inc",
    "001938": "UMB Communications Co., Ltd.",
    "4439C4": "Universal Global Scientific Industrial Co., Ltd.",
    "402CF4": "Universal Global Scientific Industrial Co., Ltd.",
    "001E37": "Universal Global Scientific Industrial Co., Ltd.",
    "001A6B": "Universal Global Scientific Industrial Co., Ltd.",
    "001641": "Universal Global Scientific Industrial Co., Ltd.",
    "0010C6": "Universal Global Scientific Industrial Co., Ltd.",
    "00247E": "Universal Global Scientific Industrial Co., Ltd.",
    "000EE8": "Zioncom Electronics (Shenzhen) Ltd.",
    "00C095": "ZNYX Networks, Inc.",
    "002025": "CONTROL TECHNOLOGY, INC.",
    "683563": "SHENZHEN LIOWN ELECTRONICS CO.,LTD.",
    "004072": "Applied Innovation Inc.",
    "00E08B": "QLogic Corporation",
    "1C57D8": "Kraftway Corporation PLC",
    "00DD0A": "UNGERMANN-BASS INC.",
    "002517": "Venntis, LLC",
    "00600F": "Westell Technologies Inc.",
    "00183A": "Westell Technologies Inc.",
    "446EE5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C8778B": "Mercury Systems \u2013 Trusted Mission Solutions, Inc. ",
    "00044B": "NVIDIA",
    "D8EB97": "TRENDnet, Inc.",
    "001C7E": "Toshiba",
    "002318": "Toshiba",
    "B86B23": "Toshiba",
    "0008F1": "Voltaire",
    "AC9B0A": "Sony Corporation",
    "104FA8": "Sony Corporation",
    "AC040B": "Peloton Interactive, Inc",
    "48FCB6": "LAVA INTERNATIONAL(H.K) LIMITED",
    "B0E235": "Xiaomi Communications Co Ltd",
    "40C729": "Sagemcom Broadband SAS",
    "14C913": "LG Electronics",
    "D8E0B8": "BULAT LLC",
    "603197": "Zyxel Communications Corporation",
    "F8A097": "ARRIS Group, Inc.",
    "0014B4": "General Dynamics United Kingdom Ltd",
    "A0B437": " GD Mission Systems",
    "E09861": "Motorola Mobility LLC, a Lenovo Company",
    "9C8ECD": "Amcrest Technologies",
    "0004A3": "Microchip Technology Inc.",
    "789CE7": "Shenzhen Aikede Technology Co., Ltd",
    "509F3B": "OI ELECTRIC CO.,LTD",
    "FC2325": "EosTek (Shenzhen) Co., Ltd.",
    "FC3D93": "LONGCHEER TELECOMMUNICATION LIMITED",
    "00F663": "Cisco Systems, Inc",
    "000B6B": "Wistron Neweb Corporation",
    "6002B4": "Wistron Neweb Corporation",
    "94DF4E": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "98EECB": "Wistron Infocomm (Zhongshan) Corporation",
    "C0C976": "Shenzhen TINNO Mobile Technology Corp.",
    "588BF3": "Zyxel Communications Corporation",
    "5067F0": "Zyxel Communications Corporation",
    "001349": "Zyxel Communications Corporation",
    "8C6D50": "SHENZHEN MTC CO LTD",
    "A009ED": "Avaya Inc",
    "683E34": "MEIZU Technology Co., Ltd.",
    "001BFE": "Zavio Inc.",
    "5410EC": "Microchip Technology Inc.",
    "A06090": "Samsung Electronics Co.,Ltd",
    "BC765E": "Samsung Electronics Co.,Ltd",
    "E0A8B8": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "F45B73": "Wanjiaan Interconnected Technology Co., Ltd",
    "B88198": "Intel Corporate",
    "C83DFC": "AlphaTheta Corporation",
    "CCD31E": "IEEE Registration Authority",
    "34B354": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C0EE6": "Chengdu Xiyida Electronic Technology Co,.Ltd",
    "005F86": "Cisco Systems, Inc",
    "381DD9": "FN-LINK TECHNOLOGY LIMITED",
    "1CB9C4": "Ruckus Wireless",
    "2CDDA3": "Point Grey Research Inc.",
    "00809F": "ALE International",
    "B824F0": "SOYO Technology Development Co., Ltd.",
    "D85B2A": "Samsung Electronics Co.,Ltd",
    "FCA89A": "Sunitec Enterprise Co.,Ltd",
    "1C7B23": "Qingdao Hisense Communications Co.,Ltd.",
    "B0D7CC": "Tridonic GmbH & Co KG",
    "8C59C3": "ADB Italia ",
    "48C663": "GTO Access Systems LLC",
    "1C6E76": "Quarion Technology Inc",
    "000763": "Sunniwell Cyber Tech. Co., Ltd.",
    "240A11": "TCT mobile ltd",
    "D8E56D": "TCT mobile ltd",
    "540593": "WOORI ELEC Co.,Ltd",
    "C02FF1": "Volta Networks",
    "E8A7F2": "sTraffic",
    "001F20": "Logitech Europe SA",
    "0062EC": "Cisco Systems, Inc",
    "CC167E": "Cisco Systems, Inc",
    "C46AB7": "Xiaomi Communications Co Ltd",
    "000AED": "HARTING Electronics GmbH",
    "CC500A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D046DC": "Southwest Research Institute",
    "70A2B3": "Apple, Inc.",
    "E41D2D": "Mellanox Technologies, Inc.",
    "F40F24": "Apple, Inc.",
    "4C57CA": "Apple, Inc.",
    "90C1C6": "Apple, Inc.",
    "0CDA41": "Hangzhou H3C Technologies Co., Limited",
    "74258A": "Hangzhou H3C Technologies Co., Limited",
    "741F4A": "Hangzhou H3C Technologies Co., Limited",
    "E42F56": "OptoMET GmbH",
    "F8DA0C": "Hon Hai Precision Ind. Co.,Ltd.",
    "1C1B0D": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "48E9F1": "Apple, Inc.",
    "903809": "Ericsson AB",
    "C83F26": "Microsoft Corporation",
    "000C49": "Dangaard Telecom Denmark A/S",
    "A0B662": "Acutvista Innovation Co., Ltd.",
    "002238": "LOGIPLUS",
    "3497F6": "ASUSTek COMPUTER INC.",
    "A08CFD": "Hewlett Packard",
    "50680A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00A006": "IMAGE DATA PROCESSING SYSTEM GROUP",
    "8019FE": "JianLing Technology CO., LTD",
    "60B4F7": "Plume Design Inc",
    "44650D": "Amazon Technologies Inc.",
    "60B387": "Synergics Technologies GmbH",
    "A4D8CA": "HONG KONG WATER WORLD TECHNOLOGY CO. LIMITED",
    "1078D2": "Elitegroup Computer Systems Co.,Ltd.",
    "002197": "Elitegroup Computer Systems Co.,Ltd.",
    "001E90": "Elitegroup Computer Systems Co.,Ltd.",
    "0022B1": "Elbit Systems Ltd.",
    "0000B4": "Edimax Technology Co. Ltd.",
    "487ADA": "Hangzhou H3C Technologies Co., Limited",
    "140C5B": "PLNetworks",
    "50FF99": "IEEE Registration Authority",
    "84E323": "Green Wave Telecommunication SDN BHD",
    "FC3F7C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "384C4F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001F45": "Enterasys",
    "000E03": "Emulex Corporation",
    "000D87": "Elitegroup Computer Systems Co.,Ltd.",
    "00168F": "GN Netcom A/S",
    "BC9889": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24615A": "China Mobile Group Device Co.,Ltd.",
    "00CAE5": "Cisco Systems, Inc",
    "004268": "Cisco Systems, Inc",
    "4883C7": "Sagemcom Broadband SAS",
    "40163B": "Samsung Electronics Co.,Ltd",
    "04A316": "Texas Instruments",
    "D4F207": "DIAODIAO(Beijing)Technology CO.,Ltd",
    "D4AD2D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F08CFB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "48555F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "405EE1": "Shenzhen H&T Intelligent Control Co.,Ltd.",
    "002578": "JSC \"Concern \"Sozvezdie\"",
    "30B49E": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "34E70B": "HAN Networks Co., Ltd",
    "007888": "Cisco Systems, Inc",
    "FC084A": "FUJITSU LIMITED",
    "0CBF3F": "Shenzhen Lencotion Technology Co.,Ltd",
    "900325": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98E7F5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001706": "Techfaithwireless Communication Technology Limited.",
    "A09D91": "SoundBridge",
    "40B688": "LEGIC Identsystems AG",
    "9CD48B": "Innolux Technology Europe BV",
    "085BDA": "CliniCare LTD",
    "1CC035": "PLANEX COMMUNICATIONS INC.",
    "34543C": "TAKAOKA TOKO CO.,LTD.",
    "9C9D5D": "Raden Inc",
    "DC4D23": "MRV Comunications",
    "0023B3": "Lyyn AB",
    "248A07": "Mellanox Technologies, Inc.",
    "C83870": "Samsung Electronics Co.,Ltd",
    "1C553A": "QianGua Corp.",
    "008E73": "Cisco Systems, Inc",
    "402E28": "MiXTelematics",
    "583277": "Reliance Communications LLC",
    "6C8FB5": "Microsoft Mobile Oy",
    "0015C1": "Sony Interactive Entertainment Inc.",
    "90A62F": "NAVER",
    "C0C522": "ARRIS Group, Inc.",
    "1C9E46": "Apple, Inc.",
    "9C4FDA": "Apple, Inc.",
    "00351A": "Cisco Systems, Inc",
    "00AF1F": "Cisco Systems, Inc",
    "C0CCF8": "Apple, Inc.",
    "C4E510": "Mechatro, Inc.",
    "0021FC": "Nokia Danmark A/S",
    "001F5D": "Nokia Danmark A/S",
    "001F01": "Nokia Danmark A/S",
    "001BEE": "Nokia Danmark A/S",
    "803896": "SHARP Corporation",
    "8489AD": "Apple, Inc.",
    "347E39": "Nokia Danmark A/S",
    "001979": "Nokia Danmark A/S",
    "0060EC": "HERMARY OPTO ELECTRONICS INC.",
    "DC3A5E": "Roku, Inc.",
    "00507F": "DrayTek Corp.",
    "001A73": "Gemtek Technology Co., Ltd.",
    "00904B": "Gemtek Technology Co., Ltd.",
    "001A7F": "GCI Science & Technology Co.,LTD",
    "0024D4": "FREEBOX SAS",
    "AC3A7A": "Roku, Inc.",
    "B83E59": "Roku, Inc.",
    "544408": "Nokia Corporation",
    "3CC243": "Nokia Corporation",
    "0090A2": "CyberTAN Technology Inc.",
    "0090D6": "Crystal Group, Inc.",
    "647791": "Samsung Electronics Co.,Ltd",
    "9CE6E7": "Samsung Electronics Co.,Ltd",
    "9C0298": "Samsung Electronics Co.,Ltd",
    "28987B": "Samsung Electronics Co.,Ltd",
    "0C715D": "Samsung Electronics Co.,Ltd",
    "00180F": "Nokia Danmark A/S",
    "C8979F": "Nokia Corporation",
    "ECF35B": "Nokia Corporation",
    "0025D0": "Nokia Danmark A/S",
    "7C1CF1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78F557": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E02861": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D0D04B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "480031": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C09D4": "Arcadyan Technology Corporation",
    "9C80DF": "Arcadyan Technology Corporation",
    "002308": "Arcadyan Technology Corporation",
    "880355": "Arcadyan Technology Corporation",
    "34BB1F": "BlackBerry RTS",
    "406F2A": "BlackBerry RTS",
    "D476EA": "zte corporation",
    "00175A": "Cisco Systems, Inc",
    "54FA3E": "Samsung Electronics Co.,Ltd",
    "0C8910": "Samsung Electronics Co.,Ltd",
    "78ABBB": "Samsung Electronics Co.,Ltd",
    "D8C4E9": "Samsung Electronics Co.,Ltd",
    "BCD11F": "Samsung Electronics Co.,Ltd",
    "F4428F": "Samsung Electronics Co.,Ltd",
    "0896D7": "AVM GmbH",
    "506A03": "NETGEAR",
    "446D6C": "Samsung Electronics Co.,Ltd",
    "00F46F": "Samsung Electronics Co.,Ltd",
    "001018": "Broadcom",
    "18C086": "Broadcom",
    "FCB4E6": "ASKEY COMPUTER CORP",
    "C8FF28": "Liteon Technology Corporation",
    "B81619": "ARRIS Group, Inc.",
    "0023ED": "ARRIS Group, Inc.",
    "001B52": "ARRIS Group, Inc.",
    "001E8D": "ARRIS Group, Inc.",
    "001BDD": "ARRIS Group, Inc.",
    "001D6B": "ARRIS Group, Inc.",
    "001DBE": "ARRIS Group, Inc.",
    "0012C9": "ARRIS Group, Inc.",
    "00192C": "ARRIS Group, Inc.",
    "00195E": "ARRIS Group, Inc.",
    "001A1B": "ARRIS Group, Inc.",
    "001A66": "ARRIS Group, Inc.",
    "001A77": "ARRIS Group, Inc.",
    "A4ED4E": "ARRIS Group, Inc.",
    "00211E": "ARRIS Group, Inc.",
    "002180": "ARRIS Group, Inc.",
    "0023A2": "ARRIS Group, Inc.",
    "B077AC": "ARRIS Group, Inc.",
    "002493": "ARRIS Group, Inc.",
    "002641": "ARRIS Group, Inc.",
    "0017E2": "ARRIS Group, Inc.",
    "001675": "ARRIS Group, Inc.",
    "000CE5": "ARRIS Group, Inc.",
    "E0469A": "NETGEAR",
    "30469A": "NETGEAR",
    "100D7F": "NETGEAR",
    "504A6E": "NETGEAR",
    "0022F4": "AMPAK Technology, Inc.",
    "001DBA": "Sony Corporation",
    "0024BE": "Sony Corporation",
    "000FDE": "Sony Corporation",
    "F8D0AC": "Sony Interactive Entertainment Inc.",
    "0CFE45": "Sony Interactive Entertainment Inc.",
    "2016D8": "Liteon Technology Corporation",
    "E8617E": "Liteon Technology Corporation",
    "18CF5E": "Liteon Technology Corporation",
    "00D0C9": "ADVANTECH CO., LTD.",
    "6487D7": "ADB Broadband Italia",
    "38229D": "ADB Broadband Italia",
    "A4526F": "ADB Broadband Italia",
    "74888B": "ADB Broadband Italia",
    "008C54": "ADB Broadband Italia",
    "F0272D": "Amazon Technologies Inc.",
    "84D6D0": "Amazon Technologies Inc.",
    "90B134": "ARRIS Group, Inc.",
    "40B7F3": "ARRIS Group, Inc.",
    "0015D0": "ARRIS Group, Inc.",
    "001596": "ARRIS Group, Inc.",
    "04E676": "AMPAK Technology, Inc.",
    "18FE34": "Espressif Inc.",
    "B8F934": "Sony Corporation",
    "8C6422": "Sony Corporation",
    "E063E5": "Sony Corporation",
    "001B59": "Sony Corporation",
    "002298": "Sony Corporation",
    "2421AB": "Sony Corporation",
    "0003E0": "ARRIS Group, Inc.",
    "20E564": "ARRIS Group, Inc.",
    "00006E": "Artisoft Inc.",
    "00DA55": "Cisco Systems, Inc",
    "985FD3": "Microsoft Corporation",
    "0010B5": "Accton Technology Corp",
    "001974": "16063",
    "E0B2F1": "FN-LINK TECHNOLOGY LIMITED",
    "0C4C39": "MitraStar Technology Corp.",
    "002243": "AzureWave Technology Inc.",
    "18E3BC": "TCT mobile ltd",
    "CC1FC4": "InVue",
    "605BB4": "AzureWave Technology Inc.",
    "64D954": "Taicang T&W Electronics",
    "5C36B8": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "00AA01": "Intel Corporation",
    "84A6C8": "Intel Corporate",
    "5891CF": "Intel Corporate",
    "0C8BFD": "Intel Corporate",
    "843A4B": "Intel Corporate",
    "5C514F": "Intel Corporate",
    "A44E31": "Intel Corporate",
    "4CEB42": "Intel Corporate",
    "00AA00": "Intel Corporation",
    "00C2C6": "Intel Corporate",
    "5CD2E4": "Intel Corporate",
    "28B2BD": "Intel Corporate",
    "4C79BA": "Intel Corporate",
    "F81654": "Intel Corporate",
    "606C66": "Intel Corporate",
    "4C8093": "Intel Corporate",
    "AC7289": "Intel Corporate",
    "448500": "Intel Corporate",
    "0CD292": "Intel Corporate",
    "00247B": "Actiontec Electronics, Inc",
    "0004E3": "Accton Technology Corp",
    "448723": "HOYA SERVICE CORPORATION",
    "D86C02": "Huaqin Telecom Technology Co.,Ltd",
    "60BEB5": "Motorola Mobility LLC, a Lenovo Company",
    "F8F1B6": "Motorola Mobility LLC, a Lenovo Company",
    "F4F1E1": "Motorola Mobility LLC, a Lenovo Company",
    "9CD917": "Motorola Mobility LLC, a Lenovo Company",
    "9068C3": "Motorola Mobility LLC, a Lenovo Company",
    "685D43": "Intel Corporate",
    "A0369F": "Intel Corporate",
    "64D4DA": "Intel Corporate",
    "4025C2": "Intel Corporate",
    "502DA2": "Intel Corporate",
    "78929C": "Intel Corporate",
    "DCA971": "Intel Corporate",
    "58946B": "Intel Corporate",
    "3CFDFE": "Intel Corporate",
    "A4C494": "Intel Corporate",
    "902E1C": "Intel Corporate",
    "A434D9": "Intel Corporate",
    "0024D7": "Intel Corporate",
    "0024D6": "Intel Corporate",
    "001DE0": "Intel Corporate",
    "A0A8CD": "Intel Corporate",
    "5CC5D4": "Intel Corporate",
    "001E64": "Intel Corporate",
    "00215C": "Intel Corporate",
    "00216B": "Intel Corporate",
    "0022FB": "Intel Corporate",
    "001517": "Intel Corporate",
    "34EF44": "2Wire Inc",
    "B0E754": "2Wire Inc",
    "B8E625": "2Wire Inc",
    "001D5A": "2Wire Inc",
    "00253C": "2Wire Inc",
    "3C197D": "Ericsson AB",
    "247C4C": "Herman Miller",
    "E46F13": "D-Link International",
    "1C4419": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "5C353B": "Compal Broadband Networks, Inc.",
    "F8E71E": "Ruckus Wireless",
    "08863B": "Belkin International Inc.",
    "90013B": "Sagemcom Broadband SAS",
    "7C034C": "Sagemcom Broadband SAS",
    "6C2E85": "Sagemcom Broadband SAS",
    "94FEF4": "Sagemcom Broadband SAS",
    "28FAA0": "vivo Mobile Communication Co., Ltd.",
    "ECDF3A": "vivo Mobile Communication Co., Ltd.",
    "F42981": "vivo Mobile Communication Co., Ltd.",
    "84F6FA": "Miovision Technologies Incorporated",
    "2C56DC": "ASUSTek COMPUTER INC.",
    "00604C": "Sagemcom Broadband SAS",
    "001F95": "Sagemcom Broadband SAS",
    "002348": "Sagemcom Broadband SAS",
    "002691": "Sagemcom Broadband SAS",
    "988B5D": "Sagemcom Broadband SAS",
    "78DEE4": "Texas Instruments",
    "001833": "Texas Instruments",
    "001834": "Texas Instruments",
    "0017E3": "Texas Instruments",
    "001830": "Texas Instruments",
    "0023D4": "Texas Instruments",
    "C0E422": "Texas Instruments",
    "D00790": "Texas Instruments",
    "3C7DB1": "Texas Instruments",
    "001783": "Texas Instruments",
    "F4FC32": "Texas Instruments",
    "90D7EB": "Texas Instruments",
    "00F871": "Demant A/S",
    "34B1F7": "Texas Instruments",
    "2CFD37": "Blue Calypso, Inc.",
    "0C6127": "Actiontec Electronics, Inc",
    "2C3033": "NETGEAR",
    "78542E": "D-Link International",
    "C4A81D": "D-Link International",
    "2435CC": "Zhongshan Scinan Internet of Things Co.,Ltd.",
    "F88FCA": "Google, Inc.",
    "002191": "D-Link Corporation",
    "ACF1DF": "D-Link International",
    "BCF685": "D-Link International",
    "3CD92B": "Hewlett Packard",
    "BC4434": "Shenzhen TINNO Mobile Technology Corp.",
    "04BF6D": "Zyxel Communications Corporation",
    "CC46D6": "Cisco Systems, Inc",
    "0041D2": "Cisco Systems, Inc",
    "2CAB00": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8CA7B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FCFFAA": "IEEE Registration Authority",
    "3898D8": "MERITECH CO.,LTD",
    "9486CD": "SEOUL ELECTRONICS&TELECOM",
    "8896B6": "Global Fire Equipment S.A.",
    "88B8D0": "Dongguan Koppo Electronic Co.,Ltd",
    "601971": "ARRIS Group, Inc.",
    "58AC78": "Cisco Systems, Inc",
    "5C571A": "ARRIS Group, Inc.",
    "E8892C": "ARRIS Group, Inc.",
    "84ACFB": "Crouzet Automatismes",
    "7CBB8A": "Nintendo Co., Ltd.",
    "3897D6": "Hangzhou H3C Technologies Co., Limited",
    "1CA770": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "0CD746": "Apple, Inc.",
    "60A37D": "Apple, Inc.",
    "68DBCA": "Apple, Inc.",
    "086698": "Apple, Inc.",
    "BC5436": "Apple, Inc.",
    "044BED": "Apple, Inc.",
    "6C8DC1": "Apple, Inc.",
    "000D0B": "BUFFALO.INC",
    "001D73": "BUFFALO.INC",
    "001601": "BUFFALO.INC",
    "7403BD": "BUFFALO.INC",
    "B8FC9A": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "94877C": "ARRIS Group, Inc.",
    "407009": "ARRIS Group, Inc.",
    "083E0C": "ARRIS Group, Inc.",
    "207355": "ARRIS Group, Inc.",
    "F8EDA5": "ARRIS Group, Inc.",
    "5465DE": "ARRIS Group, Inc.",
    "6CCA08": "ARRIS Group, Inc.",
    "000423": "Intel Corporation",
    "001111": "Intel Corporation",
    "001302": "Intel Corporate",
    "D40598": "ARRIS Group, Inc.",
    "E83381": "ARRIS Group, Inc.",
    "8C7F3B": "ARRIS Group, Inc.",
    "3C36E4": "ARRIS Group, Inc.",
    "1C1B68": "ARRIS Group, Inc.",
    "780AC7": "Baofeng TV Co., Ltd.",
    "002481": "Hewlett Packard",
    "000F61": "Hewlett Packard",
    "0014C2": "Hewlett Packard",
    "00805F": "Hewlett Packard",
    "288023": "Hewlett Packard",
    "0018FE": "Hewlett Packard",
    "001A4B": "Hewlett Packard",
    "A4516F": "Microsoft Mobile Oy",
    "FC64BA": "Xiaomi Communications Co Ltd",
    "8CAB8E": "Shanghai Feixun Communication Co.,Ltd.",
    "D40B1A": "HTC Corporation",
    "945330": "Hon Hai Precision Ind. Co.,Ltd.",
    "A08D16": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CD08A": "HUMAX Co., Ltd.",
    "CC4EEC": "HUMAX Co., Ltd.",
    "403DEC": "HUMAX Co., Ltd.",
    "EC4D47": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C44044": "RackTop Systems Inc.",
    "4CA161": "Rain Bird Corporation",
    "CC3E5F": "Hewlett Packard",
    "D89D67": "Hewlett Packard",
    "A45D36": "Hewlett Packard",
    "F0921C": "Hewlett Packard",
    "A0481C": "Hewlett Packard",
    "A01D48": "Hewlett Packard",
    "40A8F0": "Hewlett Packard",
    "8851FB": "Hewlett Packard",
    "9060F1": "Apple, Inc.",
    "542758": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "082E5F": "Hewlett Packard",
    "E4115B": "Hewlett Packard",
    "28924A": "Hewlett Packard",
    "480FCF": "Hewlett Packard",
    "00242B": "Hon Hai Precision Ind. Co.,Ltd.",
    "1C994C": "Murata Manufacturing Co., Ltd.",
    "F02765": "Murata Manufacturing Co., Ltd.",
    "5CF8A1": "Murata Manufacturing Co., Ltd.",
    "44A7CF": "Murata Manufacturing Co., Ltd.",
    "0013E0": "Murata Manufacturing Co., Ltd.",
    "90FBA6": "Hon Hai Precision Ind. Co.,Ltd.",
    "4437E6": "Hon Hai Precision Ind. Co.,Ltd.",
    "CCAF78": "Hon Hai Precision Ind. Co.,Ltd.",
    "F4B7E2": "Hon Hai Precision Ind. Co.,Ltd.",
    "785968": "Hon Hai Precision Ind. Co.,Ltd.",
    "08181A": "zte corporation",
    "001E73": "zte corporation",
    "0015EB": "zte corporation",
    "001C25": "Hon Hai Precision Ind. Co.,Ltd.",
    "00197E": "Hon Hai Precision Ind. Co.,Ltd.",
    "EC26CA": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "9471AC": "TCT mobile ltd",
    "2C088C": "HUMAX Co., Ltd.",
    "000480": "Brocade Communications Systems LLC",
    "000CDB": "Brocade Communications Systems LLC",
    "647002": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "10FEED": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "645601": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F8D111": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "B0487A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "940C6D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "001BED": "Brocade Communications Systems LLC",
    "000533": "Brocade Communications Systems LLC",
    "006069": "Brocade Communications Systems LLC",
    "0060DF": "Brocade Communications Systems LLC",
    "000088": "Brocade Communications Systems LLC",
    "ECCB30": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4DCF9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C8D15E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4559C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "80B686": "HUAWEI TECHNOLOGIES CO.,LTD",
    "10C61F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC96A0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00664B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9CC172": "HUAWEI TECHNOLOGIES CO.,LTD",
    "247F3C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "581F28": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C07009": "HUAWEI TECHNOLOGIES CO.,LTD",
    "308730": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C057BC": "Avaya Inc",
    "8038BC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4072F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F48E92": "HUAWEI TECHNOLOGIES CO.,LTD",
    "241FA0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "34CDBE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D8490B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F80113": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A49947": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64A7DD": "Avaya Inc",
    "A4251B": "Avaya Inc",
    "646A52": "Avaya Inc",
    "44322A": "Avaya Inc",
    "7038EE": "Avaya Inc",
    "703018": "Avaya Inc",
    "9C28EF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00900C": "Cisco Systems, Inc",
    "00905F": "Cisco Systems, Inc",
    "00100B": "Cisco Systems, Inc",
    "00173B": "Cisco Systems, Inc",
    "080028": "Texas Instruments",
    "405FC2": "Texas Instruments",
    "20CD39": "Texas Instruments",
    "B4994C": "Texas Instruments",
    "68DFDD": "Xiaomi Communications Co Ltd",
    "98FAE3": "Xiaomi Communications Co Ltd",
    "F0B429": "Xiaomi Communications Co Ltd",
    "7054F5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9017AC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18C58A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "006083": "Cisco Systems, Inc",
    "006009": "Cisco Systems, Inc",
    "00067C": "Cisco Systems, Inc",
    "00E0F7": "Cisco Systems, Inc",
    "00107B": "Cisco Systems, Inc",
    "0050E2": "Cisco Systems, Inc",
    "0090A6": "Cisco Systems, Inc",
    "009086": "Cisco Systems, Inc",
    "005080": "Cisco Systems, Inc",
    "005073": "Cisco Systems, Inc",
    "001BD7": "Cisco SPVTG",
    "E4D3F1": "Cisco Systems, Inc",
    "EC24B8": "Texas Instruments",
    "7CEC79": "Texas Instruments",
    "689E19": "Texas Instruments",
    "E0E5CF": "Texas Instruments",
    "1C872C": "ASUSTek COMPUTER INC.",
    "60182E": "ShenZhen Protruly Electronic Ltd co.",
    "C4143C": "Cisco Systems, Inc",
    "3C08F6": "Cisco Systems, Inc",
    "501CBF": "Cisco Systems, Inc",
    "001E8C": "ASUSTek COMPUTER INC.",
    "0013D4": "ASUSTek COMPUTER INC.",
    "20CF30": "ASUSTek COMPUTER INC.",
    "00248C": "ASUSTek COMPUTER INC.",
    "002354": "ASUSTek COMPUTER INC.",
    "BC1665": "Cisco Systems, Inc",
    "F872EA": "Cisco Systems, Inc",
    "D0C789": "Cisco Systems, Inc",
    "F84F57": "Cisco Systems, Inc",
    "7C69F6": "Cisco Systems, Inc",
    "8478AC": "Cisco Systems, Inc",
    "04DAD2": "Cisco Systems, Inc",
    "F02929": "Cisco Systems, Inc",
    "20BBC0": "Cisco Systems, Inc",
    "4C4E35": "Cisco Systems, Inc",
    "B000B4": "Cisco Systems, Inc",
    "544A00": "Cisco Systems, Inc",
    "00E16D": "Cisco Systems, Inc",
    "E0899D": "Cisco Systems, Inc",
    "C47295": "Cisco Systems, Inc",
    "E0D173": "Cisco Systems, Inc",
    "78DA6E": "Cisco Systems, Inc",
    "78BAF9": "Cisco Systems, Inc",
    "1C6E4C": "Logistic Service & Engineering Co.,Ltd",
    "34BDC8": "Cisco Systems, Inc",
    "B8782E": "Apple, Inc.",
    "000502": "Apple, Inc.",
    "000A95": "Apple, Inc.",
    "E80688": "Apple, Inc.",
    "7CC537": "Apple, Inc.",
    "78CA39": "Apple, Inc.",
    "18E7F4": "Apple, Inc.",
    "70CD60": "Apple, Inc.",
    "E4AA5D": "Cisco Systems, Inc",
    "DCEB94": "Cisco Systems, Inc",
    "84B517": "Cisco Systems, Inc",
    "188B9D": "Cisco Systems, Inc",
    "24374C": "Cisco SPVTG",
    "F45FD4": "Cisco SPVTG",
    "2CABA4": "Cisco SPVTG",
    "0022CE": "Cisco SPVTG",
    "000F66": "Cisco-Linksys, LLC",
    "00264A": "Apple, Inc.",
    "041E64": "Apple, Inc.",
    "90840D": "Apple, Inc.",
    "001124": "Apple, Inc.",
    "002241": "Apple, Inc.",
    "88CB87": "Apple, Inc.",
    "685B35": "Apple, Inc.",
    "2CB43A": "Apple, Inc.",
    "689C70": "Apple, Inc.",
    "380F4A": "Apple, Inc.",
    "3010E4": "Apple, Inc.",
    "A886DD": "Apple, Inc.",
    "F0C1F1": "Apple, Inc.",
    "B4F0AB": "Apple, Inc.",
    "80929F": "Apple, Inc.",
    "9C04EB": "Apple, Inc.",
    "5C969D": "Apple, Inc.",
    "A8968A": "Apple, Inc.",
    "D89E3F": "Apple, Inc.",
    "B8C75D": "Apple, Inc.",
    "0C74C2": "Apple, Inc.",
    "403004": "Apple, Inc.",
    "74E2F5": "Apple, Inc.",
    "E0C97A": "Apple, Inc.",
    "444C0C": "Apple, Inc.",
    "F41BA1": "Apple, Inc.",
    "041552": "Apple, Inc.",
    "CC785F": "Apple, Inc.",
    "7073CB": "Apple, Inc.",
    "EC852F": "Apple, Inc.",
    "00F4B9": "Apple, Inc.",
    "60C547": "Apple, Inc.",
    "68A86D": "Apple, Inc.",
    "7CC3A1": "Apple, Inc.",
    "5C95AE": "Apple, Inc.",
    "842999": "Apple, Inc.",
    "8C7B9D": "Apple, Inc.",
    "9803D8": "Apple, Inc.",
    "60D9C7": "Apple, Inc.",
    "3CAB8E": "Apple, Inc.",
    "609217": "Apple, Inc.",
    "84B153": "Apple, Inc.",
    "E06678": "Apple, Inc.",
    "48D705": "Apple, Inc.",
    "908D6C": "Apple, Inc.",
    "B8098A": "Apple, Inc.",
    "4C7C5F": "Apple, Inc.",
    "68644B": "Apple, Inc.",
    "C81EE7": "Apple, Inc.",
    "A43135": "Apple, Inc.",
    "68D93C": "Apple, Inc.",
    "00F76F": "Apple, Inc.",
    "C88550": "Apple, Inc.",
    "7014A6": "Apple, Inc.",
    "985AEB": "Apple, Inc.",
    "78D75F": "Apple, Inc.",
    "843835": "Apple, Inc.",
    "8C006D": "Apple, Inc.",
    "907240": "Apple, Inc.",
    "E0B52D": "Apple, Inc.",
    "6C94F8": "Apple, Inc.",
    "F82793": "Apple, Inc.",
    "C0CECD": "Apple, Inc.",
    "F44B2A": "Cisco SPVTG",
    "746F19": "ICARVISIONS (SHENZHEN) TECHNOLOGY CO., LTD.",
    "A0F9E0": "VIVATEL COMPANY LIMITED",
    "2CAE2B": "Samsung Electronics Co.,Ltd",
    "C4ADF1": "GOPEACE Inc.",
    "58FC73": "Arria Live Media, Inc.",
    "0C1A10": "Acoustic Stream",
    "C4EF70": "Home Skinovations",
    "7C5A67": "JNC Systems, Inc.",
    "5CE3B6": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C869CD": "Apple, Inc.",
    "A4B805": "Apple, Inc.",
    "C01173": "Samsung Electronics Co.,Ltd",
    "7853F2": "Roxton Systems Ltd.",
    "BCE63F": "Samsung Electronics Co.,Ltd",
    "7C9122": "Samsung Electronics Co.,Ltd",
    "6CEBB2": "Dongguan Sen DongLv Electronics Co.,Ltd",
    "F40E22": "Samsung Electronics Co.,Ltd",
    "3C7A8A": "ARRIS Group, Inc.",
    "E81363": "Comstock RD, Inc.",
    "741865": "Shanghai DareGlobal Technologies Co.,Ltd",
    "F8C372": "TSUZUKI DENKI",
    "D47208": "Bragi GmbH",
    "90C99B": "Tesorion Nederland B.V.",
    "5CADCF": "Apple, Inc.",
    "BC6C21": "Apple, Inc.",
    "B49D0B": "BQ",
    "3C8CF8": "TRENDnet, Inc.",
    "A87285": "IDT, INC.",
    "080A4E": "Planet Bingo\u00ae \u2014 3rd Rock Gaming\u00ae",
    "780541": "Queclink Wireless Solutions Co., Ltd",
    "044169": "GoPro",
    "C02DEE": "Cuff",
    "9023EC": "Availink, Inc.",
    "441CA8": "Hon Hai Precision Ind. Co.,Ltd.",
    "ACBC32": "Apple, Inc.",
    "544E90": "Apple, Inc.",
    "A4A6A9": "Private",
    "8C10D4": "Sagemcom Broadband SAS",
    "F898B9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CB559": "CNEX Labs",
    "B83A9D": "Alarm.com",
    "6858C5": "ZF TRW Automotive",
    "881B99": "SHENZHEN XIN FEI JIA ELECTRONIC CO. LTD.",
    "906F18": "Private",
    "98CB27": "Galore Networks Pvt. Ltd.",
    "CC794A": "BLU Products Inc.",
    "94D859": "TCT mobile ltd",
    "2CFCE4": "CTEK Sweden AB",
    "B4293D": "Shenzhen Urovo Technology Co.,Ltd.",
    "54FF82": "Davit Solution co.",
    "50DF95": "Lytx",
    "2827BF": "Samsung Electronics Co.,Ltd",
    "F4E926": "Tianjin Zanpu Technology Inc.",
    "E855B4": "SAI Technology Inc.",
    "340CED": "Moduel AB",
    "9CA69D": "Whaley Technology Co.Ltd",
    "5853C0": "Beijing Guang Runtong Technology Development Company co.,Ltd",
    "247260": "IOTTECH Corp",
    "E8F2E2": "LG Innotek",
    "2CA539": "Parallel Wireless, Inc",
    "245BF0": "Liteon, Inc.",
    "1005B1": "ARRIS Group, Inc.",
    "C0B713": "Beijing Xiaoyuer Technology Co. Ltd.",
    "188EF9": "G2C Co. Ltd.",
    "20635F": "Abeeway",
    "083A5C": "Junilab, Inc.",
    "B8B3DC": "DEREK (SHAOGUAN) LIMITED",
    "702A7D": "EpSpot AB",
    "4CAE31": "ShengHai Electronics (Shenzhen) Ltd",
    "F4E9D4": "QLogic Corporation",
    "44F436": "zte corporation",
    "F4B8A7": "zte corporation",
    "300C23": "zte corporation",
    "4CB76D": "Novi Security",
    "185D9A": "BobjGear LLC",
    "C47D46": "FUJITSU LIMITED",
    "609C9F": "Brocade Communications Systems LLC",
    "A8827F": "CIBN Oriental Network(Beijing) CO.,Ltd",
    "B8C3BF": "Henan Chengshi NetWork Technology Co.\uff0cLtd",
    "6CE01E": "Modcam AB",
    "74852A": "PEGATRON CORPORATION",
    "9CB6D0": "Rivet Networks",
    "40B89A": "Hon Hai Precision Ind. Co.,Ltd.",
    "1CB72C": "ASUSTek COMPUTER INC.",
    "40B837": "Sony Corporation",
    "800184": "HTC Corporation",
    "4CEEB0": "SHC Netzwerktechnik GmbH",
    "44C69B": "Wuhan Feng Tian Information Network CO.,LTD",
    "C02567": "Nexxt Solutions",
    "FCE33C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D048F3": "DATTUS Inc",
    "44962B": "Aidon Oy",
    "B89ACD": "ELITE OPTOELECTRONIC(ASIA)CO.,LTD",
    "D468BA": "Shenzhen Sundray Technologies Company Limited",
    "086266": "ASUSTek COMPUTER INC.",
    "9C3066": "RWE Effizienz GmbH",
    "C8C50E": "Shenzhen Primestone Network Technologies.Co., Ltd.",
    "D06A1F": "BSE CO.,LTD.",
    "E807BF": "SHENZHEN BOOMTECH INDUSTRY CO.,LTD",
    "84F129": "Metrascale Inc.",
    "60E6BC": "Sino-Telecom Technology Co.,Ltd.",
    "700136": "FATEK Automation Corporation",
    "FCA22A": "PT. Callysta Multi Engineering",
    "18BDAD": "L-TECH CORPORATION",
    "A45602": "fenglian Technology Co.,Ltd.",
    "D4522A": "TangoWiFi.com",
    "B008BF": "Vital Connect, Inc.",
    "E076D0": "AMPAK Technology, Inc.",
    "6CF5E8": "Mooredoll Inc.",
    "A89008": "Beijing Yuecheng Technology Co. Ltd.",
    "1CC72D": "Shenzhen Huapu Digital CO.,Ltd",
    "8CBFA6": "Samsung Electronics Co.,Ltd",
    "C8A823": "Samsung Electronics Co.,Ltd",
    "B0C559": "Samsung Electronics Co.,Ltd",
    "183864": "CAP-TECH INTERNATIONAL CO., LTD.",
    "C0335E": "Microsoft",
    "B0E03C": "TCT mobile ltd",
    "BC1485": "Samsung Electronics Co.,Ltd",
    "F01E34": "ORICO Technologies Co., Ltd",
    "DCE026": "Patrol Tag, Inc",
    "B40566": "SP Best Corporation Co., LTD.",
    "F42C56": "SENOR TECH CO LTD",
    "FCDC4A": "G-Wearables Corp.",
    "1C14B3": "Airwire Technologies",
    "9C6C15": "Microsoft Corporation",
    "94E2FD": "Boge Kompressoren OTTO Boge GmbH & Co. KG",
    "84CFBF": "Fairphone",
    "ACD1B8": "Hon Hai Precision Ind. Co.,Ltd.",
    "A48CDB": "Lenovo",
    "D85DE2": "Hon Hai Precision Ind. Co.,Ltd.",
    "3C912B": "Vexata Inc",
    "346C0F": "Pramod Telecom Pvt. Ltd",
    "E8447E": "Bitdefender SRL",
    "445ECD": "Razer Inc",
    "4CA928": "Insensi",
    "B0495F": "OMRON HEALTHCARE Co., Ltd.",
    "D0929E": "Microsoft Corporation",
    "DC0914": "Talk-A-Phone Co.",
    "BC52B4": "Nokia",
    "9405B6": "Liling FullRiver Electronics & Technology Ltd",
    "C81B6B": "Innova Security",
    "00A509": "WigWag Inc.",
    "7491BD": "Four systems Co.,Ltd.",
    "D43266": "Fike Corporation",
    "78312B": "zte corporation",
    "900CB4": "Alinket Electronic Technology Co., Ltd",
    "F0FE6B": "Shanghai High-Flying Electronics Technology Co., Ltd",
    "60F189": "Murata Manufacturing Co., Ltd.",
    "742EFC": "DirectPacket Research, Inc,",
    "48C093": "Xirrus, Inc.",
    "A0C2DE": "Costar Video Systems",
    "88E161": "Art Beijing Science and Technology Development Co., Ltd.",
    "00F3DB": "WOO Sports",
    "3CAE69": "ESA Elektroschaltanlagen Grimma GmbH",
    "1008B1": "Hon Hai Precision Ind. Co.,Ltd.",
    "E48C0F": "Discovery Insure",
    "E42354": "SHENZHEN FUZHI SOFTWARE TECHNOLOGY CO.,LTD",
    "9470D2": "WINFIRM TECHNOLOGY",
    "A44AD3": "ST Electronics(Shanghai) Co.,Ltd",
    "94BF95": "Shenzhen Coship Electronics Co., Ltd",
    "10FACE": "Reacheng Communication Technology Co.,Ltd",
    "00A2F5": "Guangzhou Yuanyun Network Technology Co.,Ltd",
    "44CE7D": "SFR",
    "344DEA": "zte corporation",
    "4C16F1": "zte corporation",
    "3438AF": "Inlab Networks GmbH",
    "B4A828": "Shenzhen Concox Information Technology Co., Ltd",
    "C4BD6A": "SKF GmbH",
    "C401CE": "PRESITION (2000) CO., LTD.",
    "587BE9": "AirPro Technology India Pvt. Ltd",
    "7CB177": "Satelco AG",
    "CC3080": "VAIO Corporation",
    "EC1D7F": "zte corporation",
    "AC3870": "Lenovo Mobile Communication Technology Ltd.",
    "70F196": "Actiontec Electronics, Inc",
    "188219": "Alibaba Cloud Computing Ltd.",
    "E4C62B": "Airware",
    "4CE933": "RailComm, LLC",
    "187117": "eta plus electronic gmbh",
    "EC0EC4": "Hon Hai Precision Ind. Co.,Ltd.",
    "8C18D9": "Shenzhen RF Technology Co., Ltd",
    "D8FB11": "AXACORE",
    "30FAB7": "Tunai Creative",
    "0809B6": "Masimo Corp",
    "4CF5A0": "Scalable Network Technologies Inc",
    "600292": "PEGATRON CORPORATION",
    "B4B859": "Texa Spa",
    "5CF9F0": "Atomos Engineering P/L",
    "D0A0D6": "ChengDu TD Tech",
    "ECB907": "CloudGenix Inc",
    "0C8C8F": "Kamo Technology Limited",
    "A4A4D3": "Bluebank Communication Technology Co.Ltd",
    "A8329A": "Digicom Futuristic Technologies Ltd.",
    "F42833": "MMPC Inc.",
    "4C83DE": "Cisco SPVTG",
    "A81374": "Panasonic Corporation AVC Networks Company",
    "F4D032": "Yunnan Ideal Information&Technology.,Ltd",
    "3C46D8": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "147590": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "50BD5F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "4CBC42": "Shenzhen Hangsheng Electronics Co.,Ltd.",
    "28A5EE": "Shenzhen SDGI CATV Co., Ltd",
    "083D88": "Samsung Electronics Co.,Ltd",
    "987E46": "Emizon Networks Limited",
    "BC4E5D": "ZhongMiao Technology Co., Ltd.",
    "7C6AC3": "GatesAir, Inc",
    "702DD1": "Newings Communication CO., LTD.",
    "F4F646": "Dediprog Technology Co. Ltd.",
    "28E6E9": "SIS Sat Internet Services GmbH",
    "F4FD2B": "ZOYI Company",
    "109266": "Samsung Electronics Co.,Ltd",
    "045C8E": "gosund GROUP CO.,LTD",
    "7CC4EF": "Devialet",
    "D85DFB": "Private",
    "5C5BC2": "YIK Corporation",
    "300D2A": "Zhejiang Wellcom Technology Co.,Ltd.",
    "3C189F": "Nokia Corporation",
    "08CD9B": "samtec automotive electronics & software GmbH",
    "30595B": "streamnow AG",
    "84850A": "Hella Sonnen- und Wetterschutztechnik GmbH",
    "EC2E4E": "HITACHI-LG DATA STORAGE INC",
    "DCF110": "Nokia Corporation",
    "908C63": "GZ Weedong Networks Technology Co. , Ltd",
    "D46761": "XonTel Technology Co.",
    "3481C4": "AVM GmbH",
    "608F5C": "Samsung Electronics Co.,Ltd",
    "E8EF89": "OPMEX Tech.",
    "F84A73": "EUMTECH CO., LTD",
    "142BD6": "Guangdong Appscomm Co.,Ltd",
    "CCB691": "NECMagnusCommunications",
    "40167E": "ASUSTek COMPUTER INC.",
    "983713": "PT.Navicom Indonesia",
    "A47E39": "zte corporation",
    "18CC23": "Philio Technology Corporation",
    "98349D": "Krauss Maffei Technologies GmbH",
    "880FB6": "Jabil Circuits India Pvt Ltd,-EHTP unit",
    "B46698": "Zealabs srl",
    "687CC8": "Measurement Systems S. de R.L.",
    "74F85D": "Berkeley Nucleonics Corp",
    "B061C7": "Ericsson-LG Enterprise",
    "400107": "Arista Corp",
    "30C750": "MIC Technology Group",
    "4411C2": "Telegartner Karl Gartner GmbH",
    "8059FD": "Noviga",
    "0092FA": "SHENZHEN WISKY TECHNOLOGY CO.,LTD",
    "100F18": "Fu Gang Electronic(KunShan)CO.,LTD",
    "D0C7C0": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "FCC2DE": "Murata Manufacturing Co., Ltd.",
    "5CE7BF": "New Singularity International Technical Development Co.,Ltd",
    "386C9B": "Ivy Biomedical",
    "B42C92": "Zhejiang Weirong Electronic Co., Ltd",
    "E0D31A": "EQUES Technology Co., Limited",
    "447E76": "Trek Technology (S) Pte Ltd",
    "B0EC8F": "GMX SAS",
    "4C7F62": "Nokia Corporation",
    "580528": "LABRIS NETWORKS",
    "407875": "IMBEL - Industria de Material Belico do Brasil",
    "D881CE": "AHN INC.",
    "28C825": "DellKing Industrial Co., Ltd",
    "80618F": "Shenzhen sangfei consumer communications co.,ltd",
    "D82A15": "Leitner SpA",
    "28DEF6": "bioMerieux Inc.",
    "987770": "Pep Digital Technology (Guangzhou) Co., Ltd",
    "BC14EF": "ITON Technology Limited",
    "D87CDD": "SANIX INCORPORATED",
    "34466F": "HiTEM Engineering",
    "68D247": "Portalis LC",
    "50B695": "Micropoint Biotechnologies,Inc.",
    "B4430D": "Broadlink Pty Ltd",
    "50A054": "Actineon",
    "B48547": "Amptown System Company GmbH",
    "748F1B": "MasterImage 3D",
    "083F76": "Intellian Technologies, Inc.",
    "A07771": "Vialis BV",
    "10DDF4": "Maxway Electronics CO.,LTD",
    "387B47": "AKELA, Inc.",
    "C064C6": "Nokia Corporation",
    "14F28E": "ShenYang ZhongKe-Allwin Technology Co.LTD",
    "5056A8": "Jolla Ltd",
    "A06518": "VNPT TECHNOLOGY",
    "7C8D91": "Shanghai Hongzhuo Information Technology co.,LTD",
    "080371": "KRG CORPORATE",
    "200E95": "IEC \u2013 TC9 WG43",
    "C8F68D": "S.E.TECHNOLOGIES LIMITED",
    "6C641A": "Penguin Computing",
    "CC89FD": "Nokia Corporation",
    "707C18": "ADATA Technology Co., Ltd",
    "78EC74": "Kyland-USA",
    "783D5B": "TELNET Redes Inteligentes S.A.",
    "D0B523": "Bestcare Cloucal Corp.",
    "24A495": "Thales Canada Inc.",
    "D0C42F": "Tamagawa Seiki Co.,Ltd.",
    "549359": "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD.",
    "C0F991": "GME Standard Communications P/L",
    "90356E": "Vodafone Omnitel N.V.",
    "5C1193": "Seal One AG",
    "847616": "Addat s.r.o.",
    "DC0575": "SIEMENS ENERGY AUTOMATION",
    "E40439": "TomTom Software Ltd",
    "E097F2": "Atomax Inc.",
    "70305E": "Nanjing Zhongke Menglian Information Technology Co.,LTD",
    "C098E5": "University of Michigan",
    "50E14A": "Private",
    "0C1262": "zte corporation",
    "3CD4D6": "WirelessWERX, Inc",
    "705986": "OOO TTV",
    "844F03": "Ablelink Electronics Ltd",
    "E8E770": "Warp9 Tech Design, Inc.",
    "609620": "Private",
    "443C9C": "Pintsch GmbH",
    "8CCDA2": "ACTP, Inc.",
    "84262B": "Nokia",
    "986CF5": "zte corporation",
    "906717": "Alphion India Private Limited",
    "6064A1": "RADiflow Ltd.",
    "9CF8DB": "shenzhen eyunmei technology co,.ltd",
    "0C473D": "Hitron Technologies. Inc",
    "680AD7": "Yancheng Kecheng Optoelectronic Technology Co., Ltd",
    "BC8893": "VILLBAU Ltd.",
    "447BC4": "DualShine Technology(SZ)Co.,Ltd",
    "407496": "aFUN TECHNOLOGY INC.",
    "701D7F": "Comtech Technology Co., Ltd.",
    "9C039E": "Beijing Winchannel Software Technology Co., Ltd",
    "708D09": "Nokia Corporation",
    "98FB12": "Grand Electronics (HK) Ltd",
    "3C1040": "daesung network",
    "28FC51": "The Electric Controller and Manufacturing Co., LLC",
    "20D21F": "Wincal Technology Corp.",
    "F89550": "Proton Products Chengdu Ltd",
    "7C49B9": "Plexus Manufacturing Sdn Bhd",
    "9C2840": "Discovery Technology,LTD..",
    "1C7B21": "Sony Corporation",
    "6CF97C": "Nanoptix Inc.",
    "F8FF5F": "Shenzhen Communication Technology Co.,Ltd",
    "44700B": "IFFU",
    "1C4158": "Gemalto M2M GmbH",
    "BC2B6B": "Beijing Haier IC Design Co.,Ltd",
    "98D331": "Shenzhen Bolutek Technology Co.,Ltd.",
    "38EC11": "Novatek Microelectronics Corp.",
    "4CCBF5": "zte corporation",
    "187ED5": "shenzhen kaism technology Co. Ltd",
    "841B38": "Shenzhen Excelsecu Data Technology Co.,Ltd",
    "E0AF4B": "Pluribus Networks, Inc.",
    "54A54B": "NSC Communications Siberia Ltd",
    "EC2257": "JiangSu NanJing University Electronic Information Technology Co.,Ltd",
    "F037A1": "Huike Electronics (SHENZHEN) CO., LTD.",
    "58B961": "SOLEM Electronique",
    "78491D": "The Will-Burt Company",
    "F46ABC": "Adonit Corp. Ltd.",
    "840F45": "Shanghai GMT Digital Technologies Co., Ltd",
    "2C5FF3": "Pertronic Industries",
    "58639A": "TPL SYSTEMES",
    "28C671": "Yota Devices OY",
    "D86960": "Steinsvik",
    "08EF3B": "MCS Logic Inc.",
    "E8EADA": "Denkovi Assembly Electronics LTD",
    "F85BC9": "M-Cube Spa",
    "907A0A": "Gebr. Bode GmbH & Co KG",
    "F47A4E": "Woojeon&Handan",
    "04848A": "7INOVA TECHNOLOGY LIMITED",
    "7CB77B": "Paradigm Electronics Inc",
    "B0CE18": "Zhejiang shenghui lighting co.,Ltd",
    "A0C6EC": "ShenZhen ANYK Technology Co.,LTD",
    "78E8B6": "zte corporation",
    "DCAE04": "CELOXICA Ltd",
    "8005DF": "Montage Technology Group Limited",
    "102279": "ZeroDesktop, Inc.",
    "7C1AFC": "Dalian Co-Edifice Video Technology Co., Ltd",
    "F08EDB": "VeloCloud Networks",
    "681D64": "Sunwave Communications Co., Ltd",
    "704CED": "TMRG, Inc.",
    "C47F51": "Inventek Systems",
    "A897DC": "IBM",
    "109AB9": "Tosibox Oy",
    "142D8B": "Incipio Technologies, Inc",
    "CCD29B": "Shenzhen Bopengfa Elec&Technology CO.,Ltd",
    "78DAB3": "GBO Technology",
    "700FEC": "Poindus Systems Corp.",
    "68EE96": "Cisco SPVTG",
    "78D38D": "HONGKONG YUNLINK TECHNOLOGY LIMITED",
    "1078CE": "Hanvit SI, Inc.",
    "D41090": "iNFORM Systems AG",
    "3495DB": "Logitec Corporation",
    "9CB793": "Creatcomm Technology Inc.",
    "5C335C": "Swissphone Telecom AG",
    "D8DA52": "APATOR S.A.",
    "587A4D": "Stonesoft Corporation",
    "F02405": "OPUS High Technology Corporation",
    "84E629": "Bluwan SA",
    "04DF69": "Car Connectivity Consortium",
    "4C21D0": "Sony Corporation",
    "78D5B5": "NAVIELEKTRO KY",
    "CCBD35": "Steinel GmbH",
    "6CECA1": "SHENZHEN CLOU ELECTRONICS CO. LTD.",
    "105C3B": "Perma-Pipe, Inc.",
    "349D90": "Heinzmann GmbH & CO. KG",
    "B03850": "Nanjing CAS-ZDC IOT SYSTEM CO.,LTD",
    "E8481F": "Advanced Automotive Antennas",
    "D40BB9": "Solid Semecs bv.",
    "F415FD": "Shanghai Pateo Electronic Equipment Manufacturing Co., Ltd.",
    "748E08": "Bestek Corp.",
    "78F5E5": "BEGA Gantenbrink-Leuchten KG",
    "C47DFE": "A.N. Solutions GmbH",
    "D862DB": "Eno Inc.",
    "8C3C07": "Skiva Technologies, Inc.",
    "E4F7A1": "Datafox GmbH",
    "381766": "PROMZAKAZ LTD.",
    "1441E2": "Monaco Enterprises, Inc.",
    "E47D5A": "Beijing Hanbang Technology Corp.",
    "70E027": "HONGYU COMMUNICATION TECHNOLOGY LIMITED",
    "A09BBD": "Total Aviation Solutions Pty Ltd",
    "38A86B": "Orga BV",
    "F07765": "Sourcefire, Inc",
    "ECD040": "GEA Farm Technologies GmbH",
    "F80DEA": "ZyCast Technology Inc.",
    "B08807": "Strata Worldwide",
    "5CF370": "CC&C Technologies, Inc",
    "A4E0E6": "FILIZOLA S.A. PESAGEM E AUTOMACAO",
    "249504": "SFR",
    "F45842": "Boxx TV Ltd",
    "106682": "NEC Platforms, Ltd.",
    "D81EDE": "B&W Group Ltd",
    "F084C9": "zte corporation",
    "D4016D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "985C93": "SBG Systems SAS",
    "A08A87": "HuiZhou KaiYue Electronic Co.,Ltd",
    "386793": "Asia Optical Co., Inc.",
    "0C8268": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C0C3B6": "Automatic Systems",
    "A0EB76": "AirCUVE Inc.",
    "FC4499": "Swarco LEA d.o.o.",
    "DC647C": "C.R.S. iiMotion GmbH",
    "148692": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A8154D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "24EA40": "Helmholz GmbH & Co. KG",
    "FCDB96": "ENERVALLEY CO., LTD",
    "1423D7": "EUTRONIX CO., LTD.",
    "28CD9C": "Shenzhen Dynamax Software Development Co.,Ltd.",
    "504F94": "Loxone Electronics GmbH",
    "60B185": "ATH system",
    "745F00": "Samsung Semiconductor Inc.",
    "E0C3F3": "zte corporation",
    "5C20D0": "Asoni Communication Co., Ltd.",
    "ACA430": "Peerless AV",
    "44184F": "Fitview",
    "18E8DD": "MODULETEK",
    "D073D5": "LIFI LABS MANAGEMENT PTY LTD",
    "149448": "BLU CASTLE S.A.",
    "48F925": "Maestronic",
    "68831A": "Pandora Mobility Corporation",
    "D429EA": "Zimory GmbH",
    "34ADE4": "Shanghai Chint Power Systems Co., Ltd.",
    "541FD5": "Advantage Electronics",
    "ACE97F": "IoT Tech Limited",
    "A0FE91": "AVAT Automation GmbH",
    "74ECF1": "Acumen",
    "90DA4E": "AVANU",
    "281878": "Microsoft Corporation",
    "7038B4": "Low Tech Solutions",
    "84ACA4": "Beijing Novel Super Digital TV Technology Co., Ltd",
    "5809E5": "Kivic Inc.",
    "DC6F08": "Bay Storage Technology",
    "BC629F": "Telenet Systems P. Ltd.",
    "380FE4": "Dedicated Network Partners Oy",
    "847A88": "HTC Corporation",
    "A4D856": "Gimbal, Inc",
    "785517": "SankyuElectronics",
    "B47F5E": "Foresight Manufacture (S) Pte Ltd",
    "B85AF7": "Ouya, Inc",
    "34F62D": "SHARP Corporation",
    "4C8FA5": "Jastec",
    "78324F": "Millennium Group, Inc.",
    "48F230": "Ubizcore Co.,LTD",
    "384369": "Patrol Products Consortium LLC",
    "E85AA7": "LLC Emzior",
    "D0D6CC": "Wintop",
    "58D071": "BW Broadcast",
    "20858C": "Assa",
    "1C52D6": "FLAT DISPLAY TECHNOLOGY CORPORATION",
    "D0DFB2": "Genie Networks Limited",
    "80FA5B": "CLEVO CO.",
    "C0B339": "Comigo Ltd.",
    "84ED33": "BBMC Co.,Ltd",
    "E82E24": "Out of the Fog Research LLC",
    "386645": "OOSIC Technology CO.,Ltd",
    "C0A0E2": "Eden Innovations",
    "6C5A34": "Shenzhen Haitianxiong Electronic Co., Ltd.",
    "DCB058": "B\u00fcrkert Werke GmbH",
    "9038DF": "Changzhou Tiannengbo System Co. Ltd.",
    "185253": "Pixord Corporation",
    "9C9C1D": "Starkey Labs Inc.",
    "9CE1D6": "Junger Audio-Studiotechnik GmbH",
    "D8182B": "Conti Temic Microelectronic GmbH",
    "80CF41": "Lenovo Mobile Communication Technology Ltd.",
    "58D6D3": "Dairy Cheq Inc",
    "046E49": "TaiYear Electronic Technology (Suzhou) Co., Ltd",
    "B49DB4": "Axion Technologies Inc.",
    "ACE87E": "Bytemark Computer Consulting Ltd",
    "60CDC5": "Taiwan Carol Electronics., Ltd",
    "2C3BFD": "Netstor Technology Co., Ltd.",
    "AC3CB4": "Nilan A/S",
    "8007A2": "Esson Technology Inc.",
    "080FFA": "KSP INC.",
    "683B1E": "Countwise LTD",
    "2091D9": "I'M SPA",
    "141BF0": "Intellimedia Systems Ltd",
    "5887E2": "Shenzhen Coship Electronics Co., Ltd.",
    "F46DE2": "zte corporation",
    "503955": "Cisco SPVTG",
    "808287": "ATCOM Technology Co.Ltd.",
    "28A186": "enblink",
    "6C9AC9": "Valentine Research, Inc.",
    "10FBF0": "KangSheng LTD.",
    "AC7236": "Lexking Technology Co., Ltd.",
    "3CD7DA": "SK Mtek microelectronics(shenzhen)limited",
    "04F8C2": "Flaircomm Microelectronics, Inc.",
    "6869F2": "ComAp s.r.o.",
    "B85AFE": "Handaer Communication Technology (Beijing) Co., Ltd",
    "ACA22C": "Baycity Technologies Ltd",
    "303294": "W-IE-NE-R Plein & Baus GmbH",
    "7C822D": "Nortec",
    "AC8D14": "Smartrove Inc",
    "388EE7": "Fanhattan LLC",
    "CCE798": "My Social Stuff",
    "A036F0": "Comprehensive Power",
    "180CAC": "CANON INC.",
    "00DB1E": "Albedo Telecom SL",
    "60748D": "Atmaca Elektronik",
    "B8B7D7": "2GIG Technologies",
    "78D129": "Vicos",
    "84DF0C": "NET2GRID BV",
    "78AB60": "ABB Australia",
    "8482F4": "Beijing Huasun Unicreate Technology Co., Ltd",
    "5CD41B": "UCZOON Technology Co., LTD",
    "2CEDEB": "Alpheus Digital Company Limited",
    "0CDCCC": "Inala Technologies",
    "0CD996": "Cisco Systems, Inc",
    "30F33A": "+plugg srl",
    "98291D": "Jaguar de Mexico, SA de CV",
    "34AF2C": "Nintendo Co., Ltd.",
    "7CD9FE": "New Cosmos Electric Co., Ltd.",
    "CCC104": "Applied Technical Systems",
    "A4B1E9": "Technicolor Delivery Technologies Belgium NV",
    "60455E": "Liptel s.r.o.",
    "D806D1": "Honeywell Fire System (Shanghai) Co,. Ltd.",
    "647657": "Innovative Security Designs",
    "0C57EB": "Mueller Systems",
    "745327": "COMMSEN CO., LIMITED",
    "D08CFF": "UPWIS AB",
    "68D1FD": "Shenzhen Trimax Technology Co.,Ltd",
    "9C066E": "Hytera Communications Corporation Limited",
    "642216": "Shandong Taixin Electronic co.,Ltd",
    "F8A03D": "Dinstar Technologies Co., Ltd.",
    "2CD444": "FUJITSU LIMITED",
    "907025": "Garea Microsys Co.,Ltd.",
    "10D1DC": "INSTAR Deutschland GmbH",
    "34996F": "VPI Engineering",
    "5869F9": "Fusion Transactive Ltd.",
    "D41E35": "TOHO Electronics INC.",
    "98A7B0": "MCST ZAO",
    "4C068A": "Basler Electric Company",
    "BC811F": "Ingate Systems",
    "D867D9": "Cisco Systems, Inc",
    "944A09": "BitWise Controls",
    "BC28D6": "Rowley Associates Limited",
    "10BD18": "Cisco Systems, Inc",
    "443839": "Cumulus Networks, inc",
    "A4E731": "Nokia Corporation",
    "4C72B9": "PEGATRON CORPORATION",
    "68D925": "ProSys Development Services",
    "848D84": "Rajant Corporation",
    "D8337F": "Office FA.com Co.,Ltd.",
    "0036F8": "Conti Temic microelectronic GmbH",
    "A4F7D0": "LAN Accessories Co., Ltd.",
    "048B42": "Skspruce Technologies",
    "5076A6": "Ecil Informatica Ind. Com. Ltda",
    "A44C11": "Cisco Systems, Inc",
    "60843B": "Soladigm, Inc.",
    "209BA5": "JIAXING GLEAD Electronics Co.,Ltd",
    "A0F450": "HTC Corporation",
    "6089B1": "Key Digital Systems",
    "44D15E": "Shanghai Kingto Information Technology Ltd",
    "0036FE": "SuperVision",
    "3CEAFB": "NSE AG",
    "8CC7AA": "Radinet Communications Inc.",
    "40336C": "Godrej & Boyce Mfg. co. ltd",
    "489153": "Weinmann Ger\u00e4te f\u00fcr Medizin GmbH + Co. KG",
    "AC9403": "Envision Peripherals Inc",
    "1C973D": "PRICOM Design",
    "BC0200": "Stewart Audio",
    "E856D6": "NCTech Ltd",
    "C08170": "Effigis GeoSolutions",
    "60B933": "Deutron Electronics Corp.",
    "54466B": "Shenzhen CZTIC Electronic Technology Co., Ltd  ",
    "44B382": "Kuang-chi Institute of Advanced Technology",
    "709E86": "X6D Limited",
    "A0F419": "Nokia Corporation",
    "0043FF": "KETRON S.R.L.",
    "7CACB2": "Bosch Software Innovations GmbH",
    "18D66A": "Inmarsat",
    "28E608": "Tokheim",
    "C47BA3": "NAVIS Inc.",
    "F44848": "Amscreen Group Ltd",
    "50D274": "Steffes Corporation",
    "F85063": "Verathon",
    "F0D14F": "LINEAR LLC",
    "5C6F4F": "S.A. SISTEL",
    "901B0E": "Fujitsu Technology Solutions GmbH",
    "74FF7D": "Wren Sound Systems, LLC",
    "ACF0B2": "Becker Electronics Taiwan Ltd.",
    "C85645": "Intermas France",
    "44348F": "MXT INDUSTRIAL LTDA",
    "2C36F8": "Cisco Systems, Inc",
    "5808FA": "Fiber Optic & telecommunication INC.",
    "845787": "DVR C&C Co., Ltd.",
    "AC3D05": "Instorescreen Aisa",
    "504A5E": "Masimo Corporation",
    "1C7C45": "Vitek Industrial Video Products, Inc.",
    "3C3888": "ConnectQuest, llc",
    "48D7FF": "BLANKOM Antennentechnik GmbH",
    "C47130": "Fon Technology S.L.",
    "D4EC0C": "Harley-Davidson Motor Company",
    "6CA96F": "TransPacket AS",
    "AC0142": "Uriel Technologies SIA",
    "542A9C": "LSY Defense, LLC.",
    "C43C3C": "CYBELEC SA",
    "B826D4": "Furukawa Industrial S.A. Produtos El\u00e9tricos",
    "B87447": "Convergence Technologies",
    "7463DF": "VTS GmbH",
    "BC125E": "Beijing  WisVideo  INC.",
    "14E4EC": "mLogic LLC",
    "C8F704": "Building Block Video",
    "508A42": "Uptmate Technology Co., LTD",
    "BCEA2B": "CityCom GmbH",
    "A45630": "Cisco Systems, Inc",
    "0C9E91": "Sankosha Corporation",
    "C8F9F9": "Cisco Systems, Inc",
    "80DB31": "Power Quotient International Co., Ltd.",
    "E05DA6": "Detlef Fink Elektronik & Softwareentwicklung",
    "846AED": "Wireless Tsukamoto.,co.LTD",
    "045A95": "Nokia Corporation",
    "04F4BC": "Xena Networks",
    "6C3A84": "Shenzhen Aero-Startech. Co.Ltd",
    "00D632": "GE Energy",
    "1C51B5": "Techaya LTD",
    "D01AA7": "UniPrint",
    "383F10": "DBL Technology Ltd.",
    "286094": "CAPELEC",
    "ACD364": "ABB SPA, ABB SACE DIV.",
    "A4EF52": "Telewave Co., Ltd.",
    "A826D9": "HTC Corporation",
    "28940F": "Cisco Systems, Inc",
    "B8DAF7": "Advanced Photonics, Inc.",
    "143AEA": "Dynapower Company LLC",
    "A086EC": "SAEHAN HITEC Co., Ltd",
    "942E17": "Schneider Electric Canada Inc",
    "98FE03": "Ericsson - North America",
    "20AA4B": "Cisco-Linksys, LLC",
    "CC944A": "Pfeiffer Vacuum GmbH",
    "0C8525": "Cisco Systems, Inc",
    "B4D8A9": "BetterBots",
    "7CC8D7": "Damalisk",
    "C46044": "Everex Electronics Limited",
    "9CB008": "Ubiquitous Computing Technology Corporation",
    "A8776F": "Zonoff",
    "00FA3B": "CLOOS ELECTRONIC GMBH",
    "2838CF": "Gen2wave",
    "E03C5B": "SHENZHEN JIAXINJIE ELECTRON CO.,LTD",
    "3828EA": "Fujian Netcom Technology Co., LTD",
    "2CEE26": "Petroleum Geo-Services",
    "FC8FC4": "Intelligent Technology Inc.",
    "541DFB": "Freestyle Energy Ltd",
    "60B606": "Phorus",
    "9092B4": "Diehl BGT Defence GmbH & Co. KG",
    "FC455F": "JIANGXI SHANSHUI OPTOELECTRONIC TECHNOLOGY CO.,LTD",
    "4833DD": "ZENNIO AVANCE Y TECNOLOGIA, S.L.",
    "10FC54": "Shany Electronic Co., Ltd. ",
    "C02973": "Audyssey Laboratories Inc.",
    "98AAD7": "BLUE WAVE NETWORKING CO LTD",
    "9C53CD": "ENGICAM s.r.l.",
    "608645": "Avery Weigh-Tronix, LLC",
    "64C5AA": "South African Broadcasting Corporation",
    "CC6DEF": "TJK Tietolaite Oy",
    "A85BF3": "Audivo GmbH",
    "B8975A": "BIOSTAR Microtech Int'l Corp.",
    "24C0B3": "RSF",
    "603553": "Buwon Technology",
    "E039D7": "Plexxi, Inc.",
    "24BC82": "Dali Wireless, Inc.",
    "087572": "Obelux Oy",
    "10C2BA": "UTT Co., Ltd.",
    "90D74F": "Bookeen",
    "500B32": "Foxda Technology Industrial(ShenZhen)Co.,LTD",
    "F04A2B": "PYRAMID Computer GmbH",
    "4C32D9": "M Rutty Holdings Pty. Ltd.",
    "68CD0F": "U Tek Company Limited",
    "A4E391": "DENY FONTAINE",
    "603FC5": "COX CO., LTD",
    "AC6FD9": "Valueplus Inc.",
    "90A783": "JSW PACIFIC CORPORATION ",
    "28AF0A": "Sirius XM Radio Inc",
    "5CD4AB": "Zektor",
    "08FC52": "OpenXS BV",
    "F8462D": "SYNTEC Incorporation",
    "78A5DD": "Shenzhen Smarteye Digital Electronics Co., Ltd",
    "ECE744": "Omntec mfg. inc",
    "28D1AF": "Nokia Corporation",
    "64E84F": "Serialway Communication Technology Co. Ltd",
    "2C9EFC": "CANON INC.",
    "DC1EA3": "Accensus LLC",
    "A40130": "ABIsystems Co., LTD",
    "302DE8": "JDA, LLC (JDA Systems)",
    "48A6D2": "GJsun Optical Science and Tech Co.,Ltd.",
    "7C336E": "MEG Electronics Inc.",
    "182B05": "8D Technologies",
    "08A12B": "ShenZhen EZL Technology Co., Ltd",
    "A00CA1": "SKTB SKiT",
    "F8F7D3": "International Communications Corporation",
    "D4E33F": "Nokia",
    "B40C25": "Palo Alto Networks",
    "240BB1": "KOSTAL Industrie Elektrik GmbH ",
    "20EEC6": "Elefirst Science & Tech Co ., ltd",
    "E01E07": "Anite Telecoms  US. Inc",
    "7C6B33": "Tenyu Tech Co. Ltd.",
    "64D989": "Cisco Systems, Inc",
    "147DC5": "Murata Manufacturing Co., Ltd.",
    "00B9F6": "Shenzhen Super Rich Electronics Co.,Ltd",
    "FCC23D": "Atmel Corporation",
    "644346": "GuangDong Quick Network Computer CO.,LTD",
    "CCE7DF": "American Magnetics, Inc.",
    "A446FA": "AmTRAN Video Corporation",
    "2804E0": "FERMAX ELECTRONICA S.A.U.",
    "0462D7": "ALSTOM HYDRO FRANCE",
    "D4507A": "CEIVA Logic, Inc",
    "88E7A6": "iKnowledge Integration Corp.",
    "D4024A": "Delphian Systems LLC",
    "0041B4": "Wuxi Zhongxing Optoelectronics Technology Co.,Ltd.",
    "F44450": "BND Co., Ltd.",
    "645DD7": "Shenzhen Lifesense Medical Electronics Co., Ltd.    ",
    "EC4670": "Meinberg Funkuhren GmbH & Co. KG",
    "D05A0F": "I-BT DIGITAL CO.,LTD",
    "EC9681": "2276427 Ontario Inc",
    "5C076F": "Thought Creator",
    "3C0FC1": "KBC Networks",
    "58E636": "EVRsafe Technologies",
    "10F9EE": "Nokia Corporation",
    "742B0F": "Infinidat Ltd.",
    "C8F981": "Seneca s.r.l.",
    "14307A": "Avermetrics",
    "A06CEC": "RIM",
    "24497B": "Innovative Converged Devices Inc",
    "98E79A": "Foxconn(NanJing) Communication Co.,Ltd.",
    "A0E9DB": "Ningbo FreeWings Technologies Co.,Ltd",
    "788973": "CMC",
    "203706": "Cisco Systems, Inc",
    "90B97D": "Johnson Outdoors Marine Electronics d/b/a Minnkota",
    "F4B164": "Lightning Telecommunications Technology Co. Ltd",
    "70B035": "Shenzhen Zowee Technology Co., Ltd",
    "8821E3": "Nebusens, S.L.",
    "7CF429": "NUUO Inc. ",
    "1045BE": "Norphonic AS",
    "8C82A8": "Insigma Technology Co.,Ltd",
    "CCB55A": "Fraunhofer ITWM",
    "AC8ACD": "ROGER D.Wensker, G.Wensker sp.j.",
    "984246": "SOL INDUSTRY PTE., LTD",
    "3429EA": "MCD ELECTRONICS SP. Z O.O.",
    "28A574": "Miller Electric Mfg. Co.",
    "90B8D0": "Joyent, Inc.",
    "F80332": "Khomp",
    "60190C": "RRAMAC",
    "D05FCE": "Hitachi Data Systems",
    "D4A425": "SMAX Technology Co., Ltd.",
    "8C11CB": "ABUS Security-Center GmbH & Co. KG",
    "D09B05": "Emtronix",
    "AC4723": "Genelec",
    "E8BA70": "Cisco Systems, Inc",
    "FC8329": "Trei technics",
    "14EB33": "BSMediasoft Co., Ltd.",
    "F4B549": "Xiamen Yeastar Information Technology Co., Ltd.",
    "88B168": "Delta Control GmbH",
    "AC8674": "Open Mesh, Inc.",
    "68876B": "INQ Mobile Limited",
    "1CAA07": "Cisco Systems, Inc",
    "685B36": "POWERTECH INDUSTRIAL CO., LTD.",
    "28EE2C": "Frontline Test Equipment",
    "782EEF": "Nokia Corporation",
    "7CF0BA": "Linkwell Telesystems Pvt Ltd",
    "94D93C": "ENELPS",
    "B8BEBF": "Cisco Systems, Inc",
    "64B64A": "ViVOtech, Inc.",
    "8C4435": "Shanghai BroadMobi Communication Technology Co., Ltd.",
    "F81D93": "Longdhua(Beijing) Controls Technology Co.,Ltd",
    "CCF841": "Lumewave",
    "D8DF0D": "beroNet GmbH",
    "ACF97E": "ELESYS INC.",
    "204005": "feno GmbH",
    "64D241": "Keith & Koep GmbH",
    "18B79E": "Invoxia",
    "281471": "Lantis co., LTD.",
    "38D135": "EasyIO Corporation Sdn. Bhd.",
    "901900": "SCS SA",
    "D45D42": "Nokia Corporation",
    "B03829": "Siliconware Precision Industries Co., Ltd.",
    "7C6C39": "PIXSYS SRL",
    "BC2846": "NextBIT Computing Pvt. Ltd.",
    "BC0F2B": "FORTUNE TECHGROUP CO.,LTD",
    "30EB25": "INTEK DIGITAL",
    "44E4D9": "Cisco Systems, Inc",
    "ACCA54": "Telldus Technologies AB",
    "648125": "Alphatron Marine BV",
    "8CF9C9": "MESADA Technology Co.,Ltd.",
    "042605": "Bosch Building Automation GmbH",
    "24F0FF": "GHT Co., Ltd.",
    "C0626B": "Cisco Systems, Inc",
    "94E226": "D. ORtiz Consulting, LLC",
    "18B3BA": "Netlogic AB",
    "D47B75": "HARTING Electronics GmbH",
    "8C5FDF": "Beijing Railway Signal Factory",
    "300B9C": "Delta Mobile Systems, Inc.",
    "D46F42": "WAXESS USA Inc",
    "04C5A4": "Cisco Systems, Inc",
    "6CAD3F": "Hubbell Building Automation, Inc.",
    "9CC0D2": "Conductix-Wampfler GmbH",
    "DCD87F": "Shenzhen JoinCyber Telecom Equipment Ltd",
    "B4E0CD": "Fusion-io, Inc",
    "286046": "Lantech Communications Global, Inc.",
    "10E2D5": "Qi Hardware Inc.",
    "303955": "Shenzhen Jinhengjia Electronic Co., Ltd.",
    "FC5B24": "Weibel Scientific A/S",
    "EC986C": "Lufft Mess- und Regeltechnik GmbH",
    "CCF67A": "Ayecka Communication Systems LTD",
    "D8C99D": "EA DISPLAY LIMITED",
    "1083D2": "Microseven Systems, LLC",
    "D093F8": "Stonestreet One LLC",
    "9C645E": "Harman Consumer Group",
    "1C334D": "ITS Telecom",
    "4CB9C8": "CONET CO., LTD.",
    "34684A": "Teraworks Co., Ltd.",
    "CCFC6D": "RIZ TRANSMITTERS",
    "E03E7D": "data-complex GmbH",
    "0CC6AC": "DAGS",
    "78593E": "RAFI GmbH & Co.KG",
    "509772": "Westinghouse Digital",
    "8CB64F": "Cisco Systems, Inc",
    "503DE5": "Cisco Systems, Inc",
    "540496": "Gigawave LTD",
    "EC4644": "TTK SAS",
    "8065E9": "BenQ Corporation",
    "204AAA": "Hanscan Spain S.A.",
    "60C980": "Trymus",
    "A036FA": "Ettus Research LLC",
    "EC836C": "RM Tech Co., Ltd.",
    "20FECD": "System In Frontier Inc.",
    "F0933A": "NxtConect",
    "5CF3FC": "IBM Corp",
    "582F42": "Universal Electric Corporation",
    "0474A1": "Aligera Equipamentos Digitais Ltda",
    "5C6984": "NUVICO",
    "B8415F": "ASP AG",
    "2CB69D": "RED Digital Cinema",
    "A86A6F": "RIM",
    "500E6D": "TrafficCast International",
    "EC3BF0": "NovelSat",
    "70DDA1": "Tellabs",
    "94D019": "Cydle Corp.",
    "8C278A": "Vocollect Inc",
    "CC0CDA": "Miljovakt AS",
    "E41C4B": "V2 TECHNOLOGY, INC.",
    "B8FF6F": "Shanghai Typrotech Technology Co.Ltd",
    "68122D": "Special Instrument Development Co., Ltd.",
    "94F720": "Tianjin Deviser Electronics Instrument Co., Ltd",
    "DC9C52": "Sapphire Technology Limited.",
    "4891F6": "Shenzhen Reach software technology CO.,LTD",
    "649B24": "V Technology Co., Ltd.",
    "846EB1": "Park Assist LLC",
    "6C504D": "Cisco Systems, Inc",
    "B41489": "Cisco Systems, Inc",
    "A0B5DA": "HongKong THTF Co., Ltd",
    "8886A0": "Simton Technologies, Ltd.",
    "A45055": "BUSWARE.DE",
    "ACAB8D": "Lyngso Marine A/S",
    "6083B2": "GkWare e.K.",
    "80D019": "Embed, Inc",
    "68EBC5": "Angstrem Telecom",
    "E8995A": "PiiGAB, Processinformation i Goteborg AB",
    "401D59": "Biometric Associates, LP",
    "B8D06F": "GUANGZHOU HKUST FOK YING TUNG RESEARCH INSTITUTE",
    "EC14F6": "BioControl AS",
    "1CBD0E": "Amplified Engineering Pty Ltd",
    "A0F217": "GE Medical System(China) Co., Ltd. ",
    "F0A764": "GST Co., Ltd.",
    "1C0656": "IDY Corporation",
    "4CEDDE": "ASKEY COMPUTER CORP",
    "E8E08F": "GRAVOTECH MARKING SAS",
    "A89B10": "inMotion Ltd.",
    "A4A80F": "Shenzhen Coship Electronics Co., Ltd.",
    "F8B599": "Guangzhou CHNAVS Digital Technology Co.,Ltd",
    "B8921D": "BG T&A",
    "80C6CA": "Endian s.r.l.",
    "C88B47": "Nolangroup S.P.A con Socio Unico",
    "C4CD45": "Beijing Boomsense Technology CO.,LTD.",
    "54FDBF": "Scheidt & Bachmann GmbH",
    "D0BB80": "SHL Telemedicine International Ltd.",
    "E061B2": "HANGZHOU ZENOINTEL TECHNOLOGY CO., LTD",
    "9411DA": "ITF Fr\u00f6schl GmbH",
    "8039E5": "PATLITE CORPORATION",
    "D08999": "APCON, Inc.",
    "C88447": "Beautiful Enterprise Co., Ltd",
    "5C17D3": "LGE ",
    "1CDF0F": "Cisco Systems, Inc",
    "68BDAB": "Cisco Systems, Inc",
    "9CADEF": "Obihai Technology, Inc.",
    "90903C": "TRISON TECHNOLOGY CORPORATION",
    "8CE7B3": "Sonardyne International Ltd",
    "088DC8": "Ryowa Electronics Co.,Ltd",
    "94E711": "Xirka Dama Persada PT",
    "7076F0": "LevelOne Communications (India) Private Limited",
    "B081D8": "I-sys Corp",
    "6C9B02": "Nokia Corporation",
    "641E81": "Dowslake Microsystems",
    "EC542E": "Shanghai XiMei Electronic Technology Co. Ltd",
    "F0E5C3": "Dr\u00e4gerwerk AG & Co. KG aA",
    "5CCA32": "Theben AG",
    "C02BFC": "iNES. applied informatics GmbH",
    "94C7AF": "Raylios Technology",
    "043604": "Gyeyoung I&T",
    "A4B2A7": "Adaxys Solutions AG",
    "D0D0FD": "Cisco Systems, Inc",
    "14FEAF": "SAGITTAR LIMITED",
    "DC7B94": "Cisco Systems, Inc",
    "D81C14": "Compacta International, Ltd.",
    "008C10": "Black Box Corp.",
    "B0B8D5": "Nanjing Nengrui Auto Equipment CO.,Ltd",
    "D82986": "Best Wish Technology LTD",
    "446132": "ecobee inc",
    "E05B70": "Innovid, Co., Ltd.",
    "F41F0B": "YAMABISHI Corporation",
    "A082C7": "P.T.I Co.,LTD",
    "E87AF3": "S5 Tech S.r.l.",
    "7415E2": "Tri-Sen Systems Corporation",
    "D48FAA": "Sogecam Industrial, S.A.",
    "98FC11": "Cisco-Linksys, LLC",
    "34E0D7": "DONGGUAN QISHENG ELECTRONICS INDUSTRIAL CO., LTD",
    "D84606": "Silicon Valley Global Marketing",
    "CC5C75": "Weightech Com. Imp. Exp. Equip. Pesagem Ltda",
    "90E0F0": "IEEE 1722a Working Group",
    "FCD4F6": "Messana Air.Ray Conditioning s.r.l.",
    "D466A8": "Riedo Networks Ltd",
    "F8AC6D": "Deltenna Ltd",
    "18B209": "Torrey Pines Logic, Inc",
    "40520D": "Pico Technology",
    "807D1B": "Neosystem Co. Ltd.",
    "C848F5": "MEDISON Xray Co., Ltd",
    "1880CE": "Barberry Solutions Ltd",
    "D84B2A": "Cognitas Technologies, Inc.",
    "684B88": "Galtronics Telemetry Inc.",
    "A4AE9A": "Maestro Wireless Solutions ltd.",
    "78A714": "Amphenol",
    "F450EB": "Telechips Inc",
    "988EDD": "TE Connectivity Limerick",
    "141BBD": "Volex Inc.",
    "A4561B": "MCOT Corporation",
    "80C63F": "Remec Broadband Wireless , LLC",
    "40D40E": "Biodata Ltd",
    "0C826A": "Wuhan Huagong Genuine Optics Technology Co., Ltd",
    "7C051E": "RAFAEL LTD.",
    "7866AE": "ZTEC Instruments, Inc.",
    "4C022E": "CMR KOREA CO., LTD",
    "34AAEE": "Mikrovisatos Servisas UAB",
    "48FCB8": "Woodstream Corporation",
    "F893F3": "VOLANS",
    "78A2A0": "Nintendo Co., Ltd.",
    "24B6B8": "FRIEM SPA",
    "D4000D": "Phoenix Broadband Technologies, LLC.",
    "AC5135": "MPI TECH",
    "44D63D": "Talari Networks",
    "08F6F8": "GET Engineering",
    "10090C": "JANOME Corporation",
    "E01CEE": "Bravo Tech, Inc.",
    "74B9EB": "JinQianMao Technology Co.,Ltd.",
    "D45297": "nSTREAMS Technologies, Inc.",
    "E0271A": "TTC Next-generation Home Network System WG",
    "0097FF": "Heimann Sensor GmbH",
    "E4AB46": "UAB Selteka",
    "945B7E": "TRILOBIT LTDA.",
    "04E548": "Cohda Wireless Pty Ltd",
    "2893FE": "Cisco Systems, Inc",
    "F4C795": "WEY Technology AG",
    "781185": "NBS Payment Solutions Inc.",
    "E85E53": "Infratec Datentechnik GmbH",
    "88BA7F": "Qfiednet Co., Ltd.",
    "64DB18": "OpenPattern",
    "D05875": "Active Control Technology Inc.",
    "D81BFE": "TWINLINX CORPORATION",
    "7071BC": "PEGATRON CORPORATION",
    "7884EE": "INDRA ESPACIO S.A.",
    "FC4463": "Universal Audio, Inc",
    "90A2DA": "GHEO SA",
    "9889ED": "Anadem Information Inc.",
    "042F56": "ATOCS (Shenzhen) LTD",
    "C8EF2E": "Beijing Gefei Tech. Co., Ltd ",
    "A4C2AB": "Hangzhou LEAD-IT Information & Technology Co.,Ltd",
    "48AA5D": "Store Electronic Systems",
    "0CC9C6": "Samwin Hong Kong Limited",
    "1062C9": "Adatis GmbH & Co. KG",
    "D8AE90": "Itibia Technologies",
    "B8653B": "Bolymin, Inc.",
    "38E8DF": "b gmbh medien + datenbanken",
    "1C129D": "IEEE PES PSRC/SUB     ",
    "E0CA4D": "Shenzhen Unistar Communication Co.,LTD",
    "A06986": "Wellav Technologies Ltd",
    "EC8EAD": "DLX",
    "F0B6EB": "Poslab Technology Co., Ltd.",
    "1C8F8A": "Phase Motion Control SpA",
    "FCCCE4": "Ascon Ltd.",
    "60B3C4": "Elber Srl",
    "04C880": "Samtec Inc",
    "5C35DA": "There Corporation Oy",
    "3C4C69": "Infinity System S.L.",
    "7830E1": "UltraClenz, LLC",
    "B09134": "Taleo",
    "34C69A": "Enecsys Ltd",
    "F88DEF": "Tenebraex",
    "042234": "Wireless Standard Extensions",
    "80C862": "Openpeak, Inc",
    "102D96": "Looxcie Inc.",
    "5850E6": "Best Buy Corporation",
    "3C1CBE": "JADAK LLC",
    "7C08D9": "Shanghai B-Star Technology Co",
    "BCD5B6": "d2d technologies",
    "FC683E": "Directed Perception, Inc",
    "28E794": "Microtime Computer Inc.",
    "A438FC": "Plastic Logic",
    "18FC9F": "Changhe Electronics Co., Ltd.",
    "94592D": "EKE Building Technology Systems Ltd",
    "CC69B0": "Global Traffic Technologies, LLC",
    "A0593A": "V.D.S. Video Display Systems srl",
    "CCEA1C": "DCONWORKS  Co., Ltd",
    "0CD502": "Westell Technologies Inc.",
    "687F74": "Cisco-Linksys, LLC",
    "38E98C": "Reco S.p.A.",
    "34EF8B": "NTT Communications Corporation",
    "2059A0": "Paragon Technologies Inc.",
    "0C2755": "Valuable Techologies Limited",
    "F8472D": "X2gen Digital Corp. Ltd",
    "849000": "Arnold&Richter Cine Technik GmbH & Co. Betriebs KG",
    "08184C": "A. S. Thomas, Inc.",
    "10880F": "Daruma Telecomunica\u00e7\u00f5es e Inform\u00e1tica S.A.",
    "FC6198": "NEC Personal Products, Ltd",
    "74D850": "Evrisko Systems",
    "78998F": "MEDILINE ITALIA SRL",
    "34CE94": "Parsec (Pty) Ltd",
    "24D2CC": "SmartDrive Systems Inc.",
    "0CEF7C": "AnaCom Inc",
    "B05B1F": "THERMO FISHER SCIENTIFIC S.P.A.",
    "ECE9F8": "Guang Zhou TRI-SUN Electronics Technology  Co., Ltd",
    "D0D286": "Beckman Coulter K.K.",
    "BCB181": "SHARP CORPORATION",
    "A0BFA5": "CORESYS",
    "C8873B": "Net Optics",
    "2C0623": "Win Leader Inc.",
    "A8CE90": "CVC",
    "E41F13": "IBM Corp",
    "E4FFDD": "ELECTRON INDIA",
    "68A1B7": "Honghao Mingchuan Technology (Beijing) CO.,Ltd.",
    "0CD7C2": "Axium Technologies, Inc.",
    "C4198B": "Dominion Voting Systems Corporation",
    "C83A35": "Tenda Technology Co., Ltd.",
    "6C8CDB": "Otus Technologies Ltd",
    "40F52E": "Leica Microsystems (Schweiz) AG",
    "906DC8": "DLG Automa\u00e7\u00e3o Industrial Ltda",
    "E84ECE": "Nintendo Co., Ltd.",
    "1045F8": "LNT-Automation GmbH",
    "DCE71C": "AUG Elektronik GmbH",
    "A870A5": "UniComm Inc.",
    "4456B7": "Spawn Labs, Inc",
    "44C9A2": "Greenwald Industries",
    "406186": "MICRO-STAR INT'L CO.,LTD",
    "584CEE": "Digital One Technologies, Limited",
    "A07332": "Cashmaster International Limited",
    "64C6AF": "AXERRA Networks Ltd",
    "701AED": "ADVAS CO., LTD.",
    "6465C0": "Nuvon, Inc",
    "40ECF8": "Siemens AG",
    "54B620": "SUHDOL E&C Co.Ltd.",
    "78C40E": "H&D Wireless",
    "44568D": "PNC Technologies  Co., Ltd.",
    "7C1EB3": "2N TELEKOMUNIKACE a.s.",
    "0026DF": "TaiDoc Technology Corp.",
    "0026D8": "Magic Point Inc.",
    "0026D6": "Ningbo Andy Optoelectronic Co., Ltd.",
    "0026F8": "Golden Highway Industry Development Co., Ltd.",
    "0026F4": "Nesslab",
    "0026EE": "TKM GmbH",
    "0026EF": "Technology Advancement Group, Inc.",
    "0026E6": "Visionhitech Co., Ltd.",
    "002701": "INCOstartec GmbH",
    "0026FB": "AirDio Wireless, Inc.",
    "0026F5": "XRPLUS Inc.",
    "0026B2": "Setrix GmbH",
    "0026AF": "Duelco A/S",
    "0026B3": "Thales Communications Inc",
    "002721": "Shenzhen Baoan Fenda Industrial Co., Ltd",
    "0026D1": "S Squared Innovations Inc.",
    "0026D3": "Zeno Information System",
    "0026CB": "Cisco Systems, Inc",
    "00271A": "Geenovo Technology Ltd.",
    "002714": "Grainmustards, Co,ltd.",
    "002715": "Rebound Telecom. Co., Ltd",
    "00270F": "Envisionnovation Inc",
    "00270A": "IEE S.A.",
    "002709": "Nintendo Co., Ltd.",
    "0026A2": "Instrumentation Technology Systems",
    "00269F": "Private",
    "002699": "Cisco Systems, Inc",
    "002653": "DaySequerra Corporation",
    "002647": "WFE TECHNOLOGY CORP.",
    "00266A": "ESSENSIUM NV",
    "00266B": "SHINE UNION ENTERPRISE LIMITED",
    "002667": "CARECOM CO.,LTD.",
    "002640": "Baustem Broadband Technologies, Ltd.",
    "002633": "MIR - Medical International Research",
    "002630": "ACOREL S.A.S",
    "00262A": "Proxense, LLC",
    "002687": "corega K.K",
    "00267B": "GSI Helmholtzzentrum f\u00fcr Schwerionenforschung GmbH",
    "00268D": "CellTel S.p.A.",
    "002628": "companytec automa\u00e7\u00e3o e controle ltda.",
    "00261F": "SAE Magnetics (H.K.) Ltd.",
    "00261E": "QINGBANG ELEC(SZ) CO., LTD",
    "002619": "FRC",
    "0025F0": "Suga Electronics Limited",
    "0025E8": "Idaho Technology",
    "0025E4": "OMNI-WiFi, LLC",
    "0025C9": "SHENZHEN HUAPU DIGITAL CO., LTD",
    "0025FA": "J&M Analytik AG",
    "0025EA": "Iphion BV",
    "0025BE": "Tektrap Systems Inc.",
    "0025B5": "Cisco Systems, Inc",
    "0025FE": "Pilot Electronics Corporation",
    "0025D7": "CEDO",
    "0025D8": "KOREA MAINTENANCE",
    "0025D2": "InpegVision Co., Ltd",
    "00255B": "CoachComm, LLC",
    "00254D": "Singapore Technologies Electronics Limited",
    "00254C": "Videon Central, Inc.",
    "002543": "MONEYTECH",
    "00257B": "STJ  ELECTRONICS  PVT  LTD",
    "00257C": "Huachentel Technology Development Co., Ltd",
    "002575": "FiberPlex Technologies, LLC",
    "002570": "Eastern Communications Company Limited",
    "002596": "GIGAVISION srl",
    "002595": "Northwest Signal Supply, Inc",
    "00258F": "Trident Microsystems, Inc.",
    "002589": "Hills Industries Limited",
    "002585": "KOKUYO S&T Co., Ltd.",
    "002551": "SE-Elektronic GmbH",
    "00256A": "inIT - Institut Industrial IT",
    "002562": "interbro Co. Ltd.",
    "002581": "x-star networks Inc.",
    "0025A0": "Nintendo Co., Ltd.",
    "00259B": "Beijing PKUNITY Microsystems Technology Co., Ltd",
    "00253A": "CEVA, Ltd.",
    "0024F7": "Cisco Systems, Inc",
    "0024F3": "Nintendo Co., Ltd.",
    "0024E0": "DS Tech, LLC",
    "0024E2": "HASEGAWA ELECTRIC CO.,LTD.",
    "002516": "Integrated Design Tools, Inc.",
    "002510": "Pico-Tesla Magnetic Therapies",
    "002520": "SMA Railway Technology GmbH",
    "002527": "Bitrode Corp.",
    "002525": "CTERA Networks Ltd.",
    "0024C8": "Broadband Solutions Group",
    "0024C5": "Meridian Audio Limited",
    "0024E5": "Seer Technology, Inc",
    "002501": "JSC \"Supertel\"",
    "002458": "PA Bastion CC",
    "00245D": "Terberg besturingstechniek B.V.",
    "002450": "Cisco Systems, Inc",
    "002496": "Ginzinger electronic systems",
    "002499": "Aquila Technologies",
    "00248A": "Kaga Electronics Co., Ltd.",
    "002487": "Transact Campus, Inc.",
    "0024B9": "Wuhan Higheasy Electronic Technology Development Co.Ltd",
    "0024BD": "Hainzl Industriesysteme GmbH",
    "002477": "Tibbo Technology",
    "00246E": "Phihong USA Corp.",
    "002473": "3COM EUROPE LTD",
    "002451": "Cisco Systems, Inc",
    "00244A": "Voyant International",
    "002447": "Kaztek Systems",
    "00246B": "Covia, Inc.",
    "002435": "WIDE CORPORATION",
    "002431": "Uni-v co.,ltd",
    "002432": "Neostar Technology Co.,LTD",
    "002430": "Ruby Tech Corp.",
    "00242E": "Datastrip Inc.",
    "0023F9": "Double-Take Software, INC.",
    "0023F5": "WILO SE",
    "0023FF": "Beijing HTTC Technology Ltd.",
    "0023F6": "Softwell Technology Co., Ltd.",
    "0023F3": "Glocom, Inc.",
    "0023F0": "Shanghai Jinghan Weighing Apparatus Co. Ltd.",
    "00241A": "Red Beetle Inc.",
    "002414": "Cisco Systems, Inc",
    "002410": "NUETEQ Technology,Inc.",
    "0023CC": "Nintendo Co., Ltd.",
    "002443": "Nortel Networks",
    "002441": "Wanzl Metallwarenfabrik GmbH",
    "00243B": "CSSI (S) Pte Ltd",
    "002408": "Pacific Biosciences",
    "00240C": "DELEC GmbH",
    "0023EA": "Cisco Systems, Inc",
    "0023E5": "IPaXiom Networks",
    "002350": "RDC, Inc. dba LynTec",
    "00234F": "Luminous Power Technologies Pvt. Ltd.",
    "002349": "Helmholtz Centre Berlin for Material and Energy",
    "002346": "Vestac",
    "002344": "Objective Interface Systems, Inc.",
    "0023B6": "SECURITE COMMUNICATIONS / HONEYWELL",
    "0023B8": "Sichuan Jiuzhou Electronic Technology Co.,Ltd",
    "0023BA": "Chroma",
    "0023BC": "EQ-SYS GmbH",
    "0023B1": "Longcheer Technology (Singapore) Pte Ltd",
    "00239A": "EasyData Hardware GmbH",
    "002396": "ANDES TECHNOLOGY CORPORATION",
    "002394": "Samjeon",
    "0023A1": "Trend Electronics Ltd",
    "0023A6": "E-Mon",
    "0023A8": "Marshall Electronics",
    "002377": "Isotek Electronics Ltd",
    "002371": "SOAM Systel",
    "002364": "Power Instruments Pte Ltd",
    "00238C": "Private",
    "002379": "Union Business Machines Co. Ltd.",
    "002362": "Goldline Controls",
    "00235E": "Cisco Systems, Inc",
    "002358": "SYSTEL SA",
    "002356": "Packet Forensics LLC",
    "0023C0": "Broadway Networks",
    "0022BB": "beyerdynamic GmbH & Co. KG",
    "0022B6": "Superflow Technologies Group",
    "0022B5": "NOVITA",
    "0022B2": "4RF Communications Ltd",
    "0022AC": "Hangzhou Siyuan Tech. Co., Ltd",
    "0022AD": "TELESIS TECHNOLOGIES, INC.",
    "0022AE": "Mattel Inc.",
    "002307": "FUTURE INNOVATION TECH CO.,LTD",
    "002305": "Cisco Systems, Inc",
    "0022FF": "NIVIS LLC",
    "002302": "Cobalt Digital, Inc.",
    "0022F5": "Advanced Realtime Tracking GmbH",
    "0022C7": "SEGGER Microcontroller GmbH & Co. KG",
    "0022C1": "Active Storage Inc.",
    "0022C2": "Proview Eletr\u00f4nica do Brasil LTDA",
    "0022BD": "Cisco Systems, Inc",
    "0022BA": "HUTH Elektronik Systeme GmbH",
    "0022D1": "Albrecht Jung GmbH & Co. KG",
    "0022D2": "All Earth Com\u00e9rcio de Eletr\u00f4nicos LTDA.",
    "0022CA": "Anviz Biometric Tech. Co., Ltd.",
    "00233C": "Alflex",
    "002333": "Cisco Systems, Inc",
    "00232E": "Kedah Electronics Engineering, LLC",
    "0022E4": "APASS TECHNOLOGY CO., LTD.",
    "0022D5": "Eaton Corp. Electrical Group Data Center Solutions - Pulizzi",
    "0022E5": "Fisher-Rosemount Systems Inc.",
    "002320": "Nicira Networks",
    "002322": "KISS Teknical Solutions, Inc.",
    "002317": "Lasercraft Inc",
    "00230E": "Gorba AG",
    "00225E": "Uwin Technologies Co.,LTD",
    "002258": "Taiyo Yuden Co., Ltd.",
    "00225B": "Teradici Corporation",
    "002259": "Guangzhou New Postcom Equipment Co.,Ltd.",
    "002253": "Entorian Technologies",
    "002239": "Indiana Life Sciences Incorporated",
    "002235": "Strukton Systems bv",
    "00222C": "Ceton Corp",
    "00222D": "SMC Networks Inc.",
    "00227E": "Chengdu 30Kaitian Communication Industry Co.Ltd",
    "00227C": "Woori SMT Co.,ltd",
    "002277": "NEC Australia Pty Ltd",
    "002279": "Nippon Conlux Co., Ltd.",
    "0022A3": "California Eastern Laboratories",
    "0022A0": "APTIV SERVICES US, LLC",
    "00224F": "Byzoro Networks Ltd.",
    "002251": "Lumasense Technologies",
    "002247": "DAC ENGINEERING CO., LTD.",
    "002271": "J\u00e4ger Computergesteuerte Me\u00dftechnik GmbH.",
    "002297": "XMOS Semiconductor",
    "00228F": "CNRS",
    "002292": "Cinetal",
    "00223D": "JumpGen Systems, LLC",
    "00222A": "SoundEar A/S",
    "0021EE": "Full Spectrum Inc.",
    "0021EC": "Solutronic GmbH",
    "0021E6": "Starlight Video Limited",
    "0021E0": "CommAgility Ltd",
    "0021CC": "Flextronics International",
    "0021CF": "The Crypto Group",
    "0021C5": "3DSP Corp",
    "0021B9": "Universal Devices Inc.",
    "0021B3": "Ross Controls",
    "0021B6": "Triacta Power Technologies Inc.",
    "0021AE": "ALCATEL-LUCENT FRANCE - WTD",
    "0021AF": "Radio Frequency Systems",
    "0021F9": "WIRECOM Technologies",
    "0021FA": "A4SP Technologies Ltd.",
    "0021F0": "EW3 Technologies LLC",
    "002211": "Rohati Systems",
    "002212": "CAI Networks, Inc.",
    "00220D": "Cisco Systems, Inc",
    "0021DE": "Firepro Wireless",
    "0021D3": "BOCOM SECURITY(ASIA PACIFIC) LIMITED",
    "002208": "Certicom Corp",
    "002205": "WeLink Solutions, Inc.",
    "002209": "Omron Healthcare Co., Ltd",
    "00221F": "eSang Technologies Co., Ltd.",
    "002217": "Neat Electronics",
    "0021A4": "Dbii Networks",
    "00214B": "Shenzhen HAMP Science & Technology Co.,Ltd",
    "002145": "Semptian Technologies Ltd.",
    "00218C": "TopControl GMBH",
    "00218A": "Electronic Design and Manufacturing Company",
    "00218B": "Wescon Technology, Inc.",
    "002184": "POWERSOFT SRL",
    "002121": "VRmagic GmbH",
    "002123": "Aerosat Avionics",
    "00211B": "Cisco Systems, Inc",
    "00217A": "Sejin Electron, Inc.",
    "002178": "Matuschek Messtechnik GmbH",
    "002173": "Ion Torrent Systems, Inc.",
    "002177": "W. L. Gore & Associates",
    "002199": "Vacon Plc",
    "0021A0": "Cisco Systems, Inc",
    "002198": "Thai Radio Co, LTD",
    "002172": "Seoultek Valley",
    "002169": "Prologix, LLC.",
    "002153": "SeaMicro Inc.",
    "002154": "D-TACQ Solutions Ltd",
    "00213E": "TomTom International BV",
    "00213F": "A-Team Technology Ltd.",
    "00212D": "SCIMOLEX CORPORATION",
    "001FD8": "A-TRUST COMPUTER CORPORATION",
    "001FD7": "TELERAD SA",
    "001FD3": "RIVA Networks Inc.",
    "001FD5": "MICRORISC s.r.o.",
    "001FCA": "Cisco Systems, Inc",
    "001FC3": "SmartSynch, Inc",
    "001FC1": "Hanlong Technology Co.,LTD",
    "001FEE": "ubisys technologies GmbH",
    "001FEF": "SHINSEI INDUSTRIES CO.,LTD",
    "001FEC": "Synapse \u00c9lectronique",
    "001FFC": "Riccius+Sohn GmbH",
    "001FFD": "Indigo Mobile Technologies Corp.",
    "001FBB": "Xenatech Co.,LTD",
    "001FB1": "Cybertech Inc.",
    "00210A": "byd:sign Corporation",
    "002107": "Seowonintech Co Ltd.",
    "001FFE": "HPN Supply Chain",
    "001FFF": "Respironics, Inc.",
    "001FE8": "KURUSUGAWA Electronics Industry Inc,.",
    "001FE0": "EdgeVelocity Corp",
    "001FAB": "I.S HIGH TECH.INC",
    "001FAC": "Goodmill Systems Ltd",
    "001FB2": "Sontheim Industrie Elektronik GmbH",
    "001F44": "GE Transportation Systems",
    "001F39": "Construcciones y Auxiliar de Ferrocarriles, S.A.",
    "001F3D": "Qbit GmbH",
    "001F36": "Bellwin Information Co. Ltd.,",
    "001F38": "POSITRON",
    "001F8E": "Metris USA Inc.",
    "001F89": "Signalion GmbH",
    "001F8A": "Ellion Digital Inc.",
    "001F7F": "Phabrix Limited",
    "001F7C": "Witelcom AS",
    "001F7A": "WiWide Inc.",
    "001F77": "HEOL DESIGN",
    "001F76": "AirLogic Systems Inc.",
    "001F73": "Teraview Technology Co., Ltd.",
    "001F11": "OPENMOKO, INC.",
    "001F2D": "Electro-Optical Imaging, Inc.",
    "001F2F": "Berker GmbH & Co. KG",
    "001F32": "Nintendo Co., Ltd.",
    "001F56": "DIGITAL FORECAST",
    "001F52": "UVT Unternehmensberatung fur Verkehr und Technik GmbH",
    "001F4F": "Thinkware Co. Ltd.",
    "001F62": "JSC \"Stilsoft\"",
    "001F67": "Hitachi,Ltd.",
    "001F61": "Talent Communication Networks Inc.",
    "001F1C": "KOBISHI ELECTRIC Co.,Ltd.",
    "001F19": "BEN-RI ELECTRONICA S.A.",
    "001EC8": "Rapid Mobile (Pty) Ltd",
    "001ECC": "CDVI",
    "001EC5": "Middle Atlantic Products Inc",
    "001EBE": "Cisco Systems, Inc",
    "001EC3": "Kozio, Inc.",
    "001EBD": "Cisco Systems, Inc",
    "001EB9": "Sing Fai Technology Limited",
    "001EF6": "Cisco Systems, Inc",
    "001EF9": "Pascom Kommunikations systeme GmbH.",
    "001EF3": "From2",
    "001EE7": "Epic Systems Inc",
    "001EE9": "Stoneridge Electronics AB",
    "001F03": "NUM AG",
    "001EFD": "Microbit 2.0 AB",
    "001EFF": "Mueller-Elektronik GmbH & Co. KG",
    "001F05": "iTAS Technology Corp.",
    "001F07": "AZTEQ Mobile",
    "001E93": "CiriTech Systems Inc",
    "001E92": "JEULIN S.A.",
    "001E91": "KIMIN Electronic Co., Ltd.",
    "001E89": "CRFS Limited",
    "001E86": "MEL Co.,Ltd.",
    "001E88": "ANDOR SYSTEM SUPPORT CO., LTD.",
    "001EB7": "TBTech, Co., Ltd.",
    "001EA2": "Symx Systems, Inc.",
    "001EA9": "Nintendo Co., Ltd.",
    "001E9E": "ddm hopt + schuler Gmbh + Co. KG",
    "001EE4": "ACS Solutions France",
    "001EED": "Adventiq Ltd.",
    "001ED4": "Doble Engineering",
    "001ED2": "Ray Shine Video Technology Inc",
    "001ECE": "BISA Technologies (Hong Kong) Limited",
    "001E9D": "Recall Technologies, Inc.",
    "001E95": "SIGMALINK",
    "001E79": "Cisco Systems, Inc",
    "001E54": "TOYO ELECTRIC Corporation",
    "001E4D": "Welkin Sciences, LLC",
    "001E4B": "City Theatrical",
    "001E4A": "Cisco Systems, Inc",
    "001E3C": "Lyngbox Media AB",
    "001E0D": "Micran Ltd.",
    "001E09": "ZEFATEK Co.,LTD",
    "001E06": "WIBRAIN",
    "001E0C": "Sherwood Information Partners, Inc.",
    "001E02": "Sougou Keikaku Kougyou Co.,Ltd.",
    "001E01": "Renesas Technology Sales Co., Ltd.",
    "001DFF": "Network Critical Solutions Ltd",
    "001E5F": "KwikByte, LLC",
    "001E5B": "Unitron Company, Inc.",
    "001E5E": "COmputime Ltd.",
    "001E76": "Thermo Fisher Scientific",
    "001E72": "PCS",
    "001E69": "Thomson Inc.",
    "001DEE": "NEXTVISION SISTEMAS DIGITAIS DE TELEVIS\u00c3O LTDA.",
    "001DEA": "Commtest Instruments Ltd",
    "001DDD": "DAT H.K. LIMITED",
    "001DE4": "Visioneered Image Systems",
    "001E00": "Shantou Institute of Ultrasonic Instruments",
    "001DF3": "SBS Science & Technology Co., Ltd",
    "001E23": "Electronic Educational Devices, Inc",
    "001E13": "Cisco Systems, Inc",
    "001E2C": "CyVerse Corporation",
    "001E26": "Digifriends Co. Ltd",
    "001D89": "VaultStor Corporation",
    "001D86": "Shinwa Industries(China) Ltd.",
    "001D88": "Clearwire",
    "001D81": "GUANGZHOU GATEWAY ELECTRONICS CO., LTD",
    "001D76": "Eyeheight Ltd.",
    "001D7A": "Wideband Semiconductor, Inc.",
    "001D68": "Thomson Telecom Belgium",
    "001D61": "BIJ Corporation",
    "001D5C": "Tom Communication Industrial Co.,Ltd.",
    "001DE2": "Radionor Communications",
    "001DD7": "Algolith",
    "001DC9": "GainSpan Corp.",
    "001DC7": "L-3 Communications Geneva Aerospace",
    "001DBC": "Nintendo Co., Ltd.",
    "001D66": "Hyundai Telecom",
    "001D77": "NSGate",
    "001DA2": "Cisco Systems, Inc",
    "001D99": "Cyan Optic, Inc.",
    "001D9B": "Hokuyo Automatic Co., Ltd.",
    "001DB6": "BestComm Networks, Inc.",
    "001DAC": "Gigamon Systems LLC",
    "001D4E": "TCM Mobile LLC",
    "001D49": "Innovation Wireless Inc.",
    "001D46": "Cisco Systems, Inc",
    "001D48": "Sensor-Technik Wiedemann GmbH",
    "001D47": "Covote GmbH & Co KG",
    "001D41": "Hardy Instruments",
    "001CE6": "INNES",
    "001CE1": "INDRA SISTEMAS, S.A.",
    "001CE0": "DASAN TPS",
    "001D11": "Analogue & Micro Ltd",
    "001D12": "ROHM CO., LTD.",
    "001D03": "Design Solutions Inc.",
    "001D3D": "Avidyne Corporation",
    "001D3C": "Muscle Corporation",
    "001D3A": "mh acoustics LLC",
    "001D35": "Viconics Electronics Inc.",
    "001D31": "HIGHPRO INTERNATIONAL R&D CO,.LTD.",
    "001CD9": "GlobalTop Technology Inc.",
    "001CDA": "Exegin Technologies Limited",
    "001CD2": "King Champion (Hong Kong) Limited",
    "001CF9": "Cisco Systems, Inc",
    "001CF1": "SUPoX Technology Co. , LTD.",
    "001CF3": "EVS BROADCAST EQUIPMENT",
    "001CF4": "Media Technology Systems Inc",
    "001D26": "Rockridgesound Technology Co.",
    "001D21": "Alcad SL",
    "001D1C": "Gennet s.a.",
    "001D15": "Shenzhen Dolphin Electronic Co., Ltd",
    "001D16": "SFR",
    "001D56": "Kramer Electronics Ltd.",
    "001C90": "Empacket Corporation",
    "001C8E": "Alcatel-Lucent IPD",
    "001C8F": "Advanced Electronic Design, Inc.",
    "001C88": "TRANSYSTEM INC.",
    "001CA9": "Audiomatica Srl",
    "001C99": "Shunra Software Ltd.",
    "001C9B": "FEIG ELECTRONIC GmbH",
    "001C95": "Opticomm Corporation",
    "001C97": "Enzytek Technology Inc.,",
    "001C58": "Cisco Systems, Inc",
    "001C5A": "Advanced Relay Corporation",
    "001C4F": "MACAB AB",
    "001C4E": "TASA International Limited",
    "001C6E": "Newbury Networks, Inc.",
    "001C6B": "COVAX  Co. Ltd",
    "001C69": "Packet Vision Ltd",
    "001C65": "JoeScan, Inc.",
    "001C5F": "Winland Electronics, Inc.",
    "001C3D": "WaveStorm",
    "001C2D": "FlexRadio Systems",
    "001C86": "Cranite Systems, Inc.",
    "001C7F": "Check Point Software Technologies",
    "001C78": "WYPLAY SAS",
    "001CCE": "By Techdesign",
    "001CB9": "KWANG SUNG ELECTRONICS CO., LTD.",
    "001C4B": "Gener8, Inc.",
    "001C40": "VDG-Security bv",
    "001C3A": "Element Labs, Inc.",
    "001CA3": "Terra",
    "001CA6": "Win4NET",
    "001BCA": "Beijing Run Technology LTD. Company",
    "001BCC": "KINGTEK CCTV ALLIANCE CO., LTD.",
    "001BC8": "MIURA CO.,LTD",
    "001BC1": "HOLUX Technology, Inc.",
    "001BBC": "Silver Peak Systems, Inc.",
    "001C1E": "emtrion GmbH",
    "001C18": "Sicert S.r.L.",
    "001C1A": "Thomas Instrumentation, Inc",
    "001C0E": "Cisco Systems, Inc",
    "001C13": "OPTSYS TECHNOLOGY CO., LTD.",
    "001BF4": "KENWIN INDUSTRIAL(HK) LTD.",
    "001BF9": "Intellitect Water Ltd",
    "001BFA": "G.i.N. mbH",
    "001BF3": "TRANSRADIO SenderSysteme Berlin AG",
    "001BDB": "Valeo VECS",
    "001BD8": "FLIR Systems Inc",
    "001BD4": "Cisco Systems, Inc",
    "001BD0": "IDENTEC SOLUTIONS",
    "001BCD": "DAVISCOMMS (S) PTE LTD",
    "001BDE": "Renkus-Heinz, Inc.",
    "001BAB": "Telchemy, Incorporated",
    "001BAE": "Micro Control Systems, Inc",
    "001BA8": "UBI&MOBI,.Inc",
    "001C2C": "Synapse",
    "001C21": "Nucsafe Inc.",
    "001C0B": "SmartAnt Telecom",
    "001C08": "Echo360, Inc.",
    "001BA0": "Awox",
    "001B42": "Wise & Blue",
    "001B35": "ChongQing JINOU Science & Technology Development CO.,Ltd",
    "001B36": "Tsubata Engineering Co.,Ltd. (Head Office)",
    "001B39": "Proxicast",
    "001B3B": "Yi-Qing CO., LTD",
    "001B51": "Vector Technology Corp.",
    "001B54": "Cisco Systems, Inc",
    "001B4A": "W&W Communications, Inc.",
    "001B44": "SanDisk Corporation",
    "001B46": "Blueone Technology Co.,Ltd",
    "001B40": "Network Automation mxc AB",
    "001B79": "FAIVELEY TRANSPORT",
    "001B71": "Telular Corp.",
    "001B73": "DTL Broadcast Ltd",
    "001B20": "TPine Technology",
    "001B1C": "Coherent",
    "001B22": "Palit Microsystems ( H.K.) Ltd.",
    "001B19": "IEEE I&M Society TC9",
    "001B65": "China Gridcom Co., Ltd",
    "001B64": "IsaacLandKorea Co., Ltd,",
    "001B97": "Violin Technologies",
    "001B88": "Divinet Access Technologies Ltd",
    "001B83": "Finsoft Ltd",
    "001B81": "DATAQ Instruments, Inc.",
    "001B7D": "CXR Anderson Jacobson",
    "001B26": "RON-Telecom ZAO",
    "001AA8": "Mamiya Digital Imaging Co., Ltd.",
    "001A99": "Smarty (HZ) Information Electronics Co., Ltd",
    "001A96": "ECLER S.A.",
    "001AFA": "Welch Allyn, Inc.",
    "001AF7": "dataschalt e+a GmbH",
    "001AF3": "Samyoung Electronics",
    "001AEF": "Loopcomm Technology, Inc.",
    "001AEC": "Keumbee Electronics Co.,Ltd.",
    "001AA9": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "001A8C": "Sophos Ltd",
    "001A91": "FusionDynamic Ltd.",
    "001AB9": "PMC",
    "001ABA": "Caton Overseas Limited",
    "001AB3": "VISIONITE INC.",
    "001ACA": "Tilera Corporation",
    "001B13": "Icron Technologies Corporation",
    "001B0F": "Petratec",
    "001AD1": "FARGO CO., LTD.",
    "001A15": "gemalto e-Payment",
    "001A0D": "HandHeld entertainment, Inc.",
    "001A0E": "Cheng Uei Precision Industry Co.,Ltd",
    "001A84": "V One Multimedia Pte Ltd",
    "001A7B": "Teleco, Inc.",
    "001A6D": "Cisco Systems, Inc",
    "001A6E": "Impro Technologies",
    "001A6C": "Cisco Systems, Inc",
    "001A3E": "Faster Technology LLC",
    "001A3A": "Dongahelecomm",
    "001A3B": "Doah Elecom Inc.",
    "001A3C": "Technowave Ltd.",
    "001A40": "A-FOUR TECH CO., LTD.",
    "001A30": "Cisco Systems, Inc",
    "001A36": "Aipermon GmbH & Co. KG",
    "001A26": "Deltanode Solutions AB",
    "001A25": "DELTA DORE",
    "001A46": "Digital Multimedia Technology Co., Ltd",
    "001A4A": "Qumranet Inc.",
    "001A63": "Elster Solutions, LLC,",
    "001A59": "Ircona",
    "0019FE": "SHENZHEN SEECOMM TECHNOLOGY CO.,LTD.",
    "0019FD": "Nintendo Co., Ltd.",
    "0019ED": "Axesstel Inc.",
    "0019F6": "Acconet (PTE) Ltd",
    "0019CD": "Chengdu ethercom information technology Ltd.",
    "0019D9": "Zeutschel GmbH",
    "0019CA": "Broadata Communications, Inc",
    "0019D3": "TRAK Microwave",
    "0019C3": "Qualitrol",
    "0019BE": "Altai Technologies Limited",
    "0019B4": "Intellio Ltd",
    "0019BA": "Paradox Security Systems Ltd",
    "001980": "Gridpoint Systems",
    "001983": "CCT R&D Limited",
    "00197F": "PLANTRONICS, INC.",
    "001A01": "Smiths Medical",
    "00198A": "Northrop Grumman Systems Corp.",
    "001989": "Sonitrol Corporation",
    "0019A1": "LG INFORMATION & COMM.",
    "0019A8": "WiQuest Communications",
    "00197A": "MAZeT GmbH",
    "001978": "Datum Systems, Inc.",
    "00191D": "Nintendo Co., Ltd.",
    "001924": "LBNL  Engineering",
    "00191A": "IRLINK",
    "001916": "PayTec AG",
    "001950": "Harman Multimedia",
    "00194C": "Fujian Stelcom information & Technology CO.,Ltd",
    "00194A": "TESTO AG",
    "001939": "Gigamips",
    "00193A": "OESOLUTIONS",
    "00192F": "Cisco Systems, Inc",
    "001922": "CM Comandos Lineares",
    "001968": "Digital Video Networks(Shanghai) CO. LTD.",
    "00196D": "Raybit Systems Korea, Inc",
    "001970": "Z-Com, Inc.",
    "00190E": "Atech Technology Co., Ltd.",
    "001960": "DoCoMo Systems, Inc.",
    "00195A": "Jenaer Antriebstechnik GmbH",
    "001905": "SCHRACK Seconet AG",
    "001907": "Cisco Systems, Inc",
    "0018C6": "OPW Fuel Management Systems",
    "0018C3": "CS Corporation",
    "0018CA": "Viprinet GmbH",
    "0018C7": "Real Time Automation",
    "0018BB": "Eliwell Controls srl",
    "0018BF": "Essence Technology Solution, Inc.",
    "0018FB": "Compro Technology",
    "0018EE": "Videology Imaging Solutions, Inc.",
    "0018EB": "Blue Zen Enterprises Private Limited",
    "001870": "E28 Shanghai Limited",
    "001872": "Expertise Engineering",
    "001874": "Cisco Systems, Inc",
    "001869": "KINGJIM",
    "0018E2": "Topdata Sistemas de Automacao Ltda",
    "0018DC": "Prostar Co., Ltd.",
    "0018D1": "Siemens Home & Office Comm. Devices",
    "0018D0": "AtRoad,  A Trimble Company",
    "0018D2": "High-Gain Antennas LLC",
    "0018D3": "TEAMCAST",
    "00188F": "Montgomery Technology, Inc.",
    "001884": "Fon Technology S.L.",
    "001880": "Maxim Integrated Products",
    "00187C": "INTERCROSS, LLC",
    "0018B9": "Cisco Systems, Inc",
    "0018B8": "New Voice International AG",
    "0018A1": "Tiqit Computers, Inc.",
    "001896": "Great Well Electronic LTD",
    "001890": "RadioCOM, s.r.o.",
    "0018AA": "Protec Fire Detection plc",
    "001810": "IPTrade S.A.",
    "001804": "E-TEK DIGITAL TECHNOLOGY LIMITED",
    "001809": "CRESYN",
    "001861": "Ooma, Inc.",
    "001855": "Aeromaritime Systembau GmbH",
    "001851": "SWsoft",
    "001856": "EyeFi, Inc",
    "00184E": "Lianhe Technologies, Inc.",
    "00184C": "Bogen Communications",
    "001812": "Beijing Xinwei Telecom Technology Co., Ltd.",
    "00180B": "Brilliant Telecommunications",
    "001800": "UNIGRAND LTD",
    "0017FC": "Suprema Inc.",
    "0017FD": "Amulet Hotkey",
    "0017FB": "FA",
    "0017D8": "Magnum Semiconductor, Inc.",
    "0017DA": "Spans Logic",
    "00185B": "Network Chemistry, Inc",
    "001841": "High Tech Computer Corp",
    "001826": "Cale Access AB",
    "00182D": "Artec Design",
    "00182A": "Taiwan Video & Monitor",
    "00183B": "CENITS Co., Ltd.",
    "00183C": "Encore Software Limited",
    "001819": "Cisco Systems, Inc",
    "00175C": "SHARP CORPORATION",
    "001759": "Cisco Systems, Inc",
    "001754": "Arkino HiTOP Corporation Limited",
    "001752": "DAGS, Inc",
    "001756": "Vinci Labs Oy",
    "001790": "HYUNDAI DIGITECH Co, Ltd.",
    "001791": "LinTech GmbH",
    "001795": "Cisco Systems, Inc",
    "001799": "SmarTire Systems Inc.",
    "00177F": "Worldsmart Retech",
    "00175F": "XENOLINK Communications Co., Ltd.",
    "001774": "Elesta GmbH",
    "001777": "Obsidian Research Corporation",
    "00176A": "Avago Technologies",
    "001786": "wisembed",
    "001782": "LoBenn Inc.",
    "00176B": "Kiyon, Inc.",
    "001778": "Central Music Co.",
    "0017B3": "Aftek Infosys Limited",
    "0017A2": "Camrivox Ltd.",
    "00179D": "Kelman Limited",
    "0017CF": "iMCA-GmbH",
    "0017B9": "Gambro Lundia AB",
    "00171E": "Theo Benning GmbH & Co. KG",
    "001712": "ISCO International",
    "00170D": "Dust Networks Inc.",
    "00170C": "Twig Com Ltd.",
    "00170B": "Contela, Inc.",
    "00170F": "Cisco Systems, Inc",
    "001704": "Shinco Electronics Group Co.,Ltd",
    "00172D": "Axcen Photonics Corporation",
    "001724": "Studer Professional Audio GmbH",
    "001707": "InGrid, Inc",
    "001702": "Osung Midicom Co., Ltd",
    "0016D8": "Senea AB",
    "0016D6": "TDA Tech Pty Ltd",
    "0016D5": "Synccom Co., Ltd",
    "001741": "DEFIDEV",
    "001738": "International Business Machines",
    "0016F7": "L-3 Communications, Aviation Recorders",
    "0016A1": "3Leaf Networks",
    "0016A4": "Ezurio Ltd",
    "001699": "Tonic DVB Marketing Ltd",
    "00169B": "Alstom Transport",
    "001690": "J-TEK INCORPORATION",
    "001698": "T&A Mobile Phones",
    "001696": "QDI Technology (H.K.) Limited",
    "001661": "Novatium Solutions (P) Ltd",
    "001664": "Prod-El SpA",
    "00165E": "Precision I/O",
    "001658": "Fusiontech Technologies Inc.",
    "001653": "LEGO System A/S IE Electronics Division",
    "001652": "Hoatech Technologies, Inc.",
    "001650": " Kratos EPD    ",
    "001666": "Quantier Communication Inc.",
    "001662": "Liyuh Technology Ltd.",
    "001680": "Bally Gaming + Systems",
    "001681": "Vector Informatik GmbH",
    "00BAC0": "Biometric Access Company",
    "0016AB": "Dansensor A/S",
    "0016A5": "Tandberg Storage ASA",
    "0016C9": "NAT Seattle, Inc.",
    "0016C6": "North Atlantic Industries",
    "0016D2": "Caspian",
    "0016BE": "INFRANET, Inc.",
    "001685": "Elisa Oyj",
    "0015FA": "Cisco Systems, Inc",
    "0015FC": "Littelfuse Startco",
    "0015F7": "Wintecronics Ltd.",
    "0015DB": "Canesta Inc.",
    "0015D7": "Reti Corporation",
    "0015CB": "Surf Communication Solutions Ltd.",
    "001630": "Vativ Technologies",
    "00162F": "Geutebr\u00fcck GmbH",
    "00162B": "Togami Electric Mfg.co.,Ltd.",
    "001642": "Pangolin",
    "00163B": "Communications & Power Industries",
    "001637": "CITEL SpA",
    "0015CC": "UQUEST, LTD.",
    "0015C6": "Cisco Systems, Inc",
    "0015C3": "Ruf Telematik AG",
    "0015F8": "Kingtronics Industrial Co. Ltd.",
    "0015E6": "MOBILE TECHNIKA Inc.",
    "001608": "Sequans Communications",
    "001624": "Teneros, Inc.",
    "00161C": "e:cue",
    "001581": "MAKUS Inc.",
    "00157A": "Telefin S.p.A.",
    "001573": "NewSoft  Technology Corporation",
    "001575": "Nevis Networks Inc.",
    "00156C": "SANE SYSTEM CO., LTD",
    "00156A": "DG2L Technologies Pvt. Ltd.",
    "0015B8": "Tahoe",
    "0015B6": "ShinMaywa Industries, Ltd.",
    "0015B0": "AUTOTELENET CO.,LTD",
    "0015B1": "Ambient Corporation",
    "00159F": "Terascala, Inc.",
    "00159E": "Mad Catz Interactive Inc",
    "0015A1": "ECA-SINTERS",
    "001593": "U4EA Technologies Inc.",
    "00158D": "Jennic Ltd",
    "001546": "ITG Worldwide Sdn Bhd",
    "00153E": "Q-Matic Sweden AB",
    "001542": "MICROHARD S.R.L.",
    "00156F": "Xiranet Communications GmbH",
    "001572": "Red-Lemon",
    "001567": "RADWIN Inc.",
    "001551": "RadioPulse Inc.",
    "001552": "Wi-Gear Inc.",
    "00154E": "IEC",
    "001550": "Nits Technology Inc",
    "00155D": "Microsoft Corporation",
    "001562": "Cisco Systems, Inc",
    "00155B": "Sampo Corporation",
    "001553": "Cytyc Corporation",
    "001584": "Schenck Process GmbH",
    "0015A7": "Robatech AG",
    "00152D": "TenX Networks, LLC",
    "001523": "Meteor Communications Corporation",
    "001524": "Numatics, Inc.",
    "001529": "N3 Corporation",
    "001509": "Plus Technology Co., Ltd",
    "0014F9": "Vantage Controls",
    "0014E9": "Nortech International",
    "0014ED": "Airak, Inc.",
    "0014E1": "Data Display AG",
    "0014E3": "mm-lab GmbH",
    "0014D9": "IP Fabrics, Inc.",
    "0014D6": "Jeongmin Electronics Co.,Ltd.",
    "0014D0": "BTI Systems Inc.",
    "0014CE": "NF CORPORATION",
    "0014F0": "Business Security OL AB",
    "0014F2": "Cisco Systems, Inc",
    "0014E7": "Stolinx,. Inc",
    "001510": "Techsphere Co., Ltd",
    "001513": "EFS sas",
    "001503": "PROFIcomms s.r.o.",
    "00151B": "Isilon Systems Inc.",
    "001459": "Moram Co., Ltd.",
    "001453": "ADVANTECH TECHNOLOGIES CO.,LTD",
    "001454": "Symwave",
    "00148B": "Globo Electronic GmbH & Co. KG",
    "001490": "ASP Corporation",
    "001448": "Inventec Multimedia & Telecom Corporation",
    "00144B": "Hifn, Inc.",
    "00143C": "Rheinmetall Canada Inc.",
    "001488": "Akorri",
    "001484": "Cermate Technologies Inc.",
    "001479": "NEC Magnus Communications,Ltd.",
    "00147B": "Iteris, Inc.",
    "00147A": "Eubus GmbH",
    "00146D": "RF Technologies",
    "001470": "Prokom Software SA",
    "001467": "ArrowSpan Inc.",
    "0014AC": "Bountiful WiFi",
    "0014A8": "Cisco Systems, Inc",
    "0014A0": "Accsense, Inc.",
    "0014BB": "Open Interface North America",
    "00145F": "ADITEC CO. LTD",
    "0013E6": "Technolution",
    "0013DF": "Ryvor Corp.",
    "0013D5": "RuggedCom",
    "0013D6": "TII NETWORK TECHNOLOGIES, INC.",
    "0013DB": "SHOEI Electric Co.,Ltd",
    "0013F2": "Klas Ltd",
    "0013F7": "SMC Networks, Inc.",
    "0013ED": "PSIA",
    "001426": "NL Technology",
    "00141C": "Cisco Systems, Inc",
    "00141A": "DEICY CORPORATION",
    "00140C": "GKB CCTV CO., LTD.",
    "0013FE": "GRANDTEC ELECTRONIC CORP.",
    "0013F8": "Dex Security Solutions",
    "0013F9": "Cavera Systems",
    "0013C5": "LIGHTRON FIBER-OPTIC DEVICES INC.",
    "0013C4": "Cisco Systems, Inc",
    "0013C2": "WACOM Co.,Ltd",
    "0013BF": "Media System Planning Corp.",
    "0013BB": "Smartvue Corporation",
    "0013B5": "Wavesat",
    "0013CD": "MTI co. LTD",
    "0013D3": "MICRO-STAR INTERNATIONAL CO., LTD.",
    "0013CA": "ATX",
    "001435": "CityCom Corp.",
    "001416": "Scosche Industries, Inc.",
    "00133C": "QUINTRON SYSTEMS INC.",
    "00133D": "Micro Memory Curtiss Wright Co",
    "00133F": "Eppendorf Instrumente GmbH",
    "001341": "Shandong New Beiyang Information Technology Co.,Ltd",
    "001331": "CellPoint Connect",
    "001335": "VS Industry Berhad",
    "00132F": "Interactek",
    "001330": "EURO PROTECTION SURVEILLANCE",
    "001325": "Cortina Systems Inc",
    "001350": "Silver Spring Networks, Inc",
    "00134C": "YDT Technology International",
    "00138D": "Kinghold",
    "001390": "Termtek Computer Co., Ltd",
    "00139A": "K-ubique ID Corp.",
    "00139E": "Ciara Technologies Inc.",
    "00139D": "MaxLinear Hispania S.L.U.",
    "0013AF": "NUMA Technology,Inc.",
    "0013B0": "Jablotron",
    "0013B1": "Intelligent Control Systems (Asia) Pte Ltd",
    "00137F": "Cisco Systems, Inc",
    "001382": "Cetacea Networks Corporation",
    "001358": "Realm Systems, Inc.",
    "001359": "ProTelevision Technologies A/S",
    "001375": "American Security Products Co.",
    "0012B4": "Work Microwave GmbH",
    "0012B5": "Vialta, Inc.",
    "0012B1": "Dai Nippon Printing Co., Ltd",
    "0012C3": "WIT S.A.",
    "0012C8": "Perfect tech",
    "0012C6": "TGC America, Inc",
    "0012CC": "Bitatek CO., LTD",
    "0012C1": "Check Point Software Technologies",
    "0012BB": "Telecommunications Industry Association TR-41 Committee",
    "0012B6": "Santa Barbara Infrared, Inc.",
    "0012B9": "Fusion Digital Technology",
    "0012ED": "AVG Advanced Technologies",
    "0012EA": "Trane",
    "0012FD": "OPTIMUS IC S.A.",
    "001305": "Epicom, Inc.",
    "001306": "Always On Wireless",
    "0012FA": "THX LTD",
    "0012E7": "Projectek Networking Electronics Corp.",
    "0012E4": "ZIEHL industrie-electronik GmbH + Co KG",
    "00129F": "RAE Systems",
    "001297": "O2Micro, Inc.",
    "00129D": "First International Computer do Brasil",
    "00129C": "Yulinet",
    "001290": "KYOWA Electric & Machinery Corp.",
    "001291": "KWS Computersysteme GmbH",
    "001295": "Aiware Inc.",
    "00128B": "Sensory Networks Inc",
    "00128F": "Montilio",
    "001246": "T.O.M TECHNOLOGY INC..",
    "00123D": "GES Co, Ltd",
    "0012A3": "Trust International B.V.",
    "0012A7": "ISR TECHNOLOGIES Inc",
    "0012AA": "IEE, Inc.",
    "00127E": "Digital Lifestyles Group, Inc.",
    "00125B": "KAIMEI ELECTRONI",
    "001259": "THERMO ELECTRON KARLSRUHE",
    "00125A": "Microsoft Corporation",
    "001289": "Advance Sterilization Products",
    "001284": "Lab33 Srl",
    "001281": "March Networks S.p.A.",
    "001256": "LG INFORMATION & COMM.",
    "00126B": "Ascalade Communications Limited",
    "0011E2": "Hua Jung Components Co., Ltd.",
    "0011DA": "Vivaas Technology Inc.",
    "0011DD": "FROMUS TEC. Co., Ltd.",
    "0011E4": "Danelec Electronics A/S",
    "0011E1": "Arcelik A.S",
    "0011BE": "AGP Telecom Co. Ltd",
    "0011BA": "Elexol Pty Ltd",
    "0011BC": "Cisco Systems, Inc",
    "0011B3": "YOSHIMIYA CO.,LTD.",
    "0011B6": "Open Systems International",
    "00122F": "Sanei Electric Inc.",
    "001230": "Picaso Infocommunication CO., LTD.",
    "0011FB": "Heidelberg Engineering GmbH",
    "0011EF": "Conitec Datensysteme GmbH",
    "0011D6": "HandEra, Inc.",
    "0011CA": "Long Range Systems, Inc.",
    "0011C5": "TEN Technology",
    "0011CD": "Axsun Technologies",
    "0011EB": "Innovative Integration",
    "0011EA": "IWICS Inc.",
    "001239": "S Net Systems Inc.",
    "001233": "JRC TOKKI Co.,Ltd.",
    "00121D": "Netfabric Corporation",
    "001214": "Koenig & Bauer AG",
    "00120F": "IEEE 802.3",
    "00120C": "CE-Infosys Pte Ltd",
    "001153": "Trident Tek, Inc.",
    "00114F": "US Digital Television, Inc",
    "0011B0": "Fortelink Inc.",
    "0011AC": "Simtec Electronics",
    "0011AD": "Shanghai Ruijie Technology",
    "0011A8": "Quest Technologies",
    "00118F": "EUTECH INSTRUMENTS PTE. LTD.",
    "001191": "CTS-Clima Temperatur Systeme GmbH",
    "00118D": "Hanchang System Corp.",
    "001189": "Aerotech Inc",
    "001163": "SYSTEM SPA DEPT. ELECTRONICS",
    "00115F": "ITX Security Co., Ltd.",
    "001182": "IMI Norgren Ltd",
    "00117A": "Singim International Corp.",
    "001148": "Prolon Control Systems",
    "001140": "Nanometrics Inc.",
    "001144": "Assurance Technology Corp",
    "0011A1": "VISION NETWARE CO.,LTD",
    "0011A0": "Vtech Engineering Canada Ltd",
    "00119B": "Telesynergy Research Inc.",
    "001172": "COTRON CORPORATION",
    "001168": "HomeLogic LLC",
    "00113A": "SHINBORAM",
    "00113F": "Alcatel DI",
    "000FFC": "Merit Li-Lin Ent.",
    "000FFB": "Nippon Denso Industry Co., Ltd.",
    "000FF2": "Loud Technologies Inc.",
    "000FF1": "nex-G Systems Pte.Ltd",
    "000FF3": "Jung Myoung Communications&Technology",
    "000FA6": "S2 Security Corporation",
    "000FAA": "Nexus Technologies",
    "000FA8": "Photometrics, Inc.",
    "000F9D": "DisplayLink (UK) Ltd",
    "001101": "CET Technologies Pte Ltd",
    "000FFF": "Control4",
    "001113": "Fraunhofer FOKUS",
    "001112": "Honeywell CMSS",
    "000FBD": "MRV Communications (Networks) LTD",
    "000FB4": "Timespace Technology",
    "001137": "AICHI ELECTRIC CO., LTD.",
    "001128": "Streamit",
    "000FCB": "3Com Ltd",
    "000FC5": "KeyMed Ltd",
    "000FBF": "DGT Sp. z o.o.",
    "000FE3": "Damm Cellular Systems A/S",
    "000FD5": "Schwechat - RISE",
    "000FE0": "NComputing Co.,Ltd.",
    "000F6B": "GateWare Communications GmbH",
    "000F67": "West Instruments",
    "000F6E": "BBox",
    "000F6F": "FTA Communication Technologies",
    "000F63": "Obzerv Technologies",
    "000F21": "Scientific Atlanta, Inc",
    "000F11": "Prodrive B.V.",
    "000F13": "Nisca corporation",
    "000F14": "Mindray Co., Ltd.",
    "000F89": "Winnertec System Co., Ltd.",
    "000F65": "icube Corp.",
    "000F5D": "Genexis BV",
    "000F58": "Adder Technology Limited",
    "000F46": "SINAR AG",
    "000F41": "Zipher Ltd",
    "000F71": "Sanmei Electronics Co.,Ltd",
    "000F92": "Microhard Systems Inc.",
    "000F4D": "TalkSwitch",
    "000EB6": "Riverbed Technology, Inc.",
    "000EB7": "Knovative, Inc.",
    "000EB1": "Newcotech,Ltd",
    "000EB4": "GUANGZHOU GAOKE COMMUNICATIONS TECHNOLOGY CO.LTD.",
    "000EA9": "Shanghai Xun Shi Communications Equipment Ltd. Co.",
    "000EFB": "Macey Enterprises",
    "000EFA": "Optoway Technology Incorporation",
    "000EFD": "FUJINON CORPORATION",
    "000EF5": "iPAC Technology Co., Ltd.",
    "000ECD": "SKOV A/S",
    "000ECA": "WTSS Inc",
    "000ECC": "Tableau, LLC",
    "000ED5": "COPAN Systems Inc.",
    "000EC8": "Zoran Corporation",
    "000ECB": "VineSys Technology",
    "000ED2": "Filtronic plc",
    "000ED9": "Aksys, Ltd.",
    "000EC2": "Lowrance Electronics, Inc.",
    "000F09": "Private",
    "000EA2": "McAfee, Inc",
    "000E9B": "Ambit Microsystems Corporation",
    "000F12": "Panasonic Europe Ltd.",
    "000F0E": "WaveSplitter Technologies, Inc.",
    "000F0C": "SYNCHRONIC ENGINEERING",
    "000F0B": "Kentima Technologies AB",
    "000EA3": "CNCR-IT CO.,LTD,HangZhou P.R.CHINA",
    "000EDD": "SHURE INCORPORATED",
    "000EDC": "Tellion INC.",
    "000E23": "Incipient, Inc.",
    "000E25": "Hannae Technology Co., Ltd",
    "000E20": "ACCESS Systems Americas, Inc.",
    "000E21": "MTU Friedrichshafen GmbH",
    "000E22": "Private",
    "000E1C": "Hach Company",
    "000E7D": "Electronics Line 3000 Ltd.",
    "000E78": "Amtelco",
    "000E71": "Gemstar Technology Development Ltd.",
    "000E70": "in2 Networks",
    "000E37": "Harms & Wende GmbH & Co.KG",
    "000E31": "Olympus Soft Imaging Solutions GmbH",
    "000E2F": "Roche Diagnostics GmbH",
    "000E2C": "Netcodec co.",
    "000E4A": "Changchun Huayu WEBPAD Co.,LTD",
    "000E41": "NIHON MECHATRONICS CO.,LTD.",
    "000E3C": "Transact Technologies Inc",
    "000E63": "Lemke Diagnostics GmbH",
    "000E5B": "ParkerVision - Direct2Data",
    "000E60": "360SUN Digital Broadband Corporation",
    "000E09": "Shenzhen Coship Software Co.,LTD.",
    "000E05": "WIRELESS MATRIX CORP.",
    "000E93": "Mil\u00e9nio 3 Sistemas Electr\u00f3nicos, Lda.",
    "000E8D": "Systems in Progress Holding GmbH",
    "000E54": "AlphaCell Wireless Ltd.",
    "000E4E": "Waveplus Technology Co., Ltd.",
    "000E76": "GEMSOC INNOVISION INC.",
    "000DE0": "ICPDAS Co.,LTD",
    "000DE3": "AT Sweden AB",
    "000DD4": "Symantec Corporation",
    "000DD2": "Simrad Optronics ASA",
    "000DD1": "Stryker Corporation",
    "000DD7": "Bright",
    "000DBE": "Bel Fuse Europe Ltd.,UK",
    "000DBC": "Cisco Systems, Inc",
    "000DEA": "Kingtel Telecommunication Corp.",
    "000DED": "Cisco Systems, Inc",
    "000DE4": "DIGINICS, Inc.",
    "000DDC": "VAC",
    "000DD9": "Anton Paar GmbH",
    "000DC9": "THALES Elektronik Systeme GmbH",
    "000D9F": "RF Micro Devices",
    "000DA5": "Fabric7 Systems, Inc",
    "000DC5": "EchoStar Global B.V. ",
    "000DC8": "AirMagnet, Inc",
    "000E02": "Advantech AMT Inc.",
    "000D89": "Bils Technology Inc",
    "000D86": "Huber + Suhner AG",
    "000D81": "Pepperl+Fuchs GmbH",
    "000D7A": "DiGATTO Asia Pacific Pte Ltd",
    "000D77": "FalconStor Software",
    "000D76": "Hokuto Denshi Co,. Ltd.",
    "000D14": "Vtech Innovation LP dba Advanced American Telephones",
    "000D17": "Turbo Networks Co.Ltd",
    "000D18": "Mega-Trend Electronics CO., LTD.",
    "000D20": "ASAHIKASEI TECHNOSYSTEM CO.,LTD.",
    "000D6F": "Ember Corporation",
    "000D5E": "NEC Personal Products",
    "000D5B": "Smart Empire Investments Limited",
    "000D59": "Amity Systems, Inc.",
    "000D50": "Galazar Networks",
    "000D25": "SANDEN CORPORATION",
    "000D24": "SENTEC E&E CO., LTD.",
    "000D22": "Unitronics LTD",
    "000D99": "Orbital Sciences Corp.; Launch Systems Group",
    "000D8F": "King Tsushin Kogyo Co., LTD.",
    "000D49": "Triton Systems of Delaware, Inc.",
    "000D48": "AEWIN Technologies Co., Ltd.",
    "000D3F": "VTI Instruments Corporation",
    "000D3A": "Microsoft Corp.",
    "000D30": "IceFyre Semiconductor",
    "000D31": "Compellent Technologies, Inc.",
    "000D7B": "Consensys Computers Inc.",
    "000D6C": "M-Audio",
    "000D70": "Datamax Corporation",
    "000CE3": "Option International N.V.",
    "000CE7": "MediaTek Inc.",
    "000CE4": "NeuroCom International, Inc.",
    "000CE8": "GuangZhou AnJuBao Co., Ltd",
    "000C34": "Vixen Co., Ltd.",
    "000CB1": "Salland Engineering (Europe) BV",
    "000CBC": "Iscutum",
    "000CCA": "HGST a Western Digital Company",
    "000CB5": "Premier Technolgies, Inc",
    "000CB6": "NANJING SEU MOBILE & INTERNET TECHNOLOGY CO.,LTD",
    "000CC3": "BeWAN systems",
    "000CD0": "Symetrix",
    "000CD9": "Itcare Co., Ltd",
    "000CD5": "Passave Inc.",
    "000CD2": "Schaffner EMV AG",
    "000D04": "Foxboro Eckardt Development GmbH",
    "000D05": "cybernet manufacturing inc.",
    "000D08": "AboveCable, Inc.",
    "000CFE": "Grand Electronic Co., Ltd",
    "000D0E": "Inqnet Systems, Inc.",
    "000D11": "DENTSPLY - Gendex",
    "000CB4": "AutoCell Laboratories, Inc.",
    "000A07": "WebWayOne Ltd",
    "000CB0": "Star Semiconductor Corporation",
    "000C96": "OQO, Inc.",
    "000C98": "LETEK Communications Inc.",
    "000C8E": "Mentor Engineering Inc",
    "000CA2": "Harmonic Video Network",
    "000CA4": "Prompttec Product Management GmbH",
    "000C24": "ANATOR",
    "000C19": "Telio Communications GmbH",
    "000C1D": "Mettler & Fuchs AG",
    "000C13": "MediaQ",
    "000C2D": "FullWave Technology Co., Ltd.",
    "000C26": "Weintek Labs. Inc.",
    "000C2B": "ELIAS Technology, Inc.",
    "000C1A": "Quest Technical Solutions Inc.",
    "000C65": "Sunin Telecom",
    "000C6F": "Amtek system co.,LTD.",
    "000C6C": "Eve Systems GmbH",
    "000C73": "TELSON ELECTRONICS CO., LTD",
    "000C7E": "Tellium Incorporated",
    "000C87": "AMD",
    "000C83": "Logical Solutions",
    "000C5B": "HANWANG TECHNOLOGY CO.,LTD",
    "000C60": "ACM Systems",
    "000C32": "Avionic Design GmbH",
    "000C33": "Compucase Enterprise Co. Ltd.",
    "000C36": "S-Takaya Electronics Industry Co.,Ltd.",
    "000C58": "M&S Systems",
    "000C51": "Scientific Technologies Inc.",
    "000C39": "Sentinel Wireless Inc.",
    "000BF6": "Nitgen Co., Ltd",
    "000C01": "Abatron AG",
    "000BFD": "Cisco Systems, Inc",
    "000BF4": "Private",
    "000BFA": "EXEMYS SRL",
    "000BFB": "D-NET International Corporation",
    "000BA9": "CloudShield Technologies, Inc.",
    "000BA1": "Fujikura Solutions Ltd.",
    "000BA6": "Miyakawa Electric Works Ltd.",
    "000B93": "Ritter Elektronik",
    "000B9B": "Sirius System Co, Ltd.",
    "000BF0": "MoTEX Products Co., Ltd.",
    "000BF1": "LAP Laser Applikations",
    "000BEE": "inc.jet, Incorporated",
    "000BE2": "Lumenera Corporation",
    "000C08": "HUMEX Technologies Corp.",
    "000C0D": "Communications & Power Industries / Satcom Division",
    "000C04": "Tecnova",
    "000BCE": "Free2move AB",
    "000BCF": "AGFA NDT INC.",
    "000BC3": "Multiplex, Inc.",
    "000BBE": "Cisco Systems, Inc",
    "000B8E": "Ascent Corporation",
    "000B8F": "AKITA ELECTRONICS SYSTEMS CO.,LTD.",
    "000BCB": "Fagor Automation , S. Coop",
    "000BC8": "AirFlow Networks",
    "000BE1": "Nokia NET Product Operations",
    "000BE0": "SercoNet Ltd.",
    "000BBD": "Connexionz Limited",
    "000BA8": "HANBACK ELECTRONICS CO., LTD.",
    "000B16": "Communication Machinery Corporation",
    "000B12": "NURI Telecom Co., Ltd.",
    "000B0F": "Bosch Rexroth",
    "000B0C": "Agile Systems Inc.",
    "000B0A": "dBm Optics",
    "000B09": "Ifoundry Systems Singapore",
    "000B8D": "Avvio Networks",
    "000B7F": "Align Engineering LLC",
    "000B85": "Cisco Systems, Inc",
    "000B81": "Kaparel Corporation",
    "000B4E": "Communications & Power Industries",
    "000B4D": "Emuzed",
    "000B40": "Cambridge Industries Group (CIG)",
    "000B44": "Concord Idea Corp.",
    "000B42": "commax Co., Ltd.",
    "000B47": "Advanced Energy",
    "000B3D": "CONTAL OK Ltd.",
    "000B24": "AirLogic",
    "000B1D": "LayerZero Power Systems, Inc.",
    "000B19": "Vernier Networks, Inc.",
    "000B82": "Grandstream Networks, Inc.",
    "000B6E": "Neff Instrument Corp.",
    "000B72": "Lawo AG",
    "000B78": "TAIFATECH INC.",
    "000B6C": "Sychip Inc.",
    "0091D6": "Crystal Group, Inc.",
    "000B31": "Yantai ZhiYang Scientific and technology industry CO., LTD",
    "000B2F": "bplan GmbH",
    "000B5A": "HyperEdge",
    "000A98": "M+F Gwinner GmbH & Co",
    "000A9B": "TB Group Inc",
    "000AC5": "Color Kinetics",
    "000ABD": "Rupprecht & Patashnick Co.",
    "000AB0": "LOYTEC electronics GmbH",
    "000AB5": "Digital Electronic Network",
    "000AFD": "Kentec Electronics",
    "000AEF": "OTRUM ASA",
    "000AE5": "ScottCare Corporation",
    "000A84": "Rainsun Enterprise Co., Ltd.",
    "000A7E": "The Advantage Group",
    "000A78": "OLITEC",
    "000AFE": "NovaPal Ltd",
    "000AA5": "MAXLINK INDUSTRIES LIMITED",
    "000AA9": "Brooks Automation GmbH",
    "000ACB": "XPAK MSA Group",
    "000AD5": "Brainchild Electronic Co., Ltd.",
    "000AD6": "BeamReach Networks",
    "000AD2": "JEPICO Corporation",
    "000AA2": "SYSTEK INC.",
    "000A91": "HemoCue AB",
    "000A6C": "Walchem Corporation",
    "000A6B": "Tadiran Telecom Business Systems LTD",
    "000A5F": "almedio inc.",
    "000A61": "Cellinx Systems Inc.",
    "000A5B": "Power-One as",
    "000A3A": "J-THREE INTERNATIONAL Holding Co., Ltd.",
    "000A47": "Allied Vision Technologies",
    "000A44": "Avery Dennison Deutschland GmbH",
    "000A3C": "Enerpoint Ltd.",
    "000A40": "Crown Audio -- Harmanm International",
    "000A7B": "Cornelius Consult",
    "000A73": "Scientific Atlanta",
    "000A69": "SUNNY bell Technology Co., Ltd.",
    "0009EB": "HuMANDATA LTD.",
    "0009E8": "Cisco Systems, Inc",
    "0009E9": "Cisco Systems, Inc",
    "0009ED": "CipherOptics",
    "0009F2": "Cohu, Inc., Electronics Division",
    "000A26": "CEIA S.p.A.",
    "000A29": "Pan Dacom Networking AG",
    "000A1D": "Optical Communications Products Inc.",
    "000A16": "Lassen Research",
    "000A18": "Vichel Inc.",
    "0009F8": "UNIMO TECHNOLOGY CO., LTD.",
    "0009FF": "X.net 2000 GmbH",
    "000A03": "ENDESA SERVICIOS, S.L.",
    "0009FE": "Daisy Technologies, Inc.",
    "000A28": "Motorola",
    "000A55": "MARKEM Corporation",
    "000A4C": "Molecular Devices Corporation",
    "000A4D": "Noritz Corporation",
    "000A06": "Teledex LLC",
    "000A0D": "Amphenol",
    "0009A2": "Interface Co., Ltd.",
    "0009A1": "Telewise Communications, Inc.",
    "000976": "Datasoft ISDN Systems GmbH",
    "00097A": "Louis Design Labs.",
    "00096B": "IBM Corp",
    "00096D": "Powernet Technologies Corp.",
    "000964": "Hi-Techniques, Inc.",
    "000965": "HyunJu Computer Co., Ltd.",
    "000982": "Loewe Opta GmbH",
    "000983": "GlobalTop Technology, Inc.",
    "00097D": "SecWell Networks Oy",
    "0009AF": "e-generis",
    "0009AD": "HYUNDAI SYSCOMM, INC.",
    "0009A9": "Ikanos Communications",
    "00099F": "VIDEX INC.",
    "0009C2": "Onity, Inc.",
    "0009C3": "NETAS",
    "0009B5": "3J Tech. Co., Ltd.",
    "0009B8": "Entise Systems",
    "0009DC": "Galaxis Technology AG",
    "0009DD": "Mavin Technology Inc.",
    "0009C6": "Visionics Corporation",
    "0009D1": "SERANOA NETWORKS INC",
    "0009CE": "SpaceBridge Semiconductor Corp.",
    "0009BD": "Epygi Technologies, Ltd.",
    "00096F": "Beijing Zhongqing Elegant Tech. Corp.,Limited",
    "00095E": "Masstech Group Inc.",
    "000939": "ShibaSoku Co.,Ltd.",
    "000933": "Ophit Co.Ltd.",
    "000932": "Omnilux",
    "00091C": "CacheVision, Inc",
    "00091A": "Macat Optics & Electronics Co., Ltd.",
    "00091B": "Digital Generation Inc.",
    "000959": "Sitecsoft",
    "000957": "Supercaller, Inc.",
    "00094F": "elmegt GmbH & Co. KG",
    "0008F2": "C&S Technology",
    "0008EA": "Motion Control Engineering, Inc",
    "000902": "Redline Communications Inc.",
    "0008FA": "KEB Automation KG",
    "000929": "Sanyo Industries (UK) Limited",
    "000928": "Telecore",
    "0008ED": "ST&T Instrument Corp.",
    "000943": "Cisco Systems, Inc",
    "000872": "Sorenson Communications",
    "00087C": "Cisco Systems, Inc",
    "000879": "CEM Corporation",
    "00087D": "Cisco Systems, Inc",
    "000875": "Acorp Electronics Corp.",
    "00086F": "Resources Computer Network Ltd.",
    "0008DF": "Alistel Inc.",
    "0008DB": "Corrigent Systems",
    "0008D8": "Dowkey Microwave",
    "0008D2": "ZOOM Networks Inc.",
    "0008B6": "RouteFree, Inc.",
    "0008B0": "HUBER+SUHNER BKtel GmbH",
    "0008AC": " BST GmbH",
    "0008A1": "CNet Technology Inc.",
    "000893": "LE INFORMATION COMMUNICATION INC.",
    "000888": "OULLIM Information Technology Inc,.",
    "000885": "EMS Dr. Thomas W\u00fcnsche",
    "0008C5": "Liontech Co., Ltd.",
    "0008CA": "TwinHan Technology Co.,Ltd",
    "0008AA": "KARAM",
    "0008AB": "EnerLinx.com, Inc.",
    "0008AD": "Toyo-Linx Co., Ltd.",
    "00089A": "Alcatel Microelectronics",
    "000826": "Colorado Med Tech",
    "00081C": "@pos.com",
    "000867": "Uptime Devices",
    "00085E": "PCO AG",
    "000856": "Gamatronic Electronic Industries Ltd.",
    "000853": "Schleicher GmbH & Co. Relaiswerke KG",
    "0007A3": "Ositis Software, Inc.",
    "00079A": "Verint Systems Inc",
    "000774": "GuangZhou Thinker Technology Co. Ltd.",
    "000798": "Selea SRL",
    "000791": "International Data Communications, Inc.",
    "0005F9": "TOA Corporation",
    "0007CA": "Creatix Polymedia Ges Fur Kommunikaitonssysteme",
    "0007C5": "Gcom, Inc.",
    "0007C8": "Brain21, Inc.",
    "0007C1": "Overture Networks, Inc.",
    "0007AE": "Britestream Networks, Inc.",
    "0007B1": "Equator Technologies",
    "0007A7": "A-Z Inc.",
    "0007A6": "Leviton Manufacturing Co., Inc.",
    "0007F3": "Thinkengine Networks",
    "0007EE": "telco Informationssysteme GmbH",
    "0007E2": "Bitworks, Inc.",
    "0007E6": "edgeflow Canada Inc.",
    "000816": "Bluelon ApS",
    "000811": "VOIX Corporation",
    "000806": "Raonet Systems, Inc.",
    "0007E5": "Coup Corporation",
    "0007DE": "eCopilt AB",
    "0007DF": "Vbrick Systems Inc.",
    "0007C2": "Netsys Telecom",
    "00078F": "Emkay Innovative Products",
    "000782": "Oracle Corporation ",
    "0007F8": "ITDevices, Inc.",
    "0006F2": "Platys Communications",
    "0006FA": "IP SQUARE Co, Ltd.",
    "000703": "CSEE Transport",
    "000706": "Sanritz Corporation",
    "0006EF": "Maxxan Systems, Inc.",
    "0006E9": "Intime Corp.",
    "000760": "TOMIS Information & Telecom Corp.",
    "00074E": "IPFRONT Inc",
    "000752": "Rhythm Watch Co., Ltd.",
    "00074F": "Cisco Systems, Inc",
    "000707": "Interalia Inc.",
    "0006F0": "Digeo, Inc.",
    "000700": "Zettamedia Korea",
    "000743": "Chelsio Communications",
    "000747": "Mecalc",
    "000744": "Unico, Inc.",
    "000749": "CENiX Inc.",
    "000730": "Hutchison OPTEL Telecom Technology Co., Ltd.",
    "000722": "The Nielsen Company",
    "00071F": "European Systems Integration",
    "000721": "Formac Elektronik GmbH",
    "000739": "Scotty Group Austria Gmbh",
    "00073D": "Nanjing Postel Telecommunications Co., Ltd.",
    "000735": "Flarion Technologies, Inc.",
    "000786": "Wireless Networks Inc.",
    "000779": "Sungil Telecom Co., Ltd.",
    "00077E": "Elrest GmbH",
    "000778": "GERSTEL GmbH & Co. KG",
    "000708": "Bitrage Inc.",
    "000712": "JAL Information Technology",
    "000713": "IP One, Inc.",
    "00076D": "Flexlight Networks",
    "00075E": "Ametek Power Instruments",
    "0006BC": "Macrolink, Inc.",
    "0006C2": "Smartmatic Corporation",
    "000654": "Winpresa Building Automation Technologies GmbH",
    "0006B4": "Vorne Industries, Inc.",
    "0006AE": "Himachal Futuristic Communications Ltd",
    "0006B0": "Comtech EF Data Corp.",
    "0006CA": "American Computer & Digital Components, Inc. (ACDC)",
    "0006CE": "DATENO",
    "0006C5": "INNOVI Technologies Limited",
    "0006C6": "lesswire AG",
    "0006B7": "TELEM GmbH",
    "0006EA": "ELZET80 Mikrocomputer GmbH&Co. KG",
    "0006E5": "Fujian Newland Computer Ltd. Co.",
    "0006DE": "Flash Technology",
    "0006DF": "AIDONIC Corporation",
    "00068E": "HID Corporation",
    "00068A": "NeuronNet Co. Ltd. R&D Center",
    "000685": "NetNearU Corporation",
    "00067F": "Digeo, Inc.",
    "000683": "Bravara Communications, Inc.",
    "000655": "Yipee, Inc.",
    "00067A": "JMP Systems",
    "000673": "TKH Security Solutions USA",
    "000676": "Novra Technologies Inc.",
    "000664": "Fostex Corporation",
    "0006DD": "AT & T Laboratories - Cambridge Ltd",
    "0006D1": "Tahoe Networks, Inc.",
    "0006D4": "Interactive Objects, Inc.",
    "00069B": "AVT Audio Video Technologies GmbH",
    "000693": "Flexus Computer Technology, Inc.",
    "000696": "Advent Networks",
    "00065D": "Heidelberg Web Systems",
    "000650": "Tiburon Networks, Inc.",
    "00065E": "Photuris, Inc.",
    "0005D7": "Vista Imaging, Inc.",
    "0005DB": "PSI Nentec GmbH",
    "0005DD": "Cisco Systems, Inc",
    "0005D9": "Techno Valley, Inc.",
    "0005DC": "Cisco Systems, Inc",
    "0005C3": "Pacific Instruments, Inc.",
    "0005B9": "Airvana, Inc.",
    "0005BC": "Resource Data Management Ltd",
    "0005BE": "Kongsberg Seatex AS",
    "0005BD": "ROAX BV",
    "0005C1": "A-Kyung Motion, Inc.",
    "0005B6": "INSYS Microelectronics GmbH",
    "000644": "NextGen Business Solutions, Inc",
    "000645": "Meisei Electric Co. Ltd.",
    "000640": "White Rock Networks",
    "00063A": "Dura Micro, Inc.",
    "000628": "Cisco Systems, Inc",
    "00061D": "MIP Telecom, Inc.",
    "000619": "Connection Technology Systems",
    "000632": "Mesco Engineering GmbH",
    "000634": "GTE Airfone Inc.",
    "000608": "At-Sky SAS",
    "0005F6": "Young Chang Co. Ltd.",
    "0005FC": "Schenck Pegasus Corp.",
    "0005E8": "TurboWave, Inc.",
    "A06A00": "Verilink Corporation",
    "0005F8": "Real Time Access, Inc.",
    "0005EB": "Blue Ridge Networks, Inc.",
    "00060C": "Melco Industries, Inc.",
    "00060E": "IGYS Systems, Inc.",
    "000614": "Prism Holdings",
    "0005D0": "Solinet Systems",
    "000580": "FibroLAN Ltd.",
    "000582": "ClearCube Technology",
    "000578": "Private",
    "000572": "Deonet Co., Ltd.",
    "000576": "NSM Technology Ltd.",
    "00054A": "Ario Data Networks, Inc.",
    "000548": "Disco Corporation",
    "00053E": "KID Systeme GmbH",
    "00053F": "VisionTek, Inc.",
    "0005B4": "Aceex Corporation",
    "000598": "CRONOS S.r.l.",
    "0005B7": "Arbor Technology Corp.",
    "00059B": "Cisco Systems, Inc",
    "000590": "Swissvoice Ltd.",
    "000595": "Alesis Corporation",
    "000562": "Digital View Limited",
    "000555": "Japan Cash Machine Co., Ltd.",
    "000552": "Xycotec Computer GmbH",
    "00059F": "Yotta Networks, Inc.",
    "000587": "Locus, Incorporated",
    "00053D": "Agere Systems",
    "000535": "Chip PC Ltd.",
    "000527": "SJ Tek Co. Ltd",
    "000529": "Shanghai Broadan Communication Technology Co., Ltd",
    "00056A": "Heuft Systemtechnik GmbH",
    "000568": "Piltofish Networks AB",
    "0004DA": "Relax Technology, Inc.",
    "0004E5": "Glonet Systems, Inc.",
    "0004D2": "Adcon Telemetry GmbH",
    "0004D3": "Toyokeiki Co., Ltd.",
    "0004D5": "Hitachi Information & Communication Engineering, Ltd.",
    "0004CA": "FreeMs Corp.",
    "00050D": "Midstream Technologies, Inc.",
    "000500": "Cisco Systems, Inc",
    "0004FD": "Japan Control Engineering Co., Ltd.",
    "0004F7": "Omega Band, Inc.",
    "0004EE": "Lincoln Electric Company",
    "0004F0": "International Computers, Ltd",
    "0004C5": "ASE Technologies, USA",
    "00052C": "Supreme Magic Corporation",
    "000519": "Siemens Building Technologies AG,",
    "000507": "Fine Appliance Corp.",
    "0004B0": "ELESIGN Co., Ltd.",
    "0004AB": "Mavenir Inc.",
    "0004A7": "FabiaTech Corporation",
    "0004AA": "Jetstream Communications",
    "0004A0": "Verity Instruments, Inc.",
    "00049E": "Wirelink Co., Ltd.",
    "00049A": "Cisco Systems, Inc",
    "000447": "Acrowave Systems Co., Ltd.",
    "00043E": "Telencomm",
    "000436": "ELANsat Technologies, Inc.",
    "000435": "InfiNet LLC",
    "000432": "Voyetra Turtle Beach, Inc.",
    "000437": "Powin Information Technology, Inc.",
    "000414": "Umezawa Musen Denki Co., Ltd.",
    "00040C": "Kanno Works, Ltd.",
    "000408": "Sanko Electronics Co., Ltd.",
    "000409": "Cratos Networks",
    "000407": "Topcon Positioning Systems, Inc.",
    "000455": "ANTARA.net",
    "000488": "Eurotherm Controls",
    "000485": "PicoLight",
    "000479": "Radius Co., Ltd.",
    "000457": "Universal Access Technology, Inc.",
    "00044D": "Cisco Systems, Inc",
    "000454": "Quadriga UK",
    "000448": "Polaroid Corporation",
    "000498": "Mahi Networks",
    "000497": "MacroSystem Digital Video AG",
    "00046D": "Cisco Systems, Inc",
    "000466": "ARMITEL Co.",
    "000428": "Cisco Systems, Inc",
    "0003D3": "Internet Energy Systems, Inc.",
    "0003CD": "Clovertech, Inc.",
    "0003CC": "Momentum Computer, Inc.",
    "00037D": "Stellcom",
    "000383": "Metera Networks, Inc.",
    "000379": "Proscend Communications, Inc.",
    "00038B": "PLUS-ONE I&T, Inc.",
    "00038C": "Total Impact",
    "000386": "Ho Net, Inc.",
    "0003E6": "Entone, Inc.",
    "0003D7": "NextNet Wireless, Inc.",
    "0003EE": "MKNet Corporation",
    "0003EA": "Mega System Technologies, Inc.",
    "0003B5": "Entra Technology Co.",
    "00039F": "Cisco Systems, Inc",
    "000390": "Digital Video Communications, Inc.",
    "0003E2": "Comspace Corporation",
    "0003FF": "Microsoft Corporation",
    "0003A9": "AXCENT Media AG",
    "000366": "ASM Pacific Technology",
    "000362": "Vodtel Communications, Inc.",
    "000364": "Scenix Semiconductor, Inc.",
    "00035E": "Metropolitan Area Networks, Inc.",
    "00035C": "Saint Song Corp.",
    "00035D": "Bosung Hi-Net Co., Ltd.",
    "000341": "Axon Digital Design",
    "008037": "Ericsson Group",
    "00033D": "ILSHin Lab",
    "000334": "Omega Engineering Inc.",
    "00033A": "Silicon Wave, Inc.",
    "0002E8": "E.D.&A.",
    "0002D9": "Reliable Controls",
    "00030E": "Core Communications Co., Ltd.",
    "000312": "TRsystems GmbH",
    "0002F2": "eDevice, Inc.",
    "0002EF": "CCC Network Systems Group Ltd.",
    "000332": "Cisco Systems, Inc",
    "000326": "Iwasaki Information Systems Co., Ltd.",
    "00036F": "Telsey SPA",
    "000372": "ULAN",
    "00034D": "Chiaro Networks, Ltd.",
    "000322": "IDIS Co., Ltd.",
    "00031D": "Taiwan Commate Computer, Inc.",
    "00025D": "Calix Networks",
    "000257": "Microcom Corp.",
    "000253": "Televideo, Inc.",
    "00024A": "Cisco Systems, Inc",
    "000249": "Aviv Infocom Co, Ltd.",
    "000245": "Lampus Co, Ltd.",
    "000246": "All-Win Tech Co., Ltd.",
    "000290": "Woorigisool, Inc.",
    "000292": "Logic Innovations, Inc.",
    "000286": "Occam Networks",
    "001095": "Thomson Inc.",
    "00027B": "Amplify Net, Inc.",
    "000273": "Coriolis Networks",
    "0002B4": "DAPHNE",
    "0002B0": "Hokubu Communication & Industrial Co., Ltd.",
    "0002AA": "PLcom Co., Ltd.",
    "00029D": "Merix Corp.",
    "0002D2": "Workstation AG",
    "0002CD": "TeleDream, Inc.",
    "0002D0": "Comdial Corporation",
    "0002CC": "M.C.C.I",
    "00026F": "Senao International Co., Ltd.",
    "0002C5": "Evertz Microsystems Ltd.",
    "0002B5": "Avnet, Inc.",
    "00027E": "Cisco Systems, Inc",
    "00027F": "ask-technologies.com",
    "00023A": "ZSK Stickmaschinen GmbH",
    "000232": "Avision, Inc.",
    "000235": "Paragon Networks International",
    "000237": "Cosmo Research Corp.",
    "000228": "Necsom, Ltd.",
    "0001AC": "Sitara Networks, Inc.",
    "0001AD": "Coach Master International  d.b.a. CMI Worldwide, Inc.",
    "00019B": "Kyoto Microcomputer Co., Ltd.",
    "000230": "Intersoft Electronics",
    "00021C": "Network Elements, Inc.",
    "00020C": "Metro-Optix",
    "000216": "Cisco Systems, Inc",
    "000214": "DTVRO",
    "00020F": "AATR",
    "0001C3": "Acromag, Inc.",
    "0001C2": "ARK Research Corp.",
    "0001ED": "SETA Corp.",
    "0001EA": "Cirilium Corp.",
    "0001E0": "Fast Systems, Inc.",
    "0001D6": "manroland AG",
    "0001F6": "Association of Musical Electronics Industry",
    "0001CB": "EVR",
    "00013A": "SHELCAD COMMUNICATIONS, LTD.",
    "000140": "Sendtek Corporation",
    "000123": "Schneider Electric Japan Holdings Ltd.",
    "000125": "YAESU MUSEN CO., LTD.",
    "000126": "PAC Labs",
    "00011B": "Unizone Technologies, Inc.",
    "000171": "Allied Data Technologies",
    "000176": "Orient Silver Enterprises",
    "000158": "Electro Industries/Gauge Tech",
    "0030AC": "Systeme Lauer GmbH & Co., Ltd.",
    "00015A": "Digital Video Broadcasting",
    "00011D": "Centillium Communications",
    "00011F": "RC Networks, Inc.",
    "00012A": "Telematica Sistems Inteligente",
    "00012D": "Komodo Technology",
    "000148": "X-traWeb Inc.",
    "000188": "LXCO Technologies ag",
    "00017F": "Experience Music Project",
    "000187": "I2SE GmbH",
    "00019A": "LEUNIG GmbH",
    "000197": "Cisco Systems, Inc",
    "000159": "S1 Corporation",
    "000166": "TC GROUP A/S",
    "000180": "AOpen, Inc.",
    "00B09A": "Morrow Technologies Corp.",
    "0030A9": "Netiverse, Inc.",
    "0030FE": "DSA GmbH",
    "0030C4": "Canon Imaging Systems Inc.",
    "00304D": "ESI",
    "00302E": "Hoft & Wessel AG",
    "0030ED": "Expert Magnetics Corp.",
    "00300F": "IMT - Information Management T",
    "003082": "TAIHAN ELECTRIC WIRE CO., LTD.",
    "0030FB": "AZS Technology AG",
    "003003": "Phasys Ltd.",
    "0030AE": "Times N System, Inc.",
    "0030E2": "GARNET SYSTEMS CO., LTD.",
    "0030D5": "DResearch GmbH",
    "003018": "Jetway Information Co., Ltd.",
    "003089": "Spectrapoint Wireless, LLC",
    "00B080": "Mannesmann Ipulsys B.V.",
    "00B01E": "Rantic Labs, Inc.",
    "00B0F0": "CALY NETWORKS",
    "003090": "CYRA TECHNOLOGIES, INC.",
    "0030A7": "SCHWEITZER ENGINEERING",
    "003023": "COGENT COMPUTER SYSTEMS, INC.",
    "00307C": "ADID SA",
    "00309A": "ASTRO TERRA CORP.",
    "00309F": "AMBER NETWORKS",
    "0030A8": "OL'E COMMUNICATIONS, INC.",
    "0030D1": "INOVA CORPORATION",
    "081443": "UNIBRAIN S.A.",
    "00B009": "Grass Valley, A Belden Brand",
    "00B0AC": "SIAE-Microelettronica S.p.A.",
    "003055": "Renesas Technology America, Inc.",
    "00302F": "GE Aviation System",
    "00300E": "Klotz Digital AG",
    "0030BB": "CacheFlow, Inc.",
    "00302D": "QUANTUM BRIDGE COMMUNICATIONS",
    "0030CB": "OMNI FLOW COMPUTERS, INC.",
    "00306B": "CMOS SYSTEMS, INC.",
    "0030AD": "SHANGHAI COMMUNICATION",
    "00D0CF": "MORETON BAY",
    "00D07F": "STRATEGY & TECHNOLOGY, LIMITED",
    "003036": "RMP ELEKTRONIKSYSTEME GMBH",
    "003035": "Corning Incorporated",
    "00307F": "IRLAN LTD.",
    "0030E6": "Draeger Medical Systems, Inc.",
    "003062": "IP Video Networks Inc",
    "00D085": "OTIS ELEVATOR COMPANY",
    "00D0E9": "Advantage Century Telecommunication Corp.",
    "00D015": "UNIVEX MICROTECHNOLOGY CORP.",
    "00D0A5": "AMERICAN ARIUM",
    "00D048": "ECTON, INC.",
    "0030BD": "BELKIN COMPONENTS",
    "0030CF": "TWO TECHNOLOGIES, INC.",
    "0030B2": "L-3 Sonoma EO",
    "00305D": "DIGITRA SYSTEMS, INC.",
    "003087": "VEGA GRIESHABER KG",
    "0030AA": "AXUS MICROSYSTEMS, INC.",
    "003007": "OPTI, INC.",
    "00D022": "INCREDIBLE TECHNOLOGIES, INC.",
    "00D071": "ECHELON CORP.",
    "00D04F": "BITRONICS, INC.",
    "00D0FB": "TEK MICROSYSTEMS, INCORPORATED",
    "00D066": "WINTRISS ENGINEERING CORP.",
    "00D082": "IOWAVE INC.",
    "00D09C": "KAPADIA COMMUNICATIONS",
    "00D0F3": "SOLARI DI UDINE SPA",
    "00D039": "UTILICOM, INC.",
    "00D067": "CAMPIO COMMUNICATIONS",
    "00D058": "Cisco Systems, Inc",
    "00D03D": "GALILEO TECHNOLOGY, LTD.",
    "00D032": "YANO ELECTRIC CO., LTD.",
    "00D0F1": "SEGA ENTERPRISES, LTD.",
    "00D090": "Cisco Systems, Inc",
    "00D0F5": "ORANGE MICRO, INC.",
    "00D078": "Eltex of Sweden AB",
    "00D0C2": "BALTHAZAR TECHNOLOGY AB",
    "00D081": "RTD Embedded Technologies, Inc.",
    "00D002": "DITECH CORPORATION",
    "00D09B": "SPECTEL LTD.",
    "00D041": "AMIGO TECHNOLOGY CO., LTD.",
    "00D09D": "VERIS INDUSTRIES",
    "00D094": "Seeion Control LLC",
    "00D069": "TECHNOLOGIC SYSTEMS",
    "00D011": "PRISM VIDEO, INC.",
    "00D0DF": "KUZUMI ELECTRONICS, INC.",
    "00D062": "DIGIGRAM",
    "00D08D": "PHOENIX GROUP, INC.",
    "00D04C": "Eseye Design Ltd",
    "00D0D9": "DEDICATED MICROCOMPUTERS",
    "00D0E1": "AVIONITEK ISRAEL INC.",
    "00D008": "MACTELL CORPORATION",
    "00D00B": "RHK TECHNOLOGY, INC.",
    "00D0A0": "MIPS DENMARK",
    "00D00A": "LANACCESS TELECOM S.A.",
    "00D01C": "SBS TECHNOLOGIES,",
    "00D0D5": "GRUNDIG AG",
    "005046": "MENICX INTERNATIONAL CO., LTD.",
    "0050B0": "TECHNOLOGY ATLANTA CORPORATION",
    "005041": "Coretronic Corporation",
    "0050DD": "SERRA SOLDADURA, S.A.",
    "005067": "AEROCOMM, INC.",
    "0050B6": "GOOD WAY IND. CO., LTD.",
    "00504B": "BARCONET N.V.",
    "00D03A": "ZONEWORX, INC.",
    "00D001": "VST TECHNOLOGIES, INC.",
    "005020": "MEDIASTAR CO., LTD.",
    "00D075": "ALARIS MEDICAL SYSTEMS, INC.",
    "0050E7": "PARADISE INNOVATIONS (ASIA)",
    "0050FB": "VSK ELECTRONICS",
    "0050A5": "CAPITOL BUSINESS SYSTEMS, LTD.",
    "005000": "NEXO COMMUNICATIONS, INC.",
    "0050C8": "Addonics Technologies, Inc.",
    "005089": "SAFETY MANAGEMENT SYSTEMS",
    "005066": "AtecoM GmbH advanced telecomunication modules",
    "005059": "iBAHN",
    "0050D9": "ENGETRON-ENGENHARIA ELETRONICA IND. e COM. LTDA",
    "0050F4": "SIGMATEK GMBH & CO. KG",
    "005021": "EIS INTERNATIONAL, INC.",
    "00505E": "DIGITEK MICROLOGIC S.A.",
    "005090": "DCTRI",
    "00503B": "MEDIAFIRE CORPORATION",
    "0050EA": "XEL COMMUNICATIONS, INC.",
    "0050E8": "Nomadix, Inc",
    "0050AE": "FDK Co., Ltd",
    "00D06D": "ACRISON, INC.",
    "00D02B": "JETCELL, INC.",
    "00503E": "Cisco Systems, Inc",
    "0050C6": "LOOP TELECOMMUNICATION INTERNATIONAL, INC.",
    "00509F": "HORIZON COMPUTER",
    "005063": "OY COMSEL SYSTEM AB",
    "00508D": "ABIT COMPUTER CORPORATION",
    "0050A0": "DELTA COMPUTER SYSTEMS, INC.",
    "005086": "TELKOM SA, LTD.",
    "00501A": "IQinVision",
    "00508F": "ASITA TECHNOLOGIES INT'L LTD.",
    "005015": "BRIGHT STAR ENGINEERING",
    "005057": "BROADBAND ACCESS SYSTEMS",
    "005088": "AMANO CORPORATION",
    "005031": "AEROFLEX LABORATORIES, INC.",
    "00907B": "E-TECH, INC.",
    "009081": "ALOHA NETWORKS, INC.",
    "00901C": "mps Software Gmbh",
    "0090DB": "NEXT LEVEL COMMUNICATIONS",
    "009056": "TELESTREAM, INC.",
    "009034": "IMAGIC, INC.",
    "009073": "GAIO TECHNOLOGY",
    "00905E": "RAULAND-BORG CORPORATION",
    "0090AF": "J. MORITA MFG. CORP.",
    "0090BB": "TAINET COMMUNICATION SYSTEM Corp.",
    "005003": "Xrite Inc",
    "0050D3": "DIGITAL AUDIO PROCESSING PTY. LTD.",
    "0050AD": "CommUnique Wireless Corp.",
    "0050AF": "INTERGON, INC.",
    "009068": "DVT CORP.",
    "0090E5": "TEKNEMA, INC.",
    "0090F4": "LIGHTNING INSTRUMENTATION",
    "009074": "ARGON NETWORKS, INC.",
    "00903B": "TriEMS Research Lab, Inc.",
    "00909F": "DIGI-DATA CORPORATION",
    "009019": "HERMES ELECTRONICS CO., LTD.",
    "009005": "PROTECH SYSTEMS CO., LTD.",
    "0090F8": "MEDIATRIX TELECOM",
    "009010": "SIMULATION LABORATORIES, INC.",
    "0090C6": "OPTIM SYSTEMS, INC.",
    "00902E": "NAMCO LIMITED",
    "00908F": "AUDIO CODES LTD.",
    "009059": "TELECOM DEVICE K.K.",
    "0090CA": "ACCORD VIDEO TELECOMMUNICATIONS, LTD.",
    "0090E9": "JANZ COMPUTER AG",
    "009037": "ACUCOMM, INC.",
    "009078": "MER TELEMANAGEMENT SOLUTIONS, LTD.",
    "0090AA": "INDIGO ACTIVE VISION SYSTEMS LIMITED",
    "00905B": "RAYMOND AND LAE ENGINEERING",
    "0090BC": "TELEMANN CO., LTD.",
    "00900A": "PROTON ELECTRONIC INDUSTRIAL CO., LTD.",
    "009090": "I-BUS",
    "00901A": "UNISPHERE SOLUTIONS",
    "0090B5": "NIKON CORPORATION",
    "0090EB": "SENTRY TELECOM SYSTEMS",
    "0090FE": "ELECOM CO., LTD.  (LANEED DIV.)",
    "0090D5": "EUPHONIX, INC.",
    "00904A": "CONCUR SYSTEM TECHNOLOGIES",
    "001047": "ECHO ELETRIC CO. LTD.",
    "00903F": "WorldCast Systems",
    "0090B8": "ROHDE & SCHWARZ GMBH & CO. KG",
    "0090D8": "WHITECROSS SYSTEMS",
    "0010FD": "COCOM A/S",
    "0010C9": "MITSUBISHI ELECTRONICS LOGISTIC SUPPORT CO.",
    "000400": "LEXMARK INTERNATIONAL, INC.",
    "0010C5": "PROTOCOL TECHNOLOGIES, INC.",
    "00101A": "PictureTel Corp.",
    "0010BA": "MARTINHO-DAVIS SYSTEMS, INC.",
    "0010C2": "WILLNET, INC.",
    "001040": "INTERMEC CORPORATION",
    "00102E": "NETWORK SYSTEMS & TECHNOLOGIES PVT. LTD.",
    "001046": "ALCORN MCBRIDE INC.",
    "0010B7": "COYOTE TECHNOLOGIES, LLC",
    "001028": "COMPUTER TECHNICA, INC.",
    "00102C": "Lasat Networks A/S",
    "0010A5": "OXFORD INSTRUMENTS",
    "0010D7": "ARGOSY RESEARCH INC.",
    "001092": "NETCORE INC.",
    "00101C": "OHM TECHNOLOGIES INTL, LLC",
    "001067": "Ericsson",
    "001021": "ENCANTO NETWORKS, INC.",
    "001064": "DNPG, LLC",
    "00109E": "AWARE, INC.",
    "001005": "UEC COMMERCIAL",
    "0010B8": "ISHIGAKI COMPUTER SYSTEM CO.",
    "00108B": "LASERANIMATION SOLLINGER GMBH",
    "0010C7": "DATA TRANSMISSION NETWORK",
    "001074": "ATEN INTERNATIONAL CO., LTD.",
    "001070": "CARADON TREND LTD.",
    "001043": "A2 CORPORATION",
    "00104E": "CEOLOGIC",
    "0010B0": "MERIDIAN TECHNOLOGY CORP.",
    "0004AC": "IBM Corp",
    "0010B4": "ATMOSPHERE NETWORKS",
    "00E08C": "NEOPARADIGM LABS, INC.",
    "00E028": "APTIX CORPORATION",
    "00E0A1": "HIMA PAUL HILDEBRANDT GmbH Co. KG",
    "00E088": "LTX-Credence CORPORATION",
    "00E03D": "FOCON ELECTRONIC SYSTEMS A/S",
    "00E046": "BENTLY NEVADA CORP.",
    "00E058": "PHASE ONE DENMARK A/S",
    "00E076": "DEVELOPMENT CONCEPTS, INC.",
    "00E07D": "NETRONIX, INC.",
    "00E05D": "UNITEC CO., LTD.",
    "00E05E": "JAPAN AVIATION ELECTRONICS INDUSTRY, LTD.",
    "00E09D": "SARNOFF CORPORATION",
    "00E0F2": "ARLOTTO COMNET, INC.",
    "00E0E1": "G2 NETWORKS, INC.",
    "00E08D": "PRESSURE SYSTEMS, INC.",
    "00E0FF": "SECURITY DYNAMICS TECHNOLOGIES, Inc.",
    "00E0AB": "DIMAT S.A.",
    "00E030": "MELITA INTERNATIONAL CORP.",
    "00E0AA": "ELECTROSONIC LTD.",
    "00E010": "HESS SB-AUTOMATENBAU GmbH",
    "006049": "VINA TECHNOLOGIES",
    "00608D": "UNIPULSE CORP.",
    "006099": "SBE, Inc.",
    "0060B3": "Z-COM, INC.",
    "006002": "SCREEN SUBTITLING SYSTEMS, LTD",
    "00E033": "E.E.P.D. GmbH",
    "00E0A2": "MICROSLATE INC.",
    "00E079": "A.T.N.R.",
    "00E075": "Verilink Corporation",
    "00E02E": "SPC ELECTRONICS CORPORATION",
    "00E0D2": "VERSANET COMMUNICATIONS, INC.",
    "00E047": "InFocus Corporation",
    "00E0C3": "SAKAI SYSTEM DEVELOPMENT CORP.",
    "00E092": "ADMTEK INCORPORATED",
    "00E03E": "ALFATECH, INC.",
    "00E09A": "Positron Inc.",
    "00E019": "ING. GIORDANO ELETTRONICA",
    "00E07B": "BAY NETWORKS",
    "00E01D": "WebTV NETWORKS, INC.",
    "006089": "XATA",
    "006021": "DSC CORPORATION",
    "0060B8": "CORELIS Inc.",
    "006039": "SanCom Technology, Inc.",
    "00E0F8": "DICNA CONTROL AB",
    "00606D": "DIGITAL EQUIPMENT CORP.",
    "0060CE": "ACCLAIM COMMUNICATIONS",
    "0060B9": "NEC Platforms, Ltd",
    "006036": "AIT Austrian Institute of Technology GmbH",
    "00608E": "HE ELECTRONICS, TECHNOLOGIE & SYSTEMTECHNIK GmbH",
    "00606A": "MITSUBISHI WIRELESS COMMUNICATIONS. INC.",
    "00601A": "KEITHLEY INSTRUMENTS",
    "0060AD": "MegaChips Corporation",
    "006055": "CORNELL UNIVERSITY",
    "00609C": "Perkin-Elmer Incorporated",
    "0060CF": "ALTEON NETWORKS, INC.",
    "006075": "PENTEK, INC.",
    "0060B7": "CHANNELMATIC, INC.",
    "006006": "SOTEC CO., LTD",
    "0060BA": "SAHARA NETWORKS, INC.",
    "006098": "HT COMMUNICATIONS",
    "0060CA": "HARMONIC SYSTEMS INCORPORATED",
    "006024": "GRADIENT TECHNOLOGIES, INC.",
    "0060DE": "Kayser-Threde GmbH",
    "0060D0": "SNMP RESEARCH INCORPORATED",
    "0060AF": "PACIFIC MICRO DATA, INC.",
    "006038": "Nortel Networks",
    "006015": "NET2NET CORPORATION",
    "00604F": "Tattile SRL ",
    "0060E8": "HITACHI COMPUTER PRODUCTS (AMERICA), INC.",
    "0060F6": "NEXTEST COMMUNICATIONS PRODUCTS, INC.",
    "006072": "VXL INSTRUMENTS, LIMITED",
    "006051": "QUALITY SEMICONDUCTOR",
    "006092": "MICRO/SYS, INC.",
    "00609E": "ASC X3 - INFORMATION TECHNOLOGY STANDARDS SECRETARIATS",
    "006010": "NETWORK MACHINES, INC.",
    "006044": "LITTON/POLY-SCIENTIFIC",
    "0060A2": "NIHON UNISYS LIMITED CO.",
    "00609D": "PMI FOOD EQUIPMENT GROUP",
    "006084": "DIGITAL VIDEO",
    "00602D": "ALERTON TECHNOLOGIES, INC.",
    "0060F8": "Loran International Technologies Inc.",
    "006078": "POWER MEASUREMENT LTD.",
    "006004": "COMPUTADORES MODULARES SA",
    "0060B4": "GLENAYRE R&D INC.",
    "00A01D": "Red Lion Controls, LP",
    "00A0B9": "EAGLE TECHNOLOGY, INC.",
    "00A019": "NEBULA CONSULTANTS, INC.",
    "00A0ED": "Brooks Automation, Inc.",
    "00A0A0": "COMPACT DATA, LTD.",
    "00A013": "TELTREND LTD.",
    "00A0A6": "M.I. SYSTEMS, K.K.",
    "00A051": "ANGIA COMMUNICATIONS. INC.",
    "00A038": "EMAIL ELECTRONICS",
    "00A077": "FUJITSU NEXION, INC.",
    "00A042": "SPUR PRODUCTS CORP.",
    "00A0C1": "ORTIVUS MEDICAL AB",
    "00A04F": "AMERITEC CORP.",
    "00A0CF": "SOTAS, INC.",
    "00A072": "OVATION SYSTEMS LTD.",
    "00A082": "NKT ELEKTRONIK A/S",
    "00A0F0": "TORONTO MICROELECTRONICS INC.",
    "00A0D7": "KASTEN CHASE APPLIED RESEARCH",
    "00A0F1": "MTI",
    "00A0FF": "TELLABS OPERATIONS, INC.",
    "00A0E5": "NHC COMMUNICATIONS",
    "00A036": "APPLIED NETWORK TECHNOLOGY",
    "00A0D2": "ALLIED TELESIS INTERNATIONAL CORPORATION",
    "00A0D3": "INSTEM COMPUTER SYSTEMS, LTD.",
    "00A0B4": "TEXAS MICROSYSTEMS, INC.",
    "00A065": "Symantec Corporation",
    "00A0A3": "RELIABLE POWER METERS",
    "00A055": "Data Device Corporation",
    "00A05B": "MARQUIP, INC.",
    "00A08C": "MultiMedia LANs, Inc.",
    "00A029": "COULTER CORPORATION",
    "00A0AA": "SPACELABS MEDICAL",
    "00A03B": "TOSHIN ELECTRIC CO., LTD.",
    "00A0F3": "STAUBLI",
    "00A060": "ACER PERIPHERALS, INC.",
    "00A083": "ASIMMPHONY TURKEY",
    "00A004": "NETPOWER, INC.",
    "00A087": "Microsemi Corporation",
    "00A043": "AMERICAN TECHNOLOGY LABS, INC.",
    "00A0D0": "TEN X TECHNOLOGY, INC.",
    "00A0BC": "VIASAT, INCORPORATED",
    "00A074": "PERCEPTION TECHNOLOGY",
    "00A07F": "GSM-SYNTEL, LTD.",
    "00A09B": "QPSX COMMUNICATIONS, LTD.",
    "00A000": "CENTILLION NETWORKS, INC.",
    "00A08A": "BROOKTROUT TECHNOLOGY, INC.",
    "00A07B": "DAWN COMPUTER INCORPORATION",
    "00A05C": "INVENTORY CONVERSION, INC./",
    "00200F": "EBRAINS Inc",
    "0020DF": "KYOSAN ELECTRIC MFG. CO., LTD.",
    "002092": "CHESS ENGINEERING B.V.",
    "00202B": "ADVANCED TELECOMMUNICATIONS MODULES, LTD.",
    "00206B": "KONICA MINOLTA HOLDINGS, INC.",
    "002004": "YAMATAKE-HONEYWELL CO., LTD.",
    "0020E5": "APEX DATA, INC.",
    "002043": "NEURON COMPANY LIMITED",
    "002071": "IBR GMBH",
    "002087": "MEMOTEC, INC.",
    "0020F9": "PARALINK NETWORKS, INC.",
    "002015": "ACTIS COMPUTER SA",
    "002099": "BON ELECTRIC CO., LTD.",
    "00207C": "AUTEC GMBH",
    "002057": "TITZE DATENTECHNIK GmbH",
    "0020BC": "Long Reach Networks Pty Ltd",
    "0020C7": "AKAI Professional M.I. Corp.",
    "0020EE": "GTECH CORPORATION",
    "00204C": "MITRON COMPUTER PTE LTD.",
    "002017": "ORBOTECH",
    "002093": "LANDINGS TECHNOLOGY CORP.",
    "002063": "WIPRO INFOTECH LTD.",
    "002056": "NEOPRODUCTS",
    "002042": "DATAMETRICS CORP.",
    "002078": "RUNTOP, INC.",
    "00C0F3": "NETWORK COMMUNICATIONS CORP.",
    "00C0C0": "SHORE MICROSYSTEMS, INC.",
    "00C00C": "RELIA TECHNOLGIES",
    "00C073": "XEDIA CORPORATION",
    "00C0D4": "AXON NETWORKS, INC.",
    "002024": "PACIFIC COMMUNICATION SCIENCES",
    "0020D1": "MICROCOMPUTER SYSTEMS (M) SDN.",
    "0020CE": "LOGICAL DESIGN GROUP, INC.",
    "002014": "GLOBAL VIEW CO., LTD.",
    "0020C2": "TEXAS MEMORY SYSTEMS, INC.",
    "0020ED": "GIGA-BYTE TECHNOLOGY CO., LTD.",
    "002085": "Eaton Corporation",
    "0020CD": "HYBRID NETWORKS, INC.",
    "00202E": "DAYSTAR DIGITAL",
    "0020B3": "Tattile SRL ",
    "002016": "SHOWA ELECTRIC WIRE & CABLE CO",
    "00204D": "INOVIS GMBH",
    "00205F": "GAMMADATA COMPUTER GMBH",
    "00201F": "BEST POWER TECHNOLOGY, INC.",
    "0020B6": "AGILE NETWORKS, INC.",
    "002029": "TELEPROCESSING PRODUCTS, INC.",
    "002069": "ISDN SYSTEMS CORPORATION",
    "00208B": "LAPIS TECHNOLOGIES, INC.",
    "002006": "GARRETT COMMUNICATIONS, INC.",
    "00C0CD": "COMELTA, S.A.",
    "00205D": "NANOMATIC OY",
    "00C04C": "DEPARTMENT OF FOREIGN AFFAIRS",
    "00C07C": "HIGHTECH INFORMATION",
    "00C0B8": "FRASER'S HILL LTD.",
    "00C062": "IMPULSE TECHNOLOGY",
    "00C0EC": "DAUPHIN TECHNOLOGY",
    "00C016": "ELECTRONIC THEATRE CONTROLS",
    "00C086": "THE LYNK CORPORATION",
    "00C013": "NETRIX",
    "00C058": "DATAEXPERT CORP.",
    "00C0D0": "RATOC SYSTEM INC.",
    "00C0BF": "TECHNOLOGY CONCEPTS, LTD.",
    "00C0BA": "NETVANTAGE",
    "00C05E": "VARI-LITE, INC.",
    "00C0D5": "Werbeagentur J\u00fcrgen Siebert",
    "00C063": "MORNING STAR TECHNOLOGIES, INC",
    "00C021": "NETEXPRESS",
    "00C0C1": "QUAD/GRAPHICS, INC.",
    "00C089": "TELINDUS DISTRIBUTION",
    "00C067": "UNITED BARCODE INDUSTRIES",
    "00C0A3": "DUAL ENTERPRISES CORPORATION",
    "00C018": "LANART CORPORATION",
    "00C077": "DAEWOO TELECOM LTD.",
    "00C0C8": "MICRO BYTE PTY. LTD.",
    "00C069": "Axxcelera Broadband Wireless",
    "00C090": "PRAIM S.R.L.",
    "00C0DE": "ZCOMM, INC.",
    "00C0DB": "IPC CORPORATION (PTE) LTD.",
    "00C09B": "Tellabs Enterprise, Inc.",
    "00C06B": "OSI PLUS CORPORATION",
    "009D8E": "CARDIAC RECORDERS, INC.",
    "00BB01": "OCTOTHORPE CORP.",
    "00C005": "LIVINGSTON ENTERPRISES, INC.",
    "00C033": "TELEBIT COMMUNICATIONS APS",
    "00C0BC": "TELECOM AUSTRALIA/CSSC",
    "00C00A": "MICRO CRAFT",
    "00C074": "TOYODA AUTOMATIC LOOM",
    "00C06C": "SVEC COMPUTER CORP.",
    "0040FF": "TELEBIT CORPORATION",
    "00401F": "COLORGRAPH LTD",
    "0040AF": "DIGITAL PRODUCTS, INC.",
    "0040F7": "Polaroid Corporation",
    "004037": "SEA-ILAN, INC.",
    "00404E": "FLUENT, INC.",
    "00408D": "THE GOODYEAR TIRE & RUBBER CO.",
    "00C026": "LANS TECHNOLOGY CO., LTD.",
    "004046": "UDC RESEARCH LIMITED",
    "00404A": "WEST AUSTRALIAN DEPARTMENT",
    "00403C": "FORKS, INC.",
    "004042": "N.A.T. GMBH",
    "0040F2": "JANICH & KLASS COMPUTERTECHNIK",
    "0040A2": "KINGSTAR TECHNOLOGY INC.",
    "00C06F": "KOMATSU LTD.",
    "00C0A7": "SEEL LTD.",
    "00401B": "PRINTER SYSTEMS CORP.",
    "0040A3": "MICROUNITY SYSTEMS ENGINEERING",
    "0040B3": "ParTech Inc.",
    "00C0E3": "OSITECH COMMUNICATIONS, INC.",
    "00C0FE": "APTEC COMPUTER SYSTEMS, INC.",
    "00407E": "EVERGREEN SYSTEMS, INC.",
    "00403E": "RASTER OPS CORPORATION",
    "00401D": "INVISIBLE SOFTWARE, INC.",
    "0040F9": "COMBINET",
    "00C0B0": "GCC TECHNOLOGIES,INC.",
    "00C0CA": "ALFA, INC.",
    "00C04A": "GROUP 2000 AG",
    "0040DC": "TRITEC ELECTRONIC GMBH",
    "004054": "CONNECTION MACHINES SERVICES",
    "004004": "ICM CO. LTD.",
    "004018": "ADOBE SYSTEMS, INC.",
    "00401A": "FUJI ELECTRIC CO., LTD.",
    "004080": "ATHENIX CORPORATION",
    "004030": "GK COMPUTER",
    "004040": "RING ACCESS, INC.",
    "008057": "ADSOFT, LTD.",
    "0080BB": "HUGHES LAN SYSTEMS",
    "00C0D7": "TAIWAN TRADING CENTER DBA",
    "004060": "COMENDEC LTD",
    "004056": "MCM JAPAN LTD.",
    "00403D": "Teradata Corporation",
    "0040DA": "TELSPEC LTD",
    "0040A6": "Cray, Inc.",
    "0040D0": "MITAC INTERNATIONAL CORP.",
    "0040AB": "ROLAND DG CORPORATION",
    "0040B6": "COMPUTERM  CORPORATION",
    "004067": "OMNIBYTE CORPORATION",
    "0040C3": "FISCHER AND PORTER CO.",
    "0040EC": "MIKASA SYSTEM ENGINEERING",
    "008072": "MICROPLEX SYSTEMS LTD.",
    "00802F": "NATIONAL INSTRUMENTS CORP.",
    "008054": "FRONTIER TECHNOLOGIES CORP.",
    "008053": "INTELLICOM, INC.",
    "008026": "NETWORK PRODUCTS CORPORATION",
    "0080B0": "ADVANCED INFORMATION",
    "0080FA": "RWT GMBH",
    "0080FD": "EXSCEED CORPRATION",
    "0040EE": "OPTIMEM",
    "004051": "Garbee and Garbee",
    "00407A": "SOCIETE D'EXPLOITATION DU CNIT",
    "0080FE": "AZURE TECHNOLOGIES, INC.",
    "00803C": "TVS ELECTRONICS LTD",
    "008046": "Tattile SRL ",
    "004025": "MOLECULAR DYNAMICS",
    "0040C7": "RUBY TECH CORPORATION",
    "004052": "STAR TECHNOLOGIES, INC.",
    "00402E": "PRECISION SOFTWARE, INC.",
    "00402B": "TRIGEM COMPUTER, INC.",
    "004002": "PERLE SYSTEMS LIMITED",
    "004049": "Roche Diagnostics International Ltd.",
    "004029": "Compex",
    "004031": "KOKUSAI ELECTRIC CO., LTD",
    "0040D3": "KIMPSION INTERNATIONAL CORP.",
    "008011": "DIGITAL SYSTEMS INT'L. INC.",
    "0080F1": "OPUS SYSTEMS",
    "008029": "EAGLE TECHNOLOGY, INC.",
    "004001": "Zero One Technology Co. Ltd.",
    "004071": "ATM COMPUTER GMBH",
    "008034": "SMT GOUPIL",
    "00802C": "THE SAGE GROUP PLC",
    "0080D6": "NUVOTECH, INC.",
    "00800A": "JAPAN COMPUTER CORP.",
    "008027": "ADAPTIVE SYSTEMS, INC.",
    "0080FC": "AVATAR CORPORATION",
    "0080E4": "NORTHWEST DIGITAL SYSTEMS, INC",
    "0080EC": "SUPERCOMPUTING SOLUTIONS, INC.",
    "0080A2": "CREATIVE ELECTRONIC SYSTEMS",
    "0080A5": "SPEED INTERNATIONAL",
    "008079": "MICROBUS DESIGNS LTD.",
    "0080AD": "CNET TECHNOLOGY, INC.",
    "00800E": "ATLANTIX CORPORATION",
    "0080AB": "DUKANE NETWORK INTEGRATION",
    "00804B": "EAGLE TECHNOLOGIES PTY.LTD.",
    "0080C8": "D-LINK SYSTEMS, INC.",
    "008012": "INTEGRATED MEASUREMENT SYSTEMS",
    "0080CC": "MICROWAVE BYPASS SYSTEMS",
    "000075": "Nortel Networks",
    "0000ED": "APRIL",
    "0000A3": "NETWORK APPLICATION TECHNOLOGY",
    "000039": "TOSHIBA CORPORATION",
    "00003C": "AUSPEX SYSTEMS INC.",
    "00007E": "CLUSTRIX CORPORATION",
    "0000CB": "COMPU-SHACK ELECTRONIC GMBH",
    "0000A5": "Tattile SRL ",
    "000036": "ATARI CORPORATION",
    "0000F8": "DIGITAL EQUIPMENT CORPORATION",
    "00807B": "ARTEL COMMUNICATIONS CORP.",
    "000044": "CASTELLE CORPORATION",
    "00805C": "AGILIS CORPORATION",
    "0080C5": "NOVELLCO DE MEXICO",
    "008014": "ESPRIT SYSTEMS",
    "00007F": "LINOTYPE-HELL AG",
    "0000CE": "MEGADATA CORP.",
    "00007B": "RESEARCH MACHINES",
    "0080F6": "SYNERGY MICROSYSTEMS",
    "008078": "PRACTICAL PERIPHERALS, INC.",
    "000079": "NETWORTH INCORPORATED",
    "000091": "ANRITSU CORPORATION",
    "00001A": "ADVANCED MICRO DEVICES",
    "0080B7": "STELLAR COMPUTER",
    "000096": "MARCONI ELECTRONICS LTD.",
    "00005E": "ICANN, IANA Department",
    "000038": "CSS LABS",
    "00009E": "MARLI S.A.",
    "000042": "METIER MANAGEMENT SYSTEMS LTD.",
    "000013": "CAMEX",
    "000095": "SONY TEKTRONIX CORP.",
    "000057": "SCITEX CORPORATION LTD.",
    "0000D6": "PUNCH LINE HOLDING",
    "000040": "APPLICON, INC.",
    "000085": "CANON INC.",
    "00004A": "ADC CODENOLL TECHNOLOGY CORP.",
    "000012": "INFORMATION TECHNOLOGY LIMITED",
    "000060": "Kontron Europe GmbH",
    "08006F": "PHILIPS APELDOORN B.V.",
    "00006A": "COMPUTER CONSOLES INC.",
    "00000F": "NEXT, INC.",
    "0000BB": "TRI-DATA",
    "00007D": "Oracle Corporation",
    "00008A": "DATAHOUSE INFORMATION SYSTEMS",
    "000032": "Marconi plc",
    "00005D": "CS TELECOM",
    "08008F": "CHIPCOM CORPORATION",
    "08007A": "INDATA",
    "080079": "THE DROID WORKS",
    "080073": "TECMAR INC.",
    "080072": "XEROX CORP UNIV GRANT PROGRAM",
    "08006A": "AT&T",
    "080019": "GENERAL ELECTRIC CORPORATION",
    "027001": "RACAL-DATACOM",
    "08003E": "CODEX CORPORATION",
    "080040": "FERRANTI COMPUTER SYS. LIMITED",
    "08003A": "ORCATECH INC.",
    "08003D": "CADNETIX CORPORATIONS",
    "080038": "BULL S.A.S.",
    "08002F": "PRIME COMPUTER INC.",
    "080062": "General Dynamics",
    "08005C": "FOUR PHASE SYSTEMS",
    "08005A": "IBM Corp",
    "08002C": "BRITTON LEE INC.",
    "08004D": "CORVUS SYSTEMS INC.",
    "08001E": "APOLLO COMPUTER INC.",
    "080052": "INSYSTEC",
    "080001": "COMPUTERVISION CORPORATION",
    "080005": "SYMBOLICS INC.",
    "00003D": "UNISYS",
    "000008": "XEROX CORPORATION",
    "00DD07": "UNGERMANN-BASS INC.",
    "00DD0D": "UNGERMANN-BASS INC.",
    "080016": "BARRISTER INFO SYS CORP",
    "000006": "XEROX CORPORATION",
    "080064": "Sitasys AG",
    "080002": "BRIDGE COMMUNICATIONS INC.",
    "08001A": "TIARA/ 10NET",
    "08008B": "PYRAMID TECHNOLOGY CORP.",
    "080012": "BELL ATLANTIC INTEGRATED SYST.",
    "00000B": "MATRIX CORPORATION",
    "00009B": "INFORMATION INTERNATIONAL, INC",
    "08000E": "NCR CORPORATION",
    "EC9A34": "Texas Instruments",
    "943F0C": "Genexis B.V.",
    "000002": "XEROX CORPORATION",
    "000003": "XEROX CORPORATION",
    "00DD09": "UNGERMANN-BASS INC.",
    "84BB26": "Texas Instruments",
    "58BAD3": "NANJING CASELA TECHNOLOGIES CORPORATION LIMITED",
    "34B1EB": "Apple, Inc.",
    "A4F841": "Apple, Inc.",
    "FCA8E0": "THE FURUKAWA ELECTRIC CO., LTD",
    "FC51B5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A05272": "Apple, Inc.",
    "200484": "Apple, Inc.",
    "F87D3F": "Huawei Device Co., Ltd.",
    "90A57D": "Huawei Device Co., Ltd.",
    "D0A0BB": "Shenzhen iComm Semiconductor CO.,LTD",
    "909E24": "ekey biometric systems gmbh",
    "8C9B2D": "PLANTRONICS, INC.",
    "284430": "Arcade Communications Ltd.",
    "604966": "Shenzhen Dingsheng Technology Co., Ltd.",
    "9803CF": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "943251": "ASKEY COMPUTER CORP",
    "606B5B": "Arista Network, Inc.",
    "DCE5D8": "zte corporation",
    "68944A": "zte corporation",
    "7433E9": "zte corporation",
    "78EDBC": "OnePlus Technology (Shenzhen) Co., Ltd",
    "40C0EE": "365mesh Pty Ltd",
    "FCC0CC": "Yunke China Information Technology Limited",
    "48EB65": "Henan KunLun Technologies CO.,Ltd.",
    "C8A3E8": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "1C2156": "Smappee NV",
    "40B70E": "Huawei Device Co., Ltd.",
    "D8D668": "Tuya Smart Inc.",
    "004CE5": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "C09120": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "4C50DD": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "407218": "Tonly Technology Co. Ltd ",
    "A841F4": "AzureWave Technology Inc.",
    "C4CB76": "Microsoft Corporation",
    "409EA4": "Juniper Networks",
    "BC4C78": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D45F7A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A80556": "vivo Mobile Communication Co., Ltd.",
    "9CB400": "zte corporation",
    "B05246": "NXP Semiconductor (Tianjin) LTD.",
    "F00727": "INTEREL BUILDING AUTOMATION",
    "886076": "Sparnex n.v.",
    "F871A6": "Apple, Inc.",
    "78A7C7": "Apple, Inc.",
    "98FEE1": "Apple, Inc.",
    "340962": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "2834FF": "Apple, Inc.",
    "D0C050": "Apple, Inc.",
    "0CD5D3": "Cisco Systems, Inc",
    "2C81BF": "Apple, Inc.",
    "100648": "IEEE Registration Authority",
    "000D2C": "Lantronix",
    "14AE68": "KLG Smartec",
    "C875F4": "China Mobile Group Device Co.,Ltd.",
    "2C532B": "TCT mobile ltd",
    "849437": "Apple, Inc.",
    "64FD29": "Zhejiang Dahua Technology Co., Ltd.",
    "44E4E6": "Extreme Networks Headquarters",
    "742E4F": "Stienen Group",
    "5C836C": "Ruckus Wireless",
    "30169D": "MERCUSYS TECHNOLOGIES CO., LTD.",
    "EC462C": "Infinix mobility limited",
    "8C8881": "Cisco Meraki",
    "D8B020": "Taicang T&W Electronics",
    "D4F32D": "Intel Corporate",
    "2C7BA0": "Intel Corporate",
    "704CA5": "Fortinet, Inc.",
    "04D590": "Fortinet, Inc.",
    "94F392": "Fortinet, Inc.",
    "24D208": "Sensata Technologies Inc.",
    "1004C1": "JD Cloud Computing Co., Ltd.",
    "5C5310": "Nanjing Qinheng Microelectronics Co., Ltd.",
    "34D262": "SZ DJI TECHNOLOGY CO.,LTD",
    "4C4CD8": "zte corporation",
    "5422E0": "Adtran Inc",
    "C40778": "New H3C Technologies Co., Ltd",
    "FC8AF7": "zte corporation",
    "C4509C": "Vantiva - Connected Home",
    "7891E9": "Raisecom Technology CO., LTD",
    "E85540": "WUXI FUNIDE DIGITAL CO.,LTD",
    "54D60D": "Hangzhou Ezviz Software Co.,Ltd.",
    "589671": "Wistron Neweb Corporation",
    "8C1A50": "China Mobile Group Device Co.,Ltd.",
    "D40068": "Fiberhome Telecommunication Technologies Co.,LTD",
    "38637F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C8FE0F": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "308ECF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D829F8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "387605": "Inogeni",
    "78392D": "IEEE Registration Authority",
    "BCE001": "Netis Technology Co., Ltd.",
    "D03E07": "Apple, Inc.",
    "50B127": "Apple, Inc.",
    "70F6CF": "Relay, Inc.",
    "C81072": "BBPOS Limited",
    "249D2A": "LinkData Technology (Tianjin) Co., LTD",
    "E4E66C": "Tiandy Technologies Co.,LTD",
    "843E03": "Sagemcom Broadband SAS",
    "549A8F": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "A8EF5F": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "F010A5": "Texas Instruments",
    "C06380": "Texas Instruments",
    "F8FB90": "Texas Instruments",
    "446B1F": "Texas Instruments",
    "1CCE51": "AzureWave Technology Inc.",
    "6833EE": "ARRIS Group, Inc.",
    "0CA138": "BLiNQ Networks Inc.",
    "C0252F": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "10634B": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "044BA5": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "D48409": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "E4F3F5": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "C0A5DD": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "D8EE42": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "28AF42": "Samsung Electronics Co.,Ltd",
    "F4ED37": "Qingdao Yuze lntelligent Technology Co.,Ltd",
    "BC026E": "Silicon Laboratories",
    "B43522": "Silicon Laboratories",
    "90395E": "Silicon Laboratories",
    "E8E07E": "Silicon Laboratories",
    "B0C7DE": "Silicon Laboratories",
    "881A14": "Silicon Laboratories",
    "44E2F8": "Silicon Laboratories",
    "E41226": "Continental Automotive Romania SLR",
    "30FB68": "Wuhan Zmvision Technology Co. Ltd.",
    "30A30F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "246830": "Shenzhen Shokzhear Co., Ltd",
    "308DD4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C90E0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "707362": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8DB38": "Google, Inc.",
    "74C64A": "AGOS Co.,Ltd",
    "E461F4": "shenzhen worldelite electronics co., LTD",
    "109F47": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "E4379F": "Cisco Systems, Inc",
    "5464BC": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "0026A4": "Novus Produtos Eletronicos Ltda",
    "24587C": "Espressif Inc.",
    "CC8DA2": "Espressif Inc.",
    "EC314A": "Chengdu Quanjing Intelligent Technology Co.,Ltd",
    "04F4D8": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "D4ADFC": "Shenzhen Intellirocks Tech. Co. Ltd.",
    "E82725": "Axis Communications AB",
    "9CC8E9": "Amazon Technologies Inc.",
    "3CB43D": "SZ Tenveo video technology co., Ltd",
    "547885": "SHENZHEN GIEC DIGITAL CO.,LTD",
    "606BB3": "zte corporation",
    "A0DD6C": "Espressif Inc.",
    "000E2A": "dormakaba USA Inc.",
    "74309D": "Infinix mobility limited",
    "54A7A0": "HUNAN AIMAG INTELLIGENT TECHNOLOGY CO.,LTD",
    "14C67D": "Vizio, Inc",
    "E8F085": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C366A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64F0AD": "Zhejiang Tmall Technology Co., Ltd.",
    "E4FAC4": "TP-LINK CORPORATION PTE. LTD.",
    "40AE30": "TP-LINK CORPORATION PTE. LTD.",
    "2C66AD": "NimbleTech Digital Inc.",
    "44E64A": "Shenzhen iComm Semiconductor CO.,LTD",
    "701301": "Vantiva - Connected Home",
    "EC5AA3": "Huawei Device Co., Ltd.",
    "98876C": "Huawei Device Co., Ltd.",
    "0C8306": "Huawei Device Co., Ltd.",
    "046ECB": "zte corporation",
    "F0ED19": "zte corporation",
    "D0EF76": "Espressif Inc.",
    "C4D8D5": "Espressif Inc.",
    "ECCD4C": "New H3C Technologies Co., Ltd",
    "D04FAB": "Yoqu Technology (Shenzhen) Co., Ltd.",
    "0025FC": "ENDA",
    "E81499": "Yoqu Technology(Shenzhen)Co.,Ltd.",
    "684DB6": "Xiaomi Communications Co Ltd",
    "205D0D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "603457": "HP Tuners LLC",
    "08DDEB": "Silicon Laboratories",
    "C06F98": "eero inc.",
    "D8A0E6": "Chongqing Yipingfang Technology Co., Ltd.",
    "7866F3": "shenzhen worldelite electronics co., LTD",
    "0C0ADF": "Texas Instruments",
    "DCBE04": "Texas Instruments",
    "E0B763": "Bosch Automotive Products (Suzhou) Co., Ltd. Changzhou Branch",
    "C8CD55": "Ruijie Networks Co.,LTD",
    "C4B25B": "Ruijie Networks Co.,LTD",
    "984A6B": "Ruijie Networks Co.,LTD",
    "102FF8": "Vicoretek (Nanjing) Co.,Ltd.",
    "C457CD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28221E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0E752": "Shenzhen Huajuxin Semiconductor Co.,ltd",
    "64BAA4": "zte corporation",
    "3824F1": "Private",
    "24EFB4": "Shanghai Neardi Technologies Co. Ltd.",
    "40D160": "Apple, Inc.",
    "BCBB58": "Apple, Inc.",
    "303B7C": "Apple, Inc.",
    "00812A": "Apple, Inc.",
    "CC6A33": "Cisco Systems, Inc",
    "1CE209": "Apple, Inc.",
    "900371": "Quectel Wireless Solutions Co.,Ltd.",
    "949F8B": "zte corporation",
    "6CC41E": "NEXSEC Incorporated",
    "F46ED6": "EM Microelectronic",
    "80283C": "Sonova AG",
    "80AE54": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "18EFC0": "Sercomm Japan Corporation",
    "3C0868": "Power Plus Communications AG",
    "901564": "IEEE Registration Authority",
    "20500F": "Fiber Groep B.V.",
    "948CD7": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "48E9CA": "creoline GmbH",
    "D47F35": "Cisco Systems, Inc",
    "408F9A": "KanEL Sweden AB",
    "001CA1": "Akamai Technologies Inc.",
    "507973": "Inagile Electronic Technology Co.,LTD.",
    "5C3548": "Aditya Infotech Ltd.",
    "88DA18": "China Mobile Group Device Co.,Ltd.",
    "98CB38": "Boxin Communications Limited Liability Company",
    "D009F5": "Hosiden Corporation",
    "286847": "Silicon Laboratories",
    "44C532": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30D4E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30C6AB": "zte corporation",
    "48E663": "IEEE Registration Authority",
    "DCECE3": "HORYS TECHNOLOGIES LLC",
    "A4403D": "Shenzhen Baseus Technology Co., Ltd.",
    "4C8125": "ZOWEE TECHNOLOGY(HEYUAN)Co.,Ltd",
    "484982": "Huawei Device Co., Ltd.",
    "2836F0": "Huawei Device Co., Ltd.",
    "1C13FA": "Huawei Device Co., Ltd.",
    "A8B0D1": "EFUN Display Technology (Shenzhen) Co., Ltd.",
    "483584": "Huawei Device Co., Ltd.",
    "88F6DC": "Huawei Device Co., Ltd.",
    "70EB74": "Ningbo Goneo Electric Appliance Co., Ltd.",
    "60D51B": "Fujitsu Limited",
    "00057E": "Eckelmann AG",
    "D4955D": "zte corporation",
    "1C3C78": "Apple, Inc.",
    "940BCD": "Apple, Inc.",
    "9C6697": "Cisco Systems, Inc",
    "D0E581": "Apple, Inc.",
    "C4B1D9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "707013": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4A459": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "E45BB3": "zte corporation",
    "24EC4A": "Espressif Inc.",
    "981DAC": "Cyviz AS",
    "F40343": "Hewlett Packard Enterprise ",
    "98F2B3": "Hewlett Packard Enterprise ",
    "70106F": "Hewlett Packard Enterprise ",
    "C8FFBF": "IEEE Registration Authority",
    "001438": "Hewlett Packard Enterprise ",
    "001A1E": "Hewlett Packard Enterprise",
    "94B40F": "Hewlett Packard Enterprise",
    "BC9FE4": "Hewlett Packard Enterprise",
    "D015A6": "Hewlett Packard Enterprise",
    "482F6B": "Hewlett Packard Enterprise",
    "0C975F": "Hewlett Packard Enterprise",
    "343A20": "Hewlett Packard Enterprise",
    "749E75": "Hewlett Packard Enterprise",
    "ECEBB8": "Hewlett Packard Enterprise ",
    "3817C3": "Hewlett Packard Enterprise ",
    "040973": "Hewlett Packard Enterprise ",
    "B88303": "Hewlett Packard Enterprise ",
    "904C81": "Hewlett Packard Enterprise ",
    "548028": "Hewlett Packard Enterprise ",
    "9C8CD8": "Hewlett Packard Enterprise ",
    "08F1EA": "Hewlett Packard Enterprise ",
    "D0D3E0": "Hewlett Packard Enterprise",
    "8C85C1": "Hewlett Packard Enterprise",
    "FC7FF1": "Hewlett Packard Enterprise",
    "946424": "Hewlett Packard Enterprise",
    "D4F5EF": "Hewlett Packard Enterprise ",
    "54778A": "Hewlett Packard Enterprise ",
    "3821C7": "Hewlett Packard Enterprise",
    "00246C": "Hewlett Packard Enterprise",
    "94A6D8": "New H3C Technologies Co., Ltd",
    "A827C8": "Edgecore Americas Networking Corporation",
    "24DD1B": "Qingdao Hi-image Technologies Co., Ltd",
    "A852D4": "Hewlett Packard Enterprise",
    "50E4E0": "Hewlett Packard Enterprise",
    "F84477": "Silicon Laboratories",
    "6CFD22": "Silicon Laboratories",
    "380B26": "Mindray Co., Ltd.",
    "CC7E0F": "Theben AG",
    "C4474E": "Intel Corporate",
    "08B4D2": "Intel Corporate",
    "5CB47E": "Intel Corporate",
    "28C5C8": "HP Inc.",
    "38AA20": "zte corporation",
    "706AC9": "zte corporation",
    "243F75": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "90D3CF": "ASKEY COMPUTER CORP",
    "483A02": "Fortinet, Inc.",
    "90CBA3": "TECNO MOBILE LIMITED",
    "BC41A0": "zte corporation",
    "C8711F": "SUZHOU TESIEN TECHNOLOGY CO., LTD.",
    "D8D261": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "246278": "sysmocom - s.f.m.c. GmbH",
    "70378E": "Tianyi Telecom Terminals Company Limited",
    "D43B8A": "Shenzhen Zhide technology Co., LTD",
    "2CED89": "HUAWEI TECHNOLOGIES CO.,LTD",
    "800518": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DCEB4D": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "247C46": "FLEXTRONICS TECHNOLOGIES MEXICO S DE RL DE CV",
    "146C27": "Dongguan Liesheng Electronic Co., Ltd.",
    "C470BD": "Mellanox Technologies, Inc.",
    "B8E924": "Mellanox Technologies, Inc.",
    "940D4B": "Cisco Systems, Inc",
    "04E387": "Cisco Systems, Inc",
    "442745": "LG Innotek",
    "8C12C2": "GLBB Japan",
    "448763": "FN-LINK TECHNOLOGY Ltd.",
    "8C08AA": "Apple, Inc.",
    "F4A310": "Apple, Inc.",
    "3C3B77": "Apple, Inc.",
    "E84A78": "Apple, Inc.",
    "10DA63": "Apple, Inc.",
    "A8BA25": "Hewlett Packard Enterprise",
    "3CA80A": "vivo Mobile Communication Co., Ltd.",
    "B0653A": "Murata Manufacturing Co., Ltd.",
    "00A41C": "Tonly Technology Co. Ltd ",
    "901506": "Espressif Inc.",
    "60F549": "Apple, Inc.",
    "D83BDA": "Espressif Inc.",
    "40BC68": "Funshion Online Technologies Co.,Ltd",
    "D0AE05": "Xiaomi Communications Co Ltd",
    "58FCE3": "Funshion Online Technologies Co.,Ltd",
    "48A170": "Cisco Systems, Inc",
    "7402E1": "Texas Instruments",
    "24F306": "ITEL MOBILE LIMITED",
    "649CF3": "Fiberhome Telecommunication Technologies Co.,LTD",
    "789912": "Flyingvoice (HongKong) Technologies Limited",
    "941042": "Fanox Electronic S.L.",
    "1090FC": "Shenzhen DOOGEE Hengtong Technology CO.,LTD",
    "4C97A1": "Silicon Laboratories",
    "B03B1B": "Kontrolnext Technology (Beijing) Ltd.",
    "BC34CA": "INOVANCE",
    "A00E98": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B41E52": "Flock Safety",
    "58B4BB": "Ruijie Networks Co.,LTD",
    "60D561": "Shenzhen Glazero Technology Co., Ltd.",
    "7CD44D": "Shanghai Moorewatt Energy Technology Co.,Ltd",
    "044BB1": "Huawei Device Co., Ltd.",
    "F0D7EE": "Huawei Device Co., Ltd.",
    "F45B29": "HUAWEI TECHNOLOGIES CO.,LTD",
    "349671": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8A25D": "Motorola Mobility LLC, a Lenovo Company",
    "B01921": "TP-LINK CORPORATION PTE. LTD.",
    "344663": "IEEE Registration Authority",
    "0CCDB4": "Sichuan AI-Link Technology Co., Ltd.",
    "7CA58F": "shenzhen Qikai Electronic Co., Ltd.",
    "647B1E": "Sagemcom Broadband SAS",
    "8C5E4D": "DragonWave Technologies DMCC",
    "7800A8": "PT HAN SUNG ELECTORONICS INDONESIA",
    "98FA2E": "Sony Interactive Entertainment Inc.",
    "90032E": "Taicang T&W Electronics",
    "F40A2E": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "406918": "EM Microelectronic",
    "A85C03": "JiangSu Fulian Communication Technology Co., Ltd",
    "20BEB8": "Amazon Technologies Inc.",
    "C095CF": "Amazon Technologies Inc.",
    "088BC8": "Google, Inc.",
    "6C3AFF": "Apple, Inc.",
    "EC97A2": "Apple, Inc.",
    "449E8B": "Apple, Inc.",
    "540A77": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2CE38E": "Cisco Systems, Inc",
    "E0C3EA": "Apple, Inc.",
    "6C1F8A": "Apple, Inc.",
    "385439": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "60B58D": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "C40FA6": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "E8122D": "UNIONMAN TECHNOLOGY CO.,LTD",
    "4C3946": "Samsung Electronics Co.,Ltd",
    "78C4FA": "Meta Platforms, Inc.",
    "F42055": "IEEE Registration Authority",
    "443B14": "MitraStar Technology Corp.",
    "446370": "LCFC(Hefei) Electronics Technology co., ltd",
    "C4C6E6": "LCFC(Hefei) Electronics Technology co., ltd",
    "182CA9": "DASAN Networks, Inc. ",
    "54A552": "Shenzhen WeSing Interactive Entertainment Technology Co., Ltd",
    "240A3F": "Samsung Electronics Co.,Ltd",
    "203B67": "Samsung Electronics Co.,Ltd",
    "2460B3": "Samsung Electronics Co.,Ltd",
    "8022FA": "ITEL MOBILE LIMITED",
    "54E1AD": "LCFC(Hefei) Electronics Technology co., ltd",
    "002B67": "LCFC(Hefei) Electronics Technology co., ltd",
    "B08184": "Espressif Inc.",
    "B4055D": "IEIT SYSTEMS Co., Ltd.",
    "3063EA": "Juniper Networks",
    "A09857": "Shenzhen ELINK Technology Co., Ltd.",
    "A84FA4": "CHINA DRAGON TECHNOLOGY LIMITED",
    "5C5DEC": "JiangSu Newcom Optical&Electrical Communication CO Ltd",
    "248625": "IEEE Registration Authority",
    "58EA1F": "Beijing Xiaomi Mobile Software Co., Ltd",
    "948B93": "Xiaomi Communications Co Ltd",
    "F820D2": "Vantiva USA LLC",
    "C0A39E": "EarthCam, Inc.",
    "8C1D55": "Hanwha NxMD (Thailand) Co., Ltd.",
    "7C8D9C": "Edgecore Americas Networking Corporation",
    "D826FA": "Jiangxi Zhentian Technology CO.,LTD",
    "F4FBF5": "EM Microelectronic",
    "E862BE": "Intel Corporate",
    "103597": "Qorvo Utrecht B.V.",
    "AC5C80": "Telink Micro LLC",
    "C8B4AB": "Inspur Computer Technology Co.,Ltd.",
    "C4D8C8": "Silicon Laboratories",
    "743C24": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8D4C3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00A91D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D0F76E": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "30F947": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "A0FA9C": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "48A964": "APEXSHA SMARTTECH PRIVATE LIMITED",
    "E4F58E": "Schneider Electric USA",
    "485F2D": "Amazon Technologies Inc.",
    "E81DEE": "i-TEK RFID",
    "C44FD5": "Vantiva - Connected Home",
    "D0F8E7": "Shenzhen Shutong Space Technology Co., Ltd",
    "8C4F00": "Espressif Inc.",
    "E808AF": "zte corporation",
    "0CC119": "Shenzhen Phaten Tech. LTD",
    "80489F": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "34A34E": "NevadaNano",
    "B4A10A": "Huawei Device Co., Ltd.",
    "306DF9": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "DC3130": "3onedata Technology Co. Ltd.",
    "F0BDEE": "Huawei Device Co., Ltd.",
    "94342F": "ITEL MOBILE LIMITED",
    "2CD8DE": "AltoBeam Inc.",
    "54C8CC": "Shenzhen SDG Telecom Equipment Co.,Ltd.",
    "00BB43": "Tiinlab Corporation",
    "04CB01": "Samsung Electronics Co.,Ltd",
    "706E10": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4D8D4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8092A5": "Valeo Interior Controls (Shenzhen) Co.,Ltd",
    "2C347B": "SHENZHEN JUNGE TECHNOLOGY CO.,LTD",
    "A0AC78": "Shenzhen iComm Semiconductor CO.,LTD",
    "E47F3C": "zte corporation",
    "2C5EAB": "Mellanox Technologies, Inc.",
    "40B215": "Extreme Networks Headquarters",
    "EC1D53": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3839CD": "vivo Mobile Communication Co., Ltd.",
    "40734D": "New H3C Technologies Co., Ltd",
    "C85CE2": "IEEE Registration Authority",
    "DCEE14": "ADT Technology",
    "10FFE0": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "1849F8": "Extreme Networks Headquarters",
    "E84A54": "Beijing Xiaomi Mobile Software Co., Ltd",
    "2CAB46": "Ruckus Wireless",
    "D88466": "Extreme Networks Headquarters",
    "000496": "Extreme Networks Headquarters",
    "00E02B": "Extreme Networks Headquarters",
    "5C0E8B": "Extreme Networks Headquarters",
    "7467F7": "Extreme Networks Headquarters",
    "4C231A": "Extreme Networks Headquarters",
    "5858CD": "Extreme Networks Headquarters",
    "DCDCC3": "Extreme Networks Headquarters",
    "C8BE35": "Extreme Networks Headquarters",
    "F06426": "Extreme Networks Headquarters",
    "F45424": "Extreme Networks Headquarters",
    "F02B7C": "Extreme Networks Headquarters",
    "7C95B1": "Extreme Networks Headquarters",
    "5859C2": "Extreme Networks Headquarters",
    "787D53": "Extreme Networks Headquarters",
    "209EF7": "Extreme Networks Headquarters",
    "809562": "Extreme Networks Headquarters",
    "90A1BA": "PNetworks Electronics Information ",
    "000AD7": "Origin Co., Ltd.",
    "A0D86F": "ARGO AI, LLC",
    "B4D7DB": "New H3C Technologies Co., Ltd",
    "3050CE": "Xiaomi Communications Co Ltd",
    "B405A1": "Xiaomi Communications Co Ltd",
    "2885BB": "Zen Exim Pvt. Ltd.",
    "F8C650": "Cisco Systems, Inc",
    "3C1060": "Fiberhome Telecommunication Technologies Co.,LTD",
    "B4565D": "Chipsea Technologies (Shenzhen) Corp.",
    "4CE136": "Private",
    "0084ED": "LEXMARK INTERNATIONAL, INC.",
    "C05D39": "Jiangsu Huitong Group Co.,Ltd.",
    "74249F": "TIBRO Corp.",
    "648505": "zte corporation",
    "E4604D": "zte corporation",
    "F4E84F": "zte corporation",
    "C87867": "Mist Systems, Inc.",
    "E822B8": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "744DBD": "Espressif Inc.",
    "00F952": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C15D9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4A923": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B810D4": "Masimo Corporation",
    "C89F0C": "Motorola Mobility LLC, a Lenovo Company",
    "28E6A9": "Samsung Electronics Co.,Ltd",
    "DC2D04": "vivo Mobile Communication Co., Ltd.",
    "A43CD7": "NTX Electronics YangZhou co.,LTD",
    "000263": "RPS S.p.A.",
    "483543": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "4C5F70": "Intel Corporate",
    "60B9C0": "Cisco Systems, Inc",
    "8C9461": "Cisco Systems, Inc",
    "5807F8": "Nokia Solutions and Networks GmbH & Co. KG",
    "DC6555": "New H3C Intelligence Terminal Co., Ltd.",
    "346F71": "TenaFe Inc. ",
    "5491AF": "IEEE Registration Authority",
    "9C3928": "Samsung Electronics Co.,Ltd",
    "BC0EAB": "Samsung Electronics Co.,Ltd",
    "98FB27": "Samsung Electronics Co.,Ltd",
    "A80BFB": "Ruckus Wireless",
    "3CFA30": "Palo Alto Networks",
    "CCECB7": "ShenZhen Linked-Z Intelligent Display Co., Ltd",
    "DCDA0C": "Espressif Inc.",
    "A8B8E0": "Changwang Technology inc.",
    "14CA56": "zte corporation",
    "48D35D": "Private",
    "5433C6": "Mist Systems, Inc.",
    "CC934A": "Sierra Wireless, ULC",
    "00A0D5": "Sierra Wireless, ULC",
    "28A331": "Sierra Wireless, ULC",
    "6C221A": "AltoBeam Inc.",
    "D843AE": "Micro-Star INTL CO., LTD.",
    "A843A4": "China Dragon Technology Limited",
    "D8094E": "Active Brains",
    "00D07C": "JTEKT ELECTRONICS CORPORATION",
    "886C60": "Xiaomi Communications Co Ltd",
    "E0DF13": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "B01886": "SmarDTV Corporation",
    "A42249": "Sagemcom Broadband SAS",
    "5CE931": "TP-Link Corporation Limited",
    "C8A6EF": "Samsung Electronics Co.,Ltd",
    "687909": "Cisco Systems, Inc",
    "E4A41C": "Cisco Systems, Inc",
    "B40AD8": "Sony Interactive Entertainment Inc.",
    "0480A7": "ShenZhen TianGang Micro Technology CO.LTD",
    "389CB2": "Apple, Inc.",
    "583653": "Apple, Inc.",
    "84D328": "Apple, Inc.",
    "504074": "Alarm.com",
    "5C7B5C": "Shenzhen SDMC Technology CO.,Ltd.",
    "407912": "Texas Instruments",
    "58A15F": "Texas Instruments",
    "10CABF": "Texas Instruments",
    "98524A": "Vantiva USA LLC",
    "A0FF70": "Vantiva USA LLC",
    "5C7D7D": "Vantiva USA LLC",
    "08A7C0": "Vantiva USA LLC",
    "8C6A8D": "Vantiva USA LLC",
    "1C9ECC": "Vantiva USA LLC",
    "CC1AA3": "Arista Networks",
    "3CC03E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "FC94E3": "Vantiva USA LLC",
    "8C04FF": "Vantiva USA LLC",
    "CC3540": "Vantiva USA LLC",
    "B0C287": "Vantiva USA LLC",
    "BC9B68": "Vantiva USA LLC",
    "80D04A": "Vantiva USA LLC",
    "3CB74B": "Vantiva USA LLC",
    "FC9114": "Vantiva USA LLC",
    "500959": "Vantiva USA LLC",
    "3817E1": "Vantiva USA LLC",
    "4075C3": "Vantiva USA LLC",
    "10A793": "Vantiva USA LLC",
    "B8A535": "Vantiva USA LLC",
    "9839C0": "FLEXTRONICS",
    "8CCB14": "TBS GmbH",
    "BC5C17": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "74B80F": "Zipline International Inc.",
    "545FA7": "Jibaiyou Technology Co.,Ltd.",
    "08BFB8": "ASUSTek COMPUTER INC.",
    "003044": "CradlePoint, Inc",
    "00E01C": "CradlePoint, Inc",
    "58D237": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "BC0FF3": "HP Inc.",
    "20BBBC": "Hangzhou Ezviz Software Co.,Ltd.",
    "64B379": "Jiangsu Viscore Technologies Co.,Ltd",
    "24FDFA": "Private",
    "D43127": "Ruijie Networks Co.,LTD",
    "ECEF17": "Sunplus Technology Co., Ltd.",
    "4CC64C": "Beijing Xiaomi Mobile Software Co., Ltd",
    "90E468": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "EC0C96": "Nokia",
    "A0779E": "Chipsea Technologies (Shenzhen) Corp.",
    "90D092": "HUMAX Co., Ltd.",
    "8430CE": "Shenzhen Jaguar Microsystems Co., Ltd",
    "70D8C2": "Intel Corporate",
    "445925": "Square Inc.",
    "E021FE": "Richer Link Technologies CO.,LTD",
    "C8A362": "ASIX Electronics Corporation",
    "34D856": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "1844CF": "B+L Industrial Measurements GmbH",
    "CC5763": "Panasonic Automotive Systems Co.,Ltd",
    "882F64": "BCOM Networks Limited",
    "BC87FA": "Bose Corporation",
    "B81EA4": "Liteon Technology Corporation",
    "840F2A": "Jiangxi Risound Electronics Co., LTD",
    "684724": "EM Microelectronic",
    "AC3B96": "NXP Semiconductor (Tianjin) LTD.",
    "0C6714": "SERNET (SUZHOU) TECHNOLOGIES CORPORATION",
    "485F08": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F86FB0": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "90F891": "Kaon Group Co., Ltd.",
    "C8A608": "Ruckus Wireless",
    "9877E7": "Kaon Group Co., Ltd.",
    "840112": "Kaon Group Co., Ltd.",
    "1834AF": "Kaon Group Co., Ltd.",
    "3447D4": "Chengdu Quanjing Intelligent Technology Co.,Ltd",
    "64FB01": "Zhongshan Camry Electronic Company Limited",
    "38F6CF": "zte corporation",
    "782AF8": "IETHCOM INFORMATION TECHNOLOGY CO., LTD.",
    "7449D2": "New H3C Technologies Co., Ltd",
    "DCD26A": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "ACD8A7": "BELLDESIGN Inc.",
    "0C8772": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "DC8D91": "Infinix mobility limited",
    "409A30": "TECNO MOBILE LIMITED",
    "90F421": "IEEE Registration Authority",
    "242361": "vivo Mobile Communication Co., Ltd.",
    "D42787": "Shanghai High-Flying Electronics  Technology Co., Ltd",
    "34D4E3": "Atom Power, Inc.",
    "906D62": "Cambium Networks Limited",
    "30B29F": "EVIDENT CORPORATION",
    "24DCC3": "Espressif Inc.",
    "F824DB": "EntryPoint Networks, Inc",
    "B44D43": "IEEE Registration Authority",
    "F8C4AE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C4A816": "eero inc.",
    "34F043": "Samsung Electronics Co.,Ltd",
    "4C66A6": "Samsung Electronics Co.,Ltd",
    "4845CF": "LLC Proizvodstvennaya Kompania \"TransService\"",
    "8C02CD": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "54F8F0": "Tesla Inc",
    "74546B": "hangzhou zhiyi communication co., ltd",
    "D8D45D": "Orbic North America",
    "0C298F": "Tesla,Inc.",
    "20B82B": "Sagemcom Broadband SAS",
    "141844": "Xenon Smart Teknoloji Ltd.",
    "A06636": "Intracom SA Telecom Solutions",
    "5843AB": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "58569F": "Cisco Systems, Inc",
    "A82AD6": "Arthrex Inc.",
    "E48EBB": "Rockwell Automation",
    "50E538": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "F4AAD0": "OHSUNG",
    "FC48C9": "Yobiiq Intelligence B.V.",
    "641C10": "Texas Instruments",
    "24BF74": "Hamamatsu Photonics K.K.",
    "0833ED": "ASKEY COMPUTER CORP",
    "C01754": "Apple, Inc.",
    "BC3340": "Cisco Meraki",
    "001227": "Franklin Electric Co., Inc.",
    "AC1A3D": "Dell Inc.",
    "F49DA7": "Private",
    "E84C4A": "Amazon Technologies Inc.",
    "748F4D": "duagon Germany GmbH",
    "00C03A": "duagon Germany GmbH",
    "2C64F6": "Wu Qi Technologies,Inc.",
    "88684B": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "C8F225": "EM Microelectronic",
    "78530D": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "785F36": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "947FD8": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "20898A": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "18AA1E": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "FC7A58": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "E88FC4": "MOBIWIRE MOBILES(NINGBO) CO.,LTD",
    "00A085": "Private",
    "FC2CFD": "dormakaba Canada Inc. - Keyscan",
    "4C9B63": "LG Innotek",
    "648F3E": "Cisco Systems, Inc",
    "CCB6C8": "Cisco Systems, Inc",
    "18FB7B": "Dell Inc.",
    "1C4024": "Dell Inc.",
    "141877": "Dell Inc.",
    "E0DB55": "Dell Inc.",
    "F04DA2": "Dell Inc.",
    "842B2B": "Dell Inc.",
    "EC2C49": "NakaoLab, The University of Tokyo",
    "246E96": "Dell Inc.",
    "00065B": "Dell Inc.",
    "B8AC6F": "Dell Inc.",
    "00219B": "Dell Inc.",
    "002170": "Dell Inc.",
    "001EC9": "Dell Inc.",
    "34E6D7": "Dell Inc.",
    "74E6E2": "Dell Inc.",
    "24B6FD": "Dell Inc.",
    "000F1F": "Dell Inc.",
    "149ECF": "Dell Inc.",
    "484D7E": "Dell Inc.",
    "6CD6E3": "Cisco Systems, Inc",
    "D4AE52": "Dell Inc.",
    "F8B156": "Dell Inc.",
    "AC3D94": "Arista Networks",
    "CC96E5": "Dell Inc.",
    "3C46A1": "Ruckus Wireless",
    "AC128E": "Shanghai Baud Data Communication Co.,Ltd.",
    "ECED73": "Motorola Mobility LLC, a Lenovo Company",
    "78AC44": "Dell Inc.",
    "C03EBA": "Dell Inc.",
    "747827": "Dell Inc.",
    "B07B25": "Dell Inc.",
    "B88584": "Dell Inc.",
    "E4B97A": "Dell Inc.",
    "684F64": "Dell Inc.",
    "D8D090": "Dell Inc.",
    "1C721D": "Dell Inc.",
    "0C29EF": "Dell Inc.",
    "A02919": "Dell Inc.",
    "588A5A": "Dell Inc.",
    "ECC018": "Cisco Systems, Inc",
    "748FC2": "Cisco Systems, Inc",
    "64E0AB": "UNION MAN TECHNOLOGY CO.,LTD",
    "0019F0": "UNION MAN TECHNOLOGY CO.,LTD",
    "A01C87": "UNION MAN TECHNOLOGY CO.,LTD",
    "40F4FD": "UNION MAN TECHNOLOGY CO.,LTD",
    "1884C1": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "D016F0": "IEEE Registration Authority",
    "B0449C": "Assa Abloy AB - Yale",
    "602A1B": "JANCUS",
    "68856A": "OuterLink Corporation",
    "E41289": "topsystem GmbH",
    "30E1F1": "Intelbras",
    "D824EC": "Plenom A/S",
    "1012D0": "zte corporation",
    "44A3C7": "zte corporation",
    "001D9C": "Rockwell Automation",
    "D019D3": "ITEL MOBILE LIMITED",
    "CCACFE": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "886EDD": "Micronet union Technology(Chengdu)Co., Ltd.",
    "80F3EF": "Meta Platforms Technologies, LLC",
    "D8B249": "Huawei Device Co., Ltd.",
    "C49D08": "Huawei Device Co., Ltd.",
    "4C5ED3": "Unisyue Technologies Co; LTD.",
    "A0FF0C": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "085411": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "743FC2": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "A4D5C2": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "F40046": "ON Semiconductor",
    "80433F": "Juniper Networks",
    "C42F90": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "54C415": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "B4A382": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "686DBC": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "08A189": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "44A642": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "5C4DBF": "zte corporation",
    "984744": "Shenzhen Boomtech Industrial Corporation",
    "4838B6": "Auhui Taoyun Technology Co., Ltd",
    "AC89D2": "Ciena Corporation",
    "E05694": "Yunhight Microelectronics",
    "242730": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "541159": "Nettrix Information Industry co.LTD",
    "BCC427": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000356": "Diebold Nixdorf",
    "00156D": "Ubiquiti Inc",
    "002722": "Ubiquiti Inc",
    "DC9FDB": "Ubiquiti Inc",
    "B0CFCB": "Amazon Technologies Inc.",
    "18E829": "Ubiquiti Inc",
    "74ACB9": "Ubiquiti Inc",
    "F492BF": "Ubiquiti Inc",
    "68D79A": "Ubiquiti Inc",
    "003052": "DZS Inc.",
    "504594": "Radisys",
    "80711F": "Juniper Networks",
    "64B708": "Espressif Inc.",
    "E8A245": "Juniper Networks",
    "840328": "Juniper Networks",
    "F4BFA8": "Juniper Networks",
    "C8FE6A": "Juniper Networks",
    "FC9643": "Juniper Networks",
    "0805E2": "Juniper Networks",
    "68F38E": "Juniper Networks",
    "8828FB": "Juniper Networks",
    "E824A6": "Juniper Networks",
    "B49882": "Brusa HyPower AG",
    "88E0F3": "Juniper Networks",
    "F8C001": "Juniper Networks",
    "A8D0E5": "Juniper Networks",
    "54E032": "Juniper Networks",
    "3C8AB0": "Juniper Networks",
    "3C6104": "Juniper Networks",
    "88E64B": "Juniper Networks",
    "D0DD49": "Juniper Networks",
    "1C9C8C": "Juniper Networks",
    "9C8ACB": "Juniper Networks",
    "1039E9": "Juniper Networks",
    "2C6BF5": "Juniper Networks",
    "B0C69A": "Juniper Networks",
    "EC13DB": "Juniper Networks",
    "F4CC55": "Juniper Networks",
    "EC94D5": "Juniper Networks",
    "A4E11A": "Juniper Networks",
    "24FC4E": "Juniper Networks",
    "A4515E": "Juniper Networks",
    "F4B52F": "Juniper Networks",
    "0014F6": "Juniper Networks",
    "28A24B": "Juniper Networks",
    "001DB5": "Juniper Networks",
    "7C273C": "Shenzhen Yunlink Technology Co., Ltd",
    "1C2AB0": "Beijing Xiaomi Electronics Co.,Ltd",
    "9817F1": "zte corporation",
    "4CC844": "Maipu Communication Technology Co.,Ltd.",
    "0C9505": "The Chamberlain Group, Inc",
    "08E6C9": "Business-intelligence of Oriental Nations Corporation Ltd.",
    "98C854": "Chiun Mai Communication System, Inc",
    "7017D7": "Shanghai Enflame Technology Co., Ltd.",
    "A08E24": "eero inc.",
    "247823": "Panasonic Entertainment & Communication Co., Ltd.",
    "844693": "Beijing Xiaomi Mobile Software Co., Ltd",
    "B86AF1": "Sagemcom Broadband SAS",
    "CC5830": "Sagemcom Broadband SAS",
    "44DF65": "Beijing Xiaomi Mobile Software Co., Ltd",
    "D8031A": "Laird Connectivity",
    "002BF5": "BUFFALO.INC",
    "5C1648": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "202141": "Universal Electronics BV",
    "28D0CB": "Adtran Inc",
    "AC51EE": "Adtran Inc",
    "C4FC22": "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD.",
    "082802": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "CC4210": "Xiaomi Communications Co Ltd",
    "A06032": "Amcrest Technologies",
    "F0D31F": "Apple, Inc.",
    "B4AEC1": "Apple, Inc.",
    "5432C7": "Apple, Inc.",
    "58E488": "Amazon Technologies Inc.",
    "D853BC": "Lenovo Information Products (Shenzhen)Co.,Ltd",
    "98F9CC": "Zhejiang Dahua Technology Co., Ltd.",
    "3868A4": "Samsung Electronics Co.,Ltd",
    "AC1E92": "Samsung Electronics Co.,Ltd",
    "80ACC8": "Phyplus Microelectronics Limited",
    "A416C0": "Apple, Inc.",
    "DC45B8": "Apple, Inc.",
    "90ECEA": "Apple, Inc.",
    "10B588": "Apple, Inc.",
    "BCAD90": "Kymeta Purchasing",
    "3C998C": "Houwa System Design Corp.",
    "5C7545": "Wayties, Inc.",
    "380FAD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B4E54C": "LLC \"Elektra\"",
    "DCC2C9": "CANON INC.",
    "4829D6": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "FC6A1C": "Mellanox Technologies, Inc.",
    "A088C2": "Mellanox Technologies, Inc.",
    "E462C4": "Cisco Systems, Inc",
    "20DE1E": "Nokia",
    "607771": "Texas Instruments",
    "ACB181": "Belden Mooresville",
    "ACE403": "Shenzhen Visteng Technology CO.,LTD",
    "A88055": "Tuya Smart Inc.",
    "508BB9": "Tuya Smart Inc.",
    "6CACC2": "Samsung Electronics Co.,Ltd",
    "BCF730": "Samsung Electronics Co.,Ltd",
    "C4DB04": "HUAWEI TECHNOLOGIES CO.,LTD",
    "947D77": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9409C9": "ALPSALPINE CO,.LTD",
    "60D039": "Apple, Inc.",
    "C4C17D": "Apple, Inc.",
    "44EE14": "Texas Instruments",
    "B4AC9D": "Texas Instruments",
    "D43F32": "eero inc.",
    "F83451": "Comcast-SRL",
    "686372": "Huawei Device Co., Ltd.",
    "A0C20D": "Huawei Device Co., Ltd.",
    "4C889E": "Huawei Device Co., Ltd.",
    "3C3174": "Google, Inc.",
    "BC9307": "Samsung Electronics Co.,Ltd",
    "503CCA": "TECNO MOBILE LIMITED",
    "9C0C35": "Shenzhenshi Xinzhongxin Technology Co.Ltd",
    "78D6D6": "eero inc.",
    "A43F51": "Shenzhen Benew Technology  Co.,Ltd.",
    "A41894": "Bosch Security Systems B.V.",
    "8CCDFE": "AMPAK Technology,Inc.",
    "DCFBB8": "Meizhou Guo Wei Electronics Co., Ltd",
    "001B8F": "Cisco Systems, Inc",
    "D04F58": "Ruckus Wireless",
    "6C2316": "TATUNG Technology Inc.,",
    "C8EDFC": "Shenzhen Ideaform Industrial Product Design Co., Ltd",
    "F82229": "Nokia Shanghai Bell Co., Ltd.",
    "485D35": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "98ACEF": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "E0BDA0": "Apple, Inc.",
    "5873D8": "Apple, Inc.",
    "F4E8C7": "Apple, Inc.",
    "148509": "Apple, Inc.",
    "443D54": "Amazon Technologies Inc.",
    "5447CC": "Sagemcom Broadband SAS",
    "A0A763": "Polytron Vertrieb GmbH",
    "24D79C": "Cisco Systems, Inc",
    "24724A": "Nile Global Inc",
    "00841E": "Cisco Meraki",
    "546503": "Quectel Wireless Solutions Co.,Ltd.",
    "F85B9B": "iMercury",
    "000941": "Allied Telesis K.K.",
    "D4925E": "Technicolor Delivery Technologies Belgium NV",
    "E8473A": "Hon Hai Precision Industry Co.,LTD",
    "D80E29": "vivo Mobile Communication Co., Ltd.",
    "10BF67": "Amazon Technologies Inc.",
    "FCD5D9": "Shenzhen SDMC Technology CO.,Ltd.",
    "80FBF0": "Quectel Wireless Solutions Co.,Ltd.",
    "90BDE6": "Quectel Wireless Solutions Co.,Ltd.",
    "500B26": "HUAWEI TECHNOLOGIES CO.,LTD",
    "504172": "HUAWEI TECHNOLOGIES CO.,LTD",
    "90A6BF": "Quectel Wireless Solutions Co.,Ltd.",
    "3C585D": "Sagemcom Broadband SAS",
    "389592": "Tendyron Corporation",
    "F0C745": "TECNO MOBILE LIMITED",
    "B46DC2": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "3CFA06": "Microsoft Corporation",
    "443262": "zte corporation",
    "F0F69C": "NIO Co., Ltd.",
    "58E403": "Wistron Neweb Corporation",
    "98CCD9": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "64D315": "HMD Global Oy",
    "606D9D": "Otto Bock Healthcare Products GmbH",
    "B06BB3": "GRT",
    "4C82A9": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "0CAF31": "Cisco Systems, Inc",
    "647C34": "Ubee Interactive Co., Limited",
    "6C38A1": "Ubee Interactive Co., Limited",
    "ACCF7B": "INGRAM MICRO SERVICES",
    "C8E306": "eero inc.",
    "AC3EB1": "Google, Inc.",
    "CCB7C4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5014C1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60A2C6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4E8A3": "Fujian Newland Auto-ID Tech Co.,Ltd.",
    "B8165F": "LG Innotek",
    "1C6349": "Texas Instruments",
    "E4521E": "Texas Instruments",
    "182C65": "Texas Instruments",
    "18FAB7": "Apple, Inc.",
    "7022FE": "Apple, Inc.",
    "881E5A": "Apple, Inc.",
    "00C585": "Apple, Inc.",
    "A87CF8": "Apple, Inc.",
    "60567D": "AM Telecom co., Ltd.",
    "D4F0C9": "KYOCERA Document Solutions Inc.",
    "64E833": "Espressif Inc.",
    "24B7DA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0846C7": "Fiberhome Telecommunication Technologies Co.,LTD",
    "78D840": "Xiaomi Communications Co Ltd",
    "C8965A": "SKY UK LIMITED",
    "C8DE41": "SKY UK LIMITED",
    "38CA84": "HP Inc.",
    "B8AB62": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "84E657": "Sony Interactive Entertainment Inc.",
    "205E97": "Nokia",
    "048680": "Quectel Wireless Solutions Co.,Ltd.",
    "FC22F4": "Zyxel Communications Corporation",
    "F0EDB8": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "F87999": "Guangdong Jiuzhi Technology Co.,Ltd",
    "CC2AAC": "Yunjing lntelligent Technology(Dongguan).,Ltd",
    "20406A": "AMPAK Technology,Inc.",
    "84DBA4": "Huawei Device Co., Ltd.",
    "245CC5": "Huawei Device Co., Ltd.",
    "F463FC": "vivo Mobile Communication Co., Ltd.",
    "B83DFB": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "5464DE": "u-blox AG",
    "38BC61": "Starkoff Co., Ltd.",
    "EC7427": "eero inc.",
    "68E1DC": "BUFFALO.INC",
    "5CAC3D": "Samsung Electronics Co.,Ltd",
    "CCE686": "Samsung Electronics Co.,Ltd",
    "74190A": "Samsung Electronics Co.,Ltd",
    "C0C4F9": "Qisda Corporation",
    "10A829": "Cisco Systems, Inc",
    "E46017": "Intel Corporate",
    "30F6EF": "Intel Corporate",
    "586D67": "Intel Corporate",
    "686CE6": "Microsoft Corporation",
    "003126": "Nokia",
    "109826": "Nokia",
    "C870D4": "IBO Technology Co,Ltd",
    "5847CA": "IEEE Registration Authority",
    "84FB43": "Central Denshi Seigyo",
    "68D40C": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "24E3DE": "China Telecom Fufu Information Technology Co., Ltd.",
    "1816E8": "Siliconware Precision Industries Co., Ltd.",
    "CC79D7": "Cisco Systems, Inc",
    "E4387E": "Cisco Systems, Inc",
    "90AC3F": "BrightSign LLC",
    "68F543": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8082F5": "STMicrolectronics International NV",
    "F4F19E": "Wistron InforComm (Zhongshan) Corporation",
    "104C43": "Fiberhome Telecommunication Technologies Co.,LTD",
    "647CE8": "Palo Alto Networks",
    "282BB9": "Shenzhen Xiongxin Technology Co.,Ltd",
    "301ABA": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "448EEC": "China Mobile Group Device Co.,Ltd.",
    "F026F8": "Worldcns Co.,Ltd.",
    "140708": "CP PLUS GMBH & CO. KG",
    "F85B6E": "Samsung Electronics Co.,Ltd",
    "14D424": "AzureWave Technology Inc.",
    "68275F": "zte corporation",
    "4441F0": "zte corporation",
    "A47952": "Huawei Device Co., Ltd.",
    "B87CD0": "Huawei Device Co., Ltd.",
    "D07E01": "Huawei Device Co., Ltd.",
    "F87907": "Huawei Device Co., Ltd.",
    "7CE87F": "Sagemcom Broadband SAS",
    "68871C": "Motorola Mobility LLC, a Lenovo Company",
    "F8A475": "PT Indoreka Jaya Wutama",
    "1449D4": "Xiaomi Communications Co Ltd",
    "F87D76": "Apple, Inc.",
    "00A093": "B/E AEROSPACE, Inc.",
    "2C7600": "Apple, Inc.",
    "7042D3": "Ruijie Networks Co.,LTD",
    "18C3F4": "IEEE Registration Authority",
    "34DF20": "Shenzhen Comstar .Technology Co.,Ltd",
    "387C76": "Universal Global Scientific Industrial Co., Ltd.",
    "E4A634": "Universal Electronics, Inc.",
    "2C8D37": "Virtium",
    "40FDF3": "AMPAK Technology,Inc.",
    "8C6A3B": "Samsung Electronics Co.,Ltd",
    "241153": "Samsung Electronics Co.,Ltd",
    "889CAD": "Cisco Systems, Inc",
    "D88332": "TaiXin Semiconductor Co., Ltd",
    "ECE61D": "Huawei Device Co., Ltd.",
    "4C63AD": "Huawei Device Co., Ltd.",
    "DCDB27": "Huawei Device Co., Ltd.",
    "ACB687": "Arcadyan Corporation",
    "506391": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E8A34E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "041892": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14656A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6CB7E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C475EA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9025F2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F41AB0": "Shenzhen Xingguodu Technology Co., Ltd.",
    "B01F47": "Heights Telecom T ltd",
    "282947": "Chipsea Technologies (Shenzhen) Corp.",
    "ACBCB5": "Apple, Inc.",
    "082573": "Apple, Inc.",
    "AC007A": "Apple, Inc.",
    "F01FC7": "Apple, Inc.",
    "88200D": "Apple, Inc.",
    "BC1541": "Nokia",
    "40E171": "Jiangsu Huitong Group Co.,Ltd.",
    "18C300": "Nokia",
    "D44D77": "Nokia",
    "946DAE": "Mellanox Technologies, Inc.",
    "E0F318": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "C464F2": "Infinix mobility limited",
    "6C724A": "Onkyo Technology K.K.",
    "D8FFC3": "Shenzhen 3SNIC information technology company Limited",
    "B88F27": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "8C255E": "VoltServer",
    "58B38F": "New H3C Technologies Co., Ltd",
    "F46D3F": "Intel Corporate",
    "DC0539": "Cisco Systems, Inc",
    "4827E2": "Espressif Inc.",
    "3886F7": "Google, Inc.",
    "F4227A": "Guangdong Seneasy Intelligent Technology Co., Ltd.",
    "DC8DB7": "ATW TECHNOLOGY, INC.",
    "543D92": "WIRELESS-TEK TECHNOLOGY LIMITED",
    "9826AD": "Quectel Wireless Solutions Co.,Ltd.",
    "640E6A": "SECO-LARM USA Inc",
    "8C5109": "IEEE Registration Authority",
    "68E154": "SiMa.ai",
    "6C60D0": "Huawei Device Co., Ltd.",
    "E8EF05": "MIND TECH INTERNATIONAL LIMITED",
    "A475B9": "Samsung Electronics Co.,Ltd",
    "80549C": "Samsung Electronics Co.,Ltd",
    "1CF8D0": "Samsung Electronics Co.,Ltd",
    "F08756": "Zyxel Communications Corporation",
    "94D331": "Xiaomi Communications Co Ltd",
    "D4430E": "Zhejiang Dahua Technology Co., Ltd.",
    "F85E0B": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "90F970": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04CAED": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68EE88": "Shenzhen TINNO Mobile Technology Corp.",
    "2C3B70": "AzureWave Technology Inc.",
    "5C8C30": "Taicang T&W Electronics",
    "34DD04": "Minut AB",
    "601E98": "Axevast Technology",
    "A8F7D9": "Mist Systems, Inc.",
    "448816": "Cisco Systems, Inc",
    "E8AC23": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68D927": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C9D65": "vivo Mobile Communication Co., Ltd.",
    "4C9D22": "ACES Co.,Ltd",
    "88C9E8": "Sony Corporation",
    "B8F0B9": "zte corporation",
    "805B65": "LG Innotek",
    "A0C98B": "Nokia Solutions and Networks GmbH & Co. KG",
    "18BB1C": "Huawei Device Co., Ltd.",
    "70A983": "Cisco Systems, Inc",
    "BCFAEB": "Cisco Systems, Inc",
    "848553": "Biznes Systema Telecom, LLC",
    "B47D76": "KNS Group LLC",
    "C0AD97": "TECNO MOBILE LIMITED",
    "580032": "Genexis B.V.",
    "BCC7DA": "Earda Technologies co Ltd",
    "1866F0": "Jupiter Systems",
    "74604C": "RODE",
    "B038E2": "Wanan Hongsheng Electronic Co.Ltd",
    "4C5369": "YanFeng Visteon(ChongQing) Automotive Electronic Co.,Ltd",
    "E048D8": "Guangzhi Wulian Technology(Guangzhou) Co., Ltd",
    "F8E4A4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "286F40": "Tonly Technology Co. Ltd ",
    "0C86C7": "Jabil Circuit (Guangzhou) Limited",
    "D88863": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C03E50": "HUAWEI TECHNOLOGIES CO.,LTD",
    "806036": "HUAWEI TECHNOLOGIES CO.,LTD",
    "74767D": "shenzhen kexint technology co.,ltd",
    "24CF24": "Beijing Xiaomi Mobile Software Co., Ltd",
    "F06C5D": "Xiaomi Communications Co Ltd",
    "40B02F": "Miele & Cie. KG",
    "0C7BC8": "Cisco Meraki",
    "C8F09E": "Espressif Inc.",
    "DC5475": "Espressif Inc.",
    "001401": "Rivertree Networks Corp.",
    "006068": "Dialogic Corporation",
    "1C5974": "IEEE Registration Authority",
    "1C0D7D": "Apple, Inc.",
    "14F287": "Apple, Inc.",
    "585595": "Apple, Inc.",
    "14946C": "Apple, Inc.",
    "90D473": "vivo Mobile Communication Co., Ltd.",
    "607D09": "Luxshare Precision Industry Co., Ltd",
    "EC6260": "Espressif Inc.",
    "B06E72": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "5CA4F4": "zte corporation",
    "246C60": "Huawei Device Co., Ltd.",
    "64C582": "China Mobile Group Device Co.,Ltd.",
    "88FC5D": "Cisco Systems, Inc",
    "F4C88A": "Intel Corporate",
    "28827C": "Bosch Automative products(Suzhou)Co.,Ltd Changzhou Branch",
    "CC3E79": "ARRIS Group, Inc.",
    "28F5D1": "ARRIS Group, Inc.",
    "10E177": "ARRIS Group, Inc.",
    "303EA7": "Intel Corporate",
    "50284A": "Intel Corporate",
    "08EBF6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B02347": "Shenzhen Giant Microelectronics Company Limited",
    "183C98": "Shenzhen Hengyi Technology Co., LTD",
    "D81068": "Murata Manufacturing Co., Ltd.",
    "5C045A": "Company NA Stage & Light",
    "58C356": "EM Microelectronic",
    "F4E204": "COYOTE SYSTEM",
    "F84E58": "Samsung Electronics Co.,Ltd",
    "B47064": "Samsung Electronics Co.,Ltd",
    "4C2E5E": "Samsung Electronics Co.,Ltd",
    "645DF4": "Samsung Electronics Co.,Ltd",
    "DCA956": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "CC827F": "Advantech Technology (CHINA) Co., Ltd.",
    "78AF08": "Intel Corporate",
    "CCDD58": "Robert Bosch GmbH",
    "787104": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "0016A3": "INGETEAM",
    "883F0C": "system a.v. co., ltd.",
    "A0EDFB": "Quectel Wireless Solutions Co.,Ltd.",
    "104D15": "Viaanix Inc",
    "50A015": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "7085C4": "Ruijie Networks Co.,LTD",
    "A0092E": "zte corporation",
    "AC330B": "Japan Computer Vision Corp.",
    "3053C1": "CRESYN",
    "5CC563": "HUNAN FN-LINK TECHNOLOGY LIMITED",
    "EC6073": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "74DDCB": "China Leadshine Technology Co.,Ltd",
    "A8B13B": "HP Inc.",
    "C43875": "Sonos, Inc.",
    "68B691": "Amazon Technologies Inc.",
    "B0AFF7": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "4827C5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BCD206": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14755B": "Intel Corporate",
    "C854A4": "Infinix mobility limited",
    "2CF295": "Huawei Device Co., Ltd.",
    "7CDAC3": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "BC6193": "Xiaomi Communications Co Ltd",
    "F8E57E": "Cisco Systems, Inc",
    "3C5D29": "Zhejiang Tmall Technology Co., Ltd.",
    "68FCCA": "Samsung Electronics Co.,Ltd",
    "6CD719": "Fiberhome Telecommunication Technologies Co.,LTD",
    "385B44": "Silicon Laboratories",
    "943469": "Silicon Laboratories",
    "E0BB0C": "Synertau LLC",
    "80015C": "Synaptics, Inc",
    "CCEB18": "OOO \"TSS\"",
    "AC77B9": "Nanjing Yufei Intelligent Control Technology Co.,LTD",
    "A85BB7": "Apple, Inc.",
    "3462B4": "Renesas Electronics (Penang) Sdn. Bhd.",
    "8C17B6": "Huawei Device Co., Ltd.",
    "B82B68": "Huawei Device Co., Ltd.",
    "0010E6": "APPLIED INTELLIGENT SYSTEMS, INC.",
    "000813": "Diskbank, Inc.",
    "744687": "Kingsignal Technology Co., Ltd.",
    "8C477F": "NambooSolution",
    "482E72": "Cisco Systems, Inc",
    "30CBC7": "Cambium Networks Limited",
    "885046": "LEAR",
    "0826AE": "IEEE Registration Authority",
    "244CAB": "Espressif Inc.",
    "E04102": "zte corporation",
    "D84008": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C047A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C558D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A42A95": "D-Link International",
    "78482C": "START USA, INC.",
    "7C45D0": "Shenzhen Wewins Wireless Co., ltd",
    "70041D": "Espressif Inc.",
    "886F29": "Pocketbook International SA",
    "BC2247": "New H3C Technologies Co., Ltd",
    "20AF1B": "SteelSeries ApS",
    "18C293": "Laird Connectivity",
    "000AC2": "Wuhan FiberHome Digital Technology Co.,Ltd.",
    "704CB6": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "549B49": "NEC Platforms, Ltd.",
    "7C7716": "Zyxel Communications Corporation",
    "943FBB": "JSC RPC Istok named after Shokin",
    "50E7B7": "vivo Mobile Communication Co., Ltd.",
    "F4F70C": "Avang - neterbit",
    "74EF4B": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "448C00": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "F0BE25": "Dongguan Cannice Precision Manufacturing Co., Ltd.",
    "149BD7": "MULI MUWAI FURNITURE QIDONG CO., LTD",
    "D0E828": "Radiant Industries Incorporated",
    "107636": "Earda Technologies co Ltd",
    "F4442C": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "E0FFF1": "Texas Instruments",
    "9C0B05": "eero inc.",
    "40F6BC": "Amazon Technologies Inc.",
    "ECDA59": "New H3C Technologies Co., Ltd",
    "98B785": "Shenzhen 10Gtek Transceivers Co., Limited",
    "485AEA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "848102": "Fiberhome Telecommunication Technologies Co.,LTD",
    "54E005": "Fiberhome Telecommunication Technologies Co.,LTD",
    "A083B4": "HeNet B.V.",
    "3C8B7F": "Cisco Systems, Inc",
    "C0F87F": "Cisco Systems, Inc",
    "5C8E8B": "Shenzhen Linghai Electronics Co.,Ltd",
    "10B1DF": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "F4F647": "zte corporation",
    "E88175": "zte corporation",
    "1C7125": "Apple, Inc.",
    "34FE77": "Apple, Inc.",
    "6CE5C9": "Apple, Inc.",
    "74B7E6": "Zegna-Daidong Limited",
    "04FF08": "Huawei Device Co., Ltd.",
    "00A45F": "Huawei Device Co., Ltd.",
    "FCE26C": "Apple, Inc.",
    "4C7975": "Apple, Inc.",
    "DC9758": "Sichuan AI-Link Technology Co., Ltd.",
    "78F238": "Samsung Electronics Co.,Ltd",
    "64D0D6": "Samsung Electronics Co.,Ltd",
    "78EB46": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C416C8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E4DCCC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3CA161": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000931": "Future Internet, Inc.",
    "B8F255": "Universal Electronics, Inc.",
    "EC656E": "The Things Industries B.V.",
    "10AEA5": "Duskrise inc.",
    "94944A": "Particle Industries Inc.",
    "5C49FA": "Shenzhen Guowei Shidai Communication Equipement Co., Ltd",
    "2CA327": "Oraimo Technology Limited",
    "9075BC": "Nokia Shanghai Bell Co., Ltd.",
    "4C53FD": "Amazon Technologies Inc.",
    "587DB6": "Northern Data AG",
    "782184": "Espressif Inc.",
    "341343": "GE Lighting",
    "185880": "Arcadyan Corporation",
    "3083D2": "Motorola Mobility LLC, a Lenovo Company",
    "AC50DE": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "00DDB6": "New H3C Technologies Co., Ltd",
    "B89EA6": "SPBEC-MINING CO.LTD",
    "E07E5F": "Renesas Electronics (Penang) Sdn. Bhd.",
    "38BEAB": "AltoBeam (China) Inc.",
    "047975": "Honor Device Co., Ltd.",
    "9C0567": "Honor Device Co., Ltd.",
    "1466B7": "Advanced Design Technology Pty Ltd",
    "B08B92": "zte corporation",
    "08C8C2": "GN Audio A/S",
    "AC8B6A": "China Mobile IOT Company Limited",
    "000056": "DR. B. STRUCK",
    "70FD88": "Nanjing Jiahao Technology Co., Ltd.",
    "98192C": "Edgecore Networks Corporation",
    "080004": "CROMEMCO INCORPORATED",
    "FC777B": "Hitron Technologies. Inc",
    "14172A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "005FBF": "Toshiba Corp.",
    "D405DE": "eero inc.",
    "FC0FE7": "Microchip Technology Inc.",
    "241281": "China Mobile Group Device Co.,Ltd.",
    "50C0F0": "Artek Microelectronics Co.,Ltd.",
    "B4AE2B": "Microsoft",
    "38A91C": "New H3C Technologies Co., Ltd",
    "F4700C": "IEEE Registration Authority",
    "549F06": "Nokia Shanghai Bell Co., Ltd.",
    "68A878": "GeoWAN Pty Ltd",
    "409B21": "Nokia",
    "4873CB": "Tiinlab Corporation",
    "789FAA": "Huawei Device Co., Ltd.",
    "E8A72F": "Microsoft Corporation",
    "94B34F": "Ruckus Wireless",
    "C85CCC": "Beijing Xiaomi Mobile Software Co., Ltd",
    "38384B": "vivo Mobile Communication Co., Ltd.",
    "140A29": "Tiinlab Corporation",
    "301A30": "Mako Networks Ltd",
    "C48BA3": "Cisco Meraki",
    "24CE33": "Amazon Technologies Inc.",
    "0CDDEF": "Nokia Corporation",
    "5C9666": "Sony Interactive Entertainment Inc.",
    "88231F": "Fibocom Wireless Inc.",
    "A4DE26": "Sumitomo Electric Industries, Ltd",
    "307BC9": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "68E478": "Qingdao Haier Technology Co.,Ltd",
    "E09D13": "Samsung Electronics Co.,Ltd",
    "A00F37": "Cisco Systems, Inc",
    "4C445B": "Intel Corporate",
    "C8CA79": "Ciena Corporation",
    "BC6EE2": "Intel Corporate",
    "7C726E": "Ericsson AB",
    "D4EEDE": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "C08B05": "HUAWEI TECHNOLOGIES CO.,LTD",
    "046865": "Apple, Inc.",
    "DC5392": "Apple, Inc.",
    "1CB3C9": "Apple, Inc.",
    "FCAA81": "Apple, Inc.",
    "609316": "Apple, Inc.",
    "646D2F": "Apple, Inc.",
    "C0F9D2": "arkona technologies GmbH",
    "387A0E": "Intel Corporate",
    "042084": "zte corporation",
    "B45F84": "zte corporation",
    "00DF1D": "Cisco Systems, Inc",
    "787264": "IEEE Registration Authority",
    "1097BD": "Espressif Inc.",
    "B01C0C": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "68403C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "903E7F": "Fiberhome Telecommunication Technologies Co.,LTD",
    "14C0A1": "UCloud Technology Co., Ltd.",
    "DC0E96": "Palo Alto Networks",
    "30C6F7": "Espressif Inc.",
    "C03653": "eero inc.",
    "940230": "Logitech",
    "701135": "Livesecu co., Ltd",
    "643216": "Weidu Technology (Beijing) Co., Ltd.",
    "C06B55": "Motorola Mobility LLC, a Lenovo Company",
    "DC84E9": "Shenzhen Qihoo Intelligent Technology Co.,Ltd",
    "78670E": "Wistron Neweb Corporation",
    "080205": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9035EA": "Silicon Laboratories",
    "FCA84A": "Sentinum GmbH",
    "6C3C7C": "CANON INC.",
    "304F00": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "1C5EE6": "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD.",
    "346F24": "AzureWave Technology Inc.",
    "A41A3A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "3484E4": "Texas Instruments",
    "AC4D16": "Texas Instruments",
    "B010A0": "Texas Instruments",
    "DC9166": "Huawei Device Co., Ltd.",
    "AC80D6": "Hexatronic AB",
    "DCE55B": "Google, Inc.",
    "4CB9EA": "iRobot Corporation",
    "B894E7": "Xiaomi Communications Co Ltd",
    "34B98D": "Xiaomi Communications Co Ltd",
    "C4D7FD": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "D876AE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C8408": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A09F7A": "D-Link Middle East FZCO",
    "B42046": "eero inc.",
    "E86E44": "zte corporation",
    "00E7E3": "zte corporation",
    "B06A41": "Google, Inc.",
    "1CED6F": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "D81F12": "Tuya Smart Inc.",
    "204B22": "Sunnovo International Limited",
    "C81CFE": "Zebra Technologies Inc.",
    "F0704F": "Samsung Electronics Co.,Ltd",
    "AC6C90": "Samsung Electronics Co.,Ltd",
    "90B622": "Samsung Electronics Co.,Ltd",
    "0C02BD": "Samsung Electronics Co.,Ltd",
    "2CD1DA": "Keysight Technologies, Inc.",
    "C4FCEF": "SambaNova Systems, Inc.",
    "5C5230": "Apple, Inc.",
    "645A36": "Apple, Inc.",
    "6C71D2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F800A1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B4AC8C": "Bern University of Applied Sciences",
    "BCCE25": "Nintendo Co.,Ltd",
    "2032C6": "Apple, Inc.",
    "FC1D2A": "vivo Mobile Communication Co., Ltd.",
    "9C54C2": "New H3C Technologies Co., Ltd",
    "78E9CF": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "000D67": "Ericsson",
    "24D7EB": "Espressif Inc.",
    "58BF25": "Espressif Inc.",
    "80D2E5": "Nintendo Co.,Ltd",
    "04D442": "GUANGDONG GENIUS TECHNOLOGY CO., LTD.",
    "1C05B7": "Chongqing Trantor Technology Co., Ltd.",
    "141973": "Beijing Yunyi Times Technology Co.,Ltd",
    "546F71": "uAvionix Corporation",
    "54EF33": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "4448B9": "MitraStar Technology Corp.",
    "5C75C6": "China Mobile Group Device Co.,Ltd.",
    "9009D0": "Synology Incorporated",
    "A0D05B": "Samsung Electronics Co.,Ltd",
    "F8CE72": "Wistron Corporation",
    "CC9DA2": "Eltex Enterprise Ltd.",
    "B437D8": "D-Link (Shanghai) Limited Corp.",
    "000A02": "ANNSO CO., LTD.",
    "9CD57D": "Cisco Systems, Inc",
    "CCE236": "Hangzhou Yaguan Technology Co. LTD",
    "000FE5": "MERCURY SECURITY CORPORATION",
    "74765B": "Quectel Wireless Solutions Co.,Ltd.",
    "204181": "ESYSE GmbH Embedded Systems Engineering",
    "605375": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78DD33": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A031DB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DCBB96": "Full Solution Telecom",
    "941F3A": "Ambiq",
    "7066E1": "dnt Innovation GmbH",
    "48188D": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "80FBF1": "Freescale Semiconductor (China) Ltd.",
    "38D57A": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "08E7E5": "Huawei Device Co., Ltd.",
    "487397": "New H3C Technologies Co., Ltd",
    "A0445C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "646D4E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "085C1B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "509A88": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B83BCC": "Xiaomi Communications Co Ltd",
    "D09CAE": "vivo Mobile Communication Co., Ltd.",
    "88D199": "Vencer Co., Ltd.",
    "701F0B": "WILOGY SRL",
    "A01842": "Comtrend Corporation",
    "506F0C": "Sagemcom Broadband SAS",
    "D48660": "Arcadyan Corporation",
    "40406C": "Icomera",
    "043926": "China Dragon Technology Limited",
    "70CD0D": "Intel Corporate",
    "5C0CE6": "Nintendo Co.,Ltd",
    "1856C3": "Apple, Inc.",
    "6881E0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CD629": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0C478": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D86D17": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C1AC0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C87CE": "Espressif Inc.",
    "8C8172": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "1C93C4": "Amazon Technologies Inc.",
    "745D43": "BSH Hausgeraete GmbH",
    "000BA5": "Quasar Cipta Mandiri, PT",
    "44E517": "Intel Corporate",
    "AC49DB": "Apple, Inc.",
    "44F09E": "Apple, Inc.",
    "08FF44": "Apple, Inc.",
    "547787": "Earda Technologies co Ltd",
    "002704": "Accelerated Concepts, Inc",
    "D0C24E": "Samsung Electronics Co.,Ltd",
    "345B98": "EM Microelectronic",
    "B88D12": "Apple, Inc.",
    "080037": "FUJIFILM Business Innovation Corp.",
    "888E7F": "ATOP CORPORATION",
    "003059": "Kontron Europe GmbH",
    "88A0BE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "949010": "HUAWEI TECHNOLOGIES CO.,LTD",
    "98F083": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C64A2": "OnePlus Technology (Shenzhen) Co., Ltd",
    "D05475": "SAVI Controls",
    "489EBD": "HP Inc.",
    "00269C": "ITUS JAPAN CO. LTD",
    "28FA19": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "BCA37F": "Rail-Mil Sp. z o.o. Sp. K.",
    "28DFEB": "Intel Corporate",
    "88FCA6": "devolo AG",
    "5009E5": "Drimsys,Inc",
    "203CC0": "Beijing Tosee Technology Co., Ltd.",
    "28FBAE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D8A011": "WiZ",
    "A47806": "Cisco Systems, Inc",
    "B8D56B": "Mirka Ltd.",
    "CCB5D1": "Beijing Xiaomi Mobile Software Co., Ltd",
    "78D6DC": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "0C4314": "Silicon Laboratories",
    "F4C02F": "BlueBite",
    "EC8AC4": "Amazon Technologies Inc.",
    "78B84B": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "508A06": "Tuya Smart Inc.",
    "D40868": "Beijing Lanxum Computer Technology CO.,LTD.",
    "E8DA00": "Kivo Technology, Inc.",
    "7886B6": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "C48025": "Huawei Device Co., Ltd.",
    "B8145C": "Huawei Device Co., Ltd.",
    "C89D18": "Huawei Device Co., Ltd.",
    "DC0398": "LG Innotek",
    "10381F": "Sichuan AI-Link Technology Co., Ltd.",
    "5414F3": "Intel Corporate",
    "ECB4E8": "Wistron Mexico SA de CV",
    "30A612": "ShenZhen Hugsun Technology Co.,Ltd.",
    "18BB41": "Huawei Device Co., Ltd.",
    "7818A8": "Huawei Device Co., Ltd.",
    "807264": "Huawei Device Co., Ltd.",
    "E8F654": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2087EC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D44165": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "7CCC1F": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "FC372B": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "248BE0": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "5C4A1F": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "54E061": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "5CC8E3": "Shintec Hozumi co.ltd.",
    "086AC5": "Intel Corporate",
    "8C367A": "Palo Alto Networks",
    "6CFE54": "Intel Corporate",
    "3C195E": "Samsung Electronics Co.,Ltd",
    "1CFF59": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "C01B23": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "187532": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "9C9C40": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "4C3B6C": "GARO AB",
    "68DDD9": "HMD Global Oy",
    "6C9466": "Intel Corporate",
    "54F6E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A85081": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3C7AC4": "Chemtronics",
    "0015E5": "Cheertek Inc.",
    "8C83FC": "Axioma Metering UAB",
    "001FC8": "Up-Today Industrial Co., Ltd.",
    "38AFD0": "Nevro",
    "FC97A8": "Cricut Inc.",
    "A0D7A0": "Huawei Device Co., Ltd.",
    "E0DA90": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4A46B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C04754": "vivo Mobile Communication Co., Ltd.",
    "8CD9D6": "Xiaomi Communications Co Ltd",
    "60A4B7": "TP-Link Corporation Limited",
    "00A00E": "NETSCOUT SYSTEMS INC",
    "A45802": "SHIN-IL TECH",
    "38F3FB": "Asperiq",
    "50AE86": "Linkintec Co., Ltd",
    "601592": "IEEE Registration Authority",
    "143B42": "Realfit(Shenzhen) Intelligent Technology Co., Ltd",
    "943CC6": "Espressif Inc.",
    "A4DAD4": "Yamato Denki Co.,Ltd.",
    "F4B1C2": "Zhejiang Dahua Technology Co., Ltd.",
    "BCFAB8": "Guangzhou\u00a0Shiyuan\u00a0Electronic\u00a0Technology\u00a0Company\u00a0Limited",
    "A04A5E": "Microsoft Corporation",
    "60C5E6": "Skullcandy",
    "109497": "Logitech Hong Kong",
    "F02F4B": "Apple, Inc.",
    "40E64B": "Apple, Inc.",
    "EC2E98": "AzureWave Technology Inc.",
    "D47798": "Cisco Systems, Inc",
    "10C9CA": "Ace Technology Corp.",
    "B4FA48": "Apple, Inc.",
    "38B3F7": "Huawei Device Co., Ltd.",
    "84E986": "Huawei Device Co., Ltd.",
    "AC1F0F": "Texas Instruments",
    "74D285": "Texas Instruments",
    "8044FD": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "F46AD7": "Microsoft Corporation",
    "481F2D": "Shenzhen Jie Shi Lian Industrial Co.,LTD",
    "A8E81E": "ATW TECHNOLOGY, INC.",
    "1409B4": "zte corporation",
    "101081": "zte corporation",
    "F81A2B": "Google, Inc.",
    "8C73A0": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F4A80D": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "3CA8ED": "smart light technology",
    "F885F9": "Calix Inc.",
    "78CFF9": "Huawei Device Co., Ltd.",
    "04C29B": "Aura Home, Inc.",
    "1C87E3": "TECNO MOBILE LIMITED",
    "20B730": "TeconGroup, Inc",
    "509707": "Xiamen Paperang Technology Co.,Ltd.",
    "3CE36B": "Zhejiang Dahua Technology Co., Ltd.",
    "488899": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "DCB131": "SHENZHEN HUARUIAN TECHNOLOGY CO.,LTD",
    "F0625A": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "843B10": " LVSWITCHES INC.",
    "94A4F9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C3491": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E84D74": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC895E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC63D7": "Intel Corporate",
    "7C9F07": "CIG SHANGHAI CO LTD",
    "203B69": "vivo Mobile Communication Co., Ltd.",
    "5C1720": "Huawei Device Co., Ltd.",
    "605E4F": "Huawei Device Co., Ltd.",
    "1CE6AD": "Huawei Device Co., Ltd.",
    "14857F": "Intel Corporate",
    "94E23C": "Intel Corporate",
    "5856C2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A03679": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8D6F6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2C52AF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48684A": "Intel Corporate",
    "58FB96": "Ruckus Wireless",
    "285B0C": "Sichuan Jiuzhou Electronic Technology Co., Ltd.",
    "1489CB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C2636": "HUAWEI TECHNOLOGIES CO.,LTD",
    "40DE17": "Shenzhen Lanfeng Times Industrial Co.,Ltd.",
    "AC2316": "Mist Systems, Inc.",
    "2C00AB": "ARRIS Group, Inc.",
    "002470": "AUROTECH ultrasound AS.",
    "BCF171": "Intel Corporate",
    "84716A": "Huawei Device Co., Ltd.",
    "488C63": "Huawei Device Co., Ltd.",
    "70DDEF": "Huawei Device Co., Ltd.",
    "54A6DB": "Huawei Device Co., Ltd.",
    "149877": "Apple, Inc.",
    "88665A": "Apple, Inc.",
    "B0E5F9": "Apple, Inc.",
    "B88DF1": "Nanjing BigFish Semiconductor Co., Ltd.",
    "6C4A74": "AERODISK LLC",
    "14EB08": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B01656": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78D4F1": "IEEE Registration Authority",
    "DC9020": "RURU TEK PRIVATE LIMITED ",
    "A842A7": "Jiangsu Huitong Group Co.,Ltd.",
    "2CD26B": "FN-LINK TECHNOLOGY LIMITED",
    "A062FB": "HISENSE VISUAL TECHNOLOGY CO.,LTD",
    "9C6865": "Fiberhome Telecommunication Technologies Co.,LTD",
    "383D5B": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24EDFD": "Siemens Canada Limited",
    "6CCDD6": "NETGEAR",
    "540764": "Huawei Device Co., Ltd.",
    "DCD7A0": "Huawei Device Co., Ltd.",
    "F01628": "Technicolor (China) Technology Co., Ltd.",
    "F88F07": "Samsung Electronics Co.,Ltd",
    "8CEA48": "Samsung Electronics Co.,Ltd",
    "005F67": "TP-Link Corporation Limited",
    "503DC6": "Xiaomi Communications Co Ltd",
    "B0BD1B": "Dongguan Liesheng Electronic Co., Ltd.",
    "D4ECAB": "vivo Mobile Communication Co., Ltd.",
    "BC3ECB": "vivo Mobile Communication Co., Ltd.",
    "08798C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C9AA1": "Huawei Device Co., Ltd.",
    "BCF45F": "zte corporation",
    "6055F9": "Espressif Inc.",
    "548ABA": "Cisco Systems, Inc",
    "64A198": "Huawei Device Co., Ltd.",
    "1418C3": "Intel Corporate",
    "54DBA2": "Fibrain",
    "44AE25": "Cisco Systems, Inc",
    "BCE712": "Cisco Systems, Inc",
    "CCCC81": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4089A8": "WiredIQ, LLC",
    "3C2093": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "E02967": "HMD Global Oy",
    "988B69": "Shenzhen hylitech Co.,LTD",
    "18146C": "Zhejiang Tmall Technology Co., Ltd.",
    "688975": "nuoxc",
    "A899DC": "i-TOP DESING TECHNOLOGY CO.,LTD",
    "001D1E": "KYUSHU TEN CO.,LTD",
    "50C3A2": "nFore Technology Co.,Ltd.",
    "1CD1BA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "804B50": "Silicon Laboratories",
    "C0E3FB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30E283": "Texas Instruments",
    "34AFB3": "Amazon Technologies Inc.",
    "683F7D": "INGRAM MICRO SERVICES",
    "945F34": "Renesas Electronics (Penang) Sdn. Bhd.",
    "603CEE": "LG Electronics (Mobile Communications)",
    "2C4A11": "Ciena Corporation",
    "20AC9C": "China Telecom Corporation Limited",
    "28AD18": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "0C354F": "Nokia",
    "C4CB54": "Fibocom Auto Inc.",
    "3024A9": "HP Inc.",
    "9409D3": "shenzhen maxtopic technology co.,ltd",
    "F8A73A": "Cisco Systems, Inc",
    "B8114B": "Cisco Systems, Inc",
    "1C4C48": "ITEL MOBILE LIMITED",
    "2C0786": "Huawei Device Co., Ltd.",
    "A86E4E": "Huawei Device Co., Ltd.",
    "102D31": "Shenzhen Americas Trading Company LLC",
    "782E56": "China Mobile Group Device Co.,Ltd.",
    "C40B31": "Apple, Inc.",
    "8C7A15": "Ruckus Wireless",
    "6420E0": "T3 Technology Co., Ltd.",
    "8411C2": "IEEE Registration Authority",
    "90DE80": "Shenzhen Century Xinyang Technology Co., Ltd",
    "109693": "Amazon Technologies Inc.",
    "E4F1D4": "vivo Mobile Communication Co., Ltd.",
    "5CC0A0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "04F352": "HUAWEI TECHNOLOGIES CO.,LTD",
    "488B0A": "Cisco Systems, Inc",
    "185BB3": "Samsung Electronics Co.,Ltd",
    "9C5FB0": "Samsung Electronics Co.,Ltd",
    "E87F6B": "Samsung Electronics Co.,Ltd",
    "ECA1D1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A46DA4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24A487": "Huawei Device Co., Ltd.",
    "C45A86": "Huawei Device Co., Ltd.",
    "786A1F": "ARRIS Group, Inc.",
    "2494CB": "ARRIS Group, Inc.",
    "FCB69D": "Zhejiang Dahua Technology Co., Ltd.",
    "94F2BB": "Valeo Vision Systems",
    "B0ECDD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "6C1414": "BUJEON ELECTRONICS Co,.Ltd",
    "AC6784": "Google, Inc.",
    "A8B088": "eero inc.",
    "EC7E91": "ITEL MOBILE LIMITED",
    "E0E1A9": "Shenzhen Four Seas Global Link Network Technology Co., Ltd.",
    "7C4E09": "Shenzhen Skyworth Wireless Technology Co.,Ltd",
    "280FC5": "Beijing Leadsec Technology Co., Ltd.",
    "1CEC72": "Allradio Co., Ltd",
    "E4DC43": "Huawei Device Co., Ltd.",
    "2430F8": "Huawei Device Co., Ltd.",
    "C43CEA": "BUFFALO.INC",
    "D4910F": "Amazon Technologies Inc.",
    "80F5B5": "Texas Instruments",
    "1C3008": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "98063A": "Home Control Singapore Pte Ltd",
    "64A200": "Xiaomi Communications Co Ltd",
    "703A2D": "Shenzhen V-Link Technology CO., LTD.",
    "1C45C2": "Huizhou City Sunsin lntelligent Technology Co.,Ltd",
    "84CC63": "Huawei Device Co., Ltd.",
    "C07831": "Huawei Device Co., Ltd.",
    "C4FBAA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0A651": "Cisco Systems, Inc",
    "183672": "Shaoxing ShunChuang Technology CO.,LTD",
    "0017C9": "Samsung Electronics Co.,Ltd",
    "5CF9FD": "Taicang T&W Electronics",
    "3898E9": "Huawei Device Co., Ltd.",
    "48A516": "Huawei Device Co., Ltd.",
    "ACDCCA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B85FB0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B4BA12": "China Mobile (Hangzhou) Information Technology Co.,Ltd.",
    "E0E0C2": "China Mobile Group Device Co.,Ltd.",
    "6C1ED7": "vivo Mobile Communication Co., Ltd.",
    "F0AA0B": "Arra Networks/ Spectramesh",
    "945641": "Palo Alto Networks",
    "ECC302": "HUMAX Co., Ltd.",
    "C4DD57": "Espressif Inc.",
    "98C97C": "Shenzhen iComm Semiconductor CO.,LTD",
    "00C343": "E-T-A Circuit Breakers Ltd",
    "A0D0DC": "Amazon Technologies Inc.",
    "143FC3": "SnapAV",
    "209A7D": "Sagemcom Broadband SAS",
    "50C68E": "Biwin Semiconductor (HK) Company Limted",
    "E475DC": "Arcadyan Corporation",
    "58208A": "IEEE Registration Authority",
    "BCA5A9": "Apple, Inc.",
    "102779": "Sadel S.p.A.",
    "80CA4B": "SHENZHEN GONGJIN ELECTRONICS CO.,LTD",
    "C41688": "Huawei Device Co., Ltd.",
    "64B0E8": "Huawei Device Co., Ltd.",
    "30A998": "Huawei Device Co., Ltd.",
    "64F54E": "EM Microelectronic",
    "04E795": "HUAWEI TECHNOLOGIES CO.,LTD",
    "20E2A8": "Apple, Inc.",
    "A0FBC5": "Apple, Inc.",
    "007D60": "Apple, Inc.",
    "74427F": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "001C45": "Chenbro Micom Co., Ltd.",
    "C0619A": "IEEE Registration Authority",
    "28C21F": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "8C7086": "Gesellschaft f\u00fcr Sonder-EDV-Anlagen mbH",
    "400634": "Huawei Device Co., Ltd.",
    "C42B44": "Huawei Device Co., Ltd.",
    "F8A26D": "CANON INC.",
    "689E0B": "Cisco Systems, Inc",
    "004238": "Intel Corporate",
    "74F9CA": "Nintendo Co.,Ltd",
    "7061EE": "Sunwoda Electronic Co.,Ltd",
    "0019DC": "ENENSYS Technologies",
    "B07D64": "Intel Corporate",
    "48902F": "LG Electronics (Mobile Communications)",
    "4C2219": "YUANFUDAO HK LIMTED",
    "E4246C": "Zhejiang Dahua Technology Co., Ltd.",
    "E8EB1B": "Microchip Technology Inc.",
    "38BAB0": "Broadcom",
    "B0A460": "Intel Corporate",
    "000083": "TADPOLE TECHNOLOGY PLC",
    "28B77C": "IEEE Registration Authority",
    "00146A": "Cisco Systems, Inc",
    "68D6ED": "GooWi Wireless Technology Co., Limited",
    "840283": "HUMAX Co., Ltd.",
    "941700": "Xiaomi Communications Co Ltd",
    "D8EF42": "Huawei Device Co., Ltd.",
    "80CC12": "Huawei Device Co., Ltd.",
    "18AA0F": "Huawei Device Co., Ltd.",
    "54D9C6": "Huawei Device Co., Ltd.",
    "308AF7": "Huawei Device Co., Ltd.",
    "64E7D8": "Samsung Electronics Co.,Ltd",
    "00E406": "HUAWEI TECHNOLOGIES CO.,LTD",
    "44227C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CCB182": "HUAWEI TECHNOLOGIES CO.,LTD",
    "089BB9": "Nokia Solutions and Networks GmbH & Co. KG",
    "D89ED4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "40D25F": "ITEL MOBILE LIMITED",
    "54F15F": "Sichuan AI-Link Technology Co., Ltd.",
    "E079C4": "iRay Technology Company Limited",
    "AC9572": "Jovision Technology Co., Ltd.",
    "001753": "nFore Technology Inc.",
    "001636": "Quanta Computer Inc.",
    "884067": "infomark",
    "A8032A": "Espressif Inc.",
    "083869": "Hong Kong AMobile Intelligent Corp. Limited Taiwan Branch",
    "2481C7": "Huawei Device Co., Ltd.",
    "FC862A": "Huawei Device Co., Ltd.",
    "AC9A96": "Maxlinear, Inc",
    "4843DD": "Amazon Technologies Inc.",
    "5894A2": "KETEK GmbH",
    "001A57": "Matrix Design Group, LLC",
    "001CFC": "Sumitomo Electric Industries, Ltd",
    "EC753E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24A160": "Espressif Inc.",
    "DCAEEB": "Ruckus Wireless",
    "882949": "Renesas Electronics (Penang) Sdn. Bhd.",
    "ECDB86": "API-K",
    "3C846A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "84D81B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "7C2ADB": "Xiaomi Communications Co Ltd",
    "F864B8": "zte corporation",
    "145120": "Huawei Device Co., Ltd.",
    "C0D193": "Huawei Device Co., Ltd.",
    "7804E3": "Huawei Device Co., Ltd.",
    "A43B0E": "Huawei Device Co., Ltd.",
    "909164": "ChongQing Lavid Technology Co., Ltd.",
    "1C3D2F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001D67": "AMEC",
    "001A62": "Data Robotics, Incorporated",
    "C06369": "BINXIN TECHNOLOGY(ZHEJIANG) LTD.",
    "1841FE": "Digital 14",
    "08B0A7": "Truebeyond Co., Ltd",
    "18AB1D": "Samsung Electronics Co.,Ltd",
    "BCE92F": "HP Inc.",
    "E092A7": "Feitian Technologies Co., Ltd",
    "B0761B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C0E60": "Nanjing Juplink Intelligent Technologies Co., Ltd.",
    "DC91BF": "Amazon Technologies Inc.",
    "6CCE44": "1MORE",
    "944444": "LG Innotek",
    "340A33": "D-Link International",
    "8437D5": "Samsung Electronics Co.,Ltd",
    "3482C5": "Samsung Electronics Co.,Ltd",
    "F0D7AF": "IEEE Registration Authority",
    "7CA96B": "Syrotech Networks. Ltd.",
    "98063C": "Samsung Electronics Co.,Ltd",
    "74A7EA": "Amazon Technologies Inc.",
    "7C6DF8": "Apple, Inc.",
    "50AD71": "Tessolve Semiconductor Private Limited",
    "5CA5BC": "eero inc.",
    "D84F37": "Proxis, spol. s r.o.",
    "74AB93": "Blink by Amazon ",
    "E87F95": "Apple, Inc.",
    "781100": "Quantumsolution",
    "88C08B": "Apple, Inc.",
    "4C7CD9": "Apple, Inc.",
    "84EAED": "Roku, Inc",
    "940853": "Liteon Technology Corporation",
    "B49E80": "Sichuan Changhong Electric Ltd.",
    "A49340": "Beijing Supvan Information Technology Co.,Ltd.",
    "F8E877": "Harman/Becker Automotive Systems GmbH",
    "A09B12": "China Mobile IOT Company Limited",
    "98F621": "Xiaomi Communications Co Ltd",
    "1413FB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D0768F": "Calix Inc.",
    "00E22C": "China Mobile Group Device Co.,Ltd.",
    "C0395A": "Zhejiang Dahua Technology Co., Ltd.",
    "2064CB": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F05501": "Huawei Device Co., Ltd.",
    "EC6CB5": "zte corporation",
    "C0B101": "zte corporation",
    "A45006": "SHENZHEN HUACHUANG SHIDAI TECHNOLOGYCO.,LTD",
    "AC3C8E": "Flextronics Computing(Suzhou)Co.,Ltd.",
    "406234": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "94FBA7": "IEEE Registration Authority",
    "FC3DA5": "Arcadyan Corporation",
    "C87125": "Johnson Outdoors Marine Electronics d/b/a Minnkota",
    "80CFA2": "Huawei Device Co., Ltd.",
    "0094EC": "Huawei Device Co., Ltd.",
    "74452D": "Huawei Device Co., Ltd.",
    "140152": "Samsung Electronics Co.,Ltd",
    "DC1BA1": "Intel Corporate",
    "BC33AC": "Silicon Laboratories",
    "001329": "VSST Co., LTD",
    "7CF2DD": "Vence Corp",
    "988E79": "Qudelix, Inc.",
    "C80739": "NAKAYO Inc",
    "6C6D09": "Kyowa Electronics Co.,Ltd.",
    "080342": "Palo Alto Networks",
    "A86ABB": "Sagemcom Broadband SAS",
    "2443E2": "DASAN Network Solutions",
    "EC237B": "zte corporation",
    "D49CF4": "Palo Alto Networks",
    "CCAB2C": "HUMAX Co., Ltd.",
    "540E2D": "vivo Mobile Communication Co., Ltd.",
    "708F47": "vivo Mobile Communication Co., Ltd.",
    "90173F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "607ECD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001876": "WowWee Ltd.",
    "C42456": "Palo Alto Networks",
    "84D412": "Palo Alto Networks",
    "B82FCB": "CMS Electracom",
    "4CFCAA": "Tesla,Inc.",
    "10CE45": "Miromico AG",
    "8C59DC": "ASR Microelectronics (Shanghai) Co., Ltd.",
    "18828C": "Arcadyan Corporation",
    "D452EE": "SKY UK LIMITED",
    "6C61F4": "SFR",
    "9CF029": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "2856C1": "Harman/Becker Automotive Systems GmbH",
    "BC4A56": "Cisco Systems, Inc",
    "F8AF05": "Huawei Device Co., Ltd.",
    "94E9EE": "Huawei Device Co., Ltd.",
    "64BC58": "Intel Corporate",
    "AC1203": "Intel Corporate",
    "28E34E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78B8D6": "Zebra Technologies Inc.",
    "F490CB": "IEEE Registration Authority",
    "185869": "Sailer Electronic Co., Ltd",
    "BC2DEF": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "7881CE": "China Mobile Iot Limited company",
    "445CE9": "Samsung Electronics Co.,Ltd",
    "C01692": "China Mobile Group Device Co.,Ltd.",
    "BCFF21": "Smart Code(shenzhen)Technology Co.,Ltd",
    "381730": "Ulrich Lippert GmbH & Co KG",
    "98C8B8": "vivo Mobile Communication Co., Ltd.",
    "402E71": "Texas Instruments",
    "F08175": "Sagemcom Broadband SAS",
    "1C63BF": "SHENZHEN BROADTEL  TELECOM CO.,LTD",
    "AC3651": "Jiangsu Hengtong Terahertz Technology Co., Ltd.",
    "684A76": "eero inc.",
    "8C683A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B46E08": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2864B0": "Huawei Device Co., Ltd.",
    "04F169": "Huawei Device Co., Ltd.",
    "5021EC": "Huawei Device Co., Ltd.",
    "0C48C6": "CELESTICA INC.",
    "A42985": "Sichuan AI-Link Technology Co., Ltd.",
    "B48107": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "706655": "AzureWave Technology Inc.",
    "C858C0": "Intel Corporate",
    "4CB16C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC0963": "Apple, Inc.",
    "D84C90": "Apple, Inc.",
    "24D0DF": "Apple, Inc.",
    "6C4A85": "Apple, Inc.",
    "28F033": "Apple, Inc.",
    "005E0C": "HMD Global Oy",
    "688FC9": "Zhuolian (Shenzhen) Communication Co., Ltd",
    "D84732": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A043B0": "Hangzhou BroadLink Technology Co.,Ltd",
    "D8C561": "CommFront Communications Pte Ltd",
    "60D89C": "HMD Global Oy",
    "F41C95": "BEIJING YUNYI TIMES TECHNOLOGY CO,.LTD",
    "A0687E": "ARRIS Group, Inc.",
    "A8705D": "ARRIS Group, Inc.",
    "9C9789": "1MORE",
    "F82E8E": "Nanjing Kechen Electric Co., Ltd.",
    "B4C9B9": "Sichuan AI-Link Technology Co., Ltd.",
    "F0463B": "Comcast Cable Corporation",
    "C402E1": "Khwahish Technologies Private Limited",
    "4C6C13": "IoT Company Solucoes Tecnologicas Ltda",
    "0009F3": "WELL Communication Corp.",
    "C8FA84": "Trusonus corp.",
    "2863BD": "APTIV SERVICES US, LLC",
    "F85B3B": "ASKEY COMPUTER CORP",
    "A885D7": "Sangfor Technologies Inc.",
    "786DEB": "GE Lighting",
    "9CA513": "Samsung Electronics Co.,Ltd",
    "309048": "Apple, Inc.",
    "444ADB": "Apple, Inc.",
    "8C5AC1": "Huawei Device Co., Ltd.",
    "A85AE0": "Huawei Device Co., Ltd.",
    "A4B61E": "Huawei Device Co., Ltd.",
    "C4FE5B": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "84F883": "Luminar Technologies",
    "B44C3B": "Zhejiang Dahua Technology Co., Ltd.",
    "40A2DB": "Amazon Technologies Inc.",
    "C03937": "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI",
    "C467D1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "9424B8": "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI",
    "CCF411": "Google, Inc.",
    "9C2DCF": "Shishi Tongyun Technology(Chengdu)Co.,Ltd.",
    "0433C2": "Intel Corporate",
    "D49AA0": "VNPT TECHNOLOGY",
    "C8C465": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1C4363": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94292F": "New H3C Technologies Co., Ltd",
    "6CF049": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "FCAA14": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "F80FF9": "Google, Inc.",
    "20F375": "ARRIS Group, Inc.",
    "84BB69": "ARRIS Group, Inc.",
    "444687": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "E0F442": "Huawei Device Co., Ltd.",
    "F0C42F": "Huawei Device Co., Ltd.",
    "C0B47D": "Huawei Device Co., Ltd.",
    "F4A59D": "Huawei Device Co., Ltd.",
    "307ECB": "SFR",
    "C803F5": "Ruckus Wireless",
    "E81B4B": "amnimo Inc.",
    "A4B439": "Cisco Systems, Inc",
    "A0B439": "Cisco Systems, Inc",
    "105FD4": "Tendyron Corporation",
    "B4265D": "Taicang T&W Electronics",
    "48A5E7": "Nintendo Co.,Ltd",
    "B81904": "Nokia Shanghai Bell Co., Ltd.",
    "B4A5AC": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "1C784E": "China Mobile Iot Limited company",
    "D4B709": "zte corporation",
    "38144E": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E0CCF8": "Xiaomi Communications Co Ltd",
    "AC675D": "Intel Corporate",
    "84C5A6": "Intel Corporate",
    "5C80B6": "Intel Corporate",
    "F07807": "Apple, Inc.",
    "082CB6": "Apple, Inc.",
    "F84E73": "Apple, Inc.",
    "3CCD36": "Apple, Inc.",
    "843E79": "Shenzhen Belon Technology CO.,LTD",
    "F8E4E3": "Intel Corporate",
    "D8CF89": "Beijing DoSee Science and Technology Co., Ltd.",
    "04AAE1": "BEIJING MICROVISION TECHNOLOGY CO.,LTD",
    "44DC4E": "ITEL MOBILE LIMITED",
    "B4E8C9": "XADA Technologies",
    "6C24A6": "vivo Mobile Communication Co., Ltd.",
    "9C5F5A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "A01C8D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4DEAF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60123C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C0D2DD": "Samsung Electronics Co.,Ltd",
    "942DDC": "Samsung Electronics Co.,Ltd",
    "7CB37B": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "88123D": "Suzhou Aquila Solutions Inc.",
    "48210B": "PEGATRON CORPORATION",
    "0068EB": "HP Inc.",
    "7C310E": "Cisco Systems, Inc",
    "484C86": "Huawei Device Co., Ltd.",
    "54F294": "Huawei Device Co., Ltd.",
    "245AB5": "Samsung Electronics Co.,Ltd",
    "382A19": "Technica Engineering GmbH",
    "74D654": "GINT",
    "7C210E": "Cisco Systems, Inc",
    "F4E5F2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "541310": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CE5EF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0EF86": "Google, Inc.",
    "E4C0CC": "China Mobile Group Device Co.,Ltd.",
    "A0946A": "Shenzhen XGTEC Technology Co,.Ltd.",
    "1C2AA3": "Shenzhen HongRui Optical Technology Co., Ltd.",
    "388E7A": "AUTOIT",
    "9C31C3": "SKY UK LIMITED",
    "5CB13E": "Sagemcom Broadband SAS",
    "E4AAEA": "Liteon Technology Corporation",
    "4C710D": "Cisco Systems, Inc",
    "4C710C": "Cisco Systems, Inc",
    "A4CD23": "Shenzhenshi Xinzhongxin  Co., Ltd",
    "E01F0A": "Xslent Energy Technologies. LLC",
    "E447B3": "zte corporation",
    "FCDB21": "SAMSARA NETWORKS INC",
    "E4671E": "SHEN ZHEN NUO XIN CHENG TECHNOLOGY co., Ltd.",
    "682719": "Microchip Technology Inc.",
    "24C17A": "BEIJING IACTIVE NETWORK CO.,LTD",
    "A4C939": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "34D772": "Xiamen Yudian Automation Technology Co., Ltd ",
    "3C86D1": "vivo Mobile Communication Co., Ltd.",
    "301B97": "Lierda Science & Technology Group Co.,Ltd",
    "48794D": "Samsung Electronics Co.,Ltd",
    "18F9C4": "BAE Systems",
    "60ABD2": "Bose Corporation",
    "C0DCDA": "Samsung Electronics Co.,Ltd",
    "04B429": "Samsung Electronics Co.,Ltd",
    "7C8AE1": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "C863FC": "ARRIS Group, Inc.",
    "54E4A9": "BHR Tech GmbH",
    "208058": "Ciena Corporation",
    "74DA88": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "CC32E5": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "1C3BF3": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "2C00F7": "XOS",
    "2CD2E3": "Guangzhou Aoshi Electronic Co.,Ltd",
    "946269": "ARRIS Group, Inc.",
    "A8C252": "Huawei Device Co., Ltd.",
    "A04147": "Huawei Device Co., Ltd.",
    "1459C3": "Creative Chips GmbH",
    "1CCCD6": "Xiaomi Communications Co Ltd",
    "A445CD": "IoT Diagnostics",
    "7CC926": "Wuhan GreeNet Information Service Co.,Ltd.",
    "68070A": "TPVision Europe B.V",
    "4CEBBD": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "70879E": "Beken Corporation",
    "684AAE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60D755": "HUAWEI TECHNOLOGIES CO.,LTD",
    "1CAECB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CF19E": "Groupe Atlantic",
    "04ED33": "Intel Corporate",
    "2036D7": "Shanghai Reacheng  Communication Technology Co.,Ltd",
    "044A6C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38FB14": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F0E4A2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B45062": "EmBestor Technology Inc.",
    "5C75AF": "Fitbit, Inc.",
    "94D505": "Fiberhome Telecommunication Technologies Co.,LTD",
    "7CB27D": "Intel Corporate",
    "582059": "Xiaomi Communications Co Ltd",
    "90272B": "Algorab S.r.l.",
    "4CBCB4": "ABB SpA - DIN Rail",
    "1063C8": "Liteon Technology Corporation",
    "7C5189": "SG Wireless Limited",
    "082697": "Zyxel Communications Corporation",
    "7869D4": "Shenyang Vibrotech Instruments Inc.",
    "5CD135": "Xtreme Power Systems",
    "74E1B6": "Apple, Inc.",
    "24A52C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "482759": "Levven Electronics Ltd.",
    "64CB9F": "TECNO MOBILE LIMITED",
    "4CFBFE": "Sercomm Japan Corporation",
    "C0CBF1": "Mobiwire Mobiles (NingBo) Co., LTD",
    "FC7D6C": "HYESUNG TECHWIN Co., Ltd",
    "1CD5E2": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "0024E9": "Samsung Electronics Co.,Ltd",
    "683B78": "Cisco Systems, Inc",
    "F4CE36": "Nordic Semiconductor ASA",
    "AC7713": "Honeywell Safety Products (Shanghai) Co.,Ltd",
    "08849D": "Amazon Technologies Inc.",
    "2440AE": "NIIC Technology Co., Ltd.",
    "F40E01": "Apple, Inc.",
    "1495CE": "Apple, Inc.",
    "50DE06": "Apple, Inc.",
    "CC660A": "Apple, Inc.",
    "FC1D43": "Apple, Inc.",
    "18A4A9": "Vanu Inc.",
    "80E82C": "Hewlett Packard",
    "D4ADBD": "Cisco Systems, Inc",
    "CCCCCC": "Silicon Laboratories",
    "5CA1E0": "EmbedWay Technologies",
    "1CB796": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3847BC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "549209": "HUAWEI TECHNOLOGIES CO.,LTD",
    "745909": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C942A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E47E9A": "zte corporation",
    "2C16BD": "IEEE Registration Authority",
    "30A889": "DECIMATOR DESIGN",
    "00F620": "Google, Inc.",
    "F43328": "CIMCON Lighting Inc.",
    "5C5AC7": "Cisco Systems, Inc",
    "B46077": "Sichuan Changhong Electric Ltd.",
    "B4A2EB": "IEEE Registration Authority",
    "68AB09": "Nokia",
    "00EDB8": "KYOCERA Corporation ",
    "C8D69D": "Arab International Optronics",
    "405BD8": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "B891C9": "Handreamnet",
    "C8A776": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A400E2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4C603": "Cisco Systems, Inc",
    "7CB59B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "2C4F52": "Cisco Systems, Inc",
    "2823F5": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "F010AB": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "B4DC09": "Guangzhou Dawei Communication Co.,Ltd",
    "98865D": "Nokia Shanghai Bell Co., Ltd.",
    "9C99CD": "Voippartners",
    "ACDB48": "ARRIS Group, Inc.",
    "68A03E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B8C385": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4CE9E4": "New H3C Technologies Co., Ltd",
    "C80D32": "Holoplot GmbH",
    "C4F7D5": "Cisco Systems, Inc",
    "1C6499": "Comtrend Corporation",
    "D05794": "Sagemcom Broadband SAS",
    "04D9F5": "ASUSTek COMPUTER INC.",
    "54EC2F": "Ruckus Wireless",
    "00257E": "NEW POS TECHNOLOGY LIMITED",
    "2899C7": "LINDSAY BROADBAND INC",
    "B4C4FC": "Xiaomi Communications Co Ltd",
    "FCBD67": "Arista Networks",
    "10DC4A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "487746": "Calix Inc.",
    "F8AE27": "John Deere Electronic Solutions",
    "88EF16": "ARRIS Group, Inc.",
    "8CA96F": "D&M Holdings Inc.",
    "98B8BA": "LG Electronics (Mobile Communications)",
    "7CD661": "Xiaomi Communications Co Ltd",
    "7445CE": "CRESYN",
    "B0FD0B": "IEEE Registration Authority",
    "B4CC04": "Piranti",
    "4CBC48": "Cisco Systems, Inc",
    "48D875": "China TransInfo Technology Co., Ltd",
    "3050FD": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "0CE99A": "ATLS ALTEC",
    "4C11AE": "Espressif Inc.",
    "8C89FA": "Zhejiang Hechuan Technology Co., Ltd.",
    "D4789B": "Cisco Systems, Inc",
    "483FE9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "143CC3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A8E544": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50C4DD": "BUFFALO.INC",
    "0040BC": "ALGORITHMICS LTD.",
    "004065": "GTE SPACENET",
    "1820D5": "ARRIS Group, Inc.",
    "E002A5": "ABB Robotics",
    "B8D526": "Zyxel Communications Corporation",
    "ECADE0": "D-Link International",
    "F0B968": "ITEL MOBILE LIMITED",
    "04E56E": "THUB Co., ltd.",
    "1C7F2C": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88BCC1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C426D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38D2CA": "Zhejiang Tmall Technology Co., Ltd.",
    "109E3A": "Zhejiang Tmall Technology Co., Ltd.",
    "78DA07": "Zhejiang Tmall Technology Co., Ltd.",
    "44A61E": "INGRAM MICRO SERVICES",
    "904DC3": "Flonidan A/S",
    "000DF1": "IONIX INC.",
    "00077C": "Westermo Network Technologies AB",
    "A8BF3C": "HDV Phoelectron Technology Limited",
    "D4F527": "SIEMENS AG",
    "1CBFCE": "Shenzhen Century Xinyang Technology Co., Ltd",
    "F83002": "Texas Instruments",
    "A8A159": "ASRock Incorporation",
    "202AC5": "Petite-En",
    "0836C9": "NETGEAR",
    "CCDC55": "Dragonchip Limited",
    "A4C3F0": "Intel Corporate",
    "906D05": "BXB ELECTRONICS CO., LTD",
    "D4BBC8": "vivo Mobile Communication Co., Ltd.",
    "489507": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "745BC5": "IEEE Registration Authority",
    "445D5E": "SHENZHEN Coolkit Technology CO.,LTD",
    "A041A7": "NL Ministry of Defense",
    "5462E2": "Apple, Inc.",
    "149D99": "Apple, Inc.",
    "B8B2F8": "Apple, Inc.",
    "98460A": "Apple, Inc.",
    "B85D0A": "Apple, Inc.",
    "7C9A1D": "Apple, Inc.",
    "103025": "Apple, Inc.",
    "70ACD7": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "147BAC": "Nokia",
    "E446DA": "Xiaomi Communications Co Ltd",
    "1C12B0": "Amazon Technologies Inc.",
    "4CBC98": "IEEE Registration Authority",
    "58D9C3": "Motorola Mobility LLC, a Lenovo Company",
    "2CF432": "Espressif Inc.",
    "28FFB2": "Toshiba Corp.",
    "1C60D2": "Fiberhome Telecommunication Technologies Co.,LTD",
    "F4B5AA": "zte corporation",
    "E8ACAD": "zte corporation",
    "647366": "Shenzhen Siera Technology Ltd",
    "744D28": "Routerboard.com",
    "041EFA": "BISSELL Homecare, Inc.",
    "2C73A0": "Cisco Systems, Inc",
    "443E07": "Electrolux",
    "04BA8D": "Samsung Electronics Co.,Ltd",
    "D85575": "Samsung Electronics Co.,Ltd",
    "D411A3": "Samsung Electronics Co.,Ltd",
    "90842B": "LEGO System A/S",
    "00267E": "PARROT SA",
    "E05A9F": "IEEE Registration Authority",
    "2C557C": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "F4BCDA": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "000915": "CAS Corp.",
    "8C5AF8": "Beijing Xiaomi Electronics Co., Ltd.",
    "D45800": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0017A0": "RoboTech srl",
    "000AA8": "ePipe Pty. Ltd.",
    "0029C2": "Cisco Systems, Inc",
    "187C0B": "Ruckus Wireless",
    "485D36": "Verizon ",
    "20C047": "Verizon ",
    "346B46": "Sagemcom Broadband SAS",
    "8485E6": "Guangdong Asano Technology CO.,Ltd.",
    "3C8375": "Microsoft Corporation",
    "78A7EB": "1MORE",
    "D47B35": "NEO Monitors AS",
    "000878": "Benchmark Storage Innovations",
    "34DAB7": "zte corporation",
    "109C70": "Prusa Research s.r.o.",
    "103D0A": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "942790": "TCT mobile ltd",
    "A41791": "Shenzhen Decnta Technology Co.,LTD.",
    "A41908": "Fiberhome Telecommunication Technologies Co.,LTD",
    "7C604A": "Avelon",
    "0014A5": "Gemtek Technology Co., Ltd.",
    "C0B5D7": "CHONGQING FUGUI ELECTRONICS CO.,LTD.",
    "108EBA": "Molekule",
    "80D336": "CERN",
    "64255E": "Observint Technologies, Inc.",
    "90940A": "Analog Devices, Inc",
    "40B076": "ASUSTek COMPUTER INC.",
    "E84C56": "INTERCEPT SERVICES LIMITED",
    "D4AD71": "Cisco Systems, Inc",
    "702B1D": "E-Domus International Limited",
    "F085C1": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "FC62B9": "ALPSALPINE CO,.LTD",
    "0002C7": "ALPSALPINE CO,.LTD",
    "001E3D": "ALPSALPINE CO,.LTD",
    "28A183": "ALPSALPINE CO,.LTD",
    "48F07B": "ALPSALPINE CO,.LTD",
    "4CDD7D": "LHP Telematics LLC",
    "B8BC5B": "Samsung Electronics Co.,Ltd",
    "60380E": "ALPSALPINE CO,.LTD",
    "D43D39": "Dialog Semiconductor",
    "4C218C": "Panasonic India Private limited",
    "94A40C": "Diehl Metering GmbH",
    "4C917A": "IEEE Registration Authority",
    "F48CEB": "D-Link International",
    "743A65": "NEC Corporation",
    "00255C": "NEC Corporation",
    "2C4E7D": "Chunghua Intelligent Network Equipment Inc.",
    "A4F465": "ITEL MOBILE LIMITED",
    "CC70ED": "Cisco Systems, Inc",
    "907E30": "LARS",
    "84EB3E": "Vivint Smart Home",
    "9C8275": "Yichip\u00a0Microelectronics (Hangzhou) Co.,Ltd",
    "5CCBCA": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "18BB26": "FN-LINK TECHNOLOGY LIMITED",
    "34F8E7": "Cisco Systems, Inc",
    "28E98E": "Mitsubishi Electric Corporation",
    "948FCF": "ARRIS Group, Inc.",
    "A8F5DD": "ARRIS Group, Inc.",
    "18B905": "Hong Kong Bouffalo Lab Limited",
    "ECF0FE": "zte corporation",
    "70B317": "Cisco Systems, Inc",
    "B00247": "AMPAK Technology, Inc.",
    "BCE796": "Wireless CCTV Ltd",
    "44D3AD": "Shenzhen TINNO Mobile Technology Corp.",
    "7485C4": "New H3C Technologies Co., Ltd",
    "D8CE3A": "Xiaomi Communications Co Ltd",
    "B0907E": "Cisco Systems, Inc",
    "2C7360": "Earda Technologies co Ltd",
    "D0BAE4": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "C82E47": "Suzhou SmartChip Semiconductor Co., LTD",
    "C02250": "Koss Corporation",
    "043385": "Nanchang BlackShark Co.,Ltd.",
    "E4CA12": "zte corporation",
    "D49E05": "zte corporation",
    "585FF6": "zte corporation",
    "40B30E": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "04CE7E": "NXP France Semiconductors France",
    "508CF5": "China Mobile Group Device Co.,Ltd.",
    "1C549E": "Universal Electronics, Inc.",
    "94B01F": "Apple, Inc.",
    "98CC4D": "Shenzhen mantunsci co., LTD",
    "B8C74A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "102C6B": "AMPAK Technology, Inc.",
    "2453BF": "Enernet",
    "94F6D6": "Apple, Inc.",
    "F82D7C": "Apple, Inc.",
    "C09AD0": "Apple, Inc.",
    "D092FA": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E85AD1": "Fiberhome Telecommunication Technologies Co.,LTD",
    "5076AF": "Intel Corporate",
    "6C5C3D": "IEEE Registration Authority",
    "84E5D8": "Guangdong UNIPOE IoT Technology Co.,Ltd.",
    "A8BC9C": "Cloud Light Technology Limited",
    "A89042": "Beijing Wanwei Intelligent Technology Co., Ltd.",
    "18BE92": "Delta Networks, Inc.",
    "C0132B": "Sichuan Changhong Electric Ltd.",
    "0CB4A4": "Xintai Automobile Intelligent Network Technology",
    "90633B": "Samsung Electronics Co.,Ltd",
    "FCAAB6": "Samsung Electronics Co.,Ltd",
    "782327": "Samsung Electronics Co.,Ltd",
    "DCF756": "Samsung Electronics Co.,Ltd",
    "2CA02F": "Veroguard Systems Pty Ltd",
    "90C54A": "vivo Mobile Communication Co., Ltd.",
    "BC7596": "Beijing Broadwit Technology Co., Ltd.",
    "A0B549": "Arcadyan Corporation",
    "001F5A": "Beckwith Electric Co.",
    "985D82": "Arista Networks",
    "1C34DA": "Mellanox Technologies, Inc.",
    "68A47D": "Sun Cupid Technology (HK) LTD",
    "184B0D": "Ruckus Wireless",
    "D41243": "AMPAK Technology, Inc.",
    "48A6B8": "Sonos, Inc.",
    "B87826": "Nintendo Co.,Ltd",
    "DCCBA8": "Explora Technologies Inc",
    "C07878": "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD.",
    "CCD4A1": "MitraStar Technology Corp.",
    "08BA5F": "Qingdao Hisense Electronics Co.,Ltd.",
    "10DFFC": "Siemens AG",
    "847F3D": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "944F4C": "Sound United LLC",
    "34DAC1": "SAE Technologies Development(Dongguan) Co., Ltd.",
    "705DCC": "EFM Networks",
    "A823FE": "LG Electronics",
    "1C599B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "806933": "HUAWEI TECHNOLOGIES CO.,LTD",
    "BC26C7": "Cisco Systems, Inc",
    "E05D5C": "Oy Everon Ab",
    "E046E5": "Gosuncn Technology Group Co., Ltd.",
    "688F2E": "Hitron Technologies. Inc",
    "D4BD1E": "5VT Technologies,Taiwan LTd.",
    "BC5EA1": "PsiKick, Inc.",
    "283926": "CyberTAN Technology Inc.",
    "C4FDE6": "DRTECH",
    "CC988B": "SONY Visual Products Inc.",
    "74B587": "Apple, Inc.",
    "FCB6D8": "Apple, Inc.",
    "3C6A2C": "IEEE Registration Authority",
    "30E3D6": "Spotify USA Inc.",
    "D80D17": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "7405A5": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "286DCD": "Beijing Winner Microelectronics Co.,Ltd. ",
    "541031": "SMARTO",
    "44A466": "GROUPE LDLC",
    "8CFCA0": "Shenzhen Smart Device Technology Co., LTD.",
    "1C427D": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "9CA525": "Shandong USR IOT Technology Limited",
    "E0456D": "China Mobile Group Device Co.,Ltd.",
    "18810E": "Apple, Inc.",
    "608C4A": "Apple, Inc.",
    "241B7A": "Apple, Inc.",
    "8CFE57": "Apple, Inc.",
    "C0A600": "Apple, Inc.",
    "E0C286": "Aisai Communication Technology Co., Ltd.",
    "F0B014": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "18C2BF": "BUFFALO.INC",
    "0C2A86": "Fiberhome Telecommunication Technologies Co.,LTD",
    "FC61E9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "00A0D1": "INVENTEC CORPORATION",
    "0018CC": "AXIOHM SAS",
    "001827": "NEC UNIFIED SOLUTIONS NEDERLAND B.V.",
    "009004": "3COM EUROPE LTD",
    "00068C": "3COM",
    "02608C": "3COM",
    "00D0D8": "3COM",
    "18937F": "AMPAK Technology, Inc.",
    "A43523": "Guangdong Donyan Network Technologies Co.,Ltd.",
    "B4A94F": "MERCURY CORPORATION",
    "803AF4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "48A0F8": "Fiberhome Telecommunication Technologies Co.,LTD",
    "405662": "GuoTengShengHua Electronics LTD.",
    "E4DB6D": "Beijing Xiaomi Electronics Co., Ltd.",
    "0C5331": "ETH Zurich",
    "0C9A42": "FN-LINK TECHNOLOGY LIMITED",
    "F85E3C": "SHENZHEN ZHIBOTONG ELECTRONICS CO.,LTD",
    "283E76": "Common Networks",
    "DC3979": "Cisco Systems, Inc",
    "58D56E": "D-Link International",
    "DC9088": "HUAWEI TECHNOLOGIES CO.,LTD",
    "54812D": "PAX Computer Technology(Shenzhen) Ltd.",
    "4062EA": "China Mobile Group Device Co.,Ltd.",
    "44657F": "Calix Inc.",
    "304B07": "Motorola Mobility LLC, a Lenovo Company",
    "345ABA": "tcloud intelligence",
    "502FA8": "Cisco Systems, Inc",
    "C08359": "IEEE Registration Authority",
    "8835C1": "OI ELECTRIC CO.,LTD",
    "3042A1": "ilumisys Inc. DBA Toggled",
    "0026B7": "Kingston Technology Company, Inc.",
    "000809": "Systemonic AG",
    "8C41F4": "IPmotion GmbH",
    "704F08": "Shenzhen Huisheng Information Technology Co., Ltd.",
    "4C0FC7": "Earda Technologies co Ltd",
    "9CF6DD": "IEEE Registration Authority",
    "001E80": "Icotera A/S",
    "48881E": "EthoSwitch LLC",
    "CC2119": "Samsung Electronics Co.,Ltd",
    "80A796": "Neuralink Corp.",
    "001EEC": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "F0761C": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "0004AE": "Sullair Corporation",
    "FCB10D": "Shenzhen Tian Kun Technology Co.,LTD.",
    "20F77C": "vivo Mobile Communication Co., Ltd.",
    "00451D": "Cisco Systems, Inc",
    "000393": "Apple, Inc.",
    "0000C3": "Harris Corporation",
    "A0D635": "WBS Technology",
    "3C71BF": "Espressif Inc.",
    "902BD2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "08D59D": "Sagemcom Broadband SAS",
    "B46921": "Intel Corporate",
    "14942F": "USYS CO.,LTD.",
    "EC83D5": "GIRD Systems Inc",
    "3C427E": "IEEE Registration Authority",
    "30D659": "Merging Technologies SA",
    "702AD5": "Samsung Electronics Co.,Ltd",
    "889765": "exands",
    "000BA3": "Siemens AG",
    "34800D": "Cavium Inc",
    "B44BD6": "IEEE Registration Authority",
    "D8912A": "Zyxel Communications Corporation",
    "000C8A": "Bose Corporation",
    "243A82": "IRTS",
    "880907": "MKT Systemtechnik GmbH & Co. KG",
    "40EEDD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC751D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "289E97": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001EB0": "ImesD Electronica S.L.",
    "001525": "Chamberlain Access Solutions",
    "58A48E": "PixArt Imaging Inc.",
    "60058A": "Hitachi Metals, Ltd.",
    "BC22FB": "RF Industries",
    "74B91E": "Nanjing Bestway Automation System Co., Ltd",
    "A019B2": "IEEE Registration Authority",
    "4C3FD3": "Texas Instruments",
    "8C15C7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "60FA9D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "386E88": "zte corporation",
    "641C67": "DIGIBRAS INDUSTRIA DO BRASILS/A",
    "DC9914": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B05365": "China Mobile IOT Company Limited",
    "308841": "Sichuan\u00a0AI-Link\u00a0Technology\u00a0Co.,\u00a0Ltd.",
    "44EFCF": "UGENE SOLUTION inc.",
    "0080B6": "Mercury Systems \u2013 Trusted Mission Solutions, Inc. ",
    "08512E": "Orion Diagnostica Oy",
    "98A404": "Ericsson AB",
    "00CC3F": "Universal Electronics, Inc.",
    "74EB80": "Samsung Electronics Co.,Ltd",
    "0CE0DC": "Samsung Electronics Co.,Ltd",
    "D868C3": "Samsung Electronics Co.,Ltd",
    "C493D9": "Samsung Electronics Co.,Ltd",
    "A82BB9": "Samsung Electronics Co.,Ltd",
    "ACFD93": "WEIFANG GOERTEK ELECTRONICS CO.,LTD",
    "000E8F": "Sercomm Corporation.",
    "00B8C2": "Heights Telecom T ltd",
    "B4B686": "Hewlett Packard",
    "F4BF80": "HUAWEI TECHNOLOGIES CO.,LTD",
    "304596": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C0F4E6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68572D": "Tuya Smart Inc.",
    "A0E617": "MATIS",
    "7001B5": "Cisco Systems, Inc",
    "001F49": "Manhattan TV Ltd",
    "7C2EBD": "Google, Inc.",
    "4CEDFB": "ASUSTek COMPUTER INC.",
    "D0B214": "PoeWit Inc",
    "88D652": "AMERGINT Technologies",
    "FC90FA": "Independent Technologies",
    "E4E130": "TCT mobile ltd",
    "6CAF15": "Webasto SE",
    "40F04E": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "0C2138": "Hengstler GmbH",
    "E0191D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "68D1BA": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "E46059": "Pingtek Co., Ltd.",
    "4050B5": "Shenzhen New Species Technology Co., Ltd.",
    "1C1AC0": "Apple, Inc.",
    "3078C2": "Innowireless / QUCELL Networks",
    "800588": "Ruijie Networks Co.,LTD",
    "3C15FB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC330D": "QING DAO HAIER TELECOM CO.,LTD.",
    "E0383F": "zte corporation",
    "D47226": "zte corporation",
    "0021F2": "EASY3CALL Technology Limited",
    "0015C4": "FLOVEL CO., LTD.",
    "00CFC0": "China Mobile Group Device Co.,Ltd.",
    "AC35EE": "FN-LINK TECHNOLOGY LIMITED",
    "0C2C54": "HUAWEI TECHNOLOGIES CO.,LTD",
    "881196": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E40EEE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "28D997": "Yuduan Mobile Co., Ltd.",
    "0011E6": "Scientific Atlanta",
    "88AE07": "Apple, Inc.",
    "40831D": "Apple, Inc.",
    "DCD3A2": "Apple, Inc.",
    "5C1DD9": "Apple, Inc.",
    "68FEF7": "Apple, Inc.",
    "F00E1D": "Megafone Limited",
    "24F128": "Telstra",
    "70695A": "Cisco Systems, Inc",
    "00BF77": "Cisco Systems, Inc",
    "D07714": "Motorola Mobility LLC, a Lenovo Company",
    "30B7D4": "Hitron Technologies. Inc",
    "B481BF": "Meta-Networks, LLC",
    "946AB0": "Arcadyan Corporation",
    "4818FA": "Nocsys",
    "587A6A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "E81AAC": "ORFEO SOUNDWORKS Inc.",
    "A038F8": "OURA Health Oy",
    "687924": "ELS-GmbH & Co. KG",
    "D4C19E": "Ruckus Wireless",
    "0CAE7D": "Texas Instruments",
    "304511": "Texas Instruments",
    "301F9A": "IEEE Registration Authority",
    "28FD80": "IEEE Registration Authority",
    "000758": "DragonWave Inc.",
    "D49398": "Nokia Corporation",
    "001937": "CommerceGuard AB",
    "FC7C02": "Phicomm (Shanghai) Co., Ltd.",
    "A8610A": "ARDUINO AG",
    "6097DD": "MicroSys Electronics GmbH",
    "047970": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A057E3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F8DF15": "Sunitec Enterprise Co.,Ltd",
    "B02680": "Cisco Systems, Inc",
    "F0FCC8": "ARRIS Group, Inc.",
    "D46D6D": "Intel Corporate",
    "B41C30": "zte corporation",
    "705AAC": "Samsung Electronics Co.,Ltd",
    "609813": "Shanghai Visking Digital Technology Co. LTD",
    "1CB044": "ASKEY COMPUTER CORP",
    "2C9569": "ARRIS Group, Inc.",
    "38DEAD": "Intel Corporate",
    "F4844C": "Texas Instruments",
    "4C82CF": "Dish Technologies Corp",
    "285767": "Dish Technologies Corp",
    "70169F": "EtherCAT Technology Group",
    "A039EE": "Sagemcom Broadband SAS",
    "689861": "Beacon Inc",
    "B4F2E8": "ARRIS Group, Inc.",
    "3C574F": "China Mobile Group Device Co.,Ltd.",
    "847460": "zte corporation",
    "506B4B": "Mellanox Technologies, Inc.",
    "1CA0B8": "Hon Hai Precision Industry Co., Ltd.",
    "2C61F6": "Apple, Inc.",
    "D4A33D": "Apple, Inc.",
    "044F17": "HUMAX Co., Ltd.",
    "0CA8A7": "Samsung Electronics Co.,Ltd",
    "B0672F": "Bowers & Wilkins",
    "10CD6E": "FISYS",
    "E4CB59": "Beijing Loveair Science and Technology Co. Ltd.",
    "B4E62D": "Espressif Inc.",
    "F0766F": "Apple, Inc.",
    "4098AD": "Apple, Inc.",
    "6C4D73": "Apple, Inc.",
    "D89C67": "Hon Hai Precision Ind. Co.,Ltd.",
    "685ACF": "Samsung Electronics Co.,Ltd",
    "64209F": "Tilgin AB",
    "A43E51": "ANOV FRANCE",
    "D86375": "Xiaomi Communications Co Ltd",
    "68EF43": "Apple, Inc.",
    "D02B20": "Apple, Inc.",
    "702605": "SONY Visual Products Inc.",
    "005013": "Seagate Cloud Systems Inc",
    "0090F1": "Seagate Cloud Systems Inc",
    "845A81": "ffly4u",
    "CC81DA": "Phicomm (Shanghai) Co., Ltd.",
    "00806C": "Secure Systems & Services",
    "1C27DD": "Datang Gohighsec(zhejiang)Information Technology Co.,Ltd.",
    "B8C8EB": "ITEL MOBILE LIMITED",
    "80C5F2": "AzureWave Technology Inc.",
    "64F88A": "China Mobile IOT Company Limited",
    "68DB54": "Phicomm (Shanghai) Co., Ltd.",
    "C8DF84": "Texas Instruments",
    "240D6C": "SMND",
    "B45253": "Seagate Technology",
    "0011C6": "Seagate Technology",
    "001D38": "Seagate Technology",
    "E48F34": "Vodafone Italia S.p.A.",
    "D8E004": "Vodia Networks Inc",
    "2CFDAB": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "30B4B8": "LG Electronics",
    "3873EA": "IEEE Registration Authority",
    "4C5262": "Fujitsu Technology Solutions GmbH",
    "803BF6": "LOOK EASY INTERNATIONAL LIMITED",
    "30EB1F": "Skylab M&C Technology Co.,Ltd",
    "549A4C": "GUANGDONG HOMECARE TECHNOLOGY CO.,LTD. ",
    "0CCEF6": "Guizhou Fortuneship Technology Co., Ltd",
    "1806FF": "Acer Computer(Shanghai) Limited.",
    "EC1D8B": "Cisco Systems, Inc",
    "EC7097": "ARRIS Group, Inc.",
    "5819F8": "ARRIS Group, Inc.",
    "D07FC4": "Ou Wei Technology Co.\uff0cLtd. of Shenzhen City",
    "1479F3": "China Mobile Group Device Co.,Ltd.",
    "48555C": "Wu Qi Technologies,Inc.",
    "18F0E4": "Xiaomi Communications Co Ltd",
    "380E4D": "Cisco Systems, Inc",
    "C4CD82": "Hangzhou Lowan Information Technology Co., Ltd.",
    "30FB94": "Shanghai Fangzhiwei Information Technology CO.,Ltd.",
    "A4B52E": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "9C8C6E": "Samsung Electronics Co.,Ltd",
    "DC4F22": "Espressif Inc.",
    "F86CE1": "Taicang T&W Electronics",
    "1C7328": "Connected Home",
    "08BA22": "Swaive Corporation",
    "28C13C": "Hon Hai Precision Industry Co., Ltd.",
    "0023A0": "Hana CNS Co., LTD.",
    "F406A5": "Hangzhou Bianfeng Networking Technology Co., Ltd.",
    "B0ECE1": "Private",
    "60E78A": "UNISEM",
    "64CBA3": "Pointmobile",
    "3CA581": "vivo Mobile Communication Co., Ltd.",
    "34E911": "vivo Mobile Communication Co., Ltd.",
    "1046B4": "FormericaOE",
    "9CE33F": "Apple, Inc.",
    "7867D7": "Apple, Inc.",
    "B8C111": "Apple, Inc.",
    "002424": "Ace Axis Limited",
    "50C9A0": "SKIPPER AS",
    "7483EF": "Arista Networks",
    "00E0F6": "DECISION EUROPE",
    "001248": "Dell EMC",
    "006048": "Dell EMC",
    "7CC95A": "Dell EMC",
    "00BF61": "Samsung Electronics Co.,Ltd",
    "ECFABC": "Espressif Inc.",
    "CC2DE0": "Routerboard.com",
    "389D92": "Seiko Epson Corporation",
    "A07099": "Beijing Huacan Electronics Co., Ltd",
    "DC5583": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "F8F21E": "Intel Corporate",
    "282986": "APC by Schneider Electric",
    "707DB9": "Cisco Systems, Inc",
    "08BEAC": "Edimax Technology Co. Ltd.",
    "8C0F83": "Angie Hospitality LLC",
    "D00401": "Motorola Mobility LLC, a Lenovo Company",
    "742857": "Mayfield Robotics",
    "589043": "Sagemcom Broadband SAS",
    "24E124": "Xiamen Milesight IoT Co., Ltd.",
    "DC68EB": "Nintendo Co.,Ltd",
    "E8361D": "Sense Labs, Inc.",
    "A407B6": "Samsung Electronics Co.,Ltd",
    "40498A": "Synapticon GmbH",
    "087808": "Samsung Electronics Co.,Ltd",
    "887598": "Samsung Electronics Co.,Ltd",
    "C0174D": "Samsung Electronics Co.,Ltd",
    "7091F3": "Universal Electronics, Inc.",
    "080069": "Silicon Graphics",
    "002291": "Cisco Systems, Inc",
    "10FCB6": "mirusystems CO.,LTD",
    "04D6AA": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "C89F42": "VDII Innovation AB",
    "50A83A": "S Mobile Devices Limited",
    "6405E9": "Shenzhen WayOS Technology Crop., Ltd.",
    "F4F5DB": "Xiaomi Communications Co Ltd",
    "D0666D": "Shenzhen Bus-Lan Technology Co., Ltd.",
    "08152F": "Samsung Electronics Co., Ltd. ARTIK",
    "0C5842": "DME Micro",
    "BC825D": "MITSUMI ELECTRIC CO.,LTD.",
    "CC2237": "IEEE Registration Authority",
    "48D6D5": "Google, Inc.",
    "20F19E": "ARRIS Group, Inc.",
    "38D620": "Limidea Concept Pte. Ltd.",
    "104963": "HARTING K.K.",
    "8CE38E": "Kioxia Corporation",
    "186024": "Hewlett Packard",
    "8CD48E": "ITEL MOBILE LIMITED",
    "642B8A": "ALL BEST Industrial Co., Ltd.",
    "00149D": "Sound ID Inc.",
    "A8E824": "INIM ELECTRONICS S.R.L.",
    "BC3D85": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2054FA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38378B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "745C4B": "GN Audio A/S",
    "74F91A": "Onface",
    "A434F1": "Texas Instruments",
    "EC0441": "ShenZhen TIGO Semiconductor Co., Ltd.",
    "B8EE0E": "Sagemcom Broadband SAS",
    "C0A53E": "Apple, Inc.",
    "B0350B": "MOBIWIRE MOBILES (NINGBO) CO.,LTD",
    "28A6AC": "seca gmbh & co. kg",
    "ECD09F": "Xiaomi Communications Co Ltd",
    "78E103": "Amazon Technologies Inc.",
    "78A6E1": "Brocade Communications Systems LLC",
    "E4EC10": "Nokia Corporation",
    "000659": "EAL (Apeldoorn) B.V.",
    "A8BE27": "Apple, Inc.",
    "002692": "Mitsubishi Electric Corporation",
    "00C08F": "Panasonic Electric Works Co., Ltd.",
    "6C090A": "GEMATICA SRL",
    "70E1FD": "FLEXTRONICS",
    "74E60F": "TECNO MOBILE LIMITED",
    "001AA7": "Torian Wireless",
    "444E6D": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "90B1E0": "Beijing Nebula Link Technology Co., Ltd",
    "8014A8": "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.",
    "586163": "Quantum Networks (SG) Pte. Ltd.",
    "002EC7": "HUAWEI TECHNOLOGIES CO.,LTD",
    "488EEF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CC121": "Panasonic Corporation AVC Networks Company",
    "ACBE75": "Ufine Technologies Co.,Ltd.",
    "409BCD": "D-Link International",
    "0CB459": "Marketech International Corp.",
    "94D029": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "308454": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "5C0339": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC3DFD": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "547595": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "C47154": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F82819": "Liteon Technology Corporation",
    "50E971": "Jibo, Inc.",
    "58C583": "ITEL MOBILE LIMITED",
    "18204C": "Kummler+Matter AG",
    "18D225": "Fiberhome Telecommunication Technologies Co.,LTD",
    "0840F3": "Tenda Technology Co.,Ltd.Dongguan branch",
    "94FBB2": "SHENZHEN GONGJIN ELECTRONICS CO.,LT",
    "00C05D": "L&N TECHNOLOGIES",
    "30F77F": "S Mobile Devices Limited",
    "D86C63": "Google, Inc.",
    "001E99": "Vantanol Industrial Corporation",
    "58B633": "Ruckus Wireless",
    "543D37": "Ruckus Wireless",
    "2CE6CC": "Ruckus Wireless",
    "5C5181": "Samsung Electronics Co.,Ltd",
    "608E08": "Samsung Electronics Co.,Ltd",
    "00227F": "Ruckus Wireless",
    "74911A": "Ruckus Wireless",
    "C8DB26": "Logitech",
    "A40E2B": "Facebook Inc",
    "F88A3C": "IEEE Registration Authority",
    "A40450": "nFore Technology Inc.",
    "FC5A1D": "Hitron Technologies. Inc",
    "94147A": "vivo Mobile Communication Co., Ltd.",
    "18B430": "Nest Labs Inc.",
    "30B164": "Power Electronics International Inc.",
    "9828A6": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "001B17": "Palo Alto Networks",
    "58493B": "Palo Alto Networks",
    "786D94": "Palo Alto Networks",
    "F430B9": "Hewlett Packard",
    "14612F": "Avaya Inc",
    "00309D": "Nimble Microsystems, Inc.",
    "8C210A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "4C189A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "CC4B73": "AMPAK Technology, Inc.",
    "681DEF": "Shenzhen CYX Technology Co., Ltd.",
    "AC203E": "Wuhan Tianyu Information Industry Co., Ltd.",
    "B01F29": "Helvetia INC.",
    "54C9DF": "FN-LINK TECHNOLOGY LIMITED",
    "A47886": "Avaya Inc",
    "0403D6": "Nintendo Co.,Ltd",
    "AC4E2E": "Shenzhen JingHanDa Electronics Co.Ltd",
    "4C910C": " Lanix Internacional, S.A. de C.V.",
    "0015DC": "KT&C Co., Ltd.",
    "00187D": "Armorlink Co .Ltd",
    "880F10": "Huami Information Technology Co.,Ltd.",
    "5C1A6F": "Cambridge Industries(Group) Co.,Ltd.",
    "3C4CD0": "CERAGON NETWORKS",
    "F40E83": "ARRIS Group, Inc.",
    "98F7D7": "ARRIS Group, Inc.",
    "206BE7": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "182CB4": "Nectarsoft Co., Ltd.",
    "2C3AE8": "Espressif Inc.",
    "88BD78": "Flaircomm Microelectronics,Inc.",
    "58C5CB": "Samsung Electronics Co.,Ltd",
    "B4BFF6": "Samsung Electronics Co.,Ltd",
    "EC363F": "Markov Corporation",
    "5804CB": "Tianjin Huisun Technology Co.,Ltd.",
    "B8FFB3": "MitraStar Technology Corp.",
    "982DBA": "Fibergate Inc.",
    "E0D55E": "GIGA-BYTE TECHNOLOGY CO.,LTD.",
    "A040A0": "NETGEAR",
    "00A38E": "Cisco Systems, Inc",
    "A0C9A0": "Murata Manufacturing Co., Ltd.",
    "74F61C": "HTC Corporation",
    "84C0EF": "Samsung Electronics Co.,Ltd",
    "000D2B": "Racal Instruments",
    "004066": "APRESIA Systems Ltd",
    "48A74E": "zte corporation",
    "BC8AE8": "QING DAO HAIER TELECOM CO.,LTD.",
    "F4DE0C": "ESPOD Ltd.",
    "28C63F": "Intel Corporate",
    "88CC45": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "CC90E8": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "3C5282": "Hewlett Packard",
    "08ED02": "IEEE Registration Authority",
    "84A9C4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0086F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D06F82": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0F479": "HUAWEI TECHNOLOGIES CO.,LTD",
    "844765": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C4FF1F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7C4F7D": "Sawwave",
    "9CAC6D": "Universal Electronics, Inc.",
    "600837": "ivvi Scientific(Nanchang)Co.Ltd",
    "E8FDE8": "CeLa Link Corporation",
    "C40BCB": "Xiaomi Communications Co Ltd",
    "60D7E3": "IEEE Registration Authority",
    "1893D7": "Texas Instruments",
    "A8B86E": "LG Electronics (Mobile Communications)",
    "84AFEC": "BUFFALO.INC",
    "7C7B8B": "Control Concepts, Inc.",
    "B83765": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "345BBB": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "34CE00": "XIAOMI Electronics,CO.,LTD",
    "D0498B": "ZOOM SERVER",
    "0827CE": "NAGANO KEIKI CO., LTD.",
    "00219E": "Sony Corporation",
    "ACB57D": "Liteon Technology Corporation",
    "D4619D": "Apple, Inc.",
    "98DDEA": "Infinix mobility limited",
    "001D44": "Krohne",
    "A8A198": "TCT mobile ltd",
    "08EA40": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "00D095": "Alcatel-Lucent Enterprise",
    "0020DA": "Alcatel-Lucent Enterprise",
    "A0C4A5": "SYGN HOUSE INC.",
    "6C5976": "Shanghai Tricheer Technology Co.,Ltd.",
    "14BD61": "Apple, Inc.",
    "5CC6E9": "Edifier International",
    "EC4F82": "Calix Inc.",
    "002CC8": "Cisco Systems, Inc",
    "C0028D": "WINSTAR Display CO.,Ltd",
    "E89FEC": "CHENGDU KT ELECTRONIC HI-TECH CO.,LTD",
    "802689": "D-Link International",
    "F8AB05": "Sagemcom Broadband SAS",
    "503A7D": "AlphaTech PLC Int\u2019l Co., Ltd.",
    "F4C4D6": "Shenzhen Xinfa Electronic Co.,ltd",
    "7C5049": "Apple, Inc.",
    "E47DEB": "Shanghai Notion Information Technology CO.,LTD.",
    "D8325A": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "9CDA3E": "Intel Corporate",
    "283F69": "Sony Corporation",
    "D83214": "Tenda Technology Co.,Ltd.Dongguan branch",
    "10954B": "Megabyte Ltd.",
    "C4B9CD": "Cisco Systems, Inc",
    "E0C0D1": "CK Telecom (Shenzhen) Limited",
    "C0D3C0": "Samsung Electronics Co.,Ltd",
    "948BC1": "Samsung Electronics Co.,Ltd",
    "14568E": "Samsung Electronics Co.,Ltd",
    "6837E9": "Amazon Technologies Inc.",
    "2CA17D": "ARRIS Group, Inc.",
    "001081": "DPS, INC.",
    "40F385": "IEEE Registration Authority",
    "887873": "Intel Corporate",
    "F87588": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F44C7F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64B473": "Xiaomi Communications Co Ltd",
    "7451BA": "Xiaomi Communications Co Ltd",
    "6CB4A7": "Landauer, Inc.",
    "F8A5C5": "Cisco Systems, Inc",
    "7C5A1C": "Sophos Ltd",
    "B0F1EC": "AMPAK Technology, Inc.",
    "542B57": "Night Owl SP",
    "7802F8": "Xiaomi Communications Co Ltd",
    "00238A": "Ciena Corporation",
    "080027": "PCS Systemtechnik GmbH",
    "2C4D54": "ASUSTek COMPUTER INC.",
    "349672": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "702084": "Hon Hai Precision Industry Co., Ltd.",
    "9C6650": "Glodio Technolies Co.,Ltd Tianjin Branch",
    "D461FE": "Hangzhou H3C Technologies Co., Limited",
    "501E2D": "StreamUnlimited Engineering GmbH",
    "A0A33B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "EC01EE": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "6049C1": "Avaya Inc",
    "E45D51": "SFR",
    "7C67A2": "Intel Corporate",
    "00E05A": "GALEA NETWORK SECURITY",
    "F06E32": "MICROTEL INNOVATION S.R.L.",
    "C43018": "MCS Logic Inc.",
    "6831FE": "Teladin Co.,Ltd.",
    "2816AD": "Intel Corporate",
    "D0FF98": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94652D": "OnePlus Technology (Shenzhen) Co., Ltd",
    "0060D3": "AT&T",
    "848DC7": "Cisco SPVTG",
    "045D4B": "Sony Corporation",
    "78AF58": "GIMASI SA",
    "1CB857": "Becon Technologies Co,.Ltd.",
    "48A380": "Gionee Communication Equipment Co.,Ltd.",
    "682737": "Samsung Electronics Co.,Ltd",
    "001992": "Adtran Inc",
    "3CF862": "Intel Corporate",
    "B0E5ED": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C486E9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "344B3D": "Fiberhome Telecommunication Technologies Co.,LTD",
    "8C9351": "Jigowatts Inc.",
    "D838FC": "Ruckus Wireless",
    "2C0BE9": "Cisco Systems, Inc",
    "90505A": "unGlue, Inc",
    "3478D7": "Gionee Communication Equipment Co.,Ltd.",
    "5CCCA0": "Gridwiz Inc.",
    "20AB37": "Apple, Inc.",
    "60F445": "Apple, Inc.",
    "E0A700": "Verkada Inc",
    "58404E": "Apple, Inc.",
    "D0C5F3": "Apple, Inc.",
    "BC9FEF": "Apple, Inc.",
    "5800E3": "Liteon Technology Corporation",
    "000AE4": "Wistron Corporation",
    "A408F5": "Sagemcom Broadband SAS",
    "00B091": "Transmeta Corp.",
    "ACC662": "MitraStar Technology Corp.",
    "886B44": "Sunnovo International Limited",
    "FC3CE9": "Tsingtong Technologies Co, Ltd.",
    "A4580F": "IEEE Registration Authority",
    "48F97C": "Fiberhome Telecommunication Technologies Co.,LTD",
    "C0BFC0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A08CF8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "101331": "Technicolor Delivery Technologies Belgium NV",
    "4CB81C": "SAM Electronics GmbH",
    "44D244": "Seiko Epson Corporation",
    "7CF95C": "U.I. Lapp GmbH",
    "C8F733": "Intel Corporate",
    "A03D6F": "Cisco Systems, Inc",
    "A0E0AF": "Cisco Systems, Inc",
    "A4E6B1": "Shanghai Joindata Technology Co.,Ltd.",
    "C09C04": "Shaanxi GuoLian Digital TV Technology Co.,Ltd.",
    "ACD657": "Shaanxi GuoLian Digital TV Technology Co.,Ltd.",
    "3CEF8C": "Zhejiang Dahua Technology Co., Ltd.",
    "8C2FA6": "Solid Optics B.V.",
    "8C192D": "IEEE Registration Authority",
    "00ACE0": "ARRIS Group, Inc.",
    "007532": "INID BV",
    "6473E2": "Arbiter Systems, Inc.",
    "E45D52": "Avaya Inc",
    "94E979": "Liteon Technology Corporation",
    "B49691": "Intel Corporate",
    "007686": "Cisco Systems, Inc",
    "D0608C": "zte corporation",
    "7C03C9": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "003048": "Super Micro Computer, Inc.",
    "9CD9CB": "Lesira Manufacturing Pty Ltd",
    "B8E937": "Sonos, Inc.",
    "AC233F": "Shenzhen Minew Technologies Co., Ltd.",
    "3C80AA": "Ransnet Singapore Pte Ltd",
    "88C626": "Logitech, Inc",
    "0023F7": "Private",
    "D4C8B0": "Prime Electronics & Satellitics Inc.",
    "0481AE": "Clack Corporation",
    "9C13AB": "Chanson Water Co., Ltd.",
    "703D15": "Hangzhou H3C Technologies Co., Limited",
    "E49E12": "FREEBOX SAS",
    "18F292": "Shannon Systems",
    "8CEA1B": "Edgecore Networks Corporation",
    "2420C7": "Sagemcom Broadband SAS",
    "446AB7": "ARRIS Group, Inc.",
    "701CE7": "Intel Corporate",
    "9C2A70": "Hon Hai Precision Ind. Co.,Ltd.",
    "58E16C": "Ying Hua Information Technology (Shanghai)Co., LTD",
    "C82158": "Intel Corporate",
    "98E476": "Zentan",
    "14A51A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "047503": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50B363": "Digitron da Amazonia S/A",
    "D4E90B": "CVT CO.,LTD",
    "CCB0DA": "Liteon Technology Corporation",
    "A4D9A4": "neXus ID Solutions AB",
    "E02CF3": "MRS Electronic GmbH",
    "2C0E3D": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "487A55": "ALE International",
    "001EAE": "Continental Automotive Systems Inc.",
    "2CBABA": "Samsung Electronics Co.,Ltd",
    "40D3AE": "Samsung Electronics Co.,Ltd",
    "2CDD95": "Taicang T&W Electronics",
    "88E87F": "Apple, Inc.",
    "9CF48E": "Apple, Inc.",
    "5CF7E6": "Apple, Inc.",
    "B853AC": "Apple, Inc.",
    "203CAE": "Apple, Inc.",
    "A03BE3": "Apple, Inc.",
    "4C3275": "Apple, Inc.",
    "9C7DA3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4C64F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00D78F": "Cisco Systems, Inc",
    "107223": "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO ",
    "D03DC3": "AQ Corporation",
    "EC01E2": "FOXCONN INTERCONNECT TECHNOLOGY",
    "E0686D": "Raybased AB",
    "CCFD17": "TCT mobile ltd",
    "502B73": "Tenda Technology Co.,Ltd.Dongguan branch",
    "B4E782": "Vivalnk",
    "4409B8": "Salcomp (Shenzhen) CO., LTD.",
    "04BA36": "Li Seng Technology Ltd",
    "B4F81E": "Kinova",
    "001628": "Magicard Ltd",
    "702E22": "zte corporation",
    "3816D1": "Samsung Electronics Co.,Ltd",
    "D0176A": "Samsung Electronics Co.,Ltd",
    "D48890": "Samsung Electronics Co.,Ltd",
    "5492BE": "Samsung Electronics Co.,Ltd",
    "1861C7": "lemonbeat GmbH",
    "205D47": "vivo Mobile Communication Co., Ltd.",
    "10C60C": "Domino UK Ltd",
    "043110": "Inspur Group Co., Ltd.",
    "949AA9": "Microsoft Corporation",
    "ACAB2E": "Beijing LasNubes Technology Co., Ltd.",
    "000678": "D&M Holdings Inc.",
    "884CCF": "Pulzze Systems, Inc",
    "E0286D": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "981888": "Cisco Meraki",
    "4CC8A1": "Cisco Meraki",
    "C8B21E": "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.",
    "600B03": "Hangzhou H3C Technologies Co., Limited",
    "A0AB1B": "D-Link International",
    "D842E2": "Canary Connect, Inc.",
    "C8E776": "PTCOM Technology",
    "302303": "Belkin International Inc.",
    "AC17C8": "Cisco Meraki",
    "E41218": "ShenZhen Rapoo Technology Co., Ltd.",
    "001984": "ESTIC Corporation",
    "00212F": "Phoebe Micro Inc.",
    "3859F9": "Hon Hai Precision Ind. Co.,Ltd.",
    "EC55F9": "Hon Hai Precision Ind. Co.,Ltd.",
    "001A8A": "Samsung Electronics Co.,Ltd",
    "3C5A37": "Samsung Electronics Co.,Ltd",
    "F49F54": "Samsung Electronics Co.,Ltd",
    "34C3AC": "Samsung Electronics Co.,Ltd",
    "44F459": "Samsung Electronics Co.,Ltd",
    "00265D": "Samsung Electronics Co.,Ltd",
    "002567": "Samsung Electronics Co.,Ltd",
    "BCB1F3": "Samsung Electronics Co.,Ltd",
    "5C0A5B": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "8056F2": "Hon Hai Precision Ind. Co.,Ltd.",
    "000278": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "002399": "Samsung Electronics Co.,Ltd",
    "7CF854": "Samsung Electronics Co.,Ltd",
    "7CE9D3": "Hon Hai Precision Ind. Co.,Ltd.",
    "1C3E84": "Hon Hai Precision Ind. Co.,Ltd.",
    "B8763F": "Hon Hai Precision Ind. Co.,Ltd.",
    "60F494": "Hon Hai Precision Ind. Co.,Ltd.",
    "906EBB": "Hon Hai Precision Ind. Co.,Ltd.",
    "18F46A": "Hon Hai Precision Ind. Co.,Ltd.",
    "4C0F6E": "Hon Hai Precision Ind. Co.,Ltd.",
    "78E400": "Hon Hai Precision Ind. Co.,Ltd.",
    "C4731E": "Samsung Electronics Co.,Ltd",
    "001C43": "Samsung Electronics Co.,Ltd",
    "0023D6": "Samsung Electronics Co.,Ltd",
    "001B98": "Samsung Electronics Co.,Ltd",
    "44D6E1": "Snuza International Pty. Ltd.",
    "486DBB": "Vestel Elektronik San ve Tic. A.S.",
    "002A10": "Cisco Systems, Inc",
    "00A289": "Cisco Systems, Inc",
    "44E9DD": "Sagemcom Broadband SAS",
    "000F5E": "Veo",
    "001328": "Westech Korea Inc.,",
    "B8BF83": "Intel Corporate",
    "84E0F4": "IEEE Registration Authority",
    "D83062": "Apple, Inc.",
    "7C79E8": "PayRange Inc.",
    "A43111": "ZIV",
    "008073": "DWB ASSOCIATES",
    "80A1D7": "Shanghai DareGlobal Technologies Co.,Ltd",
    "EC1F72": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "8C0D76": "HUAWEI TECHNOLOGIES CO.,LTD",
    "84BE52": "HUAWEI TECHNOLOGIES CO.,LTD",
    "849FB5": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A4CAA0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "14F42A": "Samsung Electronics Co.,Ltd",
    "0808C2": "Samsung Electronics Co.,Ltd",
    "CCFE3C": "Samsung Electronics Co.,Ltd",
    "28BAB5": "Samsung Electronics Co.,Ltd",
    "E440E2": "Samsung Electronics Co.,Ltd",
    "103B59": "Samsung Electronics Co.,Ltd",
    "D890E8": "Samsung Electronics Co.,Ltd",
    "E4121D": "Samsung Electronics Co.,Ltd",
    "E8508B": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "F8042E": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "1C62B8": "Samsung Electronics Co.,Ltd",
    "CCF9E8": "Samsung Electronics Co.,Ltd",
    "D857EF": "Samsung Electronics Co.,Ltd",
    "18E2C2": "Samsung Electronics Co.,Ltd",
    "9852B1": "Samsung Electronics Co.,Ltd",
    "C462EA": "Samsung Electronics Co.,Ltd",
    "182666": "Samsung Electronics Co.,Ltd",
    "30D6C9": "Samsung Electronics Co.,Ltd",
    "CC07AB": "Samsung Electronics Co.,Ltd",
    "B43A28": "Samsung Electronics Co.,Ltd",
    "78A873": "Samsung Electronics Co.,Ltd",
    "BC8AA3": "NHN Entertainment",
    "04BBF9": "Pavilion Data Systems Inc",
    "58FB84": "Intel Corporate",
    "8C6102": "Beijing Baofengmojing Technologies Co., Ltd",
    "548CA0": "Liteon Technology Corporation",
    "345760": "MitraStar Technology Corp.",
    "C0D391": "IEEE Registration Authority",
    "D49B5C": "Chongqing Miedu Technology Co., Ltd.",
    "00C017": "NetAlly",
    "5CB066": "ARRIS Group, Inc.",
    "002261": "Frontier Silicon Ltd",
    "A0C562": "ARRIS Group, Inc.",
    "8496D8": "ARRIS Group, Inc.",
    "0026D9": "ARRIS Group, Inc.",
    "E8EB11": "Texas Instruments",
    "00D037": "ARRIS Group, Inc.",
    "84E058": "ARRIS Group, Inc.",
    "707630": "ARRIS Group, Inc.",
    "D0E54D": "ARRIS Group, Inc.",
    "3C6FEA": "Panasonic India Pvt. Ltd.",
    "001988": "Wi2Wi, Inc",
    "18DC56": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "0016F2": "Dmobile System Co., Ltd.",
    "44BFE3": "Shenzhen Longtech Electronics Co.,Ltd",
    "34074F": "AccelStor, Inc.",
    "B4A984": "Symantec Corporation",
    "00E011": "UNIDEN CORPORATION",
    "002555": "Visonic Technologies 1993 Ltd.",
    "58986F": "Revolution Display",
    "C81FBE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "203DB2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48D539": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C88D83": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B0B28F": "Sagemcom Broadband SAS",
    "441441": "AudioControl Inc.",
    "001F9A": "Nortel Networks",
    "000A0E": "Invivo Research Inc.",
    "000438": "Nortel Networks",
    "000EC0": "Nortel Networks",
    "3C404F": "GUANGDONG PISEN ELECTRONICS CO.,LTD",
    "D84FB8": "LG ELECTRONICS",
    "000AEB": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "60EE5C": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "6488FF": "Sichuan Changhong Electric Ltd.",
    "C4084A": "Nokia",
    "000801": "HighSpeed Surfing Inc.",
    "000772": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "E03005": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "001E7E": "Nortel Networks",
    "001365": "Nortel Networks",
    "002162": "Nortel Networks",
    "02E6D3": "NIXDORF COMPUTER CORP.",
    "0CA402": "Alcatel-Lucent IPD",
    "A0F3E4": "Alcatel-Lucent IPD",
    "84DBFC": "Nokia",
    "0016B9": "ProCurve Networking by HP",
    "001660": "Nortel Networks",
    "00AA70": "LG Electronics (Mobile Communications)",
    "F895C7": "LG Electronics (Mobile Communications)",
    "84D931": "Hangzhou H3C Technologies Co., Limited",
    "7CFC3C": "Visteon Corporation",
    "4888CA": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "385610": "CANDY HOUSE, Inc.",
    "00A742": "Cisco Systems, Inc",
    "2C3731": "SHENZHEN YIFANG DIGITAL TECHNOLOGY CO.,LTD.",
    "00116E": "Peplink International Ltd.",
    "540955": "zte corporation",
    "C041F6": "LG ELECTRONICS INC",
    "001E75": "LG Electronics (Mobile Communications)",
    "001C62": "LG Electronics (Mobile Communications)",
    "505527": "LG Electronics (Mobile Communications)",
    "88C9D0": "LG Electronics (Mobile Communications)",
    "8C3AE3": "LG Electronics (Mobile Communications)",
    "D0052A": "Arcadyan Corporation",
    "EC6881": "Palo Alto Networks",
    "E4509A": "HW Communications Ltd",
    "702900": "Shenzhen ChipTrip Technology Co,Ltd",
    "ECAAA0": "PEGATRON CORPORATION",
    "60E3AC": "LG Electronics (Mobile Communications)",
    "90F052": "MEIZU Technology Co., Ltd.",
    "90A46A": "SISNET CO., LTD",
    "14E7C8": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "ECCD6D": "Allied Telesis, Inc.",
    "981E0F": "Jeelan (Shanghai Jeelan Technology Information Inc",
    "9CA3A9": "Guangzhou Juan Optical and Electronical Tech Joint Stock Co., Ltd",
    "7CC709": "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD.",
    "18339D": "Cisco Systems, Inc",
    "146102": "Alps Alpine",
    "54276C": "Jiangsu Houge Technology Corp.",
    "0821EF": "Samsung Electronics Co.,Ltd",
    "34145F": "Samsung Electronics Co.,Ltd",
    "A03E6B": "IEEE Registration Authority",
    "9802D8": "IEEE Registration Authority",
    "64FB81": "IEEE Registration Authority",
    "2C265F": "IEEE Registration Authority",
    "8CFDF0": "Qualcomm Inc.",
    "C4BB4C": "Zebra Information Tech Co. Ltd",
    "98CF53": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "D4A148": "HUAWEI TECHNOLOGIES CO.,LTD",
    "4C0BBE": "Microsoft",
    "0C2576": "LONGCHEER TELECOMMUNICATION LIMITED",
    "D8D43C": "Sony Corporation",
    "486B2C": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "6C25B9": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "2C282D": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "4813F3": "BBK EDUCATIONAL ELECTRONICS CORP.,LTD.",
    "00409F": "Telco Systems, Inc. ",
    "00001F": "Telco Systems, Inc. ",
    "00A012": "Telco Systems, Inc. ",
    "F0DEF1": "Wistron Infocomm (Zhongshan) Corporation",
    "F80F41": "Wistron Infocomm (Zhongshan) Corporation",
    "3C970E": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "30144A": "Wistron Neweb Corporation",
    "D065CA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8CEBC6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B08900": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00E0DD": "Zenith Electronics Corporation",
    "50CE75": "Measy Electronics Co., Ltd.",
    "001639": "Ubiquam Co., Ltd.",
    "001597": "AETA AUDIO SYSTEMS",
    "002397": "Westell Technologies Inc.",
    "00045B": "Techsan Electronics Co., Ltd.",
    "0007BA": "UTStarcom Inc",
    "90A210": "United Telecoms Ltd",
    "6C0B84": "Universal Global Scientific Industrial Co., Ltd.",
    "78CB68": "DAEHAP HYPER-TECH",
    "000B0E": "Trapeze Networks",
    "000C29": "VMware, Inc.",
    "000569": "VMware, Inc.",
    "E4029B": "Intel Corporate",
    "DC1AC5": "vivo Mobile Communication Co., Ltd.",
    "B456B9": "Teraspek Technologies Co.,Ltd",
    "9CDD1F": "Intelligent Steward Co.,Ltd",
    "3C6816": "VXi Corporation",
    "E811CA": "SHANDONG KAER ELECTRIC.CO.,LTD",
    "00C000": "LANOPTICS, LTD.",
    "845181": "Samsung Electronics Co.,Ltd",
    "A8FCB7": "Consolidated Resource Imaging",
    "C816BD": "Qingdao Hisense Communications Co.,Ltd.",
    "00EBD5": "Cisco Systems, Inc",
    "34ED0B": " Shanghai XZ-COM.CO.,Ltd.",
    "90EF68": "Zyxel Communications Corporation",
    "F45EAB": "Texas Instruments",
    "C4A366": "zte corporation",
    "6073BC": "zte corporation",
    "18B169": "Sonicwall",
    "444450": "OttoQ",
    "50F5DA": "Amazon Technologies Inc.",
    "101212": "Vivo International Corporation Pty Ltd",
    "F03EBF": "GOGORO TAIWAN LIMITED",
    "C48F07": "Shenzhen Yihao Hulian Science and Technology Co., Ltd.",
    "DC7834": "LOGICOM SA",
    "D0577B": "Intel Corporate",
    "6C9522": "Scalys",
    "A8BB50": "WiZ IoT Company Limited",
    "3C92DC": "Octopod Technology Co. Ltd.",
    "08C021": "HUAWEI TECHNOLOGIES CO.,LTD",
    "600810": "HUAWEI TECHNOLOGIES CO.,LTD",
    "48435A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "78FFCA": "TECNO MOBILE LIMITED",
    "046565": "Testop",
    "7C3548": "Transcend Information",
    "78009E": "Samsung Electronics Co.,Ltd",
    "ACC33A": "Samsung Electronics Co.,Ltd",
    "54F201": "Samsung Electronics Co.,Ltd",
    "70288B": "Samsung Electronics Co.,Ltd",
    "348A7B": "Samsung Electronics Co.,Ltd",
    "1000FD": "LaonPeople",
    "C47C8D": "IEEE Registration Authority",
    "848319": "Hangzhou Zero Zero Technology Co., Ltd.",
    "7CB0C2": "Intel Corporate",
    "A81559": "Breathometer, Inc.",
    "C4CAD9": "Hangzhou H3C Technologies Co., Limited",
    "5866BA": "Hangzhou H3C Technologies Co., Limited",
    "70BAEF": "Hangzhou H3C Technologies Co., Limited",
    "586AB1": "Hangzhou H3C Technologies Co., Limited",
    "009006": "Hamamatsu Photonics K.K.",
    "001AF4": "Handreamnet",
    "E00EDA": "Cisco Systems, Inc",
    "4C0B3A": "TCT mobile ltd",
    "E42D02": "TCT mobile ltd",
    "0CBD51": "TCT mobile ltd",
    "745C9F": "TCT mobile ltd",
    "8C99E6": "TCT mobile ltd",
    "449F7F": "DataCore Software Corporation",
    "001793": "Tigi Corporation",
    "000358": "Hanyang Digitech Co.Ltd",
    "8C8EF2": "Apple, Inc.",
    "90B0ED": "Apple, Inc.",
    "04D3CF": "Apple, Inc.",
    "4882F2": "Appel Elektronik GmbH",
    "E0C79D": "Texas Instruments",
    "00177D": "IDT Technology Limited",
    "00A045": "PHOENIX CONTACT Electronics GmbH",
    "4000E0": "Derek(Shaoguan)Limited",
    "FCBC9C": "Vimar Spa",
    "E80959": "Guoguang Electric Co.,Ltd",
    "F0407B": "Fiberhome Telecommunication Technologies Co.,LTD",
    "94885E": "Surfilter Network Technology Co., Ltd. ",
    "945089": "SimonsVoss Technologies GmbH",
    "0024F4": "Kaminario, Ltd.",
    "001A29": "Johnson Outdoors Marine Electronics d/b/a Minnkota",
    "0090AE": "ITALTEL S.p.A/RF-UP-I",
    "042AE2": "Cisco Systems, Inc",
    "E0B6F5": "IEEE Registration Authority",
    "D0A4B1": "Sonifex Ltd.",
    "50DD4F": "Automation Components, Inc",
    "341FE4": "ARRIS Group, Inc.",
    "F49EEF": "Taicang T&W Electronics",
    "002378": "GN Netcom A/S",
    "50C971": "GN Netcom A/S",
    "C4F081": "HUAWEI TECHNOLOGIES CO.,LTD",
    "801382": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94FE22": "HUAWEI TECHNOLOGIES CO.,LTD",
    "00409C": "TRANSWARE",
    "608334": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E47E66": "HUAWEI TECHNOLOGIES CO.,LTD",
    "94DBDA": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B01BD2": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "54489C": "CDOUBLES ELECTRONICS CO. LTD.",
    "1C3ADE": "Samsung Electronics Co.,Ltd",
    "002360": "Lookit Technology Co., Ltd",
    "986B3D": "ARRIS Group, Inc.",
    "F462D0": "Not for Radio, LLC",
    "84002D": "PEGATRON CORPORATION",
    "408256": "Continental Automotive GmbH",
    "E4A1E6": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "1CABC0": "Hitron Technologies. Inc",
    "C825E1": "Lemobile Information Technology (Beijing) Co., Ltd",
    "54D9E4": "BRILLIANTTS CO., LTD",
    "0090FA": "Emulex Corporation",
    "00E0D5": "Emulex Corporation",
    "84FEDC": "Borqs Beijing Ltd.",
    "001035": "Elitegroup Computer Systems Co.,Ltd.",
    "000AE6": "Elitegroup Computer Systems Co.,Ltd.",
    "7427EA": "Elitegroup Computer Systems Co.,Ltd.",
    "649968": "Elentec",
    "98DED0": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "005BA1": "shanghai huayuan chuangxin software CO., LTD.",
    "58D67A": "TCPlink",
    "5CC7D7": "AZROAD TECHNOLOGY COMPANY LIMITED",
    "509EA7": "Samsung Electronics Co.,Ltd",
    "A88195": "Samsung Electronics Co.,Ltd",
    "88ADD2": "Samsung Electronics Co.,Ltd",
    "A4BF01": "Intel Corporate",
    "10DA43": "NETGEAR",
    "B805AB": "zte corporation",
    "789682": "zte corporation",
    "C4BED4": "Avaya Inc",
    "D017C2": "ASUSTek COMPUTER INC.",
    "98072D": "Texas Instruments",
    "F0C77F": "Texas Instruments",
    "D467E7": "Fiberhome Telecommunication Technologies Co.,LTD",
    "E42F26": "Fiberhome Telecommunication Technologies Co.,LTD",
    "04C1B9": "Fiberhome Telecommunication Technologies Co.,LTD",
    "349971": "Quanta Storage Inc.",
    "9C52F8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5CF286": "IEEE Registration Authority",
    "E8FD72": "SHANGHAI LINGUO TECHNOLOGY CO., LTD.",
    "98BB1E": "BYD Precision Manufacture Company Ltd.",
    "04C103": "Clover Network, Inc.",
    "280C28": "Unigen DataStorage Corporation",
    "00CCFC": "Cisco Systems, Inc",
    "08BE77": "Green Electronics",
    "AC5F3E": "SAMSUNG ELECTRO-MECHANICS(THAILAND)",
    "546D52": "TOPVIEW OPTRONICS CORP.",
    "545AA6": "Espressif Inc.",
    "FC1A11": "vivo Mobile Communication Co., Ltd.",
    "38FDFE": "IEEE Registration Authority",
    "2C09CB": "COBS AB",
    "208B37": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "002340": "MiXTelematics",
    "B48B19": "Apple, Inc.",
    "BCEC5D": "Apple, Inc.",
    "28A02B": "Apple, Inc.",
    "0019C5": "Sony Interactive Entertainment Inc.",
    "001315": "Sony Interactive Entertainment Inc.",
    "001E1E": "Honeywell Life Safety",
    "A0C589": "Intel Corporate",
    "1C234F": "EDMI  Europe Ltd",
    "A444D1": " Wingtech Group (HongKong\uff09Limited",
    "006CFD": "Sichuan Changhong Electric Ltd.",
    "907282": "Sagemcom Broadband SAS",
    "38B8EB": "IEEE Registration Authority",
    "9897D1": "MitraStar Technology Corp.",
    "B83241": "Wuhan Tianyu Information Industry Co., Ltd.",
    "0060DC": "NEC Magnus Communications,Ltd.",
    "0023B4": "Nokia Danmark A/S",
    "002548": "Nokia Danmark A/S",
    "0022FC": "Nokia Danmark A/S",
    "0022FD": "Nokia Danmark A/S",
    "0021AA": "Nokia Danmark A/S",
    "001D6E": "Nokia Danmark A/S",
    "001370": "Nokia Danmark A/S",
    "9C1874": "Nokia Danmark A/S",
    "001BAF": "Nokia Danmark A/S",
    "001C35": "Nokia Danmark A/S",
    "001C9A": "Nokia Danmark A/S",
    "001CD6": "Nokia Danmark A/S",
    "001CD4": "Nokia Danmark A/S",
    "001D98": "Nokia Danmark A/S",
    "001DE9": "Nokia Danmark A/S",
    "001E3A": "Nokia Danmark A/S",
    "C8D10B": "Nokia Corporation",
    "A84481": "Nokia Corporation",
    "8844F6": "Nokia Corporation",
    "F44D17": "GOLDCARD HIGH-TECH CO.,LTD.",
    "0030FF": "DataFab Systems Inc.",
    "000FF6": "DARFON LIGHTING CORP",
    "CC6DA0": "Roku, Inc.",
    "0016E4": "VANGUARD SECURITY ENGINEERING CORP.",
    "3C8970": "Neosfar",
    "78CA83": "IEEE Registration Authority",
    "0C1167": "Cisco Systems, Inc",
    "74EAE8": "ARRIS Group, Inc.",
    "001742": "FUJITSU LIMITED",
    "001999": "Fujitsu Technology Solutions GmbH",
    "005A39": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "A089E4": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "5CC6D0": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "001A9A": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "50F520": "Samsung Electronics Co.,Ltd",
    "64B310": "Samsung Electronics Co.,Ltd",
    "A4EBD3": "Samsung Electronics Co.,Ltd",
    "B8C68E": "Samsung Electronics Co.,Ltd",
    "04FE31": "Samsung Electronics Co.,Ltd",
    "4CBCA5": "Samsung Electronics Co.,Ltd",
    "F88E85": "Comtrend Corporation",
    "02CF1C": "Communication Machinery Corporation",
    "0090F5": "CLEVO CO.",
    "002100": "Gemtek Technology Co., Ltd.",
    "D831CF": "Samsung Electronics Co.,Ltd",
    "B0D09C": "Samsung Electronics Co.,Ltd",
    "50C8E5": "Samsung Electronics Co.,Ltd",
    "188331": "Samsung Electronics Co.,Ltd",
    "9C65B0": "Samsung Electronics Co.,Ltd",
    "8455A5": "Samsung Electronics Co.,Ltd",
    "A87C01": "Samsung Electronics Co.,Ltd",
    "E01D3B": "Cambridge Industries(Group) Co.,Ltd.",
    "C81073": "CENTURY OPTICOMM CO.,LTD",
    "343759": "zte corporation",
    "FC2F40": "Calxeda, Inc.",
    "BC620E": "HUAWEI TECHNOLOGIES CO.,LTD",
    "74A528": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001D19": "Arcadyan Technology Corporation",
    "0012BF": "Arcadyan Technology Corporation",
    "507E5D": "Arcadyan Technology Corporation",
    "7C4FB5": "Arcadyan Technology Corporation",
    "0020D4": "Cabletron Systems, Inc.",
    "00E03A": "Cabletron Systems, Inc.",
    "70C76F": "INNO S",
    "38192F": "Nokia Corporation",
    "0010E7": "Breezecom, Ltd.",
    "5CF6DC": "Samsung Electronics Co.,Ltd",
    "0026E4": "Canal +",
    "000117": "Canal +",
    "F40B93": "BlackBerry RTS",
    "1C69A5": "BlackBerry RTS",
    "94EBCD": "BlackBerry RTS",
    "0026FF": "BlackBerry RTS",
    "A4E4B8": "BlackBerry RTS",
    "803773": "NETGEAR",
    "A42B8C": "NETGEAR",
    "28C68E": "NETGEAR",
    "04A151": "NETGEAR",
    "F87394": "NETGEAR",
    "204E7F": "NETGEAR",
    "C03F0E": "NETGEAR",
    "0026F2": "NETGEAR",
    "003067": "BIOSTAR Microtech Int'l Corp.",
    "9492BC": "SYNTECH(HK) TECHNOLOGY LIMITED",
    "00223F": "NETGEAR",
    "000FB5": "NETGEAR",
    "00095B": "NETGEAR",
    "001A4F": "AVM GmbH",
    "001C4A": "AVM GmbH",
    "00150C": "AVM GmbH",
    "000B06": "ARRIS Group, Inc.",
    "00D088": "ARRIS Group, Inc.",
    "00128A": "ARRIS Group, Inc.",
    "0011AE": "ARRIS Group, Inc.",
    "94CCB9": "ARRIS Group, Inc.",
    "001700": "ARRIS Group, Inc.",
    "0016B5": "ARRIS Group, Inc.",
    "0015A8": "ARRIS Group, Inc.",
    "00159A": "ARRIS Group, Inc.",
    "001180": "ARRIS Group, Inc.",
    "001404": "ARRIS Group, Inc.",
    "001AAD": "ARRIS Group, Inc.",
    "CC7D37": "ARRIS Group, Inc.",
    "A47AA4": "ARRIS Group, Inc.",
    "74E7C6": "ARRIS Group, Inc.",
    "0024C1": "ARRIS Group, Inc.",
    "0025F2": "ARRIS Group, Inc.",
    "0025F1": "ARRIS Group, Inc.",
    "0026BA": "ARRIS Group, Inc.",
    "00230B": "ARRIS Group, Inc.",
    "002375": "ARRIS Group, Inc.",
    "0023A3": "ARRIS Group, Inc.",
    "001ADB": "ARRIS Group, Inc.",
    "001F7E": "ARRIS Group, Inc.",
    "001C12": "ARRIS Group, Inc.",
    "00138F": "Asiarock Technology Limited",
    "34C3D2": "FN-LINK TECHNOLOGY LIMITED",
    "54F6C5": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "409558": "Aisino Corporation",
    "182861": "AirTies Wireless Networks",
    "446D57": "Liteon Technology Corporation",
    "9CB70D": "Liteon Technology Corporation",
    "68A3C4": "Liteon Technology Corporation",
    "70F1A1": "Liteon Technology Corporation",
    "EC086B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "00D9D1": "Sony Interactive Entertainment Inc.",
    "B00594": "Liteon Technology Corporation",
    "28E347": "Liteon Technology Corporation",
    "94A1A2": "AMPAK Technology, Inc.",
    "00014A": "Sony Corporation",
    "001EDC": "Sony Corporation",
    "001D28": "Sony Corporation",
    "8400D2": "Sony Corporation",
    "303926": "Sony Corporation",
    "00EB2D": "Sony Corporation",
    "B4527D": "Sony Corporation",
    "5C338E": "Alpha Networks Inc.",
    "3C438E": "ARRIS Group, Inc.",
    "983B16": "AMPAK Technology, Inc.",
    "001CA2": "ADB Broadband Italia",
    "002233": "ADB Broadband Italia",
    "3039F2": "ADB Broadband Italia",
    "0017C2": "ADB Broadband Italia",
    "001813": "Sony Corporation",
    "402BA1": "Sony Corporation",
    "9C0E4A": "Shenzhen Vastking Electronic Co.,Ltd.",
    "A85840": "Cambridge Industries(Group) Co.,Ltd.",
    "A0D37A": "Intel Corporate",
    "8896F2": "Valeo Schalter und Sensoren GmbH",
    "44D832": "AzureWave Technology Inc.",
    "E0B9A5": "AzureWave Technology Inc.",
    "781881": "AzureWave Technology Inc.",
    "6C71D9": "AzureWave Technology Inc.",
    "D0E782": "AzureWave Technology Inc.",
    "6CADF8": "AzureWave Technology Inc.",
    "A81D16": "AzureWave Technology Inc.",
    "B046FC": "MitraStar Technology Corp.",
    "E04136": "MitraStar Technology Corp.",
    "0015AF": "AzureWave Technology Inc.",
    "74F06D": "AzureWave Technology Inc.",
    "D0D412": "ADB Broadband Italia",
    "0026B8": "Actiontec Electronics, Inc",
    "0026FC": "AcSiP Technology Corp.",
    "689C5E": "AcSiP Technology Corp.",
    "001073": "TECHNOBOX, INC.",
    "20934D": "FUJIAN STAR-NET COMMUNICATION CO.,LTD",
    "009027": "Intel Corporation",
    "00A0C9": "Intel Corporation",
    "B80305": "Intel Corporate",
    "3407FB": "Ericsson AB",
    "A4A1C2": "Ericsson AB",
    "E09796": "HUAWEI TECHNOLOGIES CO.,LTD",
    "000D72": "2Wire Inc",
    "001288": "2Wire Inc",
    "00217C": "2Wire Inc",
    "001FB3": "2Wire Inc",
    "28162E": "2Wire Inc",
    "F81897": "2Wire Inc",
    "94C150": "2Wire Inc",
    "5CF821": "Texas Instruments",
    "141FBA": "IEEE Registration Authority",
    "807B85": "IEEE Registration Authority",
    "CC1BE0": "IEEE Registration Authority",
    "F40E11": "IEEE Registration Authority",
    "C48508": "Intel Corporate",
    "6805CA": "Intel Corporate",
    "8CA982": "Intel Corporate",
    "BC7737": "Intel Corporate",
    "1430C6": "Motorola Mobility LLC, a Lenovo Company",
    "141AA3": "Motorola Mobility LLC, a Lenovo Company",
    "247703": "Intel Corporate",
    "74E50B": "Intel Corporate",
    "D8FC93": "Intel Corporate",
    "0057D2": "Cisco Systems, Inc",
    "3C6716": "Lily Robotics",
    "2C228B": "CTR SRL",
    "0C6F9C": "Shaw Communications Inc.",
    "00D0BD": "Lattice Semiconductor Corp. (LPA)",
    "001F3A": "Hon Hai Precision Ind. Co.,Ltd.",
    "647BD4": "Texas Instruments",
    "D8952F": "Texas Instruments",
    "B8FFFE": "Texas Instruments",
    "88074B": "LG Electronics (Mobile Communications)",
    "F4EB38": "Sagemcom Broadband SAS",
    "001BBF": "Sagemcom Broadband SAS",
    "002569": "Sagemcom Broadband SAS",
    "C8A030": "Texas Instruments",
    "78C5E5": "Texas Instruments",
    "0CFD37": "SUSE Linux GmbH",
    "00789E": "Sagemcom Broadband SAS",
    "E8BE81": "Sagemcom Broadband SAS",
    "681590": "Sagemcom Broadband SAS",
    "10F681": "vivo Mobile Communication Co., Ltd.",
    "2CE412": "Sagemcom Broadband SAS",
    "04E451": "Texas Instruments",
    "505663": "Texas Instruments",
    "883314": "Texas Instruments",
    "44C15C": "Texas Instruments",
    "0022A5": "Texas Instruments",
    "0017E4": "Texas Instruments",
    "002275": "Belkin International Inc.",
    "78E3B5": "Hewlett Packard",
    "78ACC0": "Hewlett Packard",
    "68B599": "Hewlett Packard",
    "1CC1DE": "Hewlett Packard",
    "3C3556": "Cognitec Systems GmbH",
    "3C9066": "SmartRG, Inc.",
    "000D88": "D-Link Corporation",
    "001195": "D-Link Corporation",
    "B8A386": "D-Link International",
    "1C7EE5": "D-Link International",
    "3CCF5B": "ICOMM HK LIMITED",
    "2405F5": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "00738D": "Shenzhen TINNO Mobile Technology Corp.",
    "E043DB": "Shenzhen ViewAt Technology Co.,Ltd. ",
    "F40304": "Google, Inc.",
    "546009": "Google, Inc.",
    "A47733": "Google, Inc.",
    "00E0FC": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001346": "D-Link Corporation",
    "1CBDB9": "D-Link International",
    "9CDFB1": "Shenzhen Crave Communication Co., LTD",
    "606944": "Apple, Inc.",
    "78F882": "LG Electronics (Mobile Communications)",
    "C02C7A": "Shenzhen Horn Audio Co.,Ltd.",
    "1CCB99": "TCT mobile ltd",
    "A42BB0": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "807ABF": "HTC Corporation",
    "34BA75": "Everest Networks, Inc",
    "7C18CD": "E-TRON Co.,Ltd.",
    "94ABDE": "OMX Technology - FZE",
    "6416F0": "HUAWEI TECHNOLOGIES CO.,LTD",
    "ACCF85": "HUAWEI TECHNOLOGIES CO.,LTD",
    "188B45": "Cisco Systems, Inc",
    "F4CA24": "FreeBit Co., Ltd.",
    "00142F": "Savvius",
    "28BC18": "SourcingOverseas Co. Ltd",
    "00D0B7": "Intel Corporation",
    "001DCE": "ARRIS Group, Inc.",
    "001DD6": "ARRIS Group, Inc.",
    "3871DE": "Apple, Inc.",
    "7081EB": "Apple, Inc.",
    "000E35": "Intel Corporation",
    "0007E9": "Intel Corporation",
    "0013E8": "Intel Corporate",
    "0013CE": "Intel Corporate",
    "B8B81E": "Intel Corporate",
    "B46D83": "Intel Corporate",
    "903EAB": "ARRIS Group, Inc.",
    "306023": "ARRIS Group, Inc.",
    "14ABF0": "ARRIS Group, Inc.",
    "901ACA": "ARRIS Group, Inc.",
    "C83FB4": "ARRIS Group, Inc.",
    "E0B70A": "ARRIS Group, Inc.",
    "2C768A": "Hewlett Packard",
    "C8348E": "Intel Corporate",
    "4C3488": "Intel Corporate",
    "1002B5": "Intel Corporate",
    "001708": "Hewlett Packard",
    "0017A4": "Hewlett Packard",
    "BCEAFA": "Hewlett Packard",
    "004026": "BUFFALO.INC",
    "4CE676": "BUFFALO.INC",
    "000BCD": "Hewlett Packard",
    "000F20": "Hewlett Packard",
    "00110A": "Hewlett Packard",
    "C005C2": "ARRIS Group, Inc.",
    "1C7839": "Shenzhen Tencent Computer System Co., Ltd.",
    "0453D5": "Sysorex Global Holdings",
    "EC52DC": "WORLD MEDIA AND TECHNOLOGY Corp.",
    "94B2CC": "PIONEER CORPORATION",
    "08EB74": "HUMAX Co., Ltd.",
    "2832C5": "HUMAX Co., Ltd.",
    "88C255": "Texas Instruments",
    "CC78AB": "Texas Instruments",
    "5820B1": "Hewlett Packard",
    "9457A5": "Hewlett Packard",
    "000EB3": "Hewlett Packard",
    "080009": "Hewlett Packard",
    "0030C1": "Hewlett Packard",
    "0080A0": "Hewlett Packard",
    "D48564": "Hewlett Packard",
    "24BE05": "Hewlett Packard",
    "FC3FDB": "Hewlett Packard",
    "00092D": "HTC Corporation",
    "00265C": "Hon Hai Precision Ind. Co.,Ltd.",
    "002269": "Hon Hai Precision Ind. Co.,Ltd.",
    "D87988": "Hon Hai Precision Ind. Co.,Ltd.",
    "74A78E": "zte corporation",
    "7C6193": "HTC Corporation",
    "90E7C4": "HTC Corporation",
    "90CDB6": "Hon Hai Precision Ind. Co.,Ltd.",
    "40490F": "Hon Hai Precision Ind. Co.,Ltd.",
    "AC162D": "Hewlett Packard",
    "80C16E": "Hewlett Packard",
    "B4B52F": "Hewlett Packard",
    "D07E28": "Hewlett Packard",
    "D0BF9C": "Hewlett Packard",
    "308D99": "Hewlett Packard",
    "7446A0": "Hewlett Packard",
    "2C44FD": "Hewlett Packard",
    "443192": "Hewlett Packard",
    "A0D3C1": "Hewlett Packard",
    "38EAA7": "Hewlett Packard",
    "0452F3": "Apple, Inc.",
    "002127": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "EC888F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "6466B3": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F0F336": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "BC4699": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F483CD": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "FCD733": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "5C899A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "A81B5A": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "2C5BB8": "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD",
    "902106": "SKY UK LIMITED",
    "889471": "Brocade Communications Systems LLC",
    "8C7CFF": "Brocade Communications Systems LLC",
    "5C63BF": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "E005C5": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "388345": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "98F537": "zte corporation",
    "001FE2": "Hon Hai Precision Ind. Co.,Ltd.",
    "0016CF": "Hon Hai Precision Ind. Co.,Ltd.",
    "2002AF": "Murata Manufacturing Co., Ltd.",
    "0021E8": "Murata Manufacturing Co., Ltd.",
    "000E6D": "Murata Manufacturing Co., Ltd.",
    "D02788": "Hon Hai Precision Ind. Co.,Ltd.",
    "904CE5": "Hon Hai Precision Ind. Co.,Ltd.",
    "142D27": "Hon Hai Precision Ind. Co.,Ltd.",
    "5C4CA9": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F4C714": "HUAWEI TECHNOLOGIES CO.,LTD",
    "286ED4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001E10": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D47856": "Avaya Inc",
    "A01290": "Avaya Inc",
    "D842AC": "Shanghai Feixun Communication Co.,Ltd.",
    "5439DF": "HUAWEI TECHNOLOGIES CO.,LTD",
    "74882A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "88E3AB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "C40528": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3CDFBD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "509F27": "HUAWEI TECHNOLOGIES CO.,LTD",
    "8C34FD": "HUAWEI TECHNOLOGIES CO.,LTD",
    "587F66": "HUAWEI TECHNOLOGIES CO.,LTD",
    "64A651": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CCF954": "Avaya Inc",
    "2CF4C5": "Avaya Inc",
    "3C3A73": "Avaya Inc",
    "FC8399": "Avaya Inc",
    "283CE4": "HUAWEI TECHNOLOGIES CO.,LTD",
    "086361": "HUAWEI TECHNOLOGIES CO.,LTD",
    "38BB3C": "Avaya Inc",
    "F873A2": "Avaya Inc",
    "80717A": "HUAWEI TECHNOLOGIES CO.,LTD",
    "D4970B": "Xiaomi Communications Co Ltd",
    "8CBEBE": "Xiaomi Communications Co Ltd",
    "14F65A": "Xiaomi Communications Co Ltd",
    "009EC8": "Xiaomi Communications Co Ltd",
    "0C1DAF": "Xiaomi Communications Co Ltd",
    "3480B3": "Xiaomi Communications Co Ltd",
    "F48B32": "Xiaomi Communications Co Ltd",
    "F4B85E": "Texas Instruments",
    "68C90B": "Texas Instruments",
    "D4F513": "Texas Instruments",
    "507224": "Texas Instruments",
    "ACF7F3": "Xiaomi Communications Co Ltd",
    "0819A6": "HUAWEI TECHNOLOGIES CO.,LTD",
    "3CF808": "HUAWEI TECHNOLOGIES CO.,LTD",
    "486276": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B41513": "HUAWEI TECHNOLOGIES CO.,LTD",
    "AC4E91": "HUAWEI TECHNOLOGIES CO.,LTD",
    "283152": "HUAWEI TECHNOLOGIES CO.,LTD",
    "009021": "Cisco Systems, Inc",
    "0090B1": "Cisco Systems, Inc",
    "04BD70": "HUAWEI TECHNOLOGIES CO.,LTD",
    "001AB6": "Texas Instruments",
    "0012D1": "Texas Instruments",
    "001237": "Texas Instruments",
    "00102F": "Cisco Systems, Inc",
    "00100D": "Cisco Systems, Inc",
    "001007": "Cisco Systems, Inc",
    "001014": "Cisco Systems, Inc",
    "00400B": "Cisco Systems, Inc",
    "0090BF": "Cisco Systems, Inc",
    "0050D1": "Cisco Systems, Inc",
    "0090D9": "Cisco Systems, Inc",
    "009092": "Cisco Systems, Inc",
    "A0E6F8": "Texas Instruments",
    "70FF76": "Texas Instruments",
    "D03972": "Texas Instruments",
    "5C313E": "Texas Instruments",
    "006070": "Cisco Systems, Inc",
    "00E01E": "Cisco Systems, Inc",
    "1CE6C7": "Cisco Systems, Inc",
    "00112F": "ASUSTek COMPUTER INC.",
    "18E728": "Cisco Systems, Inc",
    "D072DC": "Cisco Systems, Inc",
    "28C7CE": "Cisco Systems, Inc",
    "F40F1B": "Cisco Systems, Inc",
    "F8C288": "Cisco Systems, Inc",
    "BCAEC5": "ASUSTek COMPUTER INC.",
    "10BF48": "ASUSTek COMPUTER INC.",
    "6C9989": "Cisco Systems, Inc",
    "1C6A7A": "Cisco Systems, Inc",
    "5067AE": "Cisco Systems, Inc",
    "F09E63": "Cisco Systems, Inc",
    "001BFC": "ASUSTek COMPUTER INC.",
    "485B39": "ASUSTek COMPUTER INC.",
    "CCD539": "Cisco Systems, Inc",
    "500604": "Cisco Systems, Inc",
    "4C0082": "Cisco Systems, Inc",
    "7C95F3": "Cisco Systems, Inc",
    "34DBFD": "Cisco Systems, Inc",
    "885A92": "Cisco Systems, Inc",
    "046C9D": "Cisco Systems, Inc",
    "84B261": "Cisco Systems, Inc",
    "00101F": "Cisco Systems, Inc",
    "54A274": "Cisco Systems, Inc",
    "A0554F": "Cisco Systems, Inc",
    "204C9E": "Cisco Systems, Inc",
    "84B802": "Cisco Systems, Inc",
    "B0AA77": "Cisco Systems, Inc",
    "BCC493": "Cisco Systems, Inc",
    "A46C2A": "Cisco Systems, Inc",
    "D0A5A6": "Cisco Systems, Inc",
    "3C5EC3": "Cisco Systems, Inc",
    "64F69D": "Cisco Systems, Inc",
    "000389": "PLANTRONICS, INC.",
    "484487": "Cisco SPVTG",
    "38C85C": "Cisco SPVTG",
    "E448C7": "Cisco SPVTG",
    "001217": "Cisco-Linksys, LLC",
    "001310": "Cisco-Linksys, LLC",
    "001EE5": "Cisco-Linksys, LLC",
    "001D4F": "Apple, Inc.",
    "002312": "Apple, Inc.",
    "80E01D": "Cisco Systems, Inc",
    "000A27": "Apple, Inc.",
    "3451C9": "Apple, Inc.",
    "406C8F": "Apple, Inc.",
    "D023DB": "Apple, Inc.",
    "70DEE2": "Apple, Inc.",
    "F0CBA1": "Apple, Inc.",
    "182032": "Apple, Inc.",
    "60FACD": "Apple, Inc.",
    "003EE1": "Apple, Inc.",
    "FC253F": "Apple, Inc.",
    "183451": "Apple, Inc.",
    "0C771A": "Apple, Inc.",
    "D8A25E": "Apple, Inc.",
    "5855CA": "Apple, Inc.",
    "DC2B61": "Apple, Inc.",
    "40A6D9": "Apple, Inc.",
    "286ABA": "Apple, Inc.",
    "4CB199": "Apple, Inc.",
    "C09F42": "Apple, Inc.",
    "705681": "Apple, Inc.",
    "04F7E4": "Apple, Inc.",
    "34C059": "Apple, Inc.",
    "F0D1A9": "Apple, Inc.",
    "040CCE": "Apple, Inc.",
    "403CFC": "Apple, Inc.",
    "4860BC": "Apple, Inc.",
    "AC3C0B": "Apple, Inc.",
    "701124": "Apple, Inc.",
    "60FB42": "Apple, Inc.",
    "64B9E8": "Apple, Inc.",
    "14109F": "Apple, Inc.",
    "042665": "Apple, Inc.",
    "EC3586": "Apple, Inc.",
    "54EAA8": "Apple, Inc.",
    "28E14C": "Apple, Inc.",
    "E4C63D": "Apple, Inc.",
    "54E43A": "Apple, Inc.",
    "04DB56": "Apple, Inc.",
    "DC9B9C": "Apple, Inc.",
    "54724F": "Apple, Inc.",
    "8C7C92": "Apple, Inc.",
    "B03495": "Apple, Inc.",
    "F437B7": "Apple, Inc.",
    "78FD94": "Apple, Inc.",
    "2CBE08": "Apple, Inc.",
    "E8802E": "Apple, Inc.",
    "006171": "Apple, Inc.",
    "04E536": "Apple, Inc.",
    "A8BBCF": "Apple, Inc.",
    "AC7F3E": "Apple, Inc.",
    "20A2E4": "Apple, Inc.",
    "BC4CC4": "Apple, Inc.",
    "280B5C": "Apple, Inc.",
    "ACFDEC": "Apple, Inc.",
    "D8CF9C": "Apple, Inc.",
    "DC3714": "Apple, Inc.",
    "6C4008": "Apple, Inc.",
    "28F076": "Apple, Inc.",
    "FCA386": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "40331A": "Apple, Inc.",
    "CCC760": "Apple, Inc.",
    "141357": "ATP Electronics, Inc.",
    "FCCF43": "HUIZHOU CITY HUIYANG DISTRICT MEISIQI INDUSTRY DEVELOPMENT CO,.LTD",
    "B4EF04": "DAIHAN Scientific Co., Ltd.",
    "A4DEC9": "QLove Mobile Intelligence Information Technology (W.H.) Co. Ltd.",
    "CCE0C3": "EXTEN Technologies, Inc.",
    "D848EE": "Hangzhou Xueji Technology Co., Ltd.",
    "4C8ECC": "SILKAN SA",
    "B8B2EB": "Googol Technology (HK) Limited",
    "646A74": "AUTH-SERVERS, LLC",
    "7C7176": "Wuxi iData Technology Company Ltd.",
    "7C0191": "Apple, Inc.",
    "0C54B9": "Nokia",
    "98E848": "Axiim",
    "2C1BC8": "Hunan Topview Network System CO.,LTD",
    "A8474A": "Hon Hai Precision Ind. Co.,Ltd.",
    "C40049": "Kamama",
    "AC6462": "zte corporation",
    "A01E0B": "MINIX Technology Limited",
    "68E8EB": "Linktel Technologies Co.,Ltd",
    "4040A7": "Sony Corporation",
    "C04A09": "Zhejiang Everbright Communication Equip. Co,. Ltd",
    "84100D": "Motorola Mobility LLC, a Lenovo Company",
    "D00F6D": "T&W Electronics Company",
    "48137E": "Samsung Electronics Co.,Ltd",
    "C025A2": "NEC Platforms, Ltd.",
    "908D78": "D-Link International",
    "E435C8": "HUAWEI TECHNOLOGIES CO.,LTD",
    "80D605": "Apple, Inc.",
    "089B4B": "iKuai Networks",
    "A845CD": "Siselectron Technology LTD.",
    "D0C193": "SKYBELL, INC",
    "D48304": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "E80734": "Champion Optical Network Engineering, LLC",
    "30F772": "Hon Hai Precision Ind. Co.,Ltd.",
    "DC3CF6": "Atomic Rules LLC",
    "4473D6": "Logitech",
    "10CC1B": "Liverock technologies,INC",
    "A43831": "RF elements s.r.o.",
    "380546": "Foctek Photonics, Inc.",
    "DC2B2A": "Apple, Inc.",
    "9C8DD3": "Leonton Technologies",
    "E41A2C": "ZPE Systems, Inc.",
    "E8BDD1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F41535": "SPON Communication Technology Co.,Ltd",
    "380AAB": "Formlabs",
    "382DE8": "Samsung Electronics Co.,Ltd",
    "C08997": "Samsung Electronics Co.,Ltd",
    "A815D6": "Shenzhen Meione Technology CO., LTD",
    "C07CD1": "PEGATRON CORPORATION",
    "90D8F3": "zte corporation",
    "D445E8": "Jiangxi Hongpai Technology Co., Ltd.",
    "30A243": "Shenzhen Prifox Innovation Technology Co., Ltd.",
    "342606": "CarePredict, Inc.",
    "38B725": "Wistron Infocomm (Zhongshan) Corporation",
    "6C7220": "D-Link International",
    "28B9D9": "Radisys Corporation",
    "AC676F": "Electrocompaniet A.S.",
    "E0553D": "Cisco Meraki",
    "640DE6": "Petra Systems",
    "F4C613": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "445F8C": "Intercel Group Limited",
    "ACEC80": "ARRIS Group, Inc.",
    "FC335F": "Polyera",
    "84D4C8": "Widex A/S",
    "EC21E5": "Toshiba",
    "380195": "Samsung Electronics Co.,Ltd",
    "44975A": "SHENZHEN FAST TECHNOLOGIES CO.,LTD",
    "5045F7": "Liuhe Intelligence Technology Ltd.",
    "B88981": "Chengdu InnoThings Technology Co., Ltd.",
    "949F3E": "Sonos, Inc.",
    "04C23E": "HTC Corporation",
    "E01AEA": "Allied Telesis, Inc.",
    "3089D3": "HONGKONG UCLOUDLINK NETWORK TECHNOLOGY LIMITED",
    "5CB395": "HUAWEI TECHNOLOGIES CO.,LTD",
    "906CAC": "Fortinet, Inc.",
    "DCA3AC": "RBcloudtech",
    "F02624": "WAFA TECHNOLOGIES CO., LTD.",
    "F8F464": "Rawe Electonic GmbH",
    "D00492": "Fiberhome Telecommunication Technologies Co.,LTD",
    "5C5188": "Motorola Mobility LLC, a Lenovo Company",
    "EC0133": "TRINUS SYSTEMS INC.",
    "00FC8D": "Hitron Technologies. Inc",
    "A013CB": "Fiberhome Telecommunication Technologies Co.,LTD",
    "58F102": "BLU Products Inc.",
    "2CC548": "IAdea Corporation",
    "0CE725": "Microsoft Corporation",
    "14DDA9": "ASUSTek COMPUTER INC.",
    "184F32": "Hon Hai Precision Ind. Co.,Ltd.",
    "DC15DB": "Ge Ruili Intelligent Technology ( Beijing ) Co., Ltd.",
    "84DF19": "Chuango Security Technology Corporation",
    "30F335": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B46D35": "Dalian Seasky Automation Co;Ltd",
    "1816C9": "Samsung Electronics Co.,Ltd",
    "842E27": "Samsung Electronics Co.,Ltd",
    "1CADD1": "Bosung Electronics Co., Ltd.",
    "082CB0": "Network Instruments",
    "E0DB10": "Samsung Electronics Co.,Ltd",
    "546172": "ZODIAC AEROSPACE SAS",
    "EC60E0": "AVI-ON LABS",
    "3CDA2A": "zte corporation",
    "1432D1": "Samsung Electronics Co.,Ltd",
    "3CA31A": "Oilfind International LLC",
    "E8F2E3": "Starcor Beijing Co.,Limited",
    "343D98": "JinQianMao Technology Co.,Ltd.",
    "F44713": "Leading Public Performance Co., Ltd.",
    "601970": "HUIZHOU QIAOXING ELECTRONICS TECHNOLOGY CO., LTD.",
    "A408EA": "Murata Manufacturing Co., Ltd.",
    "A8D409": "USA 111 Inc",
    "6C4598": "Antex Electronic Corp.",
    "68A378": "FREEBOX SAS",
    "340A22": "TOP-ACCESS ELECTRONICS CO LTD",
    "E866C4": "Diamanti",
    "D4D7A9": "Shanghai Kaixiang Info Tech LTD",
    "6C4418": "Zappware",
    "6459F8": "Vodafone Omnitel B.V.",
    "D083D4": "Xtel Wireless ApS",
    "F02A23": "Creative Next Design",
    "584704": " Shenzhen Webridge Technology Co.,Ltd",
    "5CA178": "TableTop Media (dba Ziosk)",
    "9CBEE0": "Biosoundlab Co., Ltd.",
    "0C413E": "Microsoft Corporation",
    "807459": "K's Co.,Ltd.",
    "E4695A": "Dictum Health, Inc.",
    "7C7A53": "Phytrex Technology Corp.",
    "107873": "Shenzhen Jinkeyi Communication Co., Ltd.",
    "48EE0C": "D-Link International",
    "70AD54": "Malvern Instruments Ltd",
    "9000DB": "Samsung Electronics Co.,Ltd",
    "B4EF39": "Samsung Electronics Co.,Ltd",
    "B87879": "Roche Diagnostics GmbH",
    "D06F4A": "TOPWELL INTERNATIONAL HOLDINGS LIMITED",
    "0492EE": "iway AG",
    "7C534A": "Metamako",
    "BCB308": "HONGKONG RAGENTEK COMMUNICATION TECHNOLOGY CO.,LIMITED",
    "BC6E64": "Sony Corporation",
    "6C2E72": "B&B EXPORTING LIMITED",
    "5CCCFF": "Techroutes Network Pvt Ltd",
    "FC3288": "CELOT Wireless Co., Ltd",
    "D87495": "zte corporation",
    "EC3C88": "MCNEX Co.,Ltd.",
    "08D34B": "Techman Electronics (Changshu) Co., Ltd.",
    "78A351": "SHENZHEN ZHIBOTONG ELECTRONICS CO.,LTD",
    "90C35F": "Nanjing Jiahao Technology Co., Ltd.",
    "C808E9": "LG Electronics",
    "E4FED9": "EDMI Europe Ltd",
    "5CF7C3": "SYNTECH (HK) TECHNOLOGY LIMITED",
    "ECE2FD": "SKG Electric Group(Thailand) Co., Ltd.",
    "183A2D": "Samsung Electronics Co.,Ltd",
    "EC74BA": "Hirschmann Automation and Control GmbH",
    "74A063": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C3B35": "Gehirn Inc.",
    "545146": "AMG Systems Ltd.",
    "8463D6": "Microsoft Corporation",
    "BC4DFB": "Hitron Technologies. Inc",
    "2C337A": "Hon Hai Precision Ind. Co.,Ltd.",
    "7076FF": "KERLINK",
    "1436C6": "Lenovo Mobile Communication Technology Ltd.",
    "BCE767": "Quanzhou  TDX Electronics Co., Ltd",
    "344CA4": "amazipoint technology Ltd.",
    "148F21": "Garmin International",
    "9C685B": "Octonion SA",
    "9CE230": "JULONG CO,.LTD.",
    "5C41E7": "Wiatec International Ltd.",
    "A8F038": "SHEN ZHEN SHI JIN HUA TAI ELECTRONICS CO.,LTD",
    "ACC73F": "VITSMO CO., LTD.",
    "44356F": "Neterix Ltd",
    "BC54F9": "Drogoo Technology Co., Ltd.",
    "78FC14": "Family Zone Cyber Safety Ltd ",
    "3809A4": "Firefly Integrations",
    "74E277": "Vizmonet Pte Ltd",
    "14893E": "VIXTEL TECHNOLOGIES LIMTED",
    "FCAFAC": "Socionext Inc.",
    "382C4A": "ASUSTek COMPUTER INC.",
    "08EFAB": "SAYME WIRELESS SENSOR NETWORK",
    "3C1E13": "HANGZHOU SUNRISE TECHNOLOGY CO., LTD",
    "049B9C": "Eadingcore  Intelligent Technology Co., Ltd.",
    "842690": "BEIJING THOUGHT SCIENCE CO.,LTD.",
    "84DDB7": "Cilag GmbH International",
    "CCBDD3": "Ultimaker B.V.",
    "8CE78C": "DK Networks",
    "C09879": "Acer Inc.",
    "307350": "Inpeco SA",
    "DCEC06": "Heimi Network Technology Co., Ltd.",
    "EC13B2": "Netonix",
    "104E07": "Shanghai Genvision Industries Co.,Ltd",
    "5CAAFD": "Sonos, Inc.",
    "F03D29": "Actility",
    "88708C": "Lenovo Mobile Communication Technology Ltd.",
    "08115E": "Bitel Co., Ltd.",
    "0881BC": "HongKong Ipro Technology Co., Limited",
    "5014B5": "Richfit Information Technology Co., Ltd",
    "3428F0": "ATN International Limited",
    "CC10A3": "Beijing Nan Bao Technology Co., Ltd.",
    "14EDE4": "Kaiam Corporation",
    "CC3F1D": "HMS Industrial Networks SLU",
    "DCDA4F": "GETCK TECHNOLOGY,  INC",
    "801967": "Shanghai Reallytek Information Technology  Co.,Ltd",
    "2CF7F1": "Seeed Technology Inc.",
    "101218": "Korins Inc.",
    "B84FD5": "Microsoft Corporation",
    "D84A87": "OI ELECTRIC CO.,LTD",
    "D01242": "BIOS Corporation",
    "603696": "The Sapling Company",
    "F4F26D": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "ACB74F": "METEL s.r.o.",
    "CCF538": "3isysnetworks",
    "C0EEFB": "OnePlus Tech (Shenzhen) Ltd",
    "54FFCF": "Mopria Alliance",
    "B8BD79": "TrendPoint Systems",
    "304225": "BURG-W\u00c4CHTER KG",
    "FCDBB3": "Murata Manufacturing Co., Ltd.",
    "404EEB": "Higher Way Electronic Co., Ltd.",
    "C456FE": "Lava International Ltd.",
    "74F413": "Maxwell Forest",
    "A00627": "NEXPA System",
    "303335": "Boosty",
    "F4D261": "SEMOCON Co., Ltd",
    "B009D3": "Avizia",
    "9CAD97": "Hon Hai Precision Ind. Co.,Ltd.",
    "BC9CC5": "Beijing Huafei Technology Co., Ltd.",
    "94D60E": "shenzhen yunmao information technologies co., ltd",
    "506787": "Planet Networks",
    "5CB8CB": "Allis Communications",
    "34F0CA": "Shenzhen Linghangyuan Digital Technology Co.,Ltd.",
    "44746C": "Sony Corporation",
    "4C9EFF": "Zyxel Communications Corporation",
    "6C2F2C": "Samsung Electronics Co.,Ltd",
    "8CBF9D": "Shanghai Xinyou Information Technology Ltd. Co.",
    "70720D": "Lenovo Mobile Communication Technology Ltd.",
    "3CCD5A": "Technische Alternative GmbH",
    "48D855": "Telvent",
    "08F728": "GLOBO Multimedia Sp. z o.o. Sp.k.",
    "206E9C": "Samsung Electronics Co.,Ltd",
    "BC25F0": "3D Display Technologies Co., Ltd.",
    "C03D46": "Shanghai Sango Network Technology Co.,Ltd",
    "64EAC5": "SiboTech Automation Co., Ltd.",
    "60C1CB": "Fujian Great Power PLC Equipment Co.,Ltd",
    "882950": "Netmoon Technology Co., Ltd",
    "7CE524": "Quirky, Inc.",
    "74DA38": "Edimax Technology Co. Ltd.",
    "30F7D7": "Thread Technology Co., Ltd",
    "18227E": "Samsung Electronics Co.,Ltd",
    "30C7AE": "Samsung Electronics Co.,Ltd",
    "E4F4C6": "NETGEAR",
    "90AE1B": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "60E327": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "44D4E0": "Sony Corporation",
    "B025AA": "Private",
    "D4B43E": "Messcomp Datentechnik GmbH",
    "CCA0E5": "DZG Metering GmbH",
    "60812B": "Astronics Custom Control Concepts",
    "F86601": "Suzhou Chi-tek information technology Co., Ltd",
    "145645": "Savitech Corp.",
    "1C1CFD": "Dalian Hi-Think Computer Technology, Corp",
    "48D18E": "Metis Communication Co.,Ltd",
    "6C2C06": "OOO NPP Systemotechnika-NN",
    "C4913A": "Shenzhen Sanland Electronic Co., ltd.",
    "7062B8": "D-Link International",
    "ACA919": "TrekStor GmbH",
    "C44E1F": "BlueN",
    "B0869E": "Chloride S.r.L",
    "D057A1": "Werma Signaltechnik GmbH & Co. KG",
    "B4B542": "Hubbell Power Systems, Inc.",
    "64E892": "Morio Denki Co., Ltd.",
    "88E8F8": "YONG TAI ELECTRONIC (DONGGUAN) LTD.",
    "909864": "Impex-Sat GmbH&amp;Co KG",
    "DCE578": "Experimental Factory of Scientific Engineering and Special Design Department",
    "38F098": "Vapor Stone Rail Systems",
    "54CDEE": "ShenZhen Apexis Electronic Co.,Ltd",
    "E8FC60": "ELCOM Innovations Private Limited",
    "9451BF": "Hyundai ESG",
    "F015A0": "KyungDong One Co., Ltd.",
    "1CAB01": "Innovolt",
    "B0DA00": "CERA ELECTRONIQUE",
    "D8B6D6": "Blu Tether Limited",
    "94C014": "Sorter Sp. j. Konrad Grzeszczyk MichaA, Ziomek",
    "9CFBF1": "MESOMATIC GmbH & Co.KG",
    "1027BE": "TVIP",
    "2087AC": "AES motomation",
    "709383": "Intelligent Optical Network High Tech CO.,LTD.",
    "80D433": "LzLabs GmbH",
    "8C41F2": "RDA Technologies Ltd.",
    "E036E3": "Stage One International Co., Ltd.",
    "242642": "SHARP Corporation.",
    "34DE34": "zte corporation",
    "FC1607": "Taian Technology(Wuxi) Co.,Ltd.",
    "AC02CA": "HI Solutions, Inc.",
    "04DB8A": "Suntech International Ltd.",
    "90DFB7": "s.m.s smart microwave sensors GmbH",
    "085700": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F85C45": "IC Nexus Co. Ltd.",
    "ACE069": "ISAAC Instruments",
    "30B5C2": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "FC27A2": "TRANS ELECTRIC CO., LTD.",
    "FCBBA1": "Shenzhen Minicreate Technology Co.,Ltd",
    "F08A28": "JIANGSU HENGSION ELECTRONIC S and T CO.,LTD",
    "28BB59": "RNET Technologies, Inc.",
    "E8EA6A": "StarTech.com",
    "D86595": "Toy's Myth Inc.",
    "C0F79D": "Powercode",
    "C4C919": "Energy Imports Ltd",
    "D8DD5F": "BALMUDA Inc.",
    "E07F53": "TECHBOARD SRL",
    "48FEEA": "HOMA B.V.",
    "746F3D": "Contec GmbH",
    "4C0DEE": "JABIL CIRCUIT (SHANGHAI) LTD.",
    "D0634D": "Meiko Maschinenbau GmbH &amp; Co. KG",
    "D86194": "Objetivos y Sevicios de Valor An\u0303adido",
    "589CFC": "FreeBSD Foundation",
    "702C1F": "Wisol",
    "DC052F": "National Products Inc.",
    "008B43": "RFTECH",
    "C8D429": "Muehlbauer AG",
    "8C569D": "Imaging Solutions Group",
    "40B6B1": "SUNGSAM CO,.Ltd",
    "E86183": "Black Diamond Advanced Technology, LLC",
    "38C9A9": "SMART High Reliability Solutions, Inc.",
    "8CDE99": "Comlab Inc.",
    "4CE1BB": "Zhuhai HiFocus Technology Co., Ltd.",
    "24A87D": "Panasonic Automotive Systems Asia Pacific(Thailand)Co.,Ltd.",
    "84FE9E": "RTC Industries, Inc.",
    "FC4B1C": "INTERSENSOR S.R.L.",
    "403067": "Conlog (Pty) Ltd",
    "600347": "Billion Electric Co. Ltd.",
    "F81CE5": "Telefonbau Behnke GmbH",
    "EC71DB": "Reolink Innovation Limited",
    "A409CB": "Alfred Kaercher GmbH &amp; Co KG",
    "501AC5": "Microsoft",
    "B0989F": "LG CNS",
    "C0F1C4": "Pacidal Corporation Ltd.",
    "14EDA5": "Wa\u0308chter GmbH Sicherheitssysteme",
    "D0BD01": "DS International",
    "085240": "EbV Elektronikbau- und Vertriebs GmbH",
    "B8C1A2": "Dragon Path Technologies Co., Limited",
    "80F25E": "Kyynel",
    "889166": "Viewcooper Corp.",
    "38A53C": "COMECER Netherlands",
    "5CFFFF": "Shenzhen Kezhonglong Optoelectronic Technology Co., Ltd",
    "68692E": "Zycoo Co.,Ltd",
    "D46867": "Neoventus Design Group",
    "9C216A": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "F862AA": "xn systems",
    "A4059E": "STA Infinity LLP",
    "44EE30": "Budelmann Elektronik GmbH",
    "FC1E16": "IPEVO corp",
    "3051F8": "BYK-Gardner GmbH",
    "E8F226": "MILLSON CUSTOM SOLUTIONS INC.",
    "3C6E63": "Mitron OY",
    "103378": "FLECTRON Co., LTD",
    "F0D3A7": "CobaltRay Co., Ltd",
    "2C18AE": "Trend Electronics Co., Ltd.",
    "50C006": "Carmanah Signs",
    "A4E9A3": "Honest Technology Co., Ltd",
    "A0E5E9": "enimai Inc",
    "04CB1D": "Traka plc",
    "C0C687": "Cisco SPVTG",
    "C49380": "Speedytel technology",
    "007DFA": "Volkswagen Group of America",
    "E0AEB2": "Bender GmbH &amp; Co.KG",
    "F84A7F": "Innometriks Inc",
    "0C9B13": "Shanghai Magic Mobile Telecommunication Co.Ltd.",
    "107BEF": "Zyxel Communications Corporation",
    "B462AD": "Elysia Germany GmbH",
    "2C7155": "HiveMotion",
    "20180E": "Shenzhen Sunchip Technology Co., Ltd",
    "80B219": "ELEKTRON TECHNOLOGY UK LIMITED",
    "2894AF": "Samhwa Telecom",
    "B4750E": "Belkin International Inc.",
    "94103E": "Belkin International Inc.",
    "7CB733": "ASKEY COMPUTER CORP",
    "345C40": "Cargt Holdings LLC",
    "74A4B5": "Powerleader Science and Technology Co. Ltd.",
    "909F43": "Accutron Instruments Inc.",
    "AC5036": "Pi-Coral Inc",
    "C4D655": "Tercel technology co.,ltd",
    "58BDF9": "Sigrand",
    "2464EF": "CYG SUNRI CO.,LTD.",
    "D8270C": "MaxTronic International Co., Ltd.",
    "68193F": "Digital Airways",
    "B4CCE9": "PROSYST",
    "A0BF50": "S.C. ADD-PRODUCTION S.R.L.",
    "E8D4E0": "Beijing BenyWave Technology Co., Ltd.",
    "FC019E": "VIEVU",
    "642184": "Nippon Denki Kagaku Co.,LTD",
    "94BF1E": "eflow Inc. / Smart Device Planning and Development Division",
    "E8516E": "TSMART Inc.",
    "AC220B": "ASUSTek COMPUTER INC.",
    "B887A8": "Step Ahead Innovations Inc.",
    "8C4B59": "3D Imaging & Simulations Corp",
    "5C3327": "Spazio Italia srl",
    "E0A198": "NOJA Power Switchgear Pty Ltd",
    "88354C": "Transics",
    "507691": "Tekpea, Inc.",
    "CC4AE1": "fourtec -Fourier Technologies",
    "28F532": "ADD-Engineering BV",
    "9440A2": "Anywave Communication Technologies, Inc.",
    "384233": "Wildeboer Bauteile GmbH",
    "C034B4": "Gigastone Corporation",
    "303EAD": "Sonavox Canada Inc",
    "F835DD": "Gemtek Technology Co., Ltd.",
    "D8B04C": "Jinan USR IOT Technology Co., Ltd.",
    "E8519D": "Yeonhab Precision Co.,LTD",
    "1C86AD": "MCT CO., LTD.",
    "28D93E": "Telecor Inc.",
    "640B4A": "Digital Telecom Technology Limited",
    "70F176": "Data Modul AG",
    "CCE8AC": "SOYEA Technology Co.,Ltd.",
    "CC04B4": "Select Comfort",
    "5C15E1": "AIDC TECHNOLOGY (S) PTE LTD",
    "B847C6": "SanJet Technology Corp.",
    "B8CD93": "Penetek, Inc",
    "3806B4": "A.D.C. GmbH",
    "B8241A": "SWEDA INFORMATICA LTDA",
    "A0B100": "ShenZhen Cando Electronics Co.,Ltd",
    "201D03": "Elatec GmbH",
    "C06C6D": "MagneMotion, Inc.",
    "B8C855": "Shanghai GBCOM Communication Technology Co.,Ltd.",
    "78303B": "Stephen Technologies Co.,Limited",
    "40BC73": "Cronoplast  S.L.",
    "4007C0": "Railtec Systems GmbH",
    "84E4D9": "Shenzhen NEED technology Ltd.",
    "784B08": "f.robotics acquisitions ltd",
    "B0808C": "Laser Light Engines",
    "E880D8": "GNTEK Electronics Co.,Ltd.",
    "D866C6": "Shenzhen Daystar Technology Co.,ltd",
    "D00EA4": "Porsche Cars North America",
    "188857": "Beijing Jinhong Xi-Dian Information Technology Corp.",
    "E4776B": "AARTESYS AG",
    "30F31D": "zte corporation",
    "A0EC80": "zte corporation",
    "28DB81": "Shanghai Guao Electronic Technology Co., Ltd",
    "0CC81F": "Summer Infant, Inc.",
    "A8FB70": "WiseSec L.t.d",
    "E481B3": "Shenzhen ACT Industrial Co.,Ltd.",
    "1C4AF7": "AMON INC",
    "D8DCE9": "Kunshan Erlab ductless filtration system Co.,Ltd",
    "DCD52A": "Sunny Heart Limited",
    "A4D3B5": "GLITEL Stropkov, s.r.o.",
    "44619C": "FONsystem co. ltd.",
    "88685C": "Shenzhen ChuangDao & Perpetual Eternal Technology Co.,Ltd",
    "102831": "Morion Inc.",
    "DC5726": "Power-One",
    "F8DADF": "EcoTech, Inc.",
    "30AE7B": "Deqing Dusun Electron CO., LTD",
    "68EC62": "YODO Technology Corp. Ltd.",
    "1C76CA": "Terasic Technologies Inc.",
    "F499AC": "WEBER Schraubautomaten GmbH",
    "D43A65": "IGRS Engineering Lab Ltd.",
    "7CD844": "Enmotus Inc",
    "70820E": "as electronics GmbH",
    "888964": "GSI Electronics Inc.",
    "0C1105": "AKUVOX (XIAMEN) NETWORKS CO., LTD",
    "5C22C4": "DAE EUN ELETRONICS CO., LTD",
    "F8FEA8": "Technico Japan Corporation",
    "1800DB": "Fitbit Inc.",
    "78A106": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "D05099": "ASRock Incorporation",
    "A49EDB": "AutoCrib, Inc.",
    "6C8B2F": "zte corporation",
    "CC1AFA": "zte corporation",
    "8C5AF0": "Exeltech Solar Products",
    "6C6126": "Rinicom Holdings",
    "CC047C": "G-WAY Microwave",
    "50ABBF": "Hoseo Telecom",
    "B4FE8C": "Centro Sicurezza Italia SpA",
    "2CF203": "EMKO ELEKTRONIK SAN VE TIC AS",
    "185AE8": "Zenotech.Co.,Ltd",
    "C47DCC": "Zebra Technologies Inc",
    "E0AEED": "LOENK",
    "E492E7": "Gridlink Tech. Co.,Ltd.",
    "1C37BF": "Cloudium Systems Ltd.",
    "D82916": "Ascent Communication Technology",
    "A073FC": "Rancore Technologies Private Limited",
    "64535D": "Frauscher Sensortechnik",
    "40E730": "DEY Storage Systems, Inc.",
    "68B094": "INESA ELECTRON CO.,LTD",
    "44F849": "Union Pacific Railroad",
    "CC0DEC": "Cisco SPVTG",
    "2C7B5A": "Milper Ltd",
    "0C722C": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "FC9FAE": "Fidus Systems Inc",
    "681E8B": "InfoSight Corporation",
    "F8D7BF": "REV Ritter GmbH",
    "48BE2D": "Symanitron",
    "F02329": "SHOWA DENKI CO.,LTD.",
    "5461EA": "Zaplox AB",
    "D08B7E": "Passif Semiconductor",
    "04586F": "Sichuan Whayer information industry Co.,LTD",
    "D4BF2D": "SE Controls Asia Pacific Ltd",
    "9CE635": "Nintendo Co., Ltd.",
    "60A44C": "ASUSTek COMPUTER INC.",
    "E0D9A2": "Hippih aps",
    "FC0647": "Cortland Research, LLC",
    "D052A8": "Physical Graph Corporation",
    "CC3A61": "SAMSUNG ELECTRO MECHANICS CO., LTD.",
    "3C6FF7": "EnTek Systems, Inc.",
    "6CD146": "FRAMOS GmbH",
    "3C57D5": "FiveCo",
    "F073AE": "PEAK-System Technik",
    "48B8DE": "HOMEWINS TECHNOLOGY CO.,LTD.",
    "10EA59": "Cisco SPVTG",
    "0C191F": "Inform Electronik",
    "98208E": "Definium Technologies",
    "704AE4": "Rinstrum Pty Ltd",
    "083AB8": "Shinoda Plasma Co., Ltd.",
    "A0DD97": "PolarLink Technologies, Ltd",
    "E4A7FD": "Cellco Partnership",
    "2CE2A8": "DeviceDesign",
    "60C5A8": "Beijing LT Honway Technology Co.,Ltd",
    "B4DF3B": "Chromlech",
    "A845E9": "Firich Enterprises CO., LTD.",
    "7C9A9B": "VSE valencia smart energy",
    "645A04": "Chicony Electronics Co., Ltd.",
    "AC1702": "Fibar Group sp. z o.o.",
    "984CD3": "Mantis Deposition",
    "08606E": "ASUSTek COMPUTER INC.",
    "EC89F5": "Lenovo Mobile Communication Technology Ltd.",
    "B49842": "zte corporation",
    "7054D2": "PEGATRON CORPORATION",
    "7076DD": "OxyGuard Internation A/S",
    "E89AFF": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "1065CF": "IQSIM",
    "684CA8": "Shenzhen Herotel Tech. Co., Ltd.",
    "0C8CDC": "Suunto Oy",
    "F84897": "Hitachi, Ltd.",
    "F80BD0": "Datang Telecom communication terminal (Tianjin) Co., Ltd.",
    "646223": "Cellient Co., Ltd.",
    "98473C": "SHANGHAI SUNMON COMMUNICATION TECHNOGY CO.,LTD",
    "5481AD": "Eagle Research Corporation",
    "EC473C": "Redwire, LLC",
    "3CC12C": "AES Corporation",
    "949BFD": "Trans New Technology, Inc.",
    "84E714": "Liang Herng Enterprise,Co.Ltd.",
    "B829F7": "Blaster Tech",
    "C8E1A7": "Vertu Corporation Limited",
    "A00ABF": "Wieson Technologies Co., Ltd.",
    "8CCDE8": "Nintendo Co., Ltd.",
    "00E666": "ARIMA Communications Corp.",
    "34BDFA": "Cisco SPVTG",
    "D4136F": "Asia Pacific Brands",
    "00B56D": "David Electronics Co., LTD.",
    "2C3557": "ELIIY Power CO., Ltd.",
    "B80415": "Bayan Audio",
    "4CAB33": "KST technology",
    "485261": "SOREEL",
    "F4472A": "Nanjing Rousing Sci. and Tech. Industrial Co., Ltd",
    "DC028E": "zte corporation",
    "18D949": "Qvis Labs, LLC",
    "D808F5": "Arcadia Networks Co. Ltd. ",
    "049F06": "Smobile Co., Ltd.",
    "289A4B": "SteelSeries ApS",
    "7CB232": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "54A04F": "t-mac Technologies Ltd",
    "14DB85": "S NET MEDIA",
    "B8DAF1": "Strahlenschutz- Entwicklungs- und Ausruestungsgesellschaft mbH",
    "D45C70": "Wi-Fi Alliance",
    "0CC47E": "EUCAST Co., Ltd.",
    "50724D": "BEG Brueck Electronic GmbH",
    "783CE3": "Kai-EE",
    "B898B0": "Atlona Inc.",
    "080C0B": "SysMik GmbH Dresden",
    "DCBF90": "HUIZHOU QIAOXING TELECOMMUNICATION INDUSTRY CO.,LTD.",
    "C8FB26": "Cisco SPVTG",
    "ACBD0B": "Leimac Ltd.",
    "B85810": "NUMERA, INC.",
    "A08C15": "Gerhard D. Wempe KG",
    "90CF6F": "Dlogixs Co Ltd",
    "24694A": "Jasmine Systems Inc.",
    "F41E26": "Simon-Kaloi Engineering",
    "702526": "Nokia",
    "ECD950": "IRT SA",
    "645FFF": "Nicolet Neuro",
    "A4D18F": "Shenzhen Skyee Optical Fiber Communication Technology Ltd. ",
    "58343B": "Glovast Technology Ltd.",
    "109FA9": "Actiontec Electronics, Inc",
    "C0A364": "3D Systems Massachusetts",
    "AC7A42": "iConnectivity",
    "F8051C": "DRS Imaging and Targeting Solutions",
    "78D34F": "Pace-O-Matic, Inc.",
    "A4466B": "EOC Technology",
    "7C386C": "Real Time Logic",
    "2067B1": "Pluto inc.",
    "7C02BC": "Hansung Electronics Co. LTD",
    "B82410": "Magneti Marelli Slovakia s.r.o.",
    "105F49": "Cisco SPVTG",
    "1C5C60": "Shenzhen Belzon Technology Co.,LTD.",
    "700BC0": "Dewav Technology Company",
    "CC14A6": "Yichun MyEnergy Domain, Inc",
    "889676": "TTC MARCONI s.r.o.",
    "B8B94E": "Shenzhen iBaby Labs, Inc.",
    "ACC698": "Kohzu Precision Co., Ltd.",
    "AC0A61": "Labor S.r.L.",
    "241064": "Shenzhen Ecsino Tecnical Co. Ltd",
    "7CEBEA": "ASCT",
    "189A67": "CSE-Servelec Limited",
    "087D21": "Altasec technology corporation",
    "34D7B4": "Tributary Systems, Inc.",
    "F40F9B": "WAVELINK",
    "901EDD": "GREAT COMPUTER CORPORATION",
    "9C0DAC": "Tymphany HK Limited",
    "1C43EC": "JAPAN CIRCUIT CO.,LTD",
    "785262": "Shenzhen Hojy Software Co., Ltd.",
    "20014F": "Linea Research Ltd",
    "EC0ED6": "ITECH INSTRUMENTS SAS",
    "746A89": "Rezolt Corporation",
    "702F4B": "Steelcase Inc.",
    "741489": "SRT Wireless",
    "241B13": "Shanghai Nutshell Electronic Co., Ltd.",
    "B43564": "Fujian Tian Cheng Electron Science & Technical Development Co.,Ltd.",
    "1C5FFF": "Beijing Ereneben Information Technology Co.,Ltd Shenzhen Branch",
    "6045BD": "Microsoft",
    "9C54CA": "Zhengzhou VCOM Science and Technology Co.,Ltd",
    "00BF15": "Genetec Inc.",
    "241125": "Hutek Co., Ltd.",
    "B431B8": "Aviwest",
    "508C77": "DIRMEIER Schanktechnik GmbH &Co KG",
    "5C1737": "I-View Now, LLC.",
    "70B599": "Embedded Technologies s.r.o.",
    "EC4C4D": "ZAO NPK RoTeK",
    "38EE9D": "Anedo Ltd.",
    "54D1B0": "Universal Laser Systems, Inc",
    "240917": "Devlin Electronics Limited",
    "78BEBD": "STULZ GmbH",
    "B058C4": "Broadcast Microwave Services, Inc",
    "84AF1F": "Beat System Service Co,. Ltd.",
    "A007B6": "Advanced Technical Support, Inc.",
    "0CD2B5": "Binatone Telecommunication Pvt. Ltd",
    "4846F1": "Uros Oy",
    "B827EB": "Raspberry Pi Foundation",
    "08BE09": "Astrol Electronic AG",
    "B41DEF": "Internet Laboratories, Inc.",
    "809393": "Xapt GmbH",
    "6044F5": "Easy Digital Ltd.",
    "F47F35": "Cisco Systems, Inc",
    "BCC168": "DinBox Sverige AB",
    "DC309C": "Heyrex Limited",
    "D4DF57": "Alpinion Medical Systems",
    "5048EB": "BEIJING HAIHEJINSHENG NETWORK TECHNOLOGY CO. LTD.",
    "B40E96": "HERAN ",
    "F89955": "Fortress Technology Inc",
    "4CC94F": "Nokia",
    "0CE5D3": "DH electronics GmbH",
    "40704A": "Power Idea Technology Limited",
    "545EBD": "NL Technologies",
    "CC187B": "Manzanita Systems, Inc.",
    "28F606": "Syes srl",
    "0CAF5A": "GENUS POWER INFRASTRUCTURES LIMITED",
    "7CEF8A": "Inhon International Ltd.",
    "94FAE8": "Shenzhen Eycom Technology Co., Ltd ",
    "E840F2": "PEGATRON CORPORATION",
    "50053D": "CyWee Group Ltd",
    "F88C1C": "KAISHUN ELECTRONIC TECHNOLOGY CO., LTD. BEIJING",
    "1C0B52": "EPICOM S.A",
    "747E2D": "Beijing Thomson CITIC Digital Technology Co. LTD.",
    "4C64D9": "Guangdong Leawin Group Co., Ltd",
    "9C0111": "Shenzhen Newabel Electronic Co., Ltd.",
    "400E67": "Tremol Ltd.",
    "008DDA": "Link One Co., Ltd.",
    "18193F": "Tamtron Oy",
    "885C47": "Alcatel Lucent",
    "3CC1F6": "Melange Systems Pvt. Ltd.",
    "C49805": "Minieum Networks, Inc",
    "90F4C1": "Rand McNally",
    "3CCE73": "Cisco Systems, Inc",
    "B0BD6D": "Echostreams Innovative Solutions",
    "745798": "TRUMPF Laser GmbH + Co. KG",
    "2817CE": "Omnisense Ltd",
    "E425E9": "Color-Chip",
    "78A183": "Advidia",
    "940149": "AutoHotBox",
    "1CB094": "HTC Corporation",
    "14B1C8": "InfiniWing, Inc.",
    "B48255": "Research Products Corporation",
    "8016B7": "Brunel University",
    "48ED80": "daesung eltec",
    "C80718": "TDSi",
    "B467E9": "Qingdao GoerTek Technology Co., Ltd.",
    "186751": "KOMEG Industrielle Messtechnik GmbH",
    "F0F755": "Cisco Systems, Inc",
    "1CB243": "TDC A/S",
    "38BF33": "NEC CASIO Mobile Communications",
    "645EBE": "Yahoo! JAPAN",
    "CCC50A": "SHENZHEN DAJIAHAO TECHNOLOGY CO.,LTD",
    "D8BF4C": "Victory Concept Electronics Limited",
    "3CB9A6": "Belden Deutschland GmbH",
    "C0DF77": "Conrad Electronic SE",
    "581D91": "Advanced Mobile Telecom co.,ltd.",
    "1CB17F": "NEC Platforms, Ltd.",
    "E42C56": "Lilee Systems, Ltd.",
    "B89674": "AllDSP GmbH & Co. KG",
    "3055ED": "Trex Network LLC",
    "94DF58": "IJ Electron CO.,Ltd.",
    "48D54C": "Jeda Networks",
    "1CBBA8": "OJSC \"Ufimskiy Zavod \"Promsvyaz\"",
    "70D6B6": "Metrum Technologies",
    "C0A0DE": "Multi Touch Oy",
    "8C0CA3": "Amper",
    "48E1AF": "Vity",
    "BCB852": "Cybera, Inc.",
    "C49300": "8Devices",
    "6CA682": "EDAM information & communications",
    "28D576": "Premier Wireless, Inc.",
    "F0DA7C": "RLH INDUSTRIES,INC.",
    "AC319D": "Shenzhen TG-NET Botone Technology Co.,Ltd.",
    "2486F4": "Ctek, Inc.",
    "C4237A": "WhizNets Inc.",
    "903CAE": "Yunnan KSEC Digital Technology Co.,Ltd.",
    "70704C": "Purple Communications, Inc",
    "D89760": "C2 Development, Inc.",
    "8CDE52": "ISSC Technologies Corp.",
    "082522": "ADVANSEE",
    "4C2F9D": "ICM Controls",
    "50FC30": "Treehouse Labs",
    "E467BA": "Danish Interpretation Systems A/S",
    "9CA134": "Nike, Inc.",
    "40BC8B": "itelio GmbH",
    "F47ACC": "SolidFire, Inc.",
    "905682": "Lenbrook Industries Limited",
    "C09132": "Patriot Memory",
    "90185E": "Apex Tool Group GmbH & Co OHG",
    "F8FE5C": "Reciprocal Labs Corp",
    "6C9CED": "Cisco Systems, Inc",
    "006BA0": "SHENZHEN UNIVERSAL INTELLISYS PTE LTD",
    "A898C6": "Shinbo Co., Ltd.",
    "B4211D": "Beijing GuangXin Technology Co., Ltd",
    "E843B6": "QNAP Systems, Inc.",
    "E0DADC": "JVC KENWOOD Corporation",
    "B89BC9": "SMC Networks Inc",
    "D4D249": "Power Ethernet",
    "80427C": "Adolf Tedsen GmbH & Co. KG",
    "409FC7": "BAEKCHUN I&C Co., Ltd.",
    "00FC58": "WebSilicon Ltd.",
    "983571": "Sub10 Systems Ltd",
    "F4E6D7": "Solar Power Technologies, Inc.",
    "20107A": "Gemtek Technology Co., Ltd.",
    "78DDD6": "c-scape",
    "F82F5B": "eGauge Systems LLC",
    "E4AFA1": "HES-SO",
    "A887ED": "ARC Wireless LLC",
    "186D99": "Adanis Inc.",
    "A0E201": "AVTrace Ltd.(China)",
    "9CF67D": "Ricardo Prague, s.r.o.",
    "989080": "Linkpower Network System Inc Ltd.",
    "B87424": "Viessmann Elektronik GmbH",
    "B451F9": "NB Software",
    "30168D": "ProLon",
    "4C0289": "LEX COMPUTECH CO., LTD",
    "C0E54E": "ARIES Embedded GmbH",
    "50F61A": "Kunshan JADE Technologies co., Ltd.",
    "542018": "Tely Labs",
    "581FEF": "Tuttnaer LTD",
    "58BDA3": "Nintendo Co., Ltd.",
    "187C81": "Valeo Vision Systems",
    "ACCC8E": "Axis Communications AB",
    "BC764E": "Rackspace US, Inc.",
    "5404A6": "ASUSTek COMPUTER INC.",
    "F83376": "Good Mind Innovation Co., Ltd.",
    "98C845": "PacketAccess",
    "CCC8D7": "CIAS Elettronica srl",
    "84D32A": "IEEE 1905.1",
    "8C94CF": "Encell Technology, Inc.",
    "F8F25A": "G-Lab GmbH",
    "F0DEB9": "ShangHai Y&Y Electronics Co., Ltd",
    "7CA61D": "MHL, LLC",
    "3057AC": "IRLAB LTD.",
    "842B50": "Huria Co.,Ltd.",
    "B8BB6D": "ENERES Co.,Ltd.",
    "14373B": "PROCOM Systems",
    "1897FF": "TechFaith Wireless Technology Limited",
    "4C5585": "Hamilton Systems",
    "8C8E76": "taskit GmbH",
    "A0133B": "HiTi Digital, Inc.",
    "48F7F1": "Nokia",
    "9C5711": "Feitian Xunda(Beijing) Aeronautical Information Technology Co., Ltd.",
    "88F488": "cellon communications technology(shenzhen)Co.,Ltd.",
    "448E12": "DT Research, Inc.",
    "703187": "ACX GmbH",
    "80971B": "Altenergy Power System,Inc.",
    "30E4DB": "Cisco Systems, Inc",
    "ECEA03": "DARFON LIGHTING CORP",
    "6CA780": "Nokia Corporation",
    "582EFE": "Lighting Science Group",
    "30F9ED": "Sony Corporation",
    "ECF236": "NEOMONTANA ELECTRONICS",
    "0418B6": "Private",
    "E4A5EF": "TRON LINK ELECTRONICS CO., LTD.",
    "705CAD": "Konami Gaming Inc",
    "804731": "Packet Design, Inc.",
    "F08BFE": "COSTEL.,CO.LTD",
    "3071B2": "Hangzhou Prevail Optoelectronic Equipment Co.,LTD.",
    "3C6F45": "Fiberpro Inc.",
    "DCCE41": "FE GLOBAL HONG KONG LIMITED",
    "FC6C31": "LXinstruments GmbH",
    "B09BD4": "GNH Software India Private Limited",
    "007F28": "Actiontec Electronics, Inc",
    "CC60BB": "Empower RF Systems",
    "7CDD20": "IOXOS Technologies S.A.",
    "E84E06": "EDUP INTERNATIONAL (HK) CO., LTD",
    "00077D": "Cisco Systems, Inc",
    "E0C922": "Jireh Energy Tech., Ltd.",
    "905F8D": "modas GmbH",
    "98293F": "Fujian Start Computer Equipment Co.,Ltd",
    "0C51F7": "CHAUVIN ARNOUX",
    "0CFC83": "Airoha Technology Corp.,",
    "587675": "Beijing ECHO Technologies Co.,Ltd",
    "CCD9E9": "SCR Engineers Ltd.",
    "34A709": "Trevil srl",
    "3C26D5": "Sotera Wireless",
    "ACC935": "Ness Corporation",
    "008D4E": "CJSC NII STT",
    "444F5E": "Pan Studios Co.,Ltd.",
    "D0AFB6": "Linktop Technology Co., LTD",
    "98EC65": "Cosesy ApS",
    "4C98EF": "Zeo",
    "00A1DE": "ShenZhen ShiHua Technology CO.,LTD",
    "908D1D": "GH Technologies",
    "806CBC": "NET New Electronic Technology GmbH",
    "58E808": "AUTONICS CORPORATION",
    "DC05ED": "Nabtesco  Corporation",
    "98F8DB": "Marini Impianti Industriali s.r.l.",
    "909060": "RSI VIDEO TECHNOLOGIES",
    "BC8199": "BASIC Co.,Ltd.",
    "E0F211": "Digitalwatt",
    "B45CA4": "Thing-talk Wireless Communication Technologies Corporation Limited",
    "DCA7D9": "Compressor Controls Corp",
    "38FEC5": "Ellips B.V.",
    "C455A6": "Cadac Holdings Ltd",
    "5C7757": "Haivision Network Video",
    "D4D898": "Korea CNO Tech Co., Ltd",
    "B4AA4D": "Ensequence, Inc.",
    "040A83": "Alcatel-Lucent",
    "B83D4E": "Shenzhen Cultraview Digital Technology Co.,Ltd Shanghai Branch",
    "5087B8": "Nuvyyo Inc",
    "C0EAE4": "Sonicwall",
    "0838A5": "Funkwerk plettac electronic GmbH",
    "B0A72A": "Ensemble Designs, Inc.",
    "6400F1": "Cisco Systems, Inc",
    "F86971": "Seibu Electric Co.,",
    "44AA27": "udworks Co., Ltd.",
    "E8F928": "RFTECH SRL",
    "CC1EFF": "Metrological Group BV",
    "184E94": "MESSOA TECHNOLOGIES INC.",
    "60F59C": "CRU-Dataport",
    "6C391D": "Beijing ZhongHuaHun Network Information center",
    "80B32A": "UK Grid Solutions Ltd",
    "405539": "Cisco Systems, Inc",
    "C45600": "Galleon Embedded Computing",
    "58EECE": "Icon Time Systems",
    "647FDA": "TEKTELIC Communications Inc.",
    "AC0613": "Senselogix Ltd",
    "747818": "Jurumani Solutions",
    "B8F4D0": "Herrmann Ultraschalltechnik GmbH & Co. Kg",
    "08ACA5": "Benu Video, Inc.",
    "586D8F": "Cisco-Linksys, LLC",
    "10E3C7": "Seohwa Telecom",
    "FCF1CD": "OPTEX-FA CO.,LTD.",
    "4425BB": "Bamboo Entertainment Corporation",
    "E00C7F": "Nintendo Co., Ltd.",
    "1C955D": "I-LAX ELECTRONICS INC.",
    "7465D1": "Atlinks",
    "E48AD5": "RF WINDOW CO., LTD.",
    "443719": "2 Save Energy Ltd",
    "84EA99": "Vieworks",
    "BC3E13": "Accordance Systems Inc.",
    "041D10": "Dream Ware Inc.",
    "801440": "Sunlit System Technology Corp",
    "88DD79": "Voltaire",
    "64F987": "Avvasi Inc.",
    "902E87": "LabJack",
    "DC07C1": "HangZhou QiYang Technology Co.,Ltd.",
    "A81B18": "XTS CORP",
    "D0A311": "Neuberger Geb\u00e4udeautomation GmbH",
    "A424B3": "FlatFrog Laboratories AB",
    "94CDAC": "Creowave Oy",
    "7CDA84": "Dongnian Networks Inc.",
    "C0A26D": "Abbott Point of Care",
    "00BB8E": "HME Co., Ltd.",
    "D82A7E": "Nokia Corporation",
    "801F02": "Edimax Technology Co. Ltd.",
    "180B52": "Nanotron Technologies GmbH",
    "144C1A": "Max Communication GmbH",
    "D85D84": "CAx soft GmbH",
    "D4E32C": "S. Siedle & Sohne",
    "7C6ADB": "SafeTone Technology Co.,Ltd",
    "F05D89": "Dycon Limited",
    "9CF938": "AREVA NP GmbH",
    "8CDB25": "ESG Solutions",
    "BCC61A": "SPECTRA EMBEDDED SYSTEMS",
    "0470BC": "Globalstar Inc.",
    "988E34": "ZHEJIANG BOXSAM ELECTRONIC CO.,LTD",
    "C471FE": "Cisco Systems, Inc",
    "340804": "D-Link Corporation",
    "FC3598": "Favite Inc.",
    "90D92C": "HUG-WITSCHI AG",
    "4022ED": "Digital Projection Ltd",
    "6C2E33": "Accelink Technologies Co.,Ltd.",
    "989449": "Skyworth Wireless Technology Ltd.",
    "2CA157": "acromate, Inc.",
    "942053": "Nokia Corporation",
    "28852D": "Touch Networks",
    "486B91": "Fleetwood Group Inc.",
    "643409": "BITwave Pte Ltd",
    "74CD0C": "Smith Myers Communications Ltd.",
    "CCCE40": "Janteq Corp",
    "B8EE79": "YWire Technologies, Inc.",
    "28ED58": "JAG Jakob AG",
    "B8797E": "Secure Meters (UK) Limited",
    "2005E8": "OOO InProMedia",
    "0006F6": "Cisco Systems, Inc",
    "747DB6": "Aliwei Communications, Inc",
    "B06563": "Shanghai Railway Communication Factory",
    "AC6F4F": "Enspert Inc",
    "E82877": "TMY Co., Ltd.",
    "F0C88C": "LeddarTech Inc.",
    "B0B32B": "Slican Sp. z o.o.",
    "5842E4": "Baxter International Inc",
    "CC9E00": "Nintendo Co., Ltd.",
    "58A76F": "iD corporation",
    "40987B": "Aisino Corporation",
    "BC38D2": "Pandachip Limited",
    "B8BA68": "Xi'an Jizhong Digital Communication Co.,Ltd",
    "4018D7": "Smartronix, Inc.",
    "4C2C80": "Beijing Skyway Technologies Co.,Ltd ",
    "D49E6D": "Wuhan Zhongyuan Huadian Science & Technology Co.,",
    "4037AD": "Macro Image Technology, Inc.",
    "F00248": "SmarteBuilding",
    "40C245": "Shenzhen Hexicom Technology Co., Ltd.",
    "CC55AD": "RIM",
    "E8757F": "FIRS Technologies(Shenzhen) Co., Ltd",
    "F0F7B3": "Phorm",
    "6063FD": "Transcend Communication Beijing Co.,Ltd.",
    "74BE08": "ATEK Products, LLC",
    "74D675": "WYMA Tecnologia",
    "B40EDC": "LG-Ericsson Co.,Ltd.",
    "E0EE1B": "Panasonic Automotive Systems Company of America",
    "60893C": "Thermo Fisher Scientific P.O.A.",
    "D86BF7": "Nintendo Co., Ltd.",
    "00D38D": "Hotel Technology Next Generation",
    "C83EA7": "KUNBUS GmbH",
    "D8B6C1": "NetworkAccountant, Inc.",
    "74A4A7": "QRS Music Technologies, Inc.",
    "40CD3A": "Z3 Technology",
    "482CEA": "Motorola Inc Business Light Radios",
    "F455E0": "Niceway CNC Technology Co.,Ltd.Hunan Province",
    "20FDF1": "3COM EUROPE LTD",
    "A4218A": "Nortel Networks",
    "EC2368": "IntelliVoice Co.,Ltd.",
    "749050": "Renesas Electronics Corporation",
    "7CEF18": "Creative Product Design Pty. Ltd.",
    "9088A2": "IONICS TECHNOLOGY ME LTDA",
    "00336C": "SynapSense Corporation",
    "100C24": "pomdevices, LLC",
    "58F6BF": "Kyoto University",
    "AC6123": "Drivven, Inc.",
    "F866F2": "Cisco Systems, Inc",
    "AC34CB": "Shanhai GBCOM Communication Technology Co. Ltd",
    "700258": "01DB-METRAVIB",
    "8497B8": "Memjet Inc.",
    "A0DDE5": "SHARP Corporation",
    "206AFF": "Atlas Elektronik UK Limited",
    "FC75E6": "Handreamnet",
    "0C15C5": "SDTEC Co., Ltd.",
    "F8FB2F": "Santur Corporation",
    "2CCD43": "Summit Technology Group",
    "145412": "Entis Co., Ltd.",
    "78EC22": "Shanghai Qihui Telecom Technology Co., LTD",
    "64995D": "LGE ",
    "FC7CE7": "FCI USA LLC",
    "3C106F": "ALBAHITH TECHNOLOGIES ",
    "D87157": "Lenovo Mobile Communication Technology Ltd.",
    "C00D7E": "Additech, Inc.",
    "84C7A9": "C3PO S.A.",
    "609AA4": "GVI SECURITY INC.",
    "641084": "HEXIUM Technical Development Co., Ltd.",
    "6C626D": "Micro-Star INT'L CO., LTD",
    "28068D": "ITL, LLC",
    "342109": "Jensen Scandinavia AS",
    "7C3E9D": "PATECH",
    "244597": "GEMUE Gebr. Mueller Apparatebau",
    "78818F": "Server Racks Australia Pty Ltd",
    "284846": "GridCentric Inc.",
    "30694B": "RIM",
    "10CCDB": "AXIMUM PRODUITS ELECTRONIQUES",
    "38C7BA": "CS Services Co.,Ltd.",
    "EC5C69": "MITSUBISHI HEAVY INDUSTRIES MECHATRONICS SYSTEMS,LTD.",
    "E4AD7D": "SCL Elements",
    "F09CBB": "RaonThink Inc.",
    "80EE73": "Shuttle Inc.",
    "FCE23F": "CLAY PAKY SPA",
    "58570D": "Danfoss Solar Inverters",
    "44A8C2": "SEWOO TECH CO., LTD",
    "BCA9D6": "Cyber-Rain, Inc.",
    "ECFE7E": "BlueRadios, Inc.",
    "C4823F": "Fujian Newland Auto-ID Tech. Co,.Ltd.",
    "E85B5B": "LG ELECTRONICS INC",
    "7C2CF3": "Secure Electrans Ltd",
    "081651": "SHENZHEN SEA STAR TECHNOLOGY CO.,LTD",
    "304174": "ALTEC LANSING LLC",
    "3C39C3": "JW Electronics Co., Ltd.",
    "3C05AB": "Product Creation Studio",
    "30EFD1": "Alstom Strongwish (Shenzhen) Co., Ltd.",
    "D41F0C": "JAI Manufacturing",
    "8CD628": "Ikor Metering",
    "243C20": "Dynamode Group",
    "481BD2": "Intron Scientific co., ltd.",
    "F04335": "DVN(Shanghai)Ltd.",
    "A479E4": "KLINFO Corp",
    "8C541D": "LGE ",
    "00A2DA": "INAT GmbH",
    "6C3E9C": "KE Knestel Elektronik GmbH",
    "A863DF": "DISPLAIRE CORPORATION",
    "183BD2": "BYD Precision Manufacture Company Ltd.",
    "4CC602": "Radios, Inc.",
    "D0F0DB": "Ericsson",
    "7C1476": "Damall Technologies SAS",
    "003CC5": "WONWOO Engineering Co., Ltd",
    "F077D0": "Xcellen",
    "884B39": "Siemens AG, Healthcare Sector",
    "D828C9": "General Electric Consumer and Industrial",
    "44C233": "Guangzhou Comet Technology Development Co.Ltd",
    "E43593": "Hangzhou GoTo technology Co.Ltd",
    "2C3A28": "Fagor Electr\u00f3nica",
    "80B289": "Forworld Electronics Ltd.",
    "E83A97": "Toshiba Corporation",
    "1056CA": "Peplink International Ltd.",
    "D49C28": "JayBird LLC",
    "80F593": "IRCO Sistemas de Telecomunicaci\u00f3n S.A.",
    "ECDE3D": "Lamprey Networks, Inc.",
    "6CFFBE": "MPB Communications Inc.",
    "E497F0": "Shanghai VLC Technologies Ltd. Co.",
    "B40832": "TC Communications",
    "88FD15": "LINEEYE CO., LTD",
    "24DBAD": "ShopperTrak RCT Corporation",
    "2872C5": "Smartmatic Corp",
    "486FD2": "StorSimple Inc",
    "A03A75": "PSS Belgium N.V.",
    "B45861": "CRemote, LLC",
    "B0973A": "E-Fuel Corporation",
    "204E6B": "Axxana(israel) ltd",
    "9CEBE8": "BizLink (Kunshan) Co.,Ltd",
    "F06281": "ProCurve Networking by HP",
    "C09C92": "COBY",
    "88ED1C": "Cudo Communication Co., Ltd.",
    "9CCD82": "CHENG UEI PRECISION INDUSTRY CO.,LTD",
    "040EC2": "ViewSonic Mobile China Limited",
    "C8D1D1": "AGAiT Technology Corporation",
    "00DB45": "THAMWAY CO.,LTD.",
    "74F726": "Neuron Robotics",
    "C038F9": "Nokia Danmark A/S",
    "F46349": "Diffon Corporation",
    "003A9C": "Cisco Systems, Inc",
    "889821": "TERAON",
    "E0E751": "Nintendo Co., Ltd.",
    "74F07D": "BnCOM Co.,Ltd",
    "7C6C8F": "AMS NEVE LTD",
    "9CB206": "HMS Industrial Networks",
    "ACE9AA": "Hay Systems Ltd",
    "082AD0": "SRD Innovations Inc.",
    "E08FEC": "REPOTEC CO., LTD.",
    "F852DF": "VNL Europe AB",
    "003AAF": "BlueBit Ltd.",
    "64168D": "Cisco Systems, Inc",
    "D8C3FB": "DETRACOM",
    "A8CB95": "EAST BEST CO., LTD.",
    "F45FF7": "DQ Technology Inc.",
    "7C3BD5": "Imago Group",
    "5CE223": "Delphin Technology AG",
    "F871FE": "The Goldman Sachs Group, Inc.",
    "2C1984": "IDN Telecom, Inc.",
    "40EF4C": "Fihonest communication co.,Ltd",
    "00271E": "Xagyl Communications",
    "00271D": "Comba Telecom Systems (China) Ltd.",
    "002720": "NEW-SOL COM",
    "644F74": "LENUS Co., Ltd.",
    "787F62": "GiK mbH",
    "58F67B": "Xia Men UnionCore Technology LTD.",
    "401597": "Protect America, Inc.",
    "C4FCE4": "DishTV NZ Ltd",
    "EC6C9F": "Chengdu Volans Technology CO.,LTD",
    "E80B13": "Akib Systems Taiwan, INC",
    "0026F7": "Nivetti Systems Pvt. Ltd.",
    "0026F6": "Military Communication Institute",
    "0026F0": "cTrixs International GmbH.",
    "0026EA": "Cheerchip Electronic Technology (ShangHai) Co., Ltd.",
    "0026DD": "Fival Science & Technology Co.,Ltd.",
    "0026DE": "FDI MATELEC",
    "0026DA": "Universal Media Corporation /Slovakia/ s.r.o.",
    "0026DB": "Ionics EMS Inc.",
    "0026C9": "Proventix Systems, Inc.",
    "0026BF": "ShenZhen Temobi Science&Tech Development Co.,Ltd",
    "0026D5": "Ory Solucoes em Comercio de Informatica Ltda.",
    "0026CE": "Kozumi USA Corp.",
    "0026CA": "Cisco Systems, Inc",
    "002708": "Nordiag ASA",
    "002702": "SolarEdge Technologies",
    "0026FA": "BandRich Inc.",
    "0026F9": "S.E.M. srl",
    "0026FD": "Interactive Intelligence",
    "0026B4": "Ford Motor Company",
    "0026AC": "Shanghai LUSTER Teraband photonic Co., Ltd.",
    "0026A6": "TRIXELL",
    "002711": "LanPro Inc",
    "00268F": "MTA SpA",
    "002686": "Quantenna Communcations, Inc.",
    "002684": "KISAN SYSTEM",
    "002680": "SIL3 Pty.Ltd",
    "002638": "Xia Men Joyatech Co., Ltd.",
    "00263A": "Digitec Systems",
    "002635": "Bluetechnix GmbH",
    "002617": "OEM Worldwide",
    "002613": "Engel Axil S.L.",
    "00260F": "Linn Products Ltd",
    "00260C": "Dataram",
    "002620": "ISGUS GmbH",
    "00261D": "COP SECURITY SYSTEM CORP.",
    "00262B": "Wongs Electronics Co. Ltd.",
    "002694": "Senscient Ltd",
    "002690": "I DO IT",
    "002679": "Euphonic Technologies, Inc.",
    "002676": "COMMidt AS",
    "002666": "EFM Networks",
    "002657": "OOO NPP EKRA",
    "002652": "Cisco Systems, Inc",
    "0025F6": "netTALK.com, Inc.",
    "0025F5": "DVS Korea, Co., Ltd",
    "0025EB": "Reutech Radar Systems (PTY) Ltd",
    "0025EE": "Avtex Ltd",
    "0025D6": "The Kroger Co.",
    "0025D1": "Eastern Asia Technology Limited",
    "0025CD": "Skylane Optics",
    "00258C": "ESUS ELEKTRONIK SAN. VE DIS. TIC. LTD. STI.",
    "0025A5": "Walnut Media Network",
    "0025A4": "EuroDesign embedded technologies GmbH",
    "0025AE": "Microsoft Corporation",
    "0025AF": "COMFILE Technology",
    "0025A8": "Kontron (BeiJing) Technology Co.,Ltd",
    "002601": "Cutera Inc",
    "0025BF": "Wireless Cables Inc.",
    "0025B9": "Cypress Solutions Inc",
    "0025B6": "Telecom FM",
    "0025D9": "DataFab Systems Inc.",
    "0025C1": "Nawoo Korea Corp.",
    "00257A": "CAMCO Produktions- und Vertriebs-GmbH f\u00fcr  Beschallungs- und Beleuchtungsanlagen",
    "002576": "NELI TECHNOLOGIES",
    "002574": "KUNIMI MEDIA DEVICE Co., Ltd.",
    "002573": "ST Electronics (Info-Security) Pte Ltd",
    "00256F": "Dantherm Power",
    "002554": "Pixel8 Networks",
    "00255A": "Tantalus Systems Corp.",
    "002559": "Syphan Technologies Ltd",
    "002530": "Aetas Systems Inc.",
    "00252C": "Entourage Systems, Inc.",
    "00252F": "Energy, Inc.",
    "00250F": "On-Ramp Wireless, Inc.",
    "002505": "eks Engel GmbH & Co. KG",
    "002540": "Quasar Technologies, Inc.",
    "002533": "WITTENSTEIN AG",
    "002587": "Vitality, Inc.",
    "002523": "OCP Inc.",
    "00251D": "DSA Encore, LLC",
    "00250A": "Security Expert Co. Ltd",
    "002509": "SHARETRONIC Group LTD",
    "0024D8": "IlSung Precision",
    "0024CD": "Willow Garage, Inc.",
    "0024D3": "QUALICA Inc.",
    "0024CE": "Exeltech Inc",
    "0024CF": "Inscape Data Corporation",
    "0024C6": "Hager Electro SAS",
    "0024B4": "ESCATRONIC GmbH",
    "0024B1": "Coulomb Technologies",
    "00249C": "Bimeng Comunication System Co. Ltd",
    "002498": "Cisco Systems, Inc",
    "0024FD": "Accedian Networks Inc",
    "0024F5": "NDS Surgical Imaging",
    "0024DA": "Innovar Systems Limited",
    "00246A": "Solid Year Co., Ltd.",
    "002467": "AOC International (Europe) GmbH",
    "00248B": "HYBUS CO., LTD.",
    "002492": "Motorola, Broadband Solutions Group",
    "002484": "Bang and Olufsen Medicom a/s",
    "002452": "Silicon Software GmbH",
    "002453": "Initra d.o.o.",
    "00244C": "Solartron Metrology Ltd",
    "002448": "SpiderCloud Wireless, Inc",
    "00244B": "PERCEPTRON INC",
    "00242A": "Hittite Microwave Corporation",
    "002422": "Knapp Logistik Automation GmbH",
    "00241B": "iWOW Communications Pte Ltd",
    "002428": "EnergyICT",
    "002417": "Thomson Telecom Belgium",
    "00247A": "FU YI CHENG Technology Co., Ltd.",
    "002476": "TAP.tv",
    "00246D": "Weinzierl Engineering GmbH",
    "002442": "Axona Limited",
    "00243C": "S.A.A.A.",
    "00245E": "Hivision Co.,ltd",
    "00244D": "Hokkaido Electronics Corporation",
    "002416": "Any Use",
    "002409": "The Toro Company",
    "002406": "Pointmobile",
    "002400": "Nortel Networks",
    "0023E1": "Cavena Image Products AB",
    "0023DC": "Benein, Inc",
    "0023D1": "TRG",
    "0023D3": "AirLink WiFi Networking Corp.",
    "0023FB": "IP Datatel, LLC.",
    "0023FE": "Biodevices, SA",
    "0023F4": "Masternaut",
    "0023CA": "Behind The Set, LLC",
    "0023CB": "Shenzhen Full-join Technology Co.,Ltd",
    "0023D0": "Uniloc USA Inc.",
    "0023AD": "Xmark Corporation",
    "0023AB": "Cisco Systems, Inc",
    "0023A4": "New Concepts Development Corp.",
    "002385": "ANTIPODE",
    "0023C7": "AVSystem sp. z o. o.",
    "0023C3": "LogMeIn, Inc.",
    "0023B0": "COMXION Technology Inc.",
    "002303": "LITE-ON IT Corporation",
    "0022FE": "Advanced Illumination",
    "002300": "Cayee Computer Ltd.",
    "0022F3": "SHARP Corporation",
    "0022F6": "Syracuse Research Corporation",
    "00232C": "Senticare",
    "00232B": "IRD A/S",
    "00232A": "eonas IT-Beratung und -Entwicklung GmbH",
    "002327": "Shouyo Electronics CO., LTD",
    "002328": "ALCON TELECOMMUNICATIONS CO., LTD.",
    "002372": "MORE STAR INDUSTRIAL GROUP LIMITED",
    "00236D": "ResMed Ltd",
    "00236B": "Xembedded, Inc.",
    "00237E": "ELSTER GMBH",
    "00237C": "NEOTION",
    "00237A": "RIM",
    "00234C": "KTC AB",
    "002343": "TEM AG",
    "002336": "METEL s.r.o.",
    "002338": "OJ-Electronics A/S",
    "002361": "Unigen Corporation",
    "00235F": "Silicon Micro Sensors GmbH",
    "00233B": "C-Matic Systems Ltd",
    "00231A": "ITF Co., Ltd.",
    "00230D": "Nortel Networks",
    "0022ED": "TSI Power Corporation",
    "0022E1": "ZORT Labs, LLC.",
    "0022E0": "Atlantic Software Technologies S.r.L.",
    "0022DF": "TAMUZ Monitors",
    "0022D8": "Shenzhen GST Security and Safety Technology Limited",
    "0022DC": "Vigil Health Solutions Inc.",
    "00228C": "Photon Europe GmbH",
    "00228B": "Kensington Computer Products Group",
    "00228D": "GBS Laboratories LLC",
    "002289": " Vanderlande APC inc.",
    "00226D": "Shenzhen GIEC Electronics Co., Ltd.",
    "00226E": "Gowell Electronic Limited",
    "00225D": "Digicable Network India Pvt. Ltd.",
    "002256": "Cisco Systems, Inc",
    "0022D9": "Fortex Industrial Ltd.",
    "0022D3": "Hub-Tech",
    "0022D4": "ComWorth Co., Ltd.",
    "00229F": "Sensys Traffic AB",
    "002299": "SeaMicro Inc.",
    "0022BC": "JDSU France SAS",
    "0022AA": "Nintendo Co., Ltd.",
    "0022A8": "Ouman Oy",
    "0022C9": "Lenord, Bauer & Co GmbH",
    "00227D": "YE DATA INC.",
    "00227B": "Apogee Labs, Inc.",
    "002223": "TimeKeeping Systems, Inc.",
    "00221A": "Audio Precision",
    "00224B": "AIRTECH TECHNOLOGIES, INC.",
    "00223C": "RATIO Entwicklungen GmbH",
    "002252": "ZOLL Lifecor Corporation",
    "00224D": "MITAC INTERNATIONAL CORP.",
    "00224C": "Nintendo Co., Ltd.",
    "00224A": "RAYLASE AG",
    "0021E3": "SerialTek LLC",
    "0021E7": "Informatics Services Corporation",
    "0021DC": "TECNOALARM S.r.l.",
    "0021F8": "Enseo, Inc.",
    "0021F3": "Si14 SpA",
    "0021F1": "Tutus Data AB",
    "002213": "PCI CORPORATION",
    "00221C": "Private",
    "002222": "Schaffner Deutschland GmbH",
    "002231": "SMT&C Co., Ltd.",
    "002201": "Aksys Networks Inc",
    "00219A": "Cambridge Visual Networks Ltd",
    "002193": "Videofon MV",
    "002192": "Baoding Galaxy Electronic Technology  Co.,Ltd",
    "00217B": "Bastec AB",
    "002176": "YMax Telecom Ltd.",
    "002171": "Wesung TNC Co., Ltd.",
    "00215F": "IHSE GmbH",
    "002156": "Cisco Systems, Inc",
    "002151": "Millinet Co., Ltd.",
    "0021A7": "Hantle System Co., Ltd.",
    "00219C": "Honeywld Technology Corp.",
    "0021D8": "Cisco Systems, Inc",
    "0021D7": "Cisco Systems, Inc",
    "0021D9": "SEKONIC CORPORATION",
    "0021DA": "Automation Products Group Inc.",
    "00216C": "ODVA",
    "002161": "Yournet Inc.",
    "002189": "AppTech, Inc.",
    "0021BD": "Nintendo Co., Ltd.",
    "0021B5": "Galvanic Ltd",
    "0021C4": "Consilium AB",
    "002122": "Chip-pro Ltd.",
    "002125": "KUK JE TONG SHIN Co.,LTD",
    "002126": "Shenzhen Torch Equipment Co., Ltd.",
    "00211C": "Cisco Systems, Inc",
    "001FF9": "Advanced Knowledge Associates",
    "001FF4": "Power Monitors, Inc.",
    "002135": "ALCATEL-LUCENT",
    "00210E": "Orpak Systems L.T.D.",
    "002118": "Athena Tech, Inc.",
    "002113": "Padtec S/A",
    "002112": "WISCOM SYSTEM CO.,LTD",
    "002147": "Nintendo Co., Ltd.",
    "002149": "China Daheng Group ,Inc.",
    "001FED": "Tecan Systems Inc.",
    "001FE5": "In-Circuit GmbH",
    "002138": "Cepheid",
    "001F7B": "TechNexion Ltd.",
    "001F7D": "Embedded Wireless GmbH",
    "001F74": "Eigen Development",
    "001F75": "GiBahn Media",
    "001FB6": "Chi Lin Technology Co., Ltd.",
    "001FAF": "NextIO, Inc.",
    "001FAA": "Taseon, Inc.",
    "001FBC": "EVGA Corporation",
    "001FBD": "Kyocera Wireless Corp.",
    "001FB9": "Paltronics",
    "001FB7": "WiMate Technologies Corp.",
    "001FB4": "SmartShare Systems",
    "001F9E": "Cisco Systems, Inc",
    "001FA0": "A10 Networks",
    "001F6E": "Vtech Engineering Corporation",
    "001F66": "PLANAR LLC",
    "001F5F": "Blatand GmbH",
    "001F8D": "Ingenieurbuero Stark GmbH und Ko. KG",
    "001FC5": "Nintendo Co., Ltd.",
    "001FC0": "Control Express Finland Oy",
    "001F8F": "Shanghai Bellmann Digital Source Co.,Ltd.",
    "001F84": "Gigle Semiconductor",
    "001EE6": "Shenzhen Advanced Video Info-Tech Co., Ltd.",
    "001EF5": "Hitek Automated Inc.",
    "001EF7": "Cisco Systems, Inc",
    "001ED8": "Digital United Inc.",
    "001ED6": "Alentec & Orion AB",
    "001F1E": "Astec Technology Co., Ltd",
    "001F14": "NexG",
    "001F0C": "Intelligent Digital Services GmbH",
    "001F08": "RISCO LTD",
    "001EF1": "Servimat",
    "001EF4": "L-3 Communications Display Systems",
    "001F0E": "Japan Kyastem Co., Ltd",
    "001F0B": "Federal State Unitary Enterprise Industrial Union\"Electropribor\"",
    "001F57": "Phonik Innovation Co.,LTD",
    "001F59": "Kronback Tracers",
    "001F4E": "ConMed Linvatec",
    "001F4A": "Albentia Systems S.A.",
    "001F43": "ENTES ELEKTRONIK",
    "001F28": "HPN Supply Chain",
    "001E77": "Air2App",
    "001E7B": "R.I.CO. S.r.l.",
    "001E8B": "Infra Access Korea Co., Ltd.",
    "001E85": "Lagotek Corporation",
    "001EB5": "Ever Sparkle Technologies Ltd",
    "001EB3": "Primex Wireless",
    "001EB6": "TAG Heuer SA",
    "001EAC": "Armadeus Systems",
    "001EAA": "E-Senza Technologies GmbH",
    "001E3F": "TrellisWare Technologies, Inc.",
    "001E55": "COWON SYSTEMS,Inc.",
    "001E56": "Bally Wulff Entertainment GmbH",
    "001E57": "ALCOMA, spol. s r.o.",
    "001EC4": "Celio Corp",
    "001EC1": "3COM EUROPE LTD",
    "001EBB": "BLUELIGHT TECHNOLOGY INC.",
    "001E9C": "Fidustron INC",
    "001E97": "Medium Link System Technology CO., LTD,",
    "001E6E": "Shenzhen First Mile Communications Ltd",
    "001E6D": "IT R&D Center",
    "001E50": "BATTISTONI RESEARCH",
    "001E41": "Microwave Communication & Component, Inc.",
    "001DFC": "KSIC",
    "001DF2": "Netflix, Inc.",
    "001DEF": "TRIMM, INC.",
    "001DF1": "Intego Systems, Inc.",
    "001DC3": "RIKOR TV, Ltd",
    "001DC1": "Audinate Pty L",
    "001DED": "Grid Net, Inc.",
    "001E34": "CryptoMetrics",
    "001E2D": "STIM",
    "001DB2": "HOKKAIDO ELECTRIC ENGINEERING CO.,LTD.",
    "001DAD": "Sinotech Engineering Consultants, Inc.  Geotechnical Enginee",
    "001DD8": "Microsoft Corporation",
    "001DC8": "Navionics Research Inc., dba SCADAmetrics",
    "001DCC": "Ayon Cyber Security, Inc",
    "001DAB": "SwissQual License AG",
    "001DA0": "Heng Yu Electronic Manufacturing Company Limited",
    "001E11": "ELELUX INTERNATIONAL LTD",
    "001D83": "Emitech Corporation",
    "001D74": "Tianjin China-Silicon Microelectronics Co., Ltd.",
    "001D6D": "Confidant International LLC",
    "001D7C": "ABE Elettronica S.p.A.",
    "001D4B": "Grid Connect Inc.",
    "001D4D": "Adaptive Recognition Hungary, Inc",
    "001D8A": "TechTrex Inc",
    "001D87": "VigTech Labs Sdn Bhd",
    "001D80": "Beijing Huahuan Eletronics Co.,Ltd",
    "001D58": "CQ Inc",
    "001D53": "S&O Electronics (Malaysia) Sdn. Bhd.",
    "001D54": "Sunnic Technology & Merchandise INC.",
    "001D1D": "Inter-M Corporation",
    "001D0E": "Agapha Technology co., Ltd.",
    "001D75": "Radioscape PLC",
    "001D5F": "OverSpeed SARL",
    "001D92": "MICRO-STAR INT'L CO.,LTD.",
    "001D2F": "QuantumVision Corporation",
    "001CED": "ENVIRONNEMENT SA",
    "001CE5": "MBS Electronic Systems GmbH",
    "001CDD": "COWBELL ENGINEERING CO., LTD.",
    "001CC2": "Part II Research, Inc.",
    "001CBD": "Ezze Mobile Tech., Inc.",
    "001CB8": "CBC Co., Ltd",
    "001D0A": "Davis Instruments, Inc.",
    "001D05": "Cooper Lighting Solutions",
    "001CFF": "Napera Networks Inc",
    "001C82": "Genew Technologies",
    "001C84": "STL Solution Co.,Ltd.",
    "001C79": "Cohesive Financial Technologies LLC",
    "001CA5": "Zygo Corporation",
    "001C9D": "Liecthi AG",
    "001C96": "Linkwise Technology Pte Ltd",
    "001CBA": "VerScient, Inc.",
    "001CB0": "Cisco Systems, Inc",
    "001CAC": "Qniq Technology Corp.",
    "001CDC": "Custom Computer Services, Inc.",
    "001CD1": "Waves Audio LTD",
    "001C8B": "MJ Innovations Ltd.",
    "001C57": "Cisco Systems, Inc",
    "001C5D": "Leica Microsystems",
    "001C5E": "ASTON France",
    "001C5B": "Chubb Electronic Security Systems Ltd",
    "001C55": "Shenzhen Kaifa Technology Co.",
    "001C44": "Bosch Security Systems BV",
    "001C4C": "Petrotest Instruments",
    "001C3C": "Seon Design Inc.",
    "001C29": "CORE DIGITAL ELECTRONICS CO., LTD",
    "001C24": "Formosa Wireless Systems Corp.",
    "001C20": "CLB Benelux",
    "001C1C": "Center Communication Systems GmbH",
    "001C31": "Mobile XP Technology Co., LTD",
    "001C34": "HUEY CHIAO INTERNATIONAL CO., LTD.",
    "001C36": "iNEWiT NV",
    "001C07": "Cwlinux Limited",
    "001C00": "Entry Point, LLC",
    "001BFD": "Dignsys Inc.",
    "001C6D": "KYOHRITSU ELECTRONIC INDUSTRY CO., LTD.",
    "001C64": "Landis+Gyr",
    "001BE6": "VR AG",
    "001BE2": "AhnLab,Inc.",
    "001BE0": "TELENOT ELECTRONIC GmbH",
    "001BDA": "UTStarcom Inc",
    "001BF5": "Tellink Sistemas de Telecomunicaci\u00f3n S.L.",
    "001C15": "iPhotonix LLC",
    "001BA3": "Flexit Group GmbH",
    "001B9F": "Calyptech Pty Ltd",
    "001B99": "KS System GmbH",
    "001B9A": "Apollo Fire Detectors Ltd",
    "001BBD": "FMC Kongsberg Subsea AS",
    "001BBE": "ICOP Digital",
    "001BB3": "Condalo GmbH",
    "001BB7": "Alta Heights Technology Corp.",
    "001BAC": "Curtiss Wright Controls Embedded Computing",
    "001B60": "NAVIGON AG",
    "001B57": "SEMINDIA SYSTEMS PRIVATE LIMITED",
    "001B55": "Hurco Automation Ltd.",
    "001B53": "Cisco Systems, Inc",
    "001BD1": "SOGESTMATIC",
    "001BD6": "Kelvin Hughes Ltd",
    "001BCF": "Dataupia Corporation",
    "001B8B": "NEC Platforms, Ltd.",
    "001B82": "Taiwan Semiconductor Co., Ltd.",
    "001B85": "MAN Energy Solutions",
    "001B8C": "JMicron Technology Corp.",
    "001B91": "EFKON AG",
    "001B89": "EMZA Visual Sense Ltd.",
    "001B6A": "Powerwave Technologies Sweden AB",
    "001B67": "Cisco Systems Inc",
    "001B66": "Sennheiser electronic GmbH & Co. KG",
    "001BCB": "PEMPEK SYSTEMS PTY LTD",
    "001B7B": "The Tintometer Ltd",
    "001B75": "Hypermedia Systems",
    "001AD9": "International Broadband Electric Communications, Inc.",
    "001B31": "Neural Image. Co. Ltd.",
    "001B29": "Avantis.Co.,Ltd",
    "001B28": "POLYGON, JSC",
    "001B2B": "Cisco Systems, Inc",
    "001B27": "Merlin CSI",
    "001B0E": "InoTec GmbH Organisationssysteme",
    "001B04": "Affinity International S.p.a",
    "001B06": "Ateliers R. LAUMONIER",
    "001B05": "YMC AG",
    "001AFF": "Wizyoung Tech.",
    "001B00": "Neopost Technologies",
    "001B43": "Beijing DG Telecommunications equipment Co.,Ltd",
    "001B3C": "Software Technologies Group,Inc.",
    "001B3D": "EuroTel Spa",
    "001B1B": "Siemens AG,",
    "001B1F": "FORCE Technology",
    "001B49": "Roberts Radio limited",
    "001B4E": "Navman New Zealand",
    "001B16": "Celtro Ltd.",
    "001AE6": "Atlanta Advanced Communications Holdings Limited",
    "001AF2": "Dynavisions Schweiz AG",
    "001A7A": "Lismore Instruments Limited",
    "001A78": "ubtos",
    "001A76": "SDT information Technology Co.,LTD.",
    "001A70": "Cisco-Linksys, LLC",
    "001A60": "Wave Electronics Co.,Ltd.",
    "001A56": "ViewTel Co,. Ltd.",
    "001A50": "PheeNet Technology Corp.",
    "001A53": "Zylaya",
    "001A4C": "Crossbow Technology, Inc",
    "001A7D": "cyber-blue(HK)Ltd",
    "001A82": "PROBA Building Automation Co.,LTD",
    "001A7C": "Hirschmann Multimedia B.V.",
    "001AAB": "eWings s.r.l.",
    "001AAC": "Corelatus AB",
    "001AAF": "BLUSENS TECHNOLOGY",
    "001AB0": "Signal Networks Pvt. Ltd.,",
    "001AC8": "ISL (Instrumentation Scientifique de Laboratoire)",
    "001AC6": "Micro Control Designs",
    "001A61": "PacStar Corp.",
    "001A65": "Seluxit",
    "001A54": "Hip Shing Electronics Ltd.",
    "001AA1": "Cisco Systems, Inc",
    "0019F8": "Embedded Systems Design, Inc.",
    "0019EF": "SHENZHEN LINNKING ELECTRONICS CO.,LTD",
    "0019F3": "Cetis, Inc",
    "0019F5": "Imagination Technologies Ltd",
    "0019F7": "Onset Computer Corporation",
    "0019EE": "CARLO GAVAZZI CONTROLS SPA-Controls Division",
    "0019EB": "Pyronix Ltd",
    "0019E7": "Cisco Systems, Inc",
    "0019E9": "S-Information Technolgy, Co., Ltd.",
    "0019DA": "Welltrans O&E Technology Co. , Ltd.",
    "0019D0": "Cathexis",
    "001A14": "Xin Hua Control Engineering Co.,Ltd.",
    "001A10": "LUCENT TRANS ELECTRONICS CO.,LTD",
    "001A0C": "Swe-Dish Satellite Systems AB",
    "001A07": "Arecont Vision",
    "001A08": "Simoco Ltd.",
    "001A04": "Interay Solutions BV",
    "001A02": "SECURE CARE PRODUCTS, INC",
    "001A1A": "Gentex Corporation/Electro-Acoustic Products",
    "001A12": "Essilor",
    "0019D6": "LS Cable and System Ltd.",
    "0019D7": "FORTUNETEK CO., LTD",
    "0019C9": "S&C ELECTRIC COMPANY",
    "001A42": "Techcity Technology co., Ltd.",
    "0019AC": "GSP SYSTEMS Inc.",
    "0019B0": "HanYang System",
    "001995": "Jurong Hi-Tech (Suzhou)Co.ltd",
    "00199A": "EDO-EVI",
    "001994": "Jorjin Technologies Inc.",
    "0019BF": "Citiway technology Co.,ltd",
    "0019B6": "Euro Emme s.r.l.",
    "0019B5": "Famar Fueguina S.A.",
    "00195D": "ShenZhen XinHuaTong Opto Electronics Co.,Ltd",
    "001953": "Chainleader Communications Corp.",
    "001955": "Cisco Systems, Inc",
    "001959": "Staccato Communications Inc.",
    "00194D": "Avago Technologies Sdn Bhd",
    "00194E": "Ultra Electronics - TCS (Tactical Communication Systems)",
    "00197C": "Riedel Communications GmbH",
    "00196F": "SensoPart GmbH",
    "0019A0": "NIHON DATA SYSTENS, INC.",
    "001991": "avinfo",
    "00198C": "iXSea",
    "001962": "Commerciant, LP",
    "0019B8": "Boundary Devices",
    "0018E4": "YIGUANG",
    "0018E5": "Adhoco AG",
    "0018DD": "Silicondust Engineering Ltd",
    "0018DF": "The Morey Corporation",
    "0018E1": "Verkerk Service Systemen",
    "0018DA": "W\u00fcrth Elektronik eiSos GmbH & Co. KG",
    "0018D5": "REIGNCOM",
    "0018D4": "Unified Display Interface SIG",
    "001915": "TECOM Co., Ltd.",
    "00191B": "Sputnik Engineering AG",
    "001909": "DEVI - Danfoss A/S",
    "00190A": "HASWARE INC.",
    "001904": "WB Electronics Sp. z o.o.",
    "001934": "TRENDON TOUCH TECHNOLOGY CORP.",
    "001923": "Phonex Korea Co., LTD.",
    "00191C": "Sensicast Systems",
    "001933": "Strix Systems, Inc.",
    "001928": "Siemens AG, Transportation Systems",
    "00190D": "IEEE 1394c",
    "0018C2": "Firetide, Inc",
    "0018E7": "Cameo Communications, INC.",
    "001838": "PanAccess Communications,Inc.",
    "001821": "SINDORICOH",
    "001823": "Delta Electronics, Inc.",
    "001815": "GZ Technologies, Inc.",
    "001818": "Cisco Systems, Inc",
    "0018A9": "Ethernet Direct Corporation",
    "0018A7": "Yoggie Security Systems LTD.",
    "0018A8": "AnNeal Technology Inc.",
    "0018A2": "XIP Technology AB",
    "00189C": "Weldex Corporation",
    "00189A": "HANA Micron Inc.",
    "00185F": "TAC Inc.",
    "001864": "Eaton Corporation",
    "001866": "Leutron Vision",
    "001860": "SIM Technology Group Shanghai Simcom Ltd.,",
    "00183E": "Digilent, Inc",
    "001842": "Nokia Danmark A/S",
    "001840": "3 Phoenix, Inc.",
    "001857": "Unilever R&D",
    "001853": "Atera Networks LTD.",
    "001859": "Strawberry Linux Co.,Ltd.",
    "00184F": "8 Ways Technology Corp.",
    "001873": "Cisco Systems, Inc",
    "00187A": "Wiremold",
    "00186C": "Neonode AB",
    "001844": "Heads Up Technologies, Inc.",
    "00182B": "Softier",
    "001829": "Gatsometer",
    "001881": "Buyang Electronics Industrial Co., Ltd",
    "001897": "JESS-LINK PRODUCTS Co., LTD",
    "00189E": "OMNIKEY GmbH.",
    "00181A": "AVerMedia Information Inc.",
    "001816": "Ubixon Co., Ltd.",
    "0017B5": "Peerless Systems Corporation",
    "0017AF": "Enermet",
    "0017AA": "elab-experience inc.",
    "0017AE": "GAI-Tronics",
    "0017A5": "Ralink Technology Corp",
    "0017A8": "EDM Corporation",
    "0017A9": "Sentivision",
    "0017C5": "SonicWALL",
    "0017BE": "Tratec Telecom B.V.",
    "0017C0": "PureTech Systems, Inc.",
    "0017BA": "SEDO CO., LTD.",
    "0017D4": "Monsoon Multimedia, Inc",
    "0017F1": "Renu Electronics Pvt Ltd",
    "0017FF": "PLAYLINE Co.,Ltd.",
    "0017F6": "Pyramid Meriden Inc.",
    "001806": "Hokkei Industries Co., Ltd.",
    "0017F0": "SZCOM Broadband Network Technology Co.,Ltd",
    "00178B": "Teledyne Technologies Incorporated",
    "001780": "Applied Biosystems B.V.",
    "0017DF": "Cisco Systems, Inc",
    "00172F": "NeuLion Incorporated",
    "001728": "Selex Communications",
    "001746": "Freedom9 Inc.",
    "00174D": "DYNAMIC NETWORK FACTORY, INC.",
    "001744": "Araneo Ltd.",
    "001749": "HYUNDAE YONG-O-SA CO.,LTD",
    "001743": "Deck Srl",
    "00173D": "Neology",
    "001740": "Bluberi Gaming Technologies Inc",
    "001732": "Science-Technical Center \"RISSA\"",
    "00173A": "Cloudastructure Inc",
    "001770": "Arti Industrial Electronics Ltd.",
    "00176D": "CORE CORPORATION",
    "001773": "Laketune Technologies Co. Ltd",
    "001722": "Hanazeder Electronic GmbH",
    "001723": "Summit Data Communications",
    "00176C": "Pivot3, Inc.",
    "00171F": "IMV Corporation",
    "001757": "RIX TECHNOLOGY LIMITED",
    "0016BB": "Law-Chain Computer Technology Co Ltd",
    "0016B4": "Private",
    "0016A7": "AWETA G&P",
    "0016BF": "PaloDEx Group Oy",
    "0016B7": "Seoul Commtech",
    "0016A0": "Auto-Maskin",
    "0016E1": "SiliconStor, Inc.",
    "0016E2": "American Fibertek, Inc.",
    "001713": "Tiger NetCom",
    "0016CD": "HIJI HIGH-TECH CO., LTD.",
    "0016EF": "Koko Fitness, Inc.",
    "0016FD": "Jaty Electronics",
    "00168D": "KORWIN CO., Ltd.",
    "00168E": "Vimicro corporation",
    "001689": "Pilkor Electronics Co., Ltd",
    "00168A": "id-Confirm Inc",
    "001686": "Karl Storz Imaging",
    "00167E": "DIBOSS.CO.,LTD",
    "00167B": "Haver&Boecker",
    "001679": "eOn Communications",
    "001678": "SHENZHEN BAOAN GAOKE ELECTRONICS CO., LTD",
    "001674": "EuroCB (Phils.), Inc.",
    "00164A": "Vibration Technology Limited",
    "001645": "Power Distribution, Inc.",
    "00163F": "CReTE SYSTEMS Inc.",
    "00163D": "Tsinghua Tongfang Legend Silicon Tech. Co., Ltd.",
    "00163A": "YVES TECHNOLOGY CO., LTD.",
    "001638": "TECOM Co., Ltd.",
    "001627": "embedded-logic DESIGN AND MORE GmbH",
    "001622": "BBH SYSTEMS GMBH",
    "001613": "LibreStream Technologies Inc.",
    "001633": "Oxford Diagnostics Ltd.",
    "00162D": "STNet Co., Ltd.",
    "00164F": "World Ethnic Broadcastin Inc.",
    "001670": "SKNET Corporation",
    "001667": "A-TEC Subsystem INC.",
    "00165D": "AirDefense, Inc.",
    "00165B": "Grip Audio",
    "0015CD": "Exartech International Corp.",
    "0015C9": "Gumstix, Inc",
    "0015BA": "iba AG",
    "0015BB": "SMA Solar Technology AG",
    "0015EC": "Boca Devices LLC",
    "0015EF": "NEC TOKIN Corporation",
    "0015E4": "Zimmer Elektromedizin",
    "0015BF": "technicob",
    "0015BC": "Develco",
    "0015BD": "Group 4 Technology Ltd",
    "0015B5": "CI Network Corp.",
    "0015FB": "setex schermuly textile computer gmbh",
    "0015FE": "SCHILLING ROBOTICS LLC",
    "0015AE": "kyung il",
    "00160F": "BADGER METER INC",
    "001604": "Sigpro",
    "00159C": "B-KYUNG SYSTEM Co.,Ltd.",
    "001595": "Quester Tangent Corporation",
    "0015D9": "PKC Electronics Oy",
    "0015DD": "IP Control Systems Ltd.",
    "00160D": "Be Here Corporation",
    "00151A": "Hunter Engineering Company",
    "001514": "Hu Zhou NAVA Networks&Electronics Ltd.",
    "001516": "URIEL SYSTEMS INC.",
    "001565": "XIAMEN YEALINK NETWORK TECHNOLOGY CO.,LTD",
    "001569": "PECO II, Inc.",
    "001563": "Cisco Systems, Inc",
    "001564": "BEHRINGER Spezielle Studiotechnik GmbH",
    "001582": "Pulse Eight Limited",
    "00157B": "Leuze electronic GmbH + Co. KG",
    "001578": "Audio / Video Innovations",
    "001571": "Nolan Systems",
    "001543": "Aberdeen Test Center",
    "001541": "StrataLight Communications, Inc.",
    "001561": "JJPlus Corporation",
    "001558": "FOXCONN",
    "001538": "RFID, Inc.",
    "00152E": "PacketHop, Inc.",
    "00154F": "one RF Technology",
    "001545": "SEECODE Co., Ltd.",
    "00151F": "Multivision Intelligent Surveillance (Hong Kong) Ltd",
    "001522": "Dea Security",
    "001587": "Takenaka Seisakusho Co.,Ltd",
    "0014EF": "TZero Technologies, Inc.",
    "0014EE": "Western Digital Technologies, Inc.",
    "0014A3": "Vitelec BV",
    "001497": "ZHIYUAN Eletronics co.,ltd.",
    "001499": "Helicomm Inc",
    "001492": "Liteon, Mobile Media Solution SBU",
    "001494": "ESU AG",
    "0014BC": "SYNECTIC TELECOM EXPORTS PVT. LTD.",
    "0014B9": "MSTAR SEMICONDUCTOR",
    "0014BD": "incNETWORKS, Inc",
    "0014B7": "AR Infotek Inc.",
    "0014B5": "PHYSIOMETRIX,INC",
    "0014CA": "Key Radio Systems Limited",
    "0014C8": "Contemporary Research Corp",
    "0014C5": "Alive Technologies Pty Ltd",
    "0014D3": "SEPSA",
    "0014D8": "bio-logic SA",
    "0014DA": "Huntleigh Healthcare",
    "0014D5": "Datang Telecom Technology CO. , LCD,Optical Communication Br",
    "001485": "Giga-Byte",
    "001483": "eXS Inc.",
    "00149C": "HF Company",
    "001501": "LexBox",
    "001412": "S-TEC electronics AG",
    "001411": "Deutschmann Automation GmbH & Co. KG",
    "001405": "OpenIB, Inc.",
    "004501": "Midmark RTLS",
    "001460": "Kyocera Wireless Corp.",
    "00145C": "Intronics B.V.",
    "00145A": "Westermo Neratec AG",
    "00145B": "SeekerNet Inc.",
    "001443": "Consultronics Europe Ltd",
    "001449": "Sichuan Changhong Electric Ltd.",
    "001446": "SuperVision Solutions LLC",
    "001440": "ATOMIC Corporation",
    "00142E": "77 Elektronika Kft.",
    "001430": "ViPowER, Inc",
    "001432": "Tarallax Wireless, Inc.",
    "0013F5": "Akimbi Systems",
    "0013F1": "AMOD Technology Co., Ltd.",
    "00141F": "SunKwang Electronics Co., Ltd",
    "00141D": "KEBA Industrial Automation Germany GmbH",
    "00144C": "General Meters Corp.",
    "001471": "Eastern Asia Technology Limited",
    "001403": "Renasis, LLC",
    "0013E7": "Halcro",
    "0013EA": "Kamstrup A/S",
    "0013E1": "Iprobe AB",
    "0013E3": "CoVi Technologies, Inc.",
    "0013E4": "YANGJAE SYSTEMS CORP.",
    "0013D9": "Matrix Product Development, Inc.",
    "001394": "Infohand Co.,Ltd",
    "001389": "Redes de Telefon\u00eda M\u00f3vil S.A.",
    "00138C": "Kumyoung.Co.Ltd",
    "00138E": "FOAB Elektronik AB",
    "001376": "Tabor Electronics Ltd.",
    "001380": "Cisco Systems, Inc",
    "001385": "Add-On Technology Co., LTD.",
    "0013C1": "Asoka USA Corporation",
    "0013C0": "Trix Tecnologia Ltda.",
    "0013A1": "Crow Electronic Engeneering",
    "00139C": "Exavera Technologies, Inc.",
    "00136C": "TomTom",
    "001364": "Paradigm Technology Inc..",
    "00136B": "E-TEC",
    "0013B3": "Ecom Communications Technology Co., Ltd.",
    "0013CC": "Tall Maple Systems",
    "0013B7": "Scantech ID",
    "00130B": "Mextal B.V.",
    "00130D": "GALILEO AVIONICA",
    "001308": "Nuvera Fuel Cells",
    "001307": "Paravirtual Corporation",
    "001355": "TOMEN Cyber-business Solutions, Inc.",
    "001357": "Soyal Technology Co., Ltd.",
    "001360": "Cisco Systems, Inc",
    "00135D": "NTTPC Communications, Inc.",
    "001352": "Naztec, Inc.",
    "001345": "Eaton Corporation",
    "001347": "Red Lion Controls, LP",
    "001343": "Matsushita Electronic Components (Europe) GmbH",
    "0012EC": "Movacolor b.v.",
    "0012EB": "PDH Solutions, LLC",
    "0012FC": "PLANET System Co.,LTD",
    "0012FE": "Lenovo Mobile Communication Technology Ltd.",
    "0012F7": "Xiamen Xinglian Electronics Co., Ltd.",
    "00133E": "MetaSwitch",
    "00132E": "ITian Coporation",
    "0012E1": "Alliant Networks, Inc",
    "001339": "CCV Deutschland GmbH",
    "00132B": "Phoenix Digital",
    "00131B": "BeCell Innovations Corp.",
    "001286": "ENDEVCO CORP",
    "0012C2": "Apex Electronics Factory",
    "0012BE": "Astek Corporation",
    "0012DF": "Novomatic AG",
    "0012E2": "ALAXALA Networks Corporation",
    "0012D4": "Princeton Technology, Ltd",
    "0012D9": "Cisco Systems, Inc",
    "001274": "NIT lab",
    "001270": "NGES Denro Systems",
    "00126E": "Seidel Elektronik GmbH Nfg.KG",
    "0012A0": "NeoMeridian Sdn Bhd",
    "0012AD": "VIVAVIS AG",
    "0012C5": "V-Show  Technology (China) Co.,Ltd",
    "00122D": "SiNett Corporation",
    "00122C": "Soenen Controls N.V.",
    "00122B": "Virbiage Pty Ltd",
    "001232": "LeWiz Communications Inc.",
    "0011E5": "KCodes Corporation",
    "001204": "u10 Networks, Inc.",
    "00120A": "Emerson Climate Technologies GmbH",
    "001208": "Gantner Instruments GmbH",
    "001201": "Cisco Systems, Inc",
    "001200": "Cisco Systems, Inc",
    "001263": "Data Voice Technologies GmbH",
    "00125C": "Green Hills Software, Inc.",
    "00125F": "AWIND Inc.",
    "00121A": "Techno Soft Systemnics Inc.",
    "00121B": "Sound Devices, LLC",
    "0011EE": "Estari, Inc.",
    "00126F": "Rayson Technology Co., Ltd.",
    "00124C": "BBWM Corporation",
    "001244": "Cisco Systems, Inc",
    "001255": "NetEffect Incorporated",
    "001253": "AudioDev AB",
    "0011B4": "Westermo Network Technologies AB",
    "0011B8": "Liebherr - Elektronik GmbH",
    "0011C1": "4P MOBILE DATA PROCESSING",
    "00118C": "Missouri Department of Transportation",
    "001193": "Cisco Systems, Inc",
    "00118E": "Halytech Mace",
    "001186": "Prime Systems, Inc.",
    "001178": "Chiron Technology Ltd",
    "00116A": "Domo Ltd",
    "00119D": "Diginfo Technology Corporation",
    "00119A": "Alkeria srl",
    "00119C": "EP&T Energy",
    "001183": "Datalogic ADC, Inc.",
    "00117F": "Neotune Information Technology Corporation,.LTD",
    "00117D": "ZMD America, Inc.",
    "00117C": "e-zy.net",
    "0011D7": "eWerks Inc",
    "0011C8": "Powercom Co., Ltd.",
    "0011B2": "2001 Technology Inc.",
    "0011AF": "Medialink-i,Inc",
    "0011DE": "EURILOGIC",
    "0011DF": "Current Energy",
    "00115E": "ProMinent Dosiertechnik GmbH",
    "001157": "Oki Electric Industry Co., Ltd.",
    "001139": "STOEBER ANTRIEBSTECHNIK GmbH + Co. KG.",
    "00113E": "JL Corporation",
    "001138": "TAISHIN CO., LTD.",
    "001136": "Goodrich Sensor Systems",
    "001132": "Synology Incorporated",
    "001129": "Paradise Datacom Ltd.",
    "001130": "Allied Telesis (Hong Kong) Ltd.",
    "001118": "BLX IC Design Corp., Ltd.",
    "001107": "RGB Networks Inc.",
    "001108": "Orbital Data Corporation",
    "00110C": "Atmark Techno, Inc.",
    "00114B": "Francotyp-Postalia GmbH",
    "001147": "Secom-Industry co.LTD.",
    "00114A": "KAYABA INDUSTRY Co,.Ltd.",
    "001142": "e-SMARTCOM  INC.",
    "001120": "Cisco Systems, Inc",
    "000FEC": "ARKUS Inc.",
    "000FED": "Anam Electronics Co., Ltd",
    "000FDD": "SORDIN AB",
    "00114D": "Atsumi Electric Co.,LTD.",
    "000FF0": "Sunray Co. Ltd.",
    "001103": "kawamura electric inc.",
    "000F88": "AMETEK, Inc.",
    "000F7E": "Ablerex Electronics Co., LTD",
    "000F83": "Brainium Technologies Inc.",
    "000F84": "Astute Networks, Inc.",
    "000FD9": "FlexDSL Telecommunications AG",
    "000FD0": "ASTRI",
    "000FAE": "E2O Communications",
    "000FB1": "Cognio Inc.",
    "000F9E": "Murrelektronik GmbH",
    "000F93": "Landis+Gyr Ltd.",
    "000F94": "Genexis BV",
    "000FCA": "A-JIN TECHLINE CO, LTD",
    "000FC3": "PalmPalm Technology, Inc.",
    "000F79": "Bluetooth Interest Group Inc.",
    "000F6C": "ADDI-DATA GmbH",
    "000F8F": "Cisco Systems, Inc",
    "000F7D": "Xirrus",
    "000F76": "Digital Keystone, Inc.",
    "000FA7": "Raptor Networks Technology",
    "000F23": "Cisco Systems, Inc",
    "000F22": "Helius, Inc.",
    "000F24": "Cisco Systems, Inc",
    "000F17": "Insta Elektro GmbH",
    "000F1E": "Chengdu KT Electric Co.of High & New Technology",
    "000F35": "Cisco Systems, Inc",
    "000F36": "Accurate Techhnologies, Inc.",
    "000F2B": "GREENBELL SYSTEMS",
    "000F28": "Itronix Corporation",
    "000EFC": "JTAG Technologies B.V.",
    "000EFE": "EndRun Technologies LLC",
    "000EF3": "Smartlabs, Inc. ",
    "000EF2": "Infinico Corporation",
    "000F5C": "Day One Digital Media Limited",
    "000EE4": "BOE TECHNOLOGY GROUP CO.,LTD",
    "000EDE": "REMEC, Inc.",
    "000F52": "YORK Refrigeration, Marine & Controls",
    "000F4C": "Elextech INC",
    "000F42": "Xalyo Systems",
    "000F39": "IRIS SENSORS",
    "000F3E": "CardioNet, Inc",
    "000F3F": "Big Bear Networks",
    "000F08": "Indagon Oy",
    "000F04": "cim-usa inc",
    "000F15": "Icotera A/S",
    "000F0D": "Hunt Electronic Co., Ltd.",
    "000E83": "Cisco Systems, Inc",
    "000E81": "Devicescape Software, Inc.",
    "000E88": "VIDEOTRON CORP.",
    "000E86": "Alcatel North America",
    "000E69": "China Electric Power Research Institute",
    "000E61": "MICROTROL LIMITED",
    "000E64": "Elphel, Inc",
    "000E5A": "TELEFIELD inc.",
    "000ECE": "S.I.T.T.I. S.p.A.",
    "000ED4": "CRESITT INDUSTRIE",
    "000ED6": "Cisco Systems, Inc",
    "000ED8": "Positron Access Solutions Corp",
    "000ED1": "Osaka Micro Computer.",
    "000EA5": "BLIP Systems",
    "000EA0": "NetKlass Technology Inc.",
    "000E9C": "Benchmark Electronics ",
    "000E9A": "BOE TECHNOLOGY GROUP CO.,LTD",
    "000E7E": "ionSign Oy",
    "000E77": "Decru, Inc.",
    "000E6A": "3Com Ltd",
    "000E90": "PONICO CORP.",
    "000E8A": "Avara Technologies Pty. Ltd.",
    "000EB8": "Iiga co.,Ltd",
    "000EBB": "Everbee Networks",
    "000EBE": "B&B Electronics Manufacturing Co.",
    "000EB0": "Solutions Radio BV",
    "000E4C": "Bermai Inc.",
    "000E49": "Forsway Scandinavia AB",
    "000E42": "Motic Incoporation Ltd.",
    "000E3D": "Televic N.V.",
    "000E39": "Cisco Systems, Inc",
    "000DE1": "Control Products, Inc.",
    "000DD0": "TetraTec Instruments GmbH",
    "000DD3": "SAMWOO Telecommunication Co.,Ltd.",
    "000DD8": "BBN",
    "000E34": "NexGen City, LP",
    "000E2D": "Hyundai Digital Technology Co.,Ltd.",
    "000E30": "AERAS Networks, Inc.",
    "000E0D": "Hesch Schr\u00f6der GmbH",
    "000DFF": "CHENMING MOLD INDUSTRY CORP.",
    "000DF6": "Technology Thesaurus Corp.",
    "000E29": "Shester Communications Inc",
    "000E19": "LogicaCMG Pty Ltd",
    "000E5D": "Triple Play Technologies A/S",
    "000E52": "Optium Corporation",
    "000E4D": "Numesa Inc.",
    "000E1A": "JPS Communications",
    "000E06": "Team Simoco Ltd",
    "000DAE": "SAMSUNG HEAVY INDUSTRIES CO., LTD.",
    "000DB2": "Ammasso, Inc.",
    "000DAA": "S.A.Tehnology co.,Ltd.",
    "000DA6": "Universal Switching Corporation",
    "000D9C": "K.A. Schmersal GmbH & Co. KG",
    "000D98": "S.W.A.C. Schmitt-Walter Automation Consult GmbH",
    "000D8C": "Shanghai Wedone Digital Ltd. CO.",
    "000D94": "AFAR Communications,Inc",
    "000D8D": "Prosoft Technology, Inc",
    "000D7D": "Afco Systems",
    "000D73": "Technical Support, Inc.",
    "000D69": "TMT&D Corporation",
    "000D68": "Vinci Systems, Inc.",
    "000D64": "COMAG Handels AG",
    "000D5C": "Robert Bosch GmbH, VT-ATMO",
    "000D60": "IBM Corp",
    "000D84": "Makus Inc.",
    "000D74": "Sand Network Systems, Inc.",
    "000DA2": "Infrant Technologies, Inc.",
    "000DC7": "COSMIC ENGINEERING INC.",
    "000DC2": "Private",
    "000DBF": "TekTone Sound & Signal Mfg., Inc.",
    "000DB3": "SDO Communication Corperation",
    "000D38": "NISSIN INC.",
    "000D34": "Shell International Exploration and Production, Inc.",
    "000D32": "DispenseSource, Inc.",
    "000CFC": "S2io Technologies Corp",
    "000CF6": "Sitecom Europe BV",
    "000CF2": "GAMESA E\u00f3lica",
    "000D5F": "Minds Inc",
    "000D54": "3Com Ltd",
    "000D4C": "Outline Electronics Ltd.",
    "000D4D": "Ninelanes",
    "000D12": "AXELL Corporation",
    "000D0A": "Barco Projection Systems NV",
    "000CDC": "BECS Technology, Inc",
    "000CDE": "ABB STOTZ-KONTAKT GmbH",
    "000D15": "Voipac s.r.o.",
    "000D0F": "Finlux Ltd",
    "000D2E": "Matsushita Avionics Systems Corporation",
    "000D28": "Cisco Systems, Inc",
    "000D53": "Beijing 5w Communication Corp.",
    "000CE1": "The Open Group",
    "000C71": "Wybron, Inc",
    "000C72": "Tempearl Industrial Co., Ltd.",
    "000CC2": "ControlNet (India) Private Limited",
    "000CC1": "Eaton Corporation",
    "000CA1": "SIGMACOM Co., LTD.",
    "000CA6": "Mintera Corporation",
    "000C94": "United Electronic Industries, Inc. (EUI)",
    "000C9B": "EE Solutions, Inc",
    "000C8B": "Connect Tech Inc",
    "000C90": "Octasic Inc.",
    "000C8C": "KODICOM CO.,LTD.",
    "000C55": "Microlink Communications Inc.",
    "000C59": "Indyme Electronics, Inc.",
    "000C5C": "GTN Systems B.V.",
    "000C52": "Roll Systems Inc.",
    "000C78": "In-Tech Electronics Limited",
    "000C74": "RIVERTEC CORPORATION",
    "000CC5": "Nextlink Co., Ltd.",
    "000C67": "OYO ELECTRIC CO.,LTD",
    "000C63": "Zenith Electronics Corporation",
    "000CBF": "Holy Stone Ent. Co., Ltd.",
    "000C2E": "Openet information technology(shenzhen) Co., Ltd.",
    "000C2C": "Enwiser Inc.",
    "000C28": "RIFATRON",
    "000C21": "Faculty of Science and Technology, Keio University",
    "000C1B": "ORACOM Co, Ltd.",
    "000C44": "Automated Interfaces, Inc.",
    "000C3B": "Orion Electric Co., Ltd.",
    "000C3D": "Glsystech Co., Ltd.",
    "000C14": "Diagnostic Instruments, Inc.",
    "000C53": "Private",
    "000C48": "QoStek Corporation",
    "000C4D": "Curtiss-Wright Controls Avionics & Electronics",
    "000C2F": "SeorimTechnology Co.,Ltd.",
    "000C31": "Cisco Systems, Inc",
    "000C17": "AJA Video Systems Inc",
    "000C11": "NIPPON DEMPA CO.,LTD.",
    "000BF7": "NIDEK CO.,LTD",
    "000C00": "BEB Industrie-Elektronik AG",
    "000BF3": "BAE SYSTEMS",
    "000BED": "ELM Inc.",
    "000BB5": "nStor Technologies, Inc.",
    "000BB9": "Imsys AB",
    "000BBB": "Etin Systems Co., Ltd",
    "000BBC": "En Garde Systems, Inc.",
    "000BB1": "Super Star Technology Co., Ltd.",
    "000BBF": "Cisco Systems, Inc",
    "000BAD": "PC-PoS Inc.",
    "000BA0": "T&L Information Inc.",
    "000B91": "Aglaia Gesellschaft f\u00fcr Bildverarbeitung und Kommunikation mbH",
    "000B97": "Matsushita Electric Industrial Co.,Ltd.",
    "000B92": "Ascom Danmark A/S",
    "000B52": "JOYMAX ELECTRONICS CO. LTD.",
    "000B49": "RF-Link System Inc.",
    "000B46": "Cisco Systems, Inc",
    "000BC9": "Electroline Equipment",
    "000BC2": "Corinex Communication Corp.",
    "000B9A": "Shanghai Ulink Telecom Equipment Co. Ltd.",
    "000B96": "Innotrac Diagnostics Oy",
    "000B9D": "TwinMOS Technologies Inc.",
    "000B69": "Franke Finland Oy",
    "000B66": "Teralink Communications",
    "000B56": "Cybernetics",
    "000B50": "Oxygnet",
    "000B7A": "L-3 Linkabit",
    "000B84": "BODET",
    "000B77": "Cogent Systems, Inc.",
    "000AF2": "NeoAxiom Corp.",
    "000AF5": "Airgo Networks, Inc.",
    "000AF4": "Cisco Systems, Inc",
    "000AF0": "SHIN-OH ELECTRONICS CO., LTD. R&D",
    "000AEE": "GCD Hard- & Software GmbH",
    "000AE9": "AirVast Technology Inc.",
    "000ACD": "Sunrich Technology Limited",
    "000ACC": "Winnow Networks, Inc.",
    "000ACF": "PROVIDEO Multimedia Co. Ltd.",
    "000AD1": "MWS",
    "000AD3": "INITECH Co., Ltd",
    "000AC8": "ZPSYS CO.,LTD. (Planning&Management)",
    "000B2C": "Eiki Industrial Co. Ltd.",
    "000B26": "Wetek Corporation",
    "000B28": "Quatech Inc.",
    "000B30": "Beijing Gongye Science & Technology Co.,Ltd",
    "000B2A": "HOWTEL Co., Ltd.",
    "000B13": "ZETRON INC",
    "000B10": "11wave Technonlogy Co.,Ltd",
    "000B07": "Voxpath Networks",
    "000B04": "Volktek Corporation",
    "000AC6": "Overture Networks.",
    "000AAB": "Toyota Technical Development Corporation",
    "000B38": "Kn\u00fcrr GmbH",
    "000B32": "VORMETRIC, INC.",
    "000AD8": "IPCserv Technology Corp.",
    "000B2B": "HOSTNET CORPORATION",
    "000B1F": "I CON Computer Co.",
    "000AF9": "HiConnect, Inc.",
    "000B02": "Dallmeier electronic",
    "000A7D": "Valo, Inc.",
    "000A7F": "Teradon Industries, Inc",
    "000A81": "TEIMA Audiotex S.L.",
    "000A87": "Integrated Micromachines Inc.",
    "000A77": "Bluewire Technologies LLC",
    "000A7A": "Kyoritsu Electric Co., Ltd.",
    "000A38": "Apani Networks",
    "000A41": "Cisco Systems, Inc",
    "000A3E": "EADS Telecom",
    "000A48": "Albatron Technology",
    "000A36": "Synelec Telecom Multimedia",
    "000A32": "Xsido Corporation",
    "000A2B": "Etherstuff",
    "000A9C": "Server Technology, Inc.",
    "000A8C": "Guardware Systems Ltd.",
    "000A96": "MEWTEL TECHNOLOGY INC.",
    "000A82": "TATSUTA SYSTEM ELECTRONICS CO.,LTD.",
    "000A54": "Laguna Hills, Inc.",
    "000A52": "AsiaRF Ltd.",
    "000A4F": "Brain Boxes Limited",
    "000A42": "Cisco Systems, Inc",
    "000A65": "GentechMedia.co.,ltd.",
    "000A59": "HW server",
    "000A22": "Amperion Inc",
    "000A1C": "Bridge Information Co., Ltd.",
    "000AB4": "ETIC Telecommunications",
    "000A75": "Caterpillar, Inc",
    "0009D0": "Solacom Technologies Inc.",
    "0009CC": "Moog GmbH",
    "0009C8": "SINAGAWA TSUSHIN KEISOU SERVICE",
    "0009B2": "L&F Inc.",
    "0009A8": "Eastmode Pte Ltd",
    "0009AA": "Data Comm for Business, Inc.",
    "0009A4": "HARTEC Corporation",
    "0009A6": "Ignis Optics, Inc.",
    "0009A7": "Bang & Olufsen A/S",
    "0009A0": "Microtechno Corporation",
    "00099B": "Western Telematic Inc.",
    "000990": "ACKSYS Communications & systems",
    "000A12": "Azylex Technology, Inc",
    "000A13": "Honeywell Video Systems",
    "000A09": "TaraCom Integrated Products, Inc.",
    "0009C4": "Medicore Co., Ltd",
    "0009B7": "Cisco Systems, Inc",
    "0009EF": "Vocera Communications",
    "0009E4": "K Tech Infosystem Inc.",
    "000A00": "Mediatek Corp.",
    "0009F0": "Shimizu Technology Inc.",
    "0009D9": "Neoscale Systems, Inc",
    "000942": "Wireless Technologies, Inc",
    "000945": "Palmmicro Communications Inc",
    "00093E": "C&I Technologies",
    "000940": "AGFEO GmbH & Co. KG",
    "00093B": "HYUNDAI NETWORKS INC.",
    "00097F": "Vsecure 2000 LTD.",
    "000980": "Power Zenith Inc.",
    "000972": "Securebase,Inc",
    "000978": "AIJI System Co., Ltd.",
    "000973": "Lenten Technology Co., Ltd.",
    "000975": "fSONA Communications Corporation",
    "000977": "Brunner Elektronik AG",
    "000969": "Meret Optical Communications",
    "00096E": "GIANT ELECTRONICS LTD.",
    "000920": "EpoX COMPUTER CO.,LTD.",
    "000922": "TST Biometrics GmbH",
    "000916": "Listman Home Technologies, Inc.",
    "00096C": "Imedia Semiconductor Corp.",
    "00095F": "Telebyte, Inc.",
    "000953": "Linkage System Integration Co.Ltd.",
    "00094C": "Communication Weaver Co.,Ltd.",
    "000930": "AeroConcierge Inc.",
    "00091E": "Firstech Technology Corp.",
    "000934": "Dream-Multimedia-Tv GmbH",
    "00098A": "EqualLogic Inc",
    "0008A5": "Peninsula Systems Inc.",
    "0008A2": "ADI Engineering, Inc.",
    "000898": "Gigabit Optics Corporation",
    "00089B": "ICP Electronics Inc.",
    "00089C": "Elecs Industry Co., Ltd.",
    "00089D": "UHD-Elektronik",
    "0008CF": "Nippon Koei Power Systems Co., Ltd.",
    "0008CB": "Zeta Broadband Inc.",
    "0008D3": "Hercules Technologies S.A.S.",
    "0008D0": "Musashi Engineering Co., LTD.",
    "0008F4": "Bluetake Technology Co., Ltd.",
    "0008F7": "Hitachi Ltd, Semiconductor & Integrated Circuits Gr",
    "0008F5": "YESTECHNOLOGY Co.,Ltd.",
    "000911": "Cisco Systems, Inc",
    "000912": "Cisco Systems, Inc",
    "000908": "VTech Technology Corp.",
    "00090B": "MTL  Instruments PLC",
    "0008B5": "TAI GUEN ENTERPRISE CO., LTD",
    "0008B7": "HIT Incorporated",
    "0008A9": "SangSang Technology, Inc.",
    "0008C8": "Soneticom, Inc.",
    "0008C4": "Hikari Co.,Ltd.",
    "0008BA": "Erskine Systems Ltd",
    "00088F": "ADVANCED DIGITAL TECHNOLOGY",
    "00088B": "Tropic Networks Inc.",
    "000886": "Hansung Teliann, Inc.",
    "0008EC": "Optical Zonu Corporation",
    "0008E6": "Littlefeet",
    "0008E2": "Cisco Systems, Inc",
    "000905": "iTEC Technologies Ltd.",
    "0008F9": "Artesyn Embedded Technologies",
    "0007E1": "WIS Communications Co. Ltd.",
    "0007E0": "Palm Inc.",
    "0007D6": "Commil Ltd.",
    "0007D7": "Caporis Networks AG",
    "0007D4": "Zhejiang Yutong Network Communication Co Ltd.",
    "0007CE": "Cabletime Limited",
    "0007D3": "SPGPrints B.V.",
    "0007D0": "Automat Engenharia de Automa\u00e7\u00e3o Ltda.",
    "00085D": "Mitel Corporation",
    "000855": "NASA-Goddard Space Flight Center",
    "00085A": "IntiGate Inc.",
    "000858": "Novatechnology Inc.",
    "000850": "Arizona Instrument Corp.",
    "00085B": "Hanbit Electronics Co., Ltd.",
    "0007ED": "Altera Corporation",
    "0007EA": "Massana, Inc.",
    "0007F1": "TeraBurst Networks Inc.",
    "0007F2": "IOA Corporation",
    "0007F0": "LogiSync LLC",
    "0007E7": "FreeWave Technologies",
    "0007E3": "Navcom Technology, Inc.",
    "000817": "EmergeCore Networks LLC",
    "000815": "CATS Co., Ltd.",
    "000818": "Pixelworks, Inc.",
    "00080F": "Proximion Fiber Optics AB",
    "000812": "GM-2 Corporation",
    "00080B": "Birka BPA Informationssystem AB",
    "00080A": "Espera-Werke GmbH",
    "0007FA": "ITT Co., Ltd.",
    "0007F6": "Qqest Software Systems",
    "0007FB": "Giga Stream UMTS Technologies GmbH",
    "00047C": "Skidata AG",
    "0007BE": "DataLogic SpA",
    "00082B": "Wooksung Electronics, Inc.",
    "00082E": "Multitone Electronics PLC",
    "00082A": "Powerwallz Network Security",
    "00086B": "MIPSYS",
    "00087F": "SPAUN electronic GmbH & Co. KG",
    "0007AF": "Red Lion Controls, LP",
    "0007B2": "Transaccess S.A.",
    "0007AD": "Pentacon GmbH Foto-und Feinwerktechnik",
    "0007AC": "Eolring",
    "0007AA": "Quantum Data Inc.",
    "0007A4": "GN Netcom Ltd.",
    "00079D": "Musashi Co., Ltd.",
    "00079F": "Action Digital Inc.",
    "000792": "S\u00fctron Electronic GmbH",
    "000790": "Tri-M Technologies (s) Limited",
    "00078D": "NetEngines Ltd.",
    "00078A": "Mentor Data System Inc.",
    "00076A": "NEXTEYE Co., Ltd.",
    "000767": "Yuxing Electronics Company Limited",
    "00075B": "Gibson Guitars",
    "000762": "Group Sense Limited",
    "000784": "Cisco Systems, Inc",
    "000776": "Federal APD",
    "00077A": "Infoware System Co., Ltd.",
    "00070C": "SVA-Intrusion.com Co. Ltd.",
    "00070F": "Fujant, Inc.",
    "000727": "Zi Corporation (HK) Ltd.",
    "00072E": "North Node AB",
    "0006E6": "DongYang Telecom Co., Ltd.",
    "0006D2": "Tundra Semiconductor Corp.",
    "0006D8": "Maple Optical Systems",
    "0006CF": "Thales Avionics In-Flight Systems, LLC",
    "0006B6": "Nir-Or Israel Ltd.",
    "0006A3": "Bitran Corporation",
    "00069F": "Kuokoa Networks",
    "0006A2": "Microtune, Inc.",
    "0006A6": "Artistic Licence Engineering Ltd",
    "0006AA": "VT Miltope",
    "000657": "Market Central, Inc.",
    "0006DC": "Syabas Technology (Amquest)",
    "000697": "R & D Center",
    "00069A": "e & Tel",
    "000699": "Vida Design Co.",
    "00068D": "SEPATON, Inc.",
    "000684": "Biacore AB",
    "000682": "Convedia",
    "0006BB": "ATI Technologies Inc.",
    "0006BD": "BNTECHNOLOGY Co., Ltd.",
    "0006C3": "Schindler Elevator Ltd.",
    "0006B2": "Linxtek Co.",
    "0006FE": "Ambrado, Inc",
    "0006E7": "Bit Blitz Communications Inc.",
    "0006ED": "Inara Networks",
    "000714": "Brightcom",
    "0006F3": "AcceLight Networks",
    "0006DB": "ICHIPS Co., Ltd.",
    "0006B8": "Bandspeed Pty Ltd",
    "00066B": "Sysmex Corporation",
    "00055A": "Power Dsine Ltd.",
    "000653": "Cisco Systems, Inc",
    "00064F": "PRO-NETS Technology Corporation",
    "000651": "Aspen Networks Inc.",
    "00065C": "Malachite Technologies, Inc.",
    "000622": "Chung Fu Chen Yeh Enterprise Corp.",
    "000612": "Accusys, Inc.",
    "000609": "Crossport Systems",
    "000616": "Tel Net Co., Ltd.",
    "00060D": "Wave7 Optics",
    "000642": "Genetel Systems Inc.",
    "00064A": "Honeywell Co., Ltd. (KOREA)",
    "00063F": "Everex Communications Inc.",
    "00063D": "Microwave Data Systems Inc.",
    "000639": "Newtec",
    "0005EF": "ADOIR Digital Technology",
    "0005E9": "Unicess Network, Inc.",
    "000600": "Toshiba Teli Corporation",
    "0005E6": "Egenera, Inc.",
    "00067D": "Takasago Ltd.",
    "000671": "Softing AG",
    "000672": "Netezza",
    "00062F": "Pivotech Systems Inc.",
    "000636": "Jedai Broadband Networks",
    "00CBBD": "Cambridge Broadband Networks Group",
    "0005E3": "LightSand Communications, Inc.",
    "0005ED": "Technikum Joanneum GmbH",
    "00066A": "InfiniCon Systems, Inc.",
    "000661": "NIA Home Technologies Corp.",
    "00061E": "Maxan Systems",
    "0005C6": "Triz Communications",
    "0005CC": "Sumtel Communications, Inc.",
    "0005CF": "Thunder River Technologies, Inc.",
    "0005A0": "MOBILINE Kft.",
    "0005A6": "Extron Electronics",
    "000588": "Sensoria Corp.",
    "00058D": "Lynx Photonic Networks, Inc.",
    "00058A": "Netcom Co., Ltd.",
    "000591": "Active Silicon Ltd",
    "000593": "Grammar Engine Inc.",
    "00058F": "CLCsoft co.",
    "000584": "AbsoluteValue Systems, Inc.",
    "0005D6": "L-3 Linkabit",
    "0005DA": "Apex Automationstechnik",
    "0005C7": "I/F-COM A/S",
    "000564": "Tsinghua Bitway Co., Ltd.",
    "000558": "Synchronous, Inc.",
    "000550": "Vcomms Connect Limited",
    "0005A9": "Princeton Networks, Inc.",
    "0005AF": "InnoScan Computing A/S",
    "0005A1": "Zenocom",
    "000541": "Advanced Systems Co., Ltd.",
    "000545": "Internet Photonics",
    "00057A": "Overture Networks",
    "0005BB": "Myspace AB",
    "0004F8": "QUALICABLE TV Industria E Com., Ltda",
    "0004F5": "SnowShore Networks, Inc.",
    "0004F2": "Polycom",
    "0004F3": "FS FORTH-SYSTEME GmbH",
    "0004ED": "Billion Electric Co., Ltd.",
    "0004E7": "Lightpointe Communications, Inc",
    "0004E6": "Banyan Network Private Limited",
    "0004DE": "Cisco Systems, Inc",
    "000531": "Cisco Systems, Inc",
    "000539": "A Brand New World in Sweden AB",
    "00052A": "Ikegami Tsushinki Co., Ltd.",
    "00052D": "Zoltrix International Limited",
    "000517": "Shellcomm, Inc.",
    "00051C": "Xnet Technology Corp.",
    "000512": "Zebra Technologies Inc",
    "000503": "ICONAG",
    "0004F9": "Xtera Communications, Inc.",
    "0004FA": "NBS Technologies Inc.",
    "00053A": "Willowglen Services Pte Ltd",
    "0004E1": "Infinior Microsystems",
    "000525": "Puretek Industrial Co., Ltd.",
    "0004A6": "SAF Tehnika Ltd.",
    "0004A8": "Broadmax Technologies, Inc.",
    "0004A1": "Pathway Connectivity",
    "0004A2": "L.S.I. Japan Co., Ltd.",
    "00049B": "Cisco Systems, Inc",
    "00047A": "AXXESSIT ASA",
    "00046A": "Navini Networks",
    "00046B": "Palm Wireless, Inc.",
    "000472": "Telelynx, Inc.",
    "000464": "Pulse-Link Inc",
    "000490": "Optical Access",
    "000486": "ITTC, University of Kansas",
    "00048B": "Poscon Corporation",
    "000484": "Amann GmbH",
    "000478": "G. Star Technology Corporation",
    "00044E": "Cisco Systems, Inc",
    "000452": "RocketLogix, Inc.",
    "000442": "NACT",
    "000445": "LMS Skalar Instruments GmbH",
    "00043D": "INDEL AG",
    "00043B": "Lava Computer Mfg., Inc.",
    "000431": "GlobalStreams, Inc.",
    "000499": "Chino Corporation",
    "00048E": "Ohm Tech Labs, Inc.",
    "00048D": " Teo Technologies, Inc",
    "0004CE": "Patria Ailon",
    "0004C7": "NetMount",
    "000465": "i.s.t isdn-support technik GmbH",
    "00045E": "PolyTrax Information Technology AG",
    "0004C2": "Magnipix, Inc.",
    "0003C9": "TECOM Co., Ltd.",
    "0003C1": "Packet Dynamics Ltd",
    "0003C4": "Tomra Systems ASA",
    "0003BF": "Centerpoint Broadband Technologies, Inc.",
    "0003FD": "Cisco Systems, Inc",
    "0003F9": "Pleiades Communications, Inc.",
    "0003FE": "Cisco Systems, Inc",
    "0003B9": "Hualong Telecom Co., Ltd.",
    "0003BA": "Oracle Corporation",
    "0003AF": "Paragea Communications",
    "000411": "Inkra Networks, Inc.",
    "000410": "Spinnaker Networks, Inc.",
    "000412": "WaveSmith Networks, Inc.",
    "0003AC": "Fronius Schweissmaschinen",
    "000399": "Dongju Informations & Communications Co., Ltd.",
    "00030B": "Hunter Technology, Inc.",
    "000427": "Cisco Systems, Inc",
    "000420": "Slim Devices, Inc.",
    "0003DE": "OTC Wireless",
    "0003F5": "Chip2Chip",
    "000330": "Imagenics, Co., Ltd.",
    "000328": "Mace Group, Inc.",
    "00032B": "GAI Datenfunksysteme GmbH",
    "00032C": "ABB Switzerland Ltd",
    "000323": "Cornet Technology, Inc.",
    "000313": "Access Media SPA",
    "000310": "E-Globaledge Corporation",
    "00030A": "Argus Technologies",
    "000304": "Pacific Broadband Communications",
    "000301": "EXFO",
    "0002FD": "Cisco Systems, Inc",
    "000340": "Floware Wireless Systems, Ltd.",
    "0001EC": "Ericsson Group",
    "000333": "Digitel Co., Ltd.",
    "000338": "Oak Technology",
    "000339": "Eurologic Systems, Ltd.",
    "000331": "Cisco Systems, Inc",
    "000381": "Ingenico International",
    "000380": "SSH Communications Security Corp.",
    "000382": "A-One Co., Ltd.",
    "00037A": "Taiyo Yuden Co., Ltd.",
    "000375": "NetMedia, Inc.",
    "000391": "Advanced Digital Broadcast, Ltd.",
    "00038F": "Weinschel Corporation",
    "000384": "AETA",
    "000387": "Blaze Network Products",
    "00035A": "Photron Limited",
    "000353": "Mitac, Inc.",
    "00031E": "Optranet, Inc.",
    "000315": "Cidco Incorporated",
    "000319": "Infineon AG",
    "00037B": "IDEC IZUMI Corporation",
    "000367": "Jasmine Networks, Inc.",
    "00036A": "Mainnet, Ltd.",
    "00036B": "Cisco Systems, Inc",
    "00036C": "Cisco Systems, Inc",
    "00034F": "Sur-Gard Security",
    "00034A": "RIAS Corporation",
    "0002B6": "Acrosser Technology Co., Ltd.",
    "0002B1": "Anritsu, Ltd.",
    "0002AD": "HOYA Corporation",
    "0002AE": "Scannex Electronics Ltd.",
    "0002A2": "Hilscher GmbH",
    "0002C2": "Net Vision Telecom",
    "0002BD": "Bionet Co., Ltd.",
    "0002BE": "Totsu Engineering, Inc.",
    "0002B9": "Cisco Systems, Inc",
    "0002BA": "Cisco Systems, Inc",
    "000300": "Barracuda Networks, Inc.",
    "0002F9": "MIMOS Berhad",
    "0002F3": "Media Serve Co., Ltd.",
    "0002EA": "Focus Enhancements",
    "00025A": "Catena Networks",
    "00026E": "NeGeN Access, Inc.",
    "0002E7": "CAB GmbH & Co KG",
    "0002DF": "Net Com Systems, Inc.",
    "0002DB": "NETSEC",
    "000270": "Crewave Co., Ltd.",
    "00029A": "Storage Apps",
    "00028F": "Globetek, Inc.",
    "000287": "Adapcom",
    "000281": "Madge Ltd.",
    "0002CF": "ZyGate Communications, Inc.",
    "0002D1": "Vivotek, Inc.",
    "000254": "WorldGate",
    "00024B": "Cisco Systems, Inc",
    "00024D": "Mannesman Dematic Colby Pty. Ltd.",
    "000250": "Geyser Networks, Inc.",
    "0001D9": "Sigma, Inc.",
    "0001C5": "Simpler Networks",
    "0001C9": "Cisco Systems, Inc",
    "0001C4": "NeoWave, Inc.",
    "000200": "Net & Sys Co., Ltd.",
    "0001E4": "Sitera, Inc.",
    "0001F3": "QPS, Inc.",
    "0001E3": "Siemens AG",
    "000248": "Pilz GmbH & Co.",
    "00024C": "SiByte, Inc.",
    "000240": "Seedek Co., Ltd.",
    "00023B": "Ericsson",
    "0001EB": "C-COM Corporation",
    "0001F2": "Mark of the Unicorn, Inc.",
    "0001D8": "Teltronics, Inc.",
    "000239": "Visicom",
    "000233": "Mantra Communications, Inc.",
    "00022A": "Asound Electronic",
    "000205": "Hitachi Denshi, Ltd.",
    "0001F7": "Image Display Systems, Inc.",
    "000221": "DSP Application, Ltd.",
    "000215": "Cotas Computer Technology A/B",
    "00021B": "Kollmorgen-Servotronix",
    "00021E": "SIMTEL S.R.L.",
    "00018A": "ROI COMPUTER AG",
    "000192": "Texas Digital Systems",
    "000182": "DICA TECHNOLOGIES AG",
    "000189": "Refraction Technology, Inc.",
    "000193": "Hanbyul Telecom Co., Ltd.",
    "0030F5": "Wild Lab. Ltd.",
    "000145": "WINSYSTEMS, INC.",
    "000137": "IT Farm Corporation",
    "000133": "KYOWA Electronic Instruments C",
    "00013C": "TIW SYSTEMS",
    "00014C": "Berkeley Process Control",
    "0001A3": "GENESYS LOGIC, INC.",
    "00018C": "Mega Vision",
    "00018F": "Kenetec, Inc.",
    "00017B": "Heidelberger Druckmaschinen AG",
    "000173": "AMCC",
    "00016C": "FOXCONN",
    "000175": "Radiant Communications Corp.",
    "00015C": "CADANT INC.",
    "0001AB": "Main Street Networks",
    "0001AF": "Artesyn Embedded Technologies",
    "000169": "Celestix Networks Pte Ltd.",
    "00016B": "LightChip, Inc.",
    "0001A5": "Nextcomm, Inc.",
    "000190": "SMK-M",
    "0001C1": "Vitesse Semiconductor Corporation",
    "0001B6": "SAEJIN T&M Co., Ltd.",
    "00015D": "Oracle Corporation ",
    "00B02A": "ORSYS GmbH",
    "00B069": "Honewell Oy",
    "00B0C2": "Cisco Systems, Inc",
    "00B03B": "HiQ Networks",
    "00B086": "LocSoft Limited",
    "003046": "Controlled Electronic Manageme",
    "003075": "ADTECH",
    "003098": "Global Converging Technologies",
    "00300D": "MMC Technology, Inc.",
    "0030A2": "Lightner Engineering",
    "003042": "DeTeWe-Deutsche Telephonwerke",
    "003037": "Packard Bell Nec Services",
    "003057": "QTelNet, Inc.",
    "0030FC": "Terawave Communications, Inc.",
    "000127": "OPEN Networks Pty Ltd",
    "000104": "DVICO Co., Ltd.",
    "00010E": "Bri-Link Technologies Co., Ltd",
    "00013D": "RiscStation Ltd.",
    "000143": "Cisco Systems, Inc",
    "00014B": "Ennovate Networks, Inc.",
    "000120": "OSCILLOQUARTZ S.A.",
    "0030C2": "COMONE",
    "0030F3": "At Work Computers",
    "0030CC": "Tenor Networks, Inc.",
    "0030B0": "Convergenet Technologies",
    "00029C": "3COM",
    "00B019": "UTC CCS",
    "00B0C7": "Tellabs Operations, Inc.",
    "003086": "Transistor Devices, Inc.",
    "000106": "Tews Datentechnik GmbH",
    "000109": "Nagano Japan Radio Co., Ltd.",
    "00305F": "Hasselblad",
    "0030DC": "RIGHTECH CORPORATION",
    "003025": "CHECKOUT COMPUTER SYSTEMS, LTD",
    "0030C6": "CONTROL SOLUTIONS, INC.",
    "0030E3": "SEDONA NETWORKS CORP.",
    "0030BF": "MULTIDATA GMBH",
    "003034": "SET ENGINEERING",
    "00304A": "Fraunhofer IPMS",
    "00308D": "Pinnacle Systems, Inc.",
    "0030F6": "SECURELOGIX CORPORATION",
    "0030D6": "MSC VERTRIEBS GMBH",
    "003041": "SAEJIN T & M CO., LTD.",
    "00308C": "Quantum Corporation",
    "003019": "Cisco Systems, Inc",
    "003076": "Akamba Corporation",
    "0030EC": "BORGARDT",
    "00303C": "ONNTO CORP.",
    "003024": "Cisco Systems, Inc",
    "0030D8": "SITEK",
    "003016": "ISHIDA CO., LTD.",
    "00306F": "SEYEON TECH. CO., LTD.",
    "00303D": "IVA CORPORATION",
    "0030F4": "STARDOT TECHNOLOGIES",
    "0030EB": "TURBONET COMMUNICATIONS, INC.",
    "0030A1": "WEBGATE Inc.",
    "00306A": "PENTA MEDIA CO., LTD.",
    "003053": "Basler AG",
    "0030D2": "WIN TECHNOLOGIES, CO., LTD.",
    "003097": "AB Regin",
    "00D043": "ZONAL RETAIL DATA SYSTEMS",
    "00D0C1": "HARMONIC DATA SYSTEMS, LTD.",
    "00D035": "BEHAVIOR TECH. COMPUTER CORP.",
    "00D0DB": "MCQUAY INTERNATIONAL",
    "00D070": "LONG WELL ELECTRONICS CORP.",
    "00D029": "WAKEFERN FOOD CORPORATION",
    "00D0B1": "OMEGA ELECTRONICS SA",
    "00D016": "SCM MICROSYSTEMS, INC.",
    "003012": "DIGITAL ENGINEERING LTD.",
    "00D00F": "SPEECH DESIGN GMBH",
    "003058": "API MOTION",
    "0030A6": "VIANET TECHNOLOGIES, LTD.",
    "00D0C3": "VIVID TECHNOLOGY PTE, LTD.",
    "00D0C8": "Prevas A/S",
    "00D07E": "KEYCORP LTD.",
    "00D0EA": "NEXTONE COMMUNICATIONS, INC.",
    "00D020": "AIM SYSTEM, INC.",
    "00D0CB": "DASAN CO., LTD.",
    "00D019": "DAINIPPON SCREEN CORPORATE",
    "00D013": "PRIMEX AEROSPACE COMPANY",
    "00D0A3": "VOCAL DATA, INC.",
    "00D0BF": "PIVOTAL TECHNOLOGIES",
    "00D02F": "VLSI TECHNOLOGY INC.",
    "00D0AC": "Commscope, Inc",
    "00D0BC": "Cisco Systems, Inc",
    "0050A1": "CARLO GAVAZZI, INC.",
    "005017": "RSR S.R.L.",
    "005065": "TDK-Lambda Corporation",
    "0050B9": "XITRON TECHNOLOGIES, INC.",
    "00506B": "SPX-ATEG",
    "00504A": "ELTECO A.S.",
    "00D064": "MULTITEL",
    "00D05E": "STRATABEAM TECHNOLOGY, INC.",
    "00D0AA": "CHASE COMMUNICATIONS",
    "00D0FA": "Thales e-Security Ltd.",
    "00D0EB": "LIGHTERA NETWORKS, INC.",
    "00D006": "Cisco Systems, Inc",
    "00D02A": "Voxent Systems Ltd.",
    "00D08F": "ARDENT TECHNOLOGIES, INC.",
    "00D051": "O2 MICRO, INC.",
    "00D0BB": "Cisco Systems, Inc",
    "00D06E": "TRENDVIEW RECORDERS LTD.",
    "00D05C": "KATHREIN TechnoTrend GmbH",
    "00D05D": "INTELLIWORXX, INC.",
    "00D0A1": "OSKAR VIERLING GMBH + CO. KG",
    "00D0C0": "Cisco Systems, Inc",
    "00D076": "Bank of America",
    "0050C1": "GEMFLEX NETWORKS, LTD.",
    "005075": "KESTREL SOLUTIONS",
    "0050ED": "ANDA NETWORKS",
    "005096": "SALIX TECHNOLOGIES, INC.",
    "005024": "NAVIC SYSTEMS, INC.",
    "00D068": "IWILL CORPORATION",
    "005012": "CBL - GMBH",
    "0050F2": "MICROSOFT CORP.",
    "005029": "1394 PRINTER WORKING GROUP",
    "005081": "MURATA MACHINERY, LTD.",
    "0050AC": "MAPLE COMPUTER CORPORATION",
    "005049": "Arbor Networks Inc",
    "00902C": "DATA & CONTROL EQUIPMENT LTD.",
    "0090BD": "OMNIA COMMUNICATIONS, INC.",
    "0090B4": "WILLOWBROOK TECHNOLOGIES",
    "009003": "APLIO",
    "0050CF": "VANLINK COMMUNICATION TECHNOLOGY RESEARCH INSTITUTE",
    "0050A4": "IO TECH, INC.",
    "0050A3": "TransMedia Communications, Inc.",
    "00505C": "TUNDO CORPORATION",
    "0050B3": "VOICEBOARD CORPORATION",
    "0050CD": "DIGIANSWER A/S",
    "00502D": "ACCEL, INC.",
    "00503A": "DATONG ELECTRONICS LTD.",
    "005087": "TERASAKI ELECTRIC CO., LTD.",
    "00508C": "RSI SYSTEMS",
    "0050E1": "NS TECH ELECTRONICS SDN BHD",
    "0050DE": "SIGNUM SYSTEMS CORP.",
    "00507B": "MERLOT COMMUNICATIONS",
    "00500D": "SATORI ELECTORIC CO., LTD.",
    "005026": "COSYSTEMS, INC.",
    "0090CF": "NORTEL",
    "00902F": "NETCORE SYSTEMS, INC.",
    "009098": "SBC DESIGNS, INC.",
    "009045": "Marconi Communications",
    "009036": "ens, inc.",
    "00908B": "Tattile SRL",
    "009044": "ASSURED DIGITAL, INC.",
    "00907C": "DIGITALCAST, INC.",
    "009042": "ECCS, Inc.",
    "009051": "ULTIMATE TECHNOLOGY CORP.",
    "009026": "ADVANCED SWITCHING COMMUNICATIONS, INC.",
    "0090D3": "GIESECKE & DEVRIENT GmbH",
    "009067": "WalkAbout Computers, Inc.",
    "009025": "BAE Systems Australia (Electronic Systems) Pty Ltd",
    "00904C": "Epigram, Inc.",
    "009084": "ATECH SYSTEM",
    "00906A": "TURNSTONE SYSTEMS, INC.",
    "009087": "ITIS",
    "009031": "MYSTICOM, LTD.",
    "00909D": "NovaTech Process Solutions, LLC",
    "0090DD": "MIHARU COMMUNICATIONS Inc",
    "009028": "NIPPON SIGNAL CO., LTD.",
    "00907D": "Lake Communications",
    "0090C9": "DPAC Technologies",
    "00901D": "PEC (NZ) LTD.",
    "00907E": "VETRONIX CORP.",
    "009050": "Teleste Corporation",
    "00902A": "COMMUNICATION DEVICES, INC.",
    "009072": "SIMRAD AS",
    "00900D": "Overland Storage Inc.",
    "009033": "INNOVAPHONE AG",
    "009002": "ALLGON AB",
    "0090FC": "NETWORK COMPUTING DEVICES",
    "009014": "ROTORK INSTRUMENTS, LTD.",
    "00904D": "SPEC S.A.",
    "0090FD": "CopperCom, Inc.",
    "009039": "SHASTA NETWORKS",
    "009091": "DigitalScape, Inc.",
    "00908D": "VICKERS ELECTRONICS SYSTEMS",
    "009097": "Sycamore Networks",
    "000629": "IBM Corp",
    "0010A9": "ADHOC TECHNOLOGIES",
    "00108A": "TeraLogic, Inc.",
    "001024": "NAGOYA ELECTRIC WORKS CO., LTD",
    "0010D6": "Exelis",
    "001048": "HTRC AUTOMATION, INC.",
    "001097": "WinNet Metropolitan Communications Systems, Inc.",
    "001085": "POLARIS COMMUNICATIONS, INC.",
    "00100C": "ITO CO., LTD.",
    "001006": "Thales Contact Solutions Ltd.",
    "00106F": "TRENTON TECHNOLOGY INC.",
    "001034": "GNP Computers",
    "001044": "InnoLabs Corporation",
    "00908E": "Nortel Networks Broadband Access",
    "009009": "I Controls, Inc.",
    "0090D2": "Artel Video Systems",
    "0001FE": "DIGITAL EQUIPMENT CORPORATION",
    "0010D9": "IBM JAPAN, FUJISAWA MT+D",
    "00103C": "IC ENSEMBLE, INC.",
    "001019": "SIRONA DENTAL SYSTEMS GmbH & Co. KG",
    "00107F": "CRESTRON ELECTRONICS, INC.",
    "0010D4": "STORAGE COMPUTER CORPORATION",
    "0010E2": "ArrayComm, Inc.",
    "0090DE": "CARDKEY SYSTEMS, INC.",
    "00906B": "APPLIED RESOURCES, INC.",
    "0090BE": "IBC/INTEGRATED BUSINESS COMPUTERS",
    "009066": "Troika Networks, Inc.",
    "0010D2": "NITTO TSUSHINKI CO., LTD",
    "001094": "Performance Analysis Broadband, Spirent plc",
    "001050": "RION CO., LTD.",
    "0010CE": "VOLAMP, LTD.",
    "0010B2": "COACTIVE AESTHETICS",
    "0010A1": "KENDIN SEMICONDUCTOR, INC.",
    "00105F": "ZODIAC DATA SYSTEMS",
    "00103E": "NETSCHOOLS CORPORATION",
    "0010CB": "FACIT K.K.",
    "0010E0": "Oracle Corporation ",
    "00107C": "P-COM, INC.",
    "0010BD": "THE TELECOMMUNICATION TECHNOLOGY COMMITTEE (TTC)",
    "001038": "Micro Research Ltd.",
    "0010A8": "RELIANCE COMPUTER CORP.",
    "0010D1": "Top Layer Networks, Inc.",
    "00106A": "DIGITAL MICROWAVE CORPORATION",
    "00103B": "HIPPI NETWORKING FORUM",
    "001089": "WebSonic",
    "001008": "VIENNA SYSTEMS CORPORATION",
    "00106E": "TADIRAN COM. LTD.",
    "00109A": "NETLINE",
    "00E0D1": "TELSIS LIMITED",
    "00E0F0": "ABLER TECHNOLOGY, INC.",
    "00E002": "CROSSROADS SYSTEMS, INC.",
    "00E0D6": "COMPUTER & COMMUNICATION RESEARCH LAB.",
    "00E0B7": "Cosworth Electronics Ltd",
    "00E083": "JATO TECHNOLOGIES, INC.",
    "00E072": "LYNK",
    "00E0AD": "EES TECHNOLOGY, LTD.",
    "00E094": "OSAI SRL",
    "00E032": "MISYS FINANCIAL SYSTEMS, LTD.",
    "00E0C0": "SEIWA ELECTRIC MFG. CO., LTD.",
    "00E027": "DUX, INC.",
    "00E044": "LSICS CORPORATION",
    "00E0CA": "BEST DATA PRODUCTS",
    "00E0A7": "IPC INFORMATION SYSTEMS, INC.",
    "00E097": "CARRIER ACCESS CORPORATION",
    "00E089": "ION Networks, Inc.",
    "00E070": "DH TECHNOLOGY",
    "00E055": "INGENIERIA ELECTRONICA COMERCIAL INELCOM S.A.",
    "00E042": "Pacom Systems Ltd.",
    "00E0EB": "DIGICOM SYSTEMS, INCORPORATED",
    "00E04B": "JUMP INDUSTRIELLE COMPUTERTECHNIK GmbH",
    "00E074": "TIERNAN COMMUNICATIONS, INC.",
    "00E0D9": "TAZMO CO., LTD.",
    "00E093": "ACKFIN NETWORKS",
    "00E071": "EPIS MICROCOMPUTER",
    "00E066": "ProMax Systems, Inc.",
    "00E0B4": "TECHNO SCOPE CO., LTD.",
    "00605B": "IntraServer Technology, Inc.",
    "0060D7": "ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE (EPFL)",
    "0060F3": "Performance Analysis Broadband, Spirent plc",
    "00607C": "WaveAccess, Ltd.",
    "0060C1": "WaveSpan Corporation",
    "00603C": "HAGIWARA SYS-COM CO., LTD.",
    "00607D": "SENTIENT NETWORKS INC.",
    "00606E": "DAVICOM SEMICONDUCTOR, INC.",
    "0060C7": "AMATI COMMUNICATIONS CORP.",
    "00E060": "SHERWOOD",
    "00E0BA": "BERGHOF AUTOMATIONSTECHNIK GmbH",
    "00E021": "FREEGATE CORP.",
    "00E05B": "WEST END SYSTEMS CORP.",
    "00E062": "HOST ENGINEERING",
    "00E001": "STRAND LIGHTING LIMITED",
    "00E0D8": "LANBit Computer, Inc.",
    "00E01F": "AVIDIA Systems, Inc.",
    "00E0D0": "NETSPEED, INC.",
    "006073": "REDCREEK COMMUNICATIONS, INC.",
    "0060FD": "NetICs, Inc.",
    "0060FB": "PACKETEER, INC.",
    "00E0CE": "ARN",
    "00E05F": "e-Net, Inc.",
    "00E0E7": "RAYTHEON E-SYSTEMS, INC.",
    "00E073": "NATIONAL AMUSEMENT NETWORK, INC.",
    "00E06A": "KAPSCH AG",
    "00E03C": "AdvanSys",
    "00E024": "GADZOOX NETWORKS",
    "1000E8": "NATIONAL SEMICONDUCTOR",
    "006031": "HRK SYSTEMS",
    "0060ED": "RICARDO TEST AUTOMATION LTD.",
    "006012": "POWER COMPUTING CORPORATION",
    "00604D": "MMC NETWORKS, INC.",
    "0060DD": "MYRICOM, INC.",
    "0060F4": "ADVANCED COMPUTER SOLUTIONS, Inc.",
    "006060": "Data Innovations North America",
    "0060F7": "DATAFUSION SYSTEMS",
    "006020": "PIVOTAL NETWORKING, INC.",
    "0060C0": "Nera Networks AS",
    "00600E": "WAVENET INTERNATIONAL, INC.",
    "0060A0": "SWITCHED NETWORK TECHNOLOGIES, INC.",
    "006017": "TOKIMEC INC.",
    "006026": "VIKING Modular Solutions",
    "006019": "Roche Diagnostics",
    "006059": "TECHNICAL COMMUNICATIONS CORP.",
    "006003": "TERAOKA WEIGH SYSTEM PTE, LTD.",
    "006095": "ACCU-TIME SYSTEMS, INC.",
    "0060CB": "HITACHI ZOSEN CORPORATION",
    "0060C8": "KUKA WELDING SYSTEMS & ROBOTS",
    "006023": "PERICOM SEMICONDUCTOR CORP.",
    "006063": "PSION DACOM PLC.",
    "006035": "DALLAS SEMICONDUCTOR, INC.",
    "006007": "ACRES GAMING, INC.",
    "006058": "COPPER MOUNTAIN COMMUNICATIONS, INC.",
    "00607A": "DVS GMBH",
    "006066": "LACROIX Trafic",
    "006077": "PRISA NETWORKS",
    "006094": "IBM Corp",
    "0060AB": "LARSCOM INCORPORATED",
    "00605A": "CELCORE, INC.",
    "0060A3": "CONTINUUM TECHNOLOGY CORP.",
    "00603D": "3CX",
    "006046": "VMETRO, INC.",
    "00A0DA": "INTEGRATED SYSTEMS Technology, Inc.",
    "00A02A": "TRANCELL SYSTEMS",
    "00A01C": "NASCENT NETWORKS CORPORATION",
    "00A09A": "NIHON KOHDEN AMERICA",
    "00A078": "Marconi Communications",
    "00A0BF": "WIRELESS DATA GROUP MOTOROLA",
    "00A05F": "BTG Electronics Design BV",
    "00A0CD": "DR. JOHANNES HEIDENHAIN GmbH",
    "00A02D": "1394 Trade Association",
    "00A07C": "TONYANG NYLON CO., LTD.",
    "00A0E6": "DIALOGIC CORPORATION",
    "00A04A": "NISSHIN ELECTRIC CO., LTD.",
    "00A035": "CYLINK CORPORATION",
    "00A03F": "COMPUTER SOCIETY MICROPROCESSOR & MICROPROCESSOR STANDARDS C",
    "00A03D": "OPTO-22",
    "00A026": "TELDAT, S.A.",
    "00A023": "APPLIED CREATIVE TECHNOLOGY, INC.",
    "00A08F": "DESKNET SYSTEMS, INC.",
    "00A0CC": "LITE-ON COMMUNICATIONS, INC.",
    "00A0B7": "CORDANT, INC.",
    "00A056": "MICROPROSS",
    "00A0E1": "WESTPORT RESEARCH ASSOCIATES, INC.",
    "00608A": "CITADEL COMPUTER",
    "006093": "VARIAN",
    "00A089": "XPOINT TECHNOLOGIES, INC.",
    "00A007": "APEXX TECHNOLOGY, INC.",
    "00A047": "INTEGRATED FITNESS CORP.",
    "00A032": "GES SINGAPORE PTE. LTD.",
    "00A0E3": "XKL SYSTEMS CORP.",
    "00A014": "CSIR",
    "00A015": "WYLE",
    "00A06A": "Verilink Corporation",
    "00A070": "COASTCOM",
    "002020": "MEGATRON COMPUTER INDUSTRIES PTY, LTD.",
    "00201B": "NORTHERN TELECOM/NETWORK",
    "0020C0": "PULSE ELECTRONICS, INC.",
    "00206F": "FLOWPOINT CORPORATION",
    "00A018": "CREATIVE CONTROLLERS, INC.",
    "00A0FE": "BOSTON TECHNOLOGY, INC.",
    "00A0EB": "Encore Networks, Inc.",
    "00A07D": "SEEQ TECHNOLOGY, INC.",
    "00A0D9": "CONVEX COMPUTER CORPORATION",
    "0020DE": "JAPAN DIGITAL LABORAT'Y CO.LTD",
    "00200B": "OCTAGON SYSTEMS CORP.",
    "002094": "CUBIX CORPORATION",
    "0020F7": "CYBERDATA CORPORATION",
    "0020D7": "JAPAN MINICOMPUTER SYSTEMS CO., Ltd.",
    "0020C3": "COUNTER SOLUTIONS LTD.",
    "0020FB": "OCTEL COMMUNICATIONS CORP.",
    "002047": "STEINBRECHER CORP.",
    "0020D5": "VIPA GMBH",
    "00201A": "MRV Communications, Inc.",
    "0020F2": "Oracle Corporation ",
    "0020B8": "PRIME OPTION, INC.",
    "002062": "SCORPION LOGIC, LTD.",
    "002081": "TITAN ELECTRONICS",
    "0020D9": "PANASONIC TECHNOLOGIES, INC./MIECO-US",
    "00207E": "FINECOM CO., LTD.",
    "0020AD": "LINQ SYSTEMS",
    "00207D": "ADVANCED COMPUTER APPLICATIONS",
    "00202F": "ZETA COMMUNICATIONS, LTD.",
    "00209A": "THE 3DO COMPANY",
    "002001": "DSP SOLUTIONS, INC.",
    "0020BF": "AEHR TEST SYSTEMS",
    "00204E": "NETWORK SECURITY SYSTEMS, INC.",
    "0020FF": "SYMMETRICAL TECHNOLOGIES",
    "002044": "GENITECH PTY LTD",
    "0020A1": "DOVATRON",
    "002070": "HYNET, LTD.",
    "0020BE": "LAN ACCESS CORP.",
    "00204A": "PRONET GMBH",
    "0020F3": "RAYNET CORPORATION",
    "002090": "ADVANCED COMPRESSION TECHNOLOGY, INC.",
    "002053": "HUNTSVILLE MICROSYSTEMS, INC.",
    "0020CA": "DIGITAL OCEAN",
    "002095": "RIVA ELECTRONICS",
    "00203F": "JUKI CORPORATION",
    "0020A9": "WHITE HORSE INDUSTRIAL",
    "002096": "Invensys",
    "0020EF": "USC CORPORATION",
    "002030": "ANALOG & DIGITAL SYSTEMS",
    "0020AC": "INTERFLEX DATENSYSTEME GMBH",
    "0020D8": "Nortel Networks",
    "002066": "GENERAL MAGIC, INC.",
    "002036": "BMC SOFTWARE",
    "0020F8": "CARRERA COMPUTERS, INC.",
    "00C02F": "OKUMA CORPORATION",
    "00C01E": "LA FRANCAISE DES JEUX",
    "00C0E1": "SONIC SOLUTIONS",
    "00C065": "SCOPE COMMUNICATIONS, INC.",
    "00C079": "FONSYS CO.,LTD.",
    "00406F": "SYNC RESEARCH INC.",
    "00404F": "SPACE & NAVAL WARFARE SYSTEMS",
    "00408F": "WM-DATA MINFO AB",
    "0040D7": "STUDIO GEN INC.",
    "004057": "LOCKHEED - SANDERS",
    "004017": "Silex Technology America",
    "00C071": "AREANEX COMMUNICATIONS, INC.",
    "00C0C4": "COMPUTER OPERATIONAL",
    "00C012": "NETSPAN CORPORATION",
    "00C0D9": "QUINTE NETWORK CONFIDENTIALITY",
    "00C0B1": "GENIUS NET CO.",
    "00C00F": "QUANTUM SOFTWARE SYSTEMS LTD.",
    "00C087": "UUNET TECHNOLOGIES, INC.",
    "00C006": "NIPPON AVIONICS CO., LTD.",
    "00C0D2": "SYNTELLECT, INC.",
    "00C07E": "KUBOTA CORPORATION ELECTRONIC",
    "00C0DD": "QLogic Corporation",
    "00C01B": "SOCKET COMMUNICATIONS, INC.",
    "00C03F": "STORES AUTOMATED SYSTEMS, INC.",
    "00C036": "RAYTECH ELECTRONIC CORP.",
    "00C040": "ECCI",
    "00C0A4": "UNIGRAF OY",
    "00C029": "Nexans Deutschland GmbH - ANS",
    "00C0FA": "CANARY COMMUNICATIONS, INC.",
    "00C01C": "INTERLINK COMMUNICATIONS LTD.",
    "00C096": "TAMURA CORPORATION",
    "00C04E": "COMTROL CORPORATION",
    "00C044": "EMCOM CORPORATION",
    "00C0E6": "Verilink Corporation",
    "00C042": "DATALUX CORP.",
    "00C0A2": "INTERMEDIUM A/S",
    "00C053": "Aspect Software Inc.",
    "00C0CC": "TELESCIENCES CO SYSTEMS, INC.",
    "00C0CE": "CEI SYSTEMS & ENGINEERING PTE",
    "00C08E": "NETWORK INFORMATION TECHNOLOGY",
    "00C0C7": "SPARKTRUM MICROSYSTEMS, INC.",
    "0040F3": "NETCOR",
    "00404B": "MAPLE COMPUTER SYSTEMS",
    "004033": "ADDTRON TECHNOLOGY CO., LTD.",
    "0040AD": "SMA REGELSYSTEME GMBH",
    "00406D": "LANCO, INC.",
    "0040CD": "TERA MICROSYSTEMS, INC.",
    "0040F5": "OEM ENGINES",
    "004039": "OPTEC DAIICHI DENKO CO., LTD.",
    "004079": "JUKO MANUFACTURE COMPANY, LTD.",
    "004059": "YOSHIDA KOGYO K. K.",
    "0040AE": "DELTA CONTROLS, INC.",
    "0040F6": "KATRON COMPUTERS INC.",
    "00C05F": "FINE-PAL COMPANY LIMITED",
    "0080D9": "EMK Elektronik GmbH & Co. KG",
    "00C020": "ARCO ELECTRONIC, CONTROL LTD.",
    "00C0E7": "FIBERDATA AB",
    "0040F4": "CAMEO COMMUNICATIONS, INC.",
    "0040B4": "NEXTCOM K.K.",
    "0040B0": "BYTEX CORPORATION, ENGINEERING",
    "004086": "MICHELS & KLEBERHOFF COMPUTER",
    "004092": "ASP COMPUTER PRODUCTS, INC.",
    "0040B9": "MACQ ELECTRONIQUE SA",
    "0040BB": "GOLDSTAR CABLE CO., LTD.",
    "0040B1": "CODONICS INC.",
    "00407F": "FLIR Systems",
    "004095": "R.P.T. INTERGROUPS INT'L LTD.",
    "004035": "OPCOM",
    "00405C": "FUTURE SYSTEMS, INC.",
    "004061": "DATATECH ENTERPRISES CO., LTD.",
    "00008C": "Alloy Computer Products (Australia) Pty Ltd",
    "0040C5": "MICOM COMMUNICATIONS INC.",
    "004020": "CommScope Inc",
    "004048": "SMD INFORMATICA S.A.",
    "004068": "EXTENDED SYSTEMS",
    "004078": "WEARNES AUTOMATION PTE LTD",
    "0040F8": "SYSTEMHAUS DISCOM",
    "004024": "COMPAC INC.",
    "0040D2": "PAGINE CORPORATION",
    "0040E9": "ACCORD SYSTEMS, INC.",
    "004003": "Emerson Process Management Power & Water Solutions, Inc.",
    "004090": "ANSEL COMMUNICATIONS",
    "00407C": "QUME CORPORATION",
    "004044": "QNIX COMPUTER CO., LTD.",
    "00402D": "HARRIS ADACOM CORPORATION",
    "008081": "KENDALL SQUARE RESEARCH CORP.",
    "00809C": "LUXCOM, INC.",
    "008065": "CYBERGRAPHIC SYSTEMS PTY LTD.",
    "008019": "DAYNA COMMUNICATIONS, INC.",
    "008050": "ZIATECH CORPORATION",
    "0080D2": "SHINNIHONDENKO CO., LTD.",
    "008089": "TECNETICS (PTY) LTD.",
    "00806F": "ONELAN LTD.",
    "008084": "THE CLOUD INC.",
    "0080A6": "REPUBLIC TECHNOLOGY, INC.",
    "0040FC": "IBR COMPUTER TECHNIK GMBH",
    "0040DE": "Elsag Datamat spa",
    "0040C9": "NCUBE",
    "0080CF": "EMBEDDED PERFORMANCE INC.",
    "008090": "MICROTEK INTERNATIONAL, INC.",
    "0080C4": "DOCUMENT TECHNOLOGIES, INC.",
    "0040A0": "GOLDSTAR CO., LTD.",
    "0040DF": "DIGALOG SYSTEMS, INC.",
    "004009": "TACHIBANA TECTRON CO., LTD.",
    "008032": "ACCESS CO., LTD.",
    "00805B": "CONDOR SYSTEMS, INC.",
    "008043": "NETWORLD, INC.",
    "0080AF": "ALLUMER CO., LTD.",
    "000055": "COMMISSARIAT A L`ENERGIE ATOM.",
    "000041": "ICE CORPORATION",
    "000086": "MEGAHERTZ CORPORATION",
    "000092": "COGENT DATA TECHNOLOGIES",
    "000058": "RACORE COMPUTER PRODUCTS INC.",
    "0080CE": "BROADCAST TELEVISION SYSTEMS",
    "00800C": "VIDECOM LIMITED",
    "00807D": "EQUINOX SYSTEMS INC.",
    "008063": "Hirschmann Automation and Control GmbH",
    "008074": "FISHER CONTROLS",
    "008030": "NEXUS ELECTRONICS",
    "0080C9": "ALBERTA MICROELECTRONIC CENTRE",
    "0080EE": "THOMSON CSF",
    "00808E": "RADSTONE TECHNOLOGY",
    "00801A": "BELL ATLANTIC",
    "008052": "TECHNICALLY ELITE CONCEPTS",
    "00805D": "CANSTAR",
    "00804F": "DAIKIN INDUSTRIES, LTD.",
    "008005": "CACTUS COMPUTER INC.",
    "0080CD": "MICRONICS COMPUTER, INC.",
    "008003": "HYTEC ELECTRONICS LTD.",
    "008099": "Eaton Industries GmbH",
    "00808D": "WESTCOAST TECHNOLOGY B.V.",
    "0080BE": "ARIES RESEARCH",
    "008015": "SEIKO SYSTEMS, INC.",
    "0080A4": "LIBERTY ELECTRONICS",
    "0080F3": "SUN ELECTRONICS CORP.",
    "008096": "HUMAN DESIGNED SYSTEMS, INC.",
    "0080DA": "Hottinger Br\u00fcel & Kj\u00e6r A/S",
    "00803E": "SYNERNETICS",
    "0080DE": "GIPSI S.A.",
    "00806D": "CENTURY SYSTEMS CORP.",
    "008094": "ALFA LAVAL AUTOMATION AB",
    "008002": "SATELCOM (UK) LTD",
    "008064": "WYSE TECHNOLOGY LLC",
    "008048": "COMPEX INCORPORATED",
    "008085": "H-THREE SYSTEMS CORPORATION",
    "00804C": "CONTEC CO., LTD.",
    "00808F": "C. ITOH ELECTRONICS, INC.",
    "000052": "Intrusion.com, Inc.",
    "0080FF": "SOC. DE TELEINFORMATIQUE RTC",
    "000037": "OXFORD METRICS LIMITED",
    "0000F7": "YOUTH KEEP ENTERPRISE CO LTD",
    "0000C7": "ARIX CORPORATION",
    "0000D1": "ADAPTEC INCORPORATED",
    "000016": "DU PONT PIXEL SYSTEMS     .",
    "0000E2": "ACER TECHNOLOGIES CORP.",
    "0000E1": "GRID SYSTEMS",
    "000081": "Bay Networks",
    "000029": "IMC NETWORKS CORP.",
    "0000FB": "RECHNER ZUR KOMMUNIKATION",
    "00000D": "FIBRONICS LTD.",
    "00000A": "OMRON TATEISI ELECTRONICS CO.",
    "000024": "CONNECT AS",
    "0000CF": "HAYES MICROCOMPUTER PRODUCTS",
    "0000EF": "KTI",
    "000025": "RAMTEK CORP.",
    "000067": "SOFT * RITE, INC.",
    "0000D2": "SBE, INC.",
    "0000BA": "SIIG, INC.",
    "08007B": "SANYO ELECTRIC CO. LTD.",
    "08007C": "VITALINK COMMUNICATIONS CORP.",
    "08008E": "Tandem Computers",
    "080084": "TOMEN ELECTRONICS CORP.",
    "080085": "ELXSI",
    "080082": "VERITAS SOFTWARE",
    "080080": "AES DATA INC.",
    "00002A": "TRW - SEDD/INP",
    "00002C": "AUTOTOTE LIMITED",
    "00006C": "Private",
    "0000F3": "GANDALF DATA LIMITED",
    "0000B0": "RND-RAD NETWORK DEVICES",
    "0000A9": "NETWORK SYSTEMS CORP.",
    "0000DF": "BELL & HOWELL PUB SYS DIV",
    "0000F9": "QUOTRON SYSTEMS INC.",
    "0000AF": "Canberra Industries, Inc.",
    "000076": "ABEKAS VIDEO SYSTEM",
    "080041": "RACAL-MILGO INFORMATION SYS..",
    "080035": "MICROFIVE CORPORATION",
    "080032": "TIGAN INCORPORATED",
    "080077": "TSL COMMUNICATIONS LTD.",
    "080074": "CASIO COMPUTER CO. LTD.",
    "08006E": "MASSCOMP",
    "080053": "MIDDLE EAST TECH. UNIVERSITY",
    "08004C": "HYDRA COMPUTER SYSTEMS INC.",
    "080047": "SEQUENT COMPUTER SYSTEMS INC.",
    "08004A": "BANYAN SYSTEMS INC.",
    "080044": "DAVID SYSTEMS INC.",
    "080068": "RIDGE COMPUTERS",
    "080063": "PLESSEY",
    "080060": "INDUSTRIAL NETWORKING INC.",
    "080055": "STANFORD TELECOMM. INC.",
    "08000B": "UNISYS CORPORATION",
    "00DD01": "UNGERMANN-BASS INC.",
    "08008A": "PerfTech, Inc.",
    "00DD06": "UNGERMANN-BASS INC.",
    "02BB01": "OCTOTHORPE CORP.",
    "00003E": "SIMPACT",
    "080015": "STC BUSINESS SYSTEMS",
    "080042": "MACNICA, Inc.",
    "AA0003": "DIGITAL EQUIPMENT CORPORATION",
    "080066": "AGFA CORPORATION",
    "000000": "XEROX CORPORATION",
    "F82111": "HMD Global Oy",
    "EC9161": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "08008D": "XYVISION INC.",
    "00DD02": "UNGERMANN-BASS INC.",
    "00DD04": "UNGERMANN-BASS INC.",
    "08003F": "FRED KOSCHARA ENTERPRISES",
    "C0D60A": "Texas Instruments",
    "7C49CF": "eero inc.",
    "2883C9": "Apple, Inc.",
    "080021": "3M COMPANY",
    "026086": "LOGIC REPLACEMENT TECH. LTD.",
    "DCA0D0": "Amazon Technologies Inc.",
    "F02F9E": "Amazon Technologies Inc.",
    "403668": "E&B TELECOM",
    "183F70": "Apple, Inc.",
    "A4F6E8": "Apple, Inc.",
    "6441E6": "Apple, Inc.",
    "70BB5B": "Apple, Inc.",
    "E8519E": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "740EA4": "Apple, Inc.",
    "6057C8": "Apple, Inc.",
    "D4413F": "Gen IV Technology LLC",
    "44A038": "Huawei Device Co., Ltd.",
    "F86DCC": "Wistron Neweb Corporation",
    "D0D7BE": "HUAWEI TECHNOLOGIES CO.,LTD",
    "206BF4": "Huawei Device Co., Ltd.",
    "E82BC5": "Huawei Device Co., Ltd.",
    "08276B": "Huawei Device Co., Ltd.",
    "3CCD73": "Nebula Electronic Technology Corporation",
    "3C13BB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "CC36CF": "Cisco Systems, Inc",
    "58D8A7": "Bird Home Automation GmbH",
    "ECB1E0": "Eltex Enterprise LTD",
    "B40016": " INGENICO TERMINALS SAS",
    "C89CBB": "Taicang T&W Electronics",
    "345EE7": "Hangzhou ChengFengErLai Digial Technology Co.,Ltd.",
    "54B874": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "C45781": "Wingtech Group (HongKong) Limited",
    "F4DD06": "Samsung Electronics Co.,Ltd",
    "101EDA": " INGENICO TERMINALS SAS",
    "44D47F": " INGENICO TERMINALS SAS",
    "185D6F": "N3com",
    "6451F4": "Huawei Device Co., Ltd.",
    "4C60BA": "AltoBeam Inc.",
    "486345": "Huawei Device Co., Ltd.",
    "709041": "Mist Systems, Inc.",
    "785994": "Alif Semiconductor, Inc.",
    "14CB49": "Habolink Technology Co.,LTD",
    "94A4B5": "Mitsubishi Electric Corporation",
    "30C922": "Espressif Inc.",
    "C4955F": "Anhui Saida Technology Limited Liability Company",
    "CC1B5A": "Ruckus Wireless",
    "242D6C": "eero inc.",
    "3890AF": "zte corporation",
    "1879FD": "zte corporation",
    "64A444": "Loongson Technology Corporation Limited",
    "7048B5": "CTS System Co., LTD.",
    "18314F": "AIDIN ROBOTICS",
    "681D4C": "eSystems MTG GmbH",
    "0C6743": "HUAWEI TECHNOLOGIES CO.,LTD",
    "F47946": "HUAWEI TECHNOLOGIES CO.,LTD",
    "30B851": "Siemens AG",
    "780B8C": "Private",
    "9433D8": "Cisco Systems, Inc",
    "840A9E": "Nexapp Technologies Pvt Ltd",
    "D06B78": "Apple, Inc.",
    "0C53B7": "Apple, Inc.",
    "7CF34D": "Apple, Inc.",
    "000F2C": "Lantronix",
    "FCD586": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "D802C0": "Extreme Networks Headquarters",
    "0CDC91": "Amazon Technologies Inc.",
    "E8CBF5": "Laird Connectivity",
    "981C42": "LAIIER",
    "58350F": "EM Microelectronic",
    "3067A1": "Sagemcom Broadband SAS",
    "D01BF4": "Sagemcom Broadband SAS",
    "ACA09D": "Juniper Networks",
    "78BBC1": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "E4BC96": "Versuni",
    "E023FF": "Fortinet, Inc.",
    "E8EDD6": "Fortinet, Inc.",
    "70BC48": "Cisco Systems, Inc",
    "E81CBA": "Fortinet, Inc.",
    "D0C730": "zte corporation",
    "2459E5": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "20C792": "Wuhan Maiwe communication Co.,Ltd",
    "A0BC6F": "Cisco Systems, Inc",
    "B08D57": "Cisco Systems, Inc",
    "189341": "Intel Corporate",
    "DC4546": "Intel Corporate",
    "94CE0F": "Huawei Device Co., Ltd.",
    "883F27": "Huawei Device Co., Ltd.",
    "24E29D": "Huawei Device Co., Ltd.",
    "2C2080": "Huawei Device Co., Ltd.",
    "CCBC2B": "Huawei Device Co., Ltd.",
    "C02B56": "CANDID OPTRONIX PRIVATE LIMITED",
    "DC97BA": "Intel Corporate",
    "A0D365": "Intel Corporate",
    "B402F2": "Synaptics, Inc",
    "A02DDB": "Sagemcom Broadband SAS",
    "744DDC": "Sonim Technologies, Inc",
    "B8A14A": "Raisecom Technology CO., LTD",
    "3C6D66": "NVIDIA Corporation",
    "74D5C6": "Microchip Technologies Inc",
    "BC09EB": "TECNO MOBILE LIMITED",
    "5C640F": "Sage Technologies Inc.",
    "84B4D2": "Shenzhen iComm Semiconductor CO.,LTD",
    "38A746": "COMPAL INFORMATION (KUNSHAN) CO., LTD. ",
    "6CE2D3": "New H3C Technologies Co., Ltd",
    "5C167D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "50E452": "Chipsea Technologies (Shenzhen) Corp.",
    "A4A490": "Samsung Electronics Co.,Ltd",
    "B8A825": "Samsung Electronics Co.,Ltd",
    "CCF9F0": "Samsung Electronics Co.,Ltd",
    "F0A882": "shenzhen trolink  Technology Co.,Ltd",
    "10322C": "Murata Manufacturing Co., Ltd.",
    "DCC49C": "Samsung Electronics Co.,Ltd",
    "E4C770": "Raisecom Technology CO., LTD",
    "E067B3": "Shenzhen C-Data Technology Co., Ltd.",
    "70A56A": "Shenzhen C-Data Technology Co., Ltd.",
    "DC8E8D": "Netis Technology Co., Ltd.",
    "E4BEED": "Netis Technology Co., Ltd.",
    "007263": "Netis Technology Co., Ltd.",
    "EC64C9": "Espressif Inc.",
    "F0F5BD": "Espressif Inc.",
    "F84288": "Apple, Inc.",
    "88B7EB": "Apple, Inc.",
    "10A2D3": "Apple, Inc.",
    "F44250": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "BC62CE": "Netis Technology Co., Ltd.",
    "98D7E1": "Cisco Systems, Inc",
    "F83918": "Cisco Systems, Inc",
    "84B1E2": "Microsoft Corporation",
    "FC34E2": "Shenzhen YOUHUA Technology Co., Ltd\t",
    "4C97CC": "Apple, Inc.",
    "34EE16": "Apple, Inc.",
    "48849D": "Texas Instruments",
    "600F6B": "Apple, Inc.",
    "44F971": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "386B1C": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "349677": "zte corporation",
    "2475FC": "zte corporation",
    "508965": "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.",
    "C03FBB": "Zhongshan Zhiniu Electronics Co.,Ltd.",
    "5C579E": "Chiun Mai Communication System, Inc",
    "441A84": "Quectel Wireless Solutions Co.,Ltd.",
    "705464": "Silicon Laboratories",
    "8C6FB9": "Silicon Laboratories",
    "142D41": "Silicon Laboratories",
    "048727": "Silicon Laboratories",
    "4C5BB3": "Silicon Laboratories",
    "287681": "Silicon Laboratories",
    "30FB10": "Silicon Laboratories",
    "94B216": "Silicon Laboratories",
    "8C65A3": "Silicon Laboratories",
    "B43A31": "Silicon Laboratories",
    "28F8C6": "IEEE Registration Authority",
    "C88AD8": "CHINA DRAGON TECHNOLOGY LIMITED",
    "D058AB": "Mara Tech LLC",
    "9440F3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "406F27": "HUAWEI TECHNOLOGIES CO.,LTD",
    "18CAA7": "zte corporation",
    "D487CC": "Sivantos GmbH",
    "003358": "Ruckus Wireless",
    "644EEB": "Daikin Holdings Singapore Pte Ltd",
    "E04C12": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24E8CE": "Qingdao Haier Technology Co.,Ltd",
    "9C3818": "Cisco Systems, Inc",
    "B44C90": "Cisco Systems, Inc",
    "DCEAE7": "D-Link Corporation",
    "8CA682": "IEEE Registration Authority",
    "4CA919": "Tuya Smart Inc.",
    "A8EA71": "Fiberhome Telecommunication Technologies Co.,LTD",
    "BC629C": "zte corporation",
    "D41761": "Xiaomi Communications Co Ltd",
    "98254A": "TP-LINK CORPORATION PTE. LTD.",
    "242FD0": "TP-LINK CORPORATION PTE. LTD.",
    "D825DF": "CAME UK",
    "D4B5CD": "Sagemcom Broadband SAS",
    "684983": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2CECF7": "Shenzhen SDMC Technology CO.,Ltd.",
    "90F721": "IndiNatus (IndiNatus India Private Limited)",
    "E8DA3E": "Huawei Device Co., Ltd.",
    "ECB878": "Huawei Device Co., Ltd.",
    "38F45E": "H1-Radio co.,ltd",
    "74FECE": "TP-LINK CORPORATION PTE. LTD.",
    "78F505": "Murata Manufacturing Co., Ltd.",
    "588D39": "MITSUBISHI ELECTRIC AUTOMATION (CHINA) LTD.",
    "B8E28C": "MOTOROLA SOLUTIONS MALAYSIA SDN. BHD.",
    "E0DAD7": "zte corporation",
    "94BB43": "AzureWave Technology Inc.",
    "188B0E": "Espressif Inc.",
    "30A3B5": "Jiangsu Best Tone Information Service Co., Ltd",
    "B8F774": "Fiberhome Telecommunication Technologies Co.,LTD",
    "2CCF67": "Raspberry Pi (Trading) Ltd",
    "444963": "Woven By Toyota U.S., Inc.",
    "6074B1": "Home Control Singapore Pte Ltd",
    "34C8D6": "IEEE Registration Authority",
    "281DFB": "HUAWEI TECHNOLOGIES CO.,LTD",
    "0C238D": "HUAWEI TECHNOLOGIES CO.,LTD",
    "DC6180": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A86F36": "vivo Mobile Communication Co., Ltd.",
    "C0A476": "Ruijie Networks Co.,LTD",
    "20019C": "Bigleaf Networks Inc.",
    "48A3BD": "Texas Instruments",
    "042322": "Texas Instruments",
    "28DE59": "Domus NTW CORP.",
    "B8D0F0": "FCNT LLC",
    "102F6E": "Shenzhen Sundray Technologies Company Limited",
    "702AD7": "New H3C Intelligence Terminal Co., Ltd.",
    "ECC342": "zte corporation",
    "94BE50": "Remote Solution",
    "50CF56": "China Mobile Group Device Co.,Ltd.",
    "AC0775": "Apple, Inc.",
    "842F57": "Apple, Inc.",
    "1881ED": "TERACOM TELEMATICA S.A",
    "245A5F": "TP-LINK TECHNOLOGIES CO.,LTD.",
    "00109C": "MG Co., Ltd.",
    "D44D9F": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "9CD4A6": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "A86308": "AVUTEC",
    "684216": "Steplock Access AB",
    "6CF2D8": "CANON INC.",
    "F0FC65": "SynaXG Technologies Pte. Ltd.",
    "A427A5": "Palo Alto Networks",
    "002480": "Meteocontrol GmbH",
    "C4BD8D": "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD",
    "944BF8": "TOZO INC",
    "989DB2": "GOIP Global Services Pvt. Ltd.",
    "683421": "Intel Corporate",
    "002218": "Akamai Technologies Inc.",
    "00019C": "Lumentum",
    "68E905": "Fiberhome Telecommunication Technologies Co.,LTD",
    "34E3FB": "Samsung Electronics Co.,Ltd",
    "98348C": "Great Talent Technology Limited",
    "30F600": "Sagemcom Broadband SAS",
    "985F41": "Intel Corporate",
    "303A4A": "eero inc.",
    "AC3971": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "A044B7": "PAX Computer Technology(Shenzhen) Ltd.",
    "601D16": "Med-Eng Holdings ULC",
    "985A98": "HUAWEI TECHNOLOGIES CO.,LTD",
    "E0AD9B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "B05A44": "Fibocom Wireless Inc.",
    "38F18F": "Technicolor (China) Technology Co., Ltd.",
    "4099F6": "Telink Semiconductor(Shanghai) Co.,Ltd",
    "34C6DD": "Hangzhou Ezviz Software Co.,Ltd.",
    "CC8C17": "ITEL MOBILE LIMITED",
    "08A6F7": "Espressif Inc.",
    "34CFB5": "Robotic d.o.o.",
    "4C8237": "Telink Micro LLC",
    "849CA4": "Mimosa Networks",
    "00D6CB": "Quectel Wireless Solutions Co.,Ltd.",
    "ECC9FF": "Espressif Inc.",
    "2CBCBB": "Espressif Inc.",
    "883E0D": "HD Hyundai Electric",
    "B83B8F": "Hangzhou Hylin IoT Techonology Co.,Ltd.",
    "0003C5": "Mobotix AG",
    "58A8E8": "Amazon Technologies Inc.",
    "00E05C": "PHC Corporation",
    "640C91": "Apple, Inc.",
    "5893E8": "Apple, Inc.",
    "F004E1": "Apple, Inc.",
    "708CF2": "Apple, Inc.",
    "F873DF": "Apple, Inc.",
    "141BA0": "Apple, Inc.",
    "9C4FAC": "zte corporation",
    "E4B063": "Espressif Inc.",
    "A8C246": "Gemtek Technology Co., Ltd.",
    "8457F7": "Meta Platforms Technologies, LLC",
    "CC817D": "Apple, Inc.",
    "1C6920": "Espressif Inc.",
    "20F094": "Google, Inc.",
    "60C22A": "Allwinner Technology Co., Ltd",
    "700894": "Liteon Technology Corporation",
    "B0386C": "EFM Networks",
    "BCF212": "Telink Micro LLC",
    "8813BF": "Espressif Inc.",
    "345F45": "Espressif Inc.",
    "2C0B97": "Xiaomi Communications Co Ltd",
    "74238D": "zte corporation",
    "D069C1": "HUAWEI TECHNOLOGIES CO.,LTD",
    "542F2B": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A41162": "Arlo Technology",
    "08BAB7": "Ceragon Networks Ltd.",
    "A067D6": "Nokia",
    "E82689": "Hewlett Packard Enterprise",
    "B8D4E7": "Hewlett Packard Enterprise",
    "F860F0": "Hewlett Packard Enterprise",
    "1C28AF": "Hewlett Packard Enterprise",
    "28DE65": "Hewlett Packard Enterprise",
    "88E9A4": "Hewlett Packard Enterprise ",
    "9CDC71": "Hewlett Packard Enterprise ",
    "303FBB": "Hewlett Packard Enterprise ",
    "F42E7F": "Hewlett Packard Enterprise",
    "B83A5A": "Hewlett Packard Enterprise",
    "A8BD27": "Hewlett Packard Enterprise ",
    "40B93C": "Hewlett Packard Enterprise ",
    "E0071B": "Hewlett Packard Enterprise ",
    "D89403": "Hewlett Packard Enterprise ",
    "186472": "Hewlett Packard Enterprise",
    "6CF37F": "Hewlett Packard Enterprise",
    "24DEC6": "Hewlett Packard Enterprise",
    "84D47E": "Hewlett Packard Enterprise",
    "943FC2": "Hewlett Packard Enterprise ",
    "9440C9": "Hewlett Packard Enterprise ",
    "3810F0": "Hewlett Packard Enterprise",
    "CC88C7": "Hewlett Packard Enterprise",
    "EC0273": "Hewlett Packard Enterprise",
    "D04DC6": "Hewlett Packard Enterprise",
    "9460D5": "Hewlett Packard Enterprise",
    "882510": "Hewlett Packard Enterprise",
    "D4E053": "Hewlett Packard Enterprise",
    "441244": "Hewlett Packard Enterprise",
    "54F0B1": "Hewlett Packard Enterprise",
    "B837B2": "Hewlett Packard Enterprise",
    "0CD3A1": "Monthly Kitchen",
    "001EC6": "Leviton Manufacturing Co., Inc",
    "44A3BB": "Intel Corporate",
    "94B609": "Intel Corporate",
    "7C1E4A": "FORTUNE MARKETING PRIVATE LIMITED",
    "C0BFBE": "AzureWave Technology Inc.",
    "3CBD69": "Nokia Shanghai Bell Co., Ltd.",
    "1CEAAC": "Beijing Xiaomi Mobile Software Co., Ltd",
    "84652B": "Donaldson Company",
    "58DB09": "Quectel Wireless Solutions Co.,Ltd.",
    "783409": "HUAWEI TECHNOLOGIES CO.,LTD",
    "149AA3": "HUAWEI TECHNOLOGIES CO.,LTD",
    "A0AD62": "HUAWEI TECHNOLOGIES CO.,LTD",
    "5C87D8": "IEEE Registration Authority",
    "50B140": "ELPROMA ELEKTRONIKA SP Z O O",
    "8CC5D0": "Samsung Electronics Co.,Ltd",
    "28B591": "Cisco Systems, Inc",
    "08023C": "Samsung Electronics Co.,Ltd",
    "78B6FE": "Samsung Electronics Co.,Ltd",
    "9CCE88": "Ruijie Networks Co.,LTD",
    "9054B7": "Eltex Enterprise LTD",
    "BC9424": "TCT mobile ltd",
    "849E56": "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.",
    "249E7D": "Beijing Roborock Technology Co., Ltd.",
    "5C8AAE": "CHINA DRAGON TECHNOLOGY LIMITED",
    "909F22": "Zyxel Communications Corporation",
    "2CCA16": "Apple, Inc.",
    "14147D": "Apple, Inc.",
    "DC69B5": "eero inc.",
    "102381": "Barrot Technology Co.,LTD",
    "E8B723": "Shenzhen Vatilon Electronics Co.,Ltd",
    "1889A0": "Funshion Online Technologies Co.,Ltd",
    "983F66": "Funshion Online Technologies Co.,Ltd",
    "8431A8": "Funshion Online Technologies Co.,Ltd",
    "7C2C67": "Espressif Inc.",
    "98874C": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "C08A60": "AltoBeam Inc.",
    "C8608F": "Cisco Systems, Inc",
    "B8C007": "tickIoT Inc.",
    "10907D": "Funshion Online Technologies Co.,Ltd",
    "C05D89": "Espressif Inc.",
    "400326": "NXP Semiconductor (Tianjin) LTD.",
    "1C6A1B": "Ubiquiti Inc",
    "2005B6": "OpenWrt",
    "68F125": "Data Controls Inc.",
    "008320": "Huawei Device Co., Ltd.",
    "7C68B9": "Huawei Device Co., Ltd.",
    "9028F6": "Wuhan Yangtze Optical Technology CO.,Ltd.",
    "20A200": "HUAWEI TECHNOLOGIES CO.,LTD",
    "7CA8EC": "Hewlett Packard Enterprise",
    "F4D0A7": "Zhejiang Weilai Jingling Artificial Intelligence Technology Co., Ltd.",
    "20F307": "zte corporation",
    "B4E3D0": "LG Electronics",
    "3CD1C9": "Groupe SEB",
    "EC554D": "Private",
    "084F66": "Shenzhen Skyworth Digital  Technology  CO., Ltd",
    "CCDA20": "Beijing Xiaomi Mobile Software Co., Ltd",
    "7CF17E": "TP-LINK CORPORATION PTE. LTD.",
    "B0F2F6": "Samsung Electronics Co.,Ltd",
    "241972": "Quectel Wireless Solutions Co.,Ltd.",
    "503D7F": "Nokia Shanghai Bell Co., Ltd.",
    "001710": "AxyomCore Inc.",
    "143A9A": "Hon Hai Precision Industry Co.,LTD",
    "C8E07A": "Fiberhome Telecommunication Technologies Co.,LTD",
    "D069FF": "Fiberhome Telecommunication Technologies Co.,LTD",
    "84FFC2": "Cisco Systems, Inc",
    "788517": "Cisco Systems, Inc",
    "48A4FD": "AltoBeam Inc.",
    "48F1EB": "Nintendo Co.,Ltd",
    "501B6A": "SUNITEC TECHNOLOGY CO.,LIMITED",
    "8C0DD9": "Skullcandy",
    "6C92BF": "IEIT SYSTEMS Co., Ltd.",
    "9CC2C4": "IEIT SYSTEMS Co., Ltd.",
    "4465E0": "Merlion Consulting Services (Shenzhen) Co., Ltd",
    "741EB1": "Samsung Electronics Co.,Ltd",
    "A49FE7": "Samsung Electronics Co.,Ltd",
    "A41A6E": "zte corporation",
    "CC3F36": "Apple, Inc.",
    "102FCA": "Apple, Inc.",
    "542097": "TTTech Auto  AG",
    "28D244": "LCFC(Hefei) Electronics Technology co., ltd",
    "507B9D": "LCFC(Hefei) Electronics Technology co., ltd",
    "88A4C2": "LCFC(Hefei) Electronics Technology co., ltd",
    "6C2408": "LCFC(Hefei) Electronics Technology co., ltd",
    "9C2DCD": "LCFC(Hefei) Electronics Technology co., ltd",
    "D048A1": "Juniper Networks",
    "68F728": "LCFC(Hefei) Electronics Technology co., ltd",
    "C85B76": "LCFC(Hefei) Electronics Technology co., ltd",
    "8C1645": "LCFC(Hefei) Electronics Technology co., ltd",
    "38F3AB": "LCFC(Hefei) Electronics Technology co., ltd",
    "98F1AE": "Senaisc",
    "F04B8A": "Vantiva USA LLC",
    "2CEADA": "ICC Intelligent Platforms GmbH",
    "C0470E": "Dell Inc.",
    "ACA613": "Aivres SYSTEMS INC",
    "2496D5": "NEXCON Technology Co.,ltd.",
    "DC6279": "TP-LINK CORPORATION PTE. LTD.",
    "CC38D0": "Palo Alto Networks",
    "F49EA4": "Epiq Solutions",
    "8006D9": "zte corporation",
    "94C36B": "DRD Automation GmbH",
    "D48C49": "Espressif Inc.",
    "BC0435": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "E43022": "Hanwha Vision VietNam",
    "C096A4": "Fiberhome Telecommunication Technologies Co.,LTD",
    "EC7359": "Shenzhen Cloudsky Technologies Co., Ltd.",
    "80C01E": "Intel Corporate",
    "4C4929": "TCL King Electrical Appliances(Huizhou)Co.,Ltd",
    "94EC32": "Silicon Laboratories",
    "345A60": "Micro-Star INTL CO., LTD.",
    "BCA13A": "VusionGroup",
    "C4B691": "Angel Robotics",
    "C88234": "Cisco Systems, Inc",
    "AC86D1": "IEEE Registration Authority",
    "782288": "SHENZHEN BILIAN ELECTRONIC CO.\uff0cLTD",
    "84BE8B": "Chengdu Geeker Technology Co., Ltd.",
    "98A14A": "Quectel Wireless Solutions Co.,Ltd.",
    "F8B3B7": "Espressif Inc.",
    "C4EB68": "VusionGroup",
    "AC3351": "Ericsson AB",
    "E4B323": "Espressif Inc.",
    "30DDAA": "ZHEJIANG DAHUA TECHNOLOGYCO.,LTD",
    "B89165": "Zhejiang Tmall Technology Co., Ltd.",
    "709AC4": "Huawei Device Co., Ltd.",
    "C8DD6A": "OHSUNG",
    "C4FF84": "IEEE Registration Authority",
    "1C4D89": "Hangzhou Huacheng Network Technology Co.,Ltd",
    "344A1B": "zte corporation",
    "B4BA6A": "TECNO MOBILE LIMITED",
    "045665": "Nokia Shanghai Bell Co., Ltd.",
    "B44130": "Jabil Circuit (Guangzhou) Ltd.",
    "A0334F": "Cisco Systems, Inc",
    "A0A47F": "Cisco Systems, Inc",
    "9070BF": "Mimosa Networks",
    "E09D73": "Mellanox Technologies, Inc.",
    "602D74": "Extreme Networks Headquarters",
    "44370B": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "ACB480": "Dell Inc.",
    "28A915": "SERVERCOM (INDIA) PRIVATE LIMITED",
    "10A30F": "HUAWEI TECHNOLOGIES CO.,LTD",
    "24AECC": "Honor Device Co., Ltd.",
    "8C1F64C16": "ALISONIC SRL",
    "8C1F64AFA": "DATA ELECTRONIC DEVICES, INC",
    "8C1F649B9": "QUERCUS TECHNOLOGIES, S.L.",
    "8C1F64D0F": "Mecco LLC",
    "8C1F649F8": "Exypnos - Creative Solutions LTD",
    "8C1F64129": "Navtech Radar Ltd.",
    "8C1F64B7A": "MG s.r.l.",
    "8C1F643EE": "BnB Information Technology",
    "70B3D5060": "RCH SPA",
    "70B3D5786": "RCH SPA",
    "8C1F64096": "IPCOMM GmbH",
    "8C1F64055": "Intercreate",
    "8C1F64CA9": "Avant Technologies",
    "8C1F64CFD": "Smart-VOD Pty Ltd",
    "8C1F6427B": "Oriux",
    "8C1F647E1": "HEITEC AG",
    "8C1F6403D": "HORIZON.INC",
    "8C1F6480F": "ASYS Corporation",
    "8C1F64589": "HVRND",
    "8C1F64C8D": "Aeronautics Ltd.",
    "8C1F645F1": "HD Link Co., Ltd.",
    "8C1F64048": "FieldLine Medical",
    "8C1F6456E": "Euklis srl",
    "8C1F64899": "American Edge IP",
    "8C1F64C1C": "Vektrex Electronics Systems, Inc.",
    "8C1F64C0A": "ACROLABS,INC",
    "8C1F6448F": "Mecos AG",
    "8C1F64AD8": "Novanta IMS",
    "8C1F64362": "Power Electronics Espana, S.L.",
    "8C1F64D93": "Algodue Elettronica Srl",
    "8C1F6474E": "OpenPark Technologies Kft",
    "8C1F64A36": "DONGGUAN GAGO ELECTRONICS CO.,LTD",
    "8C1F640B6": "Luke Granger-Brown",
    "8C1F64D9B": "GiS mbH",
    "8C1F64DFC": "Meiko Electronics Co.,Ltd.",
    "8C1F64620": "Solace Systems Inc.",
    "8C1F643A2": "Kron Medidores",
    "8C1F64E74": "Magosys Systems LTD",
    "70B3D5CB1": "RADAR",
    "8C1F64A34": "Potter Electric Signal Co. LLC",
    "8C1F641D6": "ZHEJIANG QIAN INFORMATION & TECHNOLOGIES",
    "8C1F64093": "MAG Audio LLC",
    "70B3D56F4": "WDI Wise Device Inc.",
    "8C1F648E8": "Cominfo, Inc.",
    "8C1F64286": "i2s",
    "70B3D52D7": "Private",
    "8C1F6403C": "Sona Business B.V.",
    "8C1F649E6": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64BDB": "Cardinal Scales Manufacturing Co",
    "8C1F64025": "SMITEC S.p.A.",
    "8C1F64CC2": "TOYOGIKEN CO.,LTD.",
    "8C1F64408": "techone system",
    "8C1F6459A": "Primalucelab isrl",
    "8C1F64EE1": "PuS GmbH und Co. KG",
    "8C1F64F24": "Albotronic",
    "8C1F64793": "Aditec GmbH",
    "8C1F64A31": "Zing Communications Inc",
    "8C1F642CD": "Taiwan Vtron",
    "70B3D523E": "Tornado Modular Systems",
    "70B3D5A87": "Tornado Modular Systems",
    "8C1F64803": "MOSCA Elektronik und Antriebstechnik GmbH",
    "70B3D5A33": "TIAMA",
    "70B3D5798": "TIAMA",
    "8C1F64756": "Star Systems International Limited",
    "8C1F6466D": "VT100 SRL",
    "8C1F6452E": "CLOUD TELECOM Inc.",
    "8C1F64293": "Landis+Gyr Equipamentos de Medi\u00e7\u00e3o Ltda",
    "8C1F64A0D": "Lumiplan Duhamel",
    "8C1F64154": "Flextronics International Kft",
    "8C1F645E3": "Nixer Ltd",
    "8C1F6477B": "DB SAS",
    "8C1F640F4": "AW-SOM Technologies LLC",
    "8C1F648E3": "UniTik Technology Co., Limited",
    "8C1F6436A": "INVENTIS S.r.l.",
    "8C1F64AFF": "Qtechnology A/S",
    "8C1F64F84": "KST technology",
    "8C1F64BF1": "Soha Jin",
    "8C1F64895": "Dacom West GmbH",
    "8C1F64593": "Brillian Network & Automation Integrated System Co., Ltd.",
    "8C1F64DA6": "Power Electronics Espana, S.L.",
    "8C1F6437F": "Scarlet Tech Co., Ltd.",
    "8C1F64251": "Watchdog Systems",
    "8C1F64B14": "Murata Manufacturing CO., Ltd.",
    "8C1F64744": "CHASEO CONNECTOME",
    "8C1F646D0": "ABB",
    "8C1F6478F": "Connection Systems",
    "8C1F64CFA": "YUYAMA MFG Co.,Ltd",
    "8C1F641D8": "Mesomat inc.",
    "8C1F649A5": "Xi\u2018an Shengxin Science& Technology Development Co.?Ltd.",
    "8C1F649E8": "GHM Messtechnik GmbH",
    "8C1F645D6": "Portrait Displays, Inc.",
    "8C1F6490D": "Algodue Elettronica Srl",
    "70B3D56CF": "Private",
    "8C1F64F2F": "Quantum Technologies Inc",
    "8C1F649B2": "Emerson Rosemount Analytical",
    "8C1F64BC3": "FoxIoT O\u00dc",
    "8C1F64B08": "Cronus Electronics",
    "8C1F642A1": "Pantherun Technologies Pvt Ltd",
    "8C1F6400C": "Guan Show Technologe Co., Ltd.",
    "8C1F64462": "REO AG",
    "8C1F649A4": "LabLogic Systems",
    "8C1F64376": "DIAS Infrared GmbH",
    "8C1F6450E": "Panoramic Power",
    "8C1F64676": "sdt.net AG",
    "8C1F64117": "Grossenbacher Systeme AG",
    "8C1F640D5": "RealD, Inc.",
    "8C1F642FE": "VERSITRON, Inc.",
    "8C1F644F9": "Photonic Science and Engineering Ltd",
    "8C1F6497D": "KSE GmbH",
    "8C1F64B9E": "Power Electronics Espana, S.L.",
    "8C1F6417E": "MI Inc.",
    "8C1F649B6": "GS Elektromedizinsiche Ger\u00e4te G. Stemple GmbH",
    "8C1F64556": "BAE Systems",
    "8C1F6483E": "Sicon srl",
    "8C1F644E7": "Circuit Solutions",
    "8C1F6444E": "GVA Lighting, Inc.",
    "8C1F647D9": "Noisewave Corporation",
    "70B3D541A": "HYOSUNG Heavy Industries Corporation",
    "8C1F64C07": "HYOSUNG Heavy Industries Corporation",
    "8C1F64208": "Sichuan AnSphere Technology Co. Ltd.",
    "8C1F64324": "Kinetic Technologies",
    "8C1F6491A": "Profcon AB",
    "70B3D511A": "Mahindra Electric Mobility Limited",
    "8C1F64A51": "BABTEL",
    "8C1F64365": "VECTOR TECHNOLOGIES, LLC",
    "8C1F64CF7": "BusPas",
    "8C1F64179": "Agrowtek Inc.",
    "8C1F64F43": "wtec GmbH",
    "8C1F64692": "Nexilis Electronics India Pvt Ltd (PICSYS)",
    "8C1F642C5": "SYSN",
    "8C1F649F4": "Grossenbacher Systeme AG",
    "8C1F649FF": "Satelles Inc",
    "8C1F644AE": "KCS Co., Ltd.",
    "8C1F64DB7": "Lambda Systems Inc.",
    "8C1F64E90": "MHE Electronics",
    "8C1F64949": "tickIoT Inc.",
    "8C1F641EF": "Tantronic AG",
    "8C1F64F2C": "Tunstall A/S",
    "8C1F64A42": "Rodgers Instruments US LLC",
    "8C1F64958": "Sanchar Telesystems limited",
    "8C1F64316": "Potter Electric Signal Company",
    "8C1F648CF": "Diffraction Limited",
    "8C1F640B0": "Bunka Shutter Co., Ltd.",
    "8C1F64C57": "Strategic Robotic Systems",
    "8C1F64274": "INVIXIUM ACCESS INC",
    "8C1F644D6": "Dan Smith LLC",
    "8C1F64622": "Logical Product",
    "8C1F64B3B": "Sicon srl",
    "8C1F6453B": "REFU Storage System GmbH",
    "8C1F64883": "DEUTA-WERKE GmbH",
    "8C1F64F45": "JBF",
    "8C1F64765": "Micro Electroninc Products",
    "8C1F644E5": "Renukas Castle Hard- and Software",
    "8C1F64F27": "Tesat-Spacecom GmbH & Co. KG",
    "8C1F64059": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64967": "DAVE SRL",
    "8C1F64703": "Calnex Solutions plc",
    "8C1F64775": "Becton Dickinson",
    "8C1F64BFB": "TechArgos",
    "8C1F64C68": "FIBERME COMMUNICATIONS LLC",
    "8C1F64660": "LLC NTPC",
    "8C1F64855": "e.kundenservice Netz GmbH",
    "8C1F64EB5": "Meiryo Denshi Corp.",
    "8C1F6440E": "Baker Hughes EMEA",
    "8C1F64FF6": "Ascon Tecnologic S.r.l.",
    "8C1F6459F": "Delta Computers LLC.",
    "8C1F647D3": "Suntech Engineering",
    "8C1F64FBA": "Onto Innovation",
    "8C1F64BF0": "Newtec A/S",
    "8C1F64A5D": "Shenzhen zhushida Technology lnformation Co.,Ltd",
    "8C1F642B6": "Stercom Power Solutions GmbH",
    "8C1F64B7B": "Gateview Technologies",
    "8C1F641AF": "EnviroNode IoT Solutions",
    "8C1F64C97": "Magnet-Physik Dr. Steingroever GmbH",
    "8C1F64099": "Pantherun Technologies Pvt Ltd",
    "8C1F64F74": "GE AVIC Civil Avionics Systems Company Limited",
    "8C1F6488D": "Pantherun Technologies Pvt Ltd",
    "8C1F64FD3": "SMILICS TECHNOLOGIES, S.L.",
    "8C1F64E99": "Pantherun Technologies Pvt Ltd",
    "8C1F64600": "Anhui Chaokun Testing Equipment Co., Ltd",
    "8C1F64103": "KRONOTECH SRL",
    "8C1F64E49": "Samwell International Inc",
    "8C1F64517": "Smart Radar System, Inc",
    "70B3D5FDF": "NARA CONTROLS INC.",
    "8C1F64656": "Optotune Switzerland AG",
    "8C1F64111": "ISAC SRL",
    "8C1F64F25": "Misaka Network, Inc.",
    "8C1F64647": "Senior Group LLC",
    "8C1F647B9": "Deviceroy",
    "8C1F6416E": "Benchmark Electronics BV",
    "8C1F64BF4": "Fluid Components Intl",
    "8C1F6419B": "FeedFlo",
    "8C1F64F31": "International Water Treatment Maritime AS",
    "8C1F6431A": "Asiga Pty Ltd",
    "8C1F6484E": "West Pharmaceutical Services, Inc.",
    "8C1F64EB2": "Aqua Broadcast Ltd",
    "70B3D5226": "Yaviar LLC",
    "8C1F64FE3": "Power Electronics Espana, S.L.",
    "8C1F6441D": "Aspen Spectra Sdn Bhd",
    "8C1F649C3": "Camozzi Automation SpA",
    "8C1F64E41": "Grossenbacher Systeme AG",
    "8C1F642C2": "TEX COMPUTER SRL ",
    "8C1F64BD7": "Union Electronic.",
    "8C1F64509": "Season Electronics Ltd",
    "8C1F64F96": "SACO Controls Inc.",
    "8C1F64F5A": "Telco Antennas Pty Ltd",
    "8C1F64177": "Emcom Systems",
    "8C1F64B77": "Carestream Dental LLC",
    "8C1F64CEE": "DISPLAX S.A.",
    "8C1F64A38": "NuGrid Power",
    "8C1F6457A": "NPO ECO-INTECH Ltd.",
    "8C1F6408B": "Shanghai Shenxu Technology Co., Ltd",
    "8C1F644C1": "Clock-O-Matic",
    "8C1F64003": "Brighten Controls LLP",
    "8C1F64A76": "DEUTA-WERKE GmbH",
    "8C1F64E52": "LcmVeloci ApS",
    "8C1F64E77": "GY-FX SAS",
    "8C1F64DB9": "Ermes Elettronica s.r.l.",
    "8C1F64CEF": "Goertek Robotics Co.,Ltd.",
    "8C1F648AE": "Shenzhen Qunfang Technology Co., LTD.",
    "8C1F644CD": "Guan Show Technologe Co., Ltd.",
    "8C1F64782": "ATM LLC",
    "70B3D5E76": "Dorsett Technologies Inc",
    "8C1F6406D": "Monnit Corporation",
    "8C1F6477C": "Orange Tree Technologies Ltd",
    "8C1F6415E": "Dynomotion, Inc",
    "8C1F64C40": "Sciospec Scientific Instruments GmbH",
    "8C1F64F3F": "Industrial Laser Machines, LLC",
    "8C1F640AB": "Norbit ODM AS",
    "8C1F648A9": "Guan Show Technologe Co., Ltd.",
    "8C1F64CAD": "General Motors",
    "8C1F64C27": "Lift Ventures, Inc",
    "8C1F64EAA": "\"KB \"Modul\", LLC",
    "8C1F647A1": "Guardian Controls International Ltd",
    "8C1F640C5": "TechnipFMC",
    "8C1F647D6": "Algodue Elettronica Srl",
    "8C1F64330": "Vision Systems Safety Tech",
    "8C1F643AD": "TowerIQ",
    "8C1F64EC1": "Actronika SAS",
    "8C1F64382": "Shenzhen ROLSTONE Technology Co., Ltd",
    "8C1F641B6": "Red Sensors Limited",
    "70B3D56BB": "LUCEO",
    "8C1F64397": "Intel Corporate",
    "8C1F64493": "Security Products International, LLC",
    "8C1F64CE3": "Pixel Design & Manufacturing Sdn. Bhd.",
    "8C1F64768": "mapna group",
    "8C1F64ACE": "Rayhaan Networks",
    "8C1F6472C": "Antai technology Co.,Ltd",
    "8C1F64984": "Abacus Peripherals Pvt Ltd",
    "8C1F64128": "YULISTA INTEGRATED SOLUTION",
    "8C1F64193": "Sicon srl",
    "8C1F64C1F": "Esys Srl",
    "8C1F6428A": "Arcopie",
    "8C1F64A5C": "Prosys",
    "8C1F64572": "ZMBIZI APP LLC",
    "8C1F64878": "Green Access Ltd",
    "8C1F64B9A": "QUERCUS TECHNOLOGIES, S.L.",
    "8C1F64611": "Siemens Industry Software Inc.",
    "8C1F64F41": "AUTOMATIZACION Y CONECTIVIDAD SA DE CV",
    "8C1F64726": "DAVE SRL",
    "8C1F6442B": "Gamber Johnson-LLC",
    "8C1F647F1": "AEM Singapore Pte Ltd",
    "8C1F6421C": "LLC \"EMS-Expert\"",
    "8C1F64460": "Solace Systems Inc.",
    "70B3D5E27": "Woodside Electronics",
    "70B3D5E41": "4neXt S.r.l.s.",
    "70B3D5D0B": "Vendanor AS",
    "70B3D5509": "Tinkerforge GmbH",
    "70B3D5036": "Vema Venturi AB",
    "70B3D5685": "LDA Audiotech",
    "70B3D5A14": "aelettronica group srl",
    "70B3D5698": "ZIEHL-ABEGG SE",
    "70B3D53B6": "MedRx, Inc",
    "70B3D5C19": "Zumbach Electronic AG",
    "70B3D5298": "Reflexion Medical",
    "70B3D52C6": "AM General Contractor",
    "70B3D5BA0": "Season Electronics Ltd",
    "70B3D5D13": "IRT Technologies",
    "70B3D54E6": "Santa Barbara Imaging Systems",
    "70B3D5EC0": "ProtoConvert Pty Ltd",
    "70B3D5D45": "Vemco Sp. z o. o.",
    "70B3D59FF": "Network Integrity Systems",
    "70B3D5137": "Subject Link Inc",
    "70B3D51FB": "Crane-elec. Co., LTD.",
    "70B3D58D6": "Beijing Xiansheng Technology Co., Ltd",
    "70B3D5D6D": "ACD Elekronik GmbH",
    "70B3D5736": "Jabil",
    "70B3D533A": "AudioTEC LLC",
    "70B3D520B": "KST technology",
    "70B3D57E6": "11811347 CANADA Inc.",
    "70B3D5D21": "biosilver .co.,ltd",
    "70B3D5E05": "Lobaro GmbH",
    "70B3D5B0A": "Mitsubishi Electric India Pvt. Ltd.",
    "70B3D516D": "BluB0X Security, Inc.",
    "70B3D559F": "Megger Germany GmbH",
    "70B3D5CE0": "M.S. CONTROL",
    "70B3D5DDA": "Hubbell Power Systems",
    "70B3D56D5": "Potter Electric Signal Co. LLC",
    "70B3D570D": "OMNISENSING PHOTONICS LLC",
    "70B3D5A9F": "Master Meter Inc.",
    "70B3D5746": "\"Smart Systems\" LLC",
    "70B3D5265": "Rapiot",
    "70B3D5B70": "Torion Plasma Corporation",
    "70B3D5DA0": "Jiangsu Etern Compamy Limited",
    "70B3D5212": "Semiconsoft, inc",
    "70B3D563D": "Topic Embedded Products B.V.",
    "70B3D51AE": "EcoG",
    "70B3D5F26": "XJ ELECTRIC CO., LTD.",
    "70B3D519D": "Automata GmbH & Co. KG",
    "70B3D50FD": "JSC \"Ural Factories\"",
    "70B3D5586": "Aliter Technologies",
    "70B3D5D04": "Plenty Unlimited Inc",
    "70B3D566E": "SIAME",
    "70B3D5EE0": "Stecomp",
    "70B3D55BD": "nexgenwave",
    "70B3D5919": "Thesycon Software Solutions GmbH & Co. KG",
    "70B3D543A": "ARKS Enterprises, Inc.",
    "70B3D51C6": "Bita-International Co., Ltd.",
    "70B3D5F49": "ZMBIZI APP LLC",
    "70B3D5B92": "N A Communications LLC",
    "70B3D5BD7": "TT Group SRL",
    "70B3D5A11": "TRIOPTICS",
    "70B3D51FA": "EBZ SysTec GmbH",
    "70B3D59EE": "Lockheed Martin - THAAD",
    "70B3D5CD8": "Nexus Electric S.A.",
    "70B3D5491": "VONSCH",
    "70B3D5B4B": "Network Customizing Technologies Inc",
    "70B3D5399": "SPE Smartico, LLC",
    "70B3D5316": "Austco Marketing & Service (USA) ltd.",
    "70B3D539F": "CT Company",
    "70B3D51B7": "ULSee Inc",
    "70B3D5560": "DaiShin Information & Communications Co., Ltd",
    "70B3D5256": "Telco Antennas Pty Ltd",
    "70B3D5D3E": "enders GmbH",
    "70B3D551F": "VALEO CDA",
    "70B3D571D": "Connido Limited",
    "70B3D544F": "Velvac Incorporated",
    "70B3D5BDC": "EDF Lab",
    "70B3D5627": "EarTex",
    "70B3D5BF4": "CreevX",
    "70B3D5453": "Foerster-Technik GmbH",
    "70B3D5D85": "BTG Instruments AB",
    "70B3D58B6": "Eldes Ltd",
    "70B3D57BD": "TableConnect GmbH",
    "70B3D523D": "Circle Consult ApS",
    "70B3D5663": "Intrinsic Group Limited",
    "70B3D5D5E": "Barcelona Smart Technologies",
    "70B3D57EC": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "70B3D5423": "Harman Connected Services Corporation India Pvt. Ltd.",
    "70B3D5E1D": "Galaxy Next Generation, Inc.",
    "70B3D5E9F": "Gigaband IP LLC",
    "70B3D593D": "Elmeasure India Pvt Ltd",
    "70B3D53A2": "Daifuku CO., Ltd.",
    "70B3D59A6": "QUNU LABS PRIVATE LIMITED",
    "70B3D57B1": "Panamera",
    "70B3D5552": "ALTIT.CO.,Ltd.",
    "70B3D599D": "Opsys-Tech",
    "70B3D593F": "Vision Sensing Co., Ltd.",
    "70B3D5F7F": "ABL Space Systems",
    "70B3D5F41": "DUEVI SRL",
    "70B3D5F32": "Elektronik Art",
    "70B3D5485": "CLARESYS LIMITED",
    "70B3D5A1A": "Nueon - The COR",
    "70B3D53FD": "NaraControls Inc",
    "70B3D5856": "Shanghai Westwell Information and Technology Company Ltd",
    "70B3D582B": "Shangnuo company",
    "70B3D5242": "Comeo Technology Co.,Ltd",
    "70B3D53B9": "BirdDog Australia",
    "70B3D52AF": "Enlaps",
    "70B3D5756": "TimeMachines Inc.",
    "70B3D5626": "KRONOTECH SRL",
    "70B3D580C": "Algra tec AG",
    "70B3D5C0D": "Clarity Medical Pvt Ltd",
    "70B3D5864": "BORMANN EDV und Zubehoer",
    "70B3D500F": "Neusoft Reach Automotive Technology (Shenyang) Co.,Ltd",
    "70B3D5C48": "Weltek Technologies Co. Ltd.",
    "70B3D5D06": "YUYAMA MFG Co.,Ltd",
    "70B3D5E46": "7thSense Design Limited",
    "70B3D56F5": "Cominfo, Inc.",
    "70B3D58D5": "Guangzhou Wanglu ",
    "70B3D569D": "JPEmbedded Mazan Filipek Sp. J.",
    "70B3D5051": "JT",
    "70B3D50EB": "Tomahawk Robotics",
    "70B3D5005": "CT Company",
    "70B3D56EB": "QUANTAFLOW",
    "70B3D5632": "Power Electronics Espana, S.L.",
    "70B3D5484": "Hermann Sewerin GmbH",
    "70B3D5629": "OZRAY",
    "70B3D5D99": "Nilar AB",
    "70B3D58DD": "Vertex Co.,Ltd.",
    "70B3D5F33": "Beijing Vizum Technology Co.,Ltd.",
    "70B3D5393": "Monnit Corporation",
    "70B3D5FD9": "eSight",
    "70B3D5687": "Volution Group UK",
    "70B3D5191": "Algodue Elettronica Srl",
    "70B3D540D": "Grupo Epelsa S.L.",
    "70B3D5F46": "Season Electronics Ltd",
    "70B3D59CD": "WEPTECH elektronik GmbH",
    "70B3D59A8": "Egag, LLC",
    "70B3D50BB": "AnaPico AG",
    "70B3D515A": "ENABLER LTD.",
    "70B3D562E": "LINEAGE POWER PVT LTD.,",
    "70B3D5872": "Nippon Safety co,ltd",
    "70B3D5898": "Salupo Sas",
    "70B3D553E": "Asiga Pty Ltd",
    "70B3D5944": "Chromateq",
    "70B3D51A7": "Elk Solutions, LLC",
    "70B3D54F2": "COMPAL ELECTRONICS, INC.",
    "70B3D5F3D": "KAYA Instruments",
    "70B3D53E0": "Gogo Business Aviation",
    "70B3D5E31": "NEUROPHET, Inc.",
    "70B3D5C99": "Remote Diagnostic Technologies Ltd",
    "70B3D54F3": "XPS ELETRONICA LTDA",
    "70B3D5331": "Firecom, Inc.",
    "70B3D562A": "DOGA",
    "70B3D51CF": "Dalcnet srl",
    "70B3D5980": "Beijing Yourong Runda Rechnology Development Co.Ltd.",
    "70B3D5EBF": "AUTOMATICA Y REGULACION S.A.",
    "70B3D5B96": "Oculii",
    "70B3D5D9F": "\"Digital Solutions\" JSC",
    "70B3D552F": "R.C. Systems Inc",
    "70B3D55AF": "JENG IoT BV",
    "70B3D5F47": "TXMission Ltd.",
    "70B3D5E72": "KDT Corp.",
    "70B3D5C52": "sensorway",
    "70B3D58A7": "Tucsen Photonics Co., Ltd. ",
    "70B3D5B1B": "Technology Link Corporation",
    "70B3D5C9A": "Todd Digital Limited",
    "70B3D5394": "Romteck Australia",
    "70B3D5642": "MB connect line GmbH Fernwartungssysteme",
    "70B3D592D": "Suzhou Wansong Electric Co.,Ltd",
    "70B3D5EF1": "Nanotok LLC",
    "70B3D51D6": "MacGray Services",
    "70B3D5201": "Leontech Limited",
    "70B3D5E0A": "Acouva, Inc.",
    "70B3D5C10": "Scanvaegt Systems A/S",
    "70B3D586F": "LLC \"NTC ACTOR\"",
    "70B3D5F91": "Solid State Disks Ltd",
    "70B3D5D30": "Leica Microsystems Ltd. Shanghai",
    "70B3D5E97": "Toptech Systems, Inc.",
    "70B3D5B95": "EPIImaging",
    "70B3D5565": "Clecell",
    "70B3D556E": "Power Electronics Espana, S.L.",
    "70B3D5082": "Sakura Seiki Co.,Ltd.",
    "70B3D5E6F": "Amazon Technologies Inc.",
    "70B3D59B8": "Loma Systems s.r.o.",
    "70B3D540F": "NEXELEC",
    "70B3D5221": "LX Design House",
    "70B3D568B": "Sadel S.p.A.",
    "70B3D5971": "RCH ITALIA SPA ",
    "70B3D5E10": "Leidos",
    "70B3D5C50": "Combilent",
    "70B3D5CA7": "i-View Communication Inc.",
    "70B3D5270": "Amazon Technologies Inc.",
    "70B3D5914": "Contec Americas Inc.",
    "70B3D59CF": "IOTIZE",
    "70B3D595F": "WiFi Nation Ltd",
    "70B3D5569": "Nuance Hearing Ltd.",
    "70B3D589C": "IHI Rotating Machinery Engineering Co.,Ltd.",
    "70B3D59AC": "Suzhou Sapa Automotive Technology Co.,Ltd",
    "70B3D5D3D": "Netzikon GmbH",
    "70B3D5C9C": "Connected Response",
    "70B3D5113": "iREA System Industry",
    "70B3D50DD": "Shenzhen Virtual Clusters Information Technology Co.,Ltd.",
    "70B3D581D": "DEUTA-WERKE GmbH",
    "70B3D53D0": "ORtek Technology, Inc.",
    "70B3D5043": "cal4care Pte Ltd",
    "70B3D5329": "Primalucelab isrl",
    "70B3D5454": "Golding Audio Ltd",
    "70B3D5537": "Biennebi s.r.l.",
    "70B3D5EC5": "TATTILE SRL",
    "70B3D5F2F": "TELEPLATFORMS",
    "70B3D5577": "DSILOG",
    "70B3D5719": "2M Technology",
    "70B3D5D33": "VECTOR.CO.,LTD.",
    "70B3D56A4": "Acrodea, Inc.",
    "70B3D5F3E": "\u041e\u041e\u041e \"\u0420\u041e\u041d\u0415\u041a\u0421\"",
    "70B3D562F": "BARCO, s.r.o.",
    "70B3D59FC": "Truecom Telesoft Private Limited",
    "70B3D5356": "BRS Sistemas Eletr\u00f4nicos",
    "70B3D5DCD": "C TECH BILISIM TEKNOLOJILERI SAN. VE TIC. A.S.",
    "70B3D503A": "Ochno AB",
    "70B3D5155": "Sanwa New Tec Co.,Ltd",
    "70B3D5A0F": "OSAKI DATATECH CO., LTD.",
    "70B3D556F": "Radikal d.o.o.",
    "70B3D5667": "CT Company",
    "70B3D59B0": "Clearly IP Inc",
    "70B3D5A68": "Zhejiang Zhaolong Interconnect Technology Co.,Ltd",
    "70B3D5D61": "VITEC",
    "70B3D5A34": "RCH ITALIA SPA ",
    "70B3D559A": "Wagner Group GmbH",
    "70B3D5C7A": "ENTEC Electric & Electronic Co., LTD.",
    "70B3D5FCE": "FX TECHNOLOGY LIMITED",
    "70B3D5588": "LLC NPO Svyazkomplektservis",
    "70B3D5171": "Aetina Corporation",
    "70B3D51FC": "Guan Show Technologe Co., Ltd.",
    "70B3D534D": "Equos Research Co., Ltd",
    "70B3D5420": "ECOINET",
    "70B3D5CD4": "Southern Ground Audio LLC",
    "70B3D53BD": "DAO QIN TECHNOLOGY CO.LTD.",
    "70B3D52C7": "Worldsensing",
    "70B3D5258": "BAYKON End\u00fcstriyel Kontrol Sistemleri San. ve Tic. A.\u015e.",
    "70B3D5B2F": "Hermann Automation GmbH",
    "70B3D513A": "DEUTA-WERKE GmbH",
    "70B3D5ABD": "wtec GmbH",
    "70B3D5CF8": "Idneo Technologies S.A.U.",
    "70B3D557E": "Ascon Tecnologic S.r.l.",
    "70B3D55E1": "Arevita",
    "70B3D51F9": "Automata GmbH & Co. KG",
    "70B3D5151": "Virsae Group Ltd",
    "70B3D5A2B": "Clever Devices",
    "70B3D550A": "AMEDTEC Medizintechnik Aue GmbH",
    "70B3D5FD5": "OCEANCCTV LTD",
    "70B3D59DA": "Blake UK",
    "70B3D5822": "Angora Networks",
    "70B3D5056": "MIRAE INFORMATION TECHNOLOGY CO., LTD.",
    "70B3D5BEE": "Sicon srl",
    "70B3D5754": "COSMOIT.CO.LTD",
    "70B3D5237": "Sikom AS",
    "70B3D533F": "XANTIA SA",
    "70B3D521A": "Acutronic Link Robotics AG",
    "70B3D5490": "Xiamen Beogold Technology Co. Ltd.",
    "70B3D59CC": "Zaxcom Inc",
    "70B3D5E29": "Invent Vision - iVision Sistemas de Imagem e Vis\u00e3o S.A.",
    "70B3D5F89": "Soehnle Industrial Solutions GmbH",
    "70B3D5E15": "Benetel",
    "70B3D566D": "Sanmina Israel",
    "70B3D5C93": "GMI Ltd",
    "70B3D51DF": "ENTEC Electric & Electronic Co., LTD.",
    "70B3D55D4": "RCH ITALIA SPA ",
    "70B3D5F3F": "comtac AG",
    "70B3D5E33": "DEUTA-WERKE GmbH",
    "70B3D5497": "ALBIRAL DISPLAY SOLUTIONS SL",
    "70B3D598A": "vision systems safety tech ",
    "70B3D59A9": "PABLO AIR Co., LTD",
    "70B3D50D0": "ProHound Controles Eirelli ",
    "70B3D570E": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D5574": "Cloud Intelligence Pty Ltd",
    "70B3D5330": "iOne",
    "70B3D5BBA": "Samriddi Automations Pvt. Ltd.",
    "70B3D5D1A": "Monnit Corporation",
    "70B3D5287": "Hypex Electronics BV",
    "70B3D5545": "Airity Technologies Inc.",
    "70B3D5C24": "Elbit Systems of America",
    "70B3D5DE5": "ASML",
    "70B3D549B": "Algodue Elettronica Srl",
    "70B3D5E40": "Siemens Mobility GmbH - MO TI SPA",
    "70B3D5FCB": "Tieline Research Pty Ltd",
    "70B3D59A2": "O-Net Communications(Shenzhen)Limited",
    "70B3D5752": "Guan Show Technologe Co., Ltd.",
    "70B3D5F71": "Sonel S.A.",
    "70B3D5E78": "Camwell India LLP",
    "70B3D5949": "National Radio & Telecommunication Corporation - NRTC",
    "70B3D57AF": "Hessware GmbH",
    "70B3D5836": "Authenticdata",
    "70B3D58AE": "FARECO",
    "70B3D54A2": "DEVAU Lemppenau GmbH",
    "70B3D536E": "Electr\u00f3nica Falc\u00f3n S.A.U",
    "70B3D507C": "ISAC SRL",
    "70B3D59C5": "LINEAGE POWER PVT LTD.,",
    "70B3D5926": "Advice",
    "70B3D50E9": "VNT electronics s.r.o.",
    "70B3D5EE8": "robert juliat",
    "70B3D508C": "Airmar Technology Corp",
    "70B3D596E": "Myostat Motion Control Inc",
    "70B3D501B": "AUDI AG",
    "70B3D5006": "Piranha EMS Inc.",
    "70B3D5CBA": "YUYAMA MFG Co.,Ltd",
    "70B3D5A44": "FSR, INC.",
    "70B3D53A0": "chiconypower",
    "70B3D5F18": "HD Vision Systems GmbH",
    "70B3D5E47": "DEUTA-WERKE GmbH",
    "70B3D52ED": "Signals and systems india pvt ltd",
    "70B3D5D28": "Toshiba Electron Tubes & Devices Co., Ltd.",
    "70B3D5AA9": "Datamars SA",
    "70B3D54D2": "Biotage Sweden AB",
    "70B3D5008": "ESYSE GmbH Embedded Systems Engineering",
    "70B3D527C": "MOTION LIB,Inc.",
    "70B3D54E0": "Microvideo",
    "70B3D5CA1": "Waldo System",
    "70B3D5ACF": "APG Cash Drawer, LLC",
    "70B3D5793": "Gastech Australia Pty Ltd",
    "70B3D5B6B": "Cambria Corporation",
    "70B3D5BC3": "eWireless",
    "70B3D59F9": "Fluid Components Intl",
    "70B3D5EBA": "Last Mile Gear",
    "70B3D5334": "Dokuen Co. Ltd.",
    "70B3D5ED7": "WAVE",
    "70B3D5BD8": "MB connect line GmbH Fernwartungssysteme",
    "70B3D53D4": "Sanmina Israel",
    "70B3D5937": "TATTILE SRL",
    "70B3D5DD5": "Cooltera Limited",
    "70B3D5868": "U-JIN Mesco Co., Ltd.",
    "70B3D515E": "Season Electronics Ltd",
    "70B3D57C7": "Sicon srl",
    "70B3D50F7": "Bespoon",
    "70B3D511B": "HoseoTelnet Inc...",
    "70B3D5BC1": "Abionic",
    "70B3D561E": "PKE Electronics AG",
    "70B3D5ACB": "TATTILE SRL",
    "70B3D5471": "SYSCO Sicherheitssysteme GmbH",
    "70B3D5E2B": "Guan Show Technologe Co., Ltd.",
    "70B3D5950": "CMT Medical technologies",
    "70B3D5F4D": "Honeywell",
    "70B3D5B94": "Cygnetic Technologies (Pty) Ltd",
    "70B3D5B1F": "TECNOWATT",
    "70B3D5948": "VISION SYSTEMS AURTOMOTIVE (SAFETY TECH)",
    "70B3D50A8": "Symetrics Industries d.b.a. Extant Aerospace",
    "70B3D584C": "CoreKinect",
    "70B3D5FC8": "Moduware PTY LTD",
    "70B3D5A9E": "Argon ST",
    "70B3D52CF": "MB connect line GmbH Fernwartungssysteme",
    "70B3D58A5": "KST technology",
    "70B3D5862": "TripleOre",
    "70B3D5406": "Acrodea, Inc.",
    "70B3D59E3": "LG Electronics",
    "70B3D567D": "Acrodea, Inc.",
    "70B3D592E": "Medical Monitoring Center OOD",
    "70B3D5460": "Guilin Tryin Technology Co.,Ltd",
    "70B3D5993": "ioThings",
    "70B3D5AB3": "MICAS AG",
    "70B3D5D24": "Microtronics Engineering GmbH",
    "70B3D50D1": "Common Sense Monitoring Solutions Ltd.",
    "70B3D500A": "FUJICOM Co.,Ltd.",
    "70B3D5984": "Sanmina Israel",
    "70B3D55ED": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D5103": "HANYOUNG NUX CO.,LTD",
    "70B3D5CA3": "Saankhya Labs Private Limited",
    "70B3D5085": "Human Systems Integration",
    "70B3D517B": "Vistec Electron Beam GmbH",
    "70B3D5375": "Adel System srl",
    "70B3D575F": "Vocality international T/A Cubic",
    "70B3D51D3": "AIROBOT O\u00dc",
    "70B3D5C11": "Ariston Thermo s.p.a.",
    "70B3D58B4": "Scenario Automation",
    "70B3D5F06": "WARECUBE,INC",
    "70B3D57AC": "Verity Studios AG",
    "70B3D53AD": "CT Company",
    "70B3D5D09": "Rishaad Brown",
    "70B3D5E0B": "ENTEC Electric & Electronic Co., LTD.",
    "70B3D53BA": "Silex Inside",
    "70B3D52C9": "SEASON DESIGN TECHNOLOGY",
    "70B3D52CA": "TATTILE SRL",
    "001BC5074": "Dynasthetics",
    "70B3D529C": "Teko Telecom Srl",
    "70B3D5A90": "ERA a.s.",
    "70B3D5D6F": "X-SPEX GmbH",
    "70B3D57D7": "Gedomo GmbH",
    "70B3D5145": "Sicon srl",
    "70B3D5981": "Zamir Recognition Systems Ltd.",
    "70B3D5700": "University Of Groningen",
    "70B3D58D0": "Raft Technologies",
    "70B3D57AD": "Insitu, Inc",
    "70B3D57C0": "TORGOVYY DOM  TEHNOLOGIY LLC",
    "70B3D5B3B": "Insitu, Inc",
    "70B3D5AEB": "Association Romandix",
    "70B3D5BAC": "AdInte, inc.",
    "70B3D5837": "HiDes, Inc.",
    "70B3D5F7A": "SENSO2ME ",
    "70B3D59A0": "ELDES",
    "70B3D525D": "Mimo Networks",
    "70B3D515D": "Vtron Pty Ltd",
    "70B3D5B18": "Abbas, a.s.",
    "70B3D5480": "Emergency Lighting Products Limited",
    "70B3D5F8F": "DIMASTEC GESTAO DE PONTO E ACESSO EIRELI-ME",
    "70B3D50A2": "TechSigno srl",
    "70B3D5782": "thou&tech",
    "70B3D5582": "VAGLER International Sdn Bhd",
    "70B3D5BE4": "Kunshan excellent Intelligent Technology Co., Ltd.",
    "70B3D5E1B": "Neuron GmbH",
    "70B3D5EB5": "JUSTEK INC",
    "70B3D595E": "BLOCKSI LLC",
    "70B3D5EF9": "Critical Link LLC",
    "70B3D5A4E": "Array Technologies Inc.",
    "70B3D5026": "Telstra",
    "70B3D5D54": "JL World Corporation Limited",
    "70B3D50B6": "Landis Gyr",
    "70B3D5A1F": "GlobalTest LLC",
    "70B3D5741": "HOW-E",
    "70B3D5A08": "BioBusiness",
    "70B3D5716": "Lode BV",
    "70B3D561D": "Telonic Berkeley Inc",
    "70B3D5D93": "PAMIR Inc",
    "70B3D5CAB": "NOTICE Co., Ltd.",
    "70B3D5252": "Sierra Nevada Corporation",
    "70B3D54FC": "Mettler Toledo",
    "70B3D55F3": "Rtone",
    "70B3D5D6B": "Uwinloc",
    "70B3D5608": "EIIT SA",
    "70B3D516B": "IOT Engineering",
    "70B3D53AA": "RCATSONE",
    "70B3D5B21": "TATTILE SRL",
    "70B3D54B8": "International Roll-Call Corporation",
    "70B3D5BA9": "Alma",
    "70B3D5842": "PLUTO Solution co.,ltd.",
    "70B3D569B": "HORIZON.INC",
    "001BC50B2": "SKODA ELECTRIC a.s.",
    "70B3D5031": "SHENZHEN GAONA ELECTRONIC CO.LTD",
    "70B3D5488": "Cardinal Scale Mfg Co",
    "70B3D5EEA": "Dameca a/s",
    "70B3D5E7D": "Nanjing Dandick Science&technology development co., LTD",
    "70B3D5C53": "S Labs sp. z o.o.",
    "70B3D50B2": "ndb technologies",
    "70B3D5319": "ISO/TC 22/SC 31",
    "70B3D5791": "Romteck Australia",
    "70B3D5674": "Fortress Cyber Security",
    "70B3D5A7E": "QUICCO SOUND Corporation",
    "70B3D57DF": "RDT Ltd",
    "70B3D5878": "Package Guard, Inc",
    "70B3D5DB8": "SISTEM SA",
    "70B3D57D2": "SDK Kristall",
    "70B3D5A69": "Leviathan Solutions Ltd.",
    "70B3D5446": "Santa Barbara Imaging Systems",
    "70B3D555E": "BRS Sistemas Eletr\u00f4nicos",
    "70B3D596B": "FOCAL-JMLab",
    "70B3D5408": "Comrod AS",
    "70B3D5007": "SENSONEO",
    "70B3D5DC9": "Sensoterra BV",
    "70B3D5E6C": "Fusar Technologies inc",
    "70B3D5371": "BEDEROV GmbH",
    "70B3D5C16": "Southern Innovation",
    "70B3D5FAD": "ARC Technology Solutions, LLC",
    "70B3D5AC7": "vivaMOS",
    "70B3D5DB2": "Micro Electroninc Products",
    "70B3D58C1": "Rievtech Electronic Co.,Ltd ",
    "70B3D5DEE": "CRDE",
    "70B3D5094": "Circuitlink Pty Ltd",
    "70B3D5AEF": "Baumtec GmbH",
    "70B3D528C": "Step Technica Co., Ltd.",
    "70B3D530B": "Ash Technologies",
    "70B3D5579": "Chelsea Technologies Group Ltd",
    "70B3D5B6A": "YUYAMA MFG Co.,Ltd",
    "70B3D5442": "Blair Companies",
    "70B3D5585": "Nefteavtomatika",
    "70B3D5AF6": "S.C.E. srl",
    "70B3D5861": "KST technology",
    "70B3D5A73": "MobiPromo",
    "70B3D5365": "CircuitMeter Inc.",
    "70B3D5B30": "Systol\u00e9 Hardware B.V.",
    "70B3D5997": "ProTom International",
    "70B3D5ADF": "Seraphim Optronics Ltd",
    "70B3D5911": "Equatel",
    "70B3D59D0": "RJ45 Technologies",
    "70B3D5B56": "Power Electronics Espana, S.L.",
    "70B3D5135": "DORLET SAU",
    "70B3D5672": "KLEIBER Infrared GmbH",
    "70B3D5D1C": "Specialised Imaging Limited",
    "70B3D507A": "ZAO ZEO",
    "70B3D5637": "INEO-SENSE",
    "70B3D5EC8": "PANASONIC LIFE SOLUTIONS ELEKTR?K SANAY? VE T?CARE",
    "70B3D5F05": "Motomuto Aps",
    "70B3D5378": "synchrotron SOLEIL",
    "70B3D5DFD": "Contiweb",
    "70B3D59D9": "ATX Networks Corp",
    "70B3D5778": "Lumacron Technology Ltd.",
    "70B3D504A": "Gecko Robotics Inc",
    "70B3D5FBD": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5A8B": "Giant Power Technology Biomedical Corporation",
    "70B3D5E2C": "Fourth Frontier Technologies Private Limited",
    "70B3D54C2": "hera Laborsysteme GmbH",
    "70B3D5AA6": "Proximus",
    "70B3D581A": "Joehl & Koeferli AG",
    "70B3D5E9C": "ATG UV Technology",
    "70B3D5FF7": "Cybercom AB",
    "70B3D52F8": "Tunstall A/S",
    "70B3D5523": "Tibit Communications",
    "70B3D5A07": "IoTrek Technology Private Limited",
    "70B3D5825": "TATTILE SRL",
    "70B3D5767": "FRANKLIN FRANCE",
    "70B3D5465": "ENERGISME",
    "70B3D59C0": "Schneider Displaytechnik GmbH",
    "70B3D5B97": "Canam Technology, Inc.",
    "70B3D5C4B": "ANKER-EAST",
    "70B3D56B1": "TTC TELEKOMUNIKACE, s.r.o.",
    "70B3D5816": "Smith Meter, Inc.",
    "70B3D5930": "The Institute of Mine Seismology",
    "70B3D537A": "APG Cash Drawer, LLC",
    "70B3D5AFF": "digital-spice",
    "70B3D5F07": "DUVAL MESSIEN",
    "70B3D5136": "Miguel Corporate Services Pte Ltd",
    "70B3D5C06": "XotonicsMED GmbH",
    "70B3D5EBB": "Beijing Wing ICT Technology Co., Ltd.",
    "70B3D560E": "HDANYWHERE",
    "70B3D5B53": "Revolution Retail Systems, LLC",
    "70B3D563C": "Pivothead",
    "70B3D5A85": "exceet electronics GesmbH",
    "70B3D55F4": "FDSTiming",
    "70B3D5D32": "Euklis by GSG International",
    "70B3D57E5": "Megaflex Oy",
    "70B3D59C7": "YUYAMA MFG Co.,Ltd",
    "70B3D5D9A": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D56B0": "PTYPE Co., LTD.",
    "70B3D58CB": "WELT Corporation",
    "70B3D5A20": "Design For Life Systems",
    "70B3D5FEC": "Finder SpA",
    "70B3D590A": "Hangzhou SunTown Intelligent Science & Technology Co.,Ltd.",
    "70B3D5594": "ATE Systems Inc",
    "70B3D527A": "TD ECOPHISIKA",
    "70B3D570F": "Alion Science & Technology",
    "70B3D5131": "Inova Design Solutions Ltd",
    "70B3D5989": "DCNS",
    "70B3D522F": "Instec, Inc.",
    "70B3D5227": "Montalvo",
    "70B3D5779": "DR.BRIDGE AQUATECH",
    "70B3D5498": "XGEM SAS",
    "70B3D5E82": "RF Track",
    "70B3D5855": "CRDE",
    "70B3D5CED": "Advanced Products Corporation Pte Ltd",
    "70B3D5938": "JETI Technische Instrumente GmbH",
    "70B3D54A7": "aelettronica group srl",
    "70B3D542C": "D.Marchiori Srl",
    "70B3D54C7": "SOLVERIS sp. z o.o.",
    "70B3D5F8D": "Flextronics Canafa Design Services",
    "70B3D59B1": "Aplex Technology Inc.",
    "70B3D5CA4": "Netemera Sp. z o.o.",
    "70B3D5A53": "GS Industrie-Elektronik GmbH",
    "70B3D568F": "PEEK TRAFFIC",
    "70B3D5CAA": "Bel Power Solutions GmbH",
    "70B3D5499": "Pycom Ltd",
    "70B3D5692": "HOSIN INDUSTRIAL LIMITED",
    "70B3D5584": "Sertone, a division of Opti-Knights Ltd",
    "70B3D57D5": "SICS Swedish ICT",
    "8C1F64C44": "Sypris Electronics",
    "70B3D5867": "Specialized Communications Corp.",
    "70B3D5E5E": "Critical Link LLC",
    "70B3D50C7": "PEEK TRAFFIC",
    "70B3D5F70": "Honeywell",
    "70B3D59D2": "ACS MOTION CONTROL",
    "70B3D502D": "NEXTtec srl",
    "70B3D56C5": "CJSC \u00abRussian telecom equipment company\u00bb (CJSC RTEC)",
    "70B3D5696": "Open Grow",
    "70B3D555A": "Sontay Ltd.",
    "70B3D578C": "Survalent Technology Corporation",
    "70B3D5B7D": "LOGIX ITS Inc",
    "70B3D58DB": "Kratos Analytical Ltd",
    "70B3D5167": "Eiden Co.,Ltd.",
    "70B3D5186": "Rohde&Schwarz Topex SA",
    "70B3D549F": "B.P.A. SRL",
    "70B3D5ABE": "MART NETWORK SOLUTIONS LTD",
    "70B3D588B": "WUHAN EASYLINKIN TECHNOLOGY co.,LTD",
    "70B3D55E6": "Mechatronics Systems Private Limited",
    "70B3D5AAC": "SensoTec GmbH",
    "70B3D56EC": "CRDE",
    "70B3D5B55": "CTAG - ESG36871424",
    "70B3D5986": "Aplex Technology Inc.",
    "70B3D551B": "Vitrea Smart Home Technologies ",
    "70B3D5A10": "w-tec AG",
    "70B3D5A2C": "TLV CO., LTD.",
    "70B3D5881": "TATTILE SRL",
    "70B3D504C": "mapna group",
    "70B3D5712": "APG Cash Drawer, LLC",
    "70B3D5CCE": "Proconex 2010 Inc.",
    "70B3D5643": "Marques,S.A.",
    "70B3D5A7C": "Transelektronik Messger\u00e4te GmbH",
    "70B3D539C": " GD Mission Systems",
    "70B3D591B": "Dolotron d.o.o.",
    "70B3D5DE4": "MAVILI ELEKTRONIK TIC. VE SAN. A.S.",
    "70B3D5AC3": "Novoptel GmbH",
    "70B3D535A": "Applied Radar, Inc.",
    "70B3D58F2": "Rimota Limited",
    "70B3D5879": "ZIGPOS GmbH",
    "70B3D5DE2": "ACD Elekronik GmbH",
    "70B3D5B81": "Instro Precision Limited",
    "70B3D59E0": "ES Industrial Systems Co., Ltd.",
    "70B3D5D65": "CRDE",
    "70B3D51AB": "Access Control Systems JSC",
    "70B3D56BF": "Otto Bihler Maschinenfabrik GmbH & Co. KG",
    "70B3D5A66": "Trapeze Software Group Inc",
    "70B3D50FA": "InsideRF Co., Ltd.",
    "70B3D5D34": "G-PHILOS CO.,LTD",
    "70B3D5028": "AT-Automation Technology GmbH",
    "70B3D5CF2": "tinnos",
    "70B3D5D94": "Dewetron GmbH",
    "70B3D5658": "emperor brands",
    "70B3D5B89": "IDA",
    "70B3D505A": "Uni Control System Sp. z o. o.",
    "70B3D5B67": "RedWave Labs Ltd",
    "70B3D5C32": "INFRASAFE/ ADVANTOR SYSTEMS ",
    "70B3D537B": "Power Ltd.",
    "70B3D5531": "ATEME",
    "70B3D57A0": "Reactec Ltd",
    "70B3D5A3B": "Grace Design/Lunatec LLC",
    "70B3D5FD8": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5CAC": "CRDE",
    "70B3D57E9": "Mecsel Oy",
    "70B3D5A9D": "VITEC MULTIMEDIA",
    "70B3D574E": "PushCorp, Inc.",
    "70B3D517E": "OCULI VISION",
    "70B3D5E39": "Thinnect, Inc,",
    "70B3D56DF": "Mango DSP, Inc.",
    "70B3D53F4": "Wincode Technology Co., Ltd.",
    "70B3D506C": "AppTek",
    "70B3D5618": "Motec Pty Ltd",
    "70B3D57CF": "ORCA Technologies, LLC",
    "70B3D5FB5": "Orange Tree Technologies Ltd",
    "70B3D58BE": "Connoiseur Electronics Private Limited",
    "70B3D5733": "SA Instrumentation Limited",
    "70B3D5F65": "MARKUS LABS",
    "70B3D5286": "Pedax Danmark",
    "70B3D5261": "Potter Electric Signal Co. LLC",
    "70B3D5B02": "Nordic Automation Systems AS",
    "70B3D509E": "MobiPromo",
    "70B3D51E3": "Hatel Elektronik LTD. STI.",
    "70B3D5187": "Elektronik & Pr\u00e4zisionsbau Saalfeld GmbH",
    "70B3D5942": "TruTeq Devices (Pty) Ltd",
    "70B3D50EE": "Picture Elements, Inc.",
    "70B3D5E49": "Kendrion Mechatronics Center GmbH",
    "70B3D5A92": "Grossenbacher Systeme AG",
    "70B3D5578": "IMAGE TECH CO.,LTD",
    "70B3D5D9E": "Grupo Epelsa S.L.",
    "70B3D5C7F": "TATTILE SRL",
    "70B3D5659": "E2G srl",
    "70B3D588A": "Perceptics, LLC",
    "70B3D5476": "FR-Team International SA",
    "70B3D5D60": "Flintab AB",
    "70B3D5729": "EMAC, Inc.",
    "70B3D5605": "Aplex Technology Inc.",
    "70B3D5A06": "Kopis Mobile LLC",
    "70B3D5943": "Abbott Medical Optics Inc.",
    "70B3D5BF2": "TWIN DEVELOPMENT",
    "70B3D52B3": "HAS co.,ltd.",
    "70B3D58F6": "Dofuntech Co.,LTD.",
    "70B3D51C4": "Smeg S.p.A.",
    "70B3D5D38": "Vista Research, Inc.",
    "70B3D545C": "AlyTech",
    "70B3D57E3": "RedLeaf Security",
    "70B3D52F9": "CONSOSPY",
    "70B3D511D": "Dakton Microlabs LLC",
    "70B3D539D": "Comark Interactive Solutions",
    "70B3D5A3C": "Wave Music Ltd",
    "70B3D5C77": "Y\u00f6nnet Ak\u0131ll\u0131 Bina ve Otomasyon Sistemleri ",
    "70B3D56FB": "Shachihata Inc.",
    "70B3D5A12": "QUERCUS TECHNOLOGIES, S.L.",
    "70B3D5A95": "DEUTA-WERKE GmbH",
    "70B3D55B1": "EPD Electronics",
    "70B3D5524": "Wuxi New Optical Communication Co.,Ltd.",
    "70B3D557D": "WICOM1 GmbH",
    "70B3D5B28": "HUSTY M.Styczen J.Hupert sp.j.",
    "70B3D5AF5": "Net And Print Inc.",
    "70B3D5C01": "SmartGuard LLC",
    "70B3D520A": "Golden Grid Systems",
    "70B3D57C8": "CRDE",
    "70B3D5D73": "ERMINE Corporation",
    "70B3D564A": "Netbric Technology Co.,Ltd.",
    "70B3D55FC": "SURTEC",
    "70B3D54EB": "INFOSOFT DIGITAL DESIGN & SERVICES PRIVATE LIMITED",
    "70B3D5D3F": "GLOBALCOM ENGINEERING SPA",
    "70B3D56BE": "VANTAGE INTEGRATED SECURITY SOLUTIONS PVT LTD",
    "70B3D576A": "Swiftnet SOC Ltd",
    "70B3D5D7E": "Triax A/S",
    "70B3D50C5": "Precitec Optronik GmbH",
    "70B3D5F00": "Aplex Technology Inc.",
    "70B3D5172": "LumiGrow, Inc",
    "70B3D5E7A": "ART SPA",
    "70B3D5EA4": "Grupo Epelsa S.L.",
    "70B3D5343": "Elektro-System s.c.",
    "70B3D53FF": "Hydra Controls",
    "70B3D52EE": "Aplex Technology Inc.",
    "70B3D5D57": "TRIUMPH BOARD a.s.",
    "70B3D5B91": "Dynetics, Inc.",
    "70B3D56B3": "DuraComm Corporation",
    "70B3D59E7": "Xiamen Maxincom Technologies Co., Ltd.",
    "70B3D5472": "Quadio Devices Private Limited",
    "70B3D5538": "sydetion UG (h.b.)",
    "70B3D52DC": "Bolide Technology Group, Inc.",
    "70B3D5530": "iSiS-Ex Limited",
    "70B3D5935": "Sensor Developments",
    "70B3D5C2A": "Array Telepresence",
    "70B3D5505": "MC2-Technologies",
    "70B3D535E": "EIDOS s.p.a.",
    "70B3D5F2D": "ID Lock AS",
    "70B3D5C85": "Solid State Disks Ltd",
    "70B3D5F36": "dinosys",
    "70B3D5347": "OAS Sweden AB",
    "70B3D54AD": "GACI",
    "70B3D53F9": "Herrick Tech Labs",
    "70B3D5FA2": "Sarokal Test Systems Oy",
    "70B3D5B8B": "Profound Medical Inc.",
    "70B3D5A1C": "MECA SYSTEM",
    "70B3D5342": "Solectrix",
    "70B3D5E20": "Signature Control Systems, LLC.",
    "70B3D552B": "GE Aviation Cheltenham",
    "70B3D5D80": "AMMT GmbH",
    "70B3D515F": "SAVRON\u0130K ELEKTRON\u0130K",
    "70B3D578B": "Jingtu Printing Systems Co., Ltd",
    "70B3D5A26": "Hear Gear, Inc.",
    "70B3D579B": "Soniclean Pty Ltd",
    "70B3D5A50": "LECIP CORPORATION",
    "70B3D5D9B": "Russian Telecom Equipment Company",
    "70B3D55FF": "Vaisala Oyj",
    "70B3D57A7": "Symbicon Ltd",
    "70B3D5A6E": "JSC Electrical Equipment Factory",
    "70B3D546B": "Airborne Engineering Limited",
    "70B3D5F7E": "Alpha Elettronica s.r.l.",
    "70B3D5D79": "GOMA ELETTRONICA SpA",
    "70B3D565D": "GEGA ELECTRONIQUE",
    "70B3D5E70": "DISK Multimedia s.r.o.",
    "70B3D583F": "Lumine Lighting Solutions Oy",
    "70B3D53E3": "Head",
    "70B3D5ED5": "hangzhou battle link technology Co.,Ltd",
    "70B3D5EB1": "CP contech electronic GmbH",
    "70B3D5C5A": "Commsignia Ltd.",
    "70B3D59DB": "CAS Medical Systems, Inc",
    "70B3D5FC6": "Tecnint HTE SRL",
    "70B3D50EC": "ACS MOTION CONTROL",
    "70B3D50A4": "Communication Technology Ltd.",
    "70B3D55C5": "Haag-Streit AG",
    "70B3D55E4": "DSP DESIGN",
    "70B3D5C25": "speedsignal GmbH",
    "70B3D5AFA": "Power Security Systems Ltd.",
    "70B3D5DF0": "astozi consulting Tomasz Zieba",
    "70B3D50A5": "FUELCELLPOWER",
    "70B3D56F3": "iungo",
    "70B3D50AB": "KST technology",
    "70B3D584A": "MOG Laboratories Pty Ltd",
    "70B3D5CDE": "Multipure International",
    "70B3D5104": "Plum sp. z o.o",
    "70B3D5731": "Phoniro Systems AB",
    "70B3D5501": "Peek Traffic",
    "70B3D582C": "NELS Ltd.",
    "70B3D5A62": "Environexus",
    "70B3D5A57": "PCSC",
    "70B3D56D9": "VECTARE Inc",
    "70B3D533C": "Videri Inc.",
    "70B3D58F5": "Stmovic",
    "70B3D5ABF": "AGR International",
    "70B3D5ABC": "BKM-Micronic Richtfunkanlagen GmbH",
    "70B3D503B": "SSL - Electrical Aerospace Ground Equipment Section",
    "70B3D5B3C": "DORLET SAU",
    "70B3D5F68": "AL ZAJEL MODERN TELECOMM",
    "70B3D554E": "RFL Electronics, Inc.",
    "70B3D5204": "TWC",
    "70B3D5099": "Schwer+Kopka GmbH",
    "70B3D5C6A": "Private",
    "70B3D519C": "Kubu, Inc.",
    "70B3D5B2A": "Myro Control, LLC",
    "70B3D561F": "Labotect Labor-Technik-G\u00f6ttingen GmbH",
    "70B3D5E53": "MI INC.",
    "70B3D57A9": "adidas AG",
    "70B3D5EE4": "O-Net Automation Technology (Shenzhen)Limited",
    "70B3D5029": "Marimo electronics Co.,Ltd.",
    "70B3D5C81": "DSP DESIGN",
    "70B3D5634": "idaqs Co.,Ltd.",
    "70B3D5E07": "Baader Planetarium GmbH",
    "70B3D5655": "AOT System GmbH",
    "70B3D56F6": "Acco Brands Europe",
    "70B3D5044": "Don Electronics Ltd",
    "70B3D5ECD": "SBS-Feintechnik GmbH & Co. KG",
    "70B3D5B8A": "Nexus Tech. VN",
    "70B3D5F38": "Scanvaegt Nordic A/S",
    "70B3D5C2C": "Dromont S.p.A.",
    "70B3D5077": "InAccess Networks SA",
    "70B3D54C1": "QUERCUS TECHNOLOGIES, S. L.",
    "70B3D5D4E": "FLSmidth",
    "70B3D542F": "SINTOKOGIO, LTD",
    "70B3D5702": "Sensor Highway Ltd",
    "70B3D529D": "XTech2 SIA",
    "70B3D50A9": "ProConnections, Inc.",
    "70B3D50E0": "PLCiS",
    "70B3D53ED": "Ultra Electronics Sonar System Division",
    "70B3D5D8B": "Lenoxi Automation s.r.o.",
    "70B3D535C": "ACS electronics srl",
    "70B3D5190": "Fantom Wireless, Inc.",
    "70B3D5814": "Ingenieurbuero SOMTRONIK",
    "001BC50C4": "ELDES",
    "001BC50C8": "Dialine",
    "001BC50C7": "WIZZILAB SAS",
    "001BC50B3": "FSM Solutions Limited",
    "001BC50B5": "Exibea AB",
    "001BC50B9": "Denki Kogyo Company, Limited",
    "001BC50B1": "Roslen Eco-Networking Products",
    "001BC50A0": "Silvair",
    "001BC509D": "Navitar Inc",
    "001BC5096": "Sanstreak Corp.",
    "001BC5093": "Ambient Devices, Inc.",
    "001BC5092": "Arnouse Digital Devices, Corp.",
    "001BC508F": "Unilever R&amp;D",
    "001BC508B": "Nistica",
    "001BC5085": "Oberon microsystems, Inc.",
    "001BC507C": "head",
    "001BC507D": "Greatcom AG",
    "001BC5080": "LUMINO GmbH",
    "001BC5069": "Datasat Digital Entertainment",
    "001BC5067": "Embit srl ",
    "001BC5064": "Enkora Oy Ltd",
    "001BC5061": "Scientific-Technical Center \"Epsilon\" Limited company",
    "001BC5058": "optiMEAS GmbH",
    "001BC5057": "EREE Electronique",
    "001BC5055": "LUMIPLAN TRANSPORT ",
    "001BC5049": "EUROCONTROL S.p.A.",
    "001BC5045": "Marvel Digital International Limited",
    "001BC5033": "JE Suunnittelu Oy",
    "001BC5031": "ADIXEIN LIMITED",
    "001BC5034": "InterCEL Pty Ltd",
    "001BC502C": "Care Everywhere LLC",
    "001BC502B": "Saturn South Pty Ltd",
    "001BC5028": "STECHWIN.CO.LTD.",
    "001BC5021": "Openpeak, Inc",
    "001BC5020": "Momentum Data Systems",
    "001BC5022": "CJSC STC  SIMOS",
    "001BC501F": "Saturn Solutions Ltd",
    "001BC5016": "Energotechnica OOO NPP Ltd",
    "001BC500A": "Mercury HMI Ltd",
    "001BC500C": "Quantum Technology Sciences, Inc.",
    "001BC5010": "Softel SA de CV",
    "001BC5009": "Solomon Systech Pte Ltd",
    "8C1F6407F": "G.S.D GROUP INC.",
    "8C1F6421E": "The Bionetics Corporation",
    "8C1F64066": "Siemens Energy Global GmbH & Co. KG",
    "8C1F64910": "Vortex IoT Ltd",
    "8C1F64AD7": "Monnit Corporation",
    "8C1F64B86": "Elektronik & Modellprodukter G\u00e4vle AB",
    "8C1F64F4F": "Leonardo Germany GmbH",
    "8C1F6480C": "Thermify Holdings Ltd",
    "8C1F64E00": "DVB-TECH S.R.L.",
    "8C1F64487": "TECHKON GmbH",
    "8C1F6481F": "ViewSonic Corp",
    "8C1F64F1D": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64881": "Flextronics International Kft",
    "8C1F64DEB": "PXM Marek Zupnik spolka komandytowa",
    "8C1F64042": "HEITEC AG",
    "70B3D5695": "Televic Rail GmbH",
    "8C1F64B00": "Gets MSS",
    "8C1F64378": "spar Power Technologies Inc.",
    "8C1F64B28": "Season Electronics Ltd",
    "8C1F6420D": "Grossenbacher Systeme AG",
    "8C1F64FEA": "AKON Co.,Ltd.",
    "8C1F64EF0": "Sonendo Inc",
    "8C1F64432": "Rebel Systems",
    "8C1F647C6": "Flextronics International Kft",
    "8C1F64327": "Deutescher Wetterdienst",
    "8C1F64A3E": "Hiwin Mikrosystem Corp.",
    "8C1F64380": "YSLAB",
    "8C1F6432E": "Trineo Systems Sp. z o.o.",
    "8C1F64F6D": "Ophir Manufacturing Solutions Pte Ltd",
    "8C1F64D38": "CUU LONG TECHNOLOGY AND TRADING COMPANY LIMITED",
    "8C1F64141": "Code Blue Corporation ",
    "8C1F64FA6": "SurveyorLabs LLC",
    "8C1F6439B": "Deviceworx Technologies Inc.",
    "8C1F6476A": "DORLET SAU",
    "8C1F64B26": "AVL DiTEST GmbH",
    "8C1F64554": "Herholdt Controls srl",
    "8C1F64D8C": "SMRI",
    "8C1F64419": "Naval Group",
    "8C1F64053": "HS.com Kft",
    "8C1F64D60": "Potter Electric Signal Co. LLC",
    "8C1F64342": "TimeMachines Inc.",
    "8C1F64DA5": "DAOM",
    "8C1F64F92": "Vision Systems Safety Tech",
    "8C1F642A6": "Radiation Solutions Inc.",
    "70B3D508B": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F64C35": "Peter Huber Kaeltemaschinenbau SE",
    "70B3D50BE": "Chamsys Ltd",
    "8C1F6456B": "Avida, Inc.",
    "8C1F6448B": "Monnit Corporation",
    "8C1F64ED6": "PowTechnology Limited",
    "8C1F64B2A": "Lumiplan Duhamel",
    "8C1F6441F": "Gigalane",
    "8C1F648D6": "ADC Global Technology Sdn Bhd",
    "8C1F645B9": "ViewSonic Corp",
    "8C1F641C9": "Pneumax Spa",
    "8C1F64364": "TILAK INTERNATIONAL",
    "8C1F64BBE": "AirScan, Inc. dba HemaTechnologies",
    "8C1F6400A": "TaskUnite Inc. (dba AMPAworks)",
    "8C1F64B18": "Grossenbacher Systeme AG",
    "8C1F6438F": "Unabiz",
    "70B3D5BFD": "Lumentum",
    "70B3D5ED8": "Wartsila Voyage Oy",
    "8C1F64722": "Artome Oy",
    "8C1F64A13": "INVENTIA Sp. z o.o.",
    "8C1F64E0B": "Laurel Electronics LLC",
    "70B3D5A05": "Wartsila Voyage Oy",
    "8C1F64696": "Emerson Rosemount Analytical",
    "8C1F64B20": "Lechpol Electronics Sp\u00f3\u0142ka z o.o. Sp.k.",
    "8C1F64AF1": "E-S-Tel",
    "8C1F64294": "nanoTRONIX Computing Inc.",
    "8C1F64F7D": "RPG INFORMATICA, S.A.",
    "8C1F647A0": "Potter Electric Signal Co. LLC",
    "8C1F64E0C": "TELESTRIDER SA",
    "8C1F64BA6": "FMC Technologies Measurement Solutions Inc",
    "8C1F6410B": "Red Lion Europe GmbH",
    "8C1F6413C": "SiFive Inc",
    "8C1F64D1F": "Free Talk Engineering Co., Ltd",
    "8C1F645E8": "Sterna Security Devices Pvt Ltd",
    "8C1F64260": "E-alarms",
    "8C1F64016": "Signtel Communications Pvt Ltd",
    "8C1F64196": "Secuinfo Co.Ltd",
    "8C1F64F7C": "General Dynamics IT",
    "8C1F648B6": "AXIS Sp z o.o.",
    "8C1F64215": "XLOGIC srl",
    "70B3D57C2": "Morgan Schaffer",
    "8C1F64DC6": "R&K",
    "8C1F64580": "SAKURA SEIKI Co., Ltd.",
    "8C1F64524": "Aski Industrie Elektronik GmbH",
    "8C1F64D2D": "Eskomar Ltd. ",
    "8C1F64C7D": "Glasson Electronics Ltd",
    "8C1F64D11": "Benetel",
    "8C1F644F1": "Abbott Diagnostics Technologies AS",
    "8C1F64715": "INTERNET PROTOCOLO LOGICA SL",
    "8C1F64561": "Deep Detection / ESB01736990",
    "8C1F64C45": "Flextroincs International (Taiwain Ltd",
    "8C1F6472D": "Hills Health Solutions",
    "8C1F6485E": "Apen Group S.p.A.  (VAT IT08767740155) ",
    "8C1F640CD": "DEUTA Werke GmbH",
    "8C1F647DF": "Secury360",
    "8C1F6449F": "Shenzhen Dongman Technology Co.,Ltd",
    "8C1F641B9": "D.T.S Illuminazione Srl ",
    "8C1F644F4": "Staco Energy Products",
    "8C1F646E7": "WiTricity Corporation",
    "8C1F64186": "Breas Medical AB",
    "70B3D5EA7": "MECCALTE SPA",
    "8C1F64527": "PDW",
    "8C1F64E89": "PADL Software Pty Ltd",
    "8C1F64904": "Hensoldt Sensors GmbH",
    "8C1F649CE": "Exi Flow Measurement Ltd",
    "8C1F64A0F": "DORLET SAU",
    "8C1F64559": "Intozi Tech Pvt Ltd",
    "8C1F649AC": "Hangzhou Jingtang Communication Technology Co.,Ltd.",
    "8C1F640C8": "EA Elektro-Automatik",
    "8C1F647BB": "Kotsu Dengyosha Co., Ltd.",
    "8C1F64BB4": "HIGH RIGHT CO.,Ltd",
    "8C1F64479": "AKSE srl",
    "8C1F64AEC": "Pixel Design & Manufacturing Sdn. Bhd.",
    "8C1F64F28": "DEUTA Werke GmbH",
    "8C1F64EF4": "Mediaport Systems Ltd",
    "8C1F64A98": "Jacobs Technology, Inc.",
    "8C1F644A9": "Martec Marine S.p.a.",
    "8C1F64DE5": "Gogo Business Aviation",
    "8C1F64B5A": "YUYAMA MFG Co.,Ltd",
    "8C1F64E80": "Power Electronics Espana, S.L.",
    "70B3D51D7": "BAE Systems Apllied Intelligence",
    "70B3D5DA9": "RCH SPA",
    "70B3D5410": "Avant Technologies",
    "70B3D5097": "Avant Technologies",
    "8C1F6450B": "Beijing Entian Technology Development Co., Ltd",
    "8C1F64CAE": "Ophir Manufacturing Solutions Pte Ltd",
    "8C1F649E4": "RMDS innovation inc.",
    "8C1F646B7": "Alpha-Omega Technology GmbH & Co. KG",
    "8C1F64E2D": "RADA Electronics Industries Ltd.",
    "8C1F647CE": "Shanghai smartlogic technology Co.,Ltd.",
    "8C1F64621": "JTL Systems Ltd.",
    "70B3D5F22": "Shengli Technologies",
    "8C1F64A89": "Mitsubishi Electric India Pvt. Ltd.",
    "8C1F64694": "Hubbell Power Systems",
    "8C1F64CD4": "Shengli Technologies",
    "8C1F64C17": "Metreg Technologies GmbH",
    "8C1F645A6": "Kinney Industries, Inc",
    "8C1F6462C": "Hangzhou EasyXR Advanced Technology Co., Ltd.",
    "8C1F64898": "Copper Connections Ltd",
    "8C1F647E3": "UNE SRL",
    "8C1F64440": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64937": "H2Ok Innovations",
    "8C1F648E0": "Reivax S/A Automa\u00e7\u00e3o e Controle",
    "8C1F6406A": "Intellisense Systems Inc.",
    "8C1F642DE": "Polar Bear Design",
    "70B3D5C1D": "Kranze Technology Solutions, Inc.",
    "8C1F64107": "SCI Technology, Inc.",
    "8C1F64863": "EngiNe srl",
    "8C1F64677": "FREY S.J.",
    "8C1F6479F": "Hiwin Mikrosystem Corp.",
    "8C1F64B6D": "Andy-L Ltd",
    "8C1F64CA7": "eumig industrie-TV GmbH.",
    "8C1F6467E": "LDA Audiotech",
    "8C1F64F53": "Beckman Coulter Inc",
    "8C1F64AE5": "Ltec Co.,Ltd",
    "8C1F64E4B": "ALGAZIRA TELECOM SOLUTIONS",
    "8C1F6498C": "PAN Business & Consulting (ANYOS]",
    "8C1F641A0": "Engage Technologies",
    "8C1F64BC1": "CominTech, LLC",
    "8C1F649A1": "Pacific Software Development Co., Ltd.",
    "8C1F64AD0": "Elektrotechnik & Elektronik Oltmann GmbH",
    "8C1F64495": "DAVE SRL",
    "70B3D5C0A": "Infosocket Co., Ltd.",
    "8C1F64E23": "Chemito Infotech PVT LTD",
    "8C1F64ECC": "Baldwin Jimek AB",
    "8C1F64E8F": "JieChuang HeYi(Beijing) Technology Co., Ltd.",
    "8C1F641AD": "Nexxto Servicos Em Tecnologia da Informacao SA",
    "8C1F6414D": "Vertesz Elektronika Kft.",
    "8C1F640BF": "Aurora Communication Technologies Corp.",
    "8C1F642F0": "Switch Science, Inc.",
    "8C1F64D21": "AMETEK CTS GMBH",
    "8C1F649E0": "Druck Ltd.",
    "8C1F64A91": "Infinitive Group Limited",
    "8C1F64D6C": "Packetalk LLC",
    "8C1F6431B": "joint analytical systems GmbH",
    "8C1F64DD4": "Midlands Technical Co., Ltd.",
    "8C1F640D8": "Power Electronics Espana, S.L.",
    "8C1F64A81": "3D perception AS",
    "8C1F642C7": "CONTRALTO AUDIO SRL",
    "8C1F641CE": "Eiden Co.,Ltd.",
    "8C1F642B8": "Veinland GmbH",
    "8C1F64423": "Hiwin Mikrosystem Corp.",
    "8C1F6432B": "Shenyang Taihua Technology Co., Ltd.",
    "8C1F64907": "Sicon srl",
    "8C1F64D42": "YUYAMA MFG Co.,Ltd",
    "8C1F64FD5": "THE WHY HOW DO COMPANY, Inc.",
    "8C1F64D51": "ZIGEN Lighting Solution co., ltd.",
    "8C1F648DF": "Grossenbacher Systeme AG",
    "70B3D594B": "RF Code",
    "8C1F6471B": "Adasky Ltd.",
    "8C1F64636": "Europe Trade",
    "8C1F641F5": "NanoThings Inc.",
    "8C1F64211": "Bipom Electronics, Inc.",
    "8C1F64F5F": "TR7 Siber Savunma A.S.",
    "8C1F64300": "Abbott Diagnostics Technologies AS",
    "8C1F64978": "Planet Innovation Products Inc.",
    "70B3D54BC": "TIAMA",
    "70B3D58BA": "TIAMA",
    "8C1F6463B": "TIAMA",
    "8C1F640B7": "TIAMA",
    "8C1F6406B": "Sanwa Supply Inc.",
    "8C1F64475": "Alpine Quantum Technologies GmbH",
    "8C1F647DC": "LINEAGE POWER PVT LTD.,",
    "8C1F64CC5": "Potter Electric Signal Co. LLC",
    "8C1F64D98": "Gnewtek photoelectric technology Ltd.",
    "8C1F6420E": "Alpha Bridge Technologies Private Limited",
    "8C1F64ECF": "Monnit Corporation",
    "8C1F6462D": "Embeddded Plus Plus",
    "8C1F649FB": "CI SYSTEMS ISRAEL LTD",
    "8C1F64A5E": "XTIA Ltd.",
    "8C1F64FDC": "Nuphoton Technologies",
    "8C1F64C59": "Tunstall A/S",
    "8C1F645FA": "PolCam Systems Sp. z o.o.",
    "8C1F64B65": "HomyHub SL",
    "8C1F64691": "Wende Tan",
    "8C1F643B7": "AI-BLOX",
    "8C1F649C0": "Header Rhyme",
    "8C1F64EE6": "LYNKX",
    "8C1F64406": "ANDA TELECOM PVT LTD",
    "8C1F642BB": "Chakra Technology Ltd",
    "8C1F642FC": "Unimar, Inc.",
    "8C1F64A9E": "Optimum Instruments Inc.",
    "8C1F64C61": "Beijing Ceresdate Technology Co.,LTD",
    "8C1F64610": "Beijing Zhongzhi Huida Technology Co., Ltd",
    "8C1F6486A": "VisionTools Bildanalyse Systeme GmbH",
    "8C1F64931": "Noptel Oy",
    "8C1F64024": "Shin Nihon Denshi Co., Ltd.",
    "70B3D5A31": "Wise Ally Holdings Limited",
    "8C1F64FCC": "GREDMANN TAIWAN LTD.",
    "8C1F64011": "DEUTA-WERKE GmbH",
    "8C1F64145": "Spectrum FiftyNine BV",
    "8C1F64573": "Ingenious Technology LLC",
    "8C1F642E2": "Mark Roberts Motion Control",
    "8C1F644FB": "MESA TECHNOLOGIES LLC",
    "8C1F64867": "Forever Engineering Systems Pvt. Ltd.",
    "8C1F647A4": "Hirotech inc.",
    "8C1F64BF3": "Alphatek AS",
    "8C1F64502": "Samwell International Inc",
    "8C1F64C91": "Soehnle Industrial Solutions GmbH",
    "70B3D530E": "Ecolonum Inc.",
    "8C1F64F2D": "HUERNER Schweisstechnik GmbH",
    "8C1F6402F": "SOLIDpower SpA",
    "8C1F64F22": "Voyage Audio LLC",
    "8C1F6491D": "enlighten",
    "8C1F64731": "ehoosys Co.,LTD.",
    "8C1F64699": "FIDICA GmbH & Co. KG",
    "8C1F64F70": "Vision Systems Safety Tech",
    "8C1F64CB2": "Dyncir Solu\u00e7\u00f5es Tecnol\u00f3gicas Ltda",
    "8C1F64BCB": "A&T Corporation",
    "8C1F64CEB": "EUREKA FOR SMART PROPERTIES CO. W.L.L",
    "8C1F64314": "Cedel BV",
    "8C1F643B6": "TEX COMPUTER SRL ",
    "8C1F64EF8": "Northwest Central Indiana Community Partnerships Inc dba Wabash Heartland Innovation Network (WHIN)",
    "8C1F6434D": "biosilver .co.,ltd",
    "8C1F646A0": "Avionica ",
    "8C1F6466F": "Elix Systems SA",
    "8C1F64309": "MECT SRL",
    "8C1F64698": "Arcus-EDS GmbH",
    "8C1F646EC": "Bit Trade One, Ltd.",
    "8C1F648F4": "Loadrite (Auckland) Limited",
    "8C1F6428D": "AVA Monitoring AB",
    "8C1F646BB": "Season Electronics Ltd",
    "8C1F64D5E": "Integer.pl S.A.",
    "8C1F6458C": "Ear Micro LLC",
    "8C1F640EF": "DAVE SRL",
    "8C1F64B13": "Abode Systems Inc",
    "8C1F6498F": "Breas Medical AB",
    "8C1F64662": "Suzhou Leamore Optronics Co., Ltd.",
    "8C1F647BC": "GO development GmbH",
    "8C1F64544": "Tinkerbee Innovations Private Limited",
    "8C1F641F0": "AVCOMM Technologies Inc",
    "8C1F64080": "Twinleaf LLC",
    "8C1F64CC6": "Genius Vision Digital Private Limited",
    "8C1F643E3": "FMTec GmbH - Future Management Technologies",
    "8C1F643D4": "e.p.g. Elettronica s.r.l.",
    "8C1F643C5": "Stratis IOT",
    "8C1F64B7C": "EVERNET CO,.LTD TAIWAN",
    "8C1F64017": "Farmote Limited",
    "8C1F64EEA": "AMESS",
    "8C1F64AB4": "Beijing Zhongchen Microelectronics Co.,Ltd",
    "8C1F64E7C": "Ashinne Technology Co., Ltd",
    "8C1F64AC5": "Forever Engineering Systems Pvt. Ltd.",
    "8C1F6454F": "Toolplanet Co., Ltd.",
    "8C1F6452D": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "8C1F64FA2": "AZD Praha s.r.o., ZOZ Olomouc",
    "8C1F64702": "AIDirections",
    "8C1F64943": "Autark GmbH",
    "8C1F64E30": "VMukti Solutions Private Limited",
    "8C1F647A7": "Timegate Instruments Ltd.",
    "70B3D5781": "Project Service S.a.s.",
    "8C1F643F4": "ACTELSER S.L.",
    "8C1F64A4C": "Flextronics International Kft",
    "8C1F6435C": "Opgal Optronic Industries ltd",
    "8C1F64D02": "Flextronics International Kft",
    "8C1F64115": "Neuralog LP",
    "8C1F648D9": "Pietro Fiorentini Spa",
    "8C1F64C24": "Alifax S.r.l.",
    "8C1F6440C": "Sichuan Aiyijan Technology Company Ltd.",
    "8C1F64194": "TIFLEX",
    "8C1F6405F": "ESCAD AUTOMATION GmbH",
    "8C1F64BD3": "IO Master Technology",
    "8C1F64A44": "Rapidev Pvt Ltd",
    "8C1F64581": "SpectraDynamics, Inc.",
    "8C1F64E02": "ITS Teknik A/S",
    "8C1F64ED9": "NETGEN HITECH SOLUTIONS LLP",
    "8C1F644AC": "Vekto",
    "8C1F6450A": "BELLCO TRADING COMPANY (PVT) LTD",
    "8C1F649D4": "Wolfspyre Labs",
    "8C1F646CF": "Italora",
    "8C1F645AE": "Suzhou Motorcomm Electronic Technology Co., Ltd",
    "8C1F64707": "OAS AG",
    "8C1F64B8D": "Tongye lnnovation Science and Technology (Shenzhen) Co.,Ltd",
    "8C1F64D0E": "Labforge Inc.",
    "8C1F64F72": "Contrader",
    "8C1F6438D": "Wilson Electronics",
    "8C1F64D40": "Breas Medical AB",
    "8C1F64417": "Fracarro srl",
    "70B3D5EB0": "Nautel LTD",
    "8C1F64848": "Jena-Optronik GmbH",
    "8C1F6477E": "Institute of geophysics, China earthquake administration",
    "70B3D5A1D": "Fluid Components Intl",
    "8C1F648D4": "Recab Sweden AB",
    "8C1F64256": "Landinger",
    "8C1F64DCA": "Porsche engineering",
    "8C1F6457B": "Potter Electric Signal Company",
    "70B3D585C": "Tabology",
    "8C1F647B8": "TimeMachines Inc.",
    "8C1F64A07": "GJD Manufacturing",
    "8C1F64780": "HME Co.,ltd",
    "8C1F64328": "Com Video Security Systems Co., Ltd.",
    "8C1F64CD8": "Gogo Business Aviation",
    "8C1F64BEE": "Sirius LLC",
    "8C1F64971": "INFRASAFE/ ADVANTOR SYSTEMS ",
    "8C1F64CDF": "Canway Technology GmbH",
    "8C1F64512": "Blik Sensing B.V.",
    "8C1F64CA1": "Pantherun Technologies Pvt Ltd",
    "8C1F641E1": "VAF Co.",
    "8C1F64973": "Dorsett Technologies Inc",
    "8C1F649F2": "MB connect line GmbH Fernwartungssysteme",
    "8C1F649F0": "ePlant, Inc.",
    "8C1F64FBD": "SAN-AI Electronic Industries Co.,Ltd.",
    "8C1F641D1": "AS Str\u00f6mungstechnik GmbH",
    "8C1F646EA": "KMtronic ltd",
    "8C1F640F9": "ikan International LLC",
    "8C1F64E98": "Luxshare Electronic Technology (Kunshan) LTD",
    "8C1F64D78": "Hunan Oushi Electronic Technology Co.,Ltd",
    "8C1F64923": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64E73": "GTR Industries",
    "8C1F64534": "SURYA ELECTRONICS",
    "8C1F64AD2": "YUYAMA MFG Co.,Ltd",
    "8C1F6407E": "FLOYD inc.",
    "8C1F64CA6": "ReliaSpeak Information Technology Co., Ltd.",
    "8C1F6430A": "XCOM Labs",
    "8C1F648E2": "ALPHA Corporation",
    "8C1F648D1": "Orlaco Products B.V.",
    "8C1F64B0C": "Barkodes Bilgisayar Sistemleri Bilgi Iletisim ve Y",
    "8C1F64C50": "Spacee",
    "8C1F645D3": "Eloy Water",
    "8C1F64242": "GIORDANO CONTROLS SPA",
    "8C1F6453D": "NEXCONTECH",
    "8C1F64D54": "Grupo Epelsa S.L.",
    "8C1F64E43": "Daedalean AG",
    "8C1F643B5": "SVMS",
    "8C1F64445": "Figment Design Laboratories",
    "8C1F64F86": "INFOSTECH Co., Ltd.",
    "8C1F64AB5": "JUSTMORPH PTE. LTD.",
    "8C1F64AA4": "HEINEN ELEKTRONIK GmbH",
    "8C1F644FA": "Sanskruti",
    "8C1F647C8": "Jacquet Dechaume",
    "8C1F64304": "Jemac Sweden AB",
    "8C1F649C1": "RealWear",
    "8C1F64AED": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64801": "Zhejiang Laolan Information Technology Co., Ltd",
    "8C1F64204": "castcore",
    "8C1F6429F": "NAGTECH LLC",
    "8C1F64947": "LLC \"TC \"Vympel\"",
    "8C1F64C03": "Abiman Engineering",
    "8C1F64438": "Integer.pl S.A.",
    "8C1F64CF1": "ROBOfiber, Inc.",
    "8C1F6419C": "Aton srl",
    "8C1F64AAB": "BlueSword Intelligent Technology Co., Ltd.",
    "8C1F64991": "DB Systel GmbH",
    "8C1F64C41": "Katronic AG & Co. KG",
    "8C1F643A4": "QLM Technology Ltd",
    "70B3D5AC2": "Wisebox.,Co.Ltd",
    "70B3D5BF7": "Fischer Connectors",
    "70B3D5A8F": "VK Integrated Systems",
    "70B3D5E03": "MBJ",
    "70B3D581F": "CAR-connect GmbH",
    "70B3D5271": "Code Blue Corporation ",
    "70B3D5975": "Coester Automa\u00e7\u00e3o Ltda",
    "70B3D5CE6": "Dynim Oy",
    "70B3D5198": "Beijing Muniulinghang Technology Co., Ltd",
    "70B3D50AA": "Wanco Inc",
    "70B3D5F74": "TESSA AGRITECH SRL",
    "70B3D538E": "China Telecom Fufu Information Technology CO.,LTD",
    "70B3D5720": "Jeio Tech ",
    "70B3D5965": "LINEAGE POWER PVT LTD.,",
    "70B3D52E4": "Schneider Electric Motion USA",
    "70B3D5E19": "BAB TECHNOLOGIE GmbH",
    "70B3D550B": "Nordson Corporation",
    "70B3D5928": "Done Design Inc",
    "70B3D5812": "TESCAN Brno, s.r.o.",
    "70B3D53DC": "XIA LLC",
    "70B3D5B79": "Dadacon GmbH",
    "70B3D5DEB": "DORLET SAU",
    "70B3D5F1B": "IndiNatus (IndiNatus India Private Limited)",
    "70B3D56B4": "Nudron IoT Solutions LLP",
    "70B3D5419": "Prodata Mobility Brasil SA",
    "70B3D5FE1": "Shenzhen Zhiting Technology Co.,Ltd",
    "70B3D5CC0": "Avionica ",
    "70B3D5FE0": "Blueprint Lab",
    "70B3D55B7": "on-systems limited",
    "70B3D5725": "Swiss Timing LTD",
    "70B3D5004": "LEIDOS",
    "70B3D5915": "DHK Storage, LLC",
    "70B3D54FB": "MAS Elettronica sas di Mascetti Sandro e C.",
    "70B3D572B": "Medipense Inc.",
    "70B3D5B4C": "AmericanPharma Technologies",
    "70B3D57CC": "MITSUBISHI HEAVY INDUSTRIES THERMAL SYSTEMS, LTD.",
    "70B3D5F31": "The-Box Development",
    "70B3D5946": "GREATWALL Infotech Co., Ltd.",
    "70B3D5998": "Kita Kirmizi Takim Bilgi Guvenligi Danismanlik ve Egitim A.S.",
    "70B3D5AD7": "Octopus IoT srl",
    "70B3D58D2": "WIZAPPLY CO.,LTD",
    "70B3D5737": "SD Biosensor",
    "70B3D582F": "SIANA Systems",
    "70B3D5B0D": "ALFI",
    "70B3D5311": "G\u00fcnther Spelsberg GmbH + Co. KG",
    "70B3D5E66": "Eneon sp. z o.o.",
    "70B3D5EC2": "Lightside Instruments AS",
    "70B3D5B42": "Samwell International Inc",
    "70B3D559E": "i2-electronics",
    "70B3D5065": "EXATEL",
    "70B3D5C13": "Guangzhou Xianhe Technology Engineering Co., Ltd",
    "70B3D5290": "GETT Geraetetechnik GmbH",
    "70B3D5CEE": "ACRIOS Systems s.r.o.",
    "70B3D5AFC": "BAE Systems",
    "70B3D5A65": "CREATIVE",
    "70B3D5C47": "ABB",
    "70B3D5FAC": "Integrated Protein Technologies, Inc.",
    "70B3D56E2": "E-Controls",
    "70B3D58B5": "xTom GmbH",
    "70B3D55D7": "Clockwork Dog",
    "70B3D535B": "Nuance Hearing Ltd.",
    "70B3D5E68": "Transit Solutions, LLC.",
    "70B3D5E65": "BIRTECH TECHNOLOGY",
    "70B3D5D82": "SUN ELECTRONICS CO.,LTD.",
    "70B3D509C": "Cardinal Kinetic",
    "70B3D558B": "Williams Sound LLC",
    "70B3D58D4": "Guangdong Transtek Medical Electronics Co., Ltd.",
    "70B3D56C8": "Sicon srl",
    "70B3D5E44": "BrainboxAI Inc",
    "70B3D5859": "HAN CHANG",
    "70B3D5B69": "Daatrics LTD",
    "70B3D583D": "Gentec",
    "70B3D51A2": "Xirgo Technologies LLC",
    "70B3D5DDE": "Abbott Diagnostics Technologies AS",
    "70B3D5B22": "YUYAMA MFG Co.,Ltd",
    "70B3D51BA": "Guan Show Technologe Co., Ltd.",
    "70B3D5B1C": "Serveron / Qualitrol",
    "70B3D5E6B": "Shenzhen Shi Fang Communication Technology Co., Ltd",
    "70B3D5769": "Barber Creations LLC",
    "70B3D53EE": "Laser Imagineering Vertriebs GmbH",
    "70B3D56A7": "Partilink Inc.",
    "70B3D571C": "Konzept Informationssysteme GmbH",
    "70B3D5F02": "ABECO Industrie Computer GmbH",
    "70B3D5DB3": "Klaxoon",
    "70B3D5FE5": "Malin Space Science System",
    "70B3D57D3": "OLEDCOMM",
    "70B3D5905": "Wexiodisk AB",
    "70B3D55CB": "ECoCoMS Ltd.",
    "70B3D5D18": "MetCom Solutions GmbH",
    "70B3D5D53": "BeiLi eTek (Zhangjiagang) Co., Ltd.",
    "70B3D53CD": "BRS Sistemas Eletr\u00f4nicos",
    "70B3D52CB": "Yongtong tech",
    "70B3D5E64": "HONG JIANG ELECTRONICS CO., LTD.",
    "70B3D5AE4": "Nuance Hearing Ltd.",
    "70B3D55FD": "Windar Photonics",
    "70B3D5D35": "King-On Technology Ltd.",
    "70B3D5EAF": "Sicon srl",
    "70B3D5C02": "Garmo Instruments S.L.",
    "70B3D5F4A": "LACS SRL",
    "70B3D536B": "Huz Electronics Ltd",
    "70B3D5110": "Orion Power Systems, Inc.",
    "70B3D543C": "Scenario Automation",
    "70B3D5468": "Shanghai Junqian Sensing Technology Co., LTD",
    "70B3D51C1": "Sphere of economical technologies Ltd",
    "70B3D5E51": "NooliTIC",
    "70B3D566F": "Simplified MFG",
    "70B3D589F": "Levelup Holding, Inc.",
    "70B3D56DC": "DEUTA-WERKE GmbH",
    "70B3D566C": "KRISTECH Krzysztof Kajstura",
    "70B3D51B2": "Cavagna Group Spa",
    "70B3D532B": "RTA srl",
    "70B3D53AC": "RF-Tuote Oy",
    "70B3D5C5E": "Frog Cellsat Limited",
    "70B3D5D88": "Nidec asi spa",
    "70B3D5E7F": "Sankyo Intec Co,ltd",
    "70B3D53B1": "Global Power Products",
    "70B3D5F66": "Seznam.cz, a.s., CZ26168685",
    "70B3D55D9": "Evident Scientific, Inc.",
    "70B3D5EB6": "EnergizeEV",
    "70B3D57FC": "Surion (Pty) Ltd",
    "70B3D5CBB": "Postmark Incorporated ",
    "70B3D58E7": "REO AG",
    "70B3D54B5": "Toolplanet Co., Ltd.",
    "70B3D553F": "Abbott Diagnostics Technologies AS",
    "70B3D595D": "GIORDANO CONTROLS SPA",
    "70B3D5801": "Glory Technology Service Inc.",
    "70B3D53D6": "Ariston Thermo s.p.a.",
    "70B3D5F28": "Yi An Electronics Co., Ltd",
    "70B3D5047": "OOO \"ORION-R\"",
    "70B3D5639": "DORLET SAU",
    "70B3D5130": "MG s.r.l.",
    "70B3D53FC": "TangRen C&S CO., Ltd",
    "70B3D50F4": "Visual Robotics",
    "70B3D5FA8": "Munters",
    "70B3D51EC": "Cherry Labs, Inc.",
    "70B3D5FFB": "QUERCUS TECHNOLOGIES, S.L.",
    "70B3D5EA5": "LOTES TM OOO",
    "70B3D56C2": "TEX COMPUTER SRL ",
    "70B3D5385": "Kamacho Scale Co., Ltd.",
    "70B3D544D": "Vessel Technology Ltd",
    "70B3D5B36": "Cetitec GmbH",
    "70B3D5038": "DONG IL VISION Co., Ltd.",
    "70B3D5109": "DiTEST Fahrzeugdiagnose GmbH",
    "70B3D51BF": "DEUTA-WERKE GmbH",
    "70B3D518F": "Newtec A/S",
    "70B3D5A7F": "AUDIO VISUAL DIGITAL SYSTEMS",
    "70B3D5F80": "Guan Show Technologe Co., Ltd.",
    "70B3D5A60": "Pneumax S.p.A.",
    "001BC5029": "2 FRANCE MARINE",
    "70B3D54FD": "ENLESS WIRELESS",
    "70B3D586A": "Stealth Communications",
    "70B3D567F": "IAAN Co., Ltd",
    "70B3D57C6": "Utrend Technology (Shanghai) Co., Ltd",
    "70B3D5B5D": "SHANDHAI LANDLEAF ARCHITECTURE TECHNOLOGY CO.,LTD",
    "70B3D52B5": "Dosepack India LLP",
    "70B3D5281": "ITG.CO.LTD",
    "70B3D55BB": "Olympus NDT Canada",
    "70B3D562C": "OOO \"NTC Rotek\"",
    "70B3D5310": "Conserv Solutions",
    "70B3D503E": "Guan Show Technologe Co., Ltd.",
    "70B3D5C44": "Franz Kessler GmbH",
    "70B3D597B": "WIKA Alexander Wiegand SE & Co. KG",
    "70B3D5F97": "Typhon Treatment Systems Ltd",
    "70B3D5FA9": "CorDes, LLC",
    "70B3D58BC": "GSI GeoSolutions International Ltd",
    "70B3D5606": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D55EB": "Loma Systems s.r.o.",
    "70B3D5321": "Yite technology",
    "70B3D5EB8": "Emporia Renewable Energy Corp",
    "70B3D567E": "Season Electronics Ltd",
    "70B3D5422": "SUS Corporation",
    "70B3D556D": "Pro-Digital Projetos Eletronicos Ltda",
    "70B3D5757": "GABO ",
    "70B3D55AD": "Profotech",
    "70B3D5143": "A & T Technologies",
    "70B3D5A80": "EVCO SPA",
    "70B3D5CB0": "Ossiaco",
    "70B3D5519": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5DAF": "INNOVATIVE CONCEPTS AND DESIGN LLC",
    "70B3D5CF7": "GENTEC ELECTRO-OPTICS",
    "70B3D5568": "Small Data Garden Oy",
    "70B3D54FF": "Shanghai AiGentoo Information Technology Co.,Ltd.",
    "70B3D54BF": "Exsom Computers LLC",
    "70B3D54E8": "Copious Imaging LLC",
    "70B3D5EC4": "hmt telematik GmbH",
    "70B3D5A52": "APEX Stabilizations GmbH",
    "70B3D5F0F": "Kyoto Denkiki",
    "70B3D5815": "Waco Giken Co.,  Ltd.",
    "70B3D5067": "NEOPATH INTEGRATED SYSTEMS LTDA",
    "70B3D51F0": "Harmonic Design GmbH",
    "70B3D58CC": "Piranha EMS Inc.",
    "70B3D5D16": "Monnit Corporation",
    "70B3D5F82": "Preston Industries dba PolyScience",
    "70B3D5299": "KMtronic ltd",
    "70B3D53EB": "Grossenbacher Systeme AG",
    "70B3D5824": "Songwoo Information & Technology Co., Ltd",
    "70B3D5292": "Boston Dynamics",
    "70B3D55F8": "Forcite Helmet Systems Pty Ltd",
    "70B3D5E8D": "Natav Services Ltd.",
    "70B3D5D23": "COTT Electronics",
    "70B3D5F94": "MB connect line GmbH Fernwartungssysteme",
    "70B3D526D": "Sorion Electronics ltd",
    "70B3D5A49": "Unipower AB",
    "70B3D5514": "Intelligent Security Systems (ISS)",
    "70B3D55E8": "VITEC",
    "70B3D505E": "VITEC",
    "70B3D5957": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D5409": "Beijing Yutian Technology Co., Ltd.",
    "70B3D5931": "MARINE INSTRUMENTS, S.A.",
    "70B3D50F6": "KSE GmbH",
    "70B3D5449": "Edgeware AB",
    "70B3D56D3": "DEUTA-WERKE GmbH",
    "70B3D5701": "COMPAR Computer GmbH",
    "70B3D591D": "Cubitech",
    "70B3D572A": "MRC Systems GmbH",
    "70B3D5463": "WARECUBE,INC",
    "70B3D5102": "Oxford Monitoring Solutions Ltd",
    "70B3D5DF4": "Heim- & B\u00fcrokommunikation Ilmert e.K.",
    "70B3D58BF": "Hangzhou Leaper Technology Co. Ltd.",
    "70B3D5DC3": "Fath Mechatronics",
    "70B3D53CB": "GeoSpectrum Technologies Inc",
    "70B3D5571": "Echogear",
    "70B3D5E54": "Beijing PanGu Company",
    "70B3D52C0": "Sensative AB",
    "70B3D524C": "Astronomical Research Cameras, Inc.",
    "70B3D53EA": "DAVE SRL",
    "70B3D50CA": "VITEC",
    "70B3D5FB9": "EYEDEA",
    "70B3D5E9D": "INTECH",
    "70B3D5C00": "BESO sp. z o.o.",
    "70B3D5210": " Eastone Century Technology Co.,Ltd.",
    "70B3D57FE": "RCH ITALIA SPA ",
    "70B3D5857": "RCH ITALIA SPA ",
    "70B3D5E14": "Automata Spa",
    "70B3D5080": "ABB",
    "70B3D5EAE": "Orlaco Products B.V.",
    "70B3D5477": "digitrol limited",
    "70B3D5BD4": "YUYAMA MFG Co.,Ltd",
    "70B3D59BE": "Izome",
    "70B3D5620": "Orlaco Products B.V.",
    "70B3D5FB2": "KJ3 Elektronik AB",
    "70B3D58ED": "NanoSense",
    "70B3D5506": "Tonbo Imaging Pte Ltd",
    "70B3D5F43": "Divelbiss Corporation",
    "70B3D58AA": "TATTILE SRL",
    "70B3D52A6": "GSI Technology",
    "70B3D55B6": "Ethical Lighting and Sensor Solutions Limited",
    "70B3D532C": "ATION Corporation",
    "70B3D5B50": "iGrid T&D",
    "70B3D5E3C": "Densitron Technologies Ltd",
    "70B3D57C4": "MECT SRL",
    "70B3D5EA1": "Qntra Technology",
    "70B3D538D": "IMP-TELEKOMUNIKACIJE DOO",
    "70B3D5803": "Grossenbacher Systeme AG",
    "70B3D5354": "IMP-Computer Systems",
    "70B3D5863": "Shenzhen Wesion Technology Co., Ltd",
    "70B3D5AB2": "Power Electronics Espana, S.L.",
    "70B3D5B80": "BIGHOUSE.,INC.",
    "70B3D5EDE": "Agrident GmbH",
    "70B3D51D9": "MondeF",
    "70B3D55E7": "Heroic Technologies Inc.",
    "70B3D57DA": "Grupo Epelsa S.L.",
    "70B3D54F1": "LG Electronics",
    "70B3D5B60": "ZAO ZEO",
    "70B3D55EC": "Creative Electronics Ltd",
    "70B3D53DD": "Kniggendorf + K\u00f6gler Security GmbH ",
    "70B3D5590": "812th AITS",
    "70B3D591C": "Alere Technologies AS",
    "70B3D5107": "OOO \"Alyans\"",
    "70B3D55A7": "ABB S.p.A.",
    "70B3D51C0": "W. H. Leary Co., Inc.",
    "70B3D5CE4": "WAVES SYSTEM",
    "70B3D5DC2": "SwineTech, Inc.",
    "70B3D5598": "Ruag Defence France SAS",
    "70B3D5B2D": "Plexus",
    "70B3D52E6": "IPG Photonics Corporation",
    "70B3D58CD": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D54AB": "TruTeq Wireless (Pty) Ltd",
    "70B3D51FF": "Audiodo AB",
    "70B3D5F04": "Scame Sistemi srl",
    "70B3D50CF": "sohonet ltd",
    "70B3D5FAB": "Open System Solutions Limited",
    "70B3D52DE": "YUYAMA MFG Co.,Ltd",
    "70B3D573C": "Centro de Ingenieria y Desarrollo industrial",
    "70B3D5553": "TAALEX Systemtechnik GmbH",
    "70B3D5ABB": "David Horn Communications Ltd",
    "70B3D59F7": "Foerster-Technik GmbH",
    "70B3D5FE9": "Camsat Przemys\u0142aw Gralak",
    "70B3D5C8E": "Coral Telecom Limited",
    "70B3D5F6F": "Smashtag Ltd",
    "70B3D56CB": "NAJIN automation",
    "70B3D5B5F": "CRDMDEVEOPPEMENTS",
    "70B3D5ACA": "Tecnint HTE SRL",
    "70B3D5543": "wallbe GmbH",
    "70B3D53A4": "Ascenix Corporation",
    "70B3D5D01": "Vision4ce Ltd",
    "70B3D530F": "Cardinal Scales Manufacturing Co",
    "70B3D5EC3": "Virtual Control Systems Ltd",
    "70B3D5B9A": "Potter Electric Signal Co. LLC",
    "70B3D5510": "PSL ELEKTRON\u0130K SANAY\u0130 VE T\u0130CARET A.S.",
    "70B3D5302": "DogWatch Inc",
    "70B3D5A84": "SOREL GmbH Mikroelektronik",
    "70B3D5990": "Energy Wall",
    "70B3D549E": "CAPTEMP, Lda",
    "70B3D5437": "Digital Way",
    "70B3D5C9D": "APG Cash Drawer, LLC",
    "70B3D5CC2": "LSC Lighting Systems (Aust) Pty Ltd",
    "70B3D5F51": "IoT Routers Limited",
    "70B3D5403": "Mighty Cube Co., Ltd.",
    "70B3D51E8": "Gogo BA ",
    "70B3D52F5": "eze System, Inc.",
    "70B3D5890": "EIDOS s.r.l.",
    "70B3D5F16": "BRS Sistemas Eletr\u00f4nicos",
    "70B3D5A5F": "Daatrics LTD",
    "70B3D56D2": "Ahrens & Birner Company GmbH",
    "70B3D5E5D": "Boffins Technologies AB",
    "70B3D540E": "Liaoyun Information Technology Co., Ltd.",
    "70B3D56B8": "BT9",
    "70B3D543F": "biosilver .co.,ltd",
    "70B3D5355": "Hongin., Ltd",
    "70B3D5FD7": "Centum Adetel Group",
    "70B3D542E": "Dr. Zinngrebe GmbH",
    "70B3D5518": "CRUXELL Corp.",
    "70B3D5FA5": "Shenzhen Hui Rui Tianyan Technology Co., Ltd.",
    "70B3D50EA": "AEV Broadcast Srl ",
    "70B3D5A9C": "Veo Technologies",
    "70B3D5EE7": "BLUE-SOLUTIONS CANADA INC.",
    "70B3D54EA": "Vocality international T/A Cubic",
    "70B3D5E94": "Lumiplan Duhamel",
    "70B3D5F58": "CDR SRL",
    "70B3D531B": "SilTerra Malaysia Sdn. Bhd.",
    "70B3D59D1": "OS42 UG (haftungsbeschraenkt)",
    "70B3D5A64": "Newshine",
    "70B3D5DB6": "csintech",
    "70B3D5848": "Aldridge Electrical Industries",
    "70B3D5675": "alfamation spa",
    "70B3D57F5": "Incusense",
    "70B3D58C6": "Onosokki Co.,Ltd",
    "70B3D502F": "LEGENDAIRE TECHNOLOGY CO., LTD.",
    "70B3D5CC3": "Fidalia Networks Inc",
    "70B3D5AC4": "Lexi Devices, Inc.",
    "70B3D55F9": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5EA8": "Dia-Stron Limited",
    "70B3D507B": "wallbe GmbH",
    "70B3D5014": "FRAKO Kondensatoren und Anlagenbau GmbH",
    "70B3D532D": "Hanwell Technology Co., Ltd.",
    "70B3D5123": "Amfitech ApS",
    "70B3D50C9": "LINEAGE POWER PVT LTD.,",
    "70B3D5B0B": "INTERNET PROTOCOLO LOGICA SL",
    "70B3D51CC": "AooGee Controls Co., LTD.",
    "70B3D5BF9": "Okolab Srl",
    "70B3D5A54": "provedo",
    "70B3D58EB": "Procon Electronics Pty Ltd",
    "70B3D5961": "TASK SISTEMAS DE COMPUTACAO LTDA",
    "70B3D51F8": "Convergent Design",
    "70B3D5F8A": "FRS GmbH & Co. KG",
    "70B3D5E9E": "MSB Elektronik und Ger\u00e4tebau GmbH",
    "70B3D51BE": "Potter Electric Signal Co. LLC",
    "70B3D5015": "EN ElectronicNetwork Hamburg GmbH",
    "70B3D52D2": "SHANGHAI IRISIAN OPTRONICS TECHNOLOGY CO.,LTD.",
    "70B3D5C41": "Merlin CSI",
    "70B3D521F": "CHRONOMEDIA",
    "70B3D5C66": "Blue Access Inc",
    "70B3D5CD0": "Ellenex Pty Ltd",
    "70B3D55FA": "TEX COMPUTER SRL ",
    "70B3D5382": "Naval Group",
    "70B3D54C0": "Technica Engineering GmbH",
    "70B3D5D6A": "KnowRoaming",
    "70B3D57D6": "Yukilab",
    "70B3D5BAF": "SYS TEC electronic GmbH",
    "70B3D58D9": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5EF4": "Orange Tree Technologies Ltd",
    "70B3D5037": "EIFFAGE ENERGIE ELECTRONIQUE",
    "70B3D5A82": "Telefrank GmbH",
    "70B3D5A76": "Pietro Fiorentini",
    "70B3D5363": "Contec Americas Inc.",
    "70B3D5B43": "ZAO ZEO",
    "70B3D5D4A": "O\u00dc ELIKO Tehnoloogia Arenduskeskus",
    "70B3D5651": "Roxford",
    "70B3D51F4": "Hangzhou Woosiyuan Communication Co.,Ltd.",
    "70B3D5B74": "OnYield Inc Ltd",
    "70B3D5749": "Granite River Labs Inc",
    "70B3D5576": "Shandong Hospot IOT Technology Co.,Ltd.",
    "70B3D52FE": "Yaham Optoelectronics Co., Ltd",
    "70B3D54A6": "HZHY TECHNOLOGY",
    "70B3D5526": "FlowNet LLC",
    "70B3D52BF": "FOSHAN VOHOM",
    "70B3D578A": "Hills Health Solutions",
    "70B3D5941": "Triax A/S",
    "70B3D5C1C": "D.E.M. SPA",
    "70B3D5A0D": "Globalcom Engineering SPA",
    "70B3D5602": "Quantum Opus, LLC",
    "70B3D5F98": "Metrum Sweden AB",
    "70B3D5228": "HEITEC AG",
    "70B3D5A86": "Divigraph (Pty) LTD",
    "70B3D57A3": "Impulse Automation",
    "70B3D5F87": "SHINWA INDUSTRIES, INC.",
    "70B3D51CB": "MatchX GmbH",
    "70B3D50F3": "MonsoonRF, Inc.",
    "70B3D5CB4": "Planewave Instruments",
    "70B3D5BFA": "NESA SRL",
    "70B3D5069": "ONDEMAND LABORATORY Co., Ltd.",
    "70B3D508A": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5670": "Particle sizing systems",
    "70B3D5E86": "YUYAMA MFG Co.,Ltd",
    "70B3D51B9": "RELISTE Ges.m.b.H.",
    "70B3D54A0": "FLUDIA",
    "70B3D5132": "Hagenuk KMT Kabelmesstechnik GmbH",
    "70B3D5492": "Jiangsu Jinheng  Information Technology Co.,Ltd.",
    "70B3D56F0": "iTelaSoft Pvt Ltd",
    "70B3D5547": "CE LINK LIMITED",
    "70B3D50C6": "Embedded Arts Co., Ltd.",
    "70B3D59E2": "Ofil USA",
    "70B3D53CF": "Systems Engineering Arts Pty Ltd",
    "70B3D5BB4": "Integritech",
    "70B3D58F7": "I.E. Sevko A.V.",
    "70B3D5A32": "Toughdog Security Systems",
    "70B3D5EF7": "DAVE SRL",
    "70B3D5BB3": "APG Cash Drawer, LLC",
    "70B3D55DB": "Movicom LLC",
    "70B3D5841": "Stanet Co.,Ltd",
    "70B3D5BBF": "Ensys srl",
    "70B3D55F6": "FreeFlight Systems",
    "70B3D5FAE": "Silixa Ltd",
    "70B3D507E": "ENTEC Electric & Electronic CO., LTD",
    "70B3D53AF": "Turbo Technologies Corporation",
    "70B3D5086": "Husty M.Styczen J.Hupert Sp.J.",
    "70B3D5991": "Javasparrow Inc.",
    "70B3D59D7": "KM OptoElektronik GmbH",
    "70B3D5D44": "ic-automation GmbH",
    "70B3D54E5": "viZaar industrial imaging AG",
    "70B3D5897": "EFG CZ spol. s r.o.",
    "70B3D5CD3": "Controlrad",
    "70B3D5284": "Globalcom Engineering SPA",
    "70B3D5809": "Tecnint HTE SRL",
    "70B3D5DFA": "Newtouch Electronics (Shanghai) Co.,Ltd.",
    "70B3D5EC7": "Neoptix Inc.",
    "70B3D5203": "WOOJIN Inc",
    "70B3D5BEA": "Virtuosys Ltd",
    "70B3D5273": "WeVo Tech",
    "70B3D5A35": "Sicon srl",
    "70B3D534E": "Risk Expert sarl",
    "70B3D5768": "Kazan Networks Corporation",
    "70B3D55FB": "TELEPLATFORMS",
    "70B3D518E": "NIPPON SEIKI CO., LTD.",
    "70B3D55D8": "LYNX Technik AG",
    "70B3D56E7": "AML",
    "70B3D5631": "SENSO2ME ",
    "70B3D5ED1": "Przemyslowy Instytut Automatyki i Pomiarow",
    "70B3D5E57": "Iradimed",
    "70B3D5DD1": "em-tec GmbH",
    "70B3D5445": "Advanced Devices SpA",
    "70B3D508D": "Clover Electronics Technology Co., Ltd.",
    "70B3D5AF0": "SEASON DESIGN TECHNOLOGY",
    "70B3D574D": "SPEECH TECHNOLOGY CENTER LIMITED",
    "70B3D569F": "T+A elektroakustik GmbH & Co.KG",
    "70B3D52B0": " Beijing Zhongyi Yue Tai Technology Co., Ltd",
    "70B3D5D3C": "HRT",
    "70B3D52E2": "Spark Lasers",
    "70B3D5888": "Zetechtics Ltd",
    "70B3D5129": "OOO \"Microlink-Svyaz\"",
    "70B3D538B": "Lookman Electroplast Industries Ltd",
    "70B3D580B": "Fischer Block, Inc.",
    "70B3D5175": "Akribis Systems",
    "70B3D564C": "ACEMIS FRANCE",
    "70B3D5370": "Inphi Corporation",
    "70B3D5276": "TELL Software Hungaria Kft.",
    "70B3D560A": "TATA POWER SED",
    "70B3D5FEF": "HANGZHOU HUALAN MICROELECTRONIQUE CO.,LTD",
    "70B3D5831": "Arnouse Digital Devices Corp",
    "70B3D5549": "Procon automatic systems GmbH",
    "70B3D5F45": "Norbit ODM AS",
    "70B3D5AD2": "Wart-Elektronik",
    "70B3D5C60": "Gogo BA ",
    "70B3D5FF1": "Data Strategy Limited",
    "70B3D5572": "CRDE",
    "70B3D59B2": "CONTINENT, Ltd",
    "70B3D5332": "InnoSenT",
    "70B3D52C2": "Quantum Detectors",
    "70B3D517A": "Gencoa Ltd",
    "70B3D5F0D": "MeQ Inc.",
    "70B3D5E04": "Combilent",
    "70B3D5E8E": "Macnica Technology",
    "70B3D5A30": "SHEN ZHEN HUAWANG TECHNOLOGY CO; LTD",
    "70B3D5040": "Savari Inc",
    "70B3D5773": "Rugged Science",
    "70B3D5288": "Bresslergroup",
    "70B3D5115": "Welltec Corp.",
    "70B3D5F77": "Satcube AB",
    "70B3D5B2B": "Vtron Pty Ltd",
    "70B3D55E0": "Hexagon Metrology SAS",
    "70B3D50F1": "Beijing One City Science & Technology Co., LTD",
    "70B3D553B": "Mr.Loop",
    "70B3D5D40": "CRDE",
    "70B3D5753": "HCH. K\u00fcndig & CIE. AG",
    "70B3D50BC": "Practical Software Studio LLC",
    "70B3D5CAE": "THEMA",
    "70B3D56EA": "Edgeware AB",
    "70B3D5D37": "Sicon srl",
    "70B3D50D9": "Brechbuehler AG",
    "70B3D536F": "BuddyGuard GmbH",
    "70B3D5AA4": "Pullnet Technology,S.L.",
    "70B3D5C68": "Mini Solution Co. Ltd.",
    "70B3D52E5": "Fl\u00e4kt Woods AB",
    "70B3D50E3": "SinTau SrL",
    "70B3D515C": "Woods Hole Oceanographic Institution",
    "70B3D5D84": "Sentry360",
    "70B3D539E": "Lanmark Controls Inc.",
    "70B3D5F3B": "Epdm Pty Ltd",
    "70B3D50A3": "Solace Systems Inc.",
    "70B3D5FC2": "HUNTER LIBERTY CORPORATION",
    "70B3D575A": "Standard Backhaul Communications",
    "70B3D5241": "Bolide Technology Group, Inc.",
    "70B3D5635": "Cosylab d.d.",
    "70B3D51E4": "Tecnologix s.r.l.",
    "70B3D52AD": "Opgal Optronic Industries",
    "70B3D55D3": "Supracon AG",
    "70B3D5FF4": "Serveron Corporation",
    "70B3D5352": "Globalcom Engineering SPA",
    "70B3D5443": "Slot3 GmbH",
    "70B3D5A4C": "Alere Technologies AS",
    "70B3D5924": "Meridian Technologies Inc",
    "70B3D53B7": "Paul Scherrer Institut (PSI)",
    "70B3D5A36": "Beijing DamingWuzhou Science&Technology Co., Ltd.",
    "70B3D5A58": "MCQ TECH GmbH",
    "70B3D5785": "Density Inc.",
    "70B3D539B": "IROC AB",
    "70B3D52DB": "ProtoPixel SL",
    "70B3D599E": "Trinity College Dublin",
    "70B3D54EF": "CMI, Inc.",
    "70B3D54CE": "Agilack",
    "70B3D52C3": "Proterra",
    "70B3D5BD5": "Synics AG",
    "70B3D5C7E": "BirdDog Australia",
    "70B3D5AC5": "ATOM GIKEN Co.,Ltd.",
    "70B3D51E9": "comtime GmbH",
    "70B3D549A": "HAXE SYSTEME",
    "70B3D5F30": "ADE Technology Inc.",
    "70B3D548A": "George Wilson Industries Ltd",
    "70B3D5F95": "Get SAT",
    "70B3D554C": "Husty M.Styczen J.Hupert Sp.J.",
    "70B3D5CD5": "Apantac LLC",
    "70B3D5947": "Checkbill Co,Ltd.",
    "70B3D5BC2": "DWEWOONG ELECTRIC Co., Ltd.",
    "70B3D5405": "MG s.r.l.",
    "70B3D5083": "ZAO ZEO",
    "70B3D528D": "Technica Engineering GmbH",
    "70B3D5BE8": "AndFun Co.,Ltd.",
    "70B3D5009": "HolidayCoro",
    "70B3D51C8": "LDA audio video profesional S.L.",
    "70B3D5C15": "Sensobox GmbH",
    "70B3D567B": "Stesalit Systems Ltd",
    "70B3D5550": "Merten GmbH&CoKG",
    "70B3D5142": "DAVE SRL",
    "70B3D5178": "Gamber Johnson-LLC",
    "70B3D5A5C": "Molekule",
    "70B3D5EBD": "midBit Technologies, LLC",
    "70B3D5A8E": "OMESH CITY GROUP",
    "70B3D51BB": "EFENTO T P SZYD\u0141OWSKI K ZAR\u0118BA SP\u00d3\u0141KA JAWNA",
    "70B3D548F": "Seiwa Giken",
    "70B3D5C4F": "AE Van de Vliet BVBA",
    "70B3D5D67": "ALPHA\u3000Corporation",
    "70B3D585D": "ATHREYA INC",
    "70B3D5BF5": "Acacia Research",
    "70B3D5C0E": "SYSDEV Srl",
    "70B3D5E71": "SiS Technology",
    "70B3D57DC": "Software Systems Plus",
    "70B3D5BA2": "MAMAC Systems, Inc.",
    "70B3D5599": "LECO Corporation",
    "70B3D57DD": "Excel Medical Electronics LLC",
    "70B3D5486": "ChongQing JianTao Technology Co., Ltd.",
    "70B3D5D91": "FoodALYT GmbH",
    "70B3D53AE": "Exicom Technologies fze",
    "70B3D5461": "TESEC Corporation",
    "70B3D5FDE": "AERONAUTICAL & GENERAL INSTRUMENTS LTD.",
    "70B3D53D5": "oxynet Solutions",
    "70B3D5B11": "CAB S.R.L.",
    "70B3D5A91": "IDEAL INDUSTRIES Ltd t/a Casella",
    "70B3D5AE5": "BeatCraft, Inc.",
    "70B3D57B4": "Zumbach Electronic AG",
    "70B3D5FC9": "Shanghai EICT Global Service Co., Ltd",
    "70B3D51A3": "Telairity Semiconductor",
    "70B3D5A4F": "Weltek Technologies Co. Ltd.",
    "70B3D5BE6": "CCII Systems (Pty) Ltd",
    "70B3D5750": "Neurio Technology Inc.",
    "70B3D533E": "Dynamic Connect (Suzhou) Hi-Tech Electronic Co.,Ltd.",
    "70B3D5243": "Rohde&Schwarz Topex SA",
    "70B3D509D": "PuS GmbH und Co. KG",
    "70B3D56E6": "Eleven Engineering Incorporated",
    "70B3D5AF1": "Emka Technologies",
    "70B3D5ABA": "CL International ",
    "70B3D535F": "Aplex Technology Inc.",
    "70B3D5C87": "Siemens AG",
    "70B3D518B": "Aplex Technology Inc.",
    "70B3D538C": "MiraeSignal Co., Ltd",
    "70B3D513E": "Stara S/A Ind\u00fastria de Implementos Agr\u00edcolas",
    "70B3D5325": "BlueMark Innovations BV",
    "70B3D568D": "\"Meta-chrom\" Co. Ltd.",
    "70B3D5840": "xm",
    "70B3D594F": "MART NETWORK SOLUTIONS LTD",
    "70B3D5A96": "\u00d6stling Marking Systems GmbH",
    "70B3D5283": "TextNinja Co.",
    "70B3D550E": "Micro Trend Automation Co., LTD",
    "70B3D5E85": "Explorer Inc.",
    "70B3D530C": "Sicon srl",
    "70B3D5BB7": "Innoflight, Inc.",
    "70B3D5D63": "CRDE",
    "70B3D54D5": "Moog Rekofa  GmbH",
    "70B3D5440": "Discover Video",
    "70B3D5640": "Electronic Equipment Company Pvt. Ltd.",
    "70B3D560F": "Tanaka Information System, LLC.",
    "001BC5087": "Onnet Technologies and Innovations LLC",
    "70B3D5259": "Zebra Elektronik A.S.",
    "70B3D5974": "Jireh Industries Ltd.",
    "70B3D54B7": "Aplex Technology Inc.",
    "70B3D501A": "Cubro Acronet GesmbH",
    "70B3D59CB": "Alligator Communications",
    "70B3D59EB": "Preston Industries dba PolyScience",
    "70B3D57E2": "Depro \u00c9lectronique inc",
    "70B3D53C5": "P4Q ELECTRONICS, S.L.",
    "70B3D5F08": "Szabo Software & Engineering UK Ltd",
    "70B3D51A8": "STC \"Rainbow\" Ltd.",
    "70B3D5070": "Lumiplan Duhamel",
    "70B3D57ED": "The Things Network Foundation",
    "70B3D58D8": "VNG Corporation",
    "70B3D5CBE": "Ensura Solutions BV",
    "70B3D5BFE": "Aplex Technology Inc.",
    "70B3D5652": "Robert Bosch, LLC",
    "70B3D545D": "Sensapex Oy",
    "70B3D5BA1": "Cathwell AS",
    "70B3D55A3": "CT Company",
    "70B3D5F7B": "KST technology",
    "70B3D5CCA": "SIEMENS AS",
    "70B3D58B2": "NPF Modem, LLC",
    "70B3D5EEE": "SOCIEDAD IBERICA DE CONSTRUCCIONES ELECTRICAS, S.A. (SICE)",
    "70B3D575C": "UPM Technology, Inc",
    "70B3D533B": "Seal Shield, LLC",
    "70B3D5054": "Groupeer Technologies",
    "70B3D5CA8": "Grupo Epelsa S.L.",
    "70B3D5609": "PBSI Group Limited",
    "70B3D5323": "TATTILE SRL",
    "70B3D58F3": "TATTILE SRL",
    "70B3D5C3B": "Vironova AB",
    "70B3D566B": "Innitive B.V.",
    "70B3D51B4": "5nines",
    "70B3D5996": "XpertSea Solutions inc.",
    "70B3D5BB8": "Al Kamel Systems S.L.",
    "70B3D5671": "Sea Shell Corporation",
    "70B3D5FC1": "InDiCor",
    "70B3D5D07": "Waversa Systems",
    "70B3D5081": "IST Technologies (SHENZHEN) Limited",
    "70B3D5E48": "TDI. Co., LTD",
    "70B3D57B9": "QIAGEN Instruments AG",
    "70B3D5066": "North Pole Engineering, Inc.",
    "70B3D5771": "Apator Miitors ApS",
    "70B3D5185": "R&D Gran-System-S LLC",
    "70B3D5C26": "Triple Play Communications",
    "70B3D5B51": "Critical Link LLC",
    "70B3D5F85": "Solystic",
    "70B3D5817": "Aplex Technology Inc.",
    "70B3D5F11": "BroadSoft Inc",
    "70B3D5504": "Xsight Systems Ltd.",
    "70B3D576F": "OTI LTD",
    "70B3D55EE": "Mikrotron Mikrocomputer, Digital- und Analogtechnik GmbH",
    "70B3D5BED": "Itrinegy Ltd.",
    "70B3D5B34": "Medtronic",
    "70B3D5740": "Prisma Telecom Testing Srl",
    "70B3D57AA": "Sadel S.p.A.",
    "70B3D5E0D": "Sigma Connectivity AB",
    "70B3D5D90": "Aplex Technology Inc.",
    "70B3D52F6": "TATTILE SRL",
    "70B3D5DD7": "DETECT Australia",
    "70B3D5F79": "Firehose Labs, Inc.",
    "70B3D5303": "Fuchu Giken, Inc.",
    "70B3D52EB": "BRNET CO.,LTD.",
    "70B3D5A56": "DORLET SAU",
    "70B3D5230": "CT Company",
    "70B3D50FC": "vitalcare",
    "70B3D54DB": "Temperature@lert",
    "70B3D5714": "Alturna Networks",
    "70B3D5D7A": "Speedifi Inc",
    "70B3D57C3": "Flexim Security Oy",
    "70B3D53E4": "Neptec Technologies Corp.",
    "70B3D58A4": "Phyton, Inc. Microsystems and Development Tools",
    "70B3D5079": "CheckBill Co,Ltd.",
    "70B3D5FCD": "Engage Technologies",
    "70B3D5703": "StromIdee GmbH",
    "70B3D5916": "Techno Mathematical Co.,Ltd",
    "70B3D51A0": "UFATECH LTD",
    "70B3D5730": "Videogenix",
    "70B3D57A8": "dieEntwickler Elektronik GmbH",
    "70B3D5B35": "Rexxam Co.,Ltd.",
    "70B3D5339": "Sierra Nevada Corporation",
    "70B3D5838": "Tofino",
    "70B3D5AB5": "BroadSoft Inc",
    "70B3D59CA": "KOMSIS ELEKTRONIK SISTEMLERI SAN. TIC. LTD.STI",
    "70B3D563B": "Lazer Safe Pty Ltd",
    "70B3D54B1": "LACE LLC.",
    "70B3D5FE7": "VEILUX INC.",
    "70B3D51A5": "METRONIC APARATURA KONTROLNO - POMIAROWA",
    "70B3D54F4": "WiTagg, Inc",
    "70B3D580D": "Data Physics Corporation",
    "70B3D50A6": "PA CONSULTING SERVICES",
    "70B3D5324": "Thales Nederland BV",
    "70B3D5361": "Parent Power",
    "70B3D5431": "Power Electronics Espana, S.L.",
    "70B3D54DC": "JK DEVICE CORPORATION",
    "70B3D5589": "Cityntel OU",
    "70B3D5231": "DELTA TAU DATA SYSTEMS, INC.",
    "70B3D54BB": "Plazma-T",
    "70B3D5CE3": "Dalcnet srl",
    "70B3D54DF": "Nidec Avtron Automation Corp",
    "70B3D5F78": "Manvish eTech Pvt. Ltd.",
    "70B3D5DF2": "AML",
    "70B3D5F72": "Hanshin Electronics",
    "70B3D59BD": "Signal Processing Devices Sweden AB",
    "70B3D5421": "North Star Bestech Co.,",
    "70B3D56E0": "ABB SPA - DMPC",
    "70B3D5811": "CJSC \u00abINTERSET\u00bb",
    "70B3D56DA": "Enovative Networks, Inc.",
    "70B3D546F": "serva transport systems GmbH",
    "70B3D5600": "Stellwerk GmbH",
    "70B3D5875": "Peek Traffic",
    "70B3D574C": "Kwant Controls BV",
    "70B3D56FA": "Dataforth Corporation",
    "70B3D55C8": "YUYAMA MFG Co.,Ltd",
    "70B3D5285": "Bentec GmbH Drilling & Oilfield Systems",
    "70B3D5D46": "Contineo s.r.o.",
    "70B3D59D3": "Communication Technology Ltd.",
    "70B3D5500": "Mistral Solutions Pvt. LTD",
    "70B3D5EA3": "Gridless Power Corperation",
    "70B3D5AE3": "Zhejiang Wellsun Electric Meter Co.,Ltd",
    "70B3D563A": "DAVE SRL",
    "70B3D57F1": "AeroVision Avionics, Inc.",
    "70B3D5E95": "BroadSoft Inc",
    "70B3D5EC6": "ESII",
    "70B3D5665": "CertUsus GmbH",
    "70B3D5C03": "XAVi Technologies Corp.",
    "70B3D5C9B": "Tieto Sweden AB",
    "70B3D5E75": "Watteco",
    "70B3D52D5": "Teuco Guzzini",
    "70B3D5DE7": "Innominds Software Private Limited",
    "70B3D57CD": "Molekuler Goruntuleme A.S.",
    "70B3D51A1": "HMicro Inc",
    "70B3D505F": "UNISOR MULTISYSTEMS LTD",
    "70B3D5EFE": "MEIDEN SYSTEM SOLUTIONS",
    "70B3D5412": "TATTILE SRL",
    "70B3D56F8": "SENSEON Corporation",
    "70B3D5092": "inomed Medizintechnik GmbH",
    "70B3D535D": "Fresh Idea Factory BV",
    "70B3D5B9B": "Elektronik Art",
    "70B3D5CF3": "Mesh Motion Inc",
    "70B3D5DAD": " GD Mission Systems",
    "70B3D57EF": "CRAVIS CO., LIMITED",
    "70B3D5112": "DiTEST Fahrzeugdiagnose GmbH",
    "70B3D5BEC": "Tokyo Communication Equipment MFG Co.,ltd.",
    "70B3D5708": "IBM Research GmbH",
    "70B3D557C": "Automata GmbH & Co. KG",
    "70B3D51F5": "Martec S.p.A.",
    "70B3D5205": "Esource Srl",
    "70B3D5182": "Kitron UAB",
    "70B3D55EA": "KYS,INC",
    "70B3D57B3": "BroadSoft Inc",
    "70B3D5062": "RM Michaelides Software & Elektronik GmbH",
    "70B3D5B8D": "JungwooEng Co., Ltd",
    "70B3D5058": "Telink Semiconductor CO, Limtied, Taiwan",
    "70B3D5956": "AeroVision Avionics, Inc.",
    "70B3D5B8C": "ePOINT Embedded Computing Limited",
    "70B3D5AA0": "Simple Works, Inc.",
    "70B3D5709": "AML",
    "70B3D554D": "Qingdao Haitian Weiye Automation Control System Co., Ltd",
    "70B3D5025": "Elsuhd Net Ltd Co.",
    "70B3D5A15": "Intercore GmbH",
    "70B3D5755": "LandmarkTech Systems Technology Co.,Ltd.",
    "70B3D5BBE": "Sunrise Systems Electronics Co. Inc.",
    "70B3D5751": "GNF",
    "70B3D5EF3": "octoScope",
    "70B3D5C78": "NETA Elektronik AS",
    "70B3D5BDA": "5-D Systems, Inc.",
    "70B3D5B08": "Secuinfo Co. Ltd",
    "70B3D5587": "INCAA Computers",
    "70B3D5CBC": "Procon Electronics Pty Ltd",
    "70B3D5D47": "YotaScope Technologies Co., Ltd.",
    "70B3D5E08": "Olssen",
    "70B3D57E7": "Atessa, Inc.",
    "70B3D5ADD": "GHL Systems Berhad",
    "70B3D523C": "Quasonix, LLC",
    "70B3D5E4A": "ICP NewTech Ltd",
    "70B3D5D86": "WPGSYS Pte Ltd",
    "70B3D5A0B": "ambiHome GmbH",
    "70B3D5E74": "Exfrontier Co., Ltd.",
    "70B3D5EDD": "Solar Network & Partners",
    "70B3D53DE": "ELOMAC Elektronik GmbH",
    "70B3D5184": "XV360 Optical Information Systems Ltd.",
    "70B3D56B6": "INRADIOS GmbH",
    "70B3D55A8": "Farmobile, LLC",
    "70B3D5AE9": "Cari Electronic",
    "001BC50C3": "inomatic GmbH",
    "001BC50BE": "YESpay International Ltd",
    "001BC509F": "ENTE Sp. z o.o.",
    "001BC50A1": "Hangzhou Zhiping Technology Co., Ltd.",
    "001BC50A2": "Hettich Benelux",
    "001BC509A": "Shenzhen Guang Lian Zhi Tong Limited",
    "001BC5095": "PREVAC sp. z o.o.",
    "001BC508A": "Topicon",
    "001BC5083": "DIWEL",
    "001BC5082": "TGS Geophysical Company (UK) Limited",
    "001BC5077": "Momentum Data Systems",
    "001BC5072": "Ohio Semitronics, Inc.",
    "001BC5075": "Kitron GmbH",
    "001BC5068": "HCS KABLOLAMA SISTEMLERI SAN. ve TIC.A.S.",
    "001BC5060": "ENSTECH",
    "001BC505C": "Suretrak Global Pty Ltd",
    "001BC504F": "Orbital Systems, Ltd.",
    "001BC504E": "Mitsubishi Electric India PVT. LTD",
    "001BC504A": "Certis Technology International Pte Ltd",
    "001BC504B": "Silicon Controls",
    "001BC5046": "G\u00c9ANT",
    "001BC503D": "rioxo GmbH",
    "001BC503C": "Xiphos Systems Corp.",
    "001BC503B": "Promixis, LLC",
    "001BC5036": "LOMAR SRL",
    "001BC502F": "Fibrain Co. Ltd.",
    "001BC502D": "DDTRONIK Dariusz Dowgiert",
    "001BC502A": "Analytical Instrument Systems, Inc.",
    "001BC5027": "CAMEA, spol. s r.o.",
    "001BC5024": "ANNECY ELECTRONIQUE SAS",
    "001BC5023": "MAGO di Della Mora Walter",
    "001BC501E": "Private",
    "001BC501C": "Coolit Systems, Inc.",
    "001BC501B": "Commonwealth Scientific and Industrial  Research Organisation",
    "001BC5017": "cPacket Networks",
    "001BC5012": "Tokyo Cosmos Electric, Inc.",
    "001BC5003": "MicroSigns Technologies Inc",
    "8C1F64451": "Guan Show Technologe Co., Ltd.",
    "8C1F64C6D": "EA Elektro-Automatik",
    "8C1F64558": "Scitel",
    "8C1F64B3A": "dream DNS",
    "8C1F64F98": "XPS ELETRONICA LTDA",
    "8C1F6450C": "Automata GmbH & Co. KG",
    "8C1F641B1": "person-AIz AS",
    "8C1F641E7": "CANON ELECTRON TUBES & DEVICES CO., LTD.",
    "8C1F64247": "Dadhwal Weighing Instrument Repairing Works",
    "8C1F647F8": "FleetSafe India Private Limited",
    "8C1F64E2B": "Glotech Exim Private Limited",
    "8C1F64D07": "Talleres de Escoriaza SAU",
    "8C1F6493A": "Rej\u00e5s of Sweden AB",
    "8C1F64B37": "Flextronics International Kft",
    "8C1F64548": "Beijing Congyun Technology Co.,Ltd",
    "8C1F64FED": "Televic Rail GmbH",
    "70B3D5120": "Televic Rail GmbH",
    "8C1F64B98": "Calamity, Inc.",
    "8C1F64B79": "AddSecure Smart Grids",
    "8C1F647B7": "James G. Biddle dba Megger",
    "8C1F64D17": "I.S.A. - Altanova group srl",
    "8C1F646A4": "Automata Spa",
    "8C1F64D96": "Smart Cabling & Transmission Corp.",
    "8C1F64FAB": "LIAN Corporation",
    "8C1F643E6": "elbe informatik GmbH",
    "8C1F6404E": "Auditdata",
    "8C1F648B2": "Abbott Diagnostics Technologies AS",
    "8C1F640D4": "Dalcnet srl",
    "8C1F6430D": "Flextronics International Kft",
    "8C1F64868": "SHENZHEN PEAKE TECHNOLOGY CO.,LTD.",
    "8C1F64970": "Potter Electric Signal Co. LLC",
    "8C1F64994": "uHave Control, Inc",
    "8C1F64624": "Canastra AG",
    "8C1F64081": "Harmony Fire Ltd",
    "8C1F64BAE": "Tieline Research Pty Ltd",
    "8C1F645E4": "Wuxi Zetai Microelectronics Co., LTD",
    "8C1F64BF2": "YUJUN ELECTRICITY INDUSTRY CO., LTD",
    "8C1F645B4": "Axion Lighting",
    "8C1F64062": "ATON GREEN STORAGE SPA",
    "8C1F64B24": "ABB",
    "8C1F6485C": "Zing 5g Communications Canada Inc.",
    "8C1F64BB1": "Transit Solutions, LLC.",
    "8C1F6414F": "NSM",
    "8C1F64C83": "Power Electronics Espana, S.L.",
    "8C1F642DD": "Flextronics International Kft",
    "8C1F64769": "Vonamic GmbH",
    "8C1F645DF": "Roesch Walter Industrie-Elektronik GmbH",
    "70B3D5C45": "STIEBEL ELTRON GMBH & CO. KG",
    "8C1F64203": "ENTOSS Co.,Ltd",
    "8C1F646DD": "ViewSonic Corp",
    "8C1F64F68": "YUYAMA MFG Co.,Ltd",
    "001BC5042": "Chamsys Ltd",
    "8C1F64D99": "INVIXIUM ACCESS INC",
    "8C1F64CBB": "Maris Tech Ltd.",
    "8C1F6430E": "Tangent Design Engineering",
    "8C1F646B0": "O-Net Technologies(Shenzhen)Group Co.,Ltd.",
    "8C1F643B8": "HUBRIS TECHNOLOGIES PRIVATE LIMITED",
    "8C1F64B47": "LINEAGE POWER PVT LTD.,",
    "8C1F6467D": "Ravi Teleinfomatics",
    "8C1F646F7": "EddyWorks Co.,Ltd",
    "8C1F64FE5": "Truenorth ",
    "8C1F64393": "GRE SYSTEM INC.",
    "8C1F64257": "Four Bars Design",
    "8C1F64800": "Shenzhen SDG Telecom Equipment Co.,Ltd.",
    "8C1F648E4": "Cominfo, Inc.",
    "8C1F649A2": "LadyBug Technologies, LLC",
    "8C1F64CA4": "Bit Part LLC",
    "8C1F64352": "Mediashare Ltd",
    "8C1F64BF5": "The Urban Jungle Project",
    "8C1F64014": "Cristal Controles Ltee",
    "8C1F64779": "INVENTIO DI NICOLO' BORDOLI",
    "70B3D59D4": "Wartsila Voyage Oy",
    "8C1F6438E": "Wartsila Voyage Oy",
    "8C1F6480E": "TxWireless Limited",
    "8C1F6443A": "Spacelite Inc",
    "8C1F64CA2": "eumig industrie-TV GmbH.",
    "8C1F64319": "Exato Company",
    "8C1F64ADB": "Hebei Weiji Electric Co.,Ltd",
    "8C1F6403B": "Orion Power Systems, Inc.",
    "8C1F643F7": "Mitsubishi Electric India Pvt. Ltd.",
    "70B3D592F": "SiFive Inc",
    "8C1F64D80": "AZTEK SA",
    "8C1F6458B": "Quectel Wireless Solutions Co.,Ltd.",
    "8C1F648D8": "MBV AG",
    "8C1F640F1": "ideaForge Technology Limited",
    "8C1F645AA": "Landis+Gyr Equipamentos de Medi\u00e7\u00e3o Ltda",
    "70B3D51F7": "Morgan Schaffer",
    "8C1F64DBF": "Rugged Controls",
    "8C1F64A06": "secutech Co.,Ltd.",
    "8C1F64A92": "Agrology, PBC",
    "8C1F647EF": "SAXOGY POWER ELECTRONICS GmbH",
    "8C1F64BB9": "SmartD Technologies Inc",
    "8C1F6437E": "SIDUS Solutions, LLC",
    "8C1F64585": "Shanghai DIDON Industry Co,. Ltd.",
    "8C1F64EFD": "Novatera(Shenzhen)Technologies Co.,Ltd.",
    "8C1F64442": "Potter Electric Signal Co LLC",
    "8C1F64021": "Savant Group",
    "8C1F64857": "roda computer GmbH",
    "8C1F642F9": "TSS COMPANY s.r.o.",
    "8C1F64E3F": "TELETECH SERVICES",
    "8C1F64AF3": "HY smart",
    "8C1F64AF9": "Grossenbacher Systeme AG",
    "8C1F64D1C": "Vesper Technologies",
    "8C1F6472F": "GMV Aerospace and Defence SAU",
    "8C1F640FA": "Nautel LTD",
    "8C1F6401D": "Nordson Corporation",
    "8C1F64A6F": "Cardinal Scales Manufacturing Co",
    "8C1F6415D": "Nhoa Energy Srl",
    "8C1F648B3": "Hubbell Power Systems",
    "8C1F646DF": "ALPHI Technology Corporation",
    "8C1F64BC4": "EasyNet Industry (Shenzhen) Co., Ltd",
    "8C1F6436F": "SP MANUFACTURING PTE LTD",
    "8C1F6467B": "Wi-DAS LLC",
    "8C1F64DC7": "Wide Swath Research, LLC",
    "8C1F6447E": "Novanta IMS",
    "8C1F64B54": "Framatome Inc.",
    "8C1F640E5": "Rugged Science",
    "70B3D59C9": "PK Sound",
    "8C1F64261": "TargaSystem S.r.L.",
    "8C1F64BAB": "YCN",
    "8C1F64C53": "Clockwork Dog",
    "8C1F6471A": "Chell Instruments Ltd",
    "70B3D509A": "AKSE srl",
    "8C1F64A17": "Pneumax Spa",
    "8C1F64E6F": "Vision Systems Safety Tech",
    "70B3D5119": "YPP Corporation",
    "8C1F64D09": "Minartime(Beijing)Science &Technology Development Co.,Ltd",
    "70B3D5900": "DCS Corp",
    "70B3D5159": "RCH SPA",
    "70B3D597D": "RCH SPA",
    "70B3D5233": "RCH SPA",
    "70B3D5294": "RCH SPA",
    "8C1F640F7": "Combilent",
    "8C1F643B1": "Panoramic Power",
    "8C1F649DF": "astTECS Communications Private Limited",
    "8C1F643A3": "Lumentum",
    "8C1F64F05": "Preston Industries dba PolyScience",
    "8C1F64F77": "Invertek Drives Ltd",
    "8C1F6433C": "HUBRIS TECHNOLOGIES PRIVATE LIMITED",
    "8C1F641BE": "MIDEUM ENG",
    "8C1F64500": "Nepean Networks Pty Ltd",
    "8C1F64267": "Karl DUNGS GmbH & Co. KG",
    "8C1F645BD": "MPT-Service project",
    "8C1F64A0E": "Elac Americas Inc.",
    "8C1F64E45": "Integer.pl S.A.",
    "8C1F64E75": "Stercom Power Soltions GmbH",
    "8C1F64648": "Gridpulse c.o.o.",
    "8C1F64BB2": "Grupo Epelsa S.L.",
    "8C1F64334": "OutdoorLink",
    "70B3D52BA": "Active Brains",
    "8C1F64148": "CAREHAWK",
    "8C1F64AAA": "Leder Elektronik Design GmbH",
    "8C1F64358": "Denso Manufacturing Tennessee",
    "8C1F64797": "Alban Giacomo S.p.a.",
    "8C1F64838": "DRIMAES INC.",
    "8C1F64E1E": "Flextronics International Kft",
    "8C1F64061": "Micron Systems",
    "8C1F643D5": "FRAKO Kondensatoren- und Anlagenbau GmbH",
    "8C1F64EF5": "Sigma Defense Systems LLC",
    "8C1F649EC": "Specialized Communications Corp.",
    "8C1F64605": "Xacti Corporation",
    "8C1F64499": "TIAMA",
    "8C1F6482F": "AnySignal",
    "8C1F645D9": "Opdi-tex GmbH",
    "70B3D5401": "Private",
    "8C1F64263": "EPC Power Corporation",
    "8C1F64560": "Dexter Laundry Inc.",
    "8C1F6456F": "ADETEC SAS",
    "8C1F642CB": "Smart Component Technologies Ltd",
    "8C1F644A0": "Tantec A/S",
    "8C1F644D9": "SECURICO ELECTRONICS INDIA LTD",
    "8C1F649F5": "YUYAMA MFG Co.,Ltd",
    "8C1F642D0": "Cambridge Research Systems Ltd",
    "8C1F64A56": "Flextronics International Kft",
    "8C1F6441C": "KSE GmbH",
    "8C1F64CB5": "Gamber-Johnson LLC",
    "8C1F64A77": "Rax-Tech International",
    "8C1F6464E": "Nilfisk Food",
    "8C1F6488E": "CubeWorks, Inc.",
    "8C1F644DC": "BESO sp. z o.o.",
    "8C1F64C85": "Potter Electric Signal Co. LLC",
    "8C1F64CB7": "ARKRAY,Inc.Kyoto Laboratory",
    "70B3D5591": "Medicomp, Inc",
    "70B3D5279": "Medicomp, Inc",
    "8C1F6418B": "M-Pulse GmbH & Co.KG",
    "8C1F64E10": "Scenario Automation",
    "8C1F64E92": "EA Elektro-Automatik",
    "8C1F6484A": "Bitmapper Integration Technologies Private Limited",
    "8C1F6447D": "EB NEURO SPA",
    "8C1F6499E": "EIDOS s.r.l.",
    "70B3D5251": "Tap Home, s.r.o.",
    "70B3D5FFF": "Private",
    "70B3D5851": "Exascend, Inc.",
    "8C1F64BB3": "Zaruc Tecnologia LTDA",
    "8C1F643BB": "Clausal Computing Oy",
    "8C1F64F50": "Vigor Electric Corp.",
    "70B3D5ADB": "RF Code",
    "8C1F64811": "Panoramic Power",
    "8C1F64D46": "End 2 End Technologies",
    "8C1F648DA": "Dart Systems Ltd",
    "8C1F64138": "Vissavi sp. z o.o.",
    "8C1F646E2": "SCU Co., Ltd.",
    "8C1F64367": "LAMTEC Mess- und Regeltechnik f\u00fcr Feuerungen GmbH & Co. KG",
    "8C1F64E53": "T PROJE MUHENDISLIK DIS TIC. LTD. STI.",
    "8C1F6497F": "Talleres de Escoriaza SA",
    "70B3D5CF6": "Tornado Modular Systems",
    "8C1F64F4C": "inomatic GmbH",
    "70B3D58A1": "TIAMA",
    "8C1F645EB": "TIAMA",
    "8C1F64591": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5FA0": "TIAMA",
    "8C1F64D7F": "Fiberstory communications Pvt Ltd",
    "8C1F64879": "ASHIDA Electronics Pvt. Ltd",
    "8C1F64056": "DONG GUAN YUNG FU ELECTRONICS LTD.",
    "8C1F64723": "Celestica Inc.",
    "8C1F64C06": "Tardis Technology",
    "8C1F644F7": "SmartD Technologies Inc",
    "8C1F64C5D": "Alfa Proxima d.o.o.",
    "8C1F64097": "FoMa Systems GmbH",
    "8C1F64AC9": "ShenYang LeShun Technology Co.,Ltd",
    "8C1F6408D": "NEETRA SRL SB",
    "8C1F6438C": "XIAMEN ZHIXIAOJIN INTELLIGENT TECHNOLOGY CO., LTD",
    "8C1F64369": "Orbital Astronautics Ltd",
    "8C1F64B6E": "Loop Technologies",
    "8C1F649E2": "Technology for Energy Corp",
    "8C1F64963": "Gogo Business Aviation",
    "8C1F646B1": "Specialist Mechanical Engineers (PTY)LTD",
    "8C1F64AFD": "Universal Robots A/S",
    "8C1F64733": "Video Network Security",
    "8C1F64852": "ABB",
    "8C1F6461C": "Automata GmbH & Co. KG",
    "8C1F64E86": "ComVetia AG",
    "8C1F649AB": "DAVE SRL",
    "8C1F64C52": "Invendis Technologies India Pvt Ltd",
    "8C1F6463F": "PREO INDUSTRIES FAR EAST LTD",
    "8C1F64187": "Sicon srl",
    "8C1F64F57": "EA Elektro-Automatik",
    "8C1F643B0": "Flextronics International Kft",
    "8C1F647B0": "AXID SYSTEM",
    "8C1F64AF5": "SANMINA ISRAEL MEDICAL SYSTEMS LTD",
    "8C1F6408E": "qiio AG",
    "8C1F64252": "TYT Electronics CO., LTD",
    "8C1F6415C": "TRON FUTURE TECH INC.",
    "8C1F64D5B": "Local Security",
    "8C1F6426E": "Koizumi Lighting Technology Corp.",
    "8C1F642A9": "Elbit Systems of America, LLC",
    "8C1F64387": "OMNIVISION",
    "8C1F642C6": "YUYAMA MFG Co.,Ltd",
    "8C1F6446A": "Pharsighted LLC",
    "8C1F64009": "Converging Systems Inc.",
    "8C1F64FFC": "Invendis Technologies India Pvt Ltd",
    "8C1F647E0": "Colombo Sales & Engineering, Inc.",
    "8C1F64366": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64BCC": "Sound Health Systems",
    "8C1F647E7": "robert juliat",
    "8C1F64354": "Paul Tagliamonte",
    "8C1F6423D": "Mokila Networks Pvt Ltd",
    "8C1F64D3A": "Applied Materials",
    "70B3D5CB7": "HKC Security Ltd.",
    "8C1F64D20": "NAS Engineering PRO",
    "8C1F647EE": "Orange Precision Measurement LLC",
    "8C1F648E5": "Druck Ltd.",
    "8C1F642FB": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64306": "Corigine,Inc.",
    "8C1F64739": "Monnit Corporation",
    "8C1F64AA8": "axelife",
    "8C1F64C04": "SANWA CORPORATION",
    "8C1F6445B": "Beijing Aoxing Technology Co.,Ltd",
    "8C1F648B9": "Zynex Monitoring Solutions",
    "8C1F64E62": "Axcend",
    "8C1F64AB7": "MClavis Co.,Ltd.",
    "8C1F645CB": "dinosys",
    "8C1F646D5": "HTK Hamburg GmbH",
    "8C1F64098": "Agvolution GmbH",
    "8C1F64240": "HuiTong intelligence Company",
    "8C1F647D8": "HIROSAWA ELECTRIC Co.,Ltd.",
    "8C1F64890": "WonATech Co., Ltd.",
    "8C1F648D5": "Agramkow A/S",
    "8C1F64092": "Gogo BA ",
    "8C1F64CDB": "EUROPEAN TELECOMMUNICATION INTERNATIONAL KFT",
    "8C1F64C0E": "Goodtech AS dep Fredrikstad",
    "8C1F641A5": "DIALTRONICS SYSTEMS PVT LTD",
    "8C1F645AC": "YUYAMA MFG Co.,Ltd",
    "8C1F64F65": "Talleres de Escoriaza SA",
    "8C1F64817": "nke marine electronics",
    "8C1F646B9": "GS Industrie-Elektronik GmbH",
    "8C1F64A6A": "Sphere Com Services Pvt Ltd",
    "8C1F64CCB": "suzhou yuecrown Electronic Technology Co.,LTD",
    "8C1F6401A": "Paragraf",
    "8C1F647CF": "Transdigital Pty Ltd",
    "8C1F64D9A": "Beijing Redlink Information Technology Co., Ltd.",
    "8C1F64254": "Zhuhai Yunzhou Intelligence Technology Ltd.",
    "8C1F64AC0": "AIQuatro",
    "8C1F640C0": "Active Research Limited",
    "8C1F6422E": "Jide Car Rastreamento e Monitoramento LTDA",
    "8C1F643B2": "Real Digital",
    "8C1F64E64": "Indefac company",
    "8C1F64672": "Farmobile LLC",
    "8C1F64FB7": "Grace Design/Lunatec LLC",
    "8C1F6428C": "Sakura Seiki Co.,Ltd.",
    "8C1F64807": "GIORDANO CONTROLS SPA",
    "8C1F64045": "VEILUX INC.",
    "8C1F640E6": "Cleanwatts Digital, S.A.",
    "8C1F640AF": "FORSEE POWER",
    "8C1F64F3C": "Microlynx Systems Ltd",
    "8C1F64911": "EOLANE",
    "8C1F646F9": "ANDDORO LLC",
    "8C1F64BC2": "Huz Electronics Ltd",
    "8C1F64641": "biosilver .co.,ltd",
    "8C1F64429": "Abbott Diagnostics Technologies AS",
    "8C1F6412B": "Beijing Tongtech Technology Co., Ltd.",
    "8C1F64085": "SORB ENGINEERING LLC",
    "70B3D55E2": "Grossenbacher Systeme AG",
    "8C1F6409B": "Taiv",
    "8C1F6469E": "AT-Automation Technology GmbH",
    "70B3D5556": "OHASHI ENGINEERING CO.,LTD.",
    "8C1F646FC": "HM Systems A/S",
    "8C1F64FB0": "MARIAN GmbH",
    "8C1F64224": "PHB Eletronica Ltda.",
    "8C1F64E61": "Stange Elektronik GmbH",
    "8C1F645BC": "HEITEC AG",
    "8C1F64F32": "Shenzhen INVT Electric Co.,Ltd",
    "8C1F649A6": "INSTITUTO DE GEST\u00c3O, REDES TECNOL\u00d3GICAS E NERGIAS",
    "8C1F644BB": "IWS Global Pty Ltd",
    "8C1F644F0": "Tieline Research Pty Ltd",
    "8C1F648C4": "Hermes Network Inc",
    "8C1F649FE": "Metroval Controle de Fluidos Ltda",
    "8C1F64498": "YUYAMA MFG Co.,Ltd",
    "8C1F64E5D": "JinYuan International Corporation",
    "8C1F64454": "KJ Klimateknik A/S",
    "8C1F64A32": "Nautel LTD",
    "8C1F64B97": "Gemini Electronics B.V.",
    "8C1F64B10": "MTU Aero Engines AG",
    "70B3D587C": "Nautel LTD",
    "8C1F640EA": "SmartSky Networks LLC",
    "8C1F64747": "VisionTIR Multispectral Technology",
    "8C1F64298": "Megger Germany GmbH",
    "8C1F648A4": "Genesis Technologies AG",
    "8C1F6445F": "Toshniwal Security Solutions Pvt Ltd",
    "8C1F647EC": "Methods2Business B.V.",
    "8C1F64956": "Paulmann Licht GmbH",
    "8C1F64B22": "BLIGHTER SURVEILLANCE SYSTEMS LTD",
    "8C1F64C01": "HORIBA ABX SAS",
    "8C1F64D56": "Wisdom Audio",
    "8C1F64535": "Columbus McKinnon",
    "8C1F64071": "DORLET SAU",
    "8C1F643FE": "Plum sp. z.o.o.",
    "8C1F641BB": "Renwei Electronics Technology (Shenzhen) Co.,LTD.",
    "8C1F64F59": "Inovonics Inc.",
    "8C1F640F0": "Xylon",
    "8C1F64638": "THUNDER DATA TAIWAN CO., LTD.",
    "8C1F64909": "MATELEX",
    "8C1F644EC": "XOR UK Corporation Limited",
    "8C1F64F9E": "DREAMSWELL Technology CO.,Ltd",
    "8C1F641DA": "Chongqing Huaxiu Technology Co.,Ltd",
    "8C1F64B82": "Seed Core Co., LTD.",
    "8C1F643FF": "UISEE(SHANGHAI) AUTOMOTIVE TECHNOLOGIES LTD.",
    "8C1F64708": "ZUUM",
    "8C1F64164": "Revo - Tec GmbH",
    "8C1F649D8": "Integer.pl S.A.",
    "8C1F64FB1": "ABB",
    "8C1F64DB5": "victtron",
    "8C1F646AD": "Potter Electric Signal Company",
    "8C1F64619": "Labtrino AB",
    "8C1F64C54": "First Mode",
    "8C1F648EE": "Abbott Diagnostics Technologies AS",
    "8C1F64892": "MDI Industrial",
    "8C1F6466C": "LINEAGE POWER PVT LTD.,",
    "8C1F64F94": "EA Elektroautomatik GmbH & Co. KG",
    "8C1F6492A": "Thermo Onix Ltd",
    "8C1F646E3": "ViewSonic International Corporation",
    "8C1F6454C": "Gemini Electronics B.V.",
    "8C1F64B84": "SPX Flow Technology",
    "8C1F6460E": "ICT International",
    "8C1F646AE": "Bray International",
    "8C1F6495A": "Shenzhen Longyun Lighting Electric Appliances Co., Ltd",
    "8C1F64219": "Guangzhou Desam Audio Co.,Ltd",
    "8C1F6490E": "Xacti Corporation",
    "70B3D5C0C": "Tech4Race",
    "8C1F64712": "Nexion Data Systems P/L",
    "8C1F6473D": "NewAgeMicro",
    "8C1F642EF": "Invisense AB",
    "8C1F64BA3": "DEUTA-WERKE GmbH",
    "8C1F64414": "INSEVIS GmbH",
    "8C1F64DFA": "ATSE LLC",
    "8C1F642E3": "Erba Lachema s.r.o.",
    "8C1F64FE0": "Potter Electric Signal Company",
    "8C1F640E0": "Autopharma",
    "8C1F6425E": "R2Sonic, LLC",
    "8C1F64CF3": "ABB S.p.A.",
    "8C1F64101": "ASW-ATI Srl",
    "8C1F64D29": "Secure Bits",
    "8C1F64144": "Langfang ENN lntelligent Technology Co.,Ltd.",
    "8C1F64B03": "Shenzhen Pisoftware Technology Co.,Ltd.",
    "8C1F643E8": "Ruichuangte",
    "70B3D5901": "ATS-CONVERS,LLC",
    "8C1F649FD": "Vishay Nobel AB",
    "70B3D5FFD": "i2Systems",
    "8C1F64918": "Abbott Diagnostics Technologies AS",
    "8C1F64DBD": "GIORDANO CONTROLS SPA",
    "70B3D5450": "Apantac LLC",
    "70B3D5318": "Exemplar Medical, LLC",
    "70B3D52B6": "HLT Micro ",
    "70B3D54CB": "Cucos Retail Systems GmbH",
    "70B3D5AFD": "dongsheng",
    "70B3D571F": "Grayshift",
    "70B3D5A63": "DesignA Electronics Limited",
    "70B3D531E": "GILLAM-FEI S.A.",
    "70B3D5CFA": "SCHEIBER",
    "70B3D5ED6": "Metrasens Limited",
    "70B3D5451": "Perform3-D LLC",
    "70B3D56D4": "Telerob Gesellschaft f\u00fcr Fernhantierungs",
    "70B3D53B3": "Movicom Electric LLC",
    "70B3D5312": "SMITEC S.p.A.",
    "70B3D5690": "Sicon srl",
    "70B3D5A1E": "Monnit Corporation",
    "70B3D5681": "DEUTA-WERKE GmbH",
    "70B3D5604": "Foxtrot Research Corp",
    "70B3D5B58": "INTERNET PROTOCOLO LOGICA SL",
    "70B3D5314": "Grau Elektronik GmbH",
    "70B3D5D2C": "microWerk GmbH",
    "70B3D546D": "Guan Show Technologe Co., Ltd.",
    "70B3D5390": "TEX COMPUTER SRL ",
    "70B3D544C": "ejoin, s.r.o.",
    "70B3D5FB4": "Array Technologies Inc.",
    "70B3D5AD0": "REO AG",
    "70B3D5520": "promedias AG",
    "70B3D5D83": "AKASAKATEC  INC.",
    "70B3D5B12": "VTEQ",
    "70B3D5D78": "Nxvi Microelectronics Technology (Jinan) Co., Ltd.",
    "001BC5030": "OctoGate IT Security Systems GmbH",
    "70B3D5887": "Entec Solar S.L.",
    "70B3D5527": "Procon Electronics Pty Ltd",
    "70B3D580E": "Utopi Ltd",
    "70B3D58E8": "PREO INDUSTRIES FAR EAST LTD",
    "70B3D5309": "ABS Applied Biometric Systems GmbH",
    "70B3D5BF0": "Alfa Elettronica srl",
    "70B3D5909": "tetronik GmbH AEN",
    "70B3D5AB1": "ISRV Zrt.",
    "70B3D58D1": "Field Design Inc.",
    "70B3D55F5": "Microvision",
    "70B3D5BBC": "Boundary Technologies Ltd",
    "70B3D5B3F": "Orbit International",
    "70B3D5624": "EBE Mobility & Green Energy GmbH",
    "70B3D5D17": "Power Element ",
    "70B3D56C4": "Veo Robotics, Inc.",
    "70B3D5CBF": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "70B3D5ED4": "WILMORE ELECTRONICS COMPANY",
    "70B3D5B61": "WuXi anktech Co., Ltd ",
    "70B3D5C57": "eBZ GmbH",
    "70B3D5ED2": "PCTEL, Inc.",
    "70B3D5CDD": "Teneo IoT B.V.",
    "70B3D5380": "SeaTech Intelligent Technology (Shanghai) Co., LTD",
    "70B3D53C8": "ABC Electric Co.",
    "70B3D58FB": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5962": "Senquire Pte. Ltd",
    "70B3D527B": "DAVE SRL",
    "70B3D56EF": "Beringar",
    "70B3D5247": "Satsky Communication Equipment Co.,Ltd.",
    "70B3D5483": "LITUM BILGI TEKNOLOJILERI SAN. VE TIC. A.S.",
    "70B3D57D8": "Nuand LLC",
    "70B3D5B71": "Optiver Pty Ltd",
    "70B3D59A3": "Shanghai Hourui Technology Co., Ltd.",
    "70B3D5619": "ZAO ZEO",
    "70B3D5573": "GEGA ELECTRONIQUE",
    "70B3D5FA4": "Energybox Limited",
    "70B3D59BB": "Jinga-hi, Inc.",
    "70B3D54D7": "Technological Ray GmbH",
    "70B3D5376": "Magenta Labs, Inc.",
    "70B3D5738": "GRYPHON SECURE INC",
    "70B3D5622": "PCS Inc.",
    "70B3D50F5": "Season Electronics Ltd",
    "70B3D5A03": "Proemion GmbH",
    "70B3D5B7B": "Doosan Digital Innovation America",
    "70B3D568A": "Advanced Telecommunications Research Institute International",
    "70B3D5E11": "Engage Technologies",
    "70B3D5386": "GPSat Systems",
    "70B3D549C": "AC Power Corp.",
    "70B3D5416": "Antlia Systems",
    "70B3D5055": "BAE SYSTEMS ",
    "70B3D5D0F": "Alto Aviation",
    "70B3D55A1": "BOE Technology Group Co., Ltd.",
    "70B3D5992": "KAEONIT",
    "70B3D5B54": "Packet Power",
    "70B3D5A23": "LG Electronics",
    "70B3D5EDA": "Breas Medical AB",
    "70B3D5536": "LARIMART SPA",
    "70B3D56AC": "Ketronixs Sdn Bhd",
    "70B3D5762": "Transformational Security, LLC",
    "70B3D5D19": "Senior Group LLC",
    "70B3D59C2": "Sportsbeams Lighting, Inc.",
    "70B3D594C": "Honeywell/Intelligrated",
    "70B3D5B90": "Amico Corporation",
    "70B3D55C9": "ICTK Holdings",
    "70B3D5846": "National Time & Signal Corp.",
    "70B3D57D4": "Computechnic AG",
    "70B3D570C": "Potter Electric Signal Co. LLC",
    "70B3D5D7D": "BESO sp. z o.o.",
    "70B3D5165": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D5AD9": "aelettronica group srl",
    "70B3D582A": "C W F Hamilton & Co Ltd",
    "70B3D54E3": "adnexo GmbH",
    "70B3D5C23": "Sumitomo Heavy Industries, Ltd.",
    "70B3D573F": "LLC Open Converged Networks",
    "70B3D546A": "Shenzhen Vikings Technology Co., Ltd.",
    "70B3D5C36": "Knowledge Resources GmbH",
    "70B3D510D": "CoreEL Technologies Pvt Ltd",
    "70B3D55AC": "LM-Instruments Oy",
    "70B3D5DBB": "Fuhr GmbH Filtertechnik",
    "70B3D506D": "Panoramic Power",
    "70B3D5669": "Panoramic Power",
    "70B3D5C18": "Sanmina Israel",
    "70B3D52C1": "Avlinkpro",
    "70B3D5683": "DECYBEN",
    "70B3D55C6": "C4I Systems Ltd",
    "70B3D5A09": "Smart Embedded Systems",
    "70B3D5F6A": "Guan Show Technologe Co., Ltd.",
    "70B3D5E24": "Gogo Business Aviation",
    "70B3D550F": "LLC Sarov Innovative Technologies (WIZOLUTION)",
    "70B3D5A77": "SPX Radiodetection",
    "70B3D5AD4": "INVISSYS",
    "70B3D525C": "ARCLAN'SYSTEM",
    "70B3D518A": "NSP Europe Ltd",
    "70B3D5C98": "Trust Automation",
    "70B3D577F": "Microchip Technology Germany II GmbH&Co.KG",
    "70B3D5F64": "silicom",
    "70B3D5466": "SYLink Technologie",
    "70B3D5249": "Kospel S.A.",
    "70B3D5EEB": "shenzhen suofeixiang technology Co.,Ltd",
    "70B3D5893": "Cubitech",
    "70B3D5E12": "SNK, Inc.",
    "70B3D5699": "Flextronics International Kft",
    "70B3D54D3": "Hefei STAROT Technology Co.,Ltd",
    "70B3D5117": "SysCom Automationstechnik GmbH",
    "70B3D55F7": "JFA Electronics Industry and Commerce EIRELI",
    "70B3D5858": "Hubbell Power Systems",
    "70B3D5467": "GreenWake Technologies",
    "70B3D5940": "Paradigm Technology Services B.V.",
    "70B3D5E5C": "Walton Hi-Tech Industries Ltd.",
    "70B3D5C82": "Sicon srl",
    "70B3D5C31": "German Power GmbH",
    "70B3D5335": "Jonsa Australia Pty Ltd",
    "70B3D5DAE": "LGE",
    "70B3D57DB": "aquila biolabs GmbH",
    "70B3D53EC": "Outsight SA",
    "70B3D5F23": "Lyse AS",
    "70B3D597E": "Public Joint Stock Company Morion",
    "70B3D5964": "Visility",
    "70B3D5B9D": "Conclusive Engineering",
    "70B3D52C4": "Hodwa Co., Ltd",
    "70B3D5DAC": "Dalian Laike Technology Development Co., Ltd ",
    "70B3D5CFB": "Screen Innovations",
    "70B3D59C3": "Sevensense Robotics AG",
    "70B3D5865": "Insitu, Inc.",
    "70B3D5425": "SinterCast",
    "70B3D5487": "ECS s.r.l.",
    "70B3D5C3A": "HAN CHANG",
    "70B3D5575": "Konrad GmbH",
    "70B3D54E4": "W.A. Benjamin Electric Co.",
    "70B3D5617": "Cominfo, Inc.",
    "70B3D54C9": "Elsist Srl",
    "70B3D56EE": "HANKOOK CTEC CO,. LTD.",
    "70B3D548B": "TATTILE SRL",
    "70B3D5A41": "THELIGHT Luminary for Cine and TV S.L.",
    "70B3D52B8": "WideNorth AS",
    "70B3D5CEF": "Ellego Powertec Oy",
    "70B3D574A": "Mettler Toledo",
    "70B3D5D64": "Mettler Toledo",
    "70B3D58B8": "GDI Technology Inc ",
    "70B3D512A": "Elvys s.r.o",
    "70B3D5AB6": "SmartD Technologies Inc",
    "70B3D5EE2": "MONTRADE SPA",
    "70B3D584F": "Mettler Toledo",
    "70B3D500B": "AXING AG",
    "70B3D5EFD": "Cambridge Technology, Inc.",
    "70B3D5CC4": "Benchmark Electronics BV",
    "70B3D5B25": "Hifocus Electronics India Private Limited",
    "70B3D50CB": "NIRECO CORPORATION",
    "70B3D5B87": "CAITRON GmbH",
    "70B3D51D2": "Xacti Corporation",
    "70B3D5DBD": "TRANSLITE GLOBAL LLC",
    "70B3D547B": "Monixo",
    "70B3D55B4": "Systems Technologies",
    "70B3D59E8": "Zerospace ICT Services B.V.",
    "70B3D506E": "GLOBAL-KING INTERNATIONAL CO., LTD.",
    "70B3D56DD": "Abbott Diagnostics Technologies AS",
    "70B3D5A8A": "JSC VIST Group ",
    "70B3D50D5": "Kahler Automation",
    "70B3D59E1": "Bolide Technology Group, Inc.",
    "70B3D545A": "Palarum LLC",
    "70B3D5B5B": "DynaMount LLC",
    "70B3D5AED": "Cubitech",
    "70B3D5BF8": "RCH ITALIA SPA ",
    "70B3D541F": "Orion S.r.l.",
    "70B3D5ECC": "Digifocus Technology Inc.",
    "70B3D54F7": "Foxtel srl",
    "70B3D5320": "CYNIX Systems Inc",
    "70B3D542B": "Guangzhou Haoxiang Computer Technology Co.,Ltd.",
    "70B3D5830": "Nordson Corporation",
    "70B3D5053": "YAMAKATSU ELECTRONICS INDUSTRY CO., LTD.",
    "70B3D5206": "ard sa",
    "70B3D57CA": "Hunan Shengyun Photoelectric Technology Co., Ltd.",
    "70B3D5F44": "Magneti Marelli S.p.A. Electronics",
    "70B3D5B84": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D5E06": "System West dba ICS Electronics",
    "70B3D5218": "Gremesh.com",
    "70B3D5358": "Nevotek",
    "70B3D5D02": "Arctos Showlasertechnik GmbH",
    "70B3D5BC0": "SENSO2ME ",
    "70B3D5C59": "R Cubed Engineering, LLC",
    "70B3D58EA": "JLCooper Electronics",
    "70B3D5DC1": "Metralight, Inc.",
    "70B3D5A70": "Gateview Technologies",
    "70B3D5521": "Selex ES Inc.",
    "70B3D583E": "The Dini Group, La Jolla inc.",
    "70B3D5B65": "Rotem Industry LTD",
    "70B3D54D6": "Operational Technology Solutions",
    "70B3D5337": "Laborie",
    "70B3D54A4": "DEUTA-WERKE GmbH",
    "70B3D5BE7": "Syscom Instruments SA",
    "70B3D5093": "Legrand Electric Ltd",
    "70B3D5D49": "Sicon srl",
    "70B3D5068": "Onethinx BV",
    "70B3D56FE": "NTO IRE-POLUS",
    "70B3D5CD7": "AutomationX GmbH",
    "70B3D57A2": "Alpha ESS Co., Ltd.",
    "70B3D5DFE": "microtec Sicherheitstechnik GmbH",
    "70B3D5D8A": "JIANGSU HORAINTEL CO.,LTD",
    "70B3D54CC": "FRESENIUS MEDICAL CARE",
    "70B3D531D": "AVA Monitoring AB",
    "70B3D505B": "PAL Inc.",
    "70B3D55A6": "TimeMachines Inc.",
    "70B3D5EAD": "Cobo, Inc.",
    "70B3D56F1": "Discover Battery",
    "70B3D5BA8": "Controlled Power Company",
    "70B3D579D": "Editech Co., Ltd",
    "70B3D59B7": "Itronics Ltd",
    "70B3D5FDC": "Tapdn",
    "70B3D5DBC": "Gamber Johnson-LLC",
    "70B3D5C08": "Talleres de Escoriaza SA",
    "70B3D5AEC": "Paratec Ltd.",
    "70B3D561B": "Nubewell Networks Pvt Ltd",
    "70B3D5788": "Slan",
    "70B3D5603": "EGISTECH CO.,LTD.",
    "70B3D5533": "Nippon Marine Enterprises, Ltd.",
    "70B3D59B9": "Aethera Technologies",
    "70B3D5FED": "Niron systems & Projects",
    "70B3D584B": "QuestHouse, Inc.",
    "70B3D5D36": "Insitu Inc.",
    "001BC50B8": "Private",
    "70B3D59FD": "amakidenki",
    "70B3D5CDC": "Dat-Con d.o.o.",
    "70B3D5A2D": "Project Service S.r.l.",
    "70B3D5A9B": "OSMOZIS",
    "70B3D5239": "Applied Silver",
    "70B3D5D00": "DKI Technology Co., Ltd",
    "70B3D5447": "Avid Controls Inc",
    "70B3D5FD0": "Alcohol Countermeasure Systems",
    "70B3D5DBE": "Hiber",
    "70B3D5020": "MICRO DEBUG, Y.K.",
    "70B3D59D6": "Crown Solar Power Fencing Systems",
    "70B3D5AA5": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5E8C": "Fracarro srl",
    "70B3D51EE": "MEGGITT",
    "70B3D52E9": "NeurIT s.r.o.",
    "70B3D5A75": "Taejin InfoTech",
    "70B3D5DAB": "SET Power Systems GmbH",
    "70B3D5A7D": "Prior Scientific Instruments Ltd",
    "70B3D51D8": "Blue Skies Global LLC",
    "70B3D5098": "Alcodex Technologies Private Limited",
    "70B3D559B": "AUTOMATIZACION Y CONECTIVIDAD SA DE CV",
    "70B3D522A": "Shishido Electrostatic, Ltd.",
    "70B3D5BC4": "Digital Media Professionals",
    "70B3D5072": "Lightdrop",
    "70B3D5802": "Qingdao CNR HITACH Railway Signal&communication co.,ltd",
    "70B3D5BB0": "WICELL TECHNOLOGY",
    "70B3D545B": "KOMZ - IZMERENIYA",
    "70B3D5B38": "GoTrustID Inc.",
    "70B3D5E0C": "Communication Systems Solutions",
    "001BC5015": "Corporate Systems Engineering ",
    "70B3D5B10": "Zumbach Electronic AG",
    "70B3D5C4C": "VTC Digicom",
    "70B3D515B": "Armstrong International, Inc.",
    "70B3D5BD6": "Consarc Corporation",
    "70B3D505C": "Amber Kinetics Inc",
    "70B3D58DA": "MicroElectronics System Co.Ltd",
    "70B3D5AE6": "Ya Batho Trading (Pty) Ltd",
    "70B3D5338": "Opti-Sciences, Inc.",
    "70B3D5B66": "Silent Gliss International Ltd",
    "70B3D5977": "Engage Technologies",
    "70B3D5E84": "ENTEC Electric & Electronic Co., LTD.",
    "70B3D5759": "AML",
    "70B3D5F27": "NIRIT- Xinwei  Telecom Technology Co., Ltd.",
    "70B3D58D7": "Schneider Electric Motion USA",
    "70B3D59DF": "DOBE Computing",
    "70B3D5673": "ACD Elekronik GmbH",
    "70B3D576C": "Aural Ltd",
    "70B3D5516": "LINEAGE POWER PVT LTD.,",
    "70B3D5441": "Videoport S.A.",
    "70B3D5402": "AKIS technologies",
    "70B3D5978": "Satixfy Israel Ltd.",
    "70B3D5B49": "ANALOGICS TECH INDIA LTD",
    "70B3D51EA": "Sense For Innovation",
    "70B3D53A1": "Reckeen HDP Media sp. z o.o. sp. k.",
    "70B3D5B4D": "Avidbots Corporation",
    "70B3D5FDD": "Laser Imagineering Vertriebs GmbH",
    "70B3D5764": "SCHMID electronic",
    "70B3D5680": "BASF Corporation",
    "70B3D553D": "ACCEL CORP",
    "70B3D5F84": "DEUTA-WERKE GmbH",
    "70B3D52B2": "Sun Creative (ZheJiang) Technology INC.",
    "70B3D5636": "Globalcom Engineering SPA",
    "70B3D5235": "CAMEON S.A.",
    "70B3D5DA5": "Roboteq",
    "70B3D556A": "Harvard Technology Ltd",
    "70B3D5280": "Computech International",
    "70B3D59AB": "Groupe Paris-Turf",
    "001BC5032": "Osborne Coinage Co",
    "001BC50B6": "VEILUX INC.",
    "70B3D585F": "YUYAMA MFG Co.,Ltd",
    "70B3D5C05": "KST technology",
    "70B3D56F7": "EGICON SRL",
    "70B3D5706": "Smith Meter, Inc.",
    "70B3D5D2E": "Coheros Oy",
    "70B3D526C": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D5876": "IONETECH",
    "70B3D506F": "Beijing Daswell Science and Technology Co.LTD",
    "70B3D5161": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5EEC": "Impolux GmbH",
    "70B3D5DA6": "Redfish Group Pty Ltd",
    "70B3D5F14": "SANYU SWITCH CO., LTD.",
    "70B3D5B4A": "MEDEX",
    "70B3D5105": "Beijing Nacao Technology Co., Ltd.",
    "70B3D5CE2": "Centero",
    "70B3D5B1E": "Fen Systems Ltd",
    "70B3D5770": "STREGA ",
    "70B3D5985": "Burk Technology",
    "70B3D5FEA": "Heng Dian Technology Co., Ltd",
    "70B3D587D": "INVIXIUM ACCESS INC.",
    "70B3D55F2": "Invisible Systems Limited",
    "70B3D5240": "Orlaco Products B.V.",
    "70B3D5B7F": "JSK System",
    "70B3D5DC8": "Enertex Bayern GmbH",
    "70B3D5073": "Liteon Technology Corporation",
    "70B3D5EBC": "Refine Technology, LLC",
    "70B3D5455": "Heartlandmicropayments",
    "70B3D564B": "Kalfire",
    "70B3D5DB1": "Biovigil Hygiene Technologies",
    "70B3D5790": "AVI Pty Ltd",
    "70B3D5154": "Walk Horizon Technology (Beijing) Co., Ltd.",
    "70B3D56C1": "R.A.I.T.88 Srl",
    "70B3D5532": "Talleres de Escoriaza SA",
    "70B3D5728": "BCD Audio",
    "70B3D5F19": "Vitro Technology Corporation",
    "70B3D5F10": "Riegl Laser Measurement Systems GmbH",
    "70B3D51A6": "Robotelf Technologies (Chengdu) Co., Ltd.",
    "70B3D5141": "M.T. S.R.L.",
    "70B3D5DF1": "CoXlab Inc.",
    "70B3D5739": "Zigencorp, Inc",
    "70B3D5FF0": "E-MetroTel",
    "70B3D593E": "Systems With Intelligence Inc.",
    "70B3D5AD8": "Euklis by GSG International",
    "70B3D5469": "Gentec Systems  Co.",
    "70B3D50D4": "Guangzhou Male Industrial Animation Technology Co.,Ltd.",
    "70B3D523A": "Mesa Labs, Inc.",
    "70B3D564E": "BigStuff3, Inc. ",
    "70B3D5EF5": "DEUTA-WERKE GmbH",
    "70B3D5253": "Wimate Technology Solutions Private Limited",
    "70B3D5D5F": "Core Balance Co., Ltd.",
    "70B3D51CD": "ELEUSI GmbH",
    "001BC50C2": "TechSolutions A/S",
    "70B3D5024": "G+D Mobile Security",
    "70B3D5DAA": "AmTote Australasia",
    "70B3D5904": "PHB Eletronica Ltda.",
    "70B3D5213": "ETON Deutschland Electro Acoustic GmbH",
    "70B3D555C": "Saratoga Speed, Inc.",
    "70B3D5C8F": "TRIDENT INFOSOL PVT LTD",
    "70B3D55DA": "Valk Welding B.V.",
    "70B3D5B64": "OSUNG LST CO.,LTD.",
    "70B3D50F8": "Special Services Group, LLC",
    "70B3D524F": "ELBIT SYSTEMS BMD AND LAND EW - ELISRA LTD",
    "70B3D5AC1": "AEM Singapore Pte. Ltd.",
    "70B3D5E69": "Fire4 Systems UK Ltd",
    "70B3D57EA": "Waterkotte GmbH",
    "70B3D53B5": "Preston Industries dba PolyScience",
    "70B3D56E9": "Krontech",
    "70B3D52B7": "Matrix Orbital Corporation",
    "70B3D59E6": "BLOCKSI LLC",
    "70B3D57BF": "Stone Three",
    "70B3D5E3A": "Cyanview",
    "70B3D5075": "Mo-Sys Engineering Ltd",
    "70B3D5FA3": "ELVA-1 MICROWAVE HANDELSBOLAG",
    "70B3D5743": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D5689": "Prisma Telecom Testing Srl",
    "70B3D58C8": "KRONOTECH SRL",
    "70B3D5096": "HAVELSAN A.\u015e.",
    "70B3D5963": "Triax A/S",
    "70B3D5351": "KST technology",
    "70B3D56FD": "Core Ak\u0131ll\u0131 Ev Sistemleri",
    "70B3D52CC": "WeWork Companies, Inc.",
    "70B3D560D": "Link Electric & Safety Control Co.",
    "70B3D5723": "LG Electronics",
    "70B3D5B3A": "Adigitalmedia",
    "70B3D581B": "bobz GmbH",
    "70B3D5B6C": "GHM-Messtechnik GmbH (Standort IMTRON)",
    "70B3D5D41": "KSE GmbH",
    "70B3D5EA6": "Galios",
    "70B3D5C4D": "RADA Electronics Industries Ltd.",
    "70B3D5E88": "Breas Medical AB",
    "70B3D595C": "Wilson Electronics",
    "70B3D5C6C": "McQ Inc",
    "70B3D5760": "QUALITTEQ LLC",
    "70B3D5828": "Xacti Corporation",
    "70B3D5705": "Digital Matter Pty Ltd",
    "70B3D52BD": "mg-sensor GmbH",
    "70B3D59B5": "Ideetron b.v.",
    "70B3D58F8": "Wi6labs",
    "70B3D5E3F": "BESTCODE LLC",
    "70B3D592B": "ENTEC Electric & Electronic Co., LTD.",
    "70B3D5894": "UnI Systech Co.,Ltd",
    "70B3D5DE6": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5727": "LP Technologies Inc.",
    "70B3D52F2": "Health Care Originals, Inc.",
    "70B3D5133": "Vidisys GmbH",
    "70B3D51E6": "Sanmina Israel",
    "70B3D5826": "Elbit Systems of America",
    "70B3D52AB": "NASA Johnson Space Center",
    "70B3D5EB9": "Thiel Audio Products Company, LLC",
    "70B3D56E8": "Blu Wireless Technology Ltd",
    "70B3D5844": "SANSFIL Technologies",
    "70B3D5254": "Spectrum Brands",
    "70B3D54A5": "Intermind Inc.",
    "70B3D5D98": "ACD Elekronik GmbH",
    "70B3D5FF8": "Dutile, Glines and Higgins Corporation",
    "70B3D558C": "OPTSYS",
    "70B3D55BF": "Aton srl",
    "70B3D56AD": "CONNIT",
    "70B3D5C33": "Dandong Dongfang Measurement & Control Technology Co., Ltd.",
    "70B3D50D3": "TSAT AS",
    "70B3D5CC8": "PROFEN COMMUNICATIONS",
    "70B3D5125": "Securolytics, Inc.",
    "70B3D5D4C": "Elystec Technology Co., Ltd",
    "70B3D57F3": "Shenzhen Virtual Clusters Information Technology Co.,Ltd.",
    "70B3D5C79": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5D69": "Thermo Fisher Scientific",
    "70B3D5C20": "Mipot S.p.a.",
    "70B3D556B": "S.E.I. CO.,LTD.",
    "70B3D5807": "Camsat Przemys\u0142aw Gralak",
    "70B3D50A0": "Cominfo, Inc.",
    "70B3D52D0": "ijin co.,ltd.",
    "70B3D5EE3": "Lithe Technology, LLC",
    "70B3D5C17": "Potter Electric Signal Co. LLC",
    "70B3D5763": "A Trap, USA",
    "70B3D5F25": "JSC \u201cScientific Industrial Enterprise \"Rubin\"",
    "70B3D59DC": "Shanghai Daorech Industry Developmnet Co.,Ltd",
    "70B3D5967": "TATTILE SRL",
    "70B3D5DA4": "CRDE",
    "70B3D5F0C": "ModulaTeam GmbH",
    "70B3D5E59": "Fracarro srl",
    "70B3D5AA3": "LINEAGE POWER PVT LTD.,",
    "70B3D59C1": "Zeroplus Technology Co.,Ltd.",
    "70B3D5D8E": "Axatel SrL",
    "70B3D5BF6": "comtac AG",
    "70B3D502E": "Monnit Corporation",
    "70B3D53F3": "SPEA SPA",
    "70B3D5138": "SMITEC S.p.A.",
    "70B3D50B0": "Raven Systems Design, Inc",
    "70B3D5C74": "Qtechnology A/S",
    "70B3D5F37": "Mitsubishi Electric Micro-Computer Application Software Co.,Ltd. ",
    "70B3D5FFC": "Symetrics Industries d.b.a. Extant Aerospace",
    "70B3D5630": "LGE",
    "70B3D5D08": "Veeco Instruments",
    "70B3D598C": "University of Wisconsin Madison - Department of High Energy Physics",
    "70B3D57A1": "Excelfore Corporation",
    "70B3D51DE": "DYCEC, S.A.",
    "70B3D585E": "XLOGIC srl",
    "70B3D5FA7": "Nordson Corporation",
    "70B3D5902": "Unlimiterhear co.,ltd. taiwan branch",
    "70B3D5880": "Skopei B.V.",
    "70B3D5C2E": "Triax A/S",
    "70B3D560B": "Edgeware AB",
    "70B3D59F6": "Edgeware AB",
    "70B3D5173": "National TeleConsultants LLC",
    "70B3D5039": "DoWoo Digitech",
    "70B3D5F83": "Tata Communications Ltd.",
    "70B3D55E5": "HAIYANG OLIX CO.,LTD.",
    "70B3D50A1": "PTN Electronics Limited ",
    "70B3D5FAF": "Radig Hard & Software",
    "70B3D54CF": "GREEN HOUSE CO., LTD.",
    "70B3D55CA": "ACD Elekronik GmbH",
    "70B3D50EF": "Dextera Labs",
    "70B3D537C": "Merus Power Dynamics Ltd.",
    "70B3D520D": "Engage Technologies",
    "70B3D5D55": "WM Design s.r.o",
    "70B3D5D25": "ENGenesis ",
    "70B3D5E67": "APPLIED PROCESSING",
    "70B3D5267": "Zehntner Testing Instruments",
    "70B3D5E98": "JSC Kaluga Astral",
    "70B3D5149": "eleven-x",
    "70B3D5542": "RTDS Technologies Inc.",
    "70B3D5952": "REQUEA",
    "70B3D516F": "NimbeLink Corp",
    "70B3D5188": "Birket Engineering",
    "70B3D511F": "Geppetto Electronics",
    "70B3D55C1": "Shanghai JaWay Information Technology Co., Ltd.",
    "70B3D5BE9": "Telecast Inc.",
    "70B3D5563": "Zhejiang Hao Teng Electronic Technology Co., Ltd.",
    "70B3D58C2": "F-domain corporation",
    "70B3D55D5": "CT Company",
    "70B3D5462": "EarTex",
    "70B3D56D0": "Code Blue Corporation ",
    "70B3D5850": "REO AG",
    "70B3D5597": "VAPE RAIL INTERNATIONAL",
    "70B3D581E": "Novathings",
    "70B3D5C0F": "Honeywell Safety Products USA, Inc",
    "70B3D50E5": "Delta Solutions LLC",
    "70B3D504B": "Dream I System Co., Ltd",
    "70B3D57E1": "Applied Materials",
    "70B3D5C3E": "DOSADORES ALLTRONIC",
    "70B3D5BCC": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5B77": "Motec Pty Ltd",
    "70B3D5AB0": "OSR R&D ISRAEL LTD",
    "70B3D5ACD": "CRDE",
    "70B3D52AC": "New Imaging Technologies",
    "70B3D5833": "Alpiq InTec Management AG",
    "70B3D51DB": "Hudson Robotics",
    "70B3D509F": "COMTECH Kft.",
    "70B3D5593": "Asis Pro",
    "70B3D5CCF": "Netberg",
    "70B3D5012": "KST technology",
    "70B3D519E": "J-Factor Embedded Technologies",
    "70B3D51EF": "ADTEK",
    "70B3D536A": "Becton Dickinson",
    "70B3D5AAA": "Xemex NV",
    "70B3D5F35": "carbonTRACK",
    "70B3D5910": "Eginity, Inc.",
    "70B3D57D1": "Schneider Electric Motion USA",
    "70B3D52B9": "BELECTRIC GmbH",
    "70B3D56A8": "Vitsch Electronics",
    "70B3D59B6": "Intercomp S.p.A.",
    "70B3D5B9E": "POLSYSTEM SI SP. Z O.O., S.K.A.",
    "70B3D5CE5": "GridBridge Inc",
    "70B3D5A21": "PPI Inc.",
    "70B3D5AAD": "Bartec GmbH",
    "70B3D5CD2": "TRUMPF Huttinger GmbH + Co. KG,",
    "70B3D55AB": "Sea Air and Land Communications Ltd",
    "70B3D5896": "Shanghai Longpal Communication Equipment Co., Ltd.",
    "70B3D58B3": "Firefly RFID Solutions",
    "70B3D5E0F": "Vtron Pty Ltd",
    "70B3D585B": "TSUBAKIMOTO CHAIN CO.",
    "8C1F64398": "Software Systems Plus",
    "70B3D5C65": "PEEK TRAFFIC",
    "70B3D512C": "CIELLE S.R.L.",
    "70B3D5885": "QuirkLogic",
    "70B3D512E": "GreenFlux",
    "70B3D5F1D": "Critical Link LLC",
    "70B3D5A28": "PEEK TRAFFIC",
    "70B3D5349": "SLAT",
    "70B3D5761": "Critical Link LLC",
    "70B3D54C5": "Moving iMage Technologies LLC",
    "70B3D59FA": "Ideas srl",
    "70B3D5FCA": "M2M Cybernetics Pvt Ltd",
    "70B3D5650": "GIFAS-ELECTRIC GmbH",
    "70B3D575B": "Netool LLC",
    "70B3D511C": "Samriddi Automations Pvt. Ltd.",
    "70B3D51DA": "Promess Inc.",
    "70B3D555B": "Procon Electronics Pty Ltd",
    "70B3D5147": "ROMO Wind A/S",
    "70B3D5721": "Zoe Medical",
    "70B3D5906": "Aplex Technology Inc.",
    "70B3D50D2": "UNMANNED SPA",
    "70B3D565A": "Aplex Technology Inc.",
    "70B3D57CE": "Aplex Technology Inc.",
    "70B3D5C91": "Grossenbacher Systeme AG",
    "70B3D56B5": "ART SPA",
    "70B3D5396": "CTG sp. z o. o.",
    "70B3D5297": "Grossenbacher Systeme AG",
    "70B3D5387": "GWF MessSysteme AG",
    "70B3D5DFF": "Spanawave Corporation",
    "70B3D54FE": "WiTagg, Inc",
    "70B3D5367": "Living Water",
    "70B3D5B72": "UB330.net d.o.o.",
    "70B3D5776": "Power Ltd.",
    "70B3D57DE": "Telaeris, Inc.",
    "70B3D53A5": "KMtronic ltd",
    "70B3D50BD": "Andium",
    "70B3D510A": "SEASON DESIGN TECHNOLOGY",
    "70B3D5E7E": "Groupe Citypassenger Inc",
    "70B3D5277": "Voltaware Limited",
    "70B3D574F": "United States Technologies Inc.",
    "70B3D547C": "Par-Tech, Inc.",
    "70B3D58EF": "Beeper Communications Ltd.",
    "70B3D58C5": "HMicro Inc",
    "70B3D53F6": "Sycomp Electronic GmbH",
    "70B3D5ADE": "ISAC SRL",
    "70B3D5D2B": "StreamPlay Oy Ltd",
    "70B3D5493": "Impulse Networks Pte Ltd",
    "70B3D5FE4": "CARE PVT LTD",
    "70B3D5544": "Silicon Safe Ltd",
    "70B3D5FEB": "Les distributions Multi-Secure incorporee",
    "70B3D5429": "Redco Audio Inc",
    "70B3D5202": "DEUTA-WERKE GmbH",
    "70B3D50B8": "Lucas-N\u00fclle GmbH",
    "70B3D5592": "CRDE",
    "70B3D5B93": "INTERNET PROTOCOLO LOGICA SL",
    "70B3D5895": "Integrated Control Corp.",
    "70B3D514D": "2-Observe",
    "70B3D5216": "FLEXTRONICS",
    "70B3D5955": "Dynacard Co., Ltd.",
    "70B3D56A1": "GLIAL TECHNOLOGY",
    "70B3D512B": "RIC Electronics",
    "70B3D50FE": "Vocality International Ltd",
    "70B3D53CA": "TTI Ltd",
    "70B3D5F56": "VirtualHere Pty. Ltd.",
    "70B3D55BE": "CASWA",
    "70B3D513C": "Detec Systems Ltd",
    "70B3D57F8": "Solvera Lynx d.d.",
    "70B3D52FA": "Toray\u3000Medical\u3000Co.,Ltd",
    "70B3D54C4": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D5E45": "Momentum Data Systems",
    "70B3D5EB7": "Skreens",
    "70B3D5892": "ABB",
    "70B3D5EA2": "Transportal Solutions Ltd",
    "70B3D5148": "Power Electronics Espana, S.L.",
    "70B3D5A2E": "Kokam Co., Ltd",
    "70B3D5F5B": "A.F.MENSAH, INC",
    "70B3D5353": "Digital Outfit",
    "70B3D5F6D": "Qowisio",
    "70B3D5A72": "Business Marketers Group, Inc.",
    "70B3D5AEA": "BBR Verkehrstechnik GmbH",
    "70B3D5551": "infrachip",
    "70B3D50D6": "TATTILE SRL",
    "70B3D5B7A": "MAHLE",
    "70B3D56C7": "Becton Dickinson",
    "70B3D5913": "Shenzhen Riitek Technology Co.,Ltd",
    "70B3D5654": "EMAC, Inc.",
    "70B3D5CF4": "Harbin Cheng Tian Technology Development Co., Ltd.",
    "70B3D5CB6": "Kuebrich Ingeniergesellschaft mbh & Co. KG",
    "70B3D5426": "Zehnder Group Nederland",
    "70B3D5D70": "Rational Production srl Unipersonale",
    "70B3D5CC1": "BEEcube Inc.",
    "70B3D5522": "Syncopated Engineering Inc",
    "70B3D54B9": "SHEN ZHEN TTK TECHNOLOGY CO,LTD",
    "70B3D5B7C": "Electronic Navigation Ltd",
    "70B3D5AF9": "Critical Link LLC",
    "70B3D5D5A": "WyreStorm Technologies Ltd",
    "70B3D5D5B": "WyreStorm Technologies Ltd",
    "70B3D583C": "Sinoembed",
    "70B3D56CD": "NORTHBOUND NETWORKS PTY. LTD.",
    "70B3D5C92": "Unitro Fleischmann",
    "70B3D5F57": "Aplex Technology Inc.",
    "70B3D5140": "Virta Laboratories, Inc.",
    "70B3D518D": "Foro Tel",
    "70B3D501F": "SPX Flow Technology BV",
    "70B3D5CA9": "Nxcontrol system Co., Ltd.",
    "70B3D513B": "Sienna Corporation",
    "70B3D5377": "Monnit Corporation",
    "70B3D534C": "GLT Exports Ltd",
    "70B3D5EDF": "GridNavigator",
    "70B3D5DCA": "DSan Corporation",
    "70B3D5C07": "ARECO",
    "70B3D5328": "HIPODROMO DE AGUA CALIENTE SA CV",
    "70B3D5517": "ISPHER",
    "70B3D5383": "LPA Excil Electronics",
    "70B3D5061": "IntelliDesign Pty Ltd",
    "70B3D5891": "neocontrol solu\u00e7\u00f5es em automa\u00e7\u00e3o",
    "70B3D58F0": "ERAESEEDS co.,ltd.",
    "70B3D5D29": "Sportzcast",
    "70B3D57F2": "TCI",
    "70B3D5ECA": "Transtronic AB",
    "70B3D5BE5": "Pantec Engineering AG",
    "70B3D552D": "Tanaka Electric Industry Co., Ltd.",
    "70B3D5E4B": "DELTA",
    "70B3D51AD": "Techworld Industries Ltd",
    "70B3D5E6D": "Domus S.C.",
    "70B3D5088": "OptiScan Biomedical Corp.",
    "70B3D5903": "Cymtec Ltd",
    "70B3D576D": "Trimble",
    "70B3D5FBB": "Vena Engineering Corporation",
    "70B3D5F54": "Revolution Retail Systems",
    "70B3D59F2": "Acorde Technologies",
    "70B3D503F": "Elesar Limited",
    "70B3D51E5": "VendNovation LLC",
    "70B3D5087": "Tempus Fugit Consoles bvba",
    "70B3D53D7": "Remote Sensing Solutions, Inc.",
    "70B3D5F73": "ASL Holdings",
    "70B3D572D": "Kron Medidores",
    "70B3D545F": "Cloud4Wi",
    "70B3D5F01": "Software Systems Plus",
    "70B3D50E6": "Nasdaq",
    "70B3D5E55": "BELT S.r.l.",
    "70B3D5B47": "DSIT Solutions LTD",
    "70B3D5789": "SEMEX-EngCon GmbH",
    "70B3D52F1": "Inspike S.R.L.",
    "70B3D5208": "DSP DESIGN LTD",
    "70B3D50DF": "B.E.A. sa",
    "70B3D5D0A": "Private",
    "70B3D5146": "3City Electronics",
    "70B3D5823": "SP Controls",
    "70B3D5435": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D5D9C": "Subinitial LLC",
    "70B3D5C97": "CSINFOTEL",
    "70B3D5B78": "HOERMANN GmbH",
    "70B3D5459": "Protium Technologies, Inc.",
    "70B3D5C55": "Intelligent Energy Ltd",
    "70B3D591E": "Creotech Instruments S.A.",
    "70B3D5078": "OrbiWise SA",
    "70B3D573E": "Trident RFID Pty Ltd",
    "70B3D5A4B": "McKay Brothers LLC",
    "70B3D5C5B": "ACD Elektronik GmbH",
    "70B3D5EAB": "APEN GROUP SpA (VAT IT08767740155)",
    "70B3D5E4E": "Midfin Systems",
    "70B3D55A2": "Wallner Automation GmbH",
    "70B3D544E": "Solace Systems Inc.",
    "70B3D5CB8": "Verti Tecnologia",
    "70B3D5D48": "HEADROOM Broadcast GmbH",
    "70B3D50D7": "Russian Telecom Equipment Company",
    "70B3D5B29": "WiViCom Co., Ltd.",
    "70B3D527D": "Telenor Connexion AB",
    "70B3D5E90": "Getein Biotechnology Co.,ltd",
    "70B3D5B24": "Datasat Digital Entertainment",
    "70B3D5BEF": "Sensortech Systems Inc.",
    "70B3D53D2": "Imagine Inc.",
    "70B3D536C": "Sicon srl",
    "70B3D5181": "Task Sistemas",
    "70B3D5A99": "Bandelin electronic GmbH & Co. KG",
    "70B3D5732": "TOFWERK AG",
    "70B3D5820": "Becker Nachrichtentechnik GmbH",
    "70B3D507D": "PANORAMIC POWER",
    "70B3D5610": "POLVISION",
    "70B3D558F": "LSL systems",
    "70B3D5D3B": "NimbeLink Corp",
    "70B3D577C": "HUSTY M.Styczen J.Hupert Sp.J.",
    "70B3D5B8F": "Assembly Contracts Ltd",
    "70B3D5CE7": "June Automation Singapore Pte. Ltd.",
    "70B3D5E58": "Thurlby Thandar Instruments LTD",
    "70B3D5EC1": "Xafax Nederland bv",
    "70B3D5C43": "Future Skies",
    "70B3D5CFF": "DTECH Labs, Inc.",
    "70B3D5E28": "iotec GmbH",
    "70B3D5CF1": "LightDec GmbH & Co. KG",
    "70B3D534A": "PAVO TASARIM \u00dcRET\u0130M T\u0130C A.\u015e.",
    "70B3D595A": "Sigmann Elektronik GmbH",
    "70B3D5DF3": "SPC Bioclinicum",
    "70B3D5682": "Rosslare Enterprises Limited",
    "70B3D5250": "Datum Electronics Limited",
    "70B3D5EB2": "Shooter Detection Systems",
    "70B3D557B": "ELAMAKATO GmbH",
    "70B3D52FD": "Special Projects Group, Inc",
    "70B3D583B": "Telefonix Incorporated",
    "70B3D5305": "CAITRON Industrial Solutions GmbH",
    "70B3D5010": "Hanwa Electronic Ind.Co.,Ltd.",
    "70B3D5AB7": "SIGLEAD INC",
    "70B3D5B88": "ARP Corporation",
    "70B3D5FDA": "ACD Elektronik GmbH",
    "70B3D5559": "Eagle Mountain Technology",
    "70B3D5DEC": "Condev-Automation GmbH",
    "70B3D579A": "Innerspec Technologies Inc.",
    "70B3D551E": "Fundaci\u00f3n Cardiovascular de Colombia",
    "70B3D5625": "VX Instruments GmbH",
    "70B3D5C58": "RMI Laser LLC",
    "70B3D5D8F": "Molu Technology Inc., LTD.",
    "70B3D5507": "Human Oriented Technology, Inc.",
    "70B3D596F": "4CAM GmbH",
    "70B3D580F": "Quickware Eng & Des LLC",
    "70B3D50F0": "Avionica",
    "70B3D5936": "FARO TECHNOLOGIES, INC.",
    "70B3D52F0": "Clock-O-Matic",
    "70B3D58CE": "CORES Corporation",
    "70B3D5346": "Ultamation Limited",
    "70B3D52A5": "Taitotekniikka",
    "70B3D5EFA": "NextEra Energy Resources, LLC",
    "70B3D51AC": "SVP Broadcast Microwave S.L.",
    "70B3D5CE9": "KINEMETRICS",
    "70B3D5090": "POWERCRAFT ELECTRONICS PVT. LTD.",
    "70B3D53F1": "Olympus NDT Canada",
    "70B3D5994": "KeFF Networks",
    "70B3D5F63": "Ars Products",
    "70B3D5F62": "FRS GmbH & Co. KG",
    "70B3D5308": "DSD MICROTECHNOLOGY,INC.",
    "70B3D5908": "Accusonic",
    "70B3D528F": "Overline Systems",
    "70B3D54C8": "Hosokawa Micron Powder Systems",
    "70B3D5FD1": "RedRat Ltd",
    "001BC50C1": "EREE Electronique",
    "001BC50BC": "kuwatec, Inc.",
    "001BC50B0": "J-D.COM",
    "001BC50B4": "COBAN SRL",
    "001BC50AA": "Senceive Ltd",
    "001BC50A6": "Balter Security GmbH",
    "001BC5099": "UAB Kitron",
    "001BC5094": "reelyActive",
    "001BC508E": "TrendPoint Systems",
    "001BC507F": "Hitechlab Inc",
    "001BC507E": "Bio Molecular System Pty Ltd",
    "001BC5073": "tado GmbH",
    "001BC5071": "Center for E-Commerce Infrastructure Development, The University of Hong Kong",
    "001BC506D": "TES Electronic Solutions (I) Pvt. Ltd.",
    "001BC5063": "Check-It Solutions Inc",
    "001BC505B": "konzeptpark GmbH",
    "001BC505F": "Klingenthaler Musikelektronik GmbH",
    "001BC505E": "Ecomed-Complex",
    "001BC505A": "POSTEC DATA SYSTEMS",
    "001BC5051": "QQ Navigation AB",
    "001BC504C": "Rhino Controls Ltd.",
    "001BC5047": "PT. Amanindo Nusapadu",
    "001BC5041": "DesignA Electronics Limited",
    "001BC503E": "Daylight  Solutions, Inc",
    "001BC503F": "ELTRADE Ltd",
    "001BC5038": "SEED International Ltd.",
    "001BC5037": "ITW Reyflex North America",
    "001BC5025": "andersen lighting GmbH",
    "001BC5026": "DIMEP Sistemas",
    "001BC5019": "Dunlop Systems & Components",
    "001BC500D": "Advanced Scientific Concepts, Inc.",
    "001BC5004": "Intellvisions Software Ltd",
    "001BC5014": "Private",
    "001BC5001": "OpenRB.com, Direct SIA",
    "8C1F6484D": "DAVE SRL",
    "8C1F64BFC": "ASiS Technologies Pte Ltd",
    "8C1F64C3E": "ISMA Microsolutions INC",
    "8C1F6436E": "Abbott Diagnostics Technologies AS",
    "8C1F64979": "Arktis Radiation Detectors",
    "70B3D5983": "Havis Inc.",
    "8C1F644D7": "Flextronics International Kft",
    "8C1F6409D": "Flextronics International Kft",
    "8C1F64FB9": "IWS Global Pty Ltd",
    "8C1F641DE": "Power Electronics Espana, S.L.",
    "8C1F64ABE": "TAIYO DENON Corporation",
    "8C1F64FC5": "SUMICO",
    "8C1F64964": "Power Electronics Espana, S.L.",
    "8C1F64F10": "Televic Rail GmbH",
    "8C1F64C9A": "Infosoft Digital Design and Services P L",
    "8C1F64C2D": "iENSO Inc.",
    "8C1F64390": "SkyLabs d.o.o.",
    "8C1F64718": "ABB",
    "8C1F64B9F": "Lithion Battery Inc",
    "8C1F64155": "SLAT",
    "8C1F646DC": "Intrinsic Innovation, LLC",
    "8C1F645A9": "Aktiebolag Solask Energi",
    "8C1F6454B": "MECT SRL",
    "8C1F64A1F": "Hitachi Energy  India Limited",
    "8C1F64E68": "LHA Systems (Pty) Ltd",
    "8C1F64C1A": "ViewSonic Corp",
    "8C1F64332": "NEXET LLC",
    "8C1F64759": "Systel Inc",
    "8C1F64338": "Rheingold Heavy LLC",
    "8C1F649D0": "Saline Lectronics, Inc.",
    "8C1F642E5": "GS Elektromedizinsiche Ger\u00e4te G. Stemple GmbH",
    "8C1F642BC": "DEUTA Werke GmbH",
    "70B3D52E1": "hiSky SCS Ltd",
    "8C1F64AF8": "Power Electronics Espana, S.L.",
    "8C1F64BF8": "CDSI",
    "8C1F640D2": "biosilver .co.,ltd",
    "8C1F64913": "Zeus Product Design Ltd",
    "8C1F64530": "SIPazon AB",
    "8C1F64FC3": "Cyclops Technology Group",
    "8C1F64A30": "TMP Srl",
    "8C1F64DD9": "Abbott Diagnostics Technologies AS",
    "8C1F6431C": "Accumetrics",
    "8C1F64DE9": "EON Technology, Corp",
    "8C1F64DBA": "Electronic Equipment Company Pvt. Ltd.",
    "8C1F64B6B": "KELC Electronics System Co., LTD.",
    "8C1F6441E": "Linxpeed Limited",
    "70B3D53F7": "CEFLA SC",
    "8C1F6463D": "Rax-Tech International",
    "8C1F647D7": "Metronic AKP sp.j.",
    "8C1F64889": "Private",
    "70B3D55B2": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F64DC9": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F646F8": "PROIKER TECHNOLOGY SL",
    "8C1F646AB": "Toho System Co., Ltd.",
    "8C1F6495B": "Qualitel Corporation",
    "8C1F6455C": "Schildknecht AG",
    "8C1F64CD1": "Flextronics International Kft",
    "8C1F64C62": "GMI Ltd",
    "8C1F64E4F": "Sabl Systems Pty Ltd",
    "8C1F642F1": "DEUTA Werke GmbH",
    "8C1F64C9E": "CytoTronics",
    "8C1F64FF9": "Vtron Pty Ltd",
    "8C1F64AB6": "EMIT GmbH",
    "8C1F645D0": "Image Engineering",
    "8C1F64481": "VirtualV Trading Limited",
    "8C1F64E47": "BRS Sistemas Eletr\u00f4nicos",
    "8C1F642A4": "YUYAMA MFG Co.,Ltd",
    "8C1F64280": "HEITEC AG",
    "8C1F642CC": "SBS SpA",
    "70B3D5304": "Wartsila Voyage Oy",
    "8C1F64D4F": "Henan Creatbot Technology Limited",
    "8C1F64BD5": "Pro-Custom Group",
    "8C1F64BFE": "PuS GmbH und Co. KG",
    "8C1F64E71": "Alma",
    "8C1F6486C": "Abbott Diagnostics Technologies AS",
    "70B3D5923": "eumig industrie-TV GmbH.",
    "8C1F6433D": "ARROW (CHINA) ELECTRONICS TRADING CO., LTD.",
    "8C1F64347": "Plut d.o.o.",
    "8C1F64920": "VuWall Technology Europe GmbH",
    "8C1F645EC": "NV BEKAERT SA",
    "8C1F64B9B": "Kromek Limited",
    "8C1F64749": "TIAMA",
    "8C1F64AA0": "Flextronics International Kft",
    "8C1F644A8": "Exact Sciences",
    "8C1F64221": "YUANSIANG OPTOELECTRONICS CO.,LTD.",
    "8C1F64116": "Sicon srl",
    "8C1F6468C": "GuangZhou HOKO Electric CO.,LTD",
    "8C1F6417B": "Bavaria Digital Technik GmbH",
    "8C1F64A86": "Global Design Tech(ZS) Co.,Ltd",
    "8C1F644A7": "Potter Electric Signal Co. LLC",
    "8C1F64AF0": "MinebeaMitsumi Inc.",
    "8C1F64CD9": "Fingoti Limited",
    "8C1F6444A": "Onbitel",
    "8C1F64AD3": "Working Set Software Solutions",
    "8C1F64F13": "ACS Motion Control",
    "8C1F64B1B": "Nov'in",
    "8C1F64C4E": "iCE-Intelligent Controlled Environments",
    "8C1F64228": "Shenzhen Chuanxin Micro Technology Co., Ltd",
    "8C1F645C4": "Cooltera Limited",
    "8C1F64A67": "Electrovymir LLC",
    "8C1F646FB": "Exatron Servers Manufacturing Pvt Ltd",
    "8C1F64E66": "ENLESS WIRELESS",
    "8C1F64B91": "CLEALINK TECHNOLOGY",
    "8C1F64E63": "Infosoft Digital Design and Services P L",
    "70B3D5E43": "SL Audio A/S",
    "8C1F64BA8": "AvMap srlu",
    "8C1F64516": "TCL OPERATIONS POLSKA SP. Z O.O.",
    "8C1F6494A": "Vision Systems Safety Tech",
    "8C1F64F1B": "Nextep Co.,Ltd.",
    "8C1F642AC": "DCO SYSTEMS LTD",
    "8C1F642FA": "RFENGINE CO., LTD.",
    "8C1F64ED5": "Smart Data (Shenzhen) Intelligent System Co., Ltd.",
    "8C1F64864": "IMI Thomson Valves",
    "8C1F64DED": "PhotonPath",
    "8C1F6482B": "Flow Power",
    "8C1F645A7": "RCH SPA",
    "8C1F64049": "NUANCES ORG",
    "8C1F644F6": "Private",
    "70B3D5C09": "RCH SPA",
    "001BC50AB": "Evondos Oy",
    "8C1F64FF3": "  Fuzhou Tucsen Photonics Co.,Ltd",
    "8C1F64384": "Tango Tango",
    "8C1F64474": "AUDIOBYTE S.R.L.",
    "8C1F64D8E": "Potter Electric Signal Co. LLC",
    "8C1F645FC": "Lance Design LLC",
    "8C1F64C43": "SHENZHEN SMARTLOG TECHNOLOGIES CO.,LTD",
    "8C1F64A87": "Morgen Technology",
    "8C1F64A9B": "Ovide Maudet SL",
    "8C1F64E79": " SHENZHEN GUANGWEN INDUSTRIAL CO.,LTD",
    "8C1F642DF": "Ubotica Technologies",
    "8C1F64126": "Harvest Technology Pty Ltd",
    "8C1F64E09": "ENLESS WIRELESS",
    "8C1F6470B": "ONICON ",
    "8C1F64D74": "TEX COMPUTER SRL ",
    "8C1F647B1": "EA Elektro-Automatik",
    "8C1F645DE": "SekureTrak Inc. dba TraknProtect",
    "8C1F6457D": "ISDI Ltd",
    "8C1F643CE": "MAHINDR & MAHINDRA",
    "8C1F6491B": "Potter Electric Signal Co. LLC",
    "8C1F64510": "Novanta IMS",
    "8C1F64E8D": "Plura",
    "8C1F647C7": "Ascon Tecnologic S.r.l.",
    "8C1F64232": "Monnit Corporation",
    "8C1F64292": "Gogo Business Aviation",
    "8C1F64020": "Utthunga Techologies Pvt Ltd",
    "8C1F647AE": "D-E-K GmbH & Co.KG",
    "70B3D5389": "2KLIC inc.",
    "8C1F645CD": "MAHINDR & MAHINDRA",
    "8C1F6477F": "TargaSystem S.r.L.",
    "70B3D5315": "Private",
    "8C1F64989": "Phe-nX B.V.",
    "8C1F64D8F": "DEUTA-WERKE GmbH",
    "8C1F64810": "Kymata Srl",
    "8C1F64A6E": "shenzhen beswave co.,ltd",
    "8C1F640FE": "Indra Heera Technology LLP",
    "8C1F64C42": "SD OPTICS",
    "8C1F640A4": "Dynamic Research, Inc.",
    "8C1F6413F": "Elsist Srl",
    "8C1F64A70": "V-teknik Elektronik AB",
    "8C1F64F63": "Quantum Media Systems",
    "70B3D5561": "Liberator Pty Ltd",
    "70B3D5F7C": "Medicomp, Inc",
    "8C1F6432C": "Taiko Audio B.V.",
    "8C1F645CE": "Packetalk LLC",
    "8C1F64317": "Bacancy Systems LLP",
    "70B3D5FFE": "Private",
    "8C1F648EB": "Numa Products LLC",
    "8C1F64DDE": "Jemac Sweden AB",
    "8C1F6440D": "PROFITT Ltd",
    "8C1F64CAB": "Spyder Controls Corp.",
    "8C1F64596": "RF Code",
    "8C1F64A3B": "Fujian Satlink Electronics Co., Ltd",
    "8C1F6483D": "L-signature",
    "8C1F64FA4": "China Information Technology Designing &Consulting Institute Co.,Ltd.",
    "8C1F645E7": "HOSCH Geb\u00e4ude Automation Neue Produkte GmbH",
    "70B3D55BC": "LAMTEC Mess- und Regeltechnik f\u00fcr Feuerungen GmbH & Co. KG",
    "70B3D5A51": "RF Code",
    "8C1F6490C": "Cool Air Incorporated",
    "70B3D540C": "Tornado Modular Systems",
    "8C1F64F79": "YUYAMA MFG Co.,Ltd",
    "8C1F6439E": "Abbott Diagnostics Technologies AS",
    "8C1F64D73": "BRS Sistemas Eletr\u00f4nicos",
    "8C1F64550": "ard sa",
    "8C1F64FD7": "Beijing Yahong Century Technology Co., Ltd",
    "8C1F64046": "American Fullway Corp.",
    "70B3D50C4": "TIAMA",
    "70B3D5C4A": "TIAMA",
    "70B3D5C75": "Planet Innovation Products Inc.",
    "8C1F6424C": "Shenzhen Link-All Technolgy Co., Ltd",
    "8C1F649CB": "Shanghai Sizhong Information Technology Co., Ltd",
    "8C1F64C1E": "VA SYD",
    "8C1F64653": "P5",
    "8C1F644AF": "miniDSP",
    "8C1F646E4": "RAB Microfluidics R&D Company Ltd",
    "8C1F648D0": "Enerthing GmbH",
    "8C1F6434C": "Kyushu Keisokki Co.,Ltd.",
    "8C1F6412E": "inomatic GmbH",
    "8C1F64FE9": "ALZAJEL MODERN TELECOMMUNICATION",
    "8C1F6443D": "Solid State Supplies Ltd",
    "8C1F640BB": "InfraChen Technology Co., Ltd.",
    "8C1F64402": "Integer.pl S.A.",
    "8C1F64BC9": "GL TECH CO.,LTD",
    "001BC5008": "Dalaj Electro-Telecom",
    "8C1F64A83": "EkspertStroyProekt",
    "8C1F648F6": "Idneo Technologies S.A.U.",
    "8C1F64E12": "Pixus Technologies Inc.",
    "8C1F64DFB": "Bobeesc Co.",
    "8C1F64AEA": "INHEMETER Co.,Ltd",
    "8C1F644E9": "EERS GLOBAL TECHNOLOGIES INC.",
    "8C1F645AF": "Teq Diligent Product Solutions Pvt. Ltd.",
    "8C1F64764": "nanoTRONIX Computing Inc.",
    "8C1F6425C": "TimeMachines Inc.",
    "8C1F64882": "TMY TECHNOLOGY INC.",
    "8C1F648DE": "Iconet Services",
    "8C1F641D0": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64C51": "EPC Energy Inc",
    "8C1F64AC3": "WAVES SYSTEM",
    "8C1F64AC4": "comelec",
    "8C1F64B2B": "Rhombus Europe",
    "8C1F64BE8": "TECHNOLOGIES BACMOVE INC.",
    "8C1F6407A": "Flextronics International Kft",
    "8C1F64A9C": "Upstart Power",
    "8C1F641FE": "Burk Technology",
    "8C1F64F23": "IDEX India Pvt Ltd",
    "8C1F64A0A": "Shanghai Wise-Tech Intelligent Technology Co.,Ltd.",
    "8C1F64E94": "ZIN TECHNOLOGIES",
    "8C1F647E2": "Aaronn Electronic GmbH",
    "8C1F64F56": "KC5 International Sdn Bhd",
    "70B3D54F8": "SICPA SA - GSS",
    "8C1F64825": "MTU Aero Engines AG",
    "70B3D5D1E": "Houston Radar LLC",
    "8C1F64B67": "M2M craft Co., Ltd.",
    "8C1F640EE": "Rich Source Precision IND., Co., LTD.",
    "8C1F6432F": "DEUTA Controls GmbH",
    "8C1F6417C": "Zelp Ltd",
    "8C1F64D53": "Gridnt",
    "8C1F641A7": "aelettronica group srl",
    "8C1F64FB4": "Thales Nederland BV",
    "70B3D5F1F": "HKC Security Ltd.",
    "8C1F64B55": "Sanchar Telesystems limited",
    "8C1F64083": "Avionica ",
    "70B3D5874": "NORTHBOUND NETWORKS PTY. LTD.",
    "8C1F6479B": "Foerster-Technik GmbH",
    "8C1F643FC": "STV Electronic GmbH",
    "8C1F641BD": "DORLET SAU",
    "8C1F64CAF": "BRS Sistemas Eletr\u00f4nicos",
    "8C1F64626": "CSIRO",
    "8C1F64C05": "SkyCell AG",
    "8C1F647AF": "E VISION INDIA PVT LTD ",
    "8C1F64683": "SLAT",
    "8C1F6494C": "BCMTECH",
    "8C1F6497C": "MB connect line GmbH Fernwartungssysteme",
    "8C1F643AC": "Benison Tech",
    "8C1F64946": "UniJet Co., Ltd.",
    "8C1F6490F": "BELIMO Automation AG",
    "8C1F64C3A": "YUSUR Technology Co., Ltd.",
    "8C1F648F8": "HIGHVOLT Pr\u00fcftechnik",
    "8C1F64625": "Stresstech OY",
    "8C1F64FB5": "Bavaria Digital Technik GmbH",
    "8C1F64CF4": "NT",
    "8C1F64DD7": "KST technology",
    "8C1F64B73": "Comm-ence, Inc.",
    "8C1F6473B": "Fink Zeitsysteme GmbH",
    "8C1F64CD3": "Pionierkraft GmbH",
    "8C1F640AC": "Patch Technologies, Inc.",
    "8C1F640BE": "BNB",
    "8C1F64BBF": "Retency",
    "8C1F64D88": "University of Geneva - Department of Particle Physics",
    "8C1F64601": "Camius",
    "8C1F649FA": "METRONA-Union GmbH",
    "8C1F64DF8": "Wittra Networks AB",
    "8C1F64B92": "Neurable",
    "8C1F64FAA": "Massar Networks",
    "8C1F64FA8": "Unitron Systems b.v.",
    "8C1F64737": "Vytahy-Vymyslicky s.r.o.",
    "8C1F6484C": "AvMap srlu",
    "8C1F64D69": "ADiCo Corporation",
    "8C1F64E5E": "BRICKMAKERS GmbH",
    "8C1F64C4C": "Lumiplan Duhamel",
    "8C1F647DE": "SOCNOC AI Inc",
    "8C1F64697": "Sontay Ltd.",
    "8C1F64F7A": "SiEngine Technology Co., Ltd.",
    "8C1F64557": "In-lite Design BV",
    "8C1F648E9": "Vesperix Corporation",
    "8C1F64FF4": "SMS group GmbH",
    "70B3D5E2F": "Flextronics International Kft",
    "8C1F64F5C": "Flextronics International Kft",
    "8C1F64A6D": "CyberneX Co., Ltd",
    "8C1F642FD": "Enestone Corporation",
    "8C1F64BD6": "NOVA Products GmbH",
    "8C1F64634": "AML",
    "8C1F64AE8": "ADETEC SAS",
    "8C1F6473C": "REO AG",
    "8C1F64C38": "ECO-ADAPT",
    "8C1F6408F": "AixControl GmbH",
    "8C1F6438B": "Borrell USA Corp",
    "8C1F64A1B": "Zilica Limited",
    "8C1F6447A": "Missing Link Electronics, Inc.",
    "8C1F6481A": "Gemini Electronics B.V.",
    "8C1F64EFB": "WARECUBE,INC",
    "8C1F64EB7": "Delta Solutions LLC",
    "8C1F64A2B": "WENet Vietnam Joint Stock company",
    "8C1F64A01": "Guan Show Technologe Co., Ltd.",
    "8C1F6465F": "Astrometric Instruments, Inc.",
    "8C1F64077": "Engage Technologies",
    "8C1F64603": "Fuku Energy Technology Co., Ltd.",
    "8C1F647D2": "Enlaps",
    "8C1F6435D": "Security&Best",
    "8C1F64EBF": "STEAMIQ, Inc.",
    "8C1F6460A": "RFENGINE CO., LTD.",
    "8C1F64E7B": "Dongguan Pengchen Earth Instrument CO. LT",
    "8C1F643C6": "Wavestream Corp",
    "8C1F64D13": "EYatsko Individual",
    "8C1F6485B": "Atlantic Pumps Ltd",
    "70B3D5A7A": "Fluid Management Technology",
    "70B3D5313": "DIEHL Controls",
    "8C1F64BC6": "Chengdu ZiChen Time&Frequency Technology Co.,Ltd",
    "8C1F646B5": "O-Net Communications(Shenzhen)Limited",
    "8C1F64675": "Transit Solutions, LLC.",
    "8C1F64521": "MP-SENSOR GmbH",
    "8C1F648AF": "Ibeos",
    "8C1F64D44": "Monarch Instrument",
    "8C1F645F5": "HongSeok Ltd.",
    "8C1F644DA": "DTDS Technology Pte Ltd",
    "8C1F64EB9": "KxS Technologies Oy",
    "8C1F646A8": "Bulwark",
    "8C1F64C71": "Yaviar LLC",
    "8C1F64787": "Tabology",
    "70B3D5CCB": "RealD, Inc.",
    "8C1F64650": "L tec Co.,Ltd",
    "70B3D53FB": "Liberty Reach",
    "8C1F64B46": "PHYGITALL SOLU\u00c7\u00d5ES EM INTERNET DAS COISAS",
    "8C1F64536": "BEIJING LXTV TECHNOLOGY CO.,LTD",
    "8C1F64ED4": "ZHEJIANG CHITIC-SAFEWAY NEW ENERGY TECHNICAL CO.,LTD.",
    "8C1F64663": "mal-tech Technological Solutions Ltd/CRISP",
    "8C1F64B2C": "SANMINA ISRAEL MEDICAL SYSTEMS LTD",
    "8C1F64F78": "Ternary Research Corporation",
    "8C1F64385": "Multilane Inc",
    "8C1F646CD": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "8C1F646C6": "FIT",
    "8C1F6425A": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "8C1F64086": "WEPTECH elektronik GmbH",
    "8C1F6453A": "TPVision Europe B.V",
    "8C1F64DAE": "Mainco automotion s.l.",
    "8C1F643E0": "YPP Corporation",
    "8C1F6479E": "Accemic Technologies GmbH",
    "8C1F64EF1": "BIOTAGE GB LTD",
    "8C1F648C2": "Cirrus Systems, Inc.",
    "8C1F6473F": "UBISCALE",
    "8C1F642F5": "Florida R&D Associates LLC",
    "8C1F642E8": "Sonora Network Solutions",
    "70B3D5417": "Figment Design Laboratories",
    "8C1F64EEF": "AiUnion Co.,Ltd",
    "8C1F64AF7": "ard sa",
    "8C1F64EE8": "Global Organ Group B.V.",
    "8C1F6479D": "Murata Manufacturing Co., Ltd.",
    "8C1F64151": "Gogo Business Aviation",
    "8C1F64296": "Roog zhi tong Technology(Beijing) Co.,Ltd",
    "8C1F64E5C": "Scientific Lightning Solutions",
    "8C1F6483A": "Grossenbacher Systeme AG",
    "8C1F64370": "WOLF Advanced Technology",
    "8C1F644C7": "SBS SpA",
    "8C1F64AE1": "YUYAMA MFG Co.,Ltd",
    "8C1F64856": "Garten Automation",
    "8C1F64903": "Portrait Displays, Inc.",
    "8C1F64549": "Brad Technology",
    "8C1F64270": "Xi\u2018an Hangguang Satellite and Control Technology Co.,Ltd",
    "8C1F64BC0": "GS Elektromedizinsiche Ger\u00e4te G. Stemple GmbH",
    "8C1F64264": "BR. Voss Ingenj\u00f6rsfirma AB",
    "8C1F64135": "Yuval Fichman",
    "70B3D5508": "INSEVIS GmbH",
    "8C1F64D7E": "Thales Belgium",
    "8C1F64A57": "EkspertStroyProekt",
    "70B3D54CD": "Power Electronics Espana, S.L.",
    "70B3D5223": "Research Laboratory of Design Automation, Ltd.",
    "70B3D5E0E": "VulcanForms",
    "70B3D5C71": "The Engineerix Group",
    "70B3D5D5D": "SEASONS 4 INC",
    "70B3D59D8": "JOLANYEE  Technology Co., Ltd.",
    "70B3D5ACE": "FARHO DOMOTICA SL",
    "70B3D51F6": "LinkAV Technology Co., Ltd",
    "70B3D5076": "Private Enterprise \"Scientific and Production Private Enterprise\"Sparing-Vist Center\"\"",
    "70B3D51BC": "Flextronics International Kft",
    "70B3D50E2": "JESE Ltd",
    "70B3D57F6": "IDZ Ltd",
    "70B3D55CE": "IP Devices",
    "70B3D58FD": "sonatest",
    "70B3D5912": "VERTEL DIGITAL PRIVATE LIMITED",
    "70B3D5D1D": "Stuyts Engineering Haarlem BV",
    "70B3D5452": "ITALIANA PONTI RADIO SRL",
    "70B3D5160": "European Synchrotron Radiation Facility",
    "70B3D5713": "Coloet S.r.l.",
    "70B3D57FA": "meoENERGY",
    "70B3D59BC": "Radian Research, Inc.",
    "70B3D55C2": "Sono-Tek Corporation",
    "70B3D55C3": "DIC Corporation",
    "70B3D5CAD": "YUYAMA MFG Co.,Ltd",
    "70B3D5E60": "Davitor AB",
    "70B3D5CF9": "Breas Medical AB",
    "70B3D5EF0": "PNETWORKS",
    "70B3D58C9": "Arwin Technology Limited",
    "70B3D596C": "Weble S\u00e0rl",
    "70B3D5CE8": "Grossenbacher Systeme AG",
    "70B3D5B01": "G.S.D GROUP INC.",
    "70B3D5CB5": "Atlas Lighting Products",
    "70B3D5C6B": "Herholdt Controls srl",
    "70B3D51B0": "NAL Research Corporation",
    "70B3D504F": "EVPU Defence a.s.",
    "70B3D55A4": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5792": "IMMOLAS",
    "70B3D5CA6": "AXING AG",
    "70B3D56C0": "LLC \"NTZ \"Mekhanotronika\"",
    "70B3D5E62": "Eon",
    "70B3D5A79": "NOREYA Technology e.U.",
    "70B3D598D": "Motohaus Powersports Limited",
    "70B3D5ED9": "AADONA Communication Pvt Ltd",
    "70B3D56CC": "ARINAX",
    "70B3D529E": "B2cloud lda",
    "70B3D5BDB": "Power Electronics Espana, S.L.",
    "70B3D54DD": "Velvac Incorporated",
    "70B3D56AA": "Intermobility",
    "70B3D5DE9": "EkspertStroyProekt LLC",
    "70B3D57AB": "Microgate Srl",
    "70B3D5D03": "Digitella Inc.",
    "70B3D541C": "Twoway Communications, Inc.",
    "70B3D564F": "GUNMA ELECTRONICS CO LTD",
    "70B3D5118": "Macromatic Industrial Controls, Inc.",
    "70B3D54C3": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D5C46": "eumig industrie-TV GmbH.",
    "70B3D5621": "SERTEC SRL",
    "70B3D5921": "QDevil",
    "70B3D53A6": "myenergi Ltd",
    "70B3D5D50": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "70B3D5156": "Rivercity Innovations Ltd.",
    "70B3D58F9": "IWS Global Pty Ltd",
    "70B3D5B14": "Pantherun Technologies Pvt Ltd",
    "70B3D5B45": "Hon Hai Precision IND.CO.,LTD",
    "70B3D5BEB": "Potter Electric Signal Co. LLC",
    "70B3D53D1": "Imenco Ltd",
    "70B3D52DF": "EASTERN SCIENCE & TECHNOLOGY CO., LTD",
    "70B3D5806": "International Super Computer Co., Ltd.",
    "70B3D5883": "Contec Americas Inc.",
    "70B3D5EC9": "Qlinx Technologies",
    "70B3D5F6B": "DEUTA-WERKE GmbH",
    "70B3D5E87": "STACKFORCE GmbH",
    "70B3D5B83": "Matrix Telematics Limited",
    "70B3D5958": "pureLiFi Ltd",
    "70B3D525E": "RFHIC",
    "70B3D5B86": "Hilo",
    "70B3D510B": "SECUREAN CO.,Ltd",
    "70B3D5DC7": "NUBURU Inc.",
    "70B3D5A3E": "Vigorcloud Co., Ltd.",
    "70B3D5BE2": "Nocix, LLC",
    "70B3D5E22": "Federated Wireless, Inc. ",
    "70B3D579C": "ADDE",
    "70B3D5B0E": "Servotronix Motion Control",
    "70B3D52DD": "Melissa Climate Jsc",
    "70B3D5A8C": "CYG CONTRON CO.LTD",
    "70B3D5AE8": "Innoknight",
    "70B3D57C5": "Projects Unlimited Inc.",
    "70B3D5D14": "LIGPT",
    "70B3D534F": "Royal Engineering Consultancy Private Limited",
    "70B3D5157": "Shanghai Jupper Technology Co.Ltd",
    "70B3D57F0": "YDK Technologies Co.,Ltd",
    "70B3D5F60": "MPM Micro Pr\u00e4zision Marx GmbH",
    "70B3D5FC4": "AERIAL CAMERA SYSTEMS Ltd",
    "70B3D54A3": "TUALCOM ELEKTRONIK A.S.",
    "70B3D573A": "DOLBY LABORATORIES, INC.",
    "70B3D52FB": "IK MULTIMEDIA PRODUCTION SRL",
    "70B3D52D3": "Hensoldt Sensors GmbH",
    "70B3D5B68": "S-Rain Control A/S",
    "70B3D5C1E": "Kron Medidores",
    "70B3D5D96": "Thermo Fisher Scientific Inc.",
    "70B3D5BC7": "Autonomic Controls, Inc.",
    "70B3D5567": "DogWatch Inc",
    "70B3D5D97": "BRS Sistemas Eletr\u00f4nicos",
    "70B3D5F7D": "2M Technology",
    "70B3D5FC7": "Invert Robotics Ltd.",
    "70B3D5B63": "Ideas srl",
    "70B3D58BD": "MAHLE ELECTRONICS, SLU",
    "70B3D5424": "Underground Systems, Inc.",
    "70B3D5B6F": "Integra Metering SAS",
    "70B3D533D": "Schneider Electric Motion USA",
    "70B3D5177": "Wired Broadcast Ltd",
    "70B3D58F1": "Paramount Bed Holdings Co., Ltd.",
    "70B3D5263": "AXING AG",
    "70B3D59E4": "K&A Electronics Inc.",
    "70B3D58DE": "Indutherm Giesstechnologie GmbH",
    "70B3D5E73": "Zeus Control Systems Ltd",
    "70B3D53A3": "CDS Institute of Management Strategy, Inc.",
    "70B3D5ED3": "Beijing Lihong Create Co., Ltd.",
    "70B3D5611": "Avionica ",
    "70B3D5DBA": "KODENSHI CORP.",
    "70B3D5291": "Sequent AG",
    "70B3D529A": "Profusion Limited",
    "70B3D5657": "ID Quantique SA",
    "70B3D53D3": "GS Elektromedizinsiche Ger\u00e4te G. Stemple GmbH",
    "70B3D5C90": "Diretta",
    "70B3D5F4E": "Hunan Lianzhong Technology Co.,Ltd.",
    "70B3D5301": "WAYNE ANALYTICS LLC",
    "70B3D5F0E": "TextSpeak Corporation",
    "70B3D5AD3": "WARECUBE,INC",
    "70B3D5FC3": "myUpTech AB",
    "70B3D530A": "HongSeok Ltd.",
    "70B3D56DB": "Techimp - Altanova group Srl",
    "70B3D5C72": "Scharco Elektronik GmbH",
    "70B3D578D": "AVL DiTEST GmbH",
    "70B3D56D7": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5E01": "EarTex",
    "70B3D5F09": "Mictrotrac Retsch GmbH",
    "70B3D5473": "KeyProd",
    "70B3D5EE6": "Vaunix Technology Corporation",
    "70B3D5529": "Inventeq B.V.",
    "70B3D5601": "Tricom Research Inc.",
    "70B3D5D77": "Portrait Displays, Inc.",
    "70B3D5B03": "Sprintshield d.o.o.",
    "70B3D5C94": "Vars Technology ",
    "70B3D5A83": "SHENZHEN HUINENGYUAN Technology Co., Ltd",
    "70B3D5641": "Burk Technology",
    "70B3D5B75": "Grossenbacher Systeme AG",
    "70B3D5D2A": "ITsynergy Ltd",
    "70B3D5869": "chargeBIG",
    "70B3D5F69": "Copper Labs, Inc.",
    "70B3D51E7": "DogWatch Inc",
    "70B3D5FB1": "TOMEI TSUSHIN KOGYO CO,.LTD",
    "70B3D532E": "A&T Corporation",
    "70B3D502B": "Scorpion Precision Industry (HK)CO. Ltd.",
    "70B3D5193": "ERA TOYS LIMITED",
    "70B3D51D5": "MIVO Technology AB",
    "70B3D50DB": "Cryptotronix LLC",
    "70B3D5E4D": "Vulcan Wireless Inc.",
    "70B3D51E2": "Shenzhen CAMERAY ELECTRONIC CO., LTD",
    "70B3D50CC": "ADMiTAS CCTV Taiwan Co. Ltd",
    "70B3D5F9F": "M.A.C. Solutions (UK) Ltd",
    "70B3D51C3": "Shanghai Tiancheng Communication Technology Corporation",
    "70B3D5B41": "T&M Media Pty Ltd",
    "70B3D5282": "SAMBO HITECH",
    "70B3D5074": "Orlaco Products B.V.",
    "70B3D50F9": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D59A4": "Nordmann International GmbH",
    "70B3D58F4": "ACQUA-SYSTEMS srls",
    "70B3D5CA5": "PTS Technologies Pte Ltd",
    "70B3D5EA9": "Zhuhai Lonl electric Co.,Ltd.",
    "70B3D5951": "Trident Systems Inc",
    "70B3D5CAF": "DAVE SRL",
    "70B3D5BE0": "Cognosos, Inc.",
    "70B3D5A3D": "SMART IN OVATION GmbH",
    "70B3D50C3": "Aug. Winkhaus GmbH & Co. KG",
    "70B3D550C": "Hangzhou landesker digital technology co. LTD",
    "70B3D5021": "HGL Dynamics Ltd",
    "70B3D550D": "CT Company",
    "70B3D56AE": "Hangzhou Weimu Technology Co,.Ltd.",
    "70B3D55DF": "Semacon Business Machines",
    "70B3D50E7": "Pure Air Filtration",
    "70B3D5F53": "HighTechSystem Co.,Ltd.",
    "70B3D512D": "S.E.I. CO.,LTD.",
    "70B3D5CA2": "De Haardt bv",
    "70B3D5843": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D574B": "Code Blue Corporation",
    "70B3D5B9F": "Yuksek Kapasite Radyolink Sistemleri San. ve Tic. A.S.",
    "70B3D526A": "Talleres de Escoriaza SA",
    "70B3D589D": "e-Matix Corporation",
    "70B3D527E": "Mettler Toledo",
    "70B3D5BB2": "Mettler Toledo",
    "70B3D5797": "Mitsubishi Electric India Pvt. Ltd.",
    "70B3D59AF": "Shanghai Brellet Telecommunication Technology Co., Ltd.",
    "70B3D506A": "Guangdong Centnet Technology Co.,Ltd",
    "70B3D5F20": "Ibercomp SA",
    "70B3D5CB9": "JSC \u00abSATIS-TL-94\u00bb",
    "70B3D5045": "Navaero Avionics AB",
    "70B3D5DD4": "ResIOT UBLSOFTWARE SRL",
    "70B3D5834": "NCE Network Consulting Engineering srl",
    "70B3D5B5E": "Dynics",
    "70B3D5F3A": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D5B5A": "GTI Technologies Inc",
    "70B3D5C69": "AZ-TECHNOLOGY SDN BHD",
    "70B3D5662": "Icon Industrial Engineering",
    "70B3D549D": "Shenzhen Chanslink Network Technology Co., Ltd",
    "70B3D57BB": "Aloxy",
    "70B3D5DD6": "Umweltanalytik Holbach GmbH",
    "70B3D56AB": "ARROW (CHINA) ELECTRONICS TRADING CO., LTD.",
    "70B3D56B9": "Becton Dickinson",
    "70B3D5438": "HBI Bisscheroux bv",
    "70B3D5E80": "Changzhou Rapid Information Technology Co,Ltd",
    "70B3D541D": "Azmoon Keifiat",
    "70B3D5EE9": "SC3 Automation",
    "70B3D5ACC": "Schneider Electric Motion USA",
    "70B3D5780": "NIDEC LEROY-SOMER",
    "70B3D5064": "AB PRECISION (POOLE) LTD",
    "70B3D5057": "RCH ITALIA SPA ",
    "70B3D5539": "Tempris GmbH",
    "70B3D55DD": "Theatrixx Technologies, Inc.",
    "70B3D5C76": "ELA INNOVATION",
    "70B3D5101": "Adolf Nissen Elektrobau GmbH + Co. KG",
    "70B3D56BC": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D5D39": "ASHIDA Electronics Pvt. Ltd",
    "70B3D5F0A": "Neuronal Innovation Control S.L.",
    "70B3D5219": "D-E-K GmbH & Co.KG",
    "70B3D5B4F": "AvMap srlu",
    "70B3D5BBB": "YUYAMA MFG Co.,Ltd",
    "70B3D5C4E": "ARKRAY, Inc. Kyoto Laboratory",
    "70B3D5A39": "SPETSSTROY-SVYAZ Ltd",
    "70B3D50B1": "AirBie AG",
    "70B3D59EA": "Blue Storm Associates, Inc.",
    "70B3D526F": "COMPAL ELECTRONICS, INC.",
    "70B3D5D15": "3DGence sp. z o.o.",
    "70B3D55AE": "TinTec Co., Ltd.",
    "70B3D59FE": "SURUGA SEIKI CO., LTD.",
    "70B3D5E56": "HIPODROMO DE AGUA CALIENTE, S.A. DE C.V.",
    "70B3D5783": "CHIeru., CO., Ltd. ",
    "70B3D5C7B": "EM Clarity Pty Ltd",
    "70B3D5414": "Smith Meter, Inc.",
    "70B3D51AA": "Echo Ridge, LLC",
    "70B3D57FF": "eumig industrie-TV GmbH.",
    "70B3D522B": "VITEC",
    "70B3D5E81": "SLAT",
    "70B3D5C83": "CertusNet Inc.",
    "70B3D5F6C": "VisioGreen",
    "70B3D56CA": "LINEAGE POWER PVT LTD.,",
    "70B3D570B": "Alere Technologies AS",
    "70B3D5158": "EAX Labs s.r.o.",
    "70B3D5458": "Ongisul Co.,Ltd.",
    "70B3D5E38": "Cursor Systems NV",
    "70B3D5DFB": "Yamamoto Works Ltd.",
    "70B3D5CC6": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5C04": "Prolan Zrt.",
    "70B3D5C51": "Innotas Elektronik GmbH",
    "70B3D56E5": "DEUTA-WERKE GmbH",
    "70B3D5B73": "Cetto Industries",
    "70B3D5E8A": "Melecs EWS GmbH",
    "70B3D5E1E": "Umano Medical Inc.",
    "70B3D5F29": "SamabaNova Systems",
    "70B3D5A4D": "LANSITEC TECHNOLOGY CO., LTD",
    "70B3D537E": "ELINKGATE JSC",
    "70B3D5B20": "ICT BUSINESS GROUP of Humanrights Center for disabled people",
    "70B3D583A": "EMDEP CENTRO TECNOLOGICO MEXICO",
    "70B3D542D": "RCH ITALIA SPA ",
    "70B3D5B52": "AEye, Inc.",
    "70B3D58C4": "APE GmbH",
    "70B3D54A8": "Acrodea, Inc.",
    "70B3D5BB5": "Grossenbacher Systeme AG",
    "70B3D5DD9": "MaNima Technologies BV",
    "70B3D5A42": "iMAR Navigation GmbH",
    "70B3D5A45": "Viper Innovations Ltd",
    "70B3D5558": "Multiple Access Communications Ltd",
    "70B3D5808": "Becton Dickinson",
    "70B3D56BA": "Integrotech sp. z o.o.",
    "70B3D5795": "TIECHE Engineered Systems",
    "70B3D5388": "Xitron",
    "70B3D57BC": "FIRST RF Corporation",
    "70B3D5D3A": "PROMOMED RUS LLC",
    "70B3D50AD": "Vega-Absolute",
    "70B3D5628": "MECT SRL",
    "70B3D5535": "SITA Messtechnik GmbH",
    "70B3D5275": "INTERNET PROTOCOLO LOGICA SL",
    "70B3D557F": "MBio Diagnostics, Inc.",
    "70B3D5648": "Magnamed Tecnologia Medica S/A",
    "70B3D5860": "KBS Industrieelektronik GmbH",
    "70B3D5B1A": "Aaronia AG",
    "70B3D5150": "YUYAMA MFG Co.,Ltd",
    "70B3D5E79": "Acrodea, Inc.",
    "70B3D51E1": "TEX COMPUTER SRL ",
    "70B3D5345": "AT-Automation Technology GmbH",
    "70B3D5D4F": "C-COM Satellite Systems Inc.",
    "70B3D50A7": "Traffic and Parking Control Co, Inc.",
    "70B3D5121": "Shenzhen Luxurite Smart Home Ltd",
    "70B3D56A3": "OutdoorLink",
    "70B3D547E": "Fiber Optika Technologies Pvt. Ltd. ",
    "70B3D5BCD": "Sasken Technologies Ltd",
    "70B3D5B46": "FAS Electronics (Fujian) Co.,LTD.",
    "70B3D589A": "Algodue Elettronica Srl",
    "70B3D5C40": "HongSeok Ltd.",
    "70B3D5A7B": "SmartSafe",
    "70B3D59C4": "aelettronica group srl",
    "70B3D5391": "Changshu Ruite Electric Co.,Ltd.",
    "70B3D594E": "BP Lubricants USA, Inc.",
    "70B3D5430": "Algodue Elettronica Srl",
    "70B3D58E2": " Zhiye Electronics Co., Ltd.",
    "70B3D5972": "AixControl GmbH",
    "70B3D5A5D": "Position Imaging",
    "70B3D572F": "Ava Technologies",
    "70B3D51B3": "Graphcore Ltd",
    "70B3D586B": "AVL DiTEST",
    "70B3D51BD": "Shenzhen Siera Technology Ltd",
    "70B3D541B": "SYS TEC electronic GmbH",
    "70B3D5C29": "SOFTLAND INDIA LTD",
    "70B3D5CFE": "Secturion Systems",
    "70B3D544B": "Open System Solutions Limited",
    "70B3D556C": "Telensa Ltd",
    "70B3D5512": "Techno Broad,Inc",
    "70B3D54EE": "NOA Co., Ltd.",
    "70B3D5003": "ANYROAM",
    "70B3D5013": "Sportsbeams Lighting, Inc.",
    "70B3D52FF": "Sunstone Engineering",
    "70B3D5071": "FSR, INC.",
    "70B3D5A22": "eSys Solutions Sweden AB",
    "70B3D5BB6": "Franke Aquarotter GmbH",
    "70B3D5F9B": "EvoLogics GmbH",
    "70B3D54B3": "Bacsoft",
    "70B3D514B": "C21 Systems Ltd",
    "70B3D53FA": "Zaklad Energoelektroniki Twerd",
    "70B3D5FC0": "CODESYSTEM Co.,Ltd",
    "70B3D52E8": "Telefire",
    "70B3D5AF8": "boekel",
    "70B3D54C6": "BlueBox Video Limited",
    "70B3D5189": "DAVE SRL",
    "70B3D5E32": "HERUTU ELECTRONICS CORPORATION",
    "70B3D5688": "MG s.r.l.",
    "70B3D51DC": "TEKVEL Ltd.",
    "70B3D5B0F": "merkur Funksysteme AG",
    "70B3D587F": "NAC Planning Co., Ltd.",
    "70B3D5293": "Solar RIg Technologies",
    "70B3D53F5": "DOLBY LABORATORIES, INC.",
    "70B3D5E61": "Adeli",
    "70B3D5707": "Koco Motion US LLC",
    "70B3D5BCB": "Smart Vision Lights",
    "70B3D5A8D": "Code Blue Corporation ",
    "70B3D52A3": "ATT Nussbaum Pr\u00fcftechnik GmbH",
    "70B3D582D": "Elektronik Art S.C.",
    "70B3D58A2": "WINNERS DIGITAL CORPORATION",
    "70B3D53F2": "H3D, Inc.",
    "70B3D5360": "PT. Emsonic Indonesia",
    "70B3D5225": "RCD Radiokomunikace",
    "70B3D5C64": "SYS TEC electronic GmbH",
    "70B3D5758": "Grossenbacher Systeme AG",
    "70B3D53B0": "Millennial Net, Inc.",
    "70B3D5415": "IDEA SPA",
    "70B3D5C0B": "FSTUDIO CO LTD",
    "70B3D5CDF": "3D Printing Specialists",
    "70B3D517F": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5AA2": "eumig industrie-TV GmbH.",
    "70B3D50BF": "Den Automation",
    "70B3D5B9C": "EDCO Technology 1993 ltd",
    "70B3D5C1A": "Xylon",
    "70B3D5C9E": "FUKUDA SANGYO CO., LTD.",
    "70B3D5084": "Rako Controls Ltd",
    "70B3D553C": "Airthings",
    "70B3D53C7": "SOFTCREATE CORP.",
    "70B3D59AA": "Tecsys do Brasil Industrial Ltda",
    "70B3D5664": "Sankyo Intec co.,ltd",
    "70B3D5A0A": "CAPSYS",
    "70B3D578F": "SoFiHa",
    "70B3D5515": "PCSC",
    "70B3D5E52": "Guangzhou Moblin Technology Co., Ltd. ",
    "70B3D519F": "Koizumi Lighting Technology Corp.",
    "70B3D5478": "Touchnet/OneCard",
    "70B3D523B": "Fink Telecom Services",
    "70B3D54EC": "Hangzhou Youshi Industry Co., Ltd.",
    "70B3D5A37": "MITSUBISHI HEAVY INDUSTRIES THERMAL SYSTEMS, LTD.",
    "70B3D5BCE": "YAWATA ELECTRIC INDUSTRIAL CO.,LTD.",
    "70B3D51D1": "Eurotek Srl",
    "70B3D510E": "Colorimetry Research, Inc",
    "70B3D5D72": "OnYield Inc Ltd",
    "70B3D5027": "Redcap Solutions s.r.o.",
    "70B3D52A9": "Power Electronics Espana, S.L.",
    "70B3D5613": "Suprock Technologies",
    "70B3D5676": "samwooeleco",
    "70B3D5257": "LG Electronics",
    "70B3D5224": "Urbana Smart Solutions Pte Ltd",
    "70B3D5D89": "Resolution Systems",
    "70B3D58B7": "Contec Americas Inc.",
    "70B3D5436": "Henrich Electronics Corporation",
    "70B3D570A": "PULLNET TECHNOLOGY, SA DE CV SSC1012302S73",
    "001BC508C": "Triax A/S",
    "70B3D54AF": "Agramkow Fluid Systems A/S",
    "70B3D5810": "Advice",
    "70B3D5A13": "Uplevel Systems Inc",
    "70B3D5D4B": "Hermann L\u00fcmmen GmbH",
    "70B3D54B4": "Hi Tech Systems Ltd",
    "70B3D527F": "ST Aerospace Systems",
    "70B3D5023": "Cambridge Pixel",
    "70B3D5FB7": "SAICE",
    "70B3D5555": "SoftLab-NSK",
    "70B3D509B": "Jacarta Ltd",
    "001BC500E": "Vigor Electric Corp",
    "70B3D5BB9": "KOSMEK.Ltd",
    "70B3D5A17": "Tunstall A/S",
    "70B3D54D4": "Nortek Global HVAC",
    "70B3D54F9": "OptoPrecision GmbH",
    "70B3D598B": "Richard Paul Russell Ltd",
    "70B3D5C2F": "ATBiS Co.,Ltd",
    "70B3D5C67": "Collini Dienstleistungs GmbH",
    "70B3D55E3": "Imecon Engineering SrL",
    "70B3D5784": "Shenzhen bayue software co. LTD",
    "70B3D5711": "X-Laser LLC",
    "70B3D563F": "DARBS Inc.",
    "70B3D51D0": "Shenzhen INVT Electric Co.,Ltd",
    "70B3D5368": "White Matter LLC",
    "70B3D5717": "Secure Systems & Services",
    "70B3D5777": "QUERCUS TECHNOLOGIES, S.L.",
    "70B3D57FD": "SYS TEC electronic GmbH",
    "70B3D598E": "Autocom Diagnostic Partner AB",
    "70B3D503C": "Ultimate Software",
    "70B3D591F": "JSC \"InformInvestGroup\"",
    "70B3D5999": "LOGICUBE INC",
    "70B3D5153": "Schneider Electric Motion USA",
    "70B3D5D22": "DEK Technologies",
    "70B3D5CD1": "Cannex Technology Inc.",
    "70B3D5CC5": "Intecom",
    "70B3D5877": "Polynet Telecommunications Consulting and Contractor Ltd.",
    "70B3D52CD": "Korea Airports Corporation",
    "70B3D524B": "TOSEI ENGINEERING CORP.",
    "70B3D5A3F": "PHPower Srl",
    "70B3D52AA": "Flirtey Inc",
    "70B3D5953": "Spectrum Techniques, LLC",
    "70B3D5246": "Saline Lectronics, Inc.",
    "70B3D519B": "Global Technical Systems",
    "70B3D59D5": "Southern Tier Technologies",
    "70B3D54B2": "Certus Operations Ltd",
    "70B3D573D": "NETWAYS GmbH",
    "70B3D5F75": "Enlaps",
    "70B3D58A8": "megatec electronic GmbH",
    "70B3D5169": "Service Plus LLC",
    "70B3D5F9C": "SureFlap Ltd",
    "70B3D5432": "DEUTA-WERKE GmbH",
    "70B3D5678": "The Dini Group, La Jolla inc.",
    "70B3D50DE": "Grossenbacher Systeme AG",
    "70B3D55B8": "Hella Gutmann Solutions GmbH ",
    "70B3D52F4": "Radixon s.r.o.",
    "70B3D5B13": "Omwave",
    "70B3D5B98": "GSF Corporation Pte Ltd",
    "70B3D530D": "Fiberbase",
    "70B3D5379": "Vensi, Inc.",
    "70B3D5035": "HKW-Elektronik GmbH",
    "70B3D577B": "AeroVision Avionics, Inc.",
    "70B3D5C14": "Grupo Epelsa S.L.",
    "70B3D5374": "OOO NPP Mars-Energo",
    "70B3D5381": "CRDE",
    "70B3D5F42": "Matsuhisa Corporation",
    "70B3D5F24": "Daavlin",
    "70B3D5FA6": "RFL Electronics, Inc.",
    "70B3D5FBC": "Twoway Communications, Inc.",
    "70B3D5413": "Axess AG",
    "70B3D5B2E": "Green Access Ltd",
    "70B3D5245": "Newtec A/S",
    "70B3D5C6E": "Orion Technologies, LLC",
    "70B3D5D6C": "GP Systems GmbH",
    "70B3D5DDB": "Intra Corporation",
    "70B3D58EE": "Network Additions",
    "70B3D584D": "Quantum Design Inc.",
    "70B3D5D4D": "The Morey Corporation",
    "70B3D5C8C": "Rollogo Limited",
    "70B3D559C": "DAVE SRL",
    "70B3D5D10": "Contec Americas Inc.",
    "70B3D590F": "DTRON Communications (Pty) Ltd",
    "70B3D5CC9": "Rapiscan Systems",
    "70B3D596D": "MSB Elektronik und Ger\u00e4tebau GmbH",
    "70B3D5677": "Fraunhofer-Institut IIS",
    "70B3D5D92": "Zamir Recognition Systems Ltd.",
    "70B3D51B6": "DACOM West GmbH",
    "70B3D5108": "TEX COMPUTER SRL ",
    "70B3D5127": "VITEC",
    "70B3D58AC": "\u200bASUNG TECHNO CO.,Ltd",
    "70B3D5211": "Fracarro srl",
    "70B3D547F": "ASE GmbH",
    "70B3D50B4": "AVER",
    "70B3D5805": "Eurotronik Kranj d.o.o.",
    "70B3D57E0": "Sanko-sha,inc.",
    "70B3D56A6": "WOW System",
    "70B3D57E8": "Mannkind Corporation",
    "70B3D5C49": "BTG Instruments AB",
    "70B3D506B": "U-Tech",
    "70B3D590D": "Modtronix Engineering",
    "70B3D5DCE": "Stahl GmbH",
    "70B3D522C": "Hiquel Elektronik- und Anlagenbau GmbH",
    "70B3D52B1": "WIXCON Co., Ltd",
    "70B3D5002": "Gogo BA ",
    "70B3D5653": "Luxar Tech, Inc.",
    "70B3D5209": "SmartNodes",
    "70B3D5F12": "Incoil Induktion AB",
    "70B3D5022": "Ravelin Ltd",
    "70B3D5215": "Dataspeed Inc",
    "70B3D510F": "neQis",
    "70B3D505D": "KOMS Co.,Ltd.",
    "70B3D59C6": "Overspeed SARL",
    "70B3D54BE": "GY-FX SAS",
    "70B3D514C": "CRDE",
    "70B3D5745": "TMSI LLC",
    "70B3D57B2": "Rail Power Systems GmbH",
    "70B3D5348": "B\u00c4R Bahnsicherung AG",
    "70B3D592A": "Miravue",
    "70B3D5AD1": "Sensile Technologies SA",
    "70B3D5D58": "Idyllic Engineering Pte Ltd",
    "70B3D5CEA": "Computerwise, Inc.",
    "70B3D5511": "Next Sight srl",
    "70B3D588D": "LG Electronics",
    "70B3D51D4": "Brinkmann Audio GmbH",
    "70B3D5A18": "Embedded Systems Lukasz Panasiuk",
    "70B3D5954": "Dot System S.r.l.",
    "70B3D586E": "Profcon AB",
    "70B3D516C": "OCEAN",
    "70B3D5222": "Marioff Corporation Oy",
    "70B3D5FBF": "SenSys (Design Electronics Ltd)",
    "70B3D5CF5": "Petring Energietechnik GmbH",
    "70B3D521D": "iRF - Intelligent RF Solutions, LLC",
    "70B3D54A9": "WARECUBE,INC",
    "70B3D5E09": "L-3 communications ComCept Division",
    "70B3D5482": "Aeryon Labs Inc",
    "70B3D51C5": "ELSAG",
    "70B3D5269": "Gilbarco Veeder-Root  \u200e",
    "70B3D5272": "TELECOM SANTE",
    "70B3D58EC": "Rudy Tellert",
    "70B3D503D": "QUERCUS TECHNOLOGIES, S.L.",
    "70B3D5122": "Henri Systems Holland bv",
    "70B3D568E": "CEA Technologies Pty Ltd",
    "70B3D57F4": "KST technology",
    "70B3D5260": "ModuSystems, Inc",
    "70B3D5939": "Invertek Drives Ltd",
    "70B3D524D": "INFO CREATIVE (HK) LTD",
    "70B3D532A": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D5E1A": "BIZERBA LUCEO",
    "70B3D58CA": "Allied Data Systems",
    "70B3D5B0C": "Vigilate srl",
    "70B3D518C": "CMC Industrial Electronics Ltd",
    "70B3D52EC": "Grupo Epelsa S.L.",
    "70B3D5C3D": "CISTECH Solutions",
    "70B3D586C": "eeas gmbh",
    "70B3D59F8": "Asymmetric Technologies",
    "70B3D5289": "Shenzhen Rongda Computer Co.,Ltd",
    "70B3D5A89": "GBS COMMUNICATIONS, LLC",
    "70B3D573B": "S-I-C",
    "70B3D5041": "FIBERNET LTD",
    "70B3D5D9D": "Electroimpact, Inc.",
    "70B3D50DA": "Aquavision Distribution Ltd",
    "70B3D5DFC": "ELECTRONIC SYSTEMS DESIGN SPRL",
    "70B3D552C": "Centuryarks Ltd.,",
    "70B3D5DD8": "EMSCAN Corp. ",
    "70B3D5A1B": "Potter Electric Signal Co. LLC",
    "70B3D587E": "Septentrio NV",
    "70B3D5A5A": "RCS Energy Management Ltd",
    "70B3D5B37": "CODEC Co., Ltd.",
    "70B3D5F96": "Ecologicsense",
    "70B3D5C86": "Woodam Co., Ltd.",
    "70B3D5666": "Aplex Technology Inc.",
    "70B3D5607": "ATEME",
    "70B3D5F5C": "Nable Communications, Inc.",
    "70B3D529F": "Code Hardware SA",
    "70B3D5F76": "Thermo Fisher Scientific",
    "70B3D5D2F": "L.I.F.E. Corporation SA",
    "70B3D5163": "BHARAT HEAVY ELECTRICALS LIMITED",
    "70B3D5DB0": "Arnouse Digital Devices Corp",
    "70B3D59AE": "Volansys technologies pvt ltd",
    "70B3D5E8F": "DISMUNTEL, S.A.",
    "70B3D521E": "Hildebrand Technology Limited",
    "70B3D5BE3": "Saratov Electrounit Production Plant named after Sergo Ordzhonikidze, OJSC",
    "70B3D571B": "elsys",
    "70B3D5E30": "QUISS AG",
    "70B3D5821": "HL2 group",
    "70B3D5DDC": "Syscom Instruments SA",
    "70B3D54B0": "Tecogen Inc.",
    "70B3D589B": "ControlWorks, Inc.",
    "70B3D5854": "Adimec Advanced Image Systems",
    "70B3D561A": "Rocket Lab Ltd.",
    "70B3D5EA0": "PARK24",
    "70B3D5697": "Alazar Technologies Inc.",
    "70B3D5AE7": "E-T-A Elektrotechnische Apparate GmbH",
    "70B3D5400": "Vtron Pty Ltd",
    "70B3D5889": "Innovative Circuit Technology",
    "70B3D558D": "DORLET SAU",
    "70B3D56FF": "AKEO PLUS",
    "70B3D5FB3": "3PS Inc",
    "70B3D5684": "LECO Corporation",
    "70B3D5D5C": "Critical Link LLC",
    "70B3D5174": "Carlson Wireless Technologies Inc. ",
    "70B3D5649": "swissled technologies AG",
    "70B3D53BB": "A-M Systems",
    "70B3D5F9A": "Krabbenh\u00f8ft og Ingolfsson",
    "70B3D57FB": "db Broadcast Products Ltd",
    "70B3D5274": "Stercom Power Solutions GmbH",
    "70B3D5513": "MB connect line GmbH Fernwartungssysteme",
    "70B3D591A": "Fujian Landfone Information Technology Co.,Ltd",
    "70B3D5106": "Aplex Technology Inc.",
    "70B3D5925": "Diamante Lighting Srl",
    "70B3D5D11": "EREE Electronique",
    "70B3D5BD2": "Burk Technology",
    "70B3D57B6": "Amada Miyachi America Inc.",
    "70B3D5C63": "Xentech Solutions Limited",
    "70B3D537D": "The DX Shop Limited",
    "70B3D510C": "Vocality International Ltd",
    "70B3D5818": "CRDE",
    "70B3D53BC": "SciTronix",
    "70B3D5932": "Rohde&Schwarz Topex SA",
    "70B3D5180": "LHA Systems (Pty) Ltd",
    "70B3D5CCC": "AEC s.r.l.",
    "70B3D54E9": "ADETEC SAS",
    "70B3D5F4F": "Power Electronics Espana, S.L.",
    "70B3D5B33": "Aplex Technology Inc.",
    "70B3D5200": "NextEV Co., Ltd.",
    "70B3D53D9": "Aplex Technology Inc.",
    "70B3D5E7C": "Aplex Technology Inc.",
    "70B3D56A0": "Active Research Limited",
    "70B3D5E9B": "NUMATA R&D Co.,Ltd",
    "70B3D53B2": "Sicon srl",
    "70B3D5A59": "Muuntos\u00e4hk\u00f6 Oy - Trafox",
    "70B3D52F3": "Scame Sistemi srl",
    "70B3D5C9F": "Triax A/S",
    "70B3D5295": "Cello Electronics (UK) Ltd",
    "70B3D5F3C": "Gigaray",
    "70B3D5C1B": "Labinvent JSC",
    "70B3D597C": "Nu-Tek Power Controls and Automation",
    "70B3D5A78": "Bionics co.,ltd.",
    "70B3D5DA8": "Tagarno AS",
    "70B3D590C": "ANTEK GmbH",
    "70B3D58B1": "M-Tech Innovations Limited",
    "70B3D517D": "Entech Electronics",
    "70B3D5AB4": "SYS TEC electronic GmbH",
    "70B3D502A": "BAE Systems Surface Ships Limited",
    "70B3D5528": "Aplex Technology Inc.",
    "70B3D5742": "YUYAMA MFG Co.,Ltd",
    "70B3D5BA7": "Digital Yacht Ltd",
    "70B3D5FE2": "Galileo T\u0131p Teknolojileri San. ve Tic. A.S.",
    "70B3D5EE1": "allora Factory BVBA",
    "70B3D50B9": "Easy Digital Concept",
    "70B3D528B": "Arnouse Digital Devices, Corp.",
    "70B3D53A9": "Vivalnk",
    "70B3D52E7": "Atos spa",
    "70B3D5772": "enModus",
    "70B3D5392": "Contec Americas Inc.",
    "70B3D5232": "UCONSYS ",
    "70B3D5B5C": "Prozess Technologie",
    "70B3D5046": "Shenzhen Rihuida Electronics Co,. Ltd",
    "70B3D58B0": "IES S.r.l.",
    "70B3D5F55": "Kohler Mira Ltd",
    "70B3D5E1C": "RoomMate AS",
    "70B3D5FD2": "DALIAN  LEVEAR ELECTRIC  CO., LTD",
    "70B3D526B": "Sorama BV",
    "70B3D5091": "PROFITT Ltd",
    "70B3D55D1": "Software Motor Corp",
    "70B3D5384": "Sensohive Technologies",
    "70B3D528A": "Transit Solutions, LLC.",
    "70B3D5A6F": "8Cups",
    "70B3D5847": "Ai-Lynx",
    "70B3D55A0": "Ascon Tecnologic S.r.l.",
    "70B3D5DB4": "YUYAMA MFG Co.,Ltd",
    "70B3D5124": "Forschungs- und Transferzentrum Leipzig e.V.",
    "70B3D552E": "Swissponic Sagl",
    "70B3D5D1B": "Grupo Epelsa S.L.",
    "70B3D569E": "PTYPE Co., LTD.",
    "70B3D5AFE": "MESOTECHNIC",
    "70B3D56A9": "OHMORI ELECTRIC INDUSTRIES CO.LTD",
    "70B3D54A1": "Herholdt Controls srl",
    "70B3D5D81": "PDD Group Ltd",
    "70B3D5326": "NEMEUS-SAS",
    "70B3D5FE6": "SHIZUKI ELECTRIC CO.,INC",
    "70B3D58AB": "EMAC, Inc.",
    "70B3D50C8": "Fin Robotics Inc",
    "70B3D5694": "MoviTHERM",
    "70B3D5033": "Sailmon BV",
    "70B3D5570": "Bayern Engineering GmbH & Co. KG",
    "70B3D5645": "Project Decibel, Inc.",
    "70B3D5DE0": "eCozy GmbH",
    "70B3D525B": "GID Industrial",
    "70B3D53E9": "APOLLO GIKEN Co.,Ltd.",
    "70B3D5E77": "OPTIX JSC",
    "70B3D514E": "Innosonix GmbH",
    "70B3D5407": "IDOSENS",
    "70B3D5B3E": "Paradigm Communication Systems Ltd",
    "70B3D53C0": "DK-Technologies A/S",
    "70B3D5364": "ADAMCZEWSKI elektronische Messtechnik GmbH",
    "70B3D5404": "RANIX,Inc.",
    "70B3D56F9": "ENVItech s.r.o.",
    "70B3D559D": "servicios de consultoria independiente S.L.",
    "70B3D5FCF": "Acc+Ess Ltd",
    "70B3D501E": "ePOINT Embedded Computing Limited",
    "70B3D5E35": "Nanospeed Technologies Limited",
    "70B3D59F0": "FUJICOM Co.,Ltd.",
    "70B3D5554": "Teletypes Manufacturing Plant",
    "70B3D5A01": "FeldTech GmbH",
    "70B3D5ADA": "Private",
    "70B3D5835": "CommBox P/L",
    "70B3D588F": "Quaesta Instruments, LLC",
    "70B3D5DC0": "ATEME",
    "70B3D5C88": "SINED srl",
    "70B3D5866": "MEPS Realtime",
    "70B3D59C8": "Applied Systems Engineering, Inc.",
    "70B3D5D8C": "Damerell Design Limited (DCL)",
    "70B3D5C21": "Aplex Technology Inc.",
    "70B3D55A9": "Bunka Shutter Co., Ltd.",
    "70B3D52BC": "EQUIPOS DE TELECOMUNICACI\u00d3N OPTOELECTR\u00d3NICOS, S.A.",
    "70B3D54BA": "Sinftech LLC",
    "70B3D582E": "PlayAlive A/S",
    "70B3D5B85": "Fenotech Inc.",
    "70B3D5D8D": "Pullnet Technology,S.L.",
    "70B3D51F3": "Smart Energy Code Company Limited",
    "70B3D5827": "Metromatics Pty Ltd",
    "70B3D52A7": "Plasmability, LLC",
    "70B3D50D8": "Laser Imagineering GmbH",
    "70B3D5A27": "HDL da Amaz\u00f4nia Industria Eletr\u00f4nica Ltda",
    "70B3D56D1": "Visual Engineering Technologies Ltd",
    "70B3D53E2": "AVI Pty Ltd",
    "70B3D5FE8": "PCME Ltd.",
    "70B3D5BAE": "WARECUBE,INC",
    "70B3D5C3C": "PEEK TRAFFIC",
    "70B3D5BAD": "Technik & Design GmbH",
    "70B3D5F4C": "PolyTech A/S",
    "70B3D5AD5": "Birdland Audio",
    "70B3D55B0": "Qxperts Italia S.r.l.",
    "70B3D5E21": "LLVISION TECHNOLOGY CO.,LTD",
    "70B3D50CD": "AML Oceanographic",
    "70B3D54AA": "Twoway Communications, Inc.",
    "70B3D5AA8": "West-Com Nurse Call Systems, Inc.",
    "70B3D5EAC": "Kentech Instruments Limited",
    "70B3D58AD": "Global Communications Technology LLC",
    "70B3D5134": "Conjing Networks Inc.",
    "70B3D5A2A": "Redwood Systems",
    "70B3D56ED": "Wiingtech International Co. LTD.",
    "70B3D520E": "Amrehn & Partner EDV-Service GmbH",
    "70B3D579E": "CW2. Gmbh & Co. KG",
    "70B3D5341": "Vtron Pty Ltd",
    "70B3D5F5A": "HAMEG GmbH",
    "70B3D5799": "Vitec System Engineering Inc.",
    "70B3D5917": "KSJ Co.Ltd",
    "70B3D5E23": "Smith Meter, Inc.",
    "70B3D5BD3": "FOTONA D.D.",
    "70B3D56A5": "Akenori PTE LTD",
    "70B3D5AB9": "Dynamic Controls",
    "70B3D541E": "Redler Computers",
    "70B3D5A00": "ATX NETWORKS LTD",
    "70B3D5DF6": "Tiab Limited",
    "70B3D5EFB": "PXM sp.k.",
    "70B3D54F0": "Li Seng Technology Ltd.,",
    "70B3D5F39": "Zenros ApS",
    "70B3D5C56": "TELETASK",
    "70B3D599F": "Confed Holding B.V.",
    "70B3D52BE": "Coherent Logix, Inc.",
    "70B3D5796": "GAMPT mbH",
    "70B3D522E": "Private",
    "70B3D5030": "Tresent Technologies",
    "70B3D56E4": "Institute of Power Engineering, Gdansk Division",
    "70B3D5E6E": "Lieron BVBA",
    "70B3D575D": "Nanjing Magewell Electronics Co., Ltd.",
    "70B3D5BE1": "FeCon GmbH",
    "70B3D5660": "Smart Service Technologies CO., LTD",
    "70B3D526E": "HI-TECH SYSTEM Co. Ltd.",
    "70B3D5C39": "MeshWorks Wireless Oy",
    "70B3D55E9": "Zehetner-Elektronik GmbH",
    "70B3D59F1": "RFEL Ltd",
    "70B3D565B": "Roush",
    "70B3D53DA": "Loop Labs, Inc.",
    "70B3D50C1": "Nexus Technologies Pty Ltd",
    "70B3D59F5": "Vickers Electronics Ltd",
    "70B3D5D0C": "Connor Winfield LTD",
    "70B3D5B82": "Lookout Portable Security",
    "70B3D539A": "Videotrend srl",
    "70B3D58E1": "WoKa-Elektronik GmbH",
    "70B3D58E0": "SOUDAX EQUIPEMENTS",
    "70B3D5502": "Glidewell Laboratories",
    "70B3D5A6D": "Metek Meteorologische Messtechnik GmbH",
    "70B3D50FF": "INTERNET PROTOCOLO LOGICA SL",
    "70B3D5882": "SIMON TECH, S.L.",
    "70B3D548D": "OMEGA BILANCE SRL SOCIETA' UNIPERSONALE",
    "70B3D508E": "Beijing CONvision Technology Co.,Ltd",
    "70B3D58DC": "Niveo International BV",
    "70B3D55F0": "managee GmbH & Co KG",
    "70B3D555D": "LunaNexus Inc",
    "70B3D536D": "Cyberteam Sp z o o",
    "70B3D5C6D": "Cyviz AS",
    "70B3D5DA1": "Qprel srl",
    "70B3D53E5": "ATEME",
    "70B3D5710": "Guardian Controls International Ltd",
    "70B3D5AEE": "DiTEST Fahrzeugdiagnose GmbH",
    "70B3D5ECF": "Ipitek",
    "70B3D5A40": "STRACK LIFT AUTOMATION GmbH",
    "70B3D5BCA": "Deymed Diagnostic",
    "70B3D543D": "Veryx Technologies Private Limited",
    "70B3D5B16": "XI'AN SHENMING ELECTRON TECHNOLOGY CO.,LTD",
    "70B3D5C96": "UNI DIMENXI SDN BHD",
    "70B3D55B5": "Lehigh Electric Products Co",
    "70B3D5BBD": "Providius Corp",
    "70B3D586D": "Census Digital Incorporated",
    "70B3D5F2A": "WIBOND Informationssysteme GmbH",
    "70B3D5A2F": "Botek Systems AB",
    "70B3D5B15": "Eta Beta Srl",
    "70B3D5C27": " GD Mission Systems",
    "70B3D59A7": "Honeywell",
    "70B3D5DDD": "BIO RAD LABORATORIES",
    "70B3D5C8D": "KST technology",
    "70B3D5871": "Oso Technologies",
    "70B3D50FB": "Cygnus LLC",
    "70B3D5918": "Glova Rail A/S",
    "001BC50C9": "UAB Kitron",
    "001BC50C5": "Gill Instruments Ltd",
    "001BC50BA": "NT MICROSYSTEMS",
    "001BC50BD": "Bridge Diagnostics, Inc.",
    "001BC50AD": "Tierra Japan Co.,Ltd",
    "001BC50AF": "Enerwise Solutions Ltd.",
    "001BC50A9": "Elektrometal SA",
    "001BC50AC": "AVnu Alliance",
    "001BC50A8": "Link Precision",
    "001BC50A5": "Tesla Controls",
    "001BC50A4": "RADMOR S.A.",
    "001BC50A7": "L.G.L. Electronics S.p.a.",
    "001BC509E": "K+K Messtechnik GmbH",
    "001BC509B": "YIK Corporation",
    "001BC5097": "Plexstar Inc.",
    "001BC5091": "3green ApS",
    "001BC5090": "Seven Solutions S.L",
    "001BC5086": "CAST Group of Companies Inc.",
    "001BC5084": "Applied Innovations Research LLC",
    "001BC507A": "Servicios Electronicos Industriales Berbel s.l.",
    "001BC507B": "QCORE Medical",
    "001BC5078": "Donbass Soft Ltd and Co.KG",
    "001BC5076": "PLAiR Media, Inc",
    "001BC506A": "IST GmbH",
    "001BC5066": "Manufacturas y transformados AB",
    "001BC5059": "INPIXAL",
    "001BC5050": "TeliSwitch Solutions",
    "001BC5053": "Metrycom Communications Ltd",
    "001BC5052": "Engineering Center ENERGOSERVICE",
    "001BC5056": "ThinKom Solutions, Inc",
    "001BC504D": "eiraku electric corp.",
    "001BC5048": "XPossible Technologies Pte Ltd",
    "001BC5044": "ZAO &quot;RADIUS Avtomatika&quot;",
    "001BC5043": "Coincident, Inc.",
    "001BC503A": "MindMade Sp. z o.o.",
    "001BC502E": "BETTINI SRL",
    "001BC500F": "Simavita Pty Ltd",
    "001BC5000": "Converging Systems Inc.",
    "001BC5002": "GORAMO - Janusz Gorecki",
    "8C1F64AFE": "Motec USA, Inc.",
    "8C1F64C92": "EQ Earthquake Ltd.",
    "8C1F64D34": "KRONOTECH SRL",
    "70B3D5B2C": "Elman srl",
    "8C1F64392": "mmc kommunikationstechnologie gmbh",
    "8C1F6486F": "NewEdge Signal Solutions LLC",
    "8C1F648FF": "Kruger DB Series Ind\u00fastria Eletr\u00f4nica ltda",
    "8C1F64EBA": "Hyve Solutions",
    "8C1F64BAA": "Mine Vision Systems",
    "8C1F6424D": "XI'AN JIAODA KAIDA NEW TECHNOLOGY CO.LTD",
    "8C1F64B64": "Televic Rail GmbH",
    "70B3D593C": "Televic Rail GmbH",
    "70B3D52A4": "Televic Rail GmbH",
    "8C1F647DA": "Xpti Tecnologias em Seguran\u00e7a Ltda",
    "8C1F644A2": "Bludigit SpA",
    "8C1F6453C": "Filgis Elektronik",
    "8C1F64A33": "RT Vision Technologies PVT LTD",
    "8C1F64962": "Umano Medical Inc.",
    "8C1F649DB": "HD Renewable Energy Co.,Ltd",
    "8C1F64804": "EA Elektro-Automatik",
    "8C1F6415A": "ASHIDA Electronics Pvt. Ltd",
    "8C1F643C1": "Suzhou Lianshichuangzhi Technology Co., Ltd",
    "8C1F64246": "Oriux",
    "8C1F641D3": "Opus-Two ICS",
    "8C1F64441": "Novanta IMS",
    "8C1F64410": "Roboteq",
    "8C1F64623": "Ryoyu-GC Co.,Ltd",
    "8C1F644C9": "Apantac LLC",
    "8C1F6496A": "EA Elektro-Automatik",
    "8C1F640ED": "Saskatchewan Research Council",
    "8C1F64FC1": "Nidec asi spa",
    "8C1F64094": "EL.EN. SPA",
    "8C1F64AF4": "Nokia Bell Labs",
    "8C1F64170": "Fracarro Radioindustrie Srl",
    "8C1F642CE": "E2 Nova Corporation",
    "8C1F6491C": "Cospowers Changsha Branch",
    "8C1F64E1A": "DAccess Security Systems P Ltd",
    "8C1F640AD": "E2 Nova Corporation",
    "8C1F64C3F": "SONIC CORPORATION",
    "8C1F644CF": "Sicon srl",
    "8C1F64082": "Zhongcheng Technology Co.,Ltd",
    "8C1F6434F": "Systec Designs BV",
    "8C1F641E6": "Radian Research, Inc.",
    "8C1F64876": "fmad engineering",
    "8C1F64149": "Clock-O-Matic",
    "8C1F64608": "CONG TY CO PHAN KY THUAT MOI TRUONG VIET AN",
    "001BC50C6": "CyanConnode",
    "8C1F6462E": "ViewSonic Corp",
    "8C1F64060": "Zadar Labs Inc",
    "8C1F64987": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F64FF8": "Chamsys Ltd",
    "8C1F64631": "HAIYANG OLIX CO.,LTD.",
    "8C1F64CDD": "The Signalling Company",
    "8C1F64F14": "Elektrosil GmbH",
    "8C1F64146": "Zhongrun Xinchan (Beijing) Technology Co., Ltd",
    "8C1F6473E": "Beijing LJ Technology Co., Ltd.",
    "8C1F64644": "DAVE SRL",
    "8C1F6411E": "Infosoft Digital Design and Services P L",
    "8C1F64035": "RealWear",
    "8C1F648CB": "CHROMAVISO A/S",
    "8C1F64BB7": "JIANGXI LV C-CHONG CHARGING TECHNOLOGY CO.LTD",
    "8C1F646BD": "IoT Water Analytics S.L.",
    "8C1F64858": "SFERA srl",
    "8C1F647F4": "G.M. International srl",
    "8C1F64C08": "Triamec Motion AG",
    "8C1F64738": "ssolgrid",
    "8C1F64BF6": "Panoramic Power",
    "8C1F64BCE": "BESO sp. z o.o.",
    "8C1F647FC": "Mitsubishi Electric Klimat Transportation Systems S.p.A.",
    "8C1F64D77": "Private",
    "8C1F6425F": "Acuris Inc",
    "8C1F6422D": "KAYSONS ELECTRICALS PRIVATE LIMITED",
    "8C1F64755": "Flextronics International Kft",
    "8C1F640A0": "TECHNIWAVE",
    "8C1F649D3": "EA Elektro-Automatik",
    "8C1F649F1": "Skymira",
    "8C1F6488C": "SAL Navigation AB",
    "8C1F64A61": "Breas Medical AB",
    "8C1F64076": "PACK'R",
    "8C1F64A12": "FUJIHENSOKUKI Co., Ltd.",
    "8C1F64F7F": "Vision Systems Safety Tech",
    "8C1F645FB": "RECOM LLC.",
    "8C1F64DC8": "DWDM.RU LLC",
    "8C1F6463C": "GALIOS",
    "8C1F64E8A": "Changzhou MITO electronics Technology Co;LTD",
    "8C1F643BD": "Oriux",
    "8C1F64B4F": "Vaunix Technology Corporation",
    "8C1F64C5E": "YUYAMA MFG Co.,Ltd",
    "8C1F64685": "Sanchar Wireless Communications Ltd",
    "8C1F6466E": "Monnit Corporation",
    "8C1F647F2": "AT-Automation Technology GmbH",
    "8C1F64D23": "PLX Inc.",
    "8C1F64E42": "Shenzhen Forddok Technology Co., Ltd",
    "8C1F646BF": "Automata GmbH & Co. KG",
    "8C1F640DF": "Leidos",
    "8C1F64F97": "Dentalhitec",
    "8C1F64A47": "Saarni Cloud Oy",
    "8C1F644E4": "Nuvation Energy",
    "8C1F648B7": "DA-Design Oy",
    "8C1F64FEC": "Newtec A/S",
    "8C1F64936": "Jiangsu Eman Electronic Technology Co., Ltd",
    "8C1F64CF9": "VeoTech",
    "8C1F645B7": "Vortex Sp. z o.o.",
    "8C1F6451A": "TELE Haase Steuerger\u00e4te Ges.m.b.H",
    "8C1F64F1C": "Rigel Engineering, LLC",
    "70B3D5AAF": "Exi Flow Measurement Ltd",
    "8C1F64993": "Applied Electro Magnetics Pvt. Ltd.",
    "8C1F64C56": "Eridan",
    "8C1F64C9F": "PeachCreek",
    "8C1F64DCF": "REO AG",
    "8C1F646D4": "Hitachi Energy Poland sp. Z o o",
    "8C1F64185": "BIOTAGE GB LTD",
    "8C1F649CA": "EDC Acoustics",
    "8C1F64368": "Crosstek Co., Ltd",
    "8C1F645C6": "Systems With Intelligence Inc.",
    "70B3D55CC": "AKSE srl",
    "70B3D5128": "AKSE srl",
    "8C1F643C0": "SNEK",
    "8C1F641F4": "EIFFAGE ENERGIE ELECTRONIQUE",
    "8C1F64778": "ERS Elektronik GmbH",
    "8C1F64482": "Vismes sarl",
    "8C1F64D63": "Mobileye",
    "70B3D590E": "Maytronics Ltd.",
    "8C1F64201": "Hiwin Mikrosystem Corp.",
    "8C1F64DDB": "Efficient Residential Heating GmbH",
    "8C1F645BE": "Benchmark Electronics BV",
    "8C1F6487B": "JSE s.r.o.",
    "8C1F64F33": "Sicon srl",
    "70B3D588E": "RCH SPA",
    "70B3D56BD": "RCH SPA",
    "8C1F643C8": "BTG Instruments AB",
    "70B3D55EF": "Star Systems International Limited",
    "8C1F64E4D": "San Telequip (P) Ltd.,",
    "8C1F642F2": "ENLESS WIRELESS",
    "8C1F64F87": "Fly Electronic (Shang Hai) Technology Co.,Ltd",
    "8C1F64C7B": "Freedom Atlantic",
    "8C1F645B0": "Sonel S.A.",
    "8C1F649E5": "Schunk Sonosystems GmbH",
    "8C1F64598": "TIRASOFT TECHNOLOGY",
    "8C1F64F91": "Consonance",
    "70B3D5F86": "NxGen Comm LLC",
    "8C1F640CA": "CLOUD TELECOM Inc.",
    "8C1F642A8": "SHALARM SECURITY Co.,LTD",
    "8C1F648B8": "Wien Energie GmbH",
    "8C1F64476": "Clair Global Corporation",
    "8C1F64B68": "All-Systems Electronics Pty Ltd",
    "8C1F64C4A": "SGi Technology Group Ltd.",
    "8C1F64C0D": "Abbott Diagnostics Technologies AS",
    "8C1F64A3F": "ViewSonic Corp",
    "8C1F64501": "QUISS GmbH",
    "8C1F64EE0": "Private",
    "8C1F64D24": "R3 IoT Ltd.",
    "70B3D54DA": "RADA Electronics Industries Ltd.",
    "8C1F648A8": "Massachusetts Institute of Technology",
    "8C1F64924": "Magics Technologies",
    "8C1F64880": "MB connect line GmbH Fernwartungssysteme",
    "8C1F64C12": "PHYSEC GmbH",
    "8C1F64B2F": "Mtechnology - Gamma Commerciale Srl",
    "8C1F6420C": "Shanghai Stairmed Technology Co.,ltd",
    "8C1F64D81": "Mitsubishi Electric India Pvt. Ltd.",
    "8C1F643D0": "TRIPLTEK",
    "8C1F64AD4": "Flextronics International Kft",
    "8C1F64461": "Kara Partners LLC",
    "8C1F64B36": "Pneumax Spa",
    "8C1F64051": "CP contech electronic GmbH",
    "8C1F64C6A": "Red Phase Technologies Limited",
    "70B3D5580": "Private",
    "8C1F6452A": "Hiwin Mikrosystem Corp.",
    "8C1F6409E": "IWS Global Pty Ltd",
    "8C1F649F6": "Vision Systems Safety Tech",
    "8C1F64E24": "COMETA SAS",
    "8C1F64BBC": "Liberator Pty Ltd",
    "70B3D5278": "Medicomp, Inc",
    "8C1F640F5": "Vishay Nobel AB",
    "8C1F64113": "Timberline Manufacturing",
    "8C1F64289": "Craft4 Digital GmbH",
    "8C1F64281": "NVP TECO LTD",
    "8C1F64B7D": "Scheurich GmbH",
    "8C1F64EDA": "DEUTA-WERKE GmbH",
    "70B3D58AF": "QBIC COMMUNICATIONS DMCC",
    "8C1F646CB": "GJD Manufacturing",
    "8C1F64830": "Vtron Pty Ltd",
    "8C1F64329": "YUYAMA MFG Co.,Ltd",
    "8C1F64439": "BORNICO",
    "8C1F645DA": "White2net srl",
    "8C1F645C9": "Abbott Diagnostics Technologies AS",
    "8C1F64D9C": "Relcom, Inc.",
    "8C1F64350": "biosilver .co.,ltd",
    "8C1F64820": "TIAMA",
    "70B3D5BA3": "TIAMA",
    "8C1F643AF": "PSA Technology Ltda.",
    "8C1F64412": "Comercial Electronica Studio-2 s.l.",
    "8C1F64349": "WAVES SYSTEM",
    "8C1F64C64": "Ajeco Oy",
    "8C1F64033": "IQ Home Kft.",
    "8C1F647D4": "Penteon Corporation ",
    "8C1F64CE4": "SL USA, LLC",
    "8C1F640AA": "DI3 INFOTECH LLP",
    "8C1F640F2": "Graphimecc Group SRL",
    "8C1F646D9": "Khimo",
    "8C1F64928": "ITG Co.Ltd",
    "8C1F64DC2": "Procon Electronics Pty Ltd",
    "8C1F641B2": "Rapid-e-Engineering Steffen Kramer",
    "8C1F649BF": "ArgusEye TECH. INC",
    "8C1F64B69": "Quanxing Tech Co.,LTD",
    "8C1F64C81": "Taolink Technologies Corporation",
    "8C1F6405C": "tickIoT Inc.",
    "8C1F642D8": "CONTROL SYSTEMS Srl",
    "8C1F645F7": "Eagle Harbor Technologies, Inc.",
    "8C1F64EA8": "Zumbach Electronic AG",
    "8C1F64F09": "Texi AS",
    "70B3D5CBD": "PREO INDUSTRIES FAR EAST LTD",
    "8C1F64842": "Potter Electric Signal Co. LLC",
    "8C1F6488B": "Taiwan Aulisa Medical Devices Technologies, Inc",
    "8C1F649B3": "B\u00f6ckelt GmbH",
    "8C1F64CC1": "VITREA Smart Home Technologies Ltd.",
    "8C1F64518": "Wagner Group GmbH",
    "8C1F64700": "QUANTAFLOW",
    "8C1F6458E": "Novanta IMS",
    "8C1F6456C": "ELTEK SpA",
    "8C1F64E46": "Nautel LTD",
    "8C1F641C0": "INVENTIA Sp. z o.o.",
    "8C1F645EA": "BTG Instruments AB",
    "8C1F6465D": "Action Streamer LLC",
    "8C1F644DB": "Private",
    "70B3D5E2D": "BAE Systems Apllied Intelligence",
    "8C1F6444F": "RealD, Inc.",
    "8C1F64A00": "BITECHNIK GmbH",
    "8C1F6439A": "Golding Audio Ltd",
    "8C1F64A60": "Active Optical Systems, LLC",
    "8C1F64F39": "Weinan Wins Future Technology Co.,Ltd",
    "70B3D5E34": "Gamber Johnson-LLC",
    "8C1F641B7": "Rax-Tech International",
    "8C1F64553": "ENIGMA SOI Sp. z o.o.",
    "8C1F64DAF": "Zhuhai Lonl electric Co.,Ltd",
    "8C1F648C5": "NextT Microwave Inc",
    "8C1F6467C": "Ensto Protrol AB",
    "8C1F64133": "Vtron Pty Ltd",
    "8C1F64F3B": " Beijing REMANG Technology Co., Ltd.",
    "8C1F64B0F": "HKC Security Ltd.",
    "8C1F64F3D": "Byte Lab Grupa d.o.o.",
    "8C1F64740": "Norvento Tecnolog\u00eda, S.L.",
    "8C1F6416D": "Xiamen Rgblink Science & Technology Co., Ltd.",
    "8C1F643CD": "Sejong security system Cor.",
    "8C1F64511": "Control Aut Tecnologia em Automa\u00e7\u00e3o LTDA",
    "8C1F64FE2": "VUV Analytics, Inc.",
    "8C1F647B6": "KEYLINE S.P.A.",
    "8C1F641B5": "Xicato",
    "8C1F64F12": "CAITRON GmbH",
    "8C1F64D2A": "Anteus Kft.",
    "8C1F647B5": "Guan Show Technologe Co., Ltd.",
    "8C1F645DB": "GlobalInvacom",
    "8C1F6414B": "Potter Electric Signal Company",
    "8C1F6454A": "Belden India Private Limited",
    "8C1F64968": "IAV ENGINEERING SARL",
    "8C1F64FC2": "I/O Controls",
    "70B3D566A": "Nomadic",
    "8C1F64F5B": "SemaConnect, Inc",
    "8C1F64043": "AperNet, LLC",
    "8C1F64268": "Astro Machine Corporation",
    "8C1F64DFE": "Nuvation Energy",
    "8C1F64301": "Agar Corporation Inc.",
    "8C1F64746": "Sensus Healthcare",
    "8C1F64A2D": "ACSL Ltd.",
    "8C1F64197": "TEKVOX, Inc",
    "8C1F6445D": "  Fuzhou Tucsen Photonics Co.,Ltd",
    "8C1F64426": "eumig industrie-TV GmbH.",
    "8C1F645E5": "Telemetrics Inc.",
    "8C1F6467F": "Hamamatsu Photonics K.K.",
    "8C1F64E0E": "Nokeval Oy",
    "8C1F64C6B": "Mediana",
    "8C1F64D08": "Power Electronics Espana, S.L.",
    "8C1F64C80": "VECOS Europe B.V.",
    "8C1F643D1": "EMIT GmbH",
    "8C1F64C8F": "JW Froehlich Maschinenfabrik GmbH",
    "8C1F64721": "M/S MILIND RAMACHANDRA RAJWADE",
    "8C1F64939": "SPIT Technology, Inc",
    "8C1F642C3": "TeraDiode / Panasonic",
    "8C1F645B3": "eumig industrie-TV GmbH.",
    "8C1F64D7C": "QUERCUS TECHNOLOGIES, S.L.",
    "8C1F64552": "Proterra, Inc",
    "8C1F64E4C": "TTC TELEKOMUNIKACE, s.r.o.",
    "8C1F649BA": "WINTUS SYSTEM",
    "8C1F64905": "Qualitrol LLC",
    "8C1F64489": "HUPI",
    "8C1F64F52": "AMF Medical SA",
    "8C1F64A84": "Beijing Wenrise Technology Co., Ltd.",
    "8C1F6401E": "SCIREQ Scientific Respiratory Equipment  Inc",
    "8C1F6467A": "MG s.r.l.",
    "8C1F64F4E": "ADAMCZEWSKI elektronische Messtechnik GmbH",
    "70B3D5679": "EMAC, Inc.",
    "8C1F647AA": "XSENSOR Technology Corp.",
    "8C1F6489E": "Cinetix Srl",
    "8C1F64D92": "Mitsubishi Electric India Pvt. Ltd.",
    "8C1F64A97": "Integer.pl S.A.",
    "8C1F649BD": "ATM SOLUTIONS",
    "8C1F641CB": "SASYS e.K.",
    "8C1F6494E": "Monnit Corporation",
    "8C1F64DD5": "Cardinal Scales Manufacturing Co",
    "8C1F64504": "EA Elektroautomatik GmbH & Co. KG",
    "8C1F64837": "runZero, Inc",
    "8C1F64525": "United States Technologies Inc.",
    "8C1F64FD4": "EMBSYS SISTEMAS EMBARCADOS",
    "8C1F644E0": "PuS GmbH und Co. KG",
    "8C1F64DC0": "Pigs Can Fly Labs LLC",
    "8C1F6461F": "Lightworks GmbH",
    "8C1F64A9A": "Signasystems Elektronik San. ve Tic. Ltd. Sti.",
    "8C1F640D6": "AVD INNOVATION LIMITED",
    "8C1F64B56": "Arcvideo",
    "8C1F641BF": "Ossia Inc",
    "8C1F64774": "navXperience GmbH",
    "8C1F64655": "S.E.I. CO.,LTD.",
    "8C1F6472A": "DORLET SAU",
    "8C1F6409F": "MB connect line GmbH Fernwartungssysteme",
    "8C1F6455E": "HANATEKSYSTEM",
    "8C1F640B8": "Signatrol Ltd",
    "8C1F64118": "Automata GmbH & Co. KG",
    "8C1F64FCD": "elbit systems - EW and sigint - Elisra",
    "8C1F64C28": "Tornado Spectral Systems Inc.",
    "8C1F64B4C": "Picocom Technology Ltd ",
    "8C1F64777": "Sicon srl",
    "8C1F64029": "Hunan Shengyun Photoelectric Technology Co.,LTD",
    "8C1F64C0C": "GIORDANO CONTROLS SPA",
    "8C1F646F4": "Elsist Srl",
    "8C1F646B3": "Feritech Ltd.",
    "70B3D5E00": "Jeaway CCTV Security Ltd,.",
    "8C1F64D52": "Critical Software SA",
    "8C1F64B3D": "RealD, Inc.",
    "8C1F64472": "Surge Networks, Inc.",
    "70B3D5D0E": "Beijing Aumiwalker technology CO.,LTD",
    "8C1F6456D": "ACOD",
    "8C1F6470E": "OvercomTech",
    "8C1F648AA": "Forever Engineering Systems Pvt. Ltd.",
    "8C1F6453F": "Velvac Incorporated",
    "8C1F64CBE": "Circa Enterprises Inc",
    "8C1F64E21": "LG-LHT Aircraft Solutions GmbH",
    "8C1F64A94": "Future wave ultra tech Company ",
    "8C1F647DD": "TAKASAKI KYODO COMPUTING CENTER Co.,LTD.",
    "8C1F64F04": "IoTSecure, LLC",
    "8C1F648AC": "BOZHON Precision Industry Technology Co.,Ltd",
    "8C1F6492D": "IVOR Intelligent Electrical Appliance Co., Ltd",
    "8C1F64DAA": "Davetech Limited",
    "8C1F64CD6": "USM Pty Ltd",
    "8C1F648B5": "Ashton Bentley Collaboration Spaces",
    "8C1F6475F": "ASTRACOM Co. Ltd",
    "8C1F641C2": "Solid Invent Ltda.",
    "8C1F64227": "Digilens",
    "8C1F64AEF": "Scenario Automation",
    "8C1F6483C": "Xtend Technologies Pvt Ltd",
    "8C1F64DE1": "Franke Aquarotter GmbH",
    "8C1F6411F": "NodeUDesign",
    "8C1F64C7C": "MERKLE Schweissanlagen-Technik GmbH",
    "8C1F644B0": "U -MEI-DAH INT'L ENTERPRISE CO.,LTD.",
    "8C1F64998": "EVLO Stockage \u00c9nergie",
    "8C1F64542": "Landis+Gyr Equipamentos de Medi\u00e7\u00e3o Ltda",
    "8C1F641E3": "WBNet",
    "8C1F64D4A": "Caproc Oy",
    "8C1F64D3C": "\"KIB Energo\" LLC",
    "8C1F642C8": "BRS Sistemas Eletr\u00f4nicos",
    "8C1F64A4E": "Syscom Instruments SA",
    "8C1F642A5": "Nonet Inc",
    "8C1F64575": "Yu-Heng Electric Co., LTD ",
    "8C1F649CF": "ASAP Electronics GmbH",
    "8C1F64372": "WINK Streaming",
    "8C1F647A6": "OTMetric",
    "8C1F64EAC": "Miracle Healthcare, Inc.",
    "8C1F64166": "Hikari Alphax Inc.",
    "8C1F64C2F": "Power Electronics Espana, S.L.",
    "8C1F64A29": "Ringtail Security",
    "8C1F640A8": "SamabaNova Systems",
    "8C1F643C4": "NavSys Technology Inc.",
    "8C1F644DD": "Griffyn Robotech Private Limited",
    "8C1F64000": "Suzhou Xingxiangyi Precision Manufacturing Co.,Ltd.",
    "8C1F64391": "CPC (UK)",
    "8C1F64FD1": "Edgeware AB",
    "8C1F64466": "Intamsys Technology Co.Ltd",
    "70B3D5A0C": "Lumiplan Duhamel",
    "70B3D51ED": "SUS Corporation",
    "70B3D5F40": "HORIZON.INC",
    "70B3D5333": "Orlaco Products B.V.",
    "70B3D54D0": "Codewerk GmbH",
    "70B3D511E": "KBPR LLC",
    "70B3D596A": "Anello Photonics",
    "70B3D54ED": "Panoramic Power",
    "70B3D5DD0": "Deep Secure Limited",
    "70B3D5411": "Mi-Fi Networks Pvt Ltd",
    "70B3D5EAA": "Druck Ltd.",
    "70B3D5E5F": "CesiumAstro Inc.",
    "70B3D55B3": "STENTORIUS by ADI",
    "70B3D5474": "CTROGERS LLC",
    "70B3D55B9": "EIZO RUGGED SOLUTIONS",
    "70B3D5DEF": "ISG Nordic AB",
    "70B3D571A": "MB connect line GmbH Fernwartungssysteme",
    "70B3D52C5": "MECT SRL",
    "70B3D5BC8": "Loma Systems s.r.o.",
    "70B3D5196": "YUYAMA MFG Co.,Ltd",
    "70B3D58E9": "COONTROL Tecnologia em Combust\u00e3o LTDA EPP",
    "70B3D5E42": "Neusoft Reach Automotive Technology (Shenyang) Co.,Ltd",
    "70B3D5829": "Guan Show Technologe Co., Ltd.",
    "70B3D580A": "SENSING LABS",
    "70B3D5581": "Thermokon Sensortechnik GmbH",
    "70B3D5D52": "Sensoronic Co.,Ltd",
    "70B3D50E4": "Walter M\u00fcller AG",
    "70B3D502C": "Iylus Inc.",
    "70B3D5373": "Hangzhou Weimu Technology Co.,Ltd.",
    "70B3D54D9": "Coda Octopus Products Limited",
    "70B3D5CA0": "Xirgo Technologies LLC",
    "70B3D5A02": "GreenFlux",
    "70B3D5CEB": "Xirgo Technologies LLC",
    "70B3D58E5": "Shanghai Armour Technology Co., Ltd.",
    "70B3D5E13": "Suzhou ZhiCai Co.,Ltd.",
    "70B3D5B27": "Naval Group",
    "70B3D5A16": "devAIs s.r.l.",
    "70B3D5DB9": "PULOON Tech",
    "70B3D581C": "QIT Co., Ltd.",
    "70B3D5C30": "Polskie Sady Nowe Podole Sp. z o.o.",
    "70B3D531A": "Terratel Technology s.r.o.",
    "70B3D5DCB": "MIJIENETRTECH CO.,LTD",
    "70B3D5C28": "Mitech Integrated Systems Inc.",
    "70B3D5CC7": "SOtM",
    "70B3D5540": "KMtronic ltd",
    "70B3D5F9D": "Teledyne API",
    "70B3D5A61": "Omsk Manufacturing Association named after A.S. Popov",
    "70B3D5018": "DELITECH GROUP",
    "70B3D5395": "ICsec S.A.",
    "70B3D5668": "\u00d6resundskraft AB",
    "70B3D5176": "Larraioz Elektronika",
    "70B3D547D": "Shenyang TECHE Technology Co.,Ltd",
    "70B3D57B5": "VOCAL Technologies Ltd.",
    "70B3D5CEC": "Deltronic Security AB",
    "70B3D5F90": "Atman Tecnologia Ltda",
    "70B3D5562": "JD Squared, Inc.",
    "70B3D5646": "Xirgo Technologies LLC",
    "70B3D5988": "Arris",
    "70B3D54E2": "Transit Solutions, LLC.",
    "70B3D5BDE": "CAST Group of Companies Inc.",
    "70B3D5116": "Momentum Data Systems",
    "70B3D53C9": "Duerkopp-Adler",
    "70B3D5B32": "GridBeyond",
    "70B3D53AB": "Camozzi Automation SpA",
    "70B3D5195": "Ci4Rail",
    "70B3D564D": "SANMINA ISRAEL MEDICAL SYSTEMS LTD",
    "70B3D55C0": "Shenzhen Lianfaxun Electronic Technology Co., Ltd",
    "70B3D5322": "PuS GmbH und Co. KG",
    "70B3D544A": "CANON ELECTRON TUBES & DEVICES CO., LTD.",
    "70B3D54F5": "Orlaco Products B.V.",
    "70B3D5F15": "ARECA EMBEDDED SYSTEMS PVT LTD",
    "70B3D5F59": "KOREA SPECTRAL PRODUCTS",
    "70B3D56D6": "KMtronic ltd",
    "70B3D5534": "Weihai Weigao Medical Imaging Technology Co., Ltd",
    "70B3D59E9": "LiveCopper Inc.",
    "70B3D53B4": "YOUSUNG",
    "70B3D5E37": "Eurotempest AB",
    "70B3D5095": "plc-tec AG",
    "70B3D5612": "Edge Power Solutions",
    "70B3D59E5": "Antek Technology ",
    "70B3D5D68": "Tobi Tribe Inc.",
    "70B3D58A3": "Loehnert Elektronik GmbH",
    "70B3D587A": "Accolade Technology Inc",
    "70B3D55C7": "QSnet Visual Technologies Ltd",
    "70B3D5886": "MB connect line GmbH Fernwartungssysteme",
    "70B3D55FE": "Grossenbacher Systeme AG",
    "70B3D52C8": "SLAT",
    "70B3D5B57": "Shanghai Qinyue Communication Technology Co., Ltd.",
    "70B3D5AAB": "QUISS GmbH",
    "70B3D58FE": "Selmatec AS",
    "70B3D52F7": "Military Research Institute",
    "70B3D5E5A": "Cardinal Scales Manufacturing Co",
    "70B3D5E83": "Talleres de Escoriaza SA",
    "70B3D5DA7": "Network Innovations",
    "70B3D517C": "Farmpro Ltd",
    "70B3D567C": "Benchmark Electronics - Secure Technology",
    "70B3D5C7C": "Beijing Aumiwalker technology CO.,LTD",
    "70B3D5089": "Kazdream Technologies LLP",
    "70B3D52D1": "Integer.pl S.A.",
    "70B3D52D9": "ZPAS S.A.",
    "70B3D56C9": "Redstone Sunshine(Beijing)Technology Co.,Ltd.",
    "70B3D51CA": "inomatic GmbH",
    "70B3D50B7": "HAI ROBOTICS Co., Ltd.",
    "70B3D553A": "Panoramic Power",
    "70B3D5481": "STEP sarl",
    "70B3D58FC": "Mianjie Technology",
    "70B3D5744": "PHYZHON Health Inc",
    "70B3D53C1": "thingdust AG",
    "70B3D5DE1": "Duplomatic MS spa",
    "70B3D5E9A": "Meta Computing Services, Corp",
    "70B3D5369": "ALVAT s.r.o.",
    "70B3D54CA": "PCB Piezotronics",
    "70B3D53E7": "JNR Sports Holdings, LLC",
    "70B3D565F": "Axnes AS",
    "70B3D5541": "Nanjing Pingguang Electronic Technology Co., Ltd",
    "70B3D5979": "eSMART Technologies SA",
    "70B3D5C2B": "YUYAMA MFG Co.,Ltd",
    "70B3D5922": "Adcole Space",
    "70B3D51F2": "YUYAMA MFG Co.,Ltd",
    "70B3D5614": "QUALITTEQ LLC",
    "70B3D562D": "elements",
    "70B3D5244": "DAT Informatics Pvt Ltd",
    "70B3D56C6": "Abbott Diagnostics Technologies AS",
    "70B3D55D2": "Contec Americas Inc.",
    "70B3D5A6A": "Privafy, Inc",
    "70B3D558A": "ITK Dr. Kassen GmbH",
    "70B3D5E2A": "CONTES, spol. s r.o.",
    "70B3D56CE": "Eredi Giuseppe Mercuri SPA",
    "70B3D5F5D": "Potter Electric Signal Co. LLC",
    "70B3D5899": "Viotec USA",
    "70B3D534B": "LEAFF ENGINEERING SRL",
    "70B3D5306": " LEMZ-T, LLC",
    "70B3D58C0": "SenseNL",
    "70B3D56FC": "MI Inc.",
    "70B3D593A": "Braemar Manufacturing, LLC",
    "70B3D53F8": "The Fire Horn, Inc.",
    "70B3D5034": "Digital Systems Engineering",
    "70B3D5262": "OOO Research and Production Center \"Computer Technologies\"",
    "70B3D58E6": "Mothonic AB",
    "70B3D5EFC": "Absolent AB",
    "70B3D5BB1": "Lumiplan Duhamel",
    "70B3D51CE": "Clear Flow by Antiference",
    "70B3D5FFA": "Barracuda Measurement Solutions",
    "70B3D5A24": "Booz Allen Hamilton",
    "70B3D589E": "Innovative Control Systems, LP",
    "70B3D5DA2": "ACD Elekronik GmbH",
    "70B3D5BDF": "H2O-YUG LLC",
    "70B3D52D8": "Unisight Digital Products",
    "70B3D59A5": "Softel",
    "70B3D51B1": "Shanghai Danyan Information Technology Co., Ltd.",
    "70B3D5C95": "Chengdu Meihuan Technology Co., Ltd",
    "70B3D5A98": "Pantec AG",
    "70B3D551A": "Shachihata Inc.",
    "70B3D55D0": "InterTalk Critical Information Systems",
    "70B3D5E89": "JSC Kaluga Astral",
    "70B3D5BA5": "fpgalabs.com",
    "70B3D593B": "Changchun FAW Yanfeng Visteon Automotive Electronics.,Ltd.",
    "70B3D5BFF": "Sunsa, Inc",
    "70B3D5A67": "Gstar Creation Co .,Ltd",
    "70B3D5E8B": "Dream D&S Co.,Ltd",
    "70B3D5126": "AddSecure Smart Grids",
    "70B3D5017": "FTG Corporation",
    "70B3D5D27": "Light field Lab",
    "70B3D51C2": "CENSIS, Uiversity of Glasgow",
    "70B3D5366": "Solarlytics, Inc.",
    "70B3D538A": "KSE GmbH",
    "70B3D5DE3": "ETL Elektrotechnik Lauter GmbH",
    "70B3D57BE": "Phytron GmbH",
    "70B3D5E5B": "Argosy Labs Inc.",
    "70B3D5A38": "Aditec GmbH",
    "70B3D5FF6": "Elektro Adrian",
    "70B3D552A": "Dataflex International BV",
    "70B3D5372": "MATELEX",
    "70B3D56E3": "SHEN ZHEN QLS ELECTRONIC TECHNOLOGY CO.,LTD.",
    "70B3D5B19": "Brayden Automation Corp",
    "70B3D51EB": "Xavant",
    "70B3D5A6C": "Controles S.A.",
    "70B3D5BFC": "Vishay Nobel AB",
    "70B3D592C": "DISMUNTEL SAL",
    "70B3D5F4B": "Chengdu Lingya Technology Co., Ltd.",
    "70B3D5A6B": "xmi systems",
    "70B3D577A": "Tecsag Innovation AG",
    "70B3D555F": "Deep BV",
    "70B3D5C84": "Linc Technology Corporation dba Data-Linc Group ",
    "70B3D579F": "Green Instruments A/S",
    "70B3D5D31": "Solace Systems Inc.",
    "70B3D58C7": "Henschel-Robotics GmbH",
    "70B3D5FB8": "Hyannis Port Research",
    "70B3D5787": "Den Automation",
    "70B3D52A0": "Airthings",
    "70B3D567A": "Micatu",
    "70B3D5170": "Mutelcor GmbH",
    "70B3D5832": "Potter Electric Signal Co. LLC",
    "70B3D5B00": "HORIBA ABX SAS",
    "70B3D5C8A": "WTE Limited",
    "70B3D5BD0": "SHS SRL",
    "70B3D5A74": "Sadel S.p.A.",
    "70B3D5F17": "VITEC",
    "70B3D5CDA": "VITEC",
    "70B3D5DED": "Simpulse",
    "70B3D5E63": "Potomac Electric Corporation",
    "70B3D5970": "Bintel AB",
    "70B3D5CDB": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D5F48": "HEITEC AG",
    "70B3D5456": "Technological Application and Production One Member Liability Company (Tecapro company)",
    "70B3D565E": "Season Electronics Ltd",
    "70B3D547A": "GlooVir Inc.",
    "70B3D5726": "ATGS",
    "70B3D5398": "SIPRO s.r.l.",
    "70B3D5496": "Profcon AB",
    "70B3D55BA": "INFRASAFE/ ADVANTOR SYSTEMS ",
    "70B3D5D56": "KRONOTECH SRL",
    "70B3D5884": "LG Electronics",
    "70B3D58DF": "DORLET SAU",
    "70B3D5BC9": "Yite technology",
    "70B3D5019": "Transit Solutions, LLC.",
    "70B3D5B06": "MULTIVOICE LLC",
    "70B3D540B": "QUERCUS TECHNOLOGIES, S.L.",
    "70B3D54FA": "Thruvision Limited",
    "70B3D51A4": "DAVEY BICKFORD",
    "70B3D5444": "AMS Controls, Inc.",
    "70B3D5982": "3S - Sensors, Signal Processing, Systems GmbH",
    "70B3D5AC0": "RITEC",
    "70B3D5EFF": "Carlo Gavazzi Industri",
    "70B3D5E25": "GJD Manufacturing",
    "70B3D57CB": "KeyW Corporation",
    "70B3D599B": "RCH ITALIA SPA ",
    "70B3D59BF": "Xiris Automation Inc.",
    "70B3D5111": "Leonardo Sistemi Integrati S.r.l.",
    "70B3D56DE": "Ametek Solidstate Controls",
    "70B3D5C54": "Flexsolution APS",
    "70B3D50F2": "TrexEdge, Inc.",
    "70B3D5C5F": "Clean-Lasersysteme GmbH",
    "70B3D569A": "Altaneos",
    "70B3D58A9": "WoKa-Elektronik GmbH",
    "70B3D500C": "EXARA Group",
    "70B3D5F1C": "Bavaria Digital Technik GmbH",
    "70B3D5F50": "Vectology,Inc",
    "70B3D546E": "Zamir Recognition Systems Ltd.",
    "70B3D57BA": "Decentlab GmbH",
    "70B3D52FC": "Loanguard T/A SE Controls",
    "70B3D5FF2": "tiga.eleven GmbH",
    "70B3D5633": "OBSERVER FOUNDATION",
    "70B3D5BA6": "Gluon Solutions Inc.",
    "70B3D5BC5": "U&R GmbH Hardware- und Systemdesign",
    "70B3D5DD3": "VITEC",
    "70B3D5FD4": "GETRALINE",
    "70B3D50ED": "Lupa Tecnologia e Sistemas Ltda",
    "70B3D5DF5": "Beijing Huanyu Zhilian Science &Technology Co., Ltd.",
    "70B3D5B7E": "Elbit Systems of America",
    "70B3D5199": "Smart Controls LLC",
    "70B3D56A2": "Root Automation",
    "70B3D5E6A": "MAC Solutions (UK) Ltd",
    "70B3D54AC": "Microsoft Research",
    "70B3D5E1F": "THETA432",
    "70B3D51F1": "DIEHL Connectivity Solutions",
    "70B3D5CF0": "SHENZHEN WITLINK CO.,LTD.",
    "70B3D5B6E": "Edgeware AB",
    "70B3D5E17": "SA Photonics",
    "70B3D531F": "Elcoma",
    "70B3D5E3B": "ComNav Technology Ltd.",
    "70B3D5114": "Project H Pty Ltd",
    "70B3D5DB7": "Pengo Technology Co., Ltd",
    "70B3D5264": "ifak technology + service GmbH",
    "70B3D5800": "HeadsafeIP PTY LTD",
    "70B3D50B5": "Capgemini Netherlands",
    "70B3D543E": "Peloton Technology",
    "70B3D516A": "4Jtech s.r.o.",
    "70B3D5C70": "Magnetek",
    "70B3D5B76": "ATL-SD",
    "70B3D54F6": "DORLET SAU",
    "70B3D5C2D": "Ensotech Limited",
    "70B3D571E": "Motec Pty Ltd",
    "70B3D524A": "Unmukti Technology Pvt Ltd",
    "70B3D5048": "AvMap srlu",
    "70B3D557A": "Rhythm Engineering, LLC.",
    "70B3D554A": "Digital Instrument Transformers",
    "70B3D525F": "COPPERNIC SAS",
    "70B3D51C9": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5638": "Parkalot Denmark ApS",
    "70B3D5765": "LG Electronics",
    "70B3D577D": "APG Cash Drawer, LLC",
    "70B3D58BB": "KST technology",
    "70B3D5F21": "dds",
    "70B3D5FEE": "Kawasaki Robot Service,Ltd.",
    "70B3D5DD2": "Insitu, Inc",
    "70B3D519A": "WiSuite USA",
    "70B3D5686": "Access Protocol Pty Ltd",
    "70B3D5DA3": "Voleatech GmbH",
    "70B3D522D": "Leder Elektronik Design",
    "70B3D5A71": "Samwell International Inc",
    "70B3D5748": "KDT",
    "70B3D5ADC": "SODAQ",
    "70B3D57F9": "Communication Systems Solutions",
    "70B3D5C1F": "Behr Technologies Inc",
    "70B3D5BCF": "APG Cash Drawer, LLC",
    "70B3D5623": "Beijing HuaLian Technology Co, Ltd.",
    "70B3D5B8E": "UR FOG S.R.L.",
    "70B3D5234": "EDFelectronics JRMM Sp z o.o. sp.k.",
    "70B3D5F88": "ODAWARAKIKI AUTO-MACHINE MFG.CO.,LTD",
    "70B3D597A": "Orion Corporation",
    "70B3D5F67": "winsun AG",
    "70B3D55A5": "Rehwork GmbH",
    "70B3D52A8": "Dynamic Perspective GmbH",
    "70B3D5050": "Compusign Systems Pty Ltd",
    "70B3D50E1": "MiWave Consulting, LLC",
    "70B3D5966": "dA Tomato Limited",
    "70B3D55CF": "PROEL TSI s.r.l.",
    "70B3D5C35": "Vibrationmaster",
    "70B3D5A94": "ETA Technology Pvt Ltd",
    "70B3D5907": "NINGBO CRRC TIMES TRANSDUCER TECHNOLOGY CO., LTD",
    "70B3D5C38": "CRESPRIT INC.",
    "70B3D56C3": "BEIJING ZGH SECURITY RESEARCH INSTITUTE CO., LTD",
    "70B3D5EB4": "Robotic Research, LLC",
    "70B3D569C": "Keepen",
    "70B3D59B4": "MyoungSung System",
    "70B3D5CFC": "VEILUX INC.",
    "70B3D558E": "VEILUX INC.",
    "70B3D523F": "ETA-USA",
    "70B3D5434": "Wit.com Inc",
    "70B3D53C4": "Hagiwara Solutions Co., Ltd.",
    "70B3D5D71": "RZB Rudolf Zimmermann, Bamberg GmbH",
    "70B3D5B39": "MB connect line GmbH Fernwartungssysteme",
    "70B3D5546": "Sensefarm AB",
    "70B3D5162": "ESPAI DE PRODUCCI\u00d3 I ELECTR\u00d3NI",
    "70B3D53E6": "machineQ",
    "70B3D5ED0": "shanghai qiaoqi zhinengkeji",
    "70B3D5F52": "Alere Technologies AS",
    "70B3D5DEA": "Advanced Ventilation Applications, Inc.",
    "70B3D5A46": "Foxconn 4Tech",
    "70B3D5960": "HORIZON TELECOM",
    "70B3D5968": "LGM Ing\u00e9nierie",
    "70B3D55F1": "Fater Rasa Noor ",
    "70B3D5C7D": "Metatronics B.V.",
    "70B3D5248": "GL TECH CO.,LTD",
    "70B3D57EE": "ADVEEZ",
    "70B3D5EEF": "TATTILE SRL",
    "70B3D5D6E": "ard sa",
    "70B3D50AC": "RoboCore Tecnologia",
    "70B3D577E": "Blue Marble Communications, Inc.",
    "70B3D5D7C": "D.T.S Illuminazione Srl ",
    "70B3D521C": "Enyx SA",
    "70B3D56B7": "Grossenbacher Systeme AG",
    "70B3D58FA": "DEA SYSTEM SPA",
    "70B3D5D62": "Andasis Elektronik San. ve Tic. A.\u015e.",
    "70B3D5656": "SonoSound ApS",
    "70B3D5F5F": "RFRain LLC",
    "70B3D5722": "UMAN",
    "70B3D5920": "SLAT",
    "70B3D52D4": "CT Company",
    "70B3D554B": "Brakels IT",
    "70B3D5144": "GS Elektromedizinsiche Ger\u00e4te G. Stemple GmbH",
    "70B3D524E": "Chengdu Cove Technology CO.,LTD",
    "70B3D57D0": "Cubitech",
    "70B3D58E3": "DORLET SAU",
    "70B3D5049": "APP Engineering, Inc.",
    "70B3D56D8": "Shanghai YuanAn Environmental Protection Technology Co.,Ltd",
    "70B3D55DE": "Hangzhou AwareTec Technology Co., Ltd",
    "70B3D5236": "Monnit Corporation",
    "70B3D57A5": "Triton Electronics Ltd",
    "001BC50BB": "Triax A/S",
    "70B3D575E": "Cardinal Health",
    "70B3D5704": "Melecs EWS GmbH",
    "70B3D52AE": "Alere Technologies AS",
    "70B3D5EB3": "KWS-Electronic GmbH",
    "70B3D521B": "Lab241 Co.,Ltd.",
    "70B3D5DBF": "Infodev Electronic Designers Intl.",
    "70B3D5525": "Plantiga Technologies Inc",
    "70B3D5340": "Renesas Electronics",
    "70B3D5B17": "Intesens",
    "70B3D5A29": "QIAGEN Instruments AG",
    "70B3D5AF2": "True Networks Ltd.",
    "70B3D5207": "Savari Inc",
    "70B3D5A3A": "EPSOFT Co., Ltd",
    "70B3D5A43": "OLEDCOMM",
    "70B3D594A": "SHENZHEN WISEWING INTERNET TECHNOLOGY CO.,LTD",
    "70B3D56E1": "Shanghai Holystar Information Technology Co.,Ltd",
    "70B3D5FE3": "CSM MACHINERY srl",
    "70B3D50AF": "KMtronic ltd",
    "70B3D54E1": "Grupo Epelsa S.L.",
    "70B3D5853": "HGH SYSTEMES INFRAROUGES",
    "70B3D55DC": "FactoryLab B.V.",
    "70B3D5F5E": "Selex ES Inc.",
    "70B3D5032": "iFreecomm Technology Co., Ltd",
    "70B3D5357": "Movimento Group AB",
    "70B3D5FF9": "InOut Communication Systems",
    "70B3D5DC6": "IDEM INC.",
    "70B3D57A6": "Electrolux",
    "70B3D5FD6": "Visual Fan",
    "70B3D5813": "Wavemed srl",
    "70B3D5BFB": "Sensor 42",
    "70B3D5C37": "Keycom Corp.",
    "70B3D5734": "MANSION INDUSTRY CO., LTD.",
    "70B3D599C": "Enerwise Solutions Ltd.",
    "70B3D500D": "Scrona AG",
    "70B3D598F": "Spaceflight Industries",
    "70B3D5E91": "NAS Australia P/L",
    "70B3D5566": "Data Informs LLC",
    "70B3D5616": "Axxess Identification Ltd",
    "70B3D563E": "RIKEN OPTECH CORPORATION",
    "70B3D5B07": "Arrowvale Electronics",
    "70B3D59DD": "HumanEyes Technologies Ltd.",
    "70B3D5255": "Asystems Corporation",
    "70B3D572E": "Maharsystem",
    "70B3D5E7B": "Shenzhen  SanYeCao  Electronics  Co.,Ltd",
    "70B3D5D26": "MI Inc.",
    "70B3D5A88": "Shangdong Bosure Automation Technology Ltd",
    "70B3D5839": "Rockwell Collins Canada",
    "70B3D5EED": "COMM-connect A/S",
    "70B3D5DF8": "RMA Mess- und Regeltechnik GmbH & Co.KG",
    "70B3D5E96": "Cellier Domesticus inc",
    "70B3D5557": "HEITEC AG",
    "70B3D568C": "ND METER",
    "70B3D5C42": "CRDE",
    "70B3D5845": "Harborside Technology",
    "70B3D5548": "DIGIVERV INC",
    "70B3D5E18": "Plasmapp Co.,Ltd.",
    "70B3D52A1": "Blink Services AB",
    "70B3D5CB3": "KST technology",
    "70B3D507F": "Abalance Corporation",
    "70B3D504E": "HUGEL GmbH",
    "70B3D5359": "Boutronic",
    "70B3D595B": "SRS Group s.r.o.",
    "70B3D57F7": "JASCO Applied Sciences Canada Ltd",
    "70B3D5D43": "EZSYS Co., Ltd.",
    "70B3D56B2": "CRDE",
    "70B3D5A9A": "Amphenol Advanced Sensors",
    "70B3D53FE": "Siemens Industry Software Inc.",
    "70B3D56AF": "Sensorberg GmbH",
    "70B3D5715": "RIOT",
    "70B3D5FDB": "Design SHIFT",
    "70B3D5D20": "Rheonics GmbH",
    "70B3D5969": "Emtel System Sp. z o.o.",
    "70B3D5B48": "DWQ Informatikai Tanacsado es Vezerlestechnikai KFT",
    "70B3D5C22": "Skyriver Communications Inc.",
    "70B3D59B3": "K&J Schmittschneider AG",
    "70B3D514A": "ExSens Technology (Pty) Ltd.",
    "70B3D54E7": "Digital Domain",
    "70B3D520F": "Tieline Research Pty Ltd",
    "70B3D5B04": "Herrmann Datensysteme GmbH",
    "70B3D5945": "Symboticware Incorporated",
    "70B3D5192": "ASPT, INC.",
    "70B3D5229": "CONTROL SYSTEMS Srl",
    "70B3D5479": "LINEAGE POWER PVT LTD.,",
    "70B3D5CE1": "EA Elektroautomatik GmbH & Co. KG",
    "70B3D59DE": "System 11 Sp. z o.o.",
    "70B3D546C": "SHANGHAI CHENZHU INSTRUMENT CO., LTD.",
    "70B3D5A48": "Applied Satellite Engineering",
    "70B3D5168": "Biwave Technologies, Inc.",
    "70B3D58C3": "Wyebot, Inc.",
    "70B3D5596": "Mencom Corporation",
    "70B3D52B4": "Foerster-Technik GmbH",
    "70B3D5E50": "Advanced Vision Technology Ltd",
    "70B3D5E16": "China Entropy Co., Ltd.",
    "70B3D5495": "Fiem Industries Ltd.",
    "70B3D5A47": "KANOA INC",
    "70B3D5489": "ard sa",
    "70B3D59A1": "ITS Industrial Turbine Services GmbH",
    "70B3D5BF3": "CG-WIRELESS",
    "70B3D5873": "Vishay Nobel AB",
    "70B3D5011": "Sumer Data S.L",
    "70B3D5661": "DesignA Electronics Limited",
    "70B3D52A2": "Visualware, Inc.",
    "70B3D5927": "LG Electronics",
    "70B3D59CE": "Terragene S.A",
    "70B3D5063": "PoolDigital GmbH & Co. KG",
    "70B3D5397": "Guangxi Hunter Information Industry Co.,Ltd",
    "70B3D531C": "FINANCIERE DE L'OMBREE (eolane)",
    "70B3D5F9E": "International Center for Elementary Particle Physics, The University of Tokyo",
    "70B3D53BF": "Star Electronics GmbH & Co. KG ",
    "70B3D53F0": "Intervala",
    "70B3D5BD1": "CableLabs",
    "70B3D5AB8": "HORIBA ABX SAS",
    "70B3D50E8": "Grossenbacher Systeme AG",
    "70B3D5EBE": "Sierra Pacific Innovations Corp",
    "70B3D5724": "Quan International Co., Ltd.",
    "70B3D50C2": "LOOK EASY INTERNATIONAL LIMITED",
    "70B3D551C": "ATX Networks Corp",
    "70B3D5427": "Key Chemical & Equipment Company",
    "70B3D5EF8": "DKS Dienstl.ges. f. Komm.anl. d. Stadt- u. Reg.verk. mbH",
    "70B3D585A": "BRUSHIES",
    "70B3D51B8": "OES Inc.",
    "70B3D561C": "Earth Works",
    "70B3D5C5C": "Layer Logic Inc",
    "70B3D5DCC": "Eutron SPA",
    "70B3D5BAA": "Device Solutions Ltd",
    "70B3D5C12": "Beijing Wisetone Information Technology Co.,Ltd.",
    "70B3D5C62": "WIZNOVA",
    "70B3D513D": "Elsist Srl",
    "70B3D525A": "DEUTA-WERKE GmbH",
    "70B3D594D": "SEASON DESIGN TECHNOLOGY",
    "70B3D5735": "Swiss Audio",
    "70B3D572C": "NuRi&G Engineering co,.Ltd.",
    "70B3D528E": "TEX COMPUTER SRL ",
    "70B3D5F8B": "IOOOTA Srl",
    "70B3D5934": "RBS Netkom GmbH",
    "70B3D590B": "Matrix Switch Corporation",
    "70B3D5693": "Altron, a.s.",
    "70B3D53BE": "MyDefence Communication ApS",
    "70B3D5503": "Itest communication Tech Co., LTD",
    "70B3D5457": "Vivaldi Clima Srl",
    "70B3D5ECB": "Re spa - Controlli Industriali - IT01782300154",
    "70B3D5D75": "Hyundai MNSOFT",
    "70B3D59FB": "Unicom Global, Inc.",
    "70B3D5F03": "GMI Ltd",
    "70B3D5B09": "FIRST LIGHT IMAGING",
    "70B3D5317": "Iotopia Solutions",
    "70B3D53E8": "COSMOS web Co., Ltd.",
    "70B3D5C5D": "FOSHAN SHILANTIAN NETWORK S.T. CO., LTD.",
    "70B3D5475": "EWATTCH",
    "70B3D5C6F": "nyantec GmbH",
    "70B3D597F": "BISTOS.,Co.,Ltd",
    "70B3D55D6": "BMT Messtechnik Gmbh",
    "70B3D5D95": "SANO SERVICE Co.,Ltd",
    "70B3D5973": "Autonomic Controls, Inc.",
    "70B3D5987": "AXIS CORPORATION",
    "70B3D5976": "Atonarp Micro-Systems India Pvt. Ltd.",
    "70B3D554F": "Assembly Contracts Limited",
    "70B3D50CE": "Innominds Software Inc",
    "70B3D5644": "ATX Networks Corp",
    "70B3D5EDB": "Netfort Solutions",
    "70B3D5197": "Lattech Systems Pty Ltd",
    "70B3D5F81": "Littlemore Scientific",
    "70B3D5A19": "Qualitronix Madrass Pvt Ltd",
    "70B3D57B0": "Medisafe International",
    "70B3D59AD": "Fortuna Impex Pvt ltd",
    "70B3D5B62": "Sakura Seiki Co.,Ltd.",
    "70B3D55C4": "TATTILE SRL",
    "70B3D57A4": "Potter Electric Signal Co. LLC",
    "70B3D5238": "Arete Associates",
    "70B3D5C80": "Link Care Services",
    "70B3D5E4F": "RWS Automation GmbH",
    "70B3D5CCD": "Suzhou PowerCore Technology Co.,Ltd.",
    "70B3D5BD9": "SolwayTech",
    "70B3D5B23": "Supervision Test et Pilotage",
    "70B3D5D05": "Colmek",
    "70B3D512F": "DSP4YOU LTd",
    "70B3D5D66": "Ascendent Technology Group",
    "70B3D53EF": "Vtron Pty Ltd",
    "70B3D5FAA": "LogiM GmbH Software und Entwicklung",
    "70B3D5448": "B/E Aerospace, Inc.",
    "70B3D542A": "Critical Link LLC",
    "70B3D5691": "PEEK TRAFFIC",
    "70B3D5718": "PEEK TRAFFIC",
    "70B3D5615": "JSC \"OTZVUK\"",
    "70B3D5A4A": "Beijing Arrow SEED Technology Co,.Ltd.",
    "70B3D5B59": "FutureTechnologyLaboratories INC.",
    "70B3D53A7": "Varikorea",
    "70B3D5B44": "ENTEC Electric & Electronic Co., LTD.",
    "70B3D5A5E": "ConectaIP Tecnologia S.L.",
    "70B3D5F99": "TEX COMPUTER SRL ",
    "70B3D51B5": "StarBridge, Inc.",
    "70B3D5C8B": "Asia Pacific Satellite Coummunication Inc.",
    "70B3D51DD": "RF CREATIONS LTD",
    "70B3D50AE": "Norsat International Inc.",
    "70B3D5307": "Energi innovation Aps",
    "70B3D55CD": "MVT Video Technologies R + H Maedler GbR",
    "70B3D59EC": "eSoftThings",
    "70B3D5AF7": "DimoSystems BV",
    "70B3D5CB2": "SECLAB",
    "70B3D5B99": "DomoSafety S.A.",
    "70B3D58CF": "Dainichi Denshi\u3000Co.,LTD",
    "70B3D5296": "Rohde&Schwarz Topex SA",
    "70B3D5FB0": "Rohde&Schwarz Topex SA",
    "70B3D5217": "Tecnint HTE SRL",
    "70B3D51FE": "MobiPromo",
    "70B3D5F0B": "RF Industries",
    "70B3D5FF3": "Aplex Technology Inc.",
    "70B3D565C": "Aplex Technology Inc.",
    "70B3D5AA1": "Shenzhen Weema TV Technology Co.,Ltd.",
    "70B3D5B1D": "Safelet BV",
    "70B3D5AAE": "Nuviz Oy",
    "70B3D5C34": "Technical Panels Co. Ltd.",
    "70B3D5E3E": "Sol Welding srl",
    "70B3D562B": "Silicann Systems GmbH",
    "70B3D50BA": "Ayre Acoustics, Inc.",
    "70B3D57D9": "ATOM GIKEN Co.,Ltd.",
    "70B3D51FD": "BRS Sistemas Eletr\u00f4nicos",
    "70B3D54AE": "Reinhardt System- und Messelectronic GmbH",
    "70B3D58E4": "Aplex Technology Inc.",
    "70B3D5F13": "MEDIAM Sp. z o.o.",
    "70B3D5350": "Tickster AB",
    "70B3D5E2E": "Merz s.r.o.",
    "70B3D5042": "Coveloz Technologies Inc.",
    "70B3D5E3D": "Leo Bodnar Electronics Ltd",
    "70B3D5B26": "INTEC International GmbH",
    "70B3D5804": "PMT Corporation",
    "70B3D5179": "ALTRAN UK",
    "70B3D58D3": "PERFORMANCE CONTROLS, INC.",
    "70B3D5B31": "Qwave Inc",
    "70B3D5194": "Husty M.Styczen J.Hupert Sp.J.",
    "70B3D52BB": "Automation Networks & Solutions LLC",
    "70B3D5AE1": "DimoCore Corporation",
    "70B3D53C3": "AIMCO",
    "70B3D5766": "Tirasoft Nederland",
    "70B3D599A": "KEVIC. inc,",
    "70B3D51C7": "Hoshin Electronics Co., Ltd.",
    "70B3D529B": "DermaLumics S.L.",
    "70B3D5DB5": "Xiamen Point Circle Technologh Co,ltd",
    "70B3D5AC6": "SMTC Corporation",
    "70B3D5FBE": "Hanbat National University",
    "70B3D5FCC": "DIgSILENT GmbH",
    "70B3D5428": "Presentation Switchers, Inc.",
    "70B3D5F1A": "Sator Controls s.r.o.",
    "70B3D59EF": "Cottonwood Creek Technologies, Inc.",
    "70B3D5AF4": "TATTILE SRL",
    "70B3D576B": "EMPELOR GmbH",
    "70B3D5268": "Cardinal Scale Mfg Co",
    "70B3D5F2E": "Shanghai JCY Technology Company",
    "70B3D538F": "Sorynorydotcom Inc",
    "70B3D5747": "Eva Automation",
    "70B3D58B9": "Toptech Systems, Inc.",
    "70B3D5059": "Pro-Digital Projetos Eletronicos Ltda",
    "70B3D5FB6": "KRONOTECH SRL",
    "70B3D5647": "KZTA",
    "70B3D58A6": "CRDE",
    "70B3D5D51": "Azcom Technology S.r.l.",
    "70B3D51A9": "OCEANIX INC.",
    "70B3D5152": "Xped Corporation Pty Ltd",
    "70B3D59BA": "ATIM Radiocommunication",
    "70B3D5E02": "YEHL & JORDAN LLC",
    "70B3D54B6": "VEILUX INC.",
    "70B3D5B05": "E-PLUS TECHNOLOGY CO., LTD",
    "70B3D53CC": "TerOpta Ltd",
    "70B3D53C6": "ACD Elekronik GmbH",
    "70B3D5EF6": "CHARGELIB",
    "70B3D5929": "OutSys",
    "70B3D5214": "signalparser",
    "70B3D5B6D": "Movis",
    "70B3D52EF": "IEM SA",
    "70B3D5794": "Shadin Avionics",
    "70B3D5B40": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "70B3D5BAB": "Axotec Technologies GmbH",
    "70B3D576E": "Grupo Epelsa S.L.",
    "70B3D5494": "Schildknecht AG",
    "70B3D5F6E": "Streambox Inc",
    "70B3D5D76": "attocube systems AG",
    "70B3D5AE0": "AnyComm.Co.,Ltd.",
    "70B3D5266": "Spectra Displays Ltd",
    "70B3D516E": "Jemac Sweden AB",
    "70B3D578E": "effectas GmbH",
    "70B3D59F3": "IEEE Registration Authority",
    "70B3D5E93": "ECON Technology Co.Ltd",
    "70B3D5001": "SOREDI touch systems GmbH",
    "70B3D54BD": "Boulder Amplifiers, Inc.",
    "70B3D51E0": "TOPROOTTechnology Corp. Ltd.",
    "70B3D57EB": "Xerox International Partners",
    "70B3D5439": "TriLED",
    "70B3D58A0": "DM RADIOCOM",
    "70B3D54D1": "Contraves Advanced Devices Sdn. Bhd.",
    "70B3D5D59": "WyreStorm Technologies Ltd",
    "70B3D514F": "Mobile Devices Unlimited",
    "70B3D5D74": "Sandia National Laboratories",
    "70B3D537F": "IDS Innomic GmbH",
    "70B3D5100": "Gupsy GmbH",
    "70B3D52CE": "KDT",
    "70B3D504D": "Sicon srl",
    "70B3D5D42": "DSP DESIGN",
    "70B3D5EE5": "Beijing Hzhytech Technology Co.Ltd",
    "70B3D5A04": "Galea Electric S.L.",
    "70B3D5D12": "FIDELTRONIK POLAND SP. Z O.O.",
    "70B3D584E": "Chromalox, Inc.",
    "70B3D52EA": "Schneider Electric Motion",
    "70B3D5DF7": "ScopeSensor Oy",
    "70B3D5BF1": "Flashnet SRL",
    "70B3D5DDF": "AeroVision Avionics, Inc.",
    "70B3D59ED": "Benchmark Electronics BV",
    "70B3D5774": "Micram Instruments Ltd",
    "70B3D5933": "SARL S@TIS",
    "70B3D548E": "Allim System Co,.Ltd.",
    "70B3D543B": "Kalycito Infotech Private Limited",
    "70B3D5C89": "ARD",
    "70B3D5166": "SERIAL IMAGE INC.",
    "70B3D5595": "PLR Prueftechnik Linke und Ruehe GmbH",
    "70B3D5CD6": "VideoRay LLC",
    "70B3D5852": "NetBoxSC, LLC",
    "70B3D532F": "Movidius SRL",
    "70B3D57C1": "Data Sciences International",
    "70B3D58FF": "IMST GmbH",
    "70B3D5A93": "Mes Communication Co., Ltd",
    "70B3D53D8": "Abitsoftware, Ltd.",
    "70B3D5470": "KITRON UAB",
    "70B3D54DE": "Oso Technologies, Inc.",
    "70B3D5F1E": "ATX NETWORKS LTD",
    "70B3D57E4": "C21 Systems Ltd",
    "70B3D5F93": "Hella Gutmann Solutions GmbH",
    "70B3D500E": "Magosys Systems LTD",
    "70B3D52D6": "Kvazar LLC",
    "70B3D5D2D": "Evolute Systems Private Limited",
    "70B3D5DF9": "Korea Plant Maintenance",
    "70B3D5E36": "Guidance Navigation Limited",
    "70B3D508F": "DEUTA-WERKE GmbH",
    "70B3D5F8E": "Isabellenh\u00fctte Heusler Gmbh &Co KG",
    "70B3D5C73": "C.D.N.CORPORATION",
    "70B3D5FBA": "Apogee Applied Research, Inc.",
    "70B3D5870": "bentrup Industriesteuerungen",
    "70B3D587B": "Liquid Instruments Pty Ltd",
    "70B3D5583": "Ducommun Inc.",
    "70B3D54D8": "Versilis Inc.",
    "70B3D5336": "Synaccess Networks Inc.",
    "70B3D5D87": "Zigen Corp",
    "70B3D5CFD": "iLOQ Oy",
    "70B3D5AD6": "Lemonade Lab Inc",
    "70B3D5AFB": "Shanghai Tianhe Automation Instrumentation Co., Ltd.",
    "70B3D5995": "LayTec AG",
    "70B3D50DC": "Talleres de Escoriaza",
    "70B3D57B7": "LSB - LA SALLE BLANCHE",
    "70B3D5AC9": "Trinity Solutions LLC",
    "70B3D5AF3": "New Japan Radio Co., Ltd",
    "70B3D5775": "Sonel S.A.",
    "70B3D5BDD": "CDR SRL",
    "70B3D5DE8": "Nation-E Ltd.",
    "70B3D5C3F": "Code Blue Corporation",
    "70B3D501C": "Kumu Networks",
    "70B3D5F2C": "Hengen Technologies GmbH",
    "70B3D5418": "DEV Systemtechnik GmbH& Co KG",
    "70B3D548C": "Integrated Systems Engineering, Inc.",
    "70B3D5AA7": "ATEME",
    "70B3D5300": "Novo DR Ltd.",
    "70B3D513F": "Farmobile, LLC",
    "70B3D5A97": "Bizwerks, LLC",
    "70B3D5A81": "Sienda New Media Technologies GmbH",
    "70B3D5D0D": "Logiwaste AB",
    "70B3D51AF": "Teenage Engineering AB",
    "70B3D5D7F": "ConectaIP Tecnologia S.L.",
    "70B3D57B8": "SerEnergy A/S",
    "70B3D53DB": "KST technology",
    "70B3D5849": "RF-Tuote Oy",
    "70B3D5DCF": "KLS Netherlands B.V.",
    "70B3D5F92": "TechOne",
    "70B3D5A25": "PulseTor LLC",
    "70B3D5D1F": "Embsec AB",
    "70B3D5A55": "Embest Technology Co., Ltd",
    "70B3D5344": "IHI Inspection & Instrumentation Co., Ltd.",
    "70B3D5959": "Zulex International Co.,Ltd.",
    "70B3D5220": "Private",
    "70B3D5A5B": "Christ Elektronik GmbH",
    "70B3D5164": "Tokyo Drawing Ltd.",
    "70B3D5327": "Seneco A/S",
    "70B3D5ECE": "COMM-connect A/S",
    "70B3D5FF5": "Prolan Process Control Co.",
    "70B3D5E92": "FUJI DATA SYSTEM CO.,LTD.",
    "70B3D5F8C": "EUROPEAN ADVANCED TECHNOLOGIES",
    "70B3D5F2B": "SENSYS GmbH",
    "70B3D50C0": "Molu Technology Inc., LTD.",
    "70B3D5BC6": "Hatteland Display AS",
    "70B3D53DF": "MultiDyne",
    "70B3D5BA4": "EIWA GIKEN INC.",
    "70B3D5C61": "JC HUNTER TECHNOLOGIES",
    "70B3D53B8": "nVideon, Inc.",
    "70B3D5E4C": "IAI-Israel Aerospace Industries MBT",
    "70B3D520C": "Siemens Healthcare Diagnostics",
    "70B3D56F2": "P&C Micro's Pty Ltd",
    "70B3D52E3": "Meiknologic GmbH",
    "70B3D5E26": "FEITIAN CO.,LTD.",
    "70B3D5E99": "Advitronics telecom bv",
    "70B3D545E": "eSOL Co.,Ltd.",
    "70B3D5362": "Asiga",
    "70B3D50B3": "Reonix Automation",
    "70B3D53CE": "Aditec GmbH",
    "70B3D55AA": "Chugoku Electric Manufacturing Co.,Inc",
    "70B3D5819": "\u00abIntellect module\u00bb LLC",
    "70B3D52DA": "Skywave Networks Private Limited",
    "70B3D5AC8": "Heartland.Data Inc.",
    "70B3D5FC5": "Eltwin A/S",
    "70B3D5016": "Guardian Controls International Ltd",
    "70B3D5FD3": "AKIS technologies",
    "70B3D5FA1": "BBI Engineering, Inc.",
    "70B3D5DC5": "Excel Medical Electronics LLC",
    "70B3D501D": "Weigl Elektronik & Mediaprojekte",
    "70B3D5564": "christmann informationstechnik + medien GmbH & Co. KG",
    "70B3D560C": "IST ElektronikgesmbH",
    "70B3D5EF2": "Kongsberg Intergrated Tactical Systems",
    "70B3D53E1": "Barnstormer Softworks",
    "70B3D53A8": "JamHub Corp.",
    "70B3D5052": "Sudo Premium Engineering",
    "70B3D5F61": "Power Diagnostic Service",
    "70B3D5433": "Flexsolution APS",
    "70B3D5F34": "MacGray Services",
    "70B3D5A0E": "Vetaphone A/S",
    "70B3D551D": "Tecnint HTE SRL",
    "70B3D5139": "Tunstall A/S",
    "70B3D5183": "Evco S.p.a.",
    "70B3D5EDC": "J.D. Koftinoff Software, Ltd.",
    "70B3D5B3D": "Inras GmbH",
    "70B3D59F4": "Tband srl",
    "70B3D540A": "Monroe Electronics, Inc.",
    "70B3D53C2": "Cellular Specialties, Inc.",
    "001BC50BF": "TN Core Co.,Ltd.",
    "001BC50C0": "Digital Loggers, Inc.",
    "001BC50B7": "Autelis, LLC",
    "001BC50AE": "Techlan Reti s.r.l.",
    "001BC50A3": "P A Network Laboratory Co.,Ltd",
    "001BC5098": "Cubic Systems, Inc.",
    "001BC508D": "EUREK  SRL",
    "001BC5089": "SIGNATURE CONTROL SYSTEMS, INC.",
    "001BC5088": "UAB Kitron",
    "001BC5081": "WonATech Co., Ltd.",
    "001BC5079": "HPI High Pressure Instrumentation GmbH",
    "001BC506F": "LLC Emzior",
    "001BC506E": "Two Dimensional Instruments, LLC",
    "001BC506B": "Verified Energy, LLC.",
    "001BC506C": "Luxcon System Limited",
    "001BC5070": "Siemens Industries, Inc, Retail & Commercial Systems",
    "001BC5062": "Sulaon Oy",
    "001BC5065": "Plair Media Inc.",
    "001BC505D": "JSC Prominform",
    "001BC5054": "Private",
    "001BC5040": "OOO Actidata",
    "001BC5039": "EURESYS S.A.",
    "001BC5035": "RTLS Ltd.",
    "001BC501D": "Rose + Herleth GbR",
    "001BC501A": "ABA ELECTRONICS TECHNOLOGY CO.,LTD",
    "001BC5013": "Zamir Recognition Systems Ltd.",
    "001BC5011": "OOO NPP Mera",
    "001BC5006": "TRIAX-HIRSCHMANN Multi-Media GmbH",
    "001BC5007": "Energy Aware Technology",
    "001BC5005": "Private",
    "001BC500B": "Private",
    "8C1F64D7B": "Global Design Solutions Korea",
    "8C1F64CFB": "YUYAMA MFG Co.,Ltd",
    "8C1F643D9": "Unlimited Bandwidth LLC",
    "8C1F64AE9": "ENNPLE",
    "8C1F64028": "eyrise B.V.",
    "8C1F64119": "Foxconn Technology Co., Ltd.",
    "8C1F649A9": "TIAMA",
    "8C1F64114": "Sanmina SCI Medical",
    "8C1F646D6": "Argosdyne Co., Ltd",
    "8C1F64CF6": "NYBSYS Inc",
    "8C1F645C3": "R3Vox Ltd",
    "8C1F64B3C": "Safepro AI Video Research Labs Pvt Ltd",
    "8C1F64F46": "Broadcast Tools, Inc.",
    "8C1F647AB": "DEUTA Werke GmbH",
    "8C1F64C29": "BRS Sistemas Eletr\u00f4nicos",
    "8C1F649E7": "MicroPilot Inc.",
    "8C1F640F3": "LSI",
    "8C1F644A1": "Breas Medical AB",
    "70B3D57C9": "Viridi Parente, Inc.",
    "8C1F643E5": "Systems Mechanics",
    "8C1F64C2B": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "8C1F6434B": "Infrared Inspection Systems",
    "8C1F64E4E": "Trivedi Advanced Technologies LLC",
    "8C1F64E70": "TELFI TECHNOLOGIES PRIVATE LIMITED",
    "8C1F641E2": "Potter Electric Signal Co. LLC",
    "8C1F64F9C": "Beijing Tong Cybsec Technology Co.,LTD",
    "8C1F64B19": "DITRON S.r.l.",
    "8C1F64303": "IntelliPlanner Software System India Pvt Ltd",
    "8C1F649CD": "JiangYu Innovative Medical Technology ",
    "8C1F64762": "Support Professionals B.V.",
    "8C1F64D61": "Advent Diamond",
    "8C1F64105": "AixControl GmbH",
    "8C1F64E8B": "Televic Rail GmbH",
    "8C1F64E6E": "HUMAN DGM. CO., LTD.",
    "8C1F6474B": "AR Modular RF",
    "8C1F64ED3": "SENSO2ME NV",
    "8C1F64AE3": "Private",
    "8C1F64657": "Bright Solutions PTE LTD",
    "8C1F64B01": "Blue Ocean UG",
    "8C1F64C9B": "J.M. Voith SE & Co. KG ",
    "8C1F64C1B": "hiSky SCS Ltd",
    "8C1F64CB9": "iC-Haus GmbH",
    "8C1F6465B": "SUS Corporation",
    "8C1F64673": "MEDIASCOPE Inc.",
    "8C1F645D1": "TWIN DEVELOPMENT",
    "8C1F64388": "MB connect line GmbH Fernwartungssysteme",
    "8C1F645E6": "ODYSSEE-SYSTEMES",
    "8C1F64496": "QUALSEN(GUANGZHOU)TECHNOLOGIES CO.,LTD",
    "8C1F64B27": "InHandPlus Inc.",
    "8C1F64BE3": "REO AG",
    "8C1F64DF9": "VuWall Technology Europe GmbH",
    "8C1F64A7B": "CPAT Flex Inc.",
    "8C1F64651": "Teledyne Cetac",
    "8C1F6495C": "Fasetto, Inc.",
    "8C1F64FAC": "Showa Electric Laboratory co.,ltd.",
    "8C1F64C13": "Glucoloop AG",
    "8C1F64FD2": "Guo He Xing Ke (ShenZhen) Technology Co.,Ltd.",
    "8C1F64DB1": "Shanghai Yamato Scale Co., Ltd",
    "8C1F64F90": "Enfabrica",
    "70B3D5CD9": "Peter Huber Kaeltemaschinenbau SE",
    "70B3D52E0": "Peter Huber Kaeltemaschinenbau SE",
    "70B3D5D7B": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F64110": "Xian Linking Backhaul Telecom Technology Co.,Ltd",
    "70B3D5DC4": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F64AA3": "Peter Huber Kaeltemaschinenbau SE",
    "8C1F64945": "Deqin Showcase",
    "8C1F64F49": "CenterClick LLC",
    "8C1F642B1": "U -MEI-DAH INT'L ENTERPRISE CO.,LTD.",
    "8C1F6436B": "ViewSonic Corp",
    "8C1F64D9E": "Wagner Group GmbH",
    "8C1F64CEC": "Zhuhai Huaya machinery Technology Co., LTD",
    "8C1F64D5D": "Genius Vision Digital Private Limited",
    "8C1F6498B": "Syscom Instruments SA",
    "8C1F64664": "Thermoeye Inc",
    "8C1F64E33": "Amiad Water Systems",
    "8C1F64375": "DUEVI SRL",
    "8C1F64F08": "ADVANTOR CORPORATION",
    "8C1F64813": "Pribusin Inc.",
    "8C1F64E14": "Proserv",
    "70B3D5AE2": "Wartsila Voyage Oy",
    "8C1F6449B": "Wartsila Voyage Oy",
    "8C1F64273": "Distran AG",
    "8C1F64ABF": "STACKIOT TECHNOLOGIES PRIVATE LIMITED",
    "8C1F6407D": "Talleres de Escoriaza SAU",
    "8C1F64ACB": "Villari B.V.",
    "8C1F642BF": "Gogo Business Aviation",
    "8C1F6471D": "Epigon spol. s r.o.",
    "8C1F6494F": "F\u00f6rster Technik GmbH",
    "8C1F64D9D": "MITSUBISHI HEAVY INDUSTRIES THERMAL SYSTEMS, LTD.",
    "8C1F641CA": "Power Electronics Espana, S.L.",
    "8C1F64457": "SHANGHAI ANGWEI INFORMATION TECHNOLOGY CO.,LTD.",
    "8C1F64D62": "Alpes recherche et d\u00e9veloppement",
    "8C1F646DE": "SUN\uff65TECTRO,Ltd.",
    "8C1F644A6": "Alaire Technologies Inc",
    "8C1F64760": "Q-Light AS",
    "8C1F64D1B": "Audiodo International AB",
    "8C1F643E2": "Agrico",
    "8C1F64125": "Hangzhou Sciener Smart Technology Co., Ltd.",
    "8C1F6413E": "BTEC INDUSTRIAL INSTRUMENT SDN. BHD.",
    "8C1F64EBD": "Esprit Digital Ltd",
    "8C1F64BEF": "Triumph SEC",
    "8C1F6400D": "T4I Sp. z o.o.",
    "8C1F648AB": "ASML US, LP",
    "8C1F64E15": "Panascais ehf.",
    "8C1F6475C": "American Energy Storage Innovations",
    "8C1F64695": "aeroLiFi GmbH",
    "8C1F64C96": "Smart Data (Shenzhen) Intelligent System Co., Ltd.",
    "8C1F64022": "Telica Telecom Private Limited",
    "8C1F64CB6": "ROWAN ELETTRONICA SRL",
    "8C1F64195": "VERIDAS Digital Authentication Solutions S.L",
    "8C1F643D2": "UVIRCO Technologies",
    "8C1F64BD2": "attocube systems AG",
    "8C1F64D27": "Taiv Inc",
    "8C1F64183": "NICE Total Cash Management Co., Ltd.",
    "8C1F6488A": "Longoo Limited",
    "8C1F642A0": "Connected Development",
    "8C1F6498D": "Aksel sp. z o.o.",
    "8C1F642A2": "SERAP",
    "8C1F64AA7": "SHENZHEN ANLIJI ELECTRONICS CO.,LTD",
    "8C1F64866": "Unitron Systems b.v.",
    "8C1F64D28": "MapleCloud Technologies",
    "8C1F64E76": "HEITEC AG",
    "8C1F64E2E": "RADA Electronics Industries Ltd.",
    "8C1F645A4": "DAVE SRL",
    "001BC509C": "MECCALTE SPA",
    "8C1F6412D": "YUYAMA MFG Co.,Ltd",
    "8C1F64104": "Timebeat.app Ltd",
    "8C1F64F83": "Vishay Nobel AB",
    "8C1F641DD": "Beijing Shengtongnaan Technology Development Co ., Ltd",
    "8C1F6449C": "Red Lion Europe GmbH",
    "70B3D57AE": "Exi Flow Measurement Ltd",
    "8C1F6416B": "TKR Spezialwerkzeuge GmbH",
    "8C1F644E3": "Exi Flow Measurement Ltd",
    "8C1F64A49": "Integer.pl S.A.",
    "8C1F647A5": "Potter Electric Signal Company",
    "8C1F6404F": "ISILINE srl",
    "8C1F644BF": "Smart Monitoring Innovations Private Limited",
    "8C1F6476C": "Guan Show Technologe Co., Ltd.",
    "8C1F6481D": "Gogo BA ",
    "8C1F643C2": "Samuel Cosgrove",
    "8C1F64C6E": "SAFE INSTRUMENTS",
    "40D8550D7": "Avant Technologies",
    "0050C2F71": "RF Code",
    "0050C29CC": "Hirotech inc.",
    "0050C2414": "Talleres de Escoriaza SA",
    "0050C21F5": "Abest Communication Corp.",
    "0050C2F05": "ESI Ventures, LLC",
    "0050C2F9B": "NEWELL TECHNOLOGIES LIMITED ",
    "0050C234D": " BMK professional electronics GmbH",
    "40D85508E": "Lyngsoe Systems",
    "0050C2D02": "SURVALENT TECHNOLOGY CORP",
    "0050C2A2A": "Astronics Custom Control Concepts",
    "0050C2462": "CT Company",
    "0050C2FB3": "CT Company",
    "0050C2F24": "CT Company",
    "40D855072": "CT Company",
    "40D855098": "DAVE SRL",
    "0050C227C": "Danlaw Inc",
    "0050C2E64": "Edgeware AB",
    "0050C27D5": "DEUTA-WERKE GmbH",
    "40D85511C": "DEUTA-WERKE GmbH",
    "40D8551A1": "KRONOTECH SRL",
    "0050C2CF9": "Elbit Systems of America",
    "0050C2F6B": "Algodue Elettronica Srl",
    "0050C21FA": "SP Controls, Inc",
    "0050C2811": "Open System Solutions Limited",
    "40D855010": "APG Cash Drawer, LLC",
    "0050C2F74": "Thor Technologies Pty Ltd",
    "0050C224E": "Scharff Weisberg Systems Integration Inc",
    "0050C28F9": "Honeywell",
    "0050C2616": "Honeywell",
    "0050C2872": "Oliotalo Oy",
    "0050C2B1F": "Atlas Copco IAS GmbH",
    "40D8550F4": "MB connect line GmbH Fernwartungssysteme",
    "0050C2B25": "Triax A/S",
    "0050C2CD3": "Covidence A/S",
    "0050C28F4": "Critical Link LLC",
    "0050C2E67": "Critical Link LLC",
    "0050C2384": "Wireless Reading Systems Holding ASA",
    "0050C2AE3": "PSD",
    "0050C225F": "Albert Handtmann Maschinenfabrik GmbH&Co.KG",
    "0050C2192": "Advanced Concepts, Inc.",
    "0050C23BA": "Phytec Messtechnik GmbH",
    "0050C28F0": "Xentras Communications",
    "0050C202C": "Narrowband Telecommunications",
    "0050C2144": "Phytec Messtechnik GmbH",
    "0050C26C6": "MedAvant Healthcare",
    "0050C2230": "AutoTOOLS group Co. Ltd.",
    "0050C2134": "DRS Photronics",
    "0050C2AF0": "Fr. Sauter AG",
    "0050C2194": "Cominfo, Inc.",
    "0050C2E4C": "Applied Micro Electronics AME bv",
    "0050C2E70": "DORLET SAU",
    "0050C2C6C": "DORLET SAU",
    "0050C2796": "DORLET SAU",
    "0050C2DDE": "DRS Imaging and Targeting Solutions",
    "0050C2172": "SAET I.S. S.r.l.",
    "0050C257D": "Sierra Video Systems",
    "40D855155": "Telefrang AB",
    "0050C2A8B": "Navicron Oy",
    "0050C23C0": "EDA Industries Pte. Ltd",
    "0050C2681": "Owasys Advanced Wireless Devices",
    "0050C2B37": "IAdea Corporation",
    "0050C2D43": "DSP4YOU LTd",
    "0050C2F90": "SecureTech Systems, Inc.",
    "0050C273A": "Naturela Ltd.",
    "0050C2178": "Partner Voxstream A/S",
    "0050C25BD": "NOMUS COMM SYSTEMS",
    "0050C2E18": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C220E": "PYRAMID Computer GmbH",
    "0050C2EAE": "Alyrica Networks",
    "0050C292A": "Rohde&Schwarz Topex SA",
    "0050C2BFA": "Rohde&Schwarz Topex SA",
    "40D855182": "Georg Neumann GmbH",
    "0050C2D47": "Rohde&Schwarz Topex SA",
    "0050C2EAD": "Rohde&Schwarz Topex SA",
    "0050C25A6": "Plastic Logic",
    "0050C2FAB": "Aplex Technology Inc.",
    "0050C23E5": "Vacon Plc",
    "0050C233A": "United Telecoms Ltd",
    "0050C2552": "Elfiq Inc.",
    "0050C2C61": "HaiVision Systems Incorporated",
    "0050C2A44": "TORC Technologies",
    "0050C2F87": "Vaisala Oyj",
    "0050C23E2": "SysNova",
    "0050C20B8": "LAN Controls, Inc.",
    "0050C2956": "Sicon srl",
    "0050C212F": "Haag-Streit AG",
    "0050C2143": "AARTESYS AG",
    "0050C25C2": "Intuitive Surgical, Inc",
    "0050C2FA5": "Intuitive Surgical, Inc",
    "0050C253F": "Ellips B.V.",
    "0050C2A0E": "ENGICAM s.r.l.",
    "0050C247A": "Efficient Channel Coding",
    "0050C27A9": "DELTA TAU DATA SYSTEMS, INC.",
    "0050C2980": "Digital Payment Technologies",
    "0050C23B6": "Arecont Vision",
    "0050C2080": "AIM",
    "0050C2953": "Grupo Epelsa S.L.",
    "40D855183": "EMAC, Inc.",
    "0050C2360": "Digital Receiver Technology",
    "0050C2974": "EMAC, Inc.",
    "40D8551C4": "QED Advanced Systems Limited",
    "40D8551C1": "Triamec Motion AG",
    "40D8551D0": "Webeasy BV",
    "40D8551E2": "ELNEC s.r.o.",
    "40D85519F": "Patria Aviation Oy",
    "40D8551A0": "Futaba Corporation",
    "40D85519B": "Northern Star Technologies",
    "40D85519C": "Parris Service Corporation",
    "40D855199": "PRESSOL Schmiergeraete GmbH",
    "40D8551AE": "Autonomous Solutions, Inc",
    "40D8551AA": "Broachlink Technology Co.,Limited",
    "40D8551A7": "ENTEC Electric & Electronic CO., LTD",
    "40D8551A8": "Multiobrabotka",
    "40D8551B6": "Magic Systems",
    "40D85519E": "Thirdwayv Inc.",
    "40D85518F": "Beat Sensing co. , ltd.",
    "40D85517F": "Telvent",
    "40D855170": "ICS Eletronics",
    "40D855181": "eROCCA",
    "40D855168": "OPASCA Systems GmbH",
    "40D85515D": "Actronic Technologies",
    "40D855159": "PLATINUM GmbH",
    "40D85514F": "TDS Software Solutions Pty Ltd",
    "40D85513C": "shanghai Anjian Information technology co. , ltd.",
    "40D85513D": "Perm Scientific-Industrial Instrument Making Company JSC",
    "40D855143": "Tokyo Drawing Ltd.",
    "40D85512B": "Mango DSP, Inc.",
    "40D855142": "Tetracore, Inc.",
    "40D85513E": "hanatech",
    "40D855137": "GDE Polska",
    "40D85512F": "Private",
    "40D855139": "WOW System",
    "40D855127": "LIGHTSTAR",
    "40D85511A": "Sicon srl",
    "40D855117": "RCS Energy Management Limited",
    "40D855120": "ObjectFab GmbH",
    "40D8550FD": "MONOGRAM technologies ltd",
    "40D855107": "Smith Meter, Inc",
    "40D8550E8": "HEITEC AG",
    "40D8550EA": "A-Z-E",
    "40D8550EE": "Siegmar Zander HuSWare",
    "40D8550ED": "IntelliDesign Pty Ltd",
    "40D8550D6": "deskontrol electronics",
    "40D8550DE": "VPG",
    "40D8550DC": "NVS Technologies Inc",
    "40D8550D4": "Mitsubishi Heavy Industries, Ltd.",
    "40D8550B5": "DATA SHARING CONSULTING",
    "40D8550B0": "Micrologic",
    "40D8550AF": "EnVerv Inc.",
    "40D8550AE": " GD Mission Systems",
    "40D8550AD": "Space Micro",
    "40D8550B2": "Ever Trend Technology Development Limited",
    "40D8550C4": "Inspired Systems",
    "40D85508F": "Excelitas",
    "40D855091": "KDT",
    "40D8550A6": "Alumbra Produtos El\u00e9tricos e Eletr\u00f4nicos Ltda",
    "40D8550A8": "Baudisch Electronic GmbH",
    "40D85509B": "Tokyo Denki Gijutsu Kogyo",
    "40D855095": "Heart Force Medical",
    "40D85508A": "Leder Elektronik Design",
    "40D85506E": "C-COM Satellite Systems Inc.",
    "40D85506B": "BRS Sistemas Eletr\u00f4nicos",
    "40D855069": "Smartcom-Bulgaria AD",
    "40D85506D": "BroadSoft, INC",
    "40D855082": "ard sa",
    "40D85505E": "inoage GmbH",
    "40D85507E": "TESCOM CORPORATION",
    "40D855073": "Diamond Technologies, Inc",
    "40D855057": "Tammermatic Group Oy",
    "40D855053": "Amantys Ltd",
    "40D855046": "Circuitlink Pty Ltd",
    "40D855048": " GD Mission Systems",
    "40D855042": "Mango Communicaitons Inc.",
    "40D85503C": "Computer System Co.,Ltd",
    "40D855040": "GHL Systems Berhad",
    "40D855033": "Ermes Elettronica s.r.l.",
    "40D855038": "Special Measurements Labs LLC",
    "40D855013": "Grande Vitesse Systems",
    "40D855015": "BITMILL srl",
    "40D85501F": "Sitep Italia Spa",
    "40D855029": "Depro Electronique",
    "40D85500D": "HuNS",
    "40D855006": "Bactest Limited",
    "40D855003": "AlphaNavigation coltd",
    "40D85500E": "Brightwell Dispensers",
    "0050C2FFA": "Nupoint Systems Inc.",
    "40D85501A": "MEGGITT DEFENSE SYSTEMS INC.",
    "0050C2FEC": "First System Technology Co., Ltd.",
    "0050C2FEA": "Brunel GmbH Section Communications",
    "0050C2FE4": "RTT Mobile Interpretation",
    "0050C2FD8": "Ease Inc.",
    "0050C2FED": "LOGISOL Kft.",
    "0050C2FCB": "Propagation Systems Limited",
    "0050C2FB2": "Preston Industries dba PolyScience",
    "0050C2FAE": "ATI Automa\u00e7\u00e3o Telecomunica\u00e7\u00f5es e Inform\u00e1tica Ltda",
    "0050C2FAF": "Vremya-CH JSC",
    "0050C2FB4": "MC-monitoring SA",
    "0050C2F9E": "Matsusada Precision Inc.",
    "0050C2FBF": "MYLOGIC",
    "0050C2FC5": "Sakura Seiki Co.,Ltd.",
    "0050C2FBE": "senTec Elektronik GmbH",
    "0050C2FAD": "Finishing Brands",
    "0050C2FAA": "YJSYSTEM",
    "0050C2F92": "CONET Solutions GmbH",
    "0050C2F93": "Baudisch Electronic GmbH",
    "0050C2F89": "CSA Engineering AG",
    "0050C2F62": "EMAC, Inc.",
    "0050C2F50": "Moritex Corporation",
    "0050C2F6D": "Pro Design Electronic GmbH",
    "0050C2F6E": "Smith Meter, Inc.",
    "0050C2F5E": "Y-cam Solutions Ltd",
    "0050C2F5C": "DSP DESIGN LTD",
    "0050C2F4B": "Supranet",
    "0050C2F5A": "Sentry 360 Security",
    "0050C2F56": "Monsoon Solutions, Inc.",
    "0050C2F3C": "Vemco Sp. z o. o.",
    "0050C2F37": "Rosslare Enterprises Limited",
    "0050C2F39": "InForce Computing, Inc.",
    "0050C2F2B": "Bio Guard component & technologies",
    "0050C2F31": "Ruetz Technologies GmbH",
    "0050C2F18": "Vitec Multimedia",
    "0050C2F3D": "Project service S.a.s",
    "0050C2F17": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2F15": "Sascal Displays Ltd",
    "0050C2F46": "Reason Tecnologia S.A.",
    "0050C2F41": "FairyDevices Inc.",
    "0050C2F25": "Samway Electronic SRL",
    "0050C2EF8": "HCL Technologies",
    "0050C2F0B": "Treehaven Technologies, Inc.",
    "0050C2F09": "Wheatstone Corporation",
    "0050C2EDB": "BELIK S.P.R.L.",
    "0050C2EDA": "Joint Stock Company \"Svyaz Inginiring M\"",
    "0050C2ED9": "Plasmatronics pty ltd",
    "0050C2EBE": "Global Tecnologia LTDA.",
    "0050C2ECF": "TAIWAN HIPLUS CORPORATION",
    "0050C2ED1": "Arcontia Technology AB",
    "0050C2EB2": "Otaki Electric Corporation",
    "0050C2EB3": "AR RF/Microwave Instrumentation",
    "0050C2EA4": "head",
    "0050C2EA7": "Saia-Burgess Controls AG",
    "0050C2EA1": "TEX COMPUTER SRL ",
    "0050C2EB8": "dspnor",
    "0050C2E9C": "SPARQ systems",
    "0050C2E63": "Prima sistemi",
    "0050C2E6D": "Allerta Inc",
    "0050C2E6E": "Power-One Italia S.p.A",
    "0050C2E6B": "Sika Technology AG",
    "0050C2E69": "Netmaker",
    "0050C2E8D": "SystemAdvanced Co,Ltd",
    "0050C2E8C": "Epec Oy",
    "0050C2E79": "MCS MICRONIC Computer Systeme GmbH",
    "0050C2E73": "ACS Motion Control Ltd.",
    "0050C2E8B": "RPA Electronic Solutions, Inc.",
    "0050C2E7D": "AEL Microsystems Limited",
    "0050C2E7E": "Swareflex GmbH",
    "0050C2E4E": "Institute For Information Industry",
    "0050C2E55": "TTi Ltd",
    "0050C2E54": "Arcos Technologies LTD",
    "0050C2E52": "Famas System S.p.A.",
    "0050C2E5C": "MCOPIA Co., Ltd",
    "0050C2E3B": "Nanosolution Inc.",
    "0050C2E39": "Telemetrics Inc.",
    "0050C2E38": "Aesir Copenhagen",
    "0050C2E45": "Stichting Sunrise",
    "0050C2E60": "HORIZON.INC",
    "0050C2E28": "Teplovodokhran",
    "0050C2E27": "CONTROL SYSTEMS Srl",
    "0050C2E26": "Cinetix s.r.l.",
    "0050C2E1E": "Lo-Q plc",
    "0050C2E22": "Michael Riedel Transformatorenbau GmbH",
    "0050C2E30": "Goennheimer Elektronic GmbH",
    "0050C2E15": "IHI Scube Co.,Ltd",
    "0050C2E0F": "Trentino Systems",
    "0050C2E10": "Radinetworks Co., Ltd",
    "0050C2DF2": "Ocean Sonics",
    "0050C2DF8": "Tommotek (WA) Pty Ltd.",
    "0050C2DDD": "A&A GENERAL SRL",
    "0050C2DD8": "Leonardo UK Ltd",
    "0050C2DD0": "IDC Solutions Pty Ltd",
    "0050C2DD9": "Metraware",
    "0050C2DA7": "Capton",
    "0050C2DB0": "IMAGO Technologies GmbH",
    "0050C2DBB": "Esensors, Inc.",
    "0050C2DBE": "WITHSYSTEM Co.,Ltd",
    "0050C2DC0": "Security Services Group (SSG)",
    "0050C2DB3": "LAUDA DR. R. WOBSER GMBH & CO. KG",
    "0050C2DCA": "Tele Data Control",
    "0050C2DC4": "Keith & Koep GmbH",
    "0050C2D92": "Manz",
    "0050C2D8E": "LSD Science&Technology Co.,Ltd.",
    "0050C2D8D": "CS-Instruments",
    "0050C2DA5": "megatec electronic GmbH",
    "0050C2DA3": " GD Mission Systems",
    "0050C2DA6": "Manitowoc Ice ",
    "0050C2DA0": "Precision Remotes",
    "0050C2D9F": "BitWise Controls",
    "0050C2D94": "Software Effect Enterprises, Inc",
    "0050C2D90": "Dumps Electronic",
    "0050C2D98": "Rong Shun Xuan Corp.",
    "0050C2D87": "Electrolight Shivuk (1994) Ltd.",
    "0050C2D7E": "LYNX Technik AG",
    "0050C2D62": "EMAC, Inc.",
    "0050C2D63": "DATAREGIS S.A.",
    "0050C2D7C": "Microcubs Systems Pvt Ltd",
    "0050C2D71": "EMAC, Inc.",
    "0050C2D68": "HiSpeed Data, Inc.",
    "0050C2D5C": "Ibetor S.L.",
    "0050C2D59": "BUANCO SYSTEM A/S",
    "0050C2D4F": "SECOM GmbH",
    "0050C2D4C": "DALOG Diagnosesysteme GmbH",
    "0050C2D53": "Telemerkki Oy",
    "0050C2D2D": "Cadi Scientific Pte Ltd",
    "0050C2D27": "Fr.Sauter AG",
    "0050C2D26": "RCH GROUP",
    "0050C2D1C": "Recon Dynamics, LLC",
    "0050C2D34": "GAON TECH corp.",
    "0050C2D20": "7+ Kft",
    "0050C2CF6": "Epec Oy",
    "0050C2D14": "SAET I.S.",
    "0050C2D10": "Rosslare Enterprises Ltd.",
    "0050C2D12": "Tokyo Weld Co.,Ltd.",
    "0050C2D0F": "Saia-Burgess Controls AG",
    "0050C2D0B": "CODACO ELECTRONIC s.r.o.",
    "0050C2D0A": "Airpoint Co., Ltd.",
    "0050C2D03": "Peekel Instruments B.V.",
    "0050C2CF7": "Armour Home Electronics LTD",
    "0050C2CE6": "APLICA TECHNOLOGIES",
    "0050C2CE7": "Echola Systems",
    "0050C2CFD": "AIRFOLC,INC.",
    "0050C2CE9": "Private",
    "0050C2CD4": "SCHRAML GmbH",
    "0050C2CD5": "Arcos Technologies Ltd.",
    "0050C2CDB": "RUTTER INC",
    "0050C2CC2": "ConectaIP Tecnologia S.L.",
    "0050C2CCA": "SANMINA SHENZHEN",
    "0050C2CDA": "taskit GmbH",
    "0050C2C94": "ISIS ENGINEERING, S.A.",
    "0050C2C91": "Media Technologies Ltd.",
    "0050C2CB1": "ZK Celltest Inc",
    "0050C2CAF": "Fr. Sauter AG",
    "0050C2C89": "Creative Micro Design",
    "0050C2C9B": "Sm electronic co.",
    "0050C2C97": "MSTRONIC CO., LTD.",
    "0050C2C83": "Gronic Systems GmbH",
    "0050C2C85": "Peek Traffic Corporation",
    "0050C2C84": "DOMIS",
    "0050C2CA9": "Intelligent Devices",
    "0050C2CAB": "SAE IT-systems GmbH & Co. KG",
    "0050C2C75": "Rovsing A/S",
    "0050C2C6D": "Deansoft CO., Ltd.",
    "0050C2C6A": "ELECTRONICA KELD",
    "0050C2C64": "Pal Software Service Co.,Ltd.",
    "0050C2C5F": "Icon Time Systems",
    "0050C2C5B": "MICRO TECHNICA",
    "0050C2C6F": "MSB Elektronik und Geraetebau GmbH",
    "0050C2C71": "Sequoia Technology Group Ltd",
    "0050C2C48": "Cytek Media Systems, INC.",
    "0050C2C18": "Linuxstamp Designs, LLC",
    "0050C2C28": "ELREHA GmbH",
    "0050C2C3F": "ANXeBusiness Corporation",
    "0050C2BF2": "Saia-Burgess Controls AG",
    "0050C2BF6": "NOLAM EMBEDDED SYSTEMS",
    "0050C2BF7": "Phytec Messtechnik GmbH",
    "0050C2BF4": "Monarch Innovative Technologies Pvt Ltd",
    "0050C2BD7": "SLAT",
    "0050C2BEE": "Private",
    "0050C2BEA": "Daeyoung inc.",
    "0050C2C03": "Volumatic Limited.",
    "0050C2BE6": "Docobo Ltd",
    "0050C2BE0": "Phaedrus Limited",
    "0050C2BE2": "Convergent Bioscience Ltd.",
    "0050C2BDE": "Evo-Teh d.o.o.",
    "0050C2BDC": "SS Systems LLC",
    "0050C2BBD": "ITS Telecom",
    "0050C2BBF": "Phytec Messtechnik GmbH",
    "0050C2BC8": "AGWTech Ltd",
    "0050C2B9B": "Telventy Energia S.A.",
    "0050C2BA1": "Transtechnik GmbH & Co.KG",
    "0050C2BA2": "Logical Tools s.r.l.",
    "0050C2BD0": "EDC wifi",
    "0050C2BCF": "Epiko, elektronski sistemi d.o.o.",
    "0050C2BAD": "Prediktor AS",
    "0050C2BAB": "iDeal Teknoloji Bilisim Cozumleri A.S.",
    "0050C2B7E": "NARETRENDS",
    "0050C2B84": "Innovate Software Solutions Pvt Ltd",
    "0050C2B90": "NAONWORKS Co., Ltd",
    "0050C2B94": "Tritech International Ltd",
    "0050C2B8F": "Gentec",
    "0050C2B8A": "MicroPoise",
    "0050C2B89": "ENTEC Electric & Electronic Co., LTD.",
    "0050C2B87": "EMAC, Inc.",
    "0050C2B59": "SLICAN sp. z o.o.",
    "0050C2B57": "Phytec Messtechnik GmbH",
    "0050C2B55": "SANYO ELECTRONIC INDUSTRIES CO.,LTD",
    "0050C2B3D": "AMS",
    "0050C2B40": "Tecnint HTE SRL",
    "0050C2B64": "FEW Bauer GmbH",
    "0050C2B65": "Peek Traffic Corporation",
    "0050C2B45": "Efftronics Systems (P) Ltd",
    "0050C2B6D": "Sound Metrics Corp",
    "0050C2B21": "Phytron-Elektronik GmbH",
    "0050C2B22": "FarSite Communications Limited",
    "0050C2B12": "CM Elektronik GmbH",
    "0050C2B27": "ATEME",
    "0050C2B24": "Teledyne Defence Limited",
    "0050C2B10": "Marine Entertainment Systems Ltd",
    "0050C2B2F": "IntelliVision Technologies, Corp",
    "0050C2B36": "CRDE",
    "0050C2AEA": "EMBEDIA",
    "0050C2AEB": "UMLogics Corporation",
    "0050C2AF3": "Palomar Products, Inc.",
    "0050C2AE5": "ABS Gesellschaft f. Automatisierung, Bildverarbeitung und Software mbH",
    "0050C2AE2": "Power Medical Interventions",
    "0050C2B0A": "Indutherm Giesstechnologie GmbH",
    "0050C2AFD": "HomeScenario, Inc.",
    "0050C2AFA": "Absolute Fire Solutions Inc.",
    "0050C2B02": "MASTER CO LTD",
    "0050C2AE1": "EVERCARE",
    "0050C2ADA": "Essepie Srl",
    "0050C2ABB": "Volantic AB",
    "0050C2AD5": "Mighty Lube Systematic Lubrication, Inc.",
    "0050C2AC8": "Palladio Systeme GmbH",
    "0050C2AA2": "ELPA sas",
    "0050C2AA1": "ELREM ELECTRONIC AG",
    "0050C2A8C": "Redwire, LLC",
    "0050C2A87": "Littlemore Scientific",
    "0050C2A90": "S.two Corporation",
    "0050C2A91": "Ceron Tech Co.,LTD",
    "0050C2AA8": "Nexans Cabling Solutions",
    "0050C2AAD": "Update Systems Inc.",
    "0050C2AAA": "Flexible Picture Systems",
    "0050C2AA6": "Bassett Electronic Systems ltd",
    "0050C2A88": "S-SYS",
    "0050C2A89": "CA Traffic Ltd",
    "0050C2A58": "EPL",
    "0050C2A5D": "MECC CO., LTD.",
    "0050C2A41": "Meiryo Denshi Corp.",
    "0050C2A54": "Diamond Point International (Europe) Ltd",
    "0050C2A53": "Quality & Design",
    "0050C2A73": "KYOEI ENGINEERING Co.,Ltd.",
    "0050C2A7C": "Pneu-Logic Corporation",
    "0050C2A6A": "Infocrossing",
    "0050C2A4C": "VasoNova, Inc.",
    "0050C2A2F": "DragonFly Scientific LLC",
    "0050C2A33": "Fuji Firmware",
    "0050C2A31": "Coolit Systems, Inc.",
    "0050C2A3B": "IPcontrols GmbH",
    "0050C2A08": "LabJack Corporation",
    "0050C2A19": "Icon Time Systems",
    "0050C2A16": "Baudisch Electronic GmbH",
    "0050C2A04": "TP Radio",
    "0050C29E5": "Halliburton Far East Pte Ltd",
    "0050C29E1": "Enreduce Energy Control AB",
    "0050C29F1": "IDEAS s.r.l.",
    "0050C29ED": "Picell B.V.",
    "0050C29E9": "Ciemme Sistemi Spa",
    "0050C29C6": "Protronic GmbH",
    "0050C29B9": "ISA - Intelligent Sensing Anywhere, S.A.",
    "0050C29B7": "St. Michael Strategies",
    "0050C29B4": "EUKREA ELECTROMATIQUE SARL",
    "0050C29A8": "DOMIS SA",
    "0050C2992": "IDT Sound Processing Corporation",
    "0050C2998": "Phytec Messtechnik GmbH",
    "0050C295A": "TechnoAP",
    "0050C2970": "Tsuji Electronics Co.,Ltd",
    "0050C298D": "Mecos AG",
    "0050C298A": "MEV Limited",
    "0050C2988": "Pantel International",
    "0050C2983": "Bogart Engineering",
    "0050C297C": "Creacon Technologies B.V.",
    "0050C297B": "SMARTQUANTUM SA",
    "0050C2966": "PCM Industries",
    "0050C295D": "full electronic system",
    "0050C2946": "MEGWARE Computer GmbH",
    "0050C2944": "Wireonair A/S",
    "0050C2936": "Margaritis Engineering",
    "0050C2931": "Korea Telecom Internet Solutions (KTIS)",
    "0050C2934": "Xilar Corp.",
    "0050C2932": "SMAVIS Inc.",
    "0050C294A": "TruMedia Measurement Ltd.",
    "0050C294B": "Piller engineering Ltd.",
    "0050C292E": "Goanna Technologies Pty Ltd",
    "0050C2914": "IO-Connect",
    "0050C2911": "Vapor Rail",
    "0050C291C": "MangoDSP",
    "0050C2929": "Flight Deck Resources",
    "0050C2903": "EcoAxis Systems Pvt. Ltd.",
    "0050C2900": "Magor Communications Corp",
    "0050C2904": "R2Sonic, LLC",
    "0050C2915": "Verint Systems Ltd.",
    "0050C290D": "EVK DI Kerschhaggl GmbH",
    "0050C28F3": "Curtis Door Systems Inc",
    "0050C28FE": "Cross Country Systems AB",
    "0050C28FC": "Symetrics Industries",
    "0050C28F8": "RMSD LTD",
    "0050C28E7": "MKT Systemtechnik",
    "0050C28DA": "DOCUTEMP, INC",
    "0050C28D5": "Peek Traffic Corp",
    "0050C28D9": "Industrial Control and Communication Limited",
    "0050C28E9": "UNIDATA",
    "0050C28F1": "Detection Technologies Ltd.",
    "0050C28D3": "IFAM GmbH",
    "0050C28B8": "DSS Networks, Inc.",
    "0050C28B9": "ACD Elektronik GmbH",
    "0050C28B7": "Calnex Solutions plc",
    "0050C28BF": "ARISTO Graphic Systeme GmbH & Co. KG",
    "0050C28BC": "Honeywell Sensotec",
    "0050C28BD": "Matrix Switch Corporation",
    "0050C28BB": "smtag international ag",
    "0050C28C9": "A+S Aktuatorik und Sensorik GmbH",
    "0050C28B1": "RingCube Technologies, Inc.",
    "0050C28AE": "DESARROLLO DE SISTEMAS INTEGRADOS DE CONTROL S.A.",
    "0050C28C3": "Byte Paradigm",
    "0050C28AF": "Xelerated",
    "0050C289D": "Systemtechnik GmbH",
    "0050C2720": "Colorado Engineering Inc.",
    "0050C2868": "Aethon, Inc.",
    "0050C2866": "Saia Burgess Controls AG",
    "0050C2891": "Admiral Secure Products, Ltd.",
    "0050C288F": "Keynote SIGOS GmbH",
    "0050C2881": "InnoTrans Communications, Inc.",
    "0050C287D": "AT&T Government Solutions",
    "0050C2879": "MILESYS",
    "0050C2875": "Phytec Messtechnik GmbH",
    "0050C2877": "Motion Analysis Corp",
    "0050C2830": "CompuShop Services LLC",
    "0050C282D": "Elmec, Inc.",
    "0050C2848": "DASA ROBOT Co., Ltd.",
    "0050C2852": "eInfochips Ltd.",
    "0050C284C": "Performance Motion Devices",
    "0050C2837": "ID-KARTA s.r.o.",
    "0050C2823": "Robot Visual Systems GmbH",
    "0050C2821": "MISCO Refractometer",
    "0050C281A": "InfoGLOBAL",
    "0050C2826": "HEWI Heinrich Wilke GmbH",
    "0050C2803": "dB Broadcast Limited",
    "0050C2813": "Intelleflex Corporation",
    "0050C280D": "Acube Systems s.r.l.",
    "0050C27E2": "DIGITROL LTD",
    "0050C27E4": "Meta Vision Systems Ltd.",
    "0050C27D4": "WR Systems, Ltd.",
    "0050C27CD": "Precision MicroControl Corporation",
    "0050C27DD": "LS Elektronik AB",
    "0050C27EE": "NH Research",
    "0050C27D6": "International Mining Technologies",
    "0050C27C1": "Primary Integration Encorp LLC",
    "0050C27BE": "Powerlinx, Inc.",
    "0050C27AE": "Global Tel-Link",
    "0050C27B0": "IMP Telekom",
    "0050C27A5": "Quantum Medical Imaging",
    "0050C279F": "ZAO NPC",
    "0050C278F": "ELMAR electronic",
    "0050C278E": "GLOBALCOM ENGINEERING SRL",
    "0050C278D": "Telairity",
    "0050C2793": "Enertex Bayern GmbH",
    "0050C2790": "GE Security-Kampro",
    "0050C278C": "Giga-tronics, Inc.",
    "0050C2767": "EID",
    "0050C2787": "Austco Marketing & Service (USA) ltd.",
    "0050C276D": "Mobilisme",
    "0050C275D": "Fluid Analytics, Inc.",
    "0050C273D": "cryptiris",
    "0050C274C": "Saia-Burgess Controls AG",
    "0050C2714": "T.E.AM., S. A.",
    "0050C2712": "Pasan SA",
    "0050C2710": "Wharton Electronics Ltd",
    "0050C272B": "Sequestered Solutions",
    "0050C2736": "Nika Ltd",
    "0050C272F": "Priority Electronics Ltd",
    "0050C270A": "Efficient Channel Coding",
    "0050C2705": "Hauch & Bach ApS",
    "0050C271E": "ASKI Industrie Elektronik Ges.m.b.H.",
    "0050C26E2": "Phytec Messtechnik GmbH",
    "0050C26F6": "AV SatCom AS",
    "0050C26E4": "MangoDSP",
    "0050C26ED": "Sechan Electronics, Inc.",
    "0050C26F8": "RV Technology Limited",
    "0050C26F1": "ITS Telecom",
    "0050C26CB": "Stream Processors",
    "0050C26A6": "Victory Concept Industries Ltd.",
    "0050C26A2": "Cadi Scientific Pte Ltd",
    "0050C26A4": "TELETASK",
    "0050C26B6": "CommoDaS GmbH",
    "0050C26AF": "Nanoradio AB",
    "0050C26BF": "Optoplan as",
    "0050C26D6": "ADL Electronics Ltd.",
    "0050C26DE": "Laser Tools & Technics Corp.",
    "0050C26D1": "Schnick-Schnack-Systems GmbH",
    "0050C268C": "Isochron Inc",
    "0050C2687": "Access Specialties, Inc",
    "0050C2685": "Datamars SA",
    "0050C2683": "MEGGITT Safety System",
    "0050C26A1": "PRICOL LIMITED",
    "0050C269D": "Dvation.co.,Ltd",
    "0050C267E": "SAIA Burgess Controls AG",
    "0050C2693": "Tech Comm, Inc.",
    "0050C267B": "Sparton Electronics",
    "0050C2673": "Ferrari electronic AG",
    "0050C268B": "SIMTEK INC.",
    "0050C269A": "StoreTech Limited",
    "0050C264F": "MA Lighting Technology GmbH",
    "0050C264E": "Northern Power",
    "0050C264B": "MangoDSP",
    "0050C264A": "CPqD",
    "0050C265F": "Invocon, Inc.",
    "0050C2670": "Naim Audio",
    "0050C2651": "STAER SPA",
    "0050C2650": "Liquid Breaker, LLC",
    "0050C2642": "Stanton Technologies Sdn Bhd",
    "0050C2617": "NARINET, INC.",
    "0050C2627": "JungleSystem Co., Ltd.",
    "0050C2622": "2N TELEKOMUNIKACE a.s.",
    "0050C261C": "TestPro Systems, Inc.",
    "0050C262E": "TDM Ing\u00e9nierie",
    "0050C262C": "AirMatrix, Inc.",
    "0050C2629": "MacDonald Humfrey (Products) Ltd",
    "0050C2623": "SAFELINE SL",
    "0050C263C": "dPict Imaging, Inc.",
    "0050C2639": "Qstreams Networks Inc.",
    "0050C2608": "Silex Industrial Automation Ltd.",
    "0050C25FC": "FilmLight Limited",
    "0050C25FD": "MEG Electronic Inc.",
    "0050C25FF": "Gazelle Monitoring Systems",
    "0050C25FA": "LEA d.o.o.",
    "0050C25E4": "Buyang Electronics Industrial Co., Ltd.",
    "0050C25F1": "Technomarine JSC",
    "0050C25F4": "TeamProjects BV",
    "0050C25B2": "Syntronic AB",
    "0050C25B1": "Rosta Ltd",
    "0050C25C4": "ProMik GmbH",
    "0050C25C5": "Radiant Imaging, Inc.",
    "0050C25B9": "Taiwan Video & Monitor",
    "0050C2593": "Broadlight",
    "0050C2591": "Grosvenor Technology Ltd",
    "0050C258F": "XoIP Systems Pty Ltd",
    "0050C25AA": "Transenna AB",
    "0050C259A": "MultiTrode Pty Ltd",
    "0050C259F": "ads-tec GmbH",
    "0050C257F": "Orderite, Inc.",
    "0050C257E": "Digital Way",
    "0050C2575": "SOLYSTIC",
    "0050C2574": "Ingenier\u00eda Almud\u00c3\u00ad S.L.",
    "0050C2568": "GeoFocus, LLC",
    "0050C2565": "WORKPOWER TECNOLOGIA ELETRONICA LTDA-EPP",
    "0050C2573": "DATAMICRO Co., Ltd.",
    "0050C2558": "Bedo Elektronik GmbH",
    "0050C2564": "Last Mile Gear",
    "0050C2562": "C21 Systems Limited",
    "0050C255F": "Moog Broad Reach",
    "0050C2563": "ORTRAT, S.L.",
    "0050C2538": "EtherTek Circuits",
    "0050C2534": "Hyundai J. Comm",
    "0050C2535": "MMS Servis s.r.o.",
    "0050C252D": "Smartcom-Bulgaria AD",
    "0050C253D": "Digital communications Technologies",
    "0050C2541": "WAVES SYSTEM",
    "0050C254F": "Valtronic SA",
    "0050C250F": "Polystar Instruments AB",
    "0050C2516": "SOWA ELECTRIC CO., LTD.",
    "0050C252A": "OMNITRONICS PTY LTD",
    "0050C2529": "Phytec Messtechnik GmbH",
    "0050C251B": "Beta Lasermike Ltd",
    "0050C250B": "Logic Beach Inc",
    "0050C2509": "Hillcrest Laboratories, Inc.",
    "0050C2526": "AC SYSTEMS, s.r.o.",
    "0050C24D5": "SEBA Design Pty Ltd",
    "0050C24F5": "Monroe Electronics, Inc.",
    "0050C24F4": "O2RUN",
    "0050C24F1": "Packet Island Inc.",
    "0050C24E4": "Embigence GmbH",
    "0050C24FB": "BES Technology Group",
    "0050C24FA": "Cambridge Technology, Inc.",
    "0050C24EB": "Mandozzi Elettronica SA",
    "0050C2502": "Criterion Systems Limited",
    "0050C24BD": "Argon ST",
    "0050C24BE": "Digital Dynamics, Inc.",
    "0050C24BA": "Mistletoe Technologies",
    "0050C24C3": "Quantum3D, Inc.",
    "0050C24C1": "Movaz Networks, Inc.",
    "0050C24AF": "Orbis Oy",
    "0050C24AE": "ads-tec GmbH",
    "0050C24C6": "Rubin Ltd.",
    "0050C24C4": "Black Diamond Video, Inc.",
    "0050C24CB": "Verint Systems Ltd",
    "0050C24A3": "Protium Technologies, Inc.",
    "0050C24A4": "IEEE P1609 WG",
    "0050C248F": "DENGYOSHA co.,LTD.",
    "0050C248A": "Mobile Matrix, Inc.",
    "0050C248B": "ads-tec GmbH",
    "0050C2312": "Dese Technologies SL",
    "0050C24A0": "Advanced technologies & Engineering (pty) Ltd",
    "0050C248C": "UNITON AG",
    "0050C249C": "Envisacor Technologies Inc.",
    "0050C247C": "AUCONET GmbH",
    "0050C2494": "Ultimate Technology, Inc.",
    "0050C248E": "Teledyne Tekmar",
    "0050C2483": "SES",
    "0050C247F": "BRIT Inc.",
    "0050C249F": "GCS, Inc",
    "0050C2472": "KOP Ltd",
    "0050C246F": "Neuroware",
    "0050C2476": "Ascon S.p.a.",
    "0050C2478": "Metafix Inc.",
    "0050C2477": "SEV Tidsystem AB",
    "0050C2463": "Codem Systems, Inc.",
    "0050C245C": "Deister Electronic GmbH",
    "0050C245B": "Matra Electronique",
    "0050C2457": "Danbridge",
    "0050C2442": "Pico Computing, Inc.",
    "0050C2444": "Offshore Systems Ltd",
    "0050C2445": "MICRONIC s.r.o.",
    "0050C2449": "BLEILE DATENTECHNIK GmbH",
    "0050C2441": "Sammi Information Systems Co.,Ltd",
    "0050C2431": "Octatron, Inc.",
    "0050C242E": "Oelmann Elektronik GmbH",
    "0050C2438": "Telecom Protection Technologies Limited",
    "0050C243F": "ARVOO Imaging Products BV",
    "0050C2420": "Boundless Technologies",
    "0050C2424": "Metrolab Technology SA",
    "0050C2425": "Pinnacle Technology",
    "0050C23FB": "nVent, Schroff GmbH",
    "0050C240F": "BIR,INC.",
    "0050C2411": "Multimessage Systems Ltd.",
    "0050C240B": "Center VOSPI JSC",
    "0050C240A": "Contec Steuerungstechnik & Automation GmbH",
    "0050C240D": "Afonics Fibreoptics Ltd",
    "0050C2407": "AIE Etudes",
    "0050C2400": "SmartMotor AS",
    "0050C2415": "SensoTech GmbH",
    "0050C2402": "Numeron Sp. z o.o.",
    "0050C241F": "Avionica, Inc",
    "0050C23DF": "BiODE Inc.",
    "0050C23DB": "Osmetech Inc.",
    "0050C23DD": "ELMIC GmbH",
    "0050C23DA": "M5 Data Limited",
    "0050C23D5": "Fluke Biomedical, Radiation Management Services",
    "0050C23D2": "Adilec Enginyeria SL",
    "0050C23D7": "TTC TELEKOMUNIKACE Ltd",
    "0050C23ED": "The Board Room Inc.",
    "0050C23CE": "Ward Leonard Electric Company",
    "0050C23E6": "CRDE",
    "0050C23E4": "CUE, a.s.",
    "0050C23C1": "MicroTek Electronics, Inc.",
    "0050C23BD": "Bigbang L.T.D.",
    "0050C23B0": "Microtarget Tecnologia Digital Ltda.",
    "0050C23A0": "StreetFire Sound Labs, LLC",
    "0050C239F": "Isensix",
    "0050C23B4": "Contr\u00f4le Analytique inc.",
    "0050C238D": "A&G Soluzioni Digitali",
    "0050C23B1": "Specstroy-Svyaz Ltd",
    "0050C23AE": "Hankuk Tapi Computer Co., Ltd",
    "0050C23AF": "Norbit ODM AS",
    "0050C23A5": "LabJack Corporation",
    "0050C2355": "IHM",
    "0050C234F": "North Pole Engineering, Inc.",
    "0050C2382": "Colorado vNet",
    "0050C237E": "Ni.Co. S.r.l.",
    "0050C237A": "IDA Corporation",
    "0050C236B": "Minerva Technology Inc",
    "0050C2365": "VPG",
    "0050C2369": "Always On Wireless",
    "0050C2387": "Inoteska s.r.o.",
    "0050C2362": "AZD Praha s.r.o.",
    "0050C235D": "NetTest A/S",
    "0050C2358": "ALCEA",
    "0050C232E": "MANUSA-GEST, S.L.",
    "0050C2320": "DTASENSOR S.p.A.",
    "0050C2324": "ODIXION",
    "0050C233E": "CA Technology, Inc",
    "0050C2341": "Novx Systems",
    "0050C2334": "Picture Elements, Inc.",
    "0050C2335": "Nimcat Networks",
    "0050C2300": "Soredex Instrumentarium Oyj",
    "0050C22F9": "Digilent Inc.",
    "0050C22F5": "ADChips",
    "0050C22EE": "SHF Communication Technologies AG",
    "0050C22F2": "Eurotek Srl",
    "0050C22EA": "QUBIsoft S.r.l.",
    "0050C22CB": "FACTS Engineering LLC",
    "0050C22D5": "PIXY AG",
    "0050C22D7": "Neo Electronics Ltd",
    "0050C22E4": "iamba LTD.",
    "0050C22DC": "Wiener, Plein & Baus GmbH",
    "0050C22BD": "StorLink Semi",
    "0050C22B9": "Richmond Sound Design Ltd.",
    "0050C22B6": "Softier Inc.",
    "0050C22B2": "Smiths Detection",
    "0050C22A8": "DVTel Israel Ltd.",
    "0050C2274": "Fundaci\u00f3n TECNALIA Research & Innovation",
    "0050C2273": "Servicios Condumex, S. A. de C. V.",
    "0050C2292": "AMIRIX Systems",
    "0050C228F": "Spellman High Voltage Electronics Corp",
    "0050C2290": "EBNEURO SPA",
    "0050C2282": "Telvent",
    "0050C2280": "AGECODAGIS SARL",
    "0050C2285": "Littwin GmbH & Co KG",
    "0050C2264": "Confidence Direct Ltd",
    "0050C2265": "BELIK S.P.R.L.",
    "0050C2254": "Thales Communications Ltd",
    "0050C2247": "Gradual Tecnologia Ltda.",
    "0050C224B": "Azimuth Systems, Inc.",
    "0050C225A": "Zendex Corporation",
    "0050C2251": "DGT Sp. z o.o.",
    "0050C2227": "Intelligent Photonics Control",
    "0050C2228": "Intelligent Media Technologies, Inc.",
    "0050C2222": "imo-elektronik GmbH",
    "0050C222D": "asetek Inc.",
    "0050C2241": "Contronics Automacao Ltda",
    "0050C2239": "Stins Coman",
    "0050C221A": "MST SYSTEMS LIMITED",
    "0050C221E": "Applied Technologies Associates",
    "0050C2216": "Level Control Systems",
    "0050C2215": "VoiceCom AG",
    "0050C21FE": "Safetran Traffic Systems Inc.",
    "0050C21F6": "BAE SYSTEMS Controls",
    "0050C220D": "Varisys Ltd",
    "0050C220A": "Ferrari electronic AG",
    "0050C21EF": "M2 Technology Pty Ltd",
    "0050C21F4": "Covia, Inc",
    "0050C21D0": "Yazaki North America, Inc.",
    "0050C21DA": "GFI Chrono Time",
    "0050C21D5": "Redpoint Controls",
    "0050C21CD": "INCAA Informatica Italia  srl",
    "0050C21C7": "TWIN DEVELOPMENT S.A.",
    "0050C21C2": "Weltronics Corp.",
    "0050C21C0": "WillMonius Inc.",
    "0050C21AF": "PESA Inc.",
    "0050C21C1": "InfinitiNetworks Inc.",
    "0050C2197": "EPM Tecnologia e Equipamentos",
    "0050C219B": "Wilcoxon Research, Inc.",
    "0050C21A1": "Portalplayer, Inc",
    "0050C218A": "Basler Electric Company",
    "0050C21A8": "ANOVA BROADBAND",
    "0050C219E": "Paltronics, Inc.",
    "0050C2179": "Verifiber LLC",
    "0050C217C": "Jeffress Engineering Pty Ltd",
    "0050C2183": "Mixbaal S.A. de C.V.",
    "0050C2185": "Optical Wireless Link Inc.",
    "0050C2181": "Task 84 Spa",
    "0050C2180": "Zultys Technologies",
    "0050C217E": "Binary Wave Technologies Inc.",
    "0050C217B": "Broadstorm Telecom",
    "0050C2018": "CAD-UL GmbH",
    "0050C215C": "Aoptix Technologies",
    "0050C215B": "Dune Networks",
    "0050C2146": "APCON, Inc.",
    "0050C2013": "Sensys Technologies Inc.",
    "0050C2001": "JMBS Developpements",
    "0050C2000": "T.L.S. Corp.",
    "0050C215D": "Cepheid",
    "0050C2153": "Advanced Devices SpA",
    "0050C2152": "AirVast Technology Inc.",
    "0050C2141": "Time Terminal Adductor Group AB",
    "0050C2142": "Instrumeter A/S",
    "0050C2145": "ELC Lighting",
    "0050C2147": "UniSUR",
    "0050C2135": "ELAD SRL",
    "0050C2125": "Tecwings GmBh",
    "0050C2130": "U.S. Traffic Corporation",
    "0050C212D": "Megisto Systems, Inc.",
    "0050C212B": "Dong A Eltek Co., Ltd.",
    "0050C213D": "Formula One Management Ltd.",
    "0050C213F": "Sentito Networks",
    "0050C211F": "CSS Industrie Computer GmbH",
    "0050C211B": "Digital Vision AB",
    "0050C211C": "Stonefly Networks",
    "0050C2117": "Wintegra Ltd.",
    "0050C2116": "DSP Design, Ltd.",
    "0050C2115": "Vidco, Inc.",
    "0050C210D": "Implementa GmbH",
    "0050C2105": "Lumentis AB",
    "0050C2103": "Green Hills Software, Inc.",
    "0050C20FD": "Inducomp Corporation",
    "0050C20E6": "RouteFree, Inc.",
    "0050C20E7": "Century Geophysical Corp.",
    "0050C20F9": "Raven Industries",
    "0050C20F4": "Sysnet Co., Ltd.",
    "0050C20E2": "Visual Circuits Corp.",
    "0050C20DC": "Elbit Systems Ltd.",
    "0050C20CD": "LINET OY",
    "0050C20CB": "STUDIEL",
    "0050C20C6": "Advanced Medical Information Technologies, Inc.",
    "0050C20C5": "SAIA Burgess Controls AG",
    "0050C20A4": "Bounty Systems Pty Ltd.",
    "0050C209E": "Infinitec Networks, Inc.",
    "0050C2098": "EPEL Industrial, S.A.",
    "0050C2096": "Utronix Elektronikutreckling AB",
    "0050C20A0": "CYNAPS",
    "0050C2092": "DigitAll World Co., Ltd",
    "0050C208A": "Rising Edge Technologies",
    "0050C207C": "PLLB elettronica spa",
    "0050C2078": "Reselec AG",
    "0050C206C": "WaveCom Electronics, Inc.",
    "0050C2071": "NetVision Telecom",
    "0050C2073": "Alstom Signalling Ltd.",
    "0050C2067": "Riverlink Computers, Ltd.",
    "0050C206A": "Unimark",
    "0050C2042": "B.E.A.R. Solutions (Australasia) Pty, Ltd",
    "0050C2049": "Computer Concepts Corp",
    "0050C2046": "Private",
    "0050C2041": "Damler Chrysler Rail System (Signal) AB",
    "0050C203F": "Celotek Corp",
    "0050C202E": "Turtle Mountain Corp",
    "0050C202B": "Nova Engineering Inc.",
    "0050C2030": "Lockheed Martin Tactical Defense Systems Eagan",
    "0050C2034": "Ing. Buero W. Kanis GmbH",
    "0050C2022": "Ashling Microsystems Ltd.",
    "0050C2023": "Zabacom, Inc.",
    "0050C201C": "Tadiran Scopus",
    "0050C2008": "Portable Add-Ons",
    "0050C2FDA": "Bucher Automation Budapest",
    "40D8550D2": "Bucher Automation Budapest",
    "0050C2A59": "Televic Rail GmbH",
    "0050C289B": "Sensata Technologies",
    "40D8550FC": "eumig industrie-TV GmbH.",
    "0050C28D4": "INTERNET PROTOCOLO LOGICA SL",
    "0050C231C": "AxyomCore Inc.",
    "0050C293B": "Cleaveland/Price, Inc.",
    "0050C20BF": "Private",
    "0050C2689": "RF Code",
    "0050C22FA": "Tornado Modular Systems",
    "0050C2B04": "Ubiquiti Inc",
    "0050C2D1F": "Olympus NDT Canada",
    "0050C21C9": "modas GmbH",
    "0050C2769": "BES GmbH",
    "0050C20B4": "Wavefly Corporation",
    "0050C2DFD": "Wotbox ltd",
    "40D855019": "Nautel LTD",
    "0050C2DC6": "Fluid Components Intl",
    "0050C2FD9": "Figment Design Laboratories",
    "40D85502A": "Tinkerforge GmbH",
    "0050C2059": "Austco Marketing & Service (USA) ltd.",
    "0050C20F1": "Steelcase Inc.",
    "40D855163": "KMtronic ltd",
    "40D85510D": "Rite-Tech Industrial Co., Ltd.",
    "0050C224D": "Mettler Toledo",
    "0050C2AB5": "Mettler Toledo",
    "0050C292D": "APProSoftware.com",
    "0050C2065": "Clever Devices",
    "0050C29F4": "FSR, INC.",
    "0050C2255": "STMicroelectronics SRL",
    "0050C2C7B": "Honeywell",
    "0050C2D95": "Honeywell",
    "0050C2EA8": "MB connect line GmbH Fernwartungssysteme",
    "0050C2AD6": "Unisensor A/S",
    "0050C22E2": "Ballard Technology, Inc,",
    "0050C2624": "Comtest Networks",
    "0050C2CEC": "Erhardt+Leimer GmbH",
    "0050C2453": "Erhardt+Leimer GmbH",
    "40D8550CE": "EST Analytical",
    "0050C2083": "ard sa",
    "0050C2D88": "T+A elektroakustik GmbH & Co.KG",
    "0050C2FBD": "EATON FHF Funke + Huster Fernsig GmbH",
    "0050C2379": "Control LAN S.A.",
    "0050C2913": "Leonardo UK Ltd",
    "0050C23DE": "ABB Power Technologies S.p.A.  Unit\u00e0\u00a0 Operativa SACE (PTMV)",
    "0050C2FD4": "Insitu, Inc",
    "40D85517C": "Critical Link LLC",
    "0050C2339": "Secure Systems & Services",
    "0050C2AB4": "n3k Informatik GmbH",
    "0050C22C0": "Magellan Technology Pty Limited",
    "0050C28AD": "Thales Communications Inc",
    "0050C2A48": "Thales UK Limited",
    "0050C2777": "Euro Display Srl",
    "0050C289A": "Neptune Technology Group Inc.",
    "0050C267C": "Gogo BA ",
    "0050C27CE": "Gogo BA ",
    "0050C2EAF": "Gogo BA ",
    "40D85500B": "Gogo BA ",
    "0050C2F8F": "Computerwise, Inc.",
    "0050C20B5": "EXTREME COPPER,  INC.",
    "0050C2E98": "i3 International Inc.",
    "0050C2C63": "Potter Electric Signal Co. LLC",
    "0050C29DF": "CODEC Co., Ltd.",
    "0050C2DE6": "Fr. Sauter AG",
    "0050C2F14": "VISION SYSTEMS AERONAUTIC",
    "0050C2E3A": "Vocality International Ltd",
    "0050C24B1": "NSFOCUS Information Technology Co., Ltd.",
    "0050C22DA": "PYRAMID Computer GmbH",
    "0050C2C0A": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2403": "Rohde&Schwarz Topex SA",
    "40D85500C": "Aplex Technology Inc.",
    "0050C2E6A": "Aplex Technology Inc.",
    "0050C2E00": "Aplex Technology Inc.",
    "0050C2DAB": "Aplex Technology Inc.",
    "0050C21F3": "Radionor Communications",
    "0050C242F": "Win4NET",
    "0050C242C": "Trapeze Software Group Inc",
    "0050C2BB7": " GD Mission Systems",
    "0050C24DE": " GD Mission Systems",
    "0050C2FDC": "QUERCUS TECHNOLOGIES, S.L.",
    "0050C213B": "Vaisala Oyj",
    "0050C233D": " GD Mission Systems",
    "0050C2CC4": " GD Mission Systems",
    "0050C22F7": "GILLAM-FEI S.A.",
    "0050C2217": "Linn Products Ltd",
    "0050C260D": "Sicon srl",
    "0050C2330": "Sicon srl",
    "40D85507C": "Agramkow Fluid Systems A/S",
    "0050C2149": "Haag-Streit AG",
    "0050C2D9B": "Intuitive Surgical, Inc",
    "0050C2060": "Private",
    "0050C25CC": "Enseo, Inc.",
    "0050C2BA0": "txtr GmbH",
    "0050C212C": "DELTA TAU DATA SYSTEMS, INC.",
    "0050C21EB": "Data Respons A/S",
    "0050C2378": "Daintree Networks Pty",
    "0050C222A": "Crescendo Networks",
    "0050C256A": "Grupo Epelsa S.L.",
    "40D855027": "Grupo Epelsa S.L.",
    "0050C2BE4": "Grupo Epelsa S.L.",
    "0050C2857": "Grupo Epelsa S.L.",
    "0050C27AA": "Grupo Epelsa S.L.",
    "0050C25F8": "Grupo Epelsa S.L.",
    "40D855071": "TATTILE SRL",
    "0050C2B51": "Aeroflex GmbH",
    "40D855131": "EMAC, Inc.",
    "0050C28E3": "bioM\u00e9rieux Italia S.p.A.",
    "40D8551CC": "NKT Photonics A/S",
    "40D8551E0": "Embedded Technology Corporation",
    "40D8551E3": "Mega Electronics Ltd",
    "40D8551DD": "BaOpt Benelux bv",
    "40D8551CA": "Rigel Engineering",
    "40D8551D1": "Founder Broadband Network Service Co.,Ltd.",
    "40D8551D3": "Kaluga Teletypes Manufacturing Plant",
    "40D855EE6": "Narinet, Inc.",
    "40D8551C5": "Private",
    "40D8551A5": "DemoPad",
    "40D8551AD": "WICHER DIGITAL TECHNIK",
    "40D8551B9": "Beking Industrieele automatisering",
    "40D85517E": "TOKHATEC",
    "40D855188": "Array Corporation",
    "40D855185": "Standard Change Makers",
    "40D855189": "Yoozma Corporation",
    "40D855177": "TRI Engineering co.,ltd.",
    "40D855173": "Contec Steuerungstechnik & Automation GmbH",
    "40D855171": "Sicon srl",
    "40D855158": "Exibea AB",
    "40D855157": "Hitachi Power Solutions Co., Ltd.",
    "40D855167": "Assembly Contracts Ltd",
    "40D855169": "Photop Koncent",
    "40D855160": "Thermo Fisher Sceintific",
    "40D855164": "NFT Automatisierungssysteme GmbH",
    "40D85514C": "PLT",
    "40D855153": "BlinkPipe Ltd",
    "40D855126": "TTI LTD",
    "40D85512D": "Biotage Sweden AB",
    "40D855132": "AeroVision Avionics, Inc",
    "40D855134": "digitech GmbH &amp; Co. KG",
    "40D85510B": "So-Cool Corporation.",
    "40D8550F7": "Comline Elektronik Elektrotechnik GmbH",
    "40D855114": " GD Mission Systems",
    "40D855101": "e.p.g. Elettronica Srl",
    "40D8550E5": "Triton Electronics LTD",
    "40D8550D8": "NEXT! s.c. S.Piela B.Dryja",
    "40D8550DB": "Top Connect OU",
    "40D8550EC": "Sentry 360 Security",
    "40D8550D3": "LECO Corporation",
    "40D8550A4": "Resch Electronic Innovation GmbH",
    "40D8550AA": "Thermal Imaging Radar, LLC",
    "40D8550C5": "M.M. Elektrolab",
    "40D8550C1": "Xepto Computing Inc",
    "40D8550C2": "SC Techswarm SRL",
    "40D85509A": "CoherentPlus Sdn Bhd",
    "40D855084": "Papendorf Software Engineering GmbH",
    "40D85507B": "IPS Technology Limited",
    "40D855050": "ATG UV Technology",
    "40D855058": "Energy Team S.p.A.",
    "40D855051": "CS Instruments Asia",
    "40D85505D": "Leica Biosystems",
    "40D85504C": "Serveron Corporation",
    "40D85505C": "Rosslare Enterprises Limited",
    "40D855064": "HIPODROMO DE AGUA CALIENTE, S.A. DE C.V.",
    "40D855068": "Oki Seatec Co., Ltd.",
    "40D855041": "T.Q.M. Itaca Technology s.r.l.",
    "40D855035": "Mesotech International, Inc.",
    "40D85501D": "Scharco Elektronik GmbH",
    "40D85502E": "Circuitec Ind. Equip. Eletr. Ltda",
    "40D85503A": "Socus networks",
    "40D85503B": "Telcomkorea",
    "40D85504B": "Vital Tech Industria e Comercio Ltda",
    "0050C2FFE": "Sensata Technologies",
    "0050C2FF8": "KST technology",
    "0050C2FF6": "Booyco Electronics",
    "40D855016": "Par-Tech, Inc.",
    "40D855009": "ClearSite Communications Inc.",
    "0050C2FE0": "Azurtest",
    "0050C2FC7": "SERCOM Regeltechniek",
    "0050C2FC4": "Kyowadensi",
    "0050C2FCE": "KOYO ELECTRIC",
    "0050C2FCF": "DINTEK Shanghai Electronic Ltd",
    "0050C2FD3": "Aster Electric Co.,Ltd.",
    "0050C2FD6": "City Computing Ltd",
    "0050C2FA3": "Xemex NV",
    "0050C2FA2": "Power-One ",
    "0050C2F9F": "Nanjing SAC Power Grid Automation Co., Ltd.",
    "0050C2F96": "JLCooper Electronics",
    "0050C2FB9": "Coral Telecom Ltd",
    "0050C2F7F": "Dynamic Design",
    "0050C2F7D": "D-Hike Electroncs Technology Co.,Ltd",
    "0050C2F79": "Tattile srl",
    "0050C2F7B": "MCM Electronics",
    "0050C2F78": "Gets MSS S.A.",
    "0050C2F8C": "UBSTechnology Co., Ltd",
    "0050C2F8B": "comlet Verteilte Systeme GmbH",
    "0050C2F84": "Dynon Instruments",
    "0050C2F73": "DELTACAST.TV",
    "0050C2F86": "Audio Power Labs",
    "0050C2F68": "Newtec A/S",
    "0050C2F5D": "SMARTB TECHNOLOGIES",
    "0050C2F5B": "Saia-Burgess Controls AG",
    "0050C2F4E": "Heinzinger electronic GmbH",
    "0050C2F51": "NDC Infrared Engineering, Inc.",
    "0050C2F49": "Green Instruments A/S",
    "0050C2F43": "Special Systems Engineering Center LLC",
    "0050C2F65": "Telebyte Inc.",
    "0050C2F40": "iBWorld co.,ltd.",
    "0050C2F06": "Micro-Key BV",
    "0050C2F03": "Wren Sound Systems",
    "0050C2F0E": "Micro Technic A/S",
    "0050C2F21": "SEITEC Co. Ltd",
    "0050C2F32": "Net4Things",
    "0050C2F2C": "Terratel Technology s.r.o.",
    "0050C2EFD": "Sanmina",
    "0050C2EF4": "RO.VE.R. Laboratories S.p.A",
    "0050C2EF0": "Homaetrix Ltd",
    "0050C2EE5": "Cytec Zylindertechnik GmbH",
    "0050C2EEB": "fibrisTerre GmbH",
    "0050C2EE9": "QUANTA S.r.l.",
    "0050C2EDF": "Monitor Business Machines",
    "0050C2F01": "Mango DSP, Inc",
    "0050C2F00": "Syscom Instruments",
    "0050C2EF6": "Netline Communication Technologies",
    "0050C2ED3": "ECO MONITORING UTILITY SYSTEMS LTD",
    "0050C2ED4": "TAMAGAWA ELECTRONICS CO.,LTD.",
    "0050C2ED2": "Klangspektrum GmbH",
    "0050C2ED7": "FBT Elettronica spa",
    "0050C2ED6": "Cat AB",
    "0050C2EC9": "Amsterdam Scientific Instruments BV",
    "0050C2EC0": "UgMO Technologies",
    "0050C2EBF": "CIVOLUTION",
    "0050C2EC1": "ANT Group s.r.l",
    "0050C2EDC": "Eyelock Corporation",
    "0050C2EB0": "Pulse Communication Systems Pvt. Ltd.",
    "0050C2EAB": "Warp9 Tech Design, Inc.",
    "0050C2EAA": "BAE Systems",
    "0050C2E96": "PROYECSON S.A.",
    "0050C2E93": "Perceptive Pixel Inc.",
    "0050C2E90": "GS Elektromedizinische Geraete G. Stemple GmbH",
    "0050C2E91": "DSP DESIGN LTD",
    "0050C2EA2": "ThinkRF Corp",
    "0050C2EA0": "Robert Bosch Healthcare Systems, Inc. ",
    "0050C2E8E": " GD Mission Systems",
    "0050C2EB5": "Covidence A/S",
    "0050C2E9D": "nicai-systems",
    "0050C2E99": "UV Networks, Inc.",
    "0050C2E9A": "Solace Systems",
    "0050C2E84": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2E61": "Detech Electronics ApS",
    "0050C2E7A": "Lightel",
    "0050C2E88": "Pivitec, LLC",
    "0050C2E6F": "Leyden Engineering",
    "0050C2E71": "traffic network solutions s.l",
    "0050C2E68": "Kyoritsu Electric Corporation",
    "0050C2E47": "ENIKA.CZ",
    "0050C2E42": "Wings for Media SL",
    "0050C2E5D": "T8 Ltd",
    "0050C2E31": "ENSIS Co., Ltd.",
    "0050C2E1F": "ELVEES",
    "0050C2E1A": "Rosslare Enterprises Limited",
    "0050C2E2E": "DS! Ingenieurbuero",
    "0050C2E17": "Gall Tankdatensysteme GmbH",
    "0050C2E05": "NOCOSIUM",
    "0050C2E04": "Sec.Eng Systems Pty Ltd",
    "0050C2E01": "Tyco Traffic & Transportation",
    "0050C2DFE": "Xitek Design Limited",
    "0050C2DF5": "EtherLight",
    "0050C2DE2": "SEQUTEC INC",
    "0050C2DDC": "Vision  & Control GmbH",
    "0050C2DCE": "Mecsel Oy",
    "0050C2DD2": "Electronic Applications, Inc.",
    "0050C2DB8": "Comsat VertriebsgmbH",
    "0050C2DBD": "Margento R&D",
    "0050C2DAA": "M & PAUL, INC",
    "0050C2D73": "Saia-Burgess Controls AG",
    "0050C2D6E": "BC Illumination, Inc.",
    "0050C2D6C": "ALPHA Corporation",
    "0050C2D83": "Blankom",
    "0050C2D7D": "Voltech Instruments",
    "0050C2D7F": "HMI Technologies",
    "0050C2D7B": "OWITA GmbH",
    "0050C2D9C": "Gamber Johnson LLC",
    "0050C2D91": "CHAUVIN ARNOUX",
    "0050C2D99": "T-Industry, s.r.o.",
    "0050C2D82": "Audio Authority Corp",
    "0050C2D7A": "Transbit Sp. z o.o.",
    "0050C2D75": "Collectric AB",
    "0050C2D76": "Telvent ",
    "0050C2D55": "Sterna Security",
    "0050C2D56": "SELEX Communications Limited",
    "0050C2D3D": "Tellabs Operations Inc.",
    "0050C2D3B": "Gitsn Inc.",
    "0050C2D49": "Smith Meter, Inc",
    "0050C2D45": "Technagon GmbH",
    "0050C2D44": "Saia-Burgess Controls AG",
    "0050C2D5E": "PRIVATECH Inc.",
    "0050C2D66": "Uvax Concepts",
    "0050C2D60": "Nihon Kessho Koogaku Co., Ltd.",
    "0050C2D52": "F+D Feinwerk- und Drucktechnik GmbH",
    "0050C2D5B": "Infinition Inc.",
    "0050C2D6B": "Nemec Automation",
    "0050C2D41": "AREA ENERGY, INC.",
    "0050C2D00": "Bodensee Gravitymeter Geosystem GmbH",
    "0050C2D1B": "TECHKON GmbH",
    "0050C2D39": "Apex NV",
    "0050C2D0E": "Weinert Engineering GmbH",
    "0050C2CFC": "Tritium Pty Ltd",
    "0050C2CE5": "Maretron, LLP",
    "0050C2CEF": "Lupatecnologia e Sistemas Ltda",
    "0050C2CDE": "Axotec Technologies GmbH",
    "0050C2CAD": "Pacific Coast Engineering",
    "0050C2CAC": "PURVIS Systems Incorporated",
    "0050C2CA0": "Kiefer technic GmbH",
    "0050C2CA5": "YOKOWO CO.,LTD",
    "0050C2CB0": "Konsmetal S.A.",
    "0050C2CB5": "Private",
    "0050C2CB7": "inotech GmbH",
    "0050C2CBF": "Megacon AB",
    "0050C2CB2": "Moravian Instruments",
    "0050C2C8F": "Keith & Koep GmbH",
    "0050C2C8D": "Emergency Message Controls LLC",
    "0050C2C80": "Reko-vek",
    "0050C2C7C": "Keysight Technologies Inc.",
    "0050C2C67": "Practical Control Ltd",
    "0050C2C79": "CODESYSTEM Co.,Ltd",
    "0050C2C76": "GridManager A/S",
    "0050C2C72": "Quail ",
    "0050C2C41": "COMPRION GmbH",
    "0050C2C3E": "Sysacom",
    "0050C2C3B": "ELEKTRO-AUTOMATIK GmbH & Co. KG",
    "0050C2C31": "4D Technology Corporation",
    "0050C2C2C": "bach-messtechnik gmbh",
    "0050C2C26": "Austco Marketing & Service (USA) ltd.",
    "0050C2C22": "Audient Ltd",
    "0050C2C39": "SECAD SA",
    "0050C2C32": "Procon Electronics",
    "0050C2C54": "HPC Platform",
    "0050C2C52": "Smartfield, Inc.",
    "0050C2C5D": "SweMet AB",
    "0050C2C5A": "Commotive A/S",
    "0050C2C4F": "Powersense A/S",
    "0050C2C49": "Elektronic Thoma GmbH",
    "0050C2C0B": "ProSourcing GmbH",
    "0050C2C19": "Ibercomp SA",
    "0050C2C1A": "SAM Co., Ltd.",
    "0050C2C1B": "Graesslin GmbH",
    "0050C2BFE": "Ingeteam Paneles S.A.U.",
    "0050C2C16": "OMICRON electronics GmbH",
    "0050C2C13": "Dantec Dynamics A/S",
    "0050C2C11": "ART Antriebs- und Regeltechnik GmbH",
    "0050C2BEF": "SOCIEDAD IBERICA DE CONSTRUCCIONES ELECTRICAS, S.A. (SICE)",
    "0050C2BB9": "Toptech Systems, Inc.",
    "0050C2BC0": "Galaxia Electronics",
    "0050C2BBC": "ImpactSystems",
    "0050C2BBE": "Onlinepizza Norden AB",
    "0050C2BEB": "Peek Traffic Corporation",
    "0050C2BC9": "Nextmove Technologies",
    "0050C2BCE": "TV Portal Co., Ltd.",
    "0050C2BD4": "Global Security Devices",
    "0050C2BB3": "ClimateWell AB (publ)",
    "0050C2BB2": "St Michael Strategies Inc",
    "0050C2B8C": "Keith & Koep GmbH",
    "0050C2B8D": "CEMSI",
    "0050C2B88": "Gigatronik  K\u00f6ln GmbH",
    "0050C2B81": "GHL Advanced Technolgy GmbH & Co. KG",
    "0050C2BA7": "RaumComputer Entwicklungs- und Vertriebs GmbH",
    "0050C2B85": "SilverNet",
    "0050C2B73": "ARKRAY, Inc. Kyoto Laboratory",
    "0050C2B66": "8185",
    "0050C2B7A": "Schnoor Industrieelektronik GmbH & Co. KG",
    "0050C2B4F": "Sencon UK Ltd.",
    "0050C2B5E": "Palgiken Co.,Ltd.",
    "0050C2B70": "Nisshin Electronics co.,ltd.",
    "0050C2B7F": "Enatel",
    "0050C2B6C": "Drinelec",
    "0050C2B18": "Rosslare Enterprises Limited",
    "0050C2B26": "Elko Systems",
    "0050C2B1D": "Telegenix",
    "0050C2B1B": "Integrated  Control Corp.",
    "0050C2B3F": "M-Tronic Design and Technology GmbH",
    "0050C2B34": "Meisol co.,ltd",
    "0050C2B4C": "ElectroCom",
    "0050C2B47": "MCS MICRONIC Computer Systeme GmbH",
    "0050C2B28": "Micromax Pty. Ltd.",
    "0050C2B00": "Saia-Burgess Controls AG",
    "0050C2AF2": "ACD Elektronik GmbH",
    "0050C2B0D": "Japan Electronics System, Inc",
    "0050C2ACB": "U-CARE INC.",
    "0050C2AC5": "E-Motion System, Inc.",
    "0050C2AE0": "AT4 wireless.S.A",
    "0050C2AD1": "Phytec Messtechnik GmbH",
    "0050C2AF1": "Green Goose",
    "0050C2A99": "Haivision Systems Inc",
    "0050C2A9A": "Absolutron. LLC",
    "0050C2A96": "FEP SRL",
    "0050C2AB9": "Showtacle",
    "0050C2AA7": "Endeas Oy",
    "0050C2A93": "SPX Flow Technology",
    "0050C2A94": "Par-Tech, Inc.",
    "0050C2AB2": "ProCom Systems, Inc.",
    "0050C2AA0": "Smith Meter, Inc.",
    "0050C2A50": "i-RED Infrarot Systeme GmbH",
    "0050C2A4F": "Deuta GmbH",
    "0050C2A4E": "Conduant Corporation",
    "0050C2A86": "LIMAB AB",
    "0050C2A4A": "DITRON S.r.l.",
    "0050C2A7B": "Orange Tree Technologies",
    "0050C2A68": "X-Pert Paint Mixing Systems",
    "0050C2A66": "DVTech",
    "0050C2A5F": "Alga Microwave Inc",
    "0050C2A14": "Elbit Systems of America - Tallahassee Operations",
    "0050C2A15": "Industrial Computing Ltd",
    "0050C2A0F": "Visualware Inc",
    "0050C2A20": "Quorum Technologies Ltd",
    "0050C2A1E": "MAMAC Systems, Inc.",
    "0050C2A1F": "Flight Data Systems Pty Ltd",
    "0050C2A1D": "SAMH Engineering Services",
    "0050C2A25": "Saia-Burgess Controls AG",
    "0050C29FE": "SPECTRA EMBEDDED SYSTEMS",
    "0050C2A17": "Winners Satellite Electronics Corp.",
    "0050C2A09": "Innovative American Technology",
    "0050C2A45": "Sofradir-EC",
    "0050C2A39": "Industrial Data Products Ltd",
    "0050C29E0": "DST Swiss AG",
    "0050C2A00": "Technovare Systems",
    "0050C29FC": "ECTEC INC.",
    "0050C29CA": "ESAB-ATAS GmbH",
    "0050C29CE": "Ronan Engineering",
    "0050C29C8": "ethermetrics",
    "0050C29D0": "J. DITTRICH ELEKTRONIC GmbH & Co. KG",
    "0050C29E6": "Kumho Electric, Inc.",
    "0050C29FA": "Teranex A Division of Silicon Optix",
    "0050C29F5": "Commex Technologies",
    "0050C29B8": "Sound Player Systems e.K.",
    "0050C29B3": "Kahler Automation",
    "0050C29B1": "Bon Hora GmbH",
    "0050C299B": "Bettini srl",
    "0050C2997": "Depro \u00c9lectronique",
    "0050C2996": "Commercial Timesharing Inc.",
    "0050C29AA": "TEKO TELECOM SpA",
    "0050C29A6": "Metodo2",
    "0050C299C": "CaTs3 Limited",
    "0050C299F": "Sietron Elektronik",
    "0050C29A3": "Computerwise, Inc.",
    "0050C29AD": "Chronos Technology Ltd.",
    "0050C29A4": "Entwicklung Hard- & Software",
    "0050C2984": "Atel Corporation",
    "0050C2985": "Earnestcom Sdn Bhd",
    "0050C2981": "Novotronik GmbH",
    "0050C297E": "RF Industries",
    "0050C2977": "Saia-Burgess Controls AG",
    "0050C2975": "Pyramid Technical Consultants",
    "0050C2967": "Watthour Engineering Co., Inc.",
    "0050C2969": "Gehrke Kommunikationssysteme GmbH",
    "0050C2955": "Roessmann Engineering",
    "0050C296E": "Peek Traffic Corporation",
    "0050C2933": "Saia-Burgess Controls AG",
    "0050C2947": "IMEXHIGHWAY cvba",
    "0050C292F": "Phytec Messtechnik GmbH",
    "0050C2941": "Rolbit",
    "0050C293E": "RCS Communication Test Systems Ltd.",
    "0050C290F": "INTEGRA Biosciences AG",
    "0050C290E": "Phytec Messtechnik GmbH",
    "0050C2902": "China Railway Signal & Communication Corp.",
    "0050C291D": "Rosendahl Studiotechnik GmbH",
    "0050C2906": "Gidel",
    "0050C28DE": "Coherix, Inc",
    "0050C28EC": "Balfour Beatty Rail GmbH",
    "0050C28ED": "AT-Automation Technology GmbH",
    "0050C28D2": "TTi Ltd",
    "0050C28DC": "Frame Systems Limited",
    "0050C28D7": "Polygon Informatics Ltd.",
    "0050C28D8": "Array Technologies Inc",
    "0050C28E6": "Sandel Avionics, Inc.",
    "0050C28EE": "PCSC",
    "0050C28C4": "Soldig Industria e Comercio de Equipamentos Eletronicos LTDA",
    "0050C28C6": "Gradual Tecnologia Ltda.",
    "0050C28C1": "Heraeus Noblelight GmbH",
    "0050C289E": "Broadcast Electronics",
    "0050C28B4": "Sandar Telecast AS",
    "0050C2896": "Blankom",
    "0050C2894": "Shockfish SA",
    "0050C28CC": "LyconSys GmbH & Co.KG",
    "0050C28B3": "VTQ Videtronik GmbH",
    "0050C28AB": "Nanomotion Ltd.",
    "0050C2867": "Syntronics",
    "0050C2864": "ATG Automatisierungstechnik GERA GmbH",
    "0050C287F": "Optoelettronica Italia S.r.l.",
    "0050C287B": "UAVNavigation S.L.",
    "0050C2860": "Eutron S.p.A.",
    "0050C2890": "BAE Systems H\u00e4gglunds AB",
    "0050C285D": "Ing. Knauseder Mechatronik GmbH",
    "0050C288A": "Continental Electronics Corp.",
    "0050C286F": "b-plus GmbH",
    "0050C286B": "OMB Sistemas Electronicos S.A.",
    "0050C287A": "Spectrum Management, LC",
    "0050C284B": "TASK SISTEMAS DE COMPUTACAO LTDA",
    "0050C2853": "Ettus Research LLC",
    "0050C2844": "Prodigy Electronics Limited",
    "0050C2840": "Residential Control Systems",
    "0050C283B": "O. Bay AG",
    "0050C2850": "K.K. Rocky",
    "0050C2833": "LaserLinc, Inc.",
    "0050C280A": "Phytec Messtechnik GmbH",
    "0050C2804": "SoftSwitching Technologies",
    "0050C27FC": "TIS Dialog LLC",
    "0050C281C": "Telcom",
    "0050C281B": "Brain Tech Co., Ltd",
    "0050C281E": "Channelot Ltd.",
    "0050C2817": "Odin TeleSystems Inc",
    "0050C27F8": "Wise Industria de Telecomunica\u00e7\u00f5es Ldta.",
    "0050C27F7": "MangoDSP",
    "0050C2808": "ITB CompuPhase",
    "0050C2822": "Winner Technology Co, Ltd.",
    "0050C27E5": "Nystrom Engineering",
    "0050C27E3": "Progentech Limited",
    "0050C27C7": "Pyrosequencing AB",
    "0050C27E0": "C&D Technologies, Inc",
    "0050C27D3": "Highrail Systems Limited",
    "0050C27C9": "Bluebell Opticom Limited",
    "0050C27C5": "Venture Research Inc.",
    "0050C27BC": "EIDOS SPA",
    "0050C27BA": "Infodev Electronic Designers Intl.",
    "0050C27B8": "Design 2000 Pty Ltd",
    "0050C27A4": "Calibre UK LTD",
    "0050C27A2": "RaySat Israel LTD",
    "0050C279A": "JMC America, LLC",
    "0050C27C0": "European Industrial Electronics B.V.",
    "0050C27BD": "PROMATE ELECTRONIC CO.LTD",
    "0050C27B3": "Elmec,  Inc.",
    "0050C27B4": "T 1 Engineering",
    "0050C277F": "ACD Elektronik GmbH",
    "0050C277C": "ATEC SRL",
    "0050C2798": "Indefia",
    "0050C2799": "AAVD",
    "0050C2795": "Ameli Spa",
    "0050C2792": "SMARTRO Co.,Ltd.",
    "0050C2771": "ZigBee Alliance",
    "0050C276F": "Control and Robotics Solutions",
    "0050C2770": "Cadex Electronics Inc.",
    "0050C275F": "B. Rexroth the identity company GmbH",
    "0050C275E": "Sky-Skan, Incorporated",
    "0050C2762": "Assembly Contracts Limited",
    "0050C277A": "Smith Meter, Inc",
    "0050C2747": "CDSA Dam Neck",
    "0050C2759": "Sequentric Energy Systems, LLC",
    "0050C2739": "Tattile srl",
    "0050C2726": "eta systemi CKB",
    "0050C2722": "Centric TSolve BV",
    "0050C270B": "B.E.A.R. Solutions (Australasia) Pty, Ltd",
    "0050C2716": "MITROL S.R.L.",
    "0050C2727": "Pelweckyj Videotechnik GmbH",
    "0050C26DD": "Zmicro Systems Inc",
    "0050C26E9": "Bando electronic communication Co.Lltd",
    "0050C26F3": "E3Switch LLC",
    "0050C26E1": "NewOnSys Ltd.",
    "0050C26DF": "QR Sciences Ltd",
    "0050C2701": "Keith & Koep GmbH",
    "0050C26DB": "Gebhardt Ventilatoren GmbH",
    "0050C26BC": "Paraytec Ltd",
    "0050C26D3": "DigiSensory technologies Pty Ltd",
    "0050C26CC": "Widmer Time Recorder Co., Inc.",
    "0050C26C8": "B&S MEDIA Co., LTD.",
    "0050C26C0": "GLOSTER SANTE EUROPE",
    "0050C26D7": "Mavenir System, Inc.",
    "0050C26DA": "Techno Fittings S.r.l.",
    "0050C26C9": "NETAMI",
    "0050C26B1": "Burk Technology",
    "0050C26A8": "Delcan Technologies, Inc",
    "0050C2688": "Elk Products",
    "0050C268E": "SELCO",
    "0050C2699": "Bulletendpoints Enterprises Inc",
    "0050C2686": "ANNAX Anzeigesysteme GmbH",
    "0050C26A0": "GFP Lab S.r.l.",
    "0050C2695": "Purelink Technology, inc.",
    "0050C2675": "Kenton Research Ltd",
    "0050C2661": "P.C.E.",
    "0050C2655": "Physik Instrumente (PI) GmbH&Co.KG",
    "0050C2656": "LDA Audio Video Profesional",
    "0050C2666": "Xworks NZ Limited",
    "0050C2662": "Asia Pacific Card & System Sdn Bhd",
    "0050C263D": "IDERs Inc",
    "0050C266D": "DIGITEK S.p.A.",
    "0050C264D": "Tera Information System Labs",
    "0050C2644": "Phytec Messtechnik GmbH",
    "0050C2671": "Skyline Products, Inc",
    "0050C266F": "Nilan A/S",
    "0050C262B": "First Control Systems AB",
    "0050C2613": "TATTILE SRL",
    "0050C2612": "IHP-GmbH",
    "0050C2641": "NEO Information Systems Co., Ltd.",
    "0050C261D": "Sutus Inc",
    "0050C263B": "Powis Corporation",
    "0050C2635": "Shockfish SA",
    "0050C2619": "Linkbit, Inc.",
    "0050C2630": "Aurora Flight Sciences",
    "0050C25F5": "Genesis inc",
    "0050C2600": "Protec Fire Detection plc",
    "0050C260E": "Automation and Control Technology, Inc.",
    "0050C260B": "Shanghai QianJin Electronic Equipment Co. Ltd.",
    "0050C2605": "Swistec GmbH",
    "0050C2606": "Shenzhen Huazhong Technology Inc",
    "0050C25F6": "CAMBRIDGE CONSULTANTS LTD",
    "0050C25E5": "Stresstech OY",
    "0050C25EB": "Garper Telecomunicaciones, S.L.",
    "0050C25BA": "SAIA Burgess Controls AG",
    "0050C25BB": "UNIC TECHNOLOGIES INC",
    "0050C25BC": "Guangzhou Hui Si Information Technologies Inc.",
    "0050C25B7": "AVerMedia Technologies, Inc.",
    "0050C25DA": "TONNA ELECTRONIQUE",
    "0050C25D6": "BioAccess Tecnologia em Biometria Ltda.",
    "0050C25D2": "DA-Design Oy",
    "0050C25AE": "CPS EUROPE B.V.",
    "0050C25AC": "Kinemetrics, Inc.",
    "0050C2587": "Dynalco",
    "0050C2582": "AllSun A/S",
    "0050C258D": "ZAO",
    "0050C258A": "Dixell S.p.a.",
    "0050C258B": "Innovative Dynamics GmbH",
    "0050C2590": "EM Motorsport Ltd",
    "0050C2592": "PaloDEx Group Oy",
    "0050C259E": "Legerity",
    "0050C257C": "\u00e9olane",
    "0050C256F": "GMA, LLC",
    "0050C256E": "LAB-EL ELEKTRONIKA LABORATORYJNA S.J.",
    "0050C2567": "Tess GmbH",
    "0050C2561": "Seitec Elektronik GmbH",
    "0050C255E": "HANZAS ELEKTRONIKA, SIA",
    "0050C2556": "Freiburger BlickZentrum",
    "0050C254E": "Convergent Design",
    "0050C2543": "DIGI SESN AG",
    "0050C2555": "Control Alternative Solutions, Inc.",
    "0050C250D": "Clearsonics Pty. Ltd.",
    "0050C250E": "Fibresavers Corporation",
    "0050C2508": "PUTERCOM ENTERPRISE CO., LTD.",
    "0050C250A": "Monitor Business Machines Ltd",
    "0050C2504": "Aphex Systems Ltd.",
    "0050C2505": "Computerwise, Inc.",
    "0050C2528": "Tattile srl",
    "0050C2521": "ARIS TECHNOLOGIES",
    "0050C2519": "DBMCORP, Inc.",
    "0050C251C": "TOA Systems",
    "0050C24E3": "Romteck Pty Ltd",
    "0050C24E7": "Computerized Elevator Contol",
    "0050C24FD": "Network Automation mxc AB",
    "0050C24DD": "Truteq Wireless (PTY) Ltd.",
    "0050C24DC": "Ace Electronics, Inc.",
    "0050C24EE": "Beijing Corelogic Communication Co., Ltd.",
    "0050C24EC": "Thales Defence and Security Systems GmbH",
    "0050C24E9": "Seachange international",
    "0050C24E6": "Photonic Bridges  Inc.",
    "0050C24FF": "Dakty GmbH",
    "0050C24C5": "eXray Broadband Inc.",
    "0050C24BF": "Westinghouse Rail Systems Ltd",
    "0050C24D4": "Herholdt Controls srl",
    "0050C24D3": "ELPROC sp. z o.o.",
    "0050C24CC": "ImpediMed Limited",
    "0050C24C0": "Bio-logic Systems Corp",
    "0050C24BB": "Protonic Holland",
    "0050C24B0": "Esmart Distribution Pte Ltd",
    "0050C24A1": "nVent, Schroff GmbH",
    "0050C248D": "Metron Sp. z o.o.",
    "0050C2489": "EREE Electronique",
    "0050C2491": "Fr. Sauter AG",
    "0050C247E": "Energie Umwelt Systemtechnik GmbH",
    "0050C2475": "ISEPOS GmbH",
    "0050C2458": "HRZ data GmbH",
    "0050C2456": "DRDC Valcartier",
    "0050C236F": "XIMEA s.r.o.",
    "0050C246C": "CAMCO GmbH",
    "0050C245D": "Digital Engineering, Inc.",
    "0050C2465": "PDTS GmbH",
    "0050C2464": "XYTAC system technologies",
    "0050C243D": "Ann Arbor Sensor Systems LLC",
    "0050C243C": "Ducommun LaBarge Technologies, Inc",
    "0050C243A": "ProDesign GmbH",
    "0050C2440": "Advanced Modular Computers Ltd.",
    "0050C2454": "Brivo Systems, LLC",
    "0050C244D": "Electro-Matic Products, Inc.",
    "0050C2432": "Topway Industries Ltd.",
    "0050C2430": "Arcom Digital",
    "0050C244E": "QQ Technology,Inc",
    "0050C2413": "Goodrich",
    "0050C242B": "VLSIP TECHNOLOGIES, INC.",
    "0050C2427": "Scheidt & Bachmann GmbH",
    "0050C2426": "STOM System",
    "0050C2429": "Matthews Australasia",
    "0050C2423": "Power-One Inc.",
    "0050C2417": "QT systems ab",
    "0050C2406": "CoreStreet, Ltd",
    "0050C23FF": "Cast Iron Systems",
    "0050C240C": "Applied Materials UK Ltd",
    "0050C23BF": "Audio Processing Technology Ltd",
    "0050C23C7": "Salent Technologies Ltd",
    "0050C23C5": "Silicon Optix Canada Inc.",
    "0050C23FC": "Weinberger Deutschland GmbH",
    "0050C23F4": "MC TECHNOLOGY GmbH",
    "0050C23F1": "Salland Electronics Holding BV",
    "0050C23F0": "megatec electronic GmbH",
    "0050C23C8": "Wheels of Zeus Inc.",
    "0050C23CF": "Technovare Systems, Inc.",
    "0050C23A8": "Engim, Inc.",
    "0050C23A4": "Silvertree Engineering Ltd",
    "0050C2383": "ICS Electronics",
    "0050C23AB": "taskit Rechnertechnik GmbH",
    "0050C23B5": "NEC TOKIN Corporation",
    "0050C2390": "TC Communications",
    "0050C2395": "vidisys gmbh",
    "0050C23A1": "Samsoft",
    "0050C238C": "EPSILON SRL",
    "0050C2375": "TIR Systems Ltd.",
    "0050C237F": "Foresearch",
    "0050C237C": "MODIA SYSTEMS Co., Ltd",
    "0050C2388": "IEE Inc",
    "0050C2368": "ASPEL S.A.",
    "0050C2325": "Federal Aviation Administration",
    "0050C2327": "Dornier GmbH",
    "0050C2328": "I.C.S. Electronics Limited",
    "0050C2331": "Broadcast Sports Inc",
    "0050C234C": "Chuo Electric Works Co., LTD.",
    "0050C2349": "SSI Schaefer Peem",
    "0050C2348": "KoolSpan, Inc.",
    "0050C233B": "MultimediaLED",
    "0050C232D": "Consens Zeiterfassung GMBH",
    "0050C2343": "ABB Xiamen Switchgear Co. Ltd.",
    "0050C2317": "Cosine Systems, Inc.",
    "0050C2316": "Dataline AB",
    "0050C231E": "C3-ilex, LLC",
    "0050C22FE": "Saab AB",
    "0050C230B": "VX Technologies Inc.",
    "0050C22D0": "Worth Data, Inc.",
    "0050C22E3": "MG Industrieelektronik GmbH",
    "0050C22DE": "Research Applications",
    "0050C22EB": "Lingg & Janke OHG",
    "0050C22E6": "DALSA",
    "0050C22E7": "SafeView, Inc.",
    "0050C22CD": "DataWind Research",
    "0050C22B8": "Admiral Secure Products, Ltd.",
    "0050C22A6": "Brannstroms Elektronik AB",
    "0050C22B1": "Private",
    "0050C22AB": "AUM Infotech Private Limited",
    "0050C22A9": "Dr. Staiger, Mohilo + Co GmbH",
    "0050C2284": "Micronet Ltd.",
    "0050C227F": "Air Broadband Communications, Inc.",
    "0050C228C": "Futaba Corporation",
    "0050C2289": "Rototype S.p.A.",
    "0050C228D": "AXODE SA",
    "0050C2299": "CAD-UL GmbH",
    "0050C2294": "EPSa GmbH",
    "0050C2276": "Tieline Research Pty Ltd",
    "0050C2253": "DSM-Messtechnik GmbH",
    "0050C224F": "Macronet s.r.l.",
    "0050C2246": "Hardmeier",
    "0050C2256": "Information Technology Corp.",
    "0050C2229": "eko systems inc.",
    "0050C2224": "PANNOCOM Ltd.",
    "0050C2232": "SIMET",
    "0050C2233": "EdenTree Technologies, Inc.",
    "0050C2235": "POLIMAR ELEKTRONIK LTD.",
    "0050C21FB": "Willowglen Systems Inc.",
    "0050C2201": "OlympusNDT",
    "0050C2213": "SysAware S.A.R.L.",
    "0050C2208": "nNovia, Inc.",
    "0050C21F2": "Tattile srl",
    "0050C21ED": "EMKA-electronic AG",
    "0050C21EE": "Perto Perif\u00e9ricos de Automa\u00e7\u00e3o S.A.",
    "0050C21E6": "United Tri-Tech Corporation",
    "0050C21E7": "Smith Meter, Inc.",
    "0050C21D7": "Pleora Technologies Inc.",
    "0050C21E2": "DIGITRONIC Automationsanlagen GmbH",
    "0050C21D4": "Phase IV Engineering Inc.",
    "0050C21F7": "ARC'Cr\u00e9ations",
    "0050C21E1": "Automaatiotekniikka Seppo Saari Oy",
    "0050C21E4": "Soronti, Inc.",
    "0050C21BD": "AIOI Systems Co., Ltd.",
    "0050C21B5": "Thrane & Thrane A/S",
    "0050C21C8": "Euphony technology CO., LTD.",
    "0050C21C5": "Flander Oy",
    "0050C21B8": "Electronic Systems Development",
    "0050C21C6": "Remco Italia Spa",
    "0050C21B1": "Axes Technologies",
    "0050C219D": "ELECTREX S.R.L",
    "0050C21A4": "Protech Optronics Co. Ltd.",
    "0050C218B": "Teradyne Inc.",
    "0050C218C": "Technodrive srl",
    "0050C2199": "Survalent Technology Corporation",
    "0050C2174": "N&P Technologies",
    "0050C217A": "WOLF Advanced Technology.",
    "0050C217F": "PDQ Manufacturing",
    "0050C216F": "Dickson Technologies",
    "0050C2009": "Datakinetics Ltd.",
    "0050C216A": "Time Domain",
    "0050C2002": "Integrated Automation Solutions",
    "0050C215F": "Pulsar GmbH",
    "0050C215A": "Plextek Limited",
    "0050C2095": "SEATECH",
    "0050C214D": "wellink, Ltd.",
    "0050C2154": "Jostra AB",
    "0050C213E": "AVerMedia Systems, Inc.",
    "0050C2133": "ChipWrights, Inc.",
    "0050C2132": "Procon Electronics",
    "0050C213C": "NBG Industrial Automation B.V.",
    "0050C213A": "Tex Computer SRL",
    "0050C2118": "Microbit 2.0 AB",
    "0050C211A": "Teamaxess Ticketing GmbH",
    "0050C212A": "Symbolic Sound Corp.",
    "0050C2113": "Ace Electronics, Inc.",
    "0050C210C": "Photonic Bridges, Inc.",
    "0050C210E": "Unipower AB",
    "0050C2109": "ITK Dr. Kassen GmbH",
    "0050C20F7": "Foss NIRSystems, Inc.",
    "0050C20E5": "Clearwater Networks",
    "0050C20F3": "Young Computer Co., Ltd.",
    "0050C20EA": "iReady Corporation",
    "0050C20E8": "Audio Design Associates, Inc.",
    "0050C20D1": "Renaissance Networking, Inc.",
    "0050C20D3": "Lake Technology, Ltd.",
    "0050C20D0": "Telefrang AB",
    "0050C20CE": "RFL Electronics, Inc.",
    "0050C20C9": "DSS Networks, Inc.",
    "0050C20C8": "The Trane Company",
    "0050C20C7": "TransComm Technology System, Inc.",
    "0050C20BB": "MAZet GmbH",
    "0050C20BD": "Tattile",
    "0050C20C0": "Imigix Ltd.",
    "0050C20AE": "Zarak Systems Corp.",
    "0050C20B1": "Beeline Networks, Inc.",
    "0050C20C1": "Casabyte",
    "0050C2094": "Analytical Spectral Devices, Inc.",
    "0050C2090": "Invensys Controls Network Systems",
    "0050C207D": "Caspian Networks",
    "0050C206D": "Advanced Signal Corp.",
    "0050C2061": "Simple Network Magic Corporation",
    "0050C203B": "VNR Electronique SA",
    "0050C204B": "Tecstar Demo Systems Division",
    "0050C2040": "MiSPO Co., Ltd.",
    "0050C203C": "BrainBoxes Ltd",
    "0050C203E": "MSU UK Ltd",
    "0050C2043": "Curtis, Inc.",
    "0050C2035": "Alliant Techsystems, Inc.",
    "0050C2036": "Arcturus Networks Inc.",
    "0050C2032": "MotionIO",
    "0050C201A": "Skylake Talix",
    "0050C201B": "Cross Products Ltd.",
    "0050C201F": "KBS Industrieelektronik GmbH",
    "0050C2026": "Abatis Systems Corp.",
    "0050C2012": "Floware System Solutions Ltd.",
    "0050C2005": "GD California, Inc.",
    "0050C2D74": "Computech International",
    "0050C255B": "MATRIX COMSEC PRIVATE LIMITED",
    "0050C2DAF": "eumig industrie-TV GmbH.",
    "0050C230E": "Leviton Manufacturing Co., Inc",
    "0050C2816": "Q-Free America, Inc.",
    "0050C2212": "4Links Limited",
    "0050C27F9": "Karl DUNGS GmbH & Co. KG",
    "0050C23DC": "3D perception AS",
    "0050C2BE5": "RF Code",
    "0050C2F1C": " GD Mission Systems",
    "0050C2FA8": "Yash SiQure Technologies India Pvt. Ltd.",
    "0050C2D30": "ACTIV Financial Systems, Inc.",
    "0050C24F6": "RealD, Inc.",
    "0050C2659": "Dorsett Technologies Inc",
    "40D8551E1": "AD QUALITE",
    "0050C2964": "IQ Automation GmbH",
    "40D85510C": "Contrans TI sp. z o.o.",
    "0050C2AA3": "Peek Traffic Corporation",
    "0050C29F8": "Austco Marketing & Service (USA) ltd.",
    "0050C2CA3": "CT Company",
    "40D85515F": "CT Company",
    "0050C2664": "Westel Wireless Systems",
    "0050C2A7E": "Densitron Technologies Ltd",
    "0050C2FD7": "DEUTA-WERKE GmbH",
    "0050C2FBA": "Elbit Systems of America",
    "0050C21DD": "peiker acustic GmbH",
    "0050C2B54": "APG Cash Drawer, LLC",
    "0050C274E": "TRONICO",
    "0050C2764": "ARGUS-SPECTRUM",
    "40D85517A": "ARGUS-SPECTRUM",
    "0050C2583": "Junger Audio-Studiotechnik GmbH",
    "40D85504E": "Honeywell International",
    "40D855080": "Honeywell",
    "40D85513F": "Zhejiang Wellsun Electric Meter Co.,Ltd",
    "0050C2A2E": "YUYAMA MFG Co.,Ltd",
    "0050C2717": "MB connect line GmbH Fernwartungssysteme",
    "0050C2FE9": "MB connect line GmbH Fernwartungssysteme",
    "0050C2D09": "Guardtec,Inc",
    "0050C21D8": "Guardian Controls International Ltd",
    "0050C231D": "Imarda New Zealand Limited",
    "0050C2FE7": "Erhardt+Leimer GmbH",
    "0050C2E24": "DiTEST Fahrzeugdiagnose GmbH",
    "0050C2A80": "ard sa",
    "0050C2FD1": "Enyx SA",
    "0050C29B2": "Mesa Labs, Inc.",
    "0050C2A60": "Arrow Central Europe GmbH - Division Spoerle",
    "40D855024": "Electrical Geodesics Incorporated",
    "0050C2F9D": "JSC \"Kaluga Teletypes Manufacturing Plant\"",
    "0050C249D": "Critical Link LLC",
    "0050C2760": "AR'S CO., LTD.",
    "0050C2485": "Phytec Messtechnik GmbH",
    "0050C2459": "Phytec Messtechnik GmbH",
    "0050C21C3": "TT electronics plc",
    "0050C2138": "Delphin Technology AG",
    "0050C2601": "MedAvant Healthcare",
    "0050C2F33": "Applied Micro Electronics AME bv",
    "40D855093": "Sentry 360 Security",
    "0050C2B30": "Applied Micro Electronics AME bv",
    "0050C221C": "Fracarro srl",
    "40D85515A": "DORLET SAU",
    "40D85506F": "DORLET SAU",
    "0050C29E7": "DORLET SAU",
    "40D855125": "Scandyna A/S",
    "0050C26B2": "Edgeware AB",
    "40D8550F8": "Better Place",
    "0050C2168": "ExtremeSpeed Inc.",
    "0050C2A78": "Apantac LLC",
    "40D855089": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "0050C2888": "IAdea Corporation",
    "0050C20B9": "Helmut Mauell GmbH Werk Weida",
    "0050C2F2D": "Robert Bosch Healthcare Systems, Inc. ",
    "0050C2667": "Vocality International Ltd",
    "40D85514B": "Vocality International Ltd",
    "0050C25FE": "NOVACOMM LTDA",
    "0050C226E": "ZP Engineering SEL",
    "0050C28C8": "Pumatronix Equipamentos Eletronicos Ltda.",
    "0050C29F9": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2D84": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2DE1": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C25E2": "PYRAMID Computer GmbH",
    "40D85506C": "Rohde&Schwarz Topex SA",
    "40D85518A": "Aplex Technology Inc.",
    "40D855110": "Aplex Technology Inc.",
    "0050C2B49": "Aplex Technology Inc.",
    "0050C28E2": "Wireless Cables Inc.",
    "40D855102": "Power Electronics Espana, S.L.",
    "0050C2ED5": "RFL Electronics, Inc.",
    "0050C26FB": "WaveIP",
    "0050C21CC": "WaveIP",
    "0050C29A9": " GD Mission Systems",
    "0050C221B": " GD Mission Systems",
    "0050C2A56": "ReaMetrix, Inc.",
    "0050C214C": "Optibase Ltd.",
    "0050C293D": "Lighting Science Group",
    "0050C20B0": "LMI Technologies",
    "0050C2F97": "Sicon srl",
    "0050C20A2": "J\u00e4ger Computergesteuerte Me\u00dftechnik GmbH.",
    "0050C2B76": "ITF Fr\u00f6schl GmbH",
    "0050C29CF": "Intuitive Surgical, Inc",
    "0050C22ED": "4RF Communications Ltd",
    "0050C24C2": "Elbit Systems Ltd.",
    "0050C22F4": "Efficient Channel Coding",
    "0050C226C": "Crystal Vision Ltd",
    "0050C2CBA": "DELTA TAU DATA SYSTEMS, INC.",
    "0050C2BE7": "Genetec Inc.",
    "0050C21BC": "DINEC International",
    "0050C2410": "Grossenbacher Systeme AG",
    "40D855133": "TATTILE SRL",
    "0050C2501": "IBEX UK Limited",
    "0050C2447": "Grupo Epelsa S.L.",
    "40D855111": "Grupo Epelsa S.L.",
    "0050C22E0": "Baxter International Inc",
    "0050C20E0": "EMAC, Inc.",
    "40D85509D": "EMAC, Inc.",
    "40D85550D": "Shenzhen MaiWei Cable TV Equipment CO.,LTD.",
    "40D8551D9": "Commercial Wireless Systems International LLC.",
    "40D8551DA": "Energy Technology and Control Ltd.",
    "40D8551D6": "EMS Computers Pty Ltd",
    "40D8551D7": "Wheatstone Corporation",
    "40D8551D4": "Prisma Engineering srl",
    "40D8551C0": "NPB Automation AB",
    "40D8551CF": "Omnik New Energy Co., Ltd",
    "40D8551BA": "Creative Lighting And Sound Systems Pty Ltd",
    "40D8551C6": "Device Solutions Ltd",
    "40D8551DE": "Vidisys GmbH",
    "40D8551B3": "BETTINI SRL",
    "40D8551B2": "AGE A. Gilg Elektronik",
    "40D8551B1": "Logos 01 S.r.l.",
    "40D8551B0": "Shin-ei Electronic Measuring Co.,Ltd.",
    "40D8551AF": "Vigitron Inc.",
    "40D8551A6": "RB-LINK Wireless",
    "40D8551AB": "Rosslare Enterprises Limited",
    "40D855198": "devboards GmbH",
    "40D85516E": "Secuinfo Co.Ltd",
    "40D85516F": "BrightLeaf Power",
    "40D85518B": "Diagnosys Test Systems Ltd",
    "40D855191": "Soukai Electric",
    "40D855178": "REDER Domotic GmbH",
    "40D855161": "Solidscape Inc",
    "40D855151": "Progress Rail Services, Inspection and Information Systems",
    "40D85514D": "SOMFY SAS",
    "40D855166": "Anhui Jiante Network Technology Co., Ltd.",
    "40D85515E": "Prodco International Inc.",
    "40D855145": "Weber Marking Systems GmbH",
    "40D85514A": " GD Mission Systems",
    "40D855146": "Pleiger Elektronik GmbH  and Co. KG",
    "40D855136": "Devriecom B.V.",
    "40D855144": "Venco",
    "40D855121": "shanghai Anjian Information technology co. , ltd.",
    "40D855124": "Debug s.r.l.",
    "40D855123": "ZAO NPC  Kompjuternie Technologii",
    "40D85511F": "KOMPAN Pawel Sokolowski",
    "40D85510F": "CAVALRY STORAGE INC",
    "40D855113": "Testbook Ltd",
    "40D855100": "TASK SISTEMAS DE COMPUTACAO S.A.",
    "40D8550DF": "Xadi Inc",
    "40D8550FA": "Marmitek BV",
    "40D8550F9": "Invisua Lighting BV",
    "40D8550F6": "Private",
    "40D8550DD": "Embed Limited",
    "40D8550E7": "LIGHTSTAR",
    "40D8550B6": "Telvent",
    "40D8550B8": "Ferlin Trading BV",
    "40D8550BB": "Whiptail",
    "40D8550BD": "iCOGNIZE GmbH",
    "40D8550BE": "Manufacturing System Insights Inc",
    "40D8550CB": "ReliOn Inc",
    "40D8550CF": "Clark-MXR, Inc.",
    "40D8550D1": "Cantada Inc",
    "40D8550D0": "Icraft Oy",
    "40D855092": "Wasserbauer GmbH",
    "40D8550A5": "WooshCom Corporation",
    "40D85509C": "Keyware Solutions Inc.",
    "40D855099": "idcell co.ltd",
    "40D8550A0": "Quantronix, Inc.",
    "40D855090": "Axxess Identification Ltd",
    "40D8550AC": "Fraunhofer HHI",
    "40D85507D": "Wuxi SiNeng New Energy Co., Ltd.",
    "40D855063": "Protonic Holland",
    "40D85505A": "Ultra Electronics Flightline Systems",
    "40D855061": "Cominfo, Inc.",
    "40D855070": "JSC Electrical Equipment Factory",
    "40D855085": "Peek Traffic Corporation",
    "40D855083": "DELOPT",
    "40D855059": "COLONIAL ASSEMBLY and  DESIGN",
    "40D855055": "Helmholtz Zentrum Dresden Rossendorf e.V.",
    "40D855056": "GROUP 57",
    "40D855052": "DAN ELECTRONICS SYSTEM (P) LIMITED",
    "40D855034": "Dacom West GmbH",
    "40D855031": "Dommel GmbH",
    "40D855049": "Thermo Fisher Scientific",
    "40D855045": "Genadsystem",
    "40D85504F": "Haein S&S Co., Ltd",
    "40D85504D": "MACHINEPERFORMANCE ApS",
    "40D855039": "CI Systems Ltd",
    "40D855004": "CR Magnetics, Inc.",
    "40D85501B": "Audio Enhancement",
    "40D855018": "STANEO SAS",
    "40D855026": "Symetrics Industries",
    "40D855021": "SMT D.O.O.",
    "40D855011": "Flexim Security Oy",
    "40D85500F": "DIGITAL DYNAMICS, INC.",
    "40D855008": "Kaori Industria Eletronica Ltda",
    "40D855002": "Hangzhou Chenxiao Technologies Co. Ltd.",
    "40D855007": "Digital Audio SA",
    "0050C2FFB": "SEFRAM",
    "0050C2FF3": "CONTROL SYSTEMS Srl",
    "0050C2FF0": " GD Mission Systems",
    "0050C2FEF": "Task Sistemas de Computacao",
    "0050C2FD2": "Autonomic Controls. Inc",
    "0050C2FD0": "Simple Solutions",
    "0050C2FE8": "Mango DSP, Inc.",
    "0050C2FE6": "Exibea AB",
    "0050C2FA0": "Amplus Communication Pte Ltd",
    "0050C2FB8": "TECHNO CO.,LTD.",
    "0050C2FB5": "Assembly Contracts Limited",
    "0050C2FA9": "Hijet Print d.o.o.",
    "0050C2F75": "PumpWell Solutions Ltd.",
    "0050C2F94": "Digital Barriers",
    "0050C2F80": "SYS TEC electronic GmbH",
    "0050C2F88": "RTC Manufacturing Inc.",
    "0050C2F7A": "C3 LLC",
    "0050C2F66": "GWT LLC",
    "0050C2F54": "Hella Gutmann Solutions GmbH",
    "0050C2F4D": "KNOWHOW INFOCOM INC.",
    "0050C2F47": "cadac,inc.",
    "0050C2F5F": "BORYEU TECHNOLOGY CO.,LTD",
    "0050C2F4F": "BAP Precision Ltd.",
    "0050C2F6C": "Pro Design Electronic GmbH",
    "0050C2F30": "Miris AB",
    "0050C2F2F": "Arcos Technologies LTD",
    "0050C2F11": "Organis GmbH",
    "0050C2F3F": "DENSEI COMMUNICATION Inc.",
    "0050C2F28": "Vertex Antennentechnik GmbH",
    "0050C2F22": "Harland Simon plc",
    "0050C2EF2": "Specialty Microwave Corp",
    "0050C2EEF": "IDTRONIC GmbH",
    "0050C2EF1": "Saia-Burgess Controls AG",
    "0050C2EED": "Future Design Controls, Inc",
    "0050C2EE3": "Tecnint HTE SRL",
    "0050C2EE1": "Procon Electronics",
    "0050C2EFC": "Private",
    "0050C2EFB": "Norbit ODM AS",
    "0050C2EFE": "PLR Information Systems Ltd.",
    "0050C2EB4": "Wishtek Technology, Inc.",
    "0050C2EC4": "Logical Electromechanical Sys Inc.",
    "0050C2EC2": "Ixonos Plc",
    "0050C2EC8": "IBERNEX INGENIERIA, S.L.",
    "0050C2EDD": "EBNEURO SPA",
    "0050C2EAC": "Alias ip",
    "0050C2E8A": "Macronet s.r.l.",
    "0050C2E89": "PROTEQSEN",
    "0050C2E95": "Dlite Comercio, Importadora e Servi\u00e7os de Automa\u00e7\u00e3o Ltda",
    "0050C2EA5": "Aerodata AG",
    "0050C2E62": "SAE IT-systems GmbH & Co. KG",
    "0050C2E5F": "Pantec Engineering AG",
    "0050C2E78": "TASK SISTEMAS DE COMPUTACAO LTDA",
    "0050C2E74": "Will corp.",
    "0050C2E7F": "LS Control A/S",
    "0050C2E80": "Saia-Burgess Controls AG",
    "0050C2E5B": "CAIPO Automazione Industriale s.r.l.",
    "0050C2E58": "Agri-hitech LLC",
    "0050C2E59": "Saia-Burgess Controls AG",
    "0050C2E86": "Multisuns Corporation",
    "0050C2E87": "Lamson Safes & Security",
    "0050C2E81": "Adaptive Technologies, Inc.",
    "0050C2E82": "Xplore Technologies Corp",
    "0050C2E37": "FUJI DATA SYSTEM Co., Ltd",
    "0050C2E4A": "GHL Systems Bhd",
    "0050C2E49": "CTF TECHNOLOGIES DO BRASIL LTDA",
    "0050C2E33": "Morita Technical Center Company",
    "0050C2E2F": "Beam Ltd",
    "0050C2E16": "Jetstream Ltd.",
    "0050C2E2C": "EN ElectronicNetwork Hamburg GmbH",
    "0050C2E1C": "Saia-Burgess Controls AG",
    "0050C2E02": "Cleverscope",
    "0050C2E0B": "Seartech",
    "0050C2E11": "RadioMobile Inc",
    "0050C2E12": "Kago Electronics BV",
    "0050C2E0C": "YOUHO ELECTRIC IND.,LTD.",
    "0050C2E08": "KST Technology",
    "0050C2DFF": "TANTAL ELECTRONICA, SL",
    "0050C2DF3": "INSEVIS GmbH",
    "0050C2DF7": "Combilent",
    "0050C2DDA": "rbz robot design s.l.",
    "0050C2DC7": "AGT Holdings Limited ",
    "0050C2DE5": "Neets",
    "0050C2DE4": "EGS Technologies Ltd",
    "0050C2DDF": "Device GmbH",
    "0050C2DE9": "Dacom West GmbH",
    "0050C2DD7": "Tornado Modular Systems",
    "0050C2DD1": "Brankamp GmbH",
    "0050C2DC3": "ZED Ziegler Electronic Devices GmbH",
    "0050C2DB9": "Peek Traffic Corporation",
    "0050C2DBC": "Nantes Systems Private Limited",
    "0050C2D97": "ERS electronic GmbH",
    "0050C2D6D": "Pro-Digital Industria Eletronica",
    "0050C2D6A": "A&T Corporation, Electrics Group , LAS R&D Unit, ",
    "0050C2D69": "GHL Systems Bhd",
    "0050C2D65": "TX Technology Corp",
    "0050C2D61": "system2 GmbH",
    "0050C2D81": "Tattile srl",
    "0050C2D85": "VITEC",
    "0050C2D3A": "WellSense Technologies",
    "0050C2D37": "LJT & Associates, Inc.",
    "0050C2D33": "Maddalena S.p.A",
    "0050C2D36": "Enatel Limited",
    "0050C2D32": "RealTime Systems Ltd",
    "0050C2D2F": "Key Systems, Inc.",
    "0050C2D40": "demmel products",
    "0050C2D4A": "ATH system",
    "0050C2D50": "Solbrig Electronics, Inc.",
    "0050C2D4E": "Keith & Koep GmbH",
    "0050C2D3C": "ASSYSTEM France",
    "0050C2D18": "Glyn GmbH & Co.KG",
    "0050C2D17": "CUE, a.s.",
    "0050C2D25": "VAF Instruments BV",
    "0050C2D22": "eMDee Technology, Inc.",
    "0050C2D0C": "JVL Industri Elektronik",
    "0050C2D0D": "DECA Card Engineering GmbH",
    "0050C2D07": "IAF GmbH",
    "0050C2D2E": "RS Gesellschaft fur Informationstechnik mbH & Co KG",
    "0050C2D13": "GUNMA ELECTRONICS CO LTD",
    "0050C2D04": "Tehama Wireless",
    "0050C2D1A": "GILLAM-FEI S.A.",
    "0050C2CD8": "IT-IS International Ltd.",
    "0050C2CD9": "NDC Infrared Engineering, Inc.",
    "0050C2CD7": "Saia-Burgess Controls AG",
    "0050C2CE1": "Satellink Inc.",
    "0050C2CD2": "SIM2 Multimedia S.p.A.",
    "0050C2CCD": "FUJI DATA SYSTEM Co.,Ltd.",
    "0050C2CCF": "TASK SISTEMAS DE COMPUTACAO LTDA",
    "0050C2CD0": "MME Mueller Mikroelektronik",
    "0050C2CEB": "Toyon Research Corporation",
    "0050C2CEA": "Keith & Koep GmbH",
    "0050C2CF8": "beks Kommunikacios Technika kft",
    "0050C2CA7": "Thermo Fisher Scientific",
    "0050C2CCB": "CaptiveAire Systems Inc.",
    "0050C2CC8": "Private",
    "0050C2CC5": "Tecnovum AG",
    "0050C2CC3": "viscount systems inc.",
    "0050C2CC1": "Level 3 Communications",
    "0050C2CBB": "Coptonix GmbH",
    "0050C2CBD": "Hi Tech Electronics Ltd",
    "0050C2CB9": "Micro Technic A/S",
    "0050C2CA6": "Vidisys GmbH",
    "0050C2CAE": "Campbell Scientific Canada Corp.",
    "0050C2C8A": "Automated Media Services, Inc.",
    "0050C2C98": "Criticare Systems, Inc",
    "0050C2C78": "9Solutions Oy",
    "0050C2C96": "CyberCraft",
    "0050C2C92": "EMAC, Inc.",
    "0050C2C6E": "TBS Holding AG",
    "0050C2C69": "REBO CO.,LTD.",
    "0050C2C9D": "Radius Sweden AB",
    "0050C2C86": "Bruckner & Jarosch Ingenieurgesellschaft mbH",
    "0050C2C82": "Kyosha Industries",
    "0050C2C6B": "SiGarden Sp z o.o.",
    "0050C2C4D": "Industrial Automation Systems",
    "0050C2C5C": "WAVECOM ELEKTRONIK AG",
    "0050C2C57": "High Speed Design, Inc.",
    "0050C2C43": "Cammegh Limited ",
    "0050C2C3D": "PLA ELECTRO APPLIANCES PVT. LTD.",
    "0050C2C62": "Zeus Systems Private Limited",
    "0050C2C45": "Galvamat & Unican Technologies SA",
    "0050C2C40": "BAE Systems Bofors AB",
    "0050C2C59": "SKD System AB",
    "0050C2C56": "Spirent Communications",
    "0050C2C30": "Wagner Group GmbH",
    "0050C2C0E": "AVItronic GmbH",
    "0050C2C09": "Globe Wireless",
    "0050C2C04": "SoGEME",
    "0050C2BFF": "I.S.A. S.r.l.",
    "0050C2C17": "Axis-Shield PoC AS",
    "0050C2C14": "Spectronix Corporation",
    "0050C2C2A": "RealTime Systems Ltd",
    "0050C2C20": "SRC Computers, LLC",
    "0050C2C1C": "Becton Dickinson",
    "0050C2C06": "ANALOG WAY",
    "0050C2C27": "Qtechnology A/S",
    "0050C2BF8": "Crtiical Link",
    "0050C2BF3": "Wanco Inc.",
    "0050C2BDB": "GasTOPS Ltd.",
    "0050C2BC7": "PTS GmbH",
    "0050C2BE3": "Jiskoot Ltd",
    "0050C2BE1": "Tattile srl",
    "0050C2BEC": "DRS Laruel Technologies",
    "0050C2BD6": "BG Systems, Inc.",
    "0050C2BD3": "Postjet Systems Ltd",
    "0050C2BBB": "GHL Systems Berhad",
    "0050C2BB8": "MoeTronix",
    "0050C2BC5": "Toptechnology SRL",
    "0050C2BC1": "Sentec Ltd",
    "0050C2BA5": "InterCel Pty Ltd",
    "0050C2BA4": "CUSTOS MOBILE S.L.",
    "0050C2BB0": "Gainbrain",
    "0050C2B91": "Finnet-Service Ltd.",
    "0050C2B9D": "W. Vershoven GmbH",
    "0050C2B9F": "AUDIOSCOPE 2K SRL",
    "0050C2B98": "Southwest Research Institute",
    "0050C2B99": "Greenlight Innovation Corp.",
    "0050C2B5A": "GREEN Center s.r.o.",
    "0050C2B78": "Folink",
    "0050C2B74": "AXED Jakubowski Wojciechowski sp.j.",
    "0050C2B6E": "Private",
    "0050C2B68": "Electronic Systems Protection, Inc.",
    "0050C2B67": "RC Systems Co. Inc.",
    "0050C2B80": "iScreen LLC",
    "0050C2B7D": "ELETECH Srl",
    "0050C2B8E": "WAC (Israel) Ltd.",
    "0050C2B63": "RO.VE.R. Laboratories S.p.A",
    "0050C2B52": "SMH Technologies",
    "0050C2B50": "SELCO",
    "0050C2B4E": "AixControl GmbH",
    "0050C2B43": "J-Systems Inc.",
    "0050C2B41": "Tata Power Company, Strategic Electronics Division",
    "0050C2B42": "ETM Electromatic Incorporated",
    "0050C2B3E": "Sage Consultants",
    "0050C2B32": "Byres Security Inc",
    "0050C2B23": "Ronyo Technologies s.r.o.",
    "0050C2B3C": "JanasCard",
    "0050C2B39": "siXis, Inc.",
    "0050C2B38": "Grenmore Ltd",
    "0050C2B13": "Measy Electronics Co., Ltd.",
    "0050C2B1E": "Abbott Medical Optics",
    "0050C2B1C": "Phytec Messtechnik GmbH",
    "0050C2B1A": "ELCUS",
    "0050C2B08": "Goerlitz AG",
    "0050C2B06": "CompuDesigns, Inc.",
    "0050C2B0C": "SMARTB TECHNOLOGIES",
    "0050C2AEF": "National CineMedia",
    "0050C2AD4": "Global Rainmakers Inc.",
    "0050C2AD3": "Peek Traffic Corporation",
    "0050C2ACF": "SP Controls, Inc",
    "0050C2AFF": "XoByte LLC",
    "0050C2AE7": "Redwood Systems",
    "0050C2AE8": "Bit-Lab PTY LTD",
    "0050C2AE6": "VECOM USA",
    "0050C2AF8": "Global Satellite Engineering",
    "0050C2AF9": "Ingenieurbuero Bickele und Buehler GmbH",
    "0050C2ADF": "Altinex, Inc",
    "0050C2ADB": "GO engineering GmbH",
    "0050C2AD8": "Incyma",
    "0050C2AD0": "Geonautics Australia Pty Ltd",
    "0050C2ACC": "ProPhotonix",
    "0050C2ACA": "Soft & Control Technology s.r.o.",
    "0050C2AC9": "Steinbeis-Transferzentrum Embedded Design und Networking",
    "0050C2AAF": "Santa Barbara Instrument Group",
    "0050C2AB8": "AHM Limited (CLiKAPAD)",
    "0050C2AB7": "Twinfalls Technologies",
    "0050C2AC6": "Marathon Products, Inc.",
    "0050C2AC2": "OpenXS B.V.",
    "0050C2ABF": "MCC Computer Company",
    "0050C2A9E": "Procon Engineering Limited",
    "0050C2A85": "JOYSYSTEM",
    "0050C2A84": "Lino Manfrotto +Co spa",
    "0050C2A9D": "Joehl & Koeferli AG",
    "0050C2A9C": "Star Electronics GmbH & Co. KG ",
    "0050C2A98": "Sentry 360 Security",
    "0050C2A8E": "BFI Industrie-Elektronik GmbH & Co.KG",
    "0050C2A76": "Roesch & Walter Industrie-Elektronik GmbH",
    "0050C2A75": "JTL Systems Ltd.",
    "0050C2A7A": "DetNet South Africa PTY (LTD)",
    "0050C2A3F": "LHA Systems CC",
    "0050C2A3D": "OWANDY",
    "0050C2A3E": "DUEVI SNC DI MORA E SANTESE",
    "0050C2A38": "Tred Displays",
    "0050C2A52": "The VON Corporation",
    "0050C2A4D": "LevelStar LLC.",
    "0050C2A47": "PRIMETECH ENGINEERING CORP.",
    "0050C2A43": "NKS Co.Ltd.",
    "0050C2A46": "Softronics Ltd.",
    "0050C2A55": "Arrowvale Electronics",
    "0050C2A57": "Juice Technologies, LLC",
    "0050C2A6C": "Figment Design Laboratories",
    "0050C2A65": "Mark-O-Print GmbH",
    "0050C2A61": "Fr. Sauter AG",
    "0050C2A37": "Software Systems Plus",
    "0050C2A0C": "Phytec Messtechnik GmbH",
    "0050C2A05": "Adgil Design Inc.",
    "0050C2A01": "Patronics International LTD",
    "0050C2A2C": "KWS-Electronic GmbH",
    "0050C2A29": "Luminex Corporation",
    "0050C2A28": "KENDA ELECTRONIC SYSTEMS LIMITED",
    "0050C2A26": "Preferred Oil, LLC",
    "0050C2A22": "Nippon Manufacturing Service Corporation (abbreviated as 'nms')",
    "0050C2A11": "OJSC Rawenstvo",
    "0050C29FB": "Villbau Kft.",
    "0050C29F6": "Ion Sense Inc.",
    "0050C2A1A": "DDL",
    "0050C29C4": "Vitel Net",
    "0050C29C3": "GE Security-Kampro",
    "0050C29BF": "2N TELEKOMUNIKACE a.s.",
    "0050C29BE": "Xad Communications Ltd",
    "0050C29D6": "Innovation, Institute, Inc",
    "0050C29D7": "Melex Inc.",
    "0050C29D1": "Bladelius Design Group AB",
    "0050C29DD": "Institut Dr. Foerster",
    "0050C29D8": "SAMSUNG HEAVY INDUSTRIES CO.,LTD.",
    "0050C29AF": "KRESS-NET Krzysztof Rutecki",
    "0050C29B5": "Telegamma srl",
    "0050C29E3": "SAI Informationstechnik",
    "0050C29CB": "NIS-time GmbH",
    "0050C29C7": "Kumera Drives Oy",
    "0050C299A": "Miromico AG",
    "0050C2999": "Cambustion Ltd",
    "0050C2995": "Inter Control Hermann K\u00f6hler  Elektrik GmbH&Co.KG",
    "0050C2994": "Xafax Nederland bv",
    "0050C2991": "UGL Limited",
    "0050C2982": "Triple Ring Technologies, Inc.",
    "0050C2979": "Far South Networks (Pty) Ltd",
    "0050C29A7": "Thales Communications & Security S.A.",
    "0050C29A5": "Conolog Corporation",
    "0050C29AC": "Questek Australia Pty Ltd",
    "0050C294E": "Zynix Original Sdn. Bhd.",
    "0050C295F": "METRONIC APARATURA KONTROLNO - POMIAROWA",
    "0050C295C": "Resurgent Health & Medical",
    "0050C2958": "Sensoptics Ltd",
    "0050C296D": "Keene Electronics Ltd.",
    "0050C2952": "Tech Fass s.r.o.",
    "0050C2940": "Phitek Systems Ltd.",
    "0050C2973": "Syst\u00e8mes Pran",
    "0050C2928": "Cinetix GmbH",
    "0050C2925": "PHB Eletronica Ltda.",
    "0050C2927": "ATIS group s.r.o.",
    "0050C293F": "TSB Solutions Inc.",
    "0050C2938": "Postec Data Systems Ltd",
    "0050C2939": "Mosaic Dynamic Solutions",
    "0050C2937": "BigBear",
    "0050C2930": "NETA Elektronik AS",
    "0050C292C": "Exatrol Corporation",
    "0050C2922": "Metrum Sweden AB",
    "0050C2923": "Amicus Wireless",
    "0050C2921": "iQue RFID Technologies BV",
    "0050C291E": "Automation Tec",
    "0050C291B": "Embedded Data Systems, LLC",
    "0050C2907": "Cristal Controles Ltee",
    "0050C28F7": "TGE Co., Ltd.",
    "0050C28FD": "Sindoma M\u00fch Mim \u00ddn\u00fe Elk San Tic Ltd.",
    "0050C2905": "Link Communications, Inc",
    "0050C291A": "Xtone Networks",
    "0050C2916": "CHK GridSense P/L",
    "0050C28FF": "Luceat",
    "0050C28D0": "Saia-Burgess Controls AG",
    "0050C28EF": "Technologies Sensio Inc",
    "0050C28CB": "Beonic Corporation",
    "0050C28CA": "Altair semiconductor Ltd",
    "0050C28DD": "GIMCON",
    "0050C28EB": "C-COM Satellite Systems Inc.",
    "0050C28C0": "S.C.E. s.r.l.",
    "0050C28C2": "Access Control Systems JSC",
    "0050C28DB": "DCOM Network Technology (Pty) Ltd",
    "0050C28AC": "Telsa s.r.l",
    "0050C28A9": "Intelligent Security Systems",
    "0050C28A8": "ALTEK ELECTRONICS",
    "0050C28AA": "ATS Elektronik GmbH",
    "0050C28A7": "PIXEYE LTD",
    "0050C28A6": "SELCO",
    "0050C289F": "Datalink Technologies Gateways Inc.",
    "0050C28B5": "Keith & Koep GmbH",
    "0050C28B0": "BK Innovation, Inc.",
    "0050C288C": "Z-App Systems",
    "0050C288E": "Cardinal Scale Mfg Co",
    "0050C28A2": "UAVISION Engenharia de Sistemas",
    "0050C2895": "Marine Communications Limited",
    "0050C286E": "HANYANG ELECTRIC CP., LTD",
    "0050C286D": "Tieline Research Pty Ltd",
    "0050C2870": "LOGEL S.R.L.",
    "0050C2878": "Acoustic Research Laboratories Pty Ltd",
    "0050C2865": "Persy Control Services B.v.",
    "0050C2861": "Grantronics Pty Ltd",
    "0050C2883": "Neocontrol Solu\u00e7\u00f5es em Automa\u00e7\u00e3o",
    "0050C2858": "Wireless Acquisition LLC",
    "0050C2845": "VisualSonics Inc.",
    "0050C2843": "Xtensor Systems Inc.",
    "0050C283F": "Phytec Messtechnik GmbH",
    "0050C283C": "hema electronic GmbH",
    "0050C2838": "T PROJE MUHENDISLIK DIS. TIC. LTD. STI.",
    "0050C2849": "Design Analysis Associates, Inc.",
    "0050C2828": "SLICAN sp. z o.o.",
    "0050C2851": "SPJ Embedded Technologies Pvt. Ltd.",
    "0050C2820": "TESCAN, s.r.o.",
    "0050C27E6": "Empirix Italy S.p.A.",
    "0050C27EA": "Monitor Business Machines Ltd.",
    "0050C280F": "Selekron Microcontrol s.l.",
    "0050C2812": "Femto SA",
    "0050C2802": "Private",
    "0050C27FB": "Qtron Pty Ltd",
    "0050C27D7": "Newtec A/S",
    "0050C27E1": "Zeltiq Aesthetics, Inc.",
    "0050C27CA": "CEDAR Audio Limited",
    "0050C27C2": "DSR Information Technologies Ltd.",
    "0050C27B9": "Technovare Systems, Inc.",
    "0050C27BB": "InRay Solutions Ltd.",
    "0050C27A7": "Guidance Navigation Limited",
    "0050C2797": "Tiefenbach Control Systems GmbH",
    "0050C27B2": "A.D.I Video technologies",
    "0050C27AD": "Turun Turvatekniikka Oy",
    "0050C27A0": "MedAvant Healthcare",
    "0050C279D": "MiraTrek",
    "0050C279E": "Benshaw Canada Controls, Inc.",
    "0050C278B": "OMICRON electronics GmbH",
    "0050C27AB": "General Microsystems Sdn Bhd",
    "0050C278A": "LEVEL TELECOM",
    "0050C2789": "Rosslare Enterprises Limited",
    "0050C2786": "Keith & Koep GmbH",
    "0050C2757": "E S P Technologies Ltd",
    "0050C2756": "Chesapeake Sciences Corp",
    "0050C2765": "Phytec Messtechnik GmbH",
    "0050C2763": "XtendWave",
    "0050C2781": "Starling Advanced Communications",
    "0050C2780": "IQ Solutions GmbH & Co. KG",
    "0050C276A": "Digidrive Audio Limited",
    "0050C2766": "Gutermann Technology GmbH",
    "0050C2773": "Pointe Conception Medical Inc.",
    "0050C2775": "Laserdyne Technologies",
    "0050C275B": "DMT System S.p.A.",
    "0050C2738": "Miracom Technology Co., Ltd.",
    "0050C272E": "SNCF EIM PAYS DE LOIRE",
    "0050C2755": "Teletek Electronics",
    "0050C2752": "LOBER, S.A.",
    "0050C2743": "Elektro-Top 3000 Ltd.",
    "0050C2740": "McQuay China",
    "0050C2734": "CardioMEMS Inc.",
    "0050C2746": "Realtronix Company",
    "0050C2744": "Avonaco Systems, Inc.",
    "0050C2751": "e&s Engineering & Software GmbH",
    "0050C2725": "DSP DESIGN",
    "0050C2704": "The Dini Group, La Jolla inc.",
    "0050C2702": "SPM Instrument AB",
    "0050C2703": "SAE IT-systems GmbH & Co. KG",
    "0050C2700": "GEM-MED SL",
    "0050C26FC": "Acte Sp. z o.o.",
    "0050C270F": "Zumbach Electronic AG",
    "0050C270C": "Exertus",
    "0050C2724": "HSC-Regelungstechnik GmbH",
    "0050C2713": "3DX-Ray Limited",
    "0050C2719": "ennovatis GmbH",
    "0050C2707": "DTech Labs Inc",
    "0050C26D8": "BL Healthcare, Inc.",
    "0050C26D9": "Ajeco Oy",
    "0050C26EA": "FIRSTEC SA",
    "0050C26EB": "Harrison Audio, LLC",
    "0050C26E5": "Boeckeler Instruments, Inc.",
    "0050C26E7": "Ace Axis Limited",
    "0050C26DC": "L-3 Communications Mobile-Vision, Inc.",
    "0050C26E0": "FIRSTTRUST Co.,Ltd.",
    "0050C26EF": "Pneumopartners LaenneXT SA",
    "0050C26F0": "Stanley Security Solutions, Inc.",
    "0050C26E3": "Miros AS",
    "0050C26F4": "Cryogenic Control Systems, Inc.",
    "0050C26AA": "Ifox - Industria e Comercio Ltda",
    "0050C26CA": "Dynamic Hearing Pty Ltd",
    "0050C26C4": "REXXON GmbH",
    "0050C26C1": "RADIUS Sweden AB",
    "0050C26D4": "Etani Electronics Co.,Ltd.",
    "0050C26A3": "CreaTech Electronics Co.",
    "0050C26C3": "iTRACS Corporation",
    "0050C26BD": "Mitron Oy",
    "0050C26B4": "SOMESCA",
    "0050C269B": "Tsien (UK) Ltd",
    "0050C2696": "Casabyte Inc.",
    "0050C269C": "Bug Labs, Inc.",
    "0050C2679": "Industrial Vacuum Systems",
    "0050C266B": "flsystem",
    "0050C268F": "BERTRONIC SRL",
    "0050C268A": "Zhuhai Jiahe Electronics Co.,LTD",
    "0050C267F": "Phytec Messtechnik GmbH",
    "0050C2690": "GHL Systems Berhad",
    "0050C265C": "VTZ d.o.o.",
    "0050C2660": "IZISOFT",
    "0050C2657": "MONYTEL S.A.",
    "0050C2634": "Sohon Inc",
    "0050C263E": "T2 Communication Ltd",
    "0050C2645": "The Software Group Limited",
    "0050C2647": "R&D Technology Solutionz Limited",
    "0050C2668": "Keith & Koep GmbH",
    "0050C2663": "COE Limited",
    "0050C2652": "Wideco Sweden AB",
    "0050C264C": "CIS Corporation",
    "0050C2626": "Winsys Informatica ltda",
    "0050C261A": "Communication Components Inc.",
    "0050C260C": "IDENTIC AB",
    "0050C261F": "Imagine Communications",
    "0050C2615": "Axis Electronics",
    "0050C25F3": "POSNET Polska S.A.",
    "0050C25DF": "Gnutek Ltd.",
    "0050C25F7": "Metrologic Group",
    "0050C25EA": "Micro Elektronische Producten",
    "0050C25E9": "Micro Technology Services Inc.",
    "0050C25FB": "All-Systems Electronics Pty Ltd",
    "0050C25D8": "TECHNIFOR SAS",
    "0050C25BF": "Techimp Systems S.r.l.",
    "0050C25C0": "Pyott-Boone Electronics",
    "0050C25C9": "Shenzhen Quanlong Technique Co.Ltd",
    "0050C25D1": "Meucci Solutions",
    "0050C25CF": "Innomed Medical Inc",
    "0050C25B8": "WestfaliaSurge GmbH",
    "0050C25B5": "RAFAEL",
    "0050C25D3": "Wexiodisk AB",
    "0050C25B0": "INCOTEC GmbH",
    "0050C25AD": "Emcom Systems",
    "0050C25A8": "ETAP NV",
    "0050C25A5": "Equipos de Telecomunicaci\u00f3n  Optoelectronicos, S.A.",
    "0050C2576": "Visi-tech Systems Ltd",
    "0050C2570": "Ellex Medical Pty Ltd",
    "0050C2571": "Oberon Service srl",
    "0050C25A4": "Federal State Unitary Enterprise Experimental Factory for Sc",
    "0050C25A0": "Rudolph Technologies, Inc.",
    "0050C2586": "Genetix Ltd",
    "0050C2550": "LJU Automatisierungstechnik GmbH",
    "0050C2566": "ubinetsys.co..ltd",
    "0050C255D": "ACD Elektronik GmbH",
    "0050C2557": "Netcomsec Co Ltd",
    "0050C255A": "Valde Systems, Inc.",
    "0050C2559": "Fail Safe Solutions LLC",
    "0050C251F": "Traquair Data Systems, Inc.",
    "0050C251E": "Alcon Technologies",
    "0050C2524": "Motec Pty Ltd",
    "0050C2523": "AMRDEC Prototype Integration Facility",
    "0050C2536": "C2 DIAGNOSTICS",
    "0050C2539": "Detection Technology Inc.",
    "0050C2537": "DST CONTROL AB",
    "0050C2533": "NanShanBridge Co.Ltd",
    "0050C2532": "NVE Corporation",
    "0050C2530": "Innovation, Institute, Inc",
    "0050C2545": "SecuInfo Co., Ltd.",
    "0050C2548": "I.T.W. Betaprint",
    "0050C2540": "Mesure Controle Commande",
    "0050C2514": "Tadian Electronics Systems LTD",
    "0050C2513": "Genie Network Resource Management Inc.",
    "0050C2510": "Summit Developmen",
    "0050C2517": "Solid State Logic",
    "0050C251A": "SpeasTech, Inc.",
    "0050C24F2": "Tantronic AG",
    "0050C24FC": "Hwayoung RF Solution Inc",
    "0050C2506": "7+ Kft",
    "0050C24E1": "SS Telecoms CC",
    "0050C24DF": "Thermo Electron",
    "0050C24E0": "Telematrix",
    "0050C24D8": "Avantry Ltd.",
    "0050C24C8": "Neets",
    "0050C24AA": "HEINEN ELEKTRONIK GmbH",
    "0050C24A8": "CYJAYA Korea",
    "0050C2496": "Acutelogic Corporation",
    "0050C2497": "Advanced Driver Information Technology GmbH",
    "0050C24A6": "BUYANG ELECTRONICS INDUSTRIAL CO., LTD.",
    "0050C24BC": "Saia Burgess Controls AG",
    "0050C246D": "Paul Scherrer Institut (PSI)",
    "0050C246B": "EASYTECH GmbH",
    "0050C2495": "VAZA Elektronik AB",
    "0050C2490": "Cloanto Corporation",
    "0050C2480": "SELKOM GmbH",
    "0050C2488": "DA SISTEMI SPA",
    "0050C2487": "Eridon Corporation",
    "0050C247B": "Pitney Bowes, Inc",
    "0050C2470": "Cybectec inc.",
    "0050C245E": "Halliburton - Sperry Drilling Service",
    "0050C2460": "Vitelnet",
    "0050C2461": "TATTILE SRL",
    "0050C245F": "T2C Marketing AB",
    "0050C2448": "Comtech Systems Inc.",
    "0050C2446": "Micro Technic A-S",
    "0050C244B": "Solace Systems, Inc.",
    "0050C2418": "Planea Oy",
    "0050C242A": "DSP DESIGN",
    "0050C243E": "Coppercom",
    "0050C2439": "Peleton Photonic Systems",
    "0050C2434": "ImperativeNetworks",
    "0050C240E": "ads-tec GmbH",
    "0050C2409": "KTEC LTD",
    "0050C2419": "Mecsel Oy",
    "0050C241B": "LogiM GmbH Software und Entwicklung",
    "0050C23F9": "Sintium Ltd",
    "0050C23F5": "Phaedrus Limited",
    "0050C2412": "TSB Solutions Inc.",
    "0050C23E8": "Conformative Systems, Inc.",
    "0050C23EB": "ISS International",
    "0050C23CD": "Micro-Measurements",
    "0050C23C9": "Dilax Intelcom AG",
    "0050C23B3": "Media Lab., Inc.",
    "0050C23D3": "American LED-gible Inc.",
    "0050C23D0": "Micro-Robotics Limited",
    "0050C23C3": "4g Technologies, L.P.",
    "0050C23C6": "Net Optics",
    "0050C23B9": "Gilbarco Autotank AB",
    "0050C2386": "Precision System Science Co.,Ltd",
    "0050C2389": "Exavio Inc.",
    "0050C23A6": "IntelliDesign Pty Ltd",
    "0050C2381": "Realtime Engineering AG",
    "0050C2344": "ads-tec GmbH",
    "0050C2342": "St. Michael Strategies",
    "0050C2345": "ACT",
    "0050C2346": "biokeysystem",
    "0050C235F": "F.Imm. S.r.L.",
    "0050C235B": "VLSIP TECHNOLOGIES, INC",
    "0050C2363": "Septentrio nv/sa",
    "0050C235E": "Jobin Yvon,Inc",
    "0050C236E": "Minicom Advanced Systems Ltd",
    "0050C2351": "Finesystem Co., Ltd",
    "0050C233C": "SkipJam",
    "0050C2326": "Navionics S.p.A.",
    "0050C2329": "Imax",
    "0050C2309": "Rackmaster Systems, Inc.",
    "0050C230F": "Digicontrole Lda",
    "0050C230D": "SETARAM",
    "0050C2310": "CYBERTRON CO., LTD.",
    "0050C2338": "Ernitec A/S",
    "0050C2333": "Radix Corporation",
    "0050C2321": "UXP",
    "0050C232B": "Digital Multimedia Technologies Spa",
    "0050C22F8": "SavvyCorp.com Ltd",
    "0050C22FB": "Arthur Industries Inc., dba On Hold Media Group",
    "0050C2307": "UNIONDIGITAL.,CO.LTD",
    "0050C2304": "COMERSON S.r.l.",
    "0050C22D8": "SYN-TECH SYSTEMS INC",
    "0050C22F1": "Geometrics, Inc.",
    "0050C22D3": "Gerber Scientific Products, Inc.",
    "0050C22D4": "Integrated System Solution Corp.",
    "0050C22CE": "Ross Video Limited",
    "0050C22BB": "TA Instruments Ltd",
    "0050C22BA": "NORCO INDUSTRIAL TECHNOLOGY INC",
    "0050C22AA": "DEUTA Werke GmbH",
    "0050C22AE": "Quest Retail Technology Pty Ltd",
    "0050C22C2": "Smarteye Corporation",
    "0050C22BE": "Lipowsky Industrie-Elektronik GmbH",
    "0050C2298": "Harvad University",
    "0050C2295": "LOGOSOL, INC.",
    "0050C229E": "SELEX Communications Ltd",
    "0050C229F": "Baudisch Electronic GmbH",
    "0050C22A7": "Micro System Architecturing srl",
    "0050C22A3": "West-Com Nurse Call Systems, Inc.",
    "0050C229D": "Globe Wireless",
    "0050C2291": "CHAUVIN ARNOUX",
    "0050C2293": "IP Unity",
    "0050C227E": "AnaLogic Computers Ltd.",
    "0050C2277": "T/R Systems, Inc.",
    "0050C226A": "FG SYNERYS",
    "0050C226B": "Continental Gateway Limited",
    "0050C2268": "Parabit Systems",
    "0050C227B": "LinkSecurity A/S",
    "0050C228A": "Real Time Systems",
    "0050C2288": "RPM Systems Corporation",
    "0050C228B": "Orion Technologies,LLC",
    "0050C2266": "ATOM GIKEN Co.,Ltd.",
    "0050C2263": "Vansco Electronics Oy",
    "0050C225E": "MITE Hradec Kralove, s.r.o.",
    "0050C225B": "Winford Engineering",
    "0050C225C": "Softhill Technologies Ltd.",
    "0050C223D": "Gauging Systems Inc",
    "0050C2237": "Tandata Systems Ltd",
    "0050C222F": "HTEC Limited",
    "0050C2248": "Dixtal Biomedica Ind. Com. Ltda.",
    "0050C2244": "intec GmbH",
    "0050C2249": "Bender GmbH & Co. KG",
    "0050C2252": "ads-tec GmbH",
    "0050C2250": "ACD Elektronik GmbH",
    "0050C223F": "Halliburton - NUMAR",
    "0050C225D": "RDTECH",
    "0050C2259": "Omicron Ceti AB",
    "0050C2257": "Digicast Networks",
    "0050C2223": "visicontrol GmbH",
    "0050C2211": "Hochschule f\u00fcr Technik, Wirtschaft und Kultur Leipzig (FH)",
    "0050C220B": "Rafael",
    "0050C222C": "Intrinsity",
    "0050C2226": "Ross Video Limited",
    "0050C2219": "Aeroflex GmbH",
    "0050C2234": "Silverback Systems",
    "0050C223A": "Chantry Networks",
    "0050C2203": "Vocality International Ltd",
    "0050C2202": "Audio Riders Oy",
    "0050C21F1": "SKY Computers, Inc.",
    "0050C21E9": "Ranch Networks",
    "0050C2205": "SystIng",
    "0050C2207": "Solectron Ind.Com.Servs.Exportadora do Brasil Ltda.",
    "0050C21CB": "quantumBEAM Limited",
    "0050C21D6": "shanghai trend intelligent systems CO.,LTD",
    "0050C21DF": "Lulea University of Technology",
    "0050C21AE": "Home Director, Inc",
    "0050C21B0": "BLANKOM Antennentechnik GmbH",
    "0050C21A7": "Alpha Beta Technologies, Inc.",
    "0050C21AA": "BitBox Ltd",
    "0050C21B9": "EmCom Technology Inc.",
    "0050C21B7": "MosChip USA",
    "0050C21BB": "Email Metering",
    "0050C21AD": "Remia s.r.o.",
    "0050C21A5": "NORCO",
    "0050C21A0": "SCA Data Systems",
    "0050C218E": "SPARR ELECTRONICS LTD",
    "0050C2191": "Partner Voxstream A/S",
    "0050C219C": "Artec Design",
    "0050C218D": "CCII Systems (Pty) Ltd",
    "0050C216D": "Postec Data Systems Ltd.",
    "0050C2064": "Private",
    "0050C216E": "PMC",
    "0050C2062": "Private",
    "0050C2017": "Hunter Technology Inc.",
    "0050C200D": "Opus Telecom Inc.",
    "0050C2150": "Torse",
    "0050C2157": "nCore, Inc.",
    "0050C2158": "Communication Solutions, Inc.",
    "0050C2155": "Enea Real Time AB",
    "0050C2137": "Uniwell Systems (UK) Ltd.",
    "0050C20F6": "Carl Baasel Lasertechnik GmbH",
    "0050C214E": "Corinex Global",
    "0050C2148": "Alltec GmbH",
    "0050C2136": "Tensilica, Inc.",
    "0050C2128": "Pycon, Inc.",
    "0050C2126": "MaxLinear Hispania S.L.U.",
    "0050C211D": "Destiny Networks, Inc.",
    "0050C2121": "COE Limited",
    "0050C211E": "Volvo Car Corporation",
    "0050C2124": "Tokai Soft Corporation",
    "0050C2075": "ENTTEC Pty Ltd.",
    "0050C2111": "Endusis Limited",
    "0050C2112": "Compuworx",
    "0050C2102": "Million Tech Development Ltd.",
    "0050C210A": "Quinx AG",
    "0050C20F5": "Spectra Technologies Holding Co., Ltd.",
    "0050C2106": "MATSUOKA",
    "0050C20EB": "iREZ Technologies LLC",
    "0050C20DB": "Cyberex",
    "0050C20E4": "Collabo Tec. Co., Ltd.",
    "0050C20DF": "Innovation Institute, Inc.",
    "0050C20D9": "Loewe Opta GmbH",
    "0050C20C2": "Alchemy Semiconductor, Inc.",
    "0050C20D5": "Zelax",
    "0050C20D2": "Real World Computing Partnership",
    "0050C209C": "RF Applications, Inc.",
    "0050C20CA": "J D Richards",
    "0050C20CC": "AlphaMedia Co., Ltd",
    "0050C20CF": "PCSC",
    "0050C20B6": "ApSecure Technologies (Canada), Inc.",
    "0050C20BE": "Stella Electronics & Tagging",
    "0050C20A7": "WaterCove Networks",
    "0050C20AD": "BMC Messsysteme GmbH",
    "0050C2099": "Case Information & Communications",
    "0050C209F": "MetaWave Vedeo Systems",
    "0050C209A": "NBO Development Center Sekusui Chemical Co. Ltd.",
    "0050C2088": "TELINC Corporation",
    "0050C2082": "GFI Chrono Time",
    "0050C207B": "Trikon Technologies Ltd.",
    "0050C2068": "Seabridge",
    "0050C2069": "EC Elettronica S.R.L.",
    "0050C2070": "Katchall Technologies Group",
    "0050C206F": "Digital Services Group",
    "0050C206E": "Avtron Manufacturing Inc.",
    "0050C206B": "NCast Corporation",
    "0050C2055": "San Castle Technologies, Inc.",
    "0050C205A": "Sonifex Ltd",
    "0050C2057": "Lite F GmBH",
    "0050C205F": "Malden Electronics Ltd",
    "0050C2056": "Base 2",
    "0050C203D": "ISDN Gateway Technology AG",
    "0050C204E": "Industrial Electronic Engineers, Inc.",
    "0050C2048": "Cybectec Inc.",
    "0050C203A": "PLLB Elettronica SPA",
    "0050C202F": "Sinetica Corp",
    "0050C201D": "Princeton Gamma Tech",
    "0050C2027": "Industrial Control Links",
    "0050C2028": "The Frensch Corporation (Pty) Ltd.",
    "0050C200F": "XLN-t",
    "40D8551AC": "Bucher Automation Budapest",
    "40D855130": "Televic Rail GmbH",
    "40D8551CE": "Peter Huber Kaeltemaschinenbau SE",
    "0050C2F16": "Peter Huber Kaeltemaschinenbau SE",
    "0050C2B5B": "Timberline Manufacturing",
    "40D855194": "RF Code",
    "0050C23F3": "Hitachi Energy Germany AG",
    "0050C2F12": "General Industrial Controls Pvt Ltd",
    "0050C26C2": "HoseoTelnet Inc...",
    "0050C29CD": "Uwe Schneider GmbH",
    "0050C2B58": "RealD, Inc.",
    "0050C29BB": "OMICRON electronics GmbH",
    "0050C2BFB": "ECS Srl",
    "0050C2398": "Inhand Electronics, Inc.",
    "0050C2E7C": "sp controls, inc ",
    "0050C2054": "Optionexist Limited",
    "0050C281F": "2N TELEKOMUNIKACE a.s.",
    "40D85506A": "elgris UG",
    "0050C20FC": "Kimmon Manufacturing Co., Ltd.",
    "0050C2A82": "CT Company",
    "0050C2EA9": "Mettler Toledo",
    "0050C227D": "ALLIED TELESIS K.K.",
    "40D855054": "VITEC",
    "0050C2DA4": "DEUTA-WERKE GmbH",
    "0050C2E44": "DEUTA-WERKE GmbH",
    "0050C27FA": "AutomationX GmbH",
    "0050C2D9A": "Saia-Burgess Controls AG",
    "0050C2761": "Elbit Systems of America",
    "0050C2025": "TERACOM TELEMATICA S.A",
    "40D8551DF": "Chengdu Meihuan Technology Co., Ltd",
    "0050C2B0B": "Honeywell",
    "0050C2F55": "Honeywell",
    "0050C23EA": "Alro Information Systems SA",
    "40D8550FF": "YUYAMA MFG Co.,Ltd",
    "0050C2EEC": "YUYAMA MFG Co.,Ltd",
    "0050C2452": "Scame Sistemi srl",
    "40D855165": "TECHBOARD SRL",
    "0050C21DC": "Imarda New Zealand Limited",
    "0050C2926": "DiTEST Fahrzeugdiagnose GmbH",
    "0050C208B": "HYPERCHIP Inc.",
    "0050C2B2C": "Concepteers, LLC",
    "40D8550C9": "QUANTAFLOW",
    "0050C2F63": "Triax A/S",
    "0050C25A7": "Phytec Messtechnik GmbH",
    "40D8550CD": "Logical Product",
    "0050C2C35": "Insitu, Inc",
    "0050C2FC6": "Critical Link LLC",
    "0050C20C4": "InterEpoch Technology,INC.",
    "0050C2301": "Delphi Display Systems, Inc.",
    "0050C2578": "Delphi Display Systems, Inc.",
    "0050C24F0": "MedAvant Healthcare",
    "40D855044": "An Chen Computer Co., Ltd.",
    "0050C22BF": "PERAX",
    "40D8551BD": "HORIBA ABX SAS",
    "0050C2EF9": "HORIBA ABX SAS",
    "0050C2CF5": "Gogo BA ",
    "0050C25AF": "DORLET SAU",
    "0050C21E5": "DORLET SAU",
    "0050C2EE2": "System Industrie Electronic GmbH",
    "0050C273B": "On Air Networks",
    "0050C2B48": "MTD GmbH",
    "0050C26FA": "DCNS",
    "0050C2374": "Owasys Advanced Wireless Devices",
    "0050C21EA": "DAVE SRL",
    "0050C21B3": "DSP DESIGN",
    "0050C2AFB": "Vocality International Ltd",
    "40D8550B7": "ACD Elektronik GmbH",
    "0050C29D3": "Telemetrie Elektronik GmbH",
    "0050C29EC": "Rohde&Schwarz Topex SA",
    "40D85519A": "Rohde&Schwarz Topex SA",
    "0050C2C9E": "Rohde&Schwarz Topex SA",
    "0050C2DD3": "Rohde&Schwarz Topex SA",
    "0050C2855": "Rohde&Schwarz Topex SA",
    "0050C2E2A": "Rohde&Schwarz Topex SA",
    "0050C2EE4": "Rohde&Schwarz Topex SA",
    "0050C20F8": "Tecnint HTE SRL",
    "40D8550BC": "Aplex Technology Inc.",
    "40D855060": "Aplex Technology Inc.",
    "0050C2F08": "Aplex Technology Inc.",
    "0050C27F4": "Wireless Cables Inc.",
    "0050C2DAC": "RFL Electronics, Inc.",
    "0050C22B7": "RAFI GmbH & Co.KG",
    "0050C2F26": "WaveIP",
    "0050C2C90": "RealD, Inc.",
    "0050C2553": "ATH system",
    "0050C23CC": "Linkwell Telesystems Pvt Ltd",
    "0050C27E9": "Sicon srl",
    "0050C2A92": "Sicon srl",
    "0050C2CE2": "Sicon srl",
    "0050C2E0A": "Sicon srl",
    "0050C2FD5": "American Microsystems, Ltd.",
    "0050C2107": "NewHer Systems",
    "40D85502F": "Advatek Lighting Pty Ltd",
    "0050C27A8": "Integrated Design Tools, Inc.",
    "0050C2527": "IRTrans GmbH",
    "0050C28A1": "Intune Networks",
    "0050C2380": "EKE-Electronics Ltd.",
    "0050C2A06": "CLOOS ELECTRONIC GMBH",
    "40D8550F1": "Grossenbacher Systeme AG",
    "0050C24F3": "Autronica Fire And Securirty",
    "0050C2BF0": "AIM",
    "0050C2CFA": "Grupo Epelsa S.L.",
    "0050C2A24": "Grupo Epelsa S.L.",
    "0050C2F58": "IEEE Registration Authority",
    "0050C29F0": "Veracity UK Ltd",
    "40D8551DB": "NIPPON TECHNO LAB.,INC,",
    "40D8551CB": "MG S.r.l.",
    "40D8551D5": "FST21 Ltd.",
    "40D8551D8": "Owl Cyber Defense Solutions, LLC",
    "40D8551B7": "TEWS Elektronik GmbH & Co. KG",
    "40D8551B8": "Orion Systems, Inc",
    "40D8551BC": "KbDevice,Inc.",
    "40D8551B4": "Inforce Computing Inc.",
    "40D855193": "FORZA SILICON CORP.",
    "40D855190": "Spider Tecnologia Ind. e Com Ltda",
    "40D85518E": "Kerun Visual Technology Co., Ltd.(Shenzhen)",
    "40D8551A3": "Noritake Itron Corporation",
    "40D8551A2": "HIPODROMO DE AGUA CALIENTE, S.A. DE C.V.",
    "40D8551A4": "cibite AG",
    "40D85516D": " GD Mission Systems",
    "40D85517B": "LUCEO",
    "40D855172": "YAWATA ELECTRIC INDUSTRIAL CO.,LTD.",
    "40D855175": "AHB Systeme GmbH",
    "40D855179": "Servo-Robot Inc.",
    "40D85516B": "TECHWAY",
    "40D855184": "Satkirit Ltd",
    "40D855186": "KST technology",
    "40D85514E": "Marposs S.p.A",
    "40D855148": "SEIKO TIME SYSTEMS INC.",
    "40D85515B": "SQF Spezialelektronik GmbH",
    "40D85515C": "Spectratech Inc.",
    "40D855154": "iart",
    "40D855152": "Home Automation Europe",
    "40D855162": "LUNA-NEXUS",
    "40D855122": "ATX Networks Ltd.",
    "40D855129": "DSP DESIGN",
    "40D855135": "GLOBALCOM ENGINEERING SRL",
    "40D85512C": "NSP Europe Ltd",
    "40D85511B": "nanoTRONIC GmbH",
    "40D855141": "Key Systems, Inc.",
    "40D855140": "InnoTrans Communications, Inc",
    "40D855104": "IMPLE SISTEMAS ELETRONICOS EMBARCADOS LTDA",
    "40D855112": "Halliburton - Sperry Drilling Service",
    "40D855103": "Peek Traffic Corporation",
    "40D855108": "ALPHA DESIGN CO.,LTD.",
    "40D855109": "Rosslare Enterprises Limited",
    "40D855116": "Uniscan LLC",
    "40D855119": "OOO Group of Industrial Technologies",
    "40D8550F3": "ECON Systems Inc.",
    "40D8550F5": "CST Group",
    "40D8550E6": "Kyoritsu Electric Corp.",
    "40D8550E3": "Medigus Ltd",
    "40D8550E4": "ARAGO SYSTEMS",
    "40D8550E2": "Keocko Holding Kft.",
    "40D8550D9": "YUKO ELECTRIC CO.,LTD",
    "40D8550E1": "STV Electronic GmbH",
    "40D8550EB": "WANTECH Networks",
    "40D8550E9": "HAMEG GmbH",
    "40D8550EF": "GeneSys Elektronik GmbH",
    "40D8550BF": "Shenzhen SETEC Power Co.,Ltd",
    "40D8550B4": "MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD.",
    "40D8550B3": "T.W.S. srl",
    "40D8550B1": "Nanjing TIANSU Automation Control System Co., Ltd.",
    "40D8550C7": "insensiv GmbH",
    "40D8550C0": "ACT",
    "40D8550BA": "PCH Engineering A/S",
    "40D8550A7": "First Design System Inc.",
    "40D8550A1": "ADVALY SYSTEM Inc.",
    "40D85509E": "NanoPulse, Inc.",
    "40D85509F": "Bascules Robbe nv",
    "40D85508B": "MeshWorks Wireless Oy",
    "40D855086": "DSP DESIGN",
    "40D855081": "Sicon srl",
    "40D8550A3": "Telefrank GmbH",
    "40D855096": "Comtel Electronics GmbH",
    "40D85508C": "Magnescale Co.,Ltd",
    "40D855067": "Tronic Control ltd.",
    "40D855066": "TeraTron GmbH",
    "40D85505F": "EPSa GmbH",
    "40D855079": "DelfiSolutions A/S",
    "40D855075": "Teraflops",
    "40D85507F": "Wheatstone Corporation",
    "40D855062": "Tech Source Inc",
    "40D855043": "SchulerControl GmbH",
    "40D85502C": "InventLab s.c.",
    "40D855023": "Shanghai o-solution electronics & Technology Co., Ltd.",
    "40D85502D": "Elgama Sistemos",
    "40D85502B": "Nomatronics",
    "40D855036": "Schweers informationstechnologie GmbH",
    "0050C2FFD": "Touchless Biometric Systems AG",
    "0050C2FFC": "Spirent Communications",
    "40D855022": "Digimerge Technology Inc",
    "40D85501E": "A2S",
    "0050C2FFF": "MSR-Solutions GmbH",
    "40D855000": "XRONOS.INC",
    "40D855001": "Vemotion",
    "40D85501C": "BERG Cloud Limited",
    "40D855017": "Franke Aquarotter GmbH",
    "40D855012": "Sencon Inc.",
    "0050C2FDD": "Toptech Systems, Inc.",
    "0050C2FCC": "Soudronic AG",
    "0050C2FEE": "Sparks Instruments SA",
    "0050C2FEB": "Axible Technologies",
    "0050C2FC3": "HSDC Sp. z o.o.",
    "0050C2FA6": "Hilkom digital GmbH",
    "0050C2FA1": "N-Hands GmbH und Co KG",
    "0050C2FAC": "ADETEL GROUP",
    "0050C2FB1": "MATELEX",
    "0050C2F8E": "GPO",
    "0050C2F8A": "EMAC, Inc.",
    "0050C2F70": "Noralta Technologies Inc",
    "0050C2F72": "MaxDeTec AG",
    "0050C2F91": "RE2 Inc",
    "0050C2F76": "Rong Jie(FuZhou)Electronics Co.,Ltd",
    "0050C2F6A": "OFI Inc. (dba 2D2C)",
    "0050C2F57": "Reach Technologies Inc.",
    "0050C2F59": "G3 Technologies",
    "0050C2F45": "HUSTY M.Styczen J.Hupert Sp.J.",
    "0050C2F4C": "Enistic Limited",
    "0050C2F1B": "Saia-Burgess Controls AG",
    "0050C2F19": "Netlink Bilisim Sistemleri San. ve Tic. Ltd. Sti.",
    "0050C2F13": "Packet Plus, Inc.",
    "0050C2F1F": "Verified Energy, LLC.",
    "0050C2F0C": "SKYCHANNEL LTD ",
    "0050C2F29": "RADYNE CORPORATION",
    "0050C2F3A": "Saia-Burgess Controls AG",
    "0050C2F3B": "TAMS firmware co.",
    "0050C2F04": "KINKI ROENTGEN INDUSTRIAL CO.,LTD",
    "0050C2F02": "BMR",
    "0050C2EF3": "Smart Power Electronics GmbH & Co. KG",
    "0050C2EE6": "B:TECH, a. s.",
    "0050C2EE7": "syes srl",
    "0050C2EDE": "Smart Grid Networks",
    "0050C2EEA": "Positioneering Limited",
    "0050C2EE8": "Kamacho Scale Co., Ltd.",
    "0050C2EC7": "LIQUID ROBOTICS, INC",
    "0050C2ED8": "AVocation Systems, Inc.",
    "0050C2EBC": "Diehl AKO Stiftung & Co. KG",
    "0050C2EBB": "TimeTerminal Adductor Group AB",
    "0050C2ED0": "Nippon Systemware Co.,Ltd.",
    "0050C2EC5": "RSUPPORT Co., Ltd.",
    "0050C2EB9": "ALPHA-MOS",
    "0050C2ECB": "FAL Corp",
    "0050C2E9E": "American Microsystems, Ltd.",
    "0050C2EA3": "Subsea Systems, Inc.",
    "0050C2E94": "ANA-U GmbH",
    "0050C2E9B": "Hentschel System GmbH",
    "0050C2EB6": "Monsoon Solutions, Inc.",
    "0050C2E57": "EOLANE MONTCEAU",
    "0050C2E56": "RFENGINE CO., LTD.",
    "0050C2E7B": "ATOM GIKEN Co.,Ltd.",
    "0050C2E6C": "SAMSUNG Electronics Co.,Ltd.(LED Division)",
    "0050C2E75": "FSM AG",
    "0050C2E76": "Embedded Solution Bank Co., Ltd.",
    "0050C2E85": "Cosmo Life Co.,Ltd",
    "0050C2E3E": "Monnit Corp.",
    "0050C2E50": "Tattile srl",
    "0050C2E51": "Motec Pty Ltd",
    "0050C2E4F": "Wine Technology Marlborough",
    "0050C2E41": "Higeco S.r.l.",
    "0050C2E43": "Technica Engineering GmbH",
    "0050C2E3F": "VISITO S.R.L.",
    "0050C2E2D": "Funkwerk IT Karlsfeld GmbH",
    "0050C2E2B": "Plant Integrity Limited",
    "0050C2E29": "Fr. Sauter AG",
    "0050C2E25": "ACD Elektronik GmbH",
    "0050C2E1B": "Embedded Labs",
    "0050C2E07": "Protagon Process Technologies GmbH",
    "0050C2DEA": "Cerner Corporation",
    "0050C2DF9": "Jenny Science AG",
    "0050C2DFA": "MAC Valves, Inc.",
    "0050C2DFB": "BETTINI SRL",
    "0050C2DF1": "Saia-Burgess Controls AG",
    "0050C2DD5": "Friend Spring Industrial Co., Ltd.",
    "0050C2DE3": "Breakaway Systems LLC",
    "0050C2DBA": "M.P. Electronics",
    "0050C2DC1": "Acrux Technology Limited",
    "0050C2DB5": "DSP DESIGN LTD",
    "0050C2DCD": "dilitronics GmbH",
    "0050C2DE8": "Visual Productions",
    "0050C2D9D": "Mistral Solutions Pvt. Ltd",
    "0050C2D8F": "Syes srl",
    "0050C2D8A": "OptoLink  Industria e Comercio Ltda",
    "0050C2DAD": "Keith & Koep GmbH",
    "0050C2D96": "CONTEC GmbH",
    "0050C2DA1": "MangoDSP",
    "0050C2D67": "KLING & FREITAG GmbH",
    "0050C2D86": "ECOMM ERA",
    "0050C2D79": "DSI RF Systems, Inc.",
    "0050C2D58": "NIK-ELEKTRONIKA Ltd",
    "0050C2D2B": "Video Tech Laboratories, Inc.",
    "0050C2D2C": "Schneider Electric Motion USA",
    "0050C2D28": "Digitale Analoge COMponenten West Electronic Vertriebs GmbH",
    "0050C2D4D": "Yardney Technical Products Inc.",
    "0050C2D46": "Thales Nederland BV",
    "0050C2D42": "Hagenuk KMT GmbH",
    "0050C2D35": "UG Systems GmbH & Co. KG",
    "0050C2D31": "UNGAVA Technologies Inc.",
    "0050C2D1E": "Tobila Systems, Inc.",
    "0050C2D21": "Innovative Circuit Technology",
    "0050C2D06": "nCk Research LLC",
    "0050C2D01": "Aanderaa Data Instruments",
    "0050C2CFF": "Infrasafe, Inc.",
    "0050C2CFE": "Techleader",
    "0050C2CF0": "Inviso B.V.",
    "0050C2CCE": "Mac-Gray Corporation ",
    "0050C2CD1": "ACD Elektronik GmbH",
    "0050C2CD6": "Arktan Systems",
    "0050C2CC0": "World Time Solutions Limited",
    "0050C2CE3": "Industrial Automatics Design Bureau ",
    "0050C2CDF": "CoreEL TEchnologies (I) Pvt Ltd",
    "0050C2CBE": "CODE BLUE CORPORATION",
    "0050C2CC9": "Promess GmbH",
    "0050C2CA8": "Systems With Intelligence Inc.",
    "0050C2CB4": "GEA Farm Technologies GmbH",
    "0050C2C9A": "PACOMP Sp. z o.o.",
    "0050C2CB8": "Raith GmbH",
    "0050C2CB6": "Krontek Pty Ltd",
    "0050C2C95": "IPSES S.r.l.",
    "0050C2C7E": "Buerkert Werke GmbH",
    "0050C2C7A": "Protonic Holland",
    "0050C2C65": "Micro I/O Servicos de Electronica, Lda",
    "0050C2C66": "KS Beschallungstechnik GmbH",
    "0050C2C8B": "OCAS AS",
    "0050C2C8C": "Lanmark Controls Inc.",
    "0050C2C60": "Integration Technologies Limited",
    "0050C2C73": "Industry Controls, Inc.",
    "0050C2C58": "Foerster-Technik GmbH",
    "0050C2C29": "Newtel Engineering S.r.l.",
    "0050C2C4C": "Lancier Monitoring GmbH",
    "0050C2C4A": "Herrick Technology Laboratories, Inc.",
    "0050C2C4B": "R.V.R. elettronica s.p.a.",
    "0050C2C53": "Eilersen Electric A/S",
    "0050C2C50": "Beceem Communications, Inc.",
    "0050C2C4E": "Elaso AG",
    "0050C2C51": "InForce Computing, Inc.",
    "0050C2C37": "B.E.A.R. Solutions (Australasia) Pty, Ltd",
    "0050C2C36": "SET GmbH",
    "0050C2C42": "Saia-Burgess Controls AG",
    "0050C2C1F": "Specialist Electronics Services Ltd",
    "0050C2C1D": "Powerbase Energy Systems Inc.",
    "0050C2C23": "Vidicon LLC",
    "0050C2C2E": "DISMUNTEL SAL",
    "0050C2C2D": "Digitale Analoge COMponenten West Electronic Vertriebs GmbH",
    "0050C2C12": "OKI DENKI BOHSAI CO.,LTD.",
    "0050C2BF1": "Amatic Industries GmbH",
    "0050C2C05": "Doppler Systems LLC",
    "0050C2C0D": "Fr. SauterAG",
    "0050C2C00": "ACD Elektronik GmbH",
    "0050C2BFC": "Altronix Corporation",
    "0050C2BE9": "ZUCCHETTI SPA",
    "0050C2BF5": "AILES ELECTRONICS CO., LTD.",
    "0050C2BD8": "b.a.b-technologie gmbh",
    "0050C2BD5": "RF-Embedded GmbH",
    "0050C2BD2": "Percello Ltd.",
    "0050C2BCD": "Highlight Parking Systems Ltd",
    "0050C2BAF": "MangoDSP",
    "0050C2BAC": "VITECO VNPT JSC",
    "0050C2BA8": "Peek Traffic Corporation",
    "0050C2BCC": "VVDN TECHNOLOGIES PVT. LTD.",
    "0050C2BCA": "Aitecsystem Co.,Ltd.",
    "0050C2BBA": "Systemteq Limited",
    "0050C2BC6": "MireroTack",
    "0050C2B97": "ikerlan",
    "0050C2B96": "Onix Electronic Systems Inc",
    "0050C2BA6": "Jomitek",
    "0050C2B9A": "Talo, NV Inc",
    "0050C2B7C": "Bettini srl",
    "0050C2B4A": "Saia-Burgess Controls AG",
    "0050C2B6B": "Phytec Messtechnik GmbH",
    "0050C2B60": "OOO NPF ATIS",
    "0050C2B61": "Nayos LTD",
    "0050C2B56": "SINOVIA SA",
    "0050C2B77": "KRISTECH",
    "0050C2B7B": "QUARTECH CORPORATION",
    "0050C2B31": "Shop Safe AG",
    "0050C2B2E": "ACT",
    "0050C2B2A": "Trench Austria GmbH",
    "0050C2B15": "PhotoTelesis LP",
    "0050C2B20": "FIVE9 NETWORK SYSTEMS LLC",
    "0050C2B05": "POLA s.r.l.",
    "0050C2B07": "FARECO",
    "0050C2AED": "3Roam",
    "0050C2B0F": "NARA Controls Inc.",
    "0050C2AF4": "Dixell S.p.A.",
    "0050C2AE9": "ClearCorp Enterprises, Inc",
    "0050C2AC4": "Orion Technologies,LLC",
    "0050C2AC3": "Diversified Control, Inc.",
    "0050C2ACE": "ChronoLogic Pty. Ltd.",
    "0050C2AC0": "DS PRO Audio Ltda",
    "0050C2ABE": "AP Labs",
    "0050C2ADE": "Neoptix Inc.",
    "0050C2ACD": "MeshWorks Wireless Oy",
    "0050C2A9F": "YellowSoft Co., Ltd.",
    "0050C2A95": "INNOVACIONES Microelectr\u00f3nicas SL (AnaFocus)",
    "0050C2AA5": "Tampere University of Technology",
    "0050C2A8F": "Quantum3D, Inc.",
    "0050C2A8D": "Frontier Electronic Systems Corp.",
    "0050C2A8A": "Audio Engineering Ltd.",
    "0050C2AAC": "VisiCon GmbH",
    "0050C2A70": "Reliable System Services Corp",
    "0050C2A6E": "Screen Technics Pty Limited",
    "0050C2A6F": "Saia-Burgess Controls AG",
    "0050C2A6D": "DTV Innovations",
    "0050C2A5A": "ITAS A/S",
    "0050C2A5B": "Phytec Messtechnik GmbH",
    "0050C2A67": "GSS Avionics Limited",
    "0050C2A64": "tetronik GmbH AEN",
    "0050C2A5E": "Ansen Investment Holdings Ltd.",
    "0050C2A72": "Gamber-Johnson LLC.",
    "0050C2A7D": "Vitel Net",
    "0050C2A79": "Saintronic",
    "0050C2A30": "D-TA Systems",
    "0050C2A2D": "Inventure Inc.",
    "0050C2A2B": "APRILIA RACING S.R.L.",
    "0050C2A1C": "Microtechnica",
    "0050C2A1B": "Realtime Systems Ltd.",
    "0050C2A42": "RealVision Inc.",
    "0050C2A4B": "L-3 Communications Mobile-Vision, Inc.",
    "0050C2A49": "Wayne Dalton Corp.",
    "0050C2A51": "Y-products co.ltd.",
    "0050C2A18": "Eoslink",
    "0050C2A27": "meconet e. K.",
    "0050C2A21": "ISAC SRL",
    "0050C2A3A": "Telecor Inc.",
    "0050C2A3C": "Br\u00e4hler ICS Konferenztechnik AG",
    "0050C2A32": "Harris Designs of NRV, Inc.",
    "0050C29DE": "CHAUVIN ARNOUX",
    "0050C29DB": "Walter Grotkasten",
    "0050C29FD": "Bitt technology-A Ltd.",
    "0050C2A12": "HCE Engineering S.r.l.",
    "0050C2A10": "Essential Design & Integration P/L",
    "0050C2A07": "Dynon Instruments",
    "0050C29EA": "SISMODULAR - Engenharia, Lda",
    "0050C29D9": "CNS Systems, Inc.",
    "0050C29E2": "E-ViEWS SAFETY SYSTEMS, INC",
    "0050C29EE": "Michael Stevens & Partners Ltd",
    "0050C2A02": "Reference, LLC.",
    "0050C29C9": "LUMINEX Lighting Control Equipment",
    "0050C29C5": "Scansonic MI GmbH",
    "0050C29BD": "Sensitron Semiconductor",
    "0050C29BC": "Kistler Straubenhardt GmbH",
    "0050C29A1": "ComAp s.r.o",
    "0050C299E": "Engage Technologies",
    "0050C29A0": "Trs Systems, Inc.",
    "0050C29D5": "Netpower Labs AB",
    "0050C29B6": "ACTECH",
    "0050C29C2": "Team Enginers",
    "0050C29C1": "Tattile srl",
    "0050C296B": "Electronic Media Services Ltd",
    "0050C296A": "Bittium Wireless Ltd",
    "0050C2993": "UNETCONVERGENCE CO., LTD",
    "0050C2560": "Procon Electronics",
    "0050C2989": "Psigenics Corporation",
    "0050C298F": "BELIK S.P.R.L.",
    "0050C298E": "Link Technologies, Inc",
    "0050C2972": "Switch Science (Panini Keikaku)",
    "0050C2986": "DSCI",
    "0050C2987": "Joinsoon Electronics MFG. Co., Ltd",
    "0050C297F": "C&I  Co.Ltd",
    "0050C283E": "KPE spol. s r.o.",
    "0050C2978": "LOGITAL DIGITAL MEDIA srl",
    "0050C2954": "Phytec Messtechnik GmbH",
    "0050C2942": "Keith & Koep GmbH",
    "0050C2949": "taskit GmbH",
    "0050C2948": "ELECTRONIA",
    "0050C2962": "Shockfish SA",
    "0050C2963": "L\u00e9cureux SA",
    "0050C294C": "TEMIX",
    "0050C2951": "EL.C.A. soc. coop.",
    "0050C2950": "Tele and Radio Research Institute",
    "0050C295E": "BEEcube Inc.",
    "0050C2959": "DECTRIS Ltd.",
    "0050C291F": "2NCOMM DESIGN SRL",
    "0050C2910": "Autotank AB",
    "0050C2912": "ASSET InterTech, Inc.",
    "0050C293C": "FractureCode Corporation",
    "0050C293A": "ALPHATRONICS nv",
    "0050C2924": "Link Electric & Safety Control Co.",
    "0050C2918": "Design Lightning Corp",
    "0050C28FA": "TELIUM s.c.",
    "0050C28F5": "tec5 AG",
    "0050C28F2": "Schneider Electric GmbH",
    "0050C2901": "Research Applications Incorp",
    "0050C2908": "Codex Digital Ltd",
    "0050C28F6": "K-MAC Corp.",
    "0050C28C5": "Vortex Engineering pvt ltd",
    "0050C28E4": "MaCaPS International Limited",
    "0050C28E1": "Deutscher Weterdienst",
    "0050C28E0": "Shenzhen Pennda Technologies Co., Ltd.",
    "0050C28B6": "Shadrinskiy Telefonny Zavod",
    "0050C28B2": "SERVAIND SA.",
    "0050C28E5": "Berthel GmbH",
    "0050C28CE": "Phytec Messtechnik GmbH",
    "0050C28CD": "Cambridge Sound Management, LLC",
    "0050C2887": "Inventis Technology Pty Limited",
    "0050C2889": "ACS Motion Control Ltd.",
    "0050C2885": "OOO \"NTK \"IMOS\"",
    "0050C2884": "IP Thinking A/S",
    "0050C2880": "Creation Technologies Chicago",
    "0050C287E": "SCM PRODUCTS, INC.",
    "0050C2898": "Veeco Process Equipment, Inc.",
    "0050C2897": "ODF Optronics, Inc.",
    "0050C28A0": "Specialized Communications Corp.",
    "0050C289C": "Mediana",
    "0050C2876": "Privatquelle Gruber GmbH & CO KG",
    "0050C2874": "Arcos Technologies Ltd.",
    "0050C2892": "Trakce a.s.",
    "0050C284D": "BMTI",
    "0050C285E": "Radiometer Medical ApS",
    "0050C2841": "Connection Electronics Ltd.",
    "0050C2842": "Quantum Controls BV",
    "0050C2839": "IMS R\u00f6ntgensysteme GmbH",
    "0050C284A": "Keystone Electronic Solutions",
    "0050C2862": "Elsys AG",
    "0050C2824": "SMT d.o.o.",
    "0050C2835": "Communications Laboratories Inc",
    "0050C2831": "St Jude Medical, Inc.",
    "0050C2836": "DSP DESIGN",
    "0050C282C": "Vitel Net",
    "0050C282A": "VDC Display Systems",
    "0050C2819": "Cabinplant A/S",
    "0050C280B": "Open Video, Inc.",
    "0050C27F6": "Saia-Burgess Controls AG",
    "0050C27F5": "ACE Carwash Systems",
    "0050C27EF": "GFI Chrono Time",
    "0050C27F0": "Network Harbor, Inc.",
    "0050C27F1": "STUHL Regelsysteme GmbH",
    "0050C27EB": "Sesol Industrial Computer",
    "0050C27ED": "Genesis Automation Inc.",
    "0050C27FF": "Shenzhen MaiWei Cable TV Equipment CO.,LTD.",
    "0050C27FD": "Adeneo",
    "0050C27FE": "Wireless Cables Inc.",
    "0050C2806": "CET",
    "0050C2805": "MultimediaLED",
    "0050C2801": "JANUS srl",
    "0050C27DC": "aiXtrusion GmbH",
    "0050C27D8": "InnoScan K/S",
    "0050C27B5": "DIT-MCO International",
    "0050C27D1": "Phytec Messtechnik GmbH",
    "0050C27D2": "Bittitalo Oy",
    "0050C27CB": "ViewPlus Technologies, Inc.",
    "0050C27AF": "C2 Microsystems",
    "0050C27B7": "Tattile srl",
    "0050C2776": "Integrated Security Corporation",
    "0050C2772": "IES Elektronikentwicklung",
    "0050C2788": "HOSA TECHNOLOGY, INC.",
    "0050C2785": "Icon Time Systems",
    "0050C276E": "Crinia Corporation",
    "0050C2791": "M Squared Lasers Limited",
    "0050C275A": "Gaisler Research AB",
    "0050C275C": "ST\u00d6RK-TRONIC St\u00f6rk GmbH&Co. KG",
    "0050C274F": "German Technologies",
    "0050C2748": "Letechnic Ltd",
    "0050C274A": "MONITOR ELECTRONICS LTD",
    "0050C2758": "AixSolve GmbH",
    "0050C2768": "Control Service do Brasil Ltda",
    "0050C2721": "Spectrum Communications FZE",
    "0050C271D": "MG s.r.l.",
    "0050C273F": "MEDAV GmbH",
    "0050C273C": "Simicon",
    "0050C2732": "Schlumberger K.K.",
    "0050C2731": "Spirent Communications",
    "0050C2728": "InterDigital Canada Ltd",
    "0050C2730": "haber & koenig electronics gmbh",
    "0050C272C": "Richard Griessbach Feinmechanik GmbH",
    "0050C272D": "Physical Acoustics Corporation",
    "0050C271C": "Elmec Inc.",
    "0050C2741": "Dain",
    "0050C273E": "Quantec Networks GmbH",
    "0050C2735": "Ant Lamp, Inc",
    "0050C2718": "illunis LLC",
    "0050C26F5": "Kitron Microelectronics AB",
    "0050C2708": "Smartek d.o.o.",
    "0050C2706": "DioDigiWorks. CO., LTD.",
    "0050C26E8": "Anymax",
    "0050C26FE": "Blue Origin",
    "0050C26FF": "St. Michael Strategies Inc.",
    "0050C2711": "LINKIT S.R.L.",
    "0050C270D": "ela-soft GmbH & Co. KG",
    "0050C26F9": "Revox GmbH",
    "0050C26F7": "infoplan Gesellschaftf\u00fcr  Informationssysteme mbH",
    "0050C26F2": "Laser Electronics Ltd",
    "0050C26BA": "Fertron Controle e Automacao Industrial Ltda.",
    "0050C26BB": "Ele.Mag S.r.l.",
    "0050C26C7": "QuickCircuit Ltd.",
    "0050C26C5": "Oerlikon Contraves AG",
    "0050C26B9": "unipo GmbH",
    "0050C26D0": "EDS Systemtechnik",
    "0050C267D": "ESA  Messtechnik GmbH",
    "0050C2680": "Honey Network Research Limited",
    "0050C26B5": "TRIUMF",
    "0050C26B0": "Smart Key International Limited",
    "0050C26AD": "Heim- & B\u00fcrokommunikation",
    "0050C26AB": "Softwareentwicklung",
    "0050C268D": "CXR Larus Corporation",
    "0050C269F": "Total RF, LLC",
    "0050C266E": "Linear Systems Ltd.",
    "0050C2665": "NetworkSound, Inc",
    "0050C2646": "TRUTOUCH TECHNOLOGIES INC",
    "0050C2677": "ProconX Pty Ltd",
    "0050C2678": "IHM",
    "0050C267A": "CC Systems AB",
    "0050C2676": "EDS",
    "0050C2654": "PaloDEx Group Oy",
    "0050C265A": "Hisstema AB",
    "0050C2653": "Doble Engineering",
    "0050C266C": "DESY",
    "0050C2643": "Enatel Limited",
    "0050C2621": "Version-T",
    "0050C2631": "Fraunhofer IIS",
    "0050C2628": "DARE Development",
    "0050C262A": "Prisma Engineering srl",
    "0050C2625": "EBNeuro SpA",
    "0050C261E": "LESTER ELECTRONICS LTD",
    "0050C2611": "Brookhaven National Laboratory",
    "0050C2610": "FDT Manufacturing, LLC",
    "0050C25EC": "ASiS Technologies Pte Ltd",
    "0050C260F": "Kommunikations- & Sicherheitssysteme Gesellschaft m.b.H",
    "0050C25F9": "ROTHARY Solutions AG",
    "0050C25CB": "Kobold Sistemi s.r.l.",
    "0050C25E0": "Phytec Messtechnik GmbH",
    "0050C25E1": "Ittiam Systems (P) Ltd",
    "0050C25CD": "RADA Electronics Industries Ltd.",
    "0050C25CE": "Roke Manor Research Ltd",
    "0050C25CA": "Buyang Electronics Industrial Co., Ltd.",
    "0050C25C8": "Georgia Tech Research Institute",
    "0050C25E6": "Musatel",
    "0050C25E7": "EADS TEST & SERVICES",
    "0050C25DD": "SomerData ltd",
    "0050C2599": "Fen Technology Limited",
    "0050C25A9": "AYC Telecom Ltd",
    "0050C259C": "DELSAT GROUP S.A.",
    "0050C2596": "SPANSION",
    "0050C2595": "Callpod, Inc.",
    "0050C259B": "SAPEC",
    "0050C25C3": "KS System GmbH",
    "0050C25BE": "Card Access Services Pty Ltd",
    "0050C25C1": "R. L. Drake Company",
    "0050C25B4": "Terrascience Systems Ltd.",
    "0050C256B": "Dataton Utvecklings AB",
    "0050C2581": "Devitech ApS",
    "0050C258E": "Penny & Giles Aerospace Ltd",
    "0050C2589": "HORIBA ABX SAS",
    "0050C258C": "Lattice Semiconductor Corp. (LPA)",
    "0050C2569": "Twinwin Technplogy Co.,Ltd.",
    "0050C2579": "Gastager Systemtechnik GmbH",
    "0050C2549": "Netsynt S.p.A.",
    "0050C2547": "BLANKOM Antennentechnik GmbH",
    "0050C2531": "Orion Technologies,LLC",
    "0050C252E": "DSP DESIGN",
    "0050C252C": "VITEC MULTIMEDIA",
    "0050C253A": "Image Control Design Limited",
    "0050C254A": "IPTC Tech. Comm. AB",
    "0050C2503": "RESPIRONICS INC.",
    "0050C24D9": "GE Security Kampro",
    "0050C24E8": "SATEL sp. z o.o.",
    "0050C24EF": "Creative Retail Entertainment",
    "0050C24FE": "GEM ELETTRONICA Srl",
    "0050C24D0": "RCS Energy Management Ltd",
    "0050C24D1": "SLICAN sp. z o.o.",
    "0050C24CF": "Ziehl-Abegg AG",
    "0050C24CE": "Open Date Equipment Limited",
    "0050C24D6": "Ingenieurb\u00fcro Schober",
    "0050C24B7": "GFI Chrono Time",
    "0050C24B8": "Shenzhen Hongdian Technologies.,Ltd",
    "0050C24B9": "Rose Technologies",
    "0050C24AC": "Doramu Co.,Ltd.",
    "0050C24AD": "OpenPeak, Inc.",
    "0050C24AB": "JVF Communications Ltd",
    "0050C24A9": "Faber Electronics BV",
    "0050C24C9": "Scirocco AB",
    "0050C2481": "Computer Sciences Corp",
    "0050C249A": "TelASIC Communications, Inc.",
    "0050C2499": "Trellia Networks",
    "0050C24A2": "SPECS GmbH",
    "0050C2486": "Safegate International AB",
    "0050C2467": "United Western Technologies",
    "0050C2466": "LONAP Limited",
    "0050C2473": "Sensus Metering Systems Israel",
    "0050C247D": "WIT Inc",
    "0050C2435": "ADATEL TELECOMUNICACIONES S.A.",
    "0050C2436": "Satellite Services BV",
    "0050C244C": "Computime Systems UK Ltd.",
    "0050C242D": "Argo-Tech",
    "0050C2428": "Roxar A/S",
    "0050C23FE": "HaiVision Systems Inc",
    "0050C23F8": "Superna Ltd",
    "0050C23FA": "Tumsan",
    "0050C2408": "TERN, Inc.",
    "0050C241D": "Altronic, Inc.",
    "0050C23EF": "PAT Industries, DBA Pacific Advanced Technology",
    "0050C23F2": "STL GmbH",
    "0050C23EE": "Commoca, Inc",
    "0050C23C4": "Sypris Electronics",
    "0050C23C2": "Casabyte Inc.",
    "0050C23F6": "dAFTdATA Limited",
    "0050C23D9": "Bavaria Digital Technik GmbH",
    "0050C23D8": "Key Systems , Inc.",
    "0050C23CA": "ABB Inc.",
    "0050C239C": "TIYODA MFG CO.,LTD.",
    "0050C23BE": "Pauly Steuer- und Regelanlagen GmbH & Co. KG",
    "0050C23AD": "Spirent Communications (Scotland) Limited",
    "0050C23A3": "Star Link Communication Pvt. Ltd.",
    "0050C23BC": "Tyzx, Inc.",
    "0050C236D": "Oplink Communications",
    "0050C236A": "Optronic Partner pr AB",
    "0050C2367": "CANMAX Technology Ltd.",
    "0050C2366": "Vanguard Technology Corp.",
    "0050C2394": "Embedit A/S",
    "0050C238F": "TTC Telecom",
    "0050C2391": "Esensors, Inc.",
    "0050C2385": "SUNGJIN NEOTECH Co.Ltd.",
    "0050C2370": "Europe Technologies",
    "0050C2371": "DIGITAL ART SYSTEM",
    "0050C237D": "VeroTrak Inc.",
    "0050C237B": "Freescale Semiconductor",
    "0050C2364": "Tattile srl",
    "0050C2372": "ELV Elektronik AG",
    "0050C233F": "EXYS bvba",
    "0050C2322": "BQT Solutions (Australia) Limited",
    "0050C234E": "ABB Power Technologies S.p.A.  Unit\u00e0\u00a0 Operativa SACE (PTMV)",
    "0050C2347": "Row Seven Ltd",
    "0050C2354": "Advanced IP Communications",
    "0050C234A": "NIE Corporation",
    "0050C232C": "Integrated Silicon Solution (Taiwan), Inc.",
    "0050C2357": "Athena Semiconductor",
    "0050C2306": "Noran Tel Communications Ltd.",
    "0050C2308": "FiveCo ",
    "0050C22EF": "Profline B.V.",
    "0050C22E9": "SRI International",
    "0050C231A": "Zodiak Data Systems",
    "0050C2319": "Invatron Systems Corp.",
    "0050C2318": "Milmega Ltd",
    "0050C2305": "Symbium Corporation",
    "0050C22FC": "Blackline Systems Corporation",
    "0050C2311": "Comodo",
    "0050C230C": "TEAMLOG",
    "0050C2323": "Red Rock Networks",
    "0050C22FF": "Patria Advanced Solutions",
    "0050C2315": "ifak system GmbH",
    "0050C22D2": "AIRNET COMMUNICATIONS CORP",
    "0050C22C9": "Roseman Engineering Ltd.",
    "0050C22CA": "PUTERCOM CO., LTD",
    "0050C22DB": "AutoTOOLS group Co. Ltd.",
    "0050C22CF": "Dise\u00f1o de Sistemas en Silicio S.A.",
    "0050C22D1": "Miritek, Inc.",
    "0050C22E8": "S.M.V. Systemelektronik GmbH",
    "0050C22DF": "MICREL-NKE",
    "0050C22E1": "Access IS",
    "0050C22DD": "Westek Technology Ltd",
    "0050C22C1": "Stage Tec Entwicklungsgesellschaft f\u00fcr professionelle Audio",
    "0050C22CC": "EMBEDDED TOOLSMITHS",
    "0050C22C4": "Invensys Energy Systens (NZ) Limited",
    "0050C22A1": "Infinetix Corp",
    "0050C22A2": "Epelsa, SL",
    "0050C22B3": "Embedded Systems Design",
    "0050C22A4": "Xipher Embedded Networking",
    "0050C2286": "ATEME",
    "0050C2281": "Cabtronix AG",
    "0050C2283": "ANSITEX CORP.",
    "0050C2296": "OpVista",
    "0050C2287": "TECNEW Electronics Engineering Cr., Ltd.",
    "0050C2271": "VLSIP TECHNOLOGIES INC.",
    "0050C2278": "Replicom Ltd.",
    "0050C2270": "S4 Technology Pty Ltd",
    "0050C226D": "DSP DESIGN",
    "0050C2262": "Shanghai Gaozhi Science&Technology Development Ltd.",
    "0050C2261": "Tattile Srl",
    "0050C2260": "BIOTAGE",
    "0050C2275": "Extreme Engineering Solutions",
    "0050C2240": "Geoquip Ltd",
    "0050C223E": "Kallastra Inc.",
    "0050C2231": "Legra Systems, Inc.",
    "0050C222B": "Riegl Laser Measurement Systems GmbH",
    "0050C223C": "Wheatstone Corporation",
    "0050C2243": "RGB Spectrum",
    "0050C224A": "CDS Rail",
    "0050C2200": "Whittier Mailing Products, Inc.",
    "0050C21FF": "Product Design Dept., Sohwa Corporation",
    "0050C220F": "OMICRON electronics GmbH",
    "0050C2214": "Oshimi System Design Inc.",
    "0050C2218": "Nansen S. A. - Instrumentos de Precis\u00e3o",
    "0050C2220": "Serveron Corporation",
    "0050C220C": "Communication and Telemechanical Systems Company Limited",
    "0050C2210": "Innovics Wireless Inc",
    "0050C21F9": "Fr. Sauter AG",
    "0050C21DE": "ReliOn Inc.",
    "0050C21F0": "EXI Wireless Systems Inc.",
    "0050C21F8": "ULTRACKER TECHNOLOGY",
    "0050C21D9": "EDC",
    "0050C21E8": "Metrotech",
    "0050C21B4": "DSP Group Inc.",
    "0050C21D2": "Shenyang Internet Technology Inc",
    "0050C21D1": "Benchmark Electronics",
    "0050C21C4": "Palm Solutions Group",
    "0050C21BF": "International Test & Engineering Services Co.,Ltd.",
    "0050C21B6": "DTS, Inc.",
    "0050C2198": "PotsTek, Inc",
    "0050C2196": "Netsynt Spa",
    "0050C219A": "AZIO TECHNOLOGY CO.",
    "0050C2195": "Momentum Data Systems",
    "0050C219F": "Fleetwood Electronics Ltd",
    "0050C21AC": "Beckmann+Egle GmbH",
    "0050C21B2": "SIGOS Systemintegration GmbH",
    "0050C218F": "MATSUI MFG CO.,LTD",
    "0050C2193": "LaserBit Communications Corp.",
    "0050C2189": "CC Systems AB",
    "0050C2175": "Sei S.p.A.",
    "0050C2187": "Cyan Technology Ltd",
    "0050C2184": "H M Computing Limited",
    "0050C2166": "Infineer Ltd.",
    "0050C2165": "IPCAST",
    "0050C2167": "Precision Filters, Inc.",
    "0050C215E": "Celite Systems, Inc.",
    "0050C2173": "DeMeTec GmbH",
    "0050C2003": "Microsoft",
    "0050C2169": "Nordson Corp.",
    "0050C2063": "Ticketmaster Corp",
    "0050C214F": "Telephonics Corp.",
    "0050C214B": "HECUBA Elektronik",
    "0050C214A": "DYCEC, S.A.",
    "0050C2140": "ITS, Inc.",
    "0050C212E": "RUNCOM",
    "0050C2120": "XStore, Inc.",
    "0050C2129": "TTPCom Ltd.",
    "0050C2104": "Adescom Inc.",
    "0050C2114": "Quest Innovations",
    "0050C2108": "Balogh S.A.",
    "0050C210B": "MarekMicro GmbH",
    "0050C210F": "Perceptics Corp.",
    "0050C20FA": "GE Transportation Systems",
    "0050C20FE": "Energy ICT",
    "0050C20EC": "Keith & Koep GmbH",
    "0050C20EF": "Movaz Networks, Inc.",
    "0050C20D8": "Charlotte's Web Networks",
    "0050C20D7": "Summit Avionics, Inc.",
    "0050C20E3": "Lanex S.A.",
    "0050C20DA": "Motion Analysis Corp.",
    "0050C20DD": "Interisa Electronica, S.A.",
    "0050C20E1": "Inspiration Technology P/L",
    "0050C20B2": "R F Micro Devices",
    "0050C20B3": "SMX Corporation",
    "0050C20BC": "Infolink Software AG",
    "0050C20A8": "Kaveri Networks",
    "0050C20A6": "Arula Systems, Inc.",
    "0050C20AB": "Fastware.Net, LLC",
    "0050C20AA": "Log-In, Inc.",
    "0050C20AC": "Honeywell GNO",
    "0050C208F": "General Industries Argentina",
    "0050C2091": "StorLogic, Inc.",
    "0050C209D": "ZELPOS",
    "0050C2097": "IMV Invertomatic",
    "0050C2085": "Crossport Systems",
    "0050C2077": "Saco Smartvision Inc.",
    "0050C2076": "Litton Guidance & Control Systems",
    "0050C207F": "Dunti Corporation",
    "0050C207A": "RadioTel",
    "0050C2074": "Edge Tech Co., Ltd.",
    "0050C2072": "Neuberger Gebaeudeautomation GmbH & Co.",
    "0050C205E": "DIVA Systems",
    "0050C205D": "Ignitus Communications, LLC",
    "0050C205C": "Nortel Networks PLC (UK)",
    "0050C205B": "Radiometer Medical A/S",
    "0050C2047": "B. R. Electronics",
    "0050C204C": "New Standard Engineering NV",
    "0050C2051": "JSR Ultrasonics",
    "0050C204F": "Luma Corporation",
    "0050C2031": "Eloquence Ltd",
    "0050C202D": "Innocor LTD",
    "0050C2039": "Apex Signal Corp",
    "0050C2020": "Icon Research Ltd.",
    "0050C2021": "DRS Technologies Canada Co.",
    "0050C2F27": "Bucher Automation Budapest",
    "0050C2DE7": "Bucher Automation Budapest",
    "0050C2FE5": "ScandiNova Systems",
    "0050C2DD6": "Wartsila Voyage Oy",
    "0050C2945": "Exi Flow Measurement Ltd",
    "0050C27C6": "SL Audio A/S",
    "40D855128": "AKSE srl",
    "0050C21A6": "RF Code",
    "0050C2DB1": "RF Code",
    "0050C2469": "Bipom Electronics, Inc.",
    "0050C2CF2": "Weiss Robotics GmbH & Co. KG",
    "0050C25F0": "DIAS Infrared GmbH",
    "0050C2F48": "SHURE INCORPORATED",
    "0050C2597": "Nautel LTD",
    "0050C2C08": "juiceboss",
    "0050C256D": "Computrol Fuel Systems Inc.",
    "40D8550F0": "Redwood Systems",
    "0050C2E66": "EMAC, Inc.",
    "0050C23D4": "Wisnu and Supak Co.,Ltd.",
    "0050C288D": "L3 Communications Nova Engineering",
    "0050C2E92": "CT Company",
    "0050C2139": "SR RESEARCH LTD",
    "0050C25E3": "Computechnic AG",
    "0050C2856": "CT Company",
    "0050C2DCB": "CT Company",
    "0050C2B6F": "CT Company",
    "40D8550C8": "Mettler Toledo",
    "0050C2B75": "Blankom",
    "0050C2DE0": "INTERNET PROTOCOLO LOGICA SL",
    "40D855076": "INTERNET PROTOCOLO LOGICA SL",
    "0050C2CB3": "DEUTA-WERKE GmbH",
    "0050C29EF": "WoKa-Elektronik GmbH",
    "0050C253E": "Honeywell",
    "0050C2204": "Algodue Elettronica Srl",
    "0050C2D3F": "Communication Systems Solutions",
    "40D8550C3": "APG Cash Drawer, LLC",
    "0050C2404": "NanShanBridge Co.Ltd",
    "0050C2FB6": "ARGUS-SPECTRUM",
    "0050C25B6": "Kontron (BeiJing) Technology Co.,Ltd",
    "0050C239B": "YUYAMA MFG Co.,Ltd",
    "0050C23CB": "Analytica GmbH",
    "0050C2FF1": "DiTEST Fahrzeugdiagnose GmbH",
    "40D855020": "ENTEC Electric & Electronic CO., LTD.",
    "0050C2E3D": "Baudisch Electronic GmbH",
    "0050C2BD1": "Ariem Technologies Pvt Ltd",
    "0050C244F": "kippdata GmbH",
    "0050C265B": "Silverbrook Research",
    "0050C239A": "Optical Air Data Systems",
    "0050C263F": "SPEECH TECHNOLOGY CENTER LIMITED",
    "0050C232A": "Phytec Messtechnik GmbH",
    "0050C2782": "Phytec Messtechnik GmbH",
    "0050C2392": "Phytec Messtechnik GmbH",
    "0050C23E9": "MedAvant Healthcare",
    "0050C277E": "Cominfo, Inc.",
    "0050C2604": "HCJB Global",
    "0050C2F64": "Chrisso Technologies LLC",
    "0050C2455": "Gogo BA ",
    "0050C250C": "AIRWISE TECHNOLOGY CO., LTD.",
    "0050C2163": "Computerwise, Inc.",
    "0050C2909": "Elisra",
    "0050C2DF4": "Potter Electric Signal Co. LLC",
    "0050C2B9C": "DAVE SRL",
    "0050C2F3E": "Vtron Pty Ltd",
    "0050C22F0": "LECO Corporation",
    "0050C2BC2": "Xslent Energy Technologies. LLC",
    "0050C2188": "dresden-elektronik",
    "0050C2D9E": "Vocality International Ltd",
    "0050C2F8D": "Guangdong East Power Co.,",
    "0050C26A5": "EATON FHF Funke + Huster Fernsig GmbH",
    "0050C266A": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C27A3": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2B92": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2CDC": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2EEE": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "0050C2594": "Pixel Velocity, Inc",
    "0050C28FB": "Alfred Kuhse GmbH",
    "0050C2E4B": "Rohde&Schwarz Topex SA",
    "0050C2F52": "Rohde&Schwarz Topex SA",
    "0050C2FC0": "Rohde&Schwarz Topex SA",
    "0050C285A": "ART SPA",
    "0050C2D11": "Aplex Technology Inc.",
    "0050C2186": "Pantec Engineering AG",
    "0050C2FB7": "Pounce Consulting",
    "40D8551DC": "Aplex Technology Inc.",
    "40D85516A": "Aplex Technology Inc.",
    "40D855147": "Aplex Technology Inc.",
    "0050C2F6F": "Aplex Technology Inc.",
    "0050C2EC3": "Aplex Technology Inc.",
    "0050C2585": "Wireless Cables Inc.",
    "0050C2723": "Power Electronics Espana, S.L.",
    "0050C2AC7": "WaveIP",
    "0050C24F7": "WaveIP",
    "0050C2ADD": " GD Mission Systems",
    "0050C2B19": "Polytron Corporation",
    "0050C2A6B": "Explorer Inc.",
    "0050C2393": "SYS TEC electronic GmbH",
    "0050C2D19": "Applied Medical Technologies, Inc DBA AirClean Systems",
    "0050C223B": "Envara",
    "0050C252B": "Sicon srl",
    "0050C2015": "LEROY AUTOMATION",
    "0050C2359": "Kramer Electronics Ltd.",
    "0050C2807": "TECHNOMARK",
    "0050C2C3A": "Sicon srl",
    "0050C2D8B": "Sicon srl",
    "0050C277B": "Itibia Technologies",
    "0050C2F53": "BAYCOM OPTO-ELECTRONICS TECHNOLGY CO., LTD.",
    "0050C25AB": "Eaton Corp. Electrical Group Data Center Solutions - Pulizzi",
    "0050C2620": "Harman/Becker Automotive Systems GmbH",
    "0050C2D5F": "Embedded Solution Bank Co., Ltd.",
    "0050C244A": "Elettronica Santerno SpA",
    "0050C24D7": "DELTA TAU DATA SYSTEMS, INC.",
    "0050C2033": "Doble Engineering",
    "0050C2029": "Grossenbacher Systeme AG",
    "0050C2A62": "Grossenbacher Systeme AG",
    "0050C2F1D": "Grossenbacher Systeme AG",
    "0050C2066": "Private",
    "0050C2C0C": "Altierre",
    "0050C2F35": "Grupo Epelsa S.L.",
    "0050C2E3C": "Grupo Epelsa S.L.",
    "0050C2493": "Artis GmbH",
    "0050C28C7": "TATTILE SRL",
    "0050C283D": "beroNet GmbH",
    "40D85519D": "EMAC, Inc.",
    "0050C2CC7": "TOPROOTTechnology Corp. Ltd.",
    "0050C24CD": "Securiton AG",
    "40D8551D2": "InventLab s.c.",
    "40D8551CD": "YXLON International A/S",
    "40D8551C3": "Cornfed Systems LLC",
    "40D8551C2": "Digital Display Systems",
    "40D8551BF": "shanghai mingding information tech co.Ltd",
    "40D8551BE": "Peek Traffic",
    "40D8551C8": "Sensata Technologies",
    "40D8551C9": "Andy-L Ltd.",
    "40D8551C7": "Wexiodisk AB",
    "40D8551E4": "STEK Ltd",
    "40D8551BB": "Micromega Dynamics SA",
    "40D8551B5": "A+EC Klein Ingenieurbuero",
    "40D8551A9": "Lubino s.r.o.",
    "40D855196": "Advanced Micro Controls Inc.",
    "40D855197": "Berg Cloud Limited",
    "40D855195": "TONNA ELECTRONIQUE",
    "40D855192": " GD Mission Systems",
    "40D85518D": "Zoe Medical",
    "40D85518C": "EOS S.r.l.",
    "40D855176": "Schneider Electric Motion, Inc. USA",
    "40D855174": "EcoGuard AB",
    "40D855180": "BroadSoft Inc",
    "40D85517D": "Kiwigrid GmbH",
    "40D855187": "CDEX Corp.",
    "40D855156": "Emphysys, Inc.",
    "40D85516C": "Private",
    "40D85513B": "Davin Technologies Co.,Ltd",
    "40D85513A": "Supplier Ind. e Com de Eletroeletr\u00f4nicos",
    "40D855138": "Calon Associates Limited",
    "40D855149": "Engage Technologies",
    "40D855150": "SHIKINO HIGH-TECH",
    "40D85512E": "Canfield Scientific, Inc.",
    "40D85512A": "Jadpod Communication Company Limited",
    "40D85510E": "HKS-Prozesstechnik GmbH",
    "40D855115": "MESA Electronic GmbH",
    "40D85510A": "DAVIS DERBY LIMITED",
    "40D855106": "Orbital A/S",
    "40D85511E": "CEMSI, Inc.",
    "40D85511D": "ACD Elektronik GmbH",
    "40D855118": "University of Nebraska -- Lincoln",
    "40D8550DA": "Devialet SA",
    "40D8550F2": "SigmaPhi Electronics",
    "40D8550FB": "InfoMac Sp. z o. o. Sp. k.",
    "40D855105": "Tieline Research Pty Ltd",
    "40D8550FE": "Cytech Technology Pte Ltd",
    "40D8550E0": "Richter",
    "40D8550CA": "NEUTRIK AG",
    "40D8550C6": "comtime GmbH",
    "40D8550B9": "WxBR Sistemas de Telecomunicacoes Ltda",
    "40D8550CC": "ATEME",
    "40D8550D5": "Shimizu Electric Co., Ltd.",
    "40D855088": "JEL SYSTEM CO., LTD.",
    "40D85508D": "Boehme Nachrichtentechnik",
    "40D8550AB": "Enel doo Belgrade",
    "40D855094": "Nomad Digital Limited",
    "40D8550A9": "Apantac LLC",
    "40D855097": "Burton Technical Services LLC",
    "40D8550A2": "Xemex NV",
    "40D855087": "Bestel China",
    "40D85507A": "4embedded",
    "40D855077": "TOEC TECHNOLOGY CO.,LTD",
    "40D855078": "NACHI-FUJIKOSHI CORP",
    "40D855065": "Parallel Wireless",
    "40D855074": "Sphere Medical Ltd",
    "40D85505B": "Data Flow Systems, Inc.",
    "40D85503D": "Tekelek Europe Ltd",
    "40D855037": "Software Workshop",
    "40D85503E": "Vishay Celtron Technologies, Inc.",
    "40D85503F": "UniSVR Global Information Technology Corp.",
    "40D85504A": "Gateway Technologies SA de CV",
    "40D855047": "Dos&Donts SRL",
    "40D855032": "BETTINI SRL",
    "40D855014": "Toni Studio",
    "40D855028": "Integrated Control Corp.",
    "40D855025": "Rosemount Analytical",
    "40D85500A": "Sarana Sistem Mikro",
    "40D855005": "Monarch Instrument",
    "40D855030": "Tecnologias Plexus",
    "0050C2FE3": "Private",
    "0050C2FF5": "Flexkom Internet Pazarlama Bilipim ve Eoitim Hiz.Inp.Mim.Muh.Oto.Enerji San. Tic. A.p.",
    "0050C2FF4": "Burk Technology",
    "0050C2FE2": "Pulsotronic Anlagentechnik GmbH",
    "0050C2FE1": "dotOcean",
    "0050C2FDE": "Peek Traffic",
    "0050C2FDF": "ACD Elektronik GmbH",
    "0050C2FDB": "The Security Center Inc",
    "0050C2FF7": "Human Intech",
    "0050C2FF9": "AVA Monitoring AB",
    "0050C2FF2": "GLOBALCOM ENGINEERING SRL",
    "0050C2FBC": "Leroy Somer",
    "0050C2FBB": "ACIDA GmbH",
    "0050C2FCD": "Jinyoung Contech",
    "0050C2FCA": "Telemisis Ltd",
    "0050C2FC9": "Mehta Tech, Inc.",
    "0050C2FC8": "Far South Networks",
    "0050C2FC2": "ELTA",
    "0050C2FC1": "Motec Pty Ltd",
    "0050C2F85": "Enetics, Inc.",
    "0050C2FB0": "Tateishi Kobisha Co.LTD",
    "0050C2FA7": "Exelis Inc.",
    "0050C2F82": "Sincair Systems International",
    "0050C2F81": "PLDA",
    "0050C2F99": "Dr. Neumann elektronik GmbH",
    "0050C2F98": "Infotech North America",
    "0050C2F95": "TTi LTD (Thurlby Thandar Instruments LTD)",
    "0050C2F9C": "R&D KOMETEH",
    "0050C2F9A": "Telvent",
    "0050C2F67": "Celestial Audio",
    "0050C2F7E": "TruTeq Wireless (Pty) Ltd",
    "0050C2F7C": "Atonometrics, Inc.",
    "0050C2F77": "SYSTEMTECHNIK GmbH",
    "0050C2F60": "Deckma GmbH",
    "0050C2F61": "Brauch Elektronik GmbH&Co.KG",
    "0050C2F69": "Safe Place Solutions Ltd",
    "0050C2F34": "Sequip S+E GmbH",
    "0050C2F2A": "ACD Elektronik GmbH",
    "0050C2F44": "Steinbichler Optotechnik GmbH",
    "0050C2F2E": "H&L Instruments, LLC",
    "0050C2F4A": "Z-App Systems, Inc.",
    "0050C2F42": "DSPCon",
    "0050C2F36": "Visitech AS",
    "0050C2F38": "AeroControl, Inc.",
    "0050C2F10": "Wincor Nixdorf Sp. z o.o.",
    "0050C2EFF": "Zephyrus Electronics LTD.",
    "0050C2F0A": "HASCOM International Pty Ltd",
    "0050C2F07": "Icon Research Ltd",
    "0050C2F20": "Unfors Instruments AB ",
    "0050C2F1E": "Dell'Orto S.P.A.",
    "0050C2F23": "Electro-Motive Diesel",
    "0050C2F1A": "Aqua Management",
    "0050C2F0D": "Bluetest AB",
    "0050C2F0F": "AeroVision Avionics, Inc.",
    "0050C2EF7": "Amstelland Electronic BV",
    "0050C2EF5": "Human Network Labs, Inc.",
    "0050C2EFA": "Predictive Sensor Technology",
    "0050C2EE0": "osf Hansjuergen Meier GmbH & Co. KG",
    "0050C2ECE": "easii ic adiis",
    "0050C2ECC": "Saia-Burgess Controls AG",
    "0050C2ECD": "Peek Traffic Corporation",
    "0050C2ECA": "BitWise Controls",
    "0050C2EA6": "Powersense A/S",
    "0050C2EB7": "Saab AB",
    "0050C2EB1": "PDU EXPERT UK LTD",
    "0050C2EC6": "INFRONICS SYSTEMS LIMITED",
    "0050C2EBD": "Droplet Measurement Technologies",
    "0050C2EBA": "West-Com Nurse Call Systems, Inc.",
    "0050C2E97": "Arista Systems Corporation",
    "0050C2E9F": "DataSoft Corporation",
    "0050C2E83": "Witree Co.,Ltd",
    "0050C2E8F": "STT Condigi A/S",
    "0050C2E77": "Fr. Sauter AG",
    "0050C2E5E": "OREP",
    "0050C2E5A": "FUTEC INC.",
    "0050C2E65": "IB Elektronik GmbH",
    "0050C2E53": "NEXT video systems Hard- and Software Development GmbH",
    "0050C2E35": "Omnica Corporation",
    "0050C2E36": "Saia-Burgess Controls AG",
    "0050C2E34": "HGL Dynamics",
    "0050C2E32": "Oshoksh Corporation",
    "0050C2E46": "Industrea Mining Technology",
    "0050C2E40": "Ecrin Systems",
    "0050C2E4D": "PCSC",
    "0050C2E48": "ITW Reyflex North America",
    "0050C2E23": "VITEC",
    "0050C2E21": "Norwia AS",
    "0050C2E20": "Divelbiss Corporation",
    "0050C2E0E": "PMAC JAPAN",
    "0050C2E0D": "Unixmedia Srl",
    "0050C2E19": "Zoe Medical",
    "0050C2E1D": "Holdline Tecnologia e Sistemas Ltda",
    "0050C2E14": "Calixto Systems Pvt Ltd",
    "0050C2E13": "Automation Assist Japan Company",
    "0050C2E09": "ATEME",
    "0050C2E06": "Ebner Electronic GmbH",
    "0050C2DFC": "I-Evo Ltd",
    "0050C2E03": "ICU Scandinavia Schweiz GmbH",
    "0050C2DF0": "Koncar Electrical Engineering Institute",
    "0050C2DEF": "Powersense A/S",
    "0050C2DF6": "HINO ENGINEERING, INC",
    "0050C2DEB": "Ruwisch & Kollegen GmbH",
    "0050C2DEC": "VendNovation LLC",
    "0050C2DED": "Lee Laser",
    "0050C2DBF": "One-Nemoto Engineering Corporation",
    "0050C2DCF": "MCS Engenharia ltda",
    "0050C2DB2": "SoftwareCannery",
    "0050C2DC9": "KinotonGmbH",
    "0050C2DC8": "T2M2 GmbH",
    "0050C2DCC": "Instrumentel Limited",
    "0050C2DC5": "Saia-Burgess Controls AG",
    "0050C2DB6": "PROSOFT-SYSTEMS LTD",
    "0050C2DB7": "SOREL GmbH Mikroelektronik",
    "0050C2DB4": "ZAO NPC \"Kompjuternie Technologii\"",
    "0050C2DD4": "SYSTECH",
    "0050C2DC2": "TESSERA TECHNOLOGY INC.",
    "0050C2DDB": "LUCEO",
    "0050C2D89": "Visual Telecommunication Network, Inc",
    "0050C2DA2": "metraTec GmbH",
    "0050C2DAE": "Spang Power Electronics",
    "0050C2D8C": "iRphotonics",
    "0050C2DA9": "Tieline Research Pty Ltd",
    "0050C2DA8": "Sine Systems, Inc.",
    "0050C2D93": "Axlon AB",
    "0050C2D72": "Scale-Tron, Inc.",
    "0050C2D70": "C. Rob. Hammerstein GmbH & Co. KG",
    "0050C2D6F": "Imtron Messtechnik GmbH",
    "0050C2D78": "P4Q Electronics",
    "0050C2D77": "Fr.SauterAG",
    "0050C2D54": "ABtrack s.r.l.",
    "0050C2D5D": "GLOBALCOM ENGINEERING SRL",
    "0050C2D5A": "Embedded Monitoring Systems Ltd.",
    "0050C2D57": "Hijikata Denki Corp.",
    "0050C2D80": "Keith & Koep GmbH",
    "0050C2D64": "TV1 GmbH",
    "0050C2D29": "Axible Technologies",
    "0050C2D24": "Expro North Sea",
    "0050C2D23": "Bluestone Technology GmbH",
    "0050C2D51": "BETTINI SRL",
    "0050C2D4B": "Indra Australia",
    "0050C2D48": "Watermark Estate Management Services, LLC",
    "0050C2D2A": "Millennium Electronics Pty.Ltd.",
    "0050C2D3E": "Synatec Electronic GmbH",
    "0050C2D1D": "Moco Media Pty Ltd",
    "0050C2D38": "Kyowa Electronics Co.,Ltd.",
    "0050C2CF3": "Daiken Automacao Ltda",
    "0050C2CF4": "Baudisch Electronic GmbH",
    "0050C2CF1": "TelGaAs, Inc.",
    "0050C2D16": "Imricor Medical Systems, Inc.",
    "0050C2D15": "MSR-Office GmbH",
    "0050C2D08": "Reimesch Kommunikationssysteme GmbH",
    "0050C2CFB": "New Embedded Technology",
    "0050C2CEE": "EMBED-IT OG",
    "0050C2CE8": "Thomas & Betts",
    "0050C2CE0": "Industrial Control Links, Inc.",
    "0050C2CDD": "K.C.C. SHOKAI LIMITED",
    "0050C2CE4": "TEKTRONIK",
    "0050C2CED": "AeroMechanical Services Ltd, FLYHT",
    "0050C2CCC": "Smartech-technology",
    "0050C2CBC": "CP ELETRONICA SA",
    "0050C2CC6": "KDT",
    "0050C2C9F": "xxter b.v.",
    "0050C2C9C": "Saia-Burgess Controls AG",
    "0050C2C93": "SENSAIR Pty Ltd",
    "0050C2CA2": "The Logical Company",
    "0050C2CA1": "Wayne Kerr Electronics",
    "0050C2CAA": "DSP DESIGN LTD",
    "0050C2CA4": "Vox Technologies",
    "0050C2C99": "HJPC Corporation dba Pactron",
    "0050C2C8E": "SDD ITG",
    "0050C2C7D": "TAE Antriebstechnik GmbH",
    "0050C2C77": "AIM Co.,Ltd",
    "0050C2C81": "Odyssee Systemes SAS",
    "0050C2C7F": "Kinects Solutions Inc",
    "0050C2C87": "LECO Corporation",
    "0050C2C88": "CSI Controles e Sistemas Industriais Ltda.",
    "0050C2C55": "Watterott electronic",
    "0050C2C5E": "CellPlus technologies, Inc.",
    "0050C2C68": "Broadsoft PacketSmart, Inc.",
    "0050C2C74": "Wapice Ltd.",
    "0050C2C70": "Wilke Technology GmbH",
    "0050C2C46": "QNE GmbH & Co. KG",
    "0050C2C47": "Weltek Technologies Co. Ltd.",
    "0050C2C44": "Beijing Zhongherongzhi Elec.&Tech.Co.,Ltd.",
    "0050C2C25": "Private",
    "0050C2C24": "Qualnetics Corporation",
    "0050C2C21": "Private",
    "0050C2C15": "INO - Institut National d'Optique",
    "0050C2C10": "Keith & Koep GmbH",
    "0050C2C38": "Computer Automation Technology Inc",
    "0050C2C34": "Kyuhen",
    "0050C2C2F": "REFLEX CES",
    "0050C2C2B": "Z-App Systems, Inc.",
    "0050C2C1E": "Peperoni-Light",
    "0050C2C3C": "ELSIST S.r.l.",
    "0050C2BDF": "Euro-Konsult Sp. z o.o.",
    "0050C2BDA": "Digital Lumens",
    "0050C2C07": "CIO Informatique Industrielle",
    "0050C2C01": "QUERCUS TECHNOLOGIES, S.L.",
    "0050C2BFD": "Ernemann Cine Tec GmbH",
    "0050C2C0F": "DYCEC, S.A.",
    "0050C2BD9": "AMS Controls, Inc.",
    "0050C2BED": "Touch Revolution Inc.",
    "0050C2BE8": "VEHICLE TESTING EQUIPMENT, S.L.",
    "0050C2C02": "Hanning Elektro-Werke GmbH & Co. KG",
    "0050C2BF9": "Vitel Net",
    "0050C2BC4": "Wheatstone Corporation",
    "0050C2BAA": "NetworkFX Communications, LLC",
    "0050C2BA9": "SISS Technology Inc.",
    "0050C2BB6": "Quarch Technology Ltd",
    "0050C2BB5": "MROAD INFORMATION SYSTEM",
    "0050C2BB1": "Pro4tech",
    "0050C2BB4": "JSC Electrical Equipment Factory",
    "0050C2BAE": "Fiber Connections Inc.",
    "0050C2BC3": "Phytec Messtechnik GmbH",
    "0050C2BCB": "ARTEIXO TELECOM",
    "0050C2B9E": "Saia-Burgess Controls AG",
    "0050C2BA3": "DSP DESIGN LTD",
    "0050C2B79": "MITSUYA LABORATORIES INC.",
    "0050C2B82": "TANABIKI Inc.",
    "0050C2B83": "Advanced Storage Concepts, Inc.",
    "0050C2B95": "Rx Monitoring Services",
    "0050C2B93": "EMC PARTNER AG",
    "0050C2B8B": "FBB",
    "0050C2B86": "ASTO",
    "0050C2B5D": "Plitron Manufacturing Inc.",
    "0050C2B6A": "Phytec Messtechnik GmbH",
    "0050C2B69": "Thetis S.p.A.",
    "0050C2B4D": "Troll Systems Corporation",
    "0050C2B44": "Ampcontrol Pty Ltd",
    "0050C2B46": "Mobileye",
    "0050C2B71": "Digitale Analoge COMponenten West Electronic Vertriebs GmbH",
    "0050C2B72": "Advanced Desktop Systems Ltd",
    "0050C2B5F": "North Bridge Technologies",
    "0050C2B5C": "ADI Video Technologies",
    "0050C2B53": "Prodco",
    "0050C2B4B": "Chitose Co.,Ltd",
    "0050C2B0E": "KYAB Lulea AB",
    "0050C2B3A": "Nikon Systems Inc.",
    "0050C2B3B": "Sportvision Inc.",
    "0050C2B33": "Numcore Ltd",
    "0050C2B35": "haneron",
    "0050C2B17": "Elcoteq Design Center Oy",
    "0050C2B16": "Neothings, Inc.",
    "0050C2B14": "Keith & Koep GmbH",
    "0050C2B2B": "CosmoData Informatica Ltda.",
    "0050C2B29": "Integra LifeSciences (Ireland) Ltd",
    "0050C2B11": "EXEL s.r.l",
    "0050C2B2D": "Datasat Digital Entertainment",
    "0050C2B03": "Spider Tecnologia Ind. e Com. Ltda.",
    "0050C2B01": "HSR Harald L. Reuter",
    "0050C2AF7": "Midwest Microwave Solutions Inc.",
    "0050C2AF6": "Energid",
    "0050C2AF5": "Kramara s.r.o.",
    "0050C2AFE": "Trolex Limited",
    "0050C2AFC": "Odus Technologies SA",
    "0050C2B09": "Harper Chalice Group Limited",
    "0050C2AEE": "IPtec, Inc.",
    "0050C2AEC": "Fritz Pauker Ingenieure GmbH",
    "0050C2ADC": "Synthesechemie Dr. Penth GmbH",
    "0050C2AD9": "elettrondata srl",
    "0050C2AD7": "Air Monitors Ltd",
    "0050C2AC1": "DAISHIN-DENSHI Co., Ltd",
    "0050C2ABC": "Barrick",
    "0050C2ABD": "Monitor Business Machines Ltd.",
    "0050C2AAE": "OUTLINE srl",
    "0050C2AB0": "FRAKO Kondensatoren- und Anlagenbau GmbH",
    "0050C2AB1": "Bitmanufaktur GmbH",
    "0050C2AB3": "Compa\u00f1\u00eda de Instrumentacion y control, S.L.",
    "0050C2ABA": "Saia-Burgess Controls AG",
    "0050C2AB6": "Gygax Embedded Engineering GEE.ch",
    "0050C2AE4": "Advanced Electronic Designs, Inc.",
    "0050C2AD2": "Rafael",
    "0050C2A9B": "PDQ Manufacturing  Inc.",
    "0050C2A81": "BPC circuits Ltd",
    "0050C2A7F": "Phytec Messtechnik GmbH",
    "0050C2A83": "Techno Sobi Co. Ltd.",
    "0050C2AAB": "BRS Sistemas Eletr\u00f4nicos",
    "0050C2AA9": "SAN GIORGIO S.E.I.N. srl",
    "0050C2AA4": "PSi Printer Systems international GmbH",
    "0050C2A97": "MICROSYSTEMES",
    "0050C2A74": "DSP DESIGN LTD",
    "0050C2A69": "Advanced Integrated Systems",
    "0050C2A63": "EMS Industries",
    "0050C2A5C": "JSC \"Component-ASU\"",
    "0050C2A71": "Purite Ltd",
    "0050C2A77": "Keith & Koep GmbH",
    "0050C2A40": "Mosberger Consulting LLC",
    "0050C2A35": "Appareo Systems, LLC",
    "0050C2A36": "Shenzhen Shangji electronic Co.Ltd",
    "0050C2A34": "Casabyte Inc.",
    "0050C2A0D": "CHARLYROBOT",
    "0050C2A0B": " IDS GeoRadar s.r.l.",
    "0050C2A0A": "ACD Elektronik GmbH",
    "0050C2A03": "EEG Enterprises Inc",
    "0050C29FF": "Humphrey Products",
    "0050C2A23": "Agility Mfg, Inc.",
    "0050C2A13": "Talyst, Inc.",
    "0050C29E8": "Hammock Corporation",
    "0050C29E4": "Pyxis Controls WLL",
    "0050C29F7": "Dave Jones Design",
    "0050C29F2": "Keith & Koep GmbH",
    "0050C29F3": "Vision Technologies, Inc.",
    "0050C29DC": "FTM Marketing Limited",
    "0050C29EB": "AFORE Solutions Inc.",
    "0050C29D4": "FIRST",
    "0050C29D2": "Saia-Burgess Controls AG",
    "0050C29DA": "NEUTRONIK e.K.",
    "0050C29C0": "Stuyts Engineering Haarlem BV",
    "0050C29BA": "Connor-Winfield",
    "0050C299D": "Powersense A/S",
    "0050C29B0": "Ebru GmbH",
    "0050C29AE": "Esensors, Inc.",
    "0050C29A2": "SAMsystems GmbH",
    "0050C2990": "Keith & Koep GmbH",
    "0050C298B": "TI2000 TECNOLOGIA INFORMATICA 2000",
    "0050C298C": "MGM-Devices Oy",
    "0050C2961": "Picsolve International Limited",
    "0050C29AB": "Electrodata Inc.",
    "0050C2957": "STRATEC Control Systems",
    "0050C2960": "kuroneko dennnou kenkyuushitsu",
    "0050C2976": "SANDS INSTRUMENTATION INDIA PVT LTD",
    "0050C297D": "Soehnle Professional GmbH & Co.KG",
    "0050C297A": "KST Technology Co., Ltd",
    "0050C296C": "Aqua Cooler Pty Ltd",
    "0050C2971": "IPITEK",
    "0050C296F": "Varec Inc.",
    "0050C2968": "BuLogics, Inc.",
    "0050C2965": "Emitech Corporation",
    "0050C295B": "AS Solar GmbH",
    "0050C294F": "IT-Designers GmbH",
    "0050C294D": "C&H technology  ltd.",
    "0050C292B": "DSP DESIGN",
    "0050C2920": "Rogue Engineering Inc.",
    "0050C2935": "Image Video",
    "0050C2943": "QuanZhou TDX Electronics Co., Ltd.",
    "0050C290C": "LSS GmbH",
    "0050C290A": "Board Level Limited",
    "0050C290B": "E.ON ES Sverige AB",
    "0050C2919": "AHV Systems, Inc.",
    "0050C2917": "CIRTEM",
    "0050C28E8": "Friedrich Kuhnt GmbH",
    "0050C28D6": "UltraVision Security Systems, Inc.",
    "0050C28CF": "GigaLinx Ltd.",
    "0050C28EA": "ATEME",
    "0050C28D1": "Bachmann Monitoring GmbH",
    "0050C28DF": "Dipl.-Ing. W. Nophut GmbH",
    "0050C28A3": "RTW GmbH & Co.KG",
    "0050C28BE": "The Pennsylvania State University",
    "0050C28BA": "Fr. Sauter AG",
    "0050C28A4": "BALOGH T.A.G Corporation",
    "0050C28A5": "Mocon, Inc.",
    "0050C2899": "Inico Technologies Ltd.",
    "0050C2893": "EIZO Technologies GmbH",
    "0050C285C": "B S E",
    "0050C285B": "Boreste",
    "0050C2859": "Nuvation",
    "0050C2871": "R-S-I Elektrotechnik GmbH & Co. KG",
    "0050C2873": "XRONET Corporation",
    "0050C286A": "USM Systems, Ltd",
    "0050C286C": "Condigi Televagt A/S",
    "0050C2869": "Funkwerk plettac electronic GmbH",
    "0050C2854": "Ratioplast-Optoelectronics GmbH",
    "0050C2863": "Advanced Technology Solutions",
    "0050C285F": " GD Mission Systems",
    "0050C287C": "Arcontia AB",
    "0050C288B": "Hollis Electronics Company LLC",
    "0050C2882": "WARECUBE,INC.",
    "0050C282B": "Keith & Koep GmbH",
    "0050C2832": "S1nn GmbH & Co. KG",
    "0050C2834": "ANTEK GmbH",
    "0050C282E": "LogiCom GmbH",
    "0050C282F": "Momentum Data Systems",
    "0050C2829": "Intellectronika",
    "0050C2827": "Enero Solutions inc.",
    "0050C2825": "Funkwerk Information Technologies Karlsfeld GmbH",
    "0050C2846": "ESP-Planning Co.",
    "0050C2847": "Lars Morich Kommunikationstechnik GmbH",
    "0050C283A": "Syr-Tec Engineering & Marketing",
    "0050C284F": "Gamber-Johnson LLC",
    "0050C284E": "DRACO SYSTEMS",
    "0050C2810": "Alphion Corporation",
    "0050C280E": "Bruno International Ltd.",
    "0050C280C": "Luxpert Technologies Co., Ltd.",
    "0050C2818": "Wireless Value BV",
    "0050C2814": "Telvent",
    "0050C2815": "microC Design SRL",
    "0050C27F3": "SOREC",
    "0050C27F2": "Logotherm Regelsysteme GmbH",
    "0050C27EC": "Lyngsoe Systems",
    "0050C2800": "Delphi Display Systems, Inc.",
    "0050C281D": "IT SALUX CO., LTD.",
    "0050C2809": "Varma Electronics Oy",
    "0050C27DE": "Cascade Technologies Ltd",
    "0050C27D9": "Volumatic Limited",
    "0050C27DA": "HTEC Limited",
    "0050C27CF": "Emitech Corporation",
    "0050C27D0": "Radar Tronic ltd.",
    "0050C27CC": "SWECO JAPS AB",
    "0050C27DB": "Mueller Elektronik",
    "0050C25A2": "iNET Systems Inc.",
    "0050C2784": "Lewis Controls Inc.",
    "0050C27E8": "Mistral Solutions Pvt. Ltd",
    "0050C27E7": "V2Green, Inc.",
    "0050C27C8": "Fr. Sauter AG",
    "0050C27C4": "MoBaCon",
    "0050C27C3": "AST INCORPORATED",
    "0050C27BF": "Zoe Medical",
    "0050C27DF": "Private",
    "0050C27AC": "IUSA SA DE CV",
    "0050C27A1": "Field Design Service",
    "0050C279B": "Schniewindt GmbH & Co. KG",
    "0050C279C": "Vital Systems Inc",
    "0050C27B6": "Alstom (Schweiz) AG",
    "0050C27B1": "ATEME",
    "0050C27A6": "ASIANA IDT",
    "0050C2794": "COMSONICS, INC.",
    "0050C2779": "Coral Telecom Ltd",
    "0050C2778": "SunGard Vivista",
    "0050C2774": "GeoSIG Ltd.",
    "0050C276B": "Putercom Enterprise Co., LTD.",
    "0050C276C": "EFG CZ spol. s r.o.",
    "0050C2783": "NORMA systems GmbH",
    "0050C277D": "Lincoln Industrial",
    "0050C2750": "Brightlights Intellectual Property Ltd",
    "0050C274D": "Beceem Communications, Inc.",
    "0050C2749": "Affolter Technologies SA",
    "0050C274B": "STAR-Dundee Ltd",
    "0050C2742": "Fantuzzi Reggiane",
    "0050C2745": "ACISA",
    "0050C2737": "Teradici Corporation",
    "0050C2754": "Abeo Corporation",
    "0050C2753": "ABB",
    "0050C2733": "Cimetrics Research Pty Ltd",
    "0050C271A": "Logus Broadband Wireless Solutions Inc.",
    "0050C2715": "RIEXINGER Elektronik",
    "0050C271B": "ADVA Optical Networking",
    "0050C26FD": "SAIA Burgess Controls AG",
    "0050C272A": "Phytec Messtechnik GmbH",
    "0050C2729": "SP Controls, Inc",
    "0050C2709": "RO.VE.R. Laboratories S.p.A",
    "0050C270E": "AUDICO SYSTEMS OY",
    "0050C271F": "ASC telecom AG",
    "0050C26CF": "Microway",
    "0050C26D2": "Lumistar Incorporated",
    "0050C26EE": "Interactive Electronic Systems",
    "0050C26EC": "Netistix Technologies Corporation",
    "0050C26E6": "Lanetco",
    "0050C26AC": "Thales UK",
    "0050C26A9": "Armida Technologies Corporation",
    "0050C26A7": "Hoer GmbH & Co. Industrie-Electronic KG",
    "0050C269E": "Ideus AB",
    "0050C26B3": "4RF Communications Ltd",
    "0050C26AE": "Qualisys AB",
    "0050C26BE": "ESTEC Co.,Ltd.",
    "0050C26D5": "Becker Electronics GmbH",
    "0050C26B7": "System LSI CO.Ltd.",
    "0050C26CD": "RGM SPA",
    "0050C26CE": "EMITALL Surveillance S.A,",
    "0050C26B8": "Epec Oy",
    "0050C2674": "Protech Optronics Co., Ltd.",
    "0050C2672": "DDS Elettronica srl",
    "0050C2682": "Commet AB",
    "0050C2684": "REASON Tecnologia S.A.",
    "0050C2698": "Navtech Radar Ltd",
    "0050C2692": "Mate Media Access Technologies",
    "0050C2691": "Interopix, Inc.",
    "0050C2697": "Monarch Instrument",
    "0050C2694": "Initel srl",
    "0050C265D": "Redfone Communications LLC",
    "0050C2658": "OpenPKG GmbH",
    "0050C2669": "DSP DESIGN",
    "0050C2637": "Omnitrol Networks, Inc.",
    "0050C2638": "HUNGAROCOM Telecommunication Ltd.",
    "0050C2636": "dSPACE GmbH",
    "0050C2632": "RoseTechnology A/S",
    "0050C2633": "Rice University",
    "0050C262F": "QES",
    "0050C2648": "Fidelity Comtech, Inc.",
    "0050C2640": "IAC",
    "0050C263A": "3DSP Corporation",
    "0050C265E": "Cantion A/S",
    "0050C2649": "Pan-STARRS",
    "0050C2609": "Toptech Systems, Inc.",
    "0050C260A": "Gradual Tecnologia Ltda.",
    "0050C2607": "Telecom FM",
    "0050C2602": "CHAUVIN ARNOUX",
    "0050C2603": "Cerus Corp",
    "0050C261B": "NCI Technologies Inc.",
    "0050C2618": "Intergrated Security Mfg. Ltd",
    "0050C2614": "Proserv",
    "0050C262D": "Procon Electronics",
    "0050C25D7": "Synrad, Inc.",
    "0050C25D5": "Cannon Technologies",
    "0050C25D4": "Buyang Electronics Industrial Co., Ltd.",
    "0050C25E8": "Info-Chip Communications Ltd.",
    "0050C25D9": "Crimson Microsystems, Inc.",
    "0050C25DC": "RM Michaelides Software & Elektronik GmbH",
    "0050C25DE": "Magal Senstar Inc.",
    "0050C25EF": "pikkerton GmbH",
    "0050C25EE": "Condre Corporation",
    "0050C25DB": "Secure Systems & Services",
    "0050C25F2": "ESEM Gr\u00fcnau GmbH & Co. KG",
    "0050C25ED": "AQUAROTTER A FRANKE COMPANY",
    "0050C25D0": "Automata Spa",
    "0050C25B3": "HITECOM System",
    "0050C25C6": "Technische Alternative GmbH",
    "0050C25C7": "InSync Technology Ltd",
    "0050C2580": "Buyang Electronics Industrial co.,Ltd.",
    "0050C257B": "ptswitch",
    "0050C2577": "Advanced Software Technologies",
    "0050C257A": "nVent, Schroff GmbH",
    "0050C2588": "Federal Electronics",
    "0050C2584": "Toyota Motorsport GmbH",
    "0050C259D": "DSS Networks, Inc.",
    "0050C2572": "Chell Instruments Ltd",
    "0050C25A3": "LUMEL S.A.",
    "0050C25A1": "Vestfold Butikkdata AS",
    "0050C256C": "Targeted Technologies, LLC",
    "0050C2598": "Bundesamt f\u00fcr Strahlenschutz",
    "0050C2551": "Innovative Neurotroncs",
    "0050C254D": "Silent System",
    "0050C2554": "Weinzierl Engineering GmbH",
    "0050C254C": "Sintecnos srl",
    "0050C254B": "Innopsys",
    "0050C255C": "ads-tec GmbH",
    "0050C2542": "AVerMedia Technologies, Inc.",
    "0050C2544": "Zetera",
    "0050C253C": "Marposs SPA",
    "0050C253B": "Teleks Co. Ltd.",
    "0050C2520": "McCain Traffic Supply",
    "0050C251D": "VELUX",
    "0050C2525": "VASTech",
    "0050C2522": "Mark IV IDS Corp.",
    "0050C252F": "Gesellschaft f\u00fcr  Rationalisierung und Rechentechnik mbH",
    "0050C2546": "Universidad de Chile Facultad de Medicina",
    "0050C2507": "Micro Connect Pty Ltd",
    "0050C2500": "Orenco Systems, Inc.",
    "0050C2511": "Tecna Srl",
    "0050C2512": "Linear Acoustic, Inc",
    "0050C2515": "Monaghan Engineering, Inc.",
    "0050C24F8": "Prodco International Inc.",
    "0050C24F9": "RTDS Technologies Inc.",
    "0050C2518": "Christ Elektronik GmbH",
    "0050C24CA": "Yarg Biometrics Limited",
    "0050C24C7": "Transbit Sp.z o.o.",
    "0050C24EA": "PMC",
    "0050C24ED": "Lab X Technologies, LLC",
    "0050C24DA": "MEDIORNET GmbH",
    "0050C24DB": "Alfing Montagetechnik GmbH",
    "0050C24E5": "Sedo Systems Ltd",
    "0050C24E2": "Applied Research Laboratories: UT",
    "0050C24D2": "Twoway CATV SERVICE INC.",
    "0050C24B4": "Matrix Audio Designs",
    "0050C24B6": "General Resources Co., LTD.",
    "0050C24B3": "ANSA Corporation",
    "0050C24B5": "Valley Tecnologia",
    "0050C24B2": "TESLA, a.s.",
    "0050C249E": "Armorlink Co .Ltd",
    "0050C249B": "vg controls, inc",
    "0050C24A5": "Teledyne Monitor Labs",
    "0050C24A7": "iseg Spezialelektronik GmbH",
    "0050C2498": "Quartet Technology, Inc.",
    "0050C2492": "TRAFSYS AS",
    "0050C246E": "Avenir Technologies Inc.",
    "0050C2471": "Pixtree Technologies, inc.",
    "0050C2484": "Kooltech LLC",
    "0050C2479": "Unlimited Bandwidth LLC",
    "0050C246A": "ISE GmbH",
    "0050C2468": "Network I/O",
    "0050C2474": "Venue 1, Inc.",
    "0050C2482": "PRIAMUS SYSTEM TECHNOLOGIES AG",
    "0050C2443": "Pickering Laboratories",
    "0050C245A": "Funkwerk plettac electronic GmbH",
    "0050C2450": "Enconair Ecological Chambers Inc.",
    "0050C2451": "HAMEG GmbH",
    "0050C2421": "EFSYS",
    "0050C2422": "ads-tec GmbH",
    "0050C243B": "A3IP",
    "0050C2437": "PowerWAN, Inc",
    "0050C241E": "Videotek Sistemas Eletronicos Ltda.",
    "0050C241C": "Infrasafe, Inc.",
    "0050C2433": "Idetech Europe S.A.",
    "0050C2416": "SELCO s.r.l.",
    "0050C241A": "Bluewater Systems Ltd",
    "0050C23E3": "CSIRO - Division of Exploration and Mining",
    "0050C23E1": "NeuLion Incorporated",
    "0050C23E0": "Oy Stinghorn Ltd",
    "0050C2405": "Guralp Systems Limited",
    "0050C2401": "Promess Incorporated",
    "0050C23EC": "Teneros",
    "0050C23FD": "HARTMANN software GbR",
    "0050C23E7": "Revolution Education Ltd",
    "0050C23F7": "Advantage R&D",
    "0050C23BB": "IMAGO Technologies GmbH",
    "0050C23B7": "Mindspeed Technologies",
    "0050C23B8": "Keith & Koep GmbH",
    "0050C23AA": "Networked Robotics Corporation",
    "0050C23A2": "Vegas Amusement",
    "0050C23A7": "Elektrotechnik & Elektronik Oltmann GmbH",
    "0050C23D6": "Comlab Inc.",
    "0050C23B2": "Tennessee Valley Authority",
    "0050C23AC": "InAccess Networks",
    "0050C23A9": "Westronic Systems Inc.",
    "0050C23D1": "Braintronics BV",
    "0050C2397": "MANGO DSP Ltd.",
    "0050C2396": "RapidWave Inc.",
    "0050C2399": "Advanced Micro Controls Inc.",
    "0050C238A": "Embedtronics Enterprise",
    "0050C2377": "Xycom VME",
    "0050C2376": "CLEODE",
    "0050C2373": "Companion Worlds, inc.",
    "0050C238E": "Nordic Alarm AB",
    "0050C239D": "DigitalDeck, Inc.",
    "0050C238B": "InterBridge,Inc.",
    "0050C239E": "A.R.G ElectroDesign Ltd",
    "0050C235C": "Ratotec GmbH",
    "0050C2361": "Contec",
    "0050C235A": "Advanced Si-Net Co., LTD.",
    "0050C2356": "Baytech Cinema",
    "0050C234B": "Ecutel Systems, Inc.",
    "0050C2352": "LUCEO",
    "0050C2350": "Kinesys Projects Limited",
    "0050C2353": "Crossing Informationssysteme GmbH",
    "0050C236C": "RISCO Group",
    "0050C2340": "Virtu",
    "0050C2313": "SAIA Burgess Controls AG",
    "0050C2314": "MicroBee Systems, Inc",
    "0050C2336": "Golden River Traffic",
    "0050C2337": "ETI",
    "0050C232F": "PULTRONICS",
    "0050C231F": "Geotech Instruments, LLC",
    "0050C2332": "PUNJAB COMMUNICATIONS LTD",
    "0050C230A": "Innings Telecom Inc.",
    "0050C231B": "Datacon",
    "0050C22F3": "Crossbow Technology, Inc.",
    "0050C22EC": "CHENGDU  BOOK DIGITAL CO., LTD",
    "0050C22F6": "Clifford Chance LLP",
    "0050C22D9": "Private",
    "0050C2302": "EuroDesign embedded technologies GmbH",
    "0050C2303": "CI Systems Ltd.",
    "0050C22D6": "WIS Technologies",
    "0050C22FD": "American Microsystems, Ltd.",
    "0050C22E5": "Transtech DSP",
    "0050C22B4": "Polatis Ltd",
    "0050C22B5": "Hobbes Computer Network Accessories",
    "0050C22B0": "Telda Electronics",
    "0050C22AF": "CSA  Computer & Antriebstechnik GmbH",
    "0050C22C7": "Siliquent Technologies Ltd",
    "0050C22C8": "SELCO",
    "0050C22C6": "Initial Electronic Security Systems",
    "0050C22C3": "Digital SP Ltd",
    "0050C22AD": "ABB T&D Spa",
    "0050C22AC": "BBI Engineering, Inc.",
    "0050C22BC": "Uster Technologies AG",
    "0050C22A5": "Septier Communication Ltd",
    "0050C2024": "IPITEK",
    "0050C229A": "Packet Techniques Inc.",
    "0050C229B": "ACD Elektronik GmbH",
    "0050C229C": "2N TELEKOMUNIKACE a.s.",
    "0050C22A0": "Sterling Industry Consult GmbH",
    "0050C2297": "KINETICS",
    "0050C227A": "Maestro Pty Ltd",
    "0050C2269": "Technisyst Pty Ltd",
    "0050C2267": "Allen Martin Conservation Ltd",
    "0050C226F": "Digital Recorders Inc",
    "0050C2279": "PATLITE Corporation",
    "0050C2272": "Verex Technology",
    "0050C228E": "Tattile srl",
    "0050C2258": "Spacesaver Corporation",
    "0050C2242": "MDS SCIEX",
    "0050C2245": "Hauppauge Computer Works, Inc.",
    "0050C224C": "Supertel",
    "0050C2221": "Getinge IT Solutions ApS",
    "0050C221D": "ESG Elektroniksystem u. Logistik GmbH",
    "0050C2238": "Schwer+Kopka GmbH",
    "0050C222E": "LORD INGENIERIE",
    "0050C2225": "nVent, Schroff GmbH",
    "0050C221F": "Monitor Business Machines Ltd",
    "0050C2236": "JLCooper Electronics",
    "0050C2209": "LK Ltd",
    "0050C2206": "Windmill Innovations",
    "0050C21FC": "EDD Srl",
    "0050C21FD": "SouthWing S.L.",
    "0050C21EC": "COSMO co.,ltd.",
    "0050C21E0": "Cognex Corporation",
    "0050C21DB": "Applied Systems Engineering, Inc.",
    "0050C21CA": "EVER Sp. z o.o.",
    "0050C21E3": "Bluesocket, Inc.",
    "0050C21CF": "LIFETIME MEMORY PRODUCTS, INC.",
    "0050C21CE": "Datatek Applications, Inc.",
    "0050C21D3": "Synopsys",
    "0050C21AB": "Streaming Networks",
    "0050C21A3": "Tidel Engineering, L.P.",
    "0050C21BA": "INTERZEAG AG",
    "0050C21A2": "ABB Switzerland Inc",
    "0050C21BE": "Sedia Electronics",
    "0050C21A9": "Axotec Technologies GmbH",
    "0050C2182": "wolf-inf-tec",
    "0050C2190": "Goerlitz AG",
    "0050C2170": "Taishodo Seiko Co., Ltd.",
    "0050C2044": "Private",
    "0050C2004": "SCI Technology Inc.",
    "0050C200B": "IO Limited",
    "0050C2014": "Canal +",
    "0050C2164": "Acunia N.V.",
    "0050C217D": "Cognex Corporation",
    "0050C2177": "Unicoi Systems",
    "0050C216B": "Masterclock, Inc.",
    "0050C200E": "TTTech",
    "0050C200C": "Vbrick Systems Inc.",
    "0050C2016": "DSP Design Ltd.",
    "0050C200A": "Tharsys",
    "0050C216C": "Brijing Embedor Embedded Internet Tech. Co. Ltd.",
    "0050C2171": "Quantronix, Inc.",
    "0050C2176": "Wavium AB",
    "0050C2156": "CommServ Solutions Inc.",
    "0050C2161": "J&B Engineering Group S.L.",
    "0050C2160": "TTI - Telecom International Ltd.",
    "0050C2162": "Teseda Corporation",
    "0050C2159": "Standard Comm. Corp.",
    "0050C2151": "Redux Communications Ltd.",
    "0050C2127": "TPA Traffic & Parking Automation BV",
    "0050C2131": "InBus Engineering, Inc.",
    "0050C2122": "Diva Systems",
    "0050C2123": "Seranoa Networks, Inc.",
    "0050C2119": "Global Opto Communication Tech. Corp",
    "0050C2110": "QuesCom",
    "0050C20ED": "Valley Products Corporation",
    "0050C20FB": "PIUSYS Co., Ltd.",
    "0050C2100": "Corelatus A.B.",
    "0050C2101": "LAUD Electronic Design AS",
    "0050C20FF": "IPAXS Corporation",
    "0050C20F2": "KMS Systems, Inc.",
    "0050C20F0": "VHB Technologies, Inc.",
    "0050C20EE": "Industrial Indexing Systems, Inc.",
    "0050C20E9": "Smartmedia LLC",
    "0050C20DE": "Frederick Engineering",
    "0050C20D4": "Palm, Inc.",
    "0050C20D6": "Inco Startec GmbH",
    "0050C20B7": "RYMIC",
    "0050C20BA": "Pro-Active",
    "0050C20C3": "Tonbu, Inc.",
    "0050C20A5": "Mobiltex Data Ltd.",
    "0050C20A9": "Radiant Networks Plc",
    "0050C20A3": "BaSyTec GmbH",
    "0050C20AF": "Latus Lightworks, Inc.",
    "0050C20A1": "Visable Genetics, Inc.",
    "0050C209B": "Seffle Instrument AB",
    "0050C2087": "Monitor Business Machines Ltd.",
    "0050C208E": "BSQUARE",
    "0050C2093": "KOREALINK",
    "0050C208D": "Kylink Communications Corp.",
    "0050C208C": "IP Unity",
    "0050C2086": "Validyne Engineering Corp.",
    "0050C2089": "Fenwal Italia S.P.A.",
    "0050C2081": "Matuschek Messtechnik GmbH",
    "0050C2084": "DIALOG4 System Engineering GmbH",
    "0050C207E": "JL-teknik",
    "0050C2079": "Flextel S.p.A",
    "0050C2052": "Mayo Foundation",
    "0050C2050": "Dataprobe, Inc.",
    "0050C204A": "Telecom Analysis Systems, LP",
    "0050C2058": "Vision Research, Inc.",
    "0050C2045": "Chase Manhattan Bank",
    "0050C2037": "E.I.S.M.",
    "0050C2038": "Etheira Technologies",
    "0050C202A": "VersaLogic Corp.",
    "0050C2019": "Emtac Technology Corp.",
    "0050C201E": "CallTech International Limited",
    "0050C2011": "Bihl + Wiedemann GmbH",
    "0050C2007": "Clive Green & Co. Ltd.",
    "0050C2010": "Moisture Systems",
    "0050C2006": "Project Management Enterprises, Inc.",
    "0050C22C5": "Elman srl",
    "0050C2F83": "Televic Rail GmbH",
    "0050C2886": "Wartsila Voyage Oy",
    "0050C2B62": "Rugged Controls",
    "EA2701": "ACCE Technology Corp.",
    "4A688D": "Shenzhen TCL New Technology Co., Ltd",
    "3A0E75": "KEVCOM LLC",
    "4A5AA9": "GoCodeIT Inc",
    "AA146B": "IronWiFi",
    "BA6791": "Kandou Bus SA",
    "DA1E56": "OSNexus",
    "AAF019": "TransferJet Consortium Incorporated Association",
    "DA7D2E": "Delta Computers LLC.",
    "0A8E62": "Delta Solutions LLC",
    "CA045A": "Ossia Inc",
    "3A5A27": "Shenzhen Longsys Electronics Co.,Ltd.",
    "6A9DA4": "Global Reach Technology",
    "3A78CA": "SoundTalks NV",
    "6A5C35": " Secr\u00e9tariat g\u00e9n\u00e9ral de la d\u00e9fense et de la s\u00e9curit\u00e9 nationale",
    "3AF521": "Single Digits",
    "4AC03F": "WUHAN HIKSTORAGE TECHNOLOGY CO.,LTD",
    "6A4065": "Open Connectivity Foundation",
    "DAA119": "Google, Inc.",
    "AA38C7": "Kaloom inc",
    "CA30BF": "IEEE 802.1 Chair",
    "FA1619": "trendytech",
    "FA610E": "Laboratory for Computational Sensing and Robotics, Johns Hopkins University",
    "FA63E1": "Samsung Electronics (UK) Ltd",
    "8A34BC": "Fiberworks AS",
    "DAFF6B": "Team Precision Public Company Limited",
    "7A4BA4": "Innostor Technology Corporation",
    "9A564B": "Cubix Corporation",
    "EA34B4": "ThinkRF Inc.",
    "2A9FEB": "AVnu Alliance",
    "5A73CF": "Gogo Business Aviation",
    "0AE91B": "PADL Software Pty Ltd",
    "7A8DD1": "TePS'EG",
    "8A0775": "HORYS TECHNOLOGIES LLC",
    "2ADBD7": "Vesper Technologies",
    "CAC276": "Saab Seaeye Ltd",
    "DAE6D7": "NVM Express, Inc. ",
    "4AED1D": "Astera Labs Inc",
    "7A3F8C": "Seagate Technology Thailand Ltd.",
    "DA9D49": "Calix Inc.",
    "4A6704": "Reliable Robotics Corporation",
    "4A1540": "sas de la Halle aux Fours",
    "8AD85E": "GigaIO Networks, Inc.",
    "AADC47": "Cachengo, Inc.",
    "0A361C": "Smardii",
    "EA2ADB": "Labs Network Industrie 4. 0 e.V.",
    "1ABBFB": "UHD Alliance",
    "EA115D": "AUTOVISION TECHNOLOGY INC.",
    "FAD988": "Momentum Data Systems",
    "BA4DEA": "Environexus",
    "EAB89B": "Thread Group, Inc.",
    "7A84F0": "McDowell Signal Processing, LLC (dba McDSP)",
    "BAA2D0": "Wireless Broadband Alliance Ltd",
    "BAF21C": "ISO/TC22/SC31",
    "DA0D38": "Farpointe Data, Inc.",
    "DAC87D": "SE Development AB",
    "0AE471": "Caterpillar Inc.",
    "5ACBD3": "Simaudio Ltd",
    "FA2EE9": "OCA Alliance, Inc.",
    "3AA3F8": "IEEE 802.1 Chair",
    "FA94F1": "IEEE 802.1 Chair",
    "3A0292": "Video Electronics Standards Association",
    "FAE51A": "RSAE Labs Inc",
    "DA28EC": "HG Genuine",
    "4A191B": "ZigBee Alliance",
    "8A00B0": "Eoptolink Technology Inc. Ltd,",
    "3ABA37": "Cirrent",
    "9A1B22": "InfoCore Co.,Ltd.",
    "AA0F85": "Yamaichi Electronics",
    "0A8736": " IEEE 1901 Working Group",
    "9AD263": "FirmTek, LLC",
    "2AFD6A": "Charter Communications",
    "7AF955": "Relay, Inc.",
    "0A35F2": "Integrated Technical Vision Ltd",
    "0AAB8C": "Midwest Microwave Solutions Inc.",
    "CAFAE6": "Tianjin Ronghui microelectronics Technology Co., Ltd.",
    "EA8F96": "Proficium, LLC",
    "3A1EC0": "GENESIS",
    "8A852E": "Kampr Systems",
    "FA00E7": "Quantum Machines ltd",
    "6A71E6": "FuelCloud",
    "FA0BBC": "CEN",
    "8AD20F": "Pro Optix AB",
    "CA3EA6": "Syng, Inc.",
    "7A384E": "PROFIBUS Nutzerorganisation e.V.",
    "3A765D": "NewCoSemi(Beijing) Technology Co., Ltd",
    "EA1ED0": "ZIGPOS GmbH",
    "0AD9C4": "Craft Designs, Inc",
    "2A0BB8": "xFusion Digital Technologies Co., Limited",
    "7A4BF3": "DILON TECHNOLOGIES",
    "7A15EC": "Yatomi Techology(ZheJiang) Co.,Ltd",
    "6A8EAF": "Waoo",
    "4AE03F": "Chengdu Superxon communication Technology Co. Ltd.",
    "5A18FF": "FiRa Consortium",
    "5A03BA": "Wireless Broadband Alliance Ltd",
    "EABC04": "G3-PLC Alliance",
    "3A3541": "Raspberry Pi (Trading) Ltd",
    "BAC485": "Fraunhofer IIS",
    "FABA85": "APAT Optoelectronics Components Co., Ltd.",
    "8ACBA4": "Resideo",
    "6AB6F2": "Eli Technology Inc",
    "FAEB6E": "xRAN.org",
    "4A5A6F": "Applied Materials",
    "EA6076": "CloudSimple, Inc.",
    "6A0E20": "GeoFrenzy, Inc",
    "BA55EC": "IEEE 802.15",
    "BA3054": "George Wilson Industries Ltd",
    "CA125C": "Microsoft Corporation",
    "FA1466": "Allegion PLC",
    "EAE0D9": "Berk-tek LLC",
    "1A75FB": "AUTOSAR",
    "CAE505": "Mobilitie",
    "FA556F": "SymbolicIO",
    "BA03C6": "Sch\u00fcco International KG",
    "DAC451": "Takaratomy",
    "BAC8C4": "Transponder & Reader Engineered Systems, Inc",
    "CA7DDD": "EmOne (Thailand) Co.,Ltd.",
    "3A796E": "LX Semicon co.,Ltd.",
    "6A621E": "DIFI Consortium",
    "BA6312": "IDTECK",
    "4AADCE": "CODAN ARGUS AG",
    "8A4F8B": "Irdeto",
    "7A35EA": "SHARP Corporation",
    "8A94AD": "Nexgen A/S",
    "0A2A33": "DIGISTOR",
    "3A72C2": "HUAWEI TECHNOLOGIES CO.,LTD",
    "2AA139": "POINTGUARD, LLC",
    "CAF25B": "Lexar Electronics(Shenzhen) Co.,Ltd.",
    "8AFB16": "TeraByte, Inc.",
    "BA4159": "ANAPASS INC.",
    "6A1F6C": "iPass, Inc.",
    "3ACFC5": "Quickline AG",
    "EADD88": "IEEE PES-PSRC Working Group H3, PC37.237",
    "4A07D6": "IEEE 802.1 Chair",
    "2AEA15": "Tibit Communications",
    "7AFD44": "Rosenberger Hochfrequenztechnik GmbH & Co. KG",
    "5AE660": "nyantec GmbH",
    "DA1C21": "Sesam GmbH",
    "8ADA26": "Eleven Inc.",
    "4A7206": "CAIRE, Inc.",
    "9AADCA": "Credo",
    "3A5E03": "Cirrus Data Solutions, Inc",
    "9AB2F8": "SFF Committee",
    "FAF9C0": "RAID Incorporated",
    "FAE190": "IN WIN DEVELOPMENT INC.",
    "4ADA10": "Emotiva Audio Corp",
    "7AAB77": "Orange",
    "FA073E": "NBASE-T Alliance",
    "CA446C": "Cypress Computer Systems, Inc.",
    "FA4C77": "OCCITALINE",
    "2AA147": "DISTRIBUCIONES CANTILEVER",
    "CA222B": "Beijing Puma Gear Power Equipment Co.,Ltd",
    "FAEEB6": "Shenzhen Jiahua Zhongli Technology Co., LTD",
    "5A6BB4": "Xconn Technologies",
    "8A7DB3": "Evolv Technology, Inc.",
    "DA19B3": "LG Uplus",
    "EABEA7": "Sonos, Inc.",
    "0AA23B": "Vitex LLC",
    "2AAC53": "Wolfspyre Labs",
    "CA1E45": "ASMedia Technology Inc.",
    "8A0AF7": "Montage Technology Co,.LTD",
    "7A4D2F": "Open Compute Project",
    "6AE732": "UNIS-WDC Storage Co.,Ltd",
    "BA07DA": "Infinix mobility limited",
    "AA34D4": "CDVI Wireless SpA",
    "CA3782": "Storbyte, Inc.",
    "CA2B5E": "Suzhou Motor-comm Electronic Technology Co.,Ltd",
    "FA6673": "NELLS ELECTRONIC SYSTEMS (NES)",
    "0ACD8F": "Cisco Systems Norway",
    "4ACA3E": "ZEUS",
    "DA4162": "Xperi Corporation",
    "4A7F74": "INTEC Inc.",
    "8AA5C1": "Ranovus USA",
    "CA8FA5": "Advanced TS Migrations, Co",
    "AA548B": "Tintri",
    "8AB3DA": "HomePlug Powerline Alliance, Inc.",
    "DA2BDD": "AT&T, Inc.",
    "8AC72E": "Roku, Inc.",
    "CAC87E": "Allen Organ Company",
    "EA9FB1": "Philips International B.V.",
    "6A737D": "25G Ethernet Consortium",
    "6AE64A": "S&O Electronics (Malaysia) Sdn. Bhd.",
    "1A9021": "Physical Security Interoperability Alliance",
    "4AE568": "leapio",
    "5AED67": "GoldiLink Technology Corp.",
}


func ManufLookup(mac string) string {
	macHex := strings.ToUpper(strings.Replace(mac, ":", "", -1))
	macLen := len(macHex)

	for offset := macLen; offset >= 2; offset -= 2 {
		part := macHex[0:offset]
		if vendor, found := manuf[part]; found {
			return vendor
		}
	}

	return ""
}